package com.ibm.btools.blm.ie.exprt;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.List;

/**
 * Provides an abstract implementation of the export operation. The 
 * extended class should provide the implementation of the <code>
 * export</code> method.
 */
public abstract class AbstractExportOperation implements IExportOperation {

	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	private List modelElements;
	private String outputDir;
	private String projectName;
	private ExportSession session;	
	private String name; 
	
	/**
	 * Sets the name of the operation
	 * @param name
	 */
	public void setName(final String name) {
		this.name = name;
	}
	
	/**
	 * Gets the name of the operation
	 * @return
	 */
	public String getName() {
		return name;	
	}
	
	/**
	 * Subclass should implements this method
	 */
	public void export() {

	}

	/**
	 * Gets the export session 
	 */
	public ExportSession getExportSession() {
		
		return session;
	}

	/**
	 * Gets the output directory to be exported to
	 */
	public String getOutputDir() {
	
		return outputDir;
	}

	/**
	 * Gets the project name that the elements of export resides
	 */
	public String getProjectName() {
	
		return projectName;
	}

	/**
	 * Gets the BOM model elements (top level) to be exported.
	 */
	public List getModelElements() {
		
		return modelElements;
	}

	/**
	 * Sets the export session
	 */
	public void setExportSession(ExportSession session) {
		this.session = session;

	}

	/**
	 * Sets the output directory to be exported to 
	 */
	public void setOutputDir(String outputDir) {
		this.outputDir = outputDir;
	}

	/**
	 * Sets the project name that the elements of export resides
	 */
	public void setProjectName(String projectName) {
		this.projectName = projectName;

	}

	/** 
	 *  Gets the BOM model elements (top level) to be exported.
	 */
	public void setModelElements(List selectedModelElements) {
		this.modelElements = selectedModelElements;

	}

}