package com.ibm.btools.blm.ie.exprt;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.osgi.framework.Bundle;

import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;

/**
 * Registry reader for the export operations	
 */
public class ExportOperationRegistryReader {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	

	protected Map exportOperations;

	public static ExportOperationRegistryReader reader;
	
	/**
	 * Returns the registry reader
	 * @return
	 */
	public static ExportOperationRegistryReader getReader() {
		if (reader == null) {
			// I didn't create a private constructor here becasue 
			// I give the application an optional to read the registry
			// everytime, so can save the memory if they don't want to 
			// use the static class
			reader = new ExportOperationRegistryReader();			
		}
		
		return reader;
	}

	/**
	 * The Empty map that contain the export operation and name pair
	 * @return
	 */
	protected Map createEmptyExportOperationCollection() {
		return new HashMap();
	}

	/**
	 * Read the Export Operations from the extensions
	 */	
	protected void readExportOperations() {
		LoggingUtil.traceEntry(this, "readExportOperations");		
		if(exportOperations == null) {
			exportOperations = createEmptyExportOperationCollection();
			// read the operations
			
			IExtensionRegistry pluginRegistry = Platform.getExtensionRegistry();
			IExtensionPoint point = null;
			IExtension[] extensions = null;
			IConfigurationElement[] elements = null;
			Object[] operationInfo = null;
			String className = null;
			String id = null;
			String name = null;
			ClassLoader classLoader = null;
			Class exportOperationClass = null;
			IExportOperation exportOperation = null;
			String elementName = null;
		
			// The extension point of the export operation is "com.ibm.btools.blm.ie.exportOperations"
			point = pluginRegistry.getExtensionPoint(IExportConstants.EXPORT_OPERATION_EXTENSION_POINT);
			LoggingUtil.trace(this, "readExportOperations", "Get Extension Point from Plugin Registry.");			
			extensions = point.getExtensions();
			for (int i = 0; i < extensions.length; i++) {
				// Get the elements
				elements = extensions[i].getConfigurationElements();						
				for (int j = 0; j < elements.length; j++) {
					elementName = elements[j].getName();
					LoggingUtil.trace(this, "readExportOperations", "Get extension element: " + elementName + ".");					
					if (IExportConstants.EXPORT_OPERATION_ELEMENT_OPERATION.compareTo(elementName) == 0) {
						// Get the operation name by id
						id = elements[j].getAttribute(IExportConstants.EXPORT_OPERATION_ATT_ID);
						if (id == null) {
							LoggingUtil.trace(this, "readExportOperations", "The element id is null.");								
							break;
						}
						className = elements[j].getAttribute(IExportConstants.EXPORT_OPERATION_ATT_CLASS);
						if (className == null) {
							LoggingUtil.trace(this, "readExportOperations", "The element class name is null.");							
							break;
						}						
						name = elements[j].getAttribute(IExportConstants.EXPORT_OPERATION_ATT_NAME);
						if (name == null) {
							// Name is required but we can change the schema to support it as optional
							// If this is the case, we just use the id as the name.
							LoggingUtil.trace(this, "readExportOperations", "The element name is null.");							
							name = id;						
						}
					
						// Load the class
						Bundle plugin = Platform.getBundle(extensions[i].getNamespace());
						try {						
							exportOperationClass = plugin.loadClass(className);
							exportOperation = (IExportOperation) exportOperationClass.newInstance();
							exportOperation.setName(name);
						} catch (Exception e) {
							// skip
							LoggingUtil.trace(this, "readExportOperations", "Cannot load the element class.");							
							break;						
						}
						operationInfo = new Object[] {name, exportOperation};
						
						exportOperations.put(id, operationInfo);						
					}		
				}					
			}
		}
		LoggingUtil.traceExit(this, "readExportOperations");		
	}	
	
	/**
	 * Returns a map of the export operations.
	 * @return Map of export operations
	 */	
	public Map getExportOperations() {
		if(exportOperations == null)
			readExportOperations();
		return exportOperations;
	}
		
	/**
	 * Return the export operation by id.
	 * @param id
	 * @return
	 */		
	public IExportOperation findExportOperation(final String id) {
		LoggingUtil.traceEntry(this, "findExportOperation");		
		Map exportOperations = getExportOperations();
		Object[] operationInfo = (Object[]) exportOperations.get(id);
		if (operationInfo == null) {
			LoggingUtil.trace(this, "findExportOperation", "Cannot find the operation: " + id + ".");			
			LoggingUtil.traceExit(this, "findExportOperation");			
			return null;
		}
		
		Object operation = operationInfo[1];
		if (operation != null) {		
		
			if (operation instanceof IExportOperation) {				
				LoggingUtil.traceExit(this, "findExportOperation");				
				return (IExportOperation) operation; 
			}
			LoggingUtil.trace(this, "findExportOperation", "The operation is not a proper class: " + operation.getClass() + ".");			
		}
		
		LoggingUtil.traceExit(this, "findExportOperation");		
		return null;
	}	
}
