package com.ibm.btools.blm.ie.exprt.option;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Vector;

/**
 * This class is to encapsulate the options contants into a java bean, which represents all the 
 * WPS export wizard options 
 * The instance of the bean is only one per export session, which will be cached in an options
 * map keyed by BpelOptions.WPS_OPTIONS 
 *
 */
public class WpsExportOptionsBean {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2008, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	//	Export options 
	public static final int TOP_IMP_LIB = 1;
	public static final int MOD_LIB = 2;
	public static final int MOD_ONLY = 3;
	public static final int LIB_ONLY = 4;
	public static final int NONE = 5;
	
	private String wpsModuleOption;//BpelOptions.WPS_MODULE_OPTION	
	private String wpsExportModuleDirectory;//WPS_EXPORT_MODULE_DIRECTORY	
	private String wpsExportProjectInterchangeName;//WPS_EXPORT_PROJECT_INTERCHANGE	
	private List <ProjectWpsOptionsBean> selectedProjectWpsOptionsBeanList;
	private List <DeployableElementWpsOptionsBean> selectedDeployableElementWpsOptionBeanList;
	//The key of the map is the WBM project name and the value of the map is a collection
	//of the elements selected for wps export for that project
	private Map <Object,String> objectToProjectNameMap;
	private List<Object> extraReferencedObjects;
	
	private boolean isBestPracticePatternEnforced;	
	private boolean isProjectInterchangeEnabled;
	private boolean isProjectInterchangeTimeStampEnabled;
	private boolean wpsEnableDefaultEvents;//WPS_ENABLE_DEFAULT_EVENTS
	private boolean isPartialExport;
	private boolean isForTestDeploy = false;
	private boolean isForDeploy = false;
	private List<String> projectNameList;
	private Map<String, String>projectVersionMap;
	//key is the project name, while value is the list of referencing project names
	private Map<String,List> referenceProjectMap;
	
	private int exportOption;
	
	private Object logicalGroupMapping;
	
	public Map<String, List> getReferenceProjectMap() {
		return referenceProjectMap;
	}

	/**
	 * Get list of project names involved in this export session
	 * @return
	 */
	public List<String> getProjectList() {
		return projectNameList;
	}

	private void addProjectNameToProjectList(String projectName) {
		if(projectNameList == null){
			projectNameList = new LinkedList<String>();
		}
		if(!projectNameList.contains(projectName)){
			projectNameList.add(projectName);
		}
	}

	/**
	 * This constructor will be called by the wizard to populate the UI attributes
	 * @param wpsExportProjectInterchangeName
	 * @param wpsEnableDefaultEvents
	 * @param isBestPracticePatternEnforced
	 * @param isProjectInterchangeEnabled
	 * @param isProjectInterchangeTimeStampEnabled
	 */
	public WpsExportOptionsBean(String wpsExportProjectInterchangeName,
			String wpsExportModuleDirectory,
			int exportOption,
			boolean isPartialExport,
			boolean wpsEnableDefaultEvents,
			boolean isBestPracticePatternEnforced,
			boolean isProjectInterchangeEnabled,
			boolean isProjectInterchangeTimeStampEnabled){
		
		this.wpsExportModuleDirectory = wpsExportModuleDirectory;
		this.wpsExportProjectInterchangeName = wpsExportProjectInterchangeName;
		this.wpsEnableDefaultEvents = wpsEnableDefaultEvents;
		this.isBestPracticePatternEnforced = isBestPracticePatternEnforced;
		this.isProjectInterchangeEnabled = isProjectInterchangeEnabled;
		this.isProjectInterchangeTimeStampEnabled = isProjectInterchangeTimeStampEnabled;
		this.wpsEnableDefaultEvents = wpsEnableDefaultEvents;
		this.wpsModuleOption = BpelOptions.WPS_EXPORT_FILE_SYSTEM;
		this.exportOption = exportOption;
		this.isPartialExport = isPartialExport;
		
		//initialize the collections
		this.objectToProjectNameMap = new HashMap<Object,String>();
		this.projectVersionMap = new HashMap();
		this.selectedProjectWpsOptionsBeanList =  new ArrayList<ProjectWpsOptionsBean>();
		this.selectedDeployableElementWpsOptionBeanList = new ArrayList<DeployableElementWpsOptionsBean>();
		this.referenceProjectMap = new HashMap<String,List>();
		this.extraReferencedObjects = new Vector<Object>();
	}
	
	/**
	 * This constructor will be called to populate the engine
	 * @param wpsExportProjectInterchangeName
	 * @param wpsEnableDefaultEvents
	 * @param isBestPracticePatternEnforced
	 * @param isProjectInterchangeEnabled
	 * @param isProjectInterchangeTimeStampEnabled
	 * @param isForTestDeploy
	 */
	public WpsExportOptionsBean(String wpsExportProjectInterchangeName,
			String wpsExportModuleDirectory,
			int exportOption,
			boolean isPartialExport,
			boolean wpsEnableDefaultEvents,
			boolean isBestPracticePatternEnforced,
			boolean isProjectInterchangeEnabled,
			boolean isProjectInterchangeTimeStampEnabled, 
			boolean isForDeploy){
		
		this(wpsExportProjectInterchangeName, wpsExportModuleDirectory,exportOption, isPartialExport, 
				wpsEnableDefaultEvents, 
				isBestPracticePatternEnforced,
				isProjectInterchangeEnabled, 
				isProjectInterchangeTimeStampEnabled);
		
		this.isForDeploy = 	isForDeploy;
		}
	
	public boolean isBestPracticePatternEnforced() {
		return isBestPracticePatternEnforced;
	}
	
	public boolean isProjectInterchangeEnabled() {
		return isProjectInterchangeEnabled;
	}	
	
	public boolean isGenerateForDeployPatternEnabled() {
		return isForDeploy;
	}	

	public String getWpsExportModuleDirectory() {
		return wpsExportModuleDirectory;
	}
	
	public String getWpsExportProjectInterchangeName() {
		return wpsExportProjectInterchangeName;
	}	

	public boolean isProjectInterchangeTimeStampEnabled() {
		return isProjectInterchangeTimeStampEnabled;
	}

	public List<ProjectWpsOptionsBean> getSelectedProjectWpsOptionsBeanList() {	
		return selectedProjectWpsOptionsBeanList;
	}	
	
	public List <DeployableElementWpsOptionsBean>getSelectedDeployableElementWpsOptionsBeanList() {
		return selectedDeployableElementWpsOptionBeanList;
	}
	
    /**
     * This interface is for the UI to capture the UI project names for each BWM project 
     * @param wpsExportGeneralModuleName
     * @param wpsExportImplementationModuleName
     * @param wpsExportSharedModuleName
     */
	public void addProjectWpsOptionsBean(String wbmProjectName,
										List<String> dependantProjectNames,
										String wpsExportGeneralModuleName,
										String wpsExportImplementationModuleName,
										String wpsExportSharedModuleName,
										boolean isPredefinedProject){
		//if no module project name or library name specified, we won't create project names bean
		//populate the bean for cross project reference
//		if(exportOption == NONE){
//			return;
//		}
		if(selectedProjectWpsOptionsBeanList == null){
			selectedProjectWpsOptionsBeanList = new ArrayList<ProjectWpsOptionsBean>();
		}
		//initialize an instance of <code>ProjectWpsOptionsBean</code>
		ProjectWpsOptionsBean bean = new ProjectWpsOptionsBean();
		bean.setWbmProjectName(wbmProjectName);
		bean.setWpsExportGeneralModuleName(wpsExportGeneralModuleName);
		bean.setWpsExportImplementationModuleName(wpsExportImplementationModuleName);
		bean.setWpsExportSharedModuleName(wpsExportSharedModuleName);
		selectedProjectWpsOptionsBeanList.add(bean);
		//AZ 06/18/07 keep a list of project name reference		
		this.addProjectNameToProjectList(wbmProjectName);
		bean.setPredefinedProject(isPredefinedProject);
		bean.setDependantProjectNames(dependantProjectNames);
		this.populateReverseReferenceMap(wbmProjectName,dependantProjectNames);
	}
	
	   /**
     * This interface is for the UI to capture the UI project names for each BWM project 
     * @param wpsExportGeneralModuleName
     * @param wpsExportImplementationModuleName
     * @param wpsExportSharedModuleName
     */
	public void addProjectWpsOptionsBean(String wbmProjectName,
										List<String> dependantProjectNames,
										String wpsExportGeneralModuleName,
										String wpsExportImplementationModuleName,
										String wpsExportSharedModuleName,
										boolean isPredefinedProject, String versionName){

		if(selectedProjectWpsOptionsBeanList == null){
			selectedProjectWpsOptionsBeanList = new ArrayList<ProjectWpsOptionsBean>();
		}
		//initialize an instance of <code>ProjectWpsOptionsBean</code>
		ProjectWpsOptionsBean bean = new ProjectWpsOptionsBean();
		bean.setWbmProjectName(wbmProjectName);
		bean.setWpsExportGeneralModuleName(wpsExportGeneralModuleName);
		bean.setWpsExportImplementationModuleName(wpsExportImplementationModuleName);
		bean.setWpsExportSharedModuleName(wpsExportSharedModuleName);
		bean.setWpsProjectVersion(versionName);
		selectedProjectWpsOptionsBeanList.add(bean);
		//AZ 06/18/07 keep a list of project name reference		
		this.addProjectNameToProjectList(wbmProjectName);
		//NOTE .. currently for the export scenario should support at most one version for the Project. 	
		if(wbmProjectName != null) projectVersionMap.put(wbmProjectName, versionName);
		if(wpsExportGeneralModuleName != null) projectVersionMap.put(wpsExportGeneralModuleName, versionName);
		if(wpsExportImplementationModuleName != null) projectVersionMap.put(wpsExportImplementationModuleName, versionName);
		if(wpsExportSharedModuleName != null) projectVersionMap.put(wpsExportSharedModuleName, versionName);
	
		
		bean.setPredefinedProject(isPredefinedProject);
		bean.setDependantProjectNames(dependantProjectNames);
		this.populateReverseReferenceMap(wbmProjectName,dependantProjectNames);
	}

	
    /**
     * This interface is for the UI to capture the UI project names for each BWM project 
     * @param wpsExportGeneralModuleName
     * @param wpsExportImplementationModuleName
     * @param wpsExportSharedModuleName
     */
	public void addDeployableElementWpsOptionsBean(String blmID,
										List<String> dependentBLMID,
										String templateName,
										String wpsLibraryName,
										String wpsLibraryVersion,
										String wpsModuleName,
										String wpsModuleVersion){

		if(selectedDeployableElementWpsOptionBeanList == null){
			selectedDeployableElementWpsOptionBeanList = new ArrayList<DeployableElementWpsOptionsBean>();
		}
		//initialize an instance of <code>ProjectWpsOptionsBean</code>
		DeployableElementWpsOptionsBean bean = new DeployableElementWpsOptionsBean();
		bean.setBlmID(blmID);
		
		bean.setTemplateSuffix(templateName);
		bean.setModule(bean.new RuntimeProjectArtifact(wpsModuleName, wpsModuleVersion));
		bean.setLibrary(bean.new RuntimeProjectArtifact(wpsLibraryName, wpsLibraryVersion));
//		bean.setWpsLibraryName(wpsLibraryName);
//		bean.setWpsLibraryVersion(wpsLibraryVersion);
//		bean.setWpsModuleName(wpsModuleName);
//		bean.setWpsModuleVersion(wpsModuleVersion);
		bean.setDependendBLMID(dependentBLMID);
		
		selectedDeployableElementWpsOptionBeanList.add(bean);
		projectVersionMap.put(wpsLibraryName, wpsLibraryVersion);
		projectVersionMap.put(wpsModuleName, wpsModuleVersion);
	}

	
	private void populateReverseReferenceMap(String wbmProjectName, List<String> dependantProjectNames) {
		if(this.referenceProjectMap != null){
			if(dependantProjectNames != null && !dependantProjectNames.isEmpty()){
				for(Iterator<String> it = dependantProjectNames.iterator();it.hasNext();){
					String projectName = it.next();
					if(!this.referenceProjectMap.containsKey(projectName)){
						List<String> referenceList = new LinkedList<String>();
						referenceList.add(wbmProjectName);
						this.referenceProjectMap.put(projectName, referenceList);
					}else{
						List<String> referenceList =this.referenceProjectMap.get(projectName);
						if(referenceList != null){
							referenceList.add(wbmProjectName);
						}						
					}
				}
			}
		}
		
	}

	/**
	 * This API intends to be called by the WPS UI wizard to populate the project name to elements collection
	 * selected for WPS export
	 * @param object
	 * @param wbmProjectName
	 */
	public void addObjectProjectNameMap(Object selectedBOMElement,String wbmProjectName){
		if(objectToProjectNameMap == null){
			objectToProjectNameMap = new HashMap<Object,String>();
		}
		objectToProjectNameMap.put(selectedBOMElement, wbmProjectName);
	}
	
	public Map<Object,String> getObjectToProjectNameMap() {				
		return objectToProjectNameMap;
	}
	
	public String getWpsModuleOption() {
		return wpsModuleOption;
	}	
	
	public void setWpsModuleOption(String wpsModuleOption) {
		this.wpsModuleOption = wpsModuleOption;
	}	

	public boolean getWpsEnableDefaultEvents() {
		return wpsEnableDefaultEvents;
	}

	public int getExportOption() {
		return exportOption;
	}

	public boolean isPartialExport() {
		return isPartialExport;
	}
	
	public List<Object> getExtraReferencedObjects() {
		return extraReferencedObjects;
	}
	public void setExtraReferencedObjects(List<Object> extraReferencedObjects) {
		this.extraReferencedObjects = extraReferencedObjects;
	}

	public Map<String, String> getProjectVersionMap() {
		return projectVersionMap;
	}

	public Object getLogicalGroupMapping() {
		
		return logicalGroupMapping;
	}
	
	public void addLogicalGroupMapping(Object logicalGroupMapping){
		this.logicalGroupMapping = logicalGroupMapping;
	}

	/**
	 * Returns true if this bean is for a test deploy session
	 * @return
	 */
	public boolean isForTestDeploy() {
		return isForTestDeploy;
	}

	/**
	 * Returns true if the Bean was created for a Test deploy Session
	 * @param isForTestDeploy
	 */
	public void setForTestDeploy(boolean isForTestDeploy) {
		this.isForTestDeploy = isForTestDeploy;
	}


}
