package com.ibm.btools.blm.ie.imprt;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.List;

import org.eclipse.emf.common.util.EList;

/**
 * The Import Operation Interface. Each Model Import Operation should implement
 * this interface. There is an abstract class <code>AbstractImportOperation</code>
 * provided by the Import framework. The Model Import Operation can extend this 
 * abstract class and provide the implementation of method <code>readObjects
 * </code>
 */
public interface IImportOperation {
	
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	/**
	 * Set the name of the operation
	 * @param name
	 */
	public void setName(final String name);
	
	/**
	 * Get the name of the operation
	 * @return
	 */
	public String getName();
	
	/**
	 * Set the selected files for import
	 * @param selectedFiles
	 */
	public void setInputFiles(final List selectedFiles);

	/**
	 * Get the selected files for import
	 * @param selectedFiles
	 */
	public List getInputFiles();

	/**
	 * Sets the project name
	 * @param projectName
	 */
	public void setProjectName(final String projectName);
	
	/**
	 * Gets the project name
	 * @param projectName
	 */
	public String getProjectName();	
	
	/**
	 * Sets the import session
	 * @param session
	 */
	public void setImportSession(final ImportSession session);	

	/**
	 * Gets the import session
	 * @param session
	 */
	public ImportSession getImportSession();	

	/**
	 * Read the objects from file
	 * @return the result of the operation
	 */
	public abstract void readObjects();

	/**
	 * Return the importedObjects
	 * @return
	 */
	public abstract EList getObjects();
	
	/**
	 * Return the predefined types
	 * @return
	 */
	public abstract EList getPredefinedTypes();	
	
	/**
	 * Clean up the operation
	 */
	public void finish();
	
	/**
	 * Set true if the objects are root models.
	 * @param flag
	 */
	public void setIsRootModels(final boolean flag);
	
	/**
	 * Return true if the objects are root models
	 * @return
	 */
	public boolean isRootModels();
	
	/**
	 * Return true if the operation can be executed.
	 * @return
	 */
	public boolean canExecute();	
}
