package com.ibm.btools.blm.ie.imprt;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import org.eclipse.core.runtime.IProgressMonitor;

/**
 * Session for each Model Import.
 */
public class ImportSession {
	
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private ImportContext context;
	private ImportContext currentContext;
	private ImportResult result;
	private IImportQuery query;
	private IProgressMonitor monitor;
	private ImportOptions options;
	
	/**
	 * Sets the progress monitor for importing progress
	 * @param monitor
	 */
	public void setProgressMonitor(final IProgressMonitor monitor) {
		this.monitor = monitor;
	}
	
	/**
	 * Sets the import query
	 * @param query
	 */
	public void setImportQuery(final IImportQuery query) {		
		this.query = query;
	}

	/**
	 * Get current import options
	 * @param options
	 */
	public void setImportOption(final ImportOptions options) {
		this.options = options;
	}
	
	/**
	 * Set current import context
	 * @param context
	 */
	public void setContext(final ImportContext context) {
		this.context = context;
	}		
	
	/**
	 * Set current import context
	 * @param context
	 */
	public void setCurrentContext(final ImportContext context) {
		this.currentContext = context;
	}		

	/**
	 * Set import result
	 * @param result
	 */
	public void setImportResult(final ImportResult result) {
		this.result = result;
	}		

	/**
	 * Set Current import status
	 * @return options
	 */
	public ImportOptions getImportOptions() {
		return options;
	}

	/**
	 * Get progress monitor
	 * @return monitor
	 */
	public IProgressMonitor getProgressMonitor() {
		return monitor;
	}

	/**
	 * Get overall import callback
	 * @return query
	 */
	public IImportQuery getOverwriteQuery() {
		return query;
	}

	/**
	 * Get import context
	 * @return context
	 */
	public ImportContext getContext() {
		if (context == null) {
			context = new ImportContext();
		}
		return context;
	}
	
	/**
	 * Get import current context
	 * @return context
	 */
	public ImportContext getCurrentContext() {
		if (currentContext == null) {
			return context;
		} else {		
			return currentContext;
		}
	}
	
	/**
	 * Get import result
	 * @return result
	 */
	public ImportResult getImportResult() {
		if (result == null) {
			result = new ImportResult();
		}
		return result;
	}	
}
