package com.ibm.btools.blm.ie.imprt.engine;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.io.File;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.process.util.PECommandConstants;
import com.ibm.btools.blm.ie.imprt.IImportConstants;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportSession;
import com.ibm.btools.blm.ie.imprt.rule.externalModel.ExternalModelRule;
import com.ibm.btools.blm.ie.imprt.rule.externalModel.bo.ComplexTypeRule;
import com.ibm.btools.blm.ie.imprt.rule.externalModel.bo.ComplexTypeTemplateRule;
import com.ibm.btools.blm.ie.imprt.rule.externalModel.bo.ExternalSchemaRule;
import com.ibm.btools.blm.ie.imprt.rule.externalModel.service.ExternalServiceRule;
import com.ibm.btools.blm.ie.imprt.rule.externalModel.service.OperationRule;
import com.ibm.btools.blm.ie.imprt.rule.externalModel.service.ServiceInterfaceRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.BulkResourceDefinitionCategoryRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.BulkResourceDefinitionRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.BulkResourceRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.BusinessItemRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.BusinessItemSampleRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.BusinessRulesTaskRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.CalendarRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.CategoryCatalogRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.CategoryInstanceRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.CategoryRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.CategoryValueInstanceRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.DataCatalogRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.DatastoreRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.FormRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.HierarchyRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.HierarchyStructureDefinitionRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.HumanTaskRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.IndividualResourceDefinitionCategoryRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.IndividualResourceDefinitionRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.IndividualResourceRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.LocationDefinitionCategoryRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.LocationDefinitionRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.LocationRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.OrganizationCatalogRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.OrganizationDefinitionCategoryRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.OrganizationDefinitionRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.OrganizationUnitRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.ProcessCatalogRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.ProcessRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.ResourceCatalogRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.RoleRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.ServiceRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.SignalCategoryRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.SignalRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.TaskRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.PredefinedTypeUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildContainerNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationBOCatalogNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationCategoryInstanceNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationDataCatalogNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationExternalServiceCatalogNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationInlineXSDSchemaNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationOrganizationCatalogNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationProcessCatalogNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationProjectNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationResourceCatalogNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationWSDLPortTypeNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationXSDFileNode;
import com.ibm.btools.bom.command.compound.PredefTypesConst;
import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.bom.model.artifacts.InstanceSpecification;
import com.ibm.btools.bom.model.artifacts.Package;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.artifacts.Signal;
import com.ibm.btools.bom.model.externalmodels.ExternalSchema;
import com.ibm.btools.bom.model.externalmodels.ExternalService;
import com.ibm.btools.bom.model.externalmodels.ServiceInterface;
import com.ibm.btools.bom.model.models.ExternalModel;
import com.ibm.btools.bom.model.models.InformationModel;
import com.ibm.btools.bom.model.models.Model;
import com.ibm.btools.bom.model.models.ModelsFactory;
import com.ibm.btools.bom.model.models.OrganizationModel;
import com.ibm.btools.bom.model.models.ProcessModel;
import com.ibm.btools.bom.model.models.ResourceModel;
import com.ibm.btools.bom.model.organizationstructures.Location;
import com.ibm.btools.bom.model.organizationstructures.LocationType;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnit;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnitType;
import com.ibm.btools.bom.model.organizationstructures.Tree;
import com.ibm.btools.bom.model.organizationstructures.TreeStructure;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.Datastore;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.humantasks.Form;
import com.ibm.btools.bom.model.resources.BulkResource;
import com.ibm.btools.bom.model.resources.BulkResourceType;
import com.ibm.btools.bom.model.resources.IndividualResource;
import com.ibm.btools.bom.model.resources.IndividualResourceType;
import com.ibm.btools.bom.model.resources.Role;
import com.ibm.btools.bom.model.time.RecurringTimeIntervals;

/**
 * Processing the BOM object.
 */
public class BOMProcessor {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private String projectName;
	private ImportSession session;
	private EList models;
	private boolean isRootModels;

	private String defaultProcessCatalogName;
	private String defaultResourceCatalogName;
	private String defaultDataCatalogName;
	private String defaultOrganizationCatalogName;

	/**
	 * Return the name of the project
	 * @return
	 */
	public String getProjectName() {
		return projectName;
	}

	/**
	 * Set the project name
	 * @param string
	 */
	public void setProjectName(final String projectName) {
		this.projectName = projectName;
	}

	/**
	 * Return the models
	 * @return
	 */
	public EList getModels() {
		return models;
	}

	/**
	 * Return the Import session
	 * @return
	 */
	public ImportSession getImportSession() {
		return session;
	}

	/**
	 * Set the import objects
	 * @param list
	 */
	public void setModels(final EList list) {
		models = list;
	}

	/**
	 * Set the Import Session
	 * @param session
	 */
	public void setImportSession(final ImportSession session) {
		this.session = session;
	}

	/**
	 * Init method
	 */
	public void init() {
		LoggingUtil.traceEntry(this, "init");		
		PredefinedTypeUtil.setPredefinedTypeToContext(getProjectName(), getImportSession().getContext());
		PredefinedTypeUtil.registerPredefinedTypes(getImportSession());

		defaultProcessCatalogName = NavigatorUtil.getDefaultProcessCatalogName();
		defaultDataCatalogName = NavigatorUtil.getDefaultDataCatalogName();
		defaultResourceCatalogName = NavigatorUtil.getDefaultResourceCatalogName();
		defaultOrganizationCatalogName = NavigatorUtil.getDefaultOrganizationCatalogName();		



		LoggingUtil.traceExit(this, "init");		 
	}

	/**
	 * The list of the models are all BOM root object with no
	 * owningPackage. If the user pass in a non-root object or
	 * root object with owningPackage, the object won't be
	 * handled.
	 * 
	 * Currently this engine will just handle BOM object.
	 * It is possible to extend to be a more generice engine
	 * later.
	 */
	public void process() {
		LoggingUtil.traceEntry(this, "process");			
		if (models == null) {
			LoggingUtil.logError(getImportSession(), IeMessageKeys.GENERIC_ERROR, IImportResultMessages.MODE_IMPORT_MODEL_IS_NULL);
			LoggingUtil.traceExit(this, "process");			
			return;	
		}

		if (projectName == null) {
			LoggingUtil.logError(getImportSession(), IeMessageKeys.GENERIC_ERROR, IImportResultMessages.PROJECT_NAME_IS_NULL);
			LoggingUtil.traceExit(this, "process");	
			return;
		}		

		Iterator iterator = models.iterator();
		Object object = null;
		while (iterator.hasNext()) {			
			object = iterator.next();
			if (object instanceof EObject) {
				if (((EObject)object).eIsProxy()) {
					LoggingUtil.trace(this, "process", "The object is a proxy.");
					continue;
				}
			}

			if (object instanceof PackageableElement) {								
				if ((((PackageableElement) object).getOwningPackage()) != null) {
					// Only handle object with no owning package
					LoggingUtil.trace(this, "process", "The object " + ((PackageableElement) object).getName() + " has an owning package.");
					continue;
				}

				if (object instanceof Model) {
					// No parent navigator node.
					processModel((Model) object, null);
					LoggingUtil.trace(this, "process", "The object is a Model.");					
				} else if (object instanceof Package) {
					LoggingUtil.trace(this, "process", "The object is a Package.");
					// ignore					
				} else {
					// No parent navigator node.
					processElement((PackageableElement) object, null);
					LoggingUtil.trace(this, "process", "The object is a PackageableElement.");					
				}				
			}
			// else ignore.
			LoggingUtil.trace(this, "process", "The object " + object.getClass() + " is not a PackageableElement.");	
		}
		//	EM_BOMMgr.instance().updateReferences(projectName);
		//defect 165068
		cleanUpTempDirectory();
		LoggingUtil.traceExit(this, "process");
	}

	/**
	 * Clean up the temp directory if there is any
	 *
	 */
	private void cleanUpTempDirectory() {
		if(session != null && session.getContext().get(IImportConstants.ROOT_TEMP_DIR) != null){
			//clean up temp dir
			if(session.getContext().get(IImportConstants.ROOT_TEMP_DIR) instanceof String){
				String dir = (String)session.getContext().get(IImportConstants.ROOT_TEMP_DIR);
				try{
					File tempFile = new File(dir);
					deleteDir(tempFile);

				}catch(Exception e){
					return;
				}
			}			
		}
	}
   
	/**
	 * Util to recursively delete a directory
	 * @param dir
	 */
	private void deleteDir(File dir) {
		if (dir.isDirectory()) {
			String[] children = dir.list();
			for (int i=0; i<children.length; i++) {
				deleteDir(new File(dir, children[i]));			
			}
		}
		// The directory is now empty so delete it
		dir.delete();
	}
	
	/**
	 * Create the models in the navigator
	 * @param model
	 */
	private void processElement(final PackageableElement element, final AbstractChildContainerNode containerNode) {
		LoggingUtil.traceEntry(this, "processElement");
		if (element == null) {
			LoggingUtil.traceExit(this, "processElement");			
			return;
		}

		AbstractChildContainerNode node = containerNode;
		//AbstractChildContainerNode node = (AbstractChildContainerNode)session.getImportOptions().getAdditionalOption("SourceNode");
		String name = element.getName();

		if (name == null) {		
			LoggingUtil.traceExit(this, "processElement");	
			return;	
		}		
		NavigationProjectNode navProjectNode = NavigatorUtil.getProjectNode(projectName);

		if (element instanceof Signal) {
			// Handle both Category and type
			if (node == null) {	
				// If there is no parent navigator node, use default catalog node as parent			
				LoggingUtil.trace(this, "processElement", "The element " + element.getName() + " does not have a Model.");

				// Add this information model to the root information model				
				DataCatalogRule rule = new DataCatalogRule();
				rule.setImportSession(session);
				rule.setParent(navProjectNode);
				InformationModel model  = ModelsFactory.eINSTANCE.createInformationModel();
				model.setName(defaultDataCatalogName);

				rule.setSource(model);
				rule.setProjectName(projectName);
				rule.setDefault(true);
				updateTask(IeMessageKeys.TASK_CREATING_DATA_CATALOG);				
				rule.invoke();
				// The parent node is the newly created navigation node
				node = rule.getNewInformationModelNode();
				if (node == null) {
					LoggingUtil.traceExit(this, "processElement");
					return;
				}
			}				
			if (node instanceof NavigationDataCatalogNode) {				
				if (((Signal)element).getIsAbstract().booleanValue() == true) {
					LoggingUtil.trace(this, "processElement", "Create a Signal Category: " + element.getName() + ".");
					SignalCategoryRule rule = new SignalCategoryRule();
					rule.setImportSession(session);
					rule.setParent((NavigationDataCatalogNode) node);
					rule.setSource(element);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_NOTIFICATION_TEMPLATE);
					rule.invoke();
				} else {
					LoggingUtil.trace(this, "processElement", "Create a Signal: " + element.getName() + ".");					
					SignalRule rule = new SignalRule();
					rule.setImportSession(session);				
					rule.setParent((NavigationDataCatalogNode) node);
					rule.setSource(element);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_NOTIFICATION);
					rule.invoke();	
				}
			} else {
				LoggingUtil.trace(this, "processElement", "The element " + element.getClass() + " is not an information model.");
				LoggingUtil.traceExit(this, "processElement");
				return;
			}			
		}  else if (element instanceof OrganizationUnitType) {
			// Handle both Category and type
			if (node == null) {	
				// If there is no parent navigator node, use default catalog node as parent			
				LoggingUtil.trace(this, "processElement", "The element " + element.getName() + " does not have a Model.");
				// Add this organization model to the root information model
				OrganizationCatalogRule rule = new OrganizationCatalogRule();
				rule.setImportSession(session);
				rule.setParent(navProjectNode);
				OrganizationModel model  = ModelsFactory.eINSTANCE.createOrganizationModel();
				model.setName(defaultOrganizationCatalogName);

				rule.setSource(model);
				rule.setProjectName(projectName);
				rule.setDefault(true);		
				updateTask(IeMessageKeys.TASK_CREATING_ORGANIZATION_CATALOG);		
				rule.invoke();
				// The parent node is the newly created navigation node
				node = rule.getNewOrganizationModelNode();
				if (node == null) {
					LoggingUtil.traceExit(this, "processElement");					
					return;
				}
			}

			if (node instanceof NavigationOrganizationCatalogNode) {
				if (((OrganizationUnitType)element).getIsAbstract().booleanValue() == true) {
					LoggingUtil.trace(this, "processElement", "Create a Ogranization Definition Category: " + element.getName() + ".");					
					OrganizationDefinitionCategoryRule rule = new OrganizationDefinitionCategoryRule();
					rule.setImportSession(session);				
					rule.setParent((NavigationOrganizationCatalogNode) node);
					rule.setSource(element);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_ORGANIZATION_DEFINITION_TEMPLATE);
					rule.invoke();
				} else {
					LoggingUtil.trace(this, "processElement", "Create an Ogranization Definition: " + element.getName() + ".");					
					OrganizationDefinitionRule rule = new OrganizationDefinitionRule();
					rule.setImportSession(session);				
					rule.setParent((NavigationOrganizationCatalogNode) node);
					rule.setSource(element);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_ORGANIZATION_DEFINITION);
					rule.invoke();	
				}
			} else {
				LoggingUtil.trace(this, "processElement", "The element " + element.getClass() + " is not an organization model.");
				LoggingUtil.traceExit(this, "processElement");
				return;
			}
		} else if (element instanceof OrganizationUnit) {
			if (node == null) {	
				// If there is no parent navigator node, use default catalog node as parent	
				LoggingUtil.trace(this, "processElement", "The element " + element.getName() + " does not have a Model.");		
				// Add this organization model to the root information model
				OrganizationCatalogRule rule = new OrganizationCatalogRule();
				rule.setImportSession(session);
				rule.setParent(navProjectNode);
				OrganizationModel model  = ModelsFactory.eINSTANCE.createOrganizationModel();
				model.setName(defaultOrganizationCatalogName);

				rule.setSource(model);
				rule.setProjectName(projectName);
				rule.setDefault(true);		
				updateTask(IeMessageKeys.TASK_CREATING_ORGANIZATION_CATALOG);		
				rule.invoke();
				// The parent node is the newly created navigation node
				node = rule.getNewOrganizationModelNode();
				if (node == null) {
					LoggingUtil.traceExit(this, "processElement");					
					return;
				}
			}				
			if (node instanceof NavigationOrganizationCatalogNode) {
				LoggingUtil.trace(this, "processElement", "Create an Ogranization Unit: " + element.getName() + ".");
				OrganizationUnitRule rule = new OrganizationUnitRule();
				rule.setImportSession(session);				
				rule.setParent((NavigationOrganizationCatalogNode) node);
				rule.setSource(element);
				rule.setProjectName(projectName);
				updateTask(IeMessageKeys.TASK_CREATING_ORGANIZATION_UNIT);
				rule.invoke();
			}
			//TODO: 2005-01-20 added for the temprary category support, should be removed for 6.0
			else if (node instanceof NavigationCategoryInstanceNode){
				LoggingUtil.trace(this, "processElement", "Create an category value instance: " + element.getName() + ".");
				CategoryValueInstanceRule rule = new CategoryValueInstanceRule();
				rule.setImportSession(session);				
				rule.setParent((NavigationCategoryInstanceNode) node);
				rule.setSource(element);
				rule.setProjectName(projectName);
				updateTask(IeMessageKeys.TASK_CREATING_ORGANIZATION_UNIT);
				rule.invoke();

			}//<end todo>.
			else {
				LoggingUtil.trace(this, "processElement", "The element " + element.getClass() + " is not an organization model.");
				LoggingUtil.traceExit(this, "processElement");
				return;
			}
		} else if (element instanceof LocationType) {
			// Handle both Category and type
			if (node == null) {	
				// If there is no parent navigator node, use default catalog node as parent		
				LoggingUtil.trace(this, "processElement", "The element " + element.getName() + " does not have a Model.");	
				// Add this organization model to the root information model
				OrganizationCatalogRule rule = new OrganizationCatalogRule();
				rule.setImportSession(session);
				rule.setParent(navProjectNode);
				OrganizationModel model  = ModelsFactory.eINSTANCE.createOrganizationModel();
				model.setName(defaultOrganizationCatalogName);

				rule.setSource(model);
				rule.setProjectName(projectName);
				rule.setDefault(true);				
				updateTask(IeMessageKeys.TASK_CREATING_ORGANIZATION_CATALOG);
				rule.invoke();
				// The parent node is the newly created navigation node
				node = rule.getNewOrganizationModelNode();
				if (node == null) {
					LoggingUtil.traceExit(this, "processElement");					
					return;
				}
			}

			if (node instanceof NavigationOrganizationCatalogNode) {
				if (((LocationType)element).getIsAbstract().booleanValue() == true) {
					LoggingUtil.trace(this, "processElement", "Create a Location Definition Category: " + element.getName() + ".");					
					LocationDefinitionCategoryRule rule = new LocationDefinitionCategoryRule();
					rule.setImportSession(session);				
					rule.setParent((NavigationOrganizationCatalogNode) node);
					rule.setSource(element);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_LOCATION_DEFINITION_TEMPLATE);
					rule.invoke();
				} else {
					LoggingUtil.trace(this, "processElement", "Create a Location Definition: " + element.getName() + ".");					
					LocationDefinitionRule rule = new LocationDefinitionRule();
					rule.setImportSession(session);				
					rule.setParent((NavigationOrganizationCatalogNode) node);
					rule.setSource(element);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_LOCATION_DEFINITION);
					rule.invoke();	
				}
			} else {
				LoggingUtil.trace(this, "processElement", "The element " + element.getClass() + " is not an organization model.");
				LoggingUtil.traceExit(this, "processElement");
				return;
			}
		} else if (element instanceof Location) {
			if (node == null) {	
				// If there is no parent navigator node, use default catalog node as parent	
				LoggingUtil.trace(this, "processElement", "The element " + element.getName() + " does not have a Model.");		
				// Add this organization model to the root information model
				OrganizationCatalogRule rule = new OrganizationCatalogRule();
				rule.setImportSession(session);
				rule.setParent(navProjectNode);
				OrganizationModel model  = ModelsFactory.eINSTANCE.createOrganizationModel();
				model.setName(defaultOrganizationCatalogName);

				rule.setSource(model);
				rule.setProjectName(projectName);
				rule.setDefault(true);	
				updateTask(IeMessageKeys.TASK_CREATING_ORGANIZATION_CATALOG);			
				rule.invoke();
				// The parent node is the newly created navigation node
				node = rule.getNewOrganizationModelNode();
				if (node == null) {
					LoggingUtil.traceExit(this, "processElement");					
					return;
				}
			}
			if (node instanceof NavigationOrganizationCatalogNode) {
				LoggingUtil.trace(this, "processElement", "Create a Location: " + element.getName() + ".");				
				LocationRule rule = new LocationRule();
				rule.setImportSession(session);				
				rule.setParent((NavigationOrganizationCatalogNode) node);
				rule.setSource(element);
				rule.setProjectName(projectName);
				updateTask(IeMessageKeys.TASK_CREATING_LOCATION);
				rule.invoke();
			} else {
				LoggingUtil.trace(this, "processElement", "The element " + element.getClass() + " is not an organization model.");
				LoggingUtil.traceExit(this, "processElement");
				return;
			}
		} else if (element instanceof TreeStructure) {
			if (node == null) {	
				// If there is no parent navigator node, use default catalog node as parent	
				LoggingUtil.trace(this, "processElement", "The element " + element.getName() + " does not have a Model.");		
				// Add this organization model to the root information model
				OrganizationCatalogRule rule = new OrganizationCatalogRule();
				rule.setImportSession(session);
				rule.setParent(navProjectNode);
				OrganizationModel model  = ModelsFactory.eINSTANCE.createOrganizationModel();
				model.setName(defaultOrganizationCatalogName);

				rule.setSource(model);
				rule.setProjectName(projectName);
				rule.setDefault(true);		
				updateTask(IeMessageKeys.TASK_CREATING_ORGANIZATION_CATALOG);		
				rule.invoke();
				// The parent node is the newly created navigation node
				node = rule.getNewOrganizationModelNode();
				if (node == null) {
					LoggingUtil.traceExit(this, "processElement");					
					return;
				}
			}
			if (node instanceof NavigationOrganizationCatalogNode) {
				LoggingUtil.trace(this, "processElement", "Create a Hierarchy Structure Definition: " + element.getName() + ".");				
				HierarchyStructureDefinitionRule rule = new HierarchyStructureDefinitionRule();
				rule.setImportSession(session);				
				rule.setParent((NavigationOrganizationCatalogNode) node);
				rule.setSource(element);
				rule.setProjectName(projectName);
				updateTask(IeMessageKeys.TASK_CREATING_HIERARCHY_STRUCTURE_DEFINITION);
				rule.invoke();
			} else {
				LoggingUtil.trace(this, "processElement", "The element " + element.getClass() + " is not an organization model.");
				LoggingUtil.traceExit(this, "processElement");
				return;
			}			
		} else if (element instanceof Tree) {
			if (node == null) {	
				// If there is no parent navigator node, use default catalog node as parent
				LoggingUtil.trace(this, "processElement", "The element " + element.getName() + " does not have a Model.");			
				// Add this organization model to the root information model
				OrganizationCatalogRule rule = new OrganizationCatalogRule();
				rule.setImportSession(session);
				rule.setParent(navProjectNode);
				OrganizationModel model  = ModelsFactory.eINSTANCE.createOrganizationModel();
				model.setName(defaultOrganizationCatalogName);

				rule.setSource(model);
				rule.setProjectName(projectName);
				rule.setDefault(true);
				updateTask(IeMessageKeys.TASK_CREATING_ORGANIZATION_CATALOG);				
				rule.invoke();
				// The parent node is the newly created navigation node
				node = rule.getNewOrganizationModelNode();
				if (node == null) {
					LoggingUtil.traceExit(this, "processElement");					
					return;
				}
			}
			if (node instanceof NavigationOrganizationCatalogNode) {
				LoggingUtil.trace(this, "processElement", "Create a Hierarchy: " + element.getName() + ".");				
				HierarchyRule rule = new HierarchyRule();
				rule.setImportSession(session);				
				rule.setParent((NavigationOrganizationCatalogNode) node);
				rule.setSource(element);
				rule.setProjectName(projectName);
				updateTask(IeMessageKeys.TASK_CREATING_HIERARCHY);
				rule.invoke();
			} else {
				LoggingUtil.trace(this, "processElement", "The element " + element.getClass() + " is not an organization model.");
				LoggingUtil.traceExit(this, "processElement");	
				return;
			}
		} else if (element instanceof IndividualResourceType ) {
			// Handle both Category and type
			if (node == null) {	
				// If there is no parent navigator node, use default catalog node as parent
				LoggingUtil.trace(this, "processElement", "The element " + element.getName() + " does not have a Model.");			
				// Add this resource model to the root information model
				ResourceCatalogRule rule = new ResourceCatalogRule();
				rule.setImportSession(session);
				rule.setParent(navProjectNode);
				ResourceModel model  = ModelsFactory.eINSTANCE.createResourceModel();
				model.setName(defaultResourceCatalogName);

				rule.setSource(model);
				rule.setProjectName(projectName);
				rule.setDefault(true);		
				updateTask(IeMessageKeys.TASK_CREATING_RESOURCE_CATALOG);		
				rule.invoke();
				// The parent node is the newly created navigation node
				node = rule.getNewResourceModelNode();
				if (node == null) {
					LoggingUtil.traceExit(this, "processElement");					
					return;
				}
			}
			if (node instanceof NavigationResourceCatalogNode) {
				LoggingUtil.trace(this, "processElement", "Create an Individual Resource Definition Category: " + element.getName() + ".");				
				if (((IndividualResourceType)element).getIsAbstract().booleanValue() == true) {
					IndividualResourceDefinitionCategoryRule rule = new IndividualResourceDefinitionCategoryRule();
					rule.setImportSession(session);				
					rule.setParent((NavigationResourceCatalogNode) node);
					rule.setSource(element);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_INDIVIDUAL_RESOURCE_DEFINITION_TEMPLATE);
					rule.invoke();
				} else {
					LoggingUtil.trace(this, "processElement", "Create an Individual Resource Definition: " + element.getName() + ".");					
					IndividualResourceDefinitionRule rule = new IndividualResourceDefinitionRule();
					rule.setImportSession(session);				
					rule.setParent((NavigationResourceCatalogNode) node);
					rule.setSource(element);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_INDIVIDUAL_RESOURCE_DEFINITION);
					rule.invoke();	
				}
			} else {
				LoggingUtil.trace(this, "processElement", "The element " + element.getClass() + " is not a resource model.");
				LoggingUtil.traceExit(this, "processElement");
				return;
			}						
		} else if (element instanceof IndividualResource) {
			if (node == null) {	
				// If there is no parent navigator node, use default catalog node as parent
				LoggingUtil.trace(this, "processElement", "The element " + element.getName() + " does not have a Model.");			
				// Add this resource model to the root information model
				ResourceCatalogRule rule = new ResourceCatalogRule();
				rule.setImportSession(session);
				rule.setParent(navProjectNode);
				ResourceModel model  = ModelsFactory.eINSTANCE.createResourceModel();
				model.setName(defaultResourceCatalogName);

				rule.setSource(model);
				rule.setProjectName(projectName);
				rule.setDefault(true);		
				updateTask(IeMessageKeys.TASK_CREATING_RESOURCE_CATALOG);		
				rule.invoke();
				// The parent node is the newly created navigation node
				node = rule.getNewResourceModelNode();
				if (node == null) {					
					LoggingUtil.traceExit(this, "processElement");					
					return;
				}
			}
			if (node instanceof NavigationResourceCatalogNode) {
				LoggingUtil.trace(this, "processElement", "Create an Individual Resource: " + element.getName() + ".");				
				IndividualResourceRule rule = new IndividualResourceRule();
				rule.setImportSession(session);				
				rule.setParent((NavigationResourceCatalogNode) node);
				rule.setSource(element);
				rule.setProjectName(projectName);
				updateTask(IeMessageKeys.TASK_CREATING_INDIVIDUAL_RESOURCE);
				rule.invoke();
			} else {
				LoggingUtil.trace(this, "processElement", "The element " + element.getClass() + " is not a resource model.");
				LoggingUtil.traceExit(this, "processElement");
				return;
			}
		} else if (element instanceof BulkResourceType ) {
			// Handle both Category and type
			if (node == null) {	
				// If there is no parent navigator node, use default catalog node as parent
				LoggingUtil.trace(this, "processElement", "The element " + element.getName() + " does not have a Model.");			
				// Add this resource model to the root information model
				ResourceCatalogRule rule = new ResourceCatalogRule();
				rule.setImportSession(session);
				rule.setParent(navProjectNode);
				ResourceModel model  = ModelsFactory.eINSTANCE.createResourceModel();
				model.setName(defaultResourceCatalogName);

				rule.setSource(model);
				rule.setProjectName(projectName);
				rule.setDefault(true);		
				updateTask(IeMessageKeys.TASK_CREATING_RESOURCE_CATALOG);		
				rule.invoke();
				// The parent node is the newly created navigation node
				node = rule.getNewResourceModelNode();
				if (node == null) {
					LoggingUtil.traceExit(this, "processElement");					
					return;
				}
			}
			if (node instanceof NavigationResourceCatalogNode) {
				LoggingUtil.trace(this, "processElement", "Create a Bulk Resource Definition Category: " + element.getName() + ".");				
				if (((BulkResourceType)element).getIsAbstract().booleanValue() == true) {
					BulkResourceDefinitionCategoryRule rule = new BulkResourceDefinitionCategoryRule();
					rule.setImportSession(session);				
					rule.setParent((NavigationResourceCatalogNode) node);
					rule.setSource(element);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_BULK_RESOURCE_DEFINITION_TEMPLATE);
					rule.invoke();
				} else {
					LoggingUtil.trace(this, "processElement", "Create a Bulk Resource Definition: " + element.getName() + ".");					
					BulkResourceDefinitionRule rule = new BulkResourceDefinitionRule();
					rule.setImportSession(session);				
					rule.setParent((NavigationResourceCatalogNode) node);
					rule.setSource(element);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_BULK_RESOURCE_DEFINITION);
					rule.invoke();	
				}
			} else {
				LoggingUtil.trace(this, "processElement", "The element " + element.getClass() + " is not a resource model.");
				LoggingUtil.traceExit(this, "processElement");
				return;
			}			
		} else if (element instanceof BulkResource) {
			if (node == null) {	
				// If there is no parent navigator node, use default catalog node as parent
				LoggingUtil.trace(this, "processElement", "The element " + element.getName() + " does not have a Model.");			
				// Add this resource model to the root information model
				ResourceCatalogRule rule = new ResourceCatalogRule();
				rule.setImportSession(session);
				rule.setParent(navProjectNode);
				ResourceModel model  = ModelsFactory.eINSTANCE.createResourceModel();
				model.setName(defaultResourceCatalogName);

				rule.setSource(model);
				rule.setProjectName(projectName);
				rule.setDefault(true);	
				updateTask(IeMessageKeys.TASK_CREATING_RESOURCE_CATALOG);			
				rule.invoke();
				// The parent node is the newly created navigation node
				node = rule.getNewResourceModelNode();
				if (node == null) {
					LoggingUtil.traceExit(this, "processElement");					
					return;
				}
			}
			if (node instanceof NavigationResourceCatalogNode) {
				LoggingUtil.trace(this, "processElement", "Create a Bulk Resource: " + element.getName() + ".");				
				BulkResourceRule rule = new BulkResourceRule();
				rule.setImportSession(session);				
				rule.setParent((NavigationResourceCatalogNode) node);
				rule.setSource(element);
				rule.setProjectName(projectName);
				updateTask(IeMessageKeys.TASK_CREATING_BULK_RESOURCE);
				rule.invoke();
			} else {
				LoggingUtil.trace(this, "processElement", "The element " + element.getClass() + " is not a resource model.");
				LoggingUtil.traceExit(this, "processElement");
				return;
			}
		} else if (element instanceof Role) {
			if (node == null) {	
				// If there is no parent navigator node, use default catalog node as parent
				LoggingUtil.trace(this, "processElement", "The element " + element.getName() + " does not have a Model.");			
				// Add this resource model to the root information model
				ResourceCatalogRule rule = new ResourceCatalogRule();
				rule.setImportSession(session);
				rule.setParent(navProjectNode);
				ResourceModel model  = ModelsFactory.eINSTANCE.createResourceModel();
				model.setName(defaultResourceCatalogName);

				rule.setSource(model);
				rule.setProjectName(projectName);
				rule.setDefault(true);	
				updateTask(IeMessageKeys.TASK_CREATING_RESOURCE_CATALOG);			
				rule.invoke();
				// The parent node is the newly created navigation node
				node = rule.getNewResourceModelNode();
				if (node == null) {
					LoggingUtil.traceExit(this, "processElement");					
					return;
				}
			}
			if (node instanceof NavigationResourceCatalogNode) {
				LoggingUtil.trace(this, "processElement", "Create a Role: " + element.getName() + ".");				
				RoleRule rule = new RoleRule();
				rule.setImportSession(session);				
				rule.setParent((NavigationResourceCatalogNode) node);
				rule.setSource(element);
				rule.setProjectName(projectName);
				updateTask(IeMessageKeys.TASK_CREATING_ROLE);
				rule.invoke();
			} else {
				LoggingUtil.trace(this, "processElement", "The element " + element.getClass() + " is not a resource model.");
				LoggingUtil.traceExit(this, "processElement");
				return;
			}
		} else if (element instanceof RecurringTimeIntervals) {
			if (node == null) {	
				// If there is no parent navigator node, use default catalog node as parent
				LoggingUtil.trace(this, "processElement", "The element " + element.getName() + " does not have a Model.");			
				// Add this resource model to the root information model
				ResourceCatalogRule rule = new ResourceCatalogRule();
				rule.setImportSession(session);
				rule.setParent(navProjectNode);
				ResourceModel model  = ModelsFactory.eINSTANCE.createResourceModel();
				model.setName(defaultResourceCatalogName);

				rule.setSource(model);
				rule.setProjectName(projectName);
				rule.setDefault(true);		
				updateTask(IeMessageKeys.TASK_CREATING_RESOURCE_CATALOG);		
				rule.invoke();
				// The parent node is the newly created navigation node
				node = rule.getNewResourceModelNode();
				if (node == null) {
					LoggingUtil.traceExit(this, "processElement");					
					return;
				}
			}
			if (node instanceof NavigationResourceCatalogNode) {
				LoggingUtil.trace(this, "processElement", "Create a Calendar: " + element.getName() + ".");				
				CalendarRule rule = new CalendarRule();
				rule.setImportSession(session);				
				rule.setParent((NavigationResourceCatalogNode) node);
				rule.setSource(element);
				rule.setProjectName(projectName);
				updateTask(IeMessageKeys.TASK_CREATING_TIMETABLE);
				rule.invoke();
			} else {
				LoggingUtil.trace(this, "processElement", "The element " + element.getClass() + " is not a resource model.");
				LoggingUtil.traceExit(this, "processElement");
				return;
			}
		} else if (element instanceof Datastore) {
			if (node == null) {	
				// If there is no parent navigator node, use default catalog node as parent
				LoggingUtil.trace(this, "processElement", "The element " + element.getName() + " does not have a Model.");			
				// Add this process model to the root information model
				ProcessCatalogRule rule = new ProcessCatalogRule();
				rule.setImportSession(session);
				rule.setParent(navProjectNode);
				ProcessModel model  = ModelsFactory.eINSTANCE.createProcessModel();
				model.setName(defaultProcessCatalogName);

				rule.setSource(model);
				rule.setProjectName(projectName);
				rule.setDefault(true);	
				updateTask(IeMessageKeys.TASK_CREATING_PROCESS_CATALOG);			
				rule.invoke();
				// The parent node is the newly created navigation node
				node = rule.getNewProcessModelNode();
				if (node == null) {
					LoggingUtil.traceExit(this, "processElement");					
					return;
				}
			}
			if (node instanceof NavigationProcessCatalogNode) {
				LoggingUtil.trace(this, "processElement", "Create a Datastore: " + element.getName() + ".");				
				DatastoreRule rule = new DatastoreRule();
				rule.setImportSession(session);				
				rule.setParent((NavigationProcessCatalogNode) node);
				rule.setSource(element);
				rule.setProjectName(projectName);
				updateTask(IeMessageKeys.TASK_CREATING_REPOSITORY);
				rule.invoke();
			} else {
				LoggingUtil.trace(this, "processElement", "The element " + element.getClass() + " is not a process model.");
				LoggingUtil.traceExit(this, "processElement");
				return;
			}
		} else if (element instanceof Activity) {
			// Process, Task and Service			
			if (node == null) {	
				// If there is no parent navigator node, use default catalog node as parent
				LoggingUtil.trace(this, "processElement", "The element " + element.getName() + " does not have a Model.");			
				// Add this process model to the root information model
				ProcessCatalogRule rule = new ProcessCatalogRule();
				rule.setImportSession(session);
				rule.setParent(navProjectNode);
				ProcessModel model  = ModelsFactory.eINSTANCE.createProcessModel();
				model.setName(defaultProcessCatalogName);

				rule.setSource(model);
				rule.setProjectName(projectName);
				rule.setDefault(true);	
				updateTask(IeMessageKeys.TASK_CREATING_PROCESS_CATALOG);			
				rule.invoke();
				// The parent node is the newly created navigation node
				node = rule.getNewProcessModelNode();
				if (node == null) {
					LoggingUtil.traceExit(this, "processElement");
					return;
				}
			}
			String aspect = null;
			StructuredActivityNode implementation = ((Activity)element).getImplementation();
			if (implementation != null) {
				aspect = implementation.getAspect();
			}

			if (aspect != null) {

				if (PECommandConstants.PROCESS_ASPECT.compareTo(aspect) == 0) {
					LoggingUtil.trace(this, "processElement", "Create a Process: " + element.getName() + ".");					
					ProcessRule rule = new ProcessRule();
					rule.setImportSession(session);				
					rule.setParent((NavigationProcessCatalogNode) node);		
					rule.setSource(element);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_PROCESS);
					rule.invoke();
				} else if (PECommandConstants.TASK_ASPECT.compareTo(aspect) == 0) {
					LoggingUtil.trace(this, "processElement", "Create a Task: " + element.getName() + ".");					
					TaskRule rule = new TaskRule();
					rule.setImportSession(session);				
					rule.setParent((NavigationProcessCatalogNode) node);		
					rule.setSource(element);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_TASK);
					rule.invoke();
				} else if (PECommandConstants.HUMAN_TASK_ASPECT.compareTo(aspect) == 0) {
					LoggingUtil.trace(this, "processElement", "Create a Task: " + element.getName() + ".");					
					HumanTaskRule rule = new HumanTaskRule();
					rule.setImportSession(session);				
					rule.setParent((NavigationProcessCatalogNode) node);		
					rule.setSource(element);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_HUMAN_TASK);
					rule.invoke();
				} else if (PECommandConstants.BUSINESS_RULE_TASK_ASPECT.compareTo(aspect) == 0) {
					LoggingUtil.trace(this, "processElement", "Create a Task: " + element.getName() + ".");					
					BusinessRulesTaskRule rule = new BusinessRulesTaskRule();
					rule.setImportSession(session);				
					rule.setParent((NavigationProcessCatalogNode) node);		
					rule.setSource(element);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_BUSINESS_RULES_TASK);
					rule.invoke();
				} else if (PECommandConstants.SERVICE_ASPECT.compareTo(aspect) == 0) {
					LoggingUtil.trace(this, "processElement", "Create a Service: " + element.getName() + ".");					
					ServiceRule rule = new ServiceRule();
					rule.setImportSession(session);				
					rule.setParent((NavigationProcessCatalogNode) node);		
					rule.setSource(element);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_SERVICE);
					rule.invoke();
				}else if(BOMUtil.ASPECT_SERVICE_OPERATION.compareTo(aspect) == 0){
					//Handle the case for operation of WSDL portType
					LoggingUtil.trace(this, "processElement", "Create an Operation: " + element.getName() + ".");					
					OperationRule rule = new OperationRule();
					rule.setImportSession(session);				
					rule.setParent((NavigationWSDLPortTypeNode) node);		
					rule.setSource(element);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_SERVICE_OPERATION,new String[]{element.getName()});
					rule.invoke();
				}
			}
		} else if (element instanceof Class) {
			//Handle complex type for the external schema
			//TODO Is there a case of abstract class to handle??
			if(BOMUtil.isSchemaType(element)||
					BOMUtil.isPrivateSchemaType(element) ||
					BOMUtil.isFileAttachment(element)||
					BOMUtil.isInlineComplexType(element)){
				//For external schema type and inline schema type
				if (((Class)element).getIsAbstract().booleanValue()) {
					//We need to support the business item template
					LoggingUtil.trace(this, "processElement", "Create a complex type template: " + element.getName() + ".");					
					ComplexTypeTemplateRule rule = new ComplexTypeTemplateRule();
					rule.setImportSession(session);				
					rule.setParent(node);
					rule.setSource(element);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_COMPLEX_TYPE_TEMPLATE,new String[]{element.getName()});
					rule.invoke();
				}
				else if(node instanceof NavigationXSDFileNode ||
						node instanceof NavigationInlineXSDSchemaNode){
					LoggingUtil.trace(this, "processElement", "Create a Complex type: " + element.getName() + ".");					
					ComplexTypeRule rule = new ComplexTypeRule();
					rule.setImportSession(session);				
					rule.setParent(node);
					rule.setSource(element);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_COMPLEX_TYPE,new String[]{element.getName()});
					rule.invoke();			
				}
				// Handle both Category and type	
			}else{
				if (node == null) {	
					// If there is no parent navigator node, use default catalog node as parent
					LoggingUtil.trace(this, "processElement", "The element " + element.getName() + " does not have a Model.");			
					// Add this information model to the root information model
					DataCatalogRule rule = new DataCatalogRule();
					rule.setImportSession(session);
					rule.setParent(navProjectNode);
					InformationModel model  = ModelsFactory.eINSTANCE.createInformationModel();
					model.setName(defaultDataCatalogName);

					rule.setSource(model);
					rule.setProjectName(projectName);
					rule.setDefault(true);	
					updateTask(IeMessageKeys.TASK_CREATING_DATA_CATALOG);			
					rule.invoke();
					// The parent node is the newly created navigation node
					node = rule.getNewInformationModelNode();
					if (node == null) {
						LoggingUtil.traceExit(this, "processElement");					
						return;
					}
				}
				if (node instanceof NavigationDataCatalogNode) {
					if (((Class)element).getIsAbstract().booleanValue() == true) {
						LoggingUtil.trace(this, "processElement", "Create a Category: " + element.getName() + ".");					
						CategoryRule rule = new CategoryRule();
						rule.setImportSession(session);				
						rule.setParent((NavigationDataCatalogNode) node);
						rule.setSource(element);
						rule.setProjectName(projectName);
						updateTask(IeMessageKeys.TASK_CREATING_BUSINESS_ITEM_TEMPLATE);
						rule.invoke();
					} else {
						LoggingUtil.trace(this, "processElement", "Create a Business Item: " + element.getName() + ".");					
						BusinessItemRule rule = new BusinessItemRule();
						rule.setImportSession(session);				
						rule.setParent((NavigationDataCatalogNode) node);
						rule.setSource(element);
						rule.setProjectName(projectName);
						updateTask(IeMessageKeys.TASK_CREATING_BUSINESS_ITEM);
						rule.invoke();	
					}
				}
				else {
					LoggingUtil.trace(this, "processElement", "The element " + element.getClass() + " is not an information model.");
					LoggingUtil.traceExit(this, "processElement");
					return;
				}	
			}
		}else if (element instanceof InstanceSpecification ) {
			if (node == null) {	
				// If there is no parent navigator node, use default catalog node as parent
				LoggingUtil.trace(this, "processElement", "The element " + element.getName() + " does not have a Model.");			
				// Add this information model to the root information model
				DataCatalogRule rule = new DataCatalogRule();
				rule.setImportSession(session);
				rule.setParent(navProjectNode);
				InformationModel model  = ModelsFactory.eINSTANCE.createInformationModel();
				model.setName(defaultDataCatalogName);

				rule.setSource(model);
				rule.setProjectName(projectName);
				rule.setDefault(true);	
				updateTask(IeMessageKeys.TASK_CREATING_DATA_CATALOG);			
				rule.invoke();
				// The parent node is the newly created navigation node
				node = rule.getNewInformationModelNode();
				if (node == null) {
					LoggingUtil.traceExit(this, "processElement");					
					return;
				}
			}
			if (node instanceof NavigationDataCatalogNode) {
				LoggingUtil.trace(this, "processElement", "Create a Business item Sample: " + element.getName() + ".");				
				BusinessItemSampleRule rule = new BusinessItemSampleRule();
				rule.setImportSession(session);				
				rule.setParent((NavigationDataCatalogNode) node);
				rule.setSource(element);
				rule.setProjectName(projectName);
				updateTask(IeMessageKeys.TASK_CREATING_BUSINESS_ITEM_INSTANCE);
				rule.invoke();
			} else {
				LoggingUtil.trace(this, "processElement", "The element " + element.getClass() + " is not an information model.");
				LoggingUtil.traceExit(this, "processElement");
				return;
			}
		}
		else if (element instanceof Form ) {
			if (node == null) {	
				Object nodeObject = session.getImportOptions().getAdditionalOption(NavigatorUtil.SOUREC_FORM_PROCESS_CATALOG_NODE);
				if(nodeObject instanceof NavigationProcessCatalogNode){
					node = (NavigationProcessCatalogNode)nodeObject;
				}else{	
					// If there is no parent navigator node, use default catalog node as parent
					LoggingUtil.trace(this, "processElement", "The element " + element.getName() + " does not have a Model.");			
					// Add this process model to the root information model
					ProcessCatalogRule rule = new ProcessCatalogRule();
					rule.setImportSession(session);
					rule.setParent(navProjectNode);
					ProcessModel model  = ModelsFactory.eINSTANCE.createProcessModel();
					model.setName(defaultProcessCatalogName);

					rule.setSource(model);
					rule.setProjectName(projectName);
					rule.setDefault(true);	
					updateTask(IeMessageKeys.TASK_CREATING_PROCESS_CATALOG);			
					rule.invoke();
					// The parent node is the newly created navigation node
					node = rule.getNewProcessModelNode();
				}
				if (node == null) {
					LoggingUtil.traceExit(this, "processElement");
					return;
				}
			}
			if (node instanceof NavigationProcessCatalogNode) {
				LoggingUtil.trace(this, "processElement", "Create a Form: " + element.getName() + ".");				
				FormRule rule = new FormRule();
				rule.setImportSession(session);				
				rule.setParent((NavigationProcessCatalogNode) node);
				rule.setSource(element);
				rule.setProjectName(projectName);
				updateTask(IeMessageKeys.TASK_CREATING_FORM);
				rule.invoke();
			} else {
				LoggingUtil.trace(this, "processElement", "The element " + element.getClass() + " is not a process model.");
				LoggingUtil.traceExit(this, "processElement");
				return;
			}
		}
	}	

	/**
	 * Create the models in the navigator
	 * @param model
	 */
	private void processModel(final Model model, final AbstractChildContainerNode node) {
		LoggingUtil.traceEntry(this, "processModel");		
		// Determine it is a root model or not.
		boolean root = false;

		if (model == null) {
			return;
		}

		String name = model.getName();

		if (name == null) {
			LoggingUtil.traceExit(this, "processModel");			
			return;		
		}

		EList models = model.getOwnedMember();
		List sortedModels = BOMUtil.sortTemplateModels(models);
		Iterator iterator = sortedModels.iterator();	
		NavigationProjectNode navProjectNode = NavigatorUtil.getProjectNode(projectName);

		Object object = null;	
		AbstractChildContainerNode parentNode = null;
		if (model instanceof ProcessModel) {	
			if (isRootModels() && IImportFramework.ROOT_PROC_MODEL_NAME.compareTo(name) == 0) {				
				// Ignore it and process it child
				LoggingUtil.trace(this, "processModel", "This is a root process model.");
				root = true;
				parentNode = null;
			} else {
				// Add this process model to the root process model
				ProcessCatalogRule rule = new ProcessCatalogRule();
				rule.setImportSession(session);
				if (node == null) {						
					rule.setParent(navProjectNode);
				} else {
					rule.setParent(node);
				}
				rule.setSource(model);
				rule.setProjectName(projectName);
				//first step may create a default model
				if(model.getUid() != null && model.getUid().equals(PredefTypesConst.PREDEF_PROCESSES_FUID)){
					rule.setDefault(true);
				}
				updateTask(IeMessageKeys.TASK_CREATING_PROCESS_CATALOG);
				rule.invoke();
				// The parent node is the newly created navigation node
				parentNode = rule.getNewProcessModelNode();
				if (parentNode == null) {
					LoggingUtil.traceExit(this, "processModel");
					return;
				}
				LoggingUtil.trace(this, "processModel", "Add the process model: " + parentNode.getLabel());				
			}		
		}
		else if(model instanceof ExternalService){
			//Add this process model to the root process model
			ExternalServiceRule rule = new ExternalServiceRule();
			rule.setImportSession(session);
			if (node == null) {	
				// If there is no parent navigator node, use the default catalog node as parent			
				Object nodeObject = session.getImportOptions().getAdditionalOption(NavigatorUtil.SOUREC_EXTERNAL_SERVICE_CATALOG_NODE);
				if(nodeObject instanceof NavigationExternalServiceCatalogNode){
					parentNode = (NavigationExternalServiceCatalogNode)nodeObject;
				}else{
					//look up the default node
					parentNode = NavigatorUtil.getDefaultCatalogForExternalService(projectName);
				}
				rule.setParent(parentNode);					
			} else {
				rule.setParent(node);
			}
			rule.setSource((ExternalService)model);
			rule.setProjectName(projectName);
			//first step may create a default model
			if(model.getUid() != null && model.getUid().equals(PredefTypesConst.PREDEF_PROCESSES_FUID)){
				rule.setDefault(true);
			}
			updateTask(IeMessageKeys.TASK_CREATING_EXTERNAL_SERVICE,new String[]{model.getName()});
			rule.invoke();
			// The parent node is the newly created navigation node
			parentNode = rule.getNewWSDLFileNode();
			if (parentNode == null) {
				LoggingUtil.traceExit(this, "externalService");
				return;
			}
			LoggingUtil.trace(this, "externalModel", "Add the external Service: " + parentNode.getLabel());			
		}else if(model instanceof ServiceInterface){			
			//Add this process model to the root process model
			ServiceInterfaceRule rule = new ServiceInterfaceRule();
			rule.setImportSession(session);
			if (node == null) {	
				return;
			} else {
				rule.setParent(node);
			}
			rule.setSource((ServiceInterface)model);
			rule.setProjectName(projectName);
			//first step may create a default model
			if(model.getUid() != null && model.getUid().equals(PredefTypesConst.PREDEF_PROCESSES_FUID)){
				rule.setDefault(true);
			}
			updateTask(IeMessageKeys.TASK_CREATING_SERVICE_INTERFACE,new String[]{model.getName()});
			rule.invoke();
			// The parent node is the newly created navigation node
			parentNode = rule.getNewServiceInterfaceNode();
			if (parentNode == null) {
				LoggingUtil.traceExit(this, "ServiceInterface");
				return;
			}
			LoggingUtil.trace(this, "serviceInterface", "Add the service interface: " + parentNode.getLabel());				
		}

		else if (model instanceof InformationModel) {
			if (isRootModels() && IImportFramework.ROOT_INFO_MODEL_NAME.compareTo(name) == 0) {				
				// Ignore it and process it child
				LoggingUtil.trace(this, "processModel", "This is a root information model.");				
				root = true;
				parentNode = null;
			} else {
				// Add this information model to the root information model
				DataCatalogRule rule = new DataCatalogRule();
				rule.setImportSession(session);
				if (node == null) {	
					// If there is no parent navigator node, use project node as parent			
					rule.setParent(navProjectNode);
				} else {
					rule.setParent(node);
				}
				rule.setSource((InformationModel)model);
				rule.setProjectName(projectName);
				updateTask(IeMessageKeys.TASK_CREATING_DATA_CATALOG);
				rule.invoke();
				// The parent node is the newly created navigation node
				parentNode = rule.getNewInformationModelNode();
				if (parentNode == null) {
					LoggingUtil.traceExit(this, "processModel");					
					return;
				}
				LoggingUtil.trace(this, "processModel", "Add the information model: " + parentNode.getLabel());					
			}
		}else if(model instanceof ExternalSchema){
			//Added to support external schema, if the external schema has owning package, it's inline schema
			ExternalSchemaRule rule = new ExternalSchemaRule();
			rule.setImportSession(session);
			if (node == null) {	
				Object nodeObject = session.getImportOptions().getAdditionalOption(NavigatorUtil.SOUREC_EXTERNAL_BO_CATALOG_NODE);
				AbstractChildContainerNode selectedNode = null;
				if(nodeObject instanceof NavigationBOCatalogNode){
					selectedNode = (NavigationBOCatalogNode)nodeObject;
				}else{
					//look up the default BO Catalog
					selectedNode = NavigatorUtil.getDefaultCatalogForExternalSchema(projectName);
				}					
				rule.setParent(selectedNode);				
			} else {
				//use the user selected BO catalog as parent			
				rule.setParent(node);
			}
			rule.setSource((ExternalSchema)model);
			rule.setProjectName(projectName);

			updateTask(IeMessageKeys.TASK_CREATING_EXTERNAL_SCHEMA,new String[]{model.getName()});
			rule.invoke();
			// The parent node is the newly created navigation node
			parentNode = rule.getNewExternalSchemaNode();
			if (parentNode == null) {
				LoggingUtil.traceExit(this, "externalSchema");
				return;
			}
			LoggingUtil.trace(this, "externalModel", "Add the external Service: " + parentNode.getLabel());				

		}else if(model instanceof ExternalModel){
			if (isRootModels() && IImportFramework.ROOT_EXTERNAL_MODEL_NAME.compareTo(name) == 0){				
				// Ignore it and process it child
				LoggingUtil.trace(this, "externalModel", "This is a root external model.");				
				root = true;
				parentNode = null;			
			}else if((BOMUtil.isModelOwnedByRootModel(model)&& IImportFramework.DEFAULT_BUSINESS_SERVICES.compareTo(name)==0)){
				LoggingUtil.trace(this, "externalModel", "This is the default external model for business services.");				
				parentNode = null;			
			}else if((BOMUtil.isModelOwnedByRootModel(model) && IImportFramework.DEFAULT_BUSINESS_SERVICE_OBJECTS.compareTo(name)==0)){
				LoggingUtil.trace(this, "externalModel", "This is the default external model for business service objects.");				
				parentNode = null;			
			}else{
				ExternalModelRule rule = new ExternalModelRule();
				rule.setImportSession(session);
				if (node == null) {	
					Object nodeObjectBO = session.getImportOptions().getAdditionalOption(NavigatorUtil.SOUREC_EXTERNAL_BO_CATALOG_NODE);
					Object nodeObjectSVC = session.getImportOptions().getAdditionalOption(NavigatorUtil.SOUREC_EXTERNAL_SERVICE_CATALOG_NODE);
					//Retrieve the root model node for BO root or service root based on flag
					AbstractChildContainerNode selectedNode = null;
					if(nodeObjectBO instanceof AbstractChildContainerNode && nodeObjectSVC instanceof AbstractChildContainerNode){
						if(NavigatorUtil.isBOCatalogNode((ExternalModel)model,getImportSession())){
							selectedNode = (AbstractChildContainerNode)nodeObjectBO;
						}else{
							selectedNode = (AbstractChildContainerNode)nodeObjectSVC;
						}
					}else{
						if(NavigatorUtil.isRootCatalogCustomCatalog((ExternalModel)model)){
							if(NavigatorUtil.isBOCatalogNode((ExternalModel)model,getImportSession())){
								selectedNode = NavigatorUtil.getRootBONode(projectName,session);
							}else{
								selectedNode = NavigatorUtil.getRootServiceNode(projectName,session);
							}
						}else{
							if(NavigatorUtil.isBOCatalogNode((ExternalModel)model,getImportSession())){
								//look up the default BO Catalog
								selectedNode = NavigatorUtil.getDefaultCatalogForExternalSchema(projectName);					
							}else{
								selectedNode = NavigatorUtil.getDefaultCatalogForExternalService(projectName);
							}	
						}
					}
					rule.setParent(selectedNode);				
				} else {
					//use the user selected BO catalog as parent			
					rule.setParent(node);
				}
				rule.setSource((ExternalModel)model);
				rule.setProjectName(projectName);

				updateTask(IeMessageKeys.TASK_CREATING_EXTERNAL_MODEL,new String[]{model.getName()});
				rule.invoke();
				// The parent node is the newly created navigation node
				parentNode = rule.getNewExternalModelNode();
				if (parentNode == null) {
					LoggingUtil.traceExit(this, "externalModel");
					return;
				}
				LoggingUtil.trace(this, "externalModel", "Add the external model catalog: " + parentNode.getLabel());				
			}
		}

		else if (model instanceof OrganizationModel) {
			if (isRootModels() && IImportFramework.ROOT_ORG_MODEL_NAME.compareTo(name) == 0 ) {				
				// Ignore it and process it child
				LoggingUtil.trace(this, "processModel", "This is a root organization model.");				
				root = true;
				parentNode = null;
			}
			//TODO: 2005-01-20 added to support temprary category solution. should be removed in 6.0
			else if(isRootModels() &&  IImportFramework.ROOT_CATEGORY_MODEL_NAME.compareTo(name) == 0){
				// Ignore it and process it child
				LoggingUtil.trace(this, "processModel", "This is a root category model.");				
				root = true;
				parentNode = null;				 
			}//<end todo>.
			else {
				//TODO:  2005-01-20 added to support temperary category solution. should be removed in 6.0
				//check if it is category model
				if(model.getAspect() != null && model.getAspect().compareTo(PredefTypesConst.PREDEF_CATEGORY_ORG_MODEL_ASPECT) == 0 &&
						model.getUid() != null && model.getUid().compareTo(PredefTypesConst.PREDEF_CATEGORY_ORG_MODEL_FUID)== 0){
					//completely ignore it
					LoggingUtil.trace(this, "processModel", "This is a predefined categories org model.");				
					root = true;
					parentNode = null;
				}
				else if(model.getAspect() != null && model.getAspect().compareTo(PredefTypesConst.CATEGORY_CATALOG_ASPECT) == 0){
					CategoryCatalogRule rule = new CategoryCatalogRule();
					rule.setImportSession(session);
					if (node == null) {	
						// If there is no parent navigator node, use project node as parent			
						rule.setParent(navProjectNode);
					} else {
						rule.setParent(node);
					}
					rule.setSource((OrganizationModel)model);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_ORGANIZATION_CATALOG);
					rule.invoke();
					// The parent node is the newly created navigation node
					parentNode = rule.getNewCategoryOrganizationModelNode();					
				}else if(model.getAspect() != null && model.getAspect().compareTo(PredefTypesConst.CATEGORY_ASPECT) == 0){
					CategoryInstanceRule rule = new CategoryInstanceRule();
					rule.setImportSession(session);
					if (node == null) {	
						// If there is no parent navigator node, use project node as parent			
						rule.setParent(navProjectNode);
					} else {
						rule.setParent(node);
					}
					rule.setSource((OrganizationModel)model);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_ORGANIZATION_CATALOG);
					rule.invoke();
					// The parent node is the newly created navigation node
					parentNode = rule.getNewCategoryInstanceOrganizationModelNode();		

				} //<end todo>.
				else{
					// Add this org model to the root org model
					OrganizationCatalogRule rule = new OrganizationCatalogRule();
					rule.setImportSession(session);
					if (node == null) {	
						// If there is no parent navigator node, use project node as parent			
						rule.setParent(navProjectNode);
					} else {
						rule.setParent(node);
					}
					rule.setSource((OrganizationModel)model);
					rule.setProjectName(projectName);
					updateTask(IeMessageKeys.TASK_CREATING_ORGANIZATION_CATALOG);
					rule.invoke();
					// The parent node is the newly created navigation node
					parentNode = rule.getNewOrganizationModelNode();
				}
				if (parentNode == null) {
					LoggingUtil.traceExit(this, "processModel");					
					return;
				}
				LoggingUtil.trace(this, "processModel", "Add the organization model: " + parentNode.getLabel());					
			}
		} else if (model instanceof ResourceModel) {
			if (isRootModels() && IImportFramework.ROOT_RES_MODEL_NAME.compareTo(name) == 0) {				
				// Ignore it and process it child
				LoggingUtil.trace(this, "processModel", "This is a root resource model.");				
				root = true;
				parentNode = null;
			} else {
				// Add this resource model to the root information model
				ResourceCatalogRule rule = new ResourceCatalogRule();
				rule.setImportSession(session);
				if (node == null) {	
					// If there is no parent navigator node, use project node as parent			
					rule.setParent(navProjectNode);
				} else {
					rule.setParent(node);
				}
				rule.setSource((ResourceModel)model);
				rule.setProjectName(projectName);
				updateTask(IeMessageKeys.TASK_CREATING_RESOURCE_CATALOG);
				rule.invoke();
				// The parent node is the newly created navigation node
				parentNode = rule.getNewResourceModelNode();
				if (parentNode == null) {
					LoggingUtil.traceExit(this, "processModel");					
					return;
				}
				LoggingUtil.trace(this, "processModel", "Add the resource model: " + parentNode.getLabel());					
			}
		}
		while (iterator.hasNext()) {			
			object = iterator.next();
			if (object instanceof EObject) {
				if (((EObject)object).eIsProxy()) {
					LoggingUtil.trace(this, "processModel", "The object is a proxy.");					
					continue;
				}
			}

			if (object instanceof PackageableElement) {								
				if (object instanceof Model) {
					processModel((Model) object, parentNode);
				} else {
					if (!root) {					
						processElement((PackageableElement) object, parentNode);
					}
				}
			}
		}		
	}

	/**
	 * Set true if the objects are root models.
	 * @param flag
	 */
	public void setIsRootModels(final boolean flag) {
		this.isRootModels = flag;
	}

	/**
	 * Return true if the objects are root models
	 * @return
	 */
	public boolean isRootModels() {
		return isRootModels;	
	}

	private void updateTask(final String key) {
		IProgressMonitor monitor = getImportSession().getProgressMonitor();
		if (monitor != null) {
			monitor.setTaskName(LoggingUtil.getLocalizedString(key, null));
		}
	}

	private void updateTask(final String key,final String[] params) {
		IProgressMonitor monitor = getImportSession().getProgressMonitor();
		if (monitor != null) {
			monitor.setTaskName(LoggingUtil.getLocalizedString(key, params));
		}
	}
}