package com.ibm.btools.blm.ie.imprt.engine;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.osgi.framework.Bundle;

import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.rule.simulationModel.UpdateProcessProfileRule;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.SimulationUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.AbstractNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationProcessNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationProcessSimulationSnapshotNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationSimulationProfileNode;
import com.ibm.btools.bom.model.models.SimulationModel;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.simulationprofiles.ProcessProfile;
import com.ibm.btools.model.modelmanager.validation.BTMessage;
import com.ibm.btools.model.modelmanager.validation.BTReporter;


public class BOMSimulationProcessor extends AbstractBOMProcessor {
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private EList models;
	private Class simCmdClass=null;
	
	public void process() {
		LoggingUtil.traceEntry(this, "process");			
		if (models == null) {
			LoggingUtil.logError(getImportSession(), IeMessageKeys.GENERIC_ERROR, IImportResultMessages.MODE_IMPORT_MODEL_IS_NULL);
			LoggingUtil.traceExit(this, "process");			
			return;	
		}
		
		if (projectName == null) {
			LoggingUtil.logError(getImportSession(), IeMessageKeys.GENERIC_ERROR, IImportResultMessages.PROJECT_NAME_IS_NULL);
			LoggingUtil.traceExit(this, "process");	
			return;
		}		

		// If the Simulation feature is disabled, then we don't proceed
		simCmdClass=getCommandClass("com.ibm.btools.sim.blm.compoundcommand.CreateProfileSIMCmd");
		if (simCmdClass == null) {
			LoggingUtil.traceExit(this, "process");	
			return;
		}		
				
		Iterator iterator = models.iterator();
		Object object = null;
		try {
			while (iterator.hasNext()) {
				object = iterator.next();
				if (object instanceof EObject) {
					if (((EObject)object).eIsProxy()) {
						LoggingUtil.trace(this, "process", "The object is a proxy.");
						continue;
					}
				}
			
				// Only interested in ProcessProfile
				if (object instanceof ProcessProfile && ((ProcessProfile)object).getOwningPackage() == null) {								
					processProcessProfile((ProcessProfile)object);
				} else if (object instanceof SimulationModel) {
					processModel((SimulationModel)object);
				}
				// else ignore.
				//LoggingUtil.trace(this, "process", "The object " + object.getClass() + " is not a PackageableElement.");	
			}
		} finally {
			// Close working copy
			Iterator it = copyIDMap.values().iterator();
			Object[] result = null;
			while(it.hasNext()) {
				result = (Object[]) it.next();
				if (result.length == 2) {
					closeWorkingCopy((String)result[1]);
				}
			}		
		}
		LoggingUtil.traceExit(this, "process");
		
	}

	/**
	 * Recursively process the model element.  Note that we will simply ignore the simulation model
	 * created in the first step because the tool has it's own simulation model structure/pattern
	 * @param simModel
	 * @return
	 */
	private void processModel(SimulationModel simModel) {
		LoggingUtil.trace(this, "processModel", "Process the SimulationModel element:" + simModel.getName());
		if (simModel != null) {
			List members = simModel.getOwnedMember();
			EObject member;
			for (Iterator it=members.iterator();it.hasNext();){
				member = (EObject)it.next();
				if (member instanceof ProcessProfile) {
					processProcessProfile((ProcessProfile)member);
				} else if (member instanceof SimulationModel) {
					processModel((SimulationModel)member);
				}
			}
		}
	}
		
	/**
	 * 
	 * @param procProfile
	 */
	private void processProcessProfile(ProcessProfile procProfile) {
		LoggingUtil.trace(this, "processElement", "Process the ProcessProfile element:" + procProfile.getName());
		Activity process = procProfile.getProcess();
		
		ImportContext context = session.getContext();
		// obtain the snapShot node for the process from the registry.  this will be null if none is found
		NavigationProcessSimulationSnapshotNode snapShotNode=SimulationUtil.getSnapShot(session.getContext(),process);
		// obtain the process navigation node from the context
		NavigationProcessNode processNode = (NavigationProcessNode)context.get(process);
//		if (!isSimulationRunnable(projectName,(Activity)BOMUtil.loadRootObject(processNode))) {
//			String[] params = new String[1];
//			params[0]=process.getName();
//			LoggingUtil.logError(getImportSession(), IeMessageKeys.VALIDATION_ERROR_ON_PROCESS, params, null);
//			return;
//		}

		AbstractNode abstractNode=null;
		if (snapShotNode==null) {
			abstractNode = processNode;
		} else {
			abstractNode = snapShotNode;
		}
		if (abstractNode != null) {
			String processProfileURI=null;
			String processProfileViewURI=null;
			String copyID = null;
			EObject workingCopy = null;		
			String processName=processNode.getLabel();
			String viewURI=(String)processNode.getEntityReferences().get(1);
			String domainURI=(String)processNode.getEntityReferences().get(0);

			Object simCmdInstance = createCreateProfileSIMCmd();
			if (simCmdInstance != null) {
				// invoke the createProfileSIMCmd to create SnapShot (if the abstractNode param is a Process NavigationNode) and
				// default ProcessProfile
				Object[] results = executeCreateProfileSIMCmd(simCmdInstance,abstractNode,processName,viewURI,domainURI);
		
				if (results!=null && results.length > 0) {
					if (snapShotNode==null) {
						//snapShotNode = (NavigationProcessSimulationSnapshotNode)results[0];
						try {
							snapShotNode = (NavigationProcessSimulationSnapshotNode)invokeMethod("getProcessSimulationSnapshotNode",simCmdInstance);
							// register the snapshot to be reused if there is more than one profile needed to be created
							// for the same snapshot
							SimulationUtil.registerSnapShot(session.getContext(),process,snapShotNode);
						}
						catch (IllegalAccessException ex) {	
							LoggingUtil.logError(getImportSession(), IeMessageKeys.GENERIC_ERROR, IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
						}
						catch (NoSuchMethodException ex) {
							LoggingUtil.logError(getImportSession(), IeMessageKeys.GENERIC_ERROR, IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
						}
						catch (InvocationTargetException ex) {
							LoggingUtil.logError(getImportSession(), IeMessageKeys.GENERIC_ERROR, IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
						}
					}
					if (snapShotNode != null) {	
						try{
							NavigationSimulationProfileNode profileNode = (NavigationSimulationProfileNode)invokeMethod("getNavigationSimulationProfileNode",simCmdInstance);
						//	processProfileURI=NavigatorUtil.getURI((NavigationSimulationProfileNode)results[results.length-1]);
							if(profileNode != null){
								processProfileURI=NavigatorUtil.getURI(profileNode);
								if (isMaster) {
									workingCopy = openMasterCopy(processProfileViewURI, processProfileURI);
								} else {
									Object[] result = openWorkingCopy(processProfileViewURI, processProfileURI);
						
									if (result != null && result.length == 2) {
										workingCopy = (EObject) result[0];
										copyID = (String) result[1];
									}
								}
		
								UpdateProcessProfileRule rule = new UpdateProcessProfileRule();
								// invoke the rule	
								rule.setImportSession(session);
								rule.setSource(procProfile);
								rule.setWorkingCopy(workingCopy);
								rule.setProjectName(projectName);
					
								try {
									rule.invoke();
								} catch (OperationCanceledException e) {
									if (!isMaster) {		
										closeWorkingCopy(copyID);
									}	
									throw e;
								}
								if (isMaster) {
									saveMasterCopy(processProfileViewURI, processProfileURI);
								} else {
									saveWorkingCopy(processProfileViewURI, processProfileURI, copyID);	
								}
							}	
						}catch (IllegalAccessException ex) {	
							LoggingUtil.logError(getImportSession(), IeMessageKeys.GENERIC_ERROR, IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
						}catch (NoSuchMethodException ex) {
							LoggingUtil.logError(getImportSession(), IeMessageKeys.GENERIC_ERROR, IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
						}
						catch (InvocationTargetException ex) {
							LoggingUtil.logError(getImportSession(), IeMessageKeys.GENERIC_ERROR, IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
						}
					}
				}
			}
		}
	}


	public static boolean isSimulationRunnable(String projectName, Activity process) {
		if (projectName!=null && process!=null) {
			List valMsgs = BTReporter.instance().getRootObjectMessages(projectName,process);
			if (valMsgs != null) {
				BTMessage msg;
				for (Iterator it=valMsgs.iterator();it.hasNext();) {
					msg = (BTMessage)it.next();
					if (msg.getId().startsWith("ZBM") && msg.getId().endsWith("E")) {
						return false;
					}
				}
			}
		} else {
			return false;
		}
		return true;
	}
	private Object createCreateProfileSIMCmd() {
		try {
			return simCmdClass.newInstance();
		}
		catch (IllegalAccessException ex) {	
			LoggingUtil.logError(getImportSession(), IeMessageKeys.GENERIC_ERROR, IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
		}
		catch (InstantiationException ex) {
			LoggingUtil.logError(getImportSession(), IeMessageKeys.GENERIC_ERROR, IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
		}
		return null;		
	}
	
	private Object[] executeCreateProfileSIMCmd(Object simCmdInstance, AbstractNode absNode,String processName,String viewURI, String domainURI) {
		try {
	
			//Object simCmdInstance = simCmdClass.newInstance();
	
			// Call the setSimulationParentNode method
			Object retobj = invokeMethod("setSimulationParentNode", absNode, AbstractNode.class,simCmdInstance);
							   
			// Call the setProjectName method
			retobj = invokeMethod("setProjectName", projectName,String.class,simCmdInstance);
								   
			// Call the setName method
			retobj = invokeMethod("setProfileName", processName,String.class,simCmdInstance);
						
			//Call the setName method
			retobj = invokeMethod("setCEFViewModelURI", viewURI,String.class,simCmdInstance);
										 
			//Call the setName method
			retobj = invokeMethod("setModelURI", domainURI,String.class,simCmdInstance);
											   
			//  Call the execute method
			retobj = invokeMethod("execute",simCmdInstance);
			//progressMonitorDialog.getProgressMonitor().worked(12);
						
			//  Get the results
			retobj = invokeMethod("getResult",simCmdInstance);
				
			//progressMonitorDialog.getProgressMonitor().worked(16);
			return ((Collection)retobj).toArray();
		}
		catch (IllegalAccessException ex) {	
			LoggingUtil.logError(getImportSession(), IeMessageKeys.GENERIC_ERROR, IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
		}
		catch (NoSuchMethodException ex) {
			LoggingUtil.logError(getImportSession(), IeMessageKeys.GENERIC_ERROR, IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
		}
		catch (InvocationTargetException ex) {
			LoggingUtil.logError(getImportSession(), IeMessageKeys.GENERIC_ERROR, IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
		}
		return null;			
	}
	private Object invokeMethod(String methodName, Object classInstance) throws NoSuchMethodException,InvocationTargetException,IllegalAccessException {
		return invokeMethod(methodName, null, null,classInstance);
	}
	private Object invokeMethod(String methodName, Object param, Class paramType, Object classInstance) throws NoSuchMethodException,InvocationTargetException,IllegalAccessException {
		//	Call the setProcessCatalogNode method
		Class paratypes[];
		if (paramType == null) {
			paratypes = new Class[0];
		} else {
			paratypes = new Class[1];
			paratypes[0] = paramType;
		}
			
		Method meth = classInstance.getClass().getMethod(
				 methodName, paratypes);			
		 // Populate the parameters for the method
		 Object arglist[];
		 if (param == null) {
			arglist = new Object[0];
		 } else {
			arglist = new Object[1];
		   	arglist[0] = param;
		 }
		 // Invoke the mothod.
		 return meth.invoke(classInstance, arglist);
	}
	
	/**
	 * Get the class form Simuation plugin.  If the plugin is not available, log a warning informing
	 * that Simulation import cannot be performed and return null
	 * @param className
	 * @return
	 */
	private Class getCommandClass(String className) {
		try {
			Bundle plugin = Platform.getBundle("com.ibm.btools.sim");
			
			// In theory this should never happen since the BLM navigator will only
			// enable the action if this plugin exists, but we'll check just to be
			// safe
			if (plugin == null) 
				return null;
			
			// Obtain the Run Simulation command
			return plugin.loadClass(className);
		} catch (Throwable e) {
			LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATION_COMMAND_NOT_AVAILABLE);
			return null;
		}
	}

	/**
	 * @return
	 */
	public EList getModels() {
		return models;
	}

	/**
	 * @param list
	 */
	public void setModels(EList list) {
		models = list;
	}

}
