package com.ibm.btools.blm.ie.imprt.engine;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.process.util.PECommandConstants;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.registry.ImportRuleRegistryReader;
import com.ibm.btools.blm.ie.imprt.rule.IImportRule;
import com.ibm.btools.blm.ie.imprt.rule.informationModel.UpdateBusinessItemRule;
import com.ibm.btools.blm.ie.imprt.rule.informationModel.UpdateSignalRule;
import com.ibm.btools.blm.ie.imprt.rule.organizationModel.UpdateOrganizationDefinitionCategoryRule;
import com.ibm.btools.blm.ie.imprt.rule.organizationModel.UpdateOrganizationDefinitionRule;
import com.ibm.btools.blm.ie.imprt.rule.processModel.UpdateDatastoreRule;
import com.ibm.btools.blm.ie.imprt.rule.resourceModel.UpdateBulkResourceTypeRule;
import com.ibm.btools.blm.ie.imprt.rule.resourceModel.UpdateIndividualResourceTypeRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.AbstractLibraryChildNode;
import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.bom.model.artifacts.InstanceSpecification;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.artifacts.PrimitiveType;
import com.ibm.btools.bom.model.artifacts.Signal;
import com.ibm.btools.bom.model.models.Model;
import com.ibm.btools.bom.model.organizationstructures.Location;
import com.ibm.btools.bom.model.organizationstructures.LocationType;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnit;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnitType;
import com.ibm.btools.bom.model.organizationstructures.Tree;
import com.ibm.btools.bom.model.organizationstructures.TreeStructure;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.Datastore;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.humantasks.Form;
import com.ibm.btools.bom.model.resources.BulkResource;
import com.ibm.btools.bom.model.resources.BulkResourceType;
import com.ibm.btools.bom.model.resources.IndividualResource;
import com.ibm.btools.bom.model.resources.IndividualResourceType;
import com.ibm.btools.bom.model.resources.Role;
import com.ibm.btools.bom.model.time.RecurringTimeIntervals;
import com.ibm.btools.cef.model.CommonContainerModel;
import com.ibm.btools.cef.model.VisualModelDocument;

/**
 * Updating the BOM object in the Import Context 
 */
public class BOMUpdateProcessor extends AbstractBOMProcessor{
	
	private int closeSequence = 3;
	private int updateProcessEndSequence = 5;
	private int updateProcessBeginSequence = 4;
	
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	/**
     * Update the BOM Object in the Import Context by sequence
	 */
	public void process() {
		LoggingUtil.traceEntry(this, "process");	
				
		if (projectName == null) {
			LoggingUtil.logError(getImportSession(), IeMessageKeys.GENERIC_ERROR, IImportResultMessages.PROJECT_NAME_IS_NULL);
			LoggingUtil.traceExit(this, "process");
			return;
		}		
		try {
			ImportRuleRegistryReader reader = ImportRuleRegistryReader.getReader();
			Map sequences = reader.getImportRules();
			int size = closeSequence;
			String sequence = null;
			//Ray - moved to superclass: copyIDMap = new HashMap();
			for (int step = 0; step<=size; step++) {			
				sequence = (new Integer(step)).toString();
				processSequence(sequence);
			}
			processActivities();
			
			//TODO AR
			ImportRuleRegistryReader.dispose();
			
			
			
		} finally {
		
			// Close working copy
			closeWorkingCopies();
			//TODO AR
			ImportRuleRegistryReader.dispose();
		}
		
		LoggingUtil.traceExit(this, "process");		
	}
	
	/**
	 * Close Working copy by ID
	 *
	 */
	private void closeWorkingCopyById(String copyID) {
		// Close working copy
		if(copyID == null){
			LoggingUtil.trace(this, "closeWorkingCopy", "The copyID should not be null.");
			return;
		}
		Iterator iterator = new ArrayList(copyIDMap.keySet()).iterator();
		Object[] result = null;
		Object key = null;
		Object domainObject = null;
		while(iterator.hasNext()) {
			key = iterator.next();
			result = (Object[]) copyIDMap.get(key);
			if (result.length == 2) {
				if (copyID.compareTo((String)result[1]) == 0) {								
					closeWorkingCopy(copyID);
					copyIDMap.remove(key);	
					break;
				}								
			}			
		}	
	}	
	
	/**
	 * Close Working copies
	 *
	 */
	private void closeWorkingCopies() {
		// Close working copy
		Iterator iterator = new ArrayList(copyIDMap.keySet()).iterator();
		Object[] result = null;
		Object key = null;
		Object domainObject = null;
		while(iterator.hasNext()) {
			key = iterator.next();
			result = (Object[]) copyIDMap.get(key);
			if (result.length == 2) {				
				closeWorkingCopy((String)result[1]);
				copyIDMap.remove(key);									
			}			
		}	
	}	
	
	private void processActivities() {
		ImportContext context = session.getContext();
		ImportContext clonedContext = (ImportContext) context.clone();
		Set keys = clonedContext.keySet();
		Iterator iterator = keys.iterator();
		Object object = null;
		AbstractLibraryChildNode node = null;
		Object content = null;	
		
		ImportRuleRegistryReader reader = ImportRuleRegistryReader.getReader();
		StructuredActivityNode implementation = null;	
		String aspect = null;
		IImportRule rule = null;
		String key = null;
		EObject workingCopy = null;
		String uri = null;
		String viewURI = null;
		String copyID = null;
		while(iterator.hasNext()) {
			object = iterator.next();
			if (object instanceof Activity) {
				content = context.get(object);
				if (content instanceof AbstractLibraryChildNode) {					
					node = (AbstractLibraryChildNode) content;
					if (node != null) {				
						implementation = ((Activity)object).getImplementation();
						aspect = implementation.getAspect();
						uri = NavigatorUtil.getURI(node);
						viewURI = NavigatorUtil.getViewURI(node);	
						if (aspect != null) {
							if (isMaster) {
								workingCopy = openMasterCopy(viewURI, uri);
							} else {
								Object[] result = openWorkingCopy(viewURI, uri);
				
								if (result != null && result.length == 2) {
									workingCopy = (EObject) result[0];
									copyID = (String) result[1];
								}
							}							
							for (int i=updateProcessBeginSequence; i<=updateProcessEndSequence; i++) {	
								//AZ added for ServiceOperation aspect of the external service operation
								if (PECommandConstants.PROCESS_ASPECT.compareTo(aspect) == 0 ) {
									rule = reader.getRule(i+"", IImportFramework.IMPORT_RULE_UPDATE_PROCESS);
									key = IeMessageKeys.TASK_UPDATING_PROCESS;
								} else if (PECommandConstants.TASK_ASPECT.compareTo(aspect) == 0) {
									rule = reader.getRule(i+"", IImportFramework.IMPORT_RULE_UPDATE_TASK);
									key = IeMessageKeys.TASK_UPDATING_TASK;									
									//Andy added for handling service operation, use the same rule as service
								} else if (PECommandConstants.SERVICE_ASPECT.compareTo(aspect) == 0 ||
										BOMUtil.isServiceOperation(implementation)) {
									rule = reader.getRule(i+"", IImportFramework.IMPORT_RULE_UPDATE_SERVICE);
									if(BOMUtil.isServiceOperation(implementation)){
										key = IeMessageKeys.TASK_UPDATING_SERVICE_OPERATION;
									}else{
										key = IeMessageKeys.TASK_UPDATING_SERVICE;
									}
								}
								if (rule != null) {
									// invoke the rule	
									rule.setImportSession(session);
									rule.setSource(object);
									rule.setWorkingCopy(workingCopy);
									rule.setProjectName(projectName);
			
									if (key != null && workingCopy != null) {
										updateTask(key, workingCopy);
									}
			
									try {
										rule.invoke();
									} catch (OperationCanceledException e) {
										if (!isMaster) {		
											closeWorkingCopyById(copyID);
										}	
										throw e;
									}	catch (RuntimeException e) {
										// e.printStackTrace();		
										throw e;
									}
					
									IProgressMonitor monitor = getImportSession().getProgressMonitor();
									if (monitor != null) {
										if (monitor.isCanceled()) {					
											throw new OperationCanceledException();					
										 }
										 monitor.worked(ProgressCalculator.getSimpleTask());
									}																	
								}						
							}

							if (isMaster) {
								saveMasterCopy(viewURI, uri);
							} else {
								saveWorkingCopy(viewURI, uri, copyID);	
							}
							closeWorkingCopyById(copyID);
						}
					}
				}				
			}
		}		
	}
	
	/**
	 * Get Domain Object from VisualDocument
	 * @return
	 */
	private Object getDomainObject(VisualModelDocument vmd) {		
		Object topContainer = null;
		
		try {			
			topContainer = (Object) vmd.getRootContent().getContentChildren().get(0);
			if (topContainer instanceof CommonContainerModel) {
				return ((CommonContainerModel)topContainer).getDomainContent().get(0);				
			}
		} catch (RuntimeException e) {
			return null;
		}

		return null;
	}
	
	/**
	 * Process the element inside the import context by sequence
	 * @param sequence
	 */
	private void processSequence(final String sequence) {
		LoggingUtil.traceEntry(this, "processSequence");
		LoggingUtil.trace(this, "processSequence", "Process the sequence: " + sequence + ".");		
				
		ImportContext context = session.getContext();
		ImportContext clonedContext = (ImportContext) context.clone();
		Set keys = clonedContext.keySet();
		Iterator iterator = keys.iterator();
		Object object = null;
		AbstractLibraryChildNode node = null;
		Object content = null;
				
		while(iterator.hasNext()) {
			object = iterator.next();
			LoggingUtil.trace(this, "processSequence", "The object in the context is: " + object.getClass()+ ".");			
			if (object instanceof Model) {
				// do nothing
			} else if (object instanceof PackageableElement) {
				content = context.get(object);
				LoggingUtil.trace(this, "processSequence", "The object's value in the context is: " + content.getClass()+ ".");
				if (content instanceof AbstractLibraryChildNode) {					
					node = (AbstractLibraryChildNode) content;
					if (node != null) {				
						processElement(object, node, sequence);
					}
				}
			}
		}		
		LoggingUtil.traceExit(this, "processSequence");
	}
	
	/**
	 * Process element by type
	 * @param object
	 * @param workingCopy
	 */
	private void processElement(final Object element, final AbstractLibraryChildNode node, final String sequence) {
		String key = null;
		LoggingUtil.traceEntry(this, "processElement");
		// Element should not be null.		
		// Node should not be null.
		
		// If the object is in already in the resolve list, don't update.
		Object list = session.getContext().get(IImportFramework.CONTEXT_RESOLVED_TYPE);
		if (list != null && list instanceof List) {
			if (((List)list).contains(node)) {
				return;
			}
		}
		
		LoggingUtil.trace(this, "processElement", "The element is: " + element.getClass() + ".");		
		
		ImportRuleRegistryReader reader = ImportRuleRegistryReader.getReader();
		IImportRule rule = null;

		// Primitive Type doesn't need to be updated
		if (element instanceof PrimitiveType) {
			LoggingUtil.trace(this, "processElement", "The element is a primitive type.");	
			LoggingUtil.traceExit(this, "processElement");
			return;
		}
		
		if (element instanceof Signal) {
			if (((Signal)element).getIsAbstract().booleanValue() == true) {				
				rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_SIGNAL_CATEGORY);
				key = IeMessageKeys.TASK_UPDATING_NOTIFICATION_TEMPLATE;
			} else {
				rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_SIGNAL);	
				key = IeMessageKeys.TASK_UPDATING_NOTIFICATION;						
			}
		}  else if (element instanceof OrganizationUnitType) {
			if (((OrganizationUnitType)element).getIsAbstract().booleanValue() == true) {
				rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_ORGANIZATION_DEFINITION_CATEGORY);
				key = IeMessageKeys.TASK_UPDATING_ORGANIZATION_DEFINITION_TEMPLATE;
			} else {
				rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_ORGANIZATION_DEFINITION);	
				key = IeMessageKeys.TASK_UPDATING_ORGANIZATION_DEFINITION;								
			}		
		} else if (element instanceof OrganizationUnit) {
			rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_ORGANIZATION_UNIT);
			key = IeMessageKeys.TASK_UPDATING_ORGANIZATION_UNIT;					
		} else if (element instanceof LocationType) {
			if (((LocationType)element).getIsAbstract().booleanValue() == true) {
				rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_LOCATION_DEFINITION_CATEGORY);
				key = IeMessageKeys.TASK_UPDATING_LOCATION_DEFINITION_TEMPLATE;
			} else {
				rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_LOCATION_DEFINITION);
				key = IeMessageKeys.TASK_UPDATING_LOCATION_DEFINITION;									
			}
		} else if (element instanceof Location) {
			rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_LOCATION);
			key = IeMessageKeys.TASK_UPDATING_LOCATION;			
		} else if (element instanceof TreeStructure) {
			rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_HIERARCHY_STRUCTURE_DEFINITION);
			key = IeMessageKeys.TASK_UPDATING_HIERARCHY_STRUCTURE_DEFINITION;						
		} else if (element instanceof Tree) {
			rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_HIERARCHY);
			key = IeMessageKeys.TASK_UPDATING_HIERARCHY;			
		} else if (element instanceof IndividualResourceType ) {
			if (((IndividualResourceType)element).getIsAbstract().booleanValue() == true) {
				rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_INDIVIDUAL_RESOURCE_DEFINITION_CATEGORY);
				key = IeMessageKeys.TASK_UPDATING_INDIVIDUAL_RESOURCE_DEFINITION_TEMPLATE;
			} else {
				rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_INDIVIDUAL_RESOURCE_DEFINITION);
				key = IeMessageKeys.TASK_UPDATING_INDIVIDUAL_RESOURCE_DEFINITION;									
			}
		} else if (element instanceof IndividualResource) {
			rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_INDIVIDUAL_RESOURCE);	
			key = IeMessageKeys.TASK_UPDATING_INDIVIDUAL_RESOURCE;			
		} else if (element instanceof BulkResourceType ) {
			if (((BulkResourceType)element).getIsAbstract().booleanValue() == true) {
				rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_BULK_RESOURCE_DEFINITION_CATEGORY);
				key = IeMessageKeys.TASK_UPDATING_BULK_RESOURCE_DEFINITION_TEMPLATE;
			} else {
				rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_BULK_RESOURCE_DEFINITION);
				key = IeMessageKeys.TASK_UPDATING_BULK_RESOURCE_DEFINITION;									
			}
		} else if (element instanceof BulkResource) {
			rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_BULK_RESOURCE);
			key = IeMessageKeys.TASK_UPDATING_BULK_RESOURCE;			
		} else if (element instanceof Role) {
			rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_ROLE);
			key = IeMessageKeys.TASK_UPDATING_ROLE;			
		} else if (element instanceof RecurringTimeIntervals) {
			rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_TIMETABLE);
			key = IeMessageKeys.TASK_UPDATING_TIMETABLE;			
		} else if (element instanceof Datastore) {
			rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_REPOSITORY);
			key = IeMessageKeys.TASK_UPDATING_REPOSITORY;			
		} else if (element instanceof Activity) {
			StructuredActivityNode implementation = ((Activity)element).getImplementation();
			String aspect = implementation.getAspect();
			if (aspect != null) {
				if (PECommandConstants.PROCESS_ASPECT.compareTo(aspect) == 0) {
					rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_PROCESS);
					key = IeMessageKeys.TASK_UPDATING_PROCESS;
				} else if (PECommandConstants.TASK_ASPECT.compareTo(aspect) == 0) {
					rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_TASK);
					key = IeMessageKeys.TASK_UPDATING_TASK;
				} else if (PECommandConstants.HUMAN_TASK_ASPECT.compareTo(aspect) == 0) {
					rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_HUMAN_TASK);
					key = IeMessageKeys.TASK_UPDATING_HUMAN_TASK;
				} else if (PECommandConstants.BUSINESS_RULE_TASK_ASPECT.compareTo(aspect) == 0) {
					rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_BUSINESS_RULES_TASK);
					key = IeMessageKeys.TASK_UPDATING_BUSINESS_RULES_TASK;
				} else if (PECommandConstants.SERVICE_ASPECT.compareTo(aspect) == 0  ||
						BOMUtil.isServiceOperation((NamedElement)implementation)) {
					//Andy.X.Z Added for new BOM Model Support
					rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_SERVICE);
					if(BOMUtil.isServiceOperation((NamedElement)implementation)){
						key = IeMessageKeys.TASK_UPDATING_SERVICE_OPERATION;
					}else{
						key = IeMessageKeys.TASK_UPDATING_SERVICE;
					}
				}
			}		
		} else if (element instanceof Class) {
			if (((Class)element).getIsAbstract().booleanValue() == true) {
				rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_BUSINESS_ITEM_CATEGORY);
				if(BOMUtil.isSchemaType((NamedElement)element)||BOMUtil.isFileAttachment((NamedElement)element)){
					key = IeMessageKeys.TASK_UPDATING_COMPLEX_TYPE_TEMPLATE;
				}else{
					key = IeMessageKeys.TASK_UPDATING_BUSINESS_ITEM_TEMPLATE;
				}
			} else {
				rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_BUSINESS_ITEM);
				if(BOMUtil.isSchemaType((NamedElement)element)||BOMUtil.isFileAttachment((NamedElement)element)){
					key = IeMessageKeys.TASK_UPDATING_COMPLEX_TYPE;
				}else{
					key = IeMessageKeys.TASK_UPDATING_BUSINESS_ITEM;	
				}
			}				
		} else if (element instanceof InstanceSpecification ) {
			rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_BUSINESS_ITEM_INSTANCE);
			key = IeMessageKeys.TASK_UPDATING_BUSINESS_ITEM_INSTANCE;			
		} else if (element instanceof Form){
			rule = reader.getRule(sequence, IImportFramework.IMPORT_RULE_UPDATE_FORM);
			key = IeMessageKeys.TASK_UPDATING_FORM;
		}else {			
			LoggingUtil.traceExit(this, "processElement");			
			return;			
		}
		
		if (rule != null) {	
			LoggingUtil.trace(this, "processElement", "Get the rule: " + rule.getClass() + ".");		
			
			String uri = NavigatorUtil.getURI(node);		
			String viewURI = NavigatorUtil.getViewURI(node);				
			String copyID = null;
			EObject workingCopy = null;

			if (isMaster) {
				workingCopy = openMasterCopy(viewURI, uri);
			} else {
				Object[] result = openWorkingCopy(viewURI, uri);
				
				if (result != null && result.length == 2) {
					workingCopy = (EObject) result[0];
					copyID = (String) result[1];
				}
			}
			
			// invoke the rule	
			rule.setImportSession(session);
			rule.setSource(element);
			rule.setWorkingCopy(workingCopy);
			rule.setProjectName(projectName);
			
			if (key != null && workingCopy != null) {
				updateTask(key, workingCopy);
			}
			
			try {
				rule.invoke();
			} catch (OperationCanceledException e) {
				if (!isMaster) {					
					closeWorkingCopyById(copyID);
				}	
				throw e;
			}	catch (RuntimeException e) {
				// e.printStackTrace();		
				throw e;
			}
					
			IProgressMonitor monitor = getImportSession().getProgressMonitor();
			if (monitor != null) {
				if (monitor.isCanceled()) {
					LoggingUtil.trace(this, "processElement", "The user cancels the operation.");					
					throw new OperationCanceledException();					
				 }
				 monitor.worked(ProgressCalculator.getSimpleTask());
			}	
			
			if (isMaster) {
				saveMasterCopy(viewURI, uri);
			} else {
				saveWorkingCopy(viewURI, uri, copyID);							
			}
			
			// hack
			if (rule instanceof UpdateBusinessItemRule || rule instanceof UpdateSignalRule || 
				rule instanceof UpdateIndividualResourceTypeRule || rule instanceof UpdateBulkResourceTypeRule ||
				rule instanceof UpdateOrganizationDefinitionRule || rule instanceof UpdateOrganizationDefinitionCategoryRule ||
				rule instanceof UpdateDatastoreRule) {
					    
			} else {
				if (!isMaster) {
					closeWorkingCopyById(copyID);
				}
			}
		}
		LoggingUtil.traceExit(this, "processElement");
	}

	private void updateTask(final String key, final Object element) {
		IProgressMonitor monitor = getImportSession().getProgressMonitor();
		String name = null;
		CommonContainerModel topContainer = null;
		if (monitor != null) {
			if (element instanceof NamedElement) {
				name = ((NamedElement)element).getName();
			} else if (element instanceof VisualModelDocument) {
				try {				
					topContainer = (CommonContainerModel) ((VisualModelDocument) element).getRootContent().getContentChildren().get(0);
					name = ((NamedElement)topContainer.getDomainContent().get(0)).getName();
				} catch (RuntimeException e) {
					// Ignore
				}
			}
			if (name != null) {				
				monitor.setTaskName(LoggingUtil.getLocalizedString(key, new String[]{name}));
			} else {
				monitor.setTaskName(LoggingUtil.getLocalizedString(key, null));
			}			
		}
	}	
}