package com.ibm.btools.blm.ie.imprt.option;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.HashMap;
import java.util.Map;

import com.ibm.btools.blm.ie.imprt.ImportOptions;
import com.ibm.btools.blm.ie.imprt.ImportSession;

public class XsdOptions {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
		
	public static final String XSD_OPTIONS = "com.ibm.btools.blm.ie.imprt.option.XsdOptions";

	public static final String XSD_OPTIONS_TARGET_TYPE = "XSD_OPTIONS_TARGET_TYPE";
	public static final String XSD_TO_BUSINESS_ITEM = "XSD_TO_BUSINESS_ITEM";
	public static final String XSD_TO_SIGNAL = "XSD_TO_SIGNAL";
	public static final String XSD_TO_INDIVIDUAL_RESOURCE_TYPE = "XSD_TO_INDIVIDUAL_RESOURCE_TYPE";
	public static final String XSD_TO_BULK_RESOURCE_TYPE = "XSD_TO_BULK_RESOURCE_TYPE";
	public static final String XSD_TO_LOCATION_TYPE = "XSD_TO_LOCATION_TYPE";
	public static final String XSD_TO_ORGANIZATION_UNIT_TYPE= "XSD_TO_ORGANIZATION_UNIT_TYPE";

	public static final String XSD_OPTIONS_IMPORT_LEVEL = "XSD_OPTIONS_IMPORT_LEVEL";
	public static final String XSD_IMPORT_LEVEL_SHALLOW = "XSD_IMPORT_LEVEL_SHALLOW";
	public static final String XSD_IMPORT_LEVEL_DEEP = "XSD_IMPORT_LEVEL_DEEP";

	public static Map getXsdOptions(ImportSession importSession) {
		ImportOptions importOptions = importSession.getImportOptions();
		if (importOptions == null) {
			importOptions = new ImportOptions();
			importSession.setImportOption(importOptions);
		}
		Map xsdOptions = (Map)importOptions.getAdditionalOption(XSD_OPTIONS);;
		if (xsdOptions == null) {
			xsdOptions = new HashMap();
			importOptions.setAdditionalOption(XSD_OPTIONS,xsdOptions);
		} 
		return xsdOptions;
	}

	public static void setXsdToBomImportLevel(ImportSession importSession, String level) {
		Map options = getXsdOptions(importSession);
		options.put(XSD_OPTIONS_IMPORT_LEVEL,level);
	}
	
	public static String getXsdToBomImportLevel(ImportSession importSession) {
		Map options = getXsdOptions(importSession);
		String tarType = (String)options.get(XSD_OPTIONS_IMPORT_LEVEL);	
		if (tarType == null) {
			tarType = XSD_IMPORT_LEVEL_DEEP; // default to deep
		}
		return tarType;	
	}
	
	public static void setXsdToBomTargetType(ImportSession importSession, String targetType) {
		Map options = getXsdOptions(importSession);
		options.put(XSD_OPTIONS_TARGET_TYPE,targetType);
	}
	
	public static String getXsdToBomTargetType(ImportSession importSession) {
		Map options = getXsdOptions(importSession);
		String tarType = (String)options.get(XSD_OPTIONS_TARGET_TYPE);	
		if (tarType == null) {
			tarType = XSD_TO_BUSINESS_ITEM; // default to bus item
		}
		return tarType;	
	}
	
	public static boolean isTargetBusItem(ImportSession importSession) {
		return getXsdToBomTargetType(importSession).equals(XSD_TO_BUSINESS_ITEM);
	}

	public static boolean isTargetSignal(ImportSession importSession) {
		return getXsdToBomTargetType(importSession).equals(XSD_TO_SIGNAL);
	}

	public static boolean isTargetIndResType(ImportSession importSession) {
		return getXsdToBomTargetType(importSession).equals(XSD_TO_INDIVIDUAL_RESOURCE_TYPE);
	}

	public static boolean isTargetBulkResType(ImportSession importSession) {
		return getXsdToBomTargetType(importSession).equals(XSD_TO_BULK_RESOURCE_TYPE);
	}

	public static boolean isTargetOrgUnitType(ImportSession importSession) {
		return getXsdToBomTargetType(importSession).equals(XSD_TO_ORGANIZATION_UNIT_TYPE);
	}
	
	public static boolean isTargetLocType(ImportSession importSession) {
		return getXsdToBomTargetType(importSession).equals(XSD_TO_LOCATION_TYPE);
	}

	public static void setTargetToBusItem(ImportSession importSession) {
		setXsdToBomTargetType(importSession,XSD_TO_BUSINESS_ITEM);	
	}

	public static void setTargetToSignal(ImportSession importSession) {
		setXsdToBomTargetType(importSession,XSD_TO_SIGNAL);	
	}

	public static void setTargetToIndResModel(ImportSession importSession) {
		setXsdToBomTargetType(importSession,XSD_TO_INDIVIDUAL_RESOURCE_TYPE);	
	}

	public static void setTargetToBulkResModel(ImportSession importSession) {
		setXsdToBomTargetType(importSession,XSD_TO_BULK_RESOURCE_TYPE);	
	}

	public static void setTargetToOrgUnitType(ImportSession importSession) {
		setXsdToBomTargetType(importSession,XSD_TO_ORGANIZATION_UNIT_TYPE);	
	}

	public static void setTargetToLocType(ImportSession importSession) {
		setXsdToBomTargetType(importSession,XSD_TO_LOCATION_TYPE);	
	}

	public static boolean isLevelShallow(ImportSession importSession) {
		return getXsdToBomImportLevel(importSession).equals(XSD_IMPORT_LEVEL_SHALLOW);
	}

	public static boolean isLevelDeep(ImportSession importSession) {
		return getXsdToBomImportLevel(importSession).equals(XSD_IMPORT_LEVEL_DEEP);
	}
	
	public static void setLevelToShallow(ImportSession importSession) {
		setXsdToBomImportLevel(importSession,XSD_IMPORT_LEVEL_SHALLOW);	
	}

	public static void setLevelToDeep(ImportSession importSession) {
		setXsdToBomImportLevel(importSession,XSD_IMPORT_LEVEL_DEEP);	
	}

}
