package com.ibm.btools.blm.ie.imprt.rule;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.EList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.PlatformUI;
import org.osgi.framework.Bundle;

import com.ibm.btools.blm.compoundcommand.pe.factory.PeBmCmdFactory;
import com.ibm.btools.blm.compoundcommand.pe.factory.PeCmdFactory;
import com.ibm.btools.blm.compoundcommand.pe.factory.PeDefaultPredefinedDataTypesLocator;
import com.ibm.btools.blm.compoundcommand.util.CCRuntimeException;
import com.ibm.btools.blm.ie.imprt.IImportConstants;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportSession;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.NavigationProjectNode;
import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.util.exception.BTRuntimeException;
import com.ibm.btools.util.resource.UtilResourceBundleSingleton;


/**
 * Abstract implementation of the Import rule
 */
public abstract class AbstractImportRule implements IImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	protected final String STATE_MAP = "STATE_MAP";
	
	private ImportSession session;
	private String projectName;
	
	/**
	 * Abstract method. Since not all rule requires working copy
	 */
	public void setWorkingCopy(final Object workingCopy) {
	}	
	
	/**
	 * Set projectName
	 * @param projectName
	 */
	public void setProjectName(final String projectName) {
		this.projectName= projectName;
	}
	
	/**
	 * Get project name
	 * @return
	 */
	public String getProjectName() {
		return projectName;
	}
	
	/**
	 * Set the import session
	 * @param session
	 */
	public void setImportSession(final ImportSession session) {
		this.session = session;
	}
	
	/**
	 * Get the import session
	 * @param session
	 */
	public ImportSession getImportSession() {
		return session;	
	}
	
	public PeCmdFactory getPeCmdFactory() {
		PeCmdFactory factory = (PeCmdFactory) session.getContext().get(IImportFramework.CONTEXT_PE_CMD_FACTORY_KEY);		
		if (factory == null) {
			NavigationProjectNode projectNode = NavigatorUtil.getProjectNode(getProjectName());
			if (projectNode == null) {
				return null;
			}
			factory = new PeBmCmdFactory(new PeDefaultPredefinedDataTypesLocator(projectNode));
			session.getContext().put(IImportFramework.CONTEXT_PE_CMD_FACTORY_KEY, factory);
		}
				
		return factory;
		
		
	}
	
	protected class DisplayErrorMessage implements Runnable {	
		
		public void run() {
			MessageBox errorBox = new MessageBox(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), SWT.OK|SWT.APPLICATION_MODAL|SWT.ICON_ERROR);
			errorBox.setText(UtilResourceBundleSingleton.INSTANCE.getMessage(IeMessageKeys.class, IeMessageKeys.ERROR));			
			errorBox.setMessage(UtilResourceBundleSingleton.INSTANCE.getMessage(IeMessageKeys.class,IeMessageKeys._UI_Path_Length_Error));
			errorBox.open();	
		}	
	}
	
	protected void showMessageBox(Exception e,String name){
		if(e instanceof CCRuntimeException && "CCB9999E".equals(((CCRuntimeException)e).getCode())){
			DisplayErrorMessage error = new DisplayErrorMessage();
			Display.getDefault().syncExec(error);	
			LoggingUtil.logError(getImportSession(), IeMessageKeys.NAME_TOO_LONG, new String[]{name}, e, 
					IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);						
			BTRuntimeException ex = new BTRuntimeException("ERROR");
			throw ex;
		}	
	}
	
	protected void registerTemplate(Object template){
		List registry = (List)getImportSession().getContext().get(IImportConstants.TEMPLATE_REGISTRY_ENTRY);
		if(registry == null){
			registry = new ArrayList();	
			getImportSession().getContext().put(IImportConstants.TEMPLATE_REGISTRY_ENTRY, registry);
		}
		registry.add(template);
	}
	
	protected void forkUICloseEditor(final String blmURI){
		PlatformUI.getWorkbench().getDisplay().syncExec(
				new Runnable() {
					public void run() {
						closeEditor(blmURI);
					}
				});
	}
	/**
	 * Close the view editor associated with this BLM URI
	 * This method has to be called by an UI thread. A NullPointerException will be raised otherwise
	 * @param blmURI
	 */
	private void closeEditor(String blmURI) {
		
		try {		
			// Load the plugin
			Bundle plugin = Platform.getBundle("com.ibm.btools.blm.ui.navigation");
			
			// Just in case it doesn't exist
			if (plugin == null) 
				return;
			
			java.lang.Class BLMEditorInputClass = plugin.loadClass("com.ibm.btools.blm.ui.navigation.manager.util.BLMEditorInput");
			
			Bundle pluginNavModel = Platform.getBundle("com.ibm.btools.blm.ui.navigation.model");
			if (pluginNavModel == null){
				return;
			}
			
			java.lang.Class AbstractChildLeafNodeClass = pluginNavModel.loadClass("com.ibm.btools.blm.ui.navigation.model.AbstractChildLeafNode");
			
			// Returns a array of references to open editors in this page. 
			//	Note that each page has its own editors; editors are never shared between pages. 
			IEditorReference[] refs =
				PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getEditorReferences();
			
			IEditorReference ref = null;
			IEditorPart editor = null;
			for (int i = 0; i < refs.length; i++) {
				ref = refs[i];
				editor = ref.getEditor(false);
				if (editor != null) {
					Object BLMEditorInputObj = editor.getEditorInput();
					if (editor.getEditorInput() != null && BLMEditorInputClass.isInstance(BLMEditorInputObj)) {						
						java.lang.Class params[] = {};
						Object paramsObj[] = {};						
						
						Method method_getNode = BLMEditorInputObj.getClass().getMethod("getNode", params);
						
						// Invoke the method
						Object abstractChildLeafNodeObj = method_getNode.invoke(BLMEditorInputObj, paramsObj);
						
						// Call EList node.getEntityReferences(); 
						java.lang.Class params2[] = {};
						Object paramsObj2[] = {};
						
						Method method_getEntityReferences  = AbstractChildLeafNodeClass.getMethod("getEntityReferences", params2);
						EList references = (EList) method_getEntityReferences.invoke(abstractChildLeafNodeObj, paramsObj2);
						if(references.get(0).equals(blmURI)) {							
							editor.getEditorSite().getPage().closeEditor(editor, false);
						}
					}
				}
			}
			
		} catch(ClassNotFoundException e) {
			LoggingUtil.logError(e.getMessage());
		} catch(NoSuchMethodException e) {
			LoggingUtil.logError(e.getMessage());
		} catch(IllegalAccessException e) {
			LoggingUtil.logError(e.getMessage());
		} catch(InvocationTargetException e) {
			LoggingUtil.logError(e.getMessage());
		}
		
	}
}
