package com.ibm.btools.blm.ie.imprt.rule.expressionModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;

import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.expression.command.AddFunctionArgumentDefinitionToFunctionDefinitionEXPCmd;
import com.ibm.btools.expression.command.AddFunctionArgumentToFunctionExpressionEXPCmd;
import com.ibm.btools.expression.command.AddFunctionDefinitionToFunctionExpressionEXPCmd;
import com.ibm.btools.expression.command.UpdateFunctionArgumentDefinitionEXPCmd;
import com.ibm.btools.expression.command.UpdateFunctionArgumentEXPCmd;
import com.ibm.btools.expression.command.UpdateFunctionDefinitionEXPCmd;
import com.ibm.btools.expression.model.Expression;
import com.ibm.btools.expression.model.FunctionArgument;
import com.ibm.btools.expression.model.FunctionArgumentDefinition;
import com.ibm.btools.expression.model.FunctionDefinition;
import com.ibm.btools.expression.model.FunctionExpression;


public class UpdateFunctionExpressionRule extends UpdateExpressionRule {

	//copyright
    static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
    
	private FunctionExpression sourceExpr;
	private FunctionExpression workingCopy;

	/**
	 * Set Original StructuredOpaqueExpression
	 * @param expr
	 */
	public void setSource(final Object expression) {
		this.sourceExpr = (FunctionExpression) expression;
	}
	
	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = (FunctionExpression) workingCopy;
	}
	

	/**
	 * Invoke the rule
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");
		
		AddFunctionDefinitionToFunctionExpressionEXPCmd addfuncDefCmd = 
			new AddFunctionDefinitionToFunctionExpressionEXPCmd(workingCopy);
		try {
			wrapperCmd.appendAndExecute(addfuncDefCmd);
		}catch(RuntimeException ex){
			LoggingUtil.logWarning(getImportSession(), IeMessageKeys.UPDATE_FUNCTION_EXPR_EXCEPTION, new String[]{sourceExpr.getDefinition().getFunctionName()}, ex);
		}		
		
		FunctionDefinition funcDef = sourceExpr.getDefinition();
		UpdateFunctionDefinitionEXPCmd updateFuncDefCmd = new UpdateFunctionDefinitionEXPCmd(workingCopy.getDefinition());
		updateFuncDefCmd.setFunctionID(funcDef.getFunctionID());
		updateFuncDefCmd.setFunctionName(funcDef.getFunctionName());
		updateFuncDefCmd.setReturnType(funcDef.getReturnType());
		try{
			wrapperCmd.appendAndExecute(updateFuncDefCmd);
		}catch(RuntimeException ex){
			LoggingUtil.logWarning(getImportSession(), IeMessageKeys.UPDATE_FUNCTION_EXPR_EXCEPTION, new String[]{funcDef.getFunctionName()}, ex);
		}
		
		
		Iterator it = sourceExpr.getArguments().iterator();
		int i=-1;
		while (it.hasNext()){
			try {			
				AddFunctionArgumentToFunctionExpressionEXPCmd addargCmd = 
					new AddFunctionArgumentToFunctionExpressionEXPCmd(workingCopy);
				wrapperCmd.appendAndExecute(addargCmd);	
			}catch (RuntimeException ex){
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.ADD_FUNCTION_ARG_EXPR_EXCEPTION, new String[]{((FunctionArgument)sourceExpr.getArguments().get(i+1)).getArgumentID()}, ex);
			}
			
			FunctionArgument arg = (FunctionArgument)it.next();
			i++;
			UpdateFunctionArgumentEXPCmd updateArgCmd = new UpdateFunctionArgumentEXPCmd((FunctionArgument)workingCopy.getArguments().get(i));
			updateArgCmd.setArgumentID(arg.getArgumentID());
			
			
			//FunctionArgumentDefinition
			try{
				AddFunctionArgumentDefinitionToFunctionDefinitionEXPCmd addargDefCmd = 
					new AddFunctionArgumentDefinitionToFunctionDefinitionEXPCmd(workingCopy.getDefinition());
				wrapperCmd.appendAndExecute(addargDefCmd);				
			}catch (RuntimeException ex){
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.UPDATE_FUNCTION_ARG_EXPR_EXCEPTION, new String[]{arg.getArgumentID()}, ex);
			}	
			if (arg.getDefinition()!=null){	
				FunctionArgumentDefinition argDef = arg.getDefinition();
				UpdateFunctionArgumentDefinitionEXPCmd updateArgDefCmd = new UpdateFunctionArgumentDefinitionEXPCmd((FunctionArgumentDefinition)workingCopy.getDefinition().getArgumentDefinition().get(i));
				updateArgDefCmd.setArgumentID(argDef.getArgumentID());
				updateArgDefCmd.setArgumentName(argDef.getArgumentName());
				updateArgDefCmd.setArgumentType(argDef.getArgumentType());
				updateArgDefCmd.setRequired(argDef.getRequired());				
				try{
					wrapperCmd.appendAndExecute(updateArgDefCmd);										
				}catch (RuntimeException ex){
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.UPDATE_FUNCTION_ARG_EXPR_EXCEPTION, new String[]{argDef.getArgumentName()}, ex);
				}					
				updateArgCmd.setDefinition((FunctionArgumentDefinition)updateArgDefCmd.getObject());
			}	
			
			try {							
				wrapperCmd.appendAndExecute(updateArgCmd);
			}catch (RuntimeException ex){
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.UPDATE_FUNCTION_ARG_EXPR_EXCEPTION, new String[]{arg.getArgumentID()}, ex);
			}
			
			if (arg.getArgumentValue()!=null){
				Expression argValueExpr =  createAndUpdateExpression(arg.getArgumentValue(), (FunctionArgument)workingCopy.getArguments().get(i), FIRST_OPERAND);
				UpdateFirstOperandRule operandRule = new UpdateFirstOperandRule();
				operandRule.setParentExpression(updateArgCmd.getObject());
				operandRule.setWorkingCopy(argValueExpr);	
				operandRule.invoke();
			}

		}	
		LoggingUtil.traceExit(this, "invoke");
	}

}
