package com.ibm.btools.blm.ie.imprt.rule.expressionModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.process.util.PEDomainViewObjectHelper;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.AbstractLibraryChildNode;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.artifacts.PrimitiveType;
import com.ibm.btools.bom.model.artifacts.Property;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.models.InformationModel;
import com.ibm.btools.bom.model.models.ProcessModel;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.expression.command.AddReferenceStepToModelPathExpressionEXPCmd;
import com.ibm.btools.expression.command.UpdateReferenceStepEXPCmd;
import com.ibm.btools.expression.model.ModelPathExpression;
import com.ibm.btools.expression.model.ReferenceStep;
import com.ibm.btools.expression.model.StaticStep;
import com.ibm.btools.expression.model.Step;


public class UpdateModelPathExpressionRule extends UpdateExpressionRule{
	
	//copyright
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	private ModelPathExpression sourceExpr;
	private ModelPathExpression workingCopy;
	private static EObject prevWorkingCopyObj;
	
	/**
	 * Set Original StructuredOpaqueExpression
	 * @param expr
	 */
	public void setSource(final Object expression) {
		this.sourceExpr = (ModelPathExpression) expression;
	}
	
	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = (ModelPathExpression) workingCopy;
	}
	

	/**
	 * Invoke the rule
	 */
	public void invoke() {	
		LoggingUtil.traceEntry(this, "invoke");
		
		EList stepsList = sourceExpr.getSteps();
		//Step lastStep = (Step)stepsList.get(stepsList.size()-1);
		//if (lastStep instanceof ReferenceStep){
		Step firstStep = (Step)stepsList.get(0);
		if (firstStep instanceof ReferenceStep){
			if ( (((ReferenceStep)firstStep).getReferencedObject()) instanceof ProcessModel ||
			(((ReferenceStep)firstStep).getReferencedObject()) instanceof Activity){
				//expressions in process model mapping
				findProcessModelReferenceSteps(stepsList);
			}else if ((((ReferenceStep)firstStep).getReferencedObject()) instanceof com.ibm.btools.bom.model.artifacts.Class){
				findinfoModelReferenceSteps(stepsList);				
			}else {
				//IndividualResource, bulkResource, role or organization
				findResOrgModelReferenceSteps(stepsList);
			}
		}else if (firstStep instanceof StaticStep){
			LoggingUtil.logWarning(getImportSession(), IeMessageKeys.STATIC_STEP_NOT_SUPPORTED, new String[]{firstStep.getName()}, null);
		}
		LoggingUtil.traceExit(this, "invoke");
	}
	
	private void findinfoModelReferenceSteps(EList stepsList){
		LoggingUtil.traceEntry(this, "findinfoModelReferenceSteps");
		Type masterType = null;
		if ( ((ReferenceStep)stepsList.get(stepsList.size()-1)).getReferencedObject() instanceof Property){				
			Iterator it = stepsList.iterator();
			while (it.hasNext()){
				Step step = (Step)it.next();
				if (step instanceof ReferenceStep){
					if ( (((ReferenceStep)step).getReferencedObject()) instanceof com.ibm.btools.bom.model.artifacts.Class){
						masterType = (Type) BOMUtil.loadRootObject((AbstractLibraryChildNode) importContext.get(((ReferenceStep)step).getReferencedObject()));
						
						//1. Found the first BusinessItem (Class) object 
						//   - find all the owningPackage of this business item
						findAllOwningPackage((InformationModel)masterType.getOwningPackage());
			
						Type businessItem = null;
						//2. Map any other BusuinessItems/Class in the ModelPathExpression
						do {
							businessItem = findBusinessItem( (com.ibm.btools.bom.model.artifacts.Class)((ReferenceStep)step).getReferencedObject());
							step = (Step)it.next();
							if (step instanceof StaticStep){
								LoggingUtil.logWarning(getImportSession(), IeMessageKeys.STATIC_STEP_NOT_SUPPORTED, new String[]{step.getName()}, null);
								break;
							}
						}while ((((ReferenceStep)step).getReferencedObject()) instanceof com.ibm.btools.bom.model.artifacts.Class );
									
						//3. Find the property from the last BusinessItem/Class's ownedAttribute, 
						//   match by name
						do{
							searchClassifiers(((ReferenceStep)step).getReferencedObject(), businessItem);
							if ( (((ReferenceStep)step).getReferencedObject()) instanceof Property){
								businessItem=((Property)((ReferenceStep)step).getReferencedObject()).getType();
							}else{
								//do nothing
							}	
							if (it.hasNext()){
								step = (Step)it.next();															
							}else{
								break;
							}							
						}while( (businessItem!=null) && !(businessItem instanceof PrimitiveType));
						
						break;
					}else{
						//do nothing
					}
				}
			}
			
		}		
		LoggingUtil.traceExit(this, "findinfoModelReferenceSteps");
		
	}
	
	private void findAllOwningPackage(EObject owningPackage){
		LoggingUtil.traceEntry(this, "findAllOwningPackage");
		if (owningPackage instanceof InformationModel){
			if (((InformationModel)owningPackage).getOwningPackage()!=null 
				&& ((InformationModel)owningPackage).getOwningPackage().getOwningPackage()!=null){
				findAllOwningPackage(((InformationModel)owningPackage).getOwningPackage());	
			}
		}
		if (owningPackage!=null){
			try {	
				AddReferenceStepToModelPathExpressionEXPCmd addCmd = 
					new AddReferenceStepToModelPathExpressionEXPCmd(workingCopy);
				//addCmd.setReferencedObject(owningPackage);
				wrapperCmd.appendAndExecute(addCmd);
				UpdateReferenceStepEXPCmd updateCmd = new UpdateReferenceStepEXPCmd((ReferenceStep)addCmd.getObject());
				updateCmd.setReferencedObject(owningPackage);
				wrapperCmd.appendAndExecute(updateCmd);			
			}catch(RuntimeException ex){
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.ADD_STEP_TO_MPE_EXCEPTION, new String[]{}, ex);
			}
		}
		LoggingUtil.traceExit(this, "findAllOwningPackage");
	}
	
	private com.ibm.btools.bom.model.artifacts.Class findBusinessItem(com.ibm.btools.bom.model.artifacts.Class businessItem){
		LoggingUtil.traceEntry(this, "findBusinessItem");
		Type masterType = (Type) BOMUtil.loadRootObject((AbstractLibraryChildNode) importContext.get(businessItem));
		AddReferenceStepToModelPathExpressionEXPCmd addCmd = 
			new AddReferenceStepToModelPathExpressionEXPCmd(workingCopy);
		try {	
			//addCmd.setReferencedObject(masterType);
			wrapperCmd.appendAndExecute(addCmd);	
			UpdateReferenceStepEXPCmd updateCmd = new UpdateReferenceStepEXPCmd((ReferenceStep)addCmd.getObject());
			updateCmd.setReferencedObject(masterType);
			wrapperCmd.appendAndExecute(updateCmd);	
		}catch (RuntimeException ex){
			LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SET_REFERENCE_OBJECT_EXCEPTION, new String[]{}, ex);
		}	
		LoggingUtil.traceExit(this, "findBusinessItem");
		return (com.ibm.btools.bom.model.artifacts.Class)((ReferenceStep)addCmd.getObject()).getReferencedObject();
	}
	
	private void searchClassifiers(EObject obj, Type businessItem){
		LoggingUtil.traceEntry(this, "searchClassifiers");
		boolean isFound = false;
		if ((obj instanceof Property) && (businessItem instanceof com.ibm.btools.bom.model.artifacts.Class)){
			isFound = findProperty(obj, businessItem);
			if (!isFound){				
				com.ibm.btools.bom.model.artifacts.Class bi = (com.ibm.btools.bom.model.artifacts.Class)businessItem;
				if (bi.getSuperClassifier()!=null && !bi.getSuperClassifier().isEmpty()){
					Iterator superIt = bi.getSuperClassifier().iterator();
					while (superIt.hasNext()){
						com.ibm.btools.bom.model.artifacts.Class newBI = (com.ibm.btools.bom.model.artifacts.Class)superIt.next();
						searchClassifiers(obj, newBI);
					}
				}
			}
		}
		LoggingUtil.traceExit(this, "searchClassifiers");
	}
	
	
	private boolean findProperty(EObject obj, Type businessItem){
		LoggingUtil.traceEntry(this, "findProperty");
		if ((obj instanceof Property) && (businessItem instanceof com.ibm.btools.bom.model.artifacts.Class)){
			Iterator it = ((com.ibm.btools.bom.model.artifacts.Class)businessItem).getOwnedAttribute().iterator();			
			while (it.hasNext()){
				EObject attr = (EObject)it.next();
				if (attr instanceof Property){
					if (((Property)attr).getName().equals(((Property)obj).getName())){
						try{
							AddReferenceStepToModelPathExpressionEXPCmd addCmd = new AddReferenceStepToModelPathExpressionEXPCmd(workingCopy);
							//addCmd.setReferencedObject(attr);							
							wrapperCmd.appendAndExecute(addCmd);
							UpdateReferenceStepEXPCmd updateCmd = new UpdateReferenceStepEXPCmd((ReferenceStep)addCmd.getObject());
							updateCmd.setReferencedObject(attr);
							wrapperCmd.appendAndExecute(updateCmd);	
							prevWorkingCopyObj = attr;	
						}catch (RuntimeException ex){
							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.ADD_STEP_TO_MPE_EXCEPTION, new String[]{}, ex);
						}
						LoggingUtil.traceExit(this, "findProperty");	
						return true;
					}
				}
			}	
		}else{
			//do nothing
		}		
		LoggingUtil.traceExit(this, "findProperty");
		return false;		
	}
	
	private void findResOrgModelReferenceSteps(EList stepsList){
		LoggingUtil.traceEntry(this, "findResOrgModelReferenceSteps");
		Iterator it = stepsList.iterator();
		while(it.hasNext()){
			Step step = (Step)it.next();
			EObject workingCopyObj = null;
			if (step instanceof ReferenceStep){
				workingCopyObj = (EObject)getModelFromContext(((ReferenceStep)step).getReferencedObject());
				if (workingCopyObj!=null){					
					try {	
						AddReferenceStepToModelPathExpressionEXPCmd addCmd = new AddReferenceStepToModelPathExpressionEXPCmd(workingCopy);
						wrapperCmd.appendAndExecute(addCmd);
						UpdateReferenceStepEXPCmd updateCmd = new UpdateReferenceStepEXPCmd((ReferenceStep)addCmd.getObject());
						updateCmd.setReferencedObject(workingCopyObj);
						wrapperCmd.appendAndExecute(updateCmd);	
					}catch (RuntimeException ex){
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SET_REFERENCE_OBJECT_EXCEPTION, new String[]{}, ex);
					}	
				}
			}
		}
		LoggingUtil.traceExit(this, "findResOrgModelReferenceSteps");
	}
	

	private void findProcessModelReferenceSteps(EList stepsList){
		LoggingUtil.traceEntry(this, "findprocessModelReferenceSteps");
		Iterator it = stepsList.iterator();	
		prevWorkingCopyObj = null;
		boolean isFirstStep = true;	
		while(it.hasNext()){			
			Step step = (Step)it.next();
			EObject workingCopyObj = null;
			if (step instanceof ReferenceStep){
				workingCopyObj = (EObject)BOMUtil.loadRootObject((AbstractLibraryChildNode) importContext.get(((ReferenceStep)step).getReferencedObject()));
				if (workingCopyObj!=null){					
					try {	
						AddReferenceStepToModelPathExpressionEXPCmd addCmd = new AddReferenceStepToModelPathExpressionEXPCmd(workingCopy);
						if (isFirstStep){
							isFirstStep = false;
							findAllOwningPackage(((PackageableElement)workingCopyObj).getOwningPackage());
						}	
						//addCmd.setReferencedObject(workingCopyObj);
						wrapperCmd.appendAndExecute(addCmd);
						UpdateReferenceStepEXPCmd updateCmd = new UpdateReferenceStepEXPCmd((ReferenceStep)addCmd.getObject());
						updateCmd.setReferencedObject(workingCopyObj);
						prevWorkingCopyObj = workingCopyObj;
						wrapperCmd.appendAndExecute(updateCmd);	
					}catch (RuntimeException ex){
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SET_REFERENCE_OBJECT_EXCEPTION, new String[]{}, ex);
					}
				}else if ( ! ((((ReferenceStep)step).getReferencedObject()) instanceof Property) ){
					workingCopyObj = PEDomainViewObjectHelper.getDomainObject((EObject)importSubContext.get(((ReferenceStep)step).getReferencedObject()));
					if (workingCopyObj!=null){						
						try {							
							AddReferenceStepToModelPathExpressionEXPCmd addCmd = new AddReferenceStepToModelPathExpressionEXPCmd(workingCopy);
							/*
							if (isFirstStep){
								isFirstStep = false;
								if ( ((ReferenceStep)step).getReferencedObject() instanceof StructuredActivityNode){
									EObject activityWorkingCopy = workingCopyObj.eContainer();
									if (activityWorkingCopy instanceof Activity)
										findAllOwningPackage(((PackageableElement)activityWorkingCopy).getOwningPackage());
								}
							}*/
							wrapperCmd.appendAndExecute(addCmd);								
							UpdateReferenceStepEXPCmd updateCmd = new UpdateReferenceStepEXPCmd((ReferenceStep)addCmd.getObject());
							updateCmd.setReferencedObject(workingCopyObj);
							prevWorkingCopyObj = workingCopyObj;
							wrapperCmd.appendAndExecute(updateCmd);
						}catch (RuntimeException ex){
							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SET_REFERENCE_OBJECT_EXCEPTION, new String[]{}, ex);
						}	
					}
				} else {
					
					//if (((ReferenceStep)stepsList.get(stepsList.size()-2)).getReferencedObject() instanceof ObjectPin){
					if (prevWorkingCopyObj!=null && prevWorkingCopyObj instanceof ObjectPin){
						//findProperty((((ReferenceStep)step).getReferencedObject()), ((ObjectPin)((ReferenceStep)stepsList.get(stepsList.size()-2)).getReferencedObject()).getType() );
						searchClassifiers((((ReferenceStep)step).getReferencedObject()), ((ObjectPin)prevWorkingCopyObj).getType() );
					//}else if (((ReferenceStep)stepsList.get(stepsList.size()-2)).getReferencedObject() instanceof Type){
					}else if (prevWorkingCopyObj!=null && prevWorkingCopyObj instanceof Type){
						//findProperty( (((ReferenceStep)step).getReferencedObject()), (Type)((ReferenceStep)stepsList.get(stepsList.size()-2)).getReferencedObject() );
						searchClassifiers( (((ReferenceStep)step).getReferencedObject()), (Type)prevWorkingCopyObj);
					}else if (prevWorkingCopyObj!=null && prevWorkingCopyObj instanceof com.ibm.btools.bom.model.artifacts.Class){
						searchClassifiers((((ReferenceStep)step).getReferencedObject()), (com.ibm.btools.bom.model.artifacts.Class)prevWorkingCopyObj);
					}else if (prevWorkingCopyObj!=null && prevWorkingCopyObj instanceof Property){
						searchClassifiers((((ReferenceStep)step).getReferencedObject()), (com.ibm.btools.bom.model.artifacts.Class)((Property)prevWorkingCopyObj).getType());
					}
					
				}
			}			
		}
		LoggingUtil.traceExit(this, "findprocessModelReferenceSteps");
	}
	
	private Object getModelFromContext(final Object key) {
		LoggingUtil.traceEntry(this, "getModelFromContext");
		ImportContext context = null;
		if (key instanceof PackageableElement) {		
			LoggingUtil.traceExit(this, "getModelFromContext");	
			return BOMUtil.getType(getProjectName(), (PackageableElement)key, importContext);
			
		} else {		
			LoggingUtil.traceExit(this, "getModelFromContext");	
			return importSubContext.get(key);
		}
						
	}	
	
}
