package com.ibm.btools.blm.ie.imprt.rule.externalModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.engine.ProgressCalculator;
import com.ibm.btools.blm.ie.imprt.rule.navigator.CatalogRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.factory.ExternalModelBOFactory;
import com.ibm.btools.blm.ie.imprt.rule.navigator.factory.ExternalModelServiceFactory;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildContainerNode;
import com.ibm.btools.bom.model.models.ExternalModel;
/**
 * This rule handles the ExternalModel BOM element to create catalogs
 * for XML import with respect to external models
 */
public class ExternalModelRule extends CatalogRule {
	//	 copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private ExternalModel newExternalModel;
	private AbstractChildContainerNode node;
	
	/**
	 * Set the source of the rule that's type of ExternalModel
	 */
	public void setSource(final Object newExternalModel){
		this.newExternalModel = (ExternalModel)newExternalModel;
	}
	
	/**
	 * get the new created or retrieved node
	 * @return AbstractChildContainerNode 
	 */	
	public AbstractChildContainerNode getNewExternalModelNode() {
		return this.node;
	}
       /**
		 * Invoke the rule
		 */	
		public void invoke() {
			LoggingUtil.traceEntry(this, "invoke");		
			if (parentNode == null) {
				LoggingUtil.trace(this, "invoke", "The parent node should not be null.");
				LoggingUtil.traceExit(this, "invoke");
				return;
			}
			
			if (newExternalModel == null) {
				LoggingUtil.trace(this, "invoke", "The source should null.");
				LoggingUtil.traceExit(this, "invoke");
				return;
			}
			
			String externalModelName = newExternalModel.getName();
			if (externalModelName == null) {
				LoggingUtil.trace(this, "invoke", "The source name should not be null.");
				LoggingUtil.traceExit(this, "invoke");
				return;
			}
			
			String description = BOMUtil.getDescription(newExternalModel);
			
			boolean resolve = false;
			Object list = getImportSession().getContext().get(IImportFramework.CONTEXT_PREDEFINED_REGISTRY_KEY);
			if (list != null && list instanceof EList) {
				resolve = ((EList) list).contains(newExternalModel);
			}		
			LoggingUtil.trace(this, "invoke", "The resolve value: " + resolve + ".");		
			
			try {
						
				if(NavigatorUtil.isBOCatalogNode((ExternalModel)newExternalModel,getImportSession())){
					ExternalModelBOFactory factory = new ExternalModelBOFactory();
					node = factory.create(
							getProjectName(),
							parentNode, 
							newExternalModel, 
							description, 
							getImportSession(), 
							isDefault, 
							resolve);	
				}else{
					ExternalModelServiceFactory factory = new ExternalModelServiceFactory();
					node = factory.create(
							getProjectName(),
							parentNode, 
							newExternalModel, 
							description, 
							getImportSession(), 
							isDefault, 
							resolve);	
				}			
			} catch (OperationCanceledException e) {			
				throw e;		
			} catch (RuntimeException e) {				
					showMessageBox(e,externalModelName);			
			}
			if (node != null) {			
				getImportSession().getContext().put(newExternalModel, node);				
			}
			NavigatorUtil.save(getProjectName());
			LoggingUtil.trace(this, "invoke", "The navigatior model is saved.");		
			
			IProgressMonitor monitor = getImportSession().getProgressMonitor();
			if (monitor != null) {
				if (monitor.isCanceled()) {
					LoggingUtil.trace(this, "invoke", "The user cancels the operation.");				
					throw new OperationCanceledException();					
				}
				monitor.worked(ProgressCalculator.getSimpleTask());
			}
			LoggingUtil.traceExit(this, "invoke");		
		}	
}
