package com.ibm.btools.blm.ie.imprt.rule.externalModel.bo;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.compoundcommand.util.CCRuntimeException;
import com.ibm.btools.blm.ie.imprt.IImportConstants;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.engine.ProgressCalculator;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.factory.ComplexTypeTemplateFactory;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildContainerNode;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildLeafNode;
import com.ibm.btools.bom.model.artifacts.Class;

/**
 * Import the complexType template from an external schema
 */
public class ComplexTypeTemplateRule extends AbstractImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	//Type of NavigationInlineXSDSchemaNode/NavigationXSDFileNode
	private AbstractChildContainerNode parentNode;	
	private Class newTemplate;
	//Type of NavigationComplexTypeTemplateNode/NavigationInlineComplexTypeTemplateNode
	private AbstractChildLeafNode node;
		
	/**
	 * Set the parent node. It can be either a project node or a data catalog node
	 * @param parentNode
	 */
	public void setParent(final AbstractChildContainerNode parentNode) {
		this.parentNode = parentNode;
	}
	
	/**
	 * Set the category for import
	 * @param newCategory
	 */
	public void setSource(final Object newCategory) {
		this.newTemplate = (Class) newCategory;
	}
	
	/**
	 * Get navigation node
	 * @return
	 */
	public AbstractChildLeafNode getNewTemplateNode() {
		return node;
	}
	
	
	/**
	 * Invoke the rule
	 */	
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");		
		if (parentNode == null) {
			LoggingUtil.trace(this, "invoke", "The parent node should not be null.");
			LoggingUtil.traceExit(this, "invoke");	
			return;
		}
		
		if (newTemplate == null) {
			LoggingUtil.trace(this, "invoke", "The source should null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		String templateName = newTemplate.getName();
		if (templateName == null) {
			LoggingUtil.trace(this, "invoke", "The source name should null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		String description = BOMUtil.getDescription(newTemplate);
		
		boolean resolve = false;
		Object list = getImportSession().getContext().get(IImportFramework.CONTEXT_PREDEFINED_REGISTRY_KEY);
		if (list != null && list instanceof EList) {
			resolve = ((EList) list).contains(newTemplate);
		}		
		LoggingUtil.trace(this, "invoke", "The resolve value: " + resolve + ".");		
		
		try {		
			ComplexTypeTemplateFactory factory = new ComplexTypeTemplateFactory(); 
			node = factory.create(
				parentNode, 
				newTemplate, 
				description, 
				getImportSession(), 
				resolve);
		} catch (OperationCanceledException e) {			
			throw e;		
		} catch (RuntimeException e) {				
				showMessageBox(e,templateName);				
		}
		if (node != null) {
			getImportSession().getContext().put(newTemplate, node);	
			Class thisObject = (Class)BOMUtil.loadRootObject(node);	
			//AZ d141256 update aspect for the template
			thisObject.setAspect(newTemplate.getAspect());		
			registerTemplate(thisObject);
		}
		
		NavigatorUtil.save(getProjectName());
		LoggingUtil.trace(this, "invoke", "The navigatior model is saved.");		
		
		IProgressMonitor monitor = getImportSession().getProgressMonitor();
		if (monitor != null) {
			if (monitor.isCanceled()) {
				LoggingUtil.trace(this, "invoke", "The user cancels the operation.");				
				throw new OperationCanceledException();					
			}
			monitor.worked(ProgressCalculator.getSimpleTask());
		}
		LoggingUtil.traceExit(this, "invoke");		
	}
	
}
