package com.ibm.btools.blm.ie.imprt.rule.externalModel.bo;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.compoundcommand.util.CCRuntimeException;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.engine.ProgressCalculator;
import com.ibm.btools.blm.ie.imprt.rule.navigator.CatalogRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.factory.ExternalSchemaFactory;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildContainerNode;
import com.ibm.btools.bom.model.externalmodels.ExternalSchema;
import com.ibm.btools.util.exception.BTRuntimeException;

/**
 * Import the information model either from the root or from 
 * another information model
 */
public class ExternalSchemaRule extends CatalogRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private ExternalSchema newExternalSchemaModel;
	private AbstractChildContainerNode node;
	
	/**
	 * Set the information model for import
	 * @param newInformationModel
	 */
	public void setSource(final Object newExternalSchemaModel) {
		this.newExternalSchemaModel = (ExternalSchema)newExternalSchemaModel;
	}
	
	/**
	 * Get navigation node
	 * @return
	 */
	public AbstractChildContainerNode getNewExternalSchemaNode() {
		return node;
	}
	
	
	/**
	 * Invoke the rule
	 */	
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");		
		if (parentNode == null) {
			LoggingUtil.trace(this, "invoke", "The parent node should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		if (newExternalSchemaModel == null) {
			LoggingUtil.trace(this, "invoke", "The source should null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		String externalSchemaName = newExternalSchemaModel.getName();
		if (externalSchemaName == null) {
			LoggingUtil.trace(this, "invoke", "The source name should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		String description = BOMUtil.getDescription(newExternalSchemaModel);
		
		boolean resolve = false;
		Object list = getImportSession().getContext().get(IImportFramework.CONTEXT_PREDEFINED_REGISTRY_KEY);
		if (list != null && list instanceof EList) {
			resolve = ((EList) list).contains(newExternalSchemaModel);
		}		
		LoggingUtil.trace(this, "invoke", "The resolve value: " + resolve + ".");		
		
		try {		
			ExternalSchemaFactory factory = new ExternalSchemaFactory();
			node = factory.create(
				getProjectName(),
				parentNode, 
				newExternalSchemaModel, 
				description, 
				getImportSession(), 
				isDefault, 
				resolve);
			
		BOMUtil.updateExternalDocument(newExternalSchemaModel,getProjectName(),node, getImportSession());
		
		} catch (OperationCanceledException e) {			
			throw e;		
		} catch (RuntimeException e) {				
				showMessageBox(e,externalSchemaName);			
		}
		if (node != null) {			
			getImportSession().getContext().put(newExternalSchemaModel, node);				
		}
		NavigatorUtil.save(getProjectName());
		LoggingUtil.trace(this, "invoke", "The navigatior model is saved.");		
		
		IProgressMonitor monitor = getImportSession().getProgressMonitor();
		if (monitor != null) {
			if (monitor.isCanceled()) {
				LoggingUtil.trace(this, "invoke", "The user cancels the operation.");				
				throw new OperationCanceledException();					
			}
			monitor.worked(ProgressCalculator.getSimpleTask());
		}
		LoggingUtil.traceExit(this, "invoke");		
	}

}
