package com.ibm.btools.blm.ie.imprt.rule.informationModel;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.bom.command.artifacts.AddCommentToElementBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddPropertyToClassBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddStateToClassBOMCmd;
import com.ibm.btools.bom.command.artifacts.RemoveCommentBOMCmd;
import com.ibm.btools.bom.command.artifacts.RemovePropertyBOMCmd;
import com.ibm.btools.bom.command.artifacts.RemoveStateBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateCommentBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdatePropertyBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateStateBOMCmd;
import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.Element;
import com.ibm.btools.bom.model.artifacts.Property;
import com.ibm.btools.bom.model.artifacts.State;
import com.ibm.btools.emf.cf.BtCompoundCommand;

/**
 * Updates a BOM Business Item
 */
public class UpdateBusinessItemRule extends UpdateClassifierRule {
	
	//copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private List templateProperties = new ArrayList();
	private boolean isBIRuleForSuperClassifier = false;
	/**
	 * Invoke the rule
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");
		//AZ 20070918 We just walk up to one level for the superclassifier to
		//avoid infinit loop in case of a circular reference for parent template
		if(!isBIRuleForSuperClassifier()){
			super.invoke();	
		}
		if(NavigatorUtil.isNewNode(getImportSession(),classifier)){
		//AZ defend the update for template overwrite
			if(((Class) workingCopy).getOwnedAttribute().size() == 0){
				createOperations();
				createPossibleStates();
				createProperties();
			}
		}else{
			if(NavigatorUtil.isOptionOverwrite(getImportSession().getImportOptions(),classifier)){			
				consolidateBusinessItem();
				createOperations();
				createPossibleStates();
				createProperties();
				templateProperties.clear();			
			}else if(NavigatorUtil.isOptionIgnore(getImportSession().getImportOptions(),classifier)){
				//no-op
			}			
		}
//		createPossibleStates();
		createOperations();
		LoggingUtil.traceExit(this, "invoke");
	}	
	
	/*
	 * This method is to compare two business items when the 
	 * user chooses to overwriter the exising one in the working 
	 * space. Source is the new BOM imported into the memory
	 * target is the one existed into the workspace. We need to compare 
	 * and refactor the properties
	 */
	private void consolidateBusinessItem(){
		Class source = (Class)super.classifier;
		Class target = (Class)super.workingCopy;		
		
		if(source == null || target == null){
			LoggingUtil.trace(this, "consolidateBusinessItem", "The workingCopy should not be null.");
			return;
		}
		
		List sourceAttrs = new ArrayList(source.getOwnedAttribute());
		List targetAttrs = new ArrayList(target.getOwnedAttribute());
		List  removable = new ArrayList();		
		
		if(sourceAttrs == null || sourceAttrs.size() == 0){
			if(targetAttrs != null && targetAttrs.size() > 0){
				for(Iterator it2 = targetAttrs.iterator();it2.hasNext();){
					EObject target2 = (EObject)it2.next();	
					if(!removable.contains(target2)){
						removable.add(target2);	
					}
				}
			}
		}
		
		for(Iterator it = sourceAttrs.iterator();it.hasNext();){	    	
			Property sourceAttribute = (Property)it.next();	
			for(Iterator it2 = targetAttrs.iterator();it2.hasNext();){
				Property targetAttribute = (Property)it2.next();
				if(nameMatch(sourceAttribute,targetAttribute)){	  				
					//update the contents but retain the object reference
					updateCollidedProperty(sourceAttribute,targetAttribute);
					//remove the source, but template is shared by other business object
					//so we keep the properties in a list, so that in the subsequent property
					//creation process, we can skip them.
					if(!classifier.getIsAbstract().booleanValue()){
						if(!removable.contains(sourceAttribute)){
							removable.add(sourceAttribute);
						}
					}else{
						if(!templateProperties.contains(sourceAttribute)){
							templateProperties.add(sourceAttribute);
						}
					}
					//break;
				}
				if(isRemovedPrperty(targetAttribute,sourceAttrs)){
					if(!removable.contains(targetAttribute)){
						removable.add(targetAttribute);
					}
				}	
			}	     
		}
		executeRemovable(removable);
	}
	
	/*
	 * There are several scenarios:
	 * 1. The imported business item has new attributes
	 * 2. The imported business item has less attributes
	 * 3. The imported business item has the attribute name as 
	 * the exisintg one's, but with different type
	 * 4. The imported business item has the same attirubte name and type
	 * but different attibute properties like readOnly, static, mutilplicity
	 * 5. The imported one is identical to the exisiting one
	 * 
	 */
	
	private boolean nameMatch(Property source,Property target){
		if(source.getName() != null && source.getName().equals(target.getName())){
			return true;
		}
		return false;
	}
	
	private boolean isRemovedPrperty(Property target, List sourceAttributes ){
		if(target == null ||sourceAttributes == null || sourceAttributes.size() == 0){
			return false;
		}
		// name search on the sourceAttributes		
		String name = target.getName();
		for(Iterator it= sourceAttributes.iterator();it.hasNext();){
			Property sourceProperty = (Property)it.next();
			if(name != null && name.equalsIgnoreCase(sourceProperty.getName())){
				//got a match
				return false;
			}
		}
		return true;	 
	}
	
	private void executeRemovable(List propertyList){
		if(propertyList == null || propertyList.size() == 0){
			return;
		}
		for(Iterator it=propertyList.iterator();it.hasNext();){
			Property removed = (Property)it.next();	
			EObject container = removed.eContainer();
			if(container instanceof Class){
				super.forkUICloseEditor(((Class)container).getUid());
			}
			
			RemovePropertyBOMCmd removeCmd = new RemovePropertyBOMCmd(removed);
			if(removeCmd.canExecute()){
				removeCmd.execute();
			}
		}			
	}
	
	/**
	 * Creates the prossible states list of this business item if the list is empty
	 */
	private void createPossibleStates() {
		LoggingUtil.traceEntry(this, "createPossibleStates");
		List pStates = ((Class)workingCopy).getPossibleStates();
		if((pStates == null) || pStates.isEmpty()) {
			EList states =((Class) classifier).getPossibleStates();
			int stateSize = states.size();
			AddStateToClassBOMCmd addStateCmd = null;
			State listElement = null;
			HashMap stateMap = (HashMap)getImportSession().getContext().get(STATE_MAP);
			if(stateMap == null) {
				stateMap = new HashMap();
			}
			for(int i = 0; i < stateSize; i++) {
				listElement = (State)states.get(i);
				if(stateMap.get(listElement) == null) {
					addStateCmd = new AddStateToClassBOMCmd((Class)workingCopy);
					addStateCmd.setName(listElement.getName());
					if(addStateCmd.canExecute()) {
						addStateCmd.execute();
						// Fix d74289: NullPointerException when xml-import tasks with input/output pins
						//             with business item states without comments. 
						stateMap.put(listElement, addStateCmd.getObject());
						
						AddCommentToElementBOMCmd addComCmd = new AddCommentToElementBOMCmd((Element)addStateCmd.getObject());
						if(listElement.getOwnedComment() == null || listElement.getOwnedComment().isEmpty()){
							continue;
						}
						addComCmd.setBody(((Comment)listElement.getOwnedComment().get(0)).getBody());
						if(addComCmd.canExecute()) {
							addComCmd.execute();
						}
					}
				}
			}
			getImportSession().getContext().put(STATE_MAP, stateMap);
		} else {
			EList states =((Class) classifier).getPossibleStates();
			int stateSize = states.size();
			HashMap stateMap = (HashMap)getImportSession().getContext().get(STATE_MAP);
			if(stateMap == null) {
				stateMap = new HashMap();
			}
			consolidateStates(pStates, states, stateSize, stateMap);		
			getImportSession().getContext().put(STATE_MAP, stateMap);		
		}		
	}
    
	/**
	 * This method handles collided <code>State</code> by name or add new incoming states
	 * @param pStates
	 * @param states
	 * @param stateSize
	 * @param stateMap
	 */
	private void consolidateStates(List pStates, EList states, int stateSize, HashMap stateMap) {
		AddStateToClassBOMCmd addStateCmd = null;
		State listElement = null;
		List<State> removedStates = new ArrayList<State>();
		searchForRemovedStates(removedStates,states,pStates);
		if(removedStates.size()>0){
			for(State removedState:removedStates){
				EObject container = removedState.eContainer();
				if(container instanceof Class){
					super.forkUICloseEditor(((Class)container).getUid());
				}
				RemoveStateBOMCmd removeCMD = new RemoveStateBOMCmd(removedState);
				if(removeCMD.canExecute()){
					removeCMD.execute();
				}
			}
		}	
		for(int i = 0; i < stateSize; i++) {
			listElement = (State)states.get(i);
			Iterator it = pStates.iterator();
			boolean stateAlreadyExists = false;
			stateAlreadyExists = refactorExistingState(listElement, it, stateAlreadyExists);
			if (!stateAlreadyExists) {
				addStateCmd = new AddStateToClassBOMCmd((Class)workingCopy);
				addStateCmd.setName(listElement.getName());
				if(addStateCmd.canExecute()) {
					addStateCmd.execute();
					AddCommentToElementBOMCmd addComCmd = new AddCommentToElementBOMCmd((Element)addStateCmd.getObject());
					if(listElement.getOwnedComment() == null || listElement.getOwnedComment().isEmpty()){
						continue;
					}
					addComCmd.setBody(((Comment)listElement.getOwnedComment().get(0)).getBody());
					if(addComCmd.canExecute()) {
						addComCmd.execute();
						stateMap.put(listElement, addComCmd.getObject().eContainer());
					}
				}
			}
		}
	}
    
	/**
	 * This method is to find out the existing state and update the comment if necessary
	 * @param listElement
	 * @param it
	 * @param stateAlreadyExists
	 * @return
	 */
	private boolean refactorExistingState(State listElement, Iterator it, boolean stateAlreadyExists) {
		while(it.hasNext()) {
			State pState = (State)it.next();
			if (listElement.getName().equals(pState.getName())) {
				stateAlreadyExists = true;
				//Update comment and icon
				if(pState.getOwnedComment()!= null && pState.getOwnedComment().size()==1){
					Comment comment = (Comment)pState.getOwnedComment().get(0);
					RemoveCommentBOMCmd removeCommentCMD = new RemoveCommentBOMCmd(comment);
					if(removeCommentCMD.canExecute()){
						removeCommentCMD.execute();
					}
				}						
				if(listElement.getOwnedComment() == null || listElement.getOwnedComment().isEmpty()){
					continue;
				}
				AddCommentToElementBOMCmd addComCmd = new AddCommentToElementBOMCmd(pState);
				addComCmd.setBody(((Comment)listElement.getOwnedComment().get(0)).getBody());
				if(addComCmd.canExecute()) {
					addComCmd.execute();							
				}						
				break;
			}
		}
		return stateAlreadyExists;
	}
	
	/**
	 * Search for state that should be removed.
	 * @param removedStates
	 * @param incomingStates
	 * @param targetStates
	 */
	private void searchForRemovedStates(List<State> removedStates,List<State> incomingStates, List<State> targetStates){
		if(removedStates == null ||incomingStates==null|| targetStates == null){
			return;
		}
		if(incomingStates.size()==0){
			if(targetStates.size() > 0){
				for(State eachState:targetStates){
					if(!removedStates.contains(eachState)){
						removedStates.add(eachState);	
					}
				}
			}
		}else{
			for(State target:targetStates){
				String name = target.getName();
				boolean matched = false;
				for(State source:incomingStates){				
					if(name != null && name.equalsIgnoreCase(source.getName())){
						matched = true;
						break;					
				     }
				}
				if(!matched){
					if(!removedStates.contains(target)){
						removedStates.add(target);	
					}
				}
			}				 
		}
	}
	
	/**
	 * Creates the properties of this business item
	 */
	private void createProperties() {
		LoggingUtil.traceEntry(this, "createProperties");
		EList attributes =((Class) classifier).getOwnedAttribute();
		Iterator iterator = attributes.iterator();		
		Property property = null;
		Property workingProperty = null;
		AddPropertyToClassBOMCmd addCmd = null;	
		UpdatePropertyBOMCmd updateCmd = null;
		while(iterator.hasNext()) {
			// Property from the ImportOperation
			property = (Property) iterator.next();
			if(templateProperties.contains(property)){
				continue;
			}
			addCmd = new AddPropertyToClassBOMCmd((Class)workingCopy);
			//name should be set first so if the command may can catch duplicate names.. 
			addCmd.setName(property.getName());
			
			if (addCmd.canExecute()) {
				try{
					addCmd.execute();
					// newly created Property
					workingProperty = (Property)addCmd.getObject();
					// Call the update property rule to update the property
					UpdatePropertyRule rule = new UpdatePropertyRule();
					rule.setImportSession(getImportSession());
					rule.setProjectName(getProjectName());
					rule.setSource(property);
					rule.setWorkingCopy(workingProperty);
					rule.invoke();	
				}catch(RuntimeException ex){
					//log an error to the file and skip the property
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.BUSINESS_ITEM_PROPERTY_EXCEPTION, new String[]{classifier.getName()}, ex);
				}
			} else {
				//log an error to the user and skip the property
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.BUSINESS_ITEM_PROPERTY_EXCEPTION, new String[]{classifier.getName()}, null );
				continue;
			}
		}
		LoggingUtil.traceExit(this, "createProperties");
	}
	
	/**
	 * Update the collided attribute contents
	 * @param source
	 * @param target
	 */
	private void updateCollidedProperty(Property source, Property target){
		UpdatePropertyRule rule = new UpdatePropertyRule();
		rule.setImportSession(getImportSession());
		rule.setProjectName(getProjectName());
		rule.setSource(source);
		rule.setWorkingCopy(target);	
		rule.invoke();	
	}
	
	/**
	 * Create the operations for this business Item.
	 *
	 */
	private void createOperations() {			
		EList operations =((Class) classifier).getOwnedOperation();
		if(operations != null && ! operations.isEmpty()){
			LoggingUtil.trace(this, "createOperations", "Unsupported operation for Business Items encountered, will skip");
			//log error... operation not supported by the tool, will be skipped
		}
	}

	public boolean isBIRuleForSuperClassifier() {
		return isBIRuleForSuperClassifier;
	}

	public void setBIRuleForSuperClassifier(boolean isBIRuleForSuperClassifier) {
		this.isBIRuleForSuperClassifier = isBIRuleForSuperClassifier;
	}
}
