package com.ibm.btools.blm.ie.imprt.rule.informationModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.bom.command.artifacts.AddCommentToElementBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddInstanceValueToSlotBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralNullToSlotBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralStringToSlotBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddOpaqueExpressionToSlotBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddSlotToInstanceSpecificationBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddUpdateValueSpecificationBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateCommentBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateInstanceSpecificationBOMCmd;
import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.bom.model.artifacts.Classifier;
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.InstanceSpecification;
import com.ibm.btools.bom.model.artifacts.InstanceValue;
import com.ibm.btools.bom.model.artifacts.LiteralNull;
import com.ibm.btools.bom.model.artifacts.LiteralSpecification;
import com.ibm.btools.bom.model.artifacts.OpaqueExpression;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.artifacts.Property;
import com.ibm.btools.bom.model.artifacts.Signal;
import com.ibm.btools.bom.model.artifacts.Slot;
import com.ibm.btools.bom.model.artifacts.StructuralFeature;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.organizationstructures.LocationType;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnitType;
import com.ibm.btools.bom.model.resources.BulkResourceType;
import com.ibm.btools.bom.model.resources.IndividualResourceType;
import com.ibm.btools.emf.cf.BtCompoundCommand;
import com.ibm.btools.expression.bom.command.AddStructuredOpaqueExpressionToSlotBEXCmd;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;



public class UpdateInstanceSpecificationRule extends AbstractImportRule{
	
	//copyright
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	protected InstanceSpecification instanceSample;
	protected InstanceSpecification workingCopy;
	protected boolean newSlots = true;

	/**
	 * Invokes the rule
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");
		updateInstanceClassifier();
		updateInstanceProperties();
		LoggingUtil.traceExit(this, "invoke");
	
	}
	
	/**
	 * Set original InstanceSpecification
	 * @param sample BOM InstanceSpecification
	 */
	public void setSource(final Object sample) {
		this.instanceSample = (InstanceSpecification) sample;
	}
	
	
	/**
	 * Set the InstanceSpecification working copy
	 * @param workingCopy a BOM InstanceSpecification
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = (InstanceSpecification) workingCopy;
	}
	
	/**
	 * Updates the <code>workingCopy</code> InstanceSpecification with the Classifier. 
	 *
	 */
	protected void updateInstanceClassifier(){
		LoggingUtil.traceEntry(this, "updateInstanceClassifier");
		// the superClassifier may have beed updated by the CreateNavigationCmd ? (step 1 of step 2)
		if(workingCopy.getClassifier()!= null && !workingCopy.getClassifier().isEmpty() && workingCopy.getClassifier().get(0) != null){
			newSlots = false;
			return;
		}
		//first set the type
		EList classifiers = instanceSample.getClassifier();
		if(classifiers != null && classifiers.size()>1){
			LoggingUtil.trace(this, "updateClassifierProperties", "Multiple superclassifier not supported.");
		}else if(classifiers.isEmpty()){
			LoggingUtil.trace(this, "updateClassifierProperties", "No classifier was found.");
			return;
		}else{
			Classifier classifier = (Classifier)instanceSample.getClassifier().get(0);
			Classifier workingClassifier = (Classifier)BOMUtil.getType(getProjectName(), classifier, getImportSession().getContext());
			if(workingClassifier != null){
				UpdateInstanceSpecificationBOMCmd updateInstanceCmd = new UpdateInstanceSpecificationBOMCmd(workingCopy);
				if (workingCopy.getName() == null) {				
					updateInstanceCmd.setName(instanceSample.getName());
				}
				updateInstanceCmd.addClassifier(workingClassifier);
				if(updateInstanceCmd.canExecute()){
					try{
						updateInstanceCmd.execute();
					}catch(RuntimeException ex){
						LoggingUtil.logError(IeMessageKeys.INSTANCE_SUPER_CLASSIFIER_EXCEPTION, new String[]{instanceSample.getName()}, ex );
					}
				}else{
					LoggingUtil.logError(IeMessageKeys.INSTANCE_SUPER_CLASSIFIER_EXCEPTION, new String[]{instanceSample.getName()}, null);
				}
			}else{
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.OBJECT_REFERENCE_NOT_FOUND_EXCEPTION, new String[]{"Classifier",classifier.getName()}, null);
			}
		}
		LoggingUtil.traceExit(this, "updateInstanceClassifier");
	}

	/**
	 * Updates the slots of the <code>workingCopy</code> InstanceSpecification
	 */
	protected void updateInstanceProperties(){
		EList slots = instanceSample.getSlot();
		if(slots != null && !slots.isEmpty()){
			Iterator it = slots.iterator();
			while (it.hasNext()){
				Slot slot = (Slot)it.next();
				updateSlot(slot);
			}
		}
	}
	
	private void updateSlot(Slot slot){
		LoggingUtil.traceEntry(this, "updateSlot");
			if(slot.getValue() == null && slot.getValue().isEmpty()){
				LoggingUtil.traceExit(this, "updateSlot");
				return;
			}
		Slot workingSlot = getWorkingSlot(slot);
		if(workingSlot != null){
			updateSlotWithValues(slot.getValue(), workingSlot);
		}
		LoggingUtil.traceEntry(this, "updateSlot");
		
	}
	
	private Slot getWorkingSlot(Slot slot){
		LoggingUtil.traceEntry(this, "getWorkingSlot");
		if(slot.getValue() == null && slot.getValue().isEmpty()){
			LoggingUtil.traceExit(this, "getWorkingSlot");
			return null;
		}
		Slot workingSlot = null;
		workingSlot = getSlot(workingCopy, slot);
		if(workingSlot != null){
			return workingSlot;
		}else{
			StructuralFeature definingFeature = slot.getDefiningFeature();
			if(definingFeature instanceof Property){
				AddSlotToInstanceSpecificationBOMCmd addSlotCmd = new AddSlotToInstanceSpecificationBOMCmd(workingCopy);
				Property property = (Property)slot.getDefiningFeature();
				Property workingProperty = getWorkingAttribute(property);
				addSlotCmd.setDefiningFeature(workingProperty);
				if(addSlotCmd.canExecute()){
					try{
						addSlotCmd.execute();
					}catch(RuntimeException ex){
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_SLOT_CREATION_EXCEPTION, new String[]{slot.getDefiningFeature().getName()}, ex );
					}
					return (Slot)addSlotCmd.getObject(); 
				}else{
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_SLOT_CREATION_EXCEPTION, new String[]{slot.getDefiningFeature().getName()}, null );
				}
			}
		}
		return null;
	}
	
	/** 
	 * Checks if a slot for the correspondent <code> slot</code> defininingFeature already was created, for example 
	 * by the CreateInstanceSpecificationCmd. 
	 * If it was created, will return the slot in the workingCopy, else, will return null.
	 * @param workingCopy the BOM InstanceSpecification workingCopy
	 * @param slot a source Slot
	 * @return workingSlot
	 */
	
	private Slot getSlot(InstanceSpecification workingCopy, Slot slot){
		if(newSlots){
			return null;
		}
		List slots = workingCopy.getSlot();
		Slot workingSlot = null;
		if(slots != null && ! slots.isEmpty()){
			Iterator it = slots.iterator();
			Slot thisSlot = null;
			while(it.hasNext()){
				thisSlot = (Slot)it.next();
				if(thisSlot.getDefiningFeature().equals(getWorkingAttribute((Property)slot.getDefiningFeature()))){
					workingSlot = thisSlot;
					break;
				}
			}
		}
		return workingSlot;
	}

	/**
	 * Updates the <code>workingSlot</code> with <code>values</code>. This is done through a CompoundCommand.
	 * A ValueSpecification for a slot should always be updated with a value. 
	 * @param values a list of BOM ValuesSpecifications
	 * @param workingSlot BOM Slot
	 */
	private void updateSlotWithValues(List values, Slot workingSlot){
		LoggingUtil.traceEntry(this, "updateSlotWithValues");
		if(values != null && ! values.isEmpty()){
			Iterator it = values.iterator();
			while(it.hasNext()){
				BtCompoundCommand wrapperCmd = new BtCompoundCommand();
				ValueSpecification valueSpec = (ValueSpecification)it.next();
				AddUpdateValueSpecificationBOMCmd addValueSpecCmd = null;
				if(valueSpec instanceof LiteralNull){
					addValueSpecCmd = new AddLiteralNullToSlotBOMCmd(workingSlot);
				}
				else if(valueSpec instanceof LiteralSpecification){
					addValueSpecCmd = new AddLiteralStringToSlotBOMCmd(workingSlot);
				}else if (valueSpec instanceof InstanceValue){
					addValueSpecCmd = new AddInstanceValueToSlotBOMCmd(workingSlot);					
				}else if (valueSpec instanceof StructuredOpaqueExpression) {
					addValueSpecCmd = new AddStructuredOpaqueExpressionToSlotBEXCmd(workingSlot);	
				}else if (valueSpec instanceof OpaqueExpression){
					addValueSpecCmd = new AddOpaqueExpressionToSlotBOMCmd(workingSlot);
				}
				if(!wrapperCmd.appendAndExecute(addValueSpecCmd)){
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_SLOT_CREATION_EXCEPTION, new String[]{workingSlot.getDefiningFeature().getName()}, null);
				}else{
					ValueSpecification workingValueSpecification = (ValueSpecification)addValueSpecCmd.getObject();
					UpdateValueSpecificationRule updateValueSpecRule = new UpdateValueSpecificationRule();
					updateValueSpecRule.setImportSession(getImportSession());
					updateValueSpecRule.setProjectName(getProjectName());
					updateValueSpecRule.setSource(valueSpec);
					updateValueSpecRule.setWorkingCopy(workingValueSpecification);
					updateValueSpecRule.setWrapperCommand(wrapperCmd);
					updateValueSpecRule.invoke();
				}
			}
		}
		LoggingUtil.traceExit(this, "updateSlotWithValues");
	}
	
//	/**
//	 * Updates the <code>workingSlot</code> with <code>values</code>. This is done through a CompoundCommand.
//	 * A ValueSpecification for a slot should always be updated with a value. 
//	 * @param values a list of BOM ValuesSpecifications
//	 * @param workingSlot BOM Slot
//	 */
//	private void updateSlotWithValues(List values, Slot workingSlot){
//		LoggingUtil.traceEntry(this, "updateSlotWithValues");
//		if(values != null && ! values.isEmpty()){
//			Iterator it = values.iterator();
//			while(it.hasNext()){
//				BtCompoundCommand wrapperCmd = new BtCompoundCommand();
//				ValueSpecification valueSpec = (ValueSpecification)it.next();
//				AddUpdateValueSpecificationBOMCmd addValueSpecCmd = null;
//				if(valueSpec instanceof LiteralSpecification){
//					LiteralSpecification literalSpec = (LiteralSpecification)valueSpec;
//					if(literalSpec instanceof LiteralString){
//						addValueSpecCmd = new AddLiteralStringToSlotBOMCmd(workingSlot);
//					}else if (literalSpec instanceof LiteralUnlimitedNatural){
//						addValueSpecCmd = new AddLiteralUnlimitedNaturalToSlotBOMCmd(workingSlot);
//					}else if(literalSpec instanceof LiteralNull){
//						addValueSpecCmd = new AddLiteralNullToSlotBOMCmd(workingSlot);
//					}else if (literalSpec instanceof LiteralInteger){
//						addValueSpecCmd = new AddLiteralIntegerToSlotBOMCmd(workingSlot);
//					}else if (literalSpec instanceof LiteralBoolean){
//						addValueSpecCmd = new AddLiteralBooleanToSlotBOMCmd(workingSlot);
//					}else if (literalSpec instanceof LiteralTime){
//						addValueSpecCmd = new AddLiteralTimeToSlotBOMCmd(workingSlot);
//					}else if (literalSpec instanceof LiteralDuration){
//						addValueSpecCmd = new AddLiteralDurationToSlotBOMCmd(workingSlot);
//					}else if (literalSpec instanceof LiteralReal){
//						addValueSpecCmd = new AddLiteralRealToSlotBOMCmd(workingSlot);
//					}
//					
//				}else if (valueSpec instanceof InstanceValue){
//					addValueSpecCmd = new AddInstanceValueToSlotBOMCmd(workingSlot);
//					
//				}else if (valueSpec instanceof OpaqueExpression){
//					addValueSpecCmd = new AddOpaqueExpressionToSlotBOMCmd(workingSlot);
//				}
//				
//				if(!wrapperCmd.appendAndExecute(addValueSpecCmd)){
//					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_SLOT_UPDATE_EXCEPTION, new String[]{workingSlot.getDefiningFeature().getName()}, null);
//				}else{
//					ValueSpecification workingValueSpecification = (ValueSpecification)addValueSpecCmd.getObject();
//					UpdateValueSpecificationRule updateValueSpecRule = new UpdateValueSpecificationRule();
//					updateValueSpecRule.setImportSession(getImportSession());
//					updateValueSpecRule.setProjectName(getProjectName());
//					updateValueSpecRule.setSource(valueSpec);
//					updateValueSpecRule.setWorkingCopy(workingValueSpecification);
//					updateValueSpecRule.setWrapperCommand(wrapperCmd);
//					updateValueSpecRule.invoke();
//				}
////				if(addValueSpecCmd.canExecute()){
////					try{
////						addValueSpecCmd.execute();			
////						ValueSpecification workingValueSpecification = (ValueSpecification)addValueSpecCmd.getObject();
////						UpdateValueSpecificationRule updateValueSpecRule = new UpdateValueSpecificationRule();
////						updateValueSpecRule.setImportSession(getImportSession());
////						updateValueSpecRule.setSource(valueSpec);
////						updateValueSpecRule.setWorkingCopy(workingValueSpecification);
////						updateValueSpecRule.invoke();
////					}catch(RuntimeException ex){
////						LoggingUtil.logWarning(getImportSession(), MessageKeys.INSTANCE_SLOT_UPDATE_EXCEPTION, new String[]{workingSlot.getDefiningFeature().getName()}, ex );
////					}
////				}else{
////					LoggingUtil.logWarning(getImportSession(), MessageKeys.INSTANCE_SLOT_UPDATE_EXCEPTION, new String[]{workingSlot.getDefiningFeature().getName()}, null);
////				}
//			}
//		}
//		LoggingUtil.traceExit(this, "updateSlotWithValues");
//	}

	/**
	 * Retrieves the BOM Property in the <code>workingClassifier</code> with the same name as
	 * the <code>attribute</code>
	 * @param workingClassifier a BOM Classifier. 
	 * @param attribute a BOM Property
	 * @return Property - a Property of the <code>workingClassifier</code> with the same name
	 * as the </code>attribute</code>
	 */
	private Property getWorkingAttribute(Property attribute){
		LoggingUtil.traceEntry(this, "getWorkingAttribute");
		if(attribute == null){
			return null;
		}
		Property property = null;
		PackageableElement classifier = (PackageableElement)attribute.eContainer();
		Classifier workingClassifier = (Classifier)BOMUtil.getType(getProjectName(), classifier, getImportSession().getContext());
		if(workingClassifier == null){
			LoggingUtil.logWarning(getImportSession(), IeMessageKeys.OBJECT_REFERENCE_NOT_FOUND_EXCEPTION, new String[]{"Classifier", classifier.getName()}, null);
			return null; 
		}
		EList attributes = null;
		if(workingClassifier instanceof Class){
			attributes = ((Class)workingClassifier).getOwnedAttribute();
		}else if (workingClassifier instanceof Signal){
			attributes = ((Signal)workingClassifier).getOwnedAttribute();
		}else if(workingClassifier instanceof IndividualResourceType){
			attributes = ((IndividualResourceType)workingClassifier).getOwnedAttribute();
		}else if(workingClassifier instanceof BulkResourceType){
			attributes = ((BulkResourceType)workingClassifier).getOwnedAttribute();
		}else if(workingClassifier instanceof OrganizationUnitType){
			attributes = ((OrganizationUnitType)workingClassifier).getOwnedAttribute();
		}else if(workingClassifier instanceof LocationType){
			attributes = ((LocationType)workingClassifier).getOwnedAttribute();
		}
		if(attributes != null && !attributes.isEmpty()){
			Iterator it = attributes.iterator();
			Property workingAttribute = null;
			while(it.hasNext()){
				workingAttribute= (Property)it.next();
				if(workingAttribute.getName().equals(attribute.getName())){
					property = workingAttribute;
					break;
				}
			}
		}
		LoggingUtil.traceEntry(this, "getWorkingAttribute");
		return property;
	}
	

	/**
	 * Updates the <code>workingCopy</code> with comments.
	 *
	 */
	private void updateComments(){
		EList comments = instanceSample.getOwnedComment();
		if(comments != null && !comments.isEmpty()){
			Iterator it = comments.iterator();
			while(it.hasNext()){
				Comment comment = (Comment)it.next();
				AddCommentToElementBOMCmd addCmd = new AddCommentToElementBOMCmd(workingCopy);
				if(addCmd.canExecute()){
					addCmd.execute();
					Comment workingComment = (Comment)addCmd.getObject();
					UpdateCommentBOMCmd updateComment = new UpdateCommentBOMCmd(workingComment);
					updateComment.setBody(comment.getBody());
					if(updateComment.canExecute()){
						updateComment.execute();
					}else{
						//log an error, comment body can not be filled
					}
				}else{
					//log an error, the comment could not be added
				}
			}
		}
	}
	


}
