package com.ibm.btools.blm.ie.imprt.rule.informationModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.bom.command.artifacts.AddLiteralIntegerLowerBoundToMultiplicityElementBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralIntegerUpperBoundToMultiplicityElementBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralUnlimitedNaturalUpperBoundToMultiplicityElementBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddUpdateLiteralSpecificationBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddUpdateValueSpecificationBOMCmd;
import com.ibm.btools.bom.command.artifacts.RemoveLiteralIntegerBOMCmd;
import com.ibm.btools.bom.command.artifacts.RemoveLiteralUnlimitedNaturalBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateLiteralIntegerBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateLiteralUnlimitedNaturalBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateMultiplicityElementBOMCmd;
import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.MultiplicityElement;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;


public class UpdateMultiplicityRule extends AbstractImportRule {
	
	//	copyright
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	

	protected MultiplicityElement property;
	protected MultiplicityElement workingCopy;
	
	/**
	 * Sets original BOM MultiplicityElement
	 * @param property
	 */
	public void setSource(final Object property) {
		this.property = (MultiplicityElement) property;
	}
	
	/**
	 * Sets the working copy a BOM MultiplicityElement.
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = (MultiplicityElement) workingCopy;
	}

	/**
	 * Invokes the rule.
	 */
	public void invoke() {
		
		UpdateMultiplicityElementBOMCmd updateCmd = new UpdateMultiplicityElementBOMCmd(workingCopy);
		if(property.getIsOrdered() != null){
			updateCmd.setIsOrdered(property.getIsOrdered().booleanValue());	
		}else{
			updateCmd.setIsOrdered(false);
		}
		if(property.getIsUnique() != null){
			updateCmd.setIsUnique(property.getIsUnique().booleanValue());	
		}else{
			// AL: The default value of the tool is false.
			updateCmd.setIsUnique(false);
		}
		if (updateCmd.canExecute()) {
			try{
				updateCmd.execute();	
			}catch(RuntimeException ex){
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_PROPERTY_EXCEPTION, null, ex );
			}
		} else {
			LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_PROPERTY_EXCEPTION);
		}
		updateLowerBound();
		updateUpperBound();
	}

	/**
	 * Updates the LowerBound of the <code>workingCopy</code>.
	 */
	private void updateLowerBound(){
		
//		AddLiteralIntegerLowerBoundToMultiplicityElementBOMCmd addLowerBoundCmd = new AddLiteralIntegerLowerBoundToMultiplicityElementBOMCmd(workingCopy);
//		LiteralInteger lowerBound = (LiteralInteger)addLowerBoundCmd.getObject();
//		if(property.getLowerBound() == null || !(property.getLowerBound() instanceof LiteralInteger)){
//			addLowerBoundCmd.setValue(1);
//		}else{
//			addLowerBoundCmd.setValue(((LiteralInteger)property.getLowerBound()).getValue().intValue());
//		}
//		if(addLowerBoundCmd.canExecute()){
//			try{
//				addLowerBoundCmd.execute();
//			}catch(RuntimeException ex){
//				if(property instanceof NamedElement){
//					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_LOWER_BOUND_NAMED_EXCEPTION, new String[]{((NamedElement)property).getName()}, ex );
//				}else{
//					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_LOWER_BOUND_EXCEPTION, null, ex );
//				}
//			
//			}
//		}else{
//			if(property instanceof NamedElement){
//				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_LOWER_BOUND_NAMED_EXCEPTION, new String[]{((NamedElement)property).getName()}, null);
//			}else{
//				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_LOWER_BOUND_EXCEPTION);
//			}
//			
//		}
		//defect 92155
		ValueSpecification lowerBound = property.getLowerBound();
		AddUpdateValueSpecificationBOMCmd addUpperBoundCmd;
		ValueSpecification workingLowerBound = workingCopy.getLowerBound();
		if(workingLowerBound == null){
			addUpperBoundCmd = new AddLiteralIntegerLowerBoundToMultiplicityElementBOMCmd(workingCopy);
			if(addUpperBoundCmd.canExecute()){
				try{
					addUpperBoundCmd.execute();
					workingLowerBound = (ValueSpecification)addUpperBoundCmd.getObject();
				}catch(RuntimeException ex){
					if(property instanceof NamedElement){
						LoggingUtil.logWarning(IeMessageKeys.PROPERTY_UPPER_BOUND_NAMED_EXCEPTION, new String[]{((NamedElement)property).getName()}, ex);
					}else{
						LoggingUtil.logWarning(IeMessageKeys.PROPERTY_UPPER_BOUND_EXCEPTION, null, ex );
					}
						
				}
			}
		}
		if(workingLowerBound != null){
			AddUpdateLiteralSpecificationBOMCmd updateUpperBound;

				updateUpperBound = new UpdateLiteralIntegerBOMCmd((LiteralInteger)workingLowerBound);
				if(property.getUpperBound() == null){
					((UpdateLiteralIntegerBOMCmd)updateUpperBound).setValue(1);
				}else{
					((UpdateLiteralIntegerBOMCmd)updateUpperBound).setValue(((LiteralInteger)lowerBound).getValue().intValue());
				}
			if(updateUpperBound.canExecute()){
				updateUpperBound.execute();
			}else{
				if(property instanceof NamedElement){
					LoggingUtil.logWarning(IeMessageKeys.PROPERTY_UPPER_BOUND_NAMED_EXCEPTION, new String[]{((NamedElement)property).getName()}, null);	
				}else{
					LoggingUtil.logWarning(IeMessageKeys.PROPERTY_UPPER_BOUND_EXCEPTION);
				}
				
			}
		}else{
			if(property instanceof NamedElement){
				LoggingUtil.logWarning(IeMessageKeys.PROPERTY_UPPER_BOUND_NAMED_EXCEPTION, new String[]{((NamedElement)property).getName()}, null);
			}else{
				LoggingUtil.logWarning(IeMessageKeys.PROPERTY_UPPER_BOUND_EXCEPTION);
			}
				
		}
		
	}

	/**
	 * Updates the upperBound of the <code>workingCopy</code>
	 *
	 */
	private void updateUpperBound(){		
		//defect 92155. Failed to update the lower and upper bound. 
		ValueSpecification upperBound = property.getUpperBound();
		AddUpdateValueSpecificationBOMCmd addUpperBoundCmd;
		ValueSpecification workingUpperBound = workingCopy.getUpperBound();
		if(workingUpperBound == null){
			if(upperBound instanceof LiteralUnlimitedNatural){
				addUpperBoundCmd = new AddLiteralUnlimitedNaturalUpperBoundToMultiplicityElementBOMCmd(workingCopy);
			}else if(upperBound instanceof LiteralInteger || upperBound == null){
				addUpperBoundCmd = new AddLiteralIntegerUpperBoundToMultiplicityElementBOMCmd(workingCopy);
			}else{
				addUpperBoundCmd = new AddLiteralIntegerUpperBoundToMultiplicityElementBOMCmd(workingCopy);
			}
			if(addUpperBoundCmd.canExecute()){
				try{
					addUpperBoundCmd.execute();
					workingUpperBound = (ValueSpecification)addUpperBoundCmd.getObject();

				}catch(RuntimeException ex){
					if(property instanceof NamedElement){
						LoggingUtil.logWarning(IeMessageKeys.PROPERTY_UPPER_BOUND_NAMED_EXCEPTION, new String[]{((NamedElement)property).getName()}, ex);
					}else{
						LoggingUtil.logWarning(IeMessageKeys.PROPERTY_UPPER_BOUND_EXCEPTION, null, ex );
					}
						
				}
			}
		}else if(workingUpperBound instanceof LiteralUnlimitedNatural && upperBound instanceof LiteralInteger){
			//remove the Literal Unlimited and dd the Literal Integer
			RemoveLiteralUnlimitedNaturalBOMCmd removeCmd = new RemoveLiteralUnlimitedNaturalBOMCmd((LiteralUnlimitedNatural)workingUpperBound);
			if(removeCmd.canExecute()){
				try{
					removeCmd.execute();
					addUpperBoundCmd = new AddLiteralIntegerUpperBoundToMultiplicityElementBOMCmd(workingCopy);
					if(addUpperBoundCmd.canExecute()){
						addUpperBoundCmd.execute();
						workingUpperBound = (ValueSpecification)addUpperBoundCmd.getObject();
					}

				}catch(RuntimeException ex){
					if(property instanceof NamedElement){
						LoggingUtil.logWarning(IeMessageKeys.PROPERTY_UPPER_BOUND_NAMED_EXCEPTION, new String[]{((NamedElement)property).getName()}, ex);
					}else{
						LoggingUtil.logWarning(IeMessageKeys.PROPERTY_UPPER_BOUND_EXCEPTION, null, ex );
					}
						
				}
			}
		}else if(workingUpperBound instanceof LiteralInteger && upperBound instanceof LiteralUnlimitedNatural){
			RemoveLiteralIntegerBOMCmd removeCmd = new RemoveLiteralIntegerBOMCmd((LiteralInteger)workingUpperBound);
			if(removeCmd.canExecute()){
				try{
					removeCmd.execute();
					addUpperBoundCmd = new AddLiteralUnlimitedNaturalUpperBoundToMultiplicityElementBOMCmd(workingCopy);
					if(addUpperBoundCmd.canExecute()){
						addUpperBoundCmd.execute();
						workingUpperBound = (ValueSpecification)addUpperBoundCmd.getObject();
					}

				}catch(RuntimeException ex){
					if(property instanceof NamedElement){
						LoggingUtil.logWarning(IeMessageKeys.PROPERTY_UPPER_BOUND_NAMED_EXCEPTION, new String[]{((NamedElement)property).getName()}, ex);
					}else{
						LoggingUtil.logWarning(IeMessageKeys.PROPERTY_UPPER_BOUND_EXCEPTION, null, ex );
					}
						
				}
			}
			
		}
		if(workingUpperBound != null){
			AddUpdateLiteralSpecificationBOMCmd updateUpperBound;
			if(workingUpperBound instanceof LiteralUnlimitedNatural){
				updateUpperBound = new UpdateLiteralUnlimitedNaturalBOMCmd((LiteralUnlimitedNatural)workingUpperBound);
				//defect 88642 upper bound  "n" denotes unlimited multiplicity
				if(((LiteralUnlimitedNatural)property.getUpperBound()).getValue() instanceof String && "-1".equals(((LiteralUnlimitedNatural)property.getUpperBound()).getValue())){
					((UpdateLiteralUnlimitedNaturalBOMCmd)updateUpperBound).setValue("n");
				}else{
					((UpdateLiteralUnlimitedNaturalBOMCmd)updateUpperBound).setValue(((LiteralUnlimitedNatural)property.getUpperBound()).getValue());
				}					
			}else{
				updateUpperBound = new UpdateLiteralIntegerBOMCmd((LiteralInteger)workingUpperBound);
				if(property.getUpperBound() == null){
					((UpdateLiteralIntegerBOMCmd)updateUpperBound).setValue(1);
				}else{
					((UpdateLiteralIntegerBOMCmd)updateUpperBound).setValue(((LiteralInteger)property.getUpperBound()).getValue().intValue());
				}
			}
			if(updateUpperBound.canExecute()){
				updateUpperBound.execute();
			}else{
				if(property instanceof NamedElement){
					LoggingUtil.logWarning(IeMessageKeys.PROPERTY_UPPER_BOUND_NAMED_EXCEPTION, new String[]{((NamedElement)property).getName()}, null);	
				}else{
					LoggingUtil.logWarning(IeMessageKeys.PROPERTY_UPPER_BOUND_EXCEPTION);
				}
				
			}
		}else{
			if(property instanceof NamedElement){
				LoggingUtil.logWarning(IeMessageKeys.PROPERTY_UPPER_BOUND_NAMED_EXCEPTION, new String[]{((NamedElement)property).getName()}, null);
			}else{
				LoggingUtil.logWarning(IeMessageKeys.PROPERTY_UPPER_BOUND_EXCEPTION);
			}
				
		}

	}
}
