package com.ibm.btools.blm.ie.imprt.rule.informationModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.AbstractLibraryChildNode;
import com.ibm.btools.bom.command.artifacts.AddInstanceSpecificationToInstanceValueBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddUpdateValueSpecificationBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateInstanceValueBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateLiteralBooleanBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateLiteralDurationBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateLiteralIntegerBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateLiteralNullBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateLiteralRealBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateLiteralStringBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateLiteralTimeBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateLiteralUnlimitedNaturalBOMCmd;
import com.ibm.btools.bom.model.artifacts.InstanceSpecification;
import com.ibm.btools.bom.model.artifacts.InstanceValue;
import com.ibm.btools.bom.model.artifacts.LiteralBoolean;
import com.ibm.btools.bom.model.artifacts.LiteralDuration;
import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.LiteralNull;
import com.ibm.btools.bom.model.artifacts.LiteralReal;
import com.ibm.btools.bom.model.artifacts.LiteralSpecification;
import com.ibm.btools.bom.model.artifacts.LiteralString;
import com.ibm.btools.bom.model.artifacts.LiteralTime;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.emf.cf.BtCompoundCommand;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.util.datatype.IBTDataTypeRegistry;


public class UpdateValueSpecificationRule extends AbstractImportRule {
	
	//copyright
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private ValueSpecification valueSpec;
	private ValueSpecification workingCopy;
	private BtCompoundCommand wrapperCmd;
	
	/**
	 * Set Original BOM ValueSpecification
	 * @param valueSpecification a BOM ValueSpecification
	 */
	public void setSource(Object valueSpecification) {
		this.valueSpec = (ValueSpecification)valueSpecification;
	}
	
	/**
	 * Set the <code>workingCopy</code> value
	 * @param workingCopy a BOM ValueSpecification
	 */
	public void setWorkingCopy(final Object workingCopy) {
			this.workingCopy = (ValueSpecification) workingCopy;
		}
		
	public void setWrapperCommand(BtCompoundCommand command){
		this.wrapperCmd = command;	
	}
	
	/**
	 * Invoke the rule
	 */
	public void invoke() {		
		LoggingUtil.traceEntry(this, "invoke");
			AddUpdateValueSpecificationBOMCmd updateSpecificationCmd = null;
			if(valueSpec instanceof LiteralSpecification){
				LiteralSpecification literalSpec = (LiteralSpecification)valueSpec;
				if(workingCopy instanceof LiteralString){
					updateSpecificationCmd = new UpdateLiteralStringBOMCmd((LiteralString)workingCopy);
					if(literalSpec instanceof LiteralString){
						((UpdateLiteralStringBOMCmd)updateSpecificationCmd).setValue(((LiteralString)literalSpec).getValue());
						if(valueSpec.getType() != null){
							updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
						}else{
							updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_STRING));
						}
					}else if (literalSpec instanceof LiteralUnlimitedNatural){
						((UpdateLiteralStringBOMCmd)updateSpecificationCmd).setValue(((LiteralUnlimitedNatural)literalSpec).getValue());
						if(valueSpec.getType() != null){
							updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
						}else{
							updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_UNLIMITED_NATURAL)); 
						}
					}else if (literalSpec instanceof LiteralInteger){
						((UpdateLiteralStringBOMCmd)updateSpecificationCmd).setValue(((LiteralInteger)literalSpec).getValue().toString());
						if(valueSpec.getType() != null){
							updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
						}else{
							updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_INTEGER));
						}
					}else if (literalSpec instanceof LiteralBoolean){
						((UpdateLiteralStringBOMCmd)updateSpecificationCmd).setValue(((LiteralBoolean)literalSpec).getValue().toString());
						if(valueSpec.getType() != null){
							updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
						}else{
							updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_BOOLEAN));
						}
					}else if (literalSpec instanceof LiteralTime){
						updateSpecificationCmd = new UpdateLiteralTimeBOMCmd((LiteralTime)workingCopy);
						((UpdateLiteralStringBOMCmd)updateSpecificationCmd).setValue(((LiteralTime)literalSpec).getValue());
						if(valueSpec.getType() != null){
							updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
						}else{
							updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_TIME));
						}
					}else if (literalSpec instanceof LiteralDuration){
						updateSpecificationCmd = new UpdateLiteralDurationBOMCmd((LiteralDuration)workingCopy);
						((UpdateLiteralStringBOMCmd)updateSpecificationCmd).setValue(((LiteralDuration)literalSpec).getValue());
						if(valueSpec.getType() != null){
							updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
						}else{
							updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_DURATION));
						}
					}else if (literalSpec instanceof LiteralReal){
						updateSpecificationCmd = new UpdateLiteralRealBOMCmd((LiteralReal)workingCopy);
						((UpdateLiteralStringBOMCmd)updateSpecificationCmd).setValue(((LiteralReal)literalSpec).getValue().toString());
						if(valueSpec.getType() != null){
							updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
						}else{
							updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_REALNUMBER));
						}
						//TODO AR added Literal null
					}else if (literalSpec instanceof LiteralNull){
						if(valueSpec.getType() != null){
							updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
						}
					}
				}else if (workingCopy instanceof LiteralUnlimitedNatural){
					updateSpecificationCmd = new UpdateLiteralUnlimitedNaturalBOMCmd((LiteralUnlimitedNatural)workingCopy);
					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
					((UpdateLiteralUnlimitedNaturalBOMCmd)updateSpecificationCmd).setValue(((LiteralUnlimitedNatural)literalSpec).getValue()); 
				}else if (workingCopy instanceof LiteralInteger){
					updateSpecificationCmd = new UpdateLiteralIntegerBOMCmd((LiteralInteger)workingCopy);
					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
					((UpdateLiteralIntegerBOMCmd)updateSpecificationCmd).setValue(((LiteralInteger)literalSpec).getValue().intValue());
				}else if (workingCopy instanceof LiteralBoolean){
					updateSpecificationCmd = new UpdateLiteralBooleanBOMCmd((LiteralBoolean)workingCopy);
					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
					((UpdateLiteralBooleanBOMCmd)updateSpecificationCmd).setValue(((LiteralBoolean)literalSpec).getValue().booleanValue());
				}else if (workingCopy instanceof LiteralTime){
					updateSpecificationCmd = new UpdateLiteralTimeBOMCmd((LiteralTime)workingCopy);
					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
					((UpdateLiteralTimeBOMCmd)updateSpecificationCmd).setValue(((LiteralTime)literalSpec).getValue());
				}else if (workingCopy instanceof LiteralDuration){
					updateSpecificationCmd = new UpdateLiteralDurationBOMCmd((LiteralDuration)workingCopy);
					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
					((UpdateLiteralDurationBOMCmd)updateSpecificationCmd).setValue(((LiteralDuration)literalSpec).getValue());
				}else if (workingCopy instanceof LiteralReal){
					updateSpecificationCmd = new UpdateLiteralRealBOMCmd((LiteralReal)workingCopy);
					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
					((UpdateLiteralRealBOMCmd)updateSpecificationCmd).setValue(((LiteralReal)literalSpec).getValue());
				}else if(workingCopy instanceof LiteralNull){
					updateSpecificationCmd = new UpdateLiteralNullBOMCmd((LiteralNull)workingCopy);
					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
				}
				if(wrapperCmd != null){
					if(!wrapperCmd.appendAndExecute(updateSpecificationCmd)){
						wrapperCmd.undo();
					}
				}else if(updateSpecificationCmd.canExecute()){
					try{
						updateSpecificationCmd.execute();
					}catch(RuntimeException ex){
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{workingCopy.getName()},ex);
					}
				}else{
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{workingCopy.getName()}, null);
				}
//		
//		LoggingUtil.traceEntry(this, "invoke");
//		AddUpdateValueSpecificationBOMCmd updateSpecificationCmd = null;
//
//		if(valueSpec instanceof LiteralSpecification){
//			LiteralSpecification literalSpec = (LiteralSpecification)valueSpec;
//			updateSpecificationCmd = new UpdateLiteralStringBOMCmd((LiteralString)workingCopy);
//			if(literalSpec instanceof LiteralString){
//				((UpdateLiteralStringBOMCmd)updateSpecificationCmd).setValue(((LiteralString)literalSpec).getValue());
//				if(valueSpec.getType() != null){
//					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
//				}else{
//					updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_STRING));
//				}
//				
//			}else if (literalSpec instanceof LiteralUnlimitedNatural){
//				((UpdateLiteralStringBOMCmd)updateSpecificationCmd).setValue(((LiteralUnlimitedNatural)literalSpec).getValue());
//				if(valueSpec.getType() != null){
//					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
//				}else{
//					updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_UNLIMITED_NATURAL)); 
//				}
//			}else if (literalSpec instanceof LiteralInteger){
//				((UpdateLiteralStringBOMCmd)updateSpecificationCmd).setValue(((LiteralInteger)literalSpec).getValue().toString());
//				if(valueSpec.getType() != null){
//					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
//				}else{
//					updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_INTEGER));
//				}
//			}else if (literalSpec instanceof LiteralBoolean){
//				((UpdateLiteralStringBOMCmd)updateSpecificationCmd).setValue(((LiteralBoolean)literalSpec).getValue().toString());
//				if(valueSpec.getType() != null){
//					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
//				}else{
//					updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_BOOLEAN));
//				}
//			}else if (literalSpec instanceof LiteralTime){
//				updateSpecificationCmd = new UpdateLiteralTimeBOMCmd((LiteralTime)workingCopy);
//				((UpdateLiteralStringBOMCmd)updateSpecificationCmd).setValue(((LiteralTime)literalSpec).getValue());
//				if(valueSpec.getType() != null){
//					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
//				}else{
//					updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_TIME));
//				}
//			}else if (literalSpec instanceof LiteralDuration){
//				updateSpecificationCmd = new UpdateLiteralDurationBOMCmd((LiteralDuration)workingCopy);
//				((UpdateLiteralStringBOMCmd)updateSpecificationCmd).setValue(((LiteralDuration)literalSpec).getValue());
//				if(valueSpec.getType() != null){
//					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
//				}else{
//					updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_DURATION));
//				}
//			}else if (literalSpec instanceof LiteralReal){
//				updateSpecificationCmd = new UpdateLiteralRealBOMCmd((LiteralReal)workingCopy);
//				((UpdateLiteralStringBOMCmd)updateSpecificationCmd).setValue(((LiteralReal)literalSpec).getValue().toString());
//				if(valueSpec.getType() != null){
//					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
//				}else{
//					updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_REALNUMBER));
//				}
//			}
//			if(wrapperCmd != null){
//				if(!wrapperCmd.appendAndExecute(updateSpecificationCmd)){
//					wrapperCmd.undo();
//				}
//			}else if(updateSpecificationCmd.canExecute()){
//				try{
//					updateSpecificationCmd.execute();
//				}catch(RuntimeException ex){
//					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{workingCopy.getName()},ex);
//				}
//			}else{
//				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{workingCopy.getName()}, null);
//			}
			
		}else if (valueSpec instanceof InstanceValue){
			InstanceValue instanceValue = (InstanceValue)valueSpec;
//TODO AR .. add the type
			updateSpecificationCmd = new UpdateInstanceValueBOMCmd((InstanceValue)workingCopy);
			updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), instanceValue.getType(), getImportSession().getContext()));
			
			
			 
			
				if(wrapperCmd != null){
					if(!wrapperCmd.appendAndExecute(updateSpecificationCmd)){
						wrapperCmd.undo();
					}
										
					UpdateInstanceValueBOMCmd updateCmd = new UpdateInstanceValueBOMCmd((InstanceValue)workingCopy);
					updateCmd.setInstance(instanceValue.getInstance());
					wrapperCmd.appendAndExecute(updateCmd);									
					
				}else if(updateSpecificationCmd.canExecute()){
					try{
						updateSpecificationCmd.execute();
					}catch(RuntimeException ex){
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{workingCopy.getName()},ex);
					}
				}else{
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{workingCopy.getName()}, null);
				}

			if(instanceValue.getOwnedInstance() != null){
				InstanceSpecification ownedInstance = instanceValue.getOwnedInstance();
				AddInstanceSpecificationToInstanceValueBOMCmd addOwnedInstanceCmd = new AddInstanceSpecificationToInstanceValueBOMCmd((InstanceValue)workingCopy);
				addOwnedInstanceCmd.setName(ownedInstance.getName());
				if(wrapperCmd != null){
					if(wrapperCmd.appendAndExecute(addOwnedInstanceCmd)){
						InstanceSpecification instanceSpecWorkingCopy = (InstanceSpecification)addOwnedInstanceCmd.getObject();
						//d97952 -Jie-  commented out following lines 						
//						UpdateInstanceValueBOMCmd updateCmd = new UpdateInstanceValueBOMCmd((InstanceValue)workingCopy);
//						updateCmd.setInstance(instanceSpecWorkingCopy);
//						if(!wrapperCmd.appendAndExecute(updateCmd)){
//							wrapperCmd.undo();
//						}else{
//							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{instanceValue.getName()},null);	
//						}
						UpdateInstanceSpecificationRule updateInstanceRule = new UpdateInstanceSpecificationRule();
						updateInstanceRule.setSource(ownedInstance);
						updateInstanceRule.setWorkingCopy(instanceSpecWorkingCopy);
						updateInstanceRule.setProjectName(getProjectName());
						updateInstanceRule.setImportSession(getImportSession());
						updateInstanceRule.invoke();
					}				
				}else if(addOwnedInstanceCmd.canExecute()){
					try{
						addOwnedInstanceCmd.execute();
						InstanceSpecification instanceSpecWorkingCopy = (InstanceSpecification)addOwnedInstanceCmd.getObject();
						UpdateInstanceValueBOMCmd updateCmd = new UpdateInstanceValueBOMCmd((InstanceValue)workingCopy);
						updateCmd.setInstance(instanceSpecWorkingCopy);
						if(updateCmd.canExecute()){
							try{
								updateCmd.execute();
							}catch(RuntimeException ex){
								LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{instanceValue.getName()},ex);
							}
						}else{
							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{instanceValue.getName()},null);
						}
						UpdateInstanceSpecificationRule updateInstanceRule = new UpdateInstanceSpecificationRule();
						updateInstanceRule.setSource(ownedInstance);
						updateInstanceRule.setWorkingCopy(instanceSpecWorkingCopy);
						updateInstanceRule.setProjectName(getProjectName());
						updateInstanceRule.setImportSession(getImportSession());
						updateInstanceRule.invoke();
					}catch(RuntimeException ex){
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{ownedInstance.getName()}, ex);
					}
				}else{
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{ownedInstance.getName()}, null);
				}
				
			}else if (instanceValue.getInstance() != null){
				InstanceSpecification workingInstance = (InstanceSpecification)BOMUtil.loadRootObject((AbstractLibraryChildNode) getImportSession().getContext().get(instanceValue.getInstance()));
				//InstanceSpecification workingInstance = (InstanceSpecification)BOMUtil.getType(getProjectName(), instanceValue.getInstance(), getImportSession().getContext());
				if(workingInstance != null){
					UpdateInstanceValueBOMCmd updateCmd = new UpdateInstanceValueBOMCmd((InstanceValue)workingCopy);
					updateCmd.setInstance(workingInstance);
				//	updateCmd.setType((Type)BOMUtil.getType(getProjectName(), instanceValue.getType(), getImportSession().getContext()));
					if(wrapperCmd != null){
						if(!wrapperCmd.appendAndExecute(updateCmd)){
							wrapperCmd.undo();
						}
					}else if(updateCmd.canExecute()){
						try{
							updateCmd.execute();
						}catch(RuntimeException ex){
							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{instanceValue.getName()},ex);
						}
					}else{
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{instanceValue.getName()},null);
					}
				}else{
					if(wrapperCmd != null){
						wrapperCmd.undo();
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.OBJECT_REFERENCE_NOT_FOUND_EXCEPTION, new String[]{"Sample", instanceValue.getInstance().getName()}, null);
					}
					
				}
			}
		}else if (valueSpec instanceof StructuredOpaqueExpression){
			com.ibm.btools.blm.ie.imprt.rule.expressionModel.UpdateExpressionRule expressionRule =  new com.ibm.btools.blm.ie.imprt.rule.expressionModel.UpdateExpressionRule();
			expressionRule.setImportSession(getImportSession());
			expressionRule.setProjectName(getProjectName());
			expressionRule.setSource((StructuredOpaqueExpression)valueSpec);
			expressionRule.setWorkingCopy((StructuredOpaqueExpression)workingCopy);
			expressionRule.invoke();	
		}
		LoggingUtil.traceExit(this, "invoke");
	}


//	/**
//	 * Invoke the rule
//	 */
//	public void invoke() {
//		LoggingUtil.traceEntry(this, "invoke");
//		AddUpdateValueSpecificationBOMCmd updateSpecificationCmd = null;
//
//
//		if(valueSpec instanceof LiteralSpecification){
//			LiteralSpecification literalSpec = (LiteralSpecification)valueSpec;
//			if(literalSpec instanceof LiteralString){
//				updateSpecificationCmd = new UpdateLiteralStringBOMCmd((LiteralString)workingCopy);
//				((UpdateLiteralStringBOMCmd)updateSpecificationCmd).setValue(((LiteralString)literalSpec).getValue());
//				if(valueSpec.getType() != null){
//					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
//				}else{
//					updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_STRING));
//				}
//				
//			}else if (literalSpec instanceof LiteralUnlimitedNatural){
//				updateSpecificationCmd = new UpdateLiteralUnlimitedNaturalBOMCmd((LiteralUnlimitedNatural)workingCopy);
//				((UpdateLiteralUnlimitedNaturalBOMCmd)updateSpecificationCmd).setValue(((LiteralUnlimitedNatural)literalSpec).getValue());
//				if(valueSpec.getType() != null){
//					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
//				}else{
//					updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_UNLIMITED_NATURAL)); 
//				}
//			}else if (literalSpec instanceof LiteralInteger){
//				updateSpecificationCmd = new UpdateLiteralIntegerBOMCmd((LiteralInteger)workingCopy);
//				((UpdateLiteralIntegerBOMCmd)updateSpecificationCmd).setValue(((LiteralInteger)literalSpec).getValue().intValue());
//				if(valueSpec.getType() != null){
//					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
//				}else{
//					updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_INTEGER));
//				}
//			}else if (literalSpec instanceof LiteralBoolean){
//				updateSpecificationCmd = new UpdateLiteralBooleanBOMCmd((LiteralBoolean)workingCopy);
//				((UpdateLiteralBooleanBOMCmd)updateSpecificationCmd).setValue(((LiteralBoolean)literalSpec).getValue().booleanValue());
//				if(valueSpec.getType() != null){
//					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
//				}else{
//					updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_BOOLEAN));
//				}
//			}else if (literalSpec instanceof LiteralTime){
//				updateSpecificationCmd = new UpdateLiteralTimeBOMCmd((LiteralTime)workingCopy);
//				((UpdateLiteralTimeBOMCmd)updateSpecificationCmd).setValue(((LiteralTime)literalSpec).getValue());
//				if(valueSpec.getType() != null){
//					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
//				}else{
//					updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_TIME));
//				}
//			}else if (literalSpec instanceof LiteralDuration){
//				updateSpecificationCmd = new UpdateLiteralDurationBOMCmd((LiteralDuration)workingCopy);
//				((UpdateLiteralDurationBOMCmd)updateSpecificationCmd).setValue(((LiteralDuration)literalSpec).getValue());
//				if(valueSpec.getType() != null){
//					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
//				}else{
//					updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_DURATION));
//				}
//			}else if (literalSpec instanceof LiteralReal){
//				updateSpecificationCmd = new UpdateLiteralRealBOMCmd((LiteralReal)workingCopy);
//				((UpdateLiteralRealBOMCmd)updateSpecificationCmd).setValue(((LiteralReal)literalSpec).getValue());
//				if(valueSpec.getType() != null){
//					updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), valueSpec.getType(), getImportSession().getContext()));
//				}else{
//					updateSpecificationCmd.setType(BOMUtil.getPrimitiveType(getProjectName(), IBTDataTypeRegistry.PREDEF_REALNUMBER));
//				}
//			}
//			if(wrapperCmd != null){
//				if(!wrapperCmd.appendAndExecute(updateSpecificationCmd)){
//					wrapperCmd.undo();
//				}
//			}else if(updateSpecificationCmd.canExecute()){
//				try{
//					updateSpecificationCmd.execute();
//				}catch(RuntimeException ex){
//					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{workingCopy.getName()},ex);
//				}
//			}else{
//				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{workingCopy.getName()}, null);
//			}
//			
//		}else if (valueSpec instanceof InstanceValue){
//			InstanceValue instanceValue = (InstanceValue)valueSpec;
////TODO AR .. add the type
//			updateSpecificationCmd = new UpdateInstanceValueBOMCmd((InstanceValue)workingCopy);
//			updateSpecificationCmd.setType((Type)BOMUtil.getType(getProjectName(), instanceValue.getType(), getImportSession().getContext()));
//				if(wrapperCmd != null){
//					if(!wrapperCmd.appendAndExecute(updateSpecificationCmd)){
//						wrapperCmd.undo();
//					}
//				}else if(updateSpecificationCmd.canExecute()){
//					try{
//						updateSpecificationCmd.execute();
//					}catch(RuntimeException ex){
//						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{workingCopy.getName()},ex);
//					}
//				}else{
//					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{workingCopy.getName()}, null);
//				}
//
//			if(instanceValue.getOwnedInstance() != null){
//				InstanceSpecification ownedInstance = instanceValue.getOwnedInstance();
//				AddInstanceSpecificationToInstanceValueBOMCmd addOwnedInstanceCmd = new AddInstanceSpecificationToInstanceValueBOMCmd((InstanceValue)workingCopy);
//				addOwnedInstanceCmd.setName(ownedInstance.getName());
//				if(wrapperCmd != null){
//					if(wrapperCmd.appendAndExecute(addOwnedInstanceCmd)){
//						InstanceSpecification instanceSpecWorkingCopy = (InstanceSpecification)addOwnedInstanceCmd.getObject();
//						UpdateInstanceValueBOMCmd updateCmd = new UpdateInstanceValueBOMCmd((InstanceValue)workingCopy);
//						updateCmd.setInstance(instanceSpecWorkingCopy);
//						if(!wrapperCmd.appendAndExecute(updateCmd)){
//							wrapperCmd.undo();
//						}else{
//							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{instanceValue.getName()},null);	
//						}
//						UpdateInstanceSpecificationRule updateInstanceRule = new UpdateInstanceSpecificationRule();
//						updateInstanceRule.setSource(ownedInstance);
//						updateInstanceRule.setWorkingCopy(instanceSpecWorkingCopy);
//						updateInstanceRule.setProjectName(getProjectName());
//						updateInstanceRule.setImportSession(getImportSession());
//						updateInstanceRule.invoke();
//					}				
//				}else if(addOwnedInstanceCmd.canExecute()){
//					try{
//						addOwnedInstanceCmd.execute();
//						InstanceSpecification instanceSpecWorkingCopy = (InstanceSpecification)addOwnedInstanceCmd.getObject();
//						UpdateInstanceValueBOMCmd updateCmd = new UpdateInstanceValueBOMCmd((InstanceValue)workingCopy);
//						updateCmd.setInstance(instanceSpecWorkingCopy);
//						if(updateCmd.canExecute()){
//							try{
//								updateCmd.execute();
//							}catch(RuntimeException ex){
//								LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{instanceValue.getName()},ex);
//							}
//						}else{
//							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{instanceValue.getName()},null);
//						}
//						UpdateInstanceSpecificationRule updateInstanceRule = new UpdateInstanceSpecificationRule();
//						updateInstanceRule.setSource(ownedInstance);
//						updateInstanceRule.setWorkingCopy(instanceSpecWorkingCopy);
//						updateInstanceRule.setProjectName(getProjectName());
//						updateInstanceRule.setImportSession(getImportSession());
//						updateInstanceRule.invoke();
//					}catch(RuntimeException ex){
//						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{ownedInstance.getName()}, ex);
//					}
//				}else{
//					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{ownedInstance.getName()}, null);
//				}
//				
//			}else if (instanceValue.getInstance() != null){
//				InstanceSpecification workingInstance = (InstanceSpecification)BOMUtil.loadRootObject((AbstractLibraryChildNode) getImportSession().getContext().get(instanceValue.getInstance()));
//				//InstanceSpecification workingInstance = (InstanceSpecification)BOMUtil.getType(getProjectName(), instanceValue.getInstance(), getImportSession().getContext());
//				if(workingInstance != null){
//					UpdateInstanceValueBOMCmd updateCmd = new UpdateInstanceValueBOMCmd((InstanceValue)workingCopy);
//					updateCmd.setInstance(workingInstance);
//				//	updateCmd.setType((Type)BOMUtil.getType(getProjectName(), instanceValue.getType(), getImportSession().getContext()));
//					if(wrapperCmd != null){
//						if(!wrapperCmd.appendAndExecute(updateCmd)){
//							wrapperCmd.undo();
//						}
//					}else if(updateCmd.canExecute()){
//						try{
//							updateCmd.execute();
//						}catch(RuntimeException ex){
//							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{instanceValue.getName()},ex);
//						}
//					}else{
//						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{instanceValue.getName()},null);
//					}
//				}else{
//					if(wrapperCmd != null){
//						wrapperCmd.undo();
//						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.OBJECT_REFERENCE_NOT_FOUND_EXCEPTION, new String[]{"Sample", instanceValue.getInstance().getName()}, null);
//					}
//					
//				}
//			}
//		}else if (valueSpec instanceof OpaqueExpression){
//			com.ibm.btools.blm.ie.imprt.rule.expressionModel.UpdateExpressionRule expressionRule =  new com.ibm.btools.blm.ie.imprt.rule.expressionModel.UpdateExpressionRule();
//			expressionRule.setImportSession(getImportSession());
//			expressionRule.setProjectName(getProjectName());
//			expressionRule.setSource((StructuredOpaqueExpression)valueSpec);
//			expressionRule.setWorkingCopy((StructuredOpaqueExpression)workingCopy);
//			expressionRule.invoke();	
//		}
//		LoggingUtil.traceExit(this, "invoke");
//	}
	
}
