package com.ibm.btools.blm.ie.imprt.rule.navigator;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.engine.ProgressCalculator;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.factory.SignalFactory;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.NavigationDataCatalogNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationSignalNode;
import com.ibm.btools.bom.model.artifacts.Signal;

/**
 * Import the signal from a information model
 */
public class SignalRule extends AbstractImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private NavigationDataCatalogNode parentNode;
	private Signal newSignal;
	private NavigationSignalNode node;
		
	/**
	 * Set the parent node. It can be either a project node or a data catalog node
	 * @param parentNode
	 */
	public void setParent(final NavigationDataCatalogNode parentNode) {
		this.parentNode = parentNode;
	}
	
	/**
	 * Set the signal model for import
	 * @param newSignal
	 */
	public void setSource(final Object newSignal) {
		this.newSignal = (Signal) newSignal;
	}
	
	/**
	 * Get navigation node
	 * @return
	 */
	public NavigationSignalNode getNewSignalNode() {
		return node;
	}
	
	
	/**
	 * Invoke the rule
	 */	
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");			
		if (parentNode == null) {
			LoggingUtil.trace(this, "invoke", "The parent node should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		if (newSignal == null) {
			LoggingUtil.trace(this, "invoke", "The source should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		String signalName = newSignal.getName();
		if (signalName == null) {
			LoggingUtil.trace(this, "invoke", "The source name should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		String description = BOMUtil.getDescription(newSignal);
		
		boolean resolve = false;
		Object list = getImportSession().getContext().get(IImportFramework.CONTEXT_PREDEFINED_REGISTRY_KEY);
		if (list != null && list instanceof EList) {
			resolve = ((EList) list).contains(newSignal);
		}		
		LoggingUtil.trace(this, "invoke", "The resolve value: " + resolve + ".");		
		
		try {		
			SignalFactory factory = new SignalFactory(); 
			node = factory.create(
				parentNode, signalName, description, getImportSession(), resolve);
		} catch (OperationCanceledException e) {			
			throw e;		
		} catch (RuntimeException e) {
			LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CANNOT_CREATE_SIGNAL, new String[]{signalName}, e, 
				IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);				
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		if (node != null) {
			getImportSession().getContext().put(newSignal, node);
		}
		NavigatorUtil.save(getProjectName());		
		LoggingUtil.trace(this, "invoke", "The navigatior model is saved.");
		
		IProgressMonitor monitor = getImportSession().getProgressMonitor();
		if (monitor != null) {
			if (monitor.isCanceled()) {
				LoggingUtil.trace(this, "invoke", "The user cancels the operation.");				
				throw new OperationCanceledException();					
			}
			monitor.worked(ProgressCalculator.getSimpleTask());
		}
		LoggingUtil.traceExit(this, "invoke");		
	}
}
