package com.ibm.btools.blm.ie.imprt.rule.navigator.factory;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.OperationCanceledException;

import com.ibm.btools.blm.compoundcommand.navigator.add.AddCategoryInstanceNAVCmd;
import com.ibm.btools.blm.compoundcommand.util.CCRuntimeException;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportMessage;
import com.ibm.btools.blm.ie.imprt.ImportSession;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.CreateOrganizationModelElementPrecondition;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.InvalidTypeException;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildContainerNode;
import com.ibm.btools.blm.ui.navigation.model.AbstractNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationCategoryCatalogNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationCategoryCatalogsNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationCategoryInstanceNode;
import com.ibm.btools.blm.ui.navigation.model.util.NavigationConstants;
import com.ibm.btools.bom.command.compound.PredefTypesConst;

/**
 * Factory class to create a category instance
 */
public class CategoryInstanceFactory extends NavigatorFactory {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	/**
	 * Create a new category instance
	 * @param navNode
	 * @param organizationModelName
	 * @param comment
	 * @return the newly created navigation node
	 */
	public NavigationCategoryInstanceNode create(final AbstractNode navNode, final String organizationModelName, 
		final String comment, ImportSession session, final boolean isDefault, final boolean resolve) {
		
		//TODO: this class is to support temprary solution of categories. should be deleted in 6.0.
		LoggingUtil.traceEntry(this, "create");	
		boolean fileLengthExceeded = false;

		AbstractChildContainerNode parentNode = null;
		String projectName = null;
		boolean isProjectNode = false;
		NavigationCategoryInstanceNode newNode = null;
					
		String name = organizationModelName;	
		name = getProperRootElementName(session, name);	
		
		if (navNode instanceof NavigationCategoryCatalogNode) {
			parentNode = (NavigationCategoryCatalogNode) navNode;
			projectName = ((NavigationCategoryCatalogNode)navNode).getProjectNode().getLabel();
		} else {
			LoggingUtil.trace(this, "create", "This is a not valid navigation node.");					
			LoggingUtil.traceExit(this, "create");
			return null;
		}
		
		// Check to see if the model is already existed.
		CreateOrganizationModelElementPrecondition precondition = new CreateOrganizationModelElementPrecondition();
		precondition.setParent(parentNode);
		precondition.setName(name);
		precondition.setType(NavigationConstants.CATEGORY_INSTANCE);		
		 
		int flag = VALID; 
		try {
			if(!precondition.isValid()) {
				LoggingUtil.trace(this, "create", "The precondition is failed.");
				if (resolve) {
					LoggingUtil.trace(this, "create", "This is a predefined type.");					
					LoggingUtil.traceExit(this, "create");	
					Object list = session.getContext().get(IImportFramework.CONTEXT_RESOLVED_TYPE);
					if (list != null && list instanceof List) {
						((List)list).add(getNode(parentNode, name));
					}
					return getNode(parentNode, name);
				}				
				if (!isDefault) {
					flag = getCollisionHandleRequest(session, organizationModelName);
					flag = OVERWRITE;
					if (flag == IGNORE) {
						LoggingUtil.trace(this, "create", "The user chooses to ignore the name collision.");
						LoggingUtil.traceExit(this, "create");	
						return null;
					} else if (flag == OVERWRITE) {
						LoggingUtil.trace(this, "create", "The user chooses to overwrite the name collision.");	
						// do nothing
					} else if (flag == NEW) {
						LoggingUtil.trace(this, "create", "The user chooses to create an element with a new name.");
						name = generateName(parentNode, name, precondition);		
						if (name == null) {												
							LoggingUtil.traceExit(this, "create");	
							return null;	
						}
					} else if (flag == CANCEL) {
						LoggingUtil.trace(this, "create", "The user chooses to cancel the import.");
						throw new OperationCanceledException();
					}	
				} else {
					flag = OVERWRITE;
				}
			}
		} catch (InvalidTypeException e) {
			// Ignore
			LoggingUtil.traceExit(this, "create");		
			return null;
		}
		if (flag != OVERWRITE) {
			// Parameters:
			// 1. project name
			// 2. organization model name
			// 3. parent Navigator node
			AddCategoryInstanceNAVCmd navCmd = new AddCategoryInstanceNAVCmd();
			navCmd.setProjectName(projectName);
			navCmd.setDomainModelName(name);
			navCmd.setDescription(comment);
			navCmd.setAbstractLibraryChildNode(parentNode);		
			navCmd.setParentInformationModelURI((String)parentNode.getEntityReference());
			if (navCmd.canExecute()) {
				try {
					navCmd.execute();
				} catch (RuntimeException e) {
					if (e instanceof CCRuntimeException) {
						CCRuntimeException ccEx = (CCRuntimeException) e;
						if (ccEx.getCode().equals(ImportMessage.FILE_LENGTH_EXCEEDED)) {
							fileLengthExceeded = true;						
							LoggingUtil.logWarning(session, IeMessageKeys.FILE_LENGTH_EXCEEDED_GENERIC, 
								new String[]{name}, null, IImportResultMessages.CANNOT_CREATE_NAVIGATION_NODE);							
						}
					}
					LoggingUtil.trace(this, "create", "Add command throws a RuntimeException.");					
				}
			} else {
				LoggingUtil.trace(this, "create", "Add command cannot be executed.");
			}		
		}
				
		// The Navigator model is created, but need to update it.
		newNode = getNode(parentNode, name);
		if (newNode == null && fileLengthExceeded == false) {
			LoggingUtil.logWarning(session, IeMessageKeys.CANNOT_CREATE_CATEGORY_INSTANCE, 
				new String[]{name}, null, IImportResultMessages.CANNOT_CREATE_NAVIGATION_NODE);		
		}
		
		LoggingUtil.traceExit(this, "create");	
		return newNode;
	}

	/**
	 * Return the category instance node by name
	 * @param parentNode
	 * @param organizationModelName
	 * @return
	 */
	private NavigationCategoryInstanceNode getNode(final AbstractChildContainerNode parentNode, final String organizationModelName) {
		LoggingUtil.traceEntry(this, "getNode");
		
		if (parentNode instanceof NavigationCategoryCatalogsNode)  {		
			Iterator catalogsNodeIterator = ((NavigationCategoryCatalogsNode) parentNode).getNavigationCategoryCatalog().iterator();
			AbstractChildContainerNode navContainerNode = null;
			String label  = null;
			NavigationCategoryCatalogNode defaultCategoryOrgModel = null;
			while(catalogsNodeIterator.hasNext()) {				
				navContainerNode = (AbstractChildContainerNode) catalogsNodeIterator.next();
				label = navContainerNode.getLabel();				
				if (label != null && PredefTypesConst.CATEGORY_ORG_MODEL_NAME.compareTo(label) == 0) {
					// This is the default category catalog org model
					defaultCategoryOrgModel = (NavigationCategoryCatalogNode) navContainerNode;
					break;
				}			
			}
			
			if(defaultCategoryOrgModel != null){
				Iterator categoryNodeIterator = defaultCategoryOrgModel.getNavigationCategoryInstance().iterator();
				while(categoryNodeIterator.hasNext()) {				
					navContainerNode = (AbstractChildContainerNode) categoryNodeIterator.next();
					label = navContainerNode.getLabel();				
					if (label != null && organizationModelName.compareTo(label) == 0) {
						// This is the category instance
						LoggingUtil.traceExit(this, "getNode");
						return (NavigationCategoryInstanceNode) navContainerNode;
					}			
				}
			}
		} else if (parentNode instanceof NavigationCategoryCatalogNode) {
			Iterator catalogsNodeIterator = ((NavigationCategoryCatalogNode)parentNode).getNavigationCategoryInstance().iterator();
			AbstractChildContainerNode navContainerNode = null;
			String label = null;
			while(catalogsNodeIterator.hasNext()) {				
				navContainerNode = (AbstractChildContainerNode) catalogsNodeIterator.next();
				label = navContainerNode.getLabel();				
				if (label != null && organizationModelName.compareTo(label) == 0) {
					// This is the Navigator model
					LoggingUtil.traceExit(this, "getNode");
					return (NavigationCategoryInstanceNode) navContainerNode;
				}			
			}		
		}
		LoggingUtil.traceExit(this, "getNode");
		return null;
	}	
}
