package com.ibm.btools.blm.ie.imprt.rule.navigator.factory;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.navigator.add.AddComplexTypeDefinitionNAVCmd;
import com.ibm.btools.blm.compoundcommand.navigator.add.AddDomainObjectNavigatorCmd;
import com.ibm.btools.blm.compoundcommand.navigator.add.AddInlineComplexTypeDefinitionNAVCmd;
import com.ibm.btools.blm.compoundcommand.util.CCRuntimeException;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportMessage;
import com.ibm.btools.blm.ie.imprt.ImportSession;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.CreateExternalModelElementPrecondition;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.InvalidTypeException;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildContainerNode;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildLeafNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationComplexTypeDefinitionNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationComplexTypeDefinitionsNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationInlineComplexTypeDefinitionNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationInlineComplexTypeDefinitionsNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationInlineXSDSchemaNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationXSDFileNode;
import com.ibm.btools.blm.ui.navigation.model.util.precondition.NameCheckConstants;
import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.infrastructure.util.UIDGenerator;
import com.ibm.btools.ui.navigation.manager.util.NavigationManagerPlugin;
import com.ibm.btools.util.resource.UtilResourceBundleSingleton;

/**
 * Factory class to create a business item
 */
public class ComplexTypeFactory extends NavigatorFactory {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	/**
	 * 
	 * @param parentNode
	 * @param complexTypeName
	 * @param comment
	 * @param session
	 * @param resolve
	 * @return the newly created node with entity references
	 */
	public AbstractChildLeafNode create(
			final AbstractChildContainerNode parentNode, 
			final Class complexType, 
			final String comment, 
			ImportSession session, 
			final boolean resolve) {
		
		LoggingUtil.traceEntry(this, "create");
		boolean fileLengthExceeded = false;
		
		String projectName = null;
		boolean isProjectNode = false;
		boolean canRemove = true;
		AbstractChildLeafNode newNode = null;
		EObject oldModel = null;
		
		String name = complexType.getName();	
		assertModelNameLength(name);
		//name = getProperRootElementName(session, name);
		projectName = parentNode.getProjectNode().getLabel();
		
		// Check to see if the model is already existed.
		CreateExternalModelElementPrecondition precondition = new CreateExternalModelElementPrecondition();
		precondition.setParent(parentNode);
		precondition.setName(name);			
		
		int flag = VALID; 
		try {
			if(!precondition.isValid(true)) {
				LoggingUtil.trace(this, "create", "The precondition is failed.");
				if (resolve) {
					LoggingUtil.trace(this, "create", "This is a predefined type.");					
					AbstractChildLeafNode node = getNode(parentNode, complexType);				
					Object list = session.getContext().get(IImportFramework.CONTEXT_RESOLVED_TYPE);
					if (list != null && list instanceof List) {
						((List)list).add(node);
					}
					LoggingUtil.traceExit(this, "create");	
					return node;
				}	
				if(precondition.getResult() == NameCheckConstants.NAME_IS_NOT_UNIQUE_DIFFERENT_CASE){
					//log the warning message
					LoggingUtil.logWarning(session, IeMessageKeys.NAME_NOT_UNIQUE_DIFFERENT_CASES, 
							new String[]{name}, null, IImportResultMessages.NAME_NOT_UNIQUE_DIFFERENT_CASES);
					return null;
				}
				flag = NavigatorUtil.getExternalSchemaOption(session.getImportOptions(),complexType);
				if (flag == IGNORE) {
					LoggingUtil.trace(this, "create", "The user chooses to ignore the name collision.");
					LoggingUtil.traceExit(this, "create");	
					//return null;
					return getNode(parentNode, complexType);
				} else if (flag == OVERWRITE) {
					LoggingUtil.trace(this, "create", "The user chooses to overwrite the name collision.");	
					AbstractChildLeafNode complexTypeNode = getNode(parentNode, complexType);
					final String blmURI = (String)complexTypeNode.getEntityReferences().get(0);				
					
					//close open editors before overwrite
					closeEditors(complexTypeNode,projectName);				
					canRemove = false;
				}
			}
		} catch (InvalidTypeException e) {
			// Ignore
			LoggingUtil.traceExit(this, "create");				
			return null;
		}			
	
		if (canRemove) {			
			NavigatorUtil.recordNewNode(session,complexType);			
			createNode(
					parentNode,
					session,
					projectName,
					//name,
					complexType,
					comment,
					fileLengthExceeded);
		}
		
		newNode =  getNode(parentNode, complexType);
		//update the node attribute1 to filter out the private schema type
		BOMUtil.updateNodeAttribute(complexType,newNode);
				
		if (newNode == null && fileLengthExceeded == false) {
			LoggingUtil.logWarning(session, IeMessageKeys.CANNOT_CREATE_COMPLEX_TYPE, 
					new String[]{name}, null, IImportResultMessages.CANNOT_CREATE_NAVIGATION_NODE);		
		} 
		LoggingUtil.traceExit(this, "create");			
		return newNode;	
	}	
	
	/**
	 * Return the complexTypeDefinition node by name
	 * @param parentNode
	 * @param definitionName
	 * @return 
	 */
	private AbstractChildLeafNode getNode(final AbstractChildContainerNode parentNode, final Class complexType) {
		LoggingUtil.traceEntry(this, "getNode");
		
		String definitionName =  complexType.getName();
		NavigationComplexTypeDefinitionsNode definitions = null;
		NavigationInlineComplexTypeDefinitionsNode inlineDefinitions = null;
		AbstractChildLeafNode navContainerNode = null;
		String label = null;
		
		if(parentNode instanceof NavigationXSDFileNode){
			definitions = ((NavigationXSDFileNode)parentNode).getComplexTypeDefinitionsNode();
			if(definitions == null){
				LoggingUtil.traceExit(this,"getNode()");
				return null;
			}
			EList defList = definitions.getComplexTypeDefinitionNode(); 
			if(defList == null){
				LoggingUtil.traceExit(this,"getNode()");
				return null;
			}
			Iterator defIterator = defList.iterator();			
			while(defIterator.hasNext()){
				navContainerNode = (NavigationComplexTypeDefinitionNode)defIterator.next();
				label = navContainerNode.getLabel();
				if(label != null && label.compareTo(definitionName) == 0 ||
						(BOMUtil.isFileAttachment(complexType)&& BOMUtil.FILE_ATTACHMENT_NODE_NAME.equals(label))){
					LoggingUtil.traceExit(this, "getNode");
					return navContainerNode;
				}
			}
			
		}else if(parentNode instanceof NavigationInlineXSDSchemaNode){
			inlineDefinitions = ((NavigationInlineXSDSchemaNode)parentNode).getInlineComplexTypeDefinitionsNode();
			if(inlineDefinitions == null){
				LoggingUtil.traceExit(this,"getNode()");
				return null;
			}
			EList inlineDefList = inlineDefinitions.getInlineComplexTypeDefinitionNode();
			if(inlineDefList == null){
				LoggingUtil.traceExit(this,"getNode()");
				return null;
			}
			Iterator inlineDefIterator = inlineDefList.iterator();			
			while(inlineDefIterator.hasNext()){
				navContainerNode = (NavigationInlineComplexTypeDefinitionNode)inlineDefIterator.next();
				label = navContainerNode.getLabel();
				if(label != null && label.compareTo(definitionName) == 0 ||
						(BOMUtil.isFileAttachment(complexType) && BOMUtil.FILE_ATTACHMENT_NODE_NAME.compareTo(label)== 0)){
					LoggingUtil.traceExit(this, "getNode");
					return navContainerNode;
				}
			}			
		}
		LoggingUtil.traceExit(this, "getNode");
		return null;
	}	
	
	/**
	 * Create and attach the node to its parent
	 * @param parentNode
	 * @param session
	 * @param projectName
	 * @param domainName
	 * @param description
	 * @param fileLengthExceeded
	 * @return
	 */
	private void createNode(
			AbstractChildContainerNode parentNode,
			ImportSession session,
			String projectName,
			//String domainName,
			Class complexType,
			String description,
			boolean fileLengthExceeded){
		
		AddDomainObjectNavigatorCmd addDefinitionCmd = null;
		//Inline definition
		if(parentNode instanceof NavigationInlineXSDSchemaNode){
		    addDefinitionCmd = new AddInlineComplexTypeDefinitionNAVCmd();
		//default or externla schema
		}else if(parentNode instanceof NavigationXSDFileNode){
			addDefinitionCmd = new AddComplexTypeDefinitionNAVCmd();
		}		
		addDefinitionCmd.setProjectName(projectName);
		addDefinitionCmd.setDefaultID("Complex type definition 1");
		addDefinitionCmd.setAbstractLibraryChildNode(parentNode);
		//addDefinitionCmd.setDefaultModelID(UIDGenerator.getUID("BLMNAV"));
		//addDefinitionCmd.setDescription("Complex Type");	
		addDefinitionCmd.setDescription(LoggingUtil.getLocalizedString(IeMessageKeys.COMPLEX_TYPE_DESCRIPTION,null));
		if(BOMUtil.isFileAttachment(complexType)){
			addDefinitionCmd.setDomainModelName(BOMUtil.FILE_ATTACHMENT_NODE_NAME);
			addDefinitionCmd.setLabel(BOMUtil.FILE_ATTACHMENT_NODE_NAME);
		}else{
			addDefinitionCmd.setDomainModelName(complexType.getName());
			addDefinitionCmd.setLabel(complexType.getName());
		}
		addDefinitionCmd.setParentInformationModelURI((String)parentNode.getEntityReference());
		if (addDefinitionCmd.canExecute()) {
			try {
				addDefinitionCmd.execute();			
			} catch (RuntimeException e) {
				if (e instanceof CCRuntimeException) {
					CCRuntimeException ccEx = (CCRuntimeException) e;
					if (ccEx.getCode().equals(ImportMessage.FILE_LENGTH_EXCEEDED)) {
						fileLengthExceeded = true;						
						LoggingUtil.logWarning(session, IeMessageKeys.FILE_LENGTH_EXCEEDED_GENERIC, 
								new String[]{complexType.getName()}, null, IImportResultMessages.CANNOT_CREATE_NAVIGATION_NODE);
						throw e;
						}
				}
				LoggingUtil.trace(this, "create", "Add command throws a RuntimeException.");					
			}
		} else {
			LoggingUtil.trace(this, "create", "Add command cannot be executed.");
		}
	}
}
