package com.ibm.btools.blm.ie.imprt.rule.navigator.factory;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.compoundcommand.navigator.add.AddComplexTypeTemplateNAVCmd;
import com.ibm.btools.blm.compoundcommand.navigator.add.AddDomainObjectNavigatorCmd;
import com.ibm.btools.blm.compoundcommand.navigator.add.AddInlineComplexTypeTemplateNAVCmd;
import com.ibm.btools.blm.compoundcommand.util.CCRuntimeException;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportMessage;
import com.ibm.btools.blm.ie.imprt.ImportSession;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.CreateExternalModelElementPrecondition;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.InvalidTypeException;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildContainerNode;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildLeafNode;
import com.ibm.btools.blm.ui.navigation.model.AbstractNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationComplexTypeTemplateNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationComplexTypeTemplatesNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationInlineComplexTypeTemplateNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationInlineComplexTypeTemplatesNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationInlineXSDSchemaNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationXSDFileNode;
import com.ibm.btools.blm.ui.navigation.model.command.business.UpdateAbstractNodeBusCmd;
import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.infrastructure.util.UIDGenerator;
import com.ibm.btools.ui.navigation.manager.util.NavigationManagerPlugin;

/**  
 * Factory class to create a business item
 */
public class   ComplexTypeTemplateFactory extends NavigatorFactory {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	/**
	 * Create a new complex type template
	 * @param parentNode
	 * @param complexTypeTemplateName
	 * @param comment
	 * @param session
	 * @param resolve
	 * @return
	 */
	public AbstractChildLeafNode create(
			final AbstractChildContainerNode parentNode, 
			final Class complexTypeTemplate, 
			final String comment, 
			ImportSession session, 
			final boolean resolve) {


		LoggingUtil.traceEntry(this, "create");	
		boolean fileLengthExceeded = false;
			
		String projectName = null;
		boolean isProjectNode = false;
		boolean canRemove = true;
		AbstractChildLeafNode newNode = null;
					
		String name = complexTypeTemplate.getName();
		assertModelNameLength(name);
		//name = getProperRootElementName(session, name);
		projectName = parentNode.getProjectNode().getLabel();
		
		// Check to see if the model is already existed.
		CreateExternalModelElementPrecondition precondition = new CreateExternalModelElementPrecondition();
		precondition.setParent(parentNode);
		precondition.setName(name);
		
		//JR31715
		if(!existTemplateCreationInfo(session)){
			LoggingUtil.logInfo(session, IeMessageKeys.TEMPLATE_CREATION_INFO, new String[]{complexTypeTemplate.getName()}, null, null);
		}
		
		int flag = VALID; 
		try {
			if(!precondition.isValid(true)) {
				LoggingUtil.trace(this, "create", "The precondition is failed.");
				if (resolve) {
					LoggingUtil.trace(this, "create", "This is a predefined type.");					
					LoggingUtil.traceExit(this, "create");	
					Object list = session.getContext().get(IImportFramework.CONTEXT_RESOLVED_TYPE);
					if (list != null && list instanceof List) {
						((List)list).add(getNode(parentNode, name));
					}
					return getNode(parentNode, name);
				}
				flag = NavigatorUtil.getExternalSchemaOption(session.getImportOptions(),complexTypeTemplate);
				if (flag == IGNORE) {
					LoggingUtil.trace(this, "create", "The user chooses to ignore the name collision.");
					LoggingUtil.traceExit(this, "create");	
					return getNode(parentNode, name);
//					return null;
				} else if (flag == OVERWRITE) {
					LoggingUtil.trace(this, "create", "The user chooses to overwrite the name collision.");	
					AbstractChildLeafNode templateNode = getNode(parentNode, name);					
					//close open editors before overwrite
					closeEditors(templateNode,projectName);					
					canRemove = false;
				} else if (flag == CANCEL) {
					LoggingUtil.trace(this, "create", "The user chooses to cancel the import.");
					throw new OperationCanceledException();
				}
			}
		} catch (InvalidTypeException e) {
			// Ignore
			LoggingUtil.traceExit(this, "create");				
			return null;
		}			
		
		if (canRemove) {				
			NavigatorUtil.recordNewNode(session,complexTypeTemplate);			
			createNode(
						parentNode,
						session,
						projectName,
						//name,
						complexTypeTemplate,
						comment,
						fileLengthExceeded);
		}
		
		newNode = getNode(parentNode,name);
		//update the node attribute
		BOMUtil.updateNodeAttribute(complexTypeTemplate,newNode);
		
		if (newNode == null && fileLengthExceeded == false) {
			LoggingUtil.logWarning(session, IeMessageKeys.CANNOT_CREATE_CATEGORY, 
				new String[]{name}, null, IImportResultMessages.CANNOT_CREATE_NAVIGATION_NODE);		
		} 		
		LoggingUtil.traceExit(this, "create");	
		return newNode;	
	}	
	
		/**
		 * Check if the template creation flag has been set, we only want to add one info to inform the UI that template(s)
		 * has been created for this import session
		 * @param session
		 * @return
		 */
		private boolean existTemplateCreationInfo(ImportSession session) {
			if(session != null){
				Object templateFlag = session.getImportOptions().getAdditionalOption(IImportFramework.BSO_TEMPLATE_CREATION_FLAG);
			if(templateFlag instanceof Boolean){
					Boolean flagObj = (Boolean)templateFlag;
					return flagObj.booleanValue();
				}
			if(session.getImportOptions().getAdditionalOptions() instanceof Map){
					session.getImportOptions().getAdditionalOptions().put(IImportFramework.BSO_TEMPLATE_CREATION_FLAG, new Boolean(true));
				}else{
					Map additionalOptionsMap = new HashMap();
					additionalOptionsMap.put(IImportFramework.BSO_TEMPLATE_CREATION_FLAG, new Boolean(true));
					session.getImportOptions().setAdditionalOptions(additionalOptionsMap);
			}
		}
		return false;
	}

	/**
	 * Retrieve the newly created node
	 * @param parentNode
	 * @param complexTypeTemplateName
	 * @return
	 */
	private AbstractChildLeafNode getNode(final AbstractChildContainerNode parentNode, final String complexTypeTemplateName) {
		LoggingUtil.traceEntry(this, "getNode");
		NavigationInlineComplexTypeTemplatesNode inlineTemplates = null;
		NavigationComplexTypeTemplatesNode templates = null;
		AbstractChildLeafNode navContainerNode = null;
		String label = null;
		
		if(parentNode instanceof NavigationInlineXSDSchemaNode){
			inlineTemplates = ((NavigationInlineXSDSchemaNode)parentNode).getInlineComplexTypeTemplatesNode();
			if(inlineTemplates == null){
				LoggingUtil.traceExit(this, "getNode");
				return null;
			}
			EList inlineList = inlineTemplates.getInlineComplexTypeTemplateNode();
			if(inlineList == null){
				LoggingUtil.traceExit(this, "getNode");
				return null;
			}			
			Iterator inlineIterator = inlineList.iterator();			
			while(inlineIterator.hasNext()){
				navContainerNode = (NavigationInlineComplexTypeTemplateNode)inlineIterator.next();
				label = navContainerNode.getLabel();
				if(label != null && label.compareTo(complexTypeTemplateName) == 0){
					LoggingUtil.traceExit(this, "getNode");
					return navContainerNode;
				}
			}
			
		}else if(parentNode instanceof NavigationXSDFileNode){
			templates = ((NavigationXSDFileNode)parentNode).getComplexTypeTemplatesNode();
			if(templates == null){
				LoggingUtil.traceExit(this, "getNode");
				return null;
			}
			EList templateList = templates.getComplexTypeTemplateNode();
			if(templateList == null){
				LoggingUtil.traceExit(this, "getNode");
				return null;
			}			
			Iterator templateIterator = templateList.iterator();			
			while(templateIterator.hasNext()){
				navContainerNode = (NavigationComplexTypeTemplateNode)templateIterator.next();
				label = navContainerNode.getLabel();
				if(label != null && label.compareTo(complexTypeTemplateName) == 0){
					LoggingUtil.traceExit(this, "getNode");
					return navContainerNode;
				}
			}
			
		}		
		LoggingUtil.traceExit(this, "getNode");
		return null;
	}	
	
	private void createNode(
			AbstractChildContainerNode parentNode,
			ImportSession session,
			String projectName,
			//String domainName,
			Class complexTypeTemplate,
			String description,
			boolean fileLengthExceeded){
		
        AddDomainObjectNavigatorCmd addTemplateCmd = null;
        //inline schema
        if(parentNode instanceof NavigationInlineXSDSchemaNode){
		 addTemplateCmd = new AddInlineComplexTypeTemplateNAVCmd();
		 
		//default or externla schema
		}else if(parentNode instanceof NavigationXSDFileNode){
			addTemplateCmd = new AddComplexTypeTemplateNAVCmd();
		}
		
		addTemplateCmd.setProjectName(projectName);
		addTemplateCmd.setDefaultID("Complex type template 1");
		addTemplateCmd.setAbstractLibraryChildNode(parentNode);
		//addTemplateCmd.setDefaultModelID(UIDGenerator.getUID("BLMNAV")); 
		addTemplateCmd.setDescription(LoggingUtil.getLocalizedString(IeMessageKeys.COMPLEX_TYPE_TEMPLATE_DESCRIPTION,null));
		//addTemplateCmd.setDescription(description);		
		addTemplateCmd.setDomainModelName(complexTypeTemplate.getName());
		addTemplateCmd.setLabel(complexTypeTemplate.getName());
		addTemplateCmd.setParentInformationModelURI((String)parentNode.getEntityReference());
		
		if (addTemplateCmd.canExecute()) {
			try {
				addTemplateCmd.execute();					
			} catch (RuntimeException e) {
				if (e instanceof CCRuntimeException) {
					CCRuntimeException ccEx = (CCRuntimeException) e;
					if (ccEx.getCode().equals(ImportMessage.FILE_LENGTH_EXCEEDED)) {
						fileLengthExceeded = true;					
						LoggingUtil.logWarning(session, IeMessageKeys.FILE_LENGTH_EXCEEDED_GENERIC, 
							new String[]{complexTypeTemplate.getName()}, null, IImportResultMessages.CANNOT_CREATE_NAVIGATION_NODE);
						throw e;
					}					
				}
				LoggingUtil.trace(this, "create", "Add command throws a RuntimeException.");					
			}
		} else {
			LoggingUtil.trace(this, "create", "Add command cannot be executed.");
		}	
	}    
}
