package com.ibm.btools.blm.ie.imprt.rule.navigator.factory;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.compoundcommand.navigator.add.AddWSDLFileNAVCmd;
import com.ibm.btools.blm.compoundcommand.navigator.remove.RemoveInlineComplexTypeDefinitionNAVCmd;
import com.ibm.btools.blm.compoundcommand.navigator.remove.RemoveInlineComplexTypeTemplateNAVCmd;
import com.ibm.btools.blm.compoundcommand.navigator.remove.RemoveOperationNAVCmd;
import com.ibm.btools.blm.compoundcommand.navigator.remove.RemoveWSDLPortTypeNAVCmd;
import com.ibm.btools.blm.compoundcommand.navigator.remove.RemoveXSDFileNAVCmd;
import com.ibm.btools.blm.compoundcommand.util.CCRuntimeException;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportMessage;
import com.ibm.btools.blm.ie.imprt.ImportSession;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.CreateExternalModelElementPrecondition;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.InvalidTypeException;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildContainerNode;
import com.ibm.btools.blm.ui.navigation.model.AbstractNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationExternalServiceCatalogNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationExternalServiceCatalogsNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationInlineComplexTypeDefinitionNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationInlineComplexTypeDefinitionsNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationInlineComplexTypeTemplateNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationInlineComplexTypeTemplatesNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationInlineXSDSchemaNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationOperationNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationWSDLFileNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationWSDLPortTypeNode;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.externalmodels.ExternalSchema;
import com.ibm.btools.bom.model.externalmodels.ExternalService;
import com.ibm.btools.bom.model.externalmodels.ServiceInterface;
import com.ibm.btools.infrastructure.util.UIDGenerator;

/**
 * @author andyxz
 * This factory is responsible for creating the catalog of <code>ExternalService</code>
 */
public class ExternalServiceFactory extends NavigatorFactory {
	//	 copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	/**
	 * Create a new external service catalog to mimic the external service catalog
	 * @param navNode
	 * @param processModelName
	 * @param comment
	 * @return the newly created navigation node
	 */
	public NavigationWSDLFileNode create(
			final String projectName,
			final AbstractNode navNode,
			final ExternalService externalServiceModel, 
			final String comment,
			ImportSession session,
			final boolean isDefault, 
			final boolean resolve) {
		
		LoggingUtil.traceEntry(this, "create");	
		boolean fileLengthExceeded = false;
		
		AbstractChildContainerNode parentNode = null;			
		boolean isProjectNode = false;
		NavigationWSDLFileNode newNode = null;
		
		String name = externalServiceModel.getName();	
		assertModelNameLength(name);
		//name = getProperRootElementName(session, name);	
		
		if (navNode instanceof NavigationExternalServiceCatalogNode) {			
			parentNode = (NavigationExternalServiceCatalogNode) navNode;			
			//if the user didn't pass in the node, we need to look up the default node,
			//which is NavigationExternalServiceCatalogsNode
		} else if(navNode == null || !(navNode instanceof NavigationExternalServiceCatalogNode )){
			parentNode = NavigatorUtil.getDefaultCatalogForExternalService(projectName);
			if(parentNode == null){
				LoggingUtil.trace(this, "create", "No default node available.");					
				LoggingUtil.traceExit(this, "create");
				return null;
			}
		}
		else{
			LoggingUtil.trace(this, "create", "This is a not valid navigation node.");					
			LoggingUtil.traceExit(this, "create");
			return null;
		}
		
		// Check to see if the model is already existed.
		//Andy leave it for now - how to validate??
		CreateExternalModelElementPrecondition precondition = new CreateExternalModelElementPrecondition();
		precondition.setParent(parentNode);
		precondition.setName(name);
		
		int flag = VALID; 
		try {
			if(!precondition.isValid(false)) {
				LoggingUtil.trace(this, "create", "The precondition is failed.");
				if (resolve) {
					LoggingUtil.trace(this, "create", "This is a predefined type.");					
					LoggingUtil.traceExit(this, "create");	
					Object list = session.getContext().get(IImportFramework.CONTEXT_RESOLVED_TYPE);
					if (list != null && list instanceof List) {
						((List)list).add(getNode(parentNode, name));
					}
					return getNode(parentNode, name);
				}				
				if (!isDefault) {				
					flag = getCollisionHandleRequest(session, name,false);				
					if (flag == IGNORE) {
						LoggingUtil.trace(this, "create", "The user chooses to ignore the name collision.");
						LoggingUtil.traceExit(this, "create");	
						return null;
					} else if (flag == OVERWRITE) {
						LoggingUtil.trace(this, "create", "The user chooses to overwrite the name collision.");	
						consolidateExternalService(getNode(parentNode, name),externalServiceModel,projectName);	
					}
				} else {
					flag = OVERWRITE;
				}				
			}
		} catch (InvalidTypeException e) {
			// Ignore
			LoggingUtil.traceExit(this, "create");		
			return null;
		}
		
		if (flag != OVERWRITE) {
			createNode(
					parentNode,
					session,
					projectName,
					name,
					comment,
					fileLengthExceeded);			
		}
		
		// The Navigator model is created, but need to update it.
		newNode = getNode(parentNode, name);
		
		if (newNode == null && fileLengthExceeded == false) {
			LoggingUtil.logWarning(session, IeMessageKeys.CANNOT_CREATE_EXTERNAL_SERVICE, 
					new String[]{name}, null, IImportResultMessages.CANNOT_CREATE_NAVIGATION_NODE);		
		} 
		
		LoggingUtil.traceExit(this, "create");	
		return newNode;
	}
	
	/**
	 * Consolidate the portTypes and operations contained by the external service
	 * @param existingNode
	 * @param importedService
	 */
	private void consolidateExternalService(NavigationWSDLFileNode existingNode,
			ExternalService importedService,
			String projectName){
		List removableOperations = new ArrayList();
		List removablePortTypes = new ArrayList();
		List removableInlineSchmas = new ArrayList();
		if(existingNode == null || importedService == null){
			return;
		}		
		EList existingPortTypeNodes = existingNode.getWsdlPortType();
		EList existingInlineSchemaNodes = existingNode.getOnlineXSDSchema();
		EList members = importedService.getOwnedMember();		
		
		if(existingPortTypeNodes != null){
			Iterator existingPortTypeNodesIterator = existingPortTypeNodes.iterator();
			while(existingPortTypeNodesIterator.hasNext()){
				boolean matchFound = false;
				AbstractChildContainerNode existingPortType = (AbstractChildContainerNode)existingPortTypeNodesIterator.next();
				//EList importedPortTypes = importedService.getOwnedMember();
				if(members != null){
					Iterator memberIterator = members.iterator();
					while(memberIterator.hasNext()){
						Object eachMember = memberIterator.next();
						if(eachMember instanceof ServiceInterface){
							ServiceInterface eachPortType = (ServiceInterface)eachMember;
							if(existingPortType.getLabel() != null && existingPortType.getLabel().equals(eachPortType.getName())){
								matchFound = true;
								//match operation
								if(existingPortType instanceof NavigationWSDLPortTypeNode){
									NavigationWSDLPortTypeNode theNode =  (NavigationWSDLPortTypeNode)existingPortType;
									EList existingOperations = theNode.getOperation();
									if(existingOperations != null && existingOperations.size() > 0){
										for(int i= 0; i < existingOperations.size();i++){
											boolean foundMatchName = false;
											NavigationOperationNode operationNode = (NavigationOperationNode)existingOperations.get(i);
											EList importedOperations = eachPortType.getOwnedMember();
											if(importedOperations != null && importedOperations.size() > 0){
												for(int j=0;j<importedOperations.size();j++){
													NamedElement eachOperation = (NamedElement)importedOperations.get(j);
													if(operationNode.getLabel()!= null && operationNode.getLabel().equals(eachOperation.getName())){
														foundMatchName = true;													
														break;
													}
												}
											}
											if(!foundMatchName){
												removableOperations.add(operationNode);
											}
										}
									}
								}
								break;
							}
						}
					}
				}				
				if(!matchFound){
					removablePortTypes.add(existingPortType);
				}
			}
		}		
		
		//AZ added to remove inline schemas for file attachment and other
		
		if(existingInlineSchemaNodes != null && existingInlineSchemaNodes.size()>0){
			Iterator existingInlineschemaIterator = existingInlineSchemaNodes.iterator();
			while(existingInlineschemaIterator.hasNext()){	
				boolean matchFound = false;
				AbstractChildContainerNode existingInlineSchema = (AbstractChildContainerNode)existingInlineschemaIterator.next();
				if(BOMUtil.FILE_ATTACHMENT_NODE_NAME.equals(existingInlineSchema.getLabel())){
					removableInlineSchmas.add(existingInlineSchema);
					continue;
				}
				Iterator memberIterator = members.iterator();
				while(memberIterator.hasNext()){
					Object eachMember = memberIterator.next();
					if(eachMember instanceof ExternalSchema){
						ExternalSchema inlineschma = (ExternalSchema)eachMember;
						if(existingInlineSchema.getLabel() != null && existingInlineSchema.getLabel().equals(inlineschma.getName())){
							matchFound = true;
							break;
						}
					}
				}
				if(!matchFound){
					removableInlineSchmas.add(existingInlineSchema);
				}
			}
		}		
		
		if(removableOperations.size() > 0){
			//remove the operations		
			Iterator removableOperationIterator = removableOperations.iterator();
			while(removableOperationIterator.hasNext()){
				Object eachOperation = removableOperationIterator.next();
				if(eachOperation instanceof NavigationOperationNode){
					NavigationOperationNode theOperation = (NavigationOperationNode)eachOperation;
					RemoveOperationNAVCmd operationRemoveCmd = new RemoveOperationNAVCmd();
					operationRemoveCmd.setProjectName(projectName);
					operationRemoveCmd.setParentNavigatorNode(theOperation);
					closeEditors(theOperation,projectName);
					if(operationRemoveCmd.canExecute()){
						operationRemoveCmd.execute();
					}					
				}
			}
		}
		
		if(removablePortTypes.size() > 0){
			//remove the port types
			Iterator removablePortTypeIterator = removablePortTypes.iterator();
			while(removablePortTypeIterator.hasNext()){
				Object eachPortType = removablePortTypeIterator.next();
				if(eachPortType instanceof NavigationWSDLPortTypeNode){
					NavigationWSDLPortTypeNode thePortType = (NavigationWSDLPortTypeNode)eachPortType;	
					RemoveWSDLPortTypeNAVCmd portTypeRemoveCmd = new RemoveWSDLPortTypeNAVCmd();
					portTypeRemoveCmd.setProjectName(projectName);
					portTypeRemoveCmd.setParentNavigatorNode(thePortType);
					List operations = new ArrayList(thePortType.getOperation());
					if(operations != null && operations.size() > 0){
						Iterator operationsIt = operations.iterator();
						while(operationsIt.hasNext()){
							NavigationOperationNode theOperation = (NavigationOperationNode)operationsIt.next();							
							RemoveOperationNAVCmd operationRemoveCmd = new RemoveOperationNAVCmd();
							operationRemoveCmd.setProjectName(projectName);
							operationRemoveCmd.setParentNavigatorNode(theOperation);
							closeEditors(theOperation,projectName);
							if(operationRemoveCmd.canExecute()){
								operationRemoveCmd.execute();
							}					
						}
					}
					if(portTypeRemoveCmd.canExecute()){
						portTypeRemoveCmd.execute();						
					}					
				}
			}
		}
		
		if(removableInlineSchmas.size() > 0){
			//remove the inline schemas
			Iterator removableInlineSchemaIterator = removableInlineSchmas.iterator();
			while(removableInlineSchemaIterator.hasNext()){
				Object eachInlineSchema = removableInlineSchemaIterator.next();
				if(eachInlineSchema instanceof NavigationInlineXSDSchemaNode){
					NavigationInlineXSDSchemaNode theSchema = (NavigationInlineXSDSchemaNode)eachInlineSchema;
					RemoveXSDFileNAVCmd inlineSchemaRemoveCmd = new RemoveXSDFileNAVCmd();					
					inlineSchemaRemoveCmd.setProjectName(projectName);
					inlineSchemaRemoveCmd.setParentNavigatorNode(theSchema);
					if(inlineSchemaRemoveCmd.canExecute()){
						//close the inline type nodes if it's opened
						NavigationInlineComplexTypeDefinitionsNode defChildren = theSchema.getInlineComplexTypeDefinitionsNode();
						List defNodes = null;
						if(defChildren != null){
							defNodes = new ArrayList(defChildren.getInlineComplexTypeDefinitionNode());							
						}
						if(defNodes != null && defNodes.size() > 0){
							Iterator defIt = defNodes.iterator();
							while(defIt.hasNext()){
								NavigationInlineComplexTypeDefinitionNode theNode = (NavigationInlineComplexTypeDefinitionNode)defIt.next();
								closeEditors(theNode,projectName);
								RemoveInlineComplexTypeDefinitionNAVCmd inlineDefNodeRemoveCmd = new RemoveInlineComplexTypeDefinitionNAVCmd();
								inlineDefNodeRemoveCmd.setProjectName(projectName);
								inlineDefNodeRemoveCmd.setParentNavigatorNode(theNode);
								if(inlineDefNodeRemoveCmd.canExecute()){
									inlineDefNodeRemoveCmd.execute();
								}
							}
						}
						
						NavigationInlineComplexTypeTemplatesNode tmpChildren = theSchema.getInlineComplexTypeTemplatesNode();
						List tmpNodes  = null;
						if(tmpChildren != null){
							tmpNodes = new ArrayList(tmpChildren.getInlineComplexTypeTemplateNode());
						}
						if(tmpNodes != null && tmpNodes.size() > 0){
							Iterator tmpIt = tmpNodes.iterator();
							while(tmpIt.hasNext()){
								NavigationInlineComplexTypeTemplateNode theNode = (NavigationInlineComplexTypeTemplateNode)tmpIt.next();
								closeEditors(theNode,projectName);
								RemoveInlineComplexTypeTemplateNAVCmd inlineTempNodeRemoveCmd = new RemoveInlineComplexTypeTemplateNAVCmd();
								inlineTempNodeRemoveCmd.setProjectName(projectName);
								inlineTempNodeRemoveCmd.setParentNavigatorNode(theNode);
								if(inlineTempNodeRemoveCmd.canExecute()){
									inlineTempNodeRemoveCmd.execute();
								}
							}
						}						
						inlineSchemaRemoveCmd.execute();
					}
				}
			}
		}
		//The node has been modified and needs to be saved
		NavigatorUtil.saveNode(existingNode,projectName);
	}
	
	/**
	 * Return the process catalog node by name
	 * @param parentNode
	 * @param processModelName
	 * @return
	 */
	private NavigationWSDLFileNode getNode(final AbstractChildContainerNode parentNode, final String wsdlFileName) {
		LoggingUtil.traceEntry(this, "getNode");
		AbstractChildContainerNode navContainerNode = null;
		String label = null;
		if (parentNode instanceof NavigationExternalServiceCatalogNode)  {	
			EList fileList = ((NavigationExternalServiceCatalogNode) parentNode).getWsdlFile();
			if(fileList == null){
				LoggingUtil.traceExit(this, "getNode");
				return null;
			}
			Iterator catalogsNodeIterator = fileList.iterator();
			
			while(catalogsNodeIterator.hasNext()) {
				navContainerNode = (AbstractChildContainerNode) catalogsNodeIterator.next();
				label = navContainerNode.getLabel();				
				if (label != null && label.compareTo(wsdlFileName) == 0) {
					// This is the Navigator model
					LoggingUtil.traceExit(this, "getNode");
					if(navContainerNode instanceof NavigationWSDLFileNode){
						return (NavigationWSDLFileNode)navContainerNode;
					}					
				}			
			}
			//handle the case that the user didn't pass in a node
		} else if(parentNode instanceof NavigationExternalServiceCatalogsNode){
			Object esCat = ((NavigationExternalServiceCatalogsNode)parentNode).getExternalServiceCatalog().get(0);
			if(esCat instanceof NavigationExternalServiceCatalogNode){
				EList fileList = ((NavigationExternalServiceCatalogNode) esCat).getWsdlFile();
				if(fileList == null){
					LoggingUtil.traceExit(this, "getNode");
					return null;
				}
				Iterator catalogsNodeIterator = fileList.iterator();				
				while(catalogsNodeIterator.hasNext()) {
					navContainerNode = (AbstractChildContainerNode) catalogsNodeIterator.next();
					label = navContainerNode.getLabel();				
					if (label != null && label.compareTo(wsdlFileName) == 0) {
						// This is the Navigator model
						LoggingUtil.traceExit(this, "getNode");
						if(navContainerNode instanceof NavigationWSDLFileNode){
							return (NavigationWSDLFileNode)navContainerNode;
						}					
					}			
				}
			}
		}
		LoggingUtil.traceExit(this, "getNode");
		return null;
	}
	
	private void createNode(
			AbstractNode parentNode,
			ImportSession session,
			String projectName,
			String domainName,
			String description,
			boolean fileLengthExceeded){
		
		AddWSDLFileNAVCmd addWsdlFileCmd = new AddWSDLFileNAVCmd();
		addWsdlFileCmd.setProjectName(projectName);
		addWsdlFileCmd.setDefaultID("WSDL file");
		addWsdlFileCmd.setAbstractLibraryChildNode((NavigationExternalServiceCatalogNode)parentNode);
		//addWsdlFileCmd.setDefaultModelID(UIDGenerator.getUID("BLMNAV")); //PredefTypesConst.PREDEF_EXT_SERVICE_FUID);
		addWsdlFileCmd.setDescription(description);
		addWsdlFileCmd.setDomainModelName(domainName);
		addWsdlFileCmd.setLabel(domainName);
		addWsdlFileCmd.setParentInformationModelURI((String)(((NavigationExternalServiceCatalogNode)parentNode).getEntityReference()));
		
		if (addWsdlFileCmd.canExecute()) {
			try {
				addWsdlFileCmd.execute();				
			} catch (RuntimeException e) {					
				if (e instanceof CCRuntimeException) {
					CCRuntimeException ccEx = (CCRuntimeException) e;
					if (ccEx.getCode().equals(ImportMessage.FILE_LENGTH_EXCEEDED)) {
						fileLengthExceeded = true;						
						LoggingUtil.logWarning(session, IeMessageKeys.FILE_LENGTH_EXCEEDED_GENERIC, 
								new String[]{domainName}, null, IImportResultMessages.CANNOT_CREATE_NAVIGATION_NODE);
						throw e;
					}
				}
				
				LoggingUtil.trace(this, "create", "Add command throws a RuntimeException.");					
			}
		} else {
			LoggingUtil.trace(this, "create", "Add command cannot be executed.");
		}		
	}
	
}
