package com.ibm.btools.blm.ie.imprt.rule.navigator.factory;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.compoundcommand.navigator.add.AddReusableHumanTaskNAVCmd;
import com.ibm.btools.blm.compoundcommand.navigator.remove.RemoveReusableHumanTaskNAVCmd;
import com.ibm.btools.blm.compoundcommand.util.CCRuntimeException;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportMessage;
import com.ibm.btools.blm.ie.imprt.ImportSession;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.CreateProcessModelElementPrecondition;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.InvalidTypeException;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.NavigationHumanTaskNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationHumanTasksNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationProcessCatalogNode;
import com.ibm.btools.blm.ui.navigation.model.util.NavigationConstants;
import com.ibm.btools.bom.model.processes.activities.Activity;

/**
 * Factory class to create a task
 */
public class HumanTaskFactory extends NavigatorFactory {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	/**
	 * Create a new task
	 * @param navNode
	 * @param taskName
	 * @param comment
	 * @return the newly created navigation node
	 */
	public NavigationHumanTaskNode create(final NavigationProcessCatalogNode parentNode, final Activity task, 
		final String comment, ImportSession session, final boolean resolve) {

		LoggingUtil.traceEntry(this, "create");	
		boolean fileLengthExceeded = false;

		String projectName = null;
		boolean isProjectNode = false;
		boolean canRemove = true;
		NavigationHumanTaskNode newNode = null;
					
		String name = task.getName();		
		name = getProperRootElementName(session, name);
		projectName = parentNode.getProjectNode().getLabel();
		
		// Check to see if the model is already existed.
		CreateProcessModelElementPrecondition precondition = new CreateProcessModelElementPrecondition();
		precondition.setParent(parentNode);
		precondition.setName(name);
		precondition.setType(NavigationConstants.HUMAN_TASK);		
		 
		int flag = VALID;
		try { 
			if(!precondition.isValid()) {
				LoggingUtil.trace(this, "create", "The precondition is failed.");
				if (resolve) {
					LoggingUtil.trace(this, "create", "This is a predefined type.");					
					LoggingUtil.traceExit(this, "create");	
					Object list = session.getContext().get(IImportFramework.CONTEXT_RESOLVED_TYPE);
					if (list != null && list instanceof List) {
						((List)list).add(getNode(parentNode, name));
					}
					return getNode(parentNode, name);
				}
				flag = getCollisionHandleRequest(session, task.getName());
				if (flag == IGNORE) {
					LoggingUtil.trace(this, "create", "The user chooses to ignore the name collision.");
					LoggingUtil.traceExit(this, "create");	
					return null;
				} else if (flag == OVERWRITE) {
					LoggingUtil.trace(this, "create", "The user chooses to overwrite the name collision.");	
					NavigationHumanTaskNode taskNode = getNode(parentNode, name);
					if (isNavigatorNodeAlreadyExisted(session, taskNode)) {
						// IGNORE
						return null;	
					}					
					closeEditors(taskNode, projectName);				
					RemoveReusableHumanTaskNAVCmd navCmd = new RemoveReusableHumanTaskNAVCmd();
					navCmd.setProjectName(projectName);
					navCmd.setParentNavigatorNode(taskNode);
					if (navCmd.canExecute()) {
						try {
							navCmd.execute();
						} catch (RuntimeException e) {
							LoggingUtil.trace(this, "create", "The remove command throws a RuntimeException.");							
							canRemove = false;
						}
					} else {
						LoggingUtil.trace(this, "create", "Cannot execute the remove command.");						
						canRemove = false;
					}	
				} else if (flag == NEW) {
					LoggingUtil.trace(this, "create", "The user chooses to create an element with a new name.");
					name = generateName(parentNode, name, precondition);		
					if (name == null) {											
						LoggingUtil.traceExit(this, "create");	
						return null;	
					}
				} else if (flag == CANCEL) {
					LoggingUtil.trace(this, "create", "The user chooses to cancel the import.");
					throw new OperationCanceledException();
				}
			}
		} catch (InvalidTypeException e) {
			// Ignore
			LoggingUtil.traceExit(this, "create");						
			return null;
		}
		
		if (canRemove) {		
			NavigatorUtil.recordNewNode(session,task);
			// Parameters:
			// 1. project name
			// 2. task name
			// 3. parent Navigator node
			AddReusableHumanTaskNAVCmd navCmd = new AddReusableHumanTaskNAVCmd();
			navCmd.setProjectName(projectName);
			navCmd.setDomainModelName(name);
			navCmd.setDescription(comment);
			navCmd.setAbstractLibraryChildNode(parentNode);		
			navCmd.setParentInformationModelURI((String)parentNode.getEntityReference());
			navCmd.setCreatePrimaryOwner(false);
			if (navCmd.canExecute()) {
				try {
					navCmd.execute();
				} catch (RuntimeException e) {
					if (e instanceof CCRuntimeException) {
						CCRuntimeException ccEx = (CCRuntimeException) e;
						if (ccEx.getCode().equals(ImportMessage.FILE_LENGTH_EXCEEDED)) {
							fileLengthExceeded = true;						
							LoggingUtil.logWarning(session, IeMessageKeys.FILE_LENGTH_EXCEEDED_GENERIC, 
								new String[]{name}, null, IImportResultMessages.CANNOT_CREATE_NAVIGATION_NODE);							
						}
					}
					LoggingUtil.trace(this, "create", "Add command throws a RuntimeException.");					
				}
			} else {
				LoggingUtil.trace(this, "create", "Add command cannot be executed.");
			}		
			
			// The Navigator model is created, but need to update it.
			newNode = getNode(parentNode, name);
		}
		
		if (newNode == null && fileLengthExceeded == false) {
			LoggingUtil.logWarning(session, IeMessageKeys.CANNOT_CREATE_TASK, 
				new String[]{name}, null, IImportResultMessages.CANNOT_CREATE_NAVIGATION_NODE);		
		} else {
			if (name.compareTo(task.getName()) != 0)
			LoggingUtil.logWarning(session, IeMessageKeys.CREATE_TASK_DIFF_NAME, 
				new String[]{task.getName(), name}, null);			
		}
		
		LoggingUtil.traceExit(this, "create");	
		return newNode;	
	}		

	/**
	 * Return the task node by name
	 * @param parentNode
	 * @param taskName
	 * @return
	 */
	private NavigationHumanTaskNode getNode(final NavigationProcessCatalogNode parentNode, final String taskName) {
		LoggingUtil.traceEntry(this, "getNode");
		NavigationHumanTasksNode tasksNode = parentNode.getHumanTasksNode();
		if (tasksNode == null) {
			LoggingUtil.traceExit(this, "getNode");
			return null;
		}
		EList nodes = tasksNode.getHumanTaskNode();		
		Iterator taskNodeIterator = nodes.iterator();
		NavigationHumanTaskNode navContainerNode = null;
		String label = null; 
		while(taskNodeIterator.hasNext()) {
			navContainerNode = (NavigationHumanTaskNode) taskNodeIterator.next();
			label = navContainerNode.getLabel();
			if (label !=null && taskName.compareTo(label) == 0) {
				// This is the Navigator model
				LoggingUtil.traceExit(this, "getNode");
				return navContainerNode;
			}
		}
		LoggingUtil.traceExit(this, "getNode");
		return null;
	}		
}
