package com.ibm.btools.blm.ie.imprt.rule.navigator.factory;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.EList;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.PlatformUI;
import org.osgi.framework.Bundle;

import com.ibm.btools.blm.compoundcommand.util.CCRuntimeException;
import com.ibm.btools.blm.ie.imprt.IImportQuery;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.ImportOptions;
import com.ibm.btools.blm.ie.imprt.ImportSession;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.CreateModelPrecondition;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NamingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.dialogs.BusinessItemImportCollisionMessageDialog;
import com.ibm.btools.blm.ui.dialogs.ImportCollisionMessageDialog;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildContainerNode;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildLeafNode;
import com.ibm.btools.blm.ui.navigation.model.AbstractNode;
import com.ibm.btools.util.exception.BTRuntimeException;



/**
 * This class provides generic method for the Navigator Factory
 */
public class NavigatorFactory {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	

	protected static final int VALID 		= 0;
	protected static final int CANCEL 		= -1;
	protected static final int IGNORE		= IImportQuery.IGNORE;//1 Skip
	protected static final int OVERWRITE	= IImportQuery.OVERWRITE;//2 Replace
	protected static final int NEW			= IImportQuery.NEW_COPY;//3 New Copy

	private final int MAX_COUNT		= 100; 
	private final static String SCHEMA_OPTION_KEY = "external_schema_collision_option";
	private final static String SERVICE_OPTION_KEY = "external_service_collision_option";
	private final static String LENGATH_MODEL_NAME_ERROR_CODE = "CCB9999E";

	protected int getCollisionHandleRequest(ImportSession session, final String name) {		
		ImportOptions options = session.getImportOptions();
		//defect 164847 handle the user UI overwrite option, the existing element
		//will be overwriten if the user check the overwriten option in the import
		//wizard, a new element will be created otherwise
		IImportQuery query = session.getOverwriteQuery();
		if(query != null){
			int wizardSelection = query.queryImportOption(name);
			if(wizardSelection == OVERWRITE ){
				options.setAlwaysIgnore(false);
				options.setAlwaysNewCopy(false);
				options.setOverwrite(true);
				options.setAlwaysOverwrite(true);
				session.setImportOption(options);
				return OVERWRITE;
			}	
		}		
		options.setAlwaysIgnore(false);
		options.setAlwaysNewCopy(true);
		options.setAlwaysOverwrite(false);
		session.setImportOption(options);		
		return NEW;		
	}	
	/**
	 * Get the handle request flag of collision
	 * @param session
	 * @param name
	 * @return
	 */
	protected int getCollisionHandleRequest(ImportSession session, final String name, boolean isSchema) {
		return getCollisionHandleRequest(session,name,false,isSchema,false);
		//		LoggingUtil.traceEntry(this, "getCollisionHandleRequest");
		//		
		//		IImportQuery query = session.getOverwriteQuery();
		//		
		//		ImportOptions options = session.getImportOptions();
		//		
		//		if (options.isAlwaysIgnore()) {		
		//			LoggingUtil.trace(this, "getCollisionHandleRequest", "always ignore");		
		//			LoggingUtil.traceExit(this, "getCollisionHandleRequest");
		//			return IGNORE;
		//		} else if (options.isAlwaysOverwrite()) {
		//		LoggingUtil.trace(this, "getCollisionHandleRequest", "overwrite");		
		//		LoggingUtil.traceExit(this, "getCollisionHandleRequest");
		//		return OVERWRITE;
		//		}else if (options.isAlwaysNewCopy()) {
		//			LoggingUtil.trace(this, "getCollisionHandleRequest", "new");		
		//			LoggingUtil.traceExit(this, "getCollisionHandleRequest");
		//			return NEW;
		//		} else {
		//			if (query == null) {
		//				// Default
		//				LoggingUtil.trace(this, "getCollisionHandleRequest", "default = ignore");		
		//				LoggingUtil.traceExit(this, "getCollisionHandleRequest");
		//				return IGNORE;
		//			}
		//			int reply = query.queryImportOption(name);
		//			if (reply == IImportQuery.CANCEL) {
		//				LoggingUtil.trace(this, "getCollisionHandleRequest", "cancel");		
		//				LoggingUtil.traceExit(this, "getCollisionHandleRequest");
		//				return CANCEL;		
		//			} else if (reply == IImportQuery.ALWAYS_IGNORE) {
		//				options.setAlwaysIgnore(true);
		//				options.setAlwaysNewCopy(false);
		//				options.setAlwaysOverwrite(false);
		//				session.setImportOption(options);
		//				LoggingUtil.trace(this, "getCollisionHandleRequest", "always ignore");		
		//				LoggingUtil.traceExit(this, "getCollisionHandleRequest");
		//				return IGNORE;					
		//			} else if (reply == IImportQuery.ALWAYS_OVERWRITE) {
		//				options.setAlwaysIgnore(false);
		//				options.setAlwaysNewCopy(false);
		//				options.setAlwaysOverwrite(true);
		//				session.setImportOption(options);
		//				LoggingUtil.trace(this, "getCollisionHandleRequest", "always overwrite");		
		//				LoggingUtil.traceExit(this, "getCollisionHandleRequest");
		//				return OVERWRITE;				
		//			} else if (reply == IImportQuery.ALWAYS_NEW_COPY) {
		//				options.setAlwaysIgnore(false);
		//				options.setAlwaysNewCopy(true);
		//				options.setAlwaysOverwrite(false);
		//				session.setImportOption(options);
		//				LoggingUtil.trace(this, "getCollisionHandleRequest", "always new");		
		//				LoggingUtil.traceExit(this, "getCollisionHandleRequest");
		//				return NEW;
		//			} else if (reply == IImportQuery.IGNORE) {
		//				LoggingUtil.trace(this, "getCollisionHandleRequest", "ignore");		
		//				LoggingUtil.traceExit(this, "getCollisionHandleRequest");
		//				return IGNORE;
		//			} else if (reply == IImportQuery.OVERWRITE) {
		//				LoggingUtil.trace(this, "getCollisionHandleRequest", "overwrite");		
		//				LoggingUtil.traceExit(this, "getCollisionHandleRequest");
		//				return OVERWRITE;
		//			} else if (reply == IImportQuery.NEW_COPY) {
		//				LoggingUtil.trace(this, "getCollisionHandleRequest", "new");		
		//				LoggingUtil.traceExit(this, "getCollisionHandleRequest");
		//				return NEW;
		//			}		
		//		}
		//		
		//		LoggingUtil.traceExit(this, "getCollisionHandleRequest");		
		//		return VALID;	
	}

	protected int getCollisionHandleRequestForBusinessItem(ImportSession session, final String name, boolean isSchema) {
		if(session != null){
			//defect 154753 skip overwrite for XML import
			if(session.getImportOptions()!= null &&
					!(session.getImportOptions().isCollisionHandlingEnabled())){
				return getCollisionHandleRequest(session,name);
			}
		}
		return getCollisionHandleRequest(session,name,false,isSchema,true);
	}


	protected int getCollisionHandleRequest(ImportSession session, final String name, boolean flag,boolean isSchema,boolean isBusinessItem) {
		int result = OVERWRITE;
		ImportOptions options = session.getImportOptions();	
		if(options.isAlwaysOverwrite()){
			recordOption(options,OVERWRITE,name,isSchema);
			return OVERWRITE;
		}else if(options.isAlwaysIgnore()){
			recordOption(options,IGNORE,name,isSchema);
			return IGNORE;
		}

		//AZ added to handle the wizard user options for overwrite
		//if the user opts for overwrite in the wizard, there shouldn't be any option prompt
		//AZ added to defend NPE for wsrr overwrite option, which is set through the import option
		IImportQuery query = session.getOverwriteQuery();
		if(query != null){
			int wizardSelection = query.queryImportOption(name);
			if(wizardSelection == OVERWRITE ){
				options.setAlwaysIgnore(false);
				options.setAlwaysNewCopy(false);
				options.setOverwrite(true);
				options.setAlwaysOverwrite(true);
				session.setImportOption(options);
				return OVERWRITE;
			}//To be compatible with the IImportQuery API, Defect 76418
		else if(wizardSelection == IImportQuery.IGNORE){
			options.setAlwaysIgnore(true);
			options.setAlwaysNewCopy(false);
			options.setOverwrite(false);
			options.setAlwaysOverwrite(false);
			options.setIgnore(true);
			session.setImportOption(options);
			return IImportQuery.IGNORE;
		}else if(wizardSelection == IImportQuery.ALWAYS_IGNORE){
			recordOption(options,IGNORE,name,isSchema);			
			return IImportQuery.IGNORE;
		}else if(wizardSelection == IImportQuery.ALWAYS_OVERWRITE){
			recordOption(options,OVERWRITE,name,isSchema);			
			return IImportQuery.OVERWRITE;
		}else if(wizardSelection == IImportQuery.NEW_COPY||
				wizardSelection == IImportQuery.ALWAYS_NEW_COPY){			
			return IImportQuery.NEW_COPY;
		}
	}

		if(isBusinessItem){
			DisplayBusinessItemOverwriteDialog biDlg = new DisplayBusinessItemOverwriteDialog(name);
			Display.getDefault().syncExec(biDlg);
			result = biDlg.getResult();
			if(result == BusinessItemImportCollisionMessageDialog.yes ){
				options.setOverwrite(true);
				result = OVERWRITE;
			}else if(result == BusinessItemImportCollisionMessageDialog.no){
				options.setIgnore(true);
				result = IGNORE;
			}else{
				options.setNewCopy(true);
				result = NEW;
			}
			if(result == OVERWRITE){
				recordOption(options,OVERWRITE,name,isSchema);
			}else if(result == IGNORE){
				recordOption(options,IGNORE,name,isSchema);
			}
			return result;
		}else{
			DisplayOverwriteDialog dlg = new DisplayOverwriteDialog(name);
			Display.getDefault().syncExec(dlg);
			result = dlg.getResult();

			if (result == ImportCollisionMessageDialog.yes){
				options.setOverwrite(true);
				result = OVERWRITE;
			}
			else if (result == ImportCollisionMessageDialog.no){
				options.setIgnore(true);
				result = IGNORE;
			}
			else if (result == ImportCollisionMessageDialog.yesToAll) {
				options.setAlwaysIgnore(false);
				options.setAlwaysNewCopy(false);
				options.setOverwrite(true);
				options.setAlwaysOverwrite(true);
				session.setImportOption(options);
				result = OVERWRITE;
			}
			else if (result == ImportCollisionMessageDialog.noToAll) {
				options.setIgnore(true);
				options.setAlwaysIgnore(true);
				options.setAlwaysNewCopy(false);
				options.setAlwaysOverwrite(false);
				session.setImportOption(options);
				result = IGNORE;
			}
			if(result == OVERWRITE){
				recordOption(options,OVERWRITE,name,isSchema);
			}else if(result == IGNORE){
				recordOption(options,IGNORE,name,isSchema);
			}
			return result;
		}
		}
		/**
		 * Generate the name for collision
		 * @param parentNode
		 * @param name
		 * @return
		 */
		protected String generateName(final AbstractChildContainerNode parentNode, final String name, CreateModelPrecondition precondition) {
			LoggingUtil.traceEntry(this, "generateName");
			String generatedName = name;
			int counter = 1;
			boolean isUnique = false; 
			while (!isUnique && counter < MAX_COUNT) {
				generatedName = name + "_" + counter;
				precondition.setParent(parentNode);
				precondition.setName(generatedName);
				isUnique = precondition.isValid();
				counter++;	
			}
			if (!isUnique) {
				LoggingUtil.trace(this, "generateName", "Cannot create an unique name.");
				LoggingUtil.traceExit(this, "generateName");
				return null;
			}
			LoggingUtil.traceExit(this, "generateName");
			return generatedName;
		}

		/**
		 * Check to see if the navigator is already exist
		 * @param session
		 * @param node
		 * @return
		 */
		protected boolean isNavigatorNodeAlreadyExisted(final ImportSession session, final AbstractNode node) {
			ImportContext context = session.getContext();

			Iterator iterator = context.values().iterator();
			Object value = null;
			while (iterator.hasNext()) {
				value = iterator.next();
				if (value == node) {
					return true;
				}
			}

			return false;
		}

		/**
		 * @param session
		 * @param name
		 * @return
		 */
		protected String getProperRootElementName(ImportSession session, String name) {
			return NamingUtil.getValidRootElementName(session, name);
		}	

		/**
		 * Assert the external model name, this assertion only applies to the external
		 * model import model creation factory
		 * This exception is meant to be caught by the BOM element creation rule and present
		 * message to the user on the failure, the constructor signiture is as the following:
		 * CCRuntimeException(Throwable exception, 
						String eMessage, 
						String eCode, 
						Object[] params, 
						String eSeverity, 
						String eResource, 
						String eClass, 
						String eMethod) {
		 * @param modelName
		 */
		protected void assertModelNameLength(String modelName){
//			if(modelName != null && modelName.length()> NamingUtil.MAX_LENGTH ){			
//			CCRuntimeException e = new CCRuntimeException(new RuntimeException(),
//			"The model name exceeds 50 characters",
//			LENGATH_MODEL_NAME_ERROR_CODE,
//			new Object[]{},
//			null,
//			null,
//			this.getClass().getName(),
//			"method create()"
//			);
//			throw e;	
//			}	
		}

		/**
		 * Close the view editor associated with this BLM URI
		 * This method has to be called by an UI thread. A NullPointerException will be raised otherwise
		 * @param blmURI
		 */
		protected void closeEditor(String blmURI) {

			try {		
				// Load the plugin
				Bundle plugin = Platform.getBundle("com.ibm.btools.blm.ui.navigation");

				// Just in case it doesn't exist
				if (plugin == null) 
					return;

				Class BLMEditorInputClass = plugin.loadClass("com.ibm.btools.blm.ui.navigation.manager.util.BLMEditorInput");

				Bundle pluginNavModel = Platform.getBundle("com.ibm.btools.blm.ui.navigation.model");
				if (pluginNavModel == null){
					return;
				}

				Class AbstractChildLeafNodeClass = pluginNavModel.loadClass("com.ibm.btools.blm.ui.navigation.model.AbstractChildLeafNode");

				// Returns a array of references to open editors in this page. 
				//	Note that each page has its own editors; editors are never shared between pages. 
				IEditorReference[] refs =
					PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getEditorReferences();

				IEditorReference ref = null;
				IEditorPart editor = null;
				for (int i = 0; i < refs.length; i++) {
					ref = refs[i];
					editor = ref.getEditor(false);
					if (editor != null) {
						Object BLMEditorInputObj = editor.getEditorInput();
						if (editor.getEditorInput() != null && BLMEditorInputClass.isInstance(BLMEditorInputObj)) {						
							Class params[] = {};
							Object paramsObj[] = {};						

							Method method_getNode = BLMEditorInputObj.getClass().getMethod("getNode", params);

							// Invoke the method
							Object abstractChildLeafNodeObj = method_getNode.invoke(BLMEditorInputObj, paramsObj);

							// Call EList node.getEntityReferences(); 
							Class params2[] = {};
							Object paramsObj2[] = {};

							Method method_getEntityReferences  = AbstractChildLeafNodeClass.getMethod("getEntityReferences", params2);
							EList references = (EList) method_getEntityReferences.invoke(abstractChildLeafNodeObj, paramsObj2);
							if(references.get(0).equals(blmURI)) {							
//								editor.getEditorSite().getPage().closeEditor(editor, false);
								//defect 85913 enable confirmation save/cancel from the user for a dirty editor
								editor.getEditorSite().getPage().closeEditor(editor, true);
							}
						}
					}
				}

			} catch(ClassNotFoundException e) {
				LoggingUtil.logError(e.getMessage());
			} catch(NoSuchMethodException e) {
				LoggingUtil.logError(e.getMessage());
			} catch(IllegalAccessException e) {
				LoggingUtil.logError(e.getMessage());
			} catch(InvocationTargetException e) {
				LoggingUtil.logError(e.getMessage());
			}

		}

		protected void closeEditors(final AbstractChildLeafNode leafNode,final String projectName){
			if(leafNode == null || projectName == null){
				return;
			}
			//first, close itself if opened
			PlatformUI.getWorkbench().getDisplay().syncExec(
					new Runnable() {
						public void run() {
							closeEditor((String)leafNode.getEntityReferences().get(0));
						}
					});
			//second, check for referenced opening editor, close it if there is any
			List references = NavigatorUtil.getDependingObjects(leafNode,projectName);
			if(references.size() > 0){
				for(int i = 0; i<references.size();i++){
					final String referenceBlmURI = (String)references.get(i);
					PlatformUI.getWorkbench().getDisplay().syncExec(
							new Runnable() {
								public void run() {
									closeEditor(referenceBlmURI);
								}
							});
				}
			}
		}

		protected class DisplayOverwriteDialog implements Runnable {

			private Object objectToOverwrite;
			private int result;			


			public DisplayOverwriteDialog(Object obj) {
				this.objectToOverwrite = obj;
				this.result = 0;				
			}

			public void run() {
				ImportCollisionMessageDialog dlg = new ImportCollisionMessageDialog(
						ImportCollisionMessageDialog.ITEM_CONFLICT,
						PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(),
						objectToOverwrite.toString(),
						objectToOverwrite.toString()
				);
				result = dlg.open();
				dlg.close();
			}

			public int getResult() {
				return result;
			}	   		

		}
		
		protected class DisplayBusinessItemOverwriteDialog implements Runnable {

			private Object objectToOverwrite;
			private int result;			


			public DisplayBusinessItemOverwriteDialog(Object obj) {
				this.objectToOverwrite = obj;
				this.result = 0;				
			}

			public void run() {
				BusinessItemImportCollisionMessageDialog dlg = new BusinessItemImportCollisionMessageDialog(
						BusinessItemImportCollisionMessageDialog.ITEM_CONFLICT,
						PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(),
						objectToOverwrite.toString(),
						objectToOverwrite.toString()
				);
				result = dlg.open();
				dlg.close();
			}

			public int getResult() {
				return result;
			}	   		

		}


		private void recordOption(ImportOptions options,int selection,String name,boolean isSchema){
			if(isSchema){
				NavigatorUtil.getSchemaOptionHash(options).put(name,new Integer(selection));
			}else{
				NavigatorUtil.getServiceOptionHash(options).put(name,new Integer(selection));
			}
		}
	}




