package com.ibm.btools.blm.ie.imprt.rule.navigator.precondition;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ui.navigation.model.AbstractNode;
import com.ibm.btools.blm.ui.navigation.model.util.NavigationConstants;
import com.ibm.btools.blm.ui.navigation.model.util.precondition.NameCheckConstants;
import com.ibm.btools.util.precondition.IPrecondition;
import com.ibm.btools.util.precondition.PreconditionRegistry;

/**
 * The abstract precondition to check does the uniqueness of the name in the 
 * container
 */
public class CreateModelPrecondition {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	protected String newName;
	protected AbstractNode parentNode;
	protected String type;
	
	protected String feature = "label";
	protected String precondition = "NameIsUnique";
	
	/**
	 * Check to see if this precondition is valid or not.
	 * @return
	 */
	public boolean isValid() {
		return validateElement(type);
	}
	
	/**
	 * Set the new name
	 * @param newName
	 */
	public void setName(final String newName) {
		this.newName = newName;
	}
	
	/**
	 * Set parent node
	 * @param parentNode
	 */
	public void setParent(final AbstractNode parentNode) {
		this.parentNode = parentNode;
	}
	
	/**
	 * Set the type of element
	 * @param type
	 */
	public void setType(final String type) {
		this.type = type;
	}
	
	public boolean validateElement(final String elementType) {
		LoggingUtil.traceEntry(this, "validateElement");
	
		// Use the NameIsUnique precondition rule
		Map args = new HashMap();
		args.put(NavigationConstants.INTENDED_NEW_NAME, newName);		
		args.put(NavigationConstants.SELECTED_OBJECT, parentNode);
		args.put(NavigationConstants.INTENDED_MODEL_NAME, type);
		args.put(NavigationConstants.ACTION, NavigationConstants.CREATE);
		args.put(NavigationConstants.RETURN_TYPE, NavigationConstants.ERROR_CODE);
			
			
		IPrecondition nameIsUniquePrecondition = PreconditionRegistry.instance().getPrecondition(precondition);				
		List results = nameIsUniquePrecondition.checkPrecondition(args);
								
		if (results == null) {
			LoggingUtil.traceEntry(this, "validateElement");
			return true;
		} else if (results.size() == 0) {
			LoggingUtil.traceEntry(this, "validateElement");
			return true;
		} else {
			int result = ((Integer)results.get(0)).intValue();
			if (result == NameCheckConstants.CONFLICT_WITHIN_SAME_TYPE || result == NameCheckConstants.NAME_IS_NOT_UNIQUE) {
				LoggingUtil.traceEntry(this, "validateElement");
				return false;
			} else if (result == NameCheckConstants.CONFLICT_WITH_DIFFERENT_TYPE ||
				result == NameCheckConstants.CONFLICT_TYPE_UNDETERMINED ||				
				result == NameCheckConstants.CONFLICT_WITH_PREDEFINED_TYPE ||
				result == NameCheckConstants.PROPERTIES_NAME_CONFLICT ||
				result == NameCheckConstants.INTERNAL_ERROR ||
				result == NameCheckConstants.CONFLICT_WITHIN_SAME_TYPE_DIFFERENT_CASE ||				
				result == NameCheckConstants.CONFLICT_WITH_DIFFERENT_TYPE_DIFFERENT_CASE ||
				result == NameCheckConstants.CONFLICT_TYPE_UNDETERMINED_DIFFERENT_CASE ||
				result == NameCheckConstants.CONFLICT_WITH_PREDEFINED_TYPE_DIFFERENT_CASE ||
				result == NameCheckConstants.PROPERTIES_NAME_CONFLICT_DIFFERENT_CASE) {	
			
				//	throw new InvalidTypeException();
				return false;	
			} 
		}
		
		return true;
	}
}