package com.ibm.btools.blm.ie.imprt.rule.processModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.information.AddConstraintAndOpaqueExpressionINFCmd;
import com.ibm.btools.blm.compoundcommand.process.util.PEDomainViewObjectHelper;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.expressionModel.UpdateExpressionRule;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.bom.command.artifacts.AddUpdateNamedElementBOMCmd;
import com.ibm.btools.bom.model.artifacts.Constraint;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.actions.AcceptSignalAction;
import com.ibm.btools.bom.model.processes.actions.ControlAction;
import com.ibm.btools.bom.model.processes.actions.Fork;
import com.ibm.btools.bom.model.processes.actions.Join;
import com.ibm.btools.bom.model.processes.actions.ObservationAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.LoopNode;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.resources.ResourceRequirement;
import com.ibm.btools.cef.model.CommonNodeModel;
import com.ibm.btools.expression.bom.command.AddLocalPostconditionToActionBEXCmd;
import com.ibm.btools.expression.bom.command.AddLocalPreconditionToActionBEXCmd;
import com.ibm.btools.expression.bom.command.AddStructuredOpaqueExpressionToAcceptSignalActionBEXCmd;
import com.ibm.btools.expression.bom.command.AddStructuredOpaqueExpressionToLoopNodeBEXCmd;
import com.ibm.btools.expression.bom.command.AddStructuredOpaqueExpressionToObservationActionBEXCmd;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;

/**
 * Update the Action's Expression
 */
public class UpdateActionExpressionRule extends AbstractImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	protected Action action;
	protected Action workingCopy;
	protected Activity process;
	
	/**
	 * Set Original process
	 * @param process
	 */
	public void setProcess(final Object process) {
		this.process = (Activity) process;
	}	
	
	/**
	 * Set Original action
	 * @param action
	 */
	public void setSource(final Object action) {
		this.action = (Action) action;
	}
	
	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = (Action) workingCopy;
	}
	
	/**
	 * Invoke the rule
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");			
		if (action == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.PROCESS_IS_NULL);			
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		if (workingCopy == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.IMPROPER_VISUAL_MODEL);			
			LoggingUtil.traceExit(this, "invoke");			
			return;	
		}

		
		createExpression();		
				
		LoggingUtil.traceExit(this, "invoke");
	}
	
	/**
	 * Create Activity expression
	 */
	protected void createExpression() {
		LoggingUtil.traceEntry(this, "createExpression");	

		ValueSpecification spec = null;
		ValueSpecification newSpec = null;
		AddUpdateNamedElementBOMCmd command = null;
		Object newConstraint = null;
		if (action instanceof AcceptSignalAction) {
			// [Expression] Attribute filter expression - AcceptSignalAction
			spec = ((AcceptSignalAction)action).getFilterExpression();
			if (spec instanceof StructuredOpaqueExpression) {
				command = new AddStructuredOpaqueExpressionToAcceptSignalActionBEXCmd((AcceptSignalAction)workingCopy);
				command.setName(spec.getName());
				if (command.canExecute()) {
					try {
						command.execute();
					} catch (RuntimeException e) {
						LoggingUtil.trace(this, "invoke",
							IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
					}
				} else {
					LoggingUtil.trace(this, "invoke",
						IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
				}
				newConstraint = command.getObject();
				
				if (newConstraint instanceof StructuredOpaqueExpression) {
					// CallExpression
					UpdateExpressionRule expressionRule =  new UpdateExpressionRule();
					expressionRule.setImportSession(getImportSession());
					expressionRule.setProjectName(getProjectName());
					expressionRule.setSource(spec);
					expressionRule.setWorkingCopy((StructuredOpaqueExpression)newConstraint);
					expressionRule.setProcess(process);
					expressionRule.invoke();
				}						
			}			
		} else if (action instanceof ObservationAction) {
			// [Expression] Attribute observationExpression - ObservationAction	
			spec = ((ObservationAction)action).getObservationExpression();
			if (spec instanceof StructuredOpaqueExpression) {
				command = new AddStructuredOpaqueExpressionToObservationActionBEXCmd((ObservationAction)workingCopy);
				command.setName(spec.getName());
				if (command.canExecute()) {
					try {
						command.execute();
					} catch (RuntimeException e) {
						LoggingUtil.trace(this, "invoke", 
							IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
					}
				} else {
					LoggingUtil.trace(this, "invoke",
						IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
				}
				newConstraint = command.getObject();
				
				if (newConstraint instanceof StructuredOpaqueExpression) {
					// CallExpression
					UpdateExpressionRule expressionRule =  new UpdateExpressionRule();
					expressionRule.setImportSession(getImportSession());
					expressionRule.setProjectName(getProjectName());
					expressionRule.setSource(spec);
					expressionRule.setWorkingCopy((StructuredOpaqueExpression)newConstraint);
					expressionRule.setProcess(process);
					expressionRule.invoke();
				}						
			}				
		} else if (action instanceof LoopNode) {
			// [Expression] Attribute loopCondition - LoopNode
			spec = ((LoopNode)action).getLoopCondition();
			newSpec = ((LoopNode)workingCopy).getLoopCondition();
			if (spec instanceof StructuredOpaqueExpression) {
				if(newSpec == null){
					//d97936 - jie: need to add the loopCondition first
					AddStructuredOpaqueExpressionToLoopNodeBEXCmd cmd = new AddStructuredOpaqueExpressionToLoopNodeBEXCmd((LoopNode)workingCopy);
					if (cmd.canExecute()) {
						try {
							cmd.execute();
							newSpec = (ValueSpecification)cmd.getObject();
						} catch (RuntimeException e) {
							LoggingUtil.trace(this, "invoke", 
							IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
						}
					} else {
						LoggingUtil.trace(this, "invoke",
						IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
					}			
				}
				if (newSpec instanceof StructuredOpaqueExpression) {
					// CallExpression
					UpdateExpressionRule expressionRule =  new UpdateExpressionRule();
					expressionRule.setImportSession(getImportSession());
					expressionRule.setProjectName(getProjectName());
					expressionRule.setSource(spec);
					expressionRule.setWorkingCopy((StructuredOpaqueExpression)newSpec);
					expressionRule.setProcess(process);
					expressionRule.invoke();
				}						
			}			
		}
		
		updateExpression();
		updateImplementationExpression();
		LoggingUtil.traceExit(this, "createExpression");		
	}
	
	/**
	 * Update Implementation
	 */
	protected void updateExpression() {
		LoggingUtil.traceEntry(this, "updateExpression");		
		AddLocalPreconditionToActionBEXCmd preCmd = null;
		AddLocalPostconditionToActionBEXCmd postCmd = null;
		AddConstraintAndOpaqueExpressionINFCmd requirementCmd = null;
		Constraint constraint = null;
		Object newConstraint = null;
		ValueSpecification spec = null;
		Iterator iterator = null;
		Iterator newIterator = null;
		EList constraints = null;
		EList newConstraints = null;
		Constraint criteria = null;
		String requirementName = null;
		
		// [Expression] requirement's constraint
		ResourceRequirement requirement = null;
		ResourceRequirement newRequirement = null;
		iterator = action.getResourceRequirement().iterator();
		newIterator = ((Action)workingCopy).getResourceRequirement().iterator();
		while(iterator.hasNext()) {
			requirement = (ResourceRequirement)iterator.next();
			requirementName = requirement.getName();
			constraints = requirement.getOwnedConstraint();			
			if (constraints.size() > 0 && constraints.get(0) instanceof Constraint) {
				criteria = (Constraint) constraints.get(0);
				spec = criteria.getSpecification();
				if (spec instanceof StructuredOpaqueExpression) {
					requirementCmd = new AddConstraintAndOpaqueExpressionINFCmd();
					while(newIterator.hasNext()) {
						newRequirement = (ResourceRequirement) newIterator.next();
						if (requirementName != null && requirementName.compareTo(newRequirement.getName()) == 0) {
							requirementCmd.setParentElement(newRequirement);
							break;					
						}
					}
					if (requirementCmd.canExecute()) {
						try {
							requirementCmd.execute();
						} catch (RuntimeException e) {
							LoggingUtil.trace(this, "invoke",
								IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
						}
					} else {
						LoggingUtil.trace(this, "invoke", 
							IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
					}
					newConstraints = newRequirement.getOwnedConstraint();
					if (newConstraints.size() >0) {
						newConstraint = newConstraints.get(0);
					}
				
					if (newConstraint instanceof Constraint) {
						// CallExpression
						UpdateExpressionRule expressionRule =  new UpdateExpressionRule();
						expressionRule.setImportSession(getImportSession());
						expressionRule.setProjectName(getProjectName());
						expressionRule.setSource(spec);
						expressionRule.setWorkingCopy(((Constraint)newConstraint).getSpecification());
						expressionRule.setProcess(process);
						expressionRule.invoke();
					}					
				}	
			}
		}
		
		// [Expression] Attribute localPrecondition - Action
		iterator = action.getLocalPrecondition().iterator();
		while(iterator.hasNext()) {
			constraint = (Constraint) iterator.next();
			spec = constraint.getSpecification();
			if (spec instanceof StructuredOpaqueExpression) {
				preCmd = new AddLocalPreconditionToActionBEXCmd();
				preCmd.setAction((Action)workingCopy);
				preCmd.setExpressionName(spec.getName());
				if (preCmd.canExecute()) {
					try {
						preCmd.execute();
					} catch (RuntimeException e) {
						LoggingUtil.trace(this, "invoke",
							IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
					}
				} else {
					LoggingUtil.trace(this, "invoke",
						IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
				}
				newConstraint = preCmd.getConstraint();
				
				if (newConstraint instanceof Constraint) {
					// CallExpression
					UpdateExpressionRule expressionRule =  new UpdateExpressionRule();
					expressionRule.setImportSession(getImportSession());
					expressionRule.setProjectName(getProjectName());
					expressionRule.setSource(spec);
					expressionRule.setWorkingCopy(((Constraint)newConstraint).getSpecification());
					expressionRule.setProcess(process);
					expressionRule.invoke();
				}						
			}
		}		
		
		// [Expression] Attribute localPostcondition - Action
		iterator = action.getLocalPostcondition().iterator();
		while(iterator.hasNext()) {
			constraint = (Constraint) iterator.next();
			spec = constraint.getSpecification();
			if (spec instanceof StructuredOpaqueExpression) {
				postCmd = new AddLocalPostconditionToActionBEXCmd();
				postCmd.setAction((Action)workingCopy);
				postCmd.setExpressionName(spec.getName());
				if (postCmd.canExecute()) {
					try {
						postCmd.execute();
					} catch (RuntimeException e) {
						LoggingUtil.trace(this, "invoke",
							IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
					 }
				} else {
					LoggingUtil.trace(this, "invoke",
						IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
				}
				newConstraint = postCmd.getConstraint();
				
				if (newConstraint instanceof Constraint) {
					// CallExpression
					UpdateExpressionRule expressionRule =  new UpdateExpressionRule();
					expressionRule.setImportSession(getImportSession());
					expressionRule.setProjectName(getProjectName());
					expressionRule.setSource(spec);
					expressionRule.setWorkingCopy(((Constraint)newConstraint).getSpecification());
					expressionRule.setProcess(process);
					expressionRule.invoke();
				}
			}						
		}
		
		if (action instanceof Join || action instanceof Fork) {
			LoggingUtil.traceExit(this, "updateExpression");		
			return;
		}
		
		// PinSet		
		PinSet pinSet = null; 
		UpdatePinSetExpressionRule pinRule = null;
		iterator = action.getInputPinSet().iterator();
		while (iterator.hasNext()) {
			pinSet = (PinSet) iterator.next();
			pinRule = new UpdatePinSetExpressionRule();
			pinRule.setWorkingCopy(getWorkingCopy(pinSet));
			pinRule.setImportSession(getImportSession());
			pinRule.setProjectName(getProjectName());
			pinRule.setSource(pinSet);
			pinRule.setProcess(process);
			pinRule.invoke();			
		}
		
		iterator = action.getOutputPinSet().iterator();
		while (iterator.hasNext()) {
			pinSet = (PinSet) iterator.next();
			pinRule = new UpdatePinSetExpressionRule();
			pinRule.setWorkingCopy(getWorkingCopy(pinSet));
			pinRule.setImportSession(getImportSession());
			pinRule.setProjectName(getProjectName());
			pinRule.setSource(pinSet);
			pinRule.setProcess(process);
			pinRule.invoke();			
		}		
		LoggingUtil.traceExit(this, "updateExpression");		
	}	
	
	/**
	 * Update Implementation
	 */
	protected void updateImplementationExpression() {	
	}
	
	
	/**
	 * Return working copy
	 * @param viewModel
	 * @return
	 */
	protected Object getWorkingCopy(Object domainModel) {
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
		Object viewModel = context.get(domainModel);
		Object viewObject = null;
		if (viewModel instanceof CommonNodeModel) {
			
			if (action instanceof ControlAction) {
				if (domainModel instanceof InputPinSet) {
					viewObject = PEDomainViewObjectHelper.getViewInputPinSetFromViewControlActionBranch((EObject)viewModel);
					return PEDomainViewObjectHelper.getDomainObject((EObject)viewObject);
				} else if (domainModel instanceof OutputPinSet) {
					viewObject = PEDomainViewObjectHelper.getViewOutputPinSetFromViewControlActionBranch((EObject)viewModel);
					return PEDomainViewObjectHelper.getDomainObject((EObject)viewObject);
				}
			} else {
				return PEDomainViewObjectHelper.getDomainObject((EObject)viewModel);
			}
		}
		return null;
	}
}
