package com.ibm.btools.blm.ie.imprt.rule.processModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.HashMap;

import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.bom.command.processes.activities.UpdateDatastoreBOMCmd;
import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.Datastore;


/**
 * Update the Datastore
 */
public class UpdateDatastoreRule extends AbstractImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	

	private Datastore datastore;
	private Datastore workingCopy;

	/**
	 * Set Original datastore
	 * @param datastore
	 */
	public void setSource(final Object datastore) {
		this.datastore = (Datastore) datastore;
	}
	
	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = (Datastore) workingCopy;
	}	
	
	/** 
	 * Invokes the rule execution.
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");	
		updateModel();
		LoggingUtil.traceExit(this, "invoke");
	}	
	
	/**
	 * Update the datastore
	 */
	private void updateModel() {
		LoggingUtil.traceEntry(this, "updateModel");			
		// Attribute aspect - ignore
		// Attribute name - command
		// Attribute ownedComment - ignore
		// Attribute ownedConstraint - ignore
		// Attribute lowerBound - always 0
		// Attribute uid - generated
		// Attribute visibility - ignore
		
		// Attribute isOrdered
		// Attribute isReadOnly
		// Attribute isUnique

		UpdateDatastoreBOMCmd updateCmd = new UpdateDatastoreBOMCmd(workingCopy);  
		// Attribute isOrdered 
		updateCmd.setIsOrdered(datastore.getIsOrdered().booleanValue());
		// Attribute isUnique
		updateCmd.setIsUnique(datastore.getIsUnique().booleanValue());
		// Attribute isReadOnly
		updateCmd.setIsReadOnly(datastore.getIsReadOnly().booleanValue()); 		
		
		// Attribute type
		Type type = datastore.getType();
		PackageableElement thePackageableElement = BOMUtil.getType(getProjectName(), type, getImportSession().getContext());
		if(thePackageableElement == null){
			thePackageableElement = BOMUtil.getTypeFromWorkspace(getImportSession(), getProjectName(), type);
		}
		if (thePackageableElement != null && thePackageableElement instanceof Type) {
			updateCmd.setType((Type)thePackageableElement);
		}
		if (updateCmd.canExecute()) {
			try {
				updateCmd.execute();
			} catch (RuntimeException e) {
				LoggingUtil.trace(this, "invoke",
					IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
			}
		} else {
			LoggingUtil.trace(this, "invoke", 
				IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);				
		}				
			
		// Attribute upperBound
		int upperBound = -1; // Default in Tool
		ValueSpecification value = datastore.getUpperBound();
		if (value instanceof LiteralInteger) {
			upperBound = ((LiteralInteger)value).getValue().intValue();				
		} else if (value instanceof LiteralUnlimitedNatural) {
			upperBound = -1;
		}		
		BOMUtil.updateUpperBound((Datastore)workingCopy, upperBound);
		LoggingUtil.traceExit(this, "updateModel");
		
		// 20050419 - create the UID hashmap for ADF Import
		if (datastore.getUid() != null) {
			HashMap processHashMap = BOMUtil.getUIDMap(getImportSession().getContext());
			// track the source process elements UID as 'key' and its corresponding working copy process element UID as 'value'
			processHashMap.put(datastore.getUid(), workingCopy.getUid());
		}
		// end of 20050419 - create the UID hashmap for ADF Import
	}
}
