package com.ibm.btools.blm.ie.imprt.rule.processModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.List;

import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.bom.command.processes.humantasks.AddFormDataToFormBOMCmd;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.processes.humantasks.Form;
import com.ibm.btools.bom.model.processes.humantasks.FormData;


/**
 * Update the form
 */
public class UpdateFormRule extends AbstractImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	

	private Form form;
	private Form workingCopy;

	/**
	 * Set Original form
	 * @param form
	 */
	public void setSource(final Object form) {
		this.form = (Form) form;
	}
	
	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = (Form) workingCopy;
	}	
	
	/** 
	 * Invokes the rule execution.
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");	
		updateModel();
		LoggingUtil.traceExit(this, "invoke");
	}	
	
	/**
	 * Update the form
	 */
	private void updateModel() {
		LoggingUtil.traceEntry(this, "updateModel");			
		//TODO: check if need to update other attribute?
		//UpdateFormBOMCmd updateCmd = new UpdateFormBOMCmd(workingCopy);  
		// Attribute aspect 
		//updateCmd.setAspect(form.getAspect());
		
		// Attribute data
		List formDataList = form.getData();
		for(int i=0; i<formDataList.size(); i++){
			FormData formData = (FormData)formDataList.get(i);
			Type type = formData.getType();
			PackageableElement thePackageableElement = BOMUtil.getType(getProjectName(), type, getImportSession().getContext());
			if(thePackageableElement == null){
				thePackageableElement = BOMUtil.getTypeFromWorkspace(getImportSession(), getProjectName(), type);
			}			
			AddFormDataToFormBOMCmd addCmd = new AddFormDataToFormBOMCmd(workingCopy);
			addCmd.setIsOrdered(formData.getIsOrdered());
			addCmd.setIsUnique(formData.getIsUnique());
			addCmd.setName(formData.getName());
			addCmd.setType((Type)thePackageableElement);
			if(addCmd.canExecute()){
				try{
					addCmd.execute();
				}catch(RuntimeException ex){
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_FORMDATA_UPDATE_EXCEPTION, new String[] {form.getName(),formData.getName()}, ex);
				}
			}else{
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_FORMDATA_UPDATE_EXCEPTION, new String[] {form.getName(), formData.getName()}, null);
			}		
					
		}
		
	}
}
