package com.ibm.btools.blm.ie.imprt.rule.processModel;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.pe.factory.PeCmdFactory;
import com.ibm.btools.blm.compoundcommand.pe.param.add.AddInputParamPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.param.add.AddOutputParamPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.param.add.AddParamPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.paramset.update.AssociateParamsWithParamSetsPeCmd;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ui.navigation.model.AbstractNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationProjectNode;
import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.bom.model.artifacts.Element;
import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.artifacts.Parameter;
import com.ibm.btools.bom.model.artifacts.ParameterDirectionKind;
import com.ibm.btools.bom.model.artifacts.State;
import com.ibm.btools.bom.model.artifacts.StateSet;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.services.ParameterSet;
import com.ibm.btools.cef.model.CommonContainerModel;
import com.ibm.btools.cef.model.CommonNodeModel;

/**
 * Update the Parameter
 */
public class UpdateParameterRule extends AbstractImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private Parameter parameter;
	private Object workingCopy;
	private Object process;	

	/**
	 * Set Original process
	 * @param process
	 */
	public void setProcess(final Object process) {
		this.process = process;
	}	
	
	/**
	 * Set Original parameter
	 * @param parameter
	 */
	public void setSource(final Object parameter) {
		this.parameter = (Parameter) parameter;
	}
	
	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = workingCopy;
	}
	
	/**
	 * Invoke the rule
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");		
		List parameterSetList = null;
		if (parameter == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.PARAMETER_IS_NULL);			
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		NavigationProjectNode projectNode = NavigatorUtil.getProjectNode(getProjectName());
		if (projectNode == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.PROJECT_NODE_IS_NULL);												
			return;
		}
		PeCmdFactory cmdFactory = getPeCmdFactory();
		AddParamPeCmd paramCmd = null;
		
		if (parameter.getDirection().getValue() == ParameterDirectionKind.IN) {
			paramCmd = cmdFactory.buildAddInputParamPeCmd((EObject) workingCopy);
			LoggingUtil.trace(this, "invoke", "Create command for input parameter");
					
		} else {
			paramCmd = cmdFactory.buildAddOutputParamPeCmd((EObject) workingCopy);	
			LoggingUtil.trace(this, "invoke", "Create command for output parameter");			
		}
		if (paramCmd != null) {		
			int lowerBound = 1; // Default in Tool and BOM
			int upperBound = 1; // Default in Tool and BOM
			// Attribute lowerBound
			ValueSpecification value = null;		
			value = parameter.getLowerBound();
			if (value instanceof LiteralInteger) {
				lowerBound = ((LiteralInteger)value).getValue().intValue();
			}
			LoggingUtil.trace(this, "invoke", "Set lower bound.");			
			paramCmd.setLowerbound(lowerBound);
			// Attribute upperBound
			value = parameter.getUpperBound();
			if (value instanceof LiteralInteger) {
				upperBound = ((LiteralInteger)value).getValue().intValue();
				paramCmd.setUpperbound(upperBound);
			} else if (value instanceof LiteralUnlimitedNatural) {
				paramCmd.setUpperbound(-1);
			} else {
				paramCmd.setUpperbound(upperBound);
			}
			LoggingUtil.trace(this, "invoke", "Set upper bound.");			
			// Attribute type
			Type type = parameter.getType();
			PackageableElement thePackageableElement = BOMUtil.getType(getProjectName(), type, getImportSession().getContext());
			if(thePackageableElement == null){
				thePackageableElement = BOMUtil.getTypeFromWorkspace(getImportSession(), getProjectName(), type);
			}
			
			if (thePackageableElement != null && thePackageableElement instanceof Type) {
				paramCmd.setBusinessItem((Type)thePackageableElement);
			}
			LoggingUtil.trace(this, "invoke", "Set Type.");	
					
			// Parameter Set
			if(NavigatorUtil.isNewNode(getImportSession(),process)){
				paramCmd.setViewParameterSetList(getParameterSet(parameter));			
			}else if(NavigatorUtil.isOptionOverwrite(getImportSession().getImportOptions(),(Activity)process)){
				//we retrieve the existing parameter set list
				parameterSetList = getParameterSetViewModel(getParameterSetFromNew(parameter));
				paramCmd.setViewParameterSetList(parameterSetList);
			}
			LoggingUtil.trace(this, "invoke", "Set parameter set list.");				
			
			// Attribute name
			paramCmd.setName(parameter.getName());
			// Attribute isOrdered
			paramCmd.setIsOrdered(parameter.getIsOrdered().booleanValue());
			// Attribute isUnique
			paramCmd.setIsUnique(parameter.getIsUnique().booleanValue());
			// Attribute state
			setParamState(paramCmd, thePackageableElement);
			if (paramCmd.canExecute()) {
				try {
					paramCmd.execute();
				} catch (RuntimeException e) {
					LoggingUtil.trace(this, "invoke",
						IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
				}
			} else {
				LoggingUtil.trace(this, "invoke",
					IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);				
			}
		}
		
		
		// Attribute aspect - ignore
		// Attribute uid - generated
		// Attribute visibility - Default		
		// Attribute ownedConstraint - not handled
		// Attribute defaultValue - not handled
		// Attribute direction - handled by command
		
		// Attribute ownedComment 
		EObject domainModel = paramCmd.getNewDomainModel();
		if (domainModel != null) {
			BOMUtil.updateDescription((Element)domainModel, BOMUtil.getDescription((Element)parameter));
		}
		if(parameterSetList != null && parameterSetList.size() > 0){
			List paramVisualList = new ArrayList();
			paramVisualList.add(paramCmd.getNewViewModel());
			AssociateParamsWithParamSetsPeCmd associateCmd = getPeCmdFactory().buildAssociateParamsWithParamSetsPeCmd(paramVisualList,parameterSetList);
			if(associateCmd != null && associateCmd.canExecute()){
				associateCmd.execute();
			}
		}
		LoggingUtil.traceExit(this, "invoke");
	}

	private void setParamState(AddParamPeCmd paramCmd, PackageableElement thePackageableElement) {
		if(process instanceof Activity) {
			if(paramCmd instanceof AddInputParamPeCmd) {
				handleInputs(paramCmd, thePackageableElement);
			} else if(paramCmd instanceof AddOutputParamPeCmd) {
				handleOutputs(paramCmd, thePackageableElement);
			}
		}
	}

	private void handleOutputs(AddParamPeCmd paramCmd, PackageableElement thePackageableElement) {
		List outputPins = ((Activity)process).getImplementation().getOutputObjectPin();
		int outputSize = outputPins.size();
		for(int j = 0; j < outputSize; j++) {
			// Ray - d154739.  For FDL import, the parameter name is not set at this stage and there is no state need to be set
			if(parameter.getName()!=null && parameter.getName().equals(((ObjectPin)outputPins.get(j)).getName())) {
				List sets = ((ObjectPin)outputPins.get(j)).getStateSets();
				if(sets != null && !sets.isEmpty()) {
					StateSet set = (StateSet)sets.get(0);
					Object state = set.getStates().get(0);
					Class classifier = (Class)thePackageableElement;
					HashMap stateMap = (HashMap)getImportSession().getContext().get(STATE_MAP);
					if(stateMap != null && !stateMap.isEmpty()) {
						State mapState = (State)stateMap.get(state);
						if(mapState != null) {
							State masterState = null;
							masterState = getMasterState(classifier, mapState, masterState);
							// Fix d74627: IO pin states are lost after xml-import
							//             process/task if the referenced business item is
							//             renamed during import.
							AbstractNode importedStateParentNode = (AbstractNode)
									getImportSession().getContext().get( ((State)state).eContainer() );
							if(((Class)mapState.eContainer()).getName().equals(importedStateParentNode.getLabel())) {
								paramCmd.setState((State)masterState);
								break;
							}
						}
					}
				}
			}
		}
	}

	private State getMasterState(Class classifier, State mapState, State masterState) {
		while(masterState == null || classifier != null) {
			List pStateList = classifier.getPossibleStates();
			int pSize = pStateList.size();
			for(int k = 0; k < pSize; k++){								
				if(((State)pStateList.get(k)).getUid().equals(mapState.getUid())){
					masterState = (State)pStateList.get(k);
					break;
				}									
			}
			List superClassList = classifier.getSuperClassifier();
			if(superClassList != null && !superClassList.isEmpty()) {
				classifier = (Class)superClassList.get(0);
			} else {
				classifier = null;
			}
		}
		return masterState;
	}

	private void handleInputs(AddParamPeCmd paramCmd, PackageableElement thePackageableElement) {
		List inputPins = ((Activity)process).getImplementation().getInputObjectPin();
		int inputSize = inputPins.size();
		for(int i = 0; i < inputSize; i++) {
			// Ray - d154739.  For FDL import, the parameter name is not set at this stage and there is no state need to be set
			if(parameter.getName()!=null && parameter.getName().equals(((ObjectPin)inputPins.get(i)).getName())) {
				List sets = ((ObjectPin)inputPins.get(i)).getStateSets();
				if(sets != null && !sets.isEmpty()) {
					StateSet set = (StateSet)sets.get(0);
					Object state = set.getStates().get(0);
					Class classifier = (Class)thePackageableElement;
					HashMap stateMap = (HashMap)getImportSession().getContext().get(STATE_MAP);
					if(stateMap != null && !stateMap.isEmpty()) {
						State mapState = (State)stateMap.get(state);
						if(mapState != null) {
							State masterState = null;
							masterState = getMasterState(classifier, mapState, masterState);
							// Fix d74627: IO pin states are lost after xml-import
							//             process/task if the referenced business item is
							//             renamed during import.
							AbstractNode importedStateParentNode = (AbstractNode)
									getImportSession().getContext().get( ((State)state).eContainer() );
							if(((Class)mapState.eContainer()).getName().equals(importedStateParentNode.getLabel())) {
								paramCmd.setState((State)masterState);
								break;
							}
						}
					}
				}
			}
		}
	}
	
	/**
	 * Returns parameter set contains this parameter
	 * @param parameter
	 * @return
	 */
	private List getParameterSet(final Parameter parameter) {
		LoggingUtil.traceEntry(this, "getParameterSet");
		
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
		List paramSetList = new ArrayList();
		
		ParameterSet paramSet = null;
		Object viewModel = null;
		if (process != null && process instanceof Activity) {
			Iterator iterator = ((Activity)process).getInputParameterSet().iterator();
			while(iterator.hasNext()) {
				paramSet = (ParameterSet) iterator.next();
				if (paramSet.getParameter().contains(parameter)) {
					viewModel = context.get(paramSet);
					paramSetList.add(viewModel);
				}
			}
			if (paramSetList.size() > 0) {
				LoggingUtil.traceExit(this, "getParameterSet");
				return paramSetList;			
			}
			iterator = ((Activity)process).getOutputParameterSet().iterator();
			while(iterator.hasNext()) {
				paramSet = (ParameterSet) iterator.next();
				if (paramSet.getParameter().contains(parameter)) {					
					viewModel = context.get(paramSet);
					paramSetList.add(viewModel);
				}
			}			
		}	
		
		LoggingUtil.traceExit(this, "getParameterSet");
		return paramSetList;
	}
	
	private ParameterSet getParameterSetFromNew(final Parameter parameter){	
		ParameterSet newParameterSet = null;		
		if (process instanceof Activity) {
			Iterator iterator = ((Activity)process).getInputParameterSet().iterator();
			boolean matchFound = false;
			while(iterator.hasNext()) {
				newParameterSet = (ParameterSet) iterator.next();
				if (newParameterSet.getParameter().contains(parameter)) {
					matchFound = true;
					break;
				}
			}
			if(!matchFound){
				Iterator iterator2 = ((Activity)process).getOutputParameterSet().iterator();
			while(iterator2.hasNext()) {
				newParameterSet = (ParameterSet) iterator2.next();
				if (newParameterSet.getParameter().contains(parameter)) {					
					break;
				}
			}			
		}	
	}
		return newParameterSet;
	}
	
	private List getParameterSetViewModel(ParameterSet associatedParamSet){	
	    List paramSetViewModelList = new ArrayList();
	    if(associatedParamSet == null){
	    	return paramSetViewModelList;
	    }			
		Object domainModel = null;
		EList contentList = null;
		if (workingCopy instanceof CommonContainerModel) {
			List children = new ArrayList(((CommonContainerModel)workingCopy).getCompositionChildren());			
			Iterator compositeChildrenIterator = children.iterator();				
			while(compositeChildrenIterator.hasNext()) {
				Object compositeChildren = compositeChildrenIterator.next();
				if (compositeChildren instanceof CommonNodeModel) {
					if (((CommonNodeModel)compositeChildren).getDescriptor() == null) {
						// Already removed.
						continue;		
					}
					LoggingUtil.trace(this, "updateParameterSets", "Common Node Model is used to represent the Parameter Set.");
					contentList = ((CommonNodeModel)compositeChildren).getDomainContent();					
					if (contentList.size() <= 0) {
						continue;
					}
					domainModel = contentList.get(0);					
					if (domainModel instanceof ParameterSet) {
						if(((ParameterSet)domainModel).getName().equals(associatedParamSet.getName())){
							paramSetViewModelList.add((CommonNodeModel)compositeChildren);
							return paramSetViewModelList;
						}
					}
				}
			}			
		} 
		return paramSetViewModelList;
	}
}
