package com.ibm.btools.blm.ie.imprt.rule.processModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.pe.factory.PeCmdFactory;
import com.ibm.btools.blm.compoundcommand.pe.paramset.add.AddOutputParamSetPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.paramset.add.AddParamSetPeCmd;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ui.navigation.model.NavigationProjectNode;
import com.ibm.btools.bom.model.artifacts.Element;
import com.ibm.btools.bom.model.services.InputParameterSet;
import com.ibm.btools.bom.model.services.OutputParameterSet;
import com.ibm.btools.bom.model.services.ParameterSet;

/**
 * Update the Input Parameter Set
 */
public class UpdateParameterSetRule extends AbstractImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private ParameterSet parameterSet;
	private Object workingCopy;
	private Object process;	
	
	/**
	 * Set Original process
	 * @param process
	 */
	public void setProcess(final Object process) {
		this.process = process;
	}
	
	/**
	 * Set Original parameterSet
	 * @param parameterSet
	 */
	public void setSource(final Object parameterSet) {
		this.parameterSet = (ParameterSet) parameterSet;
	}	
	
	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = workingCopy;
	}
	
	/**
	 * Invoke the rule
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");			
		if (parameterSet == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.PARAMETER_SET_IS_NULL);			
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		NavigationProjectNode projectNode = NavigatorUtil.getProjectNode(getProjectName());
		if (projectNode == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.PROJECT_NODE_IS_NULL);												
			return;
		}
		PeCmdFactory cmdFactory = getPeCmdFactory();
		AddParamSetPeCmd paramSetCmd = null;
		if (parameterSet instanceof InputParameterSet) {
			paramSetCmd = cmdFactory.buildAddInputParamSetPeCmd((EObject) workingCopy);
		} else if (parameterSet instanceof OutputParameterSet) {
			paramSetCmd = cmdFactory.buildAddOutputParamSetPeCmd((EObject) workingCopy);
		} else {
			LoggingUtil.trace(this, "invoke",
				IImportResultMessages.IMPROPER_BOM_MODEL);	
			LoggingUtil.traceExit(this, "invoke");
			return;				
		}
		
		// Attribute name
		paramSetCmd.setName(parameterSet.getName());
		
		// Attribute isException
		if (paramSetCmd instanceof AddOutputParamSetPeCmd) {
			((AddOutputParamSetPeCmd) paramSetCmd).setIsException(((OutputParameterSet)parameterSet).getIsException().booleanValue());
		}
				
		if (paramSetCmd.canExecute()) {
			try {
				paramSetCmd.execute();
			} catch (RuntimeException e) {
				LoggingUtil.trace(this, "invoke",
					IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
				LoggingUtil.traceExit(this, "invoke");
				return;
			}
		} else {
			LoggingUtil.trace(this, "invoke",
				IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);	
			LoggingUtil.traceExit(this, "invoke");
			return;			
		}
		
		// Attribute ownedComment		
		EObject domainModel = paramSetCmd.getNewDomainModel();
		if (domainModel != null) {
			BOMUtil.updateDescription((Element)domainModel, BOMUtil.getDescription((Element)parameterSet));
		}		
		
		// Attribute aspect - ignore
		// Attribute uid - generated
		// Attribute visibility - Default		
		// Attribute ownedConstraint - not handled
		
		LoggingUtil.trace(this, "invoke", "Put the object into the context");
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
		context.put(parameterSet, paramSetCmd.getNewViewModel());
		
		LoggingUtil.traceExit(this, "invoke");
	}
}
