package com.ibm.btools.blm.ie.imprt.rule.processModel;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.pe.factory.PeCmdFactory;
import com.ibm.btools.blm.compoundcommand.pe.pin.add.AddObjectPinPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.pin.add.AddPinPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.pin.add.AddRetrieveArtifactPinPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.pin.add.AddStoreArtifactPinPeCmd;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.informationModel.UpdateValueSpecificationRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ui.navigation.model.AbstractNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationProjectNode;
import com.ibm.btools.bom.command.artifacts.AddInstanceValueToInputValuePinBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralNullToInputValuePinBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralStringToInputValuePinBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddOpaqueExpressionToInputValuePinBOMCmd;
import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.bom.model.artifacts.Element;
import com.ibm.btools.bom.model.artifacts.InstanceSpecification;
import com.ibm.btools.bom.model.artifacts.InstanceValue;
import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.LiteralNull;
import com.ibm.btools.bom.model.artifacts.LiteralSpecification;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.OpaqueExpression;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.artifacts.State;
import com.ibm.btools.bom.model.artifacts.StateSet;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.actions.ControlAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.ControlPin;
import com.ibm.btools.bom.model.processes.activities.InputControlPin;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.InputValuePin;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputControlPin;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.RetrieveArtifactPin;
import com.ibm.btools.bom.model.processes.activities.StoreArtifactPin;
import com.ibm.btools.expression.bom.command.AddStructuredOpaqueExpressionToInputValuePinBEXCmd;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.model.modelmanager.util.AddUpdateObjectCommand;


/**
 * Update the Pin
 */
public class UpdatePinRule extends AbstractImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private Pin pin;
	private Object workingCopy;
	private Object process;
	private Action action;
	
	/**
	 * Set Original process
	 * @pin process
	 */
	public void setProcess(final Object process) {
		this.process = process;
	}	
	
	/**
	 * Set Original task
	 * @pin task
	 */
	public void setAction(final Object action) {
		this.action = (Action) action;
	}	
	
	/**
	 * Set Original pin
	 * @pin pin
	 */
	public void setSource(final Object pin) {
		this.pin = (Pin) pin;
	}
	
	/**
	 * Set the working copy
	 * @pin workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = workingCopy;
	}
	
	/**
	 * Invoke the rule
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");			
		if (pin == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.PIN_IS_NULL);			
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		NavigationProjectNode projectNode = NavigatorUtil.getProjectNode(getProjectName());
		if (projectNode == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.PROJECT_NODE_IS_NULL);												
			return;
		}
		PeCmdFactory cmdFactory = getPeCmdFactory();
		AddPinPeCmd pinCmd = null;
		
		
		if (pin instanceof ObjectPin) {
			if (pin instanceof InputValuePin) {
				pinCmd = cmdFactory.buildAddInputValuePinPeCmd((EObject) workingCopy);
				LoggingUtil.trace(this, "invoke", "Create command for input value pin");				
			} else if (pin instanceof RetrieveArtifactPin) {
				pinCmd = cmdFactory.buildAddRetrieveArtifactPinPeCmd((EObject) workingCopy);
				LoggingUtil.trace(this, "invoke", "Create command for retrieve artifact pin");				
			} else if (pin instanceof StoreArtifactPin) {
				pinCmd = cmdFactory.buildAddStoreArtifactPinPeCmd((EObject) workingCopy);
				LoggingUtil.trace(this, "invoke", "Create command for store artifact pin");				
			} else if (pin instanceof InputObjectPin) {
				pinCmd = cmdFactory.buildAddInputObjectPinPeCmd((EObject) workingCopy);
				LoggingUtil.trace(this, "invoke", "Create command for input object pin");	
			} else if (pin instanceof OutputObjectPin) {
				pinCmd = cmdFactory.buildAddOutputObjectPinPeCmd((EObject) workingCopy);	
				LoggingUtil.trace(this, "invoke", "Create command for output object pin");			
			}
			// Attribute type
			Type type = ((ObjectPin)pin).getType();
			PackageableElement thePackageableElement = BOMUtil.getType(getProjectName(), type, getImportSession().getContext());
			if(thePackageableElement == null){
				thePackageableElement = BOMUtil.getTypeFromWorkspace(getImportSession(), getProjectName(), type);
			}
			if (thePackageableElement != null && thePackageableElement instanceof Type) {
				((AddObjectPinPeCmd)pinCmd).setBusinessItem((Type)thePackageableElement);
			}
			
			// set state
			List stateSets = ((ObjectPin)pin).getStateSets();
			if(stateSets != null && !stateSets.isEmpty()) {
				State orgState = (State)((StateSet)stateSets.get(0)).getStates().get(0);
				HashMap stateMap = (HashMap)getImportSession().getContext().get(STATE_MAP);
				Class classifier = (Class)thePackageableElement;
				State masterState = null;
				State mapState = (State)stateMap.get(orgState);
				masterState = getMasterState(classifier, mapState, masterState);

				// Fix d74627: IO pin states are lost after xml-import
				//             process/task if the referenced business item is
				//             renamed during import.
				AbstractNode importedStateParentNode = (AbstractNode)
						getImportSession().getContext().get( orgState.eContainer() );
				if(((Class)mapState.eContainer()).getName().equals(importedStateParentNode.getLabel())) {
					((AddObjectPinPeCmd)pinCmd).setState((State)masterState);
				}
			}
			
			// Attribute isOrdered
			((AddObjectPinPeCmd)pinCmd).setIsOrdered(((ObjectPin)pin).getIsOrdered().booleanValue());
			
			// Attribute isUnique
			((AddObjectPinPeCmd)pinCmd).setIsUnique(((ObjectPin)pin).getIsUnique().booleanValue());			
			
			int lowerBound = 1; // Default in Tool and BOM
			int upperBound = 1; // Default in Tool and BOM
			// Attribute lowerBound
			ValueSpecification value = null;		
			value = ((ObjectPin)pin).getLowerBound();
			if (value instanceof LiteralInteger) {
				lowerBound = ((LiteralInteger)value).getValue().intValue();
			}
			LoggingUtil.trace(this, "invoke", "Set lower bound.");			
			((AddObjectPinPeCmd)pinCmd).setLowerbound(lowerBound);
			// Attribute upperBound
			value = ((ObjectPin)pin).getUpperBound();
			if (value instanceof LiteralInteger) {
				upperBound = ((LiteralInteger)value).getValue().intValue();
				((AddObjectPinPeCmd)pinCmd).setUpperbound(upperBound);
			} else if (value instanceof LiteralUnlimitedNatural) {
				((AddObjectPinPeCmd)pinCmd).setUpperbound(-1);
			} else {
				((AddObjectPinPeCmd)pinCmd).setUpperbound(upperBound);
			}
			LoggingUtil.trace(this, "invoke", "Set upper bound.");						

			if (pinCmd instanceof AddRetrieveArtifactPinPeCmd) {
				// Attribute atBeginning
				((AddRetrieveArtifactPinPeCmd)pinCmd).setAtBeginning(((RetrieveArtifactPin)pin).getAtBeginning().booleanValue());
				
				// Attribute isRemove
				((AddRetrieveArtifactPinPeCmd)pinCmd).setIsRemoved(((RetrieveArtifactPin)pin).getIsRemove().booleanValue());
			}
			
			if (pinCmd instanceof AddStoreArtifactPinPeCmd) {
				// Attribute atBeginning
				((AddStoreArtifactPinPeCmd)pinCmd).setAtBeginning(((StoreArtifactPin)pin).getAtBeginning().booleanValue());
				
				// Atribute isInsert
				((AddStoreArtifactPinPeCmd)pinCmd).setIsInsert(((StoreArtifactPin)pin).getIsInsert().booleanValue());

			}

		} else if (pin instanceof ControlPin) {
			if (pin instanceof InputControlPin) {
				pinCmd = cmdFactory.buildAddInputControlPinPeCmd((EObject) workingCopy);	
				LoggingUtil.trace(this, "invoke", "Create command for input control pin");			
			} else if (pin instanceof OutputControlPin) {
				pinCmd = cmdFactory.buildAddOutputControlPinPeCmd((EObject) workingCopy);	
				LoggingUtil.trace(this, "invoke", "Create command for output control pin");			
			}
		}
		
		if (pinCmd == null) {
			return;
		}		
		
		// Attribute owndeConstraint - not handled
		// Attribute uid - generated
		// Attribute visibility - handled
		// Attribute asspect - not handled
		// Attribute incoming - Done by edge	
		// Attribute outgoing - Done by edge
		
		if (!(action instanceof ControlAction)) {
			pinCmd.setName(pin.getName());
		}
		pinCmd.setViewPinSetList(getPinSet(pin));
		
		if (pinCmd.canExecute()) {
			try {
				pinCmd.execute();
			} catch (RuntimeException e) {
				LoggingUtil.trace(this, "invoke",
					IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
				LoggingUtil.traceExit(this, "invoke");					
				return;
			}
		} else {
			LoggingUtil.trace(this, "invoke",
				IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
			LoggingUtil.traceExit(this, "invoke");				
			return;				
		}
		
		LoggingUtil.trace(this, "invoke", "Put the object into the context");
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
		context.put(pin, pinCmd.getNewViewModel());		
		
		Pin theNewPin = (Pin) pinCmd.getNewDomainModel();
		
		// Attribute ownedComment		
		if (theNewPin != null) {
			BOMUtil.updateDescription((Element)theNewPin, BOMUtil.getDescription((Element)pin));
		}		
		
		if (pin instanceof InputValuePin) {
			addAttributeValue(theNewPin);
		}
		
		// 20050413 - create the UID hashmap for ADF Import
		if (pin.getUid() != null) {
			HashMap processHashMap = BOMUtil.getUIDMap(getImportSession().getContext());
			// track the source process elements UID as 'key' and its corresponding working copy process element UID as 'value'
			processHashMap.put(pin.getUid(), theNewPin.getUid());
		}
		// end of 20050413 - create the UID hashmap for ADF Import
		
		LoggingUtil.traceExit(this, "invoke");
	}
	
	private State getMasterState(Class classifier, State mapState, State masterState) {
		while(masterState == null && classifier != null) {
			List pStateList = classifier.getPossibleStates();
			int pSize = pStateList.size();
			for(int k = 0; k < pSize; k++){								
				if(((State)pStateList.get(k)).getUid().equals(mapState.getUid())){
					masterState = (State)pStateList.get(k);
					break;
				}									
			}
			List superClassList = classifier.getSuperClassifier();
			if(superClassList != null && !superClassList.isEmpty()) {
				classifier = (Class)superClassList.get(0);
			} else {
				classifier = null;
			}
		}
		return masterState;
	}

	/**
	 * Returns pin set contains this pin
	 * @pin pin
	 * @return
	 */
	private List getPinSet(final Pin pin) {
		LoggingUtil.traceEntry(this, "getPinSet");
		
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
		List pinSetList = new ArrayList();
		
		PinSet pinSet = null;
		Object viewModel = null;
		Iterator iterator = null;
		if (pin instanceof InputObjectPin || pin instanceof InputControlPin) {
			iterator = action.getInputPinSet().iterator();
		} else if (pin instanceof OutputObjectPin || pin instanceof OutputControlPin) {
			iterator = action.getOutputPinSet().iterator();
		}
		EList list = null;
		while(iterator.hasNext()) {
			pinSet = (PinSet) iterator.next();
			if (pinSet instanceof InputPinSet) {
				list =((InputPinSet)pinSet).getInputObjectPin();
				if (list.contains(pin)) {
					viewModel = context.get(pinSet);
					pinSetList.add(viewModel);
				}
				list =((InputPinSet)pinSet).getInputControlPin();
				if (list.contains(pin)) {
					viewModel = context.get(pinSet);
					pinSetList.add(viewModel);
				}
			}
			if (pinSet instanceof OutputPinSet) {
				list =((OutputPinSet)pinSet).getOutputObjectPin();
				if (list.contains(pin)) {
					viewModel = context.get(pinSet);
					pinSetList.add(viewModel);
				}
				list =((OutputPinSet)pinSet).getOutputControlPin();
				if (list.contains(pin)) {
					viewModel = context.get(pinSet);
					pinSetList.add(viewModel);
				}
			}
		}
		
		LoggingUtil.traceExit(this, "getPinSet");
		return pinSetList;
	}
	
	private void addAttributeValue(Pin theNewPin) {
		Iterator iterator = ((InputValuePin)pin).getValue().iterator();
		ValueSpecification value = null;
		AddUpdateObjectCommand cmd = null;
		InstanceSpecification spec = null;
		while(iterator.hasNext()) {
			value = (ValueSpecification) iterator.next();	
			if (value instanceof LiteralNull) {
				cmd = new AddLiteralNullToInputValuePinBOMCmd((InputValuePin)theNewPin);			
			} else if (value instanceof LiteralSpecification) {
				cmd = new AddLiteralStringToInputValuePinBOMCmd((InputValuePin)theNewPin);
			} else if (value instanceof StructuredOpaqueExpression){
				cmd = new AddStructuredOpaqueExpressionToInputValuePinBEXCmd((InputValuePin)theNewPin);				
			} else if (value instanceof OpaqueExpression) {
				// [Expression] Attrbibute constant valueb
				cmd = new AddOpaqueExpressionToInputValuePinBOMCmd((InputValuePin)theNewPin);
			} else if (value instanceof InstanceValue) {
				cmd = new AddInstanceValueToInputValuePinBOMCmd((InputValuePin)theNewPin);			
			}			
			if (cmd.canExecute()) {
				try {
					cmd.execute();
				} catch (RuntimeException e) {
					LoggingUtil.trace(this, "invoke",
						IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
				}
			} else {
				LoggingUtil.trace(this, "invoke", 
					IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);				
			}
			
			// After creation
			Object newDomainModel = cmd.getObject();
			if (newDomainModel != null) {
				UpdateValueSpecificationRule rule = new UpdateValueSpecificationRule();
				rule.setSource(value);
				rule.setWorkingCopy(newDomainModel);
				rule.setProjectName(getProjectName());
				rule.setImportSession(getImportSession());
				rule.invoke();
			}						
		}
	}
}
