package com.ibm.btools.blm.ie.imprt.rule.processModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;

import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.expressionModel.UpdateExpressionRule;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.bom.command.artifacts.AddUpdateNamedElementBOMCmd;
import com.ibm.btools.bom.model.artifacts.Constraint;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.actions.ObservationAction;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.DecisionOutputSet;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.expression.bom.command.AddStructuredOpaqueExpressionCorrelationPredicateToInputPinSetBEXCmd;
import com.ibm.btools.expression.bom.command.AddStructuredOpaqueExpressionInputSetConstraintToInputPinSetBEXCmd;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;

/**
 * Update the Pinset's Expression
 */
public class UpdatePinSetExpressionRule extends AbstractImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	protected PinSet pinSet;
	protected PinSet workingCopy;
	protected Activity process;
	
	/**
	 * Set Original process
	 * @param process
	 */
	public void setProcess(final Object process) {
		this.process = (Activity) process;
	}	
	
	/**
	 * Set Original process
	 * @param process
	 */
	public void setSource(final Object pinSet) {
		this.pinSet = (PinSet) pinSet;
	}
	
	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = (PinSet) workingCopy;
	}
	
	/**
	 * Invoke the rule
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");			
		if (pinSet == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.PROCESS_IS_NULL);			
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		if (workingCopy == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.IMPROPER_VISUAL_MODEL);			
			LoggingUtil.traceExit(this, "invoke");			
			return;	
		}

		LoggingUtil.trace(this, "invoke", "Create the context for process");
		
		createExpression();		
				
		LoggingUtil.traceExit(this, "invoke");
	}
	
	/**
	 * Create Activity expression
	 * @param process
	 */
	protected void createExpression() {
		AddUpdateNamedElementBOMCmd command = null;
		Constraint constraint = null;
		Object newConstraint = null;
		ValueSpecification spec = null;
		ValueSpecification newSpec = null;
		Iterator iterator = null;
		if (pinSet instanceof InputPinSet && workingCopy instanceof InputPinSet) {
			// [Expression] Attribute inputSetConstraint - InputPinSet
			iterator = ((InputPinSet)pinSet).getInputSetConstraint().iterator();
			while(iterator.hasNext()) {
				//TODO: d97940
			//	constraint = (Constraint) iterator.next();
			//	spec = constraint.getSpecification();
				spec=(StructuredOpaqueExpression)iterator.next();
				if (spec instanceof StructuredOpaqueExpression) {
					command = new AddStructuredOpaqueExpressionInputSetConstraintToInputPinSetBEXCmd((InputPinSet)workingCopy);
					command.setName(spec.getName());					
					if (((StructuredOpaqueExpression)spec).getDescription() != null) {
						command.setDescription(((StructuredOpaqueExpression)spec).getDescription());
					}					
					if (command.canExecute()) {
						try {
							command.execute();
						} catch (RuntimeException e) {
							LoggingUtil.trace(this, "invoke",
								IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
						}
					} else {
						LoggingUtil.trace(this, "invoke", 
							IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
					}
					newConstraint = command.getObject();
				
					if (newConstraint instanceof Constraint) {
						// CallExpression
						UpdateExpressionRule expressionRule =  new UpdateExpressionRule();
						expressionRule.setImportSession(getImportSession());
						expressionRule.setProjectName(getProjectName());
						expressionRule.setSource(spec);
						expressionRule.setWorkingCopy(((Constraint)newConstraint).getSpecification());
						expressionRule.setProcess(process);
						expressionRule.invoke();
					}						
				}
			}
			
			// [Expression] Attribute correlationPredicate - InputPinSet
			spec = ((InputPinSet)pinSet).getCorrelationPredicate();
			
			if (spec != null && pinSet.eContainer() instanceof ObservationAction) {
				newSpec = ((InputPinSet)workingCopy).getCorrelationPredicate();
				//TODO: d97945 - expression never created
				if(newSpec == null){
					 //create
					command = new AddStructuredOpaqueExpressionCorrelationPredicateToInputPinSetBEXCmd((InputPinSet)workingCopy);				 
					command.setName(spec.getName());
					if (command.canExecute()) {
						try {
							command.execute();
						} catch (RuntimeException e) {
							LoggingUtil.trace(this, "invoke",
								IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
						}
					} else {
						LoggingUtil.trace(this, "invoke",
							IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
					}
					newSpec = (ValueSpecification)command.getObject();
					if(newSpec instanceof StructuredOpaqueExpression){
						UpdateExpressionRule expressionRule =  new UpdateExpressionRule();
						expressionRule.setImportSession(getImportSession());
						expressionRule.setProjectName(getProjectName());
						expressionRule.setSource(spec);
						expressionRule.setWorkingCopy((StructuredOpaqueExpression)newSpec);
						expressionRule.setProcess(process);
						expressionRule.invoke();
					}
					
				}else 
				if (newSpec instanceof StructuredOpaqueExpression) {
					UpdateExpressionRule expressionRule =  new UpdateExpressionRule();
					expressionRule.setImportSession(getImportSession());
					expressionRule.setProjectName(getProjectName());
					expressionRule.setSource(spec);
					expressionRule.setWorkingCopy((StructuredOpaqueExpression)newSpec);
					expressionRule.setProcess(process);
					expressionRule.invoke();
				}
			} else {				
				if (spec instanceof StructuredOpaqueExpression) {
			//d97575 - jie: wrong expression created causing later throws classcast exception		
			//		command = new AddOpaqueExpressionCorrelationPredicateToInputPinSetBOMCmd((InputPinSet)workingCopy);
					command = new AddStructuredOpaqueExpressionCorrelationPredicateToInputPinSetBEXCmd((InputPinSet)workingCopy);				 				 
					command.setName(spec.getName());
					if (command.canExecute()) {
						try {
							command.execute();
						} catch (RuntimeException e) {
							LoggingUtil.trace(this, "invoke",
								IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
						}
					} else {
						LoggingUtil.trace(this, "invoke",
							IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
					}
					newConstraint = command.getObject();
				
					if (newConstraint instanceof StructuredOpaqueExpression) {
						// CallExpression
						UpdateExpressionRule expressionRule =  new UpdateExpressionRule();
						expressionRule.setImportSession(getImportSession());
						expressionRule.setProjectName(getProjectName());
						expressionRule.setSource(spec);
						expressionRule.setWorkingCopy((StructuredOpaqueExpression)newConstraint);
						expressionRule.setProcess(process);
						expressionRule.invoke();
					}
				}						
			}
		} else if (pinSet instanceof DecisionOutputSet && workingCopy instanceof DecisionOutputSet) {
			// [Expression] Attribute condition - DecisionOutputSet
			spec = ((DecisionOutputSet)pinSet).getCondition();
			newSpec = ((DecisionOutputSet)workingCopy).getCondition();
			if (spec instanceof StructuredOpaqueExpression) {
				if (newSpec instanceof StructuredOpaqueExpression) {
					// CallExpression
					UpdateExpressionRule expressionRule =  new UpdateExpressionRule();
					expressionRule.setImportSession(getImportSession());
					expressionRule.setProjectName(getProjectName());
					expressionRule.setSource(spec);
					expressionRule.setWorkingCopy((StructuredOpaqueExpression) newSpec);
					expressionRule.setProcess(process);
					expressionRule.invoke();
				}						
			}
		}
	}
}
