package com.ibm.btools.blm.ie.imprt.rule.processModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.pe.conn.add.AddActivityConnPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.factory.PeCmdFactory;
import com.ibm.btools.blm.compoundcommand.pe.node.add.AddDatastorePeCmd;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ui.navigation.model.AbstractLibraryChildNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationProjectNode;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Datastore;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.Repository;
import com.ibm.btools.bom.model.processes.activities.RetrieveArtifactPin;
import com.ibm.btools.bom.model.processes.activities.StoreArtifactPin;


/**
 * Update the Repository Flow
 */
public class UpdateRepositoryFlowRule extends AbstractImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private Pin pin;
	private Object workingCopy;
	private Object process;
	private Action action;
	
	/**
	 * Set Original process
	 * @param process
	 */
	public void setProcess(final Object process) {
		this.process = process;
	}	
	
	/**
	 * Set Original action
	 * @param action
	 */
	public void setAction(final Object action) {
		this.action = (Action) action;
	}	
	
	/**
	 * Set Original pin
	 * @param pin
	 */
	public void setSource(final Object pin) {
		this.pin = (Pin) pin;
	}
	
	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = workingCopy;
	}
	
	/**
	 * Invoke the rule
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");			
		if (pin == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.PIN_IS_NULL);			
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		NavigationProjectNode projectNode = NavigatorUtil.getProjectNode(getProjectName());
		if (projectNode == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.PROJECT_NODE_IS_NULL);												
			return;
		}
		PeCmdFactory cmdFactory = getPeCmdFactory();
		AddActivityConnPeCmd cmd = cmdFactory.buildAddObjectConnPeCmd((EObject)workingCopy);
		AddDatastorePeCmd addCmd = cmdFactory.buildAddDatastorePeCmd((EObject)workingCopy);
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
		
		EObject newViewModel = null;
		Object reference = null;
		Repository repository = null;
		if (pin instanceof RetrieveArtifactPin) {
			repository = ((RetrieveArtifactPin)pin).getRepository();
			
			reference = context.get(repository);
			if (reference == null) { 
				// Global
				reference = getImportSession().getContext().get(repository);
				//TODO: Jie d97786 
				if (reference instanceof AbstractLibraryChildNode) {
					reference = BOMUtil.loadRootObject((AbstractLibraryChildNode)reference);
				}	
				if (reference instanceof Datastore) {
					addCmd.setDatastore((Datastore)reference);
					if (addCmd.canExecute()) {
						try {
							addCmd.execute();
						} catch (RuntimeException e) {
							LoggingUtil.trace(this, "invoke", IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
							return;
						}
					} else {
						LoggingUtil.trace(this, "invoke",
							IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
						return;				
					}
					reference = addCmd.getNewViewModel();
					//TODO: d97857 Jie: we want to create separate repository node in the diagram.
					//context.put(repository, reference);
				}
			}
			cmd.setViewSource((EObject)reference);
			cmd.setViewTarget( (EObject) context.get(pin));
	
		} else if (pin instanceof StoreArtifactPin) {			
			repository = ((StoreArtifactPin)pin).getRepository();
			reference = context.get(repository);
			if (reference == null) {
				// Global
				reference = getImportSession().getContext().get(repository);
				//TODO: Jie d97786 
				if (reference instanceof AbstractLibraryChildNode) {
					reference = BOMUtil.loadRootObject((AbstractLibraryChildNode)reference);
				}
				if (reference instanceof Datastore) {
					addCmd.setDatastore((Datastore)reference);
					if (addCmd.canExecute()) {
						try {
							addCmd.execute();
						} catch (RuntimeException e) {
							LoggingUtil.trace(this, "invoke",
								IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
							return;
						}
					} else {
						LoggingUtil.trace(this, "invoke", 
							IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
						return;				
					}	
					reference = addCmd.getNewViewModel();
					//TODO: d97857 Jie: we want to create separate repository node in the diagram.
					//context.put(repository, reference);
				}
			}
			cmd.setViewTarget( (EObject) reference);		
			cmd.setViewSource( (EObject) context.get(pin));	
		}
		
		if (cmd.canExecute()) {
			try {
				cmd.execute();
			} catch (RuntimeException e) {
				LoggingUtil.trace(this, "invoke",
					IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
			}
		} else {
			LoggingUtil.trace(this, "invoke", 
				IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);				
		}				
		
		LoggingUtil.traceExit(this, "invoke");
	}
}
