package com.ibm.btools.blm.ie.imprt.rule.processModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.List;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.pe.factory.AbstractAddPeCmd;
import com.ibm.btools.blm.compoundcommand.process.util.PECommandConstants;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.ImportSession;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.NavigationProjectNode;
import com.ibm.btools.bom.command.resources.RemoveIndividualResourceRequirementBOMCmd;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.processes.actions.AcceptSignalAction;
import com.ibm.btools.bom.model.processes.actions.BroadcastSignalAction;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.actions.Decision;
import com.ibm.btools.bom.model.processes.actions.Fork;
import com.ibm.btools.bom.model.processes.actions.Join;
import com.ibm.btools.bom.model.processes.actions.Map;
import com.ibm.btools.bom.model.processes.actions.Merge;
import com.ibm.btools.bom.model.processes.actions.ObserverAction;
import com.ibm.btools.bom.model.processes.actions.TimerAction;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.FlowFinalNode;
import com.ibm.btools.bom.model.processes.activities.ForLoopNode;
import com.ibm.btools.bom.model.processes.activities.InitialNode;
import com.ibm.btools.bom.model.processes.activities.LoopNode;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.activities.TerminationNode;
import com.ibm.btools.bom.model.resources.IndividualResourceRequirement;
import com.ibm.btools.bom.model.services.Behavior;
import com.ibm.btools.cef.model.CommonNodeModel;

/**
 * Update the Model with a View Model
 */
public abstract class UpdateViewModelRule extends AbstractImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	

	protected boolean isFailToUpdate = false;
	protected Object workingCopy;
	protected Object process;
	protected CommonNodeModel viewModel;
	protected EObject domainModel;	
	protected NavigationProjectNode projectNode;

	/**
	 * Return the source
	 * @return
	 */
	protected abstract Object getSource();

	/**
	 * Return the command
	 * @return
	 */
	protected abstract AbstractAddPeCmd getCommand();

	/**
	 * Update the Model
	 * @return
	 */
	protected abstract void updateModel();

	/**
	 * Set Original process
	 * @param process
	 */
	public void setProcess(final Object process) {
		this.process = process;
	}
	
	/**
	 * Set the working copy - the view model contains the newly created object
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = workingCopy;
	}

	/**
	 * Invoke the rule
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");	
		Object sourceObj = getSource();
	
		if (sourceObj == null) {
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		projectNode = NavigatorUtil.getProjectNode(getProjectName());
		if (projectNode == null) {
			return;
		}
		
		AbstractAddPeCmd cmd = getCommand();
		if (cmd == null) {
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		// Attribute name
		if (sourceObj instanceof NamedElement) {
			cmd.setName(((NamedElement)sourceObj).getName());
		}
		
		if (cmd.canExecute()) {
			try {
				cmd.execute();
			} catch (RuntimeException e) {
				LoggingUtil.trace(this, "invoke", IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
				LoggingUtil.traceExit(this, "invoke");
				return;
			}
		} else {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);	
			LoggingUtil.traceExit(this, "invoke");
			return;			
		}
		
		EObject newViewModel = cmd.getNewViewModel();
		domainModel = cmd.getNewDomainModel();
		if (domainModel == null || newViewModel == null) {
			
			String elementName = getElement(sourceObj);
			String name = ((NamedElement)sourceObj).getName();
			String processName = getProcessName();
			
			LoggingUtil.logWarning(getImportSession(), IeMessageKeys.FAIL_TO_CREATE_ELEMENT, new String[]{elementName, name, processName}, null);
			LoggingUtil.traceExit(this, "invoke");
			return;
		}		
		if (newViewModel instanceof CommonNodeModel) {
			viewModel = (CommonNodeModel)newViewModel;
		}			
		LoggingUtil.trace(this, "invoke", "Put the object into the context");
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);	
		context.put(sourceObj, newViewModel);				

		updateModel();

		LoggingUtil.traceExit(this, "invoke");
	}
	
	protected String getProcessName() {
		String processName = "";
		if (process != null && process instanceof Activity) {
			processName = ((Activity)process).getName();
		}
		return processName;
	}
	
	protected String getElement() {
		Object sourceObj = getSource();		
		return getElement(sourceObj);
	}
	
	protected String getName() {
		Object sourceObj = getSource();	
		if (sourceObj != null && sourceObj instanceof NamedElement) {
			return ((NamedElement)sourceObj).getName();	
		} else {
			return "";
		}
	}	
	
	protected void logWarning(ImportSession session, final String errorCode, final String[] param, 
		final Throwable exception, final String additionalInfo) {
		
		if (errorCode != null && errorCode.compareTo(IeMessageKeys.FAIL_TO_UPDATE_ELEMENT) == 0) {
			if (isFailToUpdate) {
				return;
			} else {
				isFailToUpdate = true;
				LoggingUtil.logWarning(session, errorCode, param, exception, additionalInfo);
			}
		} else {
			LoggingUtil.logWarning(session, errorCode, param, exception, additionalInfo);
		}
	}
	
	protected String getElement(Object node) {
		String key = null;
		if (node instanceof FlowFinalNode) {
			key = IeMessageKeys.FLOW_FINAL_NODE;
		} else if (node instanceof TerminationNode) {
			key = IeMessageKeys.TERMINATION_NODE;
		} else if (node instanceof InitialNode) {
			key = IeMessageKeys.INITIAL_NODE;
		} else if (node instanceof AcceptSignalAction) {
			key = IeMessageKeys.ACCEPT_SIGNAL_ACTION;
		} else if (node instanceof Map) {
			key = IeMessageKeys.MAP;
		} else if (node instanceof Decision) {
			key = IeMessageKeys.DECISION;
		} else if (node instanceof Fork) {
			key = IeMessageKeys.FORK;
		} else if (node instanceof Join) {
			key = IeMessageKeys.JOIN;
		} else if (node instanceof Merge) {
			key = IeMessageKeys.MERGE;
		} else if (node instanceof ObserverAction) {
			key = IeMessageKeys.OBSERVATION_ACTION;
		} else if (node instanceof TimerAction) {
			key = IeMessageKeys.TIMER_ACTION;
		} else if (node instanceof ForLoopNode) {
			key = IeMessageKeys.FOR_LOOP_NODE;
		} else if (node instanceof LoopNode) {
			key = IeMessageKeys.LOOP_NODE;
		} else if (node instanceof CallBehaviorAction) {
			Behavior behavior = ((CallBehaviorAction)node).getBehavior();
			if (behavior == null || !(behavior instanceof Activity)) {
				key = IeMessageKeys.CBA_TASK;
			} else {
				StructuredActivityNode san = ((Activity)behavior).getImplementation();
				if (san == null) {
					key = IeMessageKeys.CBA_TASK;
				} else {
					String aspect = san.getAspect();
					if (aspect != null) {
						if (PECommandConstants.TASK_ASPECT.compareTo(aspect) == 0) {
							key = IeMessageKeys.CBA_TASK;
						} else if (PECommandConstants.PROCESS_ASPECT.compareTo(aspect) == 0) {
							key = IeMessageKeys.CBA_PROCESS;
						} else if (PECommandConstants.SERVICE_ASPECT.compareTo(aspect) == 0) {
							key = IeMessageKeys.CBA_SERVICE;
						}
					} else {
						key = IeMessageKeys.CBA_TASK;
					}
				}
			}
		} else if (node instanceof BroadcastSignalAction) {
			key = IeMessageKeys.BROADCAST_SIGNAL_ACTION;
		} else if (node instanceof StructuredActivityNode) {
			String aspect = ((StructuredActivityNode)node).getAspect();
			if (aspect != null) {
				if (PECommandConstants.TASK_ASPECT.compareTo(aspect) == 0) {
					key = IeMessageKeys.TASK;
				} else if (PECommandConstants.PROCESS_ASPECT.compareTo(aspect) == 0) {
					key = IeMessageKeys.SUBPROCESS;
				} else if (PECommandConstants.HUMAN_TASK_ASPECT.compareTo(aspect) == 0) {
					key = IeMessageKeys.HUMAN_TASK;
				} else if (PECommandConstants.BUSINESS_RULE_TASK_ASPECT.compareTo(aspect) == 0) {
					key = IeMessageKeys.BUSINESS_RULES_TASK;
				}
			}
		}
		if (key == null) {
			key = IeMessageKeys.ELEMENT;
		}
		return LoggingUtil.getLocalizedString(key, null);
	}
	
	
	/**
	 * Return the object from the context by the key
	 * @param key
	 * @return
	 */
	protected void putModelInContext(final Object key, final Object value) {
		ImportContext context = null;
		if (key instanceof PackageableElement) {
			context = getImportSession().getContext();
		} else {
			context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
		}
		context.put(key, value);
	}		
	
	/**
	 * Return the object from the context by the key
	 * @param key
	 * @return
	 */
	protected Object getModelFromContext(final Object key) {
		ImportContext context = null;
		if (key instanceof PackageableElement) {
			context = getImportSession().getContext();
			return BOMUtil.getType(getProjectName(), (PackageableElement)key, context);
		} else {
			context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
			return context.get(key);
		}
	}		
}
