package com.ibm.btools.blm.ie.imprt.rule.processModel.node;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.HashMap;

import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.bom.command.processes.actions.UpdateAcceptSignalActionBOMCmd;
import com.ibm.btools.bom.model.artifacts.Signal;
import com.ibm.btools.bom.model.processes.actions.AcceptSignalAction;

/**
 * Update the Accept Signal Action
 */
public class UpdateAcceptSignalActionRule extends UpdateActionRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	/**
	 * Update Model
	 */
	protected void updateModel() {		
		LoggingUtil.traceEntry(this, "updateModel");
		super.updateModel();

		// Attribute signal
		if (action instanceof AcceptSignalAction && domainModel instanceof AcceptSignalAction) {
			Signal signal = ((AcceptSignalAction)action).getSignal();
			
			Object newSignal = getModelFromContext(signal);			
			// TN - added to support reusing existing individual resource in the workspace			
			if (newSignal == null && signal != null) {
				newSignal = BOMUtil.getSignalFromWorkspace(getImportSession(), getProjectName(), signal);	
			}
			// TN - end of added to support reusing existing individual resource in the workspace
			
			if (newSignal != null && newSignal instanceof Signal) {
				UpdateAcceptSignalActionBOMCmd cmd = new UpdateAcceptSignalActionBOMCmd((AcceptSignalAction)domainModel);
				cmd.setSignal((Signal)newSignal);
				if (cmd.canExecute()) {
					try {
						cmd.execute();
					} catch (RuntimeException e) {
						logWarning(getImportSession(), 
							IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
							IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);									
					}
				} else {
					logWarning(getImportSession(), 
						IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 						
						IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);			
				}				 	
			}
			
			// 20050413 - create the UID hashmap for ADF Import
			if (action.getUid() != null) {
				HashMap processHashMap = BOMUtil.getUIDMap(getImportSession().getContext());
				// track the source process elements UID as 'key' and its corresponding working copy process element UID as 'value'
				processHashMap.put(action.getUid(), ((AcceptSignalAction)domainModel).getUid());
			}
			// end of 20050413 - create the UID hashmap for ADF Import			
		}
		
		// [Expression] Attribute filter expression
		// Attribute signalToOutputMap - not handled
		
		// operational data
		createOperationalData();		
					
		LoggingUtil.traceExit(this, "updateModel");
	}	
}
