package com.ibm.btools.blm.ie.imprt.rule.processModel.node;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.HashMap;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.gef.DomainViewObjectHelper;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.bom.command.processes.actions.UpdateBroadcastSignalActionBOMCmd;
import com.ibm.btools.bom.model.artifacts.Signal;
import com.ibm.btools.bom.model.processes.actions.BroadcastSignalAction;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;

/**
 * Update the BroadcastSignalAction
 */
public class UpdateBroadcastSignalActionRule extends UpdateInvocationActionRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	

	/**
	 * Update Model
	 */
	protected void updateModel() {		
		LoggingUtil.traceEntry(this, "updateModel");
		super.updateModel();

		if (action instanceof BroadcastSignalAction && domainModel instanceof BroadcastSignalAction) {
			UpdateBroadcastSignalActionBOMCmd cmd =  new UpdateBroadcastSignalActionBOMCmd((BroadcastSignalAction)domainModel);
			
			// Attribute signal
			Signal signal = ((BroadcastSignalAction)action).getSignal();	
			if (signal != null) {		
				Object newSignal = getModelFromContext(signal);				
				// TN - added to support reusing existing individual resource in the workspace				
				if (newSignal == null && signal != null) {
					newSignal = BOMUtil.getSignalFromWorkspace(getImportSession(), getProjectName(), signal);	
				}
				// TN - end of added to support reusing existing individual resource in the workspace				
				
				if (newSignal != null && newSignal instanceof Signal) {				
					cmd.setSignal((Signal)newSignal);
				}
			}
			
			// Attribute broadcastScope
			ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);							
			StructuredActivityNode node = ((BroadcastSignalAction)action).getBroadcastScope();
			if (node != null) {		
				Object scope = context.get(node);
				Object scopeDomainModel = DomainViewObjectHelper.getDomainObject((EObject)scope);
				if (scopeDomainModel instanceof StructuredActivityNode) {
					cmd.setBroadcastScope((StructuredActivityNode)scopeDomainModel);
				}
			}
			
			if (cmd.canExecute()) {
				try {
					cmd.execute();
				} catch (RuntimeException e) {
					logWarning(getImportSession(), 
						IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
						IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);								
				}
			} else {
				logWarning(getImportSession(), 
					IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
					IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);			
			}
			
			// 20040413 - create the UID hashmap for ADF Import
			if (action.getUid() != null) {
				HashMap processHashMap = BOMUtil.getUIDMap(getImportSession().getContext());
				// track the source process elements UID as 'key' and its corresponding working copy process element UID as 'value'
				processHashMap.put(action.getUid(), ((BroadcastSignalAction)domainModel).getUid());
			}
			// end of 20040413 - create the UID hashmap for ADF Import	
		}
		
		// Attribute inputToSignalMap - not handled
		
		// operational data
		createOperationalData();
					
		LoggingUtil.traceExit(this, "updateModel");
	}
}
