package com.ibm.btools.blm.ie.imprt.rule.processModel.node;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.gef.DomainViewObjectHelper;
import com.ibm.btools.blm.compoundcommand.pe.factory.PeCmdFactory;
import com.ibm.btools.blm.compoundcommand.pe.factory.RemovePeCmd;
import com.ibm.btools.blm.compoundcommand.pe.pin.convert.ConvertToRetrieveArtifactPinPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.pin.convert.ConvertToStoreArtifactPinPeCmd;
import com.ibm.btools.blm.compoundcommand.process.util.PEDomainViewObjectHelper;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.rule.processModel.UpdatePinRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.bom.command.processes.actions.UpdateCallBehaviorActionBOMCmd;
import com.ibm.btools.bom.command.processes.activities.UpdateInputPinSetBOMCmd;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.ControlPin;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.RetrieveArtifactPin;
import com.ibm.btools.bom.model.processes.activities.StoreArtifactPin;
import com.ibm.btools.cef.gef.model.ConnectorModel;
import com.ibm.btools.cef.model.CommonContainerModel;
import com.ibm.btools.emf.cf.BtCompoundCommand;

/**
 * Update the CallBehaviorAction
 */
public class UpdateCallBehaviorActionRule extends UpdateCallActionRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	


	/**
	 * Update Model
	 */
	protected void updateModel() {		
		LoggingUtil.traceEntry(this, "updateModel");
		super.updateModel();
		
		// [Sequence] Attribute isSynchronous
		PeCmdFactory cmdFactory = getPeCmdFactory();
		//UpdateCallActionPeCmd cmd = cmdFactory.buildUpdateCallActionPeCmd(viewModel);
		// TN - fixes for d116940
		UpdateCallBehaviorActionBOMCmd cmd = new UpdateCallBehaviorActionBOMCmd((CallBehaviorAction) domainModel);
		cmd.setIsSynchronous(((CallBehaviorAction)action).getIsSynchronous().booleanValue());
		if (cmd.canExecute()) {
			try {
				cmd.execute();
			} catch (RuntimeException e) {
				logWarning(getImportSession(), 
					IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
					IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);								
			}
		} else {
			logWarning(getImportSession(), 
				IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null,  
				IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);			
		}		
		
		if (action instanceof CallBehaviorAction && domainModel instanceof CallBehaviorAction) {
			// 20050413 - create the UID hashmap for ADF Import
			if (action.getUid() != null) {
				HashMap processHashMap = BOMUtil.getUIDMap(getImportSession().getContext());
				// track the source process elements UID as 'key' and its corresponding working copy process element UID as 'value'
				processHashMap.put(action.getUid(), ((CallBehaviorAction)domainModel).getUid());
			}
			// end of 20050413 - create the UID hashmap for ADF Import
			
			if (((CallBehaviorAction)action).getIsSynchronous().booleanValue() == false) {
				setPinSetCorrelationForAsyncCBA();				
			}		
		}		
		
		LoggingUtil.traceExit(this, "updateModel");
	}

	/**
	 * Reset the correlation between the InputPinSet and the OutputPinSet in the asynchronous CBA
	 */
	protected void setPinSetCorrelationForAsyncCBA() {
		LoggingUtil.traceEntry(this, "setPinSetCorrelationForAsyncCBA");
		// resetting the correlation between pinsets		
		List newOutputPinSets = ((Action)domainModel).getOutputPinSet();
		List newInputPinSets = ((Action)domainModel).getInputPinSet();
		
		if (newOutputPinSets != null && !newOutputPinSets.isEmpty() && newInputPinSets != null && !newInputPinSets.isEmpty()) {		
			int inputPinSetSize = newInputPinSets.size();
			int outputPinSetSize = newOutputPinSets.size();			
			for (int i=0; i<inputPinSetSize; i++) {
				InputPinSet inputPinSet = (InputPinSet) newInputPinSets.get(i);
				OutputPinSet outputPinSet = null;
				
				List corrOutputPinSets = inputPinSet.getOutputPinSet();				
				BtCompoundCommand compoundCmd = new BtCompoundCommand();
				if (corrOutputPinSets != null){				
					UpdateInputPinSetBOMCmd cmd = new UpdateInputPinSetBOMCmd(inputPinSet);
					for (Iterator it = corrOutputPinSets.iterator(); it.hasNext();) {						
						OutputPinSet corrOutputPinSet = (OutputPinSet) it.next();
						cmd.removeOutputPinSet(corrOutputPinSet);
						compoundCmd.append(cmd);
					}
				}
				UpdateInputPinSetBOMCmd cmd2 = new UpdateInputPinSetBOMCmd(inputPinSet);
				cmd2.addOutputPinSet((OutputPinSet) newOutputPinSets.get(newOutputPinSets.size() - 1) );
				compoundCmd.append(cmd2);
				if (compoundCmd.canExecute()) {
					compoundCmd.execute();
				} else {
					LoggingUtil.trace(this, "setPinSetCorrelationForAsyncCBA", IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
				}				
			}
		}
		// resetting the correlation between pinsets
		LoggingUtil.traceExit(this, "setPinSetCorrelationForAsyncCBA");
	}	
	
	/**
	 * Create pins
	 */	
	protected void createPin() {

		LoggingUtil.traceEntry(this, "createPin");
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
		
		EList list = action.getInputObjectPin();
		EList newList = ((Action)domainModel).getInputObjectPin();
		Pin eachPin  = null;
		Pin newPin = null;
		EObject pinView = null;	
		List viewList = getAllViewObjectPin(viewModel);	
		// Input Object Pin	
		if (list.size() == newList.size()) {
			for (int i=0; i<list.size(); i++) {
				eachPin = (Pin) list.get(i);
				newPin = (Pin) newList.get(i);				
				pinView = getViewObjectByPin(newPin, viewList);				
				if (eachPin instanceof RetrieveArtifactPin && newPin instanceof InputObjectPin) {
					//update the original input obj pin to a retrieve artifact pin					
					PeCmdFactory cmdFactory = new PeCmdFactory(null);
					ConvertToRetrieveArtifactPinPeCmd convertCmd = cmdFactory.buildConvertToRetrieveArtifactPinPeCmd(pinView);;					
					if (convertCmd.canExecute()) {						
						try {
							convertCmd.execute();
							pinView = convertCmd.getNewViewModel();
						} catch (RuntimeException e) {
							LoggingUtil.trace(this, "createPin", "Convert pin to retrieve artifact pin throws a RuntimeException."); 
						}
					} else {
						LoggingUtil.trace(this, "createPin", "Convert pin to retrieve artifact pin command cannot be executed"); 
					}	
				}

				if (pinView != null) {
					context.put(eachPin, pinView);
				}
			}
		}
		
		list = action.getOutputObjectPin();
		newList = ((Action)domainModel).getOutputObjectPin();		
		if (list.size() == newList.size()) {
			for (int i=0; i<list.size(); i++) {
				eachPin = (Pin) list.get(i);
				newPin = (Pin) newList.get(i);
				pinView = getViewObjectByPin(newPin, viewList);				
				if (eachPin instanceof StoreArtifactPin && newPin instanceof OutputObjectPin) {
					// update the original output obj pin to a store artifact pin
					PeCmdFactory cmdFactory = new PeCmdFactory(null);
					ConvertToStoreArtifactPinPeCmd convertCmd = cmdFactory.buildConvertToStoreArtifactPinPeCmd(pinView);;					
					if (convertCmd.canExecute()) {						
						try {
							convertCmd.execute();
							pinView = convertCmd.getNewViewModel();
						} catch (RuntimeException e) {
							LoggingUtil.trace(this, "createPin", "Convert pin to store artifact pin throws a RuntimeException."); 
						}					
					} else {
						LoggingUtil.trace(this, "createPin", "Convert pin to store artifact pin command cannot be executed");
					}				
				}
				
				if (pinView != null) {
					context.put(eachPin, pinView);
				}
			}
		}
		
		UpdatePinRule pinRule = null;
		list = action.getInputControlPin();		
		for (int i=0; i<list.size(); i++) {
			eachPin = (Pin) list.get(i);
			pinRule = new UpdatePinRule();
			pinRule.setImportSession(getImportSession());
			pinRule.setProjectName(getProjectName());
			pinRule.setSource(eachPin);
			pinRule.setWorkingCopy(viewModel);
			pinRule.setProcess(process);
			pinRule.setAction(action);
			pinRule.invoke();
		}	
		
		// Clean up the CBA first
		removeAdditionalControlPins();
		
		list = action.getOutputControlPin();
		for (int i=0; i<list.size(); i++) {
			eachPin = (Pin) list.get(i);
			pinRule = new UpdatePinRule();
			pinRule.setImportSession(getImportSession());
			pinRule.setProjectName(getProjectName());
			pinRule.setSource(eachPin);
			pinRule.setWorkingCopy(viewModel);
			pinRule.setProcess(process);
			pinRule.setAction(action);
			pinRule.invoke();
		}
		LoggingUtil.traceExit(this, "createPin");
	}
	
	private void removeAdditionalControlPins() {
		PeCmdFactory cmdFactory = getPeCmdFactory();
		RemovePeCmd cmd = null;
		List viewList = getAllViewObjectControlPin(viewModel);	
		EList newList = ((Action)domainModel).getOutputControlPin();
		Pin eachPin  = null;
		EObject pinView = null;	
		// Output Control Pin	
		for (int i=0; i<newList.size(); i++) {
			eachPin = (Pin) newList.get(i);
			pinView = getViewObjectByPin(eachPin, viewList);
			if (pinView != null) {
				cmd = cmdFactory.buildRemovePeCmd(pinView);
				if (cmd.canExecute()) {
					try {
						cmd.execute();
					} catch (RuntimeException e) {
						LoggingUtil.trace(this, "removeAdditionalControlPins", "Removed Pin Command throw a RuntimeException."); 
					}
				} else {
					LoggingUtil.trace(this, "removeAdditionalControlPins", "Removed Pin Command cannot be executed."); 
				}				
			}		
		}		 		
	}
	
	/**
	 * Create pinSet
	 */	
	protected void createPinSet() {
		LoggingUtil.traceEntry(this, "createPinSet");	
		PinSet originalPinSet = null;
		PinSet newPinSet = null;
		Object viewObject = null;	
		Iterator viewIterator = null;
		
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
		
		// register created inputpin set
		EList originalPinSets = action.getInputPinSet();
		EList newPinSets = ((Action)domainModel).getInputPinSet();

		int size = 0;
		if (originalPinSets.size() >= newPinSets.size()) {
			size = newPinSets.size();
		} else {
			size = originalPinSets.size();
		}
		
		HashMap processHashMap = BOMUtil.getUIDMap(getImportSession().getContext());
		for (int i=0; i<size; i++) {
			originalPinSet = (PinSet) originalPinSets.get(i);
			newPinSet = (PinSet) newPinSets.get(i);
			
			// 20050419 - create the UID hashmap for ADF Import
			if (originalPinSet.getUid() != null) {				
				// track the source process elements UID as 'key' and its corresponding working copy process element UID as 'value'
				processHashMap.put(originalPinSet.getUid(), newPinSet.getUid());
			}
			// end of 20050419 - create the UID hashmap for ADF Import	

			viewIterator = PEDomainViewObjectHelper.getAllViewInputPinSetFromViewAction(viewModel).iterator();
			while (viewIterator.hasNext()) {
				viewObject = viewIterator.next();
				if (PEDomainViewObjectHelper.getDomainObject((EObject)viewObject) == newPinSet) {
					context.put(originalPinSet, viewObject);
					break;
				}
			}
		}
		
		// register created outputpin set
		originalPinSets = action.getOutputPinSet();
		newPinSets = ((Action)domainModel).getOutputPinSet();

		if (originalPinSets.size() >= newPinSets.size()) {
			size = newPinSets.size();
		} else {
			size = originalPinSets.size();
		}
		
		for (int i=0; i<size; i++) {
			originalPinSet = (PinSet) originalPinSets.get(i);
			newPinSet = (PinSet) newPinSets.get(i);
			
			// 20050419 - create the UID hashmap for ADF Import
			if (originalPinSet.getUid() != null) {				
				// track the source process elements UID as 'key' and its corresponding working copy process element UID as 'value'
				processHashMap.put(originalPinSet.getUid(), newPinSet.getUid());
			}
			// end of 20050419 - create the UID hashmap for ADF Import
			
			viewIterator = PEDomainViewObjectHelper.getAllViewOutputPinSetFromViewAction(viewModel).iterator();
			while (viewIterator.hasNext()) {
				viewObject = viewIterator.next();
				if (PEDomainViewObjectHelper.getDomainObject((EObject)viewObject) == newPinSet) {
					context.put(originalPinSet, viewObject);
					break;
				}
			}
		}
		LoggingUtil.traceExit(this, "createPinSet");	
	}
	
	private List getAllViewObjectControlPin(EObject viewParent) {
		List list = new ArrayList();
		if (viewParent instanceof CommonContainerModel) {
			for (Iterator it = ((CommonContainerModel) viewParent).getCompositionChildren().iterator(); it.hasNext();) {
				EObject viewChild = (EObject) it.next();
				EObject domainChild = PEDomainViewObjectHelper.getDomainObject(viewChild);
				// Check whether the domain child is an input object pin.
				if (domainChild != null && domainChild instanceof ControlPin) {
					list.add(viewChild);
				}
			}
		}
				
		return list;
	}		
	
	private List getAllViewObjectPin(EObject viewParent) {
		List list = new ArrayList();
		if (viewParent instanceof CommonContainerModel) {
			for (Iterator it = ((CommonContainerModel) viewParent).getCompositionChildren().iterator(); it.hasNext();) {
				EObject viewChild = (EObject) it.next();
				EObject domainChild = PEDomainViewObjectHelper.getDomainObject(viewChild);
				// Check whether the domain child is an input object pin.
				if (domainChild != null && domainChild instanceof ObjectPin) {
					list.add(viewChild);
				}
			}
		}
				
		return list;
	}	
	
	/**
	 * Get the pin view object. 
	 * @param pin
	 * @param list branches
	 * @return
	 */
	protected EObject getViewObjectByPin(final Object pin, final List list) {
		Iterator iterator = list.iterator();
		Object connector = null;
		while (iterator.hasNext()) {
			connector = iterator.next();
			if (connector instanceof ConnectorModel) {
				if (pin == DomainViewObjectHelper.getDomainObject((ConnectorModel)connector)) {
					return (ConnectorModel) connector;
				}					 
			}
		}
		return null;
	}	
}
