package com.ibm.btools.blm.ie.imprt.rule.processModel.node;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.gef.DomainViewObjectHelper;
import com.ibm.btools.blm.compoundcommand.process.util.PEDomainViewObjectHelper;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.rule.processModel.UpdatePinRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.bom.command.processes.actions.AddOperationalProbabilitiesToActionBOMCmd;
import com.ibm.btools.bom.command.processes.actions.AddOutputSetProbabilitiesToOperationalProbabilitiesBOMCmd;
import com.ibm.btools.bom.command.processes.actions.AddOutputSetProbabilityToOutputSetProbabilitiesBOMCmd;
import com.ibm.btools.bom.command.processes.actions.UpdateDecisionBOMCmd;
import com.ibm.btools.bom.command.processes.activities.UpdateOutputPinSetBOMCmd;
import com.ibm.btools.bom.model.artifacts.OpaqueExpression;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.actions.Decision;
import com.ibm.btools.bom.model.processes.actions.OperationalProbabilities;
import com.ibm.btools.bom.model.processes.actions.OutputSetProbabilities;
import com.ibm.btools.bom.model.processes.actions.OutputSetProbability;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.DecisionOutputSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.cef.model.CommonModel;

/**
 * Update the Decision
 */
public class UpdateDecisionRule extends UpdateMultiplePinSetControlActionRule {

	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	

	/**
	 * Update Model
	 */
	protected void updateModel() {		
		LoggingUtil.traceEntry(this, "updateModel");
		super.updateModel();

		// Attribute multiplePaths
		if (action instanceof Decision && domainModel instanceof Decision) {
			UpdateDecisionBOMCmd cmd = new UpdateDecisionBOMCmd((Decision)domainModel);
			cmd.setMultiplePaths(((Decision)action).getMultiplePaths().booleanValue());
			if (cmd.canExecute()) {
				try {
					cmd.execute();
				} catch (RuntimeException e) {
					logWarning(getImportSession(), 
						IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
						IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);								
				}
			} else {
				logWarning(getImportSession(), 
					IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
					IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);			
			}
			
			// 20050413 - create the UID hashmap for ADF Import
			if (action.getUid() != null) {
				HashMap processHashMap = BOMUtil.getUIDMap(getImportSession().getContext());
				// track the source process elements UID as 'key' and its corresponding working copy process element UID as 'value'
				processHashMap.put(action.getUid(), ((Decision)domainModel).getUid());
			}
			// end of 20050413 - create the UID hashmap for ADF Import	
		}		
		
		//for output branch probability only
		createOutputPathProbabilities();
		
		LoggingUtil.traceExit(this, "updateModel");
	}	
	
	/**
	 * Create pins
	 */	
	protected void createPin() {
		LoggingUtil.traceEntry(this, "createPin");
		EObject inBranch = getInBranchFromViewAction();

		Pin eachPin = null;
		Pin newPin = null;
		UpdatePinRule inputObjectPinRule = null;
		UpdatePinRule inputControlPinRule = null;
		UpdatePinRule outputControlPinRule = null;
		
		Iterator pinIterator = action.getInputObjectPin().iterator();
		LoggingUtil.trace(this, "createPin", "Create input object pin");		
		while(pinIterator.hasNext()) {
			eachPin = (Pin) pinIterator.next();
			inputObjectPinRule = new UpdatePinRule();
			inputObjectPinRule.setImportSession(getImportSession());
			inputObjectPinRule.setProjectName(getProjectName());
			inputObjectPinRule.setSource(eachPin);
			inputObjectPinRule.setWorkingCopy(inBranch);
			inputObjectPinRule.setProcess(process);
			inputObjectPinRule.setAction(action);
			inputObjectPinRule.invoke(); 
		}	
	
		pinIterator = action.getInputControlPin().iterator();
		LoggingUtil.trace(this, "createPin", "Create input control pin");		
		while(pinIterator.hasNext()) {
			eachPin = (Pin) pinIterator.next();
			inputControlPinRule = new UpdatePinRule();
			inputControlPinRule.setImportSession(getImportSession());
			inputControlPinRule.setProjectName(getProjectName());
			inputControlPinRule.setSource(eachPin);
			inputControlPinRule.setWorkingCopy(inBranch);
			inputControlPinRule.setProcess(process);
			inputControlPinRule.setAction(action);
			inputControlPinRule.invoke(); 
		}
	
		// Put the output pins in the context
		List outBranches = PEDomainViewObjectHelper.getAllOutBranch(viewModel);		
		
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
		EList list = action.getOutputObjectPin();
		EList newList = ((Action)domainModel).getOutputObjectPin();
		EObject pinView = null;		
		LoggingUtil.trace(this, "createPin", "Get output object pin");	
		/*
		if (list.size() == newList.size()) {
			for (int i=0; i<list.size(); i++) {
				eachPin = (Pin) list.get(i);
				newPin = (Pin) newList.get(i);
				pinView = getViewObjectByPin(newPin, outBranches);
				if (pinView != null) {
					context.put(eachPin, pinView);
				}
			}
		}	
		*/
		List outputPinSetList = action.getOutputPinSet();
		List newOutputPinSetList = ((Action) domainModel).getOutputPinSet();
		if (outputPinSetList.size() == newOutputPinSetList.size()) {
			for (Iterator it = outputPinSetList.iterator(); it.hasNext();) {
				Object object = it.next();
				if (object instanceof OutputPinSet) {
					OutputPinSet outputPinSet = (OutputPinSet) object;
					Object newViewOutBranch = context.get(outputPinSet);
					if (newViewOutBranch instanceof CommonModel) {
						EObject newViewOutputPinSet = PEDomainViewObjectHelper.getViewOutputPinSetFromViewControlActionBranch((CommonModel)newViewOutBranch);
						EObject newObject = DomainViewObjectHelper.getDomainObject(newViewOutputPinSet);
						if (newObject instanceof OutputPinSet) {
							OutputPinSet newOutputPinSet = (OutputPinSet) newObject;
							List outputObjectPinList = outputPinSet.getOutputObjectPin();
							List newOutputObjectPinList = newOutputPinSet.getOutputObjectPin();
							if (outputObjectPinList.size() == newOutputObjectPinList.size()) {
								List mappedPins = new LinkedList();
								for (int i=0; i<outputObjectPinList.size(); i++) {
									eachPin = (Pin) outputObjectPinList.get(i);									
									newPin = (Pin) newOutputObjectPinList.get(i);																	
									pinView = getViewObjectByPin(newPin, outBranches);
									if (pinView != null) {
										context.put(eachPin, pinView);
									}
								}							
							}
							
							//update outputPinSet name
							String name = outputPinSet.getName();
							if (name != null) {
								UpdateOutputPinSetBOMCmd cmd = new UpdateOutputPinSetBOMCmd(newOutputPinSet);
								cmd.setName(name);			
								if (cmd.canExecute()) {
									try {
										cmd.execute();
									} catch (RuntimeException e) {
										logWarning(getImportSession(), 
											IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
											IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
									}
								}
								if(((DecisionOutputSet)outputPinSet).getCondition() == null){
									//we need to keep default created condition name in synch with outputPinSet
									OpaqueExpression condition = ((DecisionOutputSet)newOutputPinSet).getCondition();
									if( condition!= null){										
										condition.setName(name);
									}
								}
							}					
						}
					}					
				}
			}
		}
		


		
		list = action.getOutputControlPin();
		Iterator pinSetListIterator = null;
		EList pinSetList = action.getOutputPinSet();
		OutputPinSet pinSet = null;
		Object eachOutBranch = null;	
		LoggingUtil.trace(this, "createPin", "Get output control pin");	
		for (int i=0; i<list.size(); i++) {
			eachPin = (Pin) list.get(i);
			pinSetListIterator = pinSetList.iterator();
			while (pinSetListIterator.hasNext()) {
				pinSet = (OutputPinSet) pinSetListIterator.next();
				if (pinSet.getOutputControlPin().contains(eachPin)) {
					eachOutBranch = context.get(pinSet);
					if (eachOutBranch != null) {
						outputControlPinRule = new UpdatePinRule();
						outputControlPinRule.setImportSession(getImportSession());
						outputControlPinRule.setProjectName(getProjectName());
						outputControlPinRule.setSource(eachPin);
						outputControlPinRule.setWorkingCopy(eachOutBranch);
						outputControlPinRule.setProcess(process);
						outputControlPinRule.setAction(action);
						outputControlPinRule.invoke();
						break;
					}
				}
			}
		}
		
		updatePinName();
				
		LoggingUtil.traceExit(this, "createPin");		
	}
	
	/**
	 * Creates output path probabilities
	 * Override super class method.
	 */
	protected void createOutputPathProbabilities(){
		LoggingUtil.traceEntry(this, "createOutputPathProbabilities");
		
		OperationalProbabilities opProbs = action.getOperationalProbabilities();
		if(opProbs == null){
			LoggingUtil.traceExit(this, "createOutputPathProbabilities", "OperationalProbabilities is null");
			return;
		}	
		OutputSetProbabilities outputSetProbs = opProbs.getOutputSetProbabilities();
		if(outputSetProbs == null){
			LoggingUtil.traceExit(this, "createOutputPathProbabilities", "OutputSetProbabilities is null");
			return;
		}	
	
		//get context
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
		OperationalProbabilities domainOpProbs = ((Action)domainModel).getOperationalProbabilities();
		if(domainOpProbs == null){
			AddOperationalProbabilitiesToActionBOMCmd cmd = new AddOperationalProbabilitiesToActionBOMCmd((Action)domainModel);
			executeCmd(cmd);
			domainOpProbs = (OperationalProbabilities)cmd.getObject();
		}
		OutputSetProbabilities domainOutputSetProbs = null;
		if(domainOpProbs != null){
			domainOutputSetProbs = domainOpProbs.getOutputSetProbabilities();
			if(domainOutputSetProbs == null){								
				AddOutputSetProbabilitiesToOperationalProbabilitiesBOMCmd cmd2 = new AddOutputSetProbabilitiesToOperationalProbabilitiesBOMCmd(domainOpProbs);
				executeCmd(cmd2);
				domainOutputSetProbs = (OutputSetProbabilities)cmd2.getObject();
			}
			
		}
		if(domainOutputSetProbs != null){			
			for(Iterator it = outputSetProbs.getOutputSetProbability().iterator(); it.hasNext(); ){
				OutputSetProbability outputSetProb = (OutputSetProbability)it.next();
				OutputPinSet outputPinSet = outputSetProb.getOutputPinSet();
				ValueSpecification prob = outputSetProb.getValue();
				if(outputPinSet != null && prob != null){
					EObject outputBranchViewModel = (EObject)context.get(outputPinSet);
					if (outputBranchViewModel instanceof CommonModel) {
						EObject viewOutputPinSet = PEDomainViewObjectHelper.getViewOutputPinSetFromViewControlActionBranch((CommonModel)outputBranchViewModel);
						OutputPinSet domainOutputPinSet =(OutputPinSet)DomainViewObjectHelper.getDomainObject(viewOutputPinSet);
						OutputSetProbability domainOutputSetProb = getOutputSetProbabilityByOutputSet(domainOutputSetProbs, domainOutputPinSet);
						if(domainOutputSetProb == null){										
							AddOutputSetProbabilityToOutputSetProbabilitiesBOMCmd cmd3 = new AddOutputSetProbabilityToOutputSetProbabilitiesBOMCmd(domainOutputSetProbs);
							cmd3.setOutputPinSet(domainOutputPinSet);
							executeCmd(cmd3);
							domainOutputSetProb = (OutputSetProbability)cmd3.getObject();	
						}			
						if(domainOutputSetProb != null){
							updateValueToOutputPinSetProbability(domainOutputSetProb, prob);
						}
					}					 
				}
			}
		}
		
		LoggingUtil.traceExit(this, "createOutputPathProbabilities");
	}
	
	private OutputSetProbability getOutputSetProbabilityByOutputSet(OutputSetProbabilities outputSetProbs, OutputPinSet outputPinSet){
		LoggingUtil.traceEntry(this, "getOutputSetProbabilityByOutputSet");
		for(Iterator it = outputSetProbs.getOutputSetProbability().iterator(); it.hasNext(); ){
			OutputSetProbability nextOsProb = (OutputSetProbability)it.next();
			if(nextOsProb.getOutputPinSet().equals(outputPinSet)){
				LoggingUtil.traceExit(this, "getOutputSetProbabilityByOutputSet", "found outputSetProbability");
				return nextOsProb;
			}			
		}
		LoggingUtil.traceExit(this, "getOutputSetProbabilityByOutputSet", "null returned");
		return null;
	}
}
