package com.ibm.btools.blm.ie.imprt.rule.processModel.node;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.bom.command.artifacts.UpdateLiteralIntegerBOMCmd;
import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.ForLoopNode;
import com.ibm.btools.bom.model.processes.activities.Variable;


/**
 * Update the ForLoopNode
 */
public class UpdateForLoopNodeRule extends UpdateLoopNodeRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	// Attribute first - do no handle
	// Attribute last - do no handle
	// Attribute step - do no handle

	/**
	 * Update Model
	 */
	protected void updateModel() {		
		LoggingUtil.traceEntry(this, "updateModel");
		super.updateModel();

		// Attribute firstVariable
		// Attribute lastVariable
		// Attribute stepVariable
		if (action instanceof ForLoopNode && domainModel instanceof ForLoopNode) {
			Variable newFirstVariable = ((ForLoopNode)domainModel).getFirstVariable();
			Variable firstVariable = ((ForLoopNode)action).getFirstVariable();
			updateVariable(firstVariable, newFirstVariable);
			Variable newLastVariable = ((ForLoopNode)domainModel).getLastVariable();
			Variable lastVariable = ((ForLoopNode)action).getLastVariable();
			updateVariable(lastVariable, newLastVariable);
			Variable newStepVariable = ((ForLoopNode)domainModel).getStepVariable();
			Variable stepVariable = ((ForLoopNode)action).getStepVariable();			
			updateVariable(stepVariable, newStepVariable);
		}
					
		LoggingUtil.traceExit(this, "updateModel");
	}	
	
	/**
	 * Update variable
	 * @param variable
	 */
	private void updateVariable(final Variable variable, Variable newVariable) {		
		// Default value is used to keep the value
		EList list = null;
		EList newList = null;
		Object value = null;
		list = variable.getDefaultValue();
		newList = newVariable.getDefaultValue();
		if (list.size() >0 && newList.size() > 0) {
			updateValue((ValueSpecification)list.get(0), (ValueSpecification)newList.get(0));
		}
		//d97946 - Jie: must update the computedValue
		list = variable.getComputedValue();
		newList = newVariable.getComputedValue();
		if (list.size() >0 && newList.size() > 0) {
			updateValue((ValueSpecification)list.get(0), (ValueSpecification)newList.get(0));
		}
	}
	
	/**
	 * Update value
	 * @param value
	 */
	private void updateValue(final ValueSpecification value, ValueSpecification newValue) {
		if (value instanceof LiteralInteger && newValue instanceof LiteralInteger) {
			int number = ((LiteralInteger)value).getValue().intValue();
			UpdateLiteralIntegerBOMCmd cmd = new UpdateLiteralIntegerBOMCmd((LiteralInteger)newValue);
			cmd.setValue(number);	
			if (cmd.canExecute()) {
				try {
					cmd.execute();
				} catch (RuntimeException e) {
					logWarning(getImportSession(), 
						IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
						IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);								
				}
			} else {
				logWarning(getImportSession(), 
					IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
					IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);			
			}			
		}	
	}	
}
