package com.ibm.btools.blm.ie.imprt.rule.processModel.node;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.pe.branch.add.AddOutBranchToControlActionPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.factory.PeCmdFactory;
import com.ibm.btools.blm.compoundcommand.process.util.PEDomainViewObjectHelper;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.rule.processModel.UpdatePinRule;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.NavigationProjectNode;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Pin;

/**
 * Update the Fork
 */
public class UpdateForkRule extends UpdateSinglePinSetControlActionRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	

	/**
	 * Update Model
	 */
	protected void updateModel() {		
		LoggingUtil.traceEntry(this, "updateModel");
		super.updateModel();

		updateSinglePinSets();
		
					
		LoggingUtil.traceExit(this, "updateModel");
	}

	/**
	 * Create pins
	 */
	protected void createPin() {
		LoggingUtil.traceEntry(this, "createPin");
		EObject inBranch = getInBranchFromViewAction();

		Pin eachPin = null;
		Pin newPin = null;
		UpdatePinRule inputObjectPinRule = null;
		UpdatePinRule inputControlPinRule = null;
		
		Iterator pinIterator = action.getInputObjectPin().iterator();
		LoggingUtil.trace(this, "createPin", "Create input object pin");		
		while(pinIterator.hasNext()) {
			eachPin = (Pin) pinIterator.next();
			inputObjectPinRule = new UpdatePinRule();
			inputObjectPinRule.setImportSession(getImportSession());
			inputObjectPinRule.setProjectName(getProjectName());
			inputObjectPinRule.setSource(eachPin);
			inputObjectPinRule.setWorkingCopy(inBranch);
			inputObjectPinRule.setProcess(process);
			inputObjectPinRule.setAction(action);
			inputObjectPinRule.invoke(); 
		}	
		
		pinIterator = action.getInputControlPin().iterator();
		LoggingUtil.trace(this, "createPin", "Create input control pin");		
		while(pinIterator.hasNext()) {
			eachPin = (Pin) pinIterator.next();
			inputControlPinRule = new UpdatePinRule();
			inputControlPinRule.setImportSession(getImportSession());
			inputControlPinRule.setProjectName(getProjectName());
			inputControlPinRule.setSource(eachPin);
			inputControlPinRule.setWorkingCopy(inBranch);
			inputControlPinRule.setProcess(process);
			inputControlPinRule.setAction(action);
			inputControlPinRule.invoke(); 
		}
		
		// Put the output pins in the context		
		List outBranches = PEDomainViewObjectHelper.getAllOutBranch(viewModel);		
		
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
		EList list = action.getOutputObjectPin();
		EList newList = ((Action)domainModel).getOutputObjectPin();
		EObject pinView = null;		
		LoggingUtil.trace(this, "createPin", "Get output object pin");	
		if (list.size() <= newList.size()) {
			for (int i=0; i<list.size(); i++) {
				eachPin = (Pin) list.get(i);
				newPin = (Pin) newList.get(i);
				pinView = getViewObjectByPin(newPin, outBranches);
				if (pinView != null) {
					context.put(eachPin, pinView);
				}
			}
		}	
		
		list = action.getOutputControlPin();
		newList = ((Action)domainModel).getOutputControlPin();	
		LoggingUtil.trace(this, "createPin", "Get output control pin");			
		if (list.size() <= newList.size()) {
			for (int i=0; i<list.size(); i++) {
				eachPin = (Pin) list.get(i);
				newPin = (Pin) newList.get(i);
				pinView = getViewObjectByPin(newPin, outBranches);
				if (pinView != null) {
					context.put(eachPin, pinView);
				}
			}
		}
		
		updatePinName();
		LoggingUtil.traceExit(this, "createPin");
	}
	
	/**
	 * Create the pin set
	 */
	protected void createPinSet() {
		LoggingUtil.traceEntry(this, "createPinSet");
		// one inputSet, one outputSet
		// determine #out-branch by
		// inputControlPin = a, outputControlPin = ax
		// inputObjectPin = b, outputObjectPin = bx
		
		EList inputControlPinList = action.getInputControlPin();
		EList inputObjectPinList = action.getInputObjectPin();		
		EList outputControlPinList = action.getOutputControlPin();
		EList outputObjectPinList = action.getOutputObjectPin();
		
		int inputControlPinSize = inputControlPinList.size();		
		int inputObjectPinSize = inputObjectPinList.size();
		int outputControlPinSize = outputControlPinList.size();		
		int outputObjectPinSize = outputObjectPinList.size();		
		int outBranch = 0;
		
		
		if (inputObjectPinSize != 0) {
			outBranch = outputObjectPinSize / inputObjectPinSize;
		} else if (inputControlPinSize != 0) {
			outBranch = outputControlPinSize / inputControlPinSize;			
		}			
		if (outBranch > DEFAULT_BRANCH) {	
			NavigationProjectNode projectNode = NavigatorUtil.getProjectNode(getProjectName());
			if (projectNode == null) {
				LoggingUtil.traceExit(this, "createPinSet");											
				return;
			}
			PeCmdFactory cmdFactory = getPeCmdFactory();			
			AddOutBranchToControlActionPeCmd cmd = null; 
			for (int i=0; i< outBranch - DEFAULT_BRANCH; i++) {
				cmd = cmdFactory.buildAddOutBranchToControlActionPeCmd(viewModel);
				if (cmd.canExecute()) {
					try {
						cmd.execute();
					} catch (RuntimeException e) {
						logWarning(getImportSession(), 
							IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
							IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
							break;
					}
				} else {
					logWarning(getImportSession(), 
						IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
						IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);	
					break;			
				}		
			}
		}		
		
		//TN - March 10 
		//updatePinName();
		// end of TN - March 10
		
		LoggingUtil.traceExit(this, "createPinSet");		
	}	
}
