package com.ibm.btools.blm.ie.imprt.rule.processModel.node;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.pe.pin.convert.ConvertToInputValuePinPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.pin.convert.ConvertToRetrieveArtifactPinPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.pin.update.UpdateObjectPinPeCmd;
import com.ibm.btools.blm.compoundcommand.process.util.PEDomainViewObjectHelper;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.rule.informationModel.UpdateValueSpecificationRule;
import com.ibm.btools.blm.ie.imprt.rule.processModel.UpdatePinRule;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.bom.command.artifacts.AddInstanceValueToInputValuePinBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralNullToInputValuePinBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralStringToInputValuePinBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddOpaqueExpressionToInputValuePinBOMCmd;
import com.ibm.btools.bom.command.processes.activities.UpdateRetrieveArtifactPinBOMCmd;
import com.ibm.btools.bom.model.artifacts.InstanceSpecification;
import com.ibm.btools.bom.model.artifacts.InstanceValue;
import com.ibm.btools.bom.model.artifacts.LiteralNull;
import com.ibm.btools.bom.model.artifacts.LiteralSpecification;
import com.ibm.btools.bom.model.artifacts.OpaqueExpression;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.InputValuePin;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.RetrieveArtifactPin;
import com.ibm.btools.expression.bom.command.AddStructuredOpaqueExpressionToInputValuePinBEXCmd;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.model.modelmanager.util.AddUpdateObjectCommand;


/**
 * Update the Merge
 */
public class UpdateMergeRule extends UpdateMultiplePinSetControlActionRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	

	/**
	 * Create pins
	 */	
	protected void createPin() {
		LoggingUtil.traceEntry(this, "createPin");
		EObject outBranch = getOutBranchFromViewAction();

		Pin eachPin = null;
		Pin newPin = null;
		UpdatePinRule outputObjectPinRule = null;
		UpdatePinRule inputControlPinRule = null;
		UpdatePinRule outputControlPinRule = null;
		
		Iterator pinIterator = action.getOutputObjectPin().iterator();
		LoggingUtil.trace(this, "createPin", "Create output object pin");		
		while(pinIterator.hasNext()) {
			eachPin = (Pin) pinIterator.next();
			outputObjectPinRule = new UpdatePinRule();
			outputObjectPinRule.setImportSession(getImportSession());
			outputObjectPinRule.setProjectName(getProjectName());
			outputObjectPinRule.setSource(eachPin);
			outputObjectPinRule.setWorkingCopy(outBranch);
			outputObjectPinRule.setProcess(process);
			outputObjectPinRule.setAction(action);
			outputObjectPinRule.invoke(); 
		}	
	
		pinIterator = action.getOutputControlPin().iterator();
		LoggingUtil.trace(this, "createPin", "Create output control pin");		
		while(pinIterator.hasNext()) {
			eachPin = (Pin) pinIterator.next();
			outputControlPinRule = new UpdatePinRule();
			outputControlPinRule.setImportSession(getImportSession());
			outputControlPinRule.setProjectName(getProjectName());
			outputControlPinRule.setSource(eachPin);
			outputControlPinRule.setWorkingCopy(outBranch);
			outputControlPinRule.setProcess(process);
			outputControlPinRule.setAction(action);
			outputControlPinRule.invoke(); 
		}
	
		// Put the output pins in the context
		List inBranches = PEDomainViewObjectHelper.getAllInBranch(viewModel);		
		
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
		
		EList list = action.getInputObjectPin();
		EList newList = ((Action)domainModel).getInputObjectPin();
				
		EObject pinView = null;		
		LoggingUtil.trace(this, "createPin", "Get output object pin");	
		if (list.size() == newList.size()) {
			for (int i=0; i<list.size(); i++) {
				eachPin = (Pin) list.get(i);
				newPin = (Pin) newList.get(i);
				pinView = getViewObjectByPin(newPin, inBranches);
				if (pinView != null) {
					//d97947 - jie: missing updates for isOrdered and isUnique.
					if(eachPin instanceof InputObjectPin){
						UpdateObjectPinPeCmd pinCmd = getPeCmdFactory().buildUpdateObjectPinPeCmd(pinView);
						pinCmd.setIsOrdered(((InputObjectPin)eachPin).getIsOrdered().booleanValue());
						pinCmd.setIsUnique(((InputObjectPin)eachPin).getIsUnique().booleanValue());
						if (pinCmd.canExecute()) {
							try {
								pinCmd.execute();
							} catch (RuntimeException e) {
								LoggingUtil.trace(this, "invoke", IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
							}
						} else {
							LoggingUtil.trace(this, "invoke", IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);				
						}
					} //end jie fix
					if (eachPin instanceof InputValuePin) {
						ConvertToInputValuePinPeCmd convertCmd = getPeCmdFactory().buildConvertToInputValuePinPeCmd(pinView);
						if (convertCmd.canExecute()) {
							convertCmd.execute();
							pinView = convertCmd.getNewViewModel();
							EObject pinDomain = PEDomainViewObjectHelper.getDomainObject(pinView);
							addAttributeValue(eachPin, (InputValuePin)pinDomain);
						}
						
					} else if (eachPin instanceof RetrieveArtifactPin) {
						ConvertToRetrieveArtifactPinPeCmd convertCmd = getPeCmdFactory().buildConvertToRetrieveArtifactPinPeCmd(pinView);
						if (convertCmd.canExecute()) {
							convertCmd.execute();
							pinView = convertCmd.getNewViewModel();
							EObject pinDomain = PEDomainViewObjectHelper.getDomainObject(pinView);
							//update the RetrivedArtifactPin based on the original pin
							UpdateRetrieveArtifactPinBOMCmd updatePinCmd = new UpdateRetrieveArtifactPinBOMCmd((RetrieveArtifactPin)pinDomain);
							updatePinCmd.setAtBeginning(((RetrieveArtifactPin)eachPin).getAtBeginning().booleanValue());
							updatePinCmd.setIsRemove(((RetrieveArtifactPin)eachPin).getIsRemove().booleanValue());
							if (updatePinCmd.canExecute()) {
								try {
									updatePinCmd.execute();
								} catch (RuntimeException e) {
									LoggingUtil.trace(this, "invoke", IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
								}
							} else {
								LoggingUtil.trace(this, "invoke", IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);				
							}
							
						}
			
					}
					context.put(eachPin, pinView);
				}
			}
		}	
		
		list = action.getInputControlPin();
		Iterator pinSetListIterator = null;
		EList pinSetList = action.getInputPinSet();
		InputPinSet pinSet = null;
		Object eachInBranch = null;	
		LoggingUtil.trace(this, "createPin", "Get output control pin");	
		for (int i=0; i<list.size(); i++) {
			eachPin = (Pin) list.get(i);
			pinSetListIterator = pinSetList.iterator();
			while (pinSetListIterator.hasNext()) {
				pinSet = (InputPinSet) pinSetListIterator.next();
				if (pinSet.getInputControlPin().contains(eachPin)) {
					eachInBranch = context.get(pinSet);
					if (eachInBranch != null) {
						inputControlPinRule = new UpdatePinRule();
						inputControlPinRule.setImportSession(getImportSession());
						inputControlPinRule.setProjectName(getProjectName());
						inputControlPinRule.setSource(eachPin);
						inputControlPinRule.setWorkingCopy(eachInBranch);
						inputControlPinRule.setProcess(process);
						inputControlPinRule.setAction(action);
						inputControlPinRule.invoke();
						break;						
					}
				}
			}
		}
		
		updatePinName();
		
		LoggingUtil.traceExit(this, "createPin");		
	}	
	
	
	private void addAttributeValue(Pin pin, Pin theNewPin) {
		Iterator iterator = ((InputValuePin)pin).getValue().iterator();
		ValueSpecification value = null;
		AddUpdateObjectCommand cmd = null;
		InstanceSpecification spec = null;
		while(iterator.hasNext()) {
			value = (ValueSpecification) iterator.next();	
			if (value instanceof LiteralNull) {
				cmd = new AddLiteralNullToInputValuePinBOMCmd((InputValuePin)theNewPin);			
			} else if (value instanceof LiteralSpecification) {
				cmd = new AddLiteralStringToInputValuePinBOMCmd((InputValuePin)theNewPin);
			} else if (value instanceof StructuredOpaqueExpression){
				cmd = new AddStructuredOpaqueExpressionToInputValuePinBEXCmd((InputValuePin)theNewPin);				
			} else if (value instanceof OpaqueExpression) {
				// [Expression] Attrbibute constant valueb
				cmd = new AddOpaqueExpressionToInputValuePinBOMCmd((InputValuePin)theNewPin);
			} else if (value instanceof InstanceValue) {
				cmd = new AddInstanceValueToInputValuePinBOMCmd((InputValuePin)theNewPin);
			
			}
			
			if (cmd.canExecute()) {
				try {
					cmd.execute();
				} catch (RuntimeException e) {
					LoggingUtil.trace(this, "invoke",
						IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
				}
			} else {
				LoggingUtil.trace(this, "invoke", 
					IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);				
			}
			
			// After creation
			Object newDomainModel = cmd.getObject();
			if (newDomainModel != null) {
				UpdateValueSpecificationRule rule = new UpdateValueSpecificationRule();
				rule.setSource(value);
				rule.setWorkingCopy(newDomainModel);
				rule.setProjectName(getProjectName());
				rule.setImportSession(getImportSession());
				rule.invoke();
			}						
		}
	}
}
