package com.ibm.btools.blm.ie.imprt.rule.processModel.node;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.HashMap;

import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.bom.command.processes.actions.UpdateObserverActionBOMCmd;
import com.ibm.btools.bom.model.processes.actions.ObserverAction;
import com.ibm.btools.expression.bom.command.UpdateStructuredOpaqueExpressionBEXCmd;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;

/**
 * Update the ObserverAction
 */
public class UpdateObserverActionRule extends UpdateObservationActionRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	/**
	 * Update Model
	 */
	protected void updateModel() {		
		LoggingUtil.traceEntry(this, "updateModel");
		super.updateModel();

		// Attribute isRepeating
		if (action instanceof ObserverAction && domainModel instanceof ObserverAction) {
			
			UpdateObserverActionBOMCmd cmd = new UpdateObserverActionBOMCmd((ObserverAction)domainModel);
			cmd.setIsRepeating(((ObserverAction)action).getIsRepeating().booleanValue());
			if (cmd.canExecute()) {
				try {
					cmd.execute();
				} catch (RuntimeException e) {
					logWarning(getImportSession(), 
						IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
						IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);								
				}
			} else {
				logWarning(getImportSession(), 
					IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
					IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);			
			}
			
			// For V512 - only description of the expression is supported.
			// In V512, the actual expression will not be mapped and supported by xml export/import and the import engine.
			
			ObserverAction observerAction = (ObserverAction)action;
			ObserverAction domainModelObserver = (ObserverAction)domainModel; 
			if (observerAction.getObservationExpression() != null) {
				UpdateStructuredOpaqueExpressionBEXCmd addExprCmd = new UpdateStructuredOpaqueExpressionBEXCmd((StructuredOpaqueExpression)domainModelObserver.getObservationExpression());
				addExprCmd.setName(observerAction.getObservationExpression().getName());
				addExprCmd.setDescription(observerAction.getObservationExpression().getDescription());
				addExprCmd.setBody(observerAction.getObservationExpression().getBody());
				addExprCmd.setLanguage(observerAction.getObservationExpression().getLanguage());				
				if (addExprCmd.canExecute()) {
					try {
						addExprCmd.execute();
					} catch (RuntimeException e) {
						logWarning(getImportSession(), 
							IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
							IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);								
					}					
				}
			}
			
			// 20050413 - create the UID hashmap for ADF Import
			if (observerAction.getUid() != null) {
				HashMap processHashMap = BOMUtil.getUIDMap(getImportSession().getContext());
				// track the source process elements UID as 'key' and its corresponding working copy process element UID as 'value'
				processHashMap.put(observerAction.getUid(), domainModelObserver.getUid());
			}
			// end of 20050413 - create the UID hashmap for ADF Import		
		}
		
		// [Expression] Attribute filter expression
		// Attribute signalToOutputMap - not handled
					
		LoggingUtil.traceExit(this, "updateModel");
	}		
}