package com.ibm.btools.blm.ie.imprt.rule.resourceModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;

import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.ie.imprt.rule.informationModel.UpdateClassifierRule;
import com.ibm.btools.blm.ie.imprt.rule.informationModel.UpdatePropertyRule;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.bom.command.artifacts.AddPropertyToIndividualResourceTypeBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdatePropertyBOMCmd;
import com.ibm.btools.bom.model.artifacts.Property;
import com.ibm.btools.bom.model.resources.IndividualResourceType;


public class UpdateIndividualResourceTypeRule extends UpdateClassifierRule {
	
	//copyright
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	/**
	 * Invokes the rule.
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");
		super.invoke();
		createResourceProperties();
		LoggingUtil.traceExit(this, "invoke");
	}
	
	/**
	 * Creates and updates the properties of this Bulk Resource Type. Will delegate
	 * to the UpdatePropertyRule. 
	 *
	 */
	private void createResourceProperties() {
		LoggingUtil.traceEntry(this, "createResourceProperties");
		
		EList attributes =((IndividualResourceType) classifier).getOwnedAttribute();
		Iterator iterator = attributes.iterator();
		Property property = null;
		Property workingProperty = null;
		AddPropertyToIndividualResourceTypeBOMCmd addCmd = null;	
		UpdatePropertyBOMCmd updateCmd = null;
		while(iterator.hasNext()) {
			property = (Property) iterator.next();
			addCmd = new AddPropertyToIndividualResourceTypeBOMCmd((IndividualResourceType)workingCopy);
			if (addCmd.canExecute()) {
				try{
					addCmd.execute();
				}catch(RuntimeException ex){
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.RESOURCE_PROPERTY_EXCEPTION, new String[]{classifier.getName()}, ex);
				}
			} else {
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.RESOURCE_PROPERTY_EXCEPTION, new String[]{classifier.getName()}, null);
				continue;
			}
			// newly created Property
			workingProperty = (Property)addCmd.getObject();
			// Call the update property rule to update the property
			UpdatePropertyRule rule = new UpdatePropertyRule();
			rule.setImportSession(getImportSession());
			rule.setProjectName(getProjectName());
			rule.setSource(property);
			rule.setWorkingCopy(workingProperty);
			rule.invoke();
		}
		LoggingUtil.traceExit(this, "createResourceProperties");
	}
	

}
