package com.ibm.btools.blm.ie.imprt.rule.resourceModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildContainerNode;
import com.ibm.btools.blm.ui.navigation.model.AbstractLibraryChildNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationCalendarNode;
import com.ibm.btools.bom.command.artifacts.AddCommentToElementBOMCmd;
import com.ibm.btools.bom.command.compound.CreateTimeIntervalsBOMCmd;
import com.ibm.btools.bom.command.compound.OpenRootObjectForUpdateBOMCmd;
import com.ibm.btools.bom.command.time.AddAnchorPointToRecurringTimeIntervalsBOMCmd;
import com.ibm.btools.bom.command.time.AddOffsetDurationToTimeIntervalBOMCmd;
import com.ibm.btools.bom.command.time.AddOffsetWeekDayToTimeIntervalBOMCmd;
import com.ibm.btools.bom.command.time.AddRecurrencePeriodToRecurringTimeIntervalsBOMCmd;
import com.ibm.btools.bom.command.time.AddTimeIntervalToRecurringTimeIntervalsBOMCmd;
import com.ibm.btools.bom.command.time.AddUpdateOffsetIntoRecurrencePeriodBOMCmd;
import com.ibm.btools.bom.command.time.RemoveRecurrencePeriodBOMCmd;
import com.ibm.btools.bom.command.time.RemoveTimeIntervalBOMCmd;
import com.ibm.btools.bom.command.time.UpdateAnchorPointBOMCmd;
import com.ibm.btools.bom.command.time.UpdateRecurringTimeIntervalsBOMCmd;
import com.ibm.btools.bom.command.time.UpdateTimeIntervalsBOMCmd;
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.Element;
import com.ibm.btools.bom.model.time.AnchorPoint;
import com.ibm.btools.bom.model.time.OffsetDuration;
import com.ibm.btools.bom.model.time.OffsetIntoRecurrencePeriod;
import com.ibm.btools.bom.model.time.OffsetWeekDay;
import com.ibm.btools.bom.model.time.RecurrencePeriod;
import com.ibm.btools.bom.model.time.RecurringTimeIntervals;
import com.ibm.btools.bom.model.time.TimeInterval;
import com.ibm.btools.bom.model.time.TimeIntervals;
import com.ibm.btools.emf.cf.BtCompoundCommand;
import com.ibm.btools.model.filemanager.FileMGR;
import com.ibm.btools.model.modelmanager.CloseWorkingSetCmd;
import com.ibm.btools.model.modelmanager.SaveWorkingSetCmd;


public class UpdateRecurringTimeIntervalsRule extends AbstractImportRule {
	//copyright
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	protected RecurringTimeIntervals recurringInterval;
	protected RecurringTimeIntervals workingCopy;
	
	
	/**
	 * Set Original time recurring interval
	 * @param businessItem
	 */
	public void setSource(final Object classifier) {
		this.recurringInterval = (RecurringTimeIntervals) classifier;
	}
	
	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = (RecurringTimeIntervals) workingCopy;
	}


	/**
	 * Invokes the rule.
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");
		UpdateRecurringTimeIntervalsBOMCmd updateCmd = new UpdateRecurringTimeIntervalsBOMCmd(workingCopy);
		//update the name only if not null, else do not overwrite the command  generated default one
		
		// TN - experiment - comment it out to allow the timetable can be renamed when there is a name collision
		//if(recurringInterval.getName() != null){
		//	updateCmd.setName(recurringInterval.getName());
		//}
		// end of TN - experiment - comment it out to allow the timetable can be renamed when there is a name collision		
		
		if(recurringInterval.getNumberOfRecurrences() != null){
			updateCmd.setNumberOfRecurrences(recurringInterval.getNumberOfRecurrences());
		}else{
			updateCmd.setNumberOfRecurrences("*");
		}
		if(recurringInterval.getTimeZone() != null){
			updateCmd.setTimeZone(recurringInterval.getTimeZone());
		}else{
			updateCmd.setTimeZone("PT00H00M");
		}
		if(recurringInterval.getExemptPeriod() != null && !recurringInterval.getExemptPeriod().isEmpty()){
			updateExemptPeriods();
		}
		if(!updateCmd.canExecute()){
			LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CALENDAR_UPDATE_EXCEPTION, new String[]{recurringInterval.getName()}, null);
		}else{
			try{
				updateCmd.execute();
			}catch(RuntimeException ex){
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CALENDAR_UPDATE_EXCEPTION, new String[]{recurringInterval.getName()}, ex);
			}
		}
		if(recurringInterval.getValidityPeriod() != null && !recurringInterval.getValidityPeriod().isEmpty()){
			LoggingUtil.logWarning(getImportSession(), IeMessageKeys.UNSUPPORTED_RECURRING_INTERVAL_VALIDITY_PERIOD, new String[]{recurringInterval.getName()}, null);
		}
		updateRecurrencePeriod();
		updateAnchorPoint();
		updateIntervals();
		LoggingUtil.traceExit(this, "invoke");
	}
	/**
	 * Updates the <code>workingCopy</code> exemptPeriods as per the source <code>recurringInterval</code>
	 * NOTE: will follow the tool mechanism of managing exemptPeriods TimeIntervals. All the exempt periods
	 * of a RecurringPeriod are wrapped in a TimeIntervals object (top level BOM Packageable element)
	 * of the ResourceModel that is not shareable, as it is not visible in the BOM navigator.
	 * There is only one exemptPeriod TimeInterval for a RecurringInterval. 
	 * The name of the TimeIntervals is unique as build from the name of the recurringTimeInterval.
	 * The management of this TimeIntervals is linked with the RecurringTimeIntervals. 
	 * Upon the deletion of the RecurringTimeInterval, the TimeIntervals corresponding to the
	 * RecurringTimeInterval should be also deleted.
	 * NOTE: Although this code will create top level BOM Packageable elements (TimeIntervals)
	 * it will not handle name colision for the above mentioned reason.  
	 * TODO check the method commands and errors
	 */
	private void updateExemptPeriods(){
		LoggingUtil.traceEntry(this, "updateExemptPeriods");
		List exemptPeriods = recurringInterval.getExemptPeriod();
		Iterator it = exemptPeriods.iterator();
		//create one TimeIntervals that will hold all the RecurringTimeIntervals in all exemptPeriods TimeIntervals. 
		BtCompoundCommand wrapperCmd = new BtCompoundCommand();
		AbstractChildContainerNode resCatalogNode = (AbstractChildContainerNode)getImportSession().getContext().get(recurringInterval.getOwningPackage());
		if(resCatalogNode == null){
			//can not find the resource model, can not create the TimeIntervals
			LoggingUtil.logWarning(getImportSession(), IeMessageKeys.OBJECT_REFERENCE_NOT_FOUND_EXCEPTION, new String[]{"Resource Catalog", recurringInterval.getOwningPackage().getName()}, null);
			//can not create the TimeIntervals that will contain all the RecurringTimeInterval ExemptPeriods, return 
			LoggingUtil.logWarning(getImportSession(), IeMessageKeys.EXEMPT_PERIOD_TIME_INTERVALS_CREATION_EXCEPTION, new String[]{recurringInterval.getName()}, null, "Can not retrieve the resourceCatalog");
			LoggingUtil.traceExit(this, "updateExemptPeriods");
			return;
		}
		String intervalsURI = createTimeIntervals(resCatalogNode, wrapperCmd);
		if(intervalsURI == null){
			//could not create the TimeIntervals, return.
			LoggingUtil.logWarning(getImportSession(), IeMessageKeys.EXEMPT_PERIOD_TIME_INTERVALS_CREATION_EXCEPTION, new String[]{recurringInterval.getName()}, null, "IntervalsURI is null");
			LoggingUtil.traceExit(this, "updateExemptPeriods");
			return;
		}
		OpenRootObjectForUpdateBOMCmd openCmd = new OpenRootObjectForUpdateBOMCmd();
		openCmd.setProjectName(getProjectName());
		openCmd.setROBLM_URI(intervalsURI);
		if(!wrapperCmd.appendAndExecute(openCmd)){
			wrapperCmd.undo();
		}else{
			// need to get copy id for saving
			String copyID = openCmd.getCopyID();
			TimeIntervals workingTimeIntervals = (TimeIntervals)openCmd.getROCopy();
			//now update the TimeIntervals with the exemptPeriods 
			TimeIntervals timeIntervals = null;
			while(it.hasNext()){
				timeIntervals = (TimeIntervals)it.next();
				if(timeIntervals.getRecurringTimeIntervals()!= null && !timeIntervals.getRecurringTimeIntervals().isEmpty()){
					Iterator it1 = timeIntervals.getRecurringTimeIntervals().iterator();
					UpdateTimeIntervalsBOMCmd updateIntervalsCmd = new UpdateTimeIntervalsBOMCmd(workingTimeIntervals);
					List tempExempt = new LinkedList();
					RecurringTimeIntervals recInt = null;
					while(it1.hasNext()){
						recInt = (RecurringTimeIntervals)it1.next();
						RecurringTimeIntervals workingRecInterval =(RecurringTimeIntervals)BOMUtil.loadRootObject((AbstractLibraryChildNode) getImportSession().getContext().get(recInt));
						if(!tempExempt.contains(workingRecInterval)){
							updateIntervalsCmd.addRecurringTimeIntervals(workingRecInterval);
							tempExempt.add(workingRecInterval);
						}
					}
					wrapperCmd.appendAndExecute(updateIntervalsCmd);
				}
			}
			
			SaveWorkingSetCmd saveCmd = new SaveWorkingSetCmd();
			saveCmd.setProjectName(getProjectName());
			String projectPath = FileMGR.getProjectPath(getProjectName());
			saveCmd.setProjectPath(projectPath);
			saveCmd.setWorkingSetID(copyID);
			if(!wrapperCmd.appendAndExecute(saveCmd)){
				wrapperCmd.undo();
			}else{
				TimeIntervals masterIntervals = (TimeIntervals)BOMUtil.loadRootObject(intervalsURI, getProjectName());
				UpdateRecurringTimeIntervalsBOMCmd updateRecIntCmd = new UpdateRecurringTimeIntervalsBOMCmd(workingCopy);
				updateRecIntCmd.addExemptPeriod(masterIntervals);
				wrapperCmd.appendAndExecute(updateRecIntCmd);
			}
			
			closeWorkingCopy(copyID);

		}
		LoggingUtil.traceExit(this, "updateExemptPeriods");
	}
	
	/**
	 * Close the workingcopy by id
	 * @param copyID
	 */
	protected void closeWorkingCopy(final String copyID) {
		LoggingUtil.traceEntry(this, "closeWorkingCopy");		
		if (copyID == null) {
			LoggingUtil.trace(this, "closeWorkingCopy", "The copyID should not be null.");
			LoggingUtil.traceExit(this, "closeWorkingCopy");			
			return;
		}
		CloseWorkingSetCmd closeCmd = new CloseWorkingSetCmd();
		closeCmd.setWorkingSetID(copyID);
		if(closeCmd.canExecute()) {
			try {			
				closeCmd.execute();
			} catch (RuntimeException e) {
				LoggingUtil.trace(this, "closeWorkingCopy", "The close command throws a RuntimeException.");
				LoggingUtil.traceExit(this, "closeWorkingCopy");
				return;
			}
		} else {
			LoggingUtil.trace(this, "closeWorkingCopy", "The close command cannot be executed.");
			LoggingUtil.traceExit(this, "closeWorkingCopy");
			return;
		}
		LoggingUtil.traceExit(this, "closeWorkingCopy");		
	}

	/**
	 * Creates a hidden (tool hidden) TimeInterval that will contain all exemptPeriods RecurringTimeIntervals.
	 * The name of the TimeIntervals is created from the name of the RecurringTimeInterval and should be unique.
	 * Life cycle of this object is managed by the RecurringTimeInterval that reference it.
	 * @param node
	 * @param wrapperCmd
	 * @return the TimeIntervals root object BML URI. 
	 */
	private String createTimeIntervals(AbstractChildContainerNode node, BtCompoundCommand wrapperCmd) {
		LoggingUtil.traceEntry(this, "createTimeIntervals");
		CreateTimeIntervalsBOMCmd createCmd = new CreateTimeIntervalsBOMCmd();
		createCmd.setParentModelBLM_URI((String)node.getEntityReference());
		createCmd.setProjectName(node.getProjectNode().getLabel());
		//create an unique name for the exempt period.. unique exempt period/recurringInterval...
		//TODO !!!!!!! add precondition for interval name and not execute the command otherwise !!!!!
		//defect89894:use node label/id instead of model name
		NavigationCalendarNode obj = (NavigationCalendarNode)getImportSession().getContext().get(recurringInterval);
		//createCmd.setName("$" + recurringInterval.getName() + "_exemptPeriods");
		createCmd.setName("$" + obj.getId() + "_exemptPeriods");
		if(!wrapperCmd.appendAndExecute(createCmd)){
			wrapperCmd.undo();
		}
		
		LoggingUtil.traceExit(this, "createTimeIntervals");
		return createCmd.getROBLM_URI();
	}

	/**
	 * Updates the target <code>workingCopy</code> with the recurrencePeriod as per the
	 * source <code>recurringInterval</code>
	 */	
	protected void updateRecurrencePeriod(){
		if(recurringInterval.getRecurrencePeriod()!= null){
			
			if(workingCopy.getRecurrencePeriod() != null){
				//d90794 - need to remove the default created recurrencePeriod 
				//TODO: change message in next version indicating the remove of default recurrencePeriod failed.
				RemoveRecurrencePeriodBOMCmd removeRecPeriodCmd = new RemoveRecurrencePeriodBOMCmd(workingCopy.getRecurrencePeriod());
				if(!removeRecPeriodCmd.canExecute()){
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CALENDAR_UPDATE_EXCEPTION, new String[]{recurringInterval.getName()}, null);
				}else{
					try{
						removeRecPeriodCmd.execute();
					}catch(RuntimeException ex){
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CALENDAR_UPDATE_EXCEPTION, new String[]{recurringInterval.getName()}, ex);	
					}
				}
			}
			RecurrencePeriod recurrencePeriod = recurringInterval.getRecurrencePeriod();
			AddRecurrencePeriodToRecurringTimeIntervalsBOMCmd addRecPeriodCmd = new AddRecurrencePeriodToRecurringTimeIntervalsBOMCmd(workingCopy);
			if(recurrencePeriod.getIsCommensurable() != null){
				addRecPeriodCmd.setIsCommensurable(recurrencePeriod.getIsCommensurable().booleanValue());
			}else{
				addRecPeriodCmd.setIsCommensurable(true);
			}
			if(recurrencePeriod.getDuration() != null){
				addRecPeriodCmd.setDuration(recurrencePeriod.getDuration());
			}
			if(!addRecPeriodCmd.canExecute()){
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CALENDAR_UPDATE_EXCEPTION, new String[]{recurringInterval.getName()}, null);
			}else{
				try{
					addRecPeriodCmd.execute();
				}catch(RuntimeException ex){
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CALENDAR_UPDATE_EXCEPTION, new String[]{recurringInterval.getName()}, ex);	
				}
			}
		}
	}

	/**
	 * Updates the target <code>workingCopy</code> with the anchor point as per the
	 * source <code>recurringInterval</code>
	 */	
	protected void updateAnchorPoint(){
		if(recurringInterval.getAnchorPoint() == null){
			return;
		}
		
		if(workingCopy.getAnchorPoint() == null){
			AnchorPoint anchorPoint = recurringInterval.getAnchorPoint();
			AddAnchorPointToRecurringTimeIntervalsBOMCmd addAnchorCmd = new AddAnchorPointToRecurringTimeIntervalsBOMCmd(workingCopy);
			addAnchorCmd.setPointInTime(anchorPoint.getPointInTime());
			if(!addAnchorCmd.canExecute()){
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CALENDAR_UPDATE_EXCEPTION, new String[]{recurringInterval.getName()}, null);
			}else{
				try{
					addAnchorCmd.execute();
				}catch(RuntimeException ex){
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CALENDAR_UPDATE_EXCEPTION, new String[]{recurringInterval.getName()}, ex);	
				}
			}
		} else {			
			UpdateAnchorPointBOMCmd updateCmd = new UpdateAnchorPointBOMCmd(workingCopy.getAnchorPoint());
			AnchorPoint anchorPoint = recurringInterval.getAnchorPoint();
			updateCmd.setPointInTime(anchorPoint.getPointInTime());
			if(!updateCmd.canExecute()){
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CALENDAR_UPDATE_EXCEPTION, new String[]{recurringInterval.getName()}, null);
			}else{
				try{
					updateCmd.execute();
				}catch(RuntimeException ex){
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CALENDAR_UPDATE_EXCEPTION, new String[]{recurringInterval.getName()}, ex);	
				}
			}
		}
	}
	/**
	 * Updates the target <code>workingCopy</code> with the interval as per the
	 * source <code>recurringInterval</code>
	 */	

	protected void updateIntervals(){

		EList recurringIntervals = recurringInterval.getInterval();
		// Ray - should remove the default created time interval in the working copy in any case
		// d146358
		//if(!recurringIntervals.isEmpty()){
			if(!workingCopy.getInterval().isEmpty()){
				//d91125 - need to remove the default created time interval 
				//TODO: change message in next version indicating the remove of default time interval failed.
				RemoveTimeIntervalBOMCmd removeTimeIntervalCmd = new RemoveTimeIntervalBOMCmd((TimeInterval)workingCopy.getInterval().get(0));
				if(!removeTimeIntervalCmd.canExecute()){
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CALENDAR_UPDATE_EXCEPTION, new String[]{recurringInterval.getName()}, null);
				}else{
					try{
						removeTimeIntervalCmd.execute();
					}catch(RuntimeException ex){
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CALENDAR_UPDATE_EXCEPTION, new String[]{recurringInterval.getName()}, ex);	
					}
				}
			}
		//}
		
		Iterator it = recurringIntervals.iterator();
		AddTimeIntervalToRecurringTimeIntervalsBOMCmd addTimeIntervalCmd = null;
		TimeInterval workingTimeInterval = null;
		TimeInterval timeInterval = null;
		while(it.hasNext()){
			timeInterval = (TimeInterval)it.next();
			addTimeIntervalCmd = new AddTimeIntervalToRecurringTimeIntervalsBOMCmd(workingCopy);
				
			if(timeInterval.getDuration()!= null){
				addTimeIntervalCmd.setDuration(timeInterval.getDuration());
			}
			if(!addTimeIntervalCmd.canExecute()){
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CALENDAR_UPDATE_EXCEPTION, new String[]{recurringInterval.getName()}, null);
			}else{
				try{
					addTimeIntervalCmd.execute();
					workingTimeInterval = (TimeInterval)addTimeIntervalCmd.getObject();
					updateComments(timeInterval, workingTimeInterval);
					if(timeInterval.getOffset() != null && !timeInterval.getOffset().isEmpty()){
						EList offsets = timeInterval.getOffset();
						Iterator offsetIt = offsets.iterator();
						OffsetIntoRecurrencePeriod offset = null;
						AddUpdateOffsetIntoRecurrencePeriodBOMCmd addOffestCmd = null;
						while(offsetIt.hasNext()){
							offset =(OffsetIntoRecurrencePeriod)offsetIt.next();
							if(offset instanceof OffsetDuration){
								addOffestCmd = new AddOffsetDurationToTimeIntervalBOMCmd(workingTimeInterval);
								((AddOffsetDurationToTimeIntervalBOMCmd)addOffestCmd).setDuration(((OffsetDuration)offset).getDuration());
									
							}else if(offset instanceof OffsetWeekDay){
								addOffestCmd = new AddOffsetWeekDayToTimeIntervalBOMCmd(workingTimeInterval);
								((AddOffsetWeekDayToTimeIntervalBOMCmd)addOffestCmd).setDay(((OffsetWeekDay)offset).getDay());
								if(((OffsetWeekDay)offset).getOrdinalNumber() != null){
									((AddOffsetWeekDayToTimeIntervalBOMCmd)addOffestCmd).setOrdinalNumber(((OffsetWeekDay)offset).getOrdinalNumber().intValue());
								}
								if(((OffsetWeekDay)offset).getOffsetTime() != null){
									((AddOffsetWeekDayToTimeIntervalBOMCmd)addOffestCmd).setOffsetTime(((OffsetWeekDay)offset).getOffsetTime());
								}
							}
							if(!addOffestCmd.canExecute()){
								LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CALENDAR_UPDATE_EXCEPTION, new String[]{recurringInterval.getName()}, null);
							}else{
								try{
									addOffestCmd.execute();
								}catch(RuntimeException ex){
									LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CALENDAR_UPDATE_EXCEPTION, new String[]{recurringInterval.getName()}, ex);
								}
							}
						}
					}
				}catch(RuntimeException ex){
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CALENDAR_UPDATE_EXCEPTION, new String[]{recurringInterval.getName()}, ex);
				}
			}
		}
	}
		
	/**
	 * Update the <code>workingElement</code> with owned comments as per the
	 * <code>element</code>
	 * @param element BOM Element 
	 * @param workingElement BOM Element for update
	 */	
	private void updateComments(Element element, Element workingElement){
		LoggingUtil.traceEntry(this, "updateComments");
		EList comments = element.getOwnedComment();
		if(comments != null && !comments.isEmpty()){
			Iterator it = comments.iterator();
			Comment comment = null;
			Comment workingComment = null;
			AddCommentToElementBOMCmd addCmd = null;
			while(it.hasNext()){
				comment = (Comment)it.next();
				addCmd = new AddCommentToElementBOMCmd(workingElement);
				addCmd.setBody(comment.getBody());
				if(!addCmd.canExecute()){
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_COMMENT_UPDATE_EXCEPTION);
				}else{
					try{
						addCmd.execute();
					}catch(RuntimeException ex){
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_COMMENT_UPDATE_EXCEPTION, null, ex);
					}
				}			}
		}
		LoggingUtil.traceExit(this, "updateComments");
	}	

}
