package com.ibm.btools.blm.ie.imprt.rule.resourceModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildContainerNode;
import com.ibm.btools.blm.ui.navigation.model.AbstractLibraryChildNode;
import com.ibm.btools.bom.command.artifacts.AddCommentToElementBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralRealToMonetaryValueBOMCmd;
import com.ibm.btools.bom.command.compound.CreateTimeIntervalsBOMCmd;
import com.ibm.btools.bom.command.compound.DeleteRootObjectBOMCmd;
import com.ibm.btools.bom.command.compound.OpenRootObjectForUpdateBOMCmd;
import com.ibm.btools.bom.command.resources.AddCostPerQuantityAndTimeUnitToRoleBOMCmd;
import com.ibm.btools.bom.command.resources.AddCostPerQuantityToRoleBOMCmd;
import com.ibm.btools.bom.command.resources.AddCostPerTimeUnitToRoleBOMCmd;
import com.ibm.btools.bom.command.resources.AddCostValueToTimeDependentCostBOMCmd;
import com.ibm.btools.bom.command.resources.AddMonetaryValueToCostValueBOMCmd;
import com.ibm.btools.bom.command.resources.AddOneTimeCostToRoleBOMCmd;
import com.ibm.btools.bom.command.resources.AddScopeDimensionToRoleBOMCmd;
import com.ibm.btools.bom.command.resources.AddUpdateTimeDependentCostBOMCmd;
import com.ibm.btools.bom.command.resources.UpdateCostValueBOMCmd;
import com.ibm.btools.bom.command.resources.UpdateRoleBOMCmd;
import com.ibm.btools.bom.command.time.UpdateTimeIntervalsBOMCmd;
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.LiteralReal;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.PrimitiveType;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.resources.CostPerQuantity;
import com.ibm.btools.bom.model.resources.CostPerQuantityAndTimeUnit;
import com.ibm.btools.bom.model.resources.CostPerTimeUnit;
import com.ibm.btools.bom.model.resources.CostValue;
import com.ibm.btools.bom.model.resources.MonetaryValue;
import com.ibm.btools.bom.model.resources.OneTimeCost;
import com.ibm.btools.bom.model.resources.Role;
import com.ibm.btools.bom.model.resources.ScopeDimension;
import com.ibm.btools.bom.model.resources.TimeDependentCost;
import com.ibm.btools.bom.model.time.RecurringTimeIntervals;
import com.ibm.btools.bom.model.time.TimeIntervals;
import com.ibm.btools.emf.cf.BtCompoundCommand;
import com.ibm.btools.model.filemanager.FileMGR;
import com.ibm.btools.model.modelmanager.CloseWorkingSetCmd;
import com.ibm.btools.model.modelmanager.SaveWorkingSetCmd;
import com.ibm.btools.model.modelmanager.copyregistry.CopyRegistry;
import com.ibm.btools.model.modelmanager.util.RemoveObjectCommand;
import com.ibm.btools.model.resourcemanager.ResourceMGR;
import com.ibm.btools.model.resourcemanager.util.PredefUtil;

public class UpdateRoleRule extends AbstractImportRule {
	
	//copyright
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private Role role;
	private Role workingCopy;

	/**
	 * Invokes the rule
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");
		
		if(NavigatorUtil.isNewNode(getImportSession(),role)){
			updateRoleProperties();
			updateAvailability();
			updateTimeDependentCost();
			
		}else{
			if(NavigatorUtil.isOptionOverwrite(getImportSession().getImportOptions(),role)){
				consolidateRole();
				updateRoleProperties();
				updateAvailability();
				updateTimeDependentCost();			
			}else if(NavigatorUtil.isOptionIgnore(getImportSession().getImportOptions(),role)){
				//no-op
			}
		}
		LoggingUtil.traceExit(this, "invoke");
	}
	
	/**
	 * Sets the original BOM Role.
	 * @param role a BOM Role
	 */
	public void setSource(final Object role) {
		this.role = (Role) role;
	}
	
	/**
	 * Sets the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = (Role) workingCopy;
	}
	
	/**
	 * Updates the BOM Role <code>workingCopy</code> with properties.
	 */	
	private void updateRoleProperties(){
		LoggingUtil.traceEntry(this, "updateRoleProperties");
//		TODO check what is supported for availability
		//Note: the following properties are updated for the role:
		// comments, scopeDimension with Provided/Required Type and comments.
		if(role.getScopeDimension() != null && !role.getScopeDimension().isEmpty()){
			Iterator it = role.getScopeDimension().iterator();
			ScopeDimension scopeDimension = null;
			AddScopeDimensionToRoleBOMCmd addScopeCmd = null;
			Type workingProvidedType = null;
			Type workingRequiredType = null;
			while(it.hasNext()){
				scopeDimension = (ScopeDimension)it.next();
				addScopeCmd = new AddScopeDimensionToRoleBOMCmd(workingCopy);
				addScopeCmd.setName((scopeDimension.getName()));
				
				if(scopeDimension.getProvidedType()!= null){
					workingProvidedType = (Type)BOMUtil.loadRootObject((AbstractLibraryChildNode)getImportSession().getContext().get(scopeDimension.getProvidedType()));
					if(workingProvidedType == null){
						// Ray - Cross Project Support - Get the Primitive Type from the predefined project
						if (scopeDimension.getProvidedType() instanceof PrimitiveType) {
							workingProvidedType = BOMUtil.getPrimitiveType(PredefUtil.getPredefinedProjectName(), scopeDimension.getProvidedType().getName());
						}
						if(workingProvidedType == null){
							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.OBJECT_REFERENCE_NOT_FOUND_EXCEPTION, new String[]{"Type", scopeDimension.getProvidedType().getName()}, null);
						}
					}
						addScopeCmd.setProvidedType(workingProvidedType);
									
				}
				 if(scopeDimension.getRequiredType() != null){
					workingRequiredType = (Type)BOMUtil.loadRootObject((AbstractLibraryChildNode)getImportSession().getContext().get(scopeDimension.getRequiredType()));
					if(workingRequiredType == null){
						// Ray - Cross Project Support - Get the Primitive Type from the predefined project
						if (scopeDimension.getRequiredType() instanceof PrimitiveType) {
							workingRequiredType = BOMUtil.getPrimitiveType(PredefUtil.getPredefinedProjectName(), scopeDimension.getRequiredType().getName());
						}
						if(workingRequiredType == null){
							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.OBJECT_REFERENCE_NOT_FOUND_EXCEPTION, new String[]{"Type", scopeDimension.getRequiredType().getName()}, null);
						}
					}
					addScopeCmd.setRequiredType(workingRequiredType);
				 }
				 
				if(!addScopeCmd.canExecute()){
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.ROLE_SCOPE_DIMENSION_CREATE_EXCEPTION, new String[]{role.getName(), scopeDimension.getName()}, null);
				}else{
					try{
						addScopeCmd.execute();
						ScopeDimension workingScopeDimension = (ScopeDimension)addScopeCmd.getObject();
						updateComments(scopeDimension, workingScopeDimension);
						getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_SCOPE_DIMNENSION).put(scopeDimension, workingScopeDimension);
					}catch(RuntimeException ex){
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.ROLE_SCOPE_DIMENSION_CREATE_EXCEPTION, new String[]{role.getName(), scopeDimension.getName()}, ex);
					}
				}
			}
		}
		LoggingUtil.traceExit(this, "updateRoleProperties");
	}

	/**
	 * Updates the BOM Role <code>workingCopy</code> with properties.
	 */	
	private void updateTimeDependentCost(){
		LoggingUtil.traceEntry(this, "updateTimeDependentCost");
		List costProfiles = consolidateCostProfileList(role);
		if(costProfiles != null && !costProfiles.isEmpty()){
			Iterator it = costProfiles.iterator();
			TimeDependentCost timeCost = null;
			TimeDependentCost workingTimeCost = null;
			AddUpdateTimeDependentCostBOMCmd addTimeCmd = null;
			while(it.hasNext()){
				timeCost = (TimeDependentCost)it.next();
				if (timeCost instanceof CostPerQuantityAndTimeUnit){
					addTimeCmd = new AddCostPerQuantityAndTimeUnitToRoleBOMCmd(workingCopy);
					((AddCostPerQuantityAndTimeUnitToRoleBOMCmd)addTimeCmd).setUnitOfMeasure(((CostPerQuantityAndTimeUnit)timeCost).getUnitOfMeasure());
					((AddCostPerQuantityAndTimeUnitToRoleBOMCmd)addTimeCmd).setTimeUnit(((CostPerQuantityAndTimeUnit)timeCost).getTimeUnit());
				} else if(timeCost instanceof CostPerQuantity){
					addTimeCmd = new AddCostPerQuantityToRoleBOMCmd(workingCopy);
					((AddCostPerQuantityToRoleBOMCmd)addTimeCmd).setUnitOfMeasure(((CostPerQuantity)timeCost).getUnitOfMeasure());
				} else if(timeCost instanceof CostPerTimeUnit){
					addTimeCmd = new AddCostPerTimeUnitToRoleBOMCmd(workingCopy);
					((AddCostPerTimeUnitToRoleBOMCmd)addTimeCmd).setTimeUnit(((CostPerTimeUnit)timeCost).getTimeUnit());
				} else if (timeCost instanceof OneTimeCost){
					addTimeCmd = new AddOneTimeCostToRoleBOMCmd(workingCopy);
				}
								
				addTimeCmd.setName(timeCost.getName());
				if(addTimeCmd != null && !addTimeCmd.canExecute()){
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.RESOURCE_COST_CREATE_EXCEPTION, new String[]{role.getName()}, null);
				}else{
					try{
						addTimeCmd.execute();
						workingTimeCost = (TimeDependentCost)addTimeCmd.getObject();
						List costValues = timeCost.getCostValue();
						if(costValues != null && ! costValues.isEmpty()){
							CostValue costValue = null;
							CostValue workingCostValue = null;
							AddCostValueToTimeDependentCostBOMCmd addCostValueCmd = null;
							Iterator costIt = costValues.iterator();
							while(costIt.hasNext()){
								costValue = (CostValue)costIt.next();
								addCostValueCmd = new AddCostValueToTimeDependentCostBOMCmd(workingTimeCost);
								if(!addCostValueCmd.canExecute()){
									LoggingUtil.logWarning(getImportSession(), IeMessageKeys.RESOURCE_COST_CREATE_EXCEPTION, new String[]{role.getName()}, null);
								}else{
									try{								
										addCostValueCmd.execute();
										workingCostValue = (CostValue)addCostValueCmd.getObject();
										if(costValue.getAmount()!= null){
											MonetaryValue amount = costValue.getAmount();
											AddMonetaryValueToCostValueBOMCmd addMonetaryCmd = new AddMonetaryValueToCostValueBOMCmd(workingCostValue);
											addMonetaryCmd.setCurrency(amount.getCurrency());
											if(!addMonetaryCmd.canExecute()){
												LoggingUtil.logWarning(getImportSession(), IeMessageKeys.RESOURCE_COST_CREATE_EXCEPTION, new String[]{role.getName()}, null);
											}else{
												try{
													addMonetaryCmd.execute();
													MonetaryValue workingMonetary = (MonetaryValue)addMonetaryCmd.getObject();
													if(amount.getValue() != null && amount.getValue() instanceof LiteralReal){
														AddLiteralRealToMonetaryValueBOMCmd addValueCmd = new AddLiteralRealToMonetaryValueBOMCmd(workingMonetary);
														addValueCmd.setValue(((LiteralReal)amount.getValue()).getValue());
														if(!addValueCmd.canExecute()){
															LoggingUtil.logWarning(getImportSession(), IeMessageKeys.RESOURCE_COST_CREATE_EXCEPTION, new String[]{role.getName()}, null);
														}else{
															try{
																addValueCmd.execute();
															}catch(RuntimeException ex){
																LoggingUtil.logWarning(getImportSession(), IeMessageKeys.RESOURCE_COST_CREATE_EXCEPTION, new String[]{role.getName()}, null);
															}
														}
													}
												}catch(RuntimeException ex){
													LoggingUtil.logWarning(getImportSession(), IeMessageKeys.RESOURCE_COST_CREATE_EXCEPTION, new String[]{role.getName()}, ex);
												}
											}
										}
										updateCostValueAvailability(role, costValue, workingCostValue);
									}catch(RuntimeException ex){
										LoggingUtil.logWarning(getImportSession(), IeMessageKeys.RESOURCE_COST_CREATE_EXCEPTION, new String[]{role.getName()}, ex);
									}
								}
							}
						}
						 UpdateRoleBOMCmd updateCmd = new UpdateRoleBOMCmd(workingCopy);
						 updateCmd.addCostProfile(workingTimeCost);
						 if(updateCmd.canExecute()){
								updateCmd.execute();
						 }
					}catch(RuntimeException ex){
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.RESOURCE_COST_CREATE_EXCEPTION, new String[]{role.getName()}, ex);
					}
				}
			}
		}
		LoggingUtil.traceExit(this, "updateTimeDependentCost");
	}

	
	/**
	 * Update the <code>workingElement</code> with owned comments as per the
	 * <code>element</code>
	 * @param element BOM Element 
	 * @param workingElement BOM Element for update
	 */	
	private void updateComments(NamedElement element, NamedElement workingElement){
		LoggingUtil.traceEntry(this, "updateComments");
		EList comments = element.getOwnedComment();
		if(comments != null && !comments.isEmpty()){
			Iterator it = comments.iterator();
			Comment comment = null;
			Comment workingComment = null;
			AddCommentToElementBOMCmd addCmd = null;
			while(it.hasNext()){
				comment = (Comment)it.next();
				addCmd = new AddCommentToElementBOMCmd(workingCopy);
				addCmd.setBody(comment.getBody());
				if(!addCmd.canExecute()){
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_COMMENT_CREATE_EXCEPTION, new String[]{element.getName()}, null);
				}else{
					try{
						addCmd.execute();
					}catch(RuntimeException ex){
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_COMMENT_CREATE_EXCEPTION, new String[]{element.getName()}, ex );
					}
				}
			}
		}
		LoggingUtil.traceExit(this, "updateComments");
	}
	
	/**
	 * Creates a list that will consolidate the a resource costProfile and a resource owned CostProfile into 
	 * a single list. 
	 * Note: tool limitation : the tool does not expose the reusable costProfile. In the tool, the user will always
	 * assign cost to a resource => create an ownedCostProfile (owned TimeDependedCost instance), while the Resource 
	 * costProfile association will point to the ownedCostProfile instance.
	 * On the import, if both costProfile and ownedCostProfile are specified, the list of timeDependentCosts will be 
	 * consolidated in an unique list.
	 * @param resource a BOM Resource
	 * @return list - a List containing all the <code>resource</code> cost profiles.
	 */
	private List consolidateCostProfileList(Role role){
		
		List costProfile = new LinkedList();
		if(role.getOwnedCostProfile() != null && !role.getOwnedCostProfile().isEmpty()){
			 costProfile.addAll(role.getOwnedCostProfile());
			 if(role.getCostProfile() != null && !role.getCostProfile().isEmpty()){
				List costProfiles = role.getCostProfile();
				Iterator costIt = costProfiles.iterator();
				while(costIt.hasNext()){
					TimeDependentCost tempCost = (TimeDependentCost)costIt.next();
					if(!costProfile.contains(tempCost)){
						costProfile.add(tempCost);
					}
				}
			}
		}else{
			if(role.getCostProfile()!= null && !role.getCostProfile().isEmpty()){
				costProfile.addAll(role.getCostProfile());
			}
		}
		
		return costProfile;
	}
	
	/**
	 * Updates the <code>workingCostValue</code> availability as per the <code>costValue</code>
	 * NOTE: will follow the tool mechanism of managing costAvailability TimeIntervals. All the available intervals
	 * of a CostValue are wrapped in a TimeIntervals object (top level BOM Packageable element)
	 * of the ResourceModel that is not shareable, as it is not visible in the BOM navigator.
	 * There is only one TimeIntervals for a CostValue. 
	 * The name of the TimeIntervals is unique as build from the name of the resource, and the unique ID of the CostValue.
	 * The management of this TimeIntervals is linked with the CostValue (Resource). 
	 * Upon the deletion of the Resource, the TimeIntervals corresponding to the
	 * CostAvailability should be also deleted.
	 * NOTE: Although this code will create top level BOM Packageable elements (TimeIntervals)
	 * it will not handle name colision for the above mentioned reason.  
	 * TODO check the method commands and errors
	 * @param resource BOM Resource
	 * @param costValue BOM CostValue
	 * @param workingCostValue 
	 * TODO error log
	 */
	private void updateCostValueAvailability(Role role, CostValue costValue, CostValue workingCostValue){
		LoggingUtil.traceEntry(this, "updateCostValueAvailability");
		if(costValue.getWhen() != null && workingCostValue != null){
			TimeIntervals when = costValue.getWhen();
			BtCompoundCommand wrapperCmd = new BtCompoundCommand();
			AbstractChildContainerNode resCatalogNode = (AbstractChildContainerNode)getImportSession().getContext().get(role.getOwningPackage());
			if(resCatalogNode == null){
				//can not find the resource model, can not create the TimeIntervals
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.OBJECT_REFERENCE_NOT_FOUND_EXCEPTION, new String[]{"Resource Catalog", role.getOwningPackage().getName()}, null);
				//can not create the TimeIntervals that will contain all the RecurringTimeInterval ExemptPeriods, return 
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.COST_AVAILABILITY_TIME_INTERVALS_CREATION_EXCEPTION, new String[]{role.getName()}, null, "Can not retrieve the resourceCatalog");
				LoggingUtil.traceExit(this, "updateCostValueAvailability");
				return;
			}
			String intervalsURI = createTimeIntervalsForCostValue(role, resCatalogNode, wrapperCmd, workingCostValue, workingCopy.getName());
			if(intervalsURI == null){
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.COST_AVAILABILITY_TIME_INTERVALS_CREATION_EXCEPTION, new String[]{role.getName()}, null, "IntervalsURI is null");
				LoggingUtil.traceExit(this, "updateCostValueAvailability");
				return;
			}
			OpenRootObjectForUpdateBOMCmd openCmd = new OpenRootObjectForUpdateBOMCmd();
			openCmd.setProjectName(getProjectName());
			openCmd.setROBLM_URI(intervalsURI);
			if(!wrapperCmd.appendAndExecute(openCmd)){
				wrapperCmd.undo();
			}else{
				// need to get copy id for saving
				String copyID = openCmd.getCopyID();
				TimeIntervals workingTimeIntervals = (TimeIntervals)openCmd.getROCopy();
				if(when.getRecurringTimeIntervals() != null && !when.getRecurringTimeIntervals().isEmpty()){
					Iterator it = when.getRecurringTimeIntervals().iterator();
					UpdateTimeIntervalsBOMCmd updateTI = new UpdateTimeIntervalsBOMCmd(workingTimeIntervals);
					RecurringTimeIntervals recurringTimeInterval = null;
					RecurringTimeIntervals workingRecInterval = null;
					while(it.hasNext()){
						recurringTimeInterval = (RecurringTimeIntervals)it.next();
						workingRecInterval = (RecurringTimeIntervals) BOMUtil.loadRootObject((AbstractLibraryChildNode) getImportSession().getContext().get(recurringTimeInterval));
						updateTI.addRecurringTimeIntervals(workingRecInterval);
					}
					wrapperCmd.appendAndExecute(updateTI);
				}
				
				SaveWorkingSetCmd saveCmd = new SaveWorkingSetCmd();
				saveCmd.setProjectName(getProjectName());
				String projectPath = FileMGR.getProjectPath(getProjectName());
				saveCmd.setProjectPath(projectPath);
				saveCmd.setWorkingSetID(copyID);
				if(!wrapperCmd.appendAndExecute(saveCmd)){
					wrapperCmd.undo();
				}else{
					TimeIntervals masterIntervals = (TimeIntervals)BOMUtil.loadRootObject(intervalsURI, getProjectName());
					UpdateCostValueBOMCmd updateCostValue = new UpdateCostValueBOMCmd(workingCostValue);
					updateCostValue.setWhen(masterIntervals);
					wrapperCmd.appendAndExecute(updateCostValue);
				}
				closeWorkingCopy(copyID);
			}
		}
		LoggingUtil.traceExit(this, "updateCostValueAvailability");
		return;
	}

	/**
	* Creates a hidden (tool hidden) TimeInterval that will contain all cost available RecurringTimeIntervals.
	* The name of the TimeIntervals is created from the name of the Reosurce and the ID of the CostValue and should be unique.
	* Life cycle of this object is managed by the Role that reference it.
	* @param resource 
	* @param node
	* @param wrapperCmd
	* @return the TimeIntervals root object BML URI. 
	*/
	protected String createTimeIntervalsForCostValue(Role role, AbstractChildContainerNode node, BtCompoundCommand wrapperCmd, CostValue workingCostValue, String roleName) {
		
		CreateTimeIntervalsBOMCmd createCmd = new CreateTimeIntervalsBOMCmd();
		createCmd.setParentModelBLM_URI((String)node.getEntityReference());
		createCmd.setProjectName(node.getProjectNode().getLabel());
		String workingSetID = CopyRegistry.instance().getCopyID(workingCostValue);
		EObject master = CopyRegistry.instance().getMaster(workingSetID, workingCostValue);
		createCmd.setName("$" + roleName + "_"+ ((CostValue)master).getUid()+"_when");
		//create an unique name for the exempt period.. unique exempt period/recurringInterval...
			//TODO !!!!!!! add precondition for interval name and not execute the command otherwise !!!!!	
		if(!wrapperCmd.appendAndExecute(createCmd)){
			wrapperCmd.undo();
		}
		
		return createCmd.getROBLM_URI();
	}
	
	/**
	 * Updates the <code>workingResource</code> with BOM Resource availability as per the 
	 * source <code>resource</code>
	 * @param resource source BOM Resource
	 * @param workingResource BOM Resource
	 *
	 */
	private void updateAvailability(){
		LoggingUtil.traceEntry(this, "updateAvailability");
		//TODO change when the model will change.. should be follow cost profile example...
		if(role.getAvailability() != null){
			TimeIntervals availability = role.getAvailability();
			BtCompoundCommand wrapperCmd = new BtCompoundCommand();
			AbstractChildContainerNode resCatalogNode = (AbstractChildContainerNode)getImportSession().getContext().get(role.getOwningPackage());
			String intervalsURI = createTimeIntervals(role, resCatalogNode, wrapperCmd, workingCopy.getName());
			if(intervalsURI == null){
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.COST_AVAILABILITY_TIME_INTERVALS_CREATION_EXCEPTION, new String[]{role.getName()}, null, "IntervalsURI is null");
				LoggingUtil.traceExit(this, "updateCostValueAvailability");
				return;
			}
			OpenRootObjectForUpdateBOMCmd openCmd = new OpenRootObjectForUpdateBOMCmd();
			openCmd.setProjectName(getProjectName());
			openCmd.setROBLM_URI(intervalsURI);
			if(!wrapperCmd.appendAndExecute(openCmd)){
				wrapperCmd.undo();
			}else{
//			if (openCmd.canExecute()) {
//				openCmd.execute();
//			}
			// need to get copy id for saving
				String copyID = openCmd.getCopyID();
				
				TimeIntervals workingTimeIntervals = (TimeIntervals)openCmd.getROCopy();
				
				if(availability.getRecurringTimeIntervals() != null && !availability.getRecurringTimeIntervals().isEmpty()){
					Iterator it = availability.getRecurringTimeIntervals().iterator();
					
					UpdateTimeIntervalsBOMCmd updateTI = new UpdateTimeIntervalsBOMCmd(workingTimeIntervals);
					
					while(it.hasNext()){
						RecurringTimeIntervals recurringTimeInterval = (RecurringTimeIntervals)it.next();
						RecurringTimeIntervals workingRecInterval = (RecurringTimeIntervals) BOMUtil.loadRootObject((AbstractLibraryChildNode) getImportSession().getContext().get(recurringTimeInterval));
						updateTI.addRecurringTimeIntervals(workingRecInterval);
					}
					wrapperCmd.appendAndExecute(updateTI);
				}
				
				SaveWorkingSetCmd saveCmd = new SaveWorkingSetCmd();
				saveCmd.setProjectName(getProjectName());
				String projectPath = FileMGR.getProjectPath(getProjectName());
				saveCmd.setProjectPath(projectPath);
				saveCmd.setWorkingSetID(copyID);
				if(!wrapperCmd.appendAndExecute(saveCmd)){
					wrapperCmd.undo();
				}else{
	//			
	//			if(saveCmd.canExecute()) {
	//				saveCmd.execute();
	//			} else {
	//				//log error to error log
	//			}
				TimeIntervals masterIntervals = (TimeIntervals)BOMUtil.loadRootObject(intervalsURI, getProjectName());
				UpdateRoleBOMCmd updateResource = new UpdateRoleBOMCmd(workingCopy);
				updateResource.setAvailability(masterIntervals);
				wrapperCmd.appendAndExecute(updateResource);
			}

			closeWorkingCopy(copyID);
			}
		}
		LoggingUtil.traceExit(this, "updateAvailability");
	}


	//TODO will dissapear when move to new ResourceModel
	protected String createTimeIntervals(Role role, AbstractChildContainerNode node, BtCompoundCommand wrapperCmd, String roleName) {

		CreateTimeIntervalsBOMCmd createCmd = new CreateTimeIntervalsBOMCmd();
		createCmd.setParentModelBLM_URI((String)node.getEntityReference());
		createCmd.setProjectName(node.getProjectNode().getLabel());
		createCmd.setName("$" + roleName + "_availability");
		//create an unique name for the exempt period.. unique exempt period/recurringInterval...
		//TODO !!!!!!! add precondition for interval name and not execute the command otherwise !!!!!
		if(!wrapperCmd.appendAndExecute(createCmd)){
			wrapperCmd.undo();
		}
		return createCmd.getROBLM_URI();
	}
	
	/**
	 * Close the workingcopy by id
	 * @param copyID
	 */
	protected void closeWorkingCopy(final String copyID) {
		LoggingUtil.traceEntry(this, "closeWorkingCopy");		
		if (copyID == null) {
			LoggingUtil.trace(this, "closeWorkingCopy", "The copyID should not be null.");
			LoggingUtil.traceExit(this, "closeWorkingCopy");			
			return;
		}
		CloseWorkingSetCmd closeCmd = new CloseWorkingSetCmd();
		closeCmd.setWorkingSetID(copyID);
		if(closeCmd.canExecute()) {
			try {			
				closeCmd.execute();
			} catch (RuntimeException e) {
				LoggingUtil.trace(this, "closeWorkingCopy", "The close command throws a RuntimeException.");
				LoggingUtil.traceExit(this, "closeWorkingCopy");
				return;
			}
		} else {
			LoggingUtil.trace(this, "closeWorkingCopy", "The close command cannot be executed.");
			LoggingUtil.traceExit(this, "closeWorkingCopy");
			return;
		}
		LoggingUtil.traceExit(this, "closeWorkingCopy");		
	}

	/*
	 * This method is to compare two roles when the 
	 * user chooses to overwrite the exising one in the working 
	 * space. Source is the new BOM imported into the memory
	 * target is the one existed into the workspace. We need to compare 
	 * and refactor the properties
	 */
	private void consolidateRole(){
		LoggingUtil.traceEntry(this, "consolidateRole");
		
		Role source = this.role;
		Role target = this.workingCopy;		
		
		if(source == null || target == null){
			LoggingUtil.trace(this, "consolidateBusinessItem", "The workingCopy should not be null.");
			return;
		}
		
		List<BtCompoundCommand> commandList = new ArrayList<BtCompoundCommand>();
		
		for (int i=0; i < target.getScopeDimension().size(); i++) {
			if (target.getScopeDimension().get(i) instanceof ScopeDimension) {
				ScopeDimension scopeDimension = (ScopeDimension)target.getScopeDimension().get(i);
				RemoveObjectCommand removeCmd = new RemoveObjectCommand(scopeDimension);
				commandList.add(removeCmd);
			}
		}
			
		target.getCostProfile().clear();
		
		List<String> removeROBLM_URIs = new ArrayList<String>();
				
		for (int i=0; i < target.getOwnedCostProfile().size(); i++) {
			if (target.getOwnedCostProfile().get(i) instanceof TimeDependentCost) {
				TimeDependentCost cost = (TimeDependentCost)target.getOwnedCostProfile().get(i);
				RemoveObjectCommand removeCmd = new RemoveObjectCommand(cost);
				commandList.add(removeCmd);

				List costValues = cost.getCostValue();
				for (int j=0; j < costValues.size(); j++) {
					CostValue costValue = (CostValue)costValues.get(j);
					if (costValue.getWhen() != null) {
						TimeIntervals intervals = costValue.getWhen();
						
						String ROBLM_URI = ResourceMGR.getResourceManger().getObjectResourceID(intervals);
						if (ROBLM_URI != null && !removeROBLM_URIs.contains(ROBLM_URI))
							removeROBLM_URIs.add(ROBLM_URI);	
					}
				}
			}
		}
					
		if (target.getAvailability() != null) {
			TimeIntervals intervals = target.getAvailability();
			
			String ROBLM_URI = ResourceMGR.getResourceManger().getObjectResourceID(intervals);
			if (ROBLM_URI != null && !removeROBLM_URIs.contains(ROBLM_URI))
				removeROBLM_URIs.add(ROBLM_URI);	
		}
		
		for (int i=0; i < removeROBLM_URIs.size(); i++) {
			DeleteRootObjectBOMCmd removeCmd = new DeleteRootObjectBOMCmd();
			removeCmd.setProjectName(getProjectName());
			removeCmd.setROBLM_URI(removeROBLM_URIs.get(i));
			commandList.add(removeCmd);			
		}

		if (!commandList.isEmpty()){
			super.forkUICloseEditor(target.getUid());
			
			for (int i=0; i < commandList.size(); i++) {
				BtCompoundCommand command = commandList.get(i);
				if (command.canExecute())
					command.execute();
			}			
		}	
		
		LoggingUtil.traceExit(this, "consolidateRole");
	}

}
