package com.ibm.btools.blm.ie.imprt.rule.simulationModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import org.eclipse.emf.ecore.EObject;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.informationModel.UpdateValueSpecificationRule;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.SimulationUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.bom.command.artifacts.AddLiteralUnlimitedNaturalToSimulationGeneratedRoleOverrideBOMCmd;
import com.ibm.btools.bom.command.artifacts.RemoveValueSpecificationBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddSimulationGeneratedRoleOverrideToGeneratedRoleProfileBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.UpdateSimulationGeneratedRoleOverrideBOMCmd;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.resources.Role;
import com.ibm.btools.bom.model.simulationprofiles.GeneratedRoleProfile;
import com.ibm.btools.bom.model.simulationprofiles.SimulationGeneratedRoleOverride;
import com.ibm.btools.emf.cf.BtCompoundCommand;


public class UpdateGeneratedRoleProfileRule extends AbstractImportRule {

	//	copyright
	  static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	  private GeneratedRoleProfile genRoleProfile;
	  private GeneratedRoleProfile workingCopy;
	  
	  
	/**
	 * Sets the original BOM TransitionProfile.
	 * @param object a BOM TransitionProfile
	 */
	public void setSource(Object object) {
		this.genRoleProfile = (GeneratedRoleProfile)object;
	}
	
	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = (GeneratedRoleProfile) workingCopy;
	}
	
		/**
		 * Invokes the rule
		 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");
		updateSimGenRoleOverride();
		LoggingUtil.traceExit(this, "invoke");
	}

	private void updateSimGenRoleOverride() {
		LoggingUtil.traceEntry(this, "updateSimGenRoleOverride");
		
		SimulationGeneratedRoleOverride simGenRoleOverride = genRoleProfile.getSimulationGeneratedRoleOverride();
		if (simGenRoleOverride == null) return;
		
		SimulationGeneratedRoleOverride workingSimGenRoleOverride = workingCopy.getSimulationGeneratedRoleOverride();
	
		if (workingSimGenRoleOverride == null) {
			// if there is no simulation generated role override, create one
			AddSimulationGeneratedRoleOverrideToGeneratedRoleProfileBOMCmd addSimGenOverrideCmd = new AddSimulationGeneratedRoleOverrideToGeneratedRoleProfileBOMCmd(workingCopy);
			
			if(!addSimGenOverrideCmd.canExecute()){
				//log error
				LoggingUtil.logError(getImportSession(), IeMessageKeys.SIMULATOR_GENERATED_ROLE_OVERRIDE_EXCEPTION, new String[]{genRoleProfile.getName()}, null, "Cannot add a GeneratedRoleOverride to GeneratedRoleProfile");
			}else{
				try{
					addSimGenOverrideCmd.execute();
					workingSimGenRoleOverride = (SimulationGeneratedRoleOverride)addSimGenOverrideCmd.getObject();
				}catch(RuntimeException ex){
					//log error
					LoggingUtil.logError(getImportSession(), IeMessageKeys.SIMULATOR_GENERATED_ROLE_OVERRIDE_EXCEPTION, new String[]{genRoleProfile.getName()}, ex, "Cannot add a GeneratedRoleOverride to GeneratedRoleProfile");
				}
			}			
		}
		
		if (workingSimGenRoleOverride != null) {
			
			Boolean useDuringResGen = simGenRoleOverride.getUseDuringResourceGeneration();
			if (useDuringResGen != null) {
				UpdateSimulationGeneratedRoleOverrideBOMCmd updateCmd = new UpdateSimulationGeneratedRoleOverrideBOMCmd(workingSimGenRoleOverride);
				updateCmd.setUseDuringResourceGeneration(useDuringResGen.booleanValue());
				if(!updateCmd.canExecute()){
					//log error
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_GENERATED_ROLE_PROFILE_UPDATE_EXCEPTION, new String[]{genRoleProfile.getName()}, null, "Cannot add a GeneratedRoleOverride to GeneratedRoleProfile");
				}else{
					try{
						updateCmd.execute();
						workingSimGenRoleOverride = (SimulationGeneratedRoleOverride)updateCmd.getObject();
					}catch(RuntimeException ex){
						//log error
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_GENERATED_ROLE_PROFILE_UPDATE_EXCEPTION, new String[]{genRoleProfile.getName()}, ex, "Cannot add a GeneratedRoleOverride to GeneratedRoleProfile");
					}
				}				
			}
			
			//add the quantity required
			ValueSpecification qtyReq = simGenRoleOverride.getQuantityRequired();
			ValueSpecification workingCopyQtyReq = workingSimGenRoleOverride.getQuantityRequired();
			if (qtyReq != null) {				
				if (workingCopyQtyReq != null) {
					// Assumption: As per agreement from different parties: ADF Mapper Import, sim cmds, sim UI Attributes view
					// ONLY LiteralUnlimitedNatural will be used as the type to represent the quantity required					
					if (workingCopyQtyReq instanceof LiteralUnlimitedNatural) {
						// update the qtyReq and workingCopyQtyReq directly since they are the same type					
						UpdateValueSpecificationRule updateValueSpecRule = new UpdateValueSpecificationRule();
						updateValueSpecRule.setImportSession(getImportSession());
						updateValueSpecRule.setProjectName(getProjectName());
						updateValueSpecRule.setSource(qtyReq);
						updateValueSpecRule.setWorkingCopy(workingCopyQtyReq);
						updateValueSpecRule.invoke();
					} else {
						// remove the workingCopyQtyReq first (if different type from the source qtyReq)
						// remove the type in the workingCopyQtyReq and create a new LiteralUnlimitedNatural and then pass on the UpdateValueSpecRule for updates						
						 RemoveValueSpecificationBOMCmd removeCmd = new RemoveValueSpecificationBOMCmd(workingCopyQtyReq);
						 if(!removeCmd.canExecute()){
							//log error
							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_GENERATED_ROLE_PROFILE_UPDATE_EXCEPTION, new String[]{genRoleProfile.getName()}, null, "Cannot remove the quantity from GeneratedRoleOverride in GeneratedRoleProfile");
						 }else{
							try{
								removeCmd.execute();								
							}catch(RuntimeException ex){
								//log error
								LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_GENERATED_ROLE_PROFILE_UPDATE_EXCEPTION, new String[]{genRoleProfile.getName()}, ex, "Cannot remove the quantity from GeneratedRoleOverride in GeneratedRoleProfile");
							}							
						 }	
						 createQtyRequiredInWorkingCopy(qtyReq, workingSimGenRoleOverride);
					}
				} else {
					// create a new quantity and update it
					createQtyRequiredInWorkingCopy(qtyReq, workingSimGenRoleOverride);
				}
			}
			
			EObject roleInSnapshot = null;
			EObject resTypeInSnapshot = null;

			if (simGenRoleOverride.getRole() != null) {
				roleInSnapshot = SimulationUtil.getObjectInSnapshot(simGenRoleOverride.getRole(),getProjectName(),getImportSession().getContext());
			}				
			if (simGenRoleOverride.getResourceType() != null) {
				resTypeInSnapshot= SimulationUtil.getObjectInSnapshot(simGenRoleOverride.getResourceType(),getProjectName(),getImportSession().getContext());
			}				
			
			UpdateSimulationGeneratedRoleOverrideBOMCmd updateSimGenRoleOverrideCmd = new UpdateSimulationGeneratedRoleOverrideBOMCmd(workingSimGenRoleOverride); 
			//set the role (reference)			
			if (roleInSnapshot != null) {
				updateSimGenRoleOverrideCmd.setRole((Role)roleInSnapshot);
			}
			// set the resource type (reference)
			if (resTypeInSnapshot != null) {
				updateSimGenRoleOverrideCmd.setResourceType((Type) resTypeInSnapshot);
			}
			
			if(updateSimGenRoleOverrideCmd.canExecute()){
				try{
					updateSimGenRoleOverrideCmd.execute();
				}catch(RuntimeException ex){
					LoggingUtil.logError(getImportSession(), IeMessageKeys.SIMULATOR_GENERATED_ROLE_OVERRIDE_EXCEPTION, new String[]{genRoleProfile.getName()}, ex, "Cannot update the GeneratedRoleOverride");
				}
			}else{
				//log error
				LoggingUtil.logError(getImportSession(), IeMessageKeys.SIMULATOR_GENERATED_ROLE_OVERRIDE_EXCEPTION, new String[]{genRoleProfile.getName()}, null, "Cannot update the GeneratedRoleOverride");
			}				
		}		
		LoggingUtil.traceExit(this, "updateSimGenRoleOverride");
	}	
	
	private void createQtyRequiredInWorkingCopy(ValueSpecification qtyReq, SimulationGeneratedRoleOverride workingSimGenRoleOverride) {		
		BtCompoundCommand wrapperCmd = new BtCompoundCommand();
		AddLiteralUnlimitedNaturalToSimulationGeneratedRoleOverrideBOMCmd addQtyReqCmd = new AddLiteralUnlimitedNaturalToSimulationGeneratedRoleOverrideBOMCmd(workingSimGenRoleOverride);
		if(wrapperCmd.appendAndExecute(addQtyReqCmd)){
			ValueSpecification workingValueSpecification = (ValueSpecification)addQtyReqCmd.getObject();
			UpdateValueSpecificationRule updateValueSpecRule = new UpdateValueSpecificationRule();
			updateValueSpecRule.setImportSession(getImportSession());
			updateValueSpecRule.setSource(qtyReq);
			updateValueSpecRule.setWorkingCopy(workingValueSpecification);
			updateValueSpecRule.setWrapperCommand(wrapperCmd);
			updateValueSpecRule.invoke();
		}else{
			LoggingUtil.logError(getImportSession(), IeMessageKeys.SIMULATOR_GENERATED_ROLE_OVERRIDE_EXCEPTION, new String[]{genRoleProfile.getName()}, null, "Cannot add a required quantity to GeneratedRoleProfile");
		}				
	}
}