package com.ibm.btools.blm.ie.imprt.rule.simulationModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.informationModel.UpdateValueSpecificationRule;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.bom.command.artifacts.AddLiteralRealToSimulationOutputSetOverrideBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddSimulationOutputSetOverrideToOutputSetProfileBOMCmd;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.simulationprofiles.OutputSetProfile;
import com.ibm.btools.bom.model.simulationprofiles.SimulationOutputSetOverride;
import com.ibm.btools.emf.cf.BtCompoundCommand;

public class UpdateOutputSetProfileRule extends AbstractImportRule {

	//	copyright
	  static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	  private OutputSetProfile outputSetProfile;
	  private OutputSetProfile workingCopy;
	
	/**
	 * Sets the original BOM OutputSetProfile.
	 * @param processProfile a BOM ProcessProfile
	*/
	public void setSource(Object object) {
		LoggingUtil.traceEntry(this, "setSource");
		this.outputSetProfile = (OutputSetProfile)object;
		LoggingUtil.traceExit(this, "setSource");

	}

	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		LoggingUtil.traceEntry(this, "setWorkingCopy");
		this.workingCopy = (OutputSetProfile) workingCopy;
		LoggingUtil.traceExit(this, "setWorkingCopy");
	}

		/**
	 * Invokes the rule
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");
		updateOutputSetOverride();
		LoggingUtil.traceExit(this, "invoke");
	}
	
	/**
	 * Updates the Output Set Profile. The following will be updated: set probability
	 */
	private void updateOutputSetOverride() {
		LoggingUtil.traceEntry(this, "updateOutputSetOverride");
		
		SimulationOutputSetOverride override = outputSetProfile.getSimulationOutputSetOverride();
		if(override != null && override.getSetProbability() != null){
			SimulationOutputSetOverride workingOverride = workingCopy.getSimulationOutputSetOverride();
			// if no Simulation Transition Override, create one
			if(workingOverride == null){
				AddSimulationOutputSetOverrideToOutputSetProfileBOMCmd addOutputSetOverride = new AddSimulationOutputSetOverrideToOutputSetProfileBOMCmd(workingCopy);
				if(!addOutputSetOverride.canExecute()){
					//log error
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TRANSITION_EXCEPTION, null, null, "Can not create a Simulator Output Set  to OutputSet Profile");
				}else{
					try{
						addOutputSetOverride.execute();
						workingOverride = (SimulationOutputSetOverride)addOutputSetOverride.getObject();
					}catch(RuntimeException ex){
						//log error
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TRANSITION_EXCEPTION, null, ex, "Can not create a Simulator Transition to Transition Profile");
					}
				}
			}
			if(workingOverride != null){
				ValueSpecification workingTransition = workingOverride.getSetProbability();
				if(workingTransition == null){
					BtCompoundCommand wrapperCmd = new BtCompoundCommand();
					AddLiteralRealToSimulationOutputSetOverrideBOMCmd addProbabilityCmd = new AddLiteralRealToSimulationOutputSetOverrideBOMCmd(workingOverride);
					if(wrapperCmd.appendAndExecute(addProbabilityCmd)){
						ValueSpecification workingValueSpecification = (ValueSpecification)addProbabilityCmd.getObject();
						UpdateValueSpecificationRule updateValueSpecRule = new UpdateValueSpecificationRule();
						updateValueSpecRule.setImportSession(getImportSession());
						updateValueSpecRule.setSource(override.getSetProbability());
						updateValueSpecRule.setWorkingCopy(workingValueSpecification);
						updateValueSpecRule.setWrapperCommand(wrapperCmd);
						updateValueSpecRule.invoke();
					}else{
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TRANSITION_EXCEPTION, null, null, "Can not update a Simulator Transition to Transition Profile");
					}
				}else{
					UpdateValueSpecificationRule updateValueSpecRule = new UpdateValueSpecificationRule();
					updateValueSpecRule.setImportSession(getImportSession());
					updateValueSpecRule.setSource(override.getSetProbability());
					updateValueSpecRule.setWorkingCopy(workingTransition);
					updateValueSpecRule.invoke();
				}
			}
		}
		LoggingUtil.traceExit(this, "updateOutputSetOverride");
	}


}
