package com.ibm.btools.blm.ie.imprt.rule.simulationModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.bom.command.simulationprofiles.AddTimeMonitorTotalProcessingTrapToSimulatorProcessProfileBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.UpdateSimulatorProcessProfileBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.UpdateTimeMonitorBOMCmd;
import com.ibm.btools.bom.model.artifacts.Element;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.simulationprofiles.ConnectionProfile;
import com.ibm.btools.bom.model.simulationprofiles.GeneratedRoleProfile;
import com.ibm.btools.bom.model.simulationprofiles.InputSetProfile;
import com.ibm.btools.bom.model.simulationprofiles.OutputSetProfile;
import com.ibm.btools.bom.model.simulationprofiles.PortProfile;
import com.ibm.btools.bom.model.simulationprofiles.ProcessProfile;
import com.ibm.btools.bom.model.simulationprofiles.ResourceProfile;
import com.ibm.btools.bom.model.simulationprofiles.SimulationGeneratedRoleOverride;
import com.ibm.btools.bom.model.simulationprofiles.SimulationProcessOverride;
import com.ibm.btools.bom.model.simulationprofiles.SimulatorProcessProfile;
import com.ibm.btools.bom.model.simulationprofiles.TaskProfile;
import com.ibm.btools.bom.model.simulationprofiles.TimeMonitor;
import com.ibm.btools.bom.model.simulationprofiles.TransitionProfile;

public class UpdateProcessProfileRule extends AbstractImportRule {

	//	copyright
	  static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	  private ProcessProfile processProfile;
	  private ProcessProfile workingCopy;
	  private List workingSnapshot;
	  private ImportContext simulationContext;

	  /**
	   * Invokes the rule
	   */
	  public void invoke() {
		  LoggingUtil.traceEntry(this, "invoke");
		  //defect 95957. reinitialize the SimContext. 
		  initializeSimulationContext();
	      updateSimulatorProcessProfile();
		  updateSimulationProcessOverride();
		  updateProcessProfileContent();
		  LoggingUtil.traceExit(this, "invoke");
	  }
	
	  /**
	   * Sets the original BOM ProcessProfile.
	   * @param processProfile a BOM ProcessProfile
	   */
	  public void setSource(final Object processProfile) {
		LoggingUtil.traceEntry(this, "setSource");
		  this.processProfile = (ProcessProfile) processProfile;
		LoggingUtil.traceExit(this, "setSource");
	  }
	
	  /**
	   * Sets the working copy
	   * @param workingCopy
	   */
	  public void setWorkingCopy(final Object workingCopy) {
		LoggingUtil.traceEntry(this, "setWorkingCopy");
		  this.workingCopy = (ProcessProfile) workingCopy;
		LoggingUtil.traceExit(this, "setWorkingCopy");
	  }



	  /**
	 * @param override
	 */
	private void updateSimulationProcessOverride() {
		SimulationProcessOverride override = processProfile.getSimulationProcessOverride();
		if(override == null){
			return;
		}
	}

	/**
	 * Updates the simulatorProcessProfile. 
	 * Based on ADF import, only the following will be updated: 
	 * - randomSeed,
	 * - process start, 
	 * - totalProcessingTrap.
	 *  @param profile
	 */
	private void updateSimulatorProcessProfile() {
		LoggingUtil.traceEntry(this, "invoke");
		SimulatorProcessProfile profile = processProfile.getSimulatorProcessProfile();
		SimulatorProcessProfile workingProfile = workingCopy.getSimulatorProcessProfile();
		//UPDATE RANDOM SEED PROCESS START AND PROCESS EXPIRY
		if(profile.getRandomSeed() != null || profile.getProcessStart() != null || profile.getProcessExpiry() != null){
			UpdateSimulatorProcessProfileBOMCmd updateCmd = new UpdateSimulatorProcessProfileBOMCmd(workingProfile);
			if(profile.getRandomSeed() != null){
				updateCmd.setRandomSeed(profile.getRandomSeed());
			}
			if(profile.getProcessStart() != null){
				updateCmd.setProcessStart(profile.getProcessStart());
			}
			if(profile.getProcessExpiry() != null){
				updateCmd.setProcessExpiry(profile.getProcessExpiry());
			}
			//Jie 06/07/04- d82188 
		  	if(profile.getProcessRealExpiry() != null){
			    updateCmd.setProcessRealExpiry(profile.getProcessRealExpiry());
		  	}			
		  	if(profile.getConnectionSelectionCriteria() != null){
			    updateCmd.setConnectionSelectionCriteria(profile.getConnectionSelectionCriteria());
		  	}
		    //end of Jie added
			if(!updateCmd.canExecute()){
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_PROCESS_PROFILE_UPDATE_EXCEPTION, new String[]{processProfile.getName()}, null, "Cannot update the Simulator Process Profile");
			}else{
				try{
					updateCmd.execute();
				}catch(RuntimeException ex){
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_PROCESS_PROFILE_UPDATE_EXCEPTION, new String[]{processProfile.getName()}, ex, "Cannot update the Simulator Process Profile");
					
				}
			}
		}
		//UPDATE THE TOTAL PROCESSING TRAP
		TimeMonitor timeMonitor = profile.getTotalProcessingTrap();
		if(timeMonitor != null){
			TimeMonitor workingTimeMonitor = workingProfile.getTotalProcessingTrap();
			if(workingTimeMonitor == null){
				AddTimeMonitorTotalProcessingTrapToSimulatorProcessProfileBOMCmd addMonitorCmd = 
					new AddTimeMonitorTotalProcessingTrapToSimulatorProcessProfileBOMCmd (workingProfile);
				if(!addMonitorCmd.canExecute()){
					//log warning
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_PROCESS_PROFILE_UPDATE_EXCEPTION, new String[]{processProfile.getProcess().getName()}, null, "Cannot update a ProcessTaskProfile");
				}else{
					try{
						addMonitorCmd.execute();
						workingTimeMonitor = (TimeMonitor)addMonitorCmd.getObject();
					}catch(RuntimeException ex){
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_PROCESS_PROFILE_UPDATE_EXCEPTION, new String[]{processProfile.getProcess().getName()}, ex, "Cannot update a ProcessTaskProfile");
						//log warning.. could not update the TimeMonitor
					}
				}
			}
			if(workingTimeMonitor != null){
				UpdateTimeMonitorBOMCmd updateMonitorCmd = new UpdateTimeMonitorBOMCmd(workingTimeMonitor);
				if(timeMonitor.getIgnoreInit() != null){
					updateMonitorCmd.setIgnoreInit(timeMonitor.getIgnoreInit().intValue());
				}
				if(timeMonitor.getRatioCheck() != null){
					updateMonitorCmd.setRatioCheck(timeMonitor.getRatioCheck().booleanValue());
				}
				if(timeMonitor.getThreshold() != null){
					updateMonitorCmd.setThreshold(timeMonitor.getThreshold());
				}
				if(timeMonitor.getTimeLimit()!= null && !timeMonitor.getTimeLimit().equals("")){
					updateMonitorCmd.setTimeLimit(timeMonitor.getTimeLimit());
				}
				if(timeMonitor.getTotalChecked() != null){
					updateMonitorCmd.setTotalChecked(timeMonitor.getTotalChecked().intValue());
				}
				if(timeMonitor.getTotalTrapped()!= null){
					updateMonitorCmd.setTotalTrapped(timeMonitor.getTotalTrapped().intValue());
				}
				if(timeMonitor.getTotalViolations()!= null){
					updateMonitorCmd.setTotalViolations(timeMonitor.getTotalViolations().intValue());
				}
			
				if(!updateMonitorCmd.canExecute()){
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_PROCESS_PROFILE_UPDATE_EXCEPTION, new String[]{processProfile.getProcess().getName()}, null, "Cannot update a ProcessTaskProfile");
				}else{
					try{
						updateMonitorCmd.execute();
					}catch(RuntimeException ex){
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_PROCESS_PROFILE_UPDATE_EXCEPTION, new String[]{processProfile.getProcess().getName()}, ex, "Cannot update a ProcessTaskProfile");
					}
				}
			}
		}
		LoggingUtil.traceExit(this, "updateProfiles");
	}

	/**
	   * Updates each of the profile contained by the ProcessProfile . 
	   * @param element BOM Element 
	   * @param workingElement BOM Element for update
	   */	
	  private void updateProcessProfileContent(){
		LoggingUtil.traceEntry(this, "updateProcessProfileContent");
		
		updateProfiles(processProfile.getTaskProfile());
		updateProfiles(processProfile.getTransitionProfile());
		updateProfiles(processProfile.getResourceProfile());
		//TODO check if there is a need to update all the profiles.. only  TaskProfile
		//updateProfiles(processProfile.getConnectionProfile());
		//updateProfiles(processProfile.getInputSetProfile());
		updateProfiles(processProfile.getOutputSetProfile());
		//updateProfiles(processProfile.getPortProfile());
		
		// 20050415 - added to support the resource generation line item
		updateProfiles(processProfile.getGeneratedRoleProfile());
		// 20050415 - end of added to support the resource generation line item
		
		LoggingUtil.traceExit(this, "updateProcessProfileContent");
	  }

	/**
	 * Updates each of the <code>workingCopy</code> profile based on the input
	 * profile in the <code>profiles</code>.
	  * @param list
	  */
	private void updateProfiles(EList profiles) {
		LoggingUtil.traceEntry(this, "updateProfiles");
		if(profiles != null && !profiles.isEmpty()){
			Iterator it = profiles.iterator();
			while(it.hasNext()){
				Element profile = (Element)it.next();
				updateProfile(profile);
			}
		}
		LoggingUtil.traceExit(this, "updateProfiles");
	}	
	/**
	 * Retrieves the workingProfile that corresponds to the <code>profile</code>
	 * and updates it.
	  * @param profile
	  */
	 private void updateProfile(Element profile){
		LoggingUtil.traceEntry(this, "updateProfile");
		Element workingProfile = (Element)findToBeMappedProfile(profile); 
		AbstractImportRule updateRule = null;
		if(workingProfile != null){
			if(workingProfile instanceof TaskProfile){
				updateRule = new UpdateTaskProfileRule();
				//((UpdateTaskProfileRule)updateRule).setSnapshotContent(workingSnapshot);
			}else if(workingProfile instanceof ResourceProfile){
				updateRule = new UpdateOutputSetProfileRule();
			}else if(workingProfile instanceof ConnectionProfile){
			//	updateRule = new UpdateConnectionProfileRule();
			}else if(workingProfile instanceof InputSetProfile){
	//			updateRule = new UpdateInputProfileRule();
			}else if(workingProfile instanceof OutputSetProfile){
				updateRule = new UpdateOutputSetProfileRule();
			}else if(workingProfile instanceof PortProfile){
	//			updateRule = new UpdatePortProfileRule();
			}else if(workingProfile instanceof TransitionProfile){
				updateRule = new UpdateTransitionProfileRule();
			} else if (workingProfile instanceof GeneratedRoleProfile) {
				updateRule = new UpdateGeneratedRoleProfileRule();
			}
			if(updateRule != null){
				updateRule.setImportSession(getImportSession());
				updateRule.setProjectName(getProjectName());
				updateRule.setSource(profile);
				updateRule.setWorkingCopy(workingProfile);
				updateRule.invoke();
			}
	
		}
		LoggingUtil.traceExit(this, "updateProfile");
	 }
	/**
	 * Returns the Element Profile in the workingCopy that corresponds to the 
	 * input profile in the source. Matching will be done based on the element 
	 * canonical name.
	 * @param profile a BOM Simulation profile 
	 */
	private Element findToBeMappedProfile(Element profile){
		LoggingUtil.traceEntry(this, "findToBeMappedProfile");
		
		NamedElement element = null;
		NamedElement additionalElement = null;
		List workingProfiles = null;
		if(profile instanceof TaskProfile){
			element = ((TaskProfile)profile).getTask();
			workingProfiles = workingCopy.getTaskProfile();
		}else if(profile instanceof ResourceProfile){
			element = ((ResourceProfile)profile).getResource();
			workingProfiles = workingCopy.getTaskProfile();
		}else if (profile instanceof ConnectionProfile){
			element = ((ConnectionProfile)profile).getConnection();
			workingProfiles = workingCopy.getTaskProfile();
		}else if(profile instanceof InputSetProfile){
			element = ((InputSetProfile)profile).getInputSet();
			workingProfiles = workingCopy.getInputSetProfile();
		}else if(profile instanceof OutputSetProfile){
			element = ((OutputSetProfile)profile).getOutputSet();
			workingProfiles = workingCopy.getOutputSetProfile();
		}else if(profile instanceof PortProfile){
			element = ((PortProfile)profile).getPort();
			workingProfiles = workingCopy.getPortProfile();
		}else if(profile instanceof TransitionProfile){
			element = ((TransitionProfile)profile).getFrom();
			additionalElement = ((TransitionProfile)profile).getTo();
			workingProfiles = workingCopy.getTransitionProfile();
		} else if (profile instanceof GeneratedRoleProfile) {
			GeneratedRoleProfile genRoleProfile = (GeneratedRoleProfile) profile;
			SimulationGeneratedRoleOverride simGenRoleOverride = genRoleProfile.getSimulationGeneratedRoleOverride();	
			// The simulation generated role override should always be created
			if (simGenRoleOverride == null) {
				// log errors
				LoggingUtil.logError(getImportSession(), IeMessageKeys.SIMULATOR_GENERATED_ROLE_OVERRIDE_EXCEPTION, new String[]{genRoleProfile.getName()}, null, "The SimGeneratedRoleOverride is null in the source for update");
				return null;	
			}
			element = simGenRoleOverride.getRole();			
			workingProfiles = workingCopy.getGeneratedRoleProfile();
		}
		if(workingProfiles != null && !workingProfiles.isEmpty()){
			Iterator it = workingProfiles.iterator();
			NamedElement thisElement = null;
			NamedElement thisAdditionalElement = null;
			Element thisProfile = null;
			while(it.hasNext()){
				thisProfile = (Element)it.next();
				if(thisProfile instanceof TaskProfile){
					thisElement = ((TaskProfile)thisProfile).getTask();
				}else if(thisProfile instanceof ResourceProfile){
					thisElement = ((ResourceProfile)thisProfile).getResource();
				}else if (thisProfile instanceof ConnectionProfile){
					thisElement = ((ConnectionProfile)thisProfile).getConnection();
				}else if(thisProfile instanceof InputSetProfile){
					thisElement = ((InputSetProfile)thisProfile).getInputSet();
				}else if(thisProfile instanceof OutputSetProfile){
					thisElement = ((OutputSetProfile)thisProfile).getOutputSet();
				}else if(thisProfile instanceof PortProfile){
					thisElement = ((PortProfile)thisProfile).getPort();
				}else if(thisProfile instanceof TransitionProfile){
					thisElement = ((TransitionProfile)thisProfile).getFrom();
					thisAdditionalElement = ((TransitionProfile)thisProfile).getTo();
				} else if (thisProfile instanceof GeneratedRoleProfile) {
					GeneratedRoleProfile genRoleProfile = (GeneratedRoleProfile) thisProfile;
					SimulationGeneratedRoleOverride simGenRoleOverride = genRoleProfile.getSimulationGeneratedRoleOverride();
					// The sim cmd - GenSimProfileCmd will always create the simGenRoleOverride with default quantity
					if (simGenRoleOverride == null) {
						// log error
						LoggingUtil.logError(getImportSession(), IeMessageKeys.SIMULATOR_GENERATED_ROLE_OVERRIDE_EXCEPTION, new String[]{genRoleProfile.getName()}, null, "The SimGeneratedRoleOverride is null in the WorkingCopy for update");
						continue;
					}					
					thisElement = simGenRoleOverride.getRole();
				}
				if(thisElement != null){
					ImportContext simulationContext = getSimulationContext();
					if(BOMUtil.getCanonicalName(thisElement, simulationContext).equals(BOMUtil.getCanonicalName(element, simulationContext))){
						if(additionalElement != null && thisAdditionalElement != null){
							if(BOMUtil.getCanonicalName(thisAdditionalElement, simulationContext).equals(BOMUtil.getCanonicalName(additionalElement, simulationContext))){
								LoggingUtil.traceExit(this, "findToBeMappedProfile");
								return thisProfile;
							//Defect 94511
							}else{
								//break;
							}
						}else{
							LoggingUtil.traceExit(this, "findToBeMappedProfile");
							return thisProfile;	
						}
					}
				}else{
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATION_PROFILE_NOT_FOUND_EXCEPTION, new String[]{processProfile.getName()}, null, "Cannot retrive the Simulator Profile for update");
				}
			}
		}
		LoggingUtil.traceExit(this, "findToBeMappedProfile");
		return null;
	}

	/**
	 * Returns the Simulation subcontext from the ImportSession context.
	 * The subcontext has cached BOM Elements and their canonical names.
	 */
	private ImportContext getSimulationContext(){
		LoggingUtil.traceEntry(this, "getSimulationContext");
		if(simulationContext == null){
			simulationContext = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_SIMULATION);
		}
		LoggingUtil.traceExit(this, "getSimulationContext");
		return simulationContext;
	}
	
	private void initializeSimulationContext(){
		
		LoggingUtil.traceEntry(this, "initializeSimulationContext");
		getImportSession().getContext().removeSubContext(IImportFramework.CONTEXT_SIMULATION);
		LoggingUtil.traceExit(this, "initializeSimulationContext");
		return;
		
		
		
	}


}
