package com.ibm.btools.blm.ie.imprt.rule.simulationModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.command.AbstractCommand;
import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.informationModel.UpdateValueSpecificationRule;
import com.ibm.btools.blm.ie.imprt.rule.resourceModel.UpdateRecurringTimeIntervalsRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.SimulationUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.bom.command.artifacts.AddInstanceValueToScopeValueBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralDurationProcessingTimeToSimulationTaskOverrideBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralNullToScopeValueBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralRealMaxValueToUniformDistributionBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralRealMinValueToUniformDistributionBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralRealToListElementBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralRealToResourceQuantityBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralStringToScopeValueBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddOpaqueExpressionToScopeValueBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddStructuredDurationProcessingTimeToSimulationTaskOverrideBOMCmd;
import com.ibm.btools.bom.command.artifacts.RemoveValueSpecificationBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateLiteralRealBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateStructuredDurationBOMCmd;
import com.ibm.btools.bom.command.resources.AddBulkResourceRequirementToSimulationTaskOverrideBOMCmd;
import com.ibm.btools.bom.command.resources.AddIndividualResourceRequirementToSimulationTaskOverrideBOMCmd;
import com.ibm.btools.bom.command.resources.AddRequiredRoleToSimulationTaskOverrideBOMCmd;
import com.ibm.btools.bom.command.resources.AddResourceQuantityToBulkResourceRequirementBOMCmd;
import com.ibm.btools.bom.command.resources.AddResourceQuantityToRequiredRoleBOMCmd;
import com.ibm.btools.bom.command.resources.AddScopeValueToRequiredRoleBOMCmd;
import com.ibm.btools.bom.command.resources.AddUpdateResourceQuantityBOMCmd;
import com.ibm.btools.bom.command.resources.AddUpdateResourceRequirementBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddBernoulliDistributionToStructuredDurationBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddBetaDistributionToStructuredDurationBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddBinomialDistributionToStructuredDurationBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddContinuousRNDistributionToStructuredDurationBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddErlangRNDistributionToStructuredDurationBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddExponentialDistributionToStructuredDurationBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddGammaDistributionToStructuredDurationBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddJohnsonRNDistributionToStructuredDurationBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddListElementToRandomListBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddLognormalDistributionToStructuredDurationBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddNormalDistributionToStructuredDurationBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddPoissonDistributionToStructuredDurationBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddRandomListToStructuredDurationBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddSimulationTaskOverrideToTaskProfileBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddSimulatorTaskProfileToTaskProfileBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddTriangularRNDistributionToStructuredDurationBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddUniformDistributionToStructuredDurationBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddWeibullRNDistributionToStructuredDurationBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddWeightedListElementToWeightedListBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddWeightedListToStructuredDurationBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.UpdateBernoulliDistributionBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.UpdateBetaDistributionBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.UpdateBinomialDistributionBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.UpdateContinuousRNDistributionBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.UpdateErlangRNDistributionBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.UpdateExponentialDistributionBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.UpdateGammaDistributionBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.UpdateJohnsonRNDistributionBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.UpdateLognormalDistributionBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.UpdateNormalDistributionBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.UpdatePoissonDistributionBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.UpdateSimulatorTaskProfileBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.UpdateTriangularRNDistributionBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.UpdateWeibullRNDistributionBOMCmd;
import com.ibm.btools.bom.command.time.AddRecurringTimeIntervalsToSimulationTaskOverrideBOMCmd;
import com.ibm.btools.bom.command.time.UpdateRecurringTimeIntervalsBOMCmd;
import com.ibm.btools.bom.model.artifacts.InstanceValue;
import com.ibm.btools.bom.model.artifacts.LiteralDuration;
import com.ibm.btools.bom.model.artifacts.LiteralNull;
import com.ibm.btools.bom.model.artifacts.LiteralReal;
import com.ibm.btools.bom.model.artifacts.LiteralSpecification;
import com.ibm.btools.bom.model.artifacts.OpaqueExpression;
import com.ibm.btools.bom.model.artifacts.StructuredDuration;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.resources.BulkResource;
import com.ibm.btools.bom.model.resources.BulkResourceRequirement;
import com.ibm.btools.bom.model.resources.IndividualResource;
import com.ibm.btools.bom.model.resources.IndividualResourceRequirement;
import com.ibm.btools.bom.model.resources.RequiredRole;
import com.ibm.btools.bom.model.resources.ResourceQuantity;
import com.ibm.btools.bom.model.resources.ResourceRequirement;
import com.ibm.btools.bom.model.resources.Role;
import com.ibm.btools.bom.model.resources.ScopeDimension;
import com.ibm.btools.bom.model.resources.ScopeValue;
import com.ibm.btools.bom.model.simulationprofiles.BernoulliDistribution;
import com.ibm.btools.bom.model.simulationprofiles.BetaDistribution;
import com.ibm.btools.bom.model.simulationprofiles.BinomialDistribution;
import com.ibm.btools.bom.model.simulationprofiles.ConnectionSelectionCriteria;
import com.ibm.btools.bom.model.simulationprofiles.ContinuousRNDistribution;
import com.ibm.btools.bom.model.simulationprofiles.Distribution;
import com.ibm.btools.bom.model.simulationprofiles.ErlangRNDistribution;
import com.ibm.btools.bom.model.simulationprofiles.ExponentialDistribution;
import com.ibm.btools.bom.model.simulationprofiles.GammaDistribution;
import com.ibm.btools.bom.model.simulationprofiles.JohnsonRNDistribution;
import com.ibm.btools.bom.model.simulationprofiles.JohnsonType;
import com.ibm.btools.bom.model.simulationprofiles.ListElement;
import com.ibm.btools.bom.model.simulationprofiles.LognormalDistribution;
import com.ibm.btools.bom.model.simulationprofiles.NormalDistribution;
import com.ibm.btools.bom.model.simulationprofiles.PoissonDistribution;
import com.ibm.btools.bom.model.simulationprofiles.RandomList;
import com.ibm.btools.bom.model.simulationprofiles.SimulationTaskOverride;
import com.ibm.btools.bom.model.simulationprofiles.SimulatorTaskProfile;
import com.ibm.btools.bom.model.simulationprofiles.TaskProfile;
import com.ibm.btools.bom.model.simulationprofiles.TriangularRNDistribution;
import com.ibm.btools.bom.model.simulationprofiles.UniformDistribution;
import com.ibm.btools.bom.model.simulationprofiles.WeibullRNDistribution;
import com.ibm.btools.bom.model.simulationprofiles.WeightedList;
import com.ibm.btools.bom.model.simulationprofiles.WeightedListElement;
import com.ibm.btools.bom.model.time.RecurringTimeIntervals;
import com.ibm.btools.emf.cf.BtCompoundCommand;
import com.ibm.btools.expression.bom.command.AddStructuredOpaqueExpressionToScopeValueBEXCmd;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.model.modelmanager.util.AddUpdateObjectCommand;


public class UpdateTaskProfileRule extends AbstractImportRule {


	  
	public class UpdateSimRecurringTimeIntervalsRule extends UpdateRecurringTimeIntervalsRule{
		
		/**
			 * Invokes the rule.
			 */
			public void invoke() {
				LoggingUtil.traceEntry(this, "invoke");
				UpdateRecurringTimeIntervalsBOMCmd updateCmd = new UpdateRecurringTimeIntervalsBOMCmd(super.workingCopy);
				//update the name only if not null, else do not overwrite the command  generated default one
				if(recurringInterval.getName() != null){
					updateCmd.setName(recurringInterval.getName());
				}
				if(recurringInterval.getNumberOfRecurrences() != null){
					updateCmd.setNumberOfRecurrences(recurringInterval.getNumberOfRecurrences());
				}else{
					updateCmd.setNumberOfRecurrences("*");
				}
				if(recurringInterval.getTimeZone() != null){
					updateCmd.setTimeZone(recurringInterval.getTimeZone());
				}else{
					updateCmd.setTimeZone("PT00H00M");
				}
	
				if(!updateCmd.canExecute()){
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CALENDAR_UPDATE_EXCEPTION, new String[]{recurringInterval.getName()}, null);
				}else{
					try{
						updateCmd.execute();
					}catch(RuntimeException ex){
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CALENDAR_UPDATE_EXCEPTION, new String[]{recurringInterval.getName()}, ex);
					}
				}
				if(recurringInterval.getValidityPeriod() != null && !recurringInterval.getValidityPeriod().isEmpty()){
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.UNSUPPORTED_RECURRING_INTERVAL_VALIDITY_PERIOD, new String[]{recurringInterval.getName()}, null);
				}
				updateRecurrencePeriod();
				updateAnchorPoint();
				updateIntervals();
				LoggingUtil.traceExit(this, "invoke");
			}
		
	}

	//	copyright
	  static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	  private TaskProfile taskProfile;
	  private TaskProfile workingCopy;
	  private List workingSnapshot;

	  
	/**
	 * Sets the original BOM TaskProfile.
	 * @param object a BOM TaskProfile
	 */
	public void setSource(Object object) {
		LoggingUtil.traceEntry(this, "setSource");
		this.taskProfile = (TaskProfile)object;
		LoggingUtil.traceExit(this, "setSource");
	}
	
	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		LoggingUtil.traceEntry(this, "setWorkingCopy");
		this.workingCopy = (TaskProfile) workingCopy;
		LoggingUtil.traceExit(this, "setWorkingCopy");
	}
	
	/**
	 * Invokes the rule
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");
		updateSimulationTaskOverride();
		updateSimulatorTaskProfile();
		LoggingUtil.traceExit(this, "invoke");
	}

	/**
	 * Updates the Simulator Task Profile. The following will be updated:
	 * - connection selection criteria
	 * - max concurrent
	 *
	 */
	private void updateSimulatorTaskProfile() {
		LoggingUtil.traceEntry(this, "updateSimulatorTaskProfile");
		
		SimulatorTaskProfile simTaskProfile = taskProfile.getSimulatorTaskProfile();
		SimulatorTaskProfile workingSimTaskProfile = workingCopy.getSimulatorTaskProfile();
		if(simTaskProfile == null){
			return;
		}
		if(workingSimTaskProfile == null){
			AddSimulatorTaskProfileToTaskProfileBOMCmd addSimProfileCmd = new AddSimulatorTaskProfileToTaskProfileBOMCmd(workingCopy);
			if(addSimProfileCmd.canExecute()){
				try{
					addSimProfileCmd.execute();
					workingSimTaskProfile = (SimulatorTaskProfile)addSimProfileCmd.getObject();
				}catch(RuntimeException ex){
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_CREATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, ex, "Cannot create a SimulatorTaskProfile");
				}
			}else{
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_CREATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, null, "Cannot create a SimulatorTaskProfile");
			}
		}
		if(workingSimTaskProfile != null){
			if(simTaskProfile.getConnectionSelectionCriteria() != null || simTaskProfile.getMaxConcurrent() != null){
				UpdateSimulatorTaskProfileBOMCmd updateSimCmd = new UpdateSimulatorTaskProfileBOMCmd(workingSimTaskProfile);
				//UPDATE CONNECTION SELECTION CRITERIA
				if(simTaskProfile.getConnectionSelectionCriteria() != null){
					ConnectionSelectionCriteria selectionCriteria = simTaskProfile.getConnectionSelectionCriteria();
					updateSimCmd.setConnectionSelectionCriteria(selectionCriteria);
				}
				//UPDATE MAX CONCURRENT
				if(simTaskProfile.getMaxConcurrent() != null){
					updateSimCmd.setMaxConcurrent(simTaskProfile.getMaxConcurrent().intValue());
				}
				if(updateSimCmd.canExecute()){
					try{
						updateSimCmd.execute();
					}catch(RuntimeException ex){
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_UPDATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, ex, "Cannot update a SimulatorTaskProfile");
					}
				}else{
					//log error
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_UPDATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, null, "Cannot update a SimulatorTaskProfile");
				}
			}
		}
		LoggingUtil.traceExit(this, "updateSimulatorTaskProfile");
	}
	
	/**
	 * Updates the Simulator Task Override. The following will be updated:
	 * - processing time
	 * - availability
	 *
	 */
	private void updateSimulationTaskOverride() {
		LoggingUtil.traceEntry(this, "updateSimulationTaskOverride");
		SimulationTaskOverride override = taskProfile.getSimulationTaskOverride();
		SimulationTaskOverride workingOverride = workingCopy.getSimulationTaskOverride();
		
		if(override == null){
			return;
		}
		if(workingOverride == null){
			AddSimulationTaskOverrideToTaskProfileBOMCmd addOverrideCmd = new AddSimulationTaskOverrideToTaskProfileBOMCmd(workingCopy);
			if(!addOverrideCmd.canExecute()){
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_UPDATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, null, "Cannot update a SimulatorTaskProfile");
			}else{
				try{
					addOverrideCmd.execute();
					workingOverride = (SimulationTaskOverride)addOverrideCmd.getObject();
				}catch(RuntimeException ex){
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_UPDATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, null, "Cannot update a SimulatorTaskProfile");
				}
			}
		}
		if(workingOverride != null){
			//UPDATE Processing time
			if(override.getProcessingTime() != null){				
				// TN - 20050524: Modifications for the regenerated BOM for the Time Unit Changes				
				ValueSpecification processingTime = override.getProcessingTime();				
				//if there is no processing time for this task, create one
				if(workingOverride.getProcessingTime() == null){
					// TN - 20050524: Modifications for the regenerated BOM for the Time Unit Changes										
					addProcessingTimeToSimOverrides(workingOverride, processingTime);					
				}else{
					//if a processing time exists for this task, update it
					// TN - 20050524: Modifications for the regenerated BOM for the Time Unit Changes					
					updateProcessingTimeInSimOverrides(workingOverride, override);
				}
			}
			/*
			 * Commented out as not supported by the tool
			 //UPDATE AVAILABILITY
			// updating the Task availability will proceed as following:
			// 1. create the owned recurring intervals based on the referenced availability in the override
			// 2. create the owned TimeIntervals - availability and add the recurring time intervals to it from step 1.
			// 3. set the referenced availability to point to the owned availability
			//
			if(override.getAvailability() != null){
			if(override.getAvailability().getRecurringTimeIntervals() != null && !override.getAvailability().getRecurringTimeIntervals().isEmpty()){
					List recurringIntervalsList = override.getAvailability().getRecurringTimeIntervals();
					Iterator it = recurringIntervalsList.iterator();
					RecurringTimeIntervals workingIntervals = null;
					RecurringTimeIntervals recurringTimeIntervals = null;
					while(it.hasNext()){
						recurringTimeIntervals = (RecurringTimeIntervals)it.next();
						createTimeIntervals(workingOverride, recurringTimeIntervals);
					}
					AddTimeIntervalsToSimulationTaskOverrideBOMCmd addCmd = new AddTimeIntervalsToSimulationTaskOverrideBOMCmd(workingOverride);
					addCmd.setName(override.getAvailability().getName());
					if(addCmd.canExecute()){
						try {
							addCmd.execute();
							TimeIntervals workingTimeIntervals = (TimeIntervals)addCmd.getObject();
							UpdateTimeIntervalsBOMCmd updateCmd = new UpdateTimeIntervalsBOMCmd(workingTimeIntervals);
							List workingRecIntervalsList = workingOverride.getRecurringTimeIntervals();
							it = workingRecIntervalsList.iterator();
							while(it.hasNext()){
								updateCmd.addRecurringTimeIntervals((RecurringTimeIntervals)it.next());
							}
							if(updateCmd.canExecute()){
								try{
									updateCmd.execute();
								}catch(BTRuntimeException ex){
									LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_UPDATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, null, "Cannot update a SimulatorTaskProfile");
								}
							}else{
								LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_UPDATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, null, "Cannot update a SimulatorTaskProfile");
							}
							UpdateSimulationTaskOverrideBOMCmd updateOverrideCmd = new UpdateSimulationTaskOverrideBOMCmd(workingOverride);
							updateOverrideCmd.setAvailability(workingTimeIntervals);
							if(updateOverrideCmd.canExecute()){
								try{
									updateOverrideCmd.execute();
								}catch(RuntimeException e){
									//log warning  
									LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_UPDATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, null, "Cannot update a SimulatorTaskProfile");
								}
							}else{
								//log warning.. could not update
								LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_UPDATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, null, "Cannot update a SimulatorTaskProfile");
							}
	
						} catch (BTRuntimeException e) {
							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_UPDATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, null, "Cannot update a SimulatorTaskProfile");
							//log warning
						}
					}else{
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_UPDATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, null, "Cannot update a SimulatorTaskProfile");
						//log warning
					}
				}
			}
			*/
			//RESOURCE REQUIREMENT OVERRIDE

			if(override.getResourceRequirement() != null && !override.getResourceRequirement().isEmpty()){
				List resourceOverride = override.getResourceRequirement();
				List workingOverrideList = workingOverride.getResourceRequirement();
				workingOverrideList.removeAll(workingOverrideList);
				Iterator it = resourceOverride.iterator();
				while (it.hasNext()){
					ResourceRequirement resourceRequirement = (ResourceRequirement)it.next();
					createResourceRequirement(workingOverride, resourceRequirement);
				}
			}
		}
		LoggingUtil.traceEntry(this, "updateSimulationTaskOverride");

	}

	
//	/**
//	 * Updates the Simulator Task Override. The following will be updated:
//	 * - processing time
//	 * - availability
//	 *
//	 */
//	private void updateSimulationTaskOverride() {
//		LoggingUtil.traceEntry(this, "updateSimulationTaskOverride");
//		SimulationTaskOverride override = taskProfile.getSimulationTaskOverride();
//		SimulationTaskOverride workingOverride = workingCopy.getSimulationTaskOverride();
//		
//		if(override == null){
//			return;
//		}
//		if(workingOverride == null){
//			AddSimulationTaskOverrideToTaskProfileBOMCmd addOverrideCmd = new AddSimulationTaskOverrideToTaskProfileBOMCmd(workingCopy);
//			if(!addOverrideCmd.canExecute()){
//				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_UPDATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, null, "Cannot update a SimulatorTaskProfile");
//			}else{
//				try{
//					addOverrideCmd.execute();
//					workingOverride = (SimulationTaskOverride)addOverrideCmd.getObject();
//				}catch(RuntimeException ex){
//					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_UPDATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, ex, "Cannot update a SimulatorTaskProfile");
//				}
//			}
//		}
//		if(workingOverride != null){
//			//UPDATE Processing time
//			if(override.getProcessingTime() != null){
//				SimulationProcessingTime processingTime = override.getProcessingTime();
//				//if there is no processing time for this task, create one
//				if(workingOverride.getProcessingTime() == null){
//					AddSimulationProcessingTimeToSimulationTaskOverrideBOMCmd addProcTimeCmd = new 	AddSimulationProcessingTimeToSimulationTaskOverrideBOMCmd(workingOverride);
//					addProcTimeCmd.setTimeUnit(processingTime.getTimeUnit());
//					addProcTimeCmd.setSkew(processingTime.getSkew());
//					if(!addProcTimeCmd.canExecute()){
//						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_UPDATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, null, "Cannot create ProcessingTime for SimulatorTaskOverrideTask");
//					}else{
//						try {
//							addProcTimeCmd.execute();
//						} catch (RuntimeException e) {
//							//log warning
//							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_UPDATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, e, "Cannot create ProcessingTime for SimulatorTaskOverrideTask");
//						}
//					}
//				}else{
//					//if a processing time exists for this task, update it
//					UpdateSimulationProcessingTimeBOMCmd updateProcessingTimeCmd = new UpdateSimulationProcessingTimeBOMCmd(workingOverride.getProcessingTime());
//					updateProcessingTimeCmd.setTimeUnit(processingTime.getTimeUnit());
//					updateProcessingTimeCmd.setSkew(processingTime.getSkew());
//					if(!updateProcessingTimeCmd.canExecute()){
//						//log warning
//						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_UPDATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, null, "Cannot update ProcessingTime for SimulatorTaskOverrideTask");	
//					}else{
//						try {
//							updateProcessingTimeCmd.execute();
//						} catch (RuntimeException e) {
//							//log warning
//							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_UPDATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, e, "Cannot update ProcessingTime for SimulatorTaskOverrideTask");
//						}
//					}
//				}
//				if(processingTime.getProcessingTime() != null){
//					if(workingOverride.getProcessingTime().getProcessingTime() != null){
//						ValueSpecification workingValueSpec = workingOverride.getProcessingTime().getProcessingTime();
//						UpdateValueSpecificationRule updateValueSpecRule = new UpdateValueSpecificationRule();
//						updateValueSpecRule.setImportSession(getImportSession());
//						updateValueSpecRule.setProjectName(getProjectName());
//						updateValueSpecRule.setSource(processingTime.getProcessingTime());
//						updateValueSpecRule.setWorkingCopy(workingValueSpec);
//						updateValueSpecRule.invoke();
//					}
//				}
//			}
//			//UPDATE AVAILABILITY
//			if(override.getAvailability() != null){
//				//find in the snapshot the TimeIntervals  that was created for this override availability
//				NamedElement namedElement = getObjectInSnapshot(override.getAvailability());
//				if(namedElement instanceof TimeIntervals){
//					TimeIntervals workingTimeIntervals = (TimeIntervals)namedElement;
//					//TODO create the OwnedAvailability.. not used by the tool...
//					UpdateSimulationTaskOverrideBOMCmd updateOverrideCmd = new UpdateSimulationTaskOverrideBOMCmd(workingOverride);
//					updateOverrideCmd.setAvailability(workingTimeIntervals);
//					if(updateOverrideCmd.canExecute()){
//						try{
//							updateOverrideCmd.execute();
//						}catch(RuntimeException e){
//							//log warning  
//							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_UPDATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, e, "Cannot update ProcessingTime for SimulatorTaskOverrideTask");
//						}
//					}else{
//						//log warning.. could not update
//						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_UPDATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, null, "Cannot update ProcessingTime for SimulatorTaskOverrideTask");
//					}
//				}
//					if(override.getAvailability().getRecurringTimeIntervals() != null && !override.getAvailability().getRecurringTimeIntervals().isEmpty()){
//					//}
//					//if(override.getRecurringTimeIntervals() != null && !override.getRecurringTimeIntervals().isEmpty()){
//						List recurringIntervalsList = override.getAvailability().getRecurringTimeIntervals();
//						Iterator it = recurringIntervalsList.iterator();
//						RecurringTimeIntervals workingIntervals = null;
//						RecurringTimeIntervals recurringTimeIntervals = null;
//						while(it.hasNext()){
//							recurringTimeIntervals = (RecurringTimeIntervals)it.next();
//							createTimeIntervals(override, recurringTimeIntervals);
//						}
//					}
//				//}
//			}
//			//RESOURCE REQUIREMENT OVERRIDE
//			if(override.getResourceRequirement() != null && !override.getResourceRequirement().isEmpty()){
//				List resourceOverride = override.getResourceRequirement();
//				List workingOverrideList = workingOverride.getResourceRequirement();
//				workingOverrideList.removeAll(workingOverrideList);
//				Iterator it = resourceOverride.iterator();
//				while (it.hasNext()){
//					ResourceRequirement resourceRequirement = (ResourceRequirement)it.next();
////					ResourceRequirement workingResourceRequirement = getResourceRequirement(workingOverrideList, resourceRequirement);
////					if(workingResourceRequirement != null){
//////						TODO Consider removing the existing resource requirements first
////						if(resourceRequirement.getTimeRequired() != null){
////							UpdateResourceRequirementBOMCmd updateResReqCmd = new UpdateResourceRequirementBOMCmd(workingResourceRequirement);
////							updateResReqCmd.setTimeRequired(resourceRequirement.getTimeRequired());
////							if(updateResReqCmd.canExecute()){
////								try{
////									updateResReqCmd.execute();
////								}catch(RuntimeException ex){
////										//log warning
////								}
////							}else{
////								//log warning...
////							}
////						}						
////						//update ResourceQuantity.
////						
////					}else{
//						createResourceRequirement(workingOverride, resourceRequirement);
////					}
//				}
//			}
//		}
//		LoggingUtil.traceEntry(this, "updateSimulationTaskOverride");
//	}


	/**
	 * Returns a resourceRequirement in the in the list of TaskOverride ResourceRequirements with the same canonical name as the 
	 * <code>resRequirement</code>
	 * @param resourceRequirements
	 * @param resRequirement
	 */
	private ResourceRequirement getResourceRequirement(List resourceRequirements, ResourceRequirement resRequirement){
		LoggingUtil.traceEntry(this, "getResourceRequirement");
		ResourceRequirement thisRequirement = null;
		if(resourceRequirements != null && !resourceRequirements.isEmpty()){
			Iterator it = resourceRequirements.iterator();
			while(it.hasNext()){
				thisRequirement = (ResourceRequirement)it.next();
			//	if(BOMUtil.getCanonicalName(thisRequirement, simulationContext).equals(BOMUtil.getCanonicalName(resRequirement, simulationContext))){
				if(thisRequirement.getName().equals(resRequirement.getName())){
					LoggingUtil.traceExit(this, "getResourceRequirement");
					return thisRequirement;
				}
			}
		}
		LoggingUtil.traceExit(this, "getResourceRequirement");
		return null;
	}

	private void createTimeIntervals(SimulationTaskOverride workingOverride, RecurringTimeIntervals timeIntervals){
		LoggingUtil.traceEntry(this, "createTimeIntervals");
		
		AddRecurringTimeIntervalsToSimulationTaskOverrideBOMCmd addCmd = new AddRecurringTimeIntervalsToSimulationTaskOverrideBOMCmd(workingOverride);
		if(addCmd.canExecute()){
			try {
				addCmd.execute();
				RecurringTimeIntervals workingIntervals = (RecurringTimeIntervals) addCmd.getObject();
				//UpdateRecurringTimeIntervalsRule updateRule = new UpdateRecurringTimeIntervalsRule();
				UpdateSimRecurringTimeIntervalsRule updateRule = new UpdateSimRecurringTimeIntervalsRule();
				updateRule.setImportSession(this.getImportSession());
				updateRule.setProjectName(getProjectName());
				updateRule.setSource(timeIntervals);
				updateRule.setWorkingCopy(workingIntervals);
				updateRule.invoke();
			} catch (RuntimeException e) {
				//could not update the TimeIntervals for the task working override
			}
		}
		LoggingUtil.traceExit(this, "createTimeIntervals");
	}


	private ResourceRequirement createResourceRequirement(SimulationTaskOverride workingOverride, ResourceRequirement requirement){
		LoggingUtil.traceEntry(this, "createResourceRequirement");
		
		final Double defaultQuantity = new Double(1);
		final String defaultUnit = BOMUtil.getDefaultUnitOfMeasureName();
		
		AddUpdateResourceRequirementBOMCmd cmd = null;
		
		Object model = null;
		String unitOfMeasure = null;
		ValueSpecification value = null;
		Double quantityValue = null;
		ResourceQuantity resourceQuantity = null;
		Action task = workingCopy.getTask();
		
		// Default Tool Behavior
		quantityValue = defaultQuantity;
		unitOfMeasure = defaultUnit;

		
		if (requirement instanceof IndividualResourceRequirement) {
			cmd = new AddIndividualResourceRequirementToSimulationTaskOverrideBOMCmd(workingOverride);
			//model  = getModelFromContext(((IndividualResourceRequirement)requirement).getIndividualResource());
			model = SimulationUtil.getObjectInSnapshot(((IndividualResourceRequirement)requirement).getIndividualResource(),getProjectName(),getImportSession().getContext());
			if (model != null && model instanceof IndividualResource) {
				((AddIndividualResourceRequirementToSimulationTaskOverrideBOMCmd)cmd).setIndividualResource((IndividualResource)model);
			}
		} else if (requirement instanceof BulkResourceRequirement) {
				cmd = new AddBulkResourceRequirementToSimulationTaskOverrideBOMCmd(workingOverride);
				model = SimulationUtil.getObjectInSnapshot(((BulkResourceRequirement)requirement).getBulkResource(),getProjectName(),getImportSession().getContext());
				//model = BOMUtil.getType(this.getProjectName(), ((BulkResourceRequirement)requirement).getBulkResource(), getImportSession().getContext() );
				//model  = getModelFromContext(((BulkResourceRequirement)requirement).getBulkResource());
				if (model != null && model instanceof BulkResource) {
					((AddBulkResourceRequirementToSimulationTaskOverrideBOMCmd)cmd).setBulkResource((BulkResource)model);
				}	
				// Resource Quantiry
				resourceQuantity = ((BulkResourceRequirement)requirement).getRequiredQuantity();
				if (resourceQuantity != null) {		
					unitOfMeasure = resourceQuantity.getUnitOfMeasure();
					if (unitOfMeasure.compareTo("") == 0) {
						unitOfMeasure = defaultUnit;
					}
					value = resourceQuantity.getQuantity();
					if (value instanceof LiteralReal) {
						quantityValue = ((LiteralReal)value).getValue();
					}
				}
			} else if (requirement instanceof RequiredRole) {
				cmd = new AddRequiredRoleToSimulationTaskOverrideBOMCmd(workingOverride);
				model = SimulationUtil.getObjectInSnapshot(((RequiredRole)requirement).getRole(),getProjectName(),getImportSession().getContext());

				//model = BOMUtil.getType(this.getProjectName(),((RequiredRole)requirement).getRole() , getImportSession().getContext() );
				//model  = getModelFromContext(((RequiredRole)requirement).getRole());
				if (model != null && model instanceof Role) {
					((AddRequiredRoleToSimulationTaskOverrideBOMCmd)cmd).setRole((Role)model);
				}	
				// Resource Quantiry
				resourceQuantity = ((RequiredRole)requirement).getRequiredQuantity();
				if (resourceQuantity != null) {			
					if (unitOfMeasure.compareTo("") == 0) {
						unitOfMeasure = defaultUnit;
					}
					value = resourceQuantity.getQuantity();
					if (value instanceof LiteralReal) {
						quantityValue = ((LiteralReal)value).getValue();
					}
				}
			}
			if(requirement.getResourceType() != null){
				Type resourceType = (Type)SimulationUtil.getObjectInSnapshot(requirement.getResourceType(),getProjectName(),getImportSession().getContext());
			
				cmd.setResourceType(resourceType);
			}			
			cmd.setName(requirement.getName());
			

			String timeRequired = requirement.getTimeRequired();
			//TODO check with Alan... timeRequired format
			if (timeRequired == null || timeRequired.equals("")) {
				timeRequired = "PT0S";
			}
			cmd.setTimeRequired(timeRequired);
		
			// Create the Requirement
			if (cmd.canExecute()) {
				try {
					cmd.execute();
				} catch (RuntimeException e) {
					LoggingUtil.trace(this, "addResourceRequirement", 
						IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
				}
			} else {
				LoggingUtil.trace(this, "addResourceRequirement", 
					IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);		
			}			
		
			// Update the Requirement
			ResourceRequirement workingRequirement = (ResourceRequirement)cmd.getObject();
			Object quantity = null;
			AddUpdateResourceQuantityBOMCmd quantityCmd = null;
			if (workingRequirement != null && unitOfMeasure != null) {		
				if (workingRequirement instanceof BulkResourceRequirement) {				
					quantityCmd = new AddResourceQuantityToBulkResourceRequirementBOMCmd((BulkResourceRequirement)workingRequirement);
				} else if (workingRequirement instanceof RequiredRole) {
					quantityCmd = new AddResourceQuantityToRequiredRoleBOMCmd((RequiredRole)workingRequirement);
				}
				if (quantityCmd != null && unitOfMeasure != null) {
					quantityCmd.setUnitOfMeasure(unitOfMeasure);
					if (quantityCmd.canExecute()) {
						try {
							quantityCmd.execute();
						} catch (RuntimeException e) {
							LoggingUtil.trace(this, "addResourceRequirement", 
								IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
						}
					} else {
						LoggingUtil.trace(this, "addResourceRequirement", 
					IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);		
					}
				}
				if (quantityCmd != null) {
					quantity = quantityCmd.getObject();				
		
				// if (quantity != null && quantity instanceof ResourceQuantity && quantityValue != null) {
					AddLiteralRealToResourceQuantityBOMCmd addRealCmd = new AddLiteralRealToResourceQuantityBOMCmd((ResourceQuantity)quantity);
					addRealCmd.setValue(quantityValue);
					if (addRealCmd.canExecute()) {
						try {
							addRealCmd.execute();
						} catch (RuntimeException e) {
							LoggingUtil.trace(this, "addResourceRequirement", 
									IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
						}
					} else {
						LoggingUtil.trace(this, "addResourceRequirement", 
					IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);		
					}
				}
			}
		
			// Update RequiredRole			
			if (requirement instanceof RequiredRole && workingRequirement instanceof RequiredRole) {
				ScopeValue scopeValue = null;
				ScopeValue scopeValueWorkingCopy = null;
				Object newScopeDimension = null;
				AddScopeValueToRequiredRoleBOMCmd scopeCmd = null;
				Iterator scopeIterator = ((RequiredRole) requirement).getRequiredScope().iterator();
				EList scopeValueList = null;
				ValueSpecification thisValue = null;
				AddUpdateObjectCommand updateValueCmd = null;
				UpdateValueSpecificationRule rule  = null;
				Object newDomainModel = null;
				while (scopeIterator.hasNext()) {
					scopeValue = (ScopeValue) scopeIterator.next();
					scopeCmd = new AddScopeValueToRequiredRoleBOMCmd((RequiredRole)workingRequirement);
					// Set Scope dimension
					ScopeDimension dimension = scopeValue.getScopeDimension();
					newScopeDimension = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_SCOPE_DIMNENSION).get(dimension);
					if (newScopeDimension instanceof ScopeDimension) {
						scopeCmd.setScopeDimension((ScopeDimension)newScopeDimension);	
					}
					if (scopeCmd.canExecute()) {
						try {
							scopeCmd.execute();
							scopeValueWorkingCopy = (ScopeValue)scopeCmd.getObject();
						} catch (RuntimeException e) {
							LoggingUtil.trace(this, "addResourceRequirement", 
									IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
						}
					} else {
						LoggingUtil.trace(this, "addResourceRequirement", 
						IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);	
					}
				
					// Set Scope Value					
					scopeValueList = scopeValue.getValue();
					if (scopeValueList.size() > 0 && scopeValueWorkingCopy != null) {
						thisValue = (ValueSpecification) scopeValueList.get(0);
						if (thisValue instanceof LiteralNull) {
							updateValueCmd = new AddLiteralNullToScopeValueBOMCmd(scopeValueWorkingCopy); 
						} else if (thisValue instanceof LiteralSpecification) {
							updateValueCmd = new AddLiteralStringToScopeValueBOMCmd(scopeValueWorkingCopy);
						} else if(thisValue instanceof StructuredOpaqueExpression){
							updateValueCmd = new AddStructuredOpaqueExpressionToScopeValueBEXCmd(scopeValueWorkingCopy);	 
						} else if (thisValue instanceof OpaqueExpression) {
							// [Expression] Attrbibute constant value
							updateValueCmd = new AddOpaqueExpressionToScopeValueBOMCmd(scopeValueWorkingCopy);
						} else if (thisValue instanceof InstanceValue) {
							updateValueCmd = new AddInstanceValueToScopeValueBOMCmd(scopeValueWorkingCopy);
						}
						if (updateValueCmd.canExecute()) {
							try {
								updateValueCmd.execute();
							} catch (RuntimeException e) {
								LoggingUtil.trace(this, "addResourceRequirement", 
									IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
							}
						} else {
							LoggingUtil.trace(this, "addResourceRequirement", 
					IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);	
						}
						//	After creation
						newDomainModel = updateValueCmd.getObject();
						if (newDomainModel != null) {
							rule = new UpdateValueSpecificationRule();
							rule.setSource(value);
							rule.setWorkingCopy(newDomainModel);
							rule.setProjectName(getProjectName());
							rule.setImportSession(getImportSession());
							rule.invoke();
						}						
					}
				}
			}
		LoggingUtil.traceExit(this, "createResourceRequirement");		
		return workingRequirement;
			
	}

	
	/**
	 * Add OperationalTimes' processing times to simulation overrides
	 * @param workingOverride
	 * @param vs
	 */
	private void addProcessingTimeToSimOverrides(SimulationTaskOverride workingOverride, ValueSpecification vs){
		LoggingUtil.traceEntry(this, "addProcessingTimeToSimOverrides");		
		// NOTE: There is no support of distribution type (for processingTime) from ADF import
		// ONLY LiteralDuration is supported from ADF import (for procssingTime)
		if(vs instanceof LiteralDuration){
			// if the processingTime is a literal Duration
			AddLiteralDurationProcessingTimeToSimulationTaskOverrideBOMCmd cmd = 
				new AddLiteralDurationProcessingTimeToSimulationTaskOverrideBOMCmd(workingOverride);							
			cmd.setValue(((LiteralDuration)vs).getValue());
			executeCmd(cmd);
		} // NOTE: There is no support of distribution type (for processingTime) from ADF import
		// ONLY LiteralDuration is supported from ADF import (for processingTime)
		// The following is the "extra" implementation for the distribution types. 
		// This implementation will NOT be executed and used at all but is implemented and left here as general purpose. 
		// This part (distribution type) needs to be retested once the distribution type is supported by ADF or other types of imports
		else if (vs instanceof StructuredDuration) {			
			StructuredDuration structDur = (StructuredDuration) vs;
			AddStructuredDurationProcessingTimeToSimulationTaskOverrideBOMCmd addSDCmd = 
					new AddStructuredDurationProcessingTimeToSimulationTaskOverrideBOMCmd(workingOverride);
			//defect 90411
			if (structDur.getSkew() != null) {
				addSDCmd.setSkew(structDur.getSkew());
			} else {
				addSDCmd.setSkew(new Double(0));
			}
			if (structDur.getTimeUnit() != null) {				
				addSDCmd.setTimeUnit(structDur.getTimeUnit());			
			}
			executeCmd(addSDCmd);
			StructuredDuration newDomainStructDuration = (StructuredDuration) addSDCmd.getObject();
			
			// get the source duration value
			ValueSpecification durationVal = ((StructuredDuration)vs).getDurationValue();
			
			if(durationVal != null && durationVal instanceof Distribution){						
				if(durationVal instanceof NormalDistribution){
					AddNormalDistributionToStructuredDurationBOMCmd cmd = 
						new AddNormalDistributionToStructuredDurationBOMCmd(newDomainStructDuration);
					cmd.setMean(((NormalDistribution)durationVal).getMean());
					cmd.setStd(((NormalDistribution)durationVal).getStd());
					executeCmd(cmd);	
				}	
				else if(durationVal instanceof LognormalDistribution){
					AddLognormalDistributionToStructuredDurationBOMCmd cmd = 
						new AddLognormalDistributionToStructuredDurationBOMCmd(newDomainStructDuration);
					cmd.setMean(((LognormalDistribution)durationVal).getMean());
					cmd.setStd(((LognormalDistribution)durationVal).getStd());
					executeCmd(cmd);							
				}
				else if(durationVal instanceof PoissonDistribution){
					AddPoissonDistributionToStructuredDurationBOMCmd cmd = 
						new AddPoissonDistributionToStructuredDurationBOMCmd(newDomainStructDuration);
					cmd.setMean(((PoissonDistribution)durationVal).getMean());
					executeCmd(cmd);							
				}
				else if(durationVal instanceof GammaDistribution){
					AddGammaDistributionToStructuredDurationBOMCmd cmd = 
						new AddGammaDistributionToStructuredDurationBOMCmd(newDomainStructDuration);
					cmd.setMean(((GammaDistribution)durationVal).getMean());
					cmd.setStd(((GammaDistribution)durationVal).getStd());
					executeCmd(cmd);
				}
				else if(durationVal instanceof ExponentialDistribution){
					AddExponentialDistributionToStructuredDurationBOMCmd cmd = 
						new AddExponentialDistributionToStructuredDurationBOMCmd(newDomainStructDuration);
					cmd.setMean(((ExponentialDistribution)durationVal).getMean());
					executeCmd(cmd);			
				}
				else if(durationVal instanceof UniformDistribution){
					AddUniformDistributionToStructuredDurationBOMCmd cmd = 
						new AddUniformDistributionToStructuredDurationBOMCmd(newDomainStructDuration);
					executeCmd(cmd);
					UniformDistribution newDomainPDist = (UniformDistribution)cmd.getObject();
					if(newDomainPDist != null){
						AddLiteralRealMinValueToUniformDistributionBOMCmd cmd2 = 
							new AddLiteralRealMinValueToUniformDistributionBOMCmd(newDomainPDist);
						cmd2.setValue(((LiteralReal)((UniformDistribution)durationVal).getMinValue()).getValue());
						executeCmd(cmd2);
			
						AddLiteralRealMaxValueToUniformDistributionBOMCmd cmd3 = 
							new AddLiteralRealMaxValueToUniformDistributionBOMCmd(newDomainPDist);
						cmd3.setValue(((LiteralReal)((UniformDistribution)durationVal).getMaxValue()).getValue());
						executeCmd(cmd3);			
					}
				}
				else if(durationVal instanceof BernoulliDistribution){
					AddBernoulliDistributionToStructuredDurationBOMCmd cmd = 
						new AddBernoulliDistributionToStructuredDurationBOMCmd(newDomainStructDuration);
					cmd.setProbability(((BernoulliDistribution)durationVal).getProbability());
					executeCmd(cmd);
				}else if(durationVal instanceof BinomialDistribution){
					AddBinomialDistributionToStructuredDurationBOMCmd cmd = 
						new AddBinomialDistributionToStructuredDurationBOMCmd(newDomainStructDuration);
					cmd.setProbability(((BinomialDistribution)durationVal).getProbability());
					executeCmd(cmd);
				}else if(durationVal instanceof RandomList){
					AddRandomListToStructuredDurationBOMCmd cmd = 
						new AddRandomListToStructuredDurationBOMCmd(newDomainStructDuration);								
					executeCmd(cmd);
					RandomList newDomainRdmList = (RandomList)cmd.getObject();
					if(newDomainRdmList != null){
						BtCompoundCommand compoundCmd = new BtCompoundCommand();
						for (Iterator it = ((RandomList)durationVal).getListElement().iterator(); it.hasNext();) {
							AddListElementToRandomListBOMCmd cmd2 =
								new AddListElementToRandomListBOMCmd(newDomainRdmList);
							compoundCmd.append(cmd2);
							ListElement element = (ListElement)cmd2.getObject();
							ListElement nextElement = (ListElement)it.next();									
							LiteralReal literalReal = (LiteralReal)nextElement.getValue();
							AddLiteralRealToListElementBOMCmd cmd3 = new AddLiteralRealToListElementBOMCmd(element);
							cmd3.setValue(literalReal.getValue());
							compoundCmd.append(cmd3);
						}
						executeCmd(compoundCmd);
					}
							
				}else if(durationVal instanceof WeightedList){
					AddWeightedListToStructuredDurationBOMCmd cmd3 = 
						new AddWeightedListToStructuredDurationBOMCmd(newDomainStructDuration);								
					executeCmd(cmd3);
					WeightedList newDomainWtList = (WeightedList)cmd3.getObject();
					if(newDomainWtList != null){
						BtCompoundCommand compoundCmd = new BtCompoundCommand();
						for (Iterator it = ((WeightedList)durationVal).getWeightedListElement().iterator(); it.hasNext();) {
							WeightedListElement nextElement = (WeightedListElement)it.next();	
							AddWeightedListElementToWeightedListBOMCmd cmd4 =
								new AddWeightedListElementToWeightedListBOMCmd(newDomainWtList);
							cmd4.setProbability(nextElement.getProbability());										
							compoundCmd.append(cmd4);
							WeightedListElement element = (WeightedListElement)cmd4.getObject();																	
							LiteralReal literalRealValue = (LiteralReal)nextElement.getValue();
							AddLiteralRealToListElementBOMCmd cmd5 = new AddLiteralRealToListElementBOMCmd(element);
							cmd5.setValue(literalRealValue.getValue());
							compoundCmd.append(cmd5);
						}
						executeCmd(compoundCmd);
					}
				// support of the new distributions types in L002643					
				} else if (durationVal instanceof BetaDistribution) {
					AddBetaDistributionToStructuredDurationBOMCmd cmd = new AddBetaDistributionToStructuredDurationBOMCmd(newDomainStructDuration);
					BetaDistribution betaDist = (BetaDistribution) durationVal;				
					cmd.setA(betaDist.getA());
					cmd.setB(betaDist.getB());
					executeCmd(cmd);		
				} else if (durationVal instanceof ErlangRNDistribution) {
					AddErlangRNDistributionToStructuredDurationBOMCmd cmd = new AddErlangRNDistributionToStructuredDurationBOMCmd(newDomainStructDuration);
					ErlangRNDistribution erlangDist = (ErlangRNDistribution) durationVal;				
					cmd.setExpmean(erlangDist.getExpmean());
					cmd.setK(erlangDist.getK());
					executeCmd(cmd);				
				} else if (durationVal instanceof JohnsonRNDistribution) {
					AddJohnsonRNDistributionToStructuredDurationBOMCmd cmd = new AddJohnsonRNDistributionToStructuredDurationBOMCmd(newDomainStructDuration);
					JohnsonRNDistribution johnsonDist = (JohnsonRNDistribution) durationVal;				
					cmd.setGamma(johnsonDist.getGamma());
					cmd.setDelta(johnsonDist.getDelta());								
					JohnsonType johnsonType = johnsonDist.getJohnsonType();
					cmd.setJohnsonType(johnsonType);
					cmd.setLambda(johnsonDist.getLambda());
					cmd.setXi(johnsonDist.getXi());
					executeCmd(cmd);				
				} else if (durationVal instanceof TriangularRNDistribution) {
					AddTriangularRNDistributionToStructuredDurationBOMCmd cmd = new AddTriangularRNDistributionToStructuredDurationBOMCmd(newDomainStructDuration);
					TriangularRNDistribution triangularDist = (TriangularRNDistribution) durationVal;
					cmd.setMin(triangularDist.getMin());
					cmd.setMax(triangularDist.getMax());
					cmd.setMode(triangularDist.getMode());
					executeCmd(cmd);
				} else if (durationVal instanceof WeibullRNDistribution) {
					AddWeibullRNDistributionToStructuredDurationBOMCmd cmd = new AddWeibullRNDistributionToStructuredDurationBOMCmd(newDomainStructDuration);				
					WeibullRNDistribution weibullDist = (WeibullRNDistribution) durationVal;				
					cmd.setAlpha(weibullDist.getAlpha());
					cmd.setBeta(weibullDist.getBeta());
					executeCmd(cmd);
				} else if (durationVal instanceof ContinuousRNDistribution) {
					ContinuousRNDistribution contDist = (ContinuousRNDistribution) durationVal;
					AddContinuousRNDistributionToStructuredDurationBOMCmd cmd = new AddContinuousRNDistributionToStructuredDurationBOMCmd(newDomainStructDuration);			
					List cs = contDist.getC();
					List vals = contDist.getVal();
					
					if (cs != null && !cs.isEmpty()) {					
						for (int i=0; i<cs.size(); i++) {
							cmd.addC((Double) cs.get(i));
						}
					}
					
					if (vals != null && !vals.isEmpty()) {					
						for (int i=0; i<vals.size(); i++) {
							cmd.addVal((Double) vals.get(i));
						}
					}				
					executeCmd(cmd);		
				} else{
					LoggingUtil.trace(this,"addProcessingTimeToSimOverrides", "unknown destribution type");				
				}
			}	// end else-if for PDistribution			
		} else {
			LoggingUtil.trace(this,"addProcessingTimeToSimOverrides", "unknown data type for processingTime");			
		}		
		LoggingUtil.traceExit(this, "addProcessingTimeToSimOverrides");	
	}
	
	
	/**
	 * Update OperationalTimes' processing times in simulation overrides
	 * @param newDomainOpTimes
	 * @param vs
	 */
	private void updateProcessingTimeInSimOverrides(SimulationTaskOverride workingOverride, SimulationTaskOverride srcOverride){
		LoggingUtil.traceEntry(this, "updateProcessingTimeInSimOverrides");
		
		ValueSpecification srcProcessingTime = srcOverride.getProcessingTime();
		ValueSpecification wkCopyProcessingTime = workingOverride.getProcessingTime();
		
		// if the srcProcessingTime is the same type as the wkCopyProcessingTime, update their contents.
		// Otherwise, the wkCopyProcessingTime is removed and a new one is added to the wkCopyProcessingTime base on the srcProcessingTime
		if (srcProcessingTime instanceof LiteralDuration && wkCopyProcessingTime instanceof LiteralDuration) {										
			UpdateValueSpecificationRule updateValueSpecRule = new UpdateValueSpecificationRule();
			updateValueSpecRule.setImportSession(getImportSession());
			updateValueSpecRule.setProjectName(getProjectName());				
			updateValueSpecRule.setSource(srcProcessingTime);
			updateValueSpecRule.setWorkingCopy(wkCopyProcessingTime);
			updateValueSpecRule.invoke();			
		} else if ((srcProcessingTime instanceof LiteralDuration && wkCopyProcessingTime instanceof StructuredDuration) ||
				   (srcProcessingTime instanceof StructuredDuration && wkCopyProcessingTime instanceof LiteralDuration)) {
			// if the src and working copy has different types, remove the processingTime in workingCopy and recreate it
			RemoveValueSpecificationBOMCmd removeCmd = new RemoveValueSpecificationBOMCmd(workingOverride.getProcessingTime());
			executeCmd(removeCmd);
			addProcessingTimeToSimOverrides(workingOverride, srcOverride.getProcessingTime());			
		} // NOTE: There is no support of distribution type (for processingTime) from ADF import
		// ONLY LiteralDuration is supported from ADF import (for processingTime)
		// The following is the "extra" implementation for the distribution types. 
		// This implementation will NOT be executed and used at all but is implemented and left here as general purpose. 
		// This part (distribution type) needs to be retested once the distribution type is supported by ADF or other types of imports 
		else if (srcProcessingTime instanceof StructuredDuration && wkCopyProcessingTime instanceof StructuredDuration) {			
			StructuredDuration srcStructDur = (StructuredDuration) srcProcessingTime;
			StructuredDuration wkCopyStructDur = (StructuredDuration) wkCopyProcessingTime;
			
			UpdateStructuredDurationBOMCmd updateProcessingTimeCmd = new UpdateStructuredDurationBOMCmd(wkCopyStructDur);
			updateProcessingTimeCmd.setTimeUnit(srcStructDur.getTimeUnit());
			// defect 90411
			if(srcStructDur.getSkew() != null){
				updateProcessingTimeCmd.setSkew(srcStructDur.getSkew());	
			}else{
				updateProcessingTimeCmd.setSkew(new Double(0));
			}
			
			if(!updateProcessingTimeCmd.canExecute()){
				//log warning
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_UPDATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, null, "Cannot update a SimulatorTaskProfile");	
			}else{
				try {
					updateProcessingTimeCmd.execute();
				} catch (RuntimeException e) {
					//log warning
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TASK_PROFILE_UPDATE_EXCEPTION, new String[]{taskProfile.getTask().getName()}, null, "Cannot update a SimulatorTaskProfile");
				}
			}			
			ValueSpecification srcDurationVal = srcStructDur.getDurationValue();
			ValueSpecification wkCopyDurationVal = wkCopyStructDur.getDurationValue();
			
			if (srcDurationVal != null) {
				updateDurationValInSimOverrides(srcDurationVal, wkCopyDurationVal, wkCopyStructDur, workingOverride, srcOverride);
			} else {
				LoggingUtil.trace(this, "updateProcessingTimeInSimOverrides", "the durationVal is empty" );				
			}
		} else {
			LoggingUtil.trace(this, "updateProcessingTimeInSimOverrides", "unknown data type for processingTime" );
		}
		
		LoggingUtil.traceExit(this, "updateProcessingTimeInSimOverrides");
	}
	
	/**
	 * Update Duration Value in OperationalTimes' processing times
	 * @param srcDurationVal
	 * @param wkCopyDurationVal
	 * @param wkCopyStructDur
	 * @param workingOverride
	 * @param srcOverride	 
	 */
	private void updateDurationValInSimOverrides(ValueSpecification srcDurationVal, ValueSpecification wkCopyDurationVal, 
			StructuredDuration wkCopyStructDur, SimulationTaskOverride workingOverride, SimulationTaskOverride srcOverride){
		// NOTE: There is no support of distribution type (for processingTime) from ADF import
		// ONLY LiteralDuration is supported from ADF import (for processingTime)
		// The following is the "extra" implementation for the distribution types. 
		// This implementation will NOT be executed and used at all but is implemented and left here as general purpose.
		// This part (distribution type) needs to be retested once the distribution type is supported by ADF or other types of imports 
		LoggingUtil.traceEntry(this, "updateDurationValInSimOverrides");
		
		if (srcDurationVal instanceof NormalDistribution && wkCopyDurationVal instanceof NormalDistribution) {			
			UpdateNormalDistributionBOMCmd cmd = new UpdateNormalDistributionBOMCmd((NormalDistribution)wkCopyDurationVal);
			cmd.setMean(((NormalDistribution)srcDurationVal).getMean());
			cmd.setStd(((NormalDistribution)srcDurationVal).getStd());
			executeCmd(cmd);			
		} else if (srcDurationVal instanceof LognormalDistribution && wkCopyDurationVal instanceof LognormalDistribution) {
			UpdateLognormalDistributionBOMCmd cmd = 
				new UpdateLognormalDistributionBOMCmd((LognormalDistribution)wkCopyDurationVal);
			cmd.setMean(((LognormalDistribution)srcDurationVal).getMean());
			cmd.setStd(((LognormalDistribution)srcDurationVal).getStd());
			executeCmd(cmd);			
		} else if (srcDurationVal instanceof PoissonDistribution && wkCopyDurationVal instanceof PoissonDistribution) {
			UpdatePoissonDistributionBOMCmd cmd = 
				new UpdatePoissonDistributionBOMCmd((PoissonDistribution)wkCopyDurationVal);
			cmd.setMean(((PoissonDistribution)srcDurationVal).getMean());
			executeCmd(cmd);			
		} else if (srcDurationVal instanceof GammaDistribution && wkCopyDurationVal instanceof GammaDistribution) {
			UpdateGammaDistributionBOMCmd cmd = 
				new UpdateGammaDistributionBOMCmd((GammaDistribution)wkCopyDurationVal);
			cmd.setMean(((GammaDistribution)srcDurationVal).getMean());
			cmd.setStd(((GammaDistribution)srcDurationVal).getStd());
			executeCmd(cmd);			
		} else if (srcDurationVal instanceof ExponentialDistribution && wkCopyDurationVal instanceof ExponentialDistribution) {			
			UpdateExponentialDistributionBOMCmd cmd = 
				new UpdateExponentialDistributionBOMCmd((ExponentialDistribution)wkCopyDurationVal);
			cmd.setMean(((ExponentialDistribution)srcDurationVal).getMean());
			executeCmd(cmd);			
		} else if (srcDurationVal instanceof UniformDistribution && wkCopyDurationVal instanceof UniformDistribution) {			
				LiteralReal srcMinVal = (LiteralReal)((UniformDistribution)srcDurationVal).getMinValue();				
				LiteralReal srcMaxVal = (LiteralReal)((UniformDistribution)srcDurationVal).getMaxValue();
				LiteralReal wkCopyMinVal = (LiteralReal)((UniformDistribution)wkCopyDurationVal).getMinValue();				
				LiteralReal wkCopyMaxVal = (LiteralReal)((UniformDistribution)wkCopyDurationVal).getMaxValue();
				UpdateLiteralRealBOMCmd cmd2 = 
					new UpdateLiteralRealBOMCmd(wkCopyMinVal);
				cmd2.setValue(srcMinVal.getValue());
				executeCmd(cmd2);	
				UpdateLiteralRealBOMCmd cmd3 = 
					new UpdateLiteralRealBOMCmd(wkCopyMaxVal);
				cmd3.setValue(srcMaxVal.getValue());
				executeCmd(cmd3);			
		} else if (srcDurationVal instanceof BernoulliDistribution && wkCopyDurationVal instanceof BernoulliDistribution) {			
			UpdateBernoulliDistributionBOMCmd cmd = 
				new UpdateBernoulliDistributionBOMCmd((BernoulliDistribution)wkCopyDurationVal);
			cmd.setProbability(((BernoulliDistribution)srcDurationVal).getProbability());
			executeCmd(cmd);			
		} else if (srcDurationVal instanceof BinomialDistribution && wkCopyDurationVal instanceof BinomialDistribution) {
			UpdateBinomialDistributionBOMCmd cmd = 
				new UpdateBinomialDistributionBOMCmd((BinomialDistribution)wkCopyDurationVal);
			cmd.setProbability(((BinomialDistribution)srcDurationVal).getProbability());
			executeCmd(cmd);			
		} else if (srcDurationVal instanceof RandomList && wkCopyDurationVal instanceof RandomList) {			
			// Instead of going thru each element in the random list and update them
			// the safest way is to remove the random list, then re-create the random list and populate the elements inside the random list
			RemoveValueSpecificationBOMCmd removeRndListCmd = new RemoveValueSpecificationBOMCmd(wkCopyDurationVal);
			executeCmd(removeRndListCmd);
			
			AddRandomListToStructuredDurationBOMCmd cmd = 
				new AddRandomListToStructuredDurationBOMCmd(wkCopyStructDur);								
			executeCmd(cmd);
			RandomList newDomainRdmList = (RandomList)cmd.getObject();
			if(newDomainRdmList != null){
				BtCompoundCommand compoundCmd = new BtCompoundCommand();
				for (Iterator it = ((RandomList)srcDurationVal).getListElement().iterator(); it.hasNext();) {
					AddListElementToRandomListBOMCmd cmd2 =
						new AddListElementToRandomListBOMCmd(newDomainRdmList);
					compoundCmd.append(cmd2);
					ListElement element = (ListElement)cmd2.getObject();
					ListElement nextElement = (ListElement)it.next();									
					LiteralReal literalReal = (LiteralReal)nextElement.getValue();
					AddLiteralRealToListElementBOMCmd cmd3 = new AddLiteralRealToListElementBOMCmd(element);
					cmd3.setValue(literalReal.getValue());
					compoundCmd.append(cmd3);
				}
				executeCmd(compoundCmd);
			}
		} else if (srcDurationVal instanceof WeightedList && wkCopyDurationVal instanceof WeightedList) {
			// Instead of going thru each element in the weighted list and update them
			// the safest way is to remove the weighted list, then re-create the weighted list and populate the elements inside the weighted list
			RemoveValueSpecificationBOMCmd removeRndListCmd = new RemoveValueSpecificationBOMCmd(wkCopyDurationVal);
			executeCmd(removeRndListCmd);

			AddWeightedListToStructuredDurationBOMCmd cmd3 = 
				new AddWeightedListToStructuredDurationBOMCmd(wkCopyStructDur);								
			executeCmd(cmd3);
			WeightedList newDomainWtList = (WeightedList)cmd3.getObject();
			if(newDomainWtList != null){
				BtCompoundCommand compoundCmd = new BtCompoundCommand();
				for (Iterator it = ((WeightedList)srcDurationVal).getWeightedListElement().iterator(); it.hasNext();) {
					WeightedListElement nextElement = (WeightedListElement)it.next();	
					AddWeightedListElementToWeightedListBOMCmd cmd4 =
						new AddWeightedListElementToWeightedListBOMCmd(newDomainWtList);
					cmd4.setProbability(nextElement.getProbability());										
					compoundCmd.append(cmd4);
					WeightedListElement element = (WeightedListElement)cmd4.getObject();																	
					LiteralReal literalRealValue = (LiteralReal)nextElement.getValue();
					AddLiteralRealToListElementBOMCmd cmd5 = new AddLiteralRealToListElementBOMCmd(element);
					cmd5.setValue(literalRealValue.getValue());
					compoundCmd.append(cmd5);
				}
				executeCmd(compoundCmd);
			}
		// support of the new distributions types in L002643
		} else if (srcDurationVal instanceof BetaDistribution && wkCopyDurationVal instanceof BetaDistribution) {
			UpdateBetaDistributionBOMCmd cmd = new UpdateBetaDistributionBOMCmd((BetaDistribution) wkCopyDurationVal);
			BetaDistribution betaDist = (BetaDistribution) srcDurationVal;				
			cmd.setA(betaDist.getA());
			cmd.setB(betaDist.getB());
			executeCmd(cmd);		
		} else if (srcDurationVal instanceof ErlangRNDistribution && wkCopyDurationVal instanceof ErlangRNDistribution) {
			UpdateErlangRNDistributionBOMCmd cmd = new UpdateErlangRNDistributionBOMCmd((ErlangRNDistribution) wkCopyDurationVal);
			ErlangRNDistribution erlangDist = (ErlangRNDistribution) srcDurationVal;				
			cmd.setExpmean(erlangDist.getExpmean());
			cmd.setK(erlangDist.getK());
			executeCmd(cmd);				
		} else if (srcDurationVal instanceof JohnsonRNDistribution && wkCopyDurationVal instanceof JohnsonRNDistribution) {
			UpdateJohnsonRNDistributionBOMCmd cmd = new UpdateJohnsonRNDistributionBOMCmd((JohnsonRNDistribution) wkCopyDurationVal);
			JohnsonRNDistribution johnsonDist = (JohnsonRNDistribution) srcDurationVal;				
			cmd.setGamma(johnsonDist.getGamma());
			cmd.setDelta(johnsonDist.getDelta());								
			JohnsonType johnsonType = johnsonDist.getJohnsonType();
			cmd.setJohnsonType(johnsonType);
			cmd.setLambda(johnsonDist.getLambda());
			cmd.setXi(johnsonDist.getXi());
			executeCmd(cmd);				
		} else if (srcDurationVal instanceof TriangularRNDistribution && wkCopyDurationVal instanceof TriangularRNDistribution) {
			UpdateTriangularRNDistributionBOMCmd cmd = new UpdateTriangularRNDistributionBOMCmd((TriangularRNDistribution) wkCopyDurationVal);
			TriangularRNDistribution triangularDist = (TriangularRNDistribution)srcDurationVal;
			cmd.setMin(triangularDist.getMin());
			cmd.setMax(triangularDist.getMax());
			cmd.setMode(triangularDist.getMode());
			executeCmd(cmd);
		} else if (srcDurationVal instanceof WeibullRNDistribution && wkCopyDurationVal instanceof WeibullRNDistribution) {
			UpdateWeibullRNDistributionBOMCmd cmd = new UpdateWeibullRNDistributionBOMCmd((WeibullRNDistribution) wkCopyDurationVal);				
			WeibullRNDistribution weibullDist = (WeibullRNDistribution)srcDurationVal;				
			cmd.setAlpha(weibullDist.getAlpha());
			cmd.setBeta(weibullDist.getBeta());
			executeCmd(cmd);
		} else if (srcDurationVal instanceof ContinuousRNDistribution && wkCopyDurationVal instanceof ContinuousRNDistribution) {
			ContinuousRNDistribution contDist = (ContinuousRNDistribution) srcDurationVal;
			UpdateContinuousRNDistributionBOMCmd cmd = new UpdateContinuousRNDistributionBOMCmd((ContinuousRNDistribution) wkCopyDurationVal);
			executeCmd(cmd);
			ContinuousRNDistribution newDomainContDist = (ContinuousRNDistribution) cmd.getObject();
			UpdateContinuousRNDistributionBOMCmd cmd2 = new UpdateContinuousRNDistributionBOMCmd((ContinuousRNDistribution) newDomainContDist);								
			if (newDomainContDist != null) {
				List cs = contDist.getC();
				List vals = contDist.getVal();	
				
				if (cs != null && !cs.isEmpty()) {				
					for (int i=0; i<cs.size(); i++) {
						cmd2.addC((Double) cs.get(i));
					}
				}
				
				if (vals != null && !vals.isEmpty()) {			
					for (int i=0; i<vals.size(); i++) {
						cmd2.addVal((Double) vals.get(i));
					}
				}
				executeCmd(cmd);					
			}				
		} else {
			// if the src duration val type is not the same as the working copy duration val, remove the working copy and recreate a new one 			
			RemoveValueSpecificationBOMCmd removeCmd = new RemoveValueSpecificationBOMCmd(workingOverride.getProcessingTime());
			executeCmd(removeCmd);
			addProcessingTimeToSimOverrides(workingOverride, srcOverride.getProcessingTime());
		}
		
		LoggingUtil.traceExit(this, "updateDurationValInSimOverrides");
	}
	
	/**
	 * Execute the given command
	 * @param cmd
	 */
	protected void executeCmd(AbstractCommand cmd){
		LoggingUtil.traceEntry(this, "executeCmd");
		if(cmd.canExecute()) {
			try {
				cmd.execute();
			} catch (RuntimeException e) {
				LoggingUtil.trace(this, "executeCmd", IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
				LoggingUtil.traceExit(this, "executeCmd");
				return;
			}
		} else {
			LoggingUtil.trace(this, "executeCmd", IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);	
			LoggingUtil.traceExit(this, "executeCmd");
			return;			
		}
		LoggingUtil.traceExit(this, "executeCmd");
	}

	
	
	
}
