package com.ibm.btools.blm.ie.imprt.rule.simulationModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.informationModel.UpdateValueSpecificationRule;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.bom.command.artifacts.AddLiteralRealToSimulationTransitionOverrideBOMCmd;
import com.ibm.btools.bom.command.simulationprofiles.AddSimulationTransitionOverrideToTransitionProfileBOMCmd;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.simulationprofiles.SimulationTransitionOverride;
import com.ibm.btools.bom.model.simulationprofiles.TransitionProfile;
import com.ibm.btools.emf.cf.BtCompoundCommand;


public class UpdateTransitionProfileRule extends AbstractImportRule {

	//	copyright
	  static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	  private TransitionProfile transitionProfile;
	  private TransitionProfile workingCopy;
	  
	  
	/**
	 * Sets the original BOM TransitionProfile.
	 * @param object a BOM TransitionProfile
	 */
	public void setSource(Object object) {
		this.transitionProfile = (TransitionProfile)object;
	}
	
	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = (TransitionProfile) workingCopy;
	}
	
		/**
		 * Invokes the rule
		 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");
		updateSimulationTransitionOverride();
		LoggingUtil.traceExit(this, "invoke");
	}

	private void updateSimulationTransitionOverride(){	
		LoggingUtil.traceEntry(this, "updateSimulationTransitionOverride");

		SimulationTransitionOverride override = transitionProfile.getSimulationTransitionOverride();
		if(override != null && override.getTransitionProbability() != null){
			SimulationTransitionOverride workingOverride = workingCopy.getSimulationTransitionOverride();
			// if no Simulation Transition Override, create one
			if(workingOverride == null){
				AddSimulationTransitionOverrideToTransitionProfileBOMCmd addTransitionOverride = new AddSimulationTransitionOverrideToTransitionProfileBOMCmd(workingCopy);
				if(!addTransitionOverride.canExecute()){
					//log error
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TRANSITION_EXCEPTION, null, null, "Can not create a Simulator Transition to Transition Profile");
				}else{
					try{
						addTransitionOverride.execute();
						workingOverride = (SimulationTransitionOverride)addTransitionOverride.getObject();
					}catch(RuntimeException ex){
						//log error
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TRANSITION_EXCEPTION, null, ex, "Can not create a Simulator Transition to Transition Profile");
					}
				}
			}
			if(workingOverride != null){
				ValueSpecification workingTransition = workingOverride.getTransitionProbability();
				if(workingTransition == null){
					BtCompoundCommand wrapperCmd = new BtCompoundCommand();
					AddLiteralRealToSimulationTransitionOverrideBOMCmd addProbabilityCmd = new AddLiteralRealToSimulationTransitionOverrideBOMCmd(workingOverride);
					if(wrapperCmd.appendAndExecute(addProbabilityCmd)){
						ValueSpecification workingValueSpecification = (ValueSpecification)addProbabilityCmd.getObject();
						UpdateValueSpecificationRule updateValueSpecRule = new UpdateValueSpecificationRule();
						updateValueSpecRule.setImportSession(getImportSession());
						updateValueSpecRule.setSource(override.getTransitionProbability());
						updateValueSpecRule.setWorkingCopy(workingValueSpecification);
						updateValueSpecRule.setWrapperCommand(wrapperCmd);
						updateValueSpecRule.invoke();
					}else{
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.SIMULATOR_TRANSITION_EXCEPTION, null, null, "Can not update a Simulator Transition to Transition Profile");
					}
				}else{
					UpdateValueSpecificationRule updateValueSpecRule = new UpdateValueSpecificationRule();
					updateValueSpecRule.setImportSession(getImportSession());
					updateValueSpecRule.setSource(override.getTransitionProbability());
					updateValueSpecRule.setWorkingCopy(workingTransition);
					updateValueSpecRule.invoke();
				}
			}
		}
		LoggingUtil.traceExit(this, "updateSimulationTransitionOverride");
	}

}
