package com.ibm.btools.blm.ie.imprt.rule.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * This class is responsible for read and write byte stream from and to the document
 */
public class DocumentUtil {
	
	//copyright
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	
	/**
	 * This method is responsible for getting the byte array from the target file
	 * @param file
	 * @return
	 * @throws IOException
	 */
	public static byte[] getBytesFromDocument(File file) throws IOException {	       
		InputStream is = new FileInputStream(file);	    
		// Get the size of the file
		long length = file.length();	    
		// You cannot create an array using a long type.
		// It needs to be an int type.
		// Before converting to an int type, check
		// to ensure that file is not larger than Integer.MAX_VALUE.
		if (length > Integer.MAX_VALUE) {
			// File is too large
		}	    
		// Create the byte array to hold the data
		byte[] bytes = new byte[(int)length];
		
		// Read in the bytes
		int offset = 0;
		int numRead = 0;
		while (offset < bytes.length
				&& (numRead=is.read(bytes, offset, bytes.length-offset)) >= 0) {
			offset += numRead;
		}	    
		// Ensure all the bytes have been read in
		if (offset < bytes.length) {
			throw new IOException("Could not completely read file "+file.getName());
		}	    
		// Close the input stream and return bytes
		is.close();
		return bytes;
	}
	
	/**
	 * This method is responsible for persist the byte array into the target file structure
	 * @param content
	 * @param targetDirectory
	 * @param fileName
	 */
	public static boolean  persistDocument(byte[] content, String targetDirectory, String fileName) throws FileNotFoundException,IOException {
		FileOutputStream fos = null;		
		String fullFileName = targetDirectory+File.separator+fileName;
		try {
			File file = new File(targetDirectory);
			if (!file.exists()) {
				file.mkdir();
			}
			fos = new FileOutputStream(fullFileName);
			fos.write(content);
			fos.close();
			return true;
		} catch (FileNotFoundException e) {
			if(fos != null) {
				fos.close();
			}
			throw e;
		} catch (IOException io) {			
			if(fos != null) {
				fos.close();
			}
			throw io;
		}
	}
}
