package com.ibm.btools.blm.ie.imprt.rule.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ui.navigation.model.NavigationProcessSimulationSnapshotNode;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.models.Model;
import com.ibm.btools.bom.model.models.ResourceModel;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.model.filemanager.FileMGR;
import com.ibm.btools.model.resourcemanager.ResourceMGR;



public class SimulationUtil {

	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	public static final String CURRENT_SIM_SNAPSHOT = "com.ibm.btools.blm.ie.imprt.rule.util.SimulationUtil.CURRENT_SIM_SNAPSHOT";
	public static final String SNAPSHOT_REGISTRY="com.ibm.btools.blm.ie.imprt.rule.util.SimulationUtil.SIM_MODEL_SNAPSHOT_REGISTRY";
	
	/**
	 * Maintain a registry that keep tracks of the SnapShot created for a process so that we can
	 * reuse the snapshot for the profiles of the same process
	 * @return
	 */
	public static HashMap getSnapShotRegistry(ImportContext context) {
		HashMap registry = (HashMap)context.get(SNAPSHOT_REGISTRY);
		if (registry==null) {
			registry = new HashMap();
			context.put(SNAPSHOT_REGISTRY,registry);
		}
		return registry;	
	}

	/**
	 * Retrieve the SnapShot node of the process
	 * @param context
	 * @param process
	 * @return
	 */
	public static NavigationProcessSimulationSnapshotNode getSnapShot(ImportContext context,Activity process) {
		return (NavigationProcessSimulationSnapshotNode)getSnapShotRegistry(context).get(process);
	}

	/**
	 * Retrieve the last registered SnapShot node
	 * @param context
	 * @return
	 */
	public static NavigationProcessSimulationSnapshotNode getSnapShot(ImportContext context) {
		return (NavigationProcessSimulationSnapshotNode)getSnapShotRegistry(context).get(CURRENT_SIM_SNAPSHOT);
	}

	/**
	 * Register snapshot node for the process into the SnapShotRegistry
	 * @param context
	 * @param process
	 * @param snapShot
	 */
	public static void registerSnapShot(ImportContext context,Activity process,NavigationProcessSimulationSnapshotNode snapShot) {
		getSnapShotRegistry(context).put(CURRENT_SIM_SNAPSHOT,snapShot);		
		getSnapShotRegistry(context).put(process,snapShot);
	}

	/**
	 * Retrieve the root resource model from the snapshot
	 * @param projectName
	 * @param snapShot
	 * @return
	 */
	public static ResourceModel getRootResourceModel(String projectName,NavigationProcessSimulationSnapshotNode snapShot) {
		
		String projectPath = FileMGR.getProjectPath(projectName);
	//	String resourcePath = (String)((NavigationSimulationProfileNode)snapShot.getSimulationProfileNodes().get(0)).getEntityReferences().get(3);
	//Defect 90411
		String processPath = (String)snapShot.getAttribute1();
		
		//Jie changed on 2003.06.03 as required due to API deprecated by the ResourceManager for using absolute path
	//	Iterator tempIterator = ResourceMGR.getResourceManger().getRootObjects(projectName, projectPath, resourcePath).iterator();
	Iterator tempIterator = ResourceMGR.getResourceManger().getRootObjects(projectName, projectPath, processPath).iterator();		 		
	//	Iterator tempIterator = ResourceMGR.getResourceManger().getRootObjects(projectPath + "\\" + resourcePath + "\\Model.xmi").iterator();
		NamedElement namedElement = null;
		while (tempIterator.hasNext()) {
			namedElement = (NamedElement)tempIterator.next();
			if (namedElement instanceof ResourceModel && namedElement.getName().equals("RootResourceModel")) {
				return (ResourceModel)namedElement;
			}
		}	
		return null;				
	}
	
	public static NamedElement getObjectInSnapshot(NamedElement resourceModelElement, String projectName, ImportContext context){

		//ResourceModel model =(ResourceModel)getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_SIMULATION).get("");
		ResourceModel model = SimulationUtil.getRootResourceModel(projectName, SimulationUtil.getSnapShot(context));
		if(model == null){
			return null;	
		}else{
			List contents = model.getOwnedMember();
			Iterator it = contents.iterator();
			while (it.hasNext()){
				Object element = it.next();
				if(element instanceof Model){
					NamedElement targetElement = processModelContent((Model)element, resourceModelElement);
					if(targetElement != null){
						return targetElement;
					}
				}else if(element instanceof NamedElement && BOMUtil.getCanonicalName((NamedElement)element, null).equals(BOMUtil.getCanonicalName(resourceModelElement, null))){
					return (NamedElement)element;
				}
			}
		}
		return null;
	}

	public static NamedElement processModelContent(Model model, NamedElement resourceModelElement){
		List elements = model.getOwnedMember();
		if(elements != null && !elements.isEmpty()){
			Iterator it = elements.iterator();
			while(it.hasNext()){
				Object element = it.next();
				if(element instanceof Model){
					NamedElement targetElement = processModelContent((Model)element, resourceModelElement);
					if(targetElement != null){
						return targetElement;
					}
				}else if(element instanceof NamedElement && BOMUtil.getCanonicalName((NamedElement)element, null).equals(BOMUtil.getCanonicalName(resourceModelElement, null))){
					return (NamedElement)element;
				}
			}
			
		}
		return null;
	}
}