package com.ibm.btools.te.ilm;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.io.File;
import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.PortType;
import org.eclipse.xsd.XSDSchema;

import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.blm.ie.exprt.option.BpelOptions;
import com.ibm.btools.blm.ie.exprt.option.ProjectWpsOptionsBean;
import com.ibm.btools.blm.ie.exprt.option.WpsExportOptionsBean;
import com.ibm.btools.blm.model.blmfilemanager.BLMFileMGR;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.actions.ActionsPackage;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleGroup;
import com.ibm.btools.bom.model.processes.humantasks.Form;
import com.ibm.btools.model.filemanager.FileMGR;
import com.ibm.btools.model.modelmanager.dependencymanager.Dependency;
import com.ibm.btools.model.modelmanager.dependencymanager.DependencyManager;
import com.ibm.btools.model.modelmanager.dependencymanager.DependencyModel;
import com.ibm.btools.model.resourcemanager.ResourceMGR;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.rulehandler.RuleHandlerRegistry;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.CallBehaviorActionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.SANReusableProcessRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.impl.CallBehaviorActionRuleImpl;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionKeyConstants;
import com.ibm.btools.te.ilm.heuristics.scdl.ComponentRule;
import com.ibm.btools.te.ilm.heuristics.scdl.ScdlPackage;
import com.ibm.btools.te.ilm.heuristics.scdl.impl.ComponentRuleImpl;
import com.ibm.btools.te.ilm.heuristics.scdl.impl.ScdlPackageImpl;
import com.ibm.btools.te.ilm.heuristics.scdl.util.JavaImplementation;
import com.ibm.btools.te.ilm.heuristics.scdl.util.ScdlUtil;
import com.ibm.btools.te.ilm.model.sa.HumanTask;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingRegistry;
import com.ibm.btools.te.ilm.rulehandler.mapping.UIDUtil;
import com.ibm.btools.te.ilm.rulehandler.mapping.wps.ComponentRuleHandler;
import com.ibm.btools.util.exception.BTRuntimeException;
import com.ibm.wbiservers.businessrule.model.brgt.BusinessRuleGroupTable;
import com.ibm.wbit.ae.sacl.SACLRoot;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Case;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.bpm.map.objectdefinition.EMFRef;
import com.ibm.wbit.bpm.map.objectdefinition.URIRef;
import com.ibm.wbit.bpm.map.objectdefinition.ObjectDefinition;
import com.ibm.wbit.bpm.map.objectdefinition.URIRef;
import com.ibm.wbit.br.core.model.RuleSet;
import com.ibm.wbit.businesscalendar.ICalendar;
import com.ibm.wbit.tel.util.TaskResourceImpl;
import com.ibm.wsspi.sca.scdl.DocumentRoot;
import com.ibm.wsspi.sca.scdl.Import;
import com.ibm.wsspi.sca.scdl.util.SCDLResourceFactoryImpl;
import com.ibm.wsspi.sca.scdl.webservice.WebServiceImportBinding;

public class CrossProjectReferenceUtil {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2008, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$


	public static void registerTranformTargetWithProjectName(EObject src,EObject tar){
		registerTranformTargetWithProjectName(src, tar, null);
	}
	/**
	 * Register the transformed persistable target with the WBM project name
	 * This utility will be called at the appropriate transformation rule to capture 
	 * the persistable transformation target mapping to its project name
	 * @param exportSession
	 * @param bomSource
	 * @param transformTarget
	 */
	public static void registerTranformTargetWithProjectName(EObject src,EObject tar, String wbmProjectNameOverride){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null){
			return;
		}
		if (tar instanceof com.ibm.wbit.bpel.Activity || tar instanceof Process || tar instanceof BPELVariable ||
				tar instanceof Case || tar instanceof Link){
			if(src instanceof NamedElement){	
				Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
				if(additionalOptions != null){
					Object registry = additionalOptions.get(TransformationSessionKeyConstants.TARGET_TO_SRC_BOMUID_REGISTRY_KEY);
					if(!(registry instanceof Map)){
						registry = new HashMap();
						additionalOptions.put(TransformationSessionKeyConstants.TARGET_TO_SRC_BOMUID_REGISTRY_KEY, registry);
					}
					if(!((Map)registry).containsKey(tar)){
						String uid = UIDUtil.getUIDForBOMElement(src, tar);
						((Map)registry).put(tar, uid);
					}
				}
				
			}
			
		}
		if(isPersistableTarget(tar)){
			String wbmProjectName = wbmProjectNameOverride == null ? getProjectNameForBOMElement(src) : wbmProjectNameOverride;
			if(wbmProjectName==null && !(src instanceof NamedElement)){
				wbmProjectName=getProjectNameForTargetObject(src);
			}
			if( wbmProjectName != null){
				Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
				if(additionalOptions != null){
					Object registry = additionalOptions.get(TransformationSessionKeyConstants.TARGET_TO_PROJECT_NAME_REGISTRY_KEY);
					if(!(registry instanceof Map)){
						registry = new HashMap();
						additionalOptions.put(TransformationSessionKeyConstants.TARGET_TO_PROJECT_NAME_REGISTRY_KEY, registry);
					}
					if(!((Map)registry).containsKey(tar)){
						((Map)registry).put(tar, wbmProjectName);
					}
				}
				//register source to project name
				if(additionalOptions != null){
					Object registry = additionalOptions.get(TransformationSessionKeyConstants.SOURCE_TO_PROJECT_NAME_REGISTRY_KEY);
					if(!(registry instanceof Map)){
						registry = new HashMap();
						additionalOptions.put(TransformationSessionKeyConstants.SOURCE_TO_PROJECT_NAME_REGISTRY_KEY, registry);
					}
					if(!((Map)registry).containsKey(src)){
						((Map)registry).put(src, wbmProjectName);
					}
				}			

				//Add target to soucr map
				if(additionalOptions != null){
					Object registry = additionalOptions.get(TransformationSessionKeyConstants.TARGET_TO_SOURCE_REGISTRY_KEY);
					if(!(registry instanceof Map)){
						registry = new HashMap();
						additionalOptions.put(TransformationSessionKeyConstants.TARGET_TO_SOURCE_REGISTRY_KEY, registry);
					}
					if(!((Map)registry).containsKey(tar)){
						((Map)registry).put(tar, src);
					}
				}				
				if(additionalOptions != null){
					Object registry = additionalOptions.get(TransformationSessionKeyConstants.TARGET_TO_SOURCE_REGISTRY_KEY);
					if(!(registry instanceof Map)){
						registry = new HashMap();
						additionalOptions.put(TransformationSessionKeyConstants.TARGET_TO_SOURCE_REGISTRY_KEY, registry);
					}
					if(!((Map)registry).containsKey(tar)){
						((Map)registry).put(tar, src);
					}
				}
			}
		}
		if(tar instanceof PortType){
			//register the portType 
			Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
			if(additionalOptions != null){
				Object registry = additionalOptions.get(TransformationSessionKeyConstants.PORTTYPE_LIST_REGISTRY_KEY);
				if(!(registry instanceof List)){
					registry = new ArrayList();
					additionalOptions.put(TransformationSessionKeyConstants.PORTTYPE_LIST_REGISTRY_KEY, registry);
				}
				if(!((List)registry).contains(tar)){
					((List)registry).add(tar);
				}
			}
		}
	}
    
	/**
	 * Retrieve the bom uid string for the target
	 * @param eObj
	 * @return
	 */
	public static String getBOMUidForTarget(EObject eObj){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();		
		if(exportSession == null){
			return null;
		}
		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
		if(additionalOptions != null){
			Object registry = additionalOptions.get(TransformationSessionKeyConstants.TARGET_TO_SRC_BOMUID_REGISTRY_KEY);
			if(registry instanceof Map){
				Map targetToSourceMap = (Map)registry;
				return (String)targetToSourceMap.get(eObj);
			}				
		}
		return null;
	}
	/**
	 * Return source for the target object in the mapping
	 * @param target
	 * @return
	 */
	public static EObject getSourceForTarget(EObject target){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();		
		if(exportSession == null){
			return null;
		}
		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
		if(additionalOptions != null){
			Object registry = additionalOptions.get(TransformationSessionKeyConstants.TARGET_TO_SOURCE_REGISTRY_KEY);
			if(registry instanceof Map){
				Map targetToSourceMap = (Map)registry;
				return (EObject)targetToSourceMap.get(target);
			}				
		}
		return null;
	}

	public static NamedElement getBOMSOurceObjectFor(Object transformTarget){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null){
			return null;
		}

		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
		if(additionalOptions != null){
			Object registry = additionalOptions.get(TransformationSessionKeyConstants.TARGET_TO_SOURCE_REGISTRY_KEY);
			if(registry instanceof Map){
				Object srcObject = ((Map)registry).get(transformTarget);
				if(srcObject instanceof NamedElement){
					return (NamedElement)srcObject;
				}
			}
		}
		return null;
	}

	/**
	 * Register object definitions with the project they reside
	 * @param od
	 */
	public static void registerObjectDefintionForProjectName(EObject eObject,ObjectDefinition od){
		//This is the BOM source object
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null || eObject == null || od == null){
			return;
		}
		String projectName = null;
		if(eObject instanceof NamedElement){			
			projectName = getProjectNameForSourceObject(eObject);
		}else{
			projectName = getProjectNameForTargetObject(eObject);
		}
		//This is the transformed target object
		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
		if(additionalOptions != null){
			Object registry = additionalOptions.get(TransformationSessionKeyConstants.PROJECT_TO_OBJECTDEFINITION_REGISTRY_KEY);
			if(registry instanceof Map){
				Object values = ((Map)registry).get(projectName);
				if(values instanceof List){
					List odList = (List)values;
					if(!odList.contains(od)){
						odList.add(od);
					}
				}else{
					List odList = new LinkedList();
					odList.add(od);
					((Map)registry).put(projectName, odList);
				}
			}else{
				Map registryMap = new HashMap();
				List odList = new LinkedList();
				odList.add(od);
				registryMap.put(projectName, odList);
				additionalOptions.put(TransformationSessionKeyConstants.PROJECT_TO_OBJECTDEFINITION_REGISTRY_KEY, registryMap);
			}
		}
	}
	
	/**
	 * Special handling for typedElement multiplicity
	 * @param bomOD
	 * @param projectName
	 */
	public static void registerBOMSrcWithSharedProjectName(ObjectDefinition bomOD, String projectName){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null){
			return;
		}
//		This is the transformed target object
		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
		if(additionalOptions != null){
			Object registry = additionalOptions.get(TransformationSessionKeyConstants.PROJECT_TO_OBJECTDEFINITION_REGISTRY_KEY);
			if(registry instanceof Map){
				Object values = ((Map)registry).get(projectName);
				if(values instanceof List){
					List odList = (List)values;
					if(!odList.contains(bomOD)){
						odList.add(bomOD);
					}
				}else{
					List odList = new LinkedList();
					odList.add(bomOD);
					((Map)registry).put(projectName, odList);
				}
			}else{
				Map registryMap = new HashMap();
				List odList = new LinkedList();
				odList.add(bomOD);
				registryMap.put(projectName, odList);
				additionalOptions.put(TransformationSessionKeyConstants.PROJECT_TO_OBJECTDEFINITION_REGISTRY_KEY, registryMap);
			}
		}
	}

	/**
	 * Get the hashmap that holds the project name to a list of object definitions
	 * @return
	 */
	public static Map getProjectNameToObjectDefinitionsMap(){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null){
			return null;
		}
		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
		if(additionalOptions != null){
			Object registry = additionalOptions.get(TransformationSessionKeyConstants.PROJECT_TO_OBJECTDEFINITION_REGISTRY_KEY);
			if(registry instanceof Map){
				return (Map)registry;
			}
		}
		return null;
	}

	public static String getProjectNameForTargetObject(Object targetObject){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null){
			return null;
		}
		String wbmProjectName = null;
		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
		if(additionalOptions != null){
			Object registry = additionalOptions.get(TransformationSessionKeyConstants.TARGET_TO_PROJECT_NAME_REGISTRY_KEY);
			if(registry instanceof Map){
				Object projectNameObj = ((Map)registry).get(targetObject);
				if(projectNameObj instanceof String){
					wbmProjectName = (String)projectNameObj;
				}
			}
		}
		//AZ 06/21/07 Resolve the project name through the original WSDL/XSDSchema absolute path
		if(wbmProjectName == null){
			String location = null;
			if(targetObject instanceof Definition){
				Definition def = (Definition)targetObject;
				if(def.getLocation() != null && ExportOperationUtil.isExternalFile(def.getLocation())){				
					location = def.getLocation();	
					//we need to convert the space of thr project name
					location = ExportOperationUtil.replaceConvertedSpaces(location);
				}
				if(location != null){
					location = location.substring(0,location.indexOf(ExportOperationUtil.ROOT_EXTERNAL_MODEL)-1);
					if(location.indexOf(ExportOperationUtil.FILE_SEPARATOR) != -1){
						wbmProjectName = location.substring(location.lastIndexOf(ExportOperationUtil.FILE_SEPARATOR)+1);
					}
				}
			}else if(targetObject instanceof XSDSchema){
				XSDSchema schema = (XSDSchema)targetObject;
				if(schema.getSchemaLocation()!= null && ExportOperationUtil.isExternalFile(schema.getSchemaLocation()) ){
					location = schema.getSchemaLocation();
					location = ExportOperationUtil.replaceConvertedSpaces(location);
					if(location != null){
						location = location.substring(0,location.indexOf(ExportOperationUtil.ROOT_EXTERNAL_MODEL)-1);
						if(location.indexOf(ExportOperationUtil.FILE_SEPARATOR) != -1){
							wbmProjectName = location.substring(location.lastIndexOf(ExportOperationUtil.FILE_SEPARATOR)+1);
						}
					}
				}
			}else if(targetObject instanceof Form) {
				wbmProjectName = ResourceMGR.getResourceManger().getProjectName((Form)targetObject);
			}

		}
		return wbmProjectName;
	}

	public static String getProjectNameForSourceObject(Object srcObject){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null){
			return null;
		}
		String wbmProjectName = null;
		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
		if(additionalOptions != null){
			Object registry = additionalOptions.get(TransformationSessionKeyConstants.SOURCE_TO_PROJECT_NAME_REGISTRY_KEY);
			if(registry instanceof Map){((Map)registry).size();
			Object projectNameObj = ((Map)registry).get(srcObject);				
			if(projectNameObj instanceof String){
				wbmProjectName = (String)projectNameObj;
			}
			}
		}
		if(wbmProjectName == null && srcObject instanceof EObject ){
			//If we cannot find the project name from the registry, we query the resource manager
			wbmProjectName = ResourceMGR.getResourceManger().getProjectName((EObject)srcObject);
		}
		return wbmProjectName;
	}

	/**
	 * 
	 * @param exportSession
	 * @return
	 */
	public static Map getTargetObjectToProjectNameMap(){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null){
			return null;
		}
		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
		if(additionalOptions != null){
			Object registry = additionalOptions.get(TransformationSessionKeyConstants.TARGET_TO_PROJECT_NAME_REGISTRY_KEY);
			if(registry instanceof Map){
				return (Map)registry;
			}
		}
		return null;
	}

	private static boolean isPersistableTarget(Object targetObject){
		if(targetObject instanceof com.ibm.wbit.bpel.Process ||
				targetObject instanceof XSDSchema ||
				targetObject instanceof Definition ||
				targetObject instanceof HumanTask ||
				targetObject instanceof DocumentRoot ||
				targetObject instanceof com.ibm.wbit.tel.DocumentRoot ||
				targetObject instanceof SACLRoot ||
				targetObject instanceof com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup||
				targetObject instanceof BusinessRuleGroupTable||
				targetObject instanceof RuleSet||
				targetObject instanceof JavaImplementation ||
				targetObject instanceof ICalendar ||
				targetObject instanceof com.ibm.wbit.br.core.model.DocumentRoot ||
				targetObject instanceof com.ibm.wbit.sib.mediation.operation.model.DocumentRoot ||
				targetObject instanceof EPackage)

		{
			return true;
		}
		return false;
	}

	/**
	 * The BOM object has to be in a project, otherwise exception will be raised
	 * @param ne
	 * @return
	 */
	public static String getProjectNameForBOMElement(EObject ne){
		String projectName = null;
		if(!(ne instanceof NamedElement)){
			return projectName;
		}
		try{
			projectName = ResourceMGR.getResourceManger().getProjectName(ne);
		}catch(Exception e){
			if(projectName == null){
				throw new RuntimeException("Cannot find the project name for the bom object:["+ 
						((NamedElement)ne).getName() + "] for" + e.getLocalizedMessage());
			}
		}
		return projectName;
	}
	/**
	 * Retrieve the project name list involved in the export session
	 * @return
	 */
	public static List getProjectNameListForExportSession(){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null){
			return null;
		}
		WpsExportOptionsBean optionsBean = BpelOptions.getWpsOptions(exportSession);
		if(optionsBean != null){
			return optionsBean.getProjectList();
		}
		return null;
	}

	/**
	 * This method is to retrieve the business object target path information based on the following
	 * algorithm, where the business objects are WBM defined  business items
	 * 1. Find the process host project name
	 * 2. Look up the dependant project name list
	 * 3. Iterate through the List and append the key tns+schemaName with the dependant project name as "tns+schemaName#dependantProjectName"
	 * 4. Get the value list using the new compound key util a value is found
	 * 5. Last resort to the process project name to compute the compound key
	 * 6. If no value list found, something wrong, we throw BTRuntimeException
	 * @param process
	 */
	public static List searchForBusinessObjectValuesByDependency(String key,Map<String,List<Object>> sourcePathToTargetPathAbsoluteRegistry, com.ibm.wbit.bpel.Process process){
		List values=null;
		if(key == null ||sourcePathToTargetPathAbsoluteRegistry==null){
			return null; 
		}
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null){
			return null;
		}
		String processProjectName = getProjectNameForTargetObject(process);
		ProjectWpsOptionsBean projectBean= BpelOptionsUtil.getProjectWpsOptionsBean(exportSession, processProjectName);
		List dependencies = projectBean.getDependantProjectNames();		
		if(dependencies != null && !dependencies.isEmpty()){			
			for(Iterator it=dependencies.iterator();it.hasNext();){
				String temp = key;
				String dependantProjectName = (String)it.next();
				temp = temp+"#"+dependantProjectName;
				values = (List)sourcePathToTargetPathAbsoluteRegistry.get(temp);
				if(values != null){
					break;
				}
			}
		}
		//last resort to find in its own project
		if(values == null){
			values = (List)sourcePathToTargetPathAbsoluteRegistry.get(key+"#"+processProjectName);
		}

		if(values == null){
			String params[] = {key};
			LoggingUtil.logError(MessageKeys.ERROR_RETRIEVE_WBM_SCHEMA, params, null);
			BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_RETRIEVE_WBM_SCHEMA));
			throw runtimeEx;
		}
		return values;
	}

	/**
	 * Loop through all the referenced project libraries	 
	 * @return
	 */
	public static String[] getProjectReferences(String absPathPrefix){
		List<String> references = new ArrayList<String>();
		List wbmProjectNames = getProjectNameListForExportSession();
		if(wbmProjectNames != null && !wbmProjectNames.isEmpty()){
			for(Iterator it=wbmProjectNames.iterator();it.hasNext();){
				String projectName = (String)it.next();
				if(isPredefinedProject(projectName)&& !existSharedLib()){
					continue;
				}
				String libName = BpelOptionsUtil.getWPSLibraryName(projectName);	
				if(CrossProjectReferenceUtil.existProjectDirectory(absPathPrefix, libName)){				
					references.add(libName);
				}
			}
		}	
		return (String[])references.toArray(new String[]{});
	}
	/**
	 * Resolve module project dependency using the project's dependent library and 
	 * its own library
	 * @param projectName
	 * @param absPathPrefix
	 * @return
	 */
	public static String[] getProjectReferences(String projectName,String absPathPrefix){
		List<String> references = new ArrayList<String>();
		String[] dependentLibs = getProjectReferencesForLib(projectName, absPathPrefix);
		for(int i=0;i<dependentLibs.length;i++){
			if(dependentLibs[i] != null && dependentLibs[i].length()>0){
				references.add(dependentLibs[i]);
			}
		}
		//add callback dependency
		List<String> callbackDependency = ScdlUtil.resolveAdditionalModuleProjectDependency(projectName);
		if(callbackDependency != null && !callbackDependency.isEmpty()){
			for(String additionalDependency:callbackDependency){
				if(!references.contains(additionalDependency)){
					references.add(additionalDependency);
				}
			}
		}
		//Always depending on its own library
		String libName = BpelOptionsUtil.getWPSLibraryName(projectName);
		references.add(libName);
		return (String[])references.toArray(new String[]{});
		
	}

		/**@deprecated
		 * Loop through all the referenced project libraries	 
		 * @return
		 */
		public static String[] getProjectReferences(){
			List<String> references = new ArrayList<String>();
			List wbmProjectNames = getProjectNameListForExportSession();
			if(wbmProjectNames != null && !wbmProjectNames.isEmpty()){
				for(Iterator it=wbmProjectNames.iterator();it.hasNext();){
					String projectName = (String)it.next();
					if(isPredefinedProject(projectName)&& !existSharedLib()){
						continue;
					}
					String libName = BpelOptionsUtil.getWPSLibraryName(projectName);	
					references.add(libName);
				}
			}	
			return (String[])references.toArray(new String[]{});
		}
	/**
	 * Determine whether we need to consolidate the relative path by assuring if there 
	 * is project name involved in the path
	 * @param fixedRelativePath
	 * @return
	 */
	public static boolean skipConsolidate(String fixedRelativePath) {
		if(fixedRelativePath == null){
			return true;
		}
		List wbmProjectNames = getProjectNameListForExportSession();
		if(wbmProjectNames != null && !wbmProjectNames.isEmpty()){
			for(Iterator it=wbmProjectNames.iterator();it.hasNext();){
				String projectName = (String)it.next();
				String libName = BpelOptionsUtil.getWPSLibraryName(projectName);
				//Since we add project name in the schema name, we need to substring the last segment of the path to see if there is 
				//a lib name in the path
				String tmp = null;
				if(fixedRelativePath.indexOf(ExportOperationConstants.FILE_SEPARATOR_CHAR) != -1){
					tmp = fixedRelativePath.substring(0,fixedRelativePath.lastIndexOf(ExportOperationConstants.FILE_SEPARATOR_CHAR));
				}
				if(libName != null && tmp != null && tmp.indexOf(libName) != -1){
					return false;
				}
			}
		}
		return true;
	}

	/**
	 * Get the reference lib project list for the current project
	 * TODO cyclical reference need to be checked
	 * @param projectName
	 * @return
	 */
	public static String[] getProjectReferencesForLib(String projectName, String absPathPrefix) {
		List<String> references = new ArrayList<String>();
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null){
			return new String[]{""};
		}
		ProjectWpsOptionsBean projectBean= BpelOptionsUtil.getProjectWpsOptionsBean(exportSession, projectName);
		List dependencies = projectBean.getDependantProjectNames();		
		if(dependencies != null && !dependencies.isEmpty()){
			for(Iterator it = dependencies.iterator();it.hasNext();){
				String referenceProjectName = (String)it.next();
				String libName = BpelOptionsUtil.getWPSLibraryName(referenceProjectName);
				if(CrossProjectReferenceUtil.existProjectDirectory(absPathPrefix, BpelOptionsUtil.getWPSLibraryName(referenceProjectName))){
					references.add(libName);
				}
			}
		}	
		String shardPrjectName = getSharedLibProjectName();
		String shardWBMPrjectName = getSharedLibWBMProjectName();
		if(existSharedLib()&& shardWBMPrjectName != null && 
				!shardWBMPrjectName.equals(projectName)){
			references.add(shardPrjectName);
		}
		if(references != null && references.size()==0){
			return new String[]{""};
		}
		return (String[])references.toArray(new String[]{});
	}

		/**
		 * Get the reference lib project list for the current project
		 * TODO cyclical reference need to be checked
		 * @param projectName
		 * @return
		 */
		public static String[] getProjectReferencesForLib(String projectName) {
			List<String> references = new ArrayList<String>();
			ExportSession exportSession = BpelOptionsUtil.getExportSession();
			if(exportSession == null){
				return new String[]{""};
			}
			ProjectWpsOptionsBean projectBean= BpelOptionsUtil.getProjectWpsOptionsBean(exportSession, projectName);
			List dependencies = projectBean.getDependantProjectNames();
			if(dependencies != null && !dependencies.isEmpty()){
				for(Iterator it = dependencies.iterator();it.hasNext();){
					String referenceProjectName = (String)it.next();
					String libName = BpelOptionsUtil.getWPSLibraryName(referenceProjectName);					
					references.add(libName);
				}
			}	
			String shardPrjectName = getSharedLibProjectName();
			String shardWBMPrjectName = getSharedLibWBMProjectName();
			if(existSharedLib()&& shardWBMPrjectName != null && 
					!shardWBMPrjectName.equals(projectName)){
				references.add(shardPrjectName);
			}
			if(references != null && references.size()==0){
				return new String[]{""};
			}
			return (String[])references.toArray(new String[]{});
		}

	/**
	 * Get refencing projects 
	 * FIXME The dependency is not right
	 * @param projectName
	 * @return
	 */
	public static String[] getReferencingProjects(String projectName){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null){
			return null;
		}
		WpsExportOptionsBean optionsBean = BpelOptions.getWpsOptions(exportSession);
		Map<String,List> referencingProjectMap = optionsBean.getReferenceProjectMap();
		if(referencingProjectMap != null && !referencingProjectMap.isEmpty()){
			List referenceingProjects = referencingProjectMap.get(projectName);
			if(referenceingProjects != null){
				return (String[])referenceingProjects.toArray(new String[]{});
			}
		}
		return null;
	}

	/**
	 * Check if the current project get referenced
	 * One usage is to determine whether to create .import for the top level module
	 * artifacts
	 * @param eObject
	 * @return
	 */
	public static boolean isReferenced(EObject eObject){
		String projectName = null;	
		if(eObject instanceof NamedElement){
			projectName = getProjectNameForBOMElement(eObject);
		}else{
			projectName = getProjectNameForTargetObject(eObject);
		}
		if(projectName != null){	 
			String[] referencingProjects = getReferencingProjects(projectName);
			return (referencingProjects != null && referencingProjects.length > 0);
		}
		return false;
	}

	/**
	 * Bi-directionally check if the project has cyclical reference
	 * @param projectName
	 * @param libName
	 * @return
	 */
	public static boolean isCyclicalReferenced(String projectName) {
		String[] referencingProjects = getReferencingProjects(projectName);
		if(referencingProjects != null){
			for(int i=0;i < referencingProjects.length;i++){
				String eachOwnerProject = referencingProjects[i];
				String[] reverseList = getReferencingProjects(eachOwnerProject);
				if(reverseList != null){
					List<String> reverseReferencing = Arrays.asList(reverseList);
					if(reverseReferencing.contains(projectName)){
						return true;
					}
				}
			}
		}
		return false;
	}

	/**
	 * Check if only one project involved in the export session
	 * @return
	 */
	public static boolean isSingleProjectExport(){
		if(getProjectNameListForExportSession()!= null && 
				getProjectNameListForExportSession().size() == 1){
			return true;
		}
		return false;
	}

	public static boolean isImportDocumentRoot(Object element){
		if(element instanceof DocumentRoot){
			DocumentRoot dr = (DocumentRoot)element;
			if(dr.getImport() != null){
				return true;
			}
		}
		return false;
	}

	public static void createResourceForSCDLImport(SCDLResourceFactoryImpl resourceFactory,
			ResourceSet resourceSet,
			Object element,
			String filePath,
			String relativePath, 
			Map<String,Resource> importResourceMap, Map<Object, String> objectFilePathMap){
		if(BpelOptionsUtil.isLibraryOnly()){
			return;
		}
		 String originalProject = CrossProjectReferenceUtil.getProjectNameForTargetObject(element);
		// String[] projectNames = CrossProjectReferenceUtil.getReferencingProjects(originalProject);
		String[] projectNames = CrossProjectReferenceUtil.getDependantProjectFor(element);
		if(projectNames != null && projectNames.length > 0){
			for(int i=0;i<projectNames.length;i++){
				NamedElement target = null;
				String eachProjectName = projectNames[i];
				//We skip create .import for top level Process if
				//the dependant project is itself	defect 154754			
				if(originalProject.equals(eachProjectName)){
					target = getBOMSOurceObjectFor(element);
					//d155310
					if(shouldSkipCreateImport(target)){
						continue;
					}
				}
				String moduleProject = BpelOptionsUtil.getWPSModuleName(eachProjectName);
				//defect 164678
				if(moduleProject == null || moduleProject.length() == 0 ){
					continue;
				}
				//AR should no try to eliminate the export with WS binding
				String libProject = BpelOptionsUtil.getWPSLibraryName(eachProjectName);
					Import thisImport = ((DocumentRoot)element).getImport();
					if(thisImport.getBinding() != null && thisImport.getBinding() instanceof WebServiceImportBinding){
						//continue;
					}else{
						//replace the first segment of the relative path as the referencing module project name
						if(relativePath.indexOf("\\") != -1){
							String temp = relativePath.substring(relativePath.indexOf("\\"));
							//Consolidated relative path
							relativePath = moduleProject+temp;
						}
					}

//				//replace the first segment of the relative path as the referencing module project name
//				if(relativePath.indexOf("\\") != -1){
//					String temp = relativePath.substring(relativePath.indexOf("\\"));
//					//Consolidated relative path
//					relativePath = moduleProject+temp;
//				}
				String absoluteFilePath = filePath + relativePath;
				URI fileURI = URI.createFileURI(absoluteFilePath);
				Resource resource = resourceFactory.createResource(fileURI);		
				resourceSet.getResources().add(resource);
				
				//Create object definitions for the scdl imports and all dependent projects
				EObject elementCopy = null;
				if (originalProject.equals(eachProjectName)) {
					elementCopy = (EObject) element;
					resource.getContents().add((EObject) elementCopy);			 
				}
				else {
					//Create a copy of the document root when we're dealing with dependent projects
					elementCopy = EcoreUtil.copy((EObject) element);
					//Update the objectFilePathMap for scdl imports in dependent projects otherwise the cmt file won't get serialized to the
					//right directory
					objectFilePathMap.put(elementCopy, relativePath);
					resource.getContents().add((EObject) elementCopy);			 
					
					MappingManager mapMgr = MappingManager.getInstance();
					DocumentRoot dr = (DocumentRoot)element;
					DocumentRoot drCopy = (DocumentRoot)elementCopy;
					
					//Retrieve the source activity this scdl import maps to.  For dependent projects, we're going to map it to the 
					//same source activity
					ObjectDefinition targetDefinition = mapMgr.getObjectDefinition(dr);
					ObjectDefinition sourceDefinition = 
						(ObjectDefinition) MappingManager.uriMappings.get( ((URIRef)targetDefinition.getReferencedDefinitions().get(0)).getUri());
					Activity sourceActivity = null;
					HashMap objectRegistry = mapMgr.getMappingRegistry().getObjectRegistry();
					for (EObject eObj : (Set<EObject>)objectRegistry.keySet()) {
						if (objectRegistry.get(eObj) == sourceDefinition) {
							if (eObj instanceof Activity) {
								sourceActivity = (Activity) eObj;
							}
							else if (eObj instanceof StructuredActivityNode) {
								sourceActivity = ((StructuredActivityNode)eObj).getActivity();
							}
							break;
						}
					}
					
					if (sourceActivity != null) {
						//Establish the actual mappings
						//For dependent projects, we have special handling here.  Several internal registries maps wid artifacts to 
						//project names based on the source element.  In the case of dependant projects, we can't use the source 
						//element to determine the project name so we need to be able to override this behaviour by allowing ourselves 
						//to pass in a project name override
						mapMgr.createMapDefinitionForSCDLImport(sourceActivity,drCopy, eachProjectName);	
						if(dr.getComponent() != null){
							mapMgr.addMapDefinition(sourceActivity, drCopy.getComponent());		
						}else if(dr.getImport() != null){
							mapMgr.createMapDefinitionForSCDLImport(sourceActivity, drCopy.getImport(), eachProjectName);	
						}
						ComponentRuleHandler handler = 
							(ComponentRuleHandler) RuleHandlerRegistry.getInstance().getRuleHandlerFromRuleClass(ScdlPackage.eINSTANCE.getComponentRule()).get(0);
						handler.mapComponentInterfaces(sourceActivity.getImplementation(), drCopy, mapMgr);
						handler.mapComponentImplementation(sourceActivity.getImplementation(), drCopy, mapMgr);
						handler.mapImportBinding(sourceActivity.getImplementation(), drCopy, mapMgr);
						//******************************************************************************************************
					}
				}
				//resource.getContents().add((EObject) elementCopy);			 
				
				
				try{
					Map<String, String> options = new HashMap<String, String>();
					options.put(XMLResource.OPTION_ENCODING, "UTF-8");
					resource.save(options);
					
				}catch(Exception e){
					String params[] = {filePath};
					LoggingUtil.logError(MessageKeys.ERROR_SAVE_IMPORT_COMPONENT, params, null);
					BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_SAVE_IMPORT_COMPONENT));
					throw runtimeEx;
				}
			}
			
		}else{
			filePath = filePath + relativePath;
			URI fileURI = URI.createFileURI(filePath);
			Resource resource = resourceFactory.createResource(fileURI);		
			resourceSet.getResources().add(resource);
			resource.getContents().add((EObject) element);
			if(importResourceMap != null){
				importResourceMap.put(filePath,resource);
			}
//			try{
//			Map<String, String> options = new HashMap<String, String>();

////			options.put(XMLResource.OPTION_ENCODING, "UTF-8");
////			resource.save(options);
//			}catch(Exception e){
//			String params[] = {filePath};
//			LoggingUtil.logError(MessageKeys.ERROR_SAVE_IMPORT_COMPONENT, params, null);
//			BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_SAVE_IMPORT_COMPONENT));
//			throw runtimeEx;
//			}
		}
	}
	
	public static boolean existProjectDirectory(String location,String projectName){
		if(location == null || projectName == null){
			return false;
		}
		String projectDir = location+projectName;
		boolean exists = (new File(projectDir)).exists();
		return exists;		
	}

	/**
	 * Look up the dependant objects native projects to determine whether to create
	 * .import in that project root
	 * @param targetObject
	 * @return
	 */
	public static String[] getDependantProjectFor(Object targetObject){
		String[] dependantProjects = resolveDependantProjectsFor(targetObject);
		if(dependantProjects != null && dependantProjects.length ==0){
			//if there is no dependant project, we just create the .import in its own
			String projectName =getProjectNameForTargetObject(targetObject);
			if(projectName != null){
				dependantProjects= new String[]{projectName};
			}
		}		
		return dependantProjects;		
	}
	private static String[] resolveDependantProjectsFor(Object targetObject) {		
		List<String> dependantProjects = new ArrayList<String>();		
		String projectName =getProjectNameForTargetObject(targetObject);
		NamedElement target = getBOMSOurceObjectFor(targetObject);
		traverseDependencies(dependantProjects, projectName, target);
		if(dependantProjects != null){			
			return (String[])dependantProjects.toArray(new String[]{});
		}
		return null;
	}

	public static void traverseDependencies(List<String> dependantProjects, String projectName, NamedElement target) {
		String CBA_TO_ACTIVITY_REFERENCE_NAME = 
			ActionsPackage.eINSTANCE.getCallBehaviorAction_Behavior().getName();			
		if(projectName == null || target == null){
			return;
		}
		String baseUri = FileMGR.getProjectPath(projectName);	  
		DependencyModel dependencyModel = DependencyManager.instance().getDependencyModel(projectName, baseUri);
		if(target != null){		  
			List dependencies = dependencyModel.getAllDependencies(
					null,
					target,
					CBA_TO_ACTIVITY_REFERENCE_NAME
			);
			CallBehaviorAction callBehaviorAction = null;
			String dependencySourceProjectName = null;
			for(int i = 0; i < dependencies.size(); i++) {
				try {
					Dependency dependency = (Dependency) dependencies.get(i);
					//FIXME alina test case NPE
					if(dependency.getSource() == null){
						continue;
					}
					//defect 77174 In the case of empty or NULL source for the dependency, an InfraObj will
					//be set as a placeholder for the dependency, this may caused by an incorrect project import,
					//where the resource manager does record the project name corresponding to the process activity
					if(dependency.getSource().getEObject() instanceof CallBehaviorAction){
						callBehaviorAction = (CallBehaviorAction) dependency.getSource().getEObject();
					}
					//if(callBehaviorAction.eContainer() != null && callBehaviorAction.eContainer().eContainer() instanceof Activity){
					//defect 154206
					if(getContainingProcessActivity(callBehaviorAction) instanceof Activity){
						//Containing process
						Activity processActivity = (Activity)getContainingProcessActivity(callBehaviorAction);
						String containerProject = getProjectNameForSourceObject(processActivity);
						//This means that the containing process participates into the export session
						//we need to create import for the process
						if(containerProject != null){
							dependencySourceProjectName = getProjectNameForBOMElement(callBehaviorAction);
							if(dependencySourceProjectName != null ){
								if(dependencySourceProjectName!= null && !dependantProjects.contains(dependencySourceProjectName)){					  
									dependantProjects.add(dependencySourceProjectName);									
								}
							}
//							if(processActivity != null && containerProject != null){
//								traverseDependencies(dependantProjects, containerProject, processActivity);
//							}
						}
					}
				}
				catch(ClassCastException e) {
					String params[] = {e.getLocalizedMessage()};
					LoggingUtil.logError(MessageKeys.ERROR_SAVE_IMPORT_COMPONENT, params, null);
					BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_SAVE_IMPORT_COMPONENT));
					throw runtimeEx;
				}					
			}
		}
	}  

	/**
	 * This utility is to handle the global elements(service, task, process) share import creation 
	 * check in the case of module+library option
	 * @param componentRule
	 * @param action
	 * @return
	 */
	public static boolean createImport(TransformationRule componentRule, Action action){
		if(componentRule == null || action == null){
			return false;
		}
		if(componentRule.getParentRule() instanceof CallBehaviorActionRule){
			if(BomUtils.isReusableService(action)||BomUtils.isReusableTask(action)||BomUtils.isProcess(action)){
				if(isReferenced(action)){
					return true; 
				}
			}
		}
		return false;
	}

	/**
	 * This is used to check if the process is a CBA or top level process, we skip create import for
	 * Top level process 
	 * @param rule
	 * @param action
	 * @return
	 */
	public static boolean isTopLevelProcess(TransformationRule rule,Action action){
		if(rule == null || action == null){
			return false;
		}
		if(rule.getParentRule() instanceof SANReusableProcessRule && BomUtils.isTopLevelProcess(action)){
			return true;
		}
		return false;
	}

	/**
	 * Register the shared lib project name for the primitive type
	 * @param projectName
	 */
	public static void registerSharedLibProjectName(String projectName) {
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null || projectName == null){
			return;
		}
		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
		if(additionalOptions != null){
			Object registry = additionalOptions.get(TransformationSessionKeyConstants.SHARED_LIB_PROJECT_NAME_REGISTRY_KEY);
			if(!(registry instanceof List)){
				registry = new ArrayList();
				additionalOptions.put(TransformationSessionKeyConstants.SHARED_LIB_PROJECT_NAME_REGISTRY_KEY, registry);
			}
			if(!((List)registry).contains(projectName)){
			//defect 91531 assert the project name is really the predefined project name
				if(projectName != null && projectName.equals(getPredefinedProjectName())){
					((List)registry).add(projectName);
				}
			}
		}
	}

	/**
	 * Retrieve the shared lib project name from the registry if there is any
	 * @return
	 */
	public static String getSharedLibProjectName(){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null){
			return null;
		}
		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
		if(additionalOptions != null){
			Object registry = additionalOptions.get(TransformationSessionKeyConstants.SHARED_LIB_PROJECT_NAME_REGISTRY_KEY);
			if(registry instanceof List){
				List libList = (List)registry;
				if(!libList.isEmpty()){
					String wbmName = (String)libList.get(0);
					return BpelOptionsUtil.getWPSLibraryName(wbmName);
				}
			}
		}
		return null;
	}

	/**
	 * Get the original name of the predefined elements project
	 * @return
	 */
	public static String getSharedLibWBMProjectName(){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null){
			return null;
		}
		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
		if(additionalOptions != null){
			Object registry = additionalOptions.get(TransformationSessionKeyConstants.SHARED_LIB_PROJECT_NAME_REGISTRY_KEY);
			if(registry instanceof List){
				List libList = (List)registry;
				if(!libList.isEmpty()){					
					return (String)libList.get(0);
				}
			}
		}
		return null;
	}


	/**
	 * Check if the shared lib exists or not
	 * @return
	 */
	public static boolean existSharedLib(){
		String libName = getSharedLibProjectName();
		if(libName != null){
			return true;
		}
		return false;
	}

	public static boolean isArtificialListSchema(Object obj) {
		if(obj instanceof XSDSchema){
			String projectName = getProjectNameForTargetObject(obj);
			if(projectName != null && projectName.equals(getSharedLibWBMProjectName())){
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Determine if the project is the predefined one
	 * @param projectName
	 * @return
	 */
	public static boolean isPredefinedProject(String projectName){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null){
			return false;
		}
		ProjectWpsOptionsBean projectBean = BpelOptionsUtil.getProjectWpsOptionsBean(exportSession, projectName);
		if(projectBean != null){
			return projectBean.isPredefinedProject();
		}
		return false;
	}
	
	/**
	 * Loop through the CBA's container up to its top level, which is the container process
	 * @param cba
	 * @return
	 */
	private static Object getContainingProcessActivity(CallBehaviorAction cba){
		if(cba instanceof CallBehaviorAction){
			EObject container = (EObject)cba.eContainer();
			while(container != null){
				container = container.eContainer();
				if(container.eContainer()== null){
					return container;
				}
			}			
		}			
		return null;
	}
	
	/**
	 * We only skip creating import for top level process only
	 * @param action
	 * @return
	 */
	public static boolean shouldSkipCreateImport(NamedElement element){
		if(element instanceof Activity){
			Action action = ((Activity)element).getImplementation();
			ExportSession exportSession = BpelOptionsUtil.getExportSession();
			if(!BpelOptions.isBestPracticePatternEnabed(exportSession)){
			//defect 83260, should skip creating the new import if this is an imported Service
				if(BomUtils.isReusableService(action)||BomUtils.isReusableTask(action) || BomUtils.isServiceOperation(action)){
				//if(BomUtils.isReusableService(action)||BomUtils.isReusableTask(action)){
					return true;
				}
			}
			return (BomUtils.isProcess(action)&& BomUtils.isTopLevelProcess(action))||
					BomUtils.isReusableHumanTask(action)||
					BomUtils.isReusableBusinessRuleTask(action);
		}
		if(element instanceof Action){
			if(BomUtils.isBusinessRuleTask((Action)element) || BomUtils.isHumanTask((Action)element)){
				return true;
			}
		}
		return false;
	}
    
	/**
	 * Handle JMS and SCA import binding persistence when the user export is not best practice pattern
	 * @param element
	 * @return
	 */
	public static boolean isSCABindingImport(Object element) {
		if(isImportDocumentRoot(element)){
			DocumentRoot dr= (DocumentRoot)element;
			Import registeredImport = null;
			if(dr.getImport() != null){
				registeredImport = dr.getImport();
			}
			ExportSession exportSession = BpelOptionsUtil.getExportSession();
			Object docRootRegistry = exportSession.getExportOptions().getAdditionalOption(TransformationSessionKeyConstants.TOP_LEVEL_DOCUMENT_ROOT);
			if(docRootRegistry instanceof Set){
				Set regSet = (Set)docRootRegistry;
				if(registeredImport != null && regSet.contains(registeredImport)){
					return true;
				}
			}
		}
		return false;
	}
	
	/**
	 * This method is to resolve the local specific predefined project name in the current work space
	 * @return String name of the predefined project
	 */
	private static String getPredefinedProjectName(){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null){
			return null;
		}
		WpsExportOptionsBean optionsBean = BpelOptions.getWpsOptions(exportSession);
		if(optionsBean != null){
			List<ProjectWpsOptionsBean> beanList = optionsBean.getSelectedProjectWpsOptionsBeanList();
			if(beanList != null && !beanList.isEmpty()){
				Iterator<ProjectWpsOptionsBean> beanIt = beanList.iterator();
				while(beanIt.hasNext()){
					ProjectWpsOptionsBean bean = beanIt.next();
					if(bean !=  null && bean.isPredefinedProject()){
						return bean.getWbmProjectName();
					}
				}
			}
		}
		return null;
	}
}
