package com.ibm.btools.te.ilm;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.eclipse.core.runtime.Platform;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.PlatformUI;
import org.osgi.framework.Bundle;

import com.ibm.btools.blm.ie.exprt.ExportOptions;
import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.te.framework.rulehandler.RuleHandlerRegistry;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.emf.cf.BtCompoundCommand;
import com.ibm.btools.ie.ui.ilm.config.BOM2ILMSettingHelper;
import com.ibm.btools.ie.ui.ilm.config.model.BOM2ILMTESetting;
import com.ibm.btools.model.modelmanager.copyregistry.CopyRegistry;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionKeyConstants;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.util.exception.BTRuntimeException;

public class GenerateDefaultAttributesOperation implements TransformationSessionKeyConstants{

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	private Activity ivWorkingCopy;
	private boolean ivIsOverwrite = false;
	
	private BtCompoundCommand ivWrapperCommand;

	/**
	 * @param workingCopy
	 * @param isOverwrite
	 */
	public GenerateDefaultAttributesOperation(Activity workingCopy,
			boolean isOverwrite) {
		ivWorkingCopy = workingCopy;
		ivIsOverwrite = isOverwrite;
	}
	
	public void execute() {
		LoggingUtil.traceEntry(this, "execute");
		
		if(ivWorkingCopy == null){
			return;
		}
	
		TransformationEngine aTransformationEngine = new TransformationEngine();
		ExportSession exportSession = new ExportSession();
		
		
		TransformationSession context = new TransformationSession();
		context.put(GENERATE_DEFAULT_TECH_ATTRIBUTE, ivIsOverwrite ? Boolean.TRUE : Boolean.FALSE);
		
//		ExportOptions exportOptions = new ExportOptions();
//		exportOptions.setAdditionalOption(GENERATE_DEFAULT_TECH_ATTRIBUTE, ivIsOverwrite ? Boolean.TRUE : Boolean.FALSE);
//		exportSession.setExportOption(exportOptions);
		Activity masterCopy = (Activity)CopyRegistry.instance().getMaster(ivWorkingCopy);
		//TransformationSessionUtil.putMasterCopy(context, masterCopy);
		//defect 139703, registrer the MasterCopy with the working copy
		TransformationSessionUtil.putMasterCopy(context, masterCopy, ivWorkingCopy);
		
		//aTransformationEngine.registerExportSession(exportSession);
		aTransformationEngine.registerTransformationSession(context);
		
		BOM2ILMTESetting teSetting = BOM2ILMSettingHelper.createAndInitializeTESetting();
		BOM2ILMSettingHelper.setWBITargetType(teSetting);
		
		List modelElements = new ArrayList();
		modelElements.add(ivWorkingCopy);
		int i = 0;
		try{
			//aTransformationEngine.execute(modelElements, teSetting);
			// RK - fix NPE that happened in TransformationEngine with an exportSession with no exportOptions 
			ExportOptions exportOptions = new ExportOptions();
			exportOptions.setAdditionalOption(TransformationSessionKeyConstants.TE_SETTING,teSetting);
			exportOptions.setAdditionalOption(TransformationSessionKeyConstants.PROJECT_NAME,getProjectName(masterCopy));
//GJD 20061031 d139743
			exportOptions.setAdditionalOption(TransformationSessionKeyConstants.OBJECT_TO_FILE_NAME_REGISTRY, new HashMap());
			
			exportSession.setExportOption(exportOptions);
			
			//AZ 20070927 clean out the rule handler factory registry if there is any
			
			RuleHandlerRegistry.getInstance().clearRegisteredFactory();
			
			aTransformationEngine.execute(modelElements, exportSession);
		}catch(RuntimeException ex){
			throw ex;
		}finally{
			ivWrapperCommand = TransformationSessionUtil.getUpdateCommand(context);
			aTransformationEngine.deregisterTransformationSession();
			LoggingUtil.traceEntry(this, "execute");		
		}


//		monitor = exportSession.getProgressMonitor();
//		if(monitor != null){
//			monitor.worked(ProgressCalculator.getComplexTask());
//		}
//		
//		if(monitor != null && monitor.isCanceled()){
//			throw new OperationCanceledException();
//		}
		
//		aTransformationEngine.deregisterTransformationSession();	
		
//		if(monitor != null){
//			monitor.worked(ProgressCalculator.getSimpleTask());
//		}	
//		LoggingUtil.traceEntry(this, "execute");		
	}
	

	public Activity getWorkingCopy(){
		return ivWorkingCopy;
	}
	public void setWorkingCopy(Activity workingCopy){
		ivWorkingCopy = workingCopy;
	}
	public BtCompoundCommand getUpdateCommand(){
		return ivWrapperCommand;
	}
	/**
	 * @return Returns the isOverwrite.
	 */
	public boolean isOverwrite() {
		return ivIsOverwrite;
	}
	/**
	 * @param isOverwrite The isOverwrite to set.
	 */
	public void setOverwrite(boolean isOverwrite) {
		ivIsOverwrite = isOverwrite;
	}
	private String getProjectName(Activity masterCopy){
		String projectName = null;
		try {		
			// Load the plugin
			Bundle plugin = Platform.getBundle("com.ibm.btools.blm.ui.navigation");			
			// Just in case it doesn't exist
			if (plugin == null) {
				this.logError(masterCopy);			
			}
			Class BLMEditorInputClass = plugin.loadClass("com.ibm.btools.blm.ui.navigation.manager.util.BLMEditorInput");
			
			Bundle pluginNavModel = Platform.getBundle("com.ibm.btools.blm.ui.navigation.model");
			if (pluginNavModel == null){
				this.logError(masterCopy);
			}			
			Class AbstractChildLeafNodeClass = pluginNavModel.loadClass("com.ibm.btools.blm.ui.navigation.model.AbstractChildLeafNode");
			// Returns a array of references to open editors in this page. 
			//	Note that each page has its own editors; editors are never shared between pages. 
			IEditorReference[] refs =
				PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getEditorReferences();			
			IEditorReference ref = null;
			IEditorPart editor = null;
			if (refs.length > 0) {
				for(int i=0;i<refs.length;i++){
					ref = refs[i];
					editor = ref.getEditor(false);
					if (editor != null) {
						Object BLMEditorInputObj = editor.getEditorInput();					
						if (editor.getEditorInput() != null && BLMEditorInputClass.isInstance(BLMEditorInputObj)) {						
							Class params[] = {};
							Object paramsObj[] = {};
							
							//get the project name
							Method method_getProjectName = BLMEditorInputObj.getClass().getMethod("getProjectName", params);						
							projectName = (String)method_getProjectName.invoke(BLMEditorInputObj,paramsObj);
							
							//get the navigation node
							Method method_getNode = BLMEditorInputObj.getClass().getMethod("getNode", params);
							Object abstractChildLeafNodeObj = method_getNode.invoke(BLMEditorInputObj, paramsObj);
							
							//get the bomUID that the node refers to
							Method bomUIDMethod = AbstractChildLeafNodeClass.getMethod("getBomUID", params);
							String nodeBomUID = (String) bomUIDMethod.invoke(abstractChildLeafNodeObj, paramsObj);	
							
							if(nodeBomUID != null && nodeBomUID.equals(masterCopy.getUid())){						
								if(projectName == null){
									this.logError(masterCopy);
								}
								return projectName;
							}
						}
					}
				}	
			}
		} catch(ClassNotFoundException e) {
			this.logError(masterCopy);
		} catch(NoSuchMethodException e) {
			this.logError(masterCopy);
		} catch(IllegalAccessException e) {
			this.logError(masterCopy);
		} catch(InvocationTargetException e) {
			this.logError(masterCopy);
		}	
		if(projectName == null){
			this.logError(masterCopy);
		}
		return projectName;
	}
	
	private void logError(Activity masterCopy){
		String params[] = {masterCopy.getName()};
		LoggingUtil.logError(MessageKeys.PROJECT_NAME_NOT_FOUND, params, null);
        BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.PROJECT_NAME_NOT_FOUND));
        throw runtimeEx;
		
	}
	
}
