package com.ibm.btools.te.ilm;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Collection;

import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.ie.ui.ilm.config.model.BOM2ILMTESetting;
import com.ibm.btools.ie.ui.ilm.config.model.TESetting;
import com.ibm.btools.te.framework.rulehandler.RuleHandlerRegistry;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionKeyConstants;

/**
 *  Transformation Engine orchestrates the whole transformation 
 *  from BOM to BPGraph to IT Level Graphs to IT artifacts.
 */
public class TransformationEngine {
	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	//private static ExportSession exportSession=null;
	private static TransformationSession transformationSession = null;
		
	/**
	 * Executes all the intermediate transformers
	 * <ul>
	 * <li>BOM Transformer
	 * <li>Heuristics Transformer
	 * <li>IT level Transformer
	 * <li>ILM Transformer
	 * </ul>
	 * @param sources input to transformation engine
	 * @return Collection - the results of the transformation	 
	 */
	//public Collection execute(Collection sources, TESetting teSetting)
	public Collection execute(Collection sources, ExportSession session)
	{
		LoggingUtil.traceEntry(this, "execute");
		
		// Initialize the error and warning count
		LoggingUtil.resetErrorCount();
		LoggingUtil.resetWarningCount();	
		
		/* Reset the RuleHandlerRegistry to throw away any previous state */
		RuleHandlerRegistry.getInstance().reset();
		
		//Retrieve te setting from the exoprt options
		TESetting teSetting = (TESetting)session.getExportOptions().getAdditionalOption(TransformationSessionKeyConstants.TE_SETTING);
			
		if (teSetting instanceof BOM2ILMTESetting) {		
			// Transform to BPGraph
			Transformer bpGraphT = TransformerFactory.createTransformer(sources);
			//Collection bpGraph = bpGraphT.transform(sources,teSetting);
			Collection bpGraph = bpGraphT.transform(sources,session);
		
			// Transform to SAGraphs and Data Definitions
			Transformer heuristicsT = TransformerFactory.createTransformer(bpGraph);
			//Collection heuristicsResults = heuristicsT.transform(bpGraph,teSetting);
			Collection heuristicsResults = heuristicsT.transform(bpGraph,session);			
			//deregisterExportSession();
			LoggingUtil.traceExit(this, "execute");
			return heuristicsResults;
		}
		//deregisterExportSession();
		LoggingUtil.traceExit(this, "execute");
		return null;
	}
	
	/**
	 * Get the total error logs count
	 * @return the total error logs count
	 */
	public int getErrorCount() {
		return LoggingUtil.getErrorCount();
	}

	/**
	 * Get the total warning logs count
	 * @return the total warning logs count
	 */
	public int getWarningCount() {
		return LoggingUtil.getWarningCount();
	}

	/**
	 * @return
	 */
//	public static ExportSession getExportSession() {
//		return exportSession;
//	}
//
//	/**
//	 * @param session
//	 */
//	public void registerExportSession(ExportSession session) {
//		exportSession = session;
//	}
//
//	public void deregisterExportSession() {
//		registerExportSession(null);
//	}
	
	/**
	 * @param session
	 */
	public void registerTransformationSession(TransformationSession context) {
		transformationSession = context;
	}

	public void deregisterTransformationSession() {
		registerTransformationSession(null);
	}
	
	public static TransformationSession getTransformationSession() {
		return transformationSession;
	}	
}
