package com.ibm.btools.te.ilm;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.internal.resources.ProjectDescription;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.osgi.framework.Bundle;

import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.util.exception.BTRuntimeException;
import com.ibm.wbit.bpm.trace.model.BPMCompareMergeOperation;
import com.ibm.wsspi.sca.scdl.DocumentRoot;
import com.ibm.wsspi.sca.scdl.LibraryDependency;
import com.ibm.wsspi.sca.scdl.ModuleAndLibraryAttributes;
import com.ibm.wsspi.sca.scdl.SCDLFactory;
import com.ibm.wsspi.sca.scdl.util.SCDLResourceFactoryImpl;


/**
 * Provides a set of API to create WID Project
 * 
 */
public class WIDProjectUtil {


	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	public static final String PLUGIN_ID = "com.ibm.wbit.project";//$NON-NLS-1$
	public static final String NATURE_SHARED_ARTIFACT_MODULE = PLUGIN_ID+".sharedartifactmodulenature";//$NON-NLS-1$
	public static final String NATURE_GENERAL_MODULE = PLUGIN_ID+".generalmodulenature";//$NON-NLS-1$
	public static final String NATURE_JAVA = "org.eclipse.jdt.core.javanature";//$NON-NLS-1$

	public static final String PROJECT_TYPE_MODULE="PROJECT_TYPE_MODULE";
	public static final String PROJECT_TYPE_LIBRARY="PROJECT_TYPE_LIBRARY";
	public static final String MODULE_ATTRIBUTES_DEPENDENCY = "sca.module.attributes";
	public static final String LIBRARY_ATTRIBUTES_DEPENDENCY = "sca.library.attributes";
	public static final String DEFAULT_VERSION ="1.0.0";
	public static final String DEFAULT_VERSION_PROVIDER ="IBM_VRM";

	public static final String classpathFileName = ".classpath";
	public static  ModuleAndLibraryAttributes singleModuleAndLibraryAttributes = null;
	
	
	

	private static String builders [] = new String[]{"com.ibm.ws.rapiddeploy.annotations.core.AnnotationBuilder", "com.ibm.wbit.project.wbiindexsynchronizer",  
		"com.ibm.wbit.project.wbimodulebuilder_prebpel","com.ibm.etools.validation.validationbuilder", "com.ibm.wbit.project.wbimodulebuilder_prejdt",
		"org.eclipse.jdt.core.javabuilder", "com.ibm.wbit.project.wbimodulebuilder_postjdt", "com.ibm.wtp.j2ee.LibCopyBuilder"};

	public static void createModuleProject(String projectName, String location, String[] referencedProjects)
	{
		createProject(PROJECT_TYPE_MODULE,projectName, location, referencedProjects);  
	}

	public static void createLibraryProject(String projectName, String location, String[] referencedProjects)
	{
		createProject(PROJECT_TYPE_LIBRARY,projectName, location, referencedProjects);  
	}

	/**
	 * Creates a WID Module or Library project.  If the WID plugin com.ibm.wbit.ui is presented, 
	 * we will use the API in WID plugin to create the WID Eclipse project in the WID workspace.
	 * Otherwise, we will only create the Project descriptor and persist the descriptor directly.
	 */
	public static void createProject(String projectType,String projectName,String location, String[] referencedProjects) {
		//we need to defend the case that the project directory doesn't exist
		File projectDir = new File(location+projectName);
		if(!projectDir.exists()){
			return;
		}
		Bundle plugin = null;
		//AR avoid generating the WID Projects when direct2Deploy
		if(isShellShared()&& !BpelOptionsUtil.isProjectInterchangeEnabled() && !BpelOptionsUtil.isGenerateForDeployPatternEnabled()){
			plugin = getWidPlugin();
		}		
		if (plugin == null)	{	
			try {
				ProjectDescription des = createProjectDescription(projectType,projectName,location);
				FileOutputStream os = new FileOutputStream(location+projectName+File.separator+IProjectDescription.DESCRIPTION_FILE_NAME);
				TEModelObjectWriter mow = new TEModelObjectWriter();	
				if(referencedProjects != null && referencedProjects.length == 1 && referencedProjects[0] != null && referencedProjects[0].length()== 0){
					mow.setReferencedProjects(null);
				}else{
					mow.setReferencedProjects(referencedProjects);			
				}
				mow.write(des, os);
			} catch (IOException ioex) {
				BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_FAILED_TO_CREATE_PROJECT));
				e.setChainedException(ioex);
				LoggingUtil.logError(MessageKeys.TE_FAILED_TO_CREATE_PROJECT);
				throw e;
			}
		} else {
			createProjectInWID(projectType,projectName,location,plugin);
		}
		if (referencedProjects!=null && referencedProjects.length>0) {
			createClassPathFile(referencedProjects,location,projectName);
			createDependencyFile(projectType, referencedProjects,location, projectName);
		}
	}

	
	/**
	 * Creates the dependencyFile based on the referenced project list. The version of the referenced projects is collected from the
	 * Options bean passed into the transformation
	 * @param projectType
	 * @param referencedProjects
	 * @param location
	 * @param projectName
	 */
	private static void createDependencyFile(String projectType, String[] referencedProjects, String location, String projectName) {
			String fileName = null;
			if(projectType.equals(PROJECT_TYPE_MODULE)){
				fileName = MODULE_ATTRIBUTES_DEPENDENCY;
			}else{
				fileName = LIBRARY_ATTRIBUTES_DEPENDENCY;
			}
			ModuleAndLibraryAttributes moduleAndLibAttributes = null;
			if(fileName == MODULE_ATTRIBUTES_DEPENDENCY && BpelOptionsUtil.isGenerateForDeployPatternEnabled()){
				createDependencyFileForModule(projectType, referencedProjects, location, projectName);
				return;
			}else{
				 moduleAndLibAttributes = SCDLFactory.eINSTANCE.createModuleAndLibraryAttributes();
			}
			
			String libModuleVersion = BpelOptionsUtil.getWPSVersionForProject(projectName);
			if(libModuleVersion != null){
				moduleAndLibAttributes.setVersionValue(libModuleVersion);
			}else{
				moduleAndLibAttributes.setVersionValue(DEFAULT_VERSION);
			}
			moduleAndLibAttributes.setVersionProvider(DEFAULT_VERSION_PROVIDER);
			for (int i=0; i<referencedProjects.length; i++) {
				if(referencedProjects[i] != null && referencedProjects[i].length()!= 0){
					LibraryDependency libDependency = SCDLFactory.eINSTANCE.createLibraryDependency();
					libDependency.setName(referencedProjects[i]);
					String libVersion = BpelOptionsUtil.getWPSVersionForProject(referencedProjects[i]);
					if(libVersion != null){
						libDependency.setVersion(libVersion);
					}else{
						libDependency.setVersion(DEFAULT_VERSION);
					}
					moduleAndLibAttributes.getLibraryDependency().add(libDependency);
				}
			}
			
		SCDLResourceFactoryImpl factory = new SCDLResourceFactoryImpl();
			try{
				URI uri = URI.createFileURI(location+projectName+File.separator+fileName);
				Resource resource = factory.createResource(uri);
				
				DocumentRoot docRoot = SCDLFactory.eINSTANCE.createDocumentRoot();
				docRoot.setModuleAndLibraryAttributes(moduleAndLibAttributes);
				resource.getContents().add(docRoot);
				Map<String, String> options = new HashMap<String, String>();
				options.put(XMLResource.OPTION_ENCODING, "UTF-8");
				resource.save(options);
			
			} catch (Exception ex) {
				//cannot save the library dependency file.. 
				BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_FAILED_TO_CREATE_PROJECT));
				e.setChainedException(ex);
				LoggingUtil.logError(MessageKeys.TE_FAILED_TO_CREATE_PROJECT);
				throw e;    		
			}
	}
	
	/**
	 * Creates the dependencyFile based on the referenced project list. The version of the referenced projects is collected from the
	 * Options bean passed into the transformation
	 * @param projectType
	 * @param referencedProjects
	 * @param location
	 * @param projectName
	 */
	private static void createDependencyFileForModule(String projectType, String[] referencedProjects, String location, String projectName) {
			String fileName = null;
			if(projectType.equals(PROJECT_TYPE_MODULE)){
				fileName = MODULE_ATTRIBUTES_DEPENDENCY;
			}
				if(singleModuleAndLibraryAttributes == null){
					singleModuleAndLibraryAttributes = SCDLFactory.eINSTANCE.createModuleAndLibraryAttributes();
				}

			String libModuleVersion = BpelOptionsUtil.getWPSVersionForProject(projectName);
			if(libModuleVersion != null && singleModuleAndLibraryAttributes.getVersionValue() == null){
				singleModuleAndLibraryAttributes.setVersionValue(libModuleVersion);
			}else{
				singleModuleAndLibraryAttributes.setVersionValue(DEFAULT_VERSION);
			}
			singleModuleAndLibraryAttributes.setVersionProvider(DEFAULT_VERSION_PROVIDER);
			List incommingDependencies = singleModuleAndLibraryAttributes.getLibraryDependency();
			List<String> dependencies = new LinkedList<String>();
			if(incommingDependencies != null){
				Iterator it = incommingDependencies.iterator();
				while(it.hasNext()){
					LibraryDependency dep = (LibraryDependency)it.next();
					dependencies.add(dep.getName());
				}
			}
				for (int i=0; i<referencedProjects.length; i++) {
					if(referencedProjects[i] != null && referencedProjects[i].length()!= 0){
						String thisDepName = referencedProjects[i];
						if(!dependencies.contains(thisDepName)){
							LibraryDependency libDependency = SCDLFactory.eINSTANCE.createLibraryDependency();
							libDependency.setName(referencedProjects[i]);
							String libVersion = BpelOptionsUtil.getWPSVersionForProject(referencedProjects[i]);
							if(libVersion != null){
								libDependency.setVersion(libVersion);
							}else{
								libDependency.setVersion(DEFAULT_VERSION);
							}
							singleModuleAndLibraryAttributes.getLibraryDependency().add(libDependency);
						}
					}
				}

		SCDLResourceFactoryImpl factory = new SCDLResourceFactoryImpl();
			try{
				URI uri = URI.createFileURI(location+projectName+File.separator+fileName);
				Resource resource = factory.createResource(uri);
				
				DocumentRoot docRoot = SCDLFactory.eINSTANCE.createDocumentRoot();
				docRoot.setModuleAndLibraryAttributes(singleModuleAndLibraryAttributes);
				resource.getContents().add(docRoot);
				Map<String, String> options = new HashMap<String, String>();
				options.put(XMLResource.OPTION_ENCODING, "UTF-8");
				resource.save(options);
			
			} catch (Exception ex) {
				//cannot save the library dependency file.. 
				BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_FAILED_TO_CREATE_PROJECT));
				e.setChainedException(ex);
				LoggingUtil.logError(MessageKeys.TE_FAILED_TO_CREATE_PROJECT);
				throw e;    		
			}
	}
	/**
	 * Return the WID Plugin that contains the API to create Module/Library project
	 * @return
	 */
	public static Bundle getWidPlugin() {
	    return Platform.getBundle("com.ibm.wbit.ui");		
	}

	/**
	 * Create the Project Description (.project) that contains the minimum information (i.e. the nature)
	 * When importing to WID, WID will automatically add the builder/validator info and recreate the .classpath
	 * @param projectType
	 * @param projectName
	 * @param location
	 * @return
	 */
	protected static ProjectDescription createProjectDescription(String projectType,String projectName,String location) {
		ProjectDescription description = new ProjectDescription();
		IPath newPath = new Path(location+projectName);
		description.setName(projectName);
		description.setLocation(newPath);
		String [] natureIDs = new String[2];
		if (projectType.equals(PROJECT_TYPE_MODULE)) {
			natureIDs[0]=NATURE_GENERAL_MODULE;
		} else {
			natureIDs[0]=NATURE_SHARED_ARTIFACT_MODULE;			
		}
		natureIDs[1]=NATURE_JAVA;
		description.setNatureIds(natureIDs);

		//defect 120449 do not create the BuildSpec
//		int lenght = builders.length;
//		ICommand [] command = new ICommand[lenght];
//		//defect 120216	
//		BuildCommand[] buildCommands = new BuildCommand[lenght];
//		for (int i=0; i<lenght; i++){
//		buildCommands[i] = new BuildCommand();
//		buildCommands[i].setName(builders[i]);
//		command[i] = buildCommands[i];
//		}

//		description.setBuildSpec(command);



		return description;
	}

	/**
	 * Create the WID project using WID API.  Reflection API is used because we don't want hard dependency on WID
	 * plugin 
	 * @param projectType
	 * @param projectName
	 * @param location
	 * @param plugin
	 */
	protected static void createProjectInWID(String projectType,String projectName,String location, Bundle plugin)
	{       
		try {
			Class moduleCreationOperationClass = plugin.loadClass("com.ibm.wbit.ui.operations.ModuleCreationOperation");
			if (moduleCreationOperationClass!=null) {
				Class constParams[] = {String.class,IPath.class,int.class,IProject.class};
				Constructor constructor = moduleCreationOperationClass.getConstructor(constParams);
				IPath newPath = new Path(location);
				if(!Platform.getLocation().equals(newPath))
					newPath = new Path(location + projectName);

				//IPath newPath = new Path(location + projectName);
				Field field;
				if (projectType.equals(PROJECT_TYPE_MODULE)) {
					field = moduleCreationOperationClass.getField("GENERAL_MODULE");
				} else {
					field = moduleCreationOperationClass.getField("SHARED_ARTIFACT_MODULE");
				}
				int type = field.getInt(null);
				Object []params = {projectName,newPath,new Integer(type),null};
				Object op = constructor.newInstance(params);
				Class [] runClass = {IProgressMonitor.class};
				Method run = moduleCreationOperationClass.getMethod("run",runClass);
				Object [] runParams = {null};
				run.invoke(op,runParams);
			}    		
		} catch (Exception ex) {
			BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_FAILED_TO_CREATE_PROJECT));
			e.setChainedException(ex);
			LoggingUtil.logError(MessageKeys.TE_FAILED_TO_CREATE_PROJECT);
			throw e;
		}
	}	

	/**
	 * Create the WID project using WID API.  Reflection API is used because we don't want hard dependency on WID
	 * plugin 
	 * @param projectType
	 * @param projectName
	 * @param location
	 * @param plugin
	 */
	public static void createProjectsInWID(final String libraryName, final String moduleName, final String location, final Bundle plugin)
	{       
		try {
			final WorkspaceModifyOperation operation = new WorkspaceModifyOperation(){
				protected void execute(IProgressMonitor monitor) throws CoreException {		
					try{
						Class moduleCreationOperationClass = plugin.loadClass("com.ibm.wbit.ui.operations.ModuleCreationOperation");
						if (moduleCreationOperationClass!=null) {
							Class constParams[] = {String.class,IPath.class,int.class,IProject.class};
							Constructor constructor = moduleCreationOperationClass.getConstructor(constParams);
							//first create the Library
							IPath newPath = new Path(location);
							if(!Platform.getLocation().equals(newPath))
								newPath = new Path(location + libraryName);
							Field field = moduleCreationOperationClass.getField("SHARED_ARTIFACT_MODULE");

							//IPath newPath = new Path(location + projectName);
							int type = field.getInt(null);
							Object op = null;
							Object []params = {libraryName,newPath,new Integer(type),null};
							op = constructor.newInstance(params);

							Class [] runClass = {IProgressMonitor.class};
							Method run = moduleCreationOperationClass.getMethod("run",runClass);
							Object [] runParams = {null};
							run.invoke(op,runParams);
							//now create the Module
							if(!Platform.getLocation().equals(newPath))
								newPath = new Path(location + moduleName);
							field = moduleCreationOperationClass.getField("GENERAL_MODULE");

							//IPath newPath = new Path(location + projectName);
							type = field.getInt(null);
							Object []params1 = {moduleName, newPath,new Integer(type),null};
							op = constructor.newInstance(params1);

							Class [] runClass1 = {IProgressMonitor.class};
							run = moduleCreationOperationClass.getMethod("run",runClass1);
							Object [] runParams1 = {null};
							run.invoke(op,runParams1);
							//now update the Module with the Librart path
							IProject moduleProject = ResourcesPlugin.getWorkspace().getRoot().getProject(moduleName);
							IProject referencedProject = ResourcesPlugin.getWorkspace().getRoot().getProject(libraryName);

							IProjectDescription projDesc = moduleProject.getDescription();
							//load class path entries into a list that we can manipulate
							ArrayList classPathEntries = new ArrayList();
							IJavaProject javaProject = JavaCore.create(moduleProject);
							IClasspathEntry[] classPath = javaProject.getRawClasspath();
							for(int i=0; i<classPath.length; i++){
								classPathEntries.add(classPath[i]);
							}
							classPathEntries.add(JavaCore.newProjectEntry(referencedProject.getFullPath(), true));
							//flush changes for java
							IClasspathEntry[] newClassPath = new IClasspathEntry[classPathEntries.size()];
							classPathEntries.toArray(newClassPath);
							javaProject.setRawClasspath(newClassPath, new NullProgressMonitor());
							//flush changes for static refs
							IProject[] newRefs = new IProject[]{referencedProject};
							projDesc.setReferencedProjects(newRefs);
							moduleProject.setDescription(projDesc, new NullProgressMonitor());
						}

					}catch (Exception ex) {
						System.out.println(ex);
						BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_FAILED_TO_CREATE_PROJECT));
						e.setChainedException(ex);
						LoggingUtil.logError(MessageKeys.TE_FAILED_TO_CREATE_PROJECT);
						throw e;
					}		
				}
			};
			operation.run(new NullProgressMonitor());
		}catch (Exception ex) {
			System.out.println(ex);
			BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_FAILED_TO_CREATE_PROJECT));
			e.setChainedException(ex);
			LoggingUtil.logError(MessageKeys.TE_FAILED_TO_CREATE_PROJECT);
			throw e;
		}   			
	}	
   /**
    * This method is to handle creating projects in WID for a cross project reference setting
    * 1. WBM and WID share shell. i.e. WBM installed on top of WID
    * 2. WID export without project interchange option selected
    * Assumption:
    * The UI will validate the project names against the WID workspace to ensure the name
    * uniqueness of the incoming project. Because we cannot handle the name collision when 
    * we invoke the WID API to create the projects
    * @param location
    * @param plugin
    */
	public static void createProjectsInWIDXP(final String location, final Bundle plugin)
	{       
		try {
			final WorkspaceModifyOperation operation = new WorkspaceModifyOperation(){
				protected void execute(IProgressMonitor monitor) throws CoreException {		
					try{
						Class moduleCreationOperationClass = plugin.loadClass("com.ibm.wbit.ui.operations.ModuleCreationOperation");
						if (moduleCreationOperationClass!=null) {
							List<String> projectNameList = CrossProjectReferenceUtil.getProjectNameListForExportSession();
							Class constParams[] = {String.class,IPath.class,int.class,IProject.class};
							Constructor constructor = moduleCreationOperationClass.getConstructor(constParams);
							IPath newPath = new Path(location);
							Field field = null;
							int type = 0;
							Object op = null;
							Method run = null;
							String libraryName = null;
							String moduleName = null;
							String implName = null;

							//first create the Libraries  
							String[] libProjectNames = getLibraryProjectNames(projectNameList);
							if(libProjectNames != null && libProjectNames.length !=0){
								for(int i=0;i<libProjectNames.length;i++){
									libraryName = libProjectNames[i];
									if(!CrossProjectReferenceUtil.existProjectDirectory(location,libraryName)){
										continue;
									}
									newPath = new Path(location);
									if(!Platform.getLocation().equals(newPath))
										newPath = new Path(location + libraryName);
									field = moduleCreationOperationClass.getField("SHARED_ARTIFACT_MODULE");

									//IPath newPath = new Path(location + projectName);
									type = field.getInt(null);    						
									Object []params = {libraryName,newPath,new Integer(type),null};
									op = constructor.newInstance(params);

									Class [] runClass = {IProgressMonitor.class};
									run = moduleCreationOperationClass.getMethod("run",runClass);
									Object [] runParams = {null};
									run.invoke(op,runParams);
								}
								//Wire library project reference if there is any
								for(int j=0;j<libProjectNames.length;j++){
									String eachLibName = libProjectNames[j];
									if(!CrossProjectReferenceUtil.existProjectDirectory(location,eachLibName)){
										continue;
									}
									String[] projectReference = CrossProjectReferenceUtil.getProjectReferencesForLib(eachLibName);
									if(projectReference != null && projectReference.length != 0){
										IProject libIProject = ResourcesPlugin.getWorkspace().getRoot().getProject(eachLibName);
										List<IProject> referenceIProjectList = new ArrayList<IProject>();
										for(int k=0;k<projectReference.length;k++){
											if(projectReference[k] != null && projectReference[k].length()==0){
												continue;
											}
											IProject eachLibReferenceIProject = ResourcesPlugin.getWorkspace().getRoot().getProject(projectReference[k]);
											referenceIProjectList.add(eachLibReferenceIProject);
										}
										
										//now update the library dependencies	
										IProject[] referencedProjects = (IProject[])referenceIProjectList.toArray(new IProject[]{});
										wireProjectReference(libIProject,referencedProjects);
									}													
								}																												
							}
							
							//Resolve the lib project references
							String [] referencedLibProjects = CrossProjectReferenceUtil.getProjectReferences();
							List<IProject> libReferenceList = new ArrayList<IProject>();
							for(int k=0;k<referencedLibProjects.length;k++){
								if(referencedLibProjects[k] == null ||referencedLibProjects[k].length() == 0){
									continue;
								}
								IProject eachLibReferenceIProject = ResourcesPlugin.getWorkspace().getRoot().getProject(referencedLibProjects[k]);
								libReferenceList.add(eachLibReferenceIProject);
							}
							IProject[] referencedProjects = (IProject[])libReferenceList.toArray(new IProject[]{});
							
							if(projectNameList != null && !projectNameList.isEmpty()){
								for(Iterator projectNameIt=projectNameList.iterator();projectNameIt.hasNext();){
									String projectName = (String)projectNameIt.next();														
									//now create the Module
									if(BpelOptionsUtil.getWPSModuleName(projectName)!= null &&
											CrossProjectReferenceUtil.existProjectDirectory(location, BpelOptionsUtil.getWPSModuleName(projectName))){
										moduleName = BpelOptionsUtil.getWPSModuleName(projectName);
										if(!Platform.getLocation().equals(newPath))
											newPath = new Path(location + moduleName);
										field = moduleCreationOperationClass.getField("GENERAL_MODULE");

										type = field.getInt(null);
										Object []params1 = {moduleName, newPath,new Integer(type),null};
										op = constructor.newInstance(params1);

										Class [] runClass1 = {IProgressMonitor.class};
										run = moduleCreationOperationClass.getMethod("run",runClass1);
										Object [] runParams1 = {null};
										run.invoke(op,runParams1);
										
										//now update the Module with the Librart path										
										IProject moduleProject = ResourcesPlugin.getWorkspace().getRoot().getProject(moduleName);
										wireProjectReference(moduleProject, referencedProjects);										
									}

									//now create the implementation module
									if(BpelOptionsUtil.getImplementationModuleName(projectName)!= null && 
											CrossProjectReferenceUtil.existProjectDirectory(location, BpelOptionsUtil.getImplementationModuleName(projectName))){										
										implName = BpelOptionsUtil.getImplementationModuleName(projectName);
										if(!Platform.getLocation().equals(newPath))
											newPath = new Path(location + implName);
										field = moduleCreationOperationClass.getField("GENERAL_MODULE");
										
										type = field.getInt(null);
										Object []params1 = {implName, newPath,new Integer(type),null};
										op = constructor.newInstance(params1);

										Class [] runClass1 = {IProgressMonitor.class};
										run = moduleCreationOperationClass.getMethod("run",runClass1);
										Object [] runParams1 = {null};
										run.invoke(op,runParams1);
										
										//now update the Module with the Librart path										
										IProject implProject = ResourcesPlugin.getWorkspace().getRoot().getProject(implName);
										wireProjectReference(implProject, referencedProjects);										
									}									
								}
							}
						}
					}catch (Exception ex) {
						System.out.println(ex);
						BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_FAILED_TO_CREATE_PROJECT));
						e.setChainedException(ex);
						LoggingUtil.logError(MessageKeys.TE_FAILED_TO_CREATE_PROJECT);
						throw e;
					}		
				}
				//Wire the project references
				private void wireProjectReference(IProject project, IProject[] referencedProjects) throws CoreException, JavaModelException {
					IProjectDescription projDesc = project.getDescription();
					//load class path entries into a list that we can manipulate
					ArrayList classPathEntries = new ArrayList();
					IJavaProject javaProject = JavaCore.create(project);
					IClasspathEntry[] classPath = javaProject.getRawClasspath();
					for(int i=0; i<classPath.length; i++){
						classPathEntries.add(classPath[i]);
					}
					//Add all project references
					IProject referencedProject = null;
					for(int i=0;i<referencedProjects.length;i++){
						referencedProject = referencedProjects[i];
						if(referencedProjects[i] == null){
							continue;
						}
						
						if(project != referencedProject){
							classPathEntries.add(JavaCore.newProjectEntry(referencedProject.getFullPath(), true));
						}

						//flush changes for java
						IClasspathEntry[] newClassPath = new IClasspathEntry[classPathEntries.size()];
						classPathEntries.toArray(newClassPath);
						javaProject.setRawClasspath(newClassPath, new NullProgressMonitor());
						
						//flush changes for static refs
						IProject[] newRefs = new IProject[]{referencedProject};
						projDesc.setReferencedProjects(newRefs);
						project.setDescription(projDesc, new NullProgressMonitor());
					}
				}
			};
			operation.run(new NullProgressMonitor());
		}catch (Exception ex) {
			System.out.println(ex);
			BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_FAILED_TO_CREATE_PROJECT));
			e.setChainedException(ex);
			LoggingUtil.logError(MessageKeys.TE_FAILED_TO_CREATE_PROJECT);
			throw e;
		}   			
	}	

	private static String[] getLibraryProjectNames(List<String> projectNameList){
		List<String> libProjectNames = new ArrayList<String>();
		if(projectNameList == null || projectNameList.isEmpty()){
			return (String[])libProjectNames.toArray(new String[]{});
		}
		if(projectNameList != null && !projectNameList.isEmpty()){
			for(Iterator projectNameIt=projectNameList.iterator();projectNameIt.hasNext();){
				String projectName = (String)projectNameIt.next();	
				if(BpelOptionsUtil.getWPSLibraryName(projectName)!= null){
					libProjectNames.add(BpelOptionsUtil.getWPSLibraryName(projectName));
				}
			}
		}		
		return (String[])libProjectNames.toArray(new String[]{});
	}

	/**
	 * Create the WID project using WID API.  Reflection API is used because we don't want hard dependency on WID
	 * plugin 
	 * @param projectType
	 * @param projectName
	 * @param location
	 * @param plugin
	 */
	public static void createProjectsInWIDXProjects(final String[] libraryNames, final String[] moduleNames, final String location, final Bundle plugin)
	{       
		try {
			final WorkspaceModifyOperation operation = new WorkspaceModifyOperation(){
				protected void execute(IProgressMonitor monitor) throws CoreException {		
					try{
						Class moduleCreationOperationClass = plugin.loadClass("com.ibm.wbit.ui.operations.ModuleCreationOperation");
						if (moduleCreationOperationClass!=null) {    						
							Class constParams[] = {String.class,IPath.class,int.class,IProject.class};
							Constructor constructor = moduleCreationOperationClass.getConstructor(constParams);
							IPath newPath = new Path(location);
							Field field = null;
							int type = 0;
							Object op = null;
							Method run = null;
							String libraryName = null;
							String moduleName = null;

							//first create the Library
							//for(int i=0;i<libraryNames.length;i++){
							libraryName = libraryNames[0];    							
							if(!Platform.getLocation().equals(newPath))
								newPath = new Path(location + libraryName);
							field = moduleCreationOperationClass.getField("SHARED_ARTIFACT_MODULE");

							//IPath newPath = new Path(location + projectName);
							type = field.getInt(null);    							
							Object []params = {libraryName,newPath,new Integer(type),null};
							op = constructor.newInstance(params);

							Class [] runClass = {IProgressMonitor.class};
							run = moduleCreationOperationClass.getMethod("run",runClass);
							Object [] runParams = {null};
							run.invoke(op,runParams);
							//}
							//now create the Module
							//for(int i=0;i<moduleNames.length;i++){
							moduleName = moduleNames[0];
							if(!Platform.getLocation().equals(newPath))
								newPath = new Path(location + moduleName);
							field = moduleCreationOperationClass.getField("GENERAL_MODULE");

							//IPath newPath = new Path(location + projectName);
							type = field.getInt(null);
							Object []params1 = {moduleName, newPath,new Integer(type),null};
							op = constructor.newInstance(params1);

							Class [] runClass1 = {IProgressMonitor.class};
							run = moduleCreationOperationClass.getMethod("run",runClass1);
							Object [] runParams1 = {null};
							run.invoke(op,runParams1);
							//}
							//now update the Module with the Librart path
							IProject moduleProject = ResourcesPlugin.getWorkspace().getRoot().getProject(moduleName);
							IProject referencedProject = ResourcesPlugin.getWorkspace().getRoot().getProject(libraryName);

							IProjectDescription projDesc = moduleProject.getDescription();
							//load class path entries into a list that we can manipulate
							ArrayList classPathEntries = new ArrayList();
							IJavaProject javaProject = JavaCore.create(moduleProject);
							IClasspathEntry[] classPath = javaProject.getRawClasspath();
							for(int i=0; i<classPath.length; i++){
								classPathEntries.add(classPath[i]);
							}
							classPathEntries.add(JavaCore.newProjectEntry(referencedProject.getFullPath(), true));
							//flush changes for java
							IClasspathEntry[] newClassPath = new IClasspathEntry[classPathEntries.size()];
							classPathEntries.toArray(newClassPath);
							javaProject.setRawClasspath(newClassPath, new NullProgressMonitor());
							//flush changes for static refs
							IProject[] newRefs = new IProject[]{referencedProject};
							projDesc.setReferencedProjects(newRefs);
							moduleProject.setDescription(projDesc, new NullProgressMonitor());
						}

					}catch (Exception ex) {
						System.out.println(ex);
						BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_FAILED_TO_CREATE_PROJECT));
						e.setChainedException(ex);
						LoggingUtil.logError(MessageKeys.TE_FAILED_TO_CREATE_PROJECT);
						throw e;
					}		
				}
			};
			operation.run(new NullProgressMonitor());
		}catch (Exception ex) {
			System.out.println(ex);
			BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_FAILED_TO_CREATE_PROJECT));
			e.setChainedException(ex);
			LoggingUtil.logError(MessageKeys.TE_FAILED_TO_CREATE_PROJECT);
			throw e;
		}


	}	


	protected static void createClassPathFile(String[] referencedProjects, String location, String projectName) {
		StringBuffer classpathString = new StringBuffer(classpathFragment1);
		if(BpelOptionsUtil.isGenerateForDeployPatternEnabled()){
			classpathString.append(classpathCPEntryFragmentK);
		}
		for (int i=0; i<referencedProjects.length; i++) {
			if(referencedProjects[i] != null && referencedProjects[i].length()!= 0){
				classpathString.append(classpathCPEntryFragment1);
				classpathString.append(referencedProjects[i]);
				classpathString.append(classpathCPEntryFragment2);
			}
		}
		classpathString.append(classpathFragment2);
		try {
			File file = new File(location+projectName+File.separator+classpathFileName);
			String encoding = "UTF-8";
			PrintWriter pw = new PrintWriter(new OutputStreamWriter(new FileOutputStream(file), encoding));
			pw.print(classpathString.toString());
			pw.flush();
			pw.close();
//			FileOutputStream fos = new FileOutputStream(location+projectName+File.separator+classpathFileName);    
//			fos.write(classpathString.toString().getBytes());
//			fos.close();
		} catch (Exception ex) {
			BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_FAILED_TO_CREATE_PROJECT));
			e.setChainedException(ex);
			LoggingUtil.logError(MessageKeys.TE_FAILED_TO_CREATE_PROJECT);
			throw e;    		
		}

	}
	
	/**
	 * Check if the contribution point is present, which is contribute by the compare/merge processor plugin
	 * If the contribution point exists, then its shell sharing
	 * @return true if it's shell sharing
	 */
	public static boolean isShellShared(){
		return BPMCompareMergeOperation.isEnabled();
	}

	public static final String classpathFragment1="<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<classpath>\n";
	public static final String classpathCPEntryFragment1="\t<classpathentry exported=\"true\" kind=\"src\" path=\"/";
	public static final String classpathCPEntryFragment2="\"/>\n";
	public static final String classpathFragment2="</classpath>";
	public static final String classpathCPEntryFragmentK = "\t<classpathentry kind=\"src\" path=\"gen/src\">\n\t<attributes>\n\t\t<attribute name =\"optional\" value=\"true\"/>\n\t</attributes>\n\t</classpathentry>\n"; ;
}

