package com.ibm.btools.te.ilm.attribute.wps;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.actions.ControlAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.attributes.helper.TechnicalAttributesHelper;
import com.ibm.btools.te.attributes.helper.UpdateCommandHelper;
import com.ibm.btools.te.attributes.resource.MessageKeys;
import com.ibm.btools.te.ilm.TransformationEngine;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry;
import com.ibm.btools.te.ilm.heuristics.naming.TargetNamespaceProvider;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Empty;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.bpel.Receive;
import com.ibm.wbit.bpel.Reply;

/**
* Class responsible for providing values for BPEL type attributes. 
*/
public class BpelAttributeValueProvider extends AttributeValueProvider {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	/**
	 * Returns the <code>propertyName</code> for <code>fromSource</code>.
	 * @param propertyName the name of the property.
	 * @return the value of the attribute for the type.
	 */
	public Object getAttributeValueForType(NamedElement fromSource, String propertyName) {
		return getAttributeValueForType(getObjectType(propertyName), fromSource, propertyName, null);
	}

	/**
	 * Returns the technical attribute value for the <code>propertyName<code> for the Object <code>forType</code> from the specified <code>registry</code>
	 * @param type an EObject
	 * @param attribute the attribute name of the type
	 * @param context a TransformationContext
	 * @return the value of the attribute for the type.
	 */
	public Object getAttributeValueForType(Object forType, NamedElement fromSource, String propertyName, NamingRegistry registry) {
		if(isNameAttribute(propertyName)){
			return getName(forType, fromSource, propertyName, registry);
		}else if(isDisplayName(propertyName)){
			return getDisplayName(forType, fromSource, propertyName, registry);
		}else if(propertyName.equals(AttributeNameConstants.BPEL_PROCESS_TARGET_NAMESPACE)){
			TargetNamespaceProvider namespaceProvider = new TargetNamespaceProvider();
			namespaceProvider.setContext(getContext());
			return namespaceProvider.getTargetNamespace(forType, fromSource, propertyName, context, true);
		}else{
			return null;
		}
	}
	
	/**
	 * Returns the value of the name attribute for the <code>type</code> Object derived from the <code>fromSource</code>.
	 * @param type an BPEL Object type
	 * @param fromSource an BOM NamedElement
	 * @param attribute attribute 
	 * @param registry a NamingRegistry
	 * @return String name of BPEL element. 
	 */
	protected String getName(Object type, EObject fromSource, String attribute, NamingRegistry registry){
		NameProvider nameProvider = NameProviderFactory.getNameProvider(type);
		nameProvider.setContext(this.getContext());
		nameProvider.setParentContext(inParentContext);
		String	valueForType = nameProvider.getName(type, (NamedElement)fromSource, registry, attribute);
		return valueForType;
		
	}

	/**
	 * Returns the BPEL display name for a BPEL Element, if applicable. The name is not unique in the registry.
	 * @param type a BPEL Object
	 * @param fromSource the BOM source element.
	 * @param attribute the attribute property name 
	 * @param registry a NamingRegistry
	 * @return String BPEL display name.
	 */
	private String getDisplayName(Object type,  EObject fromSource, String attribute, NamingRegistry registry){
		//here is the logic of retrieving the displayName of the type...
		String tempName = null;
		String name = null;
		if(BpelOptionsUtil.generateDefaultValuesWithOverwrite()){
			//1.just generate default values and ignore the technical attributes
			name = getDisplayNameDefaultValue(type, attribute, fromSource, registry);
			//2c. call the commandUpdateHelper to update the BOM Model TA with new value
			Command command = UpdateCommandHelper.createUpdateCommand((NamedElement)fromSource, attribute, name);
			//2d. call the Registry Helper to add the command to the list of Commands
			TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), command);

		}else if(BpelOptionsUtil.generateDefaultValuesNoOverwrite()){
			//1. retrieve the technical attribute for this type... 
			name = (String)TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
			if(name == null || name.trim().equals("")){
				name = getDisplayNameDefaultValue(type, attribute, fromSource, registry);
				//2c. call the commandUpdateHelper to update the BOM Model TA with new value
				Command command = UpdateCommandHelper.createUpdateCommand((NamedElement)fromSource, attribute, name);
				//2d. call the Registry Helper to add the command to the list of Commands
				TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), command);
			}
		}else if(BpelOptionsUtil.runTransformation()){
			//this means we are running the transformation with user setting without validating.. 
			//1. retrieve the technical attribute for this type... 
			name = (String)TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
			if(name == null || name.trim().equals("")){
				name = getDisplayNameDefaultValue(type, attribute, fromSource, registry);
			}
		}else if(BpelOptionsUtil.runValidation()){
			//the display name should not be validated . 
		}
		return name;
	}
	
	/**
	 * Returns the default display name for a BPEL Element. This is tipycally derived from name of the <code>fromSource</code>.
	 * @param type a BPEL Object
	 * @param attribute the name of the technical attribute
	 * @param fromSource BOM source object
	 * @param context a NamingRegistry
	 * @return String default displayName.
	 */
	private String getDisplayNameDefaultValue(Object type, String attribute, EObject fromSource, NamingRegistry context) {
		String displayName = null;
		NamedElement displaySource = null;
		//tries to handle the case of a callback Invoke, where the source name is taken 
		//from the PinSet name,.
		if(fromSource instanceof OutputPinSet && type instanceof Invoke){
			displaySource = (NamedElement)fromSource;
			NameProvider nameProvider = NameProviderFactory.getNameProvider(type);
			//the display name should not be unique in the registry.. so do not pass in the registry... 
			displayName = nameProvider.getName(type, displaySource, null);
			//displayName = displayName + " Callback";
			displayName += MessageKeys.getString("TAT0031");
			return displayName;
		}
		else
		if(fromSource instanceof PinSet){
			displaySource = (Action)((PinSet)fromSource).eContainer();
		}else if(fromSource instanceof StructuredActivityNode){
			displaySource =(NamedElement)fromSource;
		}else if(fromSource instanceof ControlAction){
			displaySource = (NamedElement)fromSource;
		}
		//AZ 06/22/07 add fix for process reference in a cross project setting
		else if(fromSource instanceof CallBehaviorAction){
			displaySource = (NamedElement)fromSource;
		}
		NameProvider nameProvider = NameProviderFactory.getNameProvider(type);
		//the display name should not be unique in the registry.. so do not pass in the registry... 
		displayName = nameProvider.getName(type, displaySource, null);
		if(type instanceof Receive){
			//displayName = displayName + " Receive";
			//Remove the _Reply from the name defect 80337
		//	displayName += MessageKeys.getString("TAT0030");
		}else if(type instanceof Reply){
			//displayName = displayName + " Reply";
			displayName += MessageKeys.getString("TAT0029");
		}
		
		return displayName;
	}
	
	/**
	 * Returns true if this attribute is for a <code>name<code> property for which we can delegate to the NameProvider.
	 * @param attribute the name of the attribute
	 * @return boolean 
	 */
	protected boolean isNameAttribute(String attribute){
		return 	attribute.equals(AttributeNameConstants.BPEL_VARIABLE_NAME) || attribute.equals(AttributeNameConstants.BPEL_ACTIVITY_NAME) || attribute.equals(AttributeNameConstants.BPEL_PROCESS_NAME) || attribute.equals(AttributeNameConstants.BPEL_VARIABLE_NAME_VARIABLE)
		|| attribute.equals(AttributeNameConstants.BPEL_ACTIVITY_NAME_SWITCH) || attribute.equals(AttributeNameConstants.BPEL_ACTIVITY_NAME_WHILE) || attribute.equals(AttributeNameConstants.BPEL_ACTIVITY_NAME_SCOPE); 
	}

	/**
	 * Returns true if this attribute is for a <code>displayName<code> property.
	 * @param attribute the name of the technical attribute
	 * @return true if this attribute is for a <code>displayName<code> property 
	 */
	protected boolean isDisplayName(String attribute){
		return 	attribute.equals(AttributeNameConstants.BPEL_ACTIVITY_DISPLAY_NAME)|| attribute.equals(AttributeNameConstants.BPEL_PROCESS_DISPLAY_NAME) ||
		attribute.equals(AttributeNameConstants.BPEL_ACTIVITY_DISPLAY_NAME_SWITCH) || attribute.equals(AttributeNameConstants.BPEL_ACTIVITY_DISPLAY_NAME_WHILE) || attribute.equals(AttributeNameConstants.BPEL_ACTIVITY_DISPLAY_NAME_SCOPE);
	}

	/**
	 * Returns the object Class based on the name of the property.
	 * @param propertyName - constant for a specific property. 
	 * @return Object - object Class based on the name of the property
	 */
	private Object getObjectType(String propertyName){
		Class objectType = null;
		if(propertyName.equals(AttributeNameConstants.BPEL_VARIABLE_NAME)){
			objectType = BPELVariable.class;
		}else if(propertyName.equals(AttributeNameConstants.BPEL_PROCESS_DISPLAY_NAME) || propertyName.equals(AttributeNameConstants.BPEL_PROCESS_NAME) || propertyName.equals(AttributeNameConstants.BPEL_PROCESS_TARGET_NAMESPACE)){
			objectType = Process.class;
		}else if(propertyName.equals(AttributeNameConstants.BPEL_ACTIVITY_DISPLAY_NAME) || propertyName.equals(AttributeNameConstants.BPEL_ACTIVITY_NAME)){
			objectType = Activity.class;
		}
		return objectType;
	}
	
	

}
