package com.ibm.btools.te.ilm.attribute.wps;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry;

/**
* This class is responsible for providing values for attributes for Business Rules. 
*/

public class BusinessRuleValueProvider extends AttributeValueProvider{

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2008, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	
	
	/**
	* Return value for Business Rules attribute. 
	 * @param fromSource NamedElement
	 * @param propertyName String
	 * @return the value of the attribute for the type.
	 */
	public Object getAttributeValueForType(NamedElement fromSource, String propertyName) {
		return getAttributeValueForType(null, fromSource, propertyName, null, inParentContext);
	}

	/**
	* Return value for Business Rules attribute. 
	 * @param forType Object
	 * @param fromSource NamedElement
	 * @param propertyName String
	 * @param registry NamingRegistry
	 * @return the value of the attribute for the type.
	 */
	public Object getAttributeValueForType(Object forType, NamedElement fromSource, String propertyName, NamingRegistry registry) {
		return getAttributeValueForType(forType, fromSource, propertyName, registry, inParentContext);
	}
	
	/**
	 * Returns the technical attribute value for the <code>propertyName<code> for the Object <code>forType</code> from the specified <code>registry</code>
	 * @param type an EObject
	 * @param attribute the attribute name of the type
	 * @param context a TransformationContext
	 * @return the value of the attribute for the type.
	 */
	private Object getAttributeValueForType(Object type, NamedElement fromSource, String attribute, NamingRegistry registry, Object inParentContext) {
		if(attribute == AttributeNameConstants.BUSINESS_RULE_NAME) {
			return getName(type, fromSource, attribute, registry);
		}
		return null;
	}
	/**
	 * Returns the value of the <code>name</code> attribute for the <code>type</code> Object.
	 * @param type of BR
	 * @param fromSource an BOM NamedElement
	 * @param attribute attribute 
	 * @param registry a NamingRegistry
	 * @return
	 */
	protected String getName(Object type, EObject fromSource, String attribute, NamingRegistry registry){
		NameProvider nameProvider = NameProviderFactory.getNameProvider(type);
		nameProvider.setContext(this.getContext());
		String	valueForType = nameProvider.getName(type, (NamedElement)fromSource, registry, attribute);
		return valueForType;
	}	
}
