package com.ibm.btools.te.ilm.attribute.wps;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.attributes.helper.TechnicalAttributesHelper;
import com.ibm.btools.te.attributes.helper.UpdateCommandHelper;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.TransformationEngine;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry;
import com.ibm.wsspi.sca.scdl.Import;

/**
* This class is responsible for providing values for attributes for SCA Component. 
*/

public class ServiceComponentProvider extends AttributeValueProvider{

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2008, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	
	/* (non-Javadoc)
	 * @see com.ibm.btools.te.ilm.attribute.AttributeValueProvider#getAttributeValueForType(com.ibm.btools.bom.model.artifacts.NamedElement, java.lang.String)
	 */
	public Object getAttributeValueForType(NamedElement fromSource, String propertyName) {
		return getAttributeValueForType(null, fromSource, propertyName, null, inParentContext);
	}

	/* (non-Javadoc)
	 * @see com.ibm.btools.te.ilm.attribute.AttributeValueProvider#getAttributeValueForType(com.ibm.btools.bom.model.artifacts.NamedElement, java.lang.String, com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry)
	 */
	public Object getAttributeValueForType(Object forType, NamedElement fromSource, String propertyName, NamingRegistry registry) {
		return getAttributeValueForType(forType, fromSource, propertyName, registry, inParentContext);
	}
	
	/**
	 * Returns the technical attribute value for the <code>propertyName<code> for the Object <code>forType</code> from the specified <code>registry</code>
	 * @param type an EObject
	 * @param attribute the attribute name of the type
	 * @param context a TransformationContext
	 * @return the value of the attribute for the type.
	 */
	private Object getAttributeValueForType(Object type, NamedElement fromSource, String attribute, NamingRegistry registry, Object inParentContext) {
		if(attribute==AttributeNameConstants.COMPONENT_NAME){
			return getName(type, fromSource, attribute, registry);
		}else if(attribute==AttributeNameConstants.COMPONENT_DISPLAY_NAME){
			return getDisplayName(type, fromSource, attribute, registry);
		} else if(attribute.equals(AttributeNameConstants.COMPONENT_DESCRIPTION)){
			return getAttributeValue(type, fromSource, attribute, registry, "");
		}else if(attribute.equals(AttributeNameConstants.IMPLEMENTATION)){
			if(fromSource instanceof StructuredActivityNode && !BomUtils.isProcess((StructuredActivityNode)fromSource)){
				if(TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute) != null){
					return getAttributeValue(type, fromSource, attribute, registry, AttributeNameConstants.IMPLEMENTATION);
				}else{
					return TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
				}
			}else{
				return TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
			}
		}else{
			return TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
		}
		
	}
	/**
	 * Returns the value of the <code>name</code> attribute for the <code>type</code> Object.
	 * @param type an BPEL Object type
	 * @param fromSource an BOM NamedElement
	 * @param attribute attribute 
	 * @param registry a NamingRegistry
	 * @return
	 */
	protected String getName(Object type, EObject fromSource, String attribute, NamingRegistry registry){
		NameProvider nameProvider = NameProviderFactory.getNameProvider(type);
		nameProvider.setContext(this.getContext());
		String	valueForType = null;
		//feature 118131
		if(type instanceof Import){
			valueForType = nameProvider.getDefaultName(type, attribute, (NamedElement)fromSource, registry, this.getContext());
		}else{
			valueForType = nameProvider.getName(type, (NamedElement)fromSource, registry, attribute);
		}
		//valueForType = nameProvider.getName(type, (NamedElement)fromSource, registry, attribute);
		return valueForType;
	}

	/**
	 * Returns the BPEL display name for a BPEL Element, if applicable. The name is not unique.
	 * @param type a BPEL Object
	 * @param fromSource the BOM source element.
	 * @param attribute the attribute property name 
	 * @param registry a NamingRegistry
	 * @return String BPEL display name.
	 */
	private String getDisplayName(Object type,  NamedElement fromSource, String attribute, NamingRegistry registry){
//		here is the logic of retrieving the displayName of the type...
		String tempName = null;
		String name = null;
		//feature 118131
		if(type instanceof Import){
			name = getDisplayNameDefaultValue(type, attribute, fromSource, registry);
			return name;
		}
		if(BpelOptionsUtil.generateDefaultValuesWithOverwrite()){
			//1.just generate default values and ignore the technical attributes
			name = getDisplayNameDefaultValue(type, attribute, fromSource, registry);
			//2c. call the commandUpdateHelper to update the BOM Model TA with new value
			Command command = UpdateCommandHelper.createUpdateCommand((NamedElement)fromSource, attribute, name);
			//2d. call the Registry Helper to add the command to the list of Commands
			if(isFromCBA(fromSource)){
				ProcessUtil.putUpdateCommandsInContext(context, command);
			}else{
				TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), command);	
			}
		}else if(BpelOptionsUtil.generateDefaultValuesNoOverwrite()){
			//1. retrieve the technical attribute for this type... 
			name = (String)TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
			if(name == null || name.trim().equals("")){
				name = getDisplayNameDefaultValue(type, attribute, fromSource, registry);
				//2c. call the commandUpdateHelper to update the BOM Model TA with new value
				Command command = UpdateCommandHelper.createUpdateCommand((NamedElement)fromSource, attribute, name);
				//2d. call the Registry Helper to add the command to the list of Commands
				if(isFromCBA(fromSource)){
					ProcessUtil.putUpdateCommandsInContext(context, command);
				}else{
					TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), command);
				}
			}
		}else if(BpelOptionsUtil.runTransformation()){
			//this means we are running the transformation with user setting without validating.. 
			//1. retrieve the technical attribute for this type... 
			name = (String)TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
			//2. check if TA was set.. not null
			//3a if TA was set, return TA
			if(name == null || name.trim().equals("")){
				name = getDisplayNameDefaultValue(type, attribute, fromSource, registry);
			}
		}else if(BpelOptionsUtil.runValidation()){
			//the display name should not be validated . 
		}
		return name;
	}
	
	/**
	 * Returns the BPEL value for the <code>attribute</code> for a BPEL Element, if applicable, using the <code>defaultValue</code>.
	 * @param type a BPEL Object
	 * @param fromSource the BOM source element.
	 * @param attribute the attribute property name 
	 * @param registry a NamingRegistry
	 * @param defaultValue default value for the property
	 * @return String BPEL display name.
	 */
	private Object getAttributeValue(Object type,  NamedElement fromSource, String attribute, NamingRegistry registry, String defaultValue){
		//here is the logic of retrieving the displayName of the type...
		String tempName = null;
		Object name = null;
		if(BpelOptionsUtil.generateDefaultValuesWithOverwrite()){
			//1.just generate default values and ignore the technical attributes
			name = defaultValue;
			//2c. call the commandUpdateHelper to update the BOM Model TA with new value
			Command command = UpdateCommandHelper.createUpdateCommand((NamedElement)fromSource, attribute, name);
			//2d. call the Registry Helper to add the command to the list of Commands
			if(isFromCBA(fromSource)){
				ProcessUtil.putUpdateCommandsInContext(context, command);
			}else{
				TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), command);	
			}
		}else if(BpelOptionsUtil.generateDefaultValuesNoOverwrite()){
			//1. retrieve the technical attribute for this type... 
			Object value = TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
			if(value == null){
				name = defaultValue;
				//2c. call the commandUpdateHelper to update the BOM Model TA with new value
				Command command = UpdateCommandHelper.createUpdateCommand((NamedElement)fromSource, attribute, name);
				//2d. call the Registry Helper to add the command to the list of Commands
				if(isFromCBA(fromSource)){
					ProcessUtil.putUpdateCommandsInContext(context, command);
				}else{
					TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), command);
				}
			}
		}else if(BpelOptionsUtil.runTransformation()){
			//this means we are running the transformation with user setting without validating.. 
			//1. retrieve the technical attribute for this type... 
			name = TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
			//2. check if TA was set.. not null
			//3a if TA was set, return TA
			if(name == null){
				name = defaultValue;
			}
		}else if(BpelOptionsUtil.runValidation()){
			//the display name should not be validated . 
		}
		return name;
	}
	
	
	/**
	 * Returns the default display name for a BPEL Element.
	 * @param type
	 * @param attribute
	 * @param fromSource
	 * @param context
	 * @return
	 */
	private String getDisplayNameDefaultValue(Object type, String attribute, EObject fromSource, NamingRegistry context) {
		NamedElement displaySource = (NamedElement)fromSource;
		return displaySource.getName();
	}
	/**
	 * Returns true if the namedElement, an Input/OutputPinSet belong to an Action which is called
	 * from the main Process through a CBA
	 * @param namedElement
	 * @return
	 */
	private boolean isFromCBA(NamedElement namedElement){
		Action action = null;
		if(namedElement instanceof InputPinSet){
			action = ((InputPinSet)namedElement).getAction();
		}else if (namedElement instanceof OutputPinSet){
			action = ((OutputPinSet)namedElement).getAction();
		}else if(namedElement instanceof StructuredActivityNode){
			action = (StructuredActivityNode)namedElement;
		}
		Action thisProcess = null;
		if(getContext() == null){
			return false;
		}
		TransformationRule procDefRule = ProcessUtil.getProcessDefinitionRule(getContext());
		if(procDefRule != null){
			thisProcess = ((com.ibm.btools.bom.model.processes.activities.Activity)ProcessUtil.getProcessDefinitionRule(getContext()).getSource().get(0)).getImplementation();
		}
		if(thisProcess == null){
			return false;
		}
		if((action instanceof StructuredActivityNode) && ((StructuredActivityNode)action).getInStructuredNode() == null && 
				("SERVICE".equals(((StructuredActivityNode)action).getAspect() )|| 
						"TASK".equals(((StructuredActivityNode)action).getAspect()))){
			return true;
		}else if(BomUtils.isTopLevelProcess(action) && thisProcess != action ){
			return true;
		}else{
			return false;
		}
	}
}
