package com.ibm.btools.te.ilm.attribute.wps;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.sql.Timestamp;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.attributes.helper.OperationTypeUtil;
import com.ibm.btools.te.attributes.helper.TechnicalAttributesHelper;
import com.ibm.btools.te.attributes.helper.UpdateCommandHelper;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.TransformationEngine;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.util.exception.BTRuntimeException;

/**
* Class responsible for providing attribute values for BPEL specific types of Objects. 
*/
public class SpecificationAttributeValueProvider extends AttributeValueProvider{
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$


	/* (non-Javadoc)
	 * @see com.ibm.btools.te.ilm.attribute.AttributeValueProvider#getAttributeValueForType(com.ibm.btools.bom.model.artifacts.NamedElement, java.lang.String)
	 */
	public Object getAttributeValueForType(NamedElement fromSource, String propertyName) {

		if(propertyName.equals(AttributeNameConstants.BPEL_PROCESS_IS_LONG_RUNNING)){
			return getExecutionModeLongRunning(fromSource, propertyName);
		}else if(propertyName.equals(AttributeNameConstants.BPEL_PROCESS_RUN_AS_CHILD)){
			return getRunAsChildProcess(fromSource, propertyName);
		}else if(propertyName.equals(AttributeNameConstants.BPEL_PROCESS_VALID_FROM)){
			return getValidFrom(fromSource, propertyName);
		}else if(propertyName.equals(AttributeNameConstants.GENERATE_SWITCH)){
			return getGenerateSwitch(fromSource, propertyName);
		}else if (propertyName.equals(AttributeNameConstants.IS_ONE_WAY_OPERATION)){
			return isOneWayOperation(fromSource, propertyName);
		}else{
			return null;
		}
	}

	/* (non-Javadoc)
	 * @see com.ibm.btools.te.ilm.attribute.AttributeValueProvider#getAttributeValueForType(com.ibm.btools.bom.model.artifacts.NamedElement, java.lang.String, com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry)
	 */
	public Object getAttributeValueForType(Object forType, NamedElement fromSource, String propertyName, NamingRegistry registry) {
		return getAttributeValueForType(fromSource, propertyName);
	}
	
	/**
	 * Returns a Boolean indicating the value of "long Running" property for the Object <code>type</code>.
	 * @param type
	 * @param fromSource
	 * @param attribute
	 * @param context
	 * @return
	 */
	private Object getExecutionModeLongRunning(EObject fromSource, String attribute) {
		Boolean option = Boolean.TRUE;
		if(BpelOptionsUtil.generateDefaultValuesWithOverwrite()){
			//1.the default value is false
			//2c. call the commandUpdateHelper to update the BOM Model TA with new value
			Command command = UpdateCommandHelper.createUpdateCommand((NamedElement)fromSource, attribute, Boolean.TRUE);
			//2d. call the Registry Helper to add the command to the list of Commands
			TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), command);
		}else if(BpelOptionsUtil.generateDefaultValuesNoOverwrite()){
			//1. retrieve the technical attribute for this type... 
			Object optionObject = TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
			if(optionObject == null){
				Command command = UpdateCommandHelper.createUpdateCommand((NamedElement)fromSource, attribute, Boolean.TRUE);
				//2d. call the Registry Helper to add the command to the list of Commands
				TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), command);
			}else{
				option = (Boolean)optionObject;
			}
		}else if(BpelOptionsUtil.runTransformation()){
			//this means we are running the transformation with user setting without validating.. 
			//1. retrieve the technical attribute for this type... 
			Object optionObject = (Boolean)TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
			if(optionObject != null){
				option = (Boolean)optionObject;
			}
		}else if(BpelOptionsUtil.runValidation()){
		}
		
		return option;
	}
	
	
	/**
	 * Returns the "run as child" property value for the Object <code>type</code>.
	 * @param type
	 * @param fromSource
	 * @param attribute
	 * @param context
	 * @return
	 */
	private Boolean getRunAsChildProcess(EObject fromSource, String attribute) {
		Boolean option = Boolean.FALSE;
		if(BpelOptionsUtil.generateDefaultValuesWithOverwrite()){
			//1.the default value is false
			//2c. call the commandUpdateHelper to update the BOM Model TA with new value
			Command command = UpdateCommandHelper.createUpdateCommand((NamedElement)fromSource, attribute, Boolean.FALSE);
			//2d. call the Registry Helper to add the command to the list of Commands
			TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), command);
		}else if(BpelOptionsUtil.generateDefaultValuesNoOverwrite()){
			//1. retrieve the technical attribute for this type... 
			Object optionObject = TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
			if(optionObject == null){
				Command command = UpdateCommandHelper.createUpdateCommand((NamedElement)fromSource, attribute, Boolean.FALSE);
				//2d. call the Registry Helper to add the command to the list of Commands
				TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), command);
			}else{
				option = (Boolean)optionObject;
			}
		}else if(BpelOptionsUtil.runTransformation()){
			//this means we are running the transformation with user setting without validating.. 
			//1. retrieve the technical attribute for this type... 
			Object optionObject = (Boolean)TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
			if(optionObject != null){
				option = (Boolean)optionObject;
			}
		}else if(BpelOptionsUtil.runValidation()){
			//do nothing.. no validation should be performed on this
		}
		
		return option;
	}

	/**
	 * 	Returns true if a BPEL Switch should be generated for the Object <code>type</code>.
	 * @param type
	 * @param fromSource
	 * @param attribute
	 * @param context
	 * @return
	 */
	private Boolean getGenerateSwitch(EObject fromSource, String attribute) {
		Boolean option = Boolean.FALSE;
		if(BpelOptionsUtil.generateDefaultValuesWithOverwrite()){
			//1.the default value is true
			//2c. call the commandUpdateHelper to update the BOM Model TA with new value
			Command command = UpdateCommandHelper.createUpdateCommand((NamedElement)fromSource, attribute, option);
			//2d. call the Registry Helper to add the command to the list of Commands
			TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), command);
		}else if(BpelOptionsUtil.generateDefaultValuesNoOverwrite()){
			//1. retrieve the technical attribute for this type... 
			Object optionObject = TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
			if(optionObject == null){
				Command command = UpdateCommandHelper.createUpdateCommand((NamedElement)fromSource, attribute, option);
				//2d. call the Registry Helper to add the command to the list of Commands
				TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), command);
			}else{
				option = (Boolean)optionObject;
			}
		}else if(BpelOptionsUtil.runTransformation()){
			//this means we are running the transformation with user setting without validating.. 
			//1. retrieve the technical attribute for this type... 
			Object optionObject = (Boolean)TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
			if(optionObject != null){
				option = (Boolean)optionObject;
			}
		}else if(BpelOptionsUtil.runValidation()){
			//do nothing.. no validation should be performed on this
		}
		
		return option;
	}	

	
	/**
	 * 	Returns the the value of the "validFrom" property for the Object <code>type</code>.
	 * @param type
	 * @param fromSource
	 * @param attribute
	 * @param context
	 * @return
	 */
	private String getValidFrom(EObject fromSource, String attribute) {
		String validFrom = null;
		if(BpelOptionsUtil.generateDefaultValuesWithOverwrite()){
			//1 Generate the default value
			Timestamp timestamp = new Timestamp(System.currentTimeMillis());
			String stamp = timestamp.toString();
			StringBuffer validStamp = new StringBuffer(stamp.replace(' ', 'T'));
			validFrom = validStamp.substring(0, validStamp.toString().lastIndexOf(".")).toString();
			//2c. call the commandUpdateHelper to update the BOM Model TA with new value
			Command command = UpdateCommandHelper.createUpdateCommand((NamedElement)fromSource, attribute, validFrom);
			//2d. call the Registry Helper to add the command to the list of Commands
			TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), command);
		}else if(BpelOptionsUtil.generateDefaultValuesNoOverwrite()){
			//1. retrieve the technical attribute for this type... 
			String optionObject = (String)TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
			if(optionObject == null || optionObject.trim().equals("")){
				Timestamp timestamp = new Timestamp(System.currentTimeMillis());
				String stamp = timestamp.toString();
				StringBuffer validStamp = new StringBuffer(stamp.replace(' ', 'T'));
				validFrom = validStamp.substring(0, validStamp.toString().lastIndexOf(".")).toString();

				Command command = UpdateCommandHelper.createUpdateCommand((NamedElement)fromSource, attribute, validFrom);
				//2d. call the Registry Helper to add the command to the list of Commands
				TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), command);
			}else{
				validFrom = (String)optionObject;
			}
		}else if(BpelOptionsUtil.runTransformation()){
			//this means we are running the transformation with user setting without validating.. 
			//1. retrieve the technical attribute for this type... 
			Object optionObject = TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
			if(optionObject != null){
				validFrom = (String)optionObject;
			}
		}else if(BpelOptionsUtil.runValidation()){
			//do nothing.. no validation should be performed on this
		}
		
		return validFrom;
	}
	
	/**
	 * Returns the the value of the "isOneWayOperation" property for the Object <code>type</code>.
	 * @param type
	 * @param fromSource
	 * @param attribute
	 * @param context
	 * @return
	 */
	private Boolean isOneWayOperation(EObject fromSource, String attribute) {
		Boolean option = Boolean.TRUE;
		boolean isCBA = isFromCBA((NamedElement)fromSource);
		if(BpelOptionsUtil.generateDefaultValuesWithOverwrite()){
			if(isCBA){
				Object optionObject = TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
				if(optionObject != null){
					option = (Boolean)optionObject;
					return option;
				}
			}
			//get the default value
			option = OperationTypeUtil.isDefaultOneWayOperation((InputPinSet)fromSource);
			if(option == null){
				BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_ATTRIBUTES_INVALID_BOM_MODEL));
				LoggingUtil.logError(MessageKeys.TE_ATTRIBUTES_INVALID_BOM_MODEL);
				throw e;
			}
			//2c. call the commandUpdateHelper to update the BOM Model TA with new value
			Command command = UpdateCommandHelper.createUpdateCommand((NamedElement)fromSource, attribute, option);
			//2d. call the Registry Helper to add the command to the list of Commands
			if(isCBA){
				ProcessUtil.putUpdateCommandsInContext(context, command);
			}else{
				TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), command);
			}
			//TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), command);
		}else if(BpelOptionsUtil.generateDefaultValuesNoOverwrite()){
			//1. retrieve the technical attribute for this type... 
			Object optionObject = TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
			if(optionObject == null){
				option = OperationTypeUtil.isDefaultOneWayOperation((InputPinSet)fromSource);
				if(option == null){
					BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_ATTRIBUTES_INVALID_BOM_MODEL));
					LoggingUtil.logError(MessageKeys.TE_ATTRIBUTES_INVALID_BOM_MODEL);
					throw e;
				}
				Command command = UpdateCommandHelper.createUpdateCommand((NamedElement)fromSource, attribute, option);
				//2d. call the Registry Helper to add the command to the list of Commands
				if(isCBA){
					ProcessUtil.putUpdateCommandsInContext(context, command);
				}else{
					TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), command);
				}
				//TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), command);
			}else{
				//validate this option as if not valid we should exit
				option = (Boolean)optionObject;
				boolean isValidOperation = OperationTypeUtil.isValid((InputPinSet)fromSource);
				if(! isValidOperation){
					BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_ATTRIBUTES_INVALID_BOM_MODEL));
					LoggingUtil.logError(MessageKeys.TE_ATTRIBUTES_INVALID_BOM_MODEL);
					throw e;
				}
			}
		}else if(BpelOptionsUtil.runTransformation()){
			//this means we are running the transformation with user setting without validating.. 
			//1. retrieve the technical attribute for this type... 
			Object optionObject = (Boolean)TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
			boolean isValidOperation = OperationTypeUtil.isValid((InputPinSet)fromSource);
			if(optionObject != null && isValidOperation){
				option = (Boolean)optionObject;
			}else{
				if(optionObject == null){
					option = OperationTypeUtil.isDefaultOneWayOperation((InputPinSet)fromSource);	
					if(option == null){
						BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_ATTRIBUTES_INVALID_BOM_MODEL));
						LoggingUtil.logError(MessageKeys.TE_ATTRIBUTES_INVALID_BOM_MODEL);
						throw e;
					}
				}else {
					BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_ATTRIBUTES_INVALID_BOM_MODEL));
					LoggingUtil.logError(MessageKeys.TE_ATTRIBUTES_INVALID_BOM_MODEL);
					throw e;
				}
				
			}
		}else if(BpelOptionsUtil.runValidation()){
			//Object optionObject = TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
			boolean isValidOperation = OperationTypeUtil.isValid((InputPinSet)fromSource);
			if(! isValidOperation){
				BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_ATTRIBUTES_INVALID_BOM_MODEL));
				LoggingUtil.logError(MessageKeys.TE_ATTRIBUTES_INVALID_BOM_MODEL);
				throw e;
			}
			Object optionObject = (Boolean)TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, attribute);
			if(optionObject != null){
				option = (Boolean)optionObject;
			}
			
		}
		return option;
	}	
	
	/**
	 * Returns true if the namedElement, an Input/OutputPinSet belong to an Action which is called
	 * from the main Process through a CBA
	 * @param namedElement
	 * @return
	 */
	private boolean isFromCBA(NamedElement namedElement){
		Action action = null;
		if(namedElement instanceof InputPinSet){
			action = ((InputPinSet)namedElement).getAction();
		}else if (namedElement instanceof OutputPinSet){
			action = ((OutputPinSet)namedElement).getAction();
		}else if(namedElement instanceof StructuredActivityNode){
			action = (StructuredActivityNode)namedElement;
		}
		// we are only interested in the cases of PinSet or StructuredActivityNode
		if(action == null){
			return false;
		}
		Action thisProcess = null;
		if(getContext() == null){
			return false;
		}
		TransformationRule procDefRule = ProcessUtil.getProcessDefinitionRule(getContext());
		if(procDefRule != null){
			thisProcess = ((com.ibm.btools.bom.model.processes.activities.Activity)ProcessUtil.getProcessDefinitionRule(getContext()).getSource().get(0)).getImplementation();
		}
		if(thisProcess == null){
			return false;
		}
		if((action instanceof StructuredActivityNode) && ((StructuredActivityNode)action).getInStructuredNode() == null && 
				("SERVICE".equals(((StructuredActivityNode)action).getAspect() )|| 
						"TASK".equals(((StructuredActivityNode)action).getAspect())||
						"ServiceOperation".equals(((StructuredActivityNode)action).getAspect()))){
			return true;
		//check if this is a remote process
		}else if(BomUtils.isTopLevelProcess(action) && thisProcess != action ){
			return true;
		}else{
			return false;
		}
		
	}
	
}
