/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import org.eclipse.emf.ecore.EClass;

import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ActivityRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.BRReusableTaskRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.HTReusableTaskRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.SANReusableProcessRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.SANReusableTaskRule;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;

/**
 * Checks if a BOM Activity is representing a resuable Process or Task.
 * Based on that creates the appropriate child rules.
 * Source: BOM Activity
 * Target:
 * Target Propagated: BPEL constructs - Variables, Partners, Sequence 
 */
public class ActivityRuleImpl extends AbstractProcDefRuleImpl implements ActivityRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ActivityRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.ACTIVITY_RULE;
	}

	public boolean transformSource2Target()
		{
			LoggingUtil.traceEntry(this, "transformSource2Target");

			// this activity rule is for activity with SAN
			boolean ret=false;
			Activity activity = (Activity)getSource().get(0);
			StructuredActivityNode san = activity.getImplementation();
			if (san != null)
			{	
				if (BomUtils.isProcess((Action)san))
				{	
					SANReusableProcessRule childRule = AbstractbpelFactory.eINSTANCE.createSANReusableProcessRule();
					childRule.getSource().add(san);
					getChildRules().add(childRule);
				} else if (BomUtils.isTask((Action)san) || BomUtils.isService((Action)san) || BomUtils.isServiceOperation(san)){
					SANReusableTaskRule childRule = AbstractbpelFactory.eINSTANCE.createSANReusableTaskRule();
					childRule.getSource().add(san);
					getChildRules().add(childRule);
				} else if (BomUtils.isHumanTask((Action)san)) {
					// TODO: need to create HTReusableTaskRule
					HTReusableTaskRule childRule = AbstractbpelFactory.eINSTANCE.createHTReusableTaskRule();
					childRule.getSource().add(san);
					getChildRules().add(childRule);					
				} else if (BomUtils.isBusinessRuleTask((Action)san)) {
					BRReusableTaskRule childRule = AbstractbpelFactory.eINSTANCE.createBRReusableTaskRule();
					childRule.getSource().add(san);
					getChildRules().add(childRule);										
				}
			}
			// TODO: if SAN is null, creates an abstract Process
			ret = executeChildRules();
			propagateChildTargets();
			setComplete(ret);
			LoggingUtil.traceExit(this, "transformSource2Target");
			return isComplete();
		}


} //ActivityRuleImpl
