/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.wsdl.OperationType;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.ExtensibilityElement;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.PortType;

import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.blm.ie.exprt.option.BpelOptions;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.services.Behavior;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.CrossProjectReferenceUtil;
import com.ibm.btools.te.ilm.ExportOperationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.WIDProjectUtil;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.CallBehaviorActionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.InputRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.InvokeRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.LinkRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.OutputRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.PartnerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessInterfaceRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.SANReusableProcessRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.VariableRule;
import com.ibm.btools.te.ilm.heuristics.extservice.OperationRule;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptimizationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.CBPELUtil;
import com.ibm.btools.te.ilm.heuristics.helper.FabricIntegrationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionKeyConstants;
import com.ibm.btools.te.ilm.heuristics.scdl.ComponentRule;
import com.ibm.btools.te.ilm.heuristics.scdl.ExportRule;
import com.ibm.btools.te.ilm.heuristics.scdl.util.ScdlUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.MessageRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.PartnerLinkRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.PortTypeRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.util.OriginalWsdlUtil;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.DisplayName;
import com.ibm.wbit.bpelpp.Output;
import com.ibm.wbit.bpelpp.Catch;
import com.ibm.wbit.bpelpp.FaultSource;
import com.ibm.wbit.bpelpp.FaultSources;
import com.ibm.wbit.wpc.Parameter;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wsspi.sca.scdl.Component;
import com.ibm.wsspi.sca.scdl.DocumentRoot;
import com.ibm.wsspi.sca.scdl.InterfaceSet;
import com.ibm.wsspi.sca.scdl.SCAImportBinding;
import com.ibm.wsspi.sca.scdl.SCDLFactory;
import com.ibm.wsspi.sca.scdl.Wire;

/**
 * Triggers the creation of a BPEL invoke that maps to the BOM CallBehaviorAction
 * The invoke will be referencing to the WSDL operation of the remote Process
 * which is defined in the Remote Process' WSDL definition.  If such
 * WSDL definition is not found, a new one will be created.
 * 
 * Source: BOM CallBehaviorAction
 * Target:
 * Target Propagated: BPEL constructs - Variables, Partners, Invoke 
 */
public class CallBehaviorActionRuleImpl extends ActionRuleImpl implements CallBehaviorActionRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	private boolean ivFirstTimeExec = true;
	private PortType portTypeForCallingProcess = null;	

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected CallBehaviorActionRuleImpl() {
		super();
	}

	// The CallBehaviorAction , source of this rule
	/*
	protected CallBehaviorAction cba;
	// The StructuredActivityNode (implementation of this CallBehaviorAction behavior)
	//if it is a Process...
	protected StructuredActivityNode san;
	 */
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.eINSTANCE.getCallBehaviorActionRule();
	}

	/**
	 * 
	 * CallBehaviorActionRule creates an InvokeRule or InvokeWithCallbackRule.
	 * If the InputPinSet is a request/response type operation, then creates InvokeRule
	 * If the InputPinSet is a one-way type operation, then creates InvokeWithCallbackRule
	 * 
	 */
	public boolean transformSource2Target()
	{
		LoggingUtil.traceEntry(this, "transformSource2Target");

		//assertion
		if (getSource() == null || getSource().isEmpty()) {
			LoggingUtil.traceExit(this, "transformSource2Target","no source");
			return false;			
		}

		//check if this is first time execute this rule
		if(!ivFirstTimeExec){
			InvokeRule invokeRule = (InvokeRule)getChildRules().get(0); 
			invokeRule.transformSource2Target();
			LoggingUtil.traceExit(this, "transformSource2Target","re-execute");
			return true;		  
		}

		ivFirstTimeExec = false;

		//get source of this rule- input set of the task		
		InputPinSet inputSet = (InputPinSet)getSource().get(0);

		CallBehaviorAction cba = (CallBehaviorAction)inputSet.getAction();
		
		// If the referenced process is a first class artifact, we need to 
		// create a mirrored operation for this Process' WSDL
		Action action = (Action)findToBeProcessedAction(cba);	
		
		InputPinSet opInputPinSet = inputSet;
		if (action instanceof StructuredActivityNode)
		{
			opInputPinSet = (InputPinSet)getCorrespondingSet((NamedElement)inputSet);
		}

		//	BomUtils.PinSetAnalyzer psa = new BomUtils.PinSetAnalyzer(action); 
		BomUtils.PinSetAnalyzer psa = new BomUtils.PinSetAnalyzer(action, this.getContext());
		List osList = psa.getCallBackOutputPinSets(opInputPinSet);	

		boolean isOneWayOperation = false;
		//defect 85137, PMR 68264
		//ProcessInterfaceRule myProcIntRule = createProcessInterfaceRule(cba);
		Activity interfaceActivity = FabricIntegrationUtil.getInterfaceActivityIfAny(getParentSAN(cba));
		ProcessInterfaceRule myProcIntRule = null;
		if(interfaceActivity == null){
			myProcIntRule = createProcessInterfaceRule(getParentSAN(cba));
		}else{
			myProcIntRule = createProcessInterfaceRule(interfaceActivity.getImplementation());
		}
		ProcessDefinitionRule myProcDefRule = (ProcessDefinitionRule)ProcessUtil.getProcessDefinitionRule(getContext());

		ProcessInterfaceRule processIntRule = createProcessInterfaceRule(action);
		TransformationRule portTypeRule = createPortTypeRule(processIntRule,action);		
		Operation op=null;
		PortType pt = (PortType)portTypeRule.getTarget().get(0);
		//Record the portType for the calling process
		setPortTypeForCallingProcess(pt);
		
		if(!BomUtils.isServiceOperation(action)){
			isOneWayOperation = psa.getInputPinSetsAsOneWayOperation().contains(opInputPinSet);
		}else{
				//defect 85137, PMR 68264
			Operation operation = OriginalWsdlUtil.getOperation(getContext(), action);
			if(operation != null){
				isOneWayOperation = operation.getStyle().equals(OperationType.ONE_WAY);
			}
		}
		InvokeRule ir;
		if (isOneWayOperation) {
			ir = AbstractbpelFactory.eINSTANCE.createInvokeWithCallbackRule();			
		} else {
			ir = AbstractbpelFactory.eINSTANCE.createInvokeRule();
		}
		//add ir as child rule
		getChildRules().add(ir);	
		// Make sure the SCDL component is created.  This is the case where only the interface of
		// the Process/Global Task/Service will be created, then we need to create the SCDL Component here.
		// If the Component is already created, then this method will not recreate the component so we are safe
		DocumentRoot componentRoot=createSCDLComponent(action, pt);
		//feature 118131, only create SCDL Export if we have created a Component
		//AZ WMT 20070321 handle best practice pattern generation
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		DocumentRoot exportRoot = null;
		//AR DirectDeploy there is no need to create the export when we generate for deploy
		if(!BpelOptionsUtil.isGenerateForDeployPatternEnabled()){
			if(ScdlUtil.isComponent(componentRoot)||(BpelOptions.isBestPracticePatternEnabed(exportSession)&& componentRoot != null)){
				// Create SCDL Export
				AttributeValueProvider avp= AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.COMPONENT_NAME);
				//AR feature 1609, should always generate the export for the best practice, as MFC needs to be exposed
			//	if(BpelOptionsUtil.isGenerateForDeployPatternEnabled()){
//					exportRoot=createSCDLExport(componentRoot,(PortType)portTypeRule.getTarget().get(0));	
//				}else{
//					if(!OriginalWsdlUtil.isBindingInfoAvailable(action, (PortType)portTypeRule.getTarget().get(0),getContext())||
//							(BpelOptions.isBestPracticePatternEnabed(exportSession)&& ExportOperationUtil.isDynamicAssembler(avp, action))){
//					}
//				}
				//defect 82758
				exportRoot=createSCDLExport(componentRoot,(PortType)portTypeRule.getTarget().get(0));	
			}
		}
		// Ensure the Process and HumanTask component is put in the top level module
		// and BusinessRules (GJD 20070816)
		if(BpelOptions.isBestPracticePatternEnabed(exportSession)){
			if (BomUtils.isProcess(action) || BomUtils.isHumanTask(action) ||
					BomUtils.isBusinessRuleTask(action)) {
				Object docRootRegistry = exportSession.getExportOptions().getAdditionalOption(TransformationSessionKeyConstants.TOP_LEVEL_DOCUMENT_ROOT);
				if(docRootRegistry instanceof Set){
					((Set)docRootRegistry).add(componentRoot);
					if (exportRoot!=null) {
						((Set)docRootRegistry).add(exportRoot);					
					}
				}							
			}
		}		

		PartnerRule partnerRule;
		
		PartnerLinkRule partnerLinkRule = null;
		
		ArrayList sources = new ArrayList();
		sources.add(action);
		sources.add(cba);

		//if((isCallingProcess(cba) && BpelOptionsUtil.isTargetLongRunningReceiveCallbackMacroflow()) ||
		//	osList.size()>0 
		if (isOneWayOperation && osList.size()>0
		){
			op = findOneWayOperation(processIntRule,opInputPinSet);
			partnerLinkRule = createPartnerLinkRule(myProcIntRule,action,(PortType)portTypeRule.getTarget().get(0),(PortType)portTypeRule.getTarget().get(1));
			// If there is a callback, we need to update the Component interface and component reference
			updateComponent((PortType)portTypeRule.getTarget().get(1),action);
		} else if (isOneWayOperation && osList.size()==0) {
			op = findOneWayOperation(processIntRule,opInputPinSet);
//			partnerLinkRule = createPartnerLinkRule(myProcIntRule,action,(PortType)portTypeRule.getTarget().get(0), this);
			partnerLinkRule = createPartnerLinkRule_Overriden(myProcIntRule, action, (PortType)portTypeRule.getTarget().get(0), this, sources);
		} else {
			op = findOperation(processIntRule,opInputPinSet);
//			partnerLinkRule = createPartnerLinkRule(myProcIntRule,action,(PortType)portTypeRule.getTarget().get(0), this);
			partnerLinkRule = createPartnerLinkRule_Overriden(myProcIntRule, action, (PortType)portTypeRule.getTarget().get(0), this, sources);
		}

		
//		For the Service operation, add the portType to the source so we can more easily search by it
		//defect 75458
//		if(BomUtils.isServiceOperation(action)){
//		if(FabricIntegrationUtil.isOverriden(cba)){
//			sources.add((PortType)portTypeRule.getTarget().get(0));
//		}
		partnerRule = createPartnerRule(myProcDefRule,sources, partnerLinkRule, componentRoot);
		// Add the operation to the Component Reference
		ScdlUtil.updateSCDLComponentReference(ProcessUtil.getProcessComponent(getContext()),(PortType)portTypeRule.getTarget().get(0),op);
		ir.getSource().add(inputSet);
		ir.getSource().add(cba);
		ir.getSource().add(op);
		ir.getSource().add(partnerRule.getTarget().get(0));
		if(isOneWayOperation){
			if ((osList.size()>0)) {
				OutputPinSet ops;
				for (Iterator it=osList.iterator();it.hasNext();) {					
					ops = (OutputPinSet)it.next();				
					op = findOneWayOperation(processIntRule,ops);
					ir.getSource().add(op);				
				}
			}
		}
		ir.transformSource2Target();			

		if (ir.getTarget().size() > 0) {
			Invoke bpelInvoke=null;
			if (ir.getTarget().get(0) instanceof Invoke) {
				bpelInvoke = (Invoke)ir.getTarget().get(0);
			} else {
				for (int i=0; i<ir.getTarget().size(); i++) {
					if (ir.getTarget().get(i) instanceof Invoke) {
						bpelInvoke = (Invoke)ir.getTarget().get(i);
						break;
					}
				}
			}

			if (cba.getBehavior()!=null && cba.getBehavior().getName()!=null) {
				List extElements = bpelInvoke.getEExtensibilityElements();
				DisplayName description=null;
				// Reuse the DisplayName object if it has been created already
				if (!extElements.isEmpty()) {
					ExtensibilityElement extElement;
					for (Iterator it=extElements.iterator();it.hasNext();){
						extElement=(ExtensibilityElement)it.next();
						if (extElement instanceof DisplayName) {
							description = (DisplayName)extElement;
							break;
						}
					}
				}
				if (description==null) {
					description = BPELPlusFactory.eINSTANCE.createDisplayName();
					bpelInvoke.getEExtensibilityElements().add(description);
				}
				//description.setText(getDisplayName(cba));
				//defect 118730
				description.setText(getBPELActivityDisplayName(bpelInvoke, inputSet));

			}
			// Ray: added support for descriptions and documentations	
//			Description desc = AbstractbpelUtil.transformDescription(action);
//			if (desc!=null) {
//			bpelInvoke.addExtensibilityElement(desc);
//			}
//			Documentation doc = AbstractbpelUtil.transformDocumentation(cba);
//			if (doc!=null) {
//			bpelInvoke.addExtensibilityElement(doc);
//			}			
		}

		ProcessUtil.removeUpdateCommandsFromContext(getContext());
		//propagate all child target
		propagateChildTargets();

        
		if(CBPELUtil.shouldHanleFaultLink(inputSet)){
			CBPELUtil.processFaultSources(getContext(),ir,portTypeRule);
		}
		
		//AZ 08/24/07 
		executeHandlers();

		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}
	
	/**
	 * Get the top-level Activity that owns action 
	 * @param action
	 * @return top-level Activity
	 */
	private Activity getOwningActivity(Action action) {
		while (action.getInStructuredNode() != null) {
			action = action.getInStructuredNode();
		}
		return ((StructuredActivityNode)action).getActivity();
	}

	private boolean isCallingProcess(CallBehaviorAction cba) {
		Behavior behavior = cba.getBehavior();
		if (behavior instanceof Activity) {
			StructuredActivityNode san = ((Activity)behavior).getImplementation();
			return BomUtils.isProcess(san);
		}
		return false;
	}

	/**
	 * If there is a callback to this process, we need to 
	 * 1) add an Inteface to the Component Definition.
	 * 2) update the Reference of the calledProcess to wire to this process thru the callback interface  
	 * @param pt
	 * @param calledProcessSAN
	 */
	private void updateComponent(PortType pt, Action calledProcessSAN) {
		Component procComp = ProcessUtil.getProcessComponent(getContext());
		// Find the ComponentRule, add the PortType to the source of the rule and re-execute it to add it to the
		// component
		TransformationRule tr = TransformationUtil.getRuleForTarget(getRoot(),ComponentRule.class,procComp.eContainer());
		if (tr!=null && !tr.getSource().contains(pt)) {
			tr.getSource().add(pt);
			tr.transformSource2Target();
			TransformationRule etr = TransformationUtil.getRuleForSource(getRoot(),ExportRule.class,procComp.eContainer());
			if (etr!=null) {
				etr.getSource().add(pt);
				etr.transformSource2Target();
			}

		}
		// Find the component of the process being called
		List sources = new ArrayList();
		sources.add(calledProcessSAN);
		ComponentRule rule = (ComponentRule)TransformationUtil.getRuleForSource(getRoot(),ComponentRule.class,sources,DocumentRoot.class);
		if (rule != null) {
			// Update the reference
			Component remoteProcComp = ((DocumentRoot)rule.getTarget().get(0)).getComponent();
			ScdlUtil.updateSCDLComponentReference(remoteProcComp,procComp,pt,true);
			if(!ScdlUtil.isCallbackHandled(procComp, remoteProcComp)){
				createCallbackComponent(remoteProcComp,procComp,pt);	
				registerCallBack(remoteProcComp,procComp,pt);
			}
		}
	}
	
	/**
	 * We need to create call back export for the target component, and callback import for the
	 * source component
	 * @param srcComp
	 * @param tarComp
	 * @param callbackPT
	 */
	private void createCallbackComponent(Component srcComp, Component tarComp,PortType callbackPT) {	
		boolean isPartialExport = BpelOptionsUtil.isPartialExport();
		String targetProjectName = (String)CrossProjectReferenceUtil.getProjectNameForTargetObject(tarComp.eContainer());
		String sourceProjectName = (String)CrossProjectReferenceUtil.getProjectNameForTargetObject(srcComp.eContainer());
		TransformationRule tarcomponentRule = TransformationUtil.getRuleForTarget(getRoot(),ComponentRule.class,tarComp.eContainer());	
		
		TransformationRule srcComponentRule = TransformationUtil.getRuleForTarget(getRoot(),ComponentRule.class,srcComp.eContainer());		
		boolean isSrcProcessHomeProject = false;
		Activity srcAction = null;
		if(srcComponentRule instanceof ComponentRule){
			srcComponentRule = (ComponentRule)srcComponentRule;
			if(srcComponentRule.getSource().get(0) instanceof StructuredActivityNode){
				StructuredActivityNode san = (StructuredActivityNode)srcComponentRule.getSource().get(0);
				if(BomUtils.isTopLevelProcess(san)){
					String srcProcessHomeProjectName = CrossProjectReferenceUtil.getProjectNameForBOMElement(san);
					    List<String> dependantProjects = new ArrayList<String>();
					    srcAction = san.getActivity();
						CrossProjectReferenceUtil.traverseDependencies(dependantProjects, srcProcessHomeProjectName,srcAction);	
						if(!dependantProjects.isEmpty()){
							for(String eachProjectName:dependantProjects){
								if(eachProjectName.equals(srcProcessHomeProjectName)){
									isSrcProcessHomeProject = true;
									break;
								}
							}
						}
				}
			}
		}
		
		boolean isSelectedForExport = ScdlUtil.isActivityInExportSession(srcAction);
		
		boolean isImportCreated = ScdlUtil.isCallbackImportCreated(srcComp);
		
		com.ibm.wsspi.sca.scdl.Export callbackExport = SCDLFactory.eINSTANCE.createExport();
		InterfaceSet interfaceSet = ScdlUtil.createInterfaceSet(callbackPT);		
		callbackExport.setInterfaceSet(interfaceSet);
		String exportName = tarComp.getName()+callbackPT.getQName().getLocalPart();
		callbackExport.setName(exportName);		
		callbackExport.setDisplayName(callbackPT.getQName().getLocalPart() + " Export");	
		callbackExport.setTargetName(tarComp.getName());				
		callbackExport.setInterfaceSet(ScdlUtil.createInterfaceSet(callbackPT,false));		
		
		DocumentRoot exportRoot = SCDLFactory.eINSTANCE.createDocumentRoot();
		exportRoot.setExport(callbackExport);	
		
		boolean createImport = (!isPartialExport || isSelectedForExport) && 
		targetProjectName!= null && !targetProjectName.equals(sourceProjectName)&& 
		!isSrcProcessHomeProject && 
		!isImportCreated;
		
		DocumentRoot importRoot = null;
		if(createImport){			
			com.ibm.wsspi.sca.scdl.Import callbackImport = SCDLFactory.eINSTANCE.createImport();
			callbackImport.setName(exportName);
			callbackImport.setDisplayName(callbackPT.getQName().getLocalPart() + " Import");
			InterfaceSet interfaceSetImport = ScdlUtil.createInterfaceSet(callbackPT);	
			callbackImport.setInterfaceSet(interfaceSetImport);
			
			SCAImportBinding importBinding = SCDLFactory.eINSTANCE.createSCAImportBinding();
			importBinding.setExportName(exportName);		
			String	versionName = WIDProjectUtil.DEFAULT_VERSION;		
			importBinding.setVersionValue(versionName);
			callbackImport.setBinding(importBinding);			
			importBinding.setModuleName(BpelOptionsUtil.getWPSModuleName(targetProjectName));
			
			importRoot = SCDLFactory.eINSTANCE.createDocumentRoot();
			importRoot.setImport(callbackImport);	
		}
		
		//Register the target project name
		Map additionalOptions = BpelOptionsUtil.getExportSession().getExportOptions().getAdditionalOptions();
		if(additionalOptions != null){
			Object registry = additionalOptions.get(TransformationSessionKeyConstants.TARGET_TO_PROJECT_NAME_REGISTRY_KEY);
			if(!(registry instanceof Map)){
				registry = new HashMap();
				additionalOptions.put(TransformationSessionKeyConstants.TARGET_TO_PROJECT_NAME_REGISTRY_KEY, registry);
			}			
			((Map)registry).put(exportRoot, targetProjectName);
			if(createImport){
				((Map)registry).put(importRoot, sourceProjectName);
			}
		}
		if(tarcomponentRule != null && createImport){			
			tarcomponentRule.getTarget().add(importRoot);
			ScdlUtil.registerCallbackImport(importRoot);
		}
		ExportRule exportRule = (ExportRule)TransformationUtil.getRuleForSource(getRoot(),ExportRule.class,tarComp.eContainer());		
		if(exportRule != null && !targetProjectName.equals(sourceProjectName)){
			Object docRootRegistry =BpelOptionsUtil.getExportSession().getExportOptions().getAdditionalOption(TransformationSessionKeyConstants.TOP_LEVEL_DOCUMENT_ROOT);
			if(docRootRegistry instanceof Set){				
				((Set)docRootRegistry).add(exportRoot);
			}
			//For look-up if the export rule is not the child of the current SANReusableProcessRule
			exportRule.getTarget().add(exportRoot);
			//For propagation
			getTarget().add(exportRoot);
		}		
	}
	
	private void registerCallBack(Component srcComp, Component tarComp,PortType callbackPT){
		ScdlUtil.registerCallbackComponent((DocumentRoot)tarComp.eContainer(), (DocumentRoot)srcComp.eContainer());
		ScdlUtil.regsiterImportName2QName(tarComp,callbackPT);
	}


	private StructuredActivityNode getParentSAN(CallBehaviorAction cba) {
		StructuredActivityNode san = cba.getInStructuredNode();
		while ( san.getInStructuredNode() != null) {
			san = san.getInStructuredNode();
		}
		return san;
	}

	public PortType getPortTypeForCallingProcess() {
		return portTypeForCallingProcess;
	}

	public void setPortTypeForCallingProcess(PortType portTypeForCallingProcess) {
		this.portTypeForCallingProcess = portTypeForCallingProcess;
	}

} //CallBehaviorActionRuleImpl
