/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
 * Licensed Materials - Property of IBM
 * 5724-I74, 5724-I75
 *  Copyright IBM Corporation 2008, 2009. All rights reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */


import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.wst.wsdl.Part;

import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.actions.Merge;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.ControlFlow;
import com.ibm.btools.bom.model.processes.activities.Datastore;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.InputValuePin;
import com.ibm.btools.bom.model.processes.activities.LoopNode;
import com.ibm.btools.bom.model.processes.activities.ObjectFlow;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.RetrieveArtifactPin;
import com.ibm.btools.bom.model.processes.activities.StoreArtifactPin;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.activities.TerminationNode;
import com.ibm.btools.bom.model.processes.activities.Variable;
import com.ibm.btools.expression.model.BinaryLogicalBooleanExpression;
import com.ibm.btools.expression.model.BooleanLiteralExpression;
import com.ibm.btools.expression.model.ComparisonExpression;
import com.ibm.btools.expression.model.ComparisonOperator;
import com.ibm.btools.expression.model.Expression;
import com.ibm.btools.expression.model.FunctionArgument;
import com.ibm.btools.expression.model.FunctionDefinition;
import com.ibm.btools.expression.model.FunctionExpression;
import com.ibm.btools.expression.model.LogicalBooleanOperator;
import com.ibm.btools.expression.model.ModelFactory;
import com.ibm.btools.expression.model.StringLiteralExpression;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ConnectableRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.LinkRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.util.ScopeRegistryUtil;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelConstants;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BomWalker;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptimizationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.CBPELUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.model.abstractbpel.ExtensibilityExpression;
import com.ibm.btools.te.ilm.model.abstractbpel.JoinExtensibilityExpression;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Assign;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Empty;
import com.ibm.wbit.bpel.Flow;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Links;
import com.ibm.wbit.bpel.Scope;
import com.ibm.wbit.bpel.Sequence;
import com.ibm.wbit.bpel.Source;
import com.ibm.wbit.bpel.Target;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.DisplayName;
import com.ibm.wbit.wpc.TypeEnum;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Connectable Rule</b></em>'.
 * 
 * <p> This rule aggregates the common functionality pertaining to the mapping of nodes
 * to which connections can be made.  It handles the connection of links to / from input / output
 * sets, the updating of join conditions and the mapping of retrieve and store artifact pins. </p>
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public abstract class ConnectableRuleImpl extends AbstractProcDefRuleImpl implements ConnectableRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/* map to keep track of the activity created for an input or output set */	 
	protected Map ivActivityMap = new HashMap();

	/**
	 * <!-- begin-user-doc -->
	 * 
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ConnectableRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.CONNECTABLE_RULE;
	}

	/**
	 * This method creates the BPEL Activity (Activities) that represent the BOM pins in the 
	 * given input set.  Specifically, it creates activities to handle the input value pins
	 * and retrieve artifact pins that may be present, and initializes the termination node
	 * variable if the input set is for a nested SAN.  If more than one activity is required, 
	 * a BPEL Sequence is returned.  If a single Activity is needed it is returned, otherwise
	 * an empty list is returned if there are no activities required.   If an activity is created,
	 * the name is set to be the name of the input set.
	 * 
	 * <p>NOTE: It is the caller's responsibility to add the returned action to the list of 
	 * targets associated with this rule instance, and/or to link the returned action to
	 * other Actions created during execution of the rule instance.</p>
	 *  
	 * @param inputSet The input pin set to map
	 * @return List The BPEL objects required to represent the given input pin set.  One (and
	 * only one) element will be an activity, with rest being Variables and/or Links. 
	 */
	protected List mapInputPinSet(InputPinSet inputSet){

		/* This list will hold the activities created to map the input set complexities */
		List resultList = new ArrayList();

		/* Check the attributes that are not supported by the TE */
		//		if (inputSet.getInputSetConstraint().size() > 0){
		//			String params[]={inputSet.getName(),inputSet.getAction().getName()};
		//			LoggingUtil.logWarning(MessageKeys.INPUT_SET_CONSRTRAINTS_NOT_SUPPORTED,params,null);
		//		}
		//		if (inputSet.getCorrelationPredicate() != null){
		//			String params[]={inputSet.getName(),inputSet.getAction().getName()};
		//			LoggingUtil.logWarning(MessageKeys.CORRELATION_PREDICATES_NOT_SUPPORTED,params,null);
		//		}
		InputObjectPin p;
		for (Iterator it = inputSet.getInputObjectPin().iterator(); it.hasNext(); ){
			p = (InputObjectPin)it.next();
			if (p.getLowerBound() != null && ((LiteralInteger)p.getLowerBound()).getValue().intValue() == 0){
				String params[]={p.getName(),p.getAction().getName()};
				LoggingUtil.logWarning(MessageKeys.OPTIONAL_PINS_NOT_SUPPORTED,params,null);
			}
		}

		if (isMappedActivity(inputSet)){
			/* This input has already been examined, with a result put in the activity map, so 
			 * just return the previous findings.  It may be null */ 
			if (getMappedActivity(inputSet) != null){
				resultList.add(getMappedActivity(inputSet));
			}
			return resultList;
		}

		List activityList = new ArrayList();

		List complexPins = getPinsOfType(inputSet, InputValuePin.class);
		complexPins.addAll(getPinsOfType(inputSet, RetrieveArtifactPin.class));
		Pin complexPin = null;
		for (Iterator it = complexPins.iterator(); it.hasNext(); ){
			complexPin = (Pin)it.next();
			TransformationRule pinRule = null;

			if (complexPin instanceof InputValuePin){
				pinRule = AbstractbpelFactoryImpl.eINSTANCE.createInputValuePinRule();
			} else if (complexPin instanceof RetrieveArtifactPin){
				// we should not map the Variable
				if (((RetrieveArtifactPin)complexPin).getRepository() instanceof Variable){
					pinRule = AbstractbpelFactoryImpl.eINSTANCE.createVariableAccessRule();	
				} else 
					if (((RetrieveArtifactPin)complexPin).getRepository() instanceof Datastore){
						pinRule = AbstractbpelFactoryImpl.eINSTANCE.createDatastoreAccessRule();
					}
			}
			if(pinRule == null){
				continue;
			}
			// TODO: We need to make sure that nothing is going to try to execute all child rules later...
			getChildRules().add(pinRule);

			pinRule.getSource().add(complexPin);
			pinRule.transformSource2Target();

			/* We must remove this from the list of child rules because we don't want the target to be propagated later,
			 * and if we leave it as a child rule, it will be.  The caller can propagate the result of this method instead			 */
			getChildRules().remove(pinRule);

			/* Add the non-activity targets of the previous rule to the resultsList, and the 
			 * Activities to the activitiesList */
			Object target = null;
			for (Iterator it2=pinRule.getTarget().iterator(); it2.hasNext(); ){
				target = it2.next();
				if (!(target instanceof Activity)){
					//				if (target instanceof Link || target instanceof com.ibm.etools.ctc.bpel.Variable){
					resultList.add(target);
				} else {
					activityList.add(target);
				}
			}
		}

		//		/* Handle the termination nodes, if this is the output set of a SAN */
		//		if (isNestedSANInputSet){
		//			activityList.add(initializeTerminationVariable(inputSet));
		//		}

		/* The activityList now contains all the activities, and resultList contains everything
		 * to be propagated that is not an activity.  Now combine the activities into a single 
		 * BPEL Sequence (if there is more than one), and put everything together in one list */
		if (activityList.size() > 1){
			Sequence sequence = BPELFactory.eINSTANCE.createSequence();
			// Check to see if it's a valid name by calling NameConverter
			/* Ray Apr 13/04 - don't need name for sequence
			JavaNCNameConverter converter = new JavaNCNameConverter();
			String convertedName = converter.convertName(NamingUtil.findRegistry(this), sequence, BomUtils.getCanonicalName(inputSet));
			sequence.setName(convertedName);
			 */
			//sequence.setName(BomUtils.getCanonicalName(inputSet));
			sequence.getActivities().addAll(activityList);
			resultList.add(0, sequence);	
			setMappedActivity(inputSet, sequence);		 
		} else if (activityList.size() == 1){
			/* There is only one Activity created  */
			resultList.add(0, activityList.get(0));
			// Check to see if it's a valid name by calling NameConverter
			NameProvider nameProvider = NameProviderFactory.getNameProvider((Activity)activityList.get(0));
			String convertedName = nameProvider.getName((Activity)activityList.get(0), inputSet,  NamingUtil.findRegistry(this));

			//			JavaNCNameConverter converter = new JavaNCNameConverter();
			//			String convertedName = converter.convertName(NamingUtil.findRegistry(this), ((Activity)activityList.get(0)), BomUtils.getCanonicalName(inputSet));
			((Activity)activityList.get(0)).setName(convertedName);
			//((Activity)activityList.get(0)).setName(BomUtils.getCanonicalName(inputSet));
			setMappedActivity(inputSet, (Activity)activityList.get(0));
		} else {
			/* There were no activities created */
		}

		return resultList;
	}


	/**
	 * This method creates the BPEL Activity (Activities) that represent the BOM pins in the 
	 * given output set.  Specifically, it creates activities to handle the 
	 * store artifact pins that may be present, and sets the value of the termination 
	 * variable if the output set is for a nested SAN.  If more than one activity is required, 
	 * a BPEL Sequence is returned.  If a single Activity is needed it is returned, otherwise
	 * an empty list is returned if there are no activities required. 
	 * 
	 * <p>NOTE: It is the caller's responsibility to add the returned action to the list of 
	 * targets associated with this rule instance, and/or to link the returned action to
	 * other Actions created during execution of the rule instance.</p>
	 *  
	 * @param outputSet The output pin set to map
	 * @return List The BPEL objects required to represent the given output pin set.  One (and
	 * only one) element will be an activity, with rest being Variables and/or Links. 
	 */	
	protected List mapOutputPinSet(OutputPinSet outputSet){

		/* This list will hold the activities created to map the output set complexities */
		List resultList = new ArrayList();

		/* Check the attributes that are not supported by the TE */
		if (outputSet.getIsStream() != null && outputSet.getIsStream().booleanValue() == true){
			String params[]={outputSet.getName(),outputSet.getAction().getName()};
			LoggingUtil.logWarning(MessageKeys.STREAMING_OUTPUT_SETS_NOT_SUPPORTED,params,null);
		}
		//		if (outputSet.getIsException() != null && outputSet.getIsException().booleanValue() == true){
		//			String params[]={outputSet.getName(),outputSet.getAction().getName()};
		//			LoggingUtil.logWarning(MessageKeys.EXCEPTION_OUTPUT_SETS_NOT_SUPPORTED,params,null);
		//		}
		OutputObjectPin p;
		for (Iterator it = outputSet.getOutputObjectPin().iterator(); it.hasNext(); ){
			p = (OutputObjectPin)it.next();
			if (p.getLowerBound() != null && ((LiteralInteger)p.getLowerBound()).getValue().intValue() == 0){
				String params[]={p.getName(),p.getAction().getName()};
				LoggingUtil.logWarning(MessageKeys.OPTIONAL_PINS_NOT_SUPPORTED,params,null);
			}
		}

		if (isMappedActivity(outputSet)){
			/* This output set has already been examined, with a result put in the activity map, so 
			 * just return the previous findings.  It may be null */ 
			if (getMappedActivity(outputSet) != null){
				resultList.add(getMappedActivity(outputSet));
			}
			return resultList;
		}

		List activityList = new ArrayList();

		List complexPins = getPinsOfType(outputSet, StoreArtifactPin.class);

		//		boolean isNestedSANOutputSetOrLoopNode = 
		//			BomUtils.isProcess(outputSet.getAction()) || outputSet.getAction() instanceof LoopNode;

		Pin complexPin = null;
		for (Iterator it = complexPins.iterator(); it.hasNext(); ){
			complexPin = (Pin)it.next();
			TransformationRule pinRule = null;

			if (((StoreArtifactPin)complexPin).getRepository() instanceof Variable){
				pinRule = AbstractbpelFactoryImpl.eINSTANCE.createVariableAccessRule();	
			} else
				if (((StoreArtifactPin)complexPin).getRepository() instanceof Datastore){
					pinRule = AbstractbpelFactoryImpl.eINSTANCE.createDatastoreAccessRule();
				}


			if(pinRule != null){
				// TODO: We need to make sure that nothing is going to try to execute all child rules later...
				getChildRules().add(pinRule);

				pinRule.getSource().add(complexPin);
				pinRule.transformSource2Target();

				/* We must remove this from the list of child rules because we don't want the target to be propagated later,
				 * and if we leave it as a child rule, it will be.  The caller can propagate the result of this method instead			 */
				getChildRules().remove(pinRule);	

				/* Add all targets to resultList, which will be sorted out later */
				resultList.addAll(pinRule.getTarget());
			} 				
		}
		/**
		 * Rule: If exception output pinset contains multiple pins
		 */
		if (CBPELUtil.shouldGenerateForkActivity(outputSet)&& 
				!BomUtils.isLocalProcess(outputSet.getAction()) &&
				!(outputSet.getAction() instanceof LoopNode)&&
				!BomUtils.isTopLevelProcess(outputSet.getAction())){			
			//create Assign activity to split the fault link
			//1. Multiple pin in the exception pinset
			//2. There are multiple targets for those pins				
			Assign assign = CBPELUtil.createAssign(getContext(),this, outputSet);
			BPELVariable wrapperVariable = CBPELUtil.getBPELVariableForFaultPinSet(getContext(), this, outputSet.getName());

			//This should be interface variable, so we retain the messgae Type and mark it as interface
			//variable... TODO Test with the runtime behavior
			if (wrapperVariable != null && wrapperVariable.getXSDElement() != null){	
				wrapperVariable.setXSDElement(null);					
			}
			//Register in the non-optimization element list
			BpelOptimizationUtil.registerNonOptimizableElement(getContext(), assign);
			resultList.add(assign);		
			getTarget().add(assign);
			CBPELUtil.createGeneratedAttribute(assign, CBPELUtil.GEN_BPC_FAULT_SPLIT_ASSIGN);
		}else if(CBPELUtil.shoudlGenerateEmptyActivity(outputSet) &&
				!BomUtils.isLocalProcess(outputSet.getAction()) &&
				!(outputSet.getAction() instanceof LoopNode)&&
				!BomUtils.isTopLevelProcess(outputSet.getAction())){
				Empty empty = BPELFactory.eINSTANCE.createEmpty();
				String activityName = BomUtils.getCanonicalName((NamedElement)outputSet.eContainer());
				activityName = activityName+"_"+outputSet.getName()+"_Empty";
				JavaNCNameConverter converter = new JavaNCNameConverter();
				String actConvertedName = converter.convertName(NamingUtil.findRegistry(this), empty, activityName);
				empty.setName(actConvertedName);
				BpelOptimizationUtil.registerNonOptimizableElement(getContext(), empty);
				resultList.add(empty);		
				getTarget().add(empty);
		}
		if(CBPELUtil.isFaultLinkInParallelToRegularLink(outputSet)&& !CBPELUtil.shouldGenerateForkActivity(outputSet)){
			//we need to create empty activity to attach to the fault link before the target
			Empty empty = BPELFactory.eINSTANCE.createEmpty();
			String activityName = BomUtils.getCanonicalName((NamedElement)outputSet.eContainer());
			activityName = activityName+"_"+outputSet.getName()+"_Empty";
			JavaNCNameConverter converter = new JavaNCNameConverter();
			String actConvertedName = converter.convertName(NamingUtil.findRegistry(this), empty, activityName);
			empty.setName(actConvertedName);
			BpelOptimizationUtil.registerNonOptimizableElement(getContext(), empty);
			resultList.add(empty);		
			getTarget().add(empty);
		}

		//		/* Handle the termination nodes, if this is the output set of a SAN */
		//		if (isNestedSANOutputSetOrLoopNode){
		//			TerminationNodeRule termRule = AbstractbpelFactory.eINSTANCE.createTerminationNodeRule();
		//			termRule.getSource().addAll(BomUtils.getTerminationNodesForSet(outputSet));
		//			getChildRules().add(termRule);
		//			termRule.transformSource2Target();
		//	 				
		//			/* The target of the termination rule is an <assign> that sets the value of
		//			 * the BPEL variable associated with the termination node. */
		//			resultList.addAll(termRule.getTarget());
		//		}

		/* Break apart resultList, keeping the non-activity targets in resultsList, and putting the 
		 * Activities to the activitiesList */
		for (int i=0; i<resultList.size(); i++){
			if (resultList.get(i) instanceof Activity){
				activityList.add(resultList.get(i));
				resultList.remove(i);
				i--;
			} 
		}


		/* The activityList now contains all the activities, and resultList contains everything
		 * to be propagated that is not an activity.  Now combine the activities into a single 
		 * BPEL Sequence (if there is more than one), and put everything together in one list */
		if (activityList.size() > 1){
			Sequence sequence = BPELFactory.eINSTANCE.createSequence();
			// Check to see if it's a valid name by calling NameConverter
			//AR as per mapInputPinSet -> do not need the name for the sequence
			//			JavaNCNameConverter converter = new JavaNCNameConverter();
			//			String convertedName = converter.convertName(NamingUtil.findRegistry(this), sequence, BomUtils.getCanonicalName(outputSet));
			//			sequence.setName(convertedName);
			//sequence.setName(BomUtils.getCanonicalName(outputSet));
			sequence.getActivities().addAll(activityList);
			resultList.add(0, sequence);	
			setMappedActivity(outputSet, sequence);		 
		} else if (activityList.size() == 1){//TODO How to assign a meaningful name
			/* There is only one Activity created  */
			// Check to see if it's a valid name by calling NameConverter
			//			JavaNCNameConverter converter = new JavaNCNameConverter();
			//			String convertedName = converter.convertName(NamingUtil.findRegistry(this), ((Activity)activityList.get(0)), BomUtils.getCanonicalName(outputSet));
			NameProvider nameProvider = NameProviderFactory.getNameProvider((Activity)activityList.get(0));
			String convertedName = nameProvider.getName((Activity)activityList.get(0), outputSet,  NamingUtil.findRegistry(this));


			((Activity)activityList.get(0)).setName(convertedName);
			//((Activity)activityList.get(0)).setName(BomUtils.getCanonicalName(outputSet));
			resultList.add(0, activityList.get(0));
			setMappedActivity(outputSet, (Activity)activityList.get(0));
		} else {
			/* There were no activities created */
		}	 		
		return resultList;
	}

	/**
	 * Sets a BPEL Activity as the source of the BPEL links associated with the pins in the given set. 
	 * This also sets the given activity as the source associated with with the set itself.  
	 * @param pinSet - BOM Input/Output Set with pins associated with the link to update
	 * @param source - BPEL Activity that is the source of the link
	 */	
	protected void setSourceOfLinks(PinSet pinSet, Activity source){
		if (source == null){
			return;
		}	

		/* Keep track of the given source as being mapped to the pinSet.  We will use this info during re-execution. */
		setMappedActivity(pinSet, source);

		/* For each pin in pinSet, connect the links that have that pin as their source */
		List pinList = BomUtils.getPinsInSet(pinSet);

		//20080714 create and reuse the (Empty) activity in case of a local subprocess or loop node			
		boolean isLocalContainer = false;
		if (pinSet instanceof InputPinSet) {
			InputPinSet inputSet = (InputPinSet)pinSet;

			if ((inputSet instanceof LoopNode || BomUtils.isLocalProcess(inputSet.getAction()))) {
				if (inputSet.getAction().getInputPinSet().size() > 1) {
					Flow sourceFlowContainer = null;
					if (source.getContainer() != null && source.getContainer() instanceof Flow) {
						sourceFlowContainer = ((Flow)source.getContainer());
					}				
					
					isLocalContainer = true;
					Activity startScopeAct = ScopeRegistryUtil.getBpelActivity(getContext(), inputSet.getAction());
					if (startScopeAct == null) {
						//create an empty activity
						startScopeAct = BPELFactory.eINSTANCE.createEmpty();
						String activityName = BomUtils.getCanonicalName((NamedElement)inputSet.eContainer());
						JavaNCNameConverter converter = new JavaNCNameConverter();
						String actConvertedName = converter.convertName(NamingUtil.findRegistry(this), startScopeAct, activityName);
						startScopeAct.setName(actConvertedName);
						DisplayName dspName = BPELPlusFactory.eINSTANCE.createDisplayName();
						startScopeAct.getEExtensibilityElements().add(dspName);
						dspName.setText(((NamedElement)inputSet.eContainer()).getName());		
						if (sourceFlowContainer != null) {
							sourceFlowContainer.getActivities().add(startScopeAct);
						}				
						ScopeRegistryUtil.registerNamedElement(getContext(), inputSet.getAction(), startScopeAct);
						BpelOptimizationUtil.registerNonOptimizableElement(getContext(), startScopeAct);
					}
					// for inputSet with multiple InputObjectPins (control pin cannot be connected to within the subprocess) - create ADDITIONAL empty that will work as a FORK gateway
					if (inputSet.getInputObjectPin().size() > 1 ) {
						Activity inputSetStartScopeAct = BPELFactory.eINSTANCE.createEmpty();
						String inputSetActivityName = BomUtils.getCanonicalName((NamedElement)inputSet.eContainer()) + "_" + BomUtils.getCanonicalName(inputSet);
						JavaNCNameConverter converter = new JavaNCNameConverter();
						String isActConvertedName = converter.convertName(NamingUtil.findRegistry(this), inputSetStartScopeAct, inputSetActivityName);
						inputSetStartScopeAct.setName(isActConvertedName);
						DisplayName dspName = BPELPlusFactory.eINSTANCE.createDisplayName();
						inputSetStartScopeAct.getEExtensibilityElements().add(dspName);
						dspName.setText(((NamedElement)inputSet.eContainer()).getName() + inputSet.getName());		
						if (sourceFlowContainer != null) {
							((Flow)source.getContainer()).getActivities().add(inputSetStartScopeAct);
						}
						//connect InputPinSet activity with Scope Activity, set transition condition
						Link scopeStartToInputSetLink = linkActivities(startScopeAct, inputSetStartScopeAct, startScopeAct.getName() + "_to_" + inputSetActivityName);
						if (sourceFlowContainer != null && sourceFlowContainer.getLinks()==null) {
							Links links = BPELFactory.eINSTANCE.createLinks();
							sourceFlowContainer.setLinks(links);
						}
						sourceFlowContainer.getLinks().getChildren().add(scopeStartToInputSetLink);
						com.ibm.wbit.bpel.Variable setVariable = ScopeRegistryUtil.getBpelVariable(getContext(), inputSet);
						if (setVariable != null) {
							Source linkSource = (Source)scopeStartToInputSetLink.getSources().get(0);
							com.ibm.wbit.bpel.Condition condition = BPELFactory.eINSTANCE.createCondition();
							condition.setExpressionLanguage(AbstractbpelConstants.EXPRESSION_JAVA_NAMESPACE);
							condition.setBody(ScopeRegistryUtil.RETURN_STATEMENT_START + setVariable.getName() + ScopeRegistryUtil.RETURN_STATEMENT_END);
							linkSource.setTransitionCondition(condition);	
						}
						
						// connect pins in input set from the scopeStartInputSet activity
						for(Iterator it = pinList.iterator(); it.hasNext(); ){						
							setSourceOfLinks(pinSet, (Pin)it.next(), inputSetStartScopeAct);
						}
			
					} else {
						//set source for the link
						for(Iterator it = pinList.iterator(); it.hasNext(); ){						
							setSourceOfLinks(pinSet, (Pin)it.next(), startScopeAct);
						}
					}
				}
			}
		}	
		if (!isLocalContainer) { // code used prior to cross link clean-up
			for(Iterator it = pinList.iterator(); it.hasNext(); ){						
				setSourceOfLinks(pinSet, (Pin)it.next(), source);
			}

			/* Connect the links that have pinSet as their (only) source */
			setSourceOfLinks(pinSet, null, source);
		}

		/* Update the join condition of the target of the pins in the set */			
		setTargetJoinConditions(pinList);

	}

	/**
	 * Check if the linkRule represents the link that connects from the outputPinSet (correlates to the current activity inputPinSet)
	 * to the termination node.   
	 * @param lnkRule represnts a link that connects the activity to a termination node
	 * @param act
	 * @return
	 */
	private boolean isLinkRuleForConnectingActivityToTermNode(LinkRule lnkRule, Activity act) {
		List lnkSources = lnkRule.getSource();
		InputPinSet inPinSet = (InputPinSet)lnkSources.get(lnkSources.size()-1);
		/*
		Collection actRules=TransformationUtil.getRulesForSource(ProcessUtil.getProcessDefinitionRule(getContext()), inPinSet);
		TransformationRule actRule;
		for (Iterator it2=actRules.iterator();it2.hasNext();) {
			actRule = (TransformationRule)it2.next();
			if (actRule.getTarget().contains(act)) {
				return true;
			}
		}
		 */
		if (getSource().get(0)==inPinSet) {
			return true;
		}
		return false;
	}

	/**
	 * Check if the linkRule represents the link that connects to a TerminationNode and
	 * that the link represents a special case of a Task that has multiple inputPinSet
	 * correlates to the same outputPinSet.  See createTerminationLinkRule method in
	 * ProcessWalkingRuleImpl for more details.
	 * @param lnkRule
	 * @return
	 */
	private boolean isLinkRuleForTermNodeWithInPinSet(LinkRule lnkRule) {
		List lnkSources = lnkRule.getSource();
		Object tar;
		for (Iterator it1=lnkSources.iterator();it1.hasNext();) {
			tar = (Object)it1.next();
			if (tar instanceof TerminationNode) {
				// Normally, the LinkRule for TerminationNode connection will have a OutputPinset as the
				// outcome of the TerminationNode as the last source of the link.
				if (lnkSources.get(lnkSources.size()-1) instanceof InputPinSet) {
					return true;
				}
				return false;
			}
		}
		return false;
	}
	/**
	 * Sets a BPEL Activity as the source of the BPEL link associated with the given set and pin. 
	 * The link represents the BOM edge that is connected to the pin in the Input/Output Set.  If
	 * the pin is null, then links that have <i>only</i> the set as source will be updated. 
	 * @param pinSet - BOM Input/Output Set associated with the link to update
	 * @param pin - BOM Pin associated with the link to update.  If this is null, links associated
	 * with only the given set are updated.
	 * @param source - BPEL Activity that is the source of the link
	 */	
	private void setSourceOfLinks(PinSet pinSet, Pin pin, Activity source){
		Collection linkRules = new ArrayList();
		Collection sources = new ArrayList();
		String connectionSourceName = null;

		if (pin != null){
			// get LinkRules for the pin and set
			sources.add(pin);
			sources.add(pinSet);
			sources.add(pin.getOutgoing());						
			linkRules = TransformationUtil.getRulesForSource(ProcessUtil.getProcessDefinitionRule(getContext()), sources, Link.class);
			connectionSourceName = BomUtils.getCanonicalName(pin);
		} else if (pin == null){
			//	get LinkRules that have ONLY the set as the source,
			//  OR that have set as a source AND have an intial node as the source 
			sources.add(pinSet);
			Collection tempRules = TransformationUtil.getRulesForSource(ProcessUtil.getProcessDefinitionRule(getContext()), sources, Link.class); 
			Object tempRule = null;
			// Weed out the rules that that have other sources, in addition to 'set'
			List ruleSources = null;
			for (Iterator it=tempRules.iterator(); it.hasNext(); ){
				tempRule = it.next();

				if (! (tempRule instanceof LinkRule)){
					continue;
				}

				if (((LinkRule)tempRule).getSource().size() == 1){
					linkRules.add(tempRule);
				} else if (pinSet instanceof InputPinSet){
					List initNodeList = BomUtils.getInitialNodesForSet((InputPinSet)pinSet);
					if ( initNodeList.removeAll( ((LinkRule)tempRule).getSource()) ) {
						/* One of the sources of tempRule was an initial node for this output set, so add the rule */
						linkRules.add(tempRule);
					}					

				} else if (pinSet instanceof OutputPinSet && ((LinkRule)tempRule).getSource().size() > 1/*== 2*/ && ((LinkRule)tempRule).getSource().get(0)==pinSet) {
					// Ray - this is to handle the case where there is no
					// outgoing edge from the outputPinSet or an empty outputPinSet.
					// Support for ImplicitTermination
					// See also ProcessWalkingRule
					linkRules.add(tempRule);
				}
			}
			connectionSourceName = BomUtils.getCanonicalName(pinSet);
		}			

		if(linkRules.isEmpty()){
			return;
		}

		// Collect all the linkNames that are already in use so we don't use a duplicate
		List linkNames = new ArrayList();
		String nextName = null;
		for (Iterator it = linkRules.iterator(); it.hasNext(); ){
			nextName = ((Link)((LinkRule)it.next()).getTarget().get(0)).getName();
			if(nextName != null){
				linkNames.add(nextName);
			}
		}	

		// For each link, set the BPEL source and update the link name, if it's not already done 
		Link link= null;
		LinkRule linkRule = null;
		int index = 1;
		for (Iterator it = linkRules.iterator(); it.hasNext(); ){
			linkRule = (LinkRule)it.next();
			//Ray
			if (isLinkRuleForTermNodeWithInPinSet(linkRule)) {
				if (!isLinkRuleForConnectingActivityToTermNode(linkRule,source)) {
					continue;
				}
			}
			link = (Link)linkRule.getTarget().get(0);
			if(link.getSources().isEmpty()){
				//link doesn't have source yet, so set it
				Source src = BPELFactory.eINSTANCE.createSource();
				src.setActivity(source);
				link.getSources().add(src);
				//proposed link name:
				String oldName = (link.getName()== null)? "" : (link.getName());
				String newName = connectionSourceName + "_to_" + oldName;
				//only need to check if name is duplicated when both source and target has been set
				/*
				if(link.getTarget() != null){
					String tryName = newName;
					while(isNameExist(linkNames, tryName)){
						tryName = newName +  "_"  + index; //append index number to make the name unique
						index++;
					}
					newName = tryName;
					linkNames.add(newName);				     				
				}
				 */

				//update the name which has added first half of the link
				// Check to see if it's a valid name by calling NameConverter
				if (newName.charAt(newName.length()-1) != '_'){				
					JavaNCNameConverter converter = new JavaNCNameConverter();				
					String convertedName = converter.convertName(NamingUtil.findRegistry(this), link, newName);
					link.setName(convertedName);
					//link.setName(newName);					
				}
				else{
					link.setName(newName);
				}
				//Record the fault link that is directly connected to the downstream target without generated empty activity
				if(pinSet instanceof OutputPinSet && 
						((OutputPinSet)pinSet).getIsException().booleanValue()&& 
						(source instanceof Invoke || source instanceof Scope)){
					//Guard same type in the map or test if the outgoing link refers to a branch merge point like Merge/Join
					List nextPinSet = ProcessUtil.getBomWalker(getContext()).getNext((OutputPinSet)pinSet, BomWalker.INPUT_SET);
					boolean shouldSkip = false;
					if(nextPinSet != null && !nextPinSet.isEmpty()){
						for(Object eachPinSet:nextPinSet){
							if(eachPinSet instanceof InputPinSet){
								InputPinSet ips = (InputPinSet)eachPinSet;
								if(ips.eContainer() instanceof Merge){
									shouldSkip = true;
								}
							}
						}
					}
					if(!shouldSkip){
						CBPELUtil.registerFaultSource(getContext(), this, ((OutputPinSet)pinSet).getName(), link);
						if(source instanceof Scope){
							//register fault link for the scope
							CBPELUtil.registerFaultLinkForScope(getContext(),((OutputPinSet)pinSet),link);
						}
					}
				}

				/* Set the transition condition of the source */
				setTransitionCondition(linkRule);
			}//we need to reset the source of the link in the case of a split assign generated
			else{
				if(link.getSources().get(0)instanceof Source){
					Source theSrc = (Source)link.getSources().get(0);
					BPELVariable bpelVar = ScopeRegistryUtil.getBpelVariable(getContext(), pinSet);
					//Assert this is the case of wrapper variable
					if(bpelVar != null && bpelVar.getXSDElement()!= null && source instanceof Assign){
						theSrc.setActivity(source);
						CBPELUtil.registerFaultLinkForScope(getContext(),((OutputPinSet)pinSet),link);
					}
				}
			}
		} 				
	}

	/**
	 * Sets the given activity as the target of the links associated with pinSet, and each of the
	 * pins in pinSet.
	 * 
	 * @param pinSet The PinSet for which to get the associated link rules
	 * @param target The activity to set as the target of the links associated with pinset and its contents
	 */
	protected void setTargetOfLinks(PinSet pinSet, Activity target){
		if (target == null){
			return;
		}

		/* Keep track of the given target as being mapped to the pinSet.  We will use this info during re-execution. */
		setMappedActivity(pinSet, target);

		/* For each pin in pinSet, connect the links that have that pin as their target */
		List pinList = BomUtils.getPinsInSet(pinSet);

		for(Iterator it = pinList.iterator(); it.hasNext(); ){						
			setTargetOfLinks(pinSet, (Pin)it.next(), target);
		}

		/* Connect the links that are incoming to the pin set itself, and not a particular pin */
		setTargetOfLinks(pinSet, null, target);

		/* Update the join condition of the target of the pins in the set */			
		setJoinCondition(pinSet, target);

	}

	/**
	 * Sets the given activity as the target of the links associated with pin, in pinSet.  If
	 * pin is null, then the activity is set as the target of any links that have only pinSet as their
	 * source.
	 * @param pinSet The pinSet that contains pin
	 * @param pin The pin to get the link rules for
	 * @param target The activity to set as the target of the links
	 */
	private void setTargetOfLinks(PinSet pinSet, Pin pin, Activity target){
		Collection linkRules = new ArrayList();
		String connectionTargetName = null;
		Collection sources= new ArrayList();

		if (pin != null){
			//	get LinkRules for the pin and set
			sources.add(pin);
			sources.add(pinSet);
			sources.add(pin.getIncoming());
			linkRules = TransformationUtil.getRulesForSource(ProcessUtil.getProcessDefinitionRule(getContext()), sources, Link.class);
			connectionTargetName = BomUtils.getCanonicalName(pin);

		} else if (pin == null){
			// get LinkRules that either:
			// 1. Have ONLY the set as the source 
			//		This happens when linking from an <onMessage> to the input set of a top-level SAN,
			//      OR when linking a termination variable initialization activity in a loop node
			//		to the starting activities in that loop node 
			// 2. Have set as a source AND have a termination node as the source 
			//		This happens when linking the output set of a Nested or top-level SAN
			sources.add(pinSet);
			Collection tempRules = TransformationUtil.getRulesForSource(ProcessUtil.getProcessDefinitionRule(getContext()), sources, Link.class); 

			// Weed out the rules that that have other sources in addition to 'set'
			Object tempRule = null;
			for (Iterator it=tempRules.iterator(); it.hasNext(); ){
				tempRule = it.next();
				if (! (tempRule instanceof LinkRule)){
					continue;
				}

				if (((LinkRule)tempRule).getSource().size() == 1){
					linkRules.add(tempRule);
				} else if (pinSet instanceof OutputPinSet){
					if (((LinkRule)tempRule).getSource().size() == 2 && ((LinkRule)tempRule).getSource().get(1)==pinSet) {
						// Ray - this is to handle the case where there is no
						// outgoing edge from the outputPinSet or an empty outputPinSet.
						// Support for ImplicitTermination
						// See also ProcessWalkingRule
						linkRules.add(tempRule);
						//AR added for the case of multiplePinSets - implicit termination from a BS with multiple PinSet
					}else if (((LinkRule)tempRule).getSource().size() > 2 && ((LinkRule)tempRule).getSource().get(((LinkRule)tempRule).getSource().size() -1)==pinSet) {
						// Ray - this is to handle the case where there is no
						// outgoing edge from the outputPinSet or an empty outputPinSet.
						// Support for ImplicitTermination
						// See also ProcessWalkingRule
						linkRules.add(tempRule);
					}else {					
						List termNodeList = BomUtils.getTerminationNodesForSet((OutputPinSet)pinSet);
						if ( termNodeList.removeAll( ((LinkRule)tempRule).getSource()) ) {
							/* One of the sources of tempRule was a termination node for this output set, so add the rule */
							linkRules.add(tempRule);
						}
					}
				}
			}
			connectionTargetName = BomUtils.getCanonicalName(pinSet);
		}			

		if(linkRules.isEmpty()){
			return;
		}

		// Compile a list of the names of all the links that pertain to pin & set.
		List linkNames = new ArrayList();
		String nextName = null;
		for (Iterator it = linkRules.iterator(); it.hasNext(); ){
			nextName = ((Link)((LinkRule)it.next()).getTarget().get(0)).getName();
			if(nextName != null) {
				linkNames.add(nextName);
			}
		}

		// set the target as the target of the link and update the link name		 
		Link link= null;
		LinkRule linkRule = null;
		int index = 1; //start from 1
		for (Iterator it = linkRules.iterator(); it.hasNext(); ){
			linkRule = (LinkRule)it.next();
			link = (Link)linkRule.getTarget().get(0);
			if(link.getTargets().isEmpty()){
				//link doesn't have target yet, so set it
				Target tar = BPELFactory.eINSTANCE.createTarget();
				tar.setActivity(target);
				link.getTargets().add(tar);

				//proposed link name:
				String oldName = (link.getName()== null )? "" : (link.getName());
				// connectionTargetName was previously set to be the canonical name of either
				// the pin or the set (if there was no pin)
				String newName = oldName + connectionTargetName;

				//we only need to do this when source and target both has been set
				/*
				if(link.getSource() != null){				
					String tryName = newName;
					while(isNameExist(linkNames, tryName)){
						tryName = newName +  "_"  + index; //append index number to make the name unique
						index++;
					}
					newName = tryName; 
					//add to the name list
					linkNames.add(newName);
				}
				 */
				//update the name which has added as second half of the link
				// Check to see if it's a valid name by calling NameConverter
				JavaNCNameConverter converter = new JavaNCNameConverter();
				String convertedName = converter.convertName(NamingUtil.findRegistry(this), link, newName);
				link.setName(convertedName);
				//link.setName(newName);	  
			}		
		}

	}

	/**
	 * Sets the transition condition of the link represented by linkRule to check whether
	 * the termination node has yet been reached in the most deeply nested SAN that contains
	 * a pin that is in the list of sources of linkRule.
	 * 
	 * @param linkRule - The rule representing the link to set the transition condition of
	 * @param linkSource - The actual container for the transition condition
	 */
	private void setTransitionCondition(LinkRule linkRule){
		/* Set the transition condition based on the scope of the most deeply nested pin 
		 * that is in the list of sources of linkRule */

		if (BpelOptimizationUtil.bDoTerminationLogic) {
			Source linkSource = (Source)((Link)linkRule.getTarget().get(0)).getSources().get(0);
			StructuredActivityNode deepestSAN = null;
			for (Iterator it = linkRule.getSource().iterator(); it.hasNext(); ){
				Object srcObj = it.next();
				if (srcObj instanceof Pin){
					Action action = (Action)((Pin)srcObj).eContainer();

					StructuredActivityNode parent = action.getInStructuredNode();

					if (deepestSAN == null || deepestSAN.getNodeContents().contains(parent)){
						/* 'parent' is contained in the current 'deepestSAN', so 'parent' is deeper */
						deepestSAN = parent; 
					}
				}
			}

			/* Only set the transition condition for links that have at least one pin as a source */
			if (deepestSAN == null){
				return;
			}

			/* Now gather a list of SANs all the way up to the top.  We have to check the termination variables 
			 * associated with each SAN in the hierarchy */
			List sanList = new ArrayList();
			List terminationVarList = new ArrayList();
			for (StructuredActivityNode san = deepestSAN; san != null; san = san.getInStructuredNode()){		
				/* Get the termination variable associated with deepestSAN */
				ContainerRule cr = (ContainerRule)TransformationUtil.getRuleForSource(ProcessUtil.getProcessDefinitionRule(getContext()), ContainerRule.class, san);
				com.ibm.wbit.bpel.BPELVariable terminationVar = (com.ibm.wbit.bpel.BPELVariable)cr.getTarget().get(0);
				Part part = (Part)terminationVar.getMessageType().getParts().values().iterator().next();

				terminationVarList.add(terminationVar);
			}

			/* Create Expression Model for the transition condition of linkSource */
			if (0 != terminationVarList.size())
			{
				ExtensibilityExpression bpelExpr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();
				Expression expr = composeTransitionCondition(terminationVarList);
				bpelExpr.setExpression(expr);
				linkSource.addExtensibilityElement(bpelExpr);	
			}	
		}

		//try to find the link rules for links to subprocess or loop node				
		if (linkRule.getSource().size() > 3 && linkRule.getSource().get(3) instanceof InputPinSet) {
			InputPinSet iSet = (InputPinSet)linkRule.getSource().get(3);
			com.ibm.wbit.bpel.Variable setVariable = ScopeRegistryUtil.getBpelVariable(getContext(), iSet);
			if (setVariable != null) {
				Source linkSource = (Source)((Link)linkRule.getTarget().get(0)).getSources().get(0);
				if (linkSource.getActivity() != null && 	// this is to prevent creating Transition Condition for Subprocess with Multiple InputPinSets with Many InputPins
						(linkSource.getActivity().getTargets() == null || linkSource.getActivity().getTargets().getChildren() == null || linkSource.getActivity().getTargets().getChildren().isEmpty())) {
					com.ibm.wbit.bpel.Condition condition = BPELFactory.eINSTANCE.createCondition();
					condition.setExpressionLanguage(AbstractbpelConstants.EXPRESSION_JAVA_NAMESPACE);
					condition.setBody(ScopeRegistryUtil.RETURN_STATEMENT_START + setVariable.getName() + ScopeRegistryUtil.RETURN_STATEMENT_END);
					linkSource.setTransitionCondition(condition);
				}
			}
		} else if (linkRule.getSource().size() > 3 && linkRule.getSource().get(3) instanceof OutputPinSet) {
			OutputPinSet oSet = (OutputPinSet)linkRule.getSource().get(3);
			if (oSet.getIsException() != null && !oSet.getIsException().booleanValue()) {
				com.ibm.wbit.bpel.Variable setVariable = ScopeRegistryUtil.getBpelVariable(getContext(), oSet);
				if (setVariable != null) {
					Source linkSource = (Source)((Link)linkRule.getTarget().get(0)).getSources().get(0);
					com.ibm.wbit.bpel.Condition condition = BPELFactory.eINSTANCE.createCondition();
					condition.setExpressionLanguage(AbstractbpelConstants.EXPRESSION_JAVA_NAMESPACE);
					condition.setBody(ScopeRegistryUtil.RETURN_STATEMENT_START + setVariable.getName() + ScopeRegistryUtil.RETURN_STATEMENT_END);
					linkSource.setTransitionCondition(condition);
				}
			}
		}
	}

	/**
	 * A recursive method that returns an expression
	 * @param parentSANList - An InputPinSet or OutputPinSet in which to look for pins of type <code>type</code>
	 * @param terminationVarList - A Class indicating the type of pin to retrieve in set (e.g. StoreArtifactPin.class, etc.)
	 * @return Expression - An expression
	 */
	private Expression composeTransitionCondition(List terminationVarList)
	{
		if (BpelOptimizationUtil.bDoTerminationLogic) {
			FunctionExpression functionExpr = ModelFactory.eINSTANCE.createFunctionExpression();
			FunctionDefinition def = ModelFactory.eINSTANCE.createFunctionDefinition();
			def.setFunctionID("com.ibm.btools.te.ilm.heuristics.abstractbpel.getVariableData");
			def.setFunctionName("bpws:getVariableData");

			FunctionArgument arg1 = ModelFactory.eINSTANCE.createFunctionArgument();			
			StringLiteralExpression strExpr1 = ModelFactory.eINSTANCE.createStringLiteralExpression();
			strExpr1.setStringSymbol(((com.ibm.wbit.bpel.Variable)terminationVarList.iterator().next()).getName());
			arg1.setArgumentValue(strExpr1);

			FunctionArgument arg2 = ModelFactory.eINSTANCE.createFunctionArgument();
			Part part = (Part)((com.ibm.wbit.bpel.BPELVariable)terminationVarList.iterator().next()).getMessageType().getParts().values().iterator().next();
			StringLiteralExpression strExpr2 = ModelFactory.eINSTANCE.createStringLiteralExpression();
			strExpr2.setStringSymbol(part.getName());
			arg2.setArgumentValue(strExpr2);

			functionExpr.setDefinition(def);
			functionExpr.getArguments().add(arg1);
			functionExpr.getArguments().add(arg2);	

			if (1 == terminationVarList.size())
			{
				/* Recursion Base Case - Create a bpws:getVariableData FunctionExpression */
				BooleanLiteralExpression falseExpr = ModelFactory.eINSTANCE.createBooleanLiteralExpression();
				falseExpr.setBooleanSymbol(Boolean.FALSE);	
				ComparisonExpression cmpExpr = ModelFactory.eINSTANCE.createComparisonExpression();
				cmpExpr.setFirstOperand(functionExpr);
				cmpExpr.setOperator(ComparisonOperator.EQUAL_TO_LITERAL);
				cmpExpr.setSecondOperand(falseExpr);
				return cmpExpr;
			}
			else
			{
				/* Recursion Step */
				BooleanLiteralExpression falseExpr = ModelFactory.eINSTANCE.createBooleanLiteralExpression();
				falseExpr.setBooleanSymbol(Boolean.FALSE);			

				ComparisonExpression cmpExpr = ModelFactory.eINSTANCE.createComparisonExpression();
				cmpExpr.setFirstOperand(functionExpr);
				cmpExpr.setOperator(ComparisonOperator.EQUAL_TO_LITERAL);
				cmpExpr.setSecondOperand(falseExpr);


				BinaryLogicalBooleanExpression boolExpr = ModelFactory.eINSTANCE.createBinaryLogicalBooleanExpression();			
				boolExpr.setFirstOperand(cmpExpr);
				boolExpr.setOperator(LogicalBooleanOperator.AND_LITERAL);
				boolExpr.setSecondOperand(composeTransitionCondition(terminationVarList.subList(1, terminationVarList.size())));
				return boolExpr;			
			}
		} else {
			return null;
		}
	}


	/**
	 * For each pin in the list, update the join condition for the target of the pin's edge, 
	 * if there is one.
	 * 	 
	 * @param pins - list of pins that its target join condition should be updated
	 */
	private void setTargetJoinConditions(List pins){	

		Collection targetSets= new HashSet();
		NamedElement targetSet;
		LinkRule lr;
		Link link;
		Pin pin;
		Collection sources = new ArrayList(); 

		for(int i=0, count = pins.size(); i< count; i++){
			pin = (Pin)pins.get(i);
			targetSet = BomUtils.findTargetSet(pin.getOutgoing());
			if(targetSet != null && !targetSets.contains(targetSet)){
				targetSets.add(targetSet);
				sources.clear();
				sources.add(pin.getOutgoing().getTarget());
				sources.add(targetSet);
				sources.add(pin.getOutgoing());
				lr =(LinkRule)TransformationUtil.getRuleForSource(ProcessUtil.getProcessDefinitionRule(getContext()), sources, Link.class);
				if(lr != null && !lr.getTarget().isEmpty()){
					link = (Link)lr.getTarget().get(0);
					if(!link.getTargets().isEmpty()){
						setJoinCondition((PinSet)targetSet, ((Target)link.getTargets().get(0)).getActivity());
					}
				} 	
			}
		}		
	}


	/**
	 * Update the join condition of the target activity for the InputPinSet or OutputPinSet.
	 * The join condition of the target activity is assembled as following:
	 * 	For each pin in the pins list, find all link rules that represent the incoming edge to that pin, combine them with "OR".
	 * 	Combine these "OR"ed link rules for all pins with "AND". 
	 * If target activity only has one target, then no join condition is needed. 
	 * @param set - BOM InputPinSet or OuputSet
	 * @param target - target BPEL activity 
	 */
	protected void setJoinCondition(PinSet pinSet, Activity activity){
		List pins = BomUtils.getPinsInSet((PinSet)pinSet);
		//if target activity has one target, that means either target has only one target or target activity's target has not been complete.
		//in either cases, we don't need to do joinCondition yet.		    
		//migration AR 11.09
		if(activity.getTargets() == null || activity.getTargets().getChildren().size()<= 1){
			return;
		}

		List linkRules = null;
		Collection sources = new ArrayList();

		/* Create a join condition that will require that at least one link associated with 
		 * each pin in the input set has a status of TRUE before the target BPEL activity will execute. */

		/* Gather the link rules associated with termination nodes and this set */
		List terminationLinkRules = null;
		List implicitTerminationLinkRules = null; // Ray
		if (pinSet instanceof OutputPinSet 
				&& (BomUtils.isProcess( (Action)pinSet.eContainer()) 
						|| pinSet.eContainer() instanceof LoopNode)){
			terminationLinkRules = getLinkRulesForTerminationNode(pinSet);
			implicitTerminationLinkRules = getLinkRulesForImplicitTermination(pinSet);
		}

		/* Create Expression Model for the JoinCondition */
		JoinExtensibilityExpression bpelExpr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createJoinExtensibilityExpression();
		Expression termExpr = (terminationLinkRules != null) 
		? createLogicalOr(getLinkRulesForTerminationNode(pinSet)) 
				: null;
		// Ray
		Expression impTermExpr = (implicitTerminationLinkRules != null) 
		//							  ? createLogicalAnd(implicitTerminationLinkRules) 
		? createLogicalOr(implicitTerminationLinkRules)
				: null;
		Expression pinExpr = createLogicalAnd(pins, pinSet);		
		/*		
		if ( (termExpr!=null) && (pinExpr!=null) ){
			BinaryLogicalBooleanExpression boolExpr = ModelFactory.eINSTANCE.createBinaryLogicalBooleanExpression();
			boolExpr.setFirstOperand(termExpr);
			boolExpr.setSecondOperand(pinExpr);
			boolExpr.setOperator(LogicalBooleanOperator.AND_LITERAL);
			bpelExpr.setExpression(boolExpr);

		} else if ( (termExpr==null) && (pinExpr!=null) ) {
			bpelExpr.setExpression(pinExpr);

		} else if ( (termExpr!=null) && (pinExpr==null) ) {
			bpelExpr.setExpression(termExpr);
		}
		 */
		if ( (termExpr!=null) && (pinExpr!=null) && (impTermExpr==null)){
			BinaryLogicalBooleanExpression boolExpr = ModelFactory.eINSTANCE.createBinaryLogicalBooleanExpression();
			boolExpr.setFirstOperand(termExpr);
			boolExpr.setSecondOperand(pinExpr);
			boolExpr.setOperator(LogicalBooleanOperator.AND_LITERAL);
			bpelExpr.setExpression(boolExpr);

		} else if ( (termExpr==null) && (pinExpr!=null) && (impTermExpr==null)) {
			bpelExpr.setExpression(pinExpr);

		} else if ( (termExpr!=null) && (pinExpr==null) && (impTermExpr==null)) {
			bpelExpr.setExpression(termExpr);
		} else if ( (termExpr!=null) && (pinExpr!=null) && (impTermExpr!=null)){
			BinaryLogicalBooleanExpression boolExpr1 = ModelFactory.eINSTANCE.createBinaryLogicalBooleanExpression();
			boolExpr1.setFirstOperand(termExpr);
			boolExpr1.setSecondOperand(pinExpr);
			boolExpr1.setOperator(LogicalBooleanOperator.AND_LITERAL);
			BinaryLogicalBooleanExpression boolExpr2 = ModelFactory.eINSTANCE.createBinaryLogicalBooleanExpression();
			boolExpr2.setFirstOperand(boolExpr1);
			boolExpr2.setSecondOperand(impTermExpr);
			boolExpr2.setOperator(LogicalBooleanOperator.AND_LITERAL);
			bpelExpr.setExpression(boolExpr2);

		} else if ( (termExpr==null) && (pinExpr!=null) && (impTermExpr!=null)) {
			BinaryLogicalBooleanExpression boolExpr = ModelFactory.eINSTANCE.createBinaryLogicalBooleanExpression();
			boolExpr.setFirstOperand(impTermExpr);
			boolExpr.setSecondOperand(pinExpr);
			boolExpr.setOperator(LogicalBooleanOperator.AND_LITERAL);
			bpelExpr.setExpression(boolExpr);
		} else if ( (termExpr!=null) && (pinExpr==null) && (impTermExpr!=null)) {
			BinaryLogicalBooleanExpression boolExpr = ModelFactory.eINSTANCE.createBinaryLogicalBooleanExpression();
			boolExpr.setFirstOperand(impTermExpr);
			boolExpr.setSecondOperand(termExpr);
			boolExpr.setOperator(LogicalBooleanOperator.AND_LITERAL);
			bpelExpr.setExpression(boolExpr);
		} else if ( (termExpr==null) && (pinExpr==null) && (impTermExpr!=null)) {
			bpelExpr.setExpression(impTermExpr);
		}
		/* Remove any existing join condition */
		for (int i=activity.getExtensibilityElements().size()-1; i>=0; i--){
			if (activity.getExtensibilityElements().get(i) instanceof JoinExtensibilityExpression){
				activity.getExtensibilityElements().remove(i);
			}
		}

		/* Add the new join condition */
		activity.addExtensibilityElement(bpelExpr);		
	}


	/**
	 * Returns a list of LinkRules that represent the incoming links associated with the termination nodes
	 * of the parent SAN.
	 * @param pinSet The pinSet that is being linked to
	 * @return List - The list of link rules
	 */
	private List getLinkRulesForTerminationNode(PinSet pinSet){
		List sources = new ArrayList();
		List linkRules = new ArrayList();
		sources.clear();
		sources.add(pinSet);
		linkRules = (List)TransformationUtil.getRulesForSource(ProcessUtil.getProcessDefinitionRule(getContext()), sources, Link.class);

		for (int i=linkRules.size()-1; i>=0; i--){
			if (!(linkRules.get(i) instanceof LinkRule)){
				linkRules.remove(i);
				continue;
			}

			List termNodeList = BomUtils.getTerminationNodesForSet((OutputPinSet)pinSet);
			if ( !termNodeList.removeAll( ((LinkRule)linkRules.get(i)).getSource()) ) {
				/* None of the sources were a termination node for this output set, so remove the rule */
				linkRules.remove(i);
			}
		}
		return linkRules;

	}

	private List getLinkRulesForImplicitTermination(PinSet pinSet){
		List sources = new ArrayList();
		List linkRules = new ArrayList();
		sources.clear();
		sources.add(pinSet);
		linkRules = (List)TransformationUtil.getRulesForSource(ProcessUtil.getProcessDefinitionRule(getContext()), sources, Link.class);

		for (int i=linkRules.size()-1; i>=0; i--){
			if (!(linkRules.get(i) instanceof LinkRule)){
				linkRules.remove(i);
				continue;
			}
			LinkRule lr = (LinkRule)linkRules.get(i);
			if ( lr.getSource().size()!=2) {
				//defect 76704. This is to handle implicit termination from an Activity with
				//multiple PinSets such as a BSO with Fault set or a Process with Multiple pinsets.
				//In this case the size >2, as it will include the all the sets. The check insures that there is no ObjectFlow, control flow
				//in the source. Also eliminates the sources with TerminationNode.
				if (lr.getSource().size()>2) {
					for(Object thisSource: lr.getSource()){
						if(thisSource instanceof ObjectFlow || thisSource instanceof ControlFlow || thisSource instanceof TerminationNode){
							linkRules.remove(i);
							break;
						}
					}
				}else{
					linkRules.remove(i);
				}
			} else if (((LinkRule)linkRules.get(i)).getSource().get(0) == pinSet) {
				// For implicit termination link, there will be two sources in the LinkRule
				// Only the second source is the outputPinSet of the SAN (process/loopNode)
				// that represents the termination of a "Structured" activity  
				linkRules.remove(i);		
			}
		}
		return linkRules;

	}


	/**
	 * Returns the activity that is mapped with the given key
	 * @param key The input/output PinSet for which to look for the Activity that represents it
	 * @return Activity - the activity that was created to represent the given PinSet 
	 */
	public Activity getMappedActivity(NamedElement key){
		return (Activity)ivActivityMap.get(key);
	}

	/**
	 * Sets the given activity as the one that is associated with the InputPinSet / OutputPinSet
	 * @param key The InputPinSet / OutputPinSet to associated with <code>value</code>
	 * @param value The Activity that represents the mapped version of <code>key</code>
	 */
	private void setMappedActivity(NamedElement key, Activity value){
		ivActivityMap.put(key, value);
	}

	/**
	 * Returns true if the given key has been registered in the activity map, even if it has
	 * been registered with a null value.
	 * @param key The key to look for in the activty map.
	 * @return boolean - True if the given key has been recorded in the map, false otherwise.
	 */
	private boolean isMappedActivity(NamedElement key){
		return ivActivityMap.containsKey(key);
	}

	//	/**
	//	 * Creates the Activity required to initialize the termination node variable for
	//	 * the SAN the contains the given input set.  Both BPEL and BPEL+ versions are created, using
	//	 * the AlternativeActivity
	//	 * @param inputSet The input pin set on the SAN for which to initialize the termination onde
	//	 * @return Assign The BPEL Assign activity that initializes the termination node variable
	//	 */
	//	private Activity initializeTerminationVariable(InputPinSet inputSet){
	//		/* Find the SAN that contains inputSet... 
	//		 * We will use that SAN to find the associated BPEL Variable */
	//		StructuredActivityNode san = (StructuredActivityNode)inputSet.getAction();
	//		
	//		ContainerRule termVarRule = (ContainerRule)		
	//			TransformationUtil.getRuleForSource(ProcessUtil.getProcessDefinitionRule(getContext()),
	//												ContainerRule.class,
	//												san);
	//
	//		com.ibm.etools.ctc.bpel.Variable termVar = (com.ibm.etools.ctc.bpel.Variable)termVarRule.getTarget().get(0);
	//
	//		/* A termination variable always has only one part */
	//		Part termVarPart = (Part)termVar.getMessageType().getEParts().get(0);
	//
	//		/* We need to initalize the variable with the constant expression 'FALSE'.  But because
	//		 * BPEL+ doesn't support XPath, we need to create an AlternativeActivity so that 
	//		 * we have both a BPEL and a BPEL+ version of the <assign> */
	//		AlternativeActivity altActivity = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
	//		Alternative bpelAlt = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
	//		Alternative bpelpAlt = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
	//		altActivity.getAlternative().add(bpelAlt);
	//		altActivity.getAlternative().add(bpelpAlt);
	//		
	//		bpelAlt.setType(ProcessFlowType.BPEL_LITERAL);
	//		bpelpAlt.setType(ProcessFlowType.BPELPP_LITERAL);
	//
	//		bpelAlt.setActivity(createTerminationInitAssign(termVar, termVarPart));
	//		bpelpAlt.setActivity(createTerminationInitJava(termVar, termVarPart));	
	//		
	//		return altActivity;			
	//		
	//	}

	//	/**
	//	 * Create the <assign> version of the termination variable initialization assignment.
	//	 * @param termVar The BPEL Variable that represents the termination node to initialize
	//	 * @param termVarPart The BPEL Variable part the contains the termination value
	//	 * @return Assign - The BPEL Assign that initialize the termination variable
	//	 */
	//	private Assign createTerminationInitAssign(com.ibm.etools.ctc.bpel.Variable termVar, Part termVarPart){
	//		/* Create the BPEL Assign that we will use to set the value of the termination variable */
	//		Assign assign = BPELFactory.eINSTANCE.createAssign();
	//		Copy copy = BPELFactory.eINSTANCE.createCopy();
	//		From from = BPELFactory.eINSTANCE.createFrom();
	//		To to = BPELFactory.eINSTANCE.createTo();
	//		
	//		/* Create an Expression Model to initialize the termination variable with 'FALSE' */
	//		ExtensibilityExpression bpelExpr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();
	//		BooleanLiteralExpression strExpr = ModelFactory.eINSTANCE.createBooleanLiteralExpression();
	//		strExpr.setBooleanSymbol(Boolean.FALSE);
	//		bpelExpr.setExpression(strExpr);
	//		from.addExtensibilityElement(bpelExpr);
	//		
	//		/* Do we need to use a ToExtension object? No, because the expression is on the <from>, which
	//		 * is already capable of havine extensibility elements built in */
	//		to.setVariable(termVar);
	//		to.setPart(termVarPart);
	//		
	//		copy.setFrom(from);
	//		copy.setTo(to);
	//		assign.getCopy().add(copy);
	//		
	//		return assign;
	//		
	//	}
	//	
	//	/**
	//	 * Create the JavaScriptActivity version of the termination variable initialization assignment.
	//	 * @param termVar The BPEL Variable that represents the termination node to initialize
	//	 * @param termVarPart The BPEL Variable part the contains the termination value
	//	 * @return Invoke - The BPEL Invoke that houses the java code to initialize the variable
	//	 */
	//	private Invoke createTerminationInitJava(com.ibm.etools.ctc.bpel.Variable termVar, Part termVarPart){
	//		Invoke javaInvoke = AbstractbpelUtil.createJavaInvoke();
	//		JavaScriptActivity javaScriptActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
	//		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
	//
	//		/* TODO: NOTE: Assuming that we have to captilize the first character of the part and var name */
	//		String varName = AbstractbpelUtil.capitalizeFirstChar(termVar.getName());
	//		String partName = AbstractbpelUtil.capitalizeFirstChar(termVarPart.getName());			
	//		String javaString = "getVariable" + varName + "().set" + partName + "(false);"; 
	//		
	//		javaCode.setValue(javaString);
	//		javaScriptActivity.setJavaCode(javaCode);
	//		javaInvoke.getEExtensibilityElements().add(javaScriptActivity);
	//		
	//		return javaInvoke;
	//	}


	/**
	 * Return true if the name exist in the given name list
	 * @param linkNames - a list containing the link names
	 * @param name - name to be checked
	 * @return boolean
	 */ 
	private boolean isNameExist(List linkNames, String name){

		for(int i= 0, count = linkNames.size(); i<count; i++){
			if(name.equals(linkNames.get(i))){
				return true;
			}
		}	
		return false; 
	}

	/**
	 * Returns the pins of the given type in <code>set</code>
	 * @param set - An InputPinSet or OutputPinSet in which to look for pins of type <code>type</code>
	 * @param type - A Class indicating the type of pin to retrieve in set (e.g. StoreArtifactPin.class, etc.)
	 * @return List - The list of pins of the given type in set.
	 */
	private List getPinsOfType(NamedElement set, Class type){
		List pins = BomUtils.getPinsInSet((PinSet)set);
		List resultPins = new ArrayList();

		Pin tempPin = null;
		for (Iterator it = pins.iterator(); it.hasNext(); ){
			tempPin = (Pin)it.next();
			if (type.isAssignableFrom(tempPin.getClass())){
				resultPins.add(tempPin);
			}
		}

		return resultPins;
	}		 


	/**
	 * This is a general utility method that creates a BPEL Link between two BPEL Activities.
	 * The newly created Link object is returned, with the source and target set.
	 * @param source The BPEL Activity that will be the source of the link
	 * @param target The BPEL Activity that will be the target of the link
	 * @param linkName The String that will be set as the name of the new link
	 * @return Link The newly created BPEL Link
	 */		
	protected Link linkActivities(Activity source, Activity target, String linkName){
		Link link = BPELFactory.eINSTANCE.createLink();
		Source linkSource = BPELFactory.eINSTANCE.createSource();
		Target linkTarget = BPELFactory.eINSTANCE.createTarget();

		// Check to see if it's a valid name by calling NameConverter
		JavaNCNameConverter converter = new JavaNCNameConverter();
		String convertedName = converter.convertName(NamingUtil.findRegistry(this), link, linkName);
		link.setName(convertedName);
		//link.setName(linkName);
		linkSource.setActivity(source);
		linkTarget.setActivity(target);

		link.getSources().add(linkSource);
		link.getTargets().add(linkTarget);

		return link;

	}


	/**
	 * Re-execute this rule.
	 * <code>inputSet</code> is the source that created the BPEL activity.
	 * If <code>inputSet</code> is not null, then re-set the source and target for the links connecting to and from 
	 * the activity that the inputSet represents before re-execute the child rules.
	 * Otherwise, only re-execute the child rules.
	 * @param inputSet - an BOM InputPinSet or null
	 */
	protected void reExecute(InputPinSet inputSet){

		/* If inputSet is not null, re-execute the links connect to and from this 
		 * activity the inputSet represent */
		//AR Make sure that the child rules are always re-executed.
		if(inputSet != null){

			/* Get the BPEL Activity, activity, from the lookup table that maps an input 
			 * set to an Activity.  This information must have been set when a previous pin 
			 * was connected via setSource/TargetOfLinks(...) */
			if (getMappedActivity(inputSet) != null){

				setTargetOfLinks(inputSet, getMappedActivity(inputSet));					

				OutputPinSet outputSet = null;
				for (Iterator it = inputSet.getOutputPinSet().iterator(); it.hasNext(); ){
					outputSet = (OutputPinSet)it.next();
					/* If setSourceOfLinks(...) has been called at least once before, this will not be null.  
					 * If it hasn't then we shouldn't call it for the first time during re-execution  */
					if (getMappedActivity(outputSet) != null){
						setSourceOfLinks(outputSet, getMappedActivity(outputSet));
					}
				}
			}
		} 
		//always execute child rules
		List childRules = getChildRules();
		Iterator childRulesIterator = childRules.iterator();
		while (childRulesIterator.hasNext()){
			((TransformationRule)childRulesIterator.next()).transformSource2Target();		
		}	
	}

	/**
	 * A recursive method that returns an Logical And expression
	 * @param pins - A set of pins associated with the link
	 * @param set - the pin Set
	 * @return Expression - An Logical And expression
	 */	
	protected Expression createLogicalAnd(List pins, NamedElement set)
	{
		List linkRulesList = new ArrayList();
		Collection sources = new ArrayList();
		Collection linkRules = null;
		if (0 != pins.size())
		{
			//get the list of link rules that are associated with pin
			sources.clear();
			sources.add((Pin)pins.get(0));
			sources.add(set);
			sources.add(((Pin)pins.get(0)).getIncoming());
			linkRules = TransformationUtil.getRulesForSource(ProcessUtil.getProcessDefinitionRule(getContext()), 
					sources, 
					Link.class);			
			linkRulesList.addAll(linkRules);
		}
		if (1 == pins.size() && linkRulesList!=null && !(linkRulesList.isEmpty()))
		{
			return createLogicalOr(linkRulesList);
		}
		else if (pins.size() > 1)
		{
			if (linkRulesList!=null && !(linkRulesList.isEmpty())){
				Expression tempExpr = createLogicalAnd(pins.subList(1, pins.size()), set);
				if (tempExpr!=null) {
					BinaryLogicalBooleanExpression boolExpr = ModelFactory.eINSTANCE.createBinaryLogicalBooleanExpression();						
					boolExpr.setOperator(LogicalBooleanOperator.AND_LITERAL);
					boolExpr.setFirstOperand(createLogicalOr(linkRulesList));
					boolExpr.setSecondOperand(tempExpr);
					return boolExpr;
				}
				else{
					return createLogicalOr(linkRulesList);
				}
			}
			else{
				return createLogicalAnd(pins.subList(1, pins.size()), set);
			}			
		}
		return null;
	}

	protected Expression createLogicalAnd(List linkRulesList)
	{
		if (linkRulesList!=null && !(linkRulesList.isEmpty())){
			if (linkRulesList.size()==1) {
				return createLogicalOr(linkRulesList);				
			}
			BinaryLogicalBooleanExpression boolExpr = ModelFactory.eINSTANCE.createBinaryLogicalBooleanExpression();						
			boolExpr.setOperator(LogicalBooleanOperator.AND_LITERAL);
			boolExpr.setFirstOperand(createLogicalOr(linkRulesList.subList(0,1)));
			//boolExpr.setSecondOperand(createLogicalOr(linkRulesList.subList(1, linkRulesList.size())));
			boolExpr.setSecondOperand(createLogicalAnd(linkRulesList.subList(1, linkRulesList.size())));
			return boolExpr;
		}
		return null;
	}

	/**
	 * A recursive method that returns an Logical Or expression of bpws:getLinkStatus(...)
	 * @param linkRulesList - A list of all the Link rules	 
	 * @return Expression - A top level logial Or expression
	 */	
	protected Expression createLogicalOr(List linkRulesList)
	{
		if (linkRulesList==null || linkRulesList.isEmpty())
			return null;
		Iterator lrIterator= linkRulesList.iterator();
		LinkRule lr = null;
		if (0 != linkRulesList.size())
			lr = (LinkRule)lrIterator.next();

		FunctionExpression functionExpr = ModelFactory.eINSTANCE.createFunctionExpression();
		FunctionDefinition def = ModelFactory.eINSTANCE.createFunctionDefinition();
		def.setFunctionID("com.ibm.btools.te.ilm.heuristics.abstractbpel.getLinkStatus");
		def.setFunctionName("bpws:getLinkStatus");

		FunctionArgument arg1 = ModelFactory.eINSTANCE.createFunctionArgument();			
		StringLiteralExpression strExpr1 = ModelFactory.eINSTANCE.createStringLiteralExpression();
		strExpr1.setStringSymbol(((Link)lr.getTarget().get(0)).getName());
		arg1.setArgumentValue(strExpr1);

		functionExpr.setDefinition(def);
		functionExpr.getArguments().add(arg1);

		/* Recursion Base Case */
		if (1 == linkRulesList.size())
		{	
			return functionExpr;
		}
		else if (linkRulesList.size() > 1)
		{
			BinaryLogicalBooleanExpression boolExpr = ModelFactory.eINSTANCE.createBinaryLogicalBooleanExpression();									
			boolExpr.setFirstOperand(functionExpr);
			boolExpr.setOperator(LogicalBooleanOperator.OR_LITERAL);
			boolExpr.setSecondOperand(createLogicalOr(linkRulesList.subList(1, linkRulesList.size())));	
			return boolExpr;		
		}
		return null;		
	}
} //ConnectableRuleImpl
