/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.wst.wsdl.Message;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.services.InputParameterSet;
import com.ibm.btools.bom.model.services.Operation;
import com.ibm.btools.bom.model.services.OutputParameterSet;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessInterfaceRule;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.MessageRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.WsdlFactory;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;

/**
 * Creates a BPEL Variable for a BOM inputset or outputset or variable.
 * If the two sets are compatible, then if a BPEL Variable has been created for 
 * the opposite set, that BPEL Variable will be reused.  WSDL Message may be
 * created as a side-effect when a the WSDL message is not found for this
 * BPEL Variable
 *
 * Source: BOM InputPinSet or OutputPinSet or Variable
 * Target: BPEL Variable
 *
 */
public class ContainerRuleImpl extends AbstractProcDefRuleImpl implements ContainerRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ContainerRuleImpl() {
		super();
	}

	private static final String REG_KEY = "set-container";
	private TransformationContext context;
	private ProcessInterfaceRule procIntRule;
	// The registry
	private Map containerRegistry;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.CONTAINER_RULE;
	}

		/**	
		 * The source of the ContainerRule is the Input/OutputPinSet, or Input/OutputParameterSet the target is a BPEL Container. 
		 * A registry that keeps pairs set/containers is managed by the Context.
		 * A Container can be shared be 2 connected activities.  
		 * The ContainerRule will do the following:
		 * - Check the registry (set/container) for an entry matching the Set that it has as source.
		 *	If one is found set it as its target and finish.
		 *  Else, it iterates through all the pins in the Set and for each find the incoming (outgoing) edge (depending on the pin type) 
		 *  If the container of the source (target) of this edge (the action at the other end) is NOT the same for all the pins in the Set, 
		 *  creates a new Container for this set. Register the container with the registry - set/container
		 *  Else, if all the pins belong to the same container, for the action found at the other end, get all the output (input) sets and find the messages created by these (check in the registry). 
		 *  If no matching Message is found create Container and then register the set with the container.
		 *  If a matching  Message is found, check the registry (set/container) for a container. If none available, create the Container and register it,
		 *  Also register the Set found as match with the registry.
		 *  If a container is found, set is as the target of this rule.
		 */
		public boolean transformSource2Target() {
			LoggingUtil.traceEntry(this, "transformSource2Target");

			if (isComplete()) {
				LoggingUtil.traceExit(this, "transformSource2Target", "already completed");
				return true;				
			}
			if (getSource() == null || getSource().isEmpty()) {
				LoggingUtil.traceExit(this, "transformSource2Target", "no sources");
				return false;
			}
				
			NamedElement srcObj=(NamedElement)source.get(0);;
			NamedElement origSrcObj = srcObj;
			BPELVariable container=null;
			Message message;

			// Initialization
			context = getContext();
		//	Action owner = null;
		//	TODO AR the owner of the source can be either an action, if the source is an Input/OutputPinSet or a Variable, 
		//	or an Activity - a services.Activity if the source is an Input/OutputParameterSet 
			NamedElement owner = null;
			Operation operation = null;
			if (srcObj instanceof InputPinSet){
				owner = ((InputPinSet)srcObj).getAction();
			} else if (srcObj instanceof OutputPinSet){
				owner = ((OutputPinSet)srcObj).getAction();
			} else if (srcObj instanceof com.ibm.btools.bom.model.processes.activities.Variable){
				owner = ((com.ibm.btools.bom.model.processes.activities.Variable)srcObj).getScope();
			}else if (srcObj instanceof InputParameterSet){
				owner = ((InputParameterSet)srcObj).getBehavior();
			}else if (srcObj instanceof OutputParameterSet){
				owner = ((OutputParameterSet)srcObj).getBehavior();;
			}
						
			// Check if a containerRule already exists
			// This is the case when an outputset's container of a Merge
			// already created
			TransformationRule containerRule = 
				TransformationUtil.getRuleForSource(
						ProcessUtil.getProcessDefinitionRule(context),
						ContainerRule.class,
						origSrcObj,
						BPELVariable.class);
						
			// If a containerRule already exist, set the target to the same container
			if (containerRule != null)
			{
				getTarget().addAll(containerRule.getTarget());
				LoggingUtil.traceExit(this, "transformSource2Target", "containerRule already exists");
				return true;
			}
			
			// Otherwise, create a new container
			// First, handle the special case if the source of this rule is
			// an inputset/outputset of a BOM Variable.  Note that we don't have to 
			// treat BOM Merges specially... we want to reuse containers as much
			// as possible
			if (srcObj instanceof com.ibm.btools.bom.model.processes.activities.Variable) 
			{
				//procIntRule = (ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(context,owner);
				procIntRule = (ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(context);
				message = createMessage(srcObj);
				container = createContainer(srcObj,message);
			} else {
				
//				Handle special case for CBA
//				NamedElement action action = findToBeProcessedAction(owner);
				// Handle special case for CBA
				NamedElement action = findToBeProcessedAction(owner);

				procIntRule = createProcessInterfaceRule(action);
//				if (action != owner){
//					srcObj = getCorrespondingSet(srcObj);
//				}
				if (owner != null && action != owner){
					srcObj = getCorrespondingSet(srcObj);
				}	
				// first check if a container is not already registered.
				//container=getContainerFromRegistry(srcObj);
				
				//if (container == null){
					// check if the a compatible Container was already created
					//ie. this can be the container of a "neighbour" activity
					NamedElement compatibleSet = BomHelper.getInstance().isNextSetCompatible(srcObj, getContext()); 
					if (compatibleSet != null) {
						/* The current set is compatible with another set.  If a container already exists for the
						 * compatible set, use the same container and update the sources of the PartRules of the 
						 * container's Message to register that the pins in our current set are associated 
						 * with those same parts.	 */
						containerRule = (ContainerRule)TransformationUtil.getRuleForSource(
								ProcessUtil.getProcessDefinitionRule(context),
								ContainerRule.class,
								compatibleSet,
								BPELVariable.class);

						if (containerRule != null) {
							container = (BPELVariable)containerRule.getTarget().get(0);
							//in this case we reused the container for the new InputSet. We should register the 
							//new user defined technicalAttributes of the compatible set... 
							BomHelper.getInstance().registerContainerWithSet(container, (PinSet)origSrcObj);
							/* Register the new pins with the old parts */
							BomHelper.getInstance().registerPartsWithPins(this,(PinSet)compatibleSet, (PinSet)origSrcObj );
							//registerPartsWithPins((PinSet)compatibleSet, (PinSet)origSrcObj);
						}
					}

					//try to further reuse the container based on the user preference
					//if(container == null && BpelOptionsUtil.reuseBPELVariableForInputAndOutput(BomUtils.getContainingProcess(origSrcObj))){
					if(container == null && reuseBPELVariableForInputAndOutput(origSrcObj)){
						//container = BomHelper.getInstance().getContainerForCompatibleCorrelatedSet(this, srcObj, getContext());
						container = BomHelper.getInstance().getContainerForCompatibleCorrelatedSet(this, srcObj, getContext(), origSrcObj);
						if (container != null && origSrcObj != srcObj) {
							BomHelper.getInstance().registerPartsWithPins(this, srcObj, origSrcObj);
						}
						
					}
					//if a compatible container has already been created try to use it. 
					if(container == null ){
						//try to find a compatible container that has already been created
					//	container = TechnologyProfileHelper.getCompatibleContainerForSet(srcObj, this, NamingUtil.findRegistry(this));
					}
					if (container == null) {
						/* If we find a message with this next call, its because one was previously 
						 * created for the PinSet.  Therefore, we don't have to register the Parts
						 * with the pins in the set, since they are already registered.  */
						message = getMessageForSource(srcObj);
						if (message == null){
							/* If no message is found, create a new one. This is the case when 
							 * an Assign Rule is created first*/
							
							/* When we create a message from the source object, it will associate
							 * the parts with the pins automatically */
							message = createMessage(srcObj);
							
						}
						container = createContainer(origSrcObj,message);
						if (origSrcObj != srcObj) {
							BomHelper.getInstance().registerPartsWithPins(this, srcObj, origSrcObj);
						}
				//		registerContainer(origSrcObj,container);
						// this is optimizing performance
						// if a compatibleSet is found but no container is found
						// for it, we will register a container for the compatibleSet
						// such that it doesn't have to go through the same thing 
						// again
						//if (compatibleSet != null)
						//	registerContainer(compatibleSet,container);
					}
				//}
			}
		getTarget().add(container);
			
		executeHandlers();
		
		setComplete(true);	
		LoggingUtil.traceExit(this, "transformSource2Target");			
		return isComplete();
	}
	
	private void registerContainer(EObject keyObj, BPELVariable container)
	{
		if (containerRegistry==null)
			containerRegistry = (Map)context.get(REG_KEY);
		containerRegistry.put(keyObj,container);		
	}
	
	private BPELVariable getContainerFromRegistry(EObject keyObj)
	{
		if (containerRegistry==null)
			containerRegistry = (Map)context.get(REG_KEY);
		if (containerRegistry !=  null){
			return (BPELVariable)containerRegistry.get(keyObj);
		} else {
			containerRegistry = new HashMap();
			//register the registry with the Context
			context.put(REG_KEY, containerRegistry);
			return null;
		}
	}

	/*
	 * Creates a MessageRule and set it as a child rule to the ProcessInterfaceRule
	 */
	private Message createMessage(EObject src)
	{
		MessageRule msgRule = WsdlFactory.eINSTANCE.createMessageRule();
		msgRule.getSource().add(src);
		procIntRule.getChildRules().add(msgRule);
		if (msgRule.transformSource2Target())
		{
			return (Message)msgRule.getTarget().get(0);
		}
		return null;
		/*
		 * TODO:
		 * Need to update MessageRule:
		 * Will first check if a MessageRule is created for the source.  
		 * If yes, set the target of this rule to the target of the 
		 * MessageRule found.
		 */
	}

	/*
	 * Returns the Message that was created for the source of this Rule. 
	 * @return
	 */
	private Message getMessageForSource(NamedElement srcObj){
		Message message = null;

		if(procIntRule != null){
			TransformationRule messageRule = TransformationUtil.getRuleForSource(procIntRule,srcObj,Message.class);
			if(messageRule != null){
				message = (Message)messageRule.getTarget().get(0);	
			}
		}
		return message;
	}
	 
	/*
	 * Creates a new Container. The name of the Container is based on the source name
	 * (input/outputSet)
	 * TODO check the name of the container.
	 */
	private BPELVariable createContainer(NamedElement srcObj, Message msg){
		BPELVariable newContainer = BPELFactory.eINSTANCE.createBPELVariable();
		//newContainer.setName(msg.getQName().getLocalPart() + "Container");
		/* Convert the name of the container to a valid name for a Java identifier, since it 
		 * could be used as a methodName if ILM is our target  */
		//Check to see if it's a valid name by calling NameConverter
		AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.BPEL_VARIABLE_NAME);
		valueProvider.setContext(this.getContext());
		String convertedName = (String)valueProvider.getAttributeValueForType(newContainer, srcObj, AttributeNameConstants.BPEL_VARIABLE_NAME, NamingUtil.findRegistry(this));
//		 JavaNCNameConverter converter = new JavaNCNameConverter();
//		 String convertedName = converter.convertName(NamingUtil.findRegistry(this), newContainer, BomUtils.getCanonicalName(srcObj) + "Variable");

		newContainer.setName(convertedName);
		 //newContainer.setName(AbstractbpelUtil.convertString(BomUtils.getCanonicalName(srcObj) + "Variable"));
		newContainer.setMessageType(msg);
// Ray: Feb 05,04 - 5.1 WSADIE doesn't support DisplayName on Variable		
//		DisplayName description = BPELPlusFactory.eINSTANCE.createDisplayName();
//		description.setText(newContainer.getName());
//		newContainer.getEExtensibilityElements().add(description);
		
		return newContainer;
	}
	
	private boolean reuseBPELVariableForInputAndOutput(NamedElement origSrcObj){
		return true;
		
//		EObject sourceObject = ProcessUtil.getProcessDefinition(context);
//		AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.BPEL_PROCESS_USE_SAME_INPUT_OUTPUT_VARIABLE);
//		valueProvider.setContext(this.getContext());
//		Boolean reuseVariable = (Boolean)valueProvider.getAttributeValueForType(BomUtils.getContainingProcess(origSrcObj), AttributeNameConstants.BPEL_PROCESS_USE_SAME_INPUT_OUTPUT_VARIABLE);
//		
//		return reuseVariable.booleanValue();
		
	}

} //ContainerRuleImpl
