/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.actions.Decision;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.FlowFinalNode;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.activities.TerminationNode;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AssignRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ExclusiveBranchRule;




import com.ibm.btools.te.ilm.heuristics.abstractbpel.LinkRule;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BomWalker;
import com.ibm.btools.te.ilm.heuristics.helper.CBPELUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.Flow;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.PartnerLink;
import com.ibm.wbit.bpel.Variable;

/**
 * <!-- begin-user-doc -->
 * Source: BOM InputPinSet
 * Target: BPEL flow
 * Maps all elements in the exclusive branch region marked by the Input Set and exclusive merge point.
 * The action of exclusive merge point also mapped by this rule. 
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class ExclusiveBranchRuleImpl extends ProcessWalkingRuleImpl implements ExclusiveBranchRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/* attribute indicates whether it is first time execution of this rule, default to true */
	private boolean firstTimeExec = true;
		
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ExclusiveBranchRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.EXCLUSIVE_BRANCH_RULE;
	}

	/**
	 * Maps all elements in the exclusive branch region marked by the Input Set and exclusive merge point.
	 * The action of exclusive merge point also mapped by this rule. 
	 * 	  
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");
		
	 /*
	 *  1. create BPEL flow and set as target of this rule.
	 * 2. for each output set correlated with the input set, check if it is concurrent branch
	 * 		if it is conrrent branch, create ConcurrentBranchRule and map it.
	 * 		if not, create links representing the outgoing edge from the pins in the output set.
	 * 3. for each input set reachable by each output set, 
	 *    step through it, create child rule for each element encountered and execute child rules,
	 *    until reached the merge point including the merge point as well or util reached the end of Structured Activity Node.
	 * 4. consume and propagate child rules' targets.
	 */
		
		
	  //assertion
	  if (getSource() == null || getSource().isEmpty()) {
		LoggingUtil.traceExit(this, "transformSource2Target", "no source");
		return false;
	  }
  
	  //source of this rule is one input set
		InputPinSet inputSet = (InputPinSet)getSource().get(0);
		boolean isParallelFlowEntry = false;
		if(inputSet.getAction() instanceof Decision && ((Decision)inputSet.getAction()).getMultiplePaths().equals(Boolean.TRUE)){
			isParallelFlowEntry = true;
		}
	  //check if this is first time execute this rule
	   if(!firstTimeExec){
		  reExecute(inputSet);
		  LoggingUtil.traceExit(this, "transformSource2Target", "re-execute");		   
		  return true;
	   }
		
		firstTimeExec = false;
		
		//create BPEL flow
		Flow flow = BPELFactory.eINSTANCE.createFlow();
		//defect 139867. generate name for the flow
		String convertedName = NamingUtil.getFlowName(flow, this);
		flow.setName(convertedName);
		createDisplayName(flow, convertedName);
		getTarget().add(flow);		
		
		//for each ouput set, check if need to create Concurrent branch rule
		List osList = inputSet.getOutputPinSet();
		// Ray: if the branch leads to no OutputPinSet (no association to any OutputPinSet)
		// then assume the branch points lead to all OutputPinSet of the containing action
		if (osList.size()==0) {
			//osList = inputSet.getAction().getOutputPinSet();
			osList = BomUtils.getCorrelatedPinSets(inputSet);
		}
		
		OutputPinSet nextOS;
		boolean isCBranch = false;				
		for(int i= 0, count = osList.size(); i<count; i++){
			nextOS = (OutputPinSet)osList.get(i);
			//create concurrent branch point for nextOS if any
			isCBranch = createCBranchRuleIfAny(nextOS);
			if(!isCBranch){
				//create links for each pins in the output set
				createLinksForOutputPinSet(nextOS);
			}		
					
		}
		
		BomWalker bw = ProcessUtil.getBomWalker(getContext());
		NamedElement mergePoint = BomHelper.getInstance().findExclusiveMergePoint(inputSet);
		Action mergePointAction = null;
		if(mergePoint instanceof OutputPinSet){
			mergePointAction = ((OutputPinSet)mergePoint).getAction();
		} else if(mergePoint instanceof StructuredActivityNode) {
			mergePointAction = (Action)mergePoint;
		}
		
		List variablesOnCurrentPath = new LinkedList();
		initializePathsVariablesForBranchRegion();
		
		List nextISList;
		InputPinSet nextIS;
		//!!main logic of this method is here
		//walk through each of the output set, create and execute child rules
		for (int i = 0, count = osList.size(); i < count; i++) {
			nextOS = (OutputPinSet) osList.get(i);
			nextISList = bw.getNext(nextOS, BomWalker.INPUT_SET, true);
			
			if (nextISList == null || nextISList.isEmpty()) {
				//we have reached end of SAN ??? what to do??
				//need do the connection for the output set of SAN to internal activities of SAN?
				// The answer is no. Process rule takes case of doing this. 
				//doOutputPinSetConnection(flow, true);	
				continue;
			}							

			// TODO: watch for if this loop is needed... 
			//       ... It is believed there will be only one inputSet returned		
			for (int j = 0, nextISListCount = nextISList.size(); j < nextISListCount; j++) { 
				nextIS = (InputPinSet) nextISList.get(j);
				
				do {
					if (nextIS.getAction() == mergePointAction) {
						// we have reached the merge point, but we need to include the 
						// merge point inside the exclusive branch flow, so execute 
						// the do..while loop one more time				  	  	
						stepThroughBOM(nextIS);
						nextIS = null;
						
					} else { // nextIS not on merge point				  	  
						nextIS = stepThroughBOM(nextIS);
						if (nextIS != null) { // this means that we just created a child rule
							// we must continue
							// bw.getNext(...) return list of inputSets. Is it safe just to use 
							// the first element? The answer is yes, since we always skip branch 
							// point, we should only have one InputPinSet back
							List nextInputPinSets =	bw.getNext(nextIS, BomWalker.INPUT_SET, true);
							if (nextInputPinSets == null || nextInputPinSets.isEmpty()) {
								nextIS = null; //reached end of SAN
								// Do we need to connect the output set of the SAN to 
								// the internal activities?
								// No, since the ProcessRule will do this instead. 
								// doOutputPinSetConnection(flow, true);
							} else {
								nextIS = (InputPinSet) nextInputPinSets.get(0);
								// must pass the output set to BomHelper otherwise it doesn't work
								// Ray: need to handle input set with zero or more than one correlated outputset ... yikes!
								boolean isInBranchRegion=false;
								List outputSets = BomUtils.getCorrelatedPinSets(nextIS);
								if (outputSets==null || outputSets.isEmpty()) {
									isInBranchRegion=true;
								} else 
								if (outputSets.size()==1) {
									//isInBranchRegion=BomHelper.getInstance().isInBranchRegion(inputSet, (OutputPinSet) nextIS.getOutputPinSet().get(0));
									isInBranchRegion=BomHelper.getInstance().isInBranchRegion(inputSet, (OutputPinSet) outputSets.get(0));
								}
								//if (!BomHelper.getInstance().isInBranchRegion(inputSet, (OutputPinSet) nextIS.getOutputPinSet().get(0))){
								if (!isInBranchRegion) {
									// Ray: Additional checking - if nextIS is an EBP but is contained within the current EB region, we
									// shouldn't skip it
									//defect 76704.AR added the check for size>1. This should be a EBP - with the exception of an unconnected Activity.
									if (BomHelper.getInstance().isEBranch(nextIS) || outputSets.size() > 1) {
										NamedElement emp = BomHelper.getInstance().findExclusiveMergePoint(nextIS);
//										if (!BomHelper.getInstance().isInBranchRegion(inputSet,emp)) {
//											nextIS=null;
//										}
										//AR defect 89040.. if the emp is a CBP jump over the region and lookup the OS for the CMP . 
										// apply the logic currently in BomWalker. 
										if (emp instanceof OutputPinSet && BomHelper.getInstance().isCBranch((OutputPinSet)emp)){
											NamedElement cmp = BomHelper.getInstance().findConcurrentMergePoint((OutputPinSet)emp);
											if (cmp instanceof InputPinSet){
												List nextPinSet = ProcessUtil.getBomWalker(getContext()).getNext((InputPinSet)cmp, BomWalker.OUTPUT_SET);
												if(nextPinSet != null && !nextPinSet.isEmpty()) cmp = (OutputPinSet)nextPinSet.get(0);
											}
											if (!BomHelper.getInstance().isInBranchRegion(inputSet,cmp)) {
												nextIS=null;
											}
										}else{
											if (!BomHelper.getInstance().isInBranchRegion(inputSet,emp)) {
												nextIS=null;
											}
										}
									} else {
										// we stepped over merge point, so should stop it here
										nextIS = null;
									}

								}
							}

						} else { 
							// stepThroughBOM didn't create child rule, 
							// so we should stop tracing this path and do nothing
						}
	
					}
				} while (nextIS != null);
			}
			if(isParallelFlowEntry){
				resetPathRegistryInBranchPoint(inputSet, null);
			}
		}
		if(isParallelFlowEntry){
			resetPathsRegistry();
		}
		processChildTargets(flow);
		
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}
	
	/**
	 * Process targets produced by child rules.
	 * Links, Containers and Partners are propagated to parent.
	 * Activities are contained by this exclusive branch flow. 
	 * Empty flows are eliminated.
	 * 
	 */
	private void processChildTargets(Flow flow){
			List childRules = getChildRules();
			TransformationRule nextRule = null;
			List targetList= null;
			Object target;
			
			for(int i=0, count=childRules.size(); i<count; i++){
				nextRule = (TransformationRule)childRules.get(i);
				targetList = nextRule.getTarget();
				for (int j=0, targetCount = targetList.size(); j< targetCount; j++){
					target = targetList.get(j);
					if(target instanceof Link || target instanceof Variable || target instanceof PartnerLink){
						getTarget().add(target); //propagate all these types to parent	
					}
					else if (target instanceof Activity){		
						if(target instanceof Flow && ((Flow)target).getActivities().isEmpty()){
							//do nothing since we don't want empty flow 						
						}else				
							//add all other activity to this flow
							flow.getActivities().add(target);
					} else{
						//TODO: what other type could be?
						getTarget().add(target); //propagate all other unknown type to parent
					}
				}
			}
	
		}	
	
	/**
	 * Creates AssignRule or LinkRule for each pin in the output set. 
	 * @param outputSet
	 */
	private void createLinksForOutputPinSet(OutputPinSet outputSet){
		//pin in the output set is connected to an input pin, 
		//it is responsibility of this rule to create a BPEL representaiton of this connection.
		//check each pin in the output set	  
		Pin pin;
		List pinList = BomUtils.getPinsInSet(outputSet);
	//	boolean assignCallOperationActionTarget = false;

//		if (!hasOutGoingEdgeOnAnyPins(outputSet.getOutputControlPin()) && !hasOutGoingEdgeOnAnyPins(outputSet.getOutputObjectPin())) {
//			// Ray: Need to connect empty outputPinSet or outputPinSet with pins with no outgoing edge
//			// to the default outputCriteria
//			LinkRule linkRule = AbstractbpelFactory.eINSTANCE.createLinkRule();
//			linkRule.getSource().add(outputSet);
//			OutputPinSet outPinSet = (OutputPinSet)outputSet.getAction().getInStructuredNode().getOutputPinSet().get(0);
//			linkRule.getSource().add(outPinSet);
//			getChildRules().add(linkRule);
//			linkRule.transformSource2Target();								
//		}
		//boolean doAssign = (BomHelper.getInstance().isNextSetCompatible(outputSet, getContext())== null? true : false);
		//TODO AR new
		boolean doAssign = false;
		for(int i= 0, count= pinList.size(); i<count; i++){
			pin = (Pin)pinList.get(i);
			 
			/* Ignore pins that have no outgoing connection */
//			if (pin.getOutgoing() == null){
//				continue;
//			}
			//TODO AR 14.10 remove from code
			/* If the current edge is connected to the 'target' pin of a CallOperationAction, just
			 * create a link, even though we may have been told to create an <assign> */
//			if(pin.getOutgoing().getTarget() instanceof InputObjectPin){
//				if(((InputObjectPin)pin.getOutgoing().getTarget()).getAction() instanceof CallOperationAction){
//					CallOperationAction coa = (CallOperationAction)((InputObjectPin)pin.getOutgoing().getTarget()).getAction();
//					if(coa.getTarget() != null){
//						if(coa.getTarget() == ((InputObjectPin)pin.getOutgoing().getTarget())){
//							assignCallOperationActionTarget = true;
//						}
//					}
//				} 
//			} 
			if (!outputSet.getIsException() && i == 0 && pinsOnlyNotConnectedOrFlowFinalNode(pinList)){
				// Ray: Need to connect empty outputPinSet or outputPinSet with pins with no outgoing edge
				// to the default outputCriteria
				LinkRule linkRule = AbstractbpelFactory.eINSTANCE.createLinkRule();
				linkRule.getSource().add(outputSet);
				OutputPinSet outPinSet = (OutputPinSet)outputSet.getAction().getInStructuredNode().getOutputPinSet().get(0);
				linkRule.getSource().add(outPinSet);
				getChildRules().add(linkRule);
				linkRule.transformSource2Target();					
			} else if (pin.getOutgoing() == null || pin.getOutgoing().getTarget() instanceof FlowFinalNode){		 
			} else if (pin.getOutgoing().getTarget() instanceof TerminationNode){
				//LinkRule linkRule = createTerminationLinkRule(pin);
				LinkRule linkRule = createTerminationLinkRule(pin,null);
				getChildRules().add(linkRule);
				linkRule.transformSource2Target();

			} else if (pin.getOutgoing().getTarget() instanceof Pin) {
				//if ((pin instanceof ObjectPin) && doAssign && !assignCallOperationActionTarget){
				if ((pin instanceof ObjectPin) && doAssign){
					NamedElement targetSet = BomUtils.findTargetSet(pin.getOutgoing()); // Ray - try to avoid duplicate assigns to be created now that we support multiple outputSets
					if (needAssignRule(pin,outputSet,targetSet)){
						AssignRule ar = AbstractbpelFactory.eINSTANCE.createAssignRule();
						ar.getSource().add(pin.getOutgoing()); //edge
						ar.getSource().add(outputSet); //output set
						ar.getSource().add(BomUtils.findTargetSet(pin.getOutgoing())); //input set on the target side of the edge
						//add Assign rule as child rule
						getChildRules().add(ar);
						ar.transformSource2Target();
					}			  
				}else {//create link rule for control pin and object pin in not compatible set
					LinkRule lr = AbstractbpelFactory.eINSTANCE.createLinkRule();
					lr.getSource().add(pin.getOutgoing()); //set the outgoing edge as the source
					lr.getSource().add(pin); //set the output pin as the source
					lr.getSource().add(pin.getOutgoing().getTarget()); //input pin on the target side of the edge
					lr.getSource().add(outputSet); //set on source side of the edge
					
					/* A pin may or may not be connected to another input set. */
					NamedElement nextSet = BomUtils.findTargetSet(pin.getOutgoing());
					if (nextSet != null){
						lr.getSource().add(nextSet); //input set on target side of the edge
					}
					//add link rule as child rule
					getChildRules().add(lr);
					lr.transformSource2Target(); 
					 
				}
			}					 	 	
		}	
	}
	
//	private boolean hasOutGoingEdgeOnAnyPins(List pins) {
//		Pin pin;
//		for (Iterator it=pins.iterator();it.hasNext();) {
//			pin = (Pin)it.next();
//			if (pin.getOutgoing() != null) {
//				if (!(pin.getOutgoing().getTarget() instanceof FlowFinalNode)) {
//					return true;
//				}
//			}
//		}
//		return false;
//	}
	
	/**
	 * Creates and executes the ConcurrentBranchRule if the outputSet is a concurrent branch.
	 * @param outputSet  the output set of the exclusive branch point.
	 * @return boolean - true if created concurrentBranchRule
	 */
	private boolean createCBranchRuleIfAny(OutputPinSet outputSet){		
		if(CBPELUtil.isFaultSetNoConnections(outputSet)){
			return false;
		}
		TransformationRule rule;
	
		 //check if output is Concurrent branch
		if(BomHelper.getInstance().isCBranch(outputSet)){
			rule = AbstractbpelFactory.eINSTANCE.createConcurrentBranchRule();
			rule.getSource().add(outputSet);
			getChildRules().add(rule);
			rule.transformSource2Target();			
			return true;	 
		}else{
			//do nothing
			return false;
		}
	}	
	
	
	private List getPathAddedVariables(List currentPathVariablesSnapshot, List variablesOnCurrentPath){
		//the actual variables on CurrentPath
	//	List variablesOnCurrentPath = ProcessUtil.getCurrentPathParallelFlowsRegisteredVariables(this.getContext());
		
//		int initialSize = currentPathVariablesSnapshot.size();
//		int finalSize = variablesOnCurrentPath.size();
		List addedVariables = new LinkedList();
//		if(finalSize > initialSize){
//			for(int j= initialSize; j< finalSize; j++){
//				addedVariables.add(variablesOnCurrentPath.get(j));
//			}
//		}
		if(variablesOnCurrentPath != null){
			Iterator it = variablesOnCurrentPath.iterator();
			while(it.hasNext()){
				Object var = it.next();
				if(!currentPathVariablesSnapshot.contains(var) && !addedVariables.contains(var)){
					addedVariables.add(var);
				}
			}
		}

		return addedVariables;
		
	}

	private boolean pinsOnlyNotConnectedOrFlowFinalNode(List pinList) {
		if (pinList == null || pinList.isEmpty()) {
			return false;
		}
		
		boolean onlyNCOrFlowFinal = true;
		
		Iterator it = pinList.iterator();
		Pin pin = null;
		while (it.hasNext()) {
			pin = (Pin)it.next();
			
			if (!(pin.getOutgoing() == null || pin.getOutgoing().getTarget() instanceof FlowFinalNode)) {
				onlyNCOrFlowFinal = false;
				break;
			}
		}
		
		return onlyNCOrFlowFinal;
	}
} //ExclusiveBranchRuleImpl
