/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/



import java.util.Iterator;

import org.eclipse.emf.ecore.EClass;

import com.ibm.btools.bom.model.time.RecurringTimeIntervals;
import com.ibm.btools.bom.model.time.TimeIntervals;
import com.ibm.btools.te.framework.impl.TransformationRuleImpl;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.HeuristicsFactory;
import com.ibm.btools.te.ilm.heuristics.HeuristicsTransformation;
import com.ibm.btools.te.ilm.heuristics.TimeIntervalsRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.HTBusinessCalendarRule;
import com.ibm.btools.te.ilm.heuristics.util.TimeConstants;
import com.ibm.btools.te.ilm.heuristics.util.TimeIntervalsUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.util.exception.BTRuntimeException;
import com.ibm.wbit.businesscalendar.CalFactory;
import com.ibm.wbit.businesscalendar.ICalendar;
import com.ibm.wbit.businesscalendar.Vcalendar;
import com.ibm.wbit.businesscalendar.util.CalendarUtil;
import com.ibm.wbit.tel.DocumentRoot;

/**
 * Transform a TimeIntervals attached for HumanTask, and create Business Calendar (ICalendar).
 * Sources: TimeIntervals
 * 			HumanTask 
 * Target: ICalendar
 */
public class HTBusinessCalendarRuleImpl extends TransformationRuleImpl implements HTBusinessCalendarRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected HTBusinessCalendarRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.HT_BUSINESS_CALENDAR_RULE;
	}

	
	/**
	 * SOURCE: BOM TimeIntervals
	 * 	       BOM HumanTask (Action)
	 * TARGET: Business Calendar
	 * Maps BOM TimeIntervals to BusinessCalendar as defined in schema.
	 * @return boolean
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target()	{	
		LoggingUtil.traceEntry(this, "transformSource2Target");

		//assertion
		if (getSource() == null || getSource().size() != 2 ) {
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
			return false;			
		}
			 
		// TimeIntervals contain a list of RecurringTimeIntervals set for HumanTask
		TimeIntervals timeIntervals = (TimeIntervals)getSource().get(0);
		String taskName = ((DocumentRoot)getSource().get(1)).getTask().getName();

		if (timeIntervals.getRecurringTimeIntervals() == null || 
				timeIntervals.getRecurringTimeIntervals().isEmpty()) {
			LoggingUtil.traceExit(this, "transformSource2Target", "no content for calendars");
			return false;			
		}

		ICalendar businessCalendar = CalFactory.eINSTANCE.createICalendar();
		//set iCal name for resource to task name
		businessCalendar.setName(taskName + TimeConstants.CALENDAR_SUFFIX);
		//NOTE: set the namespace based on the HumanTask namespace - done in parent rule
		Vcalendar busCalendar = createVCalendar(taskName);
		businessCalendar.setVcalendar(busCalendar);

		Iterator it = timeIntervals.getRecurringTimeIntervals().iterator();
		RecurringTimeIntervals rti = null;
		TimeIntervalsRule tRule = null;
		while (it.hasNext()) {
			rti = (RecurringTimeIntervals) it.next();
			tRule = TimeIntervalsUtil.findTimeIntervalsRule(rti, getRoot());
			if (tRule == null) {
				tRule = HeuristicsFactory.eINSTANCE.createTimeIntervalsRule();
				tRule.getSource().add(rti);
				getParentRule().getChildRules().add(tRule);		// if no parent, add to the parent rule
				tRule.transformSource2Target();
				
				if (!(tRule.getParentRule() instanceof HeuristicsTransformation) && tRule.getTarget() != null && !tRule.getTarget().isEmpty()) {
					tRule.getParentRule().getTarget().addAll(tRule.getTarget());
				}
			} else if (tRule.getTarget() == null || tRule.getTarget().isEmpty()) {
				tRule.transformSource2Target();
			}

			// retrieve from target
			if (tRule.getTarget() != null && !tRule.getTarget().isEmpty() && 
					tRule.getTarget().get(0) instanceof ICalendar) {
				ICalendar iCal = (ICalendar)tRule.getTarget().get(0);
				String tns = iCal.getTargetNamespace();
				String name = iCal.getName();
				busCalendar.getVinclude().add(tns + TimeConstants.COLON + name);
			} else { 	// no target
				String params[] = {rti.getName()};
				LoggingUtil.logError(MessageKeys.TE_FAILED_TO_CREATE_TIMETABLE, params, null);
				BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_FAILED_TO_CREATE_TIMETABLE));
				throw runtimeEx;
			}
		}	

		//set timetable associated with HT as a target
		getTarget().add(businessCalendar);
	    
		setComplete(true);

		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}

	// the top level TimeIntervals from BOM's HT reference
	private Vcalendar createVCalendar (String taskName) {
		Vcalendar busCalendar = CalFactory.eINSTANCE.createVcalendar();
		busCalendar.setProdid(TimeConstants.PRODUCT_ID);
		// set the name of the resource calendar to the task's name
		busCalendar.setXCalname(taskName + TimeConstants.CALENDAR_SUFFIX);
		busCalendar.setVersion(CalendarUtil.VERSION_1000);

		return busCalendar;
	}	
} //HTBusinessCalendarRuleImpl