/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.InstanceSpecification;
import com.ibm.btools.bom.model.artifacts.InstanceValue;
import com.ibm.btools.bom.model.artifacts.LiteralDuration;
import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.LiteralReal;
import com.ibm.btools.bom.model.artifacts.LiteralSpecification;
import com.ibm.btools.bom.model.artifacts.LiteralString;
import com.ibm.btools.bom.model.artifacts.LiteralTime;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.PrimitiveType;
import com.ibm.btools.bom.model.artifacts.Property;
import com.ibm.btools.bom.model.artifacts.Slot;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.actions.Map;
import com.ibm.btools.bom.model.processes.actions.OutputObjectPinMap;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.expression.command.CreateModelPathExpressionEXPCmd;
import com.ibm.btools.expression.function.CoreFunctions;
import com.ibm.btools.expression.model.ArrayIndexStep;
import com.ibm.btools.expression.model.Expression;
import com.ibm.btools.expression.model.FunctionArgument;
import com.ibm.btools.expression.model.FunctionExpression;
import com.ibm.btools.expression.model.ModelFactory;
import com.ibm.btools.expression.model.ModelPathExpression;
import com.ibm.btools.expression.model.ReferenceStep;
import com.ibm.btools.expression.model.StringLiteralExpression;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ConcurrentBranchRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.MapRule;
import com.ibm.btools.te.ilm.heuristics.bpelp.util.ConcatFunctionSerializer;
import com.ibm.btools.te.ilm.heuristics.bpelp.util.SwapOrganizer;
import com.ibm.btools.te.ilm.heuristics.br.util.BrUtil;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.model.abstractbpel.Alternative;
import com.ibm.btools.te.ilm.model.abstractbpel.AlternativeActivity;
import com.ibm.btools.te.ilm.model.sa.ProcessFlowType;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Assign;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Copy;
import com.ibm.wbit.bpel.From;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Query;
import com.ibm.wbit.bpel.To;
import com.ibm.wbit.bpel.Variable;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.DisplayName;
import com.ibm.wbit.bpelpp.JavaCode;
import com.ibm.wbit.bpelpp.JavaScriptActivity;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Map Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class MapRuleImpl extends ConnectableRuleImpl implements MapRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	public static final String QUERY_LANGUAGE = "http://www.w3.org/TR/1999/REC-xpath-19991116";
	private static final String primitiveString = "String";
	private static final String primitiveBoolean = "Boolean";
	private static final String primitiveDataTime = "DateTime";
	private boolean fFirstTimeExecuting=true;
	
	private Activity fMapActivity = null;
	
	private java.util.Map<EObject, EObject> copyMap = new HashMap<EObject, EObject>();
	private java.util.Map<BPELVariable, BPELVariable> var2temp = new HashMap<BPELVariable, BPELVariable>();
	private java.util.Map<ObjectPin, BPELVariable> pin2var = new HashMap<ObjectPin, BPELVariable>();
	private List<Copy> copies4TempInit = new ArrayList<Copy>();

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected MapRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.MAP_RULE;
	}

	/**
	 * Calls the method to generate the mapped representation of the input set, and connects
	 * the result via a BPEL link to the target activity, if required
	 * @param pinSet The input set to map
	 * @param target The target activity to connect to the input set representation
	 */
	private void connectInputSet(InputPinSet pinSet, Activity target){
		/* Connect the incoming and outgoing links for this action */
		List isResultList = mapInputPinSet(pinSet);
		Activity inputSetActivity = null;
		if (isResultList.isEmpty() || !(isResultList.get(0) instanceof Activity)){
			inputSetActivity = target;
		} else {
			inputSetActivity = (Activity)isResultList.get(0);
				
			/* Link the new input set activity to the sequence */
			Link link = linkActivities(inputSetActivity, 
									   target, 
									   BomUtils.getCanonicalName(pinSet) + "_to_" + pinSet.getAction().getName());
				
			isResultList.add(link);
				
			/* Propagate results from the input set mapping */
			getTarget().addAll(isResultList);
		}

		setTargetOfLinks(pinSet, inputSetActivity);	
	}

	/**
	 * Calls the method to generate the mapped representation of the output set, and connects
	 * the source to the result via a BPEL link, if required.
	 * 
	 * @param pinSet The output pin set to map
	 * @param source The BPEL activity to connect the output set reprsentation to
	 */	
	private void connectOutputSet(OutputPinSet pinSet, Activity source){
		List osResultList = mapOutputPinSet(pinSet);
		Activity outputSetActivity = null;
		if (osResultList.isEmpty() || !(osResultList.get(0) instanceof Activity)){
			outputSetActivity = source;
		} else {
			outputSetActivity = (Activity)osResultList.get(0);
				
			/* Link the new input set activity to the sequence */
			Link link = linkActivities(source, 
									   outputSetActivity, 
									   pinSet.getAction().getName() + "_to_" + BomUtils.getCanonicalName(pinSet));
				
			osResultList.add(link);
				
			/* Propagate the results from the input set mapping */
			getTarget().addAll(osResultList);
		}

		setSourceOfLinks(pinSet, outputSetActivity);	
	}

	/**
	 * source = input set of a BOM Join node
	 * target = BPEL <sequence>
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		if (getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
			return false;
		}
		
		InputPinSet is = (InputPinSet)getSource().get(0);
		if (!fFirstTimeExecuting){
			reExecute(is);
			
			executeHandlers();
			
			LoggingUtil.traceExit(this, "transformSource2Target", "re-execute");
			if (isComplete())
				return isComplete();
		} else {
			fFirstTimeExecuting = false;
		}
		
		//TODO AR new 
		//create the list of variables for the input/outputPinSet
		
		List isVariables = createVariables(is);
		OutputPinSet os = (OutputPinSet)is.getOutputPinSet().get(0);
		List osVariables = createVariables(os);
		boolean hasMapDefinition = hasMappingDefinition((Map)is.getAction());
		
		if (hasMapDefinition && (BomUtils.getObjectPinsForSet(is).size() > isVariables.size() || BomUtils.getObjectPinsForSet(os).size() > osVariables.size())){
			setComplete(false);
			return isComplete();
		}

		fMapActivity = null;				

		//if (is.getInputObjectPin().size() > 0){
		if(os.getOutputObjectPin().size() >0){	
			/* Create an AlterativeActivity with two alternatives, one for BPEL and one for BPEL+ since
			 * BPEL+ doesn't support XPath expressions in <assign> activities */
		    fMapActivity = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
			Alternative bpelAlt = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();		
			Alternative bpelpAlt = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
		
			bpelAlt.setType(ProcessFlowType.BPEL_LITERAL);
			bpelpAlt.setType(ProcessFlowType.BPELPP_LITERAL);
			
			((AlternativeActivity)fMapActivity).getAlternative().add(bpelAlt);
			((AlternativeActivity)fMapActivity).getAlternative().add(bpelpAlt);
			
			/* BPEL: Create <assign>, with multiple BPEL Copies, if there is at least one object pin */
			//bpelAlt.setActivity(createAssign(is, isVariable, os, osVariable));
			bpelpAlt.setActivity( hasMapDefinition ? 
									createAssign((Map)is.getAction()):
									createJavaInvoke(is, isVariables, os, osVariables));
			
			/* BPEL+: Create <invoke><wpc:script><wpc:javaCode>... */
			//bpelpAlt.setActivity(createJavaInvoke(is, isVariable, os, osVariable));
			//bpelpAlt.setActivity(createJavaInvoke(is, isVariables, os, osVariables));
			
		} else {
			/* If there are no input object pins, we don't need any assignments, and therefore we 
			 * don't need an AlternativeActivity */
		    fMapActivity = BPELFactory.eINSTANCE.createEmpty();
		}
		NameProvider nameProvider = NameProviderFactory.getNameProvider(fMapActivity);
		String convertedName = nameProvider.getName(fMapActivity, is.getAction(), NamingUtil.findRegistry(this));
		
		//TODO: checking no longer necessary, to be removed
		if (hasMapDefinition &&
			(BomUtils.getObjectPinsForSet(is).size() > isVariables.size() || BomUtils.getObjectPinsForSet(os).size() > osVariables.size())){
				LoggingUtil.logWarning(MessageKeys.INCOMPLETE_ASSIGN_WARNING, new String[]{is.getAction().getName(), convertedName}, null);
		}
		
		fMapActivity.setName(convertedName);
		// Ray: added display name support
		DisplayName dspName = BPELPlusFactory.eINSTANCE.createDisplayName();
		dspName.setText(is.getAction().getName());
		fMapActivity.getEExtensibilityElements().add(dspName);
				
		/* Connect the incoming links */
		connectInputSet(is, fMapActivity);

		/* Add the mapActivity to the list of targets AFTER the input set is mapped */
		getTarget().add(fMapActivity);
		
		/* Create the concurrent branch which should be necessary, given that this is a fork */
		if (BomHelper.getInstance().isCBranch(os)){
			ConcurrentBranchRule cbr = AbstractbpelFactory.eINSTANCE.createConcurrentBranchRule();
			cbr.getSource().add(os);
			getChildRules().add(cbr);
			cbr.transformSource2Target();
			getTarget().addAll(cbr.getTarget());
		}
				
		/* Connect the outgoing links */
		connectOutputSet(os, fMapActivity);	
		
		// Ray: added support for descriptions and documentations	
		mapDescriptionAndDocumentation(is.getAction(),fMapActivity);
		
		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");
		setComplete(true);
		return true;
	}
	
	/**
	 * Returns the primary activity created for this MapRule
	 *
	 * @return The primary MapRule activity
	 */
	public Activity getMapActivity(){
	    return fMapActivity;
	}
		
	/**
	 * Creates an empty BPEL Assign activity
	 * 
	 * @param is The InputPinSet that is providing the information to copy
	 * @param isVariable The BpEL variable that actually holds the source information
	 * @param os The OutputPinSet that the information is being copied to
	 * @param osVariable The BPEL variable that will hold the information once it is copied 
	 * @return Assign - The BPEL Assign activitiy with potentially many Copy elements
	 */
	private Assign createAssign(InputPinSet is, Variable isVariable, OutputPinSet os, Variable osVariable){
		
		return BPELFactory.eINSTANCE.createAssign();

		
	}
	
	
	/**
	 * Creates an empty BPEL Assign activity
	 * 
	 * @param is The InputPinSet that is providing the information to copy
	 * @param isVariable The BpEL variable that actually holds the source information
	 * @param os The OutputPinSet that the information is being copied to
	 * @param osVariable The BPEL variable that will hold the information once it is copied 
	 * @return Assign - The BPEL Assign activitiy with potentially many Copy elements
	 */
	private Assign createAssign(InputPinSet is, List isVariables, OutputPinSet os, List osVariables){
		
		return BPELFactory.eINSTANCE.createAssign();

		
	}
	
	/**
	 * Creates the Java version of the assignment to copy the input pin information to the 
	 * corresponding output pin variable
	 */
	private Invoke createJavaInvoke(InputPinSet is, List isVariables, OutputPinSet os, List osVariables){
		Invoke mapActivity = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		String javaAssign = "// Add Implementation Codes here";
		if(isVariables != null && isVariables.isEmpty()){
			javaAssign += "\n// Create the contents of variables '" + getVariablesListName(osVariables) +"'";
		}else if (isVariables!=null && osVariables!=null) {
			javaAssign += "\n// Map the contents of variables '"+  getVariablesListName(isVariables) + "' to variable '"+ getVariablesListName(osVariables) +"'";
		}	

		jsActivity.setJavaCode(javaAssign);
		mapActivity.getEExtensibilityElements().add(jsActivity);
		
		return mapActivity;
	}
	
	private String getVariablesListName(List variables){
		StringBuffer buf = new StringBuffer();
		Iterator it = variables.iterator();
		while(it.hasNext()){
			buf.append(((BPELVariable)it.next()).getName());
			buf.append(" ");
		}
		
		return buf.toString();
	}
	
	/**
	 * Creates the Java version of the assignment to copy the input pin information to the 
	 * corresponding output pin variable
	 */
	private Invoke createJavaInvoke(InputPinSet is, Variable isVariable, OutputPinSet os, Variable osVariable){
		Invoke mapActivity = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		String javaAssign = "// Add Implementation Codes here";
		if (isVariable!=null && osVariable!=null) {
			javaAssign += "\n// Map the contents of variable '"+isVariable.getName()+"' to variable '"+osVariable.getName()+"'";
		}	
		//TODO AR migration ??? 
//		javaCode.setValue(javaAssign);
//		jsActivity.setJavaCode(javaCode);
		jsActivity.setJavaCode(javaAssign);
		mapActivity.getEExtensibilityElements().add(jsActivity);
		
		return mapActivity;
	}
	
	private List<BPELVariable> createVariables(PinSet set){
		List<BPELVariable> variables = new LinkedList<BPELVariable>();
		List<?> pins = BomUtils.getObjectPinsForSet(set);
		if(pins != null && !pins.isEmpty()){
			Iterator<?> it = pins.iterator();
			while(it.hasNext()){
				ObjectPin objectPin = (ObjectPin)it.next();
				BPELVariable variable = createVariable(objectPin);
				if (variable != null){
				    getTarget().add(variable);
				    variables.add(variable);
				    pin2var.put(objectPin, variable);
				}
			}
		}
		return variables;
	}
	
	
	

	
	
	
	// ---- Assign generation ------

	
	
	
	/**
	 * Generate assign based on mapping details stored in MapAction
	 */
	private Assign createAssign(Map map){
		
		List<?> pinMaps = map.getOutputObjectPinMappings();
		Assign result = BPELFactory.eINSTANCE.createAssign();
		List<Copy> copies = new ArrayList<Copy>();
		for (Iterator<?> i = pinMaps.iterator(); i.hasNext();){
			Copy key = null;
			EObject value = null;
			OutputObjectPinMap aMapping = (OutputObjectPinMap)i.next();
			OutputObjectPin outPin = aMapping.getOutputObjectPin();
			ValueSpecification vs = aMapping.getMapping();
			if (vs instanceof InstanceValue){
				InstanceValue iValue = (InstanceValue)aMapping.getMapping();
				createCopyFromIValue(iValue, outPin, copies);
			}
			else if (vs instanceof StructuredOpaqueExpression){
				value = vs;
				Expression exp = ((StructuredOpaqueExpression)vs).getExpression();
				if (exp instanceof ModelPathExpression){
					key = createCopyFromMPE((ModelPathExpression)exp, outPin, copies);
				}
				else if (exp instanceof FunctionExpression){
					key = createCopyFromFunctionExpression((FunctionExpression)exp,  outPin, copies);
				}
			}
			else if (vs instanceof LiteralSpecification){ 
				value = vs;
				key = createCopyFromLiteralSpec((LiteralSpecification)vs, outPin, copies);
			}
			if (key != null && value != null)
				copyMap.put(key, value);
		}
		
		List<Copy> sortedCopies = sortCopyOrder(copies);
		List<Copy> renderedCopies = handleValueSwapping(sortedCopies);
		result.getCopy().addAll(copies4TempInit); // add the copies specific for temp variable initialization, this is needed for array cases for runtime behavior.
		result.getCopy().addAll(renderedCopies);
		
		//result.getCopy().addAll(sortCopyOrder(copies));		
		return result;
	}
	
	
	/**
	 * Generate BPEL copy from the provided outputObjectPin and InstanceValue pair, which represents
	 * a data mapping. Recursive call to get to the bottom level of InstanceValue if nested InstanceValue
	 * is encountered.
	 * @param iValue
	 * @param outPin
	 * @param result
	 */
	private void createCopyFromIValue(InstanceValue iValue, OutputObjectPin outPin, List<Copy> result){		
		InstanceSpecification iSpec = iValue.getOwnedInstance();
		List<?> slots = iSpec.getSlot();
		for (Iterator<?> i = slots.iterator(); i.hasNext();){
			Slot aSlot = (Slot)i.next();
			for (Iterator<?> j = aSlot.getValue().iterator(); j.hasNext();){
				Copy key = null;
				EObject value = null;
				Object slotValue = j.next();
				if (slotValue instanceof InstanceValue){
					createCopyFromIValue((InstanceValue)slotValue, outPin, result);
				}
				else if (slotValue instanceof StructuredOpaqueExpression){
					value = (StructuredOpaqueExpression)slotValue;
					Expression exp = ((StructuredOpaqueExpression)slotValue).getExpression();
					if (exp instanceof ModelPathExpression){
						key = createCopyFromMPE((ModelPathExpression)exp, outPin, result);
					}
					else if (exp instanceof FunctionExpression){
						key = createCopyFromFunctionExpression((FunctionExpression)exp,  outPin, result);
					}
				}
				else if (slotValue instanceof LiteralSpecification){ 
					value = (LiteralSpecification)slotValue;
					key = createCopyFromLiteralSpec((LiteralSpecification)slotValue, outPin, result);
				}
				if (key != null && value != null)
					copyMap.put(key, value);
			}
		}
	}
	

	/**
	 * Call to serialize the provided functionExpression. More function type checking/handling can be
	 * added here. Currently only concat function is handled.
	 * @param fExp
	 * @return
	 */
	private String getExpressionBody(FunctionExpression fExp){
		String result = "";
		String funcId = fExp.getDefinition().getFunctionID();
		if (CoreFunctions.CONCAT_FUNCTION.equals(funcId)){
			result = ConcatFunctionSerializer.serialize(fExp, pin2var, this);
		}
		return result;
	}
	
	
	/**
	 * Create a BPEL copy and add to the result list based on the provided FunctionExpression. This method is
	 * called when the mapping target (slotValue) contains a FunctionExpression.
	 * @param fExp
	 * @param outPin
	 * @param result
	 */
	private Copy createCopyFromFunctionExpression(FunctionExpression fExp, OutputObjectPin outPin,List<Copy> result){
		String funcId = fExp.getDefinition().getFunctionID();
		if (CoreFunctions.CONCAT_FUNCTION.equals(funcId)){
			return createCopyFromConcatFunction(fExp, outPin, result);
		}
		else if (CoreFunctions.SET_INDEX_OF_COLLECTION_FUNCTION.equals(funcId)){
			return createCopyFromSetIndexFunction(fExp, outPin, result);
		}
		//future: more function handling
		return null;
	}
	
	/**
	 * Create a BPEL copy and add to the result list based on the provided set index of collection FunctionExpression. 
	 * @param fExp
	 * @param outPin
	 * @param result
	 */
	private Copy createCopyFromSetIndexFunction(FunctionExpression fExp, OutputObjectPin outPin, List<Copy> result){
		
		Copy copy = BPELFactory.eINSTANCE.createCopy();		
		
		FunctionArgument outputFnArg = (FunctionArgument)fExp.getArguments().get(0);
		ModelPathExpression toMpe = ConcatFunctionSerializer.filterSimpleTypeStep((ModelPathExpression)outputFnArg.getArgumentValue());
		String queryToString = ConcatFunctionSerializer.getQueryString(toMpe, this);
		Query queryTo = BPELFactory.eINSTANCE.createQuery();
		queryTo.setQueryLanguage(QUERY_LANGUAGE);
		queryTo.setValue(queryToString);
		To to = BPELFactory.eINSTANCE.createTo();
		to.setVariable(pin2var.get(outPin));
		to.setQuery(queryTo);
		copy.setTo(to);
		
		FunctionArgument inputFnArg = (FunctionArgument)fExp.getArguments().get(2);
		Expression fromExp = inputFnArg.getArgumentValue();		
		From from = BPELFactory.eINSTANCE.createFrom();
				
		if (fromExp instanceof StringLiteralExpression){			
			from.setLiteral(((StringLiteralExpression)fromExp).getStringSymbol());
			copy.setFrom(from);			
			result.add(copy);
			return copy;
		}		
		else if (fromExp instanceof ModelPathExpression){
			ModelPathExpression fromMPE = (ModelPathExpression)fromExp;
			InputObjectPin inputPin = (InputObjectPin)((ReferenceStep)fromMPE.getSteps().get(0)).getReferencedObject();
			Type srcType = getTypeFromModelPathExpression(fromMPE);
			if (srcType == null){
				srcType = inputPin.getType();
			}
			Type tgrType = getTypeFromModelPathExpression(toMpe);
			if (tgrType == null){
				tgrType = outPin.getType();
			}
			boolean wrapPrimitiveToString = srcType instanceof PrimitiveType && tgrType instanceof PrimitiveType && ! srcType.equals(tgrType) && tgrType.getName().equals(primitiveString);
			
			String queryFromString;
			if (wrapPrimitiveToString){
				String getVarDataString = ConcatFunctionSerializer.serializeGetVariableDataFuntion(fromMPE, pin2var, this);
				queryFromString = ConcatFunctionSerializer.serializeWarpSrcToString(getVarDataString);
			}
			else{
				queryFromString = fromMPE.getSteps().size() > 1 ? ConcatFunctionSerializer.serializeGetVariableDataFuntion(fromMPE, pin2var, this): "";
			}
			
			Query queryFrom = BPELFactory.eINSTANCE.createQuery();
			from.setVariable(pin2var.get(inputPin));
			if (! "".equals(queryFromString)){
				queryFrom.setQueryLanguage(QUERY_LANGUAGE);
				queryFrom.setValue(queryFromString);
				from.setQuery(queryFrom);
			}			
			copy.setFrom(from);	
			result.add(copy);
			return copy;
		}
		else if (fromExp instanceof FunctionExpression){
			FunctionExpression fromFnExp = (FunctionExpression)fromExp;
			String funcId = fromFnExp.getDefinition().getFunctionID();
			if (CoreFunctions.CONCAT_FUNCTION.equals(funcId)){
				com.ibm.wbit.bpel.Expression exp = BPELFactory.eINSTANCE.createExpression();
				String body = getExpressionBody(fromFnExp);
				exp.setBody(body);
				exp.setExpressionLanguage(QUERY_LANGUAGE);
				from.setExpression(exp);
				copy.setFrom(from);
				result.add(copy);
				return copy;
			}
		}
		return null;
	}
	
	/**
	 * Create a BPEL copy and add to the result list based on the provided concat FunctionExpression. 
	 * @param fExp
	 * @param outPin
	 * @param result
	 */
	private Copy createCopyFromConcatFunction(FunctionExpression fExp, OutputObjectPin outPin, List<Copy> result){
		
		com.ibm.wbit.bpel.Expression exp = BPELFactory.eINSTANCE.createExpression();
		String body = getExpressionBody(fExp);
		exp.setBody(body);
		exp.setExpressionLanguage(QUERY_LANGUAGE);
		
		ModelPathExpression toMpe = createToMPE(fExp);
		String queryToString = ConcatFunctionSerializer.getQueryString(toMpe, this);
		
		Copy copy = BPELFactory.eINSTANCE.createCopy();		
		From from = BPELFactory.eINSTANCE.createFrom();
		To to = BPELFactory.eINSTANCE.createTo();
		
		from.setExpression(exp);
		to.setVariable(pin2var.get(outPin));
		if (queryToString != null && ! queryToString.equals("")){
			Query queryTo = BPELFactory.eINSTANCE.createQuery();
			queryTo.setQueryLanguage(QUERY_LANGUAGE);
			queryTo.setValue(queryToString);
			to.setQuery(queryTo);
		}
		copy.setFrom(from);
		copy.setTo(to);
		result.add(copy);	
		return copy;
	}
	 

	/**
	 * Create a BPEL copy and add to the result list based on the provided LiteralSpecification. This method is
	 * called when the mapping target (slotValue) contains a LiteralSpecification.
	 * @param lSpec
	 * @param outPin
	 * @param result
	 */
	private Copy createCopyFromLiteralSpec(LiteralSpecification lSpec, OutputObjectPin outPin, List<Copy> result){
		ModelPathExpression toMpe = createToMPE(lSpec);
		Type tgrType = getTypeFromModelPathExpression(toMpe);
		if (tgrType == null){
			tgrType = outPin.getType();
		}
		Copy copy = BPELFactory.eINSTANCE.createCopy();		
		From from = BPELFactory.eINSTANCE.createFrom();
		To to = BPELFactory.eINSTANCE.createTo();
		String val = getStringLiteral(lSpec, tgrType);
		val = val == null || "".equals(val)? " ": val;
		
		from.setLiteral(val);
		from.setUnsafeLiteral(tgrType.getName().equals(primitiveString) ? Boolean.TRUE: Boolean.FALSE);		

		String queryToString = ConcatFunctionSerializer.getQueryString(toMpe, this);
		to.setVariable(pin2var.get(outPin));
		if (queryToString != null && !queryToString.equals("")){
			Query queryTo = BPELFactory.eINSTANCE.createQuery();
			queryTo.setQueryLanguage(QUERY_LANGUAGE);
			queryTo.setValue(queryToString);
			to.setQuery(queryTo);
		}
		copy.setFrom(from);
		copy.setTo(to);
		result.add(copy);
		return copy;
	}
	
	
	/**
	 * Return the string value of the provided LiteralSpecification.
	 * @param lSpec
	 * @return
	 */
	private String getStringLiteral(LiteralSpecification lSpec, Type type){
		String result = null;
		if (lSpec instanceof LiteralString){
			String val = ((LiteralString)lSpec).getValue() == null? "": ((LiteralString)lSpec).getValue();
			String typeName = type.getName();
			if (typeName.equals(primitiveBoolean)){
				result = val.toLowerCase();
			}
			else if (typeName.equals(primitiveDataTime)){
				if (val.indexOf(".") > 0)
					result = val.substring(0, val.indexOf("."));
			}
			else{
				result = val;
			}
		}
		else if (lSpec instanceof LiteralDuration){
			result = ((LiteralDuration)lSpec).getValue();
		}
		else if (lSpec instanceof LiteralTime){
			result = ((LiteralTime)lSpec).getValue();
		}
		else if (lSpec instanceof LiteralUnlimitedNatural){
			result = ((LiteralUnlimitedNatural)lSpec).getValue();
		}
		else if (lSpec instanceof LiteralInteger){
			result = ((LiteralInteger)lSpec).getValue().toString();
		}
		else if (lSpec instanceof LiteralReal){
			result = ((LiteralReal)lSpec).getValue().toString();
		}
		return result;
	}
	
	
	/**
	 * Create a BPEL copy and add to the result list based on the provided ModelPathExpression. This method is
	 * called when the mapping target (slotValue) contains a ModelPathExpression.
	 * @param mpe
	 * @param outPin
	 * @param result
	 */
	private Copy createCopyFromMPE(ModelPathExpression mpe, OutputObjectPin outPin, List<Copy> result){
		List<?> steps = mpe.getSteps();
		EObject refObj = ((ReferenceStep)steps.get(0)).getReferencedObject();
		if (! (refObj instanceof InputObjectPin)) return null;

		InputObjectPin inPin = (InputObjectPin)refObj;
		BPELVariable inPinVar = pin2var.get(inPin);
		ModelPathExpression toMpe = createToMPE(mpe);

		Type srcType = getTypeFromModelPathExpression(mpe);
		if (srcType == null){
			srcType = inPin.getType();
		}
		Type tgrType = getTypeFromModelPathExpression(toMpe);
		if (tgrType == null){
			tgrType = outPin.getType();
		}
		boolean wrapPrimitiveToString = srcType instanceof PrimitiveType && tgrType instanceof PrimitiveType && ! srcType.equals(tgrType) && tgrType.getName().equals("String");
		Copy copy = null;
		String fromQueryString = containsIndexStep(mpe)|| wrapPrimitiveToString ? 
			ConcatFunctionSerializer.serializeGetVariableDataFuntion(mpe, pin2var, this):
			ConcatFunctionSerializer.getQueryString(mpe, this);
		String toQueryString = ConcatFunctionSerializer.getQueryString(toMpe, this);
		if (wrapPrimitiveToString){
			copy = createCopyWrapSrcToString(fromQueryString, toQueryString, inPinVar, pin2var.get(outPin));
		}
		else{
			copy = createCopy(inPinVar, pin2var.get(outPin), fromQueryString, toQueryString);		
		}
		result.add(copy);
		return copy;
	}
	
	private Copy createCopyWrapSrcToString(String fromString, String toString, BPELVariable fromVar, BPELVariable toVar){
		Copy copy = BPELFactory.eINSTANCE.createCopy();
		From from = BPELFactory.eINSTANCE.createFrom();
		To to = BPELFactory.eINSTANCE.createTo();
		to.setVariable(toVar);
		
		if (fromString.equals("")){
			com.ibm.wbit.bpel.Expression exp = BPELFactory.eINSTANCE.createExpression();
			String body = ConcatFunctionSerializer.serializeWarpObject(fromVar);
			exp.setBody(body);
			exp.setExpressionLanguage(QUERY_LANGUAGE);
			from.setExpression(exp);
		}
		else{
			from.setVariable(fromVar);
			Query queryFrom = BPELFactory.eINSTANCE.createQuery();
			queryFrom.setQueryLanguage(QUERY_LANGUAGE);
			queryFrom.setValue(ConcatFunctionSerializer.serializeWarpSrcToString(fromString));
			from.setQuery(queryFrom);
		}
		
		if (! toString.equals("")){
			Query queryTo = BPELFactory.eINSTANCE.createQuery();
			queryTo.setQueryLanguage(QUERY_LANGUAGE);
			queryTo.setValue(toString);
			to.setQuery(queryTo);
		}
		
		copy.setFrom(from);
		copy.setTo(to);
		return copy;
	}
	
	private Type getTypeFromModelPathExpression(ModelPathExpression mpe){
		Type result = null;
		List<?> steps = mpe.getSteps();
		for (int i = steps.size() -1; i >= 0 ; i--){
			Object step = steps.get(i);
			if (step instanceof ReferenceStep && ((ReferenceStep)step).getReferencedObject() instanceof Property){
				Property property = (Property)((ReferenceStep)step).getReferencedObject();
				result = property.getType();
				break;
			}
		}
		return result;
	}
	
	private boolean containsIndexStep(ModelPathExpression mpe){
		boolean result = false;
		for (Iterator<?> i = mpe.getSteps().iterator(); i.hasNext();){
			if (i.next() instanceof ArrayIndexStep){
				result = true;
				break;
			}
		}
		return result;
	}
	
	/**
	 * Create a new ModelPathExpression based on the provided expression which is contained in the mapping target (slotValue).
	 * This method traverse up to identify the ancestor properties to be the reference steps.
	 * @param fromSpec
	 * @return
	 */
	public ModelPathExpression createToMPE(EObject fromSpec){
		CreateModelPathExpressionEXPCmd createToMpeCmd = new CreateModelPathExpressionEXPCmd();
		createToMpeCmd.execute();
		ModelPathExpression toMpe = createToMpeCmd.getExpression();
		List<EObject> refObjs = new ArrayList<EObject>();		
		EObject container = fromSpec;
		while (container.eContainer() != null){
			container = container.eContainer();
			if (container instanceof Slot){
				EObject refObj = ((Slot)container).getDefiningFeature();
				if (refObj != null){
					refObjs.add(0, refObj);
				}
			}
			if (container instanceof OutputObjectPinMap){
				ObjectPin objPin = ((OutputObjectPinMap)container).getOutputObjectPin();
				Type pinType = objPin.getType();
				if (BrUtil.isSimpleType(pinType)){
					refObjs.add(0, objPin);
				}
			}
		}		
		List<Object> toSteps = toMpe.getSteps();
		for (EObject refObj: refObjs){
			if (refObj instanceof ObjectPin && BrUtil.isSimpleType(((ObjectPin)refObj).getType())){
				break;
			}
			ReferenceStep rs = ModelFactory.eINSTANCE.createReferenceStep();
			rs.setReferencedObject(refObj);
			toSteps.add(rs);
			if (refObj instanceof Property && BrUtil.isSimpleType(((Property)refObj).getType())){
				break;
			}
		}
		return toMpe;
	}
	

	/**
	 * Create and return a BPEL copy based on the provided source and target variable, to and from query strings.
	 * @param srcVar
	 * @param tgrVar
	 * @param queryFromString
	 * @param queryToString
	 * @return
	 */
	private Copy createCopy (BPELVariable srcVar, BPELVariable tgrVar, String queryFromString, String queryToString){
		
		Copy copy = BPELFactory.eINSTANCE.createCopy();		
		From from = BPELFactory.eINSTANCE.createFrom();
		from.setVariable(srcVar);
		if (queryFromString != null && !queryFromString.equals("")){
			Query queryFrom = BPELFactory.eINSTANCE.createQuery();
			queryFrom.setQueryLanguage(QUERY_LANGUAGE);
			queryFrom.setValue(queryFromString);
			from.setQuery(queryFrom);
		}		
		To to = BPELFactory.eINSTANCE.createTo();
		to.setVariable(tgrVar);
		if (queryToString != null && !queryToString.equals("")){
			Query queryTo = BPELFactory.eINSTANCE.createQuery();
			queryTo.setQueryLanguage(QUERY_LANGUAGE);
			queryTo.setValue(queryToString);
			to.setQuery(queryTo);
		}		
		copy.setFrom(from);
		copy.setTo(to);
		return copy;
	}
	
	public EObject getObjectMappedToCopy(Copy copy) {
		return copyMap.get(copy);
	}
	
	
	private List<Copy> sortCopyOrder(List<Copy> copies){
		List<Copy> result = new ArrayList<Copy>(copies.size());
		List<Copy> noIndex = new ArrayList<Copy>();
		for (Copy copy: copies){
			int insertIndex = getInsertIndex(result, copy);
			if (insertIndex < 0){
				noIndex.add(copy);
			}
			else{
				result.add(insertIndex, copy);
			}
		}
		result.addAll(noIndex);
		return result;
	}
	
	
	private int getInsertIndex(List<Copy> copyList, Copy copyToInsert){
		int result = -1;
		int orderIndex = getOrderIndex(copyToInsert);
		if (orderIndex >= 0){
			for (result = 0; result < copyList.size(); result++){
				Copy thisCopy = copyList.get(result);
				if (thisCopy == null){
					break;
				}
				int thisOrderIndex = getOrderIndex(copyList.get(result));
				if (orderIndex < thisOrderIndex){
					break;
				}
			}
		}
		return result;
	}
	
	private int getOrderIndex(Copy copy){
		int result = -1;
		ValueSpecification vs = (ValueSpecification)copyMap.get(copy);
		if (vs != null && ! vs.getOwnedComment().isEmpty()){
			String orderIndex = ((Comment)vs.getOwnedComment().get(0)).getBody();
			try{
				Integer integer = new Integer(orderIndex);
				result = integer.intValue();
			}
			catch (NumberFormatException e){
				// do nothing as comment body is not numeric, return default value -1
			}
		}
		return result;
	}
	
	private boolean hasMappingDefinition(Map map){
		return ! map.getOutputObjectPinMappings().isEmpty();
	}
	

	
	
	//------ Values Swapping support ---------
	
	public java.util.Map<ObjectPin, BPELVariable> getPin2var() {
		return pin2var;
	}

	private List<Copy> handleValueSwapping(List<Copy> copies){
		List<Copy> result = new ArrayList<Copy>();
		SwapOrganizer mMgr = new SwapOrganizer(this, pin2var);
		for (Copy copy: copies){
			 mMgr.append(copy);
		}
		result = mMgr.getRenderedCopies();
		
		return result;
	}
	
	/**
	 * Return the temp variable for the given pin and bpelVariable. Look for the available temp in this order:
	 * 1) Local (in this rule)
	 * 2) Remote (in other map rule)
	 * 3) If non was found, create a new temp variable. 
	 * @param originalVariable
	 * @return
	 */
	public BPELVariable getTempVariable(BPELVariable originalVariable){
		BPELVariable result = getLocallyAvailableTempVariable(originalVariable);
		if (result != null){
			return result;
		}
		result = getRemotelyAvailableTempVariable(originalVariable);
		if (result != null){
			var2temp.put(originalVariable, result);
			createInitializeCopyForTempVariable(originalVariable,result);
			return result;
		}
		result = BPELFactory.eINSTANCE.createBPELVariable();
		result.setType(originalVariable.getType());
		result.setName(originalVariable.getName()+"_TEMP");
		result.setDocumentationElement(originalVariable.getDocumentationElement());
		result.setElement(originalVariable.getElement());
		result.setEnclosingDefinition(originalVariable.getEnclosingDefinition());
		result.setMessageType(originalVariable.getMessageType());
		result.setXSDElement(originalVariable.getXSDElement());
		var2temp.put(originalVariable, result);
		createInitializeCopyForTempVariable(originalVariable,result);
		getTarget().add(result);			
		return result;
	}
	
	/**
	 * Ensure the new tamp variable is initialized in this rule context by copying the originalVariable to the new temp Variable.
	 * This is required if the variable contains attributes has multiplicity of larger than 1 (i.e. collection support), speciaically
	 * runtime requires prior values in the collection to be filled before assigning the next value.
	 * @param srcVar
	 * @param tempVar
	 */
	private void createInitializeCopyForTempVariable(BPELVariable srcVar, BPELVariable tempVar){
		Copy copy = BPELFactory.eINSTANCE.createCopy();		
		From from = BPELFactory.eINSTANCE.createFrom();
		from.setVariable(srcVar);
		To to = BPELFactory.eINSTANCE.createTo();
		to.setVariable(tempVar);
		copy.setFrom(from);
		copy.setTo(to);
		copies4TempInit.add(copy);
	}
	
	/**
	 * Return the locally available temp bpel variable created for values swapping for a specified pin
	 * @param pin
	 * @return
	 */
	private BPELVariable getLocallyAvailableTempVariable(BPELVariable var){
		return var2temp.get(var);
	}
	
	/**
	 * Reach out to look for temp variable in the other MapRules.
	 * @param originalVar
	 * @return
	 */
	private BPELVariable getRemotelyAvailableTempVariable(BPELVariable originalVar){
		BPELVariable result = null;
		for (Iterator<?> i = getRoot().getRules().iterator(); i.hasNext();){
			TransformationRule tempRule = (TransformationRule)i.next();
			if (tempRule instanceof MapRuleImpl && tempRule != this){
				MapRuleImpl remoteMapRule = (MapRuleImpl)tempRule;
				result = remoteMapRule.getLocallyAvailableTempVariable(originalVar);
				if (result != null){
					break;
				}
			}
		}
		return result;	
	}
	
	/**
	 * Return the original variable given the temp variable. Null if none was found.
	 * @param tempVar
	 * @return
	 */
	public BPELVariable getOriginalVariable(BPELVariable tempVar){
		BPELVariable result = null;
		for (Iterator<BPELVariable> i = var2temp.keySet().iterator(); i.hasNext();){
			BPELVariable anOrgVar = i.next();
			if (var2temp.get(anOrgVar) == tempVar){
				result = anOrgVar;
				break;
			}
		}
		return result;
	}
		

	/**
	 * Register the copy with the expression for traceability.
	 * @param copy
	 * @param soe
	 */
	public void register(Copy copy, EObject soe){
		copyMap.put(copy, soe);
	}
	

} //MapRuleImpl
