/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.Map;
import java.util.TimeZone;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.model.resourcemanager.ResourceMGR;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.impl.TransformationRuleImpl;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.TransformationEngine;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ActivityRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.util.AbstractbpelNamingRegistry;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BomWalker;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionKeyConstants;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionUtil;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingUtil;
import com.ibm.btools.util.datatype.BTDataTypeException;
import com.ibm.btools.util.datatype.BTDateTime;
import com.ibm.btools.util.datatype.BTTime;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.CorrelationSet;
import com.ibm.wbit.bpel.CorrelationSets;
import com.ibm.wbit.bpel.PartnerLink;
import com.ibm.wbit.bpel.PartnerLinks;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.bpel.Variable;
import com.ibm.wbit.bpel.Variables;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.CustomProperty;
import com.ibm.wbit.bpelpp.Description;
import com.ibm.wbit.bpelpp.DisplayName;
import com.ibm.wbit.bpelpp.Documentation;
import com.ibm.wbit.bpelpp.ValidFrom;

/**
 * Rule:
 * Creates a BOM process and creates an Activity Rule to process
 * the BOM Activity.  
 * Source: BOM Process Activity
 * Target: BPEL Process
 */
public class ProcessDefinitionRuleImpl extends TransformationRuleImpl implements ProcessDefinitionRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private Process process;
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ProcessDefinitionRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.PROCESS_DEFINITION_RULE;
	}

	/**
	 * Source: BOM Process Activity
	 * Target: BPEL Process
	 * Rule:
	 * Creates a BOM process and creates an Activity Rule to process
	 * the BOM Activity.  Expected targets from ActivityRule is a top
	 * level BPEL activity that represents the flow, as well as Partners
	 * Containers, and Event Handlers that are created as side-effect
	 * of the child rules but not absorbed by the child rules.  This
	 * rule will process all the targets that are propagated and add them
	 * to the BPEL Process as appropriate
	 */
	public boolean transformSource2Target()
	{
		LoggingUtil.traceEntry(this, "transformSource2Target");

		boolean ret=false;
		TransformationContext context = getContext();
			
		// Create BOM Walker and store it in the context
		BomWalker bomWalker = new BomWalker();
		ProcessUtil.putBomWalker(context,bomWalker);
		// Creates a BPEL process model
		process = createBPELProcess();
		ProcessUtil.putProcessDefinitionRule(context,(NamedElement)getSource().get(0),this);
		if(process != null){
			getTarget().add(process);	
			//Register the AbstractbpelNamingRegistry
//			if (NamingUtil.getRegistry(context, process)==null)	
//				NamingUtil.putRegistry(context, process, new AbstractbpelNamingRegistry());	
			
//			JavaNCNameConverter converter = new JavaNCNameConverter();
//			String convertedName = converter.convertName(NamingUtil.findRegistry(this), process, process.getName());
//			process.setName(convertedName);

			//Ray: Looks like for WSADIE, the BPEL must have a shadow WSDL and that the filename of the WSDL must be the same
			//as the filename of BPEL.  The filename of WSDL is created based on the namespace (regardless whether it is a shadow WSDL
			//or regular WSDL).  The namespace of the shadow WSDL is the same as the namespace of the BPEL.  In order to match the
			//WSDL and BPEL filename, then the BPEL name (i.e. process name) must use the namespace (last segment) of the BPEL. 
			//AR this is not a requirement anymore... 
//			process.setName(getProcessNameFromNamespace(process.getTargetNamespace()));
			//Ray: added for displayname on BPELP.. AR .. moved to createBpelProcess 
//			DisplayName description = BPELPlusFactory.eINSTANCE.createDisplayName();
//			description.setText(((NamedElement)getSource().get(0)).getName());
//			process.getEExtensibilityElements().add(description);
					
			// Register ProcessDefinitionRule in the context with preset key (using namespace)
			//ProcessUtil.putProcessDefinitionRule(context,(NamedElement)getSource().get(0),this);		
			// A single source is expected which is a BOM Process Activity
//			ActivityRule activityRule = AbstractbpelFactory.eINSTANCE.createActivityRule();
//			getChildRules().add(activityRule);
//			activityRule.getSource().addAll(getSource());
//			ret = activityRule.transformSource2Target();
	
			// Ray: added support for descriptions and documentations		
			Action action = ((Activity)getSource().get(0)).getImplementation();
			Description desc = AbstractbpelUtil.transformDescription(action);
			if (desc!=null) {
				process.addExtensibilityElement(desc);
			}
			Documentation doc = AbstractbpelUtil.transformDocumentation(action);
			if (doc!=null) {
				 process.addExtensibilityElement(doc);
			}
		}
		// A single source is expected which is a BOM Process Activity
		ActivityRule activityRule = AbstractbpelFactory.eINSTANCE.createActivityRule();
		getChildRules().add(activityRule);
		activityRule.getSource().addAll(getSource());
		ret = activityRule.transformSource2Target();

		if(process != null){
			// Process child targets
			processChildTargets();
		}		
	
		executeHandlers();
		
		setComplete(ret);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}

	
    
	
	/**
	 * Creates a BPEL Process
	 * @return Process
	 */
	private Process createBPELProcess() 
	{
		Activity activity = (Activity)getSource().get(0);
		//AR if this is a ReusablTask or a service do not create a BPEL Process for it.
		//AZ if it's external service, we don't create a BPEL Process for it
		Action san = null;
		if(activity.getImplementation()!= null){
			san = (Action)activity.getImplementation();
			if(BomUtils.isTask(san) || BomUtils.isService(san)||BomUtils.isServiceOperation(san)){
				return null;
			}
		}
				
		Process process = BPELFactory.eINSTANCE.createProcess();	
		
		if (NamingUtil.getRegistry(getContext(), process)==null)	
			NamingUtil.putRegistry(getContext(), process, new AbstractbpelNamingRegistry());	
		
		AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.BPEL_PROCESS_DISPLAY_NAME);
		valueProvider.setContext(this.getContext());
		String processName  = (String)valueProvider.getAttributeValueForType( process, san, AttributeNameConstants.BPEL_PROCESS_NAME, NamingUtil.getRegistry(getContext(), process));
		String targetNamespace = (String)valueProvider.getAttributeValueForType(process, activity.getImplementation(), AttributeNameConstants.BPEL_PROCESS_TARGET_NAMESPACE, NamingUtil.getRegistry(getContext(), process));
		String displayName = (String)valueProvider.getAttributeValueForType( process, activity.getImplementation(), AttributeNameConstants.BPEL_PROCESS_DISPLAY_NAME, NamingUtil.getRegistry(getContext(), process));
		valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.BPEL_PROCESS_IS_LONG_RUNNING);
		String validStmp = getValidFrom(san);
	//	String validStmp = (String)valueProvider.getAttributeValueForType(activity.getImplementation(), AttributeNameConstants.BPEL_PROCESS_VALID_FROM);
		Boolean longRunningProcess = (Boolean)valueProvider.getAttributeValueForType(activity.getImplementation(), AttributeNameConstants.BPEL_PROCESS_IS_LONG_RUNNING);
		Boolean runAsChild = (Boolean)valueProvider.getAttributeValueForType(activity.getImplementation(), AttributeNameConstants.BPEL_PROCESS_RUN_AS_CHILD);
		
		process.setName(processName);
		process.setTargetNamespace(targetNamespace);
		
		
		DisplayName description = BPELPlusFactory.eINSTANCE.createDisplayName();
		description.setText(displayName);
		process.getEExtensibilityElements().add(description);
		
//		process.setName(((Activity)getSource().get(0)).getName());
//		String encodedNamespace = ProcessUtil.createNamespace((NamedElement) getSource().get(0), true);
//		process.setTargetNamespace(encodedNamespace);

		process.setSuppressJoinFailure(Boolean.TRUE);
		ValidFrom validFrom = BPELPlusFactory.eINSTANCE.createValidFrom();		
		validFrom.setValidFrom(validStmp);
		process.getEExtensibilityElements().add(validFrom);	
		
		String namespace = ProcessUtil.createNamespace(activity);
		TransformationSessionUtil.registerProcessedElementWithNamespace(TransformationEngine.getTransformationSession(), process, namespace);

		return process;
	}
	
//	private String getProcessNameFromNamespace(String namespace) {
//		String name;
//		int lastSeparator = namespace.lastIndexOf("/");
//		if (lastSeparator == -1) {
//			lastSeparator = namespace.lastIndexOf("\\");
//		}
//		if (lastSeparator != -1) {
//			name = namespace.substring(lastSeparator+ 1,namespace.length());
//		} else {
//			name = namespace.substring(namespace.indexOf(":")+ 1,namespace.length());
//		}
//		return name;
//	}
		
//	/**
//	 * Process targets of child rules and determine if it is an element
//	 * that needs to be added to the BPEL Process
//	 *
//	 */
//	private void processChildTargets() {
//		EList childRules = getChildRules();
//		Iterator childRulesIterator = childRules.iterator();
//		while (childRulesIterator.hasNext()) {
//			TransformationRule childRule =
//				(TransformationRule) childRulesIterator.next();
//			EList childTargets = childRule.getTarget();
//			for (int i = 0; i < childTargets.size(); i++) {
//				EObject childTarget = (EObject) childTargets.get(i);
//				if (childTarget instanceof PartnerLink) {
//					process.getPartnerLinks().getChildren().add(childTarget);
//				} else if (childTarget instanceof Variable) {
//					process.getVariables().getChildren().add(childTarget);
//				} else if (childTarget instanceof com.ibm.etools.ctc.bpel.Activity) {
//					process.setActivity((com.ibm.etools.ctc.bpel.Activity)childTarget);
//				} else {
//					// This case is not expected
//					super.propagateChildTargets();
//				}
//			}
//		}
//	}
	
	private void processChildTargets() {
		EList childRules = getChildRules();
		Iterator childRulesIterator = childRules.iterator();
		while (childRulesIterator.hasNext()) {
			TransformationRule childRule =
				(TransformationRule) childRulesIterator.next();
			EList childTargets = childRule.getTarget();
			for (int i = 0; i < childTargets.size(); i++) {
				EObject childTarget = (EObject) childTargets.get(i);
				if (childTarget instanceof PartnerLink) {
					if(process.getPartnerLinks() == null){
						PartnerLinks partnerLinks = BPELFactory.eINSTANCE.createPartnerLinks();
						process.setPartnerLinks(partnerLinks);
						partnerLinks.getChildren().add(childTarget);
						}else{
							process.getPartnerLinks().getChildren().add(childTarget);
						}
				} else if (childTarget instanceof Variable) {
					if(process.getVariables() == null){
						Variables variables = BPELFactory.eINSTANCE.createVariables();
						process.setVariables(variables);
						variables.getChildren().add(childTarget);
					}else{
						process.getVariables().getChildren().add(childTarget);
					}
				} else if (childTarget instanceof CorrelationSet) {
					if(process.getCorrelationSets() == null){
						CorrelationSets cSets = BPELFactory.eINSTANCE.createCorrelationSets();
						process.setCorrelationSets(cSets);
					//	cSets.getChildren().add(childTarget);
					}
						JavaNCNameConverter converter = new JavaNCNameConverter();
						String convertedName = converter.convertName(NamingUtil.findRegistry(this), childTarget, ((CorrelationSet)childTarget).getName());
						((CorrelationSet)childTarget).setName(convertedName);
							
						process.getCorrelationSets().getChildren().add(childTarget);
										
				} else if (childTarget instanceof com.ibm.wbit.bpel.Activity) {
					process.setActivity((com.ibm.wbit.bpel.Activity)childTarget);
				} else {
					// This case is not expected
					super.propagateChildTargets();
				}
			}
		}
	}

	private String getValidFrom(Action san){
		String validFromString = "";
		BTDateTime validFrom = TransformationSessionUtil.getValidFromOverride(TransformationEngine.getTransformationSession(),  (StructuredActivityNode) san);
		Calendar cal = null;
		
		//defect 112450
		if(validFrom == null){
			// Ray - get the timestamp of the Process and use it for the validFrom timestamp
			ResourceMGR resourceManager = ResourceMGR.getResourceManger();		
			Activity activity = ((StructuredActivityNode)san).getActivity();
			String projectName = MappingUtil.getProjectNameForBOM(activity);
			String timestamp = resourceManager.getLastModifid(projectName,activity.getUid());
			cal = new GregorianCalendar();
			if (timestamp!=null) {
				cal.setTimeInMillis(Long.parseLong(timestamp));
			}
			//generate the validFrom based on the current time of transformation 
			if(BpelOptionsUtil.isGenerateForDeployPatternEnabled()){
				long longTimestamp = System.currentTimeMillis();
				cal.setTimeInMillis(longTimestamp);
			}
			Date date = cal.getTime();
			Calendar gmtCalendar = new GregorianCalendar(TimeZone.getTimeZone("GMT+0"));
			gmtCalendar.setTime(date);
			cal = new GregorianCalendar(gmtCalendar.get(Calendar.YEAR), gmtCalendar.get(Calendar.MONTH), gmtCalendar.get(Calendar.DAY_OF_MONTH), gmtCalendar.get(Calendar.HOUR_OF_DAY), gmtCalendar.get(Calendar.MINUTE), gmtCalendar.get(Calendar.SECOND));
		}else{
			cal = Calendar.getInstance();
			cal.set(validFrom.getYear(), validFrom.getMonth()-1, validFrom.getDay(), validFrom.getHour(), validFrom.getMinute(), validFrom.getSecond());
			boolean inDSTBefore = cal.getTimeZone().inDaylightTime(cal.getTime());
			if (validFrom.getTimeFormat() != BTTime.Universal_TIME_ZONE_Format){
				try {
					int plusMinus = (validFrom.getTimeZone_Plus_Minus() > 0 ? -1 : 1);
					cal.add(Calendar.HOUR_OF_DAY, plusMinus * validFrom.getTimeZoneHour());
					cal.add(Calendar.MINUTE, plusMinus * validFrom.getTimeZoneMin());
					boolean inDSTAfter = cal.getTimeZone().inDaylightTime(cal.getTime());
					if (inDSTBefore ^ inDSTAfter){
						cal.add(Calendar.MILLISECOND, (inDSTAfter ? -1 : +1) * cal.getTimeZone().getDSTSavings());
					}
				} catch (BTDataTypeException e){
					/* In this case, the date may be inconsistent, so reset it to the original value */
					cal.set(validFrom.getYear(), validFrom.getMonth()-1, validFrom.getDay(), validFrom.getHour(), validFrom.getMinute(), validFrom.getSecond());
				}
			} 
		}
		SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
		validFromString = dateFormat.format(cal.getTime());
		return validFromString;
		
	}
	
	

} //ProcessDefinitionRuleImpl
