/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.wsdl.Message;
import javax.wsdl.Output;
import javax.xml.namespace.QName;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.Part;
import org.eclipse.wst.wsdl.PortType;
import org.eclipse.wst.wsdl.Types;
import org.eclipse.wst.wsdl.WSDLFactory;
import org.eclipse.wst.wsdl.XSDSchemaExtensibilityElement;
import org.eclipse.wst.wsdl.internal.impl.DefinitionImpl;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDFactory;
import org.eclipse.xsd.XSDImport;
import org.eclipse.xsd.XSDSchema;
import org.eclipse.xsd.util.XSDConstants;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.impl.TransformationRuleImpl;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessInterfaceRule;
import com.ibm.btools.te.ilm.heuristics.extservice.ExtserviceFactory;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.FabricIntegrationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.impl.ProcessFlowRuleImpl;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.WSDLDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.WsdlFactory;
import com.ibm.wbit.bpel.services.partnerlinktype.PartnerLinkType;

/**
 * Creates a WSDL representing the Process Inteface
 * Source: BOM Process SAN
 * Target: WSDL Definition
 */
public class ProcessInterfaceRuleImpl extends TransformationRuleImpl implements ProcessInterfaceRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private Definition wsdlDefinition;
	private Definition wsdlCallbackDefinition;
	

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ProcessInterfaceRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.PROCESS_INTERFACE_RULE;
	}

	/**
	 * When first called, this method will create the WSDLDefinition and set it as its
	 * target. As this rule is shared, it will only be set to complete by the ProcessFlowRule when the ProcessDefinitionRule is 
	 * completed. 
	 */
	public boolean transformSource2Target(){
		LoggingUtil.traceEntry(this, "transformSource2Target");

		if(isComplete()){
			LoggingUtil.traceExit(this, "transformSource2Target", "already completed");
			return true;
		}
		else{
			if(target == null || target.isEmpty()){
				TransformationRule wsdlDefRule = null;
				//we need to handle external service export and service operation referenced 
				//in other process
				if(BomUtils.isServiceOperation((Action)getSource().get(0))){
					 wsdlDefRule = ExtserviceFactory.eINSTANCE.createWSDLDefinitionRule();
				}else{
					 wsdlDefRule = WsdlFactory.eINSTANCE.createWSDLDefinitionRule();
				}
				getChildRules().add(wsdlDefRule);
				if(getSource().get(0) instanceof StructuredActivityNode){
					StructuredActivityNode san = (StructuredActivityNode)getSource().get(0);
					if(BomUtils.isProcess(san) && FabricIntegrationUtil.getInterfaceActivityIfAny(san)!= null){					
						wsdlDefRule.getSource().add(FabricIntegrationUtil.getInterfaceActivityIfAny(san).getImplementation());				
					}else{
						wsdlDefRule.getSource().addAll(getSource());
					}
				}
				wsdlDefRule.transformSource2Target();
				wsdlDefinition = (Definition)wsdlDefRule.getTarget().get(0);
				if(shouldCreateScalfoldDefinition()){
					wsdlDefinition = createSkeletonWSDLDefinitionForProcessVariation(wsdlDefinition);
				}
				getTarget().add(wsdlDefinition);
			}
			executeChildRules();			
			// this rule will never complete
			setComplete(false);
			executeHandlers();
		}
		setComplete(false);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}
	/**
	 * Executes each child rule and immediately process the child rule targets.
	 * This is done in order to avoid processing the target of a rule if the rule is
	 * executed several times.
	 * Interface rules are added as childs while the process is traversed and
	 * Actions are encountered. 
	 */
	protected boolean executeChildRules(){
		boolean allComplete = true;
		if (childRules != null) {
			for (int i = 0; i < childRules.size(); i++) {
				TransformationRule r = (TransformationRule) childRules.get(i);
				if (!r.isComplete() && !r.isFailed()) {
					boolean  ret = r.transformSource2Target();
					processChildTargets(r);
					if(!ret)
						allComplete = false;
				}
			}
		}		
		return allComplete;		
	}
	/**
	* A ProcessInterfaceRule will only process WSDL PortTypes from its target. If there is more than 1 PortType created for this ProcessInterfaceRule, 
	* this has to be the CallbackPortType. (Transformation now creates separate interfaces for each Local/Global Tasks). 
	* The second PortType will be added to a new WSDL Definitions (set as target of this Rule). All Messages, ElementDeclarations that pertain to the callbackPortType
	* will be added to the new WSDL Definition. In the end, the main Definition has to be cleaned of obsolete imports, namespace declcarations. 
	* The method will process the target of the input rule
	*/
	private void processChildTargets(TransformationRule rule){
		EObject childTarget = null;
		List childTargets = rule.getTarget();
		for (int i=0; i<childTargets.size(); i++){
			childTarget = (EObject) childTargets.get(i);
			if (childTarget instanceof PortType){
				if(wsdlDefinition.getEPortTypes().size() ==0){
					if(wsdlDefinition.getPortType((QName)((PortType)childTarget).getQName()) == null){
						wsdlDefinition.addPortType((PortType)childTarget);
					}
				}else{
					if(wsdlDefinition.getPortType((QName)((PortType)childTarget).getQName()) == null){
						createAndUpdateCallbackDefinition((PortType)childTarget);
					}
				}
			} else if (childTarget instanceof PartnerLinkType){
				wsdlDefinition.addExtensibilityElement((PartnerLinkType)childTarget);			
			//the child target of this rule can be a Definition containing Services and Bindings in this case propagate 
			}else if (childTarget instanceof Definition){
				if((Definition)childTarget != wsdlDefinition){
					getTarget().add(childTarget);
				}
			}
		}	
	}
	/**
	 * Creates the WSDL Callback Definition containing the callback <code>portType</code>.
	 * Will create a new WSDL Definition, updates it's name and targetNamespace, add the callback PortType and all its necessary Messages, XSD Definitions.
	 * Also cleans the main WSDL definition from obsolete XSDImports and unused namespaces.
	 * @param portType WSDL callback PortType
	 */
	private void createAndUpdateCallbackDefinition(PortType portType){
		WSDLDefinitionRule wsdlDefRule = WsdlFactory.eINSTANCE.createWSDLDefinitionRule();
		getChildRules().add(wsdlDefRule);
		wsdlDefRule.getSource().addAll(getSource());
		wsdlDefRule.transformSource2Target();
		wsdlCallbackDefinition = (Definition)wsdlDefRule.getTarget().get(0);
		//the name of the callback wsdl definition is the name of the wsdl definition + callback suffix
		String wsdlName = wsdlDefinition.getQName().getLocalPart();
		wsdlName = wsdlName + "Callback";
		//get the targetNamespace for the wsdl callback portType
		AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.WSDL_CALLBACK_PORT_TYPE_TARGET_NAMESPACE);
		valueProvider.setContext(this.getContext());
		String encodedNamespace = (String)valueProvider.getAttributeValueForType(wsdlDefinition, (NamedElement)getSource().get(0), AttributeNameConstants.WSDL_CALLBACK_PORT_TYPE_TARGET_NAMESPACE, NamingUtil.findRegistry(this));
		
		wsdlCallbackDefinition.setTargetNamespace(encodedNamespace);
		//set the name of the WSDL Definition
		QName qName = new QName(encodedNamespace, wsdlName);
		wsdlCallbackDefinition.setQName(qName);

//		wsdlCallbackDefinition.setTargetNamespace(wsdlCallbackDefinition.getQName().getNamespaceURI() + "Callback");
//		QName qName = new QName(wsdlCallbackDefinition.getQName().getNamespaceURI() + "Callback", wsdlName);
//		wsdlCallbackDefinition.setQName(qName);
		
		//replace tns with the actualTargetNamespace
		wsdlCallbackDefinition.getNamespaces().remove("tns");
		wsdlCallbackDefinition.addNamespace("tns", wsdlCallbackDefinition.getTargetNamespace());
								
		getTarget().add(wsdlCallbackDefinition);
		//change the PortType namespace
		portType.setQName(new QName(wsdlCallbackDefinition.getTargetNamespace(), portType.getQName().getLocalPart()));
		//add wsdlCallbackDefinition elements.	
		wsdlCallbackDefinition.addPortType(portType);
		portType.setElement(null);
		addPortTypeElements(portType, wsdlCallbackDefinition);			
		//cleanup the main wsdlDefinition from obsolete elements
		removeObsoleteFromWSDLDefinition();
		
	}
	
	/**
	 * Attempts to move all PortType elements -> messages, xsd elements to the wsdlCallbackDefinition. 
	 * @param portType
	 */
	private void addPortTypeElements(PortType portType, Definition wsdlDefinition){
		XSDSchema types = getSchemaForDefinition(wsdlCallbackDefinition);
		((DefinitionImpl)wsdlCallbackDefinition).updateElement();
		List messages = addMessagesForPortType(portType, wsdlDefinition);	
		addElementsForMessages(portType, messages, wsdlDefinition);  
	    //Create DOM representation for the callbackDefinition
		((DefinitionImpl)wsdlCallbackDefinition).updateElement();
		
	}
	
	/**
	 * Add all elementDeclarations, XSDImports and registers the required namespaces with the <code>newDefinition</code>
	 * @param portType a WSDL PortType (callback PortType)
	 * @param messages list of WSDL Messages belonging to the PortType.
	 * @param newDefinition callback PortTypeDefinition
	 * @return
	 */
	private void addElementsForMessages(PortType portType, List messages, Definition newDefinition){
	
		List xsdElements = new LinkedList();
		XSDSchema types = getSchemaForDefinition(newDefinition);
		

		XSDSchema oldTypes = getSchemaForDefinition(wsdlDefinition);   
		Iterator it;
		if(messages != null && !messages.isEmpty()){
			it = messages.iterator();
			while(it.hasNext()){
				org.eclipse.wst.wsdl.Message message = (org.eclipse.wst.wsdl.Message)it.next();
				Part part = (Part)message.getEParts().get(0);
				XSDElementDeclaration declaration = part.getElementDeclaration();
				//change the ElementDeclaration namespace
				declaration.setTargetNamespace(newDefinition.getTargetNamespace());
				
				oldTypes.getContents().remove(declaration);
				//Disconnect the EMF model from the DOM model
				declaration.setElement(null);			
				part.setElement(null);
				//change the enclosing definition to avoid DOMException
				part.setEnclosingDefinition(message.getEnclosingDefinition());
				message.setElement(null);
				wsdlCallbackDefinition.addMessage(message);
			
				declaration.setTargetNamespace(newDefinition.getTargetNamespace());
				types.getContents().add(declaration);
				xsdElements.add(declaration);		
			}
		}
		
		//now add the imports :
		List requiredNamespaces = new LinkedList();
		
		List imports = ProcessUtil.getRegisteredTypesForPortType(getContext(), portType);
		if(imports != null && !imports.isEmpty()){
			it = imports.iterator();
			while(it.hasNext()){
				XSDImport thisImport = (XSDImport)it.next();
				XSDImport copyImport =(XSDImport)EcoreUtil.copy(thisImport);
				copyImport.setElement(null);
				requiredNamespaces.add(copyImport.getNamespace());
				types.getContents().add(0, copyImport);
				
			}
		}
		Map map = oldTypes.getQNamePrefixToNamespaceMap();
		Set set = map.keySet();
		it = set.iterator();
		while(it.hasNext()){
			Object key = it.next();
			Object entry = map.get(key);
			if(requiredNamespaces.contains(entry))
			types.getQNamePrefixToNamespaceMap().put((String)key, (String)entry);	
		}
		
	}
	
	/**
	 * Returns the inline XSDSchema for this <code>definition</code>. If none exists, one will be created. 
	 * The new schema will have the xsd and targetNamespace registered.
	 * @param definition a WSDL Definition.
	 * @return the inline XSDSchema (types)
	 */
	private XSDSchema getSchemaForDefinition (Definition definition){
		
		XSDSchema xsdSchema = null;
		Types types = definition.getETypes();
		if(types == null){
			types = WSDLFactory.eINSTANCE.createTypes();
			definition.setTypes(types);
		}
		
		List extElements = types.getSchemas();
		if(extElements.isEmpty()){
			xsdSchema = XSDFactory.eINSTANCE.createXSDSchema();
			xsdSchema.setTargetNamespace(definition.getTargetNamespace());
			xsdSchema.setSchemaForSchemaQNamePrefix("xsd");
			Map namespaces = xsdSchema.getQNamePrefixToNamespaceMap();
			namespaces.put(xsdSchema.getSchemaForSchemaQNamePrefix(), XSDConstants.SCHEMA_FOR_SCHEMA_URI_2001);
			namespaces.put("tns", definition.getTargetNamespace());
			XSDSchemaExtensibilityElement xsdExtensibilityElement = WSDLFactory.eINSTANCE.createXSDSchemaExtensibilityElement();
			types.getEExtensibilityElements().add(xsdExtensibilityElement);
			xsdExtensibilityElement.setSchema(xsdSchema);
			
		}else{
			xsdSchema = (XSDSchema)types.getSchemas().get(0);
		}
		return xsdSchema;
	}
	
	/**
	 * Retrieves all WSDL Messages defined by the <code>PortType</code> to the <code>newDefinition</code>
	 * @param portType a WSDL PortType. (callback PortType)
	 * @param newDefinition a WSDL Definition (Callback definition)
	 * @return List WSDL Messages
	 */
	private List addMessagesForPortType(PortType portType, Definition newDefinition){
		
		List messages = new LinkedList();
		Iterator it;
		List operations = portType.getEOperations();
		if(operations != null && !operations.isEmpty()){
			it = operations.iterator();
			while(it.hasNext()){
				Operation operation = (Operation)it.next();
				org.eclipse.wst.wsdl.Input input = operation.getEInput();
				if(input != null){
					org.eclipse.wst.wsdl.Message message = input.getEMessage();
					if(message != null){
						//change the Message name namespace
						message.setQName(new QName(newDefinition.getTargetNamespace(), message.getQName().getLocalPart()));
						message.setEnclosingDefinition(portType.getEnclosingDefinition());
						//portType.getEnclosingDefinition().addMessage(message);
						//wsdlCallbackDefinition.addMessage(message);
						//Remove the message from the main interface since it will be added to the callback interface
						this.wsdlDefinition.getEMessages().remove(message);
						messages.add(message);						
					}
				}
				org.eclipse.wst.wsdl.Output output = operation.getEOutput();
				
				if(output != null){
					//Message message = output.getMessage();
					org.eclipse.wst.wsdl.Message message = output.getEMessage();
					if(message != null){
						//change the message name namespace
						message.setQName(new QName(newDefinition.getTargetNamespace(), message.getQName().getLocalPart()));
						message.setEnclosingDefinition(portType.getEnclosingDefinition());
						//portType.getEnclosingDefinition().addMessage(message);
					//	wsdlCallbackDefinition.addMessage(message);
						messages.add(message);
					}
				}
			}
		}
		return messages;
		
	}
	
	/**
	 * Removes unused XSDImports and namespace declaration from the <code<wsdlDefinition</code>
	 *
	 */
	private void removeObsoleteFromWSDLDefinition(){
		
		PortType portType = (PortType)wsdlDefinition.getEPortTypes().get(0);
		List imports = ProcessUtil.getRegisteredTypesForPortType(getContext(), portType);
		XSDSchema types = getSchemaForDefinition(wsdlDefinition);
		Map map = types.getQNamePrefixToNamespaceMap();
		
		List requiredNamespaces = new LinkedList();
		requiredNamespaces.add(XSDConstants.SCHEMA_FOR_SCHEMA_URI_2001);
		requiredNamespaces.add(wsdlDefinition.getTargetNamespace());
		

		Iterator it;
		List contents = types.getContents();
		List obsoleteImports = new LinkedList();
		it = contents.iterator();
		while(it.hasNext()){
			Object obj = it.next();
			if(obj instanceof XSDImport && (!(requiredNamespaces.contains(((XSDImport)obj).getNamespace()) || (imports!= null && imports.contains(obj))))){
				obsoleteImports.add(obj);
				//types.getContents().remove(obj);
			}
		}
		types.getContents().removeAll(obsoleteImports);		
		Set set = map.keySet();
		it = set.iterator();
		List keys = new LinkedList();
		while(it.hasNext()){
			Object key = it.next();
			Object entry = map.get(key);
			if(!requiredNamespaces.contains(entry))
			keys.add(key);	
		}
//JR26661 - do not remove xsd imports		
//		if(!keys.isEmpty()){
//			it = keys.iterator();
//			while(it.hasNext()){
//				map.remove(it.next());
//			}
//		}
		
	}

	public Definition getWsdlCallbackDefinition() {
		return wsdlCallbackDefinition;
	}
	
	/**
	 * Create a skeleton WSDL definition to represent the process's interface, which implements the business service
     * as process variation
	 * @param originalDefinition
	 * @return
	 */
	private Definition createSkeletonWSDLDefinitionForProcessVariation(Definition originalDefinition) {
		Definition wsdlDefinition = null;	
		if(originalDefinition != null){
			wsdlDefinition = WSDLFactory.eINSTANCE.createDefinition();
			QName qname = null;
			if(originalDefinition.getQName() != null){
				qname = new QName(originalDefinition.getQName().getNamespaceURI(),originalDefinition.getQName().getLocalPart());
			}else{
				String ns = originalDefinition.getTargetNamespace();
				String localName = null;
				if(ns.indexOf("/") != -1){
					localName = ns.substring(ns.lastIndexOf("/"));
				}else if(ns.indexOf(":") != -1){
					localName = ns.substring(ns.lastIndexOf(":"));
				}else{
					//shoudn't happen
					localName = "SCALFOLDDEFINITION";
				}
				qname = new QName(ns,localName);
			}
			wsdlDefinition.setQName(qname);		
			
			wsdlDefinition.setTargetNamespace(originalDefinition.getTargetNamespace());
			wsdlDefinition.addNamespace("tns", originalDefinition.getTargetNamespace());
			wsdlDefinition.addNamespace("xsd", XSDConstants.SCHEMA_FOR_SCHEMA_URI_2001);
			wsdlDefinition.addNamespace("wsdl", "http://schemas.xmlsoap.org/wsdl/");
			wsdlDefinition.setLocation(originalDefinition.getLocation());
		}
		
		return wsdlDefinition;	
	}
	/**
	 * If the interface rule's parent rule's source is a process and implements a business service and
     * the service operation has been transformed, we need to create a skeleton wsdl definition to fake
     * the process's interface definition, after creating the process shallow artifact wsdl, we skip the
     * persistence of the forfeited wsdl
	 * @return
	 */
	private boolean shouldCreateScalfoldDefinition(){
		if(getParentRule() instanceof ProcessFlowRuleImpl){
			ProcessFlowRuleImpl pflImpl = (ProcessFlowRuleImpl)getParentRule();
			boolean isServiceOperation = false;
			if(pflImpl.getSource().get(0) instanceof Activity){
				Activity activity = (Activity)pflImpl.getSource().get(0);
				Activity interfceActivity = FabricIntegrationUtil.getInterfaceActivityIfAny(activity.getImplementation());
				if(interfceActivity != null){
					isServiceOperation = BomUtils.isServiceOperation(interfceActivity.getImplementation());
				}
			}
			if(pflImpl.isInterfaceActionTransformed() && isServiceOperation){
				return true;
			}
		}
		return false;
	}

} //ProcessInterfaceRuleImpl
