/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EClass;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.actions.CallOperationAction;
import com.ibm.btools.bom.model.processes.actions.Decision;
import com.ibm.btools.bom.model.processes.actions.Fork;
import com.ibm.btools.bom.model.processes.actions.Join;
import com.ibm.btools.bom.model.processes.actions.Map;
import com.ibm.btools.bom.model.processes.actions.Merge;
import com.ibm.btools.bom.model.processes.actions.RepositoryAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Datastore;
import com.ibm.btools.bom.model.processes.activities.FlowFinalNode;
import com.ibm.btools.bom.model.processes.activities.InputControlPin;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.LoopNode;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputControlPin;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.StoreArtifactPin;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.activities.TerminationNode;
import com.ibm.btools.bom.model.processes.activities.Variable;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AssignRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.DatastoreAccessRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.DecisionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ForkRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.GenericActionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.JoinRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.LinkRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.LoopNodeRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.MapRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.MergeRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessWalkingRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.VariableAccessRule;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BomWalker;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Scope;

/**
 * <!-- begin-user-doc -->
 * This class provides common functions for rules that need to walk through BOM process.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public abstract class ProcessWalkingRuleImpl extends ConnectableRuleImpl implements ProcessWalkingRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	// entryInputPinSet is to keep track of from which InputPinSet it starts traversing the process flow
	protected InputPinSet entryInputPinSet=null;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ProcessWalkingRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.PROCESS_WALKING_RULE;
	}

	/**
	 * Steps through BOM elements starting from the <code>inputSet</code> 
	 * and creates child rules while stepping through from Input Set to Input Set.
	 * <ul>
	 * <li> If the rule for the inputSet exist, re-executes the rule and return null to indicate child rule is not created.
	 * <li> If the rule doesn't exist, creates child rule depending on the Action that the inputSet is associated with.
	 * 		For example, create SANTaskRule if the inputSet is on a Task. 
	 * <li>	Executes the child rule and return the InputPinSet.
	 * <ul>	  
	 * @param inputSet - BOM InputPinSet starting the trace through
	 * @return InputPinSet - the InputPinSet passed in argument or null. null indicates no new child rule is created. 
	 */

	protected  InputPinSet stepThroughBOM(InputPinSet inputSet){
		TransformationRule tr = getRuleForInputPinSet(inputSet);
		if(tr != null){
			// if one does exist, this input set has already been mapped, but we must execute it again so that the links that have been created that point to the input set can have their target set.
			tr.transformSource2Target(); 
			//Return null to indicate that we did not create a new child rule.
			return null;
		}
		
		/* Otherwise, if a rule for this input set does not yet exist,
		 * Based on the type of action associated with inputSet, a transformation rule must be created.  
		 * But first, we must determine whether the action inputSet either an exclusive branch point or a concurrent branch point.  
		 * This is important because if it is a branch point, we do not create the links immediately following the action, 
		 * but instead, leave the child rule to create those links itself. 
		 */
		//Check whether the Action associated with inputSet is a branch point 	
		boolean isEBranch = false;
		boolean isCBranch = false;
		OutputPinSet outputSet;
		
		isEBranch = BomHelper.getInstance().isEBranch(inputSet);
		if(!isEBranch){
			BomWalker bw = ProcessUtil.getBomWalker(getContext());
			//if inputSet is not an exclusive branch, then the there should never be multiple output set returned from following method
			
			/* We MUST set the jumping flag to TRUE in the next call.  We know that inputSet is not an E-branch, so the
			 * only option left is that it is a C-Branch, or is neither.  In the case of all actions other than a SAN, 
			 * the output set returned from the given input set will be the same, jumping or not, since in a non-compound
			 * action that output set is right next to the input set.  (Even if it's a C-Branch it won't matter, because
			 * the C-Branch starts at the output set).  However, if its a nested SAN, we don't want to step inside
			 * because the internals of the SAN will be mapped by its own rule.  Therefore, we must jump. 
			 */
			 // Ray May 06/04: Try to handle inputSet with no outputSet assoication
//			outputSet = (OutputPinSet)bw.getNext(inputSet, BomWalker.OUTPUT_SET, true).get(0);
//			isCBranch = BomHelper.getInstance().isCBranch(outputSet);
			 List outputSets = bw.getNext(inputSet, BomWalker.OUTPUT_SET, true);
			 if (outputSets!=null && outputSets.size()>0) {
			 	outputSet = (OutputPinSet)outputSets.get(0);
				isCBranch = BomHelper.getInstance().isCBranch(outputSet);
			 } else {
			 	outputSet = null;
			 	isCBranch = false;
			 }


//			outputSet = (OutputPinSet)bw.getNext(inputSet, BomWalker.OUTPUT_SET, false).get(0); 
			
//Ray May 06.04			if(!isCBranch) {			
			if(!isCBranch && outputSet!=null) {
				/*If it is neither of the two types of branch, then this Rule is 
				//responsible for creating the links that extend from the action that contains inputSet.  
				//This will involve creating either a LinkRule or an AssignRule for 
				//each pin in the output set that is correlated to is.  
				//(We know that there is only one output set correlated to is, 
				//and also that the output set only has one pin, 
				//because it is neither an exclusive nor concurrent branch point). 
				*/
				
				//Get the only output pin, if one exists 
				Pin outPin = null;
				NamedElement targetSet = null;
				if (outputSet.getOutputControlPin().size() + outputSet.getOutputObjectPin().size() == 0){
					outPin = null;
				} else {
					if (outputSet.getOutputObjectPin().isEmpty()) {
						outPin=(Pin)outputSet.getOutputControlPin().get(0);
					} else {
						// Ray: There should only be one object pin but need to filter all StoreArtifactPins
						for (int i=0; i<outputSet.getOutputObjectPin().size(); i++) {
							if (!(outputSet.getOutputObjectPin().get(i) instanceof StoreArtifactPin)) {
								outPin=(OutputObjectPin)outputSet.getOutputObjectPin().get(i);
								break;
							}
						}
					}
//					outPin =(outputSet.getOutputObjectPin().isEmpty()) 
//								? ((Pin)outputSet.getOutputControlPin().get(0)) 
//								: ((Pin)outputSet.getOutputObjectPin().get(0)); //only one expected
					if (outPin != null) //Ray
						targetSet = BomUtils.findTargetSet(outPin.getOutgoing());
				} 	  
	
				if (outPin == null || outPin.getOutgoing() == null || outPin.getOutgoing().getTarget() instanceof FlowFinalNode){
					/* Do nothing if there is no outgoing edge from pin */
					// Ray: Need to connect empty outputPinSet or outputPinSet with pins with no outgoing edge
					// to the default outputCriteria
					//OutputPinSet outPinSet=findDefaultOutputPinSet(inputSet);
					List outputPinSets = findDefaultOutputPinSet(inputSet);
					//if (outPinSet != null) {
					if (outputPinSets.size()>0) {
						OutputPinSet outPinSet;
						for (Iterator it=outputPinSets.iterator();it.hasNext();) {
							outPinSet = (OutputPinSet)it.next();
							LinkRule linkRule = AbstractbpelFactory.eINSTANCE.createLinkRule();
//							if (!inputSet.getOutputPinSet().isEmpty()) {
//								linkRule.getSource().addAll(inputSet.getOutputPinSet());
//							} else {
								List cOutputPinSets = BomUtils.getCorrelatedPinSets(inputSet);
								if (cOutputPinSets!=null && cOutputPinSets.size()>0) {
									linkRule.getSource().addAll(cOutputPinSets);
								}
//							}
							//OutputPinSet outPinSet = (OutputPinSet)inputSet.getAction().getInStructuredNode().getOutputPinSet().get(0);
							linkRule.getSource().add(outPinSet);
							getChildRules().add(linkRule);
							linkRule.transformSource2Target();					
						}
					}
				} else if (outPin.getOutgoing().getTarget() instanceof TerminationNode){
					/* If the pin is connected to a termination node, we must create the LinkRule */
					//LinkRule linkRule = createTerminationLinkRule(outPin);	
					LinkRule linkRule = createTerminationLinkRule(outPin,inputSet);
					getChildRules().add(linkRule);
					linkRule.transformSource2Target();
				} else if (outPin.getOutgoing().getTarget() instanceof Pin) {
					boolean assignCallOperationActionTarget = false;
				/* If the current edge is connected to the 'target' pin of a CallOperationAction, just
					 * create a link */
					if(outPin.getOutgoing().getTarget() instanceof InputObjectPin){
						if(((InputObjectPin)outPin.getOutgoing().getTarget()).getAction() instanceof CallOperationAction){
							CallOperationAction coa = (CallOperationAction)((InputObjectPin)outPin.getOutgoing().getTarget()).getAction();
							if(coa.getTarget() != null){
								if(coa.getTarget() == ((InputObjectPin)outPin.getOutgoing().getTarget())){
									assignCallOperationActionTarget = true;
								}
							}
						} 
					} 
								
//					if( (outPin instanceof ObjectPin) 
//							   && BomHelper.getInstance().isNextSetCompatible(outputSet)== null 
//							   && !assignCallOperationActionTarget){ 
//					if( (outPin instanceof ObjectPin) 
//												   && BomHelper.getInstance().isNextSetCompatible(outputSet, getContext())== null ){ 
					if( (outPin instanceof ObjectPin && false)){ 	
					//still need to check whether need to create one				
						if(needAssignRule(outPin, outputSet, targetSet)){
							//not compatible inputSet outputSet
							AssignRule ar = AbstractbpelFactory.eINSTANCE.createAssignRule();
							//Set the source of ar to be the BOM ActivityEdge connected to o as well as os
							ar.getSource().add(outPin.getOutgoing()); //edge
							ar.getSource().add(outputSet); //outputSet			 
							ar.getSource().add(targetSet);
							//add ar as child rule
							getChildRules().add(ar);
							ar.transformSource2Target();						
						}

					} else {  // control pin or object pin are in compatible target set, only need LinkRule						
						if (needLinkRule(outPin, outputSet, targetSet)) {			
							//then a LinkRule will be created.  
						 	LinkRule lr = AbstractbpelFactory.eINSTANCE.createLinkRule();				
						 	//Set the source of lr to be outPin, the BOM ActivityEdge connected to outPin, 
						 	//and the pin on the other end of the ActivityEdge
						 	lr.getSource().add(outPin);
						 	lr.getSource().add(outPin.getOutgoing());
						 	lr.getSource().add((Pin)outPin.getOutgoing().getTarget());
						 	lr.getSource().add(outputSet);
						 	lr.getSource().add(BomUtils.findTargetSet(outPin.getOutgoing()));
						 	getChildRules().add(lr);				
						 	lr.transformSource2Target();
						}
					}
				}
			} // if (!isCBranch)	
		} //end if (!isEbranch)	 
		
		//	Next, create the child rule depending on the type of action.  The source of the child rule will be inputSet.
		//	Call the following method to create the child rule based on the type of action the contains isNext
		TransformationRule ctr = BomHelper.getInstance().createChildRule(inputSet);

		if (ctr!= null){		
			//Add the child rule to this rule
	    	getChildRules().add(ctr);
			ctr.transformSource2Target();
		}
		    
		return inputSet;
		
	}

	/**
	 * For the purpose of supporting implicit termination.  Find the first outputPinSet of the containing SAN (parent SAN)
	 * that is not an exception output set and it is correlated with an inputPinSet.  
	 * @param inputSet
	 * @return
	 */
	protected List findDefaultOutputPinSet (PinSet pinSet) {
		OutputPinSet outPinSet;
		List ret = new ArrayList();
		List oss;
		if (pinSet instanceof InputPinSet) {
			oss = ((InputPinSet)pinSet).getAction().getInStructuredNode().getOutputPinSet();
		} else {
			oss = ((OutputPinSet)pinSet).getAction().getInStructuredNode().getOutputPinSet();
		}
		for (Iterator it=oss.iterator();it.hasNext();) {
			outPinSet = (OutputPinSet)it.next();
			if (!outPinSet.getIsException().booleanValue()) {
				InputPinSet entryInPinSet = getEntryInputPinSet(pinSet);
				if (entryInPinSet!=null && outPinSet.getInputPinSet().contains(entryInPinSet)) {
					ret.add(outPinSet);
				}
			}
		}
		return ret;

	}
	
/**
 * Returns true if it is necessary to create new AssignRule for the pin.
 * This is needed to handle the situation where the pin is on a Merge action, 
 * we don't want create multiple Assign Rules for the pin's edge.
 * @param pin  the ObjectPin or ControlPin the link is the source
 * @param sourceSet  the OutputPinSet that the pin is in
 * @param targetSet  the InputPinSet or OutputPinSet that the pin's edge targeting to  
 * @return boolean
 */
 protected boolean needAssignRule(Pin pin, OutputPinSet sourceSet, NamedElement targetSet){
 	//TODO AR new
 	return false;
 	
//// 	if(sourceSet.getAction() instanceof Merge){
// 		//check if AssignRule exist
//		Collection sources = new ArrayList();
//		sources.add(pin.getOutgoing());
//		sources.add(sourceSet);
//		sources.add(targetSet);
// 		AssignRule rule = (AssignRule)TransformationUtil.getRuleForSource(ProcessUtil.getProcessDefinitionRule(getContext()),
//						  AssignRule.class,
//						  sources,
//						  Assign.class);
//		if(rule != null)
//			return false;							   		
//// 	}	
// 	
// 	return true;		
 }
 
 
 /**
  * Returns true if it is necessary to create new LinkRule for the pin.
  * This is needed to handle the situation where the pin is on a Merge action, 
  * we don't want create multiple Link rule for the pin's edege.
  * @param pin  the ObjectPin or ControlPin, the link is the source of
  * @param sourceSet  the OutputPinSet, the pin is in
  * @param targetSet  the InputPinSet or OutputPinSet, the pin's edge targeting to 
  * @return boolean
  */
  protected boolean needLinkRule(Pin pin, OutputPinSet sourceSet, NamedElement targetSet){
	 if(sourceSet.getAction() instanceof Merge){
		 //check if LinkRule exist
		 Collection sources = new ArrayList();
		 sources.add(pin);
		 sources.add(sourceSet);
		 sources.add(pin.getOutgoing());
		 LinkRule rule = (LinkRule)TransformationUtil.getRuleForSource(ProcessUtil.getProcessDefinitionRule(getContext()),
						   LinkRule.class,
						   sources,
						   Link.class);
		 if(rule != null)
		   return false;							   		
	 }	
 	
	 return true;		
  } 
  
	/**
	 * Creates the LinkRule that represents the link from an output pin to a termination node.
	 * This method does not execute the resulting LinkRule.  If one already exists, it is returned
	 * and a new one is not created.
	 * 
	 * @param pin An input or output control or object pin that connects to a termination node.
	 * @return LinkRule The resulting link rule
	 */
	protected LinkRule createTerminationLinkRule(Pin pin,InputPinSet pinSet){
		List sources = new ArrayList();
		
		sources.add(pin);
		sources.add(pin.getOutgoing());
		sources.add(pin.getOutgoing().getTarget());
		/* The pin could be an input pin if it is in the input set of a nested SAN */
		if (pin instanceof InputControlPin || pin instanceof InputObjectPin){
			sources.addAll(BomUtils.getInputPinSets(pin));	
		} else if (pin instanceof OutputControlPin || pin instanceof OutputObjectPin){
			sources.addAll(BomUtils.getOutputPinSets(pin));
		}
		
		/* Add the output set that the termination node is associated to */
		if (((TerminationNode)pin.getOutgoing().getTarget()).getOutcome()!=null) { // Ray: in the case where the termination node has no outcome
			sources.add( ((TerminationNode)pin.getOutgoing().getTarget()).getOutcome());
		} else {
			// Ray: If there is no Outcome defined for the Target, then assume the first OutputPinSet as the outcome
			StructuredActivityNode san = ((TerminationNode)pin.getOutgoing().getTarget()).getInStructuredNode();
			sources.add(san.getOutputPinSet().get(0));
		}

		// Ray - need to support the case where there is a Task which has multiple inputPinSet correlates
		// to the same outputPinSet and the pin in outputPinSet connects to the termination node
		// We need to have the InputPinSet to be the source as well so that we can distinguish which link
		// corresponds to which BPEL Invoke activity representing the Task inputPinSet. 
		if (pinSet != null) {
			sources.add(pinSet);
		}

		/* Find a currently existing rule, if one exists */
		TransformationRule linkRule = 
			TransformationUtil.getRuleForSource(ProcessUtil.getProcessDefinitionRule(getContext()), sources, Link.class);

		if (!(linkRule instanceof LinkRule)){
			linkRule = AbstractbpelFactory.eINSTANCE.createLinkRule();
			linkRule.getSource().addAll(sources);	
		}

		return (LinkRule)linkRule;				
	}
  
	 /**
	  * Returns the rule already created for the inputSet. 
	  * Search starts from the process definition rule.
	  * @param inputSet  BOM InputPinSet that is the source of the rule 
	  * @return TrnaformationRule - the rule for the inputSet or null if not found
	  */
	 private TransformationRule getRuleForInputPinSet(InputPinSet inputSet){
		//see if a rule already exists for this input set.
		Class targetType = null;
		Class ruleType = null;
		Action sourceAction = inputSet.getAction();

		/* Note: Using the RuleType is more reliable than using the targetType */
	
		if(BomUtils.isTask(sourceAction) || BomUtils.isBusinessRuleTask(sourceAction) || BomUtils.isHumanTask(sourceAction) || sourceAction instanceof CallBehaviorAction || sourceAction instanceof CallOperationAction){
			targetType = Invoke.class;
	
		} else if(BomUtils.isProcess(sourceAction)) {
			targetType = Scope.class;
	
	 	} else if(sourceAction instanceof Decision) {		    
			ruleType = DecisionRule.class;
	
		} else if(sourceAction instanceof Merge) {
			ruleType = MergeRule.class;
	
		} else if(sourceAction instanceof Fork) {
			ruleType = ForkRule.class;
	
		} else if(sourceAction instanceof Join) {
			ruleType = JoinRule.class;		
			
		} else if(sourceAction instanceof Map) {
			ruleType = MapRule.class;		
			
		} else if(sourceAction instanceof LoopNode) {
			ruleType = LoopNodeRule.class;					
	
		} else if (sourceAction instanceof RepositoryAction){
			if (((RepositoryAction)sourceAction).getRepository() instanceof Variable){
				ruleType = VariableAccessRule.class;
			} else if (((RepositoryAction)sourceAction).getRepository() instanceof Datastore){
				ruleType = DatastoreAccessRule.class;
			}
			
		} else { 
			//TODO: more types here
			//ruleType = null;
			//targetType = null;
			// Ray 04/27/04: Support of default rule type for non-supported Action
			ruleType = GenericActionRule.class;
		}
	 		
	 	TransformationRule tr = null;
	 			
	 	if (ruleType != null){
			tr = TransformationUtil.getRuleForSource(
											ProcessUtil.getProcessDefinitionRule(getContext()), 
											ruleType,
											inputSet);
	
	 	} else if (targetType != null){
			tr = TransformationUtil.getRuleForSource(
											ProcessUtil.getProcessDefinitionRule(getContext()), 
											inputSet, 
											targetType);
	 		
	 	}
		
		return tr;
	}

	/**
	 * @return
	 */
	public InputPinSet getEntryInputPinSet(PinSet pinSet) {
		// if no entryInputPinSet that has explicitly set, attempt to find the closest parent rule that is a ProcessRule.  The source of the rule is the entryInputPinSet
		// This is not nice but until we have a better solution ...
		if (entryInputPinSet==null) {
			TransformationRule parentRule=getParentRule(),rootRule=getRoot();
			Action action;
			if (pinSet instanceof InputPinSet) {
				action = ((InputPinSet)pinSet).getAction();
			} else {
				action = ((OutputPinSet)pinSet).getAction();				
			}
			while (parentRule!=rootRule && !(parentRule instanceof ProcessRule && ((InputPinSet)parentRule.getSource().get(0)).getAction()==action.getInStructuredNode())) {
				parentRule=parentRule.getParentRule();
			}
			if (parentRule instanceof ProcessRule) {
				setEntryInputPinSet((InputPinSet)parentRule.getSource().get(0));
			}
		}
		return entryInputPinSet;
	}

	/**
	 * @param set
	 */
	public void setEntryInputPinSet(InputPinSet set) {
		entryInputPinSet = set;
	}

} //ProcessWalkingRuleImpl
