/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.PortType;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.LoopNode;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessInterfaceRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.SANNestedProcessRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.util.PinSetRegistryUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.util.ScopeRegistryUtil;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.CycleLinkUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.PortTypeRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.WsdlFactory;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.Flow;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Links;
import com.ibm.wbit.bpel.PartnerLink;
import com.ibm.wbit.bpel.Scope;
import com.ibm.wbit.bpel.Source;
import com.ibm.wbit.bpel.Sources;
import com.ibm.wbit.bpel.Target;
import com.ibm.wbit.bpel.Targets;
import com.ibm.wbit.bpel.Variable;

/**
 * <!-- begin-user-doc -->
 * Source: BOM InputPinSet of Nested Process
 * target: BPEL Scope
 * 
 * Maps a BOM Nested Process to a BPEL scope.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class SANNestedProcessRuleImpl extends ConnectableRuleImpl implements SANNestedProcessRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	/**
	 * Creates container rules for the input set and correlated output sets
	 *
	 */
	private void createContainerRule(){
		 InputPinSet inputSet = (InputPinSet)getSource().get(0);
		//create container rule for input set

		if (!inputSet.getInputObjectPin().isEmpty()) {
			for(int i=0; i<inputSet.getInputObjectPin().size(); i++){
				createVariable((NamedElement)inputSet.getInputObjectPin().get(i));
			}
		}
		  
	    // here implement correlation to multiple output set
		  List osList = inputSet.getOutputPinSet();
		  OutputPinSet nextOS;
		  for(int i= 0, count = osList.size(); i<count; i++){
			nextOS = (OutputPinSet)osList.get(i);
			if (!nextOS.getOutputObjectPin().isEmpty()) {
				for(int j=0;j<nextOS.getOutputObjectPin().size(); j++){
					createVariable((NamedElement)nextOS.getOutputObjectPin().get(j));
				}
			}
		  }
	}
	/* attribute indicates whether it is first time execution of this rule */
	private boolean firstTimeExec = true;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected SANNestedProcessRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.SAN_NESTED_PROCESS_RULE;
	}

	/**  
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		/*
		 * 1. create BPEL scope.
	 	 * 2. create container for inputSet and correlated outputSets.
	     * 3. create ProcessRule as child rule to map elements in the nested process.
	     * 4. consume and propagate the targets produced by the process rule.
		 */
		
		//assertion
		if (getSource() == null || getSource().isEmpty()) {
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
			return false;			
		}
		//first source of this rule is one input set
		InputPinSet inputSet = (InputPinSet)getSource().get(0);
	  	
		//check if this is first time execute this rule
		 if(!firstTimeExec){
			reExecute(inputSet); 
			// Need to connect the scope with preceding BPEL activity
			Scope scope = (Scope)getTarget().get(0);
			connectScope(scope,inputSet);
			
//			executeHandlers();
			LoggingUtil.traceExit(this, "transformSource2Target", "re-execute");			
			return true;
		 }
		 		
		firstTimeExec = false;
		// creating the interface for local process or loop with multiple pins in exception pin set
		if ((inputSet.getAction() instanceof LoopNode || BomUtils.isLocalProcess(inputSet.getAction())) &&
				ScopeRegistryUtil.hasExceptionWithMultiPins(inputSet.getAction())) {//has multiple pins in exception pin set
			ProcessInterfaceRule processIntRule = createProcessInterfaceRule(inputSet.getAction());
			TransformationRule portTypeRule = TransformationUtil.getRuleForSource(processIntRule, PortTypeRule.class, inputSet.getAction(), PortType.class);
			if(portTypeRule == null){
				portTypeRule = WsdlFactory.eINSTANCE.createPortTypeRule();
			}
			portTypeRule.getSource().add(inputSet.getAction());
			processIntRule.getChildRules().add(portTypeRule);
			processIntRule.transformSource2Target();
			// clean up the interface - remove all unnecessary items
			if (!processIntRule.getTarget().isEmpty() && processIntRule.getTarget().get(0) instanceof Definition) {
				Definition def = (Definition)processIntRule.getTarget().get(0);
				def.getEMessages().clear();
				def.getEPortTypes().clear();
			}		
		}		
		//create BPEL Scope
	    Scope scope;
	    List inputSets = inputSet.getAction().getInputPinSet();
		SANNestedProcessRule rule=null;
	    for (Iterator it=inputSets.iterator();it.hasNext()&&rule==null;) {
			rule = (SANNestedProcessRule)TransformationUtil.getRuleForSource(
							ProcessUtil.getProcessDefinitionRule(getContext()),
							SANNestedProcessRule.class,
							(InputPinSet)it.next(),
							Scope.class);
	    }    	
		if (rule!=null) {
			scope = (Scope)rule.getTarget().get(0);
		} else {
			scope = BPELFactory.eINSTANCE.createScope();
			String convertedName = null;
			//do not set the name of the Scope in case of processing the LoopNode
			if(inputSet.getAction() instanceof LoopNode){
				NameProvider nameProvider = NameProviderFactory.getNameProvider(scope);
				convertedName = nameProvider.getName(scope, inputSet, NamingUtil.findRegistry(this));
			}else{
				AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.BPEL_ACTIVITY_NAME_SCOPE);
				valueProvider.setContext(getContext());
				convertedName = (String)valueProvider.getAttributeValueForType(scope, inputSet.getAction(), AttributeNameConstants.BPEL_ACTIVITY_NAME_SCOPE, NamingUtil.findRegistry(this));
				createDisplayName(scope, inputSet.getAction(), AttributeNameConstants.BPEL_ACTIVITY_DISPLAY_NAME_SCOPE);
			}

			scope.setName(convertedName);
		
			//createDisplayName(scope,inputSet.getAction().getName());
			mapDescriptionAndDocumentation(inputSet.getAction(),scope);			
		}
		getTarget().add(scope);
		Flow flow;
		if (scope.getActivity()!=null && scope.getActivity() instanceof Flow) {
			flow = (Flow)scope.getActivity();
		} else {
			flow=BPELFactory.eINSTANCE.createFlow();
			//defect 139867. generate name for the flow
			String convertedName = NamingUtil.getFlowName(flow, this);
			flow.setName(convertedName);
			createDisplayName(flow, convertedName);
			
			//This flow is set for subprocess loop back link support
			scope.setActivity(flow);
		}
		
		
			    
		//create process rule
		ProcessRule pr = AbstractbpelFactory.eINSTANCE.createProcessRule();
		pr.getSource().add(inputSet);
		pr.getSource().add(flow);
		//process rule must be the first child rule	   
	    getChildRules().add(pr);	    
	   
		//create container rule for input set and output sets
		createContainerRule();
	    
	    /* do not execute pr before container rule 
	     because process rule need use container produced by pr.
	     */ 
	    pr.transformSource2Target();
	    
	    	    
	    processChildTargets(scope);

		// Need to connect the scope with preceding BPEL activity
		connectScope(scope,inputSet);	
		
		executeHandlers();
		
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}
	
	/**
	 * Processes targets produced by child rules.
	 * Links, Containers and Partners are propagated to parent.
	 * Only the flow produced by the ProcessRule is added to the scope. 
	 * All other targets are propagated to parent. 
	 * Empty flows are eliminated.
	 */
	private void processChildTargets(Scope scope){
		
		List childRules = getChildRules();
		TransformationRule nextRule = null;
		List targetList= null;
		Object target;

		for(int i=0, count=childRules.size(); i<count; i++){
			nextRule = (TransformationRule)childRules.get(i);
			targetList = nextRule.getTarget();
			for (int j=0, targetCount=targetList.size(); j< targetCount; j++){
				target = targetList.get(j);
				if (target instanceof Link) {
					if (scope.getActivity() instanceof Flow) {
						Flow flow=(Flow)scope.getActivity();
						if (!AbstractbpelUtil.isLinkToExternalActivity((Link)target,flow)) {
							Links links = flow.getLinks(); 
							if (links==null) {
								links = BPELFactory.eINSTANCE.createLinks();
								flow.setLinks(links);
							}
							links.getChildren().add(target);
						} else {
							getTarget().add(target);
						}
					} else {
						getTarget().add(target);
					}
				}
				else if(target instanceof Variable || target instanceof PartnerLink){
					getTarget().add(target); //propagate all these types to parent	
				}
				else if (target instanceof Flow){
					//we need add only the flow produced by ProcessRule to scope, leave other to parent rule.
					if( target ==(Flow)((TransformationRule)getChildRules().get(0)).getTarget().get(0)){
							if(!((Flow)target).getActivities().isEmpty())
								scope.setActivity((Flow)target);
					}			
					else
						getTarget().add((Flow)target);		
				} else{
					getTarget().add(target); //propagate all other unknown type to parent
				}
			}
		}

	}	

	/**
	 * Ray Mar01/04: We want to re-execute the ProcessRule, which is the what represents the
	 * NestedProcess instead. 
	 */
	protected void reExecute(InputPinSet inputSet){
		//reExecute the ContainerRule also as there may be Input left unprocessed... 
		createContainerRule();
		if (!getChildRules().isEmpty() && getChildRules().get(0) instanceof ProcessRule) {
			((ProcessRule)getChildRules().get(0)).transformSource2Target();
		} else {
			// This should never happen!!!
			super.reExecute(inputSet);
		}
	}	
	
	private void connectScope(Scope scope, InputPinSet ips) {
		Action localSubProcess = ips.getAction();
		// Handle connection to Subprocess' inputPinSets
		Activity bpelAct = PinSetRegistryUtil.getBpelActivity(getContext(),ips);
		if (bpelAct!=null) {
			// If there is only one pinSet in the Local SubProcess, then we can connect all the incoming links to the scope
			if (localSubProcess.getInputPinSet().size()==1) {
				if( bpelAct.getTargets() != null && !bpelAct.getTargets().getChildren().isEmpty()){
					List tars = bpelAct.getTargets().getChildren();
					Targets scopeTargets = scope.getTargets();
					if(scopeTargets == null){
						scopeTargets = BPELFactory.eINSTANCE.createTargets();
						scope.setTargets(scopeTargets);
					}
						scopeTargets.getChildren().addAll(tars);
						// make sure the links are propagate to the parent
						Target tar;
						for (Iterator it=tars.iterator();it.hasNext();) {
							tar = (Target) it.next();
							getTarget().add(tar.getLink());
						}
						
				}
				setJoinCondition(ips,scope);
			} else if (localSubProcess.getInputPinSet().size()>1) {
				Flow flow = (Flow)scope.getActivity();
				if (flow.getActivities().contains(bpelAct)) {
					Link link = BPELFactory.eINSTANCE.createLink();
					link.setName(scope.getName()+bpelAct.getName());
					Source src = BPELFactory.eINSTANCE.createSource();
					Target tar = BPELFactory.eINSTANCE.createTarget();
					src.setActivity(bpelAct);
					tar.setActivity(scope);
					link.getSources().add(src);
					link.getTargets().add(tar);
					flow.getActivities().remove(bpelAct);
					getTarget().add(bpelAct); // propagate to inputSet activity to the parent flow
					getTarget().add(link);
					// also need to propagate the links of the inputSet activity to parent flow
					// just need to handle the links that has inputSet activity as the source.  
					// The targets have been handled already (see processChildTargets and isLinkToExternalAtivity)
					List srcs = bpelAct.getSources().getChildren();
					for (Iterator it=srcs.iterator();it.hasNext();) {
						src = (Source)it.next();
						getTarget().add(src.getLink());
					}
					
				}
			}
		}
		// Handle connection to Subprocess' outputPinSets
		List opss = ips.getAction().getOutputPinSet();
		if (opss.size()==1) {
			OutputPinSet ops = (OutputPinSet)opss.get(0);
			bpelAct = PinSetRegistryUtil.getBpelActivity(getContext(),ops);
			List srcs = bpelAct.getSources().getChildren();
			if(!srcs.isEmpty()){
				if(scope.getSources() == null){
					Sources sources = BPELFactory.eINSTANCE.createSources();
					scope.setSources(sources);
				}
				scope.getSources().getChildren().addAll(srcs);

				// make sure the links are propagate to the parent
				Source src;
				for (Iterator it=srcs.iterator();it.hasNext();) {
					src = (Source) it.next();
					getTarget().add(src.getLink());
				}
			}
		}
	}
} //SANNestedProcessRuleImpl
