/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.blm.ie.exprt.option.BpelOptions;
import com.ibm.btools.bom.model.artifacts.Constraint;
import com.ibm.btools.bom.model.artifacts.InstanceSpecification;
import com.ibm.btools.bom.model.artifacts.LiteralString;
import com.ibm.btools.bom.model.artifacts.Slot;
import com.ibm.btools.bom.model.processes.humantasks.EscalationAction;
import com.ibm.btools.bom.model.processes.humantasks.HumanTask;
import com.ibm.btools.bom.model.resources.BulkResourceRequirement;
import com.ibm.btools.bom.model.resources.IndividualResource;
import com.ibm.btools.bom.model.resources.IndividualResourceRequirement;
import com.ibm.btools.bom.model.resources.RequiredRole;
import com.ibm.btools.bom.model.resources.ResourceRequirement;
import com.ibm.btools.bom.model.resources.Role;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.expression.model.BooleanLiteralExpression;
import com.ibm.btools.expression.model.Expression;
import com.ibm.btools.expression.model.FunctionArgument;
import com.ibm.btools.expression.model.FunctionExpression;
import com.ibm.btools.expression.model.ModelPathExpression;
import com.ibm.btools.expression.model.ReferenceStep;
import com.ibm.btools.expression.model.StringLiteralExpression;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.StaffActivityRule;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.HumanTaskUtil;
import com.ibm.btools.te.ilm.heuristics.helper.StaffUtil;
import com.ibm.btools.te.ilm.heuristics.helper.StaffUtil.StaffConstants;
import com.ibm.btools.te.ilm.resource.MessageKeys;
//import com.ibm.btools.te.logicalGroup.model.Group;
//import com.ibm.btools.te.logicalGroup.model.LogicalEntityRoot;
import com.ibm.wbit.tel.TPotentialOwner;
import com.ibm.wbit.tel.TStaffSettings;
import com.ibm.wbit.tel.TVerb;
import com.ibm.wbit.tel.TaskFactory;


/**
 * <!-- begin-user-doc -->
 * SOURCE: List of BOM Resource Requirements
 * TARGET: BPEL+ Staff
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class StaffActivityRuleImpl extends AbstractProcDefRuleImpl implements StaffActivityRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private boolean isBOMHumanTask=false;
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected StaffActivityRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.STAFF_ACTIVITY_RULE;
	}

	/**
	 * SOURCE: One ResourceRequirement (the case of HT) or List of BOM ResourceRequirements (Backward compatiability)
	 * TARGET: TVerb or TEL  StaffSetting
	 * SOURCE: List of BOM ResourceRequirements
	 * TARGET: TEL  StaffSetting
	 * Maps ResourceRequirements to TEL StaffSettings creating the query verb for the TEL PotentialOwner.
	 * @return boolean
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");
		//Performs as follow:
		//1. Create the BPEL Staff or a BPEL StaffSettings based on the target.
		//2. create the BPEL+ query verb based on the resource requirements source
		//3. create the BPEL Potential Owner and set the verb on the Owner
		//NOTE: BPEL reader, editor are not supported
		  //assertion
		  if (getSource() == null || getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
		  	return false;
		  }
		  if (getSource().get(0) instanceof ResourceRequirement && 
				  (((ResourceRequirement)getSource().get(0)).eContainer() instanceof HumanTask || ((ResourceRequirement)getSource().get(0)).eContainer() instanceof EscalationAction)) {
			  // This rule is triggered by BOM HumanTask transformation
			  isBOMHumanTask=true;
			  TVerb verb =TaskFactory.eINSTANCE.createTVerb(); 
			  updateQueryVerb((ResourceRequirement)(getSource().get(0)), verb);
			  getTarget().add(verb);
		  } else {
			  // This rule is triggered by BOM ResourceRequirements
			  TStaffSettings staffSettings = TaskFactory.eINSTANCE.createTStaffSettings();
			  List resourceRequirements = getSource();
			  TVerb verb =TaskFactory.eINSTANCE.createTVerb(); 
			  boolean updated = updateQueryVerb(resourceRequirements, verb);
			  if (updated){
			  	TPotentialOwner potOwner = TaskFactory.eINSTANCE.createTPotentialOwner();
				potOwner.setVerb(verb);
				staffSettings.setPotentialOwner(potOwner);
			  }
			getTarget().add(staffSettings);  
			  
		  }
		setComplete(true);
		
		executeHandlers();
		  		
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}
	
	private boolean updateQueryVerb(ResourceRequirement resourceRequirement, EObject verb){
		
		if (resourceRequirement instanceof IndividualResourceRequirement){
			IndividualResourceRequirement individual = (IndividualResourceRequirement)resourceRequirement;
			if (resourceRequirement.getOwnedConstraint()!= null && !resourceRequirement.getOwnedConstraint().isEmpty() &&
					(Constraint)resourceRequirement.getOwnedConstraint().get(0)!=null &&
					(StructuredOpaqueExpression)((Constraint)resourceRequirement.getOwnedConstraint().get(0)).getSpecification()!=null &&
					(FunctionExpression)((StructuredOpaqueExpression)((Constraint)resourceRequirement.getOwnedConstraint().get(0)).getSpecification()).getExpression()!=null					
			){
				//only known constraints are mapped to predefined queries.
//				if(isKnownConstraint((Constraint)resourceRequirement.getOwnedConstraint().get(0))){
//					updateVerbFromConstraint((Constraint)resourceRequirement.getOwnedConstraint().get(0), verb);
//					return true;
//				}else{
//					updateEverybody(verb);
//					return true;
//				}
				updateVerb((IndividualResourceRequirement)resourceRequirement,(TVerb)verb);
				return true;
			} else{
				updateEverybody(verb);
				return true;
			}
		//Only RequiredRoles with predefined specified ResourceType are mapped to Staff Queries	
		} else if (resourceRequirement instanceof RequiredRole){
			//updateMemberOfRole(((RequiredRole)resourceRequirement).getRole(), verb);
			updateMemberOfRole((RequiredRole)resourceRequirement, (TVerb)verb);
			return true;
		}
		return false;
	} 	
	
	/**
	 * Updates the BPEL+ staff query verb based on the list of ResourceRequirements 
	 * @param resourceRequirements the resource requirements for this action
	 */
	private boolean updateQueryVerb(List resourceRequirements, EObject verb){
	
		ResourceRequirement candidateRequirement = null;
		Iterator iterator = resourceRequirements.iterator();
		while(iterator.hasNext()){
			ResourceRequirement thisRequirement = (ResourceRequirement)iterator.next();
			//BulkResourceRequirement are not mapped to Staff queries.
			if(!(thisRequirement instanceof BulkResourceRequirement)){
				if(thisRequirement instanceof IndividualResourceRequirement){
					IndividualResourceRequirement individual = (IndividualResourceRequirement)thisRequirement;
					//IndividualResourcesRequirement with assigned IndividualResources of predefinedType si map to the Users by userID  
					if (individual.getIndividualResource()!= null && StaffUtil.isPredefinedStaff(individual.getIndividualResource().getClassifier())){
						updateUsersByUserID(individual.getIndividualResource(), verb);
						return true;
					}else if (individual.getResourceType()!= null && StaffUtil.isPredefinedStaff(individual.getResourceType())){
						if(thisRequirement.getOwnedConstraint()!= null && !thisRequirement.getOwnedConstraint().isEmpty()){
							//only known constraints are mapped to predefined queries.
							if(isKnownConstraint((Constraint)thisRequirement.getOwnedConstraint().get(0))){
								updateVerbFromConstraint((Constraint)thisRequirement.getOwnedConstraint().get(0), verb);
								return true;
							}else{ 
								candidateRequirement = thisRequirement;
							}
						}else{
							//individual resource with known resource type is mapped to Everybody query
							updateEverybody(verb);
							return true;
						}
					}
				//If isBOMHumanTask==false, only RequiredRoles with predefined specified ResourceType are mapped to Staff Queries	
				}else if(thisRequirement instanceof RequiredRole && (isBOMHumanTask==true || (thisRequirement.getResourceType() != null && StaffUtil.isPredefinedStaff(thisRequirement.getResourceType())))){
					updateMemberOfRole(((RequiredRole)thisRequirement).getRole(), verb);
					return true;
				}
			}
		}
		if(candidateRequirement != null){
			LoggingUtil.logWarning(MessageKeys.UNKNOWN_STAFF_ALLOCATION_CONSTRAINT);
		}
		return false;	
	} 	
	/**
	 * Updates true if the input Constraint is a known predefined SSS query constraint
	 * @param constraint BOM Constraint
	 */
	private boolean isKnownConstraint(Constraint constraint){
		if(constraint.getSpecification() instanceof StructuredOpaqueExpression){
			StructuredOpaqueExpression bomExpression = (StructuredOpaqueExpression)constraint.getSpecification();
			Expression expression = bomExpression.getExpression();
			if(expression instanceof FunctionExpression){
				FunctionExpression function = (FunctionExpression)expression;
				String functionID = function.getDefinition().getFunctionID();
				if(StaffUtil.getPredefinedResourceRequirementConstraints().contains(functionID)&& !functionID.equals(StaffUtil.MGR_OF_ORG_FUNCTION)){
					return true;
				}
			}
		}
		return false;
	}

	private void updateMemberOfRole(RequiredRole resourceRequirement, TVerb verb){	
		if (!resourceRequirement.getOwnedConstraint().isEmpty() &&
				((Constraint)resourceRequirement.getOwnedConstraint().get(0)).getSpecification()!=null &&
				((StructuredOpaqueExpression)(((Constraint)resourceRequirement.getOwnedConstraint().get(0)).getSpecification())).getExpression()!=null
				) {
//		if (resourceRequirement.getRole()!=null) {
//			updateMemberOfRole(resourceRequirement.getRole(),verb);
//		} else {
			Constraint constraint = (Constraint)resourceRequirement.getOwnedConstraint().get(0);
			StructuredOpaqueExpression resReqConstraint = (StructuredOpaqueExpression)constraint.getSpecification();
			FunctionExpression function = (FunctionExpression)resReqConstraint.getExpression();
			String functionID = function.getDefinition().getFunctionID();
			verb.setName(functionID);
			List arguments = function.getArguments();
			if(arguments != null && !arguments.isEmpty()){
				for (Iterator it=arguments.iterator();it.hasNext();) {
					FunctionArgument functionArgument = (FunctionArgument)it.next();
					if(functionArgument.getArgumentValue() != null){
						com.ibm.wbit.tel.ParameterType parameter = TaskFactory.eINSTANCE.createParameterType();
						parameter.setId(functionArgument.getArgumentID());
						Expression valueParameterExpression = functionArgument.getArgumentValue();
						if(valueParameterExpression instanceof StringLiteralExpression){
							String value = ((StringLiteralExpression)valueParameterExpression).getStringSymbol();
							if (isContextVariable(value)) {
								parameter.setValue(convertContextVariable(value));
							} else if(StaffUtil.isNameParameter(functionArgument.getArgumentID())){
								// add unique name structure if not provided by user
								parameter.setValue(parseNameValue(value, true));
							}else{
								parameter.setValue(value);							
							}
						}else if (valueParameterExpression instanceof BooleanLiteralExpression){
							parameter.setValue(((BooleanLiteralExpression)valueParameterExpression).getBooleanSymbol().toString());
						}
						verb.getParameter().add(parameter);
					}
				}
				com.ibm.wbit.tel.ParameterType includeNestedRoles = TaskFactory.eINSTANCE.createParameterType();
				includeNestedRoles.setId(StaffConstants.SSS_INCLUDE_NESTED_ROLES);
				includeNestedRoles.setValue("false");	
				verb.getParameter().add(includeNestedRoles);
			}
// Ray - d154437			
//		} else if (resourceRequirement.getRole()!=null) {
//			updateMemberOfRole(resourceRequirement.getRole(),verb);
		} else {
			updateEverybody(verb);
		}
	}
	/**
	 * Updates a "Member of Role" predefined verb. 
	 * @param requiredRole BOM Role
	 */
	private void updateMemberOfRole(Role requiredRole, EObject thisVerb){
			TVerb verb = (TVerb)thisVerb;
			//create the Member of Role verb
			verb.setName(StaffConstants.SSS_MEMBERS_OF_ROLE);
			com.ibm.wbit.tel.ParameterType parameter = TaskFactory.eINSTANCE.createParameterType();
			parameter.setId(StaffConstants.SSS_ROLE_NAME);
			if(requiredRole != null){
				//add unique name structure if not provided by user
				String nameValue = parseNameValue(requiredRole.getName(), true);
				parameter.setValue(nameValue);
			}
	
			verb.getParameter().add(parameter);
			//PMR fix
//			com.ibm.wbit.tel.ParameterType domain = TaskFactory.eINSTANCE.createParameterType();
//			domain.setId(StaffConstants.SSS_DOMAIN);
//			domain.setValue(null);
//			verb.getParameter().add(domain);
			
			com.ibm.wbit.tel.ParameterType includeNestedRoles = TaskFactory.eINSTANCE.createParameterType();
			includeNestedRoles.setId(StaffConstants.SSS_INCLUDE_NESTED_ROLES);
			includeNestedRoles.setValue("false");	
			verb.getParameter().add(includeNestedRoles);

	}

	private void updateVerb(IndividualResourceRequirement resourceRequirement, TVerb verb){

		Constraint constraint = (Constraint)resourceRequirement.getOwnedConstraint().get(0);
		StructuredOpaqueExpression resReqConstraint = (StructuredOpaqueExpression)constraint.getSpecification();
		FunctionExpression function = (FunctionExpression)resReqConstraint.getExpression();
		String functionID = function.getDefinition().getFunctionID();
		//defect 85535. Check if the Members of roles is created under the IndividualResourceRequirement
		boolean isRoleName = false;
		if(functionID.equals(StaffConstants.SSS_MEMBERS_OF_ROLE)){
			isRoleName = true;
		}
		verb.setName(functionID);
		List arguments = function.getArguments();
		boolean isGroupMembersIncludeSubGroupsParameterSet=false;
		boolean isDepartmentMembersIncludeNestedDepartmentsSet=false;
		if(arguments != null && !arguments.isEmpty()){
			for (Iterator it=arguments.iterator();it.hasNext();) {
				FunctionArgument functionArgument = (FunctionArgument)it.next();
				if(functionArgument.getArgumentValue() != null){
					com.ibm.wbit.tel.ParameterType parameter = TaskFactory.eINSTANCE.createParameterType();
					
					String funcParameterID = functionArgument.getArgumentID();
					parameter.setId(funcParameterID);
						
					if (funcParameterID.equals(StaffConstants.SSS_GROUP_MEMBERS_INCLUDE_SUBGROUPS_ARG)) {
						isGroupMembersIncludeSubGroupsParameterSet=true;
					} else if (funcParameterID.equals(StaffConstants.SSS_DEPARTMENT_MEMBERS_INCLUDENESTEDEPARTMENTS_ARG)) {
						isDepartmentMembersIncludeNestedDepartmentsSet=true;
					}
					Expression valueParameterExpression = functionArgument.getArgumentValue();
					if(valueParameterExpression instanceof StringLiteralExpression){

						String value = ((StringLiteralExpression)valueParameterExpression).getStringSymbol();
						if(StaffUtil.isNameParameter(funcParameterID)){
							//parse the value string, wrap with unique id strings if necessary
							String newValue = parseNameValue(value, isRoleName);
							parameter.setValue(newValue);
						}else if (isContextVariable(value)) {
							parameter.setValue(convertContextVariable(value));
						} else {
							parameter.setValue(value);							
						}
					}else if (valueParameterExpression instanceof BooleanLiteralExpression){
						parameter.setValue(((BooleanLiteralExpression)valueParameterExpression).getBooleanSymbol().toString());
					}
					verb.getParameter().add(parameter);
				}
			}
		}
		// Handle special params
		if (functionID.equals(StaffConstants.SSS_GROUP_MEMBERS_FUNCTION) && isGroupMembersIncludeSubGroupsParameterSet==false) {
			com.ibm.wbit.tel.ParameterType parameter = TaskFactory.eINSTANCE.createParameterType();
			parameter.setId(StaffConstants.SSS_GROUP_MEMBERS_INCLUDE_SUBGROUPS_ARG);	
			parameter.setValue(StaffConstants.QUERY_PARAM_VALUE_FALSE);
			verb.getParameter().add(parameter);
		} else if (functionID.equals(StaffConstants.SSS_DEPARTMENT_MEMBERS) && isDepartmentMembersIncludeNestedDepartmentsSet==false) {
			com.ibm.wbit.tel.ParameterType parameter = TaskFactory.eINSTANCE.createParameterType();
			parameter.setId(StaffConstants.SSS_DEPARTMENT_MEMBERS_INCLUDENESTEDEPARTMENTS_ARG);	
			parameter.setValue(StaffConstants.QUERY_PARAM_VALUE_FALSE);
			verb.getParameter().add(parameter);
		}
	}	
	
	/**
	 * Checks if value contains unique id structure.
	 * Wrappers inputted value if necessary and returns string.
	 */	
	private String parseNameValue(String value, boolean isRoleName) {
		final String delimiter = "###";
		//for Deployment to WID, continue to swap the name, thus enable for test . defect 83690
		if(isRoleName){
			//find the real roleName
			int index = value.indexOf(delimiter);
			//this is a typed value 
			if(index == -1){
				if(!BpelOptionsUtil.isGenerateForDeployPatternEnabled()){
					if (!value.toLowerCase().contains("cn="))
						value="cn=".concat(value).concat(",o=defaultWIMFileBasedRealm");	
				//we are now generating for deployment
				}else{
					//leave the value unchanged, this is probably a typed value
				}	
			}else{
				String modelerIdentifier = value.substring(value.indexOf(delimiter)+3);
				value = value.substring(0,value.indexOf(delimiter));
				//try to match in all the cases against the runtime value if exists
				String runtimeName = HumanTaskUtil.getRuntimeUniqueIdentifierForGroup(modelerIdentifier);
				if(runtimeName != null && !("".equals(runtimeName))){
					value = runtimeName;
				}else{
					if(!BpelOptionsUtil.isGenerateForDeployPatternEnabled()){
						if (!value.toLowerCase().contains("cn="))
							value="cn=".concat(value).concat(",o=defaultWIMFileBasedRealm");	
					//we are now generating for deployment
					}else{
						//we should probably log the fact that we could not find the runtime name for the entry
					}
				}
			}
		}else{
			if(!BpelOptionsUtil.isGenerateForDeployPatternEnabled()){
				if (!value.toLowerCase().contains("cn="))
					value="cn=".concat(value).concat(",o=defaultWIMFileBasedRealm");	
			//we are now generating for deployment
			}else {
				return value;
			}
			
		}
			return value;
	}

	
	/**
	 * Checks if value contains unique id structure.
	 * Wrappers inputted value if necessary and returns string.
	 */	
	private String parseNameValue_bak(String value, boolean isRoleName) {
		final String delimiter = "###";
		//for Deployment to WID, continue to swap the name, thus enable for test 
		if(isRoleName){
			//find the real roleName
			int index = value.indexOf(delimiter);
			if(index == -1){
				if(!BpelOptionsUtil.isGenerateForDeployPatternEnabled()){
					if (!value.toLowerCase().contains("cn="))
						value="cn=".concat(value).concat(",o=defaultWIMFileBasedRealm");	
				//we are now generating for deployment
				}else{
					//leave the value unchanged, this is probably a typed value
				}	
			}else{
				String modelerIdentifier = value.substring(value.indexOf(delimiter)+3);
				value = value.substring(0,value.indexOf(delimiter));
				if(!BpelOptionsUtil.isGenerateForDeployPatternEnabled()){
					if (!value.toLowerCase().contains("cn="))
						value="cn=".concat(value).concat(",o=defaultWIMFileBasedRealm");	
				//we are now generating for deployment
				}else{
					//swap the roleName with the value that exists in the mapping file.
					String runtimeName = HumanTaskUtil.getRuntimeUniqueIdentifierForGroup(modelerIdentifier);
					if(runtimeName != null && !("".equals(runtimeName))){
						value = runtimeName;
					}else{
						//we should probably log the fact that we could not find the runtime name for the entry
					}
				}
			}
		}else{
			if(!BpelOptionsUtil.isGenerateForDeployPatternEnabled()){
				if (!value.toLowerCase().contains("cn="))
					value="cn=".concat(value).concat(",o=defaultWIMFileBasedRealm");	
			//we are now generating for deployment
			}else {
				return value;
			}
			
		}
			return value;
	}

	/**
	 * Updates an TEL "Everybody" Verb. 
	 */	
	private void updateEverybody(EObject thisVerb){
		TVerb verb = (TVerb)thisVerb;
		verb.setName(StaffConstants.EVERYBODY);
	}	
	
	/**
	 * Uodates the "Users by userID" verb. 
	 * @param requiredResource an IndividualResource
	 */
	private void updateUsersByUserID(IndividualResource requiredResource, EObject thisVerb){

		TVerb verb = (TVerb)thisVerb;
			verb.setName(StaffConstants.SSS_USERS_BY_USERID);
			com.ibm.wbit.tel.ParameterType parameter = TaskFactory.eINSTANCE.createParameterType();
			parameter.setId(StaffConstants.SSS_USER_ID);
			String userID = getInstanceSpecificationValue(requiredResource, "personId");
			if(userID == null){
				//log warning...
				String[] args = new String []{"personId"};
				LoggingUtil.logWarning(MessageKeys.MISSING_MANDATORY_STAFF_PARAMETER, args, null);
			}else{	
				parameter.setValue(userID);
				verb.getParameter().add(parameter);
			}
	}	
	
	
	/**
	 * Uodates a Verb based on the input BOM Constraint
	 * @param constraint BOM Constraint that references a predefined query expression 
	 */
	private void updateVerbFromConstraint(Constraint constraint, EObject verb){
		StructuredOpaqueExpression resReqConstraint = (StructuredOpaqueExpression)constraint.getSpecification();
		FunctionExpression function = (FunctionExpression)resReqConstraint.getExpression();
		String functionID = function.getDefinition().getFunctionID();
		if(functionID.equals(StaffUtil.MGR_OF_EMPL_BY_USERID_FUNCTION)){
			updateManagerOfEmployeeByUserID(function, verb);
		}else if(functionID.equals(StaffUtil.MGR_OF_EMPL_FUNCTION)){
			updateManagerOfEmployee(function, verb);
		}else if (functionID.equals(StaffUtil.PERSON_BY_NAME_FUNCTION)){
			updatePersonByName(function, verb);
		}else if (functionID.equals(StaffUtil.PERSON_SEARCH_FUNCTION)){
			updatePersonSearch(function, verb);
		}
		else if(functionID.equals(StaffUtil.GROUP_MEMBERS_FUNCTION)){
			updateGroupMembers(function, verb);
		}else if(functionID.equals(StaffUtil.GROUP_SEARCH_FUNCTION)){
			updateGroupSearch(function, verb);
		}

	}	
	
	/**
	 * Updates the "Manager of Employee by userID" TEL Verb. 
	 * @param function predefined BOM function expression
	 */
	private void updateManagerOfEmployeeByUserID(FunctionExpression function, EObject thisVerb){
		TVerb verb = (TVerb)thisVerb;
		verb.setName(StaffConstants.SSS_MGR_OF_EMPL_BY_USERID_FUNCTION);
		List arguments = function.getArguments();
		if(arguments != null && !arguments.isEmpty()){
			Iterator it = arguments.iterator();
			while(it.hasNext()){
				FunctionArgument functionArgument = (FunctionArgument)it.next();
				if(functionArgument.getArgumentValue() != null){
					com.ibm.wbit.tel.ParameterType parameter = TaskFactory.eINSTANCE.createParameterType();
					String argumentID = functionArgument.getArgumentID();
					String searchAttribute = null;
					if(argumentID.equals(StaffUtil.MGR_OF_EMPL_BY_USERID_DOMAIN_ARG)){
						parameter.setId(StaffConstants.SSS_DOMAIN);
					}else if(argumentID.equals(StaffUtil.MGR_OF_EMPL_BY_USERID_USERID_ARG)){
						parameter.setId(StaffConstants.SSS_MGR_OF_EMPL_BY_USERID_USERID_ARG);
						searchAttribute = StaffConstants.STAFF_IDENTIFIER;
					}else{
						//LogHelper.log(0, null, "com.ibm.btolls.te.heuristics.resources.Messages","Unknow criteria, will skip");
					}
					if(parameter.getId()!= null){
						Expression valueParameterExpression = functionArgument.getArgumentValue();
						if(valueParameterExpression instanceof StringLiteralExpression){
							parameter.setValue(((StringLiteralExpression)valueParameterExpression).getStringSymbol());
						}else if (valueParameterExpression instanceof BooleanLiteralExpression){
							parameter.setValue(((BooleanLiteralExpression)valueParameterExpression).getBooleanSymbol().toString());
						}else if (valueParameterExpression instanceof ModelPathExpression){
							ModelPathExpression pathExpression = (ModelPathExpression)valueParameterExpression;
							//get the last step
							ReferenceStep lastStep = (ReferenceStep)pathExpression.getSteps().get(pathExpression.getSteps().size()-1);
							EObject eObject = lastStep.getReferencedObject();
							if(eObject instanceof InstanceSpecification){
								parameter.setValue(getInstanceSpecificationValue((InstanceSpecification)eObject, searchAttribute));
							}
						}
						verb.getParameter().add(parameter);
					}
				}
			}
		}

	}	
	/**
	 * Updates the "Manager of Employee" TEL Verb. 
	 * @param function predefined BOM function expression
	 */
	private void updateManagerOfEmployee(FunctionExpression function, EObject thisVerb){
		TVerb verb = (TVerb)thisVerb;
		verb.setName(StaffConstants.SSS_MGR_OF_EMPL_FUNCTION);
		List arguments = function.getArguments();
		if(arguments != null && !arguments.isEmpty()){
			Iterator it = arguments.iterator();
			while(it.hasNext()){
				FunctionArgument functionArgument = (FunctionArgument)it.next();
				if(functionArgument.getArgumentValue() != null){
					com.ibm.wbit.tel.ParameterType parameter = TaskFactory.eINSTANCE.createParameterType();
					String argumentID = functionArgument.getArgumentID();
					if(argumentID.equals(StaffUtil.MGR_OF_EMPL_DOMAIN_ARG)){
						parameter.setId(StaffConstants.SSS_DOMAIN);
					}else if(argumentID.equals(StaffUtil.MGR_OF_EMPL_EMPLNAME_ARG)){
						parameter.setId(StaffConstants.SSS_MGR_OF_EMPL_EMPLNAME_ARG);
					}else{
						//					LogHelper.log(0, null, "com.ibm.btools.te.ilm.heuristics.resources.Messages","Unknow criteria, will skip");
					}
					if(parameter.getId()!= null){
						Expression valueParameterExpression = functionArgument.getArgumentValue();
						//should always be a StringLiteralExpression
						if(valueParameterExpression instanceof StringLiteralExpression){
							parameter.setValue(((StringLiteralExpression)valueParameterExpression).getStringSymbol());
						}else if (valueParameterExpression instanceof BooleanLiteralExpression){
							parameter.setValue(((BooleanLiteralExpression)valueParameterExpression).getBooleanSymbol().toString());
						}
						verb.getParameter().add(parameter);
					}
				}
			}
		}
	}
	
		
	/**
	 * Updated the "Users" SSS Verb. 
	 * @param function predefined BOM function expression
	 */
	private void updatePersonByName(FunctionExpression function, EObject thisVerb){
			TVerb verb = (TVerb)thisVerb;	
			verb.setName(StaffConstants.SSS_PERSON_BY_NAME_FUNCTION);
			List arguments = function.getArguments();
			if(arguments != null && !arguments.isEmpty()){
				Iterator it = arguments.iterator();
				while(it.hasNext()){
					FunctionArgument functionArgument = (FunctionArgument)it.next();
					if(functionArgument.getArgumentValue() != null){
						com.ibm.wbit.tel.ParameterType parameter = TaskFactory.eINSTANCE.createParameterType();
						String argumentID = functionArgument.getArgumentID();
						if(argumentID.equals(StaffUtil.PERSON_BY_NAME_NAME_ARG)){
							parameter.setId(StaffConstants.SSS_PERSON_BY_NAME_NAME_ARG);
						}else if(argumentID.equals(StaffUtil.PERSON_BY_NAME_ALTNAME1_ARG)){
							parameter.setId(StaffConstants.SSS_PERSON_BY_NAME_ALTNAME1_ARG);
						}else if(argumentID.equals(StaffUtil.PERSON_BY_NAME_ALTNAME2_ARG)){
							parameter.setId(StaffConstants.SSS_PERSON_BY_NAME_ALTNAME2_ARG);
						}else{
							//					LogHelper.log(0, null, "com.ibm.btolls.te.heuristics.resources.Messages","Unknow criteria, will skip");
						}
						if(parameter.getId()!= null){
							Expression valueParameterExpression = functionArgument.getArgumentValue();
							//should only be a StringLiteralExpression
							if(valueParameterExpression instanceof StringLiteralExpression){
								parameter.setValue(((StringLiteralExpression)valueParameterExpression).getStringSymbol());
							}else if (valueParameterExpression instanceof BooleanLiteralExpression){
								parameter.setValue(((BooleanLiteralExpression)valueParameterExpression).getBooleanSymbol().toString());
							}
							verb.getParameter().add(parameter);
						}
					}
				}
			}
	}
		
	/**
	 * Updates the "Person Search" TEL Verb. 
	 * @param function predefined BOM function expression
	 */
	private void updatePersonSearch(FunctionExpression function, EObject thisVerb){
		TVerb verb = (TVerb)thisVerb;
		verb.setName(StaffConstants.SSS_PERSON_SEARCH_FUNCTION);
		List arguments = function.getArguments();
		if(arguments != null && !arguments.isEmpty()){
			Iterator it = arguments.iterator();
			while(it.hasNext()){
				FunctionArgument functionArgument = (FunctionArgument)it.next();
				if(functionArgument.getArgumentValue() != null){
					com.ibm.wbit.tel.ParameterType parameter = TaskFactory.eINSTANCE.createParameterType();
					String argumentID = functionArgument.getArgumentID();
					String searchAttribute = null;
					if(argumentID.equals(StaffUtil.PERSON_SEARCH_ASSISTANT_ARG)){
						parameter.setId(StaffConstants.SSS_PERSON_SEARCH_ASSISTANT_ARG);
						searchAttribute = StaffConstants.STAFF_IDENTIFIER;
					}else if(argumentID.equals(StaffUtil.PERSON_SEARCH_COMPANY_ARG)){
						parameter.setId(StaffConstants.SSS_PERSON_SEARCH_COMPANY_ARG);
						//searchAttribute = StaffConstants.ORGANIZATION_NAME;
						searchAttribute = StaffConstants.GROUP_IDENTIFIER;
					}else if(argumentID.equals(StaffUtil.PERSON_SEARCH_DEPARTMENT_ARG)){
						parameter.setId(StaffConstants.SSS_PERSON_SEARCH_DEPARTMENT_ARG);
						//searchAttribute = StaffConstants.ORGANIZATION_NAME;
						searchAttribute = StaffConstants.GROUP_IDENTIFIER;
					}else if(argumentID.equals(StaffUtil.PERSON_SEARCH_DISPLAYNAME_ARG)){
						parameter.setId(StaffConstants.SSS_PERSON_SEARCH_DISPLAYNAME_ARG);
					}else if(argumentID.equals(StaffUtil.PERSON_SEARCH_EMAIL_ARG)){
						parameter.setId(StaffConstants.SSS_PERSON_SEARCH_EMAIL_ARG);
					}else if(argumentID.equals(StaffUtil.PERSON_SEARCH_EMPLOYEEID_ARG)){
						parameter.setId(StaffConstants.SSS_PERSON_SEARCH_EMPLOYEEID_ARG);
					}else if(argumentID.equals(StaffUtil.PERSON_SEARCH_FAX_ARG)){
						parameter.setId(StaffConstants.SSS_PERSON_SEARCH_FAX_ARG);
					}else if(argumentID.equals(StaffUtil.PERSON_SEARCH_FIRSTNAME_ARG)){
						parameter.setId(StaffConstants.SSS_PERSON_SEARCH_FIRSTNAME_ARG);
					}else if(argumentID.equals(StaffUtil.PERSON_SEARCH_GENDER_ARG)){
						parameter.setId(StaffConstants.SSS_PERSON_SEARCH_GENDER_ARG);
					}else if(argumentID.equals(StaffUtil.PERSON_SEARCH_LASTNAME_ARG)){
						parameter.setId(StaffConstants.SSS_PERSON_SEARCH_LASTNAME_ARG);
					}else if(argumentID.equals(StaffUtil.PERSON_SEARCH_MANAGER_ARG)){
						parameter.setId(StaffConstants.SSS_PERSON_SEARCH_MANAGER_ARG);
						searchAttribute = StaffConstants.STAFF_IDENTIFIER;
					}else if(argumentID.equals(StaffUtil.PERSON_SEARCH_MIDDLENAME_ARG)){
						parameter.setId(StaffConstants.SSS_PERSON_SEARCH_MIDDLENAME_ARG);
					}else if(argumentID.equals(StaffUtil.PERSON_SEARCH_PHONE_ARG)){
						parameter.setId(StaffConstants.SSS_PERSON_SEARCH_PHONE_ARG);
					}else if(argumentID.equals(StaffUtil.PERSON_SEARCH_PREFLANGUAGE_ARG)){
						parameter.setId(StaffConstants.SSS_PERSON_SEARCH_PREFLANGUAGE_ARG);
					}else if(argumentID.equals(StaffUtil.PERSON_SEARCH_PROFILE_ARG)){
						parameter.setId(StaffConstants.SSS_PERSON_SEARCH_PROFILE_ARG);
					}else if(argumentID.equals(StaffUtil.PERSON_SEARCH_SECRETARY_ARG)){
						parameter.setId(StaffConstants.SSS_PERSON_SEARCH_SECRETARY_ARG);
						searchAttribute = StaffConstants.STAFF_IDENTIFIER;
					}else if(argumentID.equals(StaffUtil.PERSON_SEARCH_TAXPAYERID_ARG)){
						parameter.setId(StaffConstants.SSS_PERSON_SEARCH_TAXPAYERID_ARG);
					}else if(argumentID.equals(StaffUtil.PERSON_SEARCH_TIMEZONE_ARG)){
						parameter.setId(StaffConstants.SSS_PERSON_SEARCH_TIMEZONE_ARG);
					}else if(argumentID.equals(StaffUtil.PERSON_SEARCH_USERID_ARG)){
						parameter.setId(StaffConstants.SSS_PERSON_SEARCH_USERID_ARG);
					}else{
						//					LogHelper.log(0, null, "com.ibm.btolls.te.heuristics.resources.Messages","Unknow criteria, will skip");
					}
					if(parameter.getId()!= null){
						Expression valueParameterExpression = functionArgument.getArgumentValue();
						if(valueParameterExpression instanceof StringLiteralExpression){
							parameter.setValue(((StringLiteralExpression)valueParameterExpression).getStringSymbol());
						}else if (valueParameterExpression instanceof BooleanLiteralExpression){
							parameter.setValue(((BooleanLiteralExpression)valueParameterExpression).getBooleanSymbol().toString());
						}else if (valueParameterExpression instanceof ModelPathExpression){
							ModelPathExpression pathExpression = (ModelPathExpression)valueParameterExpression;
							//get the last step
							ReferenceStep lastStep = (ReferenceStep)pathExpression.getSteps().get(pathExpression.getSteps().size()-1);
							EObject eObject = lastStep.getReferencedObject();
							if(eObject instanceof InstanceSpecification){
								parameter.setValue(getInstanceSpecificationValue((InstanceSpecification)eObject, searchAttribute));
							}
						}
						if(parameter.getId() != null){
							//add parameter only if not null.. avoid empty parameter ->company
							verb.getParameter().add(parameter);	
						}
						
					}
				}
			}
		}	
	}
	/**
	 * Updates the "Group Members" SSS Verb. 
	 * @param function predefined BOM function expression
	 */
	private void updateGroupMembers(FunctionExpression function, EObject thisVerb){
		TVerb verb = (TVerb)thisVerb;
		boolean mandatoryInclude = false;
		//Verb verb = StaffFactory.eINSTANCE.createVerb();
		//verb.setId(StaffConstants.SSS_GROUP_MEMBERS_FUNCTION);
		verb.setName(StaffConstants.SSS_GROUP_MEMBERS_FUNCTION);
		List arguments = function.getArguments();
		if(arguments != null && !arguments.isEmpty()){
			Iterator it = arguments.iterator();
			while(it.hasNext()){
				String searchAttribute = null;
				FunctionArgument functionArgument = (FunctionArgument)it.next();
				if(functionArgument.getArgumentValue() != null) {
					com.ibm.wbit.tel.ParameterType parameter = TaskFactory.eINSTANCE.createParameterType();
					String argumentID = functionArgument.getArgumentID();
					if(argumentID.equals(StaffUtil.GROUP_MEMBERS_NAME_ARG)){
						parameter.setId(StaffConstants.SSS_GROUP_MEMBERS_NAME_ARG);
						//searchAttribute = StaffConstants.GROUP_IDENTIFIER;
						searchAttribute = StaffConstants.ORGANIZATION_NAME;
					}else if(argumentID.equals(StaffUtil.GROUP_MEMBERS_INCLUDE_SUBGROUPS_ARG)){
						parameter.setId(StaffConstants.SSS_GROUP_MEMBERS_INCLUDE_SUBGROUPS_ARG);
						mandatoryInclude = true;
					}else if(argumentID.equals(StaffUtil.GROUP_MEMBERS_DOMAIN_ARG)){
						parameter.setId(StaffConstants.SSS_DOMAIN);
					}else if(argumentID.equals(StaffUtil.GROUP_MEMBERS_ALTNAME1_ARG)){
						parameter.setId(StaffConstants.SSS_GROUP_MEMBERS_ALTNAME1_ARG);
						searchAttribute = StaffConstants.ORGANIZATION_NAME;
					}else if(argumentID.equals(StaffUtil.GROUP_MEMBERS_ALTNAME2_ARG)){
						parameter.setId(StaffConstants.SSS_GROUP_MEMBERS_ALTNAME2_ARG);
						searchAttribute = StaffConstants.ORGANIZATION_NAME;
					}else{
						//LogHelper.log(0, null, "com.ibm.btolls.te.heuristics.resources.Messages","Unknow criteria, will skip");
					}
					if(parameter.getId()!= null){
						Expression valueParameterExpression = functionArgument.getArgumentValue();
						//should only be a StringLiteralExpression
						if(valueParameterExpression instanceof StringLiteralExpression){
							parameter.setValue(((StringLiteralExpression)valueParameterExpression).getStringSymbol());
						}else if (valueParameterExpression instanceof BooleanLiteralExpression){
							parameter.setValue(((BooleanLiteralExpression)valueParameterExpression).getBooleanSymbol().toString());
						}else if (valueParameterExpression instanceof ModelPathExpression){
							
							ModelPathExpression pathExpression = (ModelPathExpression)valueParameterExpression;
							//get the last step
							ReferenceStep lastStep = (ReferenceStep)pathExpression.getSteps().get(pathExpression.getSteps().size()-1);
							EObject eObject = lastStep.getReferencedObject();
							if(eObject instanceof InstanceSpecification){
								parameter.setValue(getInstanceSpecificationValue((InstanceSpecification)eObject, searchAttribute));
							}
						}
						if(parameter.getId().equals(StaffConstants.SSS_GROUP_MEMBERS_INCLUDE_SUBGROUPS_ARG) && parameter.getValue()== null){
							parameter.setValue(Boolean.FALSE.toString());
						}
						verb.getParameter().add(parameter);
					}
				}
			}
			if(mandatoryInclude == false){
				com.ibm.wbit.tel.ParameterType parameter = TaskFactory.eINSTANCE.createParameterType();
				parameter.setId(StaffConstants.SSS_GROUP_MEMBERS_INCLUDE_SUBGROUPS_ARG);
				parameter.setValue(Boolean.FALSE.toString());
				verb.getParameter().add(parameter);
			}
		}
	}

			
				
	
	/**
	 * Updates the "Person Search" SSS Verb. 
	 * @param function predefined BOM function expression
	 */
	private void updateGroupSearch(FunctionExpression function, EObject thisVerb){
		TVerb verb = (TVerb)thisVerb;
		verb.setName(StaffConstants.SSS_GROUP_SEARCH_FUNCTION);
		List arguments = function.getArguments();
		if(arguments != null && !arguments.isEmpty()){
			Iterator it = arguments.iterator();
			while(it.hasNext()){
				FunctionArgument functionArgument = (FunctionArgument)it.next();
				com.ibm.wbit.tel.ParameterType parameter = TaskFactory.eINSTANCE.createParameterType();
				if(functionArgument.getArgumentValue() != null){
					String argumentID = functionArgument.getArgumentID();
					String searchAttribute = null;
					if(argumentID.equals(StaffUtil.GROUP_SEARCH_ASSISTANT_ARG)){
						parameter.setId(StaffConstants.SSS_PERSON_SEARCH_ASSISTANT_ARG);
						searchAttribute = StaffConstants.STAFF_IDENTIFIER;
					}else if(argumentID.equals(StaffUtil.GROUP_SEARCH_GROUPID_ARG)){
						parameter.setId(StaffConstants.SSS_GROUP_SEARCH_GROUPID_ARG);
						searchAttribute = StaffConstants.GROUP_IDENTIFIER;				
					}else if(argumentID.equals(StaffUtil.GROUP_SEARCH_PARENT_COMPANY_ARG)){
						parameter.setId(StaffConstants.SSS_GROUP_SEARCH_PARENT_COMPANY_ARG);
						searchAttribute = StaffConstants.GROUP_IDENTIFIER;
					}else if(argumentID.equals(StaffUtil.GROUP_SEARCH_TYPE_ARG)){
						parameter.setId(StaffConstants.SSS_GROUP_SEARCH_TYPE_ARG);
					}else if(argumentID.equals(StaffUtil.GROUP_SEARCH_DISPLAYNAME_ARG)){
						parameter.setId(StaffConstants.SSS_GROUP_SEARCH_DISPLAYNAME_ARG);
					}else if(argumentID.equals(StaffUtil.GROUP_SEARCH_INDUSTRY_TYPE_ARG)){
						parameter.setId(StaffConstants.SSS_GROUP_SEARCH_INDUSTRY_TYPE_ARG);
					}else if(argumentID.equals(StaffUtil.GROUP_SEARCH_BUSINESS_TYPE_ARG)){
						parameter.setId(StaffConstants.SSS_GROUP_SEARCH_BUSINESS_TYPE_ARG);
					}else if(argumentID.equals(StaffUtil.GROUP_SEARCH_GEOGRAPHIC_LOCATION_ARG)){
						parameter.setId(StaffConstants.SSS_GROUP_SEARCH_GEOGRAPHIC_LOCATION_ARG);
					}else if(argumentID.equals(StaffUtil.GROUP_SEARCH_AFFILIATES_ARG)){
						parameter.setId(StaffConstants.SSS_GROUP_SEARCH_AFFILIATES_ARG);
					}else if(argumentID.equals(StaffUtil.GROUP_SEARCH_BUSINESS_TYPE_ARG)){
						parameter.setId(StaffConstants.SSS_GROUP_SEARCH_BUSINESS_TYPE_ARG);
					}else if(argumentID.equals(StaffUtil.GROUP_SEARCH_MANAGER_ARG)){
						parameter.setId(StaffConstants.SSS_GROUP_SEARCH_MANAGER_ARG);
						searchAttribute = StaffConstants.STAFF_IDENTIFIER;
					}else if(argumentID.equals(StaffUtil.GROUP_SEARCH_SECRETARY_ARG)){
						parameter.setId(StaffConstants.SSS_GROUP_SEARCH_SECRETARY_ARG);
						searchAttribute = StaffConstants.STAFF_IDENTIFIER;
					}else{
						//					LogHelper.log(0, null, "com.ibm.btolls.te.heuristics.resources.Messages","Unknow criteria, will skip");
					}
					if(parameter.getId()!= null){
						Expression valueParameterExpression = functionArgument.getArgumentValue();
						if(valueParameterExpression instanceof StringLiteralExpression){
							parameter.setValue(((StringLiteralExpression)valueParameterExpression).getStringSymbol());
						}else if (valueParameterExpression instanceof BooleanLiteralExpression){
							parameter.setValue(((BooleanLiteralExpression)valueParameterExpression).getBooleanSymbol().toString());
						}else if (valueParameterExpression instanceof ModelPathExpression){
							ModelPathExpression pathExpression = (ModelPathExpression)valueParameterExpression;
							//get the last step
							ReferenceStep lastStep = (ReferenceStep)pathExpression.getSteps().get(pathExpression.getSteps().size()-1);
							EObject eObject = lastStep.getReferencedObject();
							if(eObject instanceof InstanceSpecification){
								parameter.setValue(getInstanceSpecificationValue((InstanceSpecification)eObject, searchAttribute));
							}
						}
						verb.getParameter().add(parameter);
					}
				}
			}
		}
	}


	/**
	 * Returns the attribute value on the instanceSpecification. It the attribute value desired
	 * is the organization name, the canonical name of the organization, as per the model 
	 * is returned. 
	 * @param staffInstance an InstanceSpecification of a predefined resource.
	 * @param attributeName the name of the attribute
	 * @return String the value assigned to the attribute.
	 */
	private String getInstanceSpecificationValue(InstanceSpecification staffInstance, String attributeName){
		
		String value = null;
		if(attributeName.equals(StaffConstants.ORGANIZATION_NAME)){
			value = StaffUtil.getCanonicalName(staffInstance);
		}else{
			List slots = staffInstance.getSlot();
			if(slots != null && !slots.isEmpty()){
				Iterator it = slots.iterator();
				while(it.hasNext()){
					Slot thisSlot = (Slot)it.next();
					if(thisSlot.getDefiningFeature().getName().equals(attributeName)){
						if(thisSlot.getValue() != null && !thisSlot.getValue().isEmpty()){
							value = ((LiteralString)thisSlot.getValue().get(0)).getValue();
							return value;
						}
						
					}
				}
			}
		}
		return value;
	}

	/**
	 * Check if the value is a context variable
	 * @param value
	 * @return
	 */
	private boolean isContextVariable (String value) {
		if (value.startsWith("%"+com.ibm.btools.util.PredefContextVariables.RESOURCES_THAT_ARE_POTENTIAL_OWNERS+"%") ||
				value.startsWith("%"+com.ibm.btools.util.PredefContextVariables.RESOURCES_THAT_ARE_RECEIVERS_OF_ANOTHER_ESCALATION+"%")||
				value.startsWith("%"+com.ibm.btools.util.PredefContextVariables.RESOURCE_THAT_IS_AN_OWNER+"%")) {
			return true;
		}
		return false;
	}
	
	/**
	 * transform the context variable
	 * @param contextVariable
	 * @return
	 */
	private String convertContextVariable (String contextVariable) {
		if (contextVariable.startsWith("%"+com.ibm.btools.util.PredefContextVariables.RESOURCES_THAT_ARE_POTENTIAL_OWNERS+"%")) {
			return HumanTaskUtil.CONTEXTVAR_POTENTIALOWNERS;
		} else if (contextVariable.startsWith("%"+com.ibm.btools.util.PredefContextVariables.RESOURCE_THAT_IS_AN_OWNER+"%")) {
			return HumanTaskUtil.CONTEXTVAR_OWNER;
		} else if (contextVariable.startsWith("%"+com.ibm.btools.util.PredefContextVariables.RESOURCES_THAT_ARE_RECEIVERS_OF_ANOTHER_ESCALATION+"%")) {
			String convertedContextVariable = HumanTaskUtil.CONTEXTVAR_RECEIVERFOROTHERESCALATION;
			String escalationKey = contextVariable.substring(("%"+com.ibm.btools.util.PredefContextVariables.RESOURCES_THAT_ARE_RECEIVERS_OF_ANOTHER_ESCALATION+"%").length());
			ResourceRequirement rr = (ResourceRequirement)(getSource().get(0));
			EObject rrParent = rr.eContainer();
			while (!(rrParent instanceof HumanTask) && rrParent!=null) {
				rrParent = rrParent.eContainer();
			}
			String tEscalationName=null;
			if (rrParent != null) {
				tEscalationName=HumanTaskUtil.getTelEscalationName(getContext(), (HumanTask)rrParent, escalationKey);
			}
			Pattern pattern = Pattern.compile(HumanTaskUtil.CONTEXTVAR_RECEIVERFOROTHERESCALATION_PATTERN);
		    Matcher matcher = pattern.matcher(HumanTaskUtil.CONTEXTVAR_RECEIVERFOROTHERESCALATION);
			if (tEscalationName!=null) {
			    return matcher.replaceAll(tEscalationName);				
			} else {
				if (escalationKey!= null) {
					return matcher.replaceAll(escalationKey);									
				} else {
					return HumanTaskUtil.CONTEXTVAR_RECEIVERFOROTHERESCALATION;
				}
			}
		}
		return contextVariable;
	}
} //StaffActivityRuleImpl
