/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.wst.wsdl.Part;

import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.activities.TerminationNode;
import com.ibm.btools.expression.model.BooleanLiteralExpression;
import com.ibm.btools.expression.model.ModelFactory;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.TerminationNodeRule;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.model.abstractbpel.Alternative;
import com.ibm.btools.te.ilm.model.abstractbpel.AlternativeActivity;
import com.ibm.btools.te.ilm.model.abstractbpel.ExtensibilityExpression;
import com.ibm.btools.te.ilm.model.sa.ProcessFlowType;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Assign;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Copy;
import com.ibm.wbit.bpel.From;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.To;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.JavaCode;
import com.ibm.wbit.bpelpp.JavaScriptActivity;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Termination Node Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class TerminationNodeRuleImpl extends ConnectableRuleImpl implements TerminationNodeRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected TerminationNodeRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.TERMINATION_NODE_RULE;
	}

	/* 
	 * SOURCE = {TerminationNode}
	 * TARGET = <Assign> to set the value of TRUE to the BPEL Variable associated with
	 *          the termination node.
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");
		
		if (getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return false;
		}
		
		/* We just need to get the first termination node */
		TerminationNode termNode = (TerminationNode)getSource().get(0);
		
		/* Find the SAN that contains it... We will use that SAN to find the associated BPEL Variable */
		StructuredActivityNode san = (StructuredActivityNode)termNode.eContainer();
		
		ContainerRule termVarRule = (ContainerRule)		
			TransformationUtil.getRuleForSource(ProcessUtil.getProcessDefinitionRule(getContext()),
												ContainerRule.class,
												san);

		BPELVariable termVar = (BPELVariable)termVarRule.getTarget().get(0);

		/* A termination variable always has only one part */
		Part termVarPart = (Part)termVar.getMessageType().getEParts().get(0);

		/* Create an AlterativeActivity with two alternatives, one for BPEL and one for BPEL+ since
		 * BPEL+ doesn't support XPath expressions in <assign> activities */
		Activity altActivity = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
		Alternative bpelAlt = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();		
		Alternative bpelpAlt = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
		
		bpelAlt.setType(ProcessFlowType.BPEL_LITERAL);
		bpelpAlt.setType(ProcessFlowType.BPELPP_LITERAL);
			
		((AlternativeActivity)altActivity).getAlternative().add(bpelAlt);
		((AlternativeActivity)altActivity).getAlternative().add(bpelpAlt);
			
		/* BPEL: Create <assign>, with multiple BPEL Copies, one for each input value pin value*/
		bpelAlt.setActivity(createAssign(termVar, termVarPart));
			
		/* BPEL+: Create <invoke><wpc:script><wpc:javaCode>... */
		bpelpAlt.setActivity(createJavaInvoke(termVar, termVarPart));

		/* Propagate this assign to the parent rule */
		getTarget().add(altActivity);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return true;
	}

	
	/**
	 * Creates the BPEL version  of the assign to set the value of the termination node
	 * variable to true
	 * 
	 * @param termVar The termination node variable to set the value of 
	 * @param termVarPart The part of the termination node variable to set the value of
	 * @return Assign - The BPEL Assign that sets the termination value
	 */
	private Assign createAssign(BPELVariable termVar, Part termVarPart){
		/* Create the BPEL Assign that we will use to set the value of the termination variable */
		Assign assign = BPELFactory.eINSTANCE.createAssign();
		Copy copy = BPELFactory.eINSTANCE.createCopy();
		From from = BPELFactory.eINSTANCE.createFrom();
		To to = BPELFactory.eINSTANCE.createTo();
		
		/* Create an expression model to assign the value TRUE to the terminationNode */
		ExtensibilityExpression bpelExpr = AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();
		BooleanLiteralExpression strExpr = ModelFactory.eINSTANCE.createBooleanLiteralExpression();
		strExpr.setBooleanSymbol(Boolean.TRUE);
		bpelExpr.setExpression(strExpr);
		from.addExtensibilityElement(bpelExpr);
		
		to.setVariable(termVar);
		to.setPart(termVarPart);
		
		copy.setFrom(from);
		copy.setTo(to);
		assign.getCopy().add(copy);
		
		return assign;
		
	}
	
	/**
	 * Creates the BPEL Invoke that contains a JavaScriptActivity that updates the value of the 
	 * termination node variable to TRUE.
	 * 
	 * @param termVar The termination node variable to update
	 * @param termVarPart The part of the termination node variable to update.
	 * @return Invoke - the BPEL Invoke the contains a JavaScriptActivity as an extensible element, 
	 * which in turn contains a JavaCode element that contains the actual java code!
	 */
	private Invoke createJavaInvoke(BPELVariable termVar, Part termVarPart){
		Invoke javaInvoke = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		
		String javaString = new String();
		
		/* TODO: Do we have to use eGet(...) and eSet(...) calls to be generic and avoid casting? */
		/* TODO: NOTE: Assuming that we have to captilize the first character of the part and var name */
		String varName = AbstractbpelUtil.capitalizeFirstChar(termVar.getName());
		String partName = AbstractbpelUtil.capitalizeFirstChar(termVarPart.getName());
		String elementName = termVarPart.getElementDeclaration().getTypeDefinition().getName();	
	//	javaString += AbstractbpelUtil.generateJavaGetterForVariable(varName) + "(true).set" + partName + "(true);";
		javaString += AbstractbpelUtil.generateJavaGetterForVariable(varName) + "(true)" 
						+ "." + AbstractbpelUtil.generateJavaGetterForVariable(partName)+"(" 
							+ AbstractbpelUtil.generateJavaSetterForVariable(elementName) + "(true));";
		//TODO migration ?? revisit as it seems there is no JavaCode????
		//javaCode.setValue(javaString);
		//jsActivity.setJavaCode(javaCode);
		jsActivity.setJavaCode(javaString);
		
		javaInvoke.getExtensibilityElements().add(jsActivity);
		
		return javaInvoke;
	}

} //TerminationNodeRuleImpl
