package com.ibm.btools.te.ilm.heuristics.abstractbpel.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.CorrelationSet;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.PartnerLink;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.bpel.Variable;
import com.ibm.wbit.tel.TEmail;
import com.ibm.wbit.tel.TEscalation;
import com.ibm.wbit.tel.TTask;


public class TELNamingRegistry implements NamingRegistry {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	
	private  HashMap TTaskNameRegistry = new HashMap();	
	private  HashMap TEscalationNameRegistry = new HashMap();	
	private  HashMap TEmailNameRegistry = new HashMap();		
	
	public boolean isUniqueName(Object obj, String name)
	{
		LoggingUtil.traceEntry(this, "isUniqueName");
		HashMap map; 
		map = getRegistry(obj);
		if (map == null){
			// name for this type of construct does not require name uniqueness
			LoggingUtil.traceExit(this, "isUniqueName","map==null");
			return true;		 		
		}
		else if (map.values().contains(name)){
			LoggingUtil.traceExit(this, "isUniqueName","return false");
			return false;
		}
		else {
			//Store the name in the Registry
			map.put(obj, name);
			LoggingUtil.traceExit(this, "isUniqueName");
			return true;
		}
	}

	/**
	 * Returns a HashMap for the specific obj	 
	 * @param obj  the EObject of a type of construct	
	 * @return HashMap the corresponding hashmap for obj
	 */			
	private HashMap getRegistry(Object obj)
	{
		if (obj instanceof Class) {
			return getRegistry((Class)obj);
		}
		if (obj instanceof TTask)
			return TTaskNameRegistry;
		else if (obj instanceof TEscalation)
			return TEscalationNameRegistry;
		else if (obj instanceof TEmail)
			return TEmailNameRegistry;
		else
			return null;
	}
	
	

	/**
	 * Returns a HashMap for the specific obj	 
	 * @param obj  the EObject of a type of construct	
	 * @return HashMap the corresponding hashmap for obj
	 */			
	private HashMap getRegistry(Class classType)
	{
		if (classType == TTask.class)
			return TTaskNameRegistry;
		else if (classType == TEscalation.class)
			return TEscalationNameRegistry;
		else if (classType == TEmail.class)
			return TEmailNameRegistry;
		else
			return null;
	}
//	
	/* (non-Javadoc)
	 * @see com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry#isUniqueName(java.lang.Object, java.lang.Object, java.lang.String)
	 */
	public boolean isUniqueName(Object obj, Object parentObject, String name)
	{
		HashMap map = getRegistry(obj);
		List entries = null;
		if (map == null){
			// name for this type of construct does not require name uniqueness
			return true;		 		
		}else if(map.get(parentObject) != null){
			entries = (List)map.get(parentObject);
			if(entries.contains(name)){
				return false;
			}else{
				entries.add(name);
				return true;
			}
		}else {
			entries = new LinkedList();
			entries.add(name);
			//Store the name in the Registry
			map.put(parentObject, entries);
			return true;
		}
	}	

	public void deregister(Object obj, String name) {
		HashMap map = getRegistry(obj);
		if (map == null){
			// name for this type of construct does not require name uniqueness
			return;		 		
		}
		else if (map.values().contains(name)){
			map.remove(obj);
		}
	}

	/* (non-Javadoc)
	 * @see com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry#retrieveFromRegistry(java.lang.String)
	 */
	public Object retrieveFromRegistry(Class obj, String registryEntry) {
		
		HashMap registry = getRegistry(obj);
		if(registry.containsValue(registryEntry)){
			//get the object container that was registered for that entry.. 
			Iterator it = registry.keySet().iterator();
			while(it.hasNext()){
				Object variable = it.next();
				if(registry.get(variable).equals(registryEntry)){
					return variable;
				}
			}
			
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry#register(java.lang.Object, java.lang.String)
	 */
	public void register(Object obj, String name) {
		HashMap map = getRegistry(obj);
		if(map == null){
			return;
		}
		map.put(obj, name);
	}

	
}
