package com.ibm.btools.te.ilm.heuristics.bpelp.util;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDParticle;

import com.ibm.btools.bom.model.artifacts.DataType;
import com.ibm.btools.bom.model.artifacts.Property;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.expression.function.CoreFunctions;
import com.ibm.btools.expression.function.FunctionLibrary;
import com.ibm.btools.expression.language.LanguageProtocol;
import com.ibm.btools.expression.language.Languages;
import com.ibm.btools.expression.model.ArrayIndexStep;
import com.ibm.btools.expression.model.Expression;
import com.ibm.btools.expression.model.FunctionArgument;
import com.ibm.btools.expression.model.FunctionExpression;
import com.ibm.btools.expression.model.IntegerLiteralExpression;
import com.ibm.btools.expression.model.ModelPathExpression;
import com.ibm.btools.expression.model.ReferenceStep;
import com.ibm.btools.expression.model.StaticStep;
import com.ibm.btools.expression.model.Step;
import com.ibm.btools.expression.model.StringLiteralExpression;
import com.ibm.btools.expression.xpath.XPathExpressionSerializer;
import com.ibm.btools.expression.xpath.XPathSerializationException;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.heuristics.br.util.BrUtil;
import com.ibm.btools.te.ilm.heuristics.xsd.PropertyRule;
import com.ibm.wbit.bpel.BPELVariable;

/**
 * A static serializer class to help serialize the MapNode's concart function.
 * 
 *
 */
public class ConcatFunctionSerializer {
	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private static final String GETDATA = "bpws:getVariableData";
	
	/**
	 * To serialize the provided function expression which is assumed to be a MapNode's concatFunction.
	 * The pin2Var map should contain the ObjectPin and BPELVariable mapping to allow retriving the
	 * bpel variable used in the enclosed getVariableData function.
	 * @param concatFunction
	 * @param pin2Var
	 * @return
	 */
	public static String serialize(FunctionExpression concatFunction, Map<ObjectPin, BPELVariable> pin2Var, TransformationRule rule){
		return serialize(concatFunction, pin2Var, null, rule);
	}
	
	
	/**
	 * Serialize method for the functionExpresion. Execute recursively for nested cancat functionExpression.
	 * @param concatFunction
	 * @param pin2Var
	 * @param dDelimiter
	 * @return
	 */
	private static String serialize(FunctionExpression concatFunction, Map<ObjectPin, BPELVariable> pin2Var, String dDelimiter, TransformationRule rule){
		
		final StringBuffer sb = new StringBuffer();
		final LanguageProtocol protocol = Languages.XPATH_10_PROTOCOL;
		String funcId = concatFunction.getDefinition().getFunctionID();
		String funcName = FunctionLibrary.getInstance().getFunctionNameForProtocol(funcId, protocol);	
		if (funcName == null) 
			funcName = concatFunction.getDefinition().getFunctionName();
		sb.append(funcName);
		sb.append(XPathExpressionSerializer.PARENTHESIS_START);
		
		String prefix = null, postfix = null, temp  = null, defaultDelimiter = dDelimiter;
		List<FunctionArgument> inputElementExpression = new ArrayList<FunctionArgument>();
		for (Iterator<?> i = concatFunction.getArguments().iterator(); i.hasNext();){
			FunctionArgument fArg = (FunctionArgument)i.next();
			String argId = fArg.getArgumentID();
			Expression argValue = fArg.getArgumentValue();			
			if (argValue instanceof StringLiteralExpression){ // future: use evaluator to obtain string value ?
				temp = ((StringLiteralExpression)argValue).getStringSymbol();
				if (CoreFunctions.CONCAT_FUNCTION_PREFIX_ARG.equals(argId))
					prefix = temp;
				else if (CoreFunctions.CONCAT_FUNCTION_DEFAULT_DELIMITER_ARG.equals(argId))
					defaultDelimiter = temp;
				else if (CoreFunctions.CONCAT_FUNCTION_POSTFIX_ARG.equals(argId))
					postfix = temp;
				else
					inputElementExpression.add(fArg);
			}
			else 
				inputElementExpression.add(fArg);
		}
		
		if (prefix != null){
			addLiteral(sb, prefix);
			sb.append(XPathExpressionSerializer.ARGUMENT_SEP);
		}
		int argSize = inputElementExpression.size();
		for (int i = 0; i < argSize; i++){
			FunctionArgument fArg = (FunctionArgument)inputElementExpression.get(i);
			Expression exp = fArg.getArgumentValue();
			if (exp instanceof FunctionExpression){
				FunctionExpression fExp = (FunctionExpression) exp;
				if (CoreFunctions.CONCAT_FUNCTION.equals(fExp.getDefinition().getFunctionID())){
					sb.append(serialize(fExp, pin2Var, defaultDelimiter, rule));
				}
			}
			else if (exp instanceof ModelPathExpression){
				ModelPathExpression arg = (ModelPathExpression)exp;
				String varDataFn = serializeGetVariableDataFuntion(arg, pin2Var, rule);
				if (varDataFn != null || ! "".equals(varDataFn)){
					sb.append(varDataFn);
				}
			}
			else if (exp instanceof StringLiteralExpression){
				addLiteral(sb, ((StringLiteralExpression)exp).getStringSymbol()); //evaluator
			}
			else{
				continue;
			}
			// if DD not null, current argument is not FE or delimiter. 
			if (defaultDelimiter != null && ! (exp instanceof FunctionExpression) && ! isDelimiter(fArg)){
				// if next argument is available, ensure it is not delimiter
				if (i == argSize - 1 || (i < argSize -1 && ! isDelimiter((FunctionArgument)inputElementExpression.get(i+1)))){
					sb.append(XPathExpressionSerializer.ARGUMENT_SEP);
					addLiteral(sb, defaultDelimiter);
				}
			}
			if (i < argSize -1 || postfix != null){
				sb.append(XPathExpressionSerializer.ARGUMENT_SEP);
			}
		}		
		if (postfix != null){
			addLiteral(sb, postfix);
		}
		sb.append(XPathExpressionSerializer.PARENTHESIS_END);
		return sb.toString();
	}
	
	
	/**
	 * Helper method to add a literal string contained by a single quote into the provided string buffer.
	 * @param sb
	 * @param literal
	 */
	private static void addLiteral(StringBuffer sb, String literal){
		sb.append(XPathExpressionSerializer.STRING_DECOR);
		sb.append(literal);
		sb.append(XPathExpressionSerializer.STRING_DECOR);
	}
	
	
	/**
	 * Helper method to check if the given argument represents a user delimiter.
	 * @param fArg
	 * @return
	 */
	private static boolean isDelimiter(FunctionArgument fArg){
		return CoreFunctions.CONCAT_FUNCTION_INPUT_DELIMITER_ARG.equals(fArg.getArgumentID());
	}
	
	/**
	 * Return a string representation for the getVariableData function which is enclosed in
	 * the concat function represented by a function expression. The getVariableData function
	 * is based on the provided path expression which is one of the function argument of the 
	 * concat function.
	 * @param mpe
	 * @param pin2var
	 * @return
	 */
	public static String serializeGetVariableDataFuntion(ModelPathExpression mpe, Map<ObjectPin, BPELVariable> pin2var, TransformationRule rule){
		final StringBuffer sb = new StringBuffer();
		ReferenceStep step0 = (ReferenceStep)mpe.getSteps().get(0);
		if (step0.getReferencedObject() instanceof ObjectPin){
			ObjectPin pin = (ObjectPin)step0.getReferencedObject();
			BPELVariable pinVar = pin2var.get(pin);
			sb.append(GETDATA);
			sb.append(XPathExpressionSerializer.PARENTHESIS_START);
			addLiteral(sb, pinVar != null ? pinVar.getName(): "");			
			String queryString = getQueryString(mpe, rule);
			if (queryString != null && ! queryString.equals("")){			
				sb.append(XPathExpressionSerializer.ARGUMENT_SEP);
				addLiteral(sb, XPathExpressionSerializer.STEP_SEP + queryString);
			}
			sb.append(XPathExpressionSerializer.PARENTHESIS_END);			
		}
		return sb.toString();
	}
	
	
	/**
	 * Return a string representation of xpath based on the provided ModelPathExpression.
	 * The resulting string is assumed to be used as a query string of a from/to query or
	 * getDVariableData query. 
	 * @param mpe
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static String getQueryString(ModelPathExpression mpe, TransformationRule rule){
		ModelPathExpression mpe_org = filterSimpleTypeStep(mpe);
		ModelPathExpression mpeCopy = (ModelPathExpression)mpe_org.copy();
		List<?> temp = new ArrayList(mpeCopy.getSteps());
		
		for (Iterator i = temp.iterator(); i.hasNext();){
			Step aStep = (Step) i.next();
			if (aStep instanceof ReferenceStep){
				ReferenceStep rStep = (ReferenceStep)aStep;
				EObject refObj = rStep.getReferencedObject();
				if (refObj instanceof ObjectPin || refObj instanceof com.ibm.btools.bom.model.artifacts.Class || refObj instanceof DataType){
					mpeCopy.getSteps().remove(rStep);
				}
			}
		}
		String result = serialize(mpeCopy, rule);
		return result.startsWith(XPathExpressionSerializer.STEP_SEP) ? result.substring(1): result;
	}


	/**
	 * Serializes an expression into an equivalent XPath String
	 * @param expression the expression
	 * @return the XPath String
	 * @throws XPathSerializationException
	 */
	private static String serialize(final ModelPathExpression pathExpression, TransformationRule rule) {
						
		final StringBuffer sb = new StringBuffer();
		if (Boolean.TRUE.equals(pathExpression.getIsAbsolute())) {
			sb.append(XPathExpressionSerializer.STEP_SEP);
		}
		final List<?> steps = pathExpression.getSteps();
		String stepName = null;
		final int size = steps.size();		
		for (int i=0; i<size; i++) {
			Step step = (Step)steps.get(i);
			if (step != null) {
				if (step instanceof StaticStep) {								
					stepName = ((StaticStep)steps.get(i)).getName();
				} 
				else if (step instanceof ReferenceStep) {							
					stepName = getReferenceStepName((ReferenceStep)step, rule);
				}
				else if (step instanceof ArrayIndexStep){
					stepName = getArrayIndexStepName((ArrayIndexStep)step);
				}
				else {
					continue;
				}				
				if (stepName == null) {
					stepName = XPathExpressionSerializer.CURRENT_STEP_SEP;
				}				
//				if (XPathExpressionSerializer.CURRENT_STEP_SEP.equals(stepName) || step instanceof ArrayIndexStep) {
//					sb.append(stepName);
//				} 
//				else{
//					sb.append(NCNameConverter.stringToNcname(stepName));
//				}
				sb.append(stepName);
			}			
			if (i < steps.size() - 1 && ! (steps.get(i+1) instanceof ArrayIndexStep)) {
				sb.append(XPathExpressionSerializer.STEP_SEP);
			}
		}
		String result = sb.toString();		
		return result;
	}
	
	
	private static String getReferenceStepName(ReferenceStep step, TransformationRule rule){
		String result = null;
		EObject eObj = step.getReferencedObject();
		if (eObj instanceof Property){
			Property prop = (Property)eObj;
			XSDParticle particle = (XSDParticle)TransformationUtil.getRuleForSource(rule.getRoot(),PropertyRule.class,prop).getTarget().get(0);
			XSDElementDeclaration decl = (XSDElementDeclaration)particle.getContent();
			result = decl.getName();
		}
		if (result == null){
			result = step.getReferenceName();
		}
		return result;
	}
	
	private static String getArrayIndexStepName(ArrayIndexStep step){
		String result = null;
		if (step.getIndex() instanceof IntegerLiteralExpression){
			IntegerLiteralExpression exp = (IntegerLiteralExpression)step.getIndex();
			result = "" + XPathExpressionSerializer.PREDICATE_START + exp.getNumericSymbol() + XPathExpressionSerializer.PREDICATE_END;
		}
		return result;
	}
	
	//<![CDATA[string(bpws:getVariableData('durationVariable') )]]>
	public static String serializeWarpObject(BPELVariable variable){
		final StringBuffer sb = new StringBuffer();
		sb.append(GETDATA);
		sb.append(XPathExpressionSerializer.PARENTHESIS_START);
		addLiteral(sb, variable.getName());
		sb.append(XPathExpressionSerializer.PARENTHESIS_END);
		return serializeWarpSrcToString(sb.toString());
	}
	
	//string(bpws:getVariableData('Businessitem2Variable','/Attribute[2]'))
	public static String serializeWarpSrcToString(String srcString){
		final StringBuffer sb = new StringBuffer();
		sb.append("string");
		sb.append(XPathExpressionSerializer.PARENTHESIS_START);
		sb.append(srcString);
		sb.append(XPathExpressionSerializer.PARENTHESIS_END);
		return sb.toString();
	}


	/**
	 * Return a new instance of MPE without the last dummy step created from simple type BSO import.
	 * @param toMpe
	 * @return ModelPathExpression
	 */
	@SuppressWarnings("unchecked")
	public static ModelPathExpression filterSimpleTypeStep(ModelPathExpression toMpe){
		ModelPathExpression mpe = (ModelPathExpression)toMpe.copy();
		List steps = new ArrayList();
		for (Iterator<?> i = mpe.getSteps().iterator(); i.hasNext();){
			Object obj = i.next();
			if (obj instanceof ReferenceStep && ((ReferenceStep)obj).getReferencedObject() instanceof ObjectPin){
				Type type = ((ObjectPin)(((ReferenceStep)obj).getReferencedObject())).getType();
				if (BrUtil.isSimpleType(type)){
					break;
				}
			}
			steps.add(obj);
			if (obj instanceof ReferenceStep && ((ReferenceStep)obj).getReferencedObject() instanceof Property){
				Type type = ((Property)((ReferenceStep)obj).getReferencedObject()).getType();
				if (BrUtil.isSimpleType(type)){
					break;
				}
			}
		}
		mpe.getSteps().clear();
		mpe.getSteps().addAll(steps);
		return mpe;
	}
}
