package com.ibm.btools.te.ilm.heuristics.br.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.io.File;
import java.util.Iterator;

import org.eclipse.emf.ecore.EClass;

import com.ibm.btools.blm.ie.BusinessRuleTaskPresentationUtil;
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.models.ProcessModel;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRule;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleAction;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleTemplate;
import com.ibm.btools.bom.model.processes.businessrules.IfThenRule;
import com.ibm.btools.bom.model.processes.businessrules.TemplateInstanceRule;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.impl.TransformationRuleImpl;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.ExportOperationConstants;
import com.ibm.btools.te.ilm.ExportOperationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.BRReusableTaskRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.BRTaskRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.impl.BRReusableTaskRuleImpl;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.impl.BRTaskRuleImpl;
import com.ibm.btools.te.ilm.heuristics.br.BRRule;
import com.ibm.btools.te.ilm.heuristics.br.BRSetRule;
import com.ibm.btools.te.ilm.heuristics.br.BRTemplateRule;
import com.ibm.btools.te.ilm.heuristics.br.BrPackage;
import com.ibm.btools.te.ilm.heuristics.br.util.BRActionSerializer;
import com.ibm.btools.te.ilm.heuristics.br.util.BRConditionSerializer;
import com.ibm.btools.te.ilm.heuristics.br.util.BRExpressionSerializer;
import com.ibm.btools.te.ilm.heuristics.br.util.BrUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup;
import com.ibm.wbit.br.core.model.Condition;
import com.ibm.wbit.br.core.model.ConditionExpression;
import com.ibm.wbit.br.core.model.Expression;
import com.ibm.wbit.br.core.model.ModelFactory;
import com.ibm.wbit.br.core.model.ParameterValue;
import com.ibm.wbit.br.core.model.Rule;
import com.ibm.wbit.br.core.model.RuleTemplate;
import com.ibm.wbit.br.core.model.Variable;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>BR Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class BRRuleImpl extends TransformationRuleImpl implements BRRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	/* attribute indicate whether it is first time execution of this rule */
	private boolean firstTimeExec = true;	

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected BRRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return BrPackage.Literals.BR_RULE;
	}

	/**
	 * Map BOM BusinessRule to Business Rule constructs.
	 */
	@SuppressWarnings("unchecked")
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");
		
		//assertion
		if (getSource() == null || getSource().size() != 1) {
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
			return false;			
		}
			 
		//get sources of this rule - BusinessRule		
		BusinessRule businessRule = (BusinessRule)getSource().get(0);
		
  		//check if this is first time execute this rule
		if(!firstTimeExec){
			 return true;		  
		} else {
			firstTimeExec = false;
		}

		// create Business Rule (businessRule);		
		// Check if this is a template or standalone rule	
		// For a template based rule:
		// 		- populate the paramater values
		
		// For a stand alone rule:
		// 		- create and populate condition
		// 		- create and populate action		
		// populate presentation string
		
		Rule rule = null;
		if (businessRule instanceof IfThenRule) {
			rule = createIfThenRule((IfThenRule)businessRule);
		} else if (businessRule instanceof TemplateInstanceRule) {
			rule = createTemplateInstanceRule((TemplateInstanceRule)businessRule);
		}		
		
		// set the rule details and add it as a target		
		getTarget().add(rule);		
		setComplete(true);
		
		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();		
	}
	
	@SuppressWarnings("unchecked")
	private Rule createIfThenRule(IfThenRule ifThenRule) {
		if (ifThenRule == null) {
			return null;
		}
		
		// create a Rule that a template is based on
		// 		- create and populate condition
		// 		- create and populate action
		com.ibm.wbit.br.core.model.IfThenRule rule = ModelFactory.eINSTANCE.createIfThenRule();
		
		// set attributes
		AttributeValueProvider avp = null;
		BRConditionSerializer brConditionSerializer = BRConditionSerializer.getInstance();		
		BRActionSerializer brActionSerializer = BRActionSerializer.getInstance();
		BRExpressionSerializer brExpressionSerializer = BRExpressionSerializer.getInstance();
		if (brExpressionSerializer.getRoot() == null) {
			brExpressionSerializer.setRoot(getRoot());
		}
//		brExpressionSerializer.setBrRule((ActionRule)getParentRule().getParentRule());

		if (getParentRule() instanceof BRSetRule) {
			BRSetRuleImpl brSetRule = (BRSetRuleImpl)getParentRule();
			avp = brSetRule.getAvp();
		}
		
		String rName = BrUtil.createRuleName(ifThenRule, rule, avp, getContext());
		rule.setLabel(rName);
		// display name and description
		rule.setDisplayName(ifThenRule.getName());
		if (ifThenRule.getOwnedComment() != null && !ifThenRule.getOwnedComment().isEmpty()) {
			rule.setDescription(((Comment)ifThenRule.getOwnedComment().get(0)).getBody());
		}
			
		Condition convertedCondition = ModelFactory.eINSTANCE.createCondition();;
		ConditionExpression conditionExpression = null;
		
		// get, convert and set the expression for condition
		if (ifThenRule.getRuleCondition() != null) {			
			if (ifThenRule.getRuleCondition() != null && ifThenRule.getRuleCondition() instanceof StructuredOpaqueExpression &&
					((StructuredOpaqueExpression)ifThenRule.getRuleCondition()).getExpression() != null &&
					brConditionSerializer != null) {
				conditionExpression = brConditionSerializer.createConditionExpression(((StructuredOpaqueExpression)ifThenRule.getRuleCondition()).getExpression(), null);
				convertedCondition.setConditionExpression(conditionExpression);
			}				
		} 
		
		if (convertedCondition.getConditionExpression() == null) {
			conditionExpression = ModelFactory.eINSTANCE.createBooleanExpression();
			convertedCondition.setConditionExpression(conditionExpression);
		}
				
		rule.setIf(convertedCondition);

		rule.setThen(brActionSerializer.createActionBlock(ifThenRule.getRuleConsequence()));
		
		//set presentation
		if (ifThenRule.getPresentation() != null && ifThenRule.getPresentation().length() > 0) {
			rule.setComment(ifThenRule.getPresentation());
		} else {
			String presentation = BusinessRuleTaskPresentationUtil.getInstance().getDisplayablePresentation(ifThenRule);
			rule.setComment(presentation);
		}
		
		return rule;
	}

	@SuppressWarnings("unchecked")
	private Rule createTemplateInstanceRule(TemplateInstanceRule templateInstanceRule) {
		if (templateInstanceRule == null || templateInstanceRule.getTemplate() == null){
			return null;
		}
		
		com.ibm.wbit.br.core.model.TemplateInstanceRule rule = ModelFactory.eINSTANCE.createTemplateInstanceRule();
		// set attributes
		AttributeValueProvider avp = null;
		if (getParentRule() instanceof BRSetRule) {
			avp = ((BRSetRuleImpl)getParentRule()).getAvp();
		}
		
		String rName = BrUtil.createRuleName(templateInstanceRule, rule, avp, getContext());
		rule.setLabel(rName);
		// display name and description
		rule.setDisplayName(templateInstanceRule.getName());
		if (templateInstanceRule.getOwnedComment() != null && !templateInstanceRule.getOwnedComment().isEmpty()) {
			rule.setDescription(((Comment)templateInstanceRule.getOwnedComment().get(0)).getBody());
		}

		rule.setTemplateRef(findRuleTemplate(templateInstanceRule));

		// get all parameter values
		if (rule.getTemplateRef() != null &&
				rule.getTemplateRef().getParameter() != null &&
				!rule.getTemplateRef().getParameter().isEmpty() &&
				templateInstanceRule.getTemplate().getParameters() != null) {
			
			Iterator defIt = rule.getTemplateRef().getParameter().iterator();		//definitions from template
			Iterator valIt = templateInstanceRule.getParameterValues().iterator();				//values from instance
			
			Variable parameterDefinition = null;
			ValueSpecification parameterValueInstance = null;
			
			ParameterValue parameterValue = null;
			Expression value = null;
			StringBuffer povParmNames = new StringBuffer();
			
			while (valIt.hasNext() && defIt.hasNext()) {	// assert: same size of lists
				parameterDefinition = (Variable)defIt.next();
				parameterValueInstance = (ValueSpecification)valIt.next();
//				com.ibm.btools.bom.model.processes.businessrules.ParameterValue valueWrapper = (com.ibm.btools.bom.model.processes.businessrules.ParameterValue)valIt.next();
//				parameterValueInstance= valueWrapper.getValue();
				
//				handlePOV(getContext(),rule, valueWrapper, parameterDefinition.getVarName(), povParmNames);
				
				parameterValue = ModelFactory.eINSTANCE.createParameterValue();			
				parameterValue.setName(parameterDefinition.getVarName());
				value = ModelFactory.eINSTANCE.createExpression();
	
				value.setValue(BrUtil.getStringValueFromValueSpecification(parameterValueInstance));
				parameterValue.setValue(value);
				rule.getParameterValue().add(parameterValue);
			}
			if(povParmNames != null && povParmNames.length()>1){
				povParmNames.append(ExportOperationConstants.RIGHT_BRACKET_POV);
				//Add property to the rule
				ExportOperationUtil.createSystemCustomProperty(rule, ExportOperationConstants.BRR_CUSTOM_PROPERTY_NAME,povParmNames.toString());
			}
		}			
		//Note: presentation is inherited from template

		return rule;
	}
	// find RuleTemplate from the parent list of targets
	private RuleTemplate findRuleTemplate (TemplateInstanceRule templateInstanceRule) {
		if (templateInstanceRule == null) {
			return null;
		}
		
		RuleTemplate ruleTemplate = null;
		
		//RuleSet is parent for Rules and Templates
		if(getParentRule().getChildRules() != null) {
			Iterator it = getParentRule().getChildRules().iterator();
			TransformationRule transformationRule = null;
			while(it.hasNext()) {
				transformationRule = (TransformationRule) it.next();
				
				// check if UID of BOM BusinessRuleTemplate elements match
				if (transformationRule instanceof BRTemplateRule) {
					if (transformationRule.getSource() != null &&
							transformationRule.getSource().size() == 1 &&
							((BusinessRuleTemplate)transformationRule.getSource().get(0)).getUid().equals(templateInstanceRule.getTemplate().getUid())) {
						ruleTemplate = (RuleTemplate)transformationRule.getTarget().get(0);	//RuleTemplate is one and only target for BRTemplateRule
						break;
					}
				}
			}
		}
		
		return ruleTemplate;
	}
	
	/**
	 * We only handle local business rule task for process hint, which is the full qualified path of the process
	 * @param context
	 * @param templateInstanceRule
	 * @param valueWrapper
	 * @param paramName 
	 * @param povParams
	 */
//	private void handlePOV(TransformationContext context,com.ibm.wbit.br.core.model.TemplateInstanceRule templateInstanceRule, com.ibm.btools.bom.model.processes.businessrules.ParameterValue valueWrapper, String paramName, StringBuffer povParams) {
//		if(context == null || valueWrapper == null || !valueWrapper.isIsPointOfVariability()){
//			return;
//		}
//		if(povParams == null){
//			povParams = new StringBuffer();
//		}
//		if(povParams.length()== 0){
//			povParams.append(ExportOperationConstants.LEFT_BRACKET_POV);
//		}		
//		if(paramName != null && povParams.length()==1){
//			povParams.append(paramName);
//		}else if (paramName != null && povParams.length()> 1){
//			povParams.append(ExportOperationConstants.PARAM_NAME_SEPARATOR_POV);
//			povParams.append(paramName);
//		}
//		//Find the enclosing Business Rule Group		
//		TransformationRule enclosingTaskRule = findEnclosingBRG();
//		BusinessRuleGroup brg = null;
//		BRReusableTaskRuleImpl brrtri = null;
//		BRTaskRuleImpl brtri = null;
//		String processHint = null;
//		if(enclosingTaskRule.getTarget() != null && enclosingTaskRule.getTarget().size() == 1){
//			if(enclosingTaskRule.getTarget().get(0) instanceof BusinessRuleGroup){
//				if(enclosingTaskRule instanceof BRReusableTaskRuleImpl){
//					brrtri = (BRReusableTaskRuleImpl)enclosingTaskRule;
//					brrtri.setExpose2BusinessSpace(true);						
//				}else if(enclosingTaskRule instanceof BRTaskRuleImpl){
//					brtri = (BRTaskRuleImpl)enclosingTaskRule;
//					brtri.setExpose2BusinessSpace(true);	
//					InputPinSet inputSet = (InputPinSet)brtri.getSource().get(0);
//					BusinessRuleAction businessRuleAction = (BusinessRuleAction)inputSet.getAction();
//					processHint = resolveProcessHintForAction(businessRuleAction);
//				}
//				brg = (BusinessRuleGroup)enclosingTaskRule.getTarget().get(0);
//			}				
//		}
//		if(brg != null){
//			if((brrtri != null && brrtri.isExpose2BusinessSpace()&& brrtri.isFirstTimeCreateSystemProperty())||
//					(brtri != null && brtri.isExpose2BusinessSpace()&& brtri.isFirstTimeCreateSystemProperty())){
//				ExportOperationUtil.createSystemCustomProperty(brg, ExportOperationConstants.BRR_CUSTOM_PROPERTY_NAME,ExportOperationConstants.TRUE_LIETRAL);
////				if(processHint != null && processHint.length() > 0 ){
////					ExportOperationUtil.createSystemCustomProperty(brg, ExportOperationConstants.PROCESS_HINT_POV,processHint);
////				}
//				if(brrtri != null){
//					brrtri.setFirstTimeCreateSystemProperty(false);
//				}else if(brtri != null){
//					brtri.setFirstTimeCreateSystemProperty(false);					
//				}
//			}
//		}
//		//Markt enclosing rule set
//		markRuleSet();
//	}
    
//	/**
//	 * Resolve the local business rule action's enclosing process full qualified path
//	 * like: My Cat1\My Cat2\My Process1
//	 * @param businessRuleAction
//	 * @return
//	 */
//	private String resolveProcessHintForAction(BusinessRuleAction businessRuleAction) {
//		String processHint = null;
//		if(businessRuleAction == null){
//			return processHint;
//		}
//		if(businessRuleAction.eContainer()instanceof StructuredActivityNode){
//			StructuredActivityNode action = ( StructuredActivityNode)businessRuleAction.eContainer();
//			 if(BomUtils.isProcess(action)){
//				 if(action.eContainer() instanceof Activity){
//					 StringBuffer sb = new StringBuffer();
//					 Activity processActivity = (Activity)action.eContainer();
//					 sb.append(processActivity.getName());
//					 com.ibm.btools.bom.model.artifacts.Package processCatalog = processActivity.getOwningPackage();
//					 while(processCatalog instanceof ProcessModel && !(ExportOperationConstants.ROOT_PROCESS_MODEL_POV.equals(processCatalog.getName()))){
//						 sb.insert(0, File.separator);
//						 sb.insert(0, processCatalog.getName());
//						 processCatalog = processCatalog.getOwningPackage();
//					 }
//					 processHint = sb.toString();
//				 }
//			 }
//		}
//		return processHint;
//	}

	/**
	 * Retrieve the owning business rule group
	 * @return
	 */
//	private TransformationRule findEnclosingBRG() {
//		TransformationRule parentRule = null;
//		parentRule = this.getParentRule();
//		while(parentRule != null && !(parentRule instanceof BRReusableTaskRule ||parentRule instanceof BRTaskRule)){
//			parentRule = parentRule.getParentRule();
//		}
//		if(parentRule instanceof BRReusableTaskRule ||parentRule instanceof BRTaskRule){
//			return parentRule;
//		}
//		return null;
//	}	
	/**
	 * Mark the business rule set flag as "expose to business space" so that we can set the 
	 * system property after the target (rule set) is available
	 * @return
	 */
//	private void markRuleSet() {
//		TransformationRule parentRule = null;
//		parentRule = this.getParentRule();
//		while(parentRule != null && !(parentRule instanceof BRSetRule)){
//			parentRule = parentRule.getParentRule();
//		}
//		if(parentRule instanceof BRSetRule){
//			BRSetRuleImpl bsri = (BRSetRuleImpl)parentRule;
//			if(bsri.isFirstTimeCreateSystemProperty()){
//				bsri.setExpose2BusinessSpace(true);				
//			}
//		}		
//	}	
} //BRRuleImpl