package com.ibm.btools.te.ilm.heuristics.br.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;

import com.ibm.btools.bom.model.artifacts.LiteralTime;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.businessrules.AssignmentConsequence;
import com.ibm.btools.bom.model.processes.businessrules.RuleConsequence;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.expression.model.DateLiteralExpression;
import com.ibm.btools.expression.model.DateTimeLiteralExpression;
import com.ibm.btools.expression.model.ModelPathExpression;
import com.ibm.btools.expression.model.PredefinedType;
import com.ibm.btools.expression.model.TimeLiteralExpression;
import com.ibm.btools.expression.util.LogUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.wbit.br.core.model.ActionBlock;
import com.ibm.wbit.br.core.model.Expression;
import com.ibm.wbit.br.core.model.ModelFactory;

/**
 * The BRExpressionSerializer is used to populate expressions for Business Rules.
 */
public class BRActionSerializer implements BRExpressionConstants {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	private static BRActionSerializer instance = new BRActionSerializer();

	private BRActionSerializer() {
		super();
	}

	public static BRActionSerializer getInstance() {
		return instance;
	}	

	/**
	 * Serializes a list of consequences an ActionBlock
	 * @param expression Expression
	 * @param parentCondition ConditionExpression
	 * @return ConditionExpression
	 * @throws BRSerializationException
	 */
	@SuppressWarnings("unchecked")
	public ActionBlock createActionBlock(final List<RuleConsequence> consequences) throws BRSerializationException {

		LogUtil.traceEntry(this, "createActionExpression(List consequences)", new String[] {"expression"}, new Object[] {consequences});

		// process multiple assignements, and add multiple actions
		ActionBlock actionBlock =  ModelFactory.eINSTANCE.createActionBlock();
		List<Expression> actionList = actionBlock.getAction();
		com.ibm.wbit.br.core.model.Expression actionExpression = null;		

		if (consequences == null || consequences.isEmpty()) {	// empty expression part
			actionExpression = ModelFactory.eINSTANCE.createExpression();
			actionList.add(actionExpression);	
			return actionBlock;
		}
		BRExpressionSerializer brExpressionSerializer = BRExpressionSerializer.getInstance();

		Iterator it = consequences.iterator();
		AssignmentConsequence aConsequence = null;
		ModelPathExpression assignementTarget = null;
		while (it.hasNext()) {
			aConsequence = (AssignmentConsequence)it.next();
			
			// assignement target is a MPE
			if ((aConsequence.getAssignmentTarget() instanceof StructuredOpaqueExpression) && 
					(((StructuredOpaqueExpression)aConsequence.getAssignmentTarget()).getExpression() instanceof ModelPathExpression)) {
				assignementTarget = (ModelPathExpression)((StructuredOpaqueExpression)aConsequence.getAssignmentTarget()).getExpression();
			} else {
				throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "createActionBlock");
			}

			if (PredefinedType.DATETIME.equals(assignementTarget.getEvaluatesToType())) {
				// create up to 6 assignements
				List <Expression> actionExpressionList = createExpressionsForDateTime(assignementTarget, aConsequence.getAssignedValue());
				actionList.addAll(actionExpressionList);
			} else if (PredefinedType.DATE.equals(assignementTarget.getEvaluatesToType())) {
				// create up to 3 assignements
				List <Expression> actionExpressionList = createExpressionsForDate(assignementTarget, aConsequence.getAssignedValue());
				actionList.addAll(actionExpressionList);
			} else if (PredefinedType.TIME.equals(assignementTarget.getEvaluatesToType())) {
				// create up to 3 assignements
				List <Expression> actionExpressionList = createExpressionsForTime(assignementTarget, aConsequence.getAssignedValue());
				actionList.addAll(actionExpressionList);
			} else {
				actionExpression = ModelFactory.eINSTANCE.createExpression();
				StringBuffer expressionValue = new StringBuffer(brExpressionSerializer.serialize(assignementTarget, true));
				if (expressionValue != null && expressionValue.toString() != null &&
						expressionValue.toString().endsWith(GET_FIRST_ELEMENT_OF_COLLECTION)) {
					expressionValue.delete(expressionValue.length() - 7, expressionValue.length());	// delete GET_FIRST_ELEMENT_OF_COLLECTION
					expressionValue.append(SET_FIRST_ELEMENT_OF_COLLECTION);
					expressionValue.append(brExpressionSerializer.serializeAssignement(aConsequence.getAssignedValue()));
					expressionValue.append(PARENTHESIS_END);
				} else {
					expressionValue.append(EQUALS_OP).append(brExpressionSerializer.serializeAssignement(aConsequence.getAssignedValue()));
				}
//				String expressionValue = brExpressionSerializer.serialize(assignementTarget) +
//							EQUALS_OP +
//							brExpressionSerializer.serializeAssignement(aConsequence.getAssignedValue());
				actionExpression.setValue(expressionValue.toString());
				actionList.add(actionExpression);
			}
		}
		
		LogUtil.traceExit(this, "createActionBlock(List consequences)", actionBlock);
		
		return actionBlock;
	}

	/**
	 * Serializes consequence of date time assignement.
	 * @param assignementTarget ModelPathExpression
	 * @param assignementValue ValueSpecification
	 * @return List
	 * @throws BRSerializationException
	 */
	private List <Expression> createExpressionsForDateTime(ModelPathExpression assignementTarget, ValueSpecification assignementValue) {
		if (assignementTarget == null || assignementValue == null) {
			return null;
		}
		
		if (assignementValue instanceof LiteralTime) {
			return createDateTimeLiteralActionAssignement(assignementTarget, (LiteralTime)assignementValue);
		} else if (assignementValue instanceof StructuredOpaqueExpression){
			com.ibm.btools.expression.model.Expression assignementExpressionValue = ((StructuredOpaqueExpression)assignementValue).getExpression();
			if (assignementExpressionValue instanceof DateTimeLiteralExpression) {
				return createDateTimeLiteralActionAssignement(assignementTarget, assignementExpressionValue);
			} else if (assignementExpressionValue instanceof ModelPathExpression) {
				return createDatesModelPathAssignement(assignementTarget, (ModelPathExpression)assignementExpressionValue);
			} else {
				return createDateTimeVariableActionAssignement(assignementTarget, assignementExpressionValue);
			}
		} else {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "createExpressionsForDateTime");
		}
	}
	
	/**
	 * Serializes consequence of date assignement.
	 * @param assignementTarget ModelPathExpression
	 * @param assignementValue ValueSpecification
	 * @return List
	 * @throws BRSerializationException
	 */
	private List <Expression> createExpressionsForDate(ModelPathExpression assignementTarget, ValueSpecification assignementValue) {
		if (assignementTarget == null || assignementValue == null) {
			return null;
		}
		List <Expression> expressions = null;
		if (assignementValue instanceof LiteralTime) {
			expressions = createDateLiteralActionAssignement(assignementTarget, (LiteralTime)assignementValue);
		} else if (assignementValue instanceof StructuredOpaqueExpression){			
			com.ibm.btools.expression.model.Expression assignementExpressionValue = ((StructuredOpaqueExpression)assignementValue).getExpression();
			if (assignementExpressionValue instanceof DateLiteralExpression) {
				expressions = createDateLiteralActionAssignement(assignementTarget, assignementExpressionValue);
			} else if (assignementExpressionValue instanceof ModelPathExpression) {
				expressions = createDatesModelPathAssignement(assignementTarget, (ModelPathExpression)assignementExpressionValue);
			} else {
				expressions = createDateVariableActionAssignement(assignementTarget, assignementExpressionValue);
			}
		}
		return expressions;
	}
	
	/**
	 * Serializes consequence of date assignement.
	 * @param assignementTarget ModelPathExpression
	 * @param assignementValue ValueSpecification
	 * @return List
	 * @throws BRSerializationException
	 */
	private List <Expression> createExpressionsForTime(ModelPathExpression assignementTarget, ValueSpecification assignementValue) {
		if (assignementTarget == null || assignementValue == null) {
			return null;
		}
		List <Expression> expressions = null;
		
		if (assignementValue instanceof LiteralTime) {
			expressions = createTimeLiteralActionAssignement(assignementTarget, (LiteralTime)assignementValue);
		} else if (assignementValue instanceof StructuredOpaqueExpression){			
			com.ibm.btools.expression.model.Expression assignementExpressionValue = ((StructuredOpaqueExpression)assignementValue).getExpression();
			if (assignementExpressionValue instanceof TimeLiteralExpression) {
				return createTimeLiteralActionAssignement(assignementTarget, assignementExpressionValue);
			} else if (assignementExpressionValue instanceof ModelPathExpression) {
				return createDatesModelPathAssignement(assignementTarget, (ModelPathExpression)assignementExpressionValue);
			} else{
				return createTimeVariableActionAssignement(assignementTarget, assignementExpressionValue);
			}
		}
		return expressions;
	}
	
	/**
	 * Serializes consequence of date time assignement for DateTimeLiteral.
	 * @param assignementTarget ModelPathExpression
	 * @param assignementValue Expression
	 * @return List
	 * @throws BRSerializationException
	 */
	private List <Expression> createDateTimeLiteralActionAssignement(ModelPathExpression assignementTarget, com.ibm.btools.expression.model.Expression assignementValue) {
		BRExpressionSerializer brExpressionSerializer = BRExpressionSerializer.getInstance();

		DateTimeLiteralExpression assignement = (DateTimeLiteralExpression)assignementValue;
		
		List <Expression> expressions = new ArrayList<Expression>(6);
		com.ibm.wbit.br.core.model.Expression actionExpression = null;
		String serializedAssignee = brExpressionSerializer.serialize(assignementTarget);
		
		// year
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		String expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					YEAR +
					EQUALS_OP +
					assignement.getYear();
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);
		
		// month
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					MONTH +
					EQUALS_OP +
					assignement.getMonth();
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// day
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					DAY +
					EQUALS_OP +
					assignement.getDay();
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// hour
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					HOUR +
					EQUALS_OP +
					assignement.getHour();
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// minute
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					MINUTE +
					EQUALS_OP +
					assignement.getMinute();
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// second
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					SECOND +
					EQUALS_OP +
					assignement.getSecond();
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);
		
		return expressions;
	}

	/**
	 * Serializes consequence of date time assignement for DateTimeLiteral.
	 * @param assignementTarget ModelPathExpression
	 * @param assignementValue LiteralTime
	 * @return List
	 * @throws BRSerializationException
	 */
	private List <Expression> createDateTimeLiteralActionAssignement(ModelPathExpression assignementTarget, LiteralTime assignementValue) {
		BRExpressionSerializer brExpressionSerializer = BRExpressionSerializer.getInstance();

		Calendar date = BrUtil.timeStringToDateTime(((LiteralTime)assignementValue).getValue());
		
		List <Expression> expressions = new ArrayList<Expression>(6);
		com.ibm.wbit.br.core.model.Expression actionExpression = null;
		String serializedAssignee = brExpressionSerializer.serialize(assignementTarget);
		
		// year
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		String expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					YEAR +
					EQUALS_OP +
					date.get(Calendar.YEAR);
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);
		
		// month
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					MONTH +
					EQUALS_OP +
					date.get(Calendar.MONTH);
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// day
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					DAY +
					EQUALS_OP +
					date.get(Calendar.DAY_OF_MONTH);
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// hour
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					HOUR +
					EQUALS_OP +
					date.get(Calendar.HOUR_OF_DAY);
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// minute
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					MINUTE +
					EQUALS_OP +
					date.get(Calendar.MINUTE);
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// second
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					SECOND +
					EQUALS_OP +
					date.get(Calendar.SECOND);
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);
		
		return expressions;
	}

	/**
	 * Serializes consequence of date time assignement for a variable.
	 * @param assignementTarget ModelPathExpression
	 * @param assignementValue Expression
	 * @return List
	 * @throws BRSerializationException
	 */
	private List <Expression> createDateTimeVariableActionAssignement(ModelPathExpression assignementTarget, com.ibm.btools.expression.model.Expression assignementValue) {	
		BRExpressionSerializer brExpressionSerializer = BRExpressionSerializer.getInstance();
		
		List <Expression>expressions = new ArrayList<Expression>(6);
		com.ibm.wbit.br.core.model.Expression actionExpression = null;

		String serializedAssignee = brExpressionSerializer.serialize(assignementTarget);
		String serializedAssignement = brExpressionSerializer.serialize(assignementValue);

		// year
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		String expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					YEAR +
					EQUALS_OP +
					serializedAssignement +
					CURRENT_STEP_SEP +
					YEAR;
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);
		
		// month
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					MONTH +
					EQUALS_OP +
					serializedAssignement +
					CURRENT_STEP_SEP +
					MONTH;
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// day
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					DAY +
					EQUALS_OP +
					serializedAssignement +
					CURRENT_STEP_SEP +
					DAY;
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// hour
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					HOUR +
					EQUALS_OP +
					serializedAssignement +
					CURRENT_STEP_SEP +
					HOUR;
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// minute
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					MINUTE +
					EQUALS_OP +
					serializedAssignement +
					CURRENT_STEP_SEP +
					MINUTE;
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// second
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					SECOND +
					EQUALS_OP +
					serializedAssignement +
					CURRENT_STEP_SEP +
					SECOND;
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);
		
		return expressions;
	}

	/**
	 * Serializes consequence of date assignement for a DateLiteralExpression.
	 * @param assignementTarget ModelPathExpression
	 * @param assignementValue Expression
	 * @return List
	 * @throws BRSerializationException
	 */
	private List <Expression> createDateLiteralActionAssignement(ModelPathExpression assignementTarget, com.ibm.btools.expression.model.Expression assignementValue) {
		BRExpressionSerializer brExpressionSerializer = BRExpressionSerializer.getInstance();
		DateLiteralExpression assignement = (DateLiteralExpression)assignementValue;
		
		List <Expression> expressions = new ArrayList<Expression>(3);
		com.ibm.wbit.br.core.model.Expression actionExpression = null;
		String serializedAssignee = brExpressionSerializer.serialize(assignementTarget);
		
		// year
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		String expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					YEAR +
					EQUALS_OP +
					assignement.getYear();
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);
		
		// month
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					MONTH +
					EQUALS_OP +
					assignement.getMonth();
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// day
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					DAY +
					EQUALS_OP +
					assignement.getDay();
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		return expressions;
	}

	/**
	 * Serializes consequence of date assignement for a LiteralTime.
	 * @param assignementTarget ModelPathExpression
	 * @param assignementValue LiteralTime
	 * @return List
	 * @throws BRSerializationException
	 */
	private List <Expression> createDateLiteralActionAssignement(ModelPathExpression assignementTarget, LiteralTime assignementValue) {
		BRExpressionSerializer brExpressionSerializer = BRExpressionSerializer.getInstance();
		Calendar date = BrUtil.timeStringToDateTime(((LiteralTime)assignementValue).getValue());
		
		List <Expression> expressions = new ArrayList<Expression>(3);
		com.ibm.wbit.br.core.model.Expression actionExpression = null;
		String serializedAssignee = brExpressionSerializer.serialize(assignementTarget);
		
		// year
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		String expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					YEAR +
					EQUALS_OP +
					date.get(Calendar.YEAR);
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);
		
		// month
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					MONTH +
					EQUALS_OP +
					date.get(Calendar.MONTH);
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// day
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					DAY +
					EQUALS_OP +
					date.get(Calendar.DAY_OF_MONTH);
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		return expressions;
	}

	/**
	 * Serializes consequence of date assignement for a variable.
	 * @param assignementTarget ModelPathExpression
	 * @param assignementValue Expression
	 * @return List
	 * @throws BRSerializationException
	 */
	private List <Expression> createDateVariableActionAssignement(ModelPathExpression assignementTarget, com.ibm.btools.expression.model.Expression assignementValue) {	
		BRExpressionSerializer brExpressionSerializer = BRExpressionSerializer.getInstance();
		List <Expression> expressions = new ArrayList<Expression>(3);
		com.ibm.wbit.br.core.model.Expression actionExpression = null;

		String serializedAssignee = brExpressionSerializer.serialize(assignementTarget);
		String serializedAssignement = brExpressionSerializer.serialize(assignementValue);

		// year
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		String expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					YEAR +
					EQUALS_OP +
					serializedAssignement +
					CURRENT_STEP_SEP +
					YEAR;
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);
		
		// month
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					MONTH +
					EQUALS_OP +
					serializedAssignement +
					CURRENT_STEP_SEP +
					MONTH;
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// day
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					DAY +
					EQUALS_OP +
					serializedAssignement +
					CURRENT_STEP_SEP +
					DAY;
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);
		
		return expressions;
	}

	/**
	 * Serializes consequence of time assignement for a LiteralTime.
	 * @param assignementTarget ModelPathExpression
	 * @param assignementValue LiteralTime
	 * @return List
	 * @throws BRSerializationException
	 */
	private List <Expression> createTimeLiteralActionAssignement(ModelPathExpression assignementTarget, LiteralTime assignementValue) {
		BRExpressionSerializer brExpressionSerializer = BRExpressionSerializer.getInstance();

		Calendar date = BrUtil.timeStringToTime(((LiteralTime)assignementValue).getValue());
		//TimeLiteralExpression assignement = (TimeLiteralExpression)assignementValue;
		Expression actionExpression = null;

		List <Expression> expressions = new ArrayList<Expression>(3);
		String serializedAssignee = brExpressionSerializer.serialize(assignementTarget);
		
		// hour
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		String expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					HOUR +
					EQUALS_OP +
					date.get(Calendar.HOUR_OF_DAY);
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// minute
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					MINUTE +
					EQUALS_OP +
					date.get(Calendar.MINUTE);
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// second
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					SECOND +
					EQUALS_OP +
					date.get(Calendar.SECOND);
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);
		
		return expressions;
	}
	
	/**
	 * Serializes consequence of time assignement for a TimeLiteralExpression.
	 * @param assignementTarget ModelPathExpression
	 * @param assignementValue Expression
	 * @return List
	 * @throws BRSerializationException
	 */
	private List <Expression> createTimeLiteralActionAssignement(ModelPathExpression assignementTarget, com.ibm.btools.expression.model.Expression assignementValue) {
		BRExpressionSerializer brExpressionSerializer = BRExpressionSerializer.getInstance();
		TimeLiteralExpression assignement = (TimeLiteralExpression)assignementValue;
		Expression actionExpression = null;

		List <Expression> expressions = new ArrayList<Expression>(3);
		String serializedAssignee = brExpressionSerializer.serialize(assignementTarget);
		
		// hour
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		String expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					HOUR +
					EQUALS_OP +
					assignement.getHour();
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// minute
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					MINUTE +
					EQUALS_OP +
					assignement.getMinute();
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// second
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					SECOND +
					EQUALS_OP +
					assignement.getSecond();
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);
		
		return expressions;
	}

	/**
	 * Serializes consequence of time assignement for a variable.
	 * @param assignementTarget ModelPathExpression
	 * @param assignementValue Expression
	 * @return List
	 * @throws BRSerializationException
	 */
	private List <Expression> createTimeVariableActionAssignement(ModelPathExpression assignementTarget, com.ibm.btools.expression.model.Expression assignementValue) {	
		BRExpressionSerializer brExpressionSerializer = BRExpressionSerializer.getInstance();
		List <Expression>expressions = new ArrayList<Expression>(3);
		com.ibm.wbit.br.core.model.Expression actionExpression = null;

		String serializedAssignee = brExpressionSerializer.serialize(assignementTarget);
		String serializedAssignement = brExpressionSerializer.serialize(assignementValue);

		// hour
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		String expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					HOUR +
					EQUALS_OP +
					serializedAssignement +
					CURRENT_STEP_SEP +
					HOUR;
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// minute
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					MINUTE +
					EQUALS_OP +
					serializedAssignement +
					CURRENT_STEP_SEP +
					MINUTE;
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);

		// second
		actionExpression = ModelFactory.eINSTANCE.createExpression();
		expressionValue = serializedAssignee +
					CURRENT_STEP_SEP +
					SECOND +
					EQUALS_OP +
					serializedAssignement +
					CURRENT_STEP_SEP +
					SECOND;
		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);
		
		return expressions;
	}

	/**
	 * Serializes dates for ModelPathExpression assignement.
	 * @param assignementTarget ModelPathExpression
	 * @param assignementValue ModelPathExpression
	 * @return List
	 * @throws BRSerializationException
	 */
	private List <Expression> createDatesModelPathAssignement(ModelPathExpression assignementTarget, ModelPathExpression assignementValue) {	
		BRExpressionSerializer brExpressionSerializer = BRExpressionSerializer.getInstance();
		List <Expression>expressions = new ArrayList<Expression>(1);
		com.ibm.wbit.br.core.model.Expression actionExpression = null;

		String serializedAssignee = brExpressionSerializer.serialize(assignementTarget);
		String serializedAssignement = brExpressionSerializer.serialize(assignementValue);

		actionExpression = ModelFactory.eINSTANCE.createExpression();
		String expressionValue = serializedAssignee +
					EQUALS_OP +
					serializedAssignement;

		actionExpression.setValue(expressionValue);
		expressions.add(actionExpression);
		return expressions;
	}

}