package com.ibm.btools.te.ilm.heuristics.br.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import com.ibm.btools.expression.model.BinaryLogicalBooleanExpression;
import com.ibm.btools.expression.model.BinaryOperatorExpression;
import com.ibm.btools.expression.model.ComparisonExpression;
import com.ibm.btools.expression.model.DateLiteralExpression;
import com.ibm.btools.expression.model.DateTimeLiteralExpression;
import com.ibm.btools.expression.model.Expression;
import com.ibm.btools.expression.model.FunctionExpression;
import com.ibm.btools.expression.model.IsKindOfExpression;
import com.ibm.btools.expression.model.LiteralExpression;
import com.ibm.btools.expression.model.LogicalBooleanOperator;
import com.ibm.btools.expression.model.ModelPathExpression;
import com.ibm.btools.expression.model.PredefinedType;
import com.ibm.btools.expression.model.TimeLiteralExpression;
import com.ibm.btools.expression.model.UnaryOperatorExpression;
import com.ibm.btools.expression.model.VariableExpression;
import com.ibm.btools.expression.resource.MessageKeys;
import com.ibm.btools.expression.util.LogUtil;
import com.ibm.wbit.br.core.model.BooleanExpression;
import com.ibm.wbit.br.core.model.ConditionExpression;
import com.ibm.wbit.br.core.model.LogicalAndExpression;
import com.ibm.wbit.br.core.model.LogicalExpression;
import com.ibm.wbit.br.core.model.LogicalOrExpression;
import com.ibm.wbit.br.core.model.ModelFactory;

/**
 * The BRExpressionSerializer is used to populate expressions for Business Rules.
 */
public class BRConditionSerializer implements BRExpressionConstants {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private static BRConditionSerializer instance = new BRConditionSerializer();

	private BRConditionSerializer() {
		super();
	}

	public static BRConditionSerializer getInstance() {
		return instance;
	}	

	/**
	 * Serializes an expression into an ConditionExpression, or adds to a parent condition
	 * @param expression Expression
	 * @param parentCondition ConditionExpression
	 * @return ConditionExpression
	 * @throws BRSerializationException
	 */
	public ConditionExpression createConditionExpression(final Expression expression, ConditionExpression parentCondition) throws BRSerializationException {

		LogUtil.traceEntry(this, "createConditionExpression(Expression expression, ConditionExpression parentCondition)", new String[] {"expression"}, new Object[] {expression});

		if (expression == null) {
			return null;
		}
		
		ConditionExpression newCondition = null;
		
		if (expression instanceof BinaryLogicalBooleanExpression) {
			newCondition = createBooleanConditionExpression((BinaryLogicalBooleanExpression)expression);
			if (parentCondition != null) {
				if (parentCondition instanceof LogicalAndExpression) {
					((LogicalAndExpression)parentCondition).addCondition(newCondition);
				} else if (parentCondition instanceof LogicalOrExpression) {
					((LogicalOrExpression)parentCondition).addCondition(newCondition);
				} else {
					throw BrUtil.createException(MessageKeys.INVALID_BOOLEAN_OP, null, "createConditionExpression");
				}	
			}
		} else if (expression instanceof ComparisonExpression &&
					((ComparisonExpression)expression).getFirstOperand() != null &&
					PredefinedType.DATETIME.equals(((ComparisonExpression)expression).getFirstOperand().getEvaluatesToType())) {
			newCondition = createBooleanConditionExpressionForDateTime((ComparisonExpression)expression);
			if (parentCondition != null) {
				if (parentCondition instanceof LogicalAndExpression) {
					((LogicalAndExpression)parentCondition).addCondition(newCondition);
				} else if (parentCondition instanceof LogicalOrExpression) {
					((LogicalOrExpression)parentCondition).addCondition(newCondition);
				} else {
					throw BrUtil.createException(MessageKeys.INVALID_BOOLEAN_OP, null, "createConditionExpression");
				}	
			}		
		} else if (expression instanceof ComparisonExpression &&
					((ComparisonExpression)expression).getFirstOperand() != null &&
					PredefinedType.DATE.equals(((ComparisonExpression)expression).getFirstOperand().getEvaluatesToType())) {
			newCondition = createBooleanConditionExpressionForDate((ComparisonExpression)expression);
			if (parentCondition != null) {
				if (parentCondition instanceof LogicalAndExpression) {
					((LogicalAndExpression)parentCondition).addCondition(newCondition);
				} else if (parentCondition instanceof LogicalOrExpression) {
					((LogicalOrExpression)parentCondition).addCondition(newCondition);
				} else {
					throw BrUtil.createException(MessageKeys.INVALID_BOOLEAN_OP, null, "createConditionExpression");
				}	
			}
		} else if (expression instanceof ComparisonExpression &&
					((ComparisonExpression)expression).getFirstOperand() != null &&
					PredefinedType.TIME.equals(((ComparisonExpression)expression).getFirstOperand().getEvaluatesToType())){
			newCondition = createBooleanConditionExpressionForTime((ComparisonExpression)expression);
			if (parentCondition != null) {
				if (parentCondition instanceof LogicalAndExpression) {
					((LogicalAndExpression)parentCondition).addCondition(newCondition);
				} else if (parentCondition instanceof LogicalOrExpression) {
					((LogicalOrExpression)parentCondition).addCondition(newCondition);
				} else {
					throw BrUtil.createException(MessageKeys.INVALID_BOOLEAN_OP, null, "createConditionExpression");
				}	
			}
		} else if (expression instanceof LiteralExpression ||		
					expression instanceof VariableExpression ||
					expression instanceof BinaryOperatorExpression ||
					expression instanceof UnaryOperatorExpression ||
					expression instanceof FunctionExpression ||
					expression instanceof ModelPathExpression ||
					expression instanceof IsKindOfExpression) {
			newCondition = ModelFactory.eINSTANCE.createBooleanExpression();
			BRExpressionSerializer brExpressionSerializer = BRExpressionSerializer.getInstance();
			((BooleanExpression)newCondition).setExpString(brExpressionSerializer.serialize(expression));
			
			if (parentCondition != null) {
				if (parentCondition instanceof LogicalAndExpression) {
					((LogicalAndExpression)parentCondition).addCondition(newCondition);
				} else if (parentCondition instanceof LogicalOrExpression) {
					((LogicalOrExpression)parentCondition).addCondition(newCondition);
				}
			}
		} else {
			throw BrUtil.createException(MessageKeys.INVALID_EXP, null, "createConditionExpression");
		}
		
		LogUtil.traceExit(this, "createConditionExpression(Expression expression, ConditionExpression parentCondition)", expression);
		
		return newCondition;
	}

	/**
	 * Create boolean logical condition based on expression
	 * @param logicExpression BinaryLogicalBooleanExpression
	 * @return LogicalExpression
	 * @throws BRSerializationException
	 */
	private LogicalExpression createBooleanConditionExpression(BinaryLogicalBooleanExpression logicExpression) throws BRSerializationException {
				
		LogUtil.traceEntry(this, "createBooleanConditionExpression(BinaryLogicalBooleanExpression logicExpression, ConditionExpression parentCondition)", new String[] {"logicExpression"}, new Object[] {logicExpression});
		
		LogicalExpression newCondition = null;
		
		LogicalBooleanOperator op = logicExpression.getOperator();
		if (LogicalBooleanOperator.AND_LITERAL.equals(op)) {
			newCondition = ModelFactory.eINSTANCE.createLogicalAndExpression();
			((LogicalAndExpression)newCondition).addCondition(createConditionExpression(logicExpression.getFirstOperand(), newCondition));
			((LogicalAndExpression)newCondition).addCondition(createConditionExpression(logicExpression.getSecondOperand(), newCondition));
		} else if (LogicalBooleanOperator.OR_LITERAL.equals(op)) {
			newCondition = ModelFactory.eINSTANCE.createLogicalOrExpression();
			((LogicalOrExpression)newCondition).addCondition(createConditionExpression(logicExpression.getFirstOperand(), newCondition));
			((LogicalOrExpression)newCondition).addCondition(createConditionExpression(logicExpression.getSecondOperand(), newCondition));
		} else {
			throw BrUtil.createException(MessageKeys.INVALID_BOOLEAN_OP, null, "createBooleanConditionExpression");
		}

		LogUtil.traceExit(this, "createBooleanConditionExpression(BinaryLogicalBooleanExpression logicExpression, ConditionExpression parentCondition)", newCondition);

		return newCondition;
	}

	
	/**
	 * Create boolean logical condition based on comparison expression on dateTime type operands
	 * @param logicExpression ComparisonExpression
	 * @return LogicalExpression
	 * @throws BRSerializationException
	 */
	private LogicalExpression createBooleanConditionExpressionForDateTime(ComparisonExpression compExpression) throws BRSerializationException {
				
		LogUtil.traceEntry(this, "createBooleanConditionExpressionForDateTime(ComparisonExpression compExpression)", new String[] {"compExpression"}, new Object[] {compExpression});
		BRExpressionSerializer brExpressionSerializer = BRExpressionSerializer.getInstance();
		LogicalAndExpression newCondition = ModelFactory.eINSTANCE.createLogicalAndExpression();
		
		final Expression firstOperand = compExpression.getFirstOperand();
		final Expression secondOperand = compExpression.getSecondOperand();		
		final String comparisonOperator = BRExpressionSerializer.serializeComparisonOperator(compExpression.getOperator());
		
		// special case for first and second operand being a ModelPathExpression AND (operator EQUAL or NOT_EQUAL)
		if (firstOperand instanceof ModelPathExpression && secondOperand instanceof ModelPathExpression &&
				(comparisonOperator.equals(EQUAL_TO_OP) || comparisonOperator.equals(NOT_EQUAL_TO_OP))) {
			String firstArgument = brExpressionSerializer.serialize(firstOperand);
			String secondArgument = brExpressionSerializer.serialize(secondOperand);
			newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForDates(firstArgument, secondArgument, comparisonOperator));			
		} else if (firstOperand instanceof DateTimeLiteralExpression) {
			DateTimeLiteralExpression firstArgument = (DateTimeLiteralExpression) firstOperand;
			if (secondOperand instanceof DateTimeLiteralExpression) {
				DateTimeLiteralExpression secondArgument = (DateTimeLiteralExpression) secondOperand;
				if (comparisonOperator.equals(EQUAL_TO_OP) || comparisonOperator.equals(NOT_EQUAL_TO_OP)) {
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForSecond(firstArgument, secondArgument, comparisonOperator));
				} else {	// >, >=, <, <= 
					LogicalOrExpression newORCondition = ModelFactory.eINSTANCE.createLogicalOrExpression();
					newORCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, comparisonOperator));
					LogicalAndExpression andConditionForMonth = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForMonth.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMonth.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForMonth);
					LogicalAndExpression andConditionForDay = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForDay);
					LogicalAndExpression andConditionForHour = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForHour.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForHour.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForHour.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForHour.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForHour);
					LogicalAndExpression andConditionForMinute = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForMinute);
					LogicalAndExpression andConditionForSecond = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForSecond(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForSecond);
					return newORCondition;
				}
			} else {	// string
				String secondArgument = null;
				if (secondOperand instanceof ModelPathExpression) {
					secondArgument = brExpressionSerializer.serialize(secondOperand);
				} else {
					secondArgument = PARENTHESIS_START + brExpressionSerializer.serialize(secondOperand) + PARENTHESIS_END;
				}
				if (comparisonOperator.equals(EQUAL_TO_OP) || comparisonOperator.equals(NOT_EQUAL_TO_OP)) {
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForSecond(firstArgument, secondArgument, comparisonOperator));
				} else {	// >, >=, <, <= 
					LogicalOrExpression newORCondition = ModelFactory.eINSTANCE.createLogicalOrExpression();
					newORCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, comparisonOperator));
					LogicalAndExpression andConditionForMonth = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForMonth.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMonth.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForMonth);
					LogicalAndExpression andConditionForDay = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForDay);
					LogicalAndExpression andConditionForHour = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForHour.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForHour.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForHour.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForHour.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForHour);
					LogicalAndExpression andConditionForMinute = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForMinute);
					LogicalAndExpression andConditionForSecond = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForSecond(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForSecond);
					return newORCondition;
				}
			}
		} else {	//string
			String firstArgument = null;	// this can be ModelPathExpression or SubExpression
			if (firstOperand instanceof ModelPathExpression) {
				firstArgument = brExpressionSerializer.serialize(firstOperand);
			} else {
				firstArgument = PARENTHESIS_START + brExpressionSerializer.serialize(firstOperand) + PARENTHESIS_END;
			}
			if (secondOperand instanceof DateTimeLiteralExpression) {
				DateTimeLiteralExpression secondArgument = (DateTimeLiteralExpression) secondOperand;
				if (comparisonOperator.equals(EQUAL_TO_OP) || comparisonOperator.equals(NOT_EQUAL_TO_OP)) {
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForSecond(firstArgument, secondArgument, comparisonOperator));
				} else {	// >, >=, <, <= 
					LogicalOrExpression newORCondition = ModelFactory.eINSTANCE.createLogicalOrExpression();
					newORCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, comparisonOperator));
					LogicalAndExpression andConditionForMonth = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForMonth.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMonth.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForMonth);
					LogicalAndExpression andConditionForDay = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForDay);
					LogicalAndExpression andConditionForHour = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForHour.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForHour.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForHour.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForHour.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForHour);
					LogicalAndExpression andConditionForMinute = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForMinute);
					LogicalAndExpression andConditionForSecond = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForSecond(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForSecond);
					return newORCondition;
				}					
			} else {	// string
				String secondArgument = null;
				if (secondOperand instanceof ModelPathExpression) {
					secondArgument = brExpressionSerializer.serialize(secondOperand);
				} else {
					secondArgument = PARENTHESIS_START + brExpressionSerializer.serialize(secondOperand) + PARENTHESIS_END;
				}
				if (comparisonOperator.equals(EQUAL_TO_OP) || comparisonOperator.equals(NOT_EQUAL_TO_OP)) {
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForSecond(firstArgument, secondArgument, comparisonOperator));
				} else {	// >, >=, <, <= 
					LogicalOrExpression newORCondition = ModelFactory.eINSTANCE.createLogicalOrExpression();
					newORCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, comparisonOperator));
					LogicalAndExpression andConditionForMonth = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForMonth.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMonth.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForMonth);
					LogicalAndExpression andConditionForDay = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForDay);
					LogicalAndExpression andConditionForHour = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForHour.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForHour.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForHour.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForHour.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForHour);
					LogicalAndExpression andConditionForMinute = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForMinute);
					LogicalAndExpression andConditionForSecond = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForSecond(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForSecond);
					return newORCondition;
				}					
			}
		}

		LogUtil.traceExit(this, "createBooleanConditionExpressionForDateTime(ComparisonExpression compExpression)", newCondition);

		return newCondition;
	}


	/**
	 * Create boolean logical condition based on comparison expression on date type operands
	 * @param logicExpression ComparisonExpression
	 * @return LogicalExpression
	 * @throws BRSerializationException
	 */
	private LogicalExpression createBooleanConditionExpressionForDate(ComparisonExpression compExpression) throws BRSerializationException {
				
		LogUtil.traceEntry(this, "createBooleanConditionExpressionForDate(ComparisonExpression compExpression)", new String[] {"compExpression"}, new Object[] {compExpression});
		
		LogicalAndExpression newCondition = ModelFactory.eINSTANCE.createLogicalAndExpression();;
		BRExpressionSerializer brExpressionSerializer = BRExpressionSerializer.getInstance();
		
		final Expression firstOperand = compExpression.getFirstOperand();
		final Expression secondOperand = compExpression.getSecondOperand();		
		final String comparisonOperator = BRExpressionSerializer.serializeComparisonOperator(compExpression.getOperator());

		// special case for first and second operand being a ModelPathExpression AND (operator EQUAL or NOT_EQUAL)
		if (firstOperand instanceof ModelPathExpression && secondOperand instanceof ModelPathExpression &&
				(comparisonOperator.equals(EQUAL_TO_OP) || comparisonOperator.equals(NOT_EQUAL_TO_OP))) {
			String firstArgument = brExpressionSerializer.serialize(firstOperand);
			String secondArgument = brExpressionSerializer.serialize(secondOperand);
			newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForDates(firstArgument, secondArgument, comparisonOperator));			
		} else if (firstOperand instanceof DateLiteralExpression) {
			DateLiteralExpression firstArgument = (DateLiteralExpression) firstOperand;
			if (secondOperand instanceof DateLiteralExpression) {
				DateLiteralExpression secondArgument = (DateLiteralExpression) secondOperand;
				if (comparisonOperator.equals(EQUAL_TO_OP) || comparisonOperator.equals(NOT_EQUAL_TO_OP)) {
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, comparisonOperator));
				} else {	// >, >=, <, <= 
					LogicalOrExpression newORCondition = ModelFactory.eINSTANCE.createLogicalOrExpression();
					newORCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, comparisonOperator));
					LogicalAndExpression andConditionForMonth = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForMonth.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMonth.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForMonth);
					LogicalAndExpression andConditionForDay = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForDay);
					return newORCondition;
				}
			} else {	// string
				String secondArgument = null;
				if (secondOperand instanceof ModelPathExpression) {
					secondArgument = brExpressionSerializer.serialize(secondOperand);
				} else {
					secondArgument = PARENTHESIS_START + brExpressionSerializer.serialize(secondOperand) + PARENTHESIS_END;
				}
				if (comparisonOperator.equals(EQUAL_TO_OP) || comparisonOperator.equals(NOT_EQUAL_TO_OP)) {
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, comparisonOperator));
				} else {	// >, >=, <, <= 
					LogicalOrExpression newORCondition = ModelFactory.eINSTANCE.createLogicalOrExpression();
					newORCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, comparisonOperator));
					LogicalAndExpression andConditionForMonth = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForMonth.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMonth.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForMonth);
					LogicalAndExpression andConditionForDay = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForDay);
					return newORCondition;
				}					
			}
		} else {	//string
			String firstArgument = null;	// this can be ModelPathExpression or SubExpression
			if (firstOperand instanceof ModelPathExpression) {
				firstArgument = brExpressionSerializer.serialize(firstOperand);
			} else {
				firstArgument = PARENTHESIS_START + brExpressionSerializer.serialize(firstOperand) + PARENTHESIS_END;
			}
			if (secondOperand instanceof DateLiteralExpression) {
				DateLiteralExpression secondArgument = (DateLiteralExpression) secondOperand;
				if (comparisonOperator.equals(EQUAL_TO_OP) || comparisonOperator.equals(NOT_EQUAL_TO_OP)) {
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, comparisonOperator));
				} else {	// >, >=, <, <= 
					LogicalOrExpression newORCondition = ModelFactory.eINSTANCE.createLogicalOrExpression();
					newORCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, comparisonOperator));
					LogicalAndExpression andConditionForMonth = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForMonth.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMonth.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForMonth);
					LogicalAndExpression andConditionForDay = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForDay);
					return newORCondition;
				}										
			} else {	// string
				String secondArgument = null;
				if (secondOperand instanceof ModelPathExpression) {
					secondArgument = brExpressionSerializer.serialize(secondOperand);
				} else {
					secondArgument = PARENTHESIS_START + brExpressionSerializer.serialize(secondOperand) + PARENTHESIS_END;
				}
				if (comparisonOperator.equals(EQUAL_TO_OP) || comparisonOperator.equals(NOT_EQUAL_TO_OP)) {
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, comparisonOperator));
				} else {	// >, >=, <, <= 
					LogicalOrExpression newORCondition = ModelFactory.eINSTANCE.createLogicalOrExpression();
					newORCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, comparisonOperator));
					LogicalAndExpression andConditionForMonth = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForMonth.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMonth.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForMonth);
					LogicalAndExpression andConditionForDay = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForYear(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForMonth(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForDay.addCondition(BRDateTimeSerializer.createBooleanExpressionForDay(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForDay);
					return newORCondition;
				}										
			}
		}

		LogUtil.traceExit(this, "createBooleanConditionExpressionForDate(ComparisonExpression compExpression)", newCondition);

		return newCondition;
	}

	/**
	 * Create boolean logical condition based on comparison expression on time type operands
	 * @param logicExpression ComparisonExpression
	 * @return LogicalExpression
	 * @throws BRSerializationException
	 */
	private LogicalExpression createBooleanConditionExpressionForTime(ComparisonExpression compExpression) throws BRSerializationException {
				
		LogUtil.traceEntry(this, "createBooleanConditionExpressionForDateTime(ComparisonExpression compExpression)", new String[] {"compExpression"}, new Object[] {compExpression});
		
		BRExpressionSerializer brExpressionSerializer = BRExpressionSerializer.getInstance();
		LogicalAndExpression newCondition = ModelFactory.eINSTANCE.createLogicalAndExpression();;
		
		final Expression firstOperand = compExpression.getFirstOperand();
		final Expression secondOperand = compExpression.getSecondOperand();		
		final String comparisonOperator = BRExpressionSerializer.serializeComparisonOperator(compExpression.getOperator());
		
		// special case for first and second operand being a ModelPathExpression AND (operator EQUAL or NOT_EQUAL)
		if (firstOperand instanceof ModelPathExpression && secondOperand instanceof ModelPathExpression &&
				(comparisonOperator.equals(EQUAL_TO_OP) || comparisonOperator.equals(NOT_EQUAL_TO_OP))) {
			String firstArgument = brExpressionSerializer.serialize(firstOperand);
			String secondArgument = brExpressionSerializer.serialize(secondOperand);
			newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForDates(firstArgument, secondArgument, comparisonOperator));			
		} else if (firstOperand instanceof TimeLiteralExpression) {
			TimeLiteralExpression firstArgument = (TimeLiteralExpression) firstOperand;
			if (secondOperand instanceof TimeLiteralExpression) {
				TimeLiteralExpression secondArgument = (TimeLiteralExpression) secondOperand;
				if (comparisonOperator.equals(EQUAL_TO_OP) || comparisonOperator.equals(NOT_EQUAL_TO_OP)) {
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForSecond(firstArgument, secondArgument, comparisonOperator));
				} else {	// >, >=, <, <= 
					LogicalOrExpression newORCondition = ModelFactory.eINSTANCE.createLogicalOrExpression();
					newORCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, comparisonOperator));
					LogicalAndExpression andConditionForMinute = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForMinute);
					LogicalAndExpression andConditionForSecond = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForSecond(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForSecond);
					return newORCondition;
				}
			} else {	// string
				String secondArgument = null;
				if (secondOperand instanceof ModelPathExpression) {
					secondArgument = brExpressionSerializer.serialize(secondOperand);
				} else {
					secondArgument = PARENTHESIS_START + brExpressionSerializer.serialize(secondOperand) + PARENTHESIS_END;
				}
				if (comparisonOperator.equals(EQUAL_TO_OP) || comparisonOperator.equals(NOT_EQUAL_TO_OP)) {
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForSecond(firstArgument, secondArgument, comparisonOperator));
				} else {	// >, >=, <, <= 
					LogicalOrExpression newORCondition = ModelFactory.eINSTANCE.createLogicalOrExpression();
					newORCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, comparisonOperator));
					LogicalAndExpression andConditionForMinute = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForMinute);
					LogicalAndExpression andConditionForSecond = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForSecond(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForSecond);
					return newORCondition;
				}					
			}
		} else {	//string
			String firstArgument = null;	// this can be ModelPathExpression or SubExpression
			if (firstOperand instanceof ModelPathExpression) {
				firstArgument = brExpressionSerializer.serialize(firstOperand);
			} else {
				firstArgument = PARENTHESIS_START + brExpressionSerializer.serialize(firstOperand) + PARENTHESIS_END;
			}
			if (secondOperand instanceof TimeLiteralExpression) {
				TimeLiteralExpression secondArgument = (TimeLiteralExpression) secondOperand;
				if (comparisonOperator.equals(EQUAL_TO_OP) || comparisonOperator.equals(NOT_EQUAL_TO_OP)) {
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForSecond(firstArgument, secondArgument, comparisonOperator));
				} else {	// >, >=, <, <= 
					LogicalOrExpression newORCondition = ModelFactory.eINSTANCE.createLogicalOrExpression();
					newORCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, comparisonOperator));
					LogicalAndExpression andConditionForMinute = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForMinute);
					LogicalAndExpression andConditionForSecond = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForSecond(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForSecond);
					return newORCondition;
				}					
			} else {	// string
				String secondArgument = null;
				if (secondOperand instanceof ModelPathExpression) {
					secondArgument = brExpressionSerializer.serialize(secondOperand);
				} else {
					secondArgument = PARENTHESIS_START + brExpressionSerializer.serialize(secondOperand) + PARENTHESIS_END;
				}
				if (comparisonOperator.equals(EQUAL_TO_OP) || comparisonOperator.equals(NOT_EQUAL_TO_OP)) {
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, comparisonOperator));
					newCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForSecond(firstArgument, secondArgument, comparisonOperator));
				} else {	// >, >=, <, <= 
					LogicalOrExpression newORCondition = ModelFactory.eINSTANCE.createLogicalOrExpression();
					newORCondition.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, comparisonOperator));
					LogicalAndExpression andConditionForMinute = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForMinute.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForMinute);
					LogicalAndExpression andConditionForSecond = ModelFactory.eINSTANCE.createLogicalAndExpression();
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForHour(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForMinute(firstArgument, secondArgument, EQUAL_TO_OP));
					andConditionForSecond.addCondition(BRDateTimeSerializer.createBooleanExpressionForSecond(firstArgument, secondArgument, comparisonOperator));
					newORCondition.addCondition(andConditionForSecond);
					return newORCondition;
				}					
			}
		}

		LogUtil.traceExit(this, "createBooleanConditionExpressionForTime(ComparisonExpression compExpression)", newCondition);

		return newCondition;
	}

}
