package com.ibm.btools.te.ilm.heuristics.br.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import com.ibm.btools.expression.model.DateLiteralExpression;
import com.ibm.btools.expression.model.DateTimeLiteralExpression;
import com.ibm.btools.expression.model.TimeLiteralExpression;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.wbit.br.core.model.BooleanExpression;
import com.ibm.wbit.br.core.model.ModelFactory;

public class BRDateTimeSerializer implements BRExpressionConstants {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	/**
	 * Serialize year for a string operand.
	 * @param serializedOperand String 
	 * @return String
	 */	
	protected static String serializeYear(String serializedOperand) {
		if (serializedOperand == null) {
			return null;
		}
			
		StringBuffer sb = new StringBuffer(serializedOperand);			
		sb.append(CURRENT_STEP_SEP).append(YEAR);
		
		return sb.toString();
	}

	/**
	 * Serialize year for a DateTimeLiteralExpression operand.
	 * @param serializedOperand String 
	 * @return String
	 */	
	protected static String serializeYear(DateTimeLiteralExpression operand) {
		if (operand == null) {
			return null;
		}
		StringBuffer sb = new StringBuffer();
		sb.append(operand.getYear());
		return sb.toString();
	}

	/**
	 * Serialize year for a DateLiteralExpression operand.
	 * @param serializedOperand String 
	 * @return String
	 */	
	protected static String serializeYear(DateLiteralExpression operand) {
		if (operand == null) {
			return null;
		}
			
		StringBuffer sb = new StringBuffer();
		sb.append(operand.getYear());
		return sb.toString();
	}

	/**
	 * Serialize month for a string operand.
	 * @param serializedOperand String 
	 * @return String
	 */	
	protected static String serializeMonth(String serializedOperand) {
		if (serializedOperand == null) {
			return null;
		}
			
		StringBuffer sb = new StringBuffer(serializedOperand);			
		sb.append(CURRENT_STEP_SEP).append(MONTH);
		
		return sb.toString();
	}

	/**
	 * Serialize month for a DateTimeLiteralExpression operand.
	 * @param serializedOperand String 
	 * @return String
	 */	
	protected static String serializeMonth(DateTimeLiteralExpression operand) {
		if (operand == null) {
			return null;
		}
		StringBuffer sb = new StringBuffer();
		sb.append(operand.getMonth());
		return sb.toString();
	}

	/**
	 * Serialize month for a DateLiteralExpression operand.
	 * @param serializedOperand String 
	 * @return String
	 */	
	protected static String serializeMonth(DateLiteralExpression operand) {
		if (operand == null) {
			return null;
		}
			
		StringBuffer sb = new StringBuffer();
		sb.append(operand.getMonth());
		return sb.toString();
	}

	/**
	 * Serialize day for a string operand.
	 * @param serializedOperand String 
	 * @return String
	 */	
	protected static String serializeDay(String serializedOperand) {
		if (serializedOperand == null) {
			return null;
		}
			
		StringBuffer sb = new StringBuffer(serializedOperand);			
		sb.append(CURRENT_STEP_SEP).append(DAY);
		
		return sb.toString();
	}

	/**
	 * Serialize day for a DateTimeLiteralExpression operand.
	 * @param serializedOperand String 
	 * @return String
	 */	
	protected static String serializeDay(DateTimeLiteralExpression operand) {
		if (operand == null) {
			return null;
		}
		StringBuffer sb = new StringBuffer();
		sb.append(operand.getDay());
		return sb.toString();
	}
	
	/**
	 * Serialize day for a DateLiteralExpression operand.
	 * @param serializedOperand String 
	 * @return String
	 */	
	protected static String serializeDay(DateLiteralExpression operand) {
		if (operand == null) {
			return null;
		}
			
		StringBuffer sb = new StringBuffer();
		sb.append(operand.getDay());
		return sb.toString();
	}

	/**
	 * Serialize hour for a string operand.
	 * @param serializedOperand String 
	 * @return String
	 */	
	protected static String serializeHour(String serializedOperand) {
		if (serializedOperand == null) {
			return null;
		}
			
		StringBuffer sb = new StringBuffer(serializedOperand);			
		sb.append(CURRENT_STEP_SEP).append(HOUR);
		
		return sb.toString();
	}

	/**
	 * Serialize hour for a DateTimeLiteralExpression operand.
	 * @param serializedOperand String 
	 * @return String
	 */	
	protected static String serializeHour(DateTimeLiteralExpression operand) {
		if (operand == null) {
			return null;
		}
		StringBuffer sb = new StringBuffer();
		sb.append(operand.getHour());
		return sb.toString();
	}

	/**
	 * Serialize hour for a TimeLiteralExpression operand.
	 * @param serializedOperand String 
	 * @return String
	 */	
	protected static String serializeHour(TimeLiteralExpression operand) {
		if (operand == null) {
			return null;
		}
			
		StringBuffer sb = new StringBuffer();
		sb.append(operand.getHour());
		return sb.toString();
	}

	/**
	 * Serialize minute for a string operand.
	 * @param serializedOperand String 
	 * @return String
	 */	
	protected static String serializeMinute(String serializedOperand) {
		if (serializedOperand == null) {
			return null;
		}
			
		StringBuffer sb = new StringBuffer(serializedOperand);			
		sb.append(CURRENT_STEP_SEP).append(MINUTE);
		
		return sb.toString();
	}

	/**
	 * Serialize minute for a DateTimeLiteralExpression operand.
	 * @param serializedOperand String 
	 * @return String
	 */	
	protected static String serializeMinute(DateTimeLiteralExpression operand) {
		if (operand == null) {
			return null;
		}
		StringBuffer sb = new StringBuffer();
		sb.append(operand.getMinute());
		return sb.toString();
	}

	/**
	 * Serialize minute for a TimeLiteralExpression operand.
	 * @param serializedOperand String 
	 * @return String
	 */	
	protected static String serializeMinute(TimeLiteralExpression operand) {
		if (operand == null) {
			return null;
		}
		StringBuffer sb = new StringBuffer();
		sb.append(operand.getMinute());
		return sb.toString();
	}

	/**
	 * Serialize seconds for a string operand.
	 * @param serializedOperand String 
	 * @return String
	 */	
	protected static String serializeSecond(String serializedOperand) {
		if (serializedOperand == null) {
			return null;
		}
			
		StringBuffer sb = new StringBuffer(serializedOperand);			
		sb.append(CURRENT_STEP_SEP).append(SECOND);
		
		return sb.toString();
	}

	/**
	 * Serialize seconds for a DateTimeLiteralExpression operand.
	 * @param serializedOperand String 
	 * @return String
	 */	
	protected static String serializeSecond(DateTimeLiteralExpression operand) {
		if (operand == null) {
			return null;
		}
		StringBuffer sb = new StringBuffer();
		sb.append(operand.getSecond());
		return sb.toString();
	}

	/**
	 * Serialize second for a TimeLiteralExpression operand.
	 * @param serializedOperand String 
	 * @return String
	 */	
	protected static String serializeSecond(TimeLiteralExpression operand) {
		if (operand == null) {
			return null;
		}
			
		StringBuffer sb = new StringBuffer();
		sb.append(operand.getSecond());
		return sb.toString();
	}

	/**
	 * Serialize date time.
	 * @param firstArgument DateTimeLiteralExpression 
	 * @param secondArgument DateTimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return String
	 */	
	protected static String serializeDateTime(DateTimeLiteralExpression firstArgument, DateTimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		final StringBuffer sb = new StringBuffer();
	
		// year
		sb.append(serializeYear(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeYear(secondArgument));
		
		sb.append(AND_OP);
		
		// month
		sb.append(serializeMonth(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeMonth(secondArgument));
		
		sb.append(AND_OP);
		
		// day
		sb.append(serializeDay(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeDay(secondArgument));
		
		sb.append(AND_OP);
		
		// hour
		sb.append(serializeHour(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeHour(secondArgument));
		
		sb.append(AND_OP);
		
		// minute
		sb.append(serializeMinute(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeMinute(secondArgument));
		
		sb.append(AND_OP);

		// second
		sb.append(serializeSecond(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeSecond(secondArgument));
				
		return sb.toString();
	}
	
	/**
	 * Serialize date time.
	 * @param firstArgument String 
	 * @param secondArgument DateTimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return String
	 */	
	protected static String serializeDateTime(String firstArgument, DateTimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		final StringBuffer sb = new StringBuffer();
	
		// year
		sb.append(serializeYear(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeYear(secondArgument));
		
		sb.append(AND_OP);
		
		// month
		sb.append(serializeMonth(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeMonth(secondArgument));
		
		sb.append(AND_OP);
		
		// day
		sb.append(serializeDay(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeDay(secondArgument));
		
		sb.append(AND_OP);
		
		// hour
		sb.append(serializeHour(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeHour(secondArgument));
		
		sb.append(AND_OP);
		
		// minute
		sb.append(serializeMinute(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeMinute(secondArgument));
		
		sb.append(AND_OP);

		// second
		sb.append(serializeSecond(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeSecond(secondArgument));
				
		return sb.toString();
	}
	
	/**
	 * Serialize date time.
	 * @param firstArgument DateTimeLiteralExpression 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return String
	 */	
	protected static String serializeDateTime(DateTimeLiteralExpression firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		final StringBuffer sb = new StringBuffer();
	
		// year
		sb.append(serializeYear(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeYear(secondArgument));
		
		sb.append(AND_OP);
		
		// month
		sb.append(serializeMonth(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeMonth(secondArgument));
		
		sb.append(AND_OP);
		
		// day
		sb.append(serializeDay(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeDay(secondArgument));
		
		sb.append(AND_OP);
		
		// hour
		sb.append(serializeHour(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeHour(secondArgument));
		
		sb.append(AND_OP);
		
		// minute
		sb.append(serializeMinute(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeMinute(secondArgument));
		
		sb.append(AND_OP);

		// second
		sb.append(serializeSecond(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeSecond(secondArgument));
				
		return sb.toString();
	}
	
	/**
	 * Serialize date time.
	 * @param firstArgument String 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return String
	 */	
	protected static String serializeDateTime(String firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		final StringBuffer sb = new StringBuffer();
	
		// year
		sb.append(serializeYear(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeYear(secondArgument));
		
		sb.append(AND_OP);
		
		// month
		sb.append(serializeMonth(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeMonth(secondArgument));
		
		sb.append(AND_OP);
		
		// day
		sb.append(serializeDay(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeDay(secondArgument));
		
		sb.append(AND_OP);
		
		// hour
		sb.append(serializeHour(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeHour(secondArgument));
		
		sb.append(AND_OP);
		
		// minute
		sb.append(serializeMinute(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeMinute(secondArgument));
		
		sb.append(AND_OP);

		// second
		sb.append(serializeSecond(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeSecond(secondArgument));
				
		return sb.toString();
	}
	
	/**
	 * Serialize date.
	 * @param firstArgument DateLiteralExpression 
	 * @param secondArgument DateLiteralExpression 
	 * @param comparisonOperator String 
	 * @return String
	 */	
	protected static String serializeDate(DateLiteralExpression firstArgument, DateLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		final StringBuffer sb = new StringBuffer();
	
		// year
		sb.append(serializeYear(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeYear(secondArgument));
		
		sb.append(AND_OP);
		
		// month
		sb.append(serializeMonth(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeMonth(secondArgument));
		
		sb.append(AND_OP);
		
		// day
		sb.append(serializeDay(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeDay(secondArgument));
		
		return sb.toString();
	}
	

	/**
	 * Serialize date.
	 * @param firstArgument String 
	 * @param secondArgument DateLiteralExpression 
	 * @param comparisonOperator String 
	 * @return String
	 */	
	protected static String serializeDate(String firstArgument, DateLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		final StringBuffer sb = new StringBuffer();
	
		// year
		sb.append(serializeYear(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeYear(secondArgument));
		
		sb.append(AND_OP);
		
		// month
		sb.append(serializeMonth(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeMonth(secondArgument));
		
		sb.append(AND_OP);
		
		// day
		sb.append(serializeDay(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeDay(secondArgument));
		
		return sb.toString();
	}
	

	/**
	 * Serialize date.
	 * @param firstArgument DateLiteralExpression 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return String
	 */	
	protected static String serializeDate(DateLiteralExpression firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		final StringBuffer sb = new StringBuffer();
	
		// year
		sb.append(serializeYear(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeYear(secondArgument));
		
		sb.append(AND_OP);
		
		// month
		sb.append(serializeMonth(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeMonth(secondArgument));
		
		sb.append(AND_OP);
		
		// day
		sb.append(serializeDay(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeDay(secondArgument));
						
		return sb.toString();
	}
	

	/**
	 * Serialize date.
	 * @param firstArgument String 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return String
	 */	
	protected static String serializeDate(String firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		final StringBuffer sb = new StringBuffer();
	
		// year
		sb.append(serializeYear(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeYear(secondArgument));
		
		sb.append(AND_OP);
		
		// month
		sb.append(serializeMonth(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeMonth(secondArgument));
		
		sb.append(AND_OP);
		
		// day
		sb.append(serializeDay(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeDay(secondArgument));
		
		sb.append(AND_OP);
		
		return sb.toString();
	}
	
	/**
	 * Serialize time.
	 * @param firstArgument TimeLiteralExpression 
	 * @param secondArgument TimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return String
	 */	
	protected static String serializeTime(TimeLiteralExpression firstArgument, TimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		final StringBuffer sb = new StringBuffer();
	
		// hour
		sb.append(serializeHour(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeHour(secondArgument));
		
		sb.append(AND_OP);
		
		// minute
		sb.append(serializeMinute(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeMinute(secondArgument));
		
		sb.append(AND_OP);

		// second
		sb.append(serializeSecond(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeSecond(secondArgument));
				
		return sb.toString();
	}
	

	/**
	 * Serialize time.
	 * @param firstArgument String 
	 * @param secondArgument TimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return String
	 */	
	protected static String serializeTime(String firstArgument, TimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		final StringBuffer sb = new StringBuffer();
	
		// hour
		sb.append(serializeHour(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeHour(secondArgument));
		
		sb.append(AND_OP);
		
		// minute
		sb.append(serializeMinute(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeMinute(secondArgument));
		
		sb.append(AND_OP);

		// second
		sb.append(serializeSecond(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeSecond(secondArgument));
				
		return sb.toString();
	}
	

	/**
	 * Serialize time.
	 * @param firstArgument TimeLiteralExpression 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return String
	 */	
	protected static String serializeTime(TimeLiteralExpression firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		final StringBuffer sb = new StringBuffer();
	
		// hour
		sb.append(serializeHour(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeHour(secondArgument));
		
		sb.append(AND_OP);
		
		// minute
		sb.append(serializeMinute(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeMinute(secondArgument));
		
		sb.append(AND_OP);

		// second
		sb.append(serializeSecond(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeSecond(secondArgument));
				
		return sb.toString();
	}
	

	/**
	 * Serialize time.
	 * @param firstArgument String 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return String
	 */	
	protected static String serializeTime(String firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		final StringBuffer sb = new StringBuffer();
	
		// hour
		sb.append(serializeHour(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeHour(secondArgument));
		
		sb.append(AND_OP);
		
		// minute
		sb.append(serializeMinute(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeMinute(secondArgument));
		
		sb.append(AND_OP);

		// second
		sb.append(serializeSecond(firstArgument));
		sb.append(comparisonOperator);
		sb.append(serializeSecond(secondArgument));
				
		return sb.toString();
	}

	
	/**
	 * Create boolean expression for year.
	 * @param firstArgument DateTimeLiteralExpression 
	 * @param secondArgument DateTimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForYear(DateTimeLiteralExpression firstArgument, DateTimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeYear(firstArgument) + comparisonOperator + serializeYear(secondArgument));

		return booleanExpression;
	}

	/**
	 * Create boolean expression for year.
	 * @param firstArgument DateTimeLiteralExpression 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForYear(DateTimeLiteralExpression firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeYear(firstArgument) + comparisonOperator + serializeYear(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for year.
	 * @param firstArgument String 
	 * @param secondArgument DateTimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForYear(String firstArgument, DateTimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeYear(firstArgument) + comparisonOperator + serializeYear(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for year.
	 * @param firstArgument DateLiteralExpression 
	 * @param secondArgument DateLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForYear(DateLiteralExpression firstArgument, DateLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeYear(firstArgument) + comparisonOperator + serializeYear(secondArgument));

		return booleanExpression;
	}

	/**
	 * Create boolean expression for year.
	 * @param firstArgument DateLiteralExpression 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForYear(DateLiteralExpression firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeYear(firstArgument) + comparisonOperator + serializeYear(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for year.
	 * @param firstArgument String 
	 * @param secondArgument DateLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForYear(String firstArgument, DateLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeYear(firstArgument) + comparisonOperator + serializeYear(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for year.
	 * @param firstArgument String 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForYear(String firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeYear(firstArgument) + comparisonOperator + serializeYear(secondArgument));

		return booleanExpression;
	}

	/**
	 * Create boolean expression for month.
	 * @param firstArgument DateTimeLiteralExpression 
	 * @param secondArgument DateTimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForMonth(DateTimeLiteralExpression firstArgument, DateTimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeMonth(firstArgument) + comparisonOperator + serializeMonth(secondArgument));

		return booleanExpression;
	}

	/**
	 * Create boolean expression for month.
	 * @param firstArgument DateTimeLiteralExpression 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForMonth(DateTimeLiteralExpression firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeMonth(firstArgument) + comparisonOperator + serializeMonth(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for month.
	 * @param firstArgument String 
	 * @param secondArgument DateTimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForMonth(String firstArgument, DateTimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeMonth(firstArgument) + comparisonOperator + serializeMonth(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for month.
	 * @param firstArgument DateLiteralExpression 
	 * @param secondArgument DateLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForMonth(DateLiteralExpression firstArgument, DateLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeMonth(firstArgument) + comparisonOperator + serializeMonth(secondArgument));

		return booleanExpression;
	}

	/**
	 * Create boolean expression for month.
	 * @param firstArgument DateLiteralExpression 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForMonth(DateLiteralExpression firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeMonth(firstArgument) + comparisonOperator + serializeMonth(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for month.
	 * @param firstArgument String 
	 * @param secondArgument DateLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForMonth(String firstArgument, DateLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeMonth(firstArgument) + comparisonOperator + serializeMonth(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for month.
	 * @param firstArgument String 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForMonth(String firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeMonth(firstArgument) + comparisonOperator + serializeMonth(secondArgument));

		return booleanExpression;
	}

	/**
	 * Create boolean expression for day.
	 * @param firstArgument DateTimeLiteralExpression 
	 * @param secondArgument DateTimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForDay(DateTimeLiteralExpression firstArgument, DateTimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeDay(firstArgument) + comparisonOperator + serializeDay(secondArgument));

		return booleanExpression;
	}

	/**
	 * Create boolean expression for day.
	 * @param firstArgument DateTimeLiteralExpression 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForDay(DateTimeLiteralExpression firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeDay(firstArgument) + comparisonOperator + serializeDay(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for day.
	 * @param firstArgument String 
	 * @param secondArgument DateTimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForDay(String firstArgument, DateTimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeDay(firstArgument) + comparisonOperator + serializeDay(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for day.
	 * @param firstArgument DateLiteralExpression 
	 * @param secondArgument DateLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForDay(DateLiteralExpression firstArgument, DateLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeDay(firstArgument) + comparisonOperator + serializeDay(secondArgument));

		return booleanExpression;
	}

	/**
	 * Create boolean expression for day.
	 * @param firstArgument DateLiteralExpression 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForDay(DateLiteralExpression firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeDay(firstArgument) + comparisonOperator + serializeDay(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for day.
	 * @param firstArgument String 
	 * @param secondArgument DateLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForDay(String firstArgument, DateLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeDay(firstArgument) + comparisonOperator + serializeDay(secondArgument));

		return booleanExpression;
	}

	/**
	 * Create boolean expression for day.
	 * @param firstArgument String 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForDay(String firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeDay(firstArgument) + comparisonOperator + serializeDay(secondArgument));

		return booleanExpression;
	}

	/**
	 * Create boolean expression for hour.
	 * @param firstArgument DateTimeLiteralExpression 
	 * @param secondArgument DateTimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForHour(DateTimeLiteralExpression firstArgument, DateTimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeHour(firstArgument) + comparisonOperator + serializeHour(secondArgument));

		return booleanExpression;
	}

	/**
	 * Create boolean expression for hour.
	 * @param firstArgument DateTimeLiteralExpression 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForHour(DateTimeLiteralExpression firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeHour(firstArgument) + comparisonOperator + serializeHour(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for hour.
	 * @param firstArgument String 
	 * @param secondArgument DateTimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForHour(String firstArgument, DateTimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeHour(firstArgument) + comparisonOperator + serializeHour(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for hour.
	 * @param firstArgument TimeLiteralExpression 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForHour(TimeLiteralExpression firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeHour(firstArgument) + comparisonOperator + serializeHour(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for hour.
	 * @param firstArgument TimeLiteralExpression 
	 * @param secondArgument TimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForHour(TimeLiteralExpression firstArgument, TimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeHour(firstArgument) + comparisonOperator + serializeHour(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for hour.
	 * @param firstArgument String 
	 * @param secondArgument TimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForHour(String firstArgument, TimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeHour(firstArgument) + comparisonOperator + serializeHour(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for hour.
	 * @param firstArgument String 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForHour(String firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeHour(firstArgument) + comparisonOperator + serializeHour(secondArgument));

		return booleanExpression;
	}

	/**
	 * Create boolean expression for minute.
	 * @param firstArgument DateTimeLiteralExpression 
	 * @param secondArgument DateTimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForMinute(DateTimeLiteralExpression firstArgument, DateTimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeMinute(firstArgument) + comparisonOperator + serializeMinute(secondArgument));

		return booleanExpression;
	}

	/**
	 * Create boolean expression for minute.
	 * @param firstArgument DateTimeLiteralExpression 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForMinute(DateTimeLiteralExpression firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeMinute(firstArgument) + comparisonOperator + serializeMinute(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for minute.
	 * @param firstArgument String 
	 * @param secondArgument DateTimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForMinute(String firstArgument, DateTimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeMinute(firstArgument) + comparisonOperator + serializeMinute(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for minute.
	 * @param firstArgument TimeLiteralExpression 
	 * @param secondArgument TimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForMinute(TimeLiteralExpression firstArgument, TimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeMinute(firstArgument) + comparisonOperator + serializeMinute(secondArgument));

		return booleanExpression;
	}

	/**
	 * Create boolean expression for minute.
	 * @param firstArgument TimeLiteralExpression 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForMinute(TimeLiteralExpression firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeMinute(firstArgument) + comparisonOperator + serializeMinute(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for minute.
	 * @param firstArgument String 
	 * @param secondArgument TimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForMinute(String firstArgument, TimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeMinute(firstArgument) + comparisonOperator + serializeMinute(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for minute.
	 * @param firstArgument String 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForMinute(String firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeMinute(firstArgument) + comparisonOperator + serializeMinute(secondArgument));

		return booleanExpression;
	}

	/**
	 * Create boolean expression for seconds.
	 * @param firstArgument DateTimeLiteralExpression 
	 * @param secondArgument DateTimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForSecond(DateTimeLiteralExpression firstArgument, DateTimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeSecond(firstArgument) + comparisonOperator + serializeSecond(secondArgument));

		return booleanExpression;
	}

	/**
	 * Create boolean expression for seconds.
	 * @param firstArgument DateTimeLiteralExpression 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForSecond(DateTimeLiteralExpression firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeSecond(firstArgument) + comparisonOperator + serializeSecond(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for seconds.
	 * @param firstArgument String 
	 * @param secondArgument DateTimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForSecond(String firstArgument, DateTimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeSecond(firstArgument) + comparisonOperator + serializeSecond(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for seconds.
	 * @param firstArgument TimeLiteralExpression 
	 * @param secondArgument TimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForSecond(TimeLiteralExpression firstArgument, TimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeSecond(firstArgument) + comparisonOperator + serializeSecond(secondArgument));

		return booleanExpression;
	}

	/**
	 * Create boolean expression for seconds.
	 * @param firstArgument TimeLiteralExpression 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForSecond(TimeLiteralExpression firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeSecond(firstArgument) + comparisonOperator + serializeSecond(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for seconds.
	 * @param firstArgument String 
	 * @param secondArgument TimeLiteralExpression 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForSecond(String firstArgument, TimeLiteralExpression secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeSecond(firstArgument) + comparisonOperator + serializeSecond(secondArgument));

		return booleanExpression;
	}
	
	/**
	 * Create boolean expression for seconds.
	 * @param firstArgument String 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForSecond(String firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(serializeSecond(firstArgument) + comparisonOperator + serializeSecond(secondArgument));

		return booleanExpression;
	}

	/**
	 * Create boolean expression for dates.
	 * @param firstArgument String 
	 * @param secondArgument String 
	 * @param comparisonOperator String 
	 * @return BooleanExpression
	 */	
	protected static BooleanExpression createBooleanExpressionForDates(String firstArgument, String secondArgument, String comparisonOperator) {
		if (firstArgument == null || secondArgument == null || comparisonOperator == null) {
			throw BrUtil.createException(MessageKeys.INVALID_OPERAND, null, "serialize");
		}

		BooleanExpression booleanExpression = ModelFactory.eINSTANCE.createBooleanExpression();
		booleanExpression.setExpString(firstArgument + comparisonOperator + secondArgument);

		return booleanExpression;
	}

}
