package com.ibm.btools.te.ilm.heuristics.br.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.List;

import org.eclipse.emf.common.util.AbstractEnumerator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xsd.XSDComplexTypeDefinition;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDParticle;
import org.eclipse.xsd.XSDSimpleTypeDefinition;
import org.eclipse.xsd.XSDTypeDefinition;
import org.eclipse.xsd.XSDVariety;

import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.bom.model.artifacts.LiteralBoolean;
import com.ibm.btools.bom.model.artifacts.LiteralDuration;
import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.LiteralReal;
import com.ibm.btools.bom.model.artifacts.LiteralSpecification;
import com.ibm.btools.bom.model.artifacts.LiteralString;
import com.ibm.btools.bom.model.artifacts.LiteralTime;
import com.ibm.btools.bom.model.artifacts.Property;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleTemplate;
import com.ibm.btools.bom.model.processes.businessrules.TemplateParameter;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.expression.function.CoreFunctions;
import com.ibm.btools.expression.language.serialization.ExpressionSerializer;
import com.ibm.btools.expression.model.BinaryLogicalBooleanExpression;
import com.ibm.btools.expression.model.BinaryMathematicalOperator;
import com.ibm.btools.expression.model.BinaryNumericExpression;
import com.ibm.btools.expression.model.BinaryOperatorExpression;
import com.ibm.btools.expression.model.BooleanLiteralExpression;
import com.ibm.btools.expression.model.ComparisonExpression;
import com.ibm.btools.expression.model.ComparisonOperator;
import com.ibm.btools.expression.model.DateLiteralExpression;
import com.ibm.btools.expression.model.DateTimeLiteralExpression;
import com.ibm.btools.expression.model.DurationLiteralExpression;
import com.ibm.btools.expression.model.Expression;
import com.ibm.btools.expression.model.FunctionArgument;
import com.ibm.btools.expression.model.FunctionExpression;
import com.ibm.btools.expression.model.IntegerLiteralExpression;
import com.ibm.btools.expression.model.IsKindOfExpression;
import com.ibm.btools.expression.model.LiteralExpression;
import com.ibm.btools.expression.model.LogicalBooleanOperator;
import com.ibm.btools.expression.model.ModelPathExpression;
import com.ibm.btools.expression.model.NegationExpression;
import com.ibm.btools.expression.model.NotExpression;
import com.ibm.btools.expression.model.NumericLiteralExpression;
import com.ibm.btools.expression.model.PredefinedType;
import com.ibm.btools.expression.model.RealLiteralExpression;
import com.ibm.btools.expression.model.ReferenceStep;
import com.ibm.btools.expression.model.StringLiteralExpression;
import com.ibm.btools.expression.model.TimeLiteralExpression;
import com.ibm.btools.expression.model.UnaryOperatorExpression;
import com.ibm.btools.expression.model.UnionExpression;
import com.ibm.btools.expression.model.VariableExpression;
import com.ibm.btools.expression.resource.MessageKeys;
import com.ibm.btools.expression.util.LogUtil;
import com.ibm.btools.expression.util.TypeUtil;
import com.ibm.btools.expression.xpath.XPathOperatorPrecedenceHelper;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.wbit.br.core.model.RuleTemplate;
import com.ibm.wbit.br.core.model.Variable;

/**
 * The BRExpressionSerializer is used to populate expressions for Business Rules.
 */
public class BRExpressionSerializer extends ExpressionSerializer implements BRExpressionConstants {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	private static BRExpressionSerializer instance = new BRExpressionSerializer();
	private TransformationRoot root = null;
	
	private TransformationRule inputPinSetRule = null;
	private TransformationRule outputPinSetRule = null;
	private TransformationRule processInterfaceRule = null;
	private TransformationRule businessRuleSetRule = null;
	private JavaNCNameConverter javaNameConverter = new JavaNCNameConverter();

	private BRExpressionSerializer() {
		super();
	}

	public static BRExpressionSerializer getInstance() {
		return instance;
	}

	public TransformationRoot getRoot() {
		return root;
	}

	public void setRoot(TransformationRoot root) {
		this.root = root;
	}
	
	private TransformationRule getInputPinSetRule() {
		return inputPinSetRule;
	}

	public void setInputPinSetRule(TransformationRule inputPinSetRule) {
		this.inputPinSetRule = inputPinSetRule;
	}

	private TransformationRule getOutputPinSetRule() {
		return outputPinSetRule;
	}

	public void setOutputPinSetRule(TransformationRule outputPinSetRule) {
		this.outputPinSetRule = outputPinSetRule;
	}

	private TransformationRule getBusinessRuleSetRule() {
		return businessRuleSetRule;
	}

	public void setBusinessRuleSetRule(TransformationRule businessRuleSetRule) {
		this.businessRuleSetRule = businessRuleSetRule;
	}

	private TransformationRule getProcessInterfaceRule() {
		return processInterfaceRule;
	}

	public void setProcessInterfaceRule(TransformationRule processInterfaceRule) {
		this.processInterfaceRule = processInterfaceRule;
	}


	/**
	 * Serializes an expression into a string in table
	 * @param expression Expression
	 * @return expression String
	 * @throws BRSerializationException
	 */
	public String serialize(final Expression expression, boolean isAction) throws BRSerializationException {

		LogUtil.traceEntry(this, "serialize(Expression expression)", new String[] {"expression"}, new Object[] {expression});

		String result = null;
		if (expression == null) {
			return "";
		}
		if (expression instanceof LiteralExpression) {		
			result = serialize((LiteralExpression)expression);
		} else if (expression instanceof VariableExpression) {
			throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERATION, null, "serialize");
		} else if (expression instanceof BinaryOperatorExpression) {
			result = serialize((BinaryOperatorExpression)expression);
		} else if (expression instanceof UnaryOperatorExpression) {
			result = serialize((UnaryOperatorExpression)expression);
		} else if (expression instanceof FunctionExpression) {
			result = serialize((FunctionExpression)expression);
		} else if (expression instanceof ModelPathExpression) {
			result = serialize((ModelPathExpression)expression, isAction);
		} else if (expression instanceof IsKindOfExpression) {
			result = serialize((IsKindOfExpression)expression);
		} else {
			throw BrUtil.createException(MessageKeys.INVALID_EXP, null, "serialize");
		}
		
		LogUtil.traceExit(this, "serialize(Expression expression)", result);

		return result;
	}

	
	/**
	 * Serializes an expression into a string in table
	 * @param expression Expression
	 * @return expression String
	 * @throws BRSerializationException
	 */
	public String serialize(final Expression expression) throws BRSerializationException {

		LogUtil.traceEntry(this, "serialize(Expression expression)", new String[] {"expression"}, new Object[] {expression});

		String result = null;
		if (expression == null) {
			return "";
		}
		if (expression instanceof LiteralExpression) {		
			result = serialize((LiteralExpression)expression);
		} else if (expression instanceof VariableExpression) {
			throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERATION, null, "serialize");
		} else if (expression instanceof BinaryOperatorExpression) {
			result = serialize((BinaryOperatorExpression)expression);
		} else if (expression instanceof UnaryOperatorExpression) {
			result = serialize((UnaryOperatorExpression)expression);
		} else if (expression instanceof FunctionExpression) {
			result = serialize((FunctionExpression)expression);
		} else if (expression instanceof ModelPathExpression) {
			result = serialize((ModelPathExpression)expression, false);
		} else if (expression instanceof IsKindOfExpression) {
			result = serialize((IsKindOfExpression)expression);
		} else {
			throw BrUtil.createException(MessageKeys.INVALID_EXP, null, "serialize");
		}
		
		LogUtil.traceExit(this, "serialize(Expression expression)", result);

		return result;
	}

	
	/**
	 * Serializes an expression used in assignement into a string with brackets for binary expressions
	 * @param expression Expression
	 * @return expression String
	 * @throws BRSerializationException
	 */
	public String serializeAssignement(final Expression expression) throws BRSerializationException {

		LogUtil.traceEntry(this, "serializeAssignement(Expression expression)", new String[] {"expression"}, new Object[] {expression});

		String result = null;
		if (expression == null) {
			return "";
		}
		if (expression instanceof BinaryOperatorExpression) {
			result = serializeAssignement((BinaryOperatorExpression)expression);
		} else {
			result = serialize(expression);
		}
		
		LogUtil.traceExit(this, "serializeAssignement(Expression expression)", result);

		return result;
	}


	/**
	 * Serializes an expression into a String
	 * @param litExpression LiteralExpression
	 * @return String
	 * @throws BRSerializationException
	 */
	private String serialize(final LiteralExpression litExpression) throws BRSerializationException {

		LogUtil.traceEntry(this, "serialize(LiteralExpression litExpression)", new String[] {"litExpression"}, new Object[] {litExpression});
		
		boolean parenthesesRequired = areParenthesesRequired(litExpression);		
		final StringBuffer sb = new StringBuffer();
		
		if (parenthesesRequired) {
			sb.append(PARENTHESIS_START);
		}
		
		if (litExpression instanceof StringLiteralExpression) {
			sb.append(STRING_DECOR);
			if (((StringLiteralExpression)litExpression).getStringSymbol() != null) {
				sb.append(((StringLiteralExpression)litExpression).getStringSymbol());
			}
			sb.append(STRING_DECOR);
		} else if (litExpression instanceof BooleanLiteralExpression) {
			if (((BooleanLiteralExpression)litExpression).getBooleanSymbol() != null) {
				if (((BooleanLiteralExpression)litExpression).getBooleanSymbol().booleanValue()) {
					sb.append(TRUE_VALUE);
				} else {
					sb.append(FALSE_VALUE);
				}
			}
		} else if (litExpression instanceof IntegerLiteralExpression) {
			if (((IntegerLiteralExpression)litExpression).getIntegerSymbol() != null) {
				sb.append(((IntegerLiteralExpression)litExpression).getIntegerSymbol().toString());
			}
		} else if (litExpression instanceof RealLiteralExpression) {
			if (((RealLiteralExpression)litExpression).getRealSymbol() != null) {
				sb.append(((RealLiteralExpression)litExpression).getRealSymbol().toString());
			}
		} else if (litExpression instanceof NumericLiteralExpression) {
			if (((NumericLiteralExpression)litExpression).getNumericSymbol() != null) {
				sb.append(((NumericLiteralExpression)litExpression).getNumericSymbol());
			}
		} else if (litExpression instanceof DateLiteralExpression) {
			if (((DateLiteralExpression)litExpression).getDateSymbol() != null) {
				sb.append(((DateLiteralExpression)litExpression).getDateSymbol());
			}
		} else if (litExpression instanceof TimeLiteralExpression) {
			if (((TimeLiteralExpression)litExpression).getTimeSymbol() != null) {
				sb.append(((TimeLiteralExpression)litExpression).getTimeSymbol());
			}
		} else if (litExpression instanceof DateTimeLiteralExpression) {
			if (((DateTimeLiteralExpression)litExpression).getDateTimeSymbol() != null) {
				sb.append(((DateTimeLiteralExpression)litExpression).getDateTimeSymbol());
			}
		} else if (litExpression instanceof DurationLiteralExpression) {
			sb.append(STRING_DECOR);
			if (((DurationLiteralExpression)litExpression).getDurationSymbol() != null) {
				sb.append(((DurationLiteralExpression)litExpression).getDurationSymbol());
			}
			sb.append(STRING_DECOR);
		} else {
			throw BrUtil.createException(MessageKeys.INVALID_LITERAL_EXP, null, "serialize");
		}
		
		if (parenthesesRequired) {
			sb.append(PARENTHESIS_END);
		}

		LogUtil.traceExit(this, "serialize(LiteralExpression litExpression)", sb.toString());
		
		return sb.toString();
	}

	/**
	 * Serializes an expression into a String
	 * @param litSpecification LiteralSpecification
	 * @return String
	 * @throws BRSerializationException
	 */
	private String serialize(final LiteralSpecification litSpecification) throws BRSerializationException {

		LogUtil.traceEntry(this, "serialize(LiteralSpecification litSpecification)", new String[] {"litSpecification"}, new Object[] {litSpecification});
		
		final StringBuffer sb = new StringBuffer();
		
		if (litSpecification instanceof LiteralString) {
			LiteralString ls = (LiteralString)litSpecification;
			if (ls.getValue() != null) {
				sb.append(STRING_DECOR);
				sb.append(ls.getValue());
				sb.append(STRING_DECOR);
			}
		} else if (litSpecification instanceof LiteralBoolean) {
			LiteralBoolean lb = (LiteralBoolean)litSpecification;
			if (lb.getValue() != null) {
				if (((LiteralBoolean)litSpecification).getValue().booleanValue()) {
					sb.append(TRUE_VALUE);
				} else {
					sb.append(FALSE_VALUE);
				}
			}
		} else if (litSpecification instanceof LiteralInteger) {
			LiteralInteger li = (LiteralInteger)litSpecification;
			if (li.getValue() != null) {
				sb.append(li.getValue().toString());
			}
		} else if (litSpecification instanceof LiteralReal) {
			LiteralReal lr = (LiteralReal)litSpecification;
			if (lr.getValue() != null) {
				sb.append(lr.getValue().toString());
			}
		} else if (litSpecification instanceof LiteralTime) {		// handled separately
			throw BrUtil.createException(MessageKeys.INVALID_LITERAL_EXP, null, "serialize");
		} else if (litSpecification instanceof LiteralDuration) {	// not supported
			throw BrUtil.createException(MessageKeys.INVALID_LITERAL_EXP, null, "serialize");
		} else {
			throw BrUtil.createException(MessageKeys.INVALID_LITERAL_EXP, null, "serialize");
		}

		LogUtil.traceExit(this, "serialize(LiteralSpecification litSpecification)", sb.toString());
		
		return sb.toString();
	}

	/**
	 * Serializes a binary expression into a String
	 * @param expression the expression
	 * @return String
	 * @throws BRSerializationException
	 */
	private String serialize(BinaryOperatorExpression binExpression) throws BRSerializationException {
				
		LogUtil.traceEntry(this, "serialize(BinaryOperatorExpression binExpression)", new String[] {"binExpression"}, new Object[] {binExpression});
		
		String result = null;
		if (binExpression instanceof BinaryLogicalBooleanExpression) {
			result = serialize((BinaryLogicalBooleanExpression)binExpression);
		} else if (binExpression instanceof BinaryNumericExpression) {
			result = serialize((BinaryNumericExpression)binExpression);
		} else if (binExpression instanceof ComparisonExpression) {
			result = serialize((ComparisonExpression)binExpression);
		} else if (binExpression instanceof UnionExpression) {
			throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERATION, null, "serialize");
		} else {
			throw BrUtil.createException(MessageKeys.INVALID_BINARY_EXP, null, "serialize");
		}
		
		LogUtil.traceExit(this, "serialize(BinaryOperatorExpression binExpression)", result);

		return result;
	}
	
	/**
	 * Serializes a binary expression for assignement into a String
	 * @param expression the expression
	 * @return String
	 * @throws BRSerializationException
	 */
	private String serializeAssignement(BinaryOperatorExpression binExpression) throws BRSerializationException {
				
		LogUtil.traceEntry(this, "serializeAssignement(BinaryOperatorExpression binExpression)", new String[] {"binExpression"}, new Object[] {binExpression});
		
		String result = null;
		if (binExpression instanceof BinaryLogicalBooleanExpression) {
			result = serializeAssignement((BinaryLogicalBooleanExpression)binExpression);
		} else if (binExpression instanceof BinaryNumericExpression) {
			result = serializeAssignement((BinaryNumericExpression)binExpression);
		} else if (binExpression instanceof ComparisonExpression) {
			result = serializeAssignement((ComparisonExpression)binExpression);
		} else if (binExpression instanceof UnionExpression) {
			throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERATION, null, "serialize");
		} else {
			throw BrUtil.createException(MessageKeys.INVALID_BINARY_EXP, null, "serialize");
		}
		
		LogUtil.traceExit(this, "serializeAssignement(BinaryOperatorExpression binExpression)", result);

		return result;
	}
	
	/**
	 * Returns true if parentheses are required around the given binary expression.
	 */
	private boolean areParenthesesRequired(BinaryOperatorExpression binExpression) throws BRSerializationException {
		
		EObject container = binExpression.eContainer();
		
		if (container instanceof BinaryOperatorExpression) {
			
			AbstractEnumerator parentOperator = null;
			if (container instanceof BinaryLogicalBooleanExpression) {
				parentOperator = ((BinaryLogicalBooleanExpression)container).getOperator();
			} else if (container instanceof BinaryNumericExpression) {
				parentOperator = ((BinaryNumericExpression)container).getOperator();
			} else if (container instanceof ComparisonExpression) {
				parentOperator = ((ComparisonExpression)container).getOperator();
			} else {
				throw BrUtil.createException(MessageKeys.INVALID_BINARY_EXP, null, "areParenthesesRequired");
			}
			
			AbstractEnumerator ownOperator = null;
			if (binExpression instanceof BinaryLogicalBooleanExpression) {
				ownOperator = ((BinaryLogicalBooleanExpression)binExpression).getOperator();
			} else if (binExpression instanceof BinaryNumericExpression) {
				ownOperator = ((BinaryNumericExpression)binExpression).getOperator();
			} else if (binExpression instanceof ComparisonExpression) {
				ownOperator = ((ComparisonExpression)binExpression).getOperator();
			} else {
				throw BrUtil.createException(MessageKeys.INVALID_BINARY_EXP, null, "areParenthesesRequired");
			}
			
			// if own operator has a lower precedence than the parent operator
			// then parentheses are required
			if (XPathOperatorPrecedenceHelper.compareOperatorsPrecedence(ownOperator, parentOperator) < 0) {
				return true;
			}
			return false;
		} else if (container instanceof NegationExpression) {
			return true;
		}
		return false;
	}
	
	/**
	 * Returns true if parentheses are required around the given unary expression.
	 */
	private boolean areParenthesesRequired(UnaryOperatorExpression uniExpression) throws BRSerializationException {
		
		EObject container = uniExpression.eContainer();
		
		if (container instanceof NegationExpression) {
			return true;
		}
		return false;
	}
	
	/**
	 * Returns true if parentheses are required around the given literary expression.
	 */
	private boolean areParenthesesRequired(LiteralExpression litExpression) throws BRSerializationException {
		
		EObject container = litExpression.eContainer();
		
		if (container instanceof NegationExpression) {
			if (litExpression instanceof IntegerLiteralExpression) {
				if (((IntegerLiteralExpression)litExpression).getIntegerSymbol().intValue() < 0) {
					return true;
				}
			}
		}
		return false;
	}	
	
	/**
	 * Serializes a binary boolean expression into a String
	 * @param expression BinaryLogicalBooleanExpression
	 * @return String
	 * @throws BRSerializationException
	 */
	private String serialize(BinaryLogicalBooleanExpression logicExpression) throws BRSerializationException {
						
		LogUtil.traceEntry(this, "serialize(BinaryLogicalBooleanExpression logicExpression)", new String[] {"logicExpression"}, new Object[] {logicExpression});
		
		boolean parenthesesRequired = areParenthesesRequired(logicExpression);
		
		StringBuffer sb = new StringBuffer();
		
		if (parenthesesRequired) {
			sb.append(PARENTHESIS_START);
		}
		sb.append(serialize(logicExpression.getFirstOperand()));
		
		LogicalBooleanOperator op = logicExpression.getOperator();
		if (LogicalBooleanOperator.AND_LITERAL.equals(op)) {
			sb.append(AND_OP);
		} else if (LogicalBooleanOperator.OR_LITERAL.equals(op)) {
			sb.append(OR_OP);
		} else {
			throw BrUtil.createException(MessageKeys.INVALID_BOOLEAN_OP, null, "serialize");
		}
		
		sb.append(serialize(logicExpression.getSecondOperand()));
		if (parenthesesRequired) {
			sb.append(PARENTHESIS_END);
		}
				
		LogUtil.traceExit(this, "serialize(BinaryLogicalBooleanExpression logicExpression)", sb.toString());

		return sb.toString();
	}
	
	/**
	 * Serializes a binary boolean expression from assignement into a String
	 * @param expression BinaryLogicalBooleanExpression
	 * @return String
	 * @throws BRSerializationException
	 */
	private String serializeAssignement(BinaryLogicalBooleanExpression logicExpression) throws BRSerializationException {
						
		LogUtil.traceEntry(this, "serializeAssignement(BinaryLogicalBooleanExpression logicExpression)", new String[] {"logicExpression"}, new Object[] {logicExpression});
		
		StringBuffer sb = new StringBuffer();
		
		sb.append(PARENTHESIS_START);
		sb.append(serializeAssignement(logicExpression.getFirstOperand()));
		
		LogicalBooleanOperator op = logicExpression.getOperator();
		if (LogicalBooleanOperator.AND_LITERAL.equals(op)) {
			sb.append(AND_OP);
		} else if (LogicalBooleanOperator.OR_LITERAL.equals(op)) {
			sb.append(OR_OP);
		} else {
			throw BrUtil.createException(MessageKeys.INVALID_BOOLEAN_OP, null, "serialize");
		}
		
		sb.append(serializeAssignement(logicExpression.getSecondOperand()));
		sb.append(PARENTHESIS_END);
				
		LogUtil.traceExit(this, "serializeAssignement(BinaryLogicalBooleanExpression logicExpression)", sb.toString());

		return sb.toString();
	}
	

	/**
	 * Serializes a BinaryNumericExpression expression into a String
	 * @param expression BinaryNumericExpression
	 * @return String
	 * @throws BRSerializationException
	 */
	private String serialize(BinaryNumericExpression numExpression) throws BRSerializationException {
						
		LogUtil.traceEntry(this, "serialize(BinaryNumericExpression numExpression)", new String[] {"numExpression"}, new Object[] {numExpression});
		
		final Expression firstOperand = numExpression.getFirstOperand();
		final Expression secondOperand = numExpression.getSecondOperand();
		
		if (firstOperand != null) {			
			final String firstType = firstOperand.getEvaluatesToType();
			if (TypeUtil.isDateOrTimeType(firstType)
					|| PredefinedType.DURATION.equals(firstType)) {
				throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERAND, null, "serialize");
			}
		}
		
		if (secondOperand != null) {
			final String secondType = secondOperand.getEvaluatesToType();
			if (TypeUtil.isDateOrTimeType(secondType)
					|| PredefinedType.DURATION.equals(secondType)) {
				throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERAND, null, "serialize");
			}
		}
		
		boolean parenthesesRequired = areParenthesesRequired(numExpression) ;
		
		final StringBuffer sb = new StringBuffer();
		
		if (parenthesesRequired) {
			sb.append(PARENTHESIS_START);
		}
		sb.append(serialize(numExpression.getFirstOperand()));
		
		BinaryMathematicalOperator op = numExpression.getOperator();
		if (BinaryMathematicalOperator.ADDITION_LITERAL.equals(op)) {
			sb.append(ADDITION_OP);
		} else if (BinaryMathematicalOperator.SUBTRACTION_LITERAL.equals(op)) {
			sb.append(SUBTRACTION_OP);
		} else if (BinaryMathematicalOperator.MULTIPLICATION_LITERAL.equals(op)) {
			sb.append(MULTIPLICATION_OP);
		} else if (BinaryMathematicalOperator.DIVISION_LITERAL.equals(op)) {
			sb.append(DIVISION_OP);
		} else if (BinaryMathematicalOperator.MODULUS_LITERAL.equals(op)) {
			sb.append(MODULUS_OP);
		} else {
			throw BrUtil.createException(MessageKeys.INVALID_MATHEMATICAL_OP, null, "serialize");
		}
		
		sb.append(serialize(numExpression.getSecondOperand()));
		if (parenthesesRequired) {
			sb.append(PARENTHESIS_END);
		}
				
		LogUtil.traceExit(this, "serialize(BinaryNumericExpression numExpression)", sb.toString());

		return sb.toString();
	}
	
	/**
	 * Serializes a BinaryNumericExpression expression into a String
	 * @param expression BinaryNumericExpression
	 * @return String
	 * @throws BRSerializationException
	 */
	private String serializeAssignement(BinaryNumericExpression numExpression) throws BRSerializationException {
						
		LogUtil.traceEntry(this, "serialize(BinaryNumericExpression numExpression)", new String[] {"numExpression"}, new Object[] {numExpression});
		
		final Expression firstOperand = numExpression.getFirstOperand();
		final Expression secondOperand = numExpression.getSecondOperand();
		
		if (firstOperand != null) {			
			final String firstType = firstOperand.getEvaluatesToType();
			if (TypeUtil.isDateOrTimeType(firstType)
					|| PredefinedType.DURATION.equals(firstType)) {
				throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERAND, null, "serialize");
			}
		}
		
		if (secondOperand != null) {
			final String secondType = secondOperand.getEvaluatesToType();
			if (TypeUtil.isDateOrTimeType(secondType)
					|| PredefinedType.DURATION.equals(secondType)) {
				throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERAND, null, "serialize");
			}
		}
		
		final StringBuffer sb = new StringBuffer();
		
		sb.append(PARENTHESIS_START);
		sb.append(serializeAssignement(numExpression.getFirstOperand()));
		
		BinaryMathematicalOperator op = numExpression.getOperator();
		if (BinaryMathematicalOperator.ADDITION_LITERAL.equals(op)) {
			sb.append(ADDITION_OP);
		} else if (BinaryMathematicalOperator.SUBTRACTION_LITERAL.equals(op)) {
			sb.append(SUBTRACTION_OP);
		} else if (BinaryMathematicalOperator.MULTIPLICATION_LITERAL.equals(op)) {
			sb.append(MULTIPLICATION_OP);
		} else if (BinaryMathematicalOperator.DIVISION_LITERAL.equals(op)) {
			sb.append(DIVISION_OP);
		} else if (BinaryMathematicalOperator.MODULUS_LITERAL.equals(op)) {
			sb.append(MODULUS_OP);
		} else {
			throw BrUtil.createException(MessageKeys.INVALID_MATHEMATICAL_OP, null, "serialize");
		}
		
		sb.append(serializeAssignement(numExpression.getSecondOperand()));
		sb.append(PARENTHESIS_END);
				
		LogUtil.traceExit(this, "serialize(BinaryNumericExpression numExpression)", sb.toString());

		return sb.toString();
	}
	
	/**
	 * Serializes a ComparisonExpression expression into a String
	 * @param expression ComparisonExpression
	 * @return String
	 * @throws BRSerializationException
	 */
	private String serialize(final ComparisonExpression compExpression) throws BRSerializationException {
						
		LogUtil.traceEntry(this, "serialize(ComparisonExpression compExpression)", new String[] {"compExpression"}, new Object[] {compExpression});

		boolean parenthesesRequired = areParenthesesRequired(compExpression);
		
		final StringBuffer sb = new StringBuffer();

		if (parenthesesRequired) {
			sb.append(PARENTHESIS_START);
		}
		
		final Expression firstOperand = compExpression.getFirstOperand();
		final Expression secondOperand = compExpression.getSecondOperand();		
		final ComparisonOperator op = compExpression.getOperator();
		
		if (firstOperand == null || secondOperand == null || op == null) {
			throw BrUtil.createException(MessageKeys.INVALID_COMPARISION_OP, null, "serialize");
		}
		
		if (ComparisonOperator.EQUAL_TO_LITERAL.equals(op)) {
			if (PredefinedType.DURATION.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DURATION.equals(secondOperand.getEvaluatesToType())) {
				throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERAND, null, "serialize");
			//datetime
			} else if (PredefinedType.DATETIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATETIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateTimeComparison(compExpression));
			//date	
			} else if (PredefinedType.DATE.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATE.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateComparison(compExpression));
			//time	
			} else if (PredefinedType.TIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.TIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeTimeComparison(compExpression));
			} else {
				sb.append(serialize(firstOperand));
				sb.append(EQUAL_TO_OP);
				sb.append(serialize(secondOperand));
			}
			
		} else if (ComparisonOperator.NOT_EQUAL_TO_LITERAL.equals(op)) {
			if (PredefinedType.DURATION.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DURATION.equals(secondOperand.getEvaluatesToType())) {
				throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERAND, null, "serialize");
			//datetime
			} else if (PredefinedType.DATETIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATETIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateTimeComparison(compExpression));
			//date	
			} else if (PredefinedType.DATE.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATE.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateComparison(compExpression));
			//time	
			} else if (PredefinedType.TIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.TIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeTimeComparison(compExpression));
			} else {
				sb.append(serialize(firstOperand));
				sb.append(NOT_EQUAL_TO_OP);
				sb.append(serialize(secondOperand));
			}
		} else if (ComparisonOperator.LESS_THAN_LITERAL.equals(op)) {
			if (PredefinedType.DURATION.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DURATION.equals(secondOperand.getEvaluatesToType())) {
				throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERAND, null, "serialize");
			//datetime
			} else if (PredefinedType.DATETIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATETIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateTimeComparison(compExpression));
			//date	
			} else if (PredefinedType.DATE.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATE.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateComparison(compExpression));
			//time	
			} else if (PredefinedType.TIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.TIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeTimeComparison(compExpression));
			} else {
				sb.append(serialize(firstOperand));
				sb.append(LESS_THAN_OP);
				sb.append(serialize(secondOperand));
			}
		} else if (ComparisonOperator.LESS_THAN_OR_EQUAL_TO_LITERAL.equals(op)) {
			if (PredefinedType.DURATION.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DURATION.equals(secondOperand.getEvaluatesToType())) {
				throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERAND, null, "serialize");
			//datetime
			} else if (PredefinedType.DATETIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATETIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateTimeComparison(compExpression));
			//date	
			} else if (PredefinedType.DATE.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATE.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateComparison(compExpression));
			//time	
			} else if (PredefinedType.TIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.TIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeTimeComparison(compExpression));
			} else {
				sb.append(serialize(firstOperand));
				sb.append(LESS_THAN_OR_EQUAL_TO_OP);
				sb.append(serialize(secondOperand));
			}
		} else if (ComparisonOperator.GREATER_THAN_LITERAL.equals(op)) {
			if (PredefinedType.DURATION.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DURATION.equals(secondOperand.getEvaluatesToType())) {
				throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERAND, null, "serialize");
			//datetime
			} else if (PredefinedType.DATETIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATETIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateTimeComparison(compExpression));
			//date	
			} else if (PredefinedType.DATE.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATE.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateComparison(compExpression));
			//time	
			} else if (PredefinedType.TIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.TIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeTimeComparison(compExpression));
			} else {
				sb.append(serialize(firstOperand));
				sb.append(GREATER_THAN_OP);
				sb.append(serialize(secondOperand));
			}
		} else if (ComparisonOperator.GREATER_THAN_OR_EQUAL_TO_LITERAL.equals(op)) {
			if (PredefinedType.DURATION.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DURATION.equals(secondOperand.getEvaluatesToType())) {
				throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERAND, null, "serialize");
			//datetime
			} else if (PredefinedType.DATETIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATETIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateTimeComparison(compExpression));
			//date	
			} else if (PredefinedType.DATE.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATE.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateComparison(compExpression));
			//time	
			} else if (PredefinedType.TIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.TIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeTimeComparison(compExpression));
			} else {
				sb.append(serialize(firstOperand));
				sb.append(GREATER_THAN_OR_EQUAL_TO_OP);
				sb.append(serialize(secondOperand));
			}
		} else if (ComparisonOperator.IS_AFTER_LITERAL.equals(op)) {
			//datetime
			if (PredefinedType.DATETIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATETIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateTimeComparison(compExpression));
			//date	
			} else if (PredefinedType.DATE.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATE.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateComparison(compExpression));
			//time	
			} else if (PredefinedType.TIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.TIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeTimeComparison(compExpression));
			}
		} else if (ComparisonOperator.IS_BEFORE_LITERAL.equals(op)) {
			//datetime
			if (PredefinedType.DATETIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATETIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateTimeComparison(compExpression));
			//date	
			} else if (PredefinedType.DATE.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATE.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateComparison(compExpression));
			//time	
			} else if (PredefinedType.TIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.TIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeTimeComparison(compExpression));
			}
		} else {
			throw BrUtil.createException(MessageKeys.INVALID_COMPARISION_OP, null, "serialize");
		}
		
		if (parenthesesRequired) {
			sb.append(PARENTHESIS_END);
		}
						
		LogUtil.traceExit(this, "serialize(ComparisonExpression compExpression)", sb.toString());

		return sb.toString();
	}
	
	/**
	 * Serializes a ComparisonExpression expression into a String
	 * @param expression ComparisonExpression
	 * @return String
	 * @throws BRSerializationException
	 */
	private String serializeAssignement(final ComparisonExpression compExpression) throws BRSerializationException {
						
		LogUtil.traceEntry(this, "serialize(ComparisonExpression compExpression)", new String[] {"compExpression"}, new Object[] {compExpression});

		final StringBuffer sb = new StringBuffer();

		sb.append(PARENTHESIS_START);
		
		final Expression firstOperand = compExpression.getFirstOperand();
		final Expression secondOperand = compExpression.getSecondOperand();		
		final ComparisonOperator op = compExpression.getOperator();
		
		if (firstOperand == null || secondOperand == null || op == null) {
			throw BrUtil.createException(MessageKeys.INVALID_COMPARISION_OP, null, "serialize");
		}
		
		if (ComparisonOperator.EQUAL_TO_LITERAL.equals(op)) {
			if (PredefinedType.DURATION.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DURATION.equals(secondOperand.getEvaluatesToType())) {
				throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERAND, null, "serialize");
			//datetime
			} else if (PredefinedType.DATETIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATETIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateTimeComparison(compExpression));
			//date	
			} else if (PredefinedType.DATE.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATE.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateComparison(compExpression));
			//time	
			} else if (PredefinedType.TIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.TIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeTimeComparison(compExpression));
			} else {
				sb.append(serializeAssignement(firstOperand));
				sb.append(EQUAL_TO_OP);
				sb.append(serializeAssignement(secondOperand));
			}
		} else if (ComparisonOperator.NOT_EQUAL_TO_LITERAL.equals(op)) {
			if (PredefinedType.DURATION.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DURATION.equals(secondOperand.getEvaluatesToType())) {
				throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERAND, null, "serialize");
			//datetime
			} else if (PredefinedType.DATETIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATETIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateTimeComparison(compExpression));
			//date	
			} else if (PredefinedType.DATE.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATE.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateComparison(compExpression));
			//time	
			} else if (PredefinedType.TIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.TIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeTimeComparison(compExpression));
			} else {
				sb.append(serializeAssignement(firstOperand));
				sb.append(NOT_EQUAL_TO_OP);
				sb.append(serializeAssignement(secondOperand));
			}
		} else if (ComparisonOperator.LESS_THAN_LITERAL.equals(op)) {
			if (PredefinedType.DURATION.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DURATION.equals(secondOperand.getEvaluatesToType())) {
				throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERAND, null, "serialize");
			//datetime
			} else if (PredefinedType.DATETIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATETIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateTimeComparison(compExpression));
			//date	
			} else if (PredefinedType.DATE.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATE.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateComparison(compExpression));
			//time	
			} else if (PredefinedType.TIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.TIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeTimeComparison(compExpression));
			} else {
				sb.append(serializeAssignement(firstOperand));
				sb.append(LESS_THAN_OP);
				sb.append(serializeAssignement(secondOperand));
			}
		} else if (ComparisonOperator.LESS_THAN_OR_EQUAL_TO_LITERAL.equals(op)) {
			if (PredefinedType.DURATION.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DURATION.equals(secondOperand.getEvaluatesToType())) {
				throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERAND, null, "serialize");
			//datetime
			} else if (PredefinedType.DATETIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATETIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateTimeComparison(compExpression));
			//date	
			} else if (PredefinedType.DATE.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATE.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateComparison(compExpression));
			//time	
			} else if (PredefinedType.TIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.TIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeTimeComparison(compExpression));
			} else {
				sb.append(serializeAssignement(firstOperand));
				sb.append(LESS_THAN_OR_EQUAL_TO_OP);
				sb.append(serializeAssignement(secondOperand));
			}
		} else if (ComparisonOperator.GREATER_THAN_LITERAL.equals(op)) {
			if (PredefinedType.DURATION.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DURATION.equals(secondOperand.getEvaluatesToType())) {
				throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERAND, null, "serialize");
			//datetime
			} else if (PredefinedType.DATETIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATETIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateTimeComparison(compExpression));
			//date	
			} else if (PredefinedType.DATE.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATE.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateComparison(compExpression));
			//time	
			} else if (PredefinedType.TIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.TIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeTimeComparison(compExpression));
			} else {
				sb.append(serializeAssignement(firstOperand));
				sb.append(GREATER_THAN_OP);
				sb.append(serializeAssignement(secondOperand));
			}
		} else if (ComparisonOperator.GREATER_THAN_OR_EQUAL_TO_LITERAL.equals(op)) {
			if (PredefinedType.DURATION.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DURATION.equals(secondOperand.getEvaluatesToType())) {
				throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERAND, null, "serialize");
			//datetime
			} else if (PredefinedType.DATETIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATETIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateTimeComparison(compExpression));
			//date	
			} else if (PredefinedType.DATE.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATE.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateComparison(compExpression));
			//time	
			} else if (PredefinedType.TIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.TIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeTimeComparison(compExpression));
			} else {
				sb.append(serializeAssignement(firstOperand));
				sb.append(GREATER_THAN_OR_EQUAL_TO_OP);
				sb.append(serializeAssignement(secondOperand));
			}
		} else if (ComparisonOperator.IS_AFTER_LITERAL.equals(op)) {
			//datetime
			if (PredefinedType.DATETIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATETIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateTimeComparison(compExpression));
			//date	
			} else if (PredefinedType.DATE.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATE.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateComparison(compExpression));
			//time	
			} else if (PredefinedType.TIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.TIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeTimeComparison(compExpression));
			}
		} else if (ComparisonOperator.IS_BEFORE_LITERAL.equals(op)) {
			//datetime
			if (PredefinedType.DATETIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATETIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateTimeComparison(compExpression));
			//date	
			} else if (PredefinedType.DATE.equals(firstOperand.getEvaluatesToType()) && PredefinedType.DATE.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeDateComparison(compExpression));
			//time	
			} else if (PredefinedType.TIME.equals(firstOperand.getEvaluatesToType()) && PredefinedType.TIME.equals(secondOperand.getEvaluatesToType())) {
				sb.append(serializeTimeComparison(compExpression));
			}
		} else {
			throw BrUtil.createException(MessageKeys.INVALID_COMPARISION_OP, null, "serialize");
		}
		
		sb.append(PARENTHESIS_END);
		
		LogUtil.traceExit(this, "serialize(ComparisonExpression compExpression)", sb.toString());

		return sb.toString();
	}
	

	/**
	 * Serializes a UnaryOperatorExpression into a String
	 * @param expression UnaryOperatorExpression
	 * @return String
	 * @throws BRSerializationException
	 */
	private String serialize(UnaryOperatorExpression unaryExpression) throws BRSerializationException {
		
		if (unaryExpression instanceof NotExpression) {
			throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERATION, null, "serialize");
		} else if (unaryExpression instanceof NegationExpression) {
			return serialize((NegationExpression)unaryExpression);
		} else {
			throw new BRSerializationException(MessageKeys.INVALID_UNARY_EXP);
		}
	}
	
	/**
	 * Serializes a NegationExpression into a String
	 * @param expression NegationExpression
	 * @return String
	 * @throws BRSerializationException
	 */
	private String serialize(final NegationExpression negExpression) throws BRSerializationException {
						
		LogUtil.traceEntry(this, "serialize(NegationExpression negExpression)", new String[] {"negExpression"}, new Object[] {negExpression});
		
		boolean parenthesesRequired = areParenthesesRequired(negExpression);
		
		final StringBuffer sb = new StringBuffer();
		
		if (parenthesesRequired) {
			sb.append(PARENTHESIS_START);
		}
		
		sb.append(NEGATION_OP);
		sb.append(serialize(negExpression.getExpression()));
		
		if (parenthesesRequired) {
			sb.append(PARENTHESIS_END);
		}
						
		LogUtil.traceExit(this, "serialize(NegationExpression negExpression)", sb.toString());
		
		return sb.toString();
	}

	/**
	 * Serializes a FunctionExpression into a String
	 * @param expression FunctionExpression
	 * @return String
	 * @throws BRSerializationException
	 */
	private String serialize(final FunctionExpression funcExpression) throws BRSerializationException {
						
		LogUtil.traceEntry(this, "serialize(FunctionExpression funcExpression)", new String[] {"funcExpression"}, new Object[] {funcExpression});
		
		final StringBuffer sb = new StringBuffer();		
		String funcId = funcExpression.getDefinition().getFunctionID();
		
		if (CoreFunctions.SELECT_FUNCTION.equals(funcId)) {
			throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERATION, null, "serialize");
		} else if (CoreFunctions.FOR_ALL_FUNCTION.equals(funcId)) {
			throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERATION, null, "serialize");
		} else if (CoreFunctions.EXISTS_FUNCTION.equals(funcId)) {
			throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERATION, null, "serialize");
		} else if (CoreFunctions.GET_ALL_FUNCTION.equals(funcId)) {
			throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERATION, null, "serialize");
		} else if (CoreFunctions.SUM_FUNCTION.equals(funcId)) {
			throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERATION, null, "serialize");
		}else if (CoreFunctions.STARTS_WITH_FUNCTION.equals(funcId)) {
			sb.append(serializeStartFunction(funcExpression));
		}else if (CoreFunctions.CONTAINS_FUNCTION.equals(funcId)) {
			sb.append(serializeContainsFunction(funcExpression));
		}else if (CoreFunctions.SIZE_FUNCTION.equals(funcId)) {
			sb.append(serializeSizeFunction(funcExpression));
		}else {
			throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.NOT_SUPPORTED_OPERATION, null, "serialize");
		}
								
		LogUtil.traceExit(this, "serialize(FunctionExpression funcExpression)", sb.toString());

		return sb.toString();
	}

	/**
	 * @param funcExpression
	 * @return
	 */
	private String serializeStartFunction(FunctionExpression funcExpression) {
		
		LogUtil.traceEntry(this, "serializeStartFunction(FunctionExpression funcExpression)", new String[] {"funcExpression"}, new Object[] {funcExpression});
		
		if (funcExpression.getArguments() == null || funcExpression.getArguments().size() != 2) {
			BrUtil.createException(MessageKeys.INVALID_SELECT_FIRST_ARG, null, "serializeContainsFunction");
		}
		
		final StringBuffer sb = new StringBuffer();
		FunctionArgument firstArgument = (FunctionArgument)funcExpression.getArguments().get(0);
		Expression firstArgValue = firstArgument.getArgumentValue();		
		sb.append(serialize(firstArgValue));
		
		sb.append(CURRENT_STEP_SEP).append(STARTS_WITH_FUNC).append(PARENTHESIS_START);
		
		FunctionArgument secondArgument = (FunctionArgument)funcExpression.getArguments().get(1);
		Expression secondArgValue = secondArgument.getArgumentValue();
		sb.append(serialize(secondArgValue));
		sb.append(PARENTHESIS_END);

		return sb.toString();
	}

	/**
	 * Used to check if a string contains a substring
	 * @param funcExpression
	 * @return
	 */
	private String serializeContainsFunction(FunctionExpression funcExpression) {
		LogUtil.traceEntry(this, "serializeContainsFunction(FunctionExpression funcExpression)", new String[] {"funcExpression"}, new Object[] {funcExpression});

		if (funcExpression.getArguments() == null || funcExpression.getArguments().size() != 2) {
			BrUtil.createException(MessageKeys.INVALID_SELECT_FIRST_ARG, null, "serializeContainsFunction");
		}
		
		final StringBuffer sb = new StringBuffer();
		FunctionArgument firstArgument = (FunctionArgument)funcExpression.getArguments().get(0);
		Expression firstArgValue = firstArgument.getArgumentValue();		
		sb.append(serialize(firstArgValue));
		
		sb.append(CURRENT_STEP_SEP).append(CONTAINS_FUNC).append(PARENTHESIS_START);
		
		FunctionArgument secondArgument = (FunctionArgument)funcExpression.getArguments().get(1);
		Expression secondArgValue = secondArgument.getArgumentValue();
		sb.append(serialize(secondArgValue));
		sb.append(PARENTHESIS_END);
		
		sb.append(CHECK_STRING_EXIST);

		return sb.toString();
	}

	/**
 	 * Serialize assignement in business rule action assignement.
	 * @param assignedValue ValueSpecification
	 * @return String
	 */
	public String serializeAssignement(ValueSpecification assignedValue) {
		
		LogUtil.traceEntry(this, "serializeAssignement(ValueSpecification assignedValue)", new String[] {"funcExpression"}, new Object[] {assignedValue});
		
		if (assignedValue == null) {
			throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.INVALID_OPERAND, null, "serialize");
		}
		
		final StringBuffer sb = new StringBuffer();
		if (assignedValue instanceof StructuredOpaqueExpression) {
			final Expression expression = ((StructuredOpaqueExpression)assignedValue).getExpression();
			sb.append(serializeAssignement(expression));
		} else if (assignedValue instanceof LiteralSpecification) {
			sb.append(serialize((LiteralSpecification)assignedValue));
		} else {
			throw BrUtil.createException(MessageKeys.INVALID_EXP, null, "serialize");
		}
		
		LogUtil.traceExit(this, "serializeAssignement(ValueSpecification assignedValue)", sb.toString());

		return sb.toString();
	}

	
	/**
	 * Serializes "Size" function into a String
	 * @param expression FunctionExpression
	 * @return String
	 * @throws BRSerializationException
	 */
	private String serializeSizeFunction(final FunctionExpression funcExpression) throws BRSerializationException {
						
		LogUtil.traceEntry(this, "serializeSizeFunction(FunctionExpression funcExpression)", new String[] {"funcExpression"}, new Object[] {funcExpression});
		
		final StringBuffer sb = new StringBuffer();
		
		final List arguments = funcExpression.getArguments();
		FunctionArgument argument = null;
		
		if (arguments.size() != 1) {
			throw BrUtil.createException("Incorrect number of arguments for size function.  Arguments received: {0}", new String[] {arguments.toString()}, "serialize");
		}

		// elements argument			
		argument = (FunctionArgument)arguments.get(0);
		Expression argValue = argument.getArgumentValue();
		
		sb.append(serializeCollection(argValue));
		sb.append(CURRENT_STEP_SEP).append(SIZE_FUNC).append(PARENTHESIS_START).append(PARENTHESIS_END);
											
		LogUtil.traceExit(this, "serializeSizeFunction(FunctionExpression funcExpression)", sb.toString());

		return sb.toString();
	}

	/**
	 * Serializes a ModelPathExpression into a String
	 * @param expression ModelPathExpression
	 * @return String
	 * @throws BRSerializationException
	 */
	private String serialize(final ModelPathExpression pathExpression, boolean isAction) throws BRSerializationException {
						
		LogUtil.traceEntry(this, "serialize(ModelPathExpression pathExpression)", new String[] {"pathExpression"}, new Object[] {pathExpression});
				
		final StringBuffer sb = new StringBuffer();
		final List steps = pathExpression.getSteps();

		Object step = null;
		ReferenceStep referenceStep = null;
		String serializedName = null;
		XSDElementDeclaration xsdComplexTypePropertyElement = null;
		
		for (int i=0; i < steps.size(); i++) {
			serializedName = null;
			step = steps.get(i);
			if (step != null && step instanceof ReferenceStep) {
				referenceStep = (ReferenceStep)steps.get(i);
				if (referenceStep.getReferencedObject() instanceof InputObjectPin) {
					InputObjectPin pin = (InputObjectPin)referenceStep.getReferencedObject();
					InputPinSet iPinSet = (InputPinSet)((InputObjectPin)pin).getAction().getInputPinSet().get(0);
					int pinIndex = BrUtil.findInputPinIndex(pin, iPinSet);
					TransformationRule pinSetRule = null;
					if (getInputPinSetRule() == null) {
						if (getProcessInterfaceRule() != null) {
							pinSetRule = (TransformationRule)TransformationUtil.getRuleForSource(getProcessInterfaceRule(), iPinSet, XSDTypeDefinition.class);
						}
						if (pinSetRule == null) {
							pinSetRule = (TransformationRule)TransformationUtil.getRuleForSource(root, iPinSet, XSDTypeDefinition.class);
							if (pinSetRule != null) {
								setInputPinSetRule(pinSetRule);
							}
						} else {
							setInputPinSetRule(pinSetRule);
						}
					} else {
						pinSetRule = getInputPinSetRule();
					}
					if (pinSetRule != null) {	// unwrap the wrapper type
						serializedName = BrUtil.getNameFromPinSetRule(pinSetRule, pinIndex);
					}
			
					if (serializedName == null) {
						serializedName = javaNameConverter.convertName(referenceStep.getReferenceName());						
					} else if (BomUtils.isMultipled(pin)) {
						// get the wrapper type for the pin from the PinSetRule
						String wrappedType = BrUtil.getTypeForCollectionPinFromPinSetRule(pinSetRule, pinIndex);
						if (wrappedType != null) {
							serializedName = serializedName + CURRENT_STEP_SEP + wrappedType + GET_FIRST_ELEMENT_OF_COLLECTION;
						} else {
							serializedName = serializedName + GET_FIRST_ELEMENT_OF_COLLECTION;
						}
					}
				} else if (referenceStep.getReferencedObject() instanceof OutputObjectPin) {
					OutputObjectPin pin = (OutputObjectPin)referenceStep.getReferencedObject();
					OutputPinSet oPinSet = (OutputPinSet)((OutputObjectPin)pin).getAction().getOutputPinSet().get(0);
					int pinIndex = BrUtil.findOutputPinIndex(pin, oPinSet);
						
					TransformationRule pinSetRule = null;
					if (getOutputPinSetRule() == null) {
						if (getProcessInterfaceRule() != null) {
							pinSetRule = (TransformationRule)TransformationUtil.getRuleForSource(getProcessInterfaceRule(), oPinSet, XSDTypeDefinition.class);
						}
						if (pinSetRule == null) {
							pinSetRule = (TransformationRule)TransformationUtil.getRuleForSource(root, oPinSet, XSDTypeDefinition.class);
							if (pinSetRule != null) {
								setOutputPinSetRule(pinSetRule);
							}
						} else {
							setOutputPinSetRule(pinSetRule);
						}
					} else {
						pinSetRule = getOutputPinSetRule();
					}
					
					if (pinSetRule != null) {	// unwrap the wrapper type
						serializedName = BrUtil.getNameFromPinSetRule(pinSetRule, pinIndex);
					}
					if (serializedName == null) {
						serializedName = javaNameConverter.convertName(referenceStep.getReferenceName());						
					} else if (BomUtils.isMultipled(pin)) {
						// get the wrapper type for the pin from the PinSetRule
						String wrappedType = BrUtil.getTypeForCollectionPinFromPinSetRule(pinSetRule, pinIndex);
						if (wrappedType != null) {
							serializedName = serializedName + CURRENT_STEP_SEP + wrappedType + GET_FIRST_ELEMENT_OF_COLLECTION;
						} else {
							serializedName = serializedName + GET_FIRST_ELEMENT_OF_COLLECTION;
						}
					}
				// assert: template cannot be a collection
				} else if (referenceStep.getReferencedObject() instanceof TemplateParameter) {
					TemplateParameter templateParameter = (TemplateParameter)referenceStep.getReferencedObject();
					BusinessRuleTemplate businessRuleTemplate = (BusinessRuleTemplate)templateParameter.eContainer();
					TransformationRule brRule = null;
					
					if (getBusinessRuleSetRule() != null) {
						brRule = (TransformationRule)TransformationUtil.getRuleForSource(getBusinessRuleSetRule(), businessRuleTemplate, RuleTemplate.class);
					}
					if (brRule == null) {
						brRule = (TransformationRule)TransformationUtil.getRuleForSource(root, businessRuleTemplate, RuleTemplate.class);
					}
					
					RuleTemplate ruleTemplate = null;
					String propertyName = null;
					if (brRule != null) {
						ruleTemplate = (RuleTemplate)brRule.getTarget().get(0);
					
						// get index of template - retrieve Variable parameter of the same index
						int indexOfTemplateParameter = BrUtil.findIndexOfTemplateParameter(templateParameter, businessRuleTemplate);
						
						if (indexOfTemplateParameter > ruleTemplate.getParameter().size() - 1) {
							throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.INVALID_OPERAND, null, "findIndexOfTemplateParamter");						
						}
						
						Variable variable = (Variable)ruleTemplate.getParameter().get(indexOfTemplateParameter);						
						propertyName = variable.getVarName();
					}

					if (propertyName == null) {
						serializedName = javaNameConverter.convertName(referenceStep.getReferenceName());
					} else {
						serializedName = propertyName;
					}
				} else if (referenceStep.getReferencedObject() instanceof Property) {
					Property property = (Property)referenceStep.getReferencedObject();
					TransformationRule propertyRuleAttribute = (TransformationRule)TransformationUtil.getRuleForSource(root, property, XSDParticle.class);
					XSDParticle xsdPropertyParticle = null;
					if (propertyRuleAttribute != null) {
						xsdPropertyParticle = (XSDParticle)propertyRuleAttribute.getTarget().get(0);
//					} else {	// check if it is a collection 
//						TransformationRule propertyListRuleAttribute = (TransformationRule)TransformationUtil.getRuleForSource(root, property, XSDComplexTypeDefinition.class);						
					}

					String propertyName = null;
					if (xsdPropertyParticle != null && xsdPropertyParticle.getContent() instanceof XSDElementDeclaration) {
						XSDElementDeclaration xsdPropertyElement = (XSDElementDeclaration)xsdPropertyParticle.getContent();
						propertyName  = xsdPropertyElement.getName();
						if (xsdPropertyElement.getType() != null &&
								xsdPropertyElement.getType() instanceof XSDComplexTypeDefinition) {
							xsdComplexTypePropertyElement = xsdPropertyElement;
						}
					}
					
					if (propertyName == null) {
						if (BomUtils.isMultipled(property)) {
							serializedName = javaNameConverter.convertName(referenceStep.getReferenceName()) + GET_FIRST_ELEMENT_OF_COLLECTION;
						} else {
							serializedName = javaNameConverter.convertName(referenceStep.getReferenceName());
							if (referenceStep.getReferenceName() != null && referenceStep.getReferenceName().startsWith("@")) {
								serializedName = "@" + serializedName;
							}
						}
					} else if (BomUtils.isMultipled(property)) {
						serializedName = propertyName + GET_FIRST_ELEMENT_OF_COLLECTION;			
					} else {
						serializedName = propertyName;
					}
				}
			} else {
				throw BrUtil.createException(MessageKeys.INVALID_STEP, null, "serialize");
			}
				
			if (serializedName != null && !serializedName.equals("")) {
				sb.append(serializedName);
				// for simple types exit (skip property)
				if (referenceStep != null) {
					if (referenceStep.getReferencedObject() instanceof ObjectPin) {
						Type objectType = ((ObjectPin)(referenceStep.getReferencedObject())).getType();
						if (BrUtil.isSimpleType(objectType)) {
							break;
						}	
					} else if (referenceStep.getReferencedObject() instanceof Property) {
						Property prop = (Property) referenceStep.getReferencedObject();
						if (prop.getType() != null) {
							Type objectType = ((Property)referenceStep.getReferencedObject()).getType();
							if (BrUtil.isSimpleType(objectType)) {
								break;
							}
						}
						//d88902 - for derived, imported simpleTypes, check the owning Class, and remove the property name
						if (prop.eContainer() instanceof Class) {
							Class propOwner = (Class)prop.eContainer();
							if ("SchemaType".equals(propOwner.getAspect()) &&
									BrUtil.isSimpleType(propOwner)) {
								int sbLength = sb.length();
								sb.delete(sbLength - serializedName.length() - 1, sbLength);
								
								//for condition and complexTypes with simpleContent in case of XSD:UNION skip value
								if (!isAction && xsdComplexTypePropertyElement != null && xsdComplexTypePropertyElement.getType() != null &&
										xsdComplexTypePropertyElement.getType() instanceof XSDComplexTypeDefinition &&
										xsdComplexTypePropertyElement.getType().getBaseType() instanceof XSDSimpleTypeDefinition &&
										XSDVariety.UNION == ((XSDSimpleTypeDefinition)xsdComplexTypePropertyElement.getType().getBaseType()).getVariety().getValue()) {
									break;
								// for complexTypes with simpleContent or derived simpleTypes add value to the serialized Condition and Action string
								} else {
									sb.append(XSD_SIMPLE_CONTENT_VALUE);
								}
								break;
							}
						}
					}
				}
				if (i < steps.size() - 1) {
					sb.append(CURRENT_STEP_SEP);
				}
			}
		}
		
		String result = sb.toString();
		
		LogUtil.traceExit(this, "serialize(ModelPathExpression pathExpression)", result);
		
		return result;
	}
	
	
	/**
	 * Serializes a date time comparison into a String.
	 * @param compExpression ComparisonExpression
	 * @return String
	 */
	private String serializeDateTimeComparison(ComparisonExpression compExpression) {
		if (compExpression == null) {
			return null;
		}
		
		LogUtil.traceEntry(this, "serializeDateTimeComparison(ComparisonExpression compExpression)", new String[] {"serializeDateTimeComparison"}, new Object[] {compExpression});

		final StringBuffer sb = new StringBuffer();
		
		// always put date time comparison in brackets
		sb.append(PARENTHESIS_START);
		
		final Expression firstOperand = compExpression.getFirstOperand();
		final Expression secondOperand = compExpression.getSecondOperand();		
		String comparisonOperator = serializeComparisonOperator(compExpression.getOperator());
		
		if (firstOperand instanceof DateTimeLiteralExpression) {
			DateTimeLiteralExpression firstArgumentAsDTLE = (DateTimeLiteralExpression) firstOperand;
			if (secondOperand instanceof DateTimeLiteralExpression) {
				DateTimeLiteralExpression secondArgumentAsDTLE = (DateTimeLiteralExpression) secondOperand;
				sb.append(BRDateTimeSerializer.serializeDateTime(firstArgumentAsDTLE, secondArgumentAsDTLE, comparisonOperator));
			} else {	// string
				String secondArgumentAsString = null;
				if (secondOperand instanceof ModelPathExpression) {
					secondArgumentAsString = serialize(secondOperand);
				} else {
					secondArgumentAsString = PARENTHESIS_START + serialize(secondOperand) + PARENTHESIS_END;
				}
				sb.append(BRDateTimeSerializer.serializeDateTime(firstArgumentAsDTLE, secondArgumentAsString, comparisonOperator));
			}
		} else {	//string
			String firstArgumentAsString = null;	// this can be ModelPathExpression or SubExpression
			if (firstOperand instanceof ModelPathExpression) {
				firstArgumentAsString = serialize(firstOperand);
			} else {
				firstArgumentAsString = PARENTHESIS_START + serialize(firstOperand) + PARENTHESIS_END;
			}
			if (secondOperand instanceof DateTimeLiteralExpression) {
				DateTimeLiteralExpression secondArgumentAsDTLE = (DateTimeLiteralExpression) secondOperand;
				sb.append(BRDateTimeSerializer.serializeDateTime(firstArgumentAsString, secondArgumentAsDTLE, comparisonOperator));
			} else {	// string
				String secondArgumentAsString = null;
				if (secondOperand instanceof ModelPathExpression) {
					secondArgumentAsString = serialize(secondOperand);
				} else {
					secondArgumentAsString = PARENTHESIS_START + serialize(secondOperand) + PARENTHESIS_END;
				}
				sb.append(BRDateTimeSerializer.serializeDateTime(firstArgumentAsString, secondArgumentAsString, comparisonOperator));
			}
		}
		sb.append(PARENTHESIS_END);
		
		LogUtil.traceExit(this, "serializeDateTimeComparison(ComparisonExpression compExpression)", sb.toString());

		return sb.toString();
	}

	
	/**
	 * Serializes a date comparison into a String.
	 * @param compExpression ComparisonExpression
	 * @return String
	 */
	private String serializeDateComparison(ComparisonExpression compExpression) {
		if (compExpression == null) {
			return null;
		}
		
		LogUtil.traceEntry(this, "serializeDateComparison(ComparisonExpression compExpression)", new String[] {"serializeDateComparison"}, new Object[] {compExpression});

		final StringBuffer sb = new StringBuffer();
		
		// always put date comparison in brackets
		sb.append(PARENTHESIS_START);
		
		final Expression firstOperand = compExpression.getFirstOperand();
		final Expression secondOperand = compExpression.getSecondOperand();		
		String comparisonOperator = serializeComparisonOperator(compExpression.getOperator());
		
		if (firstOperand instanceof DateLiteralExpression) {
			DateLiteralExpression firstArgument = (DateLiteralExpression) firstOperand;
			if (secondOperand instanceof DateLiteralExpression) {
				DateLiteralExpression secondArgument = (DateLiteralExpression) secondOperand;
				sb.append(BRDateTimeSerializer.serializeDate(firstArgument, secondArgument, comparisonOperator));
			} else {	// string
				String secondArgumentAsString = null;
				if (secondOperand instanceof ModelPathExpression) {
					secondArgumentAsString = serialize(secondOperand);
				} else {
					secondArgumentAsString = PARENTHESIS_START + serialize(secondOperand) + PARENTHESIS_END;
				}
				sb.append(BRDateTimeSerializer.serializeDate(firstArgument, secondArgumentAsString, comparisonOperator));
			}
		} else {	//string
			String firstArgumentAsString = null;	// this can be ModelPathExpression or SubExpression
			if (firstOperand instanceof ModelPathExpression) {
				firstArgumentAsString = serialize(firstOperand);
			} else {
				firstArgumentAsString = PARENTHESIS_START + serialize(firstOperand) + PARENTHESIS_END;
			}
			if (secondOperand instanceof DateLiteralExpression) {
				DateLiteralExpression secondArgument = (DateLiteralExpression) secondOperand;
				sb.append(BRDateTimeSerializer.serializeDate(firstArgumentAsString, secondArgument, comparisonOperator));
			} else {	// string
				String secondArgumentAsString = null;
				if (secondOperand instanceof ModelPathExpression) {
					secondArgumentAsString = serialize(secondOperand);
				} else {
					secondArgumentAsString = PARENTHESIS_START + serialize(secondOperand) + PARENTHESIS_END;
				}
				sb.append(BRDateTimeSerializer.serializeDate(firstArgumentAsString, secondArgumentAsString, comparisonOperator));
			}
		}
		sb.append(PARENTHESIS_END);
						
		LogUtil.traceExit(this, "serializeDateComparison(ComparisonExpression compExpression)", sb.toString());

		return sb.toString();
	}
	
	
	/**
	 * Serializes time comparison into a String.
	 * @param compExpression ComparisonExpression
	 * @return String
	 */
	private String serializeTimeComparison(ComparisonExpression compExpression) {
		if (compExpression == null) {
			return null;
		}
		
		LogUtil.traceEntry(this, "serializeTimeComparison(ComparisonExpression compExpression)", new String[] {"serializeTimeComparison"}, new Object[] {compExpression});
		final StringBuffer sb = new StringBuffer();

		// always put date comparison in brackets
		sb.append(PARENTHESIS_START);
		
		final Expression firstOperand = compExpression.getFirstOperand();
		final Expression secondOperand = compExpression.getSecondOperand();		
		String comparisonOperator = serializeComparisonOperator(compExpression.getOperator());
		
		if (firstOperand instanceof TimeLiteralExpression) {
			TimeLiteralExpression firstArgument = (TimeLiteralExpression) firstOperand;
			if (secondOperand instanceof TimeLiteralExpression) {
				TimeLiteralExpression secondArgument = (TimeLiteralExpression) secondOperand;
				sb.append(BRDateTimeSerializer.serializeTime(firstArgument, secondArgument, comparisonOperator));
			} else {	// string
				String secondArgumentAsString = null;
				if (secondOperand instanceof ModelPathExpression) {
					secondArgumentAsString = serialize(secondOperand);
				} else {
					secondArgumentAsString = PARENTHESIS_START + serialize(secondOperand) + PARENTHESIS_END;
				}
				sb.append(BRDateTimeSerializer.serializeTime(firstArgument, secondArgumentAsString, comparisonOperator));
			}
		} else {	//string
			String firstArgumentAsString = null;	// this can be ModelPathExpression or SubExpression
			if (firstOperand instanceof ModelPathExpression) {
				firstArgumentAsString = serialize(firstOperand);
			} else {
				firstArgumentAsString = PARENTHESIS_START + serialize(firstOperand) + PARENTHESIS_END;
			}
			if (secondOperand instanceof TimeLiteralExpression) {
				TimeLiteralExpression secondArgument = (TimeLiteralExpression) secondOperand;
				sb.append(BRDateTimeSerializer.serializeTime(firstArgumentAsString, secondArgument, comparisonOperator));
			} else {	// string
				String secondArgumentAsString = null;
				if (secondOperand instanceof ModelPathExpression) {
					secondArgumentAsString = serialize(secondOperand);
				} else {
					secondArgumentAsString = PARENTHESIS_START + serialize(secondOperand) + PARENTHESIS_END;
				}
				sb.append(BRDateTimeSerializer.serializeTime(firstArgumentAsString, secondArgumentAsString, comparisonOperator));
			}
		}
		sb.append(PARENTHESIS_END);
						
		LogUtil.traceExit(this, "serializeDateTimeComparison(ComparisonExpression compExpression)", sb.toString());

		return sb.toString();
	}
	
	/**
	 * Serializes a collection expression into a string.
	 * @param expression Expression
	 * @return expression String
	 * @throws BRSerializationException
	 */
	public String serializeCollection(final Expression expression) throws BRSerializationException {

		LogUtil.traceEntry(this, "serializeCollection(Expression expression)", new String[] {"expression"}, new Object[] {expression});

		String result = null;
		if (expression == null) {
			return "";
		}
		if (expression instanceof ModelPathExpression) {
			result = serializeCollection((ModelPathExpression)expression);
		} else {		
			result = serialize(expression);
		}		
		LogUtil.traceExit(this, "serializeCollection(Expression expression)", result);

		return result;
	}


	/**
	 * Serializes a collection Expression into a String
	 * @param expression ModelPathExpression
	 * @return String
	 * @throws BRSerializationException
	 */
	private String serializeCollection(final ModelPathExpression pathExpression) throws BRSerializationException {
						
		LogUtil.traceEntry(this, "serializeCollection(ModelPathExpression pathExpression)", new String[] {"pathExpression"}, new Object[] {pathExpression});
				
		final StringBuffer sb = new StringBuffer();
		final List steps = pathExpression.getSteps();

		Object step = null;
		ReferenceStep referenceStep = null;
		String serializedName = null;
		
		for (int i=0; i < steps.size(); i++) {
			serializedName = null;
			step = steps.get(i);
			if (step != null && step instanceof ReferenceStep) {
				referenceStep = (ReferenceStep)steps.get(i);
				if (referenceStep.getReferencedObject() instanceof InputObjectPin) {
					InputObjectPin pin = (InputObjectPin)referenceStep.getReferencedObject();
					InputPinSet iPinSet = (InputPinSet)((InputObjectPin)pin).getAction().getInputPinSet().get(0);
					int pinIndex = BrUtil.findInputPinIndex(pin, iPinSet);
					
					TransformationRule pinSetRule = null;					
					if (getInputPinSetRule() == null) {
						if (getProcessInterfaceRule() != null) {
							pinSetRule = (TransformationRule)TransformationUtil.getRuleForSource(getProcessInterfaceRule(), iPinSet, XSDTypeDefinition.class);
						}
						if (pinSetRule == null) {
							pinSetRule = (TransformationRule)TransformationUtil.getRuleForSource(root, iPinSet, XSDTypeDefinition.class);
							if (pinSetRule != null) {
								setInputPinSetRule(pinSetRule);
							}
						} else {
							setInputPinSetRule(pinSetRule);
						}
					} else {
						pinSetRule = getInputPinSetRule();
					}

					if (pinSetRule != null) {	// unwrap the wrapper type
						serializedName = BrUtil.getNameFromPinSetRule(pinSetRule, pinIndex);
					}
			
					if (serializedName == null) {
						serializedName = javaNameConverter.convertName(referenceStep.getReferenceName());						
					}					
				} else if (referenceStep.getReferencedObject() instanceof OutputObjectPin) {
					OutputObjectPin pin = (OutputObjectPin)referenceStep.getReferencedObject();
					OutputPinSet oPinSet = (OutputPinSet)((OutputObjectPin)pin).getAction().getOutputPinSet().get(0);
					int pinIndex = BrUtil.findOutputPinIndex(pin, oPinSet);
						
					TransformationRule pinSetRule = null;

					if (getOutputPinSetRule() == null) {
						if (getProcessInterfaceRule() != null) {
							pinSetRule = (TransformationRule)TransformationUtil.getRuleForSource(getProcessInterfaceRule(), oPinSet, XSDTypeDefinition.class);
						}
						if (pinSetRule == null) {
							pinSetRule = (TransformationRule)TransformationUtil.getRuleForSource(root, oPinSet, XSDTypeDefinition.class);
							if (pinSetRule != null) {
								setOutputPinSetRule(pinSetRule);
							}
						} else {
							setOutputPinSetRule(pinSetRule);
						}
					} else {
						pinSetRule = getOutputPinSetRule();
					}
					
					if (pinSetRule != null) {	// unwrap the wrapper type
						serializedName = BrUtil.getNameFromPinSetRule(pinSetRule, pinIndex);
					}
					if (serializedName == null) {
						serializedName = javaNameConverter.convertName(referenceStep.getReferenceName());						
					}
				// assert: template cannot be a collection
				} else if (referenceStep.getReferencedObject() instanceof TemplateParameter) {
					TemplateParameter templateParameter = (TemplateParameter)referenceStep.getReferencedObject();
					BusinessRuleTemplate businessRuleTemplate = (BusinessRuleTemplate)templateParameter.eContainer();

					TransformationRule brRule = null;
					
					if (getBusinessRuleSetRule() != null) {
						brRule = (TransformationRule)TransformationUtil.getRuleForSource(getBusinessRuleSetRule(), businessRuleTemplate, RuleTemplate.class);
					}
					if (brRule == null) {
						brRule = (TransformationRule)TransformationUtil.getRuleForSource(root, businessRuleTemplate, RuleTemplate.class);
					}
					
					RuleTemplate ruleTemplate = null;
					String propertyName = null;
					if (brRule != null) {
						ruleTemplate = (RuleTemplate)brRule.getTarget().get(0);
					
						// get index of template - retrieve Variable parameter of the same index
						int indexOfTemplateParameter = BrUtil.findIndexOfTemplateParameter(templateParameter, businessRuleTemplate);
						
						if (indexOfTemplateParameter > ruleTemplate.getParameter().size() - 1) {
							throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.INVALID_OPERAND, null, "findIndexOfTemplateParamter");						
						}
						
						Variable variable = (Variable)ruleTemplate.getParameter().get(indexOfTemplateParameter);						
						propertyName = variable.getVarName();
					}

					if (propertyName == null) {
						serializedName = javaNameConverter.convertName(referenceStep.getReferenceName());
					} else {
						serializedName = propertyName;
					}
				} else if (referenceStep.getReferencedObject() instanceof Property) {
					Property property = (Property)referenceStep.getReferencedObject();
					TransformationRule propertyRuleAttribute = (TransformationRule)TransformationUtil.getRuleForSource(root, property, XSDParticle.class);

					XSDParticle xsdPropertyParticle = null;
					if (propertyRuleAttribute != null) {
						xsdPropertyParticle = (XSDParticle)propertyRuleAttribute.getTarget().get(0);
					}

					String propertyName = null;
					if (xsdPropertyParticle != null && xsdPropertyParticle.getContent() instanceof XSDElementDeclaration) {
						propertyName  = ((XSDElementDeclaration)xsdPropertyParticle.getContent()).getName();
					}
					
					if (propertyName == null) {
						serializedName = javaNameConverter.convertName(referenceStep.getReferenceName());
					} else {
						serializedName = propertyName;
					}
				}
			} else {
				throw BrUtil.createException(MessageKeys.INVALID_STEP, null, "serialize");
			}
				
			if (serializedName != null && !serializedName.equals("")) {
				sb.append(serializedName);
				// for simple types exit (skip property)
				if (referenceStep != null) {
					if (referenceStep.getReferencedObject() instanceof ObjectPin) {
						Type objectType = ((ObjectPin)(referenceStep.getReferencedObject())).getType();
						if (BrUtil.isSimpleType(objectType)) {
							break;
						}	//d88902
					} else if (referenceStep.getReferencedObject() instanceof Property &&
							((Property)referenceStep.getReferencedObject()).getType() != null) {
						Type objectType = ((Property)referenceStep.getReferencedObject()).getType();
						if (BrUtil.isSimpleType(objectType)) {
							break;
						}						
					}
				}
				if (i < steps.size() - 1) {
					sb.append(CURRENT_STEP_SEP);
				}
			}
		}
		
		String result = sb.toString();
		
		LogUtil.traceExit(this, "serializeCollection(ModelPathExpression pathExpression)", result);
		
		return result;
	}
	
	/**
	 * Serialize comparison operator.
	 * @param op ComparisonOperator 
	 * @return String
	 */	
	protected static String serializeComparisonOperator(ComparisonOperator op) {
		if (op == null) {
			throw BrUtil.createException(MessageKeys.INVALID_COMPARISION_OP, null, "serialize");		
		}
		
		if (ComparisonOperator.EQUAL_TO_LITERAL.equals(op)) {
			return EQUAL_TO_OP;
		} else if (ComparisonOperator.NOT_EQUAL_TO_LITERAL.equals(op)) {
			return NOT_EQUAL_TO_OP;
		} else if (ComparisonOperator.LESS_THAN_LITERAL.equals(op)) {
			return LESS_THAN_OP;
		} else if (ComparisonOperator.LESS_THAN_OR_EQUAL_TO_LITERAL.equals(op)) {
			return LESS_THAN_OR_EQUAL_TO_OP;
		} else if (ComparisonOperator.GREATER_THAN_LITERAL.equals(op)) {
			return GREATER_THAN_OP;
		} else if (ComparisonOperator.GREATER_THAN_OR_EQUAL_TO_LITERAL.equals(op)) {
			return GREATER_THAN_OR_EQUAL_TO_OP;
		} else if (ComparisonOperator.IS_AFTER_LITERAL.equals(op)) {
			return GREATER_THAN_OP;
		} else if (ComparisonOperator.IS_BEFORE_LITERAL.equals(op)) {
			return LESS_THAN_OP;
		} else {
			throw BrUtil.createException(MessageKeys.INVALID_COMPARISION_OP, null, "serialize");
		}
	}
}
