package com.ibm.btools.te.ilm.heuristics.extservice.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.wst.wsdl.Input;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.Output;
import org.eclipse.wst.wsdl.PortType;

import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.impl.TransformationRuleImpl;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.extservice.ExtservicePackage;
import com.ibm.btools.te.ilm.heuristics.extservice.OperationRule;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.wsdl.MessageRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.ParameterRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.WsdlFactory;
import com.ibm.btools.te.ilm.heuristics.wsdl.util.OriginalWsdlUtil;

/**
 * <!-- begin-user-doc -->
* An implementation of the model object '<em><b>Operation Rule</b></em>'.
* Source: Activity with aspect="ServiceOperation"
* Target: WSDL Operation
* <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class OperationRuleImpl extends TransformationRuleImpl implements OperationRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	private List pinSet = null;
	private Operation operation = null;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected OperationRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return ExtservicePackage.Literals.OPERATION_RULE;
	}

	/*
	 * SOURCE: Activity with aspect="ServiceOperation"
	 * TARGET: WSDL Operation 
	 * Transforms the source a BOM Activity into a WSDL operation. It is responsible for 
	 * creating child Message and ParameterRules.  
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target(){
		/* The source of an OperationRule is an Activity with aspect="ServiceOperation"		
		 * Create childRules as follows:
		 * - For each inputSet and/or output set create a parameter (inputSet -> inputParameter
		 * outputSet -> outputParameter)
		 * The Rule will create as childs ParameterRules and MessageRules.
		 * Execute the child rules and attach the input/output to the operation. 
		 */
		LoggingUtil.traceEntry(this, "transformSource2Target");
		if( isComplete()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return true;
		}
		if( getSource() == null || getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return false;
		}		
		
		if(getSource().get(0) instanceof PinSet){
			PinSet ps = (PinSet)getSource().get(0);
			StructuredActivityNode san = (StructuredActivityNode)(ps.eContainer());			
			PortType portType = OriginalWsdlUtil.getPortType(getContext(),BomUtils.resolveOwningInterface(san));
			if(portType != null){
				Operation originalOperation = OriginalWsdlUtil.getOperation(portType,san.getName());
				if(originalOperation != null){
					operation = originalOperation;
					OriginalWsdlUtil.registerWsdlElement(getContext(),san,operation);
				}
			}
		}		
		getTarget().add(operation);			
		// setup child parameter rules. For each input/output set in the source, a ParameterRule and a MessageRule will be created
		setupChildRules();
		boolean ret = executeChildRules();
		processChildTargets();
		setComplete(ret);
		executeHandlers();
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();	
	}
	
	/**
	 * Create child ParameterRules for each source element (BOM Input and/or OutputPinSet)
	 */
	private void createParameterRules(){
		// create a parameter for the each pin in the input/output Set
		ParameterRule parameter; 		
		pinSet = getSource();
		for (int i=0; i< pinSet.size(); i++){
			parameter = WsdlFactory.eINSTANCE.createParameterRule();
			parameter.getSource().add(pinSet.get(i));
			// Add another source to indicate this is for callback such that the parameter rule will create an input for an OutputPinSet
			// May need to think of a better way ....
			if ((pinSet.get(0) instanceof OutputPinSet) && (getSource().size() == 1)) {
				parameter.getSource().add(((OutputPinSet)(pinSet.get(0))).getAction());
			}
			getChildRules().add(parameter);
		}
		
	}	
	
	/**
	 * Creates a MessageRule for each Input/OutputSet (Input/OutputParameterSet). If a WSDL Message was already 
	 * created this source, the message should be reused.
	 */	
	private void createMessageRules(){
		MessageRule rule = null;	
		pinSet = getSource();
		for (int i=0; i< pinSet.size(); i++){
			//AZ 20070213 skip create empty message for the PinSet with no object pins
			//if(pinSet.get(i) instanceof OutputPinSet && BomUtils.getObjectPinsForSet((PinSet)pinSet.get(i)).size() > 0){
			if(pinSet.get(i) instanceof PinSet && BomUtils.getObjectPinsForSet((PinSet)pinSet.get(i)).size() > 0){
				//check first if the message was already created.			
				rule = (MessageRule)TransformationUtil.getRuleForSource(getRoot(),(EObject)pinSet.get(i), Message.class);
				// could not find the message, create a message rule for the input/outputSet source
				if(rule == null){
					rule = WsdlFactory.eINSTANCE.createMessageRule();
					rule.getSource().add(pinSet.get(i));
					getChildRules().add(rule);
				}
			}
		}
	}
	/**
	 * Creates child rules for that will create WSDL messages and parameters.
	 */
	private void setupChildRules(){
		// NOTE: to be called in this order, as a WSDL Parameter will reference a Message.
		createMessageRules();
		createFaultMessageRules();
		createParameterRules();	
		createFaultParameterRules();
	} 
	
	/**
	 * Create message rules for the exceptional outputs 
	 *
	 */
	private void createFaultMessageRules(){
		// create a message for each fault pin in the output Set
		MessageRule faultMessageRule = null;	
		pinSet = getSource();
		for (int i=0; i< pinSet.size(); i++){
			if(pinSet.get(i) instanceof InputPinSet){
				InputPinSet ips = (InputPinSet)pinSet.get(i);
				EList opsList = ips.getOutputPinSet();
				if(!opsList.isEmpty()){
					Iterator it = opsList.iterator();
					while(it.hasNext()){
						OutputPinSet ops = (OutputPinSet)it.next();
						if(pinSet.contains(ops))
							continue;				
						//check first if the message was already created.
						faultMessageRule = (MessageRule)TransformationUtil.getRuleForSource(getRoot(),(EObject)ops, Message.class);
						// could not find the message, create a message rule for the outputSet source
						if(faultMessageRule == null){
							faultMessageRule = WsdlFactory.eINSTANCE.createMessageRule();				
							faultMessageRule.getSource().add(ops);
							getChildRules().add(faultMessageRule);
						}
					}
				}
			}
			
		}
	}
	
	/**
	 * Create parameter rules for the exceptional outputs 
	 *
	 */
	private void createFaultParameterRules(){
		//	 create a parameter for each fault pin in the output Set
		ParameterRule faultParameterRule; 		
		pinSet = getSource();
		for (int i=0; i< pinSet.size(); i++){
			if(pinSet.get(i) instanceof InputPinSet){
				InputPinSet ips = (InputPinSet)pinSet.get(i);
				EList opsList = ips.getOutputPinSet();
				if(!opsList.isEmpty()){
					Iterator it = opsList.iterator();
					while(it.hasNext()){
						OutputPinSet ops = (OutputPinSet)it.next();
						if(pinSet.contains(ops))
							continue;				
						faultParameterRule = WsdlFactory.eINSTANCE.createParameterRule();			
						faultParameterRule.getSource().add(ops);		
						getChildRules().add(faultParameterRule);
					}
				}
			}			
		}		
	}
	/**
	 * Process child targets. This rule will only process WSDL Input and/or Output. 
	 */
	protected void processChildTargets(){
		List childRules = getChildRules();
		EObject childTarget;
		List childTargets;
		Iterator childRulesIterator = childRules.iterator();
		while (childRulesIterator.hasNext()){
			TransformationRule childRule = (TransformationRule)childRulesIterator.next();
			childTargets = childRule.getTarget();
			for (int i=0; i<childTargets.size(); i++){
				childTarget = (EObject) childTargets.get(i);
				if (childTarget instanceof Output){					
					if (operation.getEOutput()!=null) {
						if(operation.getEOutput().getName() != null &&
								operation.getEOutput().getName().equals(((Output)childTarget).getName())){
							continue;
						}
					}				
				}else if (childTarget instanceof Input){					
					if (operation.getEInput()!=null) {
						if(operation.getEInput().getName() != null && 
								operation.getEInput().getName().equals(((Input)childTarget).getName())){
							continue;
						}
					}
					
				}
			}	
		}
	}
} //OperationRuleImpl
