/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.BlockRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.ProcessRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.fdl.model.Block;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.Sink;
import com.ibm.btools.fdl.model.Source;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Block Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class BlockRuleImpl extends ContainmentRuleImpl implements BlockRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected BlockRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.BLOCK_RULE;
	}

	/**
	 * Source: BOM InputPinSet of nested StructuredActivityNode
	 * Target: FDL Block
	 * 
	 * Maps BOM nested process to an FDL Block.
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");
	
		InputPinSet inputPinSet = (InputPinSet)(getSource().get(0));
		OutputPinSet outputPinSet = (OutputPinSet)inputPinSet.getOutputPinSet().get(0);
		
		if(!isFirstTimeExec()){
			//will connect additional connectors with Block 
			reExecute(inputPinSet);
			LoggingUtil.traceExit(this, "transformSource2Target","re-execute");
			return isComplete();
		}
		
		setFirstTimeExec(false);		
		
		//create Block
		Block block = getFdlModelFactoryInstance().createBlock();
		block.setName(getNameRegistry().getActivityName(inputPinSet.getAction()));
		getTarget().add(block);	
						
		//cache the block for re-execution of this rule
		setTargetActivity(block);		
		
		//create input and output containers for block
		DataStructure inputDs = createDataStructure(inputPinSet);
		DataStructure outputDs = createDataStructure(outputPinSet);
		block.setInputDataStructure(inputDs);
		block.setOutputDataStructure(outputDs);
		
		//map input value pin if any to block input container initialization
		List initialList = mapInputValuePin(inputPinSet, inputDs);
		if((initialList != null) && (!initialList.isEmpty())){
			block.getInputContainerInitials().addAll(initialList);
		}
		
		//connect incoming connectors of the InputPinSet to the Block 
		setTargetOfConnectors(inputPinSet, block);	
		
		//create source and sink
		Source src = createSource(inputPinSet);
		Sink sink = createSink(outputPinSet);	
			
		//create connectors for outgoing edges of the InputPinSet
		if(src != null){
			block.getSource().add(src);
			connectInputPinSet(inputPinSet, src);		
		}
		//create process rule to map internal of the block
		ProcessRule pr = FdlFactory.eINSTANCE.createProcessRule();
		getChildRules().add(pr);		
		pr.getSource().add(inputPinSet);
		pr.transformSource2Target();
		
		//connector incoming connectors of the OutputPinSet with the Sink.
		
		if(sink != null){
			block.getSink().add(sink);	
			connectOutputPinSet(outputPinSet, sink);
		}	
		
		//process all child rules targets so far, so we exclude any rules may be created later
	    processChildTargets(block);
		
		TransformationRule branchRule =  createBranchRuleIfAny(inputPinSet);		
		if(branchRule != null){
			getTarget().addAll(branchRule.getTarget());
		}
				
		//connect outgoing connectors of the OutputPinSet to the Block
		setSourceOfConnectors(outputPinSet, block);	   

		// Ray: added support for descriptions and documentations		
		Action action = inputPinSet.getAction();
		String desc = FdlUtil.transformDescription(action);
		if (desc!=null) {
			block.setDescription(getNameRegistry().getDescription(desc));
		}
		String doc = FdlUtil.transformDocumentation(action);
		if (doc!=null) {
			block.setDocumentation(getNameRegistry().getDocumentation(doc));
		}
		
		setComplete(true);
		
		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}



	
		
	

} //BlockRuleImpl
