/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.blm.ie.imprt.rule.util.PredefinedTypeUtil;
import com.ibm.btools.bom.command.compound.PredefTypesConst;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.resources.BulkResourceRequirement;
import com.ibm.btools.bom.model.resources.IndividualResource;
import com.ibm.btools.bom.model.resources.IndividualResourceRequirement;
import com.ibm.btools.bom.model.resources.IndividualResourceType;
import com.ibm.btools.bom.model.resources.RequiredRole;
import com.ibm.btools.bom.model.resources.ResourceRequirement;
import com.ibm.btools.bom.model.resources.Role;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.CallBehaviorActionRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.PersonRule;
import com.ibm.btools.te.ilm.heuristics.fdl.RoleRule;
import com.ibm.btools.te.ilm.heuristics.fdl.StaffAssignmentRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.heuristics.helper.StaffUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.fdl.model.BasicActivity;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.DoneBy;
import com.ibm.btools.fdl.model.DoneByEnum;
import com.ibm.btools.fdl.model.ProcessActivity;
import com.ibm.btools.fdl.model.StartExitOption;
import com.ibm.btools.util.exception.BTRuntimeException;

/**
 * <!-- begin-user-doc -->
 *
 * Source: BOM InputPinSet of CallBehaviorAction
 * Target: FDL ProcessActivity
 * 
 * Maps one InputPinSet of BOM CallBehaviorAction to one FDL ProcessActivity.
 * Note: the Process that CallBehaviorAction referenced will not be created.
 *
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class CallBehaviorActionRuleImpl extends ConnectableRuleImpl implements CallBehaviorActionRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected CallBehaviorActionRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.CALL_BEHAVIOR_ACTION_RULE;
	}

	/**
	 * Source: BOM InputPinSet of CallBehaviorAction
	 * Target: FDL ProcessActivity
	 * 
	 * Maps one InputPinSet of BOM CallBehaviorAction to one FDL ProcessActivity.
	 */
	public boolean transformSource2Target()
	{
		LoggingUtil.traceEntry(this, "transformSource2Target");

		InputPinSet inputPinSet = (InputPinSet)getSource().get(0);
		if(!isFirstTimeExec()){
			reExecute(inputPinSet);
			LoggingUtil.traceExit(this, "transformSource2Target","re-execute");
			return true;
		}
		
		setFirstTimeExec(false);
		
		ProcessActivity procAct = getFdlModelFactoryInstance().createProcessActivity();
		CallBehaviorAction cba = (CallBehaviorAction)inputPinSet.getAction();
		
		String procActName = getNameRegistry().getActivityName(inputPinSet);
		procAct.setName(procActName);
//		String dsc = FdlUtil.getFullName(inputPinSet);
//		procAct.setDescription(getNameRegistry().getDescription(dsc));
//		procAct.setDocumentation(getNameRegistry().getDocumentation(cba.getEffect()));
	  
		List correlatedOutputPinSets = inputPinSet.getOutputPinSet();
		if (correlatedOutputPinSets.size()>1)
		{
			String params[]={inputPinSet.getName(),cba.getName()};
			LoggingUtil.logError(MessageKeys.CBA_MULTIPLE_CORRELATION_NOT_SUPPORTED,params,null);
			//BTRuntimeException e = new BTRuntimeException(
			//	TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.CBA_MULTIPLE_CORRELATION_NOT_SUPPORTED));							
			//throw e;
		}
		
		if (cba.getIsSynchronous() != null && cba.getIsSynchronous().booleanValue() == false){
			String params[]={cba.getName()};			
			LoggingUtil.logWarning(MessageKeys.ASYNCH_COA_NOT_SUPPORTED);		
		}
		
		OutputPinSet outputPinSet = (OutputPinSet)correlatedOutputPinSets.get(0);
		// Set input/output data structure
		DataStructure inputDs = createDataStructure(inputPinSet);
		procAct.setInputDataStructure(inputDs);
		procAct.setOutputDataStructure(createDataStructure(outputPinSet));
		
		//map any input value pin in the input pin set to activity input container initialization
		List initialList = mapInputValuePin(inputPinSet, inputDs);
		if((initialList != null) && (!initialList.isEmpty())){
			procAct.getInputContainerInitials().addAll(initialList);
		}
		
		Integer index = null;
		if(cba.getInputPinSet().size() > 1){
			index = new Integer(cba.getInputPinSet().indexOf(inputPinSet) + 1);
		}
		String processName = getNameRegistry().getProcessName((Activity)cba.getBehavior(), index);
		procAct.setProcessName(processName);
		
		getTarget().add(procAct);
		//cache it for reExecute
		setTargetActivity(procAct);	
	
		// set connector's target for incoming connectors
		setTargetOfConnectors(inputPinSet, procAct);
		
		createBranchRuleIfAny(inputPinSet);
		
		// Set connector's source for outgoing connectors
		setSourceOfConnectors(outputPinSet, procAct);
		
		Action action = inputPinSet.getAction();
		
		//Check to see if any Resource Requirements
		createResource(action, procAct);	
		// Ray: added support for descriptions and documentations		
		//Action action = inputPinSet.getAction();
		String desc = FdlUtil.transformDescription(((Activity)cba.getBehavior()).getImplementation());
		if (desc!=null) {
			procAct.setDescription(getNameRegistry().getDescription(desc));
		}
		String doc = FdlUtil.transformDocumentation(action);
		if (doc!=null) {
			procAct.setDocumentation(getNameRegistry().getDocumentation(doc));
		}		
		//Check to see if any organization units
		//createOrganization(action);		
		
		propagateChildTargets();
		
		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");		
		return true;
	}
	
/*
	private void createOrganization(Action action){
		if (action.getResponsibleOrganization()!=null && !action.getResponsibleOrganization().isEmpty()){
			List responsibleOrganizations = action.getResponsibleOrganization();
			Iterator iterator = responsibleOrganizations.iterator();
			while(iterator.hasNext()){		
				OrganizationRule orgRule = ModelFactory.eINSTANCE.createOrganizationRule();
				OrganizationUnit orgUnit = (OrganizationUnit)iterator.next();
				orgRule.getSource().add(orgUnit);
				getRoot().getChildRules().add(orgRule);
				orgRule.transformSource2Target();				
			}
		}		
	}
*/	


	/**
	 * Creates a Resource to represent the a Person or a Role or a Query
	 * @param action - a BOM action
	 * @param ProcessActivity - a fdl process activity
	 * @return
	 */
	private void createResource(Action action, ProcessActivity activity)
	{
		//FDL Roles
		ResourceRequirement candidateRequirement = null;
		if(StaffUtil.isStaffRequired(action)){
			List resourceRequirements = action.getResourceRequirement();			
			Iterator iterator = resourceRequirements.iterator();
			while(iterator.hasNext()){
				ResourceRequirement thisRequirement = (ResourceRequirement)iterator.next();
				//BulkResourceRequirement are not mapped to Staff queries.
				if(!(thisRequirement instanceof BulkResourceRequirement)){
					if(thisRequirement instanceof IndividualResourceRequirement){
						IndividualResourceRequirement individual = (IndividualResourceRequirement)thisRequirement;
						//IndividualResourcesRequirement with assigned IndividualResources of predefinedType si map to the Users by userID  
						if (individual.getIndividualResource()!= null && StaffUtil.isPredefinedStaff(individual.getIndividualResource().getClassifier())){
							//Create FDL Person
							if (individual.getIndividualResource()!=null && !findPerson(individual.getIndividualResource())){
								PersonRule personRule = FdlFactory.eINSTANCE.createPersonRule();
								personRule.getSource().add(individual.getIndividualResource());
								getRoot().getChildRules().add(personRule);
								personRule.transformSource2Target();	
							}																		
						}
					//Only RequiredRoles with predefined specified ResourceType are mapped to Staff Queries	
					}else if(thisRequirement instanceof RequiredRole){
						if (((RequiredRole)thisRequirement).getResourceType()!=null && StaffUtil.isPredefinedStaff(((RequiredRole)thisRequirement).getResourceType())){							
							if (((RequiredRole)thisRequirement).getRole()!=null && !findRole(((RequiredRole)thisRequirement).getRole())){
								RoleRule roleRule = FdlFactory.eINSTANCE.createRoleRule();
								roleRule.getSource().add(((RequiredRole)thisRequirement).getRole());
								getRoot().getChildRules().add(roleRule);
								roleRule.transformSource2Target();			
							}	
						}		
					}					
					//if(thisRequirement.getOwnedConstraint()!= null && !thisRequirement.getOwnedConstraint().isEmpty()){
						//Call StaffAssignmentRule to map Queries
						StaffAssignmentRule staffRule = FdlFactory.eINSTANCE.createStaffAssignmentRule();
						staffRule.getSource().add(thisRequirement);					
						staffRule.transformSource2Target();	
						//get target and save the doneBy object to activity
						if (!staffRule.getTarget().isEmpty() && activity instanceof BasicActivity){					
							((BasicActivity)activity).getActivityExtensionSetting().getDoneBy().add((DoneBy)staffRule.getTarget().get(0));
							((BasicActivity)activity).getActivityExtensionSetting().setStart(StartExitOption.MANUAL_LITERAL);					
						}	
					//}
					
				}
			}				 
		}
	}	
	
	private boolean findPerson(IndividualResource res){
		TransformationRule rule = TransformationUtil.getRuleForSource(getRoot(), PersonRule.class, res);
		if (rule==null)
			return false;
		else
			return true;
	}
	
	private boolean findRole(Role role){
		TransformationRule rule = TransformationUtil.getRuleForSource(getRoot(), RoleRule.class, role);
		if (rule==null)
			return false;
		else
			return true;
		
	}

} //CallBehaviorActionRuleImpl
