/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.processes.actions.Fork;
import com.ibm.btools.bom.model.processes.activities.ActivityEdge;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ConnectorRule;
import com.ibm.btools.te.ilm.heuristics.fdl.DataMappingRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.fdl.model.Activity;
import com.ibm.btools.fdl.model.Block;
import com.ibm.btools.fdl.model.ControlFlow;
import com.ibm.btools.fdl.model.DataFlow;
import com.ibm.btools.fdl.model.DataMapping;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.MemberDeclaration;
import com.ibm.btools.fdl.model.Process;
import com.ibm.btools.fdl.model.Sink;
import com.ibm.btools.fdl.model.Source;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlConstants;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Connector Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public abstract class ConnectorRuleImpl extends AbstractFdlProcDefRuleImpl implements ConnectorRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	protected DataFlow dataFlow=null;
	protected ControlFlow ctrlFlow=null;
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ConnectorRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.CONNECTOR_RULE;
	}

	protected void updateDataMapping(ActivityEdge edge,PinSet srcPinSet,PinSet tarPinSet)
	{
		// Check if both the To and From FDL Activity or Source/Sink of the DataFlow has been
		// set.  If 'yes', create the DataMapping
		if (dataFlow != null)
		{
			DataStructure fromContainer=null;
			DataStructure toContainer=null;

			Activity toAct = dataFlow.getToActivity();
			Activity fromAct = dataFlow.getFromActivity();
			if (fromAct == null) {
				Source source = dataFlow.getSource();
				if (source != null) {
					if (source.eContainer() instanceof Process) {
						fromContainer = ((Process)source.eContainer()).getInputDataStructure();
					} else {
						fromContainer = ((Block)source.eContainer()).getInputDataStructure();						
					}
				}
			} else {
				fromContainer = fromAct.getOutputDataStructure();
			}
			if (toAct == null) {
				Sink sink = dataFlow.getSink();
				if (sink != null) {
					if (sink.eContainer() instanceof Process) {
						toContainer = ((Process)sink.eContainer()).getOutputDataStructure();
					} else {
						toContainer = ((Block)sink.eContainer()).getOutputDataStructure();						
					}
				}
			} else {
				toContainer = toAct.getInputDataStructure();
			}
//			if (toAct!=null && fromAct!=null)
			if (fromContainer!=null && toContainer!=null)
			{
				// Create DataMapping
				List dataMaps = dataFlow.getDataMappings();
				// If the From and To Container are the same, then we
				// only need a single _STRUCT to _STRUCT mapping.  If a map
				// is found already exist, that means the _STRUCT to _STRUCT mapping
				// has already been created and there is no need to recreate it
				if ((fromContainer == toContainer) && (dataMaps.size()==0))
				{
					DataMapping dMap = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createDataMapping();
					dMap.setFromDataStructureMemberName(FdlConstants.TOP_STRUCTURE_NAME);
					dMap.setToDataStructureMemberName(FdlConstants.TOP_STRUCTURE_NAME);
					dataMaps.add(dMap);
				} else if (fromContainer != toContainer){
					ObjectPin srcPin = (ObjectPin)edge.getSource();
					ObjectPin tarPin = (ObjectPin)edge.getTarget();
					dataMaps.addAll(createDataMappings(srcPin,tarPin,srcPinSet,tarPinSet,fromContainer,toContainer));										
				}		
			}
		}			
	}
	
	private Collection createDataMappings(ObjectPin srcPin, ObjectPin tarPin, PinSet srcPinSet, PinSet tarPinSet, DataStructure srcDs, DataStructure tarDs)
	{
		/*
		ContainerRule srcCr = (ContainerRule)TransformationUtil.getRuleForSource(getRoot(),ContainerRule.class,srcPinSet);
		ContainerRule tarCr = (ContainerRule)TransformationUtil.getRuleForSource(getRoot(),ContainerRule.class,tarPinSet);
		MemberDeclRule srcMdr = (MemberDeclRule)TransformationUtil.getRuleForSource(srcCr,MemberDeclRule.class,srcPin);
		MemberDeclRule tarMdr = (MemberDeclRule)TransformationUtil.getRuleForSource(tarCr,MemberDeclRule.class,tarPin);
		MemberDeclaration srcMd = (MemberDeclaration)srcMdr.getTarget().get(0);
		MemberDeclaration tarMd = (MemberDeclaration)tarMdr.getTarget().get(0);
		*/
		
//		DataStructure srcDs = (DataStructure)srcCr.getTarget().get(0);
//		DataStructure tarDs = (DataStructure)tarCr.getTarget().get(0);
		DataMappingRule dmr = FdlFactory.eINSTANCE.createDataMappingRule();
		if (FdlUtil.bOptimizeSingleObjPinContainer) {
			int numObjPins;
			if (srcPinSet instanceof InputPinSet) {
				numObjPins = ((InputPinSet)srcPinSet).getInputObjectPin().size();
			} else {
				// We need a special case for Fork because whether the DS generated is optimized
				// is always determined by number of object pin on the inputPinSet not the outputPinSet of the fork
				if (srcPinSet.eContainer() instanceof Fork) {
					numObjPins = ((InputPinSet)((OutputPinSet)srcPinSet).getInputPinSet().get(0)).getInputObjectPin().size();
				} else {
					numObjPins = ((OutputPinSet)srcPinSet).getOutputObjectPin().size();
				}
			}
			if (numObjPins==1 && srcPin.getType() == tarPin.getType()) {
				dmr.getSource().add(srcDs);
			} else {
				dmr.getSource().add(getMemberDecl(srcDs,srcPin,srcPinSet));
			}
			if (tarPinSet instanceof InputPinSet) {
				numObjPins = ((InputPinSet)tarPinSet).getInputObjectPin().size();
			} else {
				numObjPins = ((OutputPinSet)tarPinSet).getOutputObjectPin().size();
			}
			if (numObjPins==1 && srcPin.getType() == tarPin.getType()) {
				dmr.getSource().add(tarDs);
			} else {
				dmr.getSource().add(getMemberDecl(tarDs,tarPin,tarPinSet));
			}
		} else {
			MemberDeclaration srcMd = getMemberDecl(srcDs,srcPin,srcPinSet);
			MemberDeclaration tarMd = getMemberDecl(tarDs,tarPin,tarPinSet);
		
			dmr.getSource().add(srcMd);
			dmr.getSource().add(tarMd);
		}
		getChildRules().add(dmr);
		dmr.transformSource2Target();
		return dmr.getTarget();
	}
	
	private MemberDeclaration getMemberDecl(DataStructure ds, ObjectPin pin, PinSet pinset)
	{
		MemberDeclaration md;
		List objectPins;
		if (pinset instanceof InputPinSet)
		{
			objectPins = ((InputPinSet)pinset).getInputObjectPin();
		} else {
			objectPins = ((OutputPinSet)pinset).getOutputObjectPin();
		}
		int pinIndex = objectPins.indexOf(pin);
		int memDeclSize = ds.getMemberDeclarations().size();
		// The following mod operator is applied for the case of Fork's outputset or Join inputset
		if (pinIndex >= memDeclSize) {
			pinIndex = pinIndex % memDeclSize;
		}
		return (MemberDeclaration)ds.getMemberDeclarations().get(pinIndex);
	}
//	private Collection createDataMappings(ObjectPin srcPin, ObjectPin tarPin, PinSet srcPinSet, PinSet tarPinSet)
//	{
//		ContainerRule srcCr = (ContainerRule)TransformationUtil.getRuleForSource(getRoot(),ContainerRule.class,srcPinSet);
//		ContainerRule tarCr = (ContainerRule)TransformationUtil.getRuleForSource(getRoot(),ContainerRule.class,tarPinSet);
//		MemberDeclRule srcMdr = (MemberDeclRule)TransformationUtil.getRuleForSource(srcCr,MemberDeclRule.class,srcPin);
//		MemberDeclRule tarMdr = (MemberDeclRule)TransformationUtil.getRuleForSource(tarCr,MemberDeclRule.class,tarPin);
//		MemberDeclaration srcMd = (MemberDeclaration)srcMdr.getTarget().get(0);
//		MemberDeclaration tarMd = (MemberDeclaration)tarMdr.getTarget().get(0);
//
//		ArrayList mappings = new ArrayList();
//		if (srcMd.getType() == tarMd.getType())
//		{
//			Integer srcMdNum = srcMd.getArraySize();
//			Integer tarMdNum = tarMd.getArraySize();
//			// if both Source and Target arraysizes is null (that means they
//			// are not arrays) or if the arraysizes are the same, then
//			// we only need to create a single data mapping.  Otherwise we
//			// have to create multiple mappings for the array
//			if ((srcMdNum==null && tarMdNum==null) ||
//				(srcMdNum.equals(tarMdNum)))
//			{
//				mappings.add(createDataMapping(srcMd.getMemberName(),tarMd.getMemberName()));				
//			} else {
//				String indexStr;
//				for (int i=0; i<srcMdNum.intValue();i++)
//				{
//					indexStr="("+i+")";
//					mappings.add(createDataMapping(srcMd.getMemberName()+indexStr,tarMd.getMemberName()+indexStr));									
//				}
//			}
//		} else {
//			// this is the case where the source Pin type is a subClass of the target Pin type
//			
//			// Need to get the DataStructureRule that transformed the type of the source Pin
//			// to FDL DataStructure so that we can later on use getRuleForSource to retrieve 
//			// the MemberDeclaration for the flatten super class
//			com.ibm.btools.bom.model.artifacts.Class srcClass =
//				(com.ibm.btools.bom.model.artifacts.Class)srcPin.getType();
//			DataStructureRule srcPinDsr = (DataStructureRule)
//				TransformationUtil.getRuleForSource(getRoot(),DataStructureRule.class,srcClass);
//
//			// Need to get the DataStructureRule that transformed the type of the target Pin
//			// to FDL DataStructure so that we can later on use getRuleForSource to retrieve 
//			// the MemberDeclaration for the super class		
//			com.ibm.btools.bom.model.artifacts.Class tarClass =
//				(com.ibm.btools.bom.model.artifacts.Class)tarPin.getType();
//			DataStructureRule tarPinDsr = (DataStructureRule)
//				TransformationUtil.getRuleForSource(getRoot(),DataStructureRule.class,tarClass);
//				
//			mappings.addAll(createSuperClassMemberMappings(tarClass,srcPinDsr,tarPinDsr));
//		}
//		return mappings;
//	}

//	private Collection createDataMappings(MemberDeclaration srcMd, MemberDeclaration tarMd)
//	{
//
//		ArrayList mappings = new ArrayList();
//		DataType srcMdType = srcMd.getType();
//		DataType tarMdType = tarMd.getType();
//		if (srcMdType == tarMdType)
//		{
//			Integer srcMdNum = srcMd.getArraySize();
//			Integer tarMdNum = tarMd.getArraySize();
//			// if both Source and Target arraysizes is null (that means they
//			// are not arrays) or if the arraysizes are the same, then
//			// we only need to create a single data mapping.  Otherwise we
//			// have to create multiple mappings for the array
//			if ((srcMdNum==null && tarMdNum==null) ||
//				(srcMdNum.equals(tarMdNum)))
//			{
//				mappings.add(createDataMapping(srcMd.getMemberName(),tarMd.getMemberName()));				
//			} else {
//				String indexStr;
//				for (int i=0; i<srcMdNum.intValue();i++)
//				{
//					indexStr="("+i+")";
//					mappings.add(createDataMapping(srcMd.getMemberName()+indexStr,tarMd.getMemberName()+indexStr));									
//				}
//			}
//		} else {
//			// this is the case where the source Pin type is a subClass of the target Pin type
//			
//			// Need to get the DataStructureRule that transformed the type of the source Pin
//			// to FDL DataStructure so that we can later on use getRuleForSource to retrieve 
//			// the MemberDeclaration for the flatten super class
//			DataStructureRule srcPinDsr = (DataStructureRule)
//				TransformationUtil.getRuleForTarget(getRoot(),DataStructureRule.class,srcMdType);
//				
//			// Need to get the DataStructureRule that transformed the type of the target Pin
//			// to FDL DataStructure so that we can later on use getRuleForSource to retrieve 
//			// the MemberDeclaration for the super class		
//			DataStructureRule tarPinDsr = (DataStructureRule)
//				TransformationUtil.getRuleForTarget(getRoot(),DataStructureRule.class,tarMdType);
//				
//			mappings.addAll(createSuperClassMemberMappings(
//				(com.ibm.btools.bom.model.artifacts.Class)tarPinDsr.getSource().get(0),srcPinDsr,tarPinDsr));
//		}
//		return mappings;
//	}
//
//	
//	private Collection createSuperClassMemberMappings(
//		com.ibm.btools.bom.model.artifacts.Class superClass,
//		DataStructureRule srcPinDsr,
//		DataStructureRule tarPinDsr
//		)
//	{
//		ArrayList mappings = new ArrayList();
//		List attributes = superClass.getOwnedAttribute();
//		Iterator it = attributes.iterator();
//		Property prop;
//		MemberDeclaration srcMd, tarMd;
//		while (it.hasNext())
//		{
//			prop = (Property)it.next();
//			// get the source member declaration that represents the flatten super class property/attribute
//			srcMd = (MemberDeclaration)
//					((MemberDeclRule)TransformationUtil.getRuleForSource(srcPinDsr,MemberDeclRule.class,prop)).getTarget().get(0);
//			// get the target member declaration that represents the super class property/attribute
//			tarMd =	(MemberDeclaration)
//					((MemberDeclRule)TransformationUtil.getRuleForSource(tarPinDsr,MemberDeclRule.class,prop)).getTarget().get(0);
//			mappings.add(createDataMapping(srcMd.getMemberName(),tarMd.getMemberName()));
//		}
//		// Recursively create mappings for superclasses
//		List superClasses = superClass.getSuperClassifier();
//		it = superClasses.iterator();
//		while (it.hasNext())
//		{
//			mappings.addAll(
//				createSuperClassMemberMappings(
//					(com.ibm.btools.bom.model.artifacts.Class)it.next(),
//					srcPinDsr,
//					tarPinDsr
//					)
//				);
//		}
//		return mappings;
//	}
//	
//	private DataMapping createDataMapping(String fromName, String toName)
//	{
//		DataMapping dMap = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createDataMapping();
//		dMap.setFromDataStructureMemberName(fromName);
//		dMap.setToDataStructureMemberName(toName);
//		return dMap;		
//		
//	}
} //ConnectorRuleImpl
