/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ControlActionRule;
import com.ibm.btools.te.ilm.heuristics.fdl.DataMappingRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.ProgramRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlConstants;
import com.ibm.btools.fdl.model.Activity;
import com.ibm.btools.fdl.model.ControlFlow;
import com.ibm.btools.fdl.model.DataFlow;
import com.ibm.btools.fdl.model.DataMapping;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.MemberDeclaration;
import com.ibm.btools.fdl.model.ProgramActivity;
import com.ibm.btools.fdl.model.SynchronizationType;


/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Control Action Rule</b></em>'.
 * Superclass of all ControlAction rules that provide a set of common methods in support of
 * transformation of BOM ControlAction to FDL equivalent representation
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public abstract class ControlActionRuleImpl extends ConnectableRuleImpl implements ControlActionRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private HashMap joinIndices=null;
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ControlActionRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.CONTROL_ACTION_RULE;
	}

	/**
	 * Creates an FDL DataDefault connector
	 * @param act
	 * @return
	 */
	protected DataFlow createDataDefaultConnector(Activity act)
	{
		DataFlow dataFlow = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createDataFlow();
		dataFlow.setDefaultActivity(act);
		DataMapping dataMapping = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createDataMapping();
		dataMapping.setFromDataStructureMemberName(FdlConstants.TOP_STRUCTURE_NAME);
		dataMapping.setToDataStructureMemberName(FdlConstants.TOP_STRUCTURE_NAME);
		dataFlow.getDataMappings().add(dataMapping);
		return dataFlow;
	}
	
	/**
	 * Creates am FDL noop program activity
	 * @return
	 */
	protected ProgramActivity createNoopProgramActivity()
	{
		ProgramActivity nopa = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createProgramActivity();
		nopa.setProgramName(FdlConstants.NOOP_PROGRAM_NAME);
		nopa.setSynchronization(SynchronizationType.CHAINED_LITERAL);
		//create noop program
		ProgramRule pr = FdlFactory.eINSTANCE.createProgramRule();
		pr.getSource().add(nopa);
		getRoot().getChildRules().add(pr);
		pr.transformSource2Target();
		return nopa;
	}
	
	/**
	 * Creates an 'dummy' noop activity for the case when the inputpinset and outputpinset of
	 * a control action are not identical and therefore not be able to share the same input/ouput
	 * DataStructure.  If a noop activity is creates, re-wiring the flow connector will be handled
	 * as well
	 * @param srcAct
	 * @param tarAct
	 * @param srcConRule
	 * @param tarConRule
	 * @param accumulateCopy
	 * @return
	 */
	protected ProgramActivity createCompatibleContainerActivity(
		Activity srcAct, Activity tarAct, ContainerRule srcConRule, ContainerRule tarConRule, boolean accumulateCopy)
	{
		ProgramActivity noopAct = createNoopProgramActivity();
		DataStructure tarCon = (DataStructure)tarConRule.getTarget().get(0);
		DataStructure srcCon = (DataStructure)srcConRule.getTarget().get(0);
		noopAct.setInputDataStructure(tarCon);
		noopAct.setOutputDataStructure(tarCon);
		getTarget().add(createDataDefaultConnector(noopAct));
		ControlFlow ctrlFlow = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createControlFlow();
		DataFlow dataFlow = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createDataFlow();
		getTarget().add(ctrlFlow);
		getTarget().add(dataFlow);
		PinSet pinSet = (PinSet)tarConRule.getSource().get(0);
		if (srcAct == null) {
			noopAct.setName(createNoopActivityName(tarAct,pinSet));
			// This is the case for Merge and Join
			// link between the noopAct and tarAct
			ctrlFlow.setFromActivity(noopAct);
			ctrlFlow.setToActivity(tarAct);
			dataFlow.setFromActivity(noopAct);
			dataFlow.setToActivity(tarAct);
			// Create map from tarCon to srcCon
			List tarMembers = tarCon.getMemberDeclarations();
			List srcMembers = srcCon.getMemberDeclarations();
			int size = tarMembers.size();
			int srcSize = srcMembers.size();
			MemberDeclaration srcMd;
			MemberDeclaration tarMd;
			Collection dataMappings;
			for (int i=0; i<size; i++) {
				if (accumulateCopy)	{
					tarMd = (MemberDeclaration)tarMembers.get(i);
					srcMd = (MemberDeclaration)srcMembers.get(i%srcSize);
					dataMappings = createDataMappings(tarMd,srcMd);
					
					// Case of Join, we need to map the pin in 
					// tarCon to the next available slot of the 
					// pin in srcCon (for Merge we will always 
					// overwrite).  This can be done by checking 
					// which is the next slot in the target container 
					// field (array) that represents the pin doesnt have a map yet.
					if (srcMd.getArraySize() != null) {
						if (joinIndices == null) {
							joinIndices = new HashMap();
						}
						Integer indexInt = (Integer)joinIndices.get(srcMd);
						int index;
						if (indexInt == null) {
							index = 0;
						} else {
							index = indexInt.intValue();
						}
						// Update the dataMappings array index
						Iterator it = dataMappings.iterator();
						DataMapping dataMapping;
						String memberName;
						int preNameInd,postNameInd,newInd;
						String memberNamePre, memberNamePost;
						while (it.hasNext())
						{
							dataMapping = (DataMapping)it.next();
							memberName = dataMapping.getToDataStructureMemberName();
							preNameInd = memberName.indexOf('(')+1;
							postNameInd = memberName.indexOf(')');
							memberNamePre = memberName.substring(0,preNameInd);
							memberNamePost = memberName.substring(postNameInd);
							newInd = Integer.parseInt(memberName.substring(preNameInd,postNameInd))+index;
							dataMapping.setToDataStructureMemberName(memberNamePre+newInd+memberNamePost);
						}
						joinIndices.put(srcMd,new Integer(index+getArraySize(tarMd)));
					}
				} else {
					tarMd = (MemberDeclaration)tarMembers.get(i);
					srcMd = (MemberDeclaration)srcMembers.get(i);
					dataMappings = createDataMappings(tarMd,srcMd);					
				}
				dataFlow.getDataMappings().addAll(dataMappings);				
			}
			setTargetOfConnectors(pinSet,noopAct);
		} else {
			noopAct.setName(createNoopActivityName(srcAct,pinSet));
			// This is the case for Decision and Fork
			// link between the srcAct and noopAct
			ctrlFlow.setFromActivity(srcAct);
			ctrlFlow.setToActivity(noopAct);
			dataFlow.setFromActivity(srcAct);
			dataFlow.setToActivity(noopAct);
			// Create map from srcCon to tarCon
			List srcMembers = srcCon.getMemberDeclarations();
			List tarMembers = tarCon.getMemberDeclarations();
			int size = srcMembers.size();
			int tarSize = tarMembers.size();
			MemberDeclaration srcMd;
			MemberDeclaration tarMd;
			for (int i=0; i<tarSize; i++) { 
				if (accumulateCopy)	{
					srcMd = (MemberDeclaration)srcMembers.get(i%size);
					tarMd = (MemberDeclaration)tarMembers.get(i);
					dataFlow.getDataMappings().addAll(createDataMappings(srcMd,tarMd));
				} else {
					srcMd = (MemberDeclaration)srcMembers.get(i);
					tarMd = (MemberDeclaration)tarMembers.get(i);
					dataFlow.getDataMappings().addAll(createDataMappings(srcMd,tarMd));
				}
			}
			setSourceOfConnectors(pinSet,noopAct);
		}
		return noopAct;
		
	}

	/**
	 * Creates a ContainerRule based on the pinSet
	 * @param pinSet
	 * @return
	 */
	protected ContainerRule createContainerRule(PinSet pinSet)
	{
		ContainerRule rule = FdlFactory.eINSTANCE.createContainerRule();
		rule.getSource().add(pinSet);
		getRoot().getChildRules().add(rule);
		rule.transformSource2Target();
		return rule;		
	}

	/**
	 * Checks if two pin sets are identical.  Two pin sets are identical if they both
	 * have the same number of pins in the same order, with the same type, and same
	 * multiplicity
	 * @param ps1
	 * @param ps2
	 * @return
	 */
	protected boolean isPinSetIdentical(PinSet ps1, PinSet ps2)
	{

		boolean isIdentical=true;

		List objPins1 = getObjectPins(ps1);
		List objPins2 = getObjectPins(ps2);
		int numObjPins = objPins1.size();
		if (numObjPins != objPins2.size()) {
			isIdentical = false;
		} else {
			ObjectPin objPin1, objPin2;
			for (int i=0; i<numObjPins && isIdentical; i++) {
				objPin1 = (ObjectPin)objPins1.get(i);
				objPin2 = (ObjectPin)objPins2.get(i);
				if (!isSameType(objPin1,objPin2) || !isSameMultiplicity(objPin1,objPin2)) {
					isIdentical=false;
				}
			}
		}
		return isIdentical;
	}

	/**
	 * Check if two object pins op1 and op2 are the same type
	 * @param op1
	 * @param op2
	 * @return
	 */
	private boolean isSameType(ObjectPin op1, ObjectPin op2) {
		// Ray: try to handle null type
		if (op1.getType()==null || op2.getType()==null) {
			return true;
		}
		return op1.getType().equals(op2.getType());
	}
	
	/**
	 * Check if the multiplicity (incl lower and upper bound) of the two pins are the same
	 * @param op1
	 * @param op2
	 * @return
	 */
	private boolean isSameMultiplicity(ObjectPin op1, ObjectPin op2) {
		return (getMultiplicity(op1.getLowerBound()).equals(getMultiplicity(op2.getLowerBound())) &&
				getMultiplicity(op1.getUpperBound()).equals(getMultiplicity(op2.getUpperBound())) );
	}
	
	/**
	 * Convert the multiplicity ValueSpecification to a string
	 * @param mul
	 * @return
	 */
	private String getMultiplicity(ValueSpecification mul) 
	{
		if (mul != null)
		{
			if (mul instanceof LiteralUnlimitedNatural)
			{
				return ((LiteralUnlimitedNatural)mul).getValue();
			} else {
				return ((LiteralInteger)mul).getValue().toString();
			}
		} else {
			return "";
		}
	}
	
	/**
	 * Get a list of object pins for a pin set (input or output pin set)
	 * @param ps
	 * @return
	 */
	private List getObjectPins(PinSet ps)
	{
		if (ps instanceof InputPinSet) {
			return ((InputPinSet)ps).getInputObjectPin();
		} else {
			return ((OutputPinSet)ps).getOutputObjectPin();
		}
	}
	
	/**
	 * Creates a name for the noop activity based on the pinset
	 * @param act
	 * @param pinset
	 * @return
	 */
	private String createNoopActivityName(Activity act,PinSet pinSet)
	{
		//return act.getName()+"_"+pinset.getName();
	
		//return getNameRegistry().getActivityName(pinSet);
		String noopName = getNameRegistry().getActivityName(pinSet);
		// The returned noopName will be the same as the original control action name in the case
		// of Join (how about fork?).  Therefore need to call the registry and recreate a unique name again
		// using a different method
		return getNameRegistry().getActivityName(noopName,"");
	}
	
	/**
	 * Creates DataMapping (Rule) between two memberDeclarations
	 * @param srcMd
	 * @param tarMd
	 * @return
	 */
	private Collection createDataMappings(MemberDeclaration srcMd, MemberDeclaration tarMd)
	{
		DataMappingRule dmr = FdlFactory.eINSTANCE.createDataMappingRule();
		dmr.getSource().add(srcMd);
		dmr.getSource().add(tarMd);
		getChildRules().add(dmr);
		dmr.transformSource2Target();
		return dmr.getTarget();		
	}
	
	/**
	 * Get size of the array from a member declaration and return it as an int
	 * @param md
	 * @return
	 */
	private int getArraySize (MemberDeclaration md)
	{
		Integer mdInt = md.getArraySize();
		if (mdInt == null) {
			return 1;
		} else {
			return mdInt.intValue();
		}
	}	
} //ControlActionRuleImpl
