/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;

import com.ibm.btools.te.framework.impl.TransformationRuleImpl;
import com.ibm.btools.te.framework.util.TransformationUtil;

import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.DataFlowRule;
import com.ibm.btools.te.ilm.heuristics.fdl.DecisionRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FDLOptimizationRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.JoinRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ProgramRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlConstants;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;

import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.processes.activities.ActivityEdge;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.fdl.model.Activity;
import com.ibm.btools.fdl.model.ActivityExtensionSetting;
import com.ibm.btools.fdl.model.BasicActivity;
import com.ibm.btools.fdl.model.Block;
import com.ibm.btools.fdl.model.ControlFlow;
import com.ibm.btools.fdl.model.DataFlow;
import com.ibm.btools.fdl.model.DataMapping;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.MemberDeclaration;
import com.ibm.btools.fdl.model.ModelFactory;
import com.ibm.btools.fdl.model.Process;
import com.ibm.btools.fdl.model.Program;
import com.ibm.btools.fdl.model.ProgramActivity;
import com.ibm.btools.fdl.model.Sink;
import com.ibm.btools.fdl.model.Source;
import com.ibm.btools.fdl.model.StartConditionEnum;
import com.ibm.btools.fdl.model.SynchronizationType;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>FDL Optimization Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class FDLOptimizationRuleImpl extends TransformationRuleImpl implements FDLOptimizationRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	//private List nonReducibleNoopActivities=new ArrayList();
	private List nonReducibleLinks = new ArrayList();
		
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected FDLOptimizationRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.FDL_OPTIMIZATION_RULE;
	}

	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");
		Process process = (Process)getSource().get(0);
		List constructs = process.getConstructs();
		collectNonReducibleLinks(constructs);
		optimizeConstructs(constructs);
		getTarget().add(process);
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");		
		return isComplete();
	}
	
	private void collectNonReducibleLinks(List constructs) {
		for (Iterator it=constructs.iterator();it.hasNext();) {
			Object construct = it.next();
			if (isNoopActivity(construct) && isNoopForDecision((Activity)construct)) {
				nonReducibleLinks.addAll(((Activity)construct).getOutgoingControlFlows());
			} else if (construct instanceof Block) {
				collectNonReducibleLinks(((Block)construct).getConstructs());
			}
		}	
	}
	
	private void optimizeConstructs(List constructs) {
		List noopActs=new ArrayList();
		for (Iterator it=constructs.iterator();it.hasNext();) {
			Object construct = it.next();
			if (isNoopActivity(construct)) {
				noopActs.add(construct);
			} else if (construct instanceof Block) {
				optimizeConstructs(((Block)construct).getConstructs());
			}
		}	
		optimizeNoopActivities(noopActs);
	}
	
	private void optimizeNoopActivities(List noopActs) {
		int numNoopActs = noopActs.size();
		ProgramActivity pa;
		for (int i=numNoopActs-1;i>=0;i--) {
			pa = (ProgramActivity)noopActs.get(i);
			if (optimizeNoopActivity(pa)) {
				noopActs.remove(pa);
			}
		}
		// if there is at least one noop activities that has been removed, then it means there is a chance
		// we can further remove the remaining noop activites.  Thus we can this method recursivelt until
		// we can no longer further remove any noop activites.
		if (noopActs.size()!=numNoopActs) {
			optimizeNoopActivities(noopActs);
		}
	}
	

	private boolean optimizeNoopActivity(ProgramActivity noop) {
//		if (noop.getIncomingControlFlows().size()>1 && noop.getActivityExtensionSetting().getStartWhen()==StartConditionEnum.ALL_CONNECTORS_LITERAL) {
//			return false;
//		}
		if (noop.getIncomingControlFlows().size()<=1 || noop.getOutgoingControlFlows().size()<=1) {
			// There is a 'bug' that when a merge is connected to the outputpin of a Process/SubProcess,
			// A ControlFlow is created but doesn't connect to any activity.  Here we should eliminate this
			// controlflow before we continue
//			if (noop.getOutgoingControlFlows().size()==1 && ((ControlFlow)noop.getOutgoingControlFlows().get(0)).getToActivity()==null) {
//				removeConstructs(noop.eContainer(),noop.getOutgoingControlFlows());
//				noop.getOutgoingControlFlows().clear();
//			}
			
			if (noop.getIncomingControlFlows().size()==1) {
				if (!nonReducibleLinks.contains(noop.getIncomingControlFlows().get(0))) {
					removeActivity(noop);
					return true;
				}
			} 
			//if (noop.getOutgoingControlFlows().size()==1 && !(noop.getIncomingControlFlows().size()>1 && noop.getActivityExtensionSetting().getStartWhen()==StartConditionEnum.ALL_CONNECTORS_LITERAL)) {
			if (noop.getOutgoingControlFlows().size()==1) {
				ControlFlow ctrlFlow = (ControlFlow)noop.getOutgoingControlFlows().get(0);
				if (!nonReducibleLinks.contains(ctrlFlow)) {
//					if (ctrlFlow.getToActivity()!=null) {
						Activity act = ctrlFlow.getToActivity();
						if (act.getIncomingControlFlows().size()<=1||isSameStartCondition(noop,act)) {
							removeActivity(noop);
							return true;
						}
//					} else {
//						removeConstructs(noop.eContainer(),noop.getOutgoingControlFlows());
//						noop.getOutgoingControlFlows().clear();
//						removeActivity(noop);
//						return true;
//					}
				}
			}  
			if (noop.getIncomingControlFlows().size()==0 || noop.getOutgoingControlFlows().size()==0){
				// This is the case where the Control node is the first or last node in the process without any preceding tasks
				if (noop.getOutgoingControlFlows().size()>0) {
					ControlFlow ctrlFlow;
					for (Iterator it=noop.getOutgoingControlFlows().iterator();it.hasNext();) {
						ctrlFlow = (ControlFlow)it.next();
						//if (ctrlFlow.getWhen() != null) {
						if (nonReducibleLinks.contains(ctrlFlow)) {
							return false;
						}
					}
				} else if (noop.getIncomingControlFlows().size()>0) {
					if ((noop.getIncomingControlFlows().size()>1 && noop.getActivityExtensionSetting().getStartWhen()==StartConditionEnum.ALL_CONNECTORS_LITERAL)) {
						return false;
					}
					ControlFlow ctrlFlow;
					for (Iterator it=noop.getIncomingControlFlows().iterator();it.hasNext();) {
						ctrlFlow = (ControlFlow)it.next();
						//if (ctrlFlow.getWhen() != null) {
						if (nonReducibleLinks.contains(ctrlFlow)) {						
							return false;
						}
					}
				}
				removeActivity(noop);
				return true;
			} 
		}
		return false;
	}
	
	private boolean isNoopForDecision(Activity noop) {
		Collection trs = TransformationUtil.getRulesForTarget(getRoot(),DecisionRule.class,noop);
		TransformationRule tr=null;
		boolean bFound=false;
		for (Iterator it=trs.iterator();it.hasNext();) {
			tr = (TransformationRule)it.next();
			if (((DecisionRuleImpl)tr).getNoopActivity()==noop) {
				bFound=true;
				break;
			}
		}
		//TransformationRule tr = TransformationUtil.getRuleForTarget(getRoot(),DecisionRule.class,noop);
		//if (tr != null && ((DecisionRuleImpl)tr).getNoopActivity()==noop) {
		if (bFound == true) {
			return true;
		}		
		return false;
	}

	private boolean isNoopForJoin(Activity noop) {
		TransformationRule tr = TransformationUtil.getRuleForTarget(getRoot(),JoinRule.class,noop);
		if (tr != null && ((JoinRuleImpl)tr).getIsNoopActivity()==noop) {
			return true;
		}		
		return false;
	}
		
	private boolean isSameStartCondition(Activity srcAct, Activity tarAct) {
		if (getStartCondition(srcAct)==getStartCondition(tarAct)) {
			return true;
		}
		return false;
	}
	
	private StartConditionEnum getStartCondition (Activity act) {
		if (act instanceof BasicActivity) {
			ActivityExtensionSetting aes = ((BasicActivity)act).getActivityExtensionSetting();
			if (aes != null) {
				return aes.getStartWhen();
			}
		} else if (act instanceof Block) {
			return ((Block)act).getStartWhen();
		}
		return null;
	}
	
	private void setStartCondition (Activity act, StartConditionEnum startCondition) {
		if (startCondition != null) {
			if (act instanceof BasicActivity) {
				ActivityExtensionSetting aes = ((BasicActivity)act).getActivityExtensionSetting();
				if (aes == null) {
					aes = ModelFactory.eINSTANCE.createActivityExtensionSetting();
					((BasicActivity)act).setActivityExtensionSetting(aes);
				}
				aes.setStartWhen(startCondition);
			} else if (act instanceof Block) {
				((Block)act).setStartWhen(startCondition);
			}
		}
	}
	
	/*
	 * Assumption: act has either 0 or 1 incoming control flow and/or 0 or 1 outgoing control flow
	 */
	private void removeActivity(Activity act) {
		//System.out.println(act.getName()+" is removed");
		ControlFlow ctrlFlow;
		if (act.getIncomingControlFlows().size()==1 && 
				((ControlFlow)act.getIncomingControlFlows().get(0)).getFromActivity()!=null
				&& !nonReducibleLinks.contains(act.getIncomingControlFlows().get(0))
				) {
			ctrlFlow=(ControlFlow)act.getIncomingControlFlows().get(0);
			Activity srcAct = ctrlFlow.getFromActivity();
			srcAct.getOutgoingControlFlows().remove(ctrlFlow);
			removeConstructs(act.eContainer(),act.getIncomingControlFlows());
			if (act.getIncomingDataFlows().size()>0) {
				fixDataMappingsForDecisionOrForkNode((DataFlow)act.getIncomingDataFlows().get(0),act.getOutgoingDataFlows());
				srcAct.getOutgoingDataFlows().removeAll(act.getIncomingDataFlows());
				removeConstructs(act.eContainer(),act.getIncomingDataFlows());
			}
			srcAct.getOutgoingControlFlows().addAll(act.getOutgoingControlFlows());
			srcAct.getOutgoingDataFlows().addAll(act.getOutgoingDataFlows());
		} else if (act.getOutgoingControlFlows().size()==1 && 
				((ControlFlow)act.getOutgoingControlFlows().get(0)).getToActivity()!=null 
				&& !nonReducibleLinks.contains(act.getOutgoingControlFlows().get(0))
				) {
			ctrlFlow=(ControlFlow)act.getOutgoingControlFlows().get(0);
			Activity tarAct = ctrlFlow.getToActivity();
			tarAct.getIncomingControlFlows().remove(ctrlFlow);
			removeConstructs(act.eContainer(),act.getOutgoingControlFlows());
			if (act.getOutgoingDataFlows().size()>0) {
				if (isNoopForJoin(act)) {
					fixDataMappingsForJoinNode(act.getIncomingDataFlows(),(DataFlow)act.getOutgoingDataFlows().get(0));
				} else {
					removeIncomingDataFlows(act.getIncomingDataFlows(),(DataFlow)act.getOutgoingDataFlows().get(0));				
				}
				tarAct.getIncomingDataFlows().removeAll(act.getOutgoingDataFlows());
				removeConstructs(act.eContainer(),act.getOutgoingDataFlows());
			}
			tarAct.getIncomingControlFlows().addAll(act.getIncomingControlFlows());
			tarAct.getIncomingDataFlows().addAll(act.getIncomingDataFlows());
			// update join condition
			setStartCondition(tarAct,getStartCondition(act));
		} else {
			if (act.getOutgoingControlFlows().size()>0) {
				// try to connect Source to the target activities
				if (act.getIncomingDataFlows().size()>0 && ((DataFlow)act.getIncomingDataFlows().get(0)).getSource()!=null) {
					Source src = ((DataFlow)act.getIncomingDataFlows().get(0)).getSource();
					src.getDataFlows().addAll(act.getOutgoingDataFlows());
					src.getDataFlows().removeAll(act.getIncomingDataFlows());
					removeConstructs(act.eContainer(),act.getIncomingDataFlows());
					act.getOutgoingDataFlows().clear();
				} else {
					removeConstructs(act.eContainer(),act.getOutgoingDataFlows());
				}
			} else if (act.getIncomingControlFlows().size()>0) {
				// try to connect source activities to Sink
				if (act.getOutgoingDataFlows().size()>0 && ((DataFlow)act.getOutgoingDataFlows().get(0)).getSink()!=null) {
					Sink sink = ((DataFlow)act.getOutgoingDataFlows().get(0)).getSink();
					sink.getDataFlows().addAll(act.getIncomingDataFlows());
					sink.getDataFlows().removeAll(act.getOutgoingDataFlows());
					removeConstructs(act.eContainer(),act.getOutgoingDataFlows());
					act.getIncomingDataFlows().clear();
				} else {
					removeConstructs(act.eContainer(),act.getIncomingDataFlows());					
				}
			} else {
				removeConstructs(act.eContainer(),act.getOutgoingDataFlows());
				removeConstructs(act.eContainer(),act.getIncomingDataFlows());												
			}
			removeConstructs(act.eContainer(),act.getOutgoingControlFlows());
			removeConstructs(act.eContainer(),act.getIncomingControlFlows());
		}
		if (act.getDefaultDataFlow()!=null) {
			replaceElement(act.getDefaultDataFlow(),null);
		}
//		if (act instanceof ProgramActivity) {
//			Program prg = getProgram((ProgramActivity)act);
//			replaceElement(prg,null);
//		}
		replaceElement(act,null);
	}

	private Program getProgram(ProgramActivity pa) {
		TransformationRule tr = TransformationUtil.getRuleForSource(getRoot(),ProgramRule.class,pa);
		if (tr != null && tr.getTarget().get(0) instanceof Program) {
			 return (Program)tr.getTarget().get(0);
		}
		return null;
	}
	
	private void fixDataMappingsForDecisionOrForkNode(DataFlow incomingDataFlow, List outgoingDataFlows) {
		Activity srcAct = incomingDataFlow.getFromActivity();
		Activity tarAct = incomingDataFlow.getToActivity();
		/**
		 * We need to fix the mapping if the followings are true:
		 * 1) the output datastructure of Task1 is not the same as the Input datastructure of Decision
		 * 2) any data mapping in the outgoing data flows of the Decision is the default top level mapping (i.e. _STRUCT to _STRUCT)
		 */
		if (srcAct!=null && tarAct!=null) {
			if (srcAct.getOutputDataStructure()!=tarAct.getInputDataStructure()) {
				DataFlow outDF;
				for (Iterator it=outgoingDataFlows.iterator();it.hasNext();) {
					outDF = (DataFlow)it.next();
					if (outDF.getDataMappings().size()==1) {
						DataMapping dm = (DataMapping)outDF.getDataMappings().get(0);
						if (dm.getFromDataStructureMemberName()!=null && dm.getToDataStructureMemberName()!=null) {
							if (dm.getFromDataStructureMemberName().equals(FdlConstants.TOP_STRUCTURE_NAME)
								&& dm.getToDataStructureMemberName().equals(FdlConstants.TOP_STRUCTURE_NAME)) {
								outDF.getDataMappings().remove(0);
									if (incomingDataFlow.getDataMappings().size()>0) {
										Collection copyOfDataMappings = EcoreUtil.copyAll(incomingDataFlow.getDataMappings());
										if (copyOfDataMappings!=null && copyOfDataMappings.size()>0) {
											outDF.getDataMappings().addAll(copyOfDataMappings);
										}
									}
								}
						}
					}
				}
			}
		}
	}
	
	private void fixDataMappingsForJoinNode(List incomingDataFlows, DataFlow outgoingDataFlow) {
		DataFlow df;
		DataMapping srcDM,tarDM;
		if (outgoingDataFlow != null && outgoingDataFlow.getDataMappings().size()!=0) {
			for (int i=0; i<incomingDataFlows.size();i++){
				df = (DataFlow)incomingDataFlows.get(i);	
				List mappings = df.getDataMappings();
				if (df.getDataMappings().size()>0) {
					if (df.getToActivity()!=null && df.getToActivity().getInputDataStructure()!=null) {
						DataStructure ds = df.getToActivity().getInputDataStructure();
						for (int j=0; j<df.getDataMappings().size();j++) {
							srcDM = (DataMapping)df.getDataMappings().get(j);
							int mapPos = -1;
							String dsMemName = getTopLevelMemberName(srcDM.getToDataStructureMemberName());
							if (dsMemName != null) {
								mapPos = getDataMemberPos(ds,dsMemName);
							}
							if (mapPos!= -1 && outgoingDataFlow.getDataMappings().size()>mapPos) {
								tarDM = (DataMapping)outgoingDataFlow.getDataMappings().get(mapPos);
								if (srcDM.getToDataStructureMemberName().equals(tarDM.getFromDataStructureMemberName())) {
									srcDM.setToDataStructureMemberName(tarDM.getToDataStructureMemberName());
								}
							}
						}
					}
				}
			}
		}
	}

	/**
	 * Remove any incoming data flows that become unnecessary if the activity is to be removed
	 * An incoming data flow is unnecessary when
	 * 1) the output datastructure of the activity is different than the input datastructure of the connected downstream task
	 * 2) the target data member name in the data mapping of the incomingDataFlow is not found in the input datastructure of 
	 * the connected downstream task (on the safe side, we should also check if the types of the data member matches even if
	 * the data memeber name is found in the target (input) data structure)
	 * @param incomingDataFlows
	 * @param outgoingDataFlow
	 */
	private void removeIncomingDataFlows(List incomingDataFlows, DataFlow outgoingDataFlow) {
		DataFlow inDF;
		DataMapping inDataFlowDataMapping,outDataFlowDataMapping;
		if (outgoingDataFlow != null && outgoingDataFlow.getDataMappings().size()>0) {
			if (outgoingDataFlow.getFromActivity()!=null && outgoingDataFlow.getToActivity()!=null) {
				Activity fromAct = outgoingDataFlow.getFromActivity();
				Activity toAct = outgoingDataFlow.getToActivity();
				if (fromAct.getOutputDataStructure()!=toAct.getInputDataStructure()) {
					for (int i=incomingDataFlows.size()-1; i>=0;i--){
						inDF = (DataFlow)incomingDataFlows.get(i);	
						List mappings = inDF.getDataMappings();
						// Look for each data mapping.  Remove any that doesn't apply when the activity is removed
						for (int j=mappings.size()-1;j>=0;j--) {
							inDataFlowDataMapping = (DataMapping)inDF.getDataMappings().get(j);
							boolean foundMatchedDataMapping=false;
							for (Iterator it=outgoingDataFlow.getDataMappings().iterator();it.hasNext();) {
								outDataFlowDataMapping = (DataMapping)it.next();
								if (getTopLevelMemberName(inDataFlowDataMapping.getToDataStructureMemberName()).equals(getTopLevelMemberName(outDataFlowDataMapping.getFromDataStructureMemberName()))) {
									if (inDataFlowDataMapping.getToDataStructureMemberName().equals(outDataFlowDataMapping.getFromDataStructureMemberName())) {
										inDataFlowDataMapping.setToDataStructureMemberName(outDataFlowDataMapping.getToDataStructureMemberName());
									}
									foundMatchedDataMapping=true;
									break;
								}
							}
							if (!foundMatchedDataMapping) {
								inDF.getDataMappings().remove(j);
							}
							
//							String dsmn = getTopLevelMemberName(inDataFlowDataMapping.getToDataStructureMemberName());
//							if (dsmn != null) {								
//								if (getDataMemberPos(toAct.getInputDataStructure(),dsmn)==-1) {
//									inDF.getDataMappings().remove(j);
//								}
//							}
						}
						// If the data flow has no more mapping, then it should be removed
						if (inDF.getDataMappings().size()==0) {
							inDF.setFromActivity(null);
							inDF.setToActivity(null);
							removeConstruct(inDF.eContainer(),inDF);
						}
					}	
				}
			}
		}	
			
	}
	
	private String getTopLevelMemberName(String memberPath) {
		String topLevelMemberName = memberPath;
		if (topLevelMemberName != null) {
			//Get the top level member name
			int nestedMemPos = topLevelMemberName.indexOf('.');
			if (nestedMemPos >0) {
				topLevelMemberName = topLevelMemberName.substring(0,nestedMemPos);
			}
			int arrayPos = topLevelMemberName.indexOf('(');
			if (arrayPos >0) {
				topLevelMemberName = topLevelMemberName.substring(0,arrayPos);				
			}
		}		
		return topLevelMemberName;
	}
	
	private int getDataMemberPos(DataStructure ds,String memberName) {
		if (ds!=null && memberName!=null) {
			List mds = ds.getMemberDeclarations();
			MemberDeclaration md;
			for (int i=0; i<mds.size(); i++) {
				md = (MemberDeclaration)mds.get(i);
				if (md.getMemberName()!=null && md.getMemberName().equals(memberName)) {
					return i;
				}
			}
		}
		return -1;

	}

	private void removeConstruct(EObject container,Object construct) {
		if (construct!=null && container !=null) {
			if (container instanceof Process) {
				((Process)container).getConstructs().remove(construct);
			} else if (container instanceof Block) {
				((Block)container).getConstructs().remove(construct);
			}	
		}
	}
			
	private void removeConstructs(EObject container,List constructs) {
		if (constructs!=null && container !=null) {
			if (container instanceof Process) {
				//((Process)container).getConstructs().removeAll(constructs);
				removeAll(((Process)container).getConstructs(),constructs);
			} else if (container instanceof Block) {
				//((Block)container).getConstructs().removeAll(constructs);
				removeAll(((Block)container).getConstructs(),constructs);
			}	
		}
	}
	
	// There is a strange problem with EList.removeAll method if the type of objects in the two list are not the same.  
	// This method is to workaround it
	private void removeAll(List srcList, List removeList) {
		for (int i=srcList.size()-1;i>=0;i--) {
			Object obj = srcList.get(i);
			for (Iterator it=removeList.iterator();it.hasNext();) {
				if (obj == it.next()) {
					srcList.remove(i);
				}
			}
		}
	}
	
	private boolean isNoopActivity (Object construct) {
		if (construct instanceof ProgramActivity) {
			ProgramActivity pa = (ProgramActivity)construct;
			if (pa.getProgramName()==FdlConstants.NOOP_PROGRAM_NAME) {
				return true;
			}
		}
		return false;
	}

	private void replaceElement(EObject origElement, EObject newElement) {
		if (origElement.eContainer() != null) {
			if (origElement.eContainer().eGet(origElement.eContainmentFeature()) instanceof List){
				List list = (List)origElement.eContainer().eGet(origElement.eContainmentFeature());
				if (newElement != null) {
					int index = list.indexOf(origElement);
					list.add(index, newElement);
					/* The altActivity has shifted up one in the list -- remove it from its new position */
					list.remove(index + 1);
				} else {
					list.remove(origElement);
				}
			} else {
				/* If the container isn't a list, just set the new containment value directly */
				origElement.eContainer().eSet(origElement.eContainmentFeature(), newElement);
			}
		}		
	}
	
} //FDLOptimizationRuleImpl
