/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.processes.activities.ActivityEdge;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.ConnectorRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.FlattenRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ProcessRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ProgramRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlConstants;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.fdl.model.Activity;
import com.ibm.btools.fdl.model.ControlFlow;
import com.ibm.btools.fdl.model.DataFlow;
import com.ibm.btools.fdl.model.DataMapping;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.ProgramActivity;
import com.ibm.btools.fdl.model.SynchronizationType;

/**
 * <!-- begin-user-doc -->
 * Source: BOM InputPinSet
 * target: Empty ProgramActivity(s) 
 * 
 * Flattens the SAN and maps all elements inside the SAN. 
 * 
 * Flatten means creating empty activities to represent the InputPinSet and OutputPinSet(s).
 * Connects these empty activites with the incoming connectors and outgoing connectors.  
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class FlattenRuleImpl extends ConnectableRuleImpl implements FlattenRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	protected FlattenRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.FLATTEN_RULE;
	}

	/**
	 * Source: BOM InputPinSet
	 * target: Empty ProgramActivity(s) 
	 * Flatten the SAN and maps all elements inside the SAN. 
	 * 
	 * Note: Flatten means creating empty activities to represent the InputPinSet and OutputPinSet(s).
	 * Connects these empty activites with the incoming connectors and outgoing connectors.  
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		InputPinSet inputPinSet = (InputPinSet)(getSource().get(0));
	
		if(!isFirstTimeExec()){
			//will connect additional connectors with Empty ProgramActivity 
			reExecute(inputPinSet);
			LoggingUtil.traceExit(this, "transformSource2Target");
			return isComplete();
		}
	
		setFirstTimeExec(false);	
		
		//connect input set empty program with left side and right side edges.
		doInputPinSetConnection();
	
		//create process rule to map internal of the SAN
		ProcessRule pr = FdlFactory.eINSTANCE.createProcessRule();
		getChildRules().add(pr);	
		pr.getSource().add(inputPinSet);
		pr.transformSource2Target();
		
		createBranchRuleIfAny(inputPinSet);
	
		//connect output set empty program with left side and right side edges
		doOutputPinSetsConnection(); 	
   
		//propagate all child rules targets
		propagateChildTargets();
		setComplete(true);
		
		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");		
		return isComplete();
	}
	
	/**
	 * Creates empty program activity to represent the pinSet
	 * 
	 * @param pinSet
	 * @return
	 */
	private ProgramActivity createNoopProgramActivity(PinSet pinSet){
		ProgramActivity pa = getFdlModelFactoryInstance().createProgramActivity();
		getTarget().add(pa);
		pa.setName(getNameRegistry().getActivityName(FdlUtil.getCanonicalName(pinSet), FdlConstants.EMPTY_NAME_SUFFIX));
		//set program to the MQ predefined NOOP program
		pa.setDescription(FdlConstants.EMPTY_DESCRIPTION);
		pa.setProgramName(FdlConstants.NOOP_PROGRAM_NAME);
		pa.setSynchronization(SynchronizationType.CHAINED_LITERAL);
		
		//create NOOP Program
		ProgramRule pr = FdlFactory.eINSTANCE.createProgramRule();
		pr.getSource().add(pa);
		getRoot().getChildRules().add(pr);
		pr.transformSource2Target();
		
		ContainerRule cr = FdlFactory.eINSTANCE.createContainerRule();
		getRoot().getChildRules().add(cr); 
		cr.getSource().add(pinSet);
		cr.transformSource2Target();
	  	if(!cr.getTarget().isEmpty()){	  	
			DataStructure ds = (DataStructure)cr.getTarget().get(0);
			pa.setInputDataStructure(ds);
			pa.setOutputDataStructure(ds);
	  	}
		
		//Set Default DataFlow for the empty program activity
		DataFlow df = getFdlModelFactoryInstance().createDataFlow();
		DataMapping dm = getFdlModelFactoryInstance().createDataMapping();
		dm.setFromDataStructureMemberName(FdlConstants.TOP_STRUCTURE_NAME);
		dm.setToDataStructureMemberName(FdlConstants.TOP_STRUCTURE_NAME);
		df.getDataMappings().add(dm);
		df.setDefaultActivity(pa);
		getTarget().add(df);
		
		return pa;
		
	}
	/**
	 * Connects incoming connectors of the input pin set with the empty program activity that represent the input pin set.
	 * Creates connectors for the outgoing edges of the input pin set.  
	 * 
	 */
	private void doInputPinSetConnection(){
		InputPinSet inputPinSet = (InputPinSet)(getSource().get(0));
		//create Empty ProgramActivity to represent the InputPinSet
		ProgramActivity pa = createNoopProgramActivity(inputPinSet);
		
		//map input value pin if any to NOOP activity input container initialization
		List initialList = mapInputValuePin(inputPinSet, pa.getInputDataStructure());
		if((initialList != null) && (!initialList.isEmpty())){
			pa.getInputContainerInitials().addAll(initialList);
		}
				
		//cache the pa for re-execution of this rule
		setTargetActivity(pa);
		//connect incoming connectors of the InputPinSet to the Block 
		setTargetOfConnectors(inputPinSet, pa);
		
		//create connectors for outgoing edges of the input set
	
 		List pinList = new ArrayList();
		//get control pin from input set
		pinList.addAll(inputPinSet.getInputControlPin());
		pinList.addAll(inputPinSet.getInputObjectPin());  
   
		//pin in the input set is connected to an input pin on the inside of the SAN, 
	   	//create a ConnectorRule representaiton of this connection.
		//check each pin in the input set	  
		Pin pin;	
		ConnectorRule cr;	
		for (Iterator it= pinList.iterator(); it.hasNext(); ) {
	    	pin = (Pin) it.next();
	    	ActivityEdge edge = pin.getOutgoing();
			PinSet targetPinSet = null;
			if((edge != null) && ((targetPinSet=(PinSet)BomUtils.findTargetSet(edge)) != null)){
		    	cr = FdlFactory.eINSTANCE.createFlowRule();
		    	cr.getSource().add(pin.getOutgoing()); //edge
				cr.getSource().add(inputPinSet); //input set
				cr.getSource().add(targetPinSet);
	
				getChildRules().add(cr);
				cr.transformSource2Target();
				//set fromActivity for the ControlFlow, and DataFlow created by the cr.
				((ControlFlow)cr.getTarget().get(0)).setFromActivity(pa);
				if(cr.getTarget().size()>1){			
					((DataFlow)cr.getTarget().get(1)).setFromActivity(pa);
				}
			}
   		} 	
				 
	}
	
	/**
	 * Connects the incoming and outgoing connectors of output pin set with 
	 * the empty program activity that representing the output pin set. 
	 */
	private void doOutputPinSetsConnection(){
		InputPinSet inputPinSet = (InputPinSet)(getSource().get(0));
		List opList = inputPinSet.getOutputPinSet();
		OutputPinSet outputPinSet; 
		ProgramActivity pa; 
		for(Iterator it = opList.iterator(); it.hasNext(); ){
			outputPinSet = (OutputPinSet)it.next();
			pa = (ProgramActivity)getCachedActivity(outputPinSet);
			if(pa == null){
				pa = createNoopProgramActivity(outputPinSet);
				setCachedActivity(outputPinSet, pa);				
			}
			setTargetOfConnectors(outputPinSet, pa);
			setSourceOfConnectors(outputPinSet, pa);
		}
		
	}
	
/**
 * Gets the cached activity for the pin set
 * @param pinSet
 * @return
 */
public Activity getCachedActivity(PinSet pinSet){

	return (Activity)getActivityRegistry().get(FdlUtil.getFullName(pinSet));
}

/**
 * Caches the activity for the pin set
 * @param pinSet
 * @param activity
 */
private void setCachedActivity(PinSet pinSet, Activity activity){
	getActivityRegistry().put(FdlUtil.getFullName(pinSet), activity);
		
}
/**
 * Returns the pin set activity registry in the current transformation context.
 * The registry is a map that keeps a pinSet and actvity pair. 
 * This is needed when mapping of the output pin set is done by one path of mapping, 
 * and the mapped activity need to be retrieved by another path of mapping.
 * Note: registry cannot be kept in instance variable because each path of mapping (start for input pin set) 
 * creates one instance of Flattenrule.
 * @return
 */
private Map getActivityRegistry(){
	Map registry = ((Map)getContext().get(FdlConstants.FDL_MAPPED_OUTPUT_PIN_SET_ACTIVITY_REGISTRY));
	if(registry == null){
		registry = new HashMap(3);
		getContext().put(FdlConstants.FDL_MAPPED_OUTPUT_PIN_SET_ACTIVITY_REGISTRY, registry);
	}
	return registry;
}

	

} //FlattenRuleImpl