/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.processes.actions.Fork;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.ActivitiesFactory;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.ConcurrentBranchRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.ForkRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.fdl.model.Activity;
import com.ibm.btools.fdl.model.DataFlow;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.ProgramActivity;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Fork Rule</b></em>'.
 * Source: BOM InputPinSet of the Fork Action
 * Target: FDL Noop Program Activity representing the Fork
 * Transform a Fork to an FDL Noop Program Activity representing the Fork
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class ForkRuleImpl extends ControlActionRuleImpl implements ForkRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private ProgramActivity nopa;
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ForkRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.FORK_RULE;
	}

	/**
	 * Source: BOM InputPinSet of the Fork Action
	 * Target: FDL Noop Program Activity representing the Fork
	 */
	public boolean transformSource2Target() {	
		LoggingUtil.traceEntry(this, "transformSource2Target");
	
		if (isComplete())
			return true;
		if (getSource() == null || getSource().isEmpty())
			return false;
		InputPinSet inputPinSet = (InputPinSet)getSource().get(0);
		if (isFirstTimeExec()) {
			setFirstTimeExec(false);
			
			Fork fork = (Fork)inputPinSet.getAction();
			
			nopa = createNoopProgramActivity();
			nopa.setName(getNameRegistry().getActivityName(fork));
			getTarget().add(nopa);
				
			OutputPinSet outputPinSet = (OutputPinSet)inputPinSet.getAction().getOutputPinSet().get(0);

			createConcurrentBranchRule(outputPinSet);
			DataStructure inDataStruct=null;
			ContainerRule isCR = createContainerRule(inputPinSet);
			List targets = isCR.getTarget();
			if (targets.size() > 0) {
				inDataStruct = (DataStructure)(targets.get(0));
			}
			nopa.setInputDataStructure(inDataStruct);
			nopa.setOutputDataStructure(inDataStruct);
			
			//map input value pin if any to nopa input container initialization
		    List initialList = mapInputValuePin(inputPinSet, inDataStruct);
		    if((initialList != null) && (!initialList.isEmpty())){
			  nopa.getInputContainerInitials().addAll(initialList);
		    }	
		    
			handleOutputSets(nopa,inputPinSet,outputPinSet,isCR);
			DataFlow df = createDataDefaultConnector(nopa);
			getTarget().add(df);

			// Ray: added support for descriptions and documentations		
			Action action = inputPinSet.getAction();
			String desc = FdlUtil.transformDescription(action);
			if (desc!=null) {
				nopa.setDescription(getNameRegistry().getDescription(desc));
			}
			String doc = FdlUtil.transformDocumentation(action);
			if (doc!=null) {
				nopa.setDocumentation(getNameRegistry().getDocumentation(doc));
			}
						
			//propagate all targets from children to parent
			propagateChildTargets();					
		}
		setTargetOfConnectors(inputPinSet,nopa);
		
		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");		
		return true;		
	}

	/**
	 * Check if the InputPinSet and the corresponding OutputPinSet (see createCorrespondingOutputPinSets method)
	 * are identical and creates a FDL no-op activity if they are not.
	 * @param nopa
	 * @param is
	 * @param os
	 * @param isCR
	 */
	private void handleOutputSets(ProgramActivity nopa, InputPinSet is, OutputPinSet os, ContainerRule isCR) {
		ArrayList corOPSets = createCorrespondingOutputPinSets(is, os);
		Iterator it=corOPSets.iterator();
		OutputPinSet corOs;
		boolean allOutputPinSetIdentical=true;
		ContainerRule osCR;
		Activity compatContainerAct;
		while (it.hasNext() && allOutputPinSetIdentical) {
			corOs = (OutputPinSet)it.next();
			if (!isPinSetIdentical(is,corOs)) {
				allOutputPinSetIdentical = false;
				osCR = createContainerRule(os);
				compatContainerAct = createCompatibleContainerActivity(nopa,null,isCR,osCR,true);
				getTarget().add(compatContainerAct);
				//TODO: Jie added in -watch for this
				mapOutputPinSet(os, compatContainerAct);
				//end jie added
			} 
		}
		if (allOutputPinSetIdentical) {
			//TODO: Jie added in -watch for this
			mapOutputPinSet(os, nopa);
			//end jie added
			setSourceOfConnectors(os,nopa);
			
		}
	}
	
	/**
	 * Try to find the matching pins in the outputpinset that matches to the pins 
	 * in an inputpinset of a Fork
	 * @param is
	 * @param os
	 * @return
	 */
	private ArrayList createCorrespondingOutputPinSets(InputPinSet is, OutputPinSet os) {
		ArrayList corOPSets = new ArrayList();
		int numObjPins = is.getInputObjectPin().size();
		if (numObjPins > 0) {
			List osObjPins = os.getOutputObjectPin();
			int totalIteration = osObjPins.size()/numObjPins;
			OutputPinSet corOPSet;
			OutputObjectPin osObjPin,corOsObjPin;
			for (int i=0; i < totalIteration; i++) {
				corOPSet = ActivitiesFactory.eINSTANCE.createOutputPinSet();
				for (int j=0; j<numObjPins; j++) {
					osObjPin = (OutputObjectPin)osObjPins.get(j+(i*numObjPins));
					//corOsObjPin = (OutputObjectPin)copyObjectPin(osObjPin);
					corOPSet.getOutputObjectPin().add(osObjPin);
				}
				corOPSets.add(corOPSet);
			}
		}
		return corOPSets;
	}
	
	/**
	 * Creats a concurrent branch rule for a Fork
	 * @param outputPinSet
	 * @return
	 */
	private ConcurrentBranchRule createConcurrentBranchRule(OutputPinSet outputPinSet) {
		ConcurrentBranchRule rule=null;
		if (BomHelper.getInstance().isCBranch(outputPinSet)) {
			rule = FdlFactory.eINSTANCE.createConcurrentBranchRule();
			rule.getSource().add(outputPinSet);
			getChildRules().add(rule);
			rule.transformSource2Target();	 
		}
		return rule;
	}
	
} //ForkRuleImpl
