/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.artifacts.MultiplicityElement;
import com.ibm.btools.bom.model.artifacts.TypedElement;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.DataTypeRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.MemberDeclRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.fdl.model.DataType;
import com.ibm.btools.fdl.model.MemberDeclaration;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Member Decl Rule</b></em>'.
 * Source: BOM TypedElement
 * Target: FDL MemberDeclaration 
 * Transform a BOM TypedElement (pin, property, etc.) to an FDL MemberDeclaration
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class MemberDeclRuleImpl extends AbstractFdlDataStructRuleImpl implements MemberDeclRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected MemberDeclRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.MEMBER_DECL_RULE;
	}

	/**
	 * Source: BOM TypedElement
	 * Target: FDL MemberDeclaration
	 */
	public boolean transformSource2Target()
	{
		LoggingUtil.traceEntry(this, "transformSource2Target");

		if (isComplete())
			return true;
		if (getSource() == null || getSource().isEmpty())
			return false;
		TypedElement element = (TypedElement)getSource().get(0);
		MemberDeclaration md = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createMemberDeclaration();
		if (element instanceof MultiplicityElement)
		{
			int arraySize = getArraySize((MultiplicityElement)element);
			if (arraySize > 1)
			{
				md.setArraySize(new Integer(arraySize));
			}
		}
	
		md.setMemberName(element.getName());
		// Ray: try to fix null type
		if (element.getType() != null) {
			DataTypeRule dtRule = FdlFactory.eINSTANCE.createDataTypeRule();
			dtRule.getSource().add(element.getType());
			getChildRules().add(dtRule);
			dtRule.transformSource2Target();
			md.setType((DataType)dtRule.getTarget().get(0));
		} else {
			// Log a warning
			String params[] = new String[1];
			params[0] = BomUtils.getCanonicalName(element,false,true);
			LoggingUtil.logWarning(MessageKeys.NO_TYPE_SPECIFIED,params,null);
		}
		// Ray: added support for descriptions and documentations		
		String desc = FdlUtil.transformDescription(element);
		if (desc!=null) {
			md.setDescription(getNameRegistry().getDescription(desc));
		}		
		getTarget().add(md);
		setComplete(true);
		
		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();						
	}
	
	/**
	 * obtain the size of an array for the multiplicity element
	 * @param element
	 * @return
	 */
	private int getArraySize(MultiplicityElement element)
	{
		validateMultiplicity(element);
		return getMultiplicity(element.getUpperBound());
	}
} //MemberDeclRuleImpl
