/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.processes.actions.Merge;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.ConcurrentBranchRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.MergeRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.fdl.model.Activity;
import com.ibm.btools.fdl.model.ActivityExtensionSetting;
import com.ibm.btools.fdl.model.DataFlow;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.ProgramActivity;
import com.ibm.btools.fdl.model.StartConditionEnum;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Merge Rule</b></em>'.
 * Source: BOM Merge InputSet
 * Target: FDL Noop Activity representing BOM Merge
 * Transform a Merge InputSet to a FDL Noop Activity representing BOM Merge
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class MergeRuleImpl extends ControlActionRuleImpl implements MergeRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private HashMap isMap=null;
	private ProgramActivity nopa=null;
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected MergeRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.MERGE_RULE;
	}

	/**
	 * Source: BOM Merge InputSet
	 * Target: FDL Noop Activity representing BOM Merge
	 */
	public boolean transformSource2Target()
	{
		LoggingUtil.traceEntry(this, "transformSource2Target");

		if (isComplete())
			return true;
		if (getSource() == null || getSource().isEmpty())
			return false;
		InputPinSet inputPinSet = (InputPinSet)getSource().get(0);
		Merge merge = (Merge)inputPinSet.getAction();
		MergeRule mergeRule = (MergeRule) TransformationUtil.getRuleForSource(
											ProcessUtil.getProcessDefinitionRule(getContext()),
											MergeRule.class,
											merge
											);
		if (mergeRule != null && mergeRule != this) {
			mergeRule.transformSource2Target();
			getTarget().addAll(mergeRule.getTarget());
			
			/* Don't executeHanlders() here because this rule is not really executing...
			 * The actual rule executing is 'mergeRule' above, which will call executeHandlers() */
			
			return true;
		}
		
		if (isFirstTimeExec()) {
			setFirstTimeExec(false);
			
			getSource().add(merge);
			
			nopa = createNoopProgramActivity();
			nopa.setName(getNameRegistry().getActivityName(merge));
			ActivityExtensionSetting actExtSetting = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createActivityExtensionSetting();
			actExtSetting.setStartWhen(StartConditionEnum.AT_LEAST_ONE_CONNECTOR_LITERAL);
			nopa.setActivityExtensionSetting(actExtSetting);

			getTarget().add(nopa);

			OutputPinSet outputPinSet = (OutputPinSet)inputPinSet.getAction().getOutputPinSet().get(0);
			createConcurrentBranchRule(outputPinSet);
			setSourceOfConnectors(outputPinSet, nopa);
								
			DataStructure dataStruct = null;
			ContainerRule osCR = createContainerRule(outputPinSet);
			List targets = osCR.getTarget();	
			if (targets.size() > 0) {
				dataStruct = (DataStructure)(targets.get(0));
			}
			nopa.setInputDataStructure(dataStruct);
			nopa.setOutputDataStructure(dataStruct);		
			isMap = handleInputSets(nopa, outputPinSet, osCR);

			DataFlow df = createDataDefaultConnector(nopa);
			getTarget().add(df);

			// Ray: added support for descriptions and documentations		
			Action action = inputPinSet.getAction();
			String desc = FdlUtil.transformDescription(action);
			if (desc!=null) {
				nopa.setDescription(getNameRegistry().getDescription(desc));
			}
			String doc = FdlUtil.transformDocumentation(action);
			if (doc!=null) {
				nopa.setDocumentation(getNameRegistry().getDocumentation(doc));
			}			
			propagateChildTargets();				
		}
		updateInputPinSetsLink(merge.getInputPinSet());
		
		executeHandlers();

		LoggingUtil.traceExit(this, "transformSource2Target");		
		return true;
	}

	/**
	 * Find the MergeRule of 'merge' Merge node
	 * @param merge
	 * @return
	 */
	private MergeRule findMergeRule(Merge merge) {
		return (MergeRule) TransformationUtil.getRuleForSource(
			ProcessUtil.getProcessDefinitionRule(getContext()),
			MergeRule.class,
			merge
			);
	}
	
	/**
	 * Update target of all links of all the Merge Node's inputpinset (see updateInputPinSetLink for details)
	 * @param inputPinSets
	 */
	private void updateInputPinSetsLink(List inputPinSets) {
		Iterator it = inputPinSets.iterator();
		while (it.hasNext()) {
			updateInputPinSetLink((InputPinSet)it.next());
		}
	}
	/**
	 * Update the target of the link that is going into the Merge node
	 * because we may have to create an extra no-op acitivity to represent an inputpinset
	 * of a merge node if it is not identical to the outputpinset
	 * @param is
	 */	
	private void updateInputPinSetLink(InputPinSet is)
	{
		ProgramActivity pa = null;

		if (isMap != null)
		{
			pa = (ProgramActivity)isMap.get(is);
			if (pa==null) {
				pa = nopa;
			}
		} else {
			pa = nopa;
		}
		setTargetOfConnectors(is, pa);
	}
	
	/**
	 * Create a HashMap hm (if not already created) that collects
	 * the relationship between the InputPinSet is (key) and the
	 * noopAct (value).  This allows linking the target of the
	 * Control/Data flow to the right Program Activity later
	 * 
	 * @param nopa
	 * @param os
	 * @param osCR
	 * @return
	 */
	private HashMap handleInputSets(ProgramActivity nopa,OutputPinSet os,ContainerRule osCR)
	{
		HashMap isMap = null;
		List inputPinSets = os.getAction().getInputPinSet();
		InputPinSet is;
		Iterator it = inputPinSets.iterator();	
		Activity compatContainerAct;
		ContainerRule isCR;
		while (it.hasNext()) {
			is = (InputPinSet)it.next();
			isCR = createContainerRule(is);
			if (is.getInputObjectPin().size()>1 || !isPinSetIdentical(is,os)) {
				compatContainerAct = createCompatibleContainerActivity(null,nopa,osCR,isCR,false);
				getTarget().add(compatContainerAct);
				if (isMap == null) {
					isMap = new HashMap();
				}
				isMap.put(is,compatContainerAct);
			}
		}
		return isMap;
	}	
	
	/**
	 * Creates a concurrent branch rule is a concurrent branch is detected
	 * @param outputPinSet
	 * @return
	 */
	private ConcurrentBranchRule createConcurrentBranchRule(OutputPinSet outputPinSet) {
		ConcurrentBranchRule rule=null;
		if (BomHelper.getInstance().isCBranch(outputPinSet)){
			rule = FdlFactory.eINSTANCE.createConcurrentBranchRule();
			rule.getSource().add(outputPinSet);
			getChildRules().add(rule);
			rule.transformSource2Target();	 
		}
		return rule;
	}
	
	/**
	 * Returns the NOOP ProgramActivity created to represent this input pin set, 
	 * if it exists.  Returns null otherwise.
	 * @param is
	 * @return
	 */
	public ProgramActivity getIsNoopActivity(InputPinSet is){
		if (isMap != null){
			return (ProgramActivity)isMap.get(is);
		}
		
		return null;
	}
} //MergeRuleImpl
