/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.actions.RetrieveArtifactAction;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.InputValuePin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Repository;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.RetrieveDatastoreArtifactActionRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.ProgramActivity;

/**
 * <!-- begin-user-doc -->
 * Source: RetrieveArtifactAction (referencing Datastore)
 * Target: FDL ProgramActivity (referencing the retrieve from Datastore Program)
 *  
 * Maps RetrieveArtifactAction to FDL ProgramActivity.
 * Connects the activity with incoming connector and outgoing connectors.
 * 
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class RetrieveDatastoreArtifactActionRuleImpl extends RetrieveDatastoreArtifactRuleImpl implements RetrieveDatastoreArtifactActionRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected RetrieveDatastoreArtifactActionRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.RETRIEVE_DATASTORE_ARTIFACT_ACTION_RULE;
	}

	/**
	 * Source: RetrieveArtifactAction (referencing Datastore)
	 * Target: FDL ProgramActivity (referencing retrieve from Datastore Program)
	 *  
	 * Maps RetrieveArtifactAction to FDL ProgramActivity.
	 * Connects the activity with incoming connector and outgoing connectors.
	 * 
	 *  
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");
		
		if (getSource() == null || getSource().isEmpty())
			return false;		
		InputPinSet inputSet = (InputPinSet)getSource().get(0);
		OutputPinSet outputSet = (OutputPinSet)inputSet.getOutputPinSet().get(0);
		RetrieveArtifactAction action = (RetrieveArtifactAction)inputSet.getAction();

		if(!(isFirstTimeExec())){
			LoggingUtil.traceExit(this, "transformSource2Target","re-execute");
			return true;	
		}

		//first time execute this rule, do following
		setFirstTimeExec(false);
	
		ProgramActivity pa = createProgramActivity(action);
		//do InputPinSet connection
		setTargetOfConnectors(inputSet, pa);
		
		createBranchRuleIfAny(inputSet);
		//do OutputPinSet connection
		setSourceOfConnectors(outputSet, pa);
		//all targets propagate to parent
		propagateChildTargets();

		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
		
	}
		
/**
 * Creates a program activity for the <code>action</code> that retrieve data from the datastore.
 * 
 * @param action The retireve artifact action
 * @return ProgramActivity - The retrieve datastore program activity created
 */
	private ProgramActivity createProgramActivity(RetrieveArtifactAction action){
		
		Repository datastore = action.getRepository();
				
		// create output datastructure use the OutputPinSet according to the pin multiplicity, assuming datstore program can handle any multiplicity
		ContainerRule cr = FdlFactory.eINSTANCE.createContainerRule();
		cr.getSource().add(action.getOutputPinSet().get(0));
		getRoot().getChildRules().add(cr);
		cr.transformSource2Target();
		DataStructure outputDs = (DataStructure)cr.getTarget().get(0);
		
	    String paName = getNameRegistry().getActivityName(action);
			
		InputObjectPin pin =(InputObjectPin) action.getLength();
		String lengthValueString = "1";
	    if(pin instanceof InputValuePin){
		  //need to extract the value and set as initial value to program input datastructure
		  //assume there will be only one value for ArtifactAction
		  ValueSpecification valueSpec = (ValueSpecification)((InputValuePin)pin).getValue().get(0);
		  lengthValueString = FdlUtil.getValueSpecificationString(valueSpec);
		  if(lengthValueString == null){
			 lengthValueString = "1";
		  }			
		}else{					
			//log warning, we cannot support this
			String params[]={pin.getName(),pin.getAction().getName()};		
			LoggingUtil.logWarning(MessageKeys.IVP_NOT_LIT_SPEC_NOT_SUPPORTED,params,null);					
	 	}  	
		//create program activity for the retrieve datastore program	
		ProgramActivity pa = createRetrieveDatastoreProgramActivity(
			datastore, 
			paName, 
			outputDs,
			action.getIsRemove(),
			action.getAtBeginning(),
			lengthValueString);
	
	    String dsc = FdlUtil.getFullName(action);
	    pa.setDescription(getNameRegistry().getDescription(dsc));
	    		 
	    getTarget().add(pa);
	    return pa;

	}	

} //RetrieveDatastoreArtifactActionRuleImpl
