/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.processes.activities.Repository;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.DatastoreRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.RetrieveDatastoreArtifactRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlConstants;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.fdl.model.ContainerInitial;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.MemberDeclaration;
import com.ibm.btools.fdl.model.Program;
import com.ibm.btools.fdl.model.ProgramActivity;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Retrieve Datastore Artifact Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public abstract class RetrieveDatastoreArtifactRuleImpl
	extends ConnectableRuleImpl
	implements RetrieveDatastoreArtifactRule {
		
		public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
		
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected RetrieveDatastoreArtifactRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.RETRIEVE_DATASTORE_ARTIFACT_RULE;
	}

	/**
	 * Creates a retrieve datastore ProgramActivity and creates input container initialization
	 * 	
	 * @param datastore The datastore that the program activity is accessing
	 * @param actName	The name of the program activity to be created
	 * @param outputDs	The output datastructure for the retrieve program activity
	 * @param isRemove	The value of isRemove attribute for the input container
	 * @param atBeginning The value of atBeginning attribute for the input container
	 * @param length	The value of length attribute for the input container 
	 * @return ProgramActivity - The retrieve datastore program activity created
	 */
	protected ProgramActivity createRetrieveDatastoreProgramActivity(
		Repository datastore,
		String actName,
		DataStructure outputDs,
		Boolean isRemove,
		Boolean atBeginning,
		String length) {
		/*	
		 * 1. creates DatastoreRule to get the retrieve from datastore program.
		 * 2. gets the input container from the program and set it to the ProgramActivity.
		 * 3. creates input container initialization using the pin's attributes.
		*/
	
		if (datastore instanceof com.ibm.btools.bom.model.processes.activities.Variable) {
			LoggingUtil.logWarning(MessageKeys.VARIABLE_REPOSITORY_NOT_SUPPORTED);
			//TODO: throw exception may cause some test cases not run
		//	BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.VARIABLE_REPOSITORY_NOT_SUPPORTED));							
		//	throw e;			
		}

		//	create datastore rule for the datastore
		DatastoreRule dsr = FdlFactory.eINSTANCE.createDatastoreRule();
		dsr.getSource().add(datastore);
		getChildRules().add(dsr);
		dsr.transformSource2Target();
		//we know the first target is the retrieve database program
		Program prm = (Program) dsr.getTarget().get(0);

		DataStructure inputDs = prm.getInputDataStructure();
		ProgramActivity pa = getFdlModelFactoryInstance().createProgramActivity();
		pa.setName(actName);
		pa.setProgramName(prm.getName());
		pa.setInputDataStructure(inputDs);
		pa.setOutputDataStructure(outputDs);

		//set initial value of input datastucture
		ContainerInitial ci =
			getFdlModelFactoryInstance().createContainerInitial();
		//get isRemove attribute
		ci.setMemberName(
			((MemberDeclaration) inputDs.getMemberDeclarations().get(0))
				.getMemberName());
		String initValue = isRemove.booleanValue() ? (FdlConstants.TRUE_INT_STRING) : (FdlConstants.FALSE_INT_STRING);
		ci.setInitialValue(initValue);
		pa.getInputContainerInitials().add(ci);

		ci = getFdlModelFactoryInstance().createContainerInitial();
		//get atBegining attribute
		ci.setMemberName(
			((MemberDeclaration) inputDs.getMemberDeclarations().get(1))
				.getMemberName());
		initValue = atBeginning.booleanValue() ? (FdlConstants.TRUE_INT_STRING) : (FdlConstants.FALSE_INT_STRING);
		ci.setInitialValue(initValue);
		pa.getInputContainerInitials().add(ci);


		ci = getFdlModelFactoryInstance().createContainerInitial();
		//get length attribute
		ci.setMemberName(
			((MemberDeclaration) inputDs.getMemberDeclarations().get(2))
				.getMemberName());
		ci.setInitialValue(length);
		pa.getInputContainerInitials().add(ci);

		return pa;
	}

} //RetrieveDatastoreArtifactRuleImpl
