/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Calendar;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.artifacts.Constraint;
import com.ibm.btools.bom.model.artifacts.LiteralString;
import com.ibm.btools.bom.model.artifacts.Slot;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnit;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.resources.BulkResourceRequirement;
import com.ibm.btools.bom.model.resources.IndividualResource;
import com.ibm.btools.bom.model.resources.IndividualResourceRequirement;
import com.ibm.btools.bom.model.resources.RequiredRole;
import com.ibm.btools.bom.model.resources.ResourceRequirement;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.TransformationEngine;
import com.ibm.btools.te.ilm.TransformationSession;
import com.ibm.btools.te.ilm.heuristics.fdl.CategoryRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.PersonRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ProcessRule;
import com.ibm.btools.te.ilm.heuristics.fdl.RoleRule;
import com.ibm.btools.te.ilm.heuristics.fdl.SANReusableProcessRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomWalker;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.helper.StaffUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.util.datatype.BTDataTypeException;
import com.ibm.btools.util.datatype.BTDateTime;
import com.ibm.btools.util.datatype.BTTime;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.expression.model.Expression;
import com.ibm.btools.expression.model.FunctionArgument;
import com.ibm.btools.expression.model.FunctionExpression;
import com.ibm.btools.expression.model.ModelPathExpression;
import com.ibm.btools.expression.model.ReferenceStep;
import com.ibm.btools.expression.model.StringLiteralExpression;
import com.ibm.btools.fdl.model.AuditOption;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.DefaultProcessSetting;
import com.ibm.btools.fdl.model.Person;
import com.ibm.btools.fdl.model.Process;
import com.ibm.btools.fdl.model.ProcessCategory;
import com.ibm.btools.fdl.model.ProcessStaffAssignmentSetting;
import com.ibm.btools.fdl.model.Role;
import com.ibm.btools.fdl.model.Sink;
import com.ibm.btools.fdl.model.Source;
import com.ibm.btools.fdl.model.StaffOption;
import com.ibm.btools.fdl.model.TimeStamp;

/**
 * <!-- begin-user-doc -->
 * Maps one path of the BOM process flow to a FDL Process, 
 * starting from the InputPinSet and any InitialNode associated with the InputPinSet. 
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class SANReusableProcessRuleImpl extends ContainmentRuleImpl implements SANReusableProcessRule {
	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected SANReusableProcessRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.SAN_REUSABLE_PROCESS_RULE;
	}

	/*
	 * Source: InputPinSet of SAN reusable process
	 * Target: FDL Process
	 * 
	 * Maps one path of the BOM process flow to a FDL Process, 
	 * starting from the InputPinSet and any InitialNode associated with the InputPinSet. 
	 */
	public boolean transformSource2Target(){
		LoggingUtil.traceEntry(this, "transformSource2Target");
		
		if(isComplete())
		   return true;
		if (getSource() == null || getSource().isEmpty())
			   return false;			
		
		TransformationContext context = getContext();
		// Create BOM Walker and store it in the context
		BomWalker bomWalker = new BomWalker();
		ProcessUtil.putBomWalker(context,bomWalker);
				
		// Put ProcessDefinitionRule in the context with preset key for easy access
		 // During the traversal of this process
		 ProcessUtil.putProcessDefinitionRule(context,this);
		//----------------------------
		
		//clear name registry for new process
		getNameRegistry().resetForProcess();
		
		InputPinSet inputSet = (InputPinSet)getSource().get(0);
		if(inputSet.getOutputPinSet().size() > 1){
			String params[]={inputSet.getName(),inputSet.getAction().getName()};
			LoggingUtil.logError(MessageKeys.TOP_LEVEL_MULTIPLE_CORR_NOT_SUPPORTED,params,null);
		}
		OutputPinSet outputSet = (OutputPinSet)inputSet.getOutputPinSet().get(0);
		StructuredActivityNode san = (StructuredActivityNode)inputSet.getAction();

		//create fdl Process to hold this particular process
		Process p = getFdlModelFactoryInstance().createProcess();
		
		Integer index = null;
		if(san.getInputPinSet().size() > 1){
			index = new Integer(san.getInputPinSet().indexOf(inputSet) + 1);
		}
		String pName = getNameRegistry().getProcessName(san.getActivity(), index);
		p.setName(pName);
		TransformationSession s = TransformationEngine.getTransformationSession();
		BTDateTime validFrom = TransformationSessionUtil.getValidFromOverride(s, san); 
		if (validFrom != null){
			Calendar cal = Calendar.getInstance();
			cal.set(validFrom.getYear(), validFrom.getMonth()-1, validFrom.getDay(), validFrom.getHour(), validFrom.getMinute(), validFrom.getSecond());
			if (validFrom.getTimeFormat() != BTTime.Universal_TIME_ZONE_Format){
				try {
					int plusMinus = (validFrom.getTimeZone_Plus_Minus() > 0 ? -1 : 1);
					cal.add(Calendar.HOUR_OF_DAY, plusMinus * validFrom.getTimeZoneHour());
					cal.add(Calendar.MINUTE, plusMinus * validFrom.getTimeZoneMin());
				} catch (BTDataTypeException e){
					/* In this case, the date may be inconsistent, so reset it to the original value */
					cal.set(validFrom.getYear(), validFrom.getMonth()-1, validFrom.getDay(), validFrom.getHour(), validFrom.getMinute(), validFrom.getSecond());
				}
			} 
			
			TimeStamp t = getFdlModelFactoryInstance().createTimeStamp();
			t.setYear(new Integer(cal.get(Calendar.YEAR)));
			t.setMonth(new Integer(cal.get(Calendar.MONTH)+1));
			t.setDay(new Integer(cal.get(Calendar.DATE)));
			t.setHours(new Integer(cal.get(Calendar.HOUR_OF_DAY)));
			t.setMinutes(new Integer(cal.get(Calendar.MINUTE)));
			t.setSeconds(new Integer(cal.get(Calendar.SECOND)));
			p.setValidFrom(t);
		}
		// Required by OME
		DefaultProcessSetting dps = getFdlModelFactoryInstance().createDefaultProcessSetting();
		dps.setAuditOption(AuditOption.FULL_LITERAL);
		dps.setAuditToDb(Boolean.TRUE);
		p.setDefaultProcessSetting(dps);
		
		getTarget().add(p);		
		
		ProcessCategory pc;
		//create process category	
		com.ibm.btools.bom.model.processes.activities.Activity 
			activity =((StructuredActivityNode)inputSet.getAction()).getActivity();
		CategoryRule catRule = FdlFactory.eINSTANCE.createCategoryRule();
		catRule.getSource().add(activity);
		getRoot().getChildRules().add(catRule);
		catRule.transformSource2Target();
		pc = (ProcessCategory)catRule.getTarget().get(0);	
		p.setProcessCategory(pc);
		
		//create input container and output container rule
		DataStructure inputDs = createDataStructure(inputSet);
		DataStructure outputDs = createDataStructure(outputSet); 
		
		p.setInputDataStructure(inputDs);
		p.setOutputDataStructure(outputDs);
		
		//map any input value pin in the input pin set to process input container initialization
		List initialList = mapInputValuePin(inputSet, inputDs);
		if((initialList != null) && (!initialList.isEmpty())){
			p.getInputContainerInitials().addAll(initialList);
		}
		//create source and sink
		Source src = createSource(inputSet);		
		Sink sink = createSink(outputSet);
		
		//create connectors for the outgoing edge of the InputPinSet
		if(src != null){
			p.getSource().add(src);
			connectInputPinSet(inputSet, src);
		}			
					
		//create process rule to map the content of the SAN reusable process
		ProcessRule pr = FdlFactory.eINSTANCE.createProcessRule();
		getChildRules().add(pr);
		pr.getSource().add(inputSet);
		pr.transformSource2Target();
		
		//connect incoming connectors of the OutputPinSet with the fdl sink.
		if(sink != null){
			p.getSink().add(sink);
			connectOutputPinSet(outputSet, sink);
		}	
		createResource(inputSet.getAction(), p);
		//process child targets
		processChildTargets(p);		

		// Ray: added support for descriptions and documentations		
		Action action = inputSet.getAction();
		String desc = FdlUtil.transformDescription(action);
		if (desc!=null) {
			p.setDescription(getNameRegistry().getDescription(desc));
		}
		String doc = FdlUtil.transformDocumentation(action);
		if (doc!=null) {
			p.setDocumentation(getNameRegistry().getDocumentation(doc));
		}		
		setComplete(true);
		
		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");		
		return isComplete();		
	}
	
	/**
	 * Creates a Resource to represent the a Person or a Role or a Query
	 * @param action - a BOM action
	 * @param activity - a program activity
	 * @return
	 */
	private void createResource(Action action, Process process)
	{
		ProcessStaffAssignmentSetting saSettings = null;
		//FDL Roles
		ResourceRequirement candidateRequirement = null;
		if(StaffUtil.isStaffRequired(action)){
			List resourceRequirements = action.getResourceRequirement();			
			Iterator iterator = resourceRequirements.iterator();
			while(iterator.hasNext()){
				ResourceRequirement thisRequirement = (ResourceRequirement)iterator.next();
				//BulkResourceRequirement are not mapped to Staff queries.
				if(!(thisRequirement instanceof BulkResourceRequirement)){
					saSettings = process.getProcessStaffAssignmentSetting();
					if( saSettings == null){
						saSettings = getFdlModelFactoryInstance().createProcessStaffAssignmentSetting();
					}
					if(thisRequirement instanceof IndividualResourceRequirement){
						IndividualResourceRequirement individual = (IndividualResourceRequirement)thisRequirement;
						//IndividualResourcesRequirement with assigned IndividualResources of predefinedType si map to the Users by userID  
						if (individual.getIndividualResource()!= null && StaffUtil.isPredefinedStaff(individual.getIndividualResource().getClassifier())){
							Person person = findPerson(individual.getIndividualResource());
							if (person==null){
								//Create FDL Person
								  PersonRule personRule = FdlFactory.eINSTANCE.createPersonRule();
								  personRule.getSource().add(individual.getIndividualResource());
								  getRoot().getChildRules().add(personRule);							
								  personRule.transformSource2Target();
								  person = (Person)personRule.getTarget().get(0);
							}
							
							if(saSettings.getProcessAdministrator() == null && person != null){
								StaffOption staffOption = getFdlModelFactoryInstance().createStaffOption();
								staffOption.setValue(person.getUserID());
								saSettings.setProcessAdministrator(staffOption);
							}
						}else if (individual.getResourceType()!= null && StaffUtil.isPredefinedStaff(individual.getResourceType())){
							if(thisRequirement.getOwnedConstraint()!= null && !thisRequirement.getOwnedConstraint().isEmpty()){
								Constraint constraint = (Constraint)thisRequirement.getOwnedConstraint().get(0);
								//only known constraints are mapped to predefined queries.
								if(isKnownConstraint(constraint)){
									//Query suport for GroupMembers and Manager of Organization
									StructuredOpaqueExpression resReqConstraint = (StructuredOpaqueExpression)constraint.getSpecification();
									FunctionExpression function = (FunctionExpression)resReqConstraint.getExpression();
									String functionID = function.getDefinition().getFunctionID();				
									if(functionID.equals(StaffUtil.GROUP_MEMBERS_FUNCTION)){
										String organizationID = getOrganizationID(function);
										if(saSettings.getOrganization() == null && organizationID != null ){
											StaffOption staffOption = getFdlModelFactoryInstance().createStaffOption();
											staffOption.setValue(organizationID);
											saSettings.setOrganization(staffOption);
										}
									}
								}
							}
						}
												
					//Only RequiredRoles with predefined specified ResourceType are mapped to Staff Queries	
					}else if(thisRequirement instanceof RequiredRole){
						if (((RequiredRole)thisRequirement).getResourceType()!=null && StaffUtil.isPredefinedStaff(((RequiredRole)thisRequirement).getResourceType())){
							if (((RequiredRole)thisRequirement).getRole()!=null ){
								Role role = findRole(((RequiredRole)thisRequirement).getRole());
								if (role==null){
									RoleRule roleRule = FdlFactory.eINSTANCE.createRoleRule();
									roleRule.getSource().add(((RequiredRole)thisRequirement).getRole());
									getRoot().getChildRules().add(roleRule);
									roleRule.transformSource2Target();	
									role = (Role)roleRule.getTarget().get(0);
								}								
								if(saSettings.getRole() == null && role != null){
									StaffOption staffOption = getFdlModelFactoryInstance().createStaffOption();
									staffOption.setValue(role.getName());
									saSettings.setRole(staffOption);
								}	
							}
						}				
					}
					
				}
				if(saSettings != null){
					process.setProcessStaffAssignmentSetting(saSettings);
				}		
				
			}				 
		}
	}

	/**
	 * Returns true if the input Constraint is a known predefined SSS query constraint
	 * @param constraint BOM Constraint
	 * @return boolean
	 */
	private boolean isKnownConstraint(Constraint constraint){
		if(constraint.getSpecification() instanceof StructuredOpaqueExpression){
			StructuredOpaqueExpression bomExpression = (StructuredOpaqueExpression)constraint.getSpecification();
			Expression expression = bomExpression.getExpression();
			if(expression instanceof FunctionExpression){
				FunctionExpression function = (FunctionExpression)expression;
				String functionID = function.getDefinition().getFunctionID();
				if(StaffUtil.getPredefinedFDLResourceRequirementConstraints().contains(functionID)){
					return true;
				}
			}
		}
		return false;
	}
	
	private Person findPerson (IndividualResource resource){
		TransformationRule rule = TransformationUtil.getRuleForSource(getRoot(), PersonRule.class, resource);
		if (rule!=null && rule.getTarget().get(0) instanceof Person)
			return (Person)rule.getTarget().get(0);
		return null;
	}
	
	private Role findRole (com.ibm.btools.bom.model.resources.Role role){
		TransformationRule rule = TransformationUtil.getRuleForSource(getRoot(), RoleRule.class, role);
		if (rule!=null && (rule.getTarget().get(0) instanceof Role))
			return (Role)rule.getTarget().get(0);
		return null;
	}
		
	
	/**
	 * Creates a Group Members Query
	 * @param function - a  BOM FunctionExpression
	 * @return DoneBy - FDL Done By, this is a query specified in Activity Extension Setting
	 */		
	private String getOrganizationID(FunctionExpression function)
	{
		String organizationID = null;
		List arguments = function.getArguments();
		if(arguments != null && !arguments.isEmpty()){
			Iterator it = arguments.iterator();
			while(it.hasNext()){
				FunctionArgument functionArgument = (FunctionArgument)it.next();				
				String argumentID = functionArgument.getArgumentID();	
				if(argumentID.equals(StaffUtil.GROUP_MEMBERS_NAME_ARG)){
					Expression valueParameterExpression = functionArgument.getArgumentValue();
					if(valueParameterExpression instanceof StringLiteralExpression){
						organizationID = "'" + ((StringLiteralExpression)valueParameterExpression).getStringSymbol() + "'";
					}else if (valueParameterExpression instanceof ModelPathExpression){
						ModelPathExpression pathExpression = (ModelPathExpression)valueParameterExpression;
						//get the last step
						ReferenceStep lastStep = (ReferenceStep)pathExpression.getSteps().get(pathExpression.getSteps().size()-1);
						EObject eObject = lastStep.getReferencedObject();
						if(eObject instanceof OrganizationUnit){
							//Should be an OrganizationUnit
							List orgSlots = ((OrganizationUnit)eObject).getSlot();							
							if(orgSlots != null && !orgSlots.isEmpty()){
								Iterator orgIt = orgSlots.iterator();
								while (orgIt.hasNext()){		
									Slot thisOrgSlot = (Slot)orgIt.next();					
									if (thisOrgSlot.getDefiningFeature().getName()!=null){					
										if(thisOrgSlot.getDefiningFeature().getName().equals("organizationID")){
											if (thisOrgSlot.getValue()!=null && !thisOrgSlot.getValue().isEmpty()){
												organizationID = "'" + ((LiteralString)thisOrgSlot.getValue().get(0)).getValue() + "' ";
											}
											break;								
										}
									}
								}
							}
						}
					}					
				}
			}
		}
		if (organizationID!=null){
			return FdlUtil.getResourceOrganizationName(organizationID);
		}else{
			return null;
		}
		
	}
	
		
} //SANReusableProcessRuleImpl
