/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.ProgramRule;
import com.ibm.btools.te.ilm.heuristics.fdl.SANReusableTaskRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.Program;
import com.ibm.btools.fdl.model.ProgramActivity;

/**
 * <!-- begin-user-doc -->
 * Source: BOM InputPinSet (on a Reusable Task)
 * Target: FDL ProgramActivity
 * 
 * Maps a BOM Reusable Task to an FDL Process Activity.
 * Connects the mapped activity with the incoming and outgoing connectors.
 * 
 * Note: each InputPinSet of BOM Reusable Task is mapped to one FDL Process Activity.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class SANReusableTaskRuleImpl extends ConnectableRuleImpl implements SANReusableTaskRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected SANReusableTaskRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.SAN_REUSABLE_TASK_RULE;
	}

/**
 * Source: BOM InputPinSet (on a Reusable Task)
 * Target: FDL ProgramActivity
 * 
 * Maps a BOM Reusable Task to an FDL Process Activity.
 * Connects the mapped activity with the incoming and outgoing connectors.
 * 
 * Note: each InputPinSet of BOM Reusable Task is mapped to one FDL Process Activity.
 *  
 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		InputPinSet inputPinSet = (InputPinSet)getSource().get(0);
		if(inputPinSet.getOutputPinSet().size() > 1){
			String params[]={inputPinSet.getName(),inputPinSet.getAction().getName()};
			LoggingUtil.logError(MessageKeys.TASK_MULTIPLE_CORR_NOT_SUPPORTED,params,null);
		}
		OutputPinSet outputPinSet = (OutputPinSet)inputPinSet.getOutputPinSet().get(0);
		
		if(!(isFirstTimeExec())){
			reExecute(inputPinSet);
			LoggingUtil.traceExit(this, "transformSource2Target","re-execute");
			return true;	
		}

		//first time execute this rule
		setFirstTimeExec(false);
		DataStructure inputDs = createDataStructure(inputPinSet);
		DataStructure outputDs = createDataStructure(outputPinSet);

		ProgramActivity pa = createProgramActivity(inputDs, outputDs);
		//map InputPinSet to see if there is any activity created
		mapInputPinSet(inputPinSet, pa); 
	
		//do InputPinSet connection		
		setTargetOfConnectors(inputPinSet, pa);
		
		createBranchRuleIfAny(inputPinSet);
		
		mapOutputPinSet(outputPinSet, pa);
		//do OutputPinSet connection
		setSourceOfConnectors(outputPinSet, pa);

		// Ray: added support for descriptions and documentations		
		Action action = inputPinSet.getAction();
		Action refAction=action;
		if (action instanceof CallBehaviorAction) {
			refAction = ((Activity)((CallBehaviorAction)action).getBehavior()).getImplementation();
		}
		String desc = FdlUtil.transformDescription(refAction);
		if (desc!=null) {
			pa.setDescription(getNameRegistry().getDescription(desc));
		}
		String doc = FdlUtil.transformDocumentation(action);
		if (doc!=null) {
			pa.setDocumentation(getNameRegistry().getDocumentation(doc));
		}		

		//all targets propagate to parent
		propagateChildTargets();

		setComplete(true);
		
		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	} 
		
	/**
	 * Creates a ProgramActivity to represent the Reusable Task.
	 * Program will be re-used for the same CBA it references.
	 * @param inputDs
	 * @param outputDs
	 * @return ProgramActivity
	 */
	private ProgramActivity createProgramActivity(DataStructure inputDs, DataStructure outputDs){	

		ProgramActivity pa;
		
		InputPinSet inputPinSet = (InputPinSet)getSource().get(0);
		//TODO: find if tool will give different name for the reusable task.
		String paName = getNameRegistry().getActivityName(inputPinSet); 
		//use the cba's inputSet to get program name
		int index = inputPinSet.getAction().getInputPinSet().indexOf(inputPinSet); 
		Activity activity = (Activity)((CallBehaviorAction)inputPinSet.getAction()).getBehavior();
		InputPinSet topLevelSANInputPinSet = (InputPinSet)(activity.getImplementation().getInputPinSet().get(index));		
		String prmName = getNameRegistry().getProgramName(topLevelSANInputPinSet);
			
		pa = getFdlModelFactoryInstance().createProgramActivity();
		pa.setName(paName);
		pa.setProgramName(prmName);
		
		pa.setInputDataStructure(inputDs);
		pa.setOutputDataStructure(outputDs);		
		
//		String dsc = FdlUtil.getFullName(inputPinSet);
//		pa.setDescription(getNameRegistry().getDescription(dsc));
		
		//map input value pin if any to activity input container initialization
		List initialList = mapInputValuePin(inputPinSet, inputDs);
		if((initialList != null) && (!initialList.isEmpty())){
			pa.getInputContainerInitials().addAll(initialList);
		}

		//set pa as target of this rule
		getTarget().add(pa);
		//** cache pa for reExecute
		setTargetActivity(pa);

		//program rule will find if program already exist
		ProgramRule pr = FdlFactory.eINSTANCE.createProgramRule();
		pr.getSource().add(pa);
		getRoot().getChildRules().add(pr);
		pr.transformSource2Target();
		//override the prm description since it is reusable program, its description should not use activities.
//		dsc = FdlUtil.getFullName(topLevelSANInputPinSet);
//		Program prm = (Program)pr.getTarget().get(0);
//		prm.setDescription(getNameRegistry().getDescription(dsc));			
		
		return pa;
	}	
	
} //SANReusableTaskRuleImpl
