/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.blm.ie.imprt.rule.util.PredefinedTypeUtil;
import com.ibm.btools.bom.command.compound.PredefTypesConst;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.resources.BulkResourceRequirement;
import com.ibm.btools.bom.model.resources.IndividualResource;
import com.ibm.btools.bom.model.resources.IndividualResourceRequirement;
import com.ibm.btools.bom.model.resources.IndividualResourceType;
import com.ibm.btools.bom.model.resources.RequiredRole;
import com.ibm.btools.bom.model.resources.ResourceRequirement;
import com.ibm.btools.bom.model.resources.Role;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.PersonRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ProgramRule;
import com.ibm.btools.te.ilm.heuristics.fdl.RoleRule;
import com.ibm.btools.te.ilm.heuristics.fdl.SANTaskRule;
import com.ibm.btools.te.ilm.heuristics.fdl.StaffAssignmentRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.heuristics.helper.StaffUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.fdl.model.Activity;
import com.ibm.btools.fdl.model.BasicActivity;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.DoneBy;
import com.ibm.btools.fdl.model.ProgramActivity;
import com.ibm.btools.fdl.model.StartExitOption;
import com.ibm.btools.util.exception.BTRuntimeException;

/**
 * <!-- begin-user-doc -->
 * Source: BOM InputPinSet (on a Task)
 * Target: FDL ProgramActivity
 * 
 * Maps a BOM Task to a FDL ProgramActivity.
 * Connects the mapped activity with the incoming and outgoing connectors.
 * 
 * Note: Each InputPinSet of BOM Task is mapped to one FDL ProgramActivity.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class SANTaskRuleImpl extends ConnectableRuleImpl implements SANTaskRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected SANTaskRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.SAN_TASK_RULE;
	}

	/**
	 * Source: BOM InputPinSet (on a Task)
	 * Target: FDL ProgramActivity
	 * 
	 * Maps a BOM Task to a FDL ProgramActivity.
	 * Connects the mapped activity with the incoming and outgoing connectors.
	 * 
	 * Note: Each InputPinSet of BOM Task is mapped to one FDL ProgramActivity.
	 *  
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");
		
		InputPinSet inputPinSet = (InputPinSet)getSource().get(0);
		if(inputPinSet.getOutputPinSet().size() > 1){
			String params[]={inputPinSet.getName(),inputPinSet.getAction().getName()};
			LoggingUtil.logError(MessageKeys.TASK_MULTIPLE_CORR_NOT_SUPPORTED);
//			BTRuntimeException e = new BTRuntimeException(
//				TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TASK_MULTIPLE_CORR_NOT_SUPPORTED));							
//			throw e;
		}
		// Ray May 07,04 - Try to handle the case when inputPinSet has no correlated outputPinSet
//		OutputPinSet outputPinSet = (OutputPinSet)inputPinSet.getOutputPinSet().get(0);
		OutputPinSet outputPinSet = null;
		if (inputPinSet.getOutputPinSet().size()>0) {
			outputPinSet = (OutputPinSet)inputPinSet.getOutputPinSet().get(0);
		}
		
		if(!(isFirstTimeExec())){
			reExecute(inputPinSet);
			LoggingUtil.traceExit(this, "transformSource2Target", "re-execute");	
			
			executeHandlers();
					
			return true;	
		}

		//first time execute this rule
		setFirstTimeExec(false);
		DataStructure inputDs = createDataStructure(inputPinSet);
		DataStructure outputDs = createDataStructure(outputPinSet);

		Activity pa = createProgramActivity(inputDs, outputDs);
		//map InputPinSet to see if there is any activity created
		mapInputPinSet(inputPinSet, pa); 
	
		//do InputPinSet connection		
		setTargetOfConnectors(inputPinSet, pa);
		
		
		if (outputPinSet != null) { // Ray May 07,04
			createBranchRuleIfAny(inputPinSet);
			mapOutputPinSet(outputPinSet, pa);
			//do OutputPinSet connection
			setSourceOfConnectors(outputPinSet, pa);
		}
		
		// Ray: added support for descriptions and documentations		
		Action action = inputPinSet.getAction();
		String desc = FdlUtil.transformDescription(action);
		if (desc!=null) {
			pa.setDescription(getNameRegistry().getDescription(desc));
		}
		String doc = FdlUtil.transformDocumentation(action);
		if (doc!=null) {
			pa.setDocumentation(getNameRegistry().getDocumentation(doc));
		}
		//Check to see if any organization units
		//createOrganization(action);		
		
		//Check to see if any Resource Requirements
		createResource(action, pa);	
		
		//all targets propagate to parent
		propagateChildTargets();

		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");
		
		executeHandlers();
		
		return isComplete();
	}
	
/*
	private void createOrganization(Action action){
		if (action.getResponsibleOrganization()!=null && !action.getResponsibleOrganization().isEmpty()){
			List responsibleOrganizations = action.getResponsibleOrganization();
			Iterator iterator = responsibleOrganizations.iterator();
			while(iterator.hasNext()){		
				OrganizationRule orgRule = ModelFactory.eINSTANCE.createOrganizationRule();
				OrganizationUnit orgUnit = (OrganizationUnit)iterator.next();
				orgRule.getSource().add(orgUnit);
				getRoot().getChildRules().add(orgRule);
				orgRule.transformSource2Target();				
			}
		}		
	}
*/	


	/**
	 * Creates a Resource to represent the a Person or a Role or a Query
	 * @param action - a BOM action
	 * @param activity - a program activity
	 * @return
	 */
	private void createResource(Action action, Activity activity)
	{
		//FDL Roles
		ResourceRequirement candidateRequirement = null;
		if(StaffUtil.isStaffRequired(action)){
			List resourceRequirements = action.getResourceRequirement();			
			Iterator iterator = resourceRequirements.iterator();
			while(iterator.hasNext()){
				ResourceRequirement thisRequirement = (ResourceRequirement)iterator.next();
				//BulkResourceRequirement are not mapped to Staff queries.
				if(!(thisRequirement instanceof BulkResourceRequirement)){
					if(thisRequirement instanceof IndividualResourceRequirement){
						IndividualResourceRequirement individual = (IndividualResourceRequirement)thisRequirement;
						//IndividualResourcesRequirement with assigned IndividualResources of predefinedType si map to the Users by userID  
						if (individual.getIndividualResource()!= null && StaffUtil.isPredefinedStaff(individual.getIndividualResource().getClassifier())){
							//Create FDL Person
							//check to see if person already existed
							if (individual.getIndividualResource()!=null && !findPerson(individual.getIndividualResource())){
								PersonRule personRule = FdlFactory.eINSTANCE.createPersonRule();
								personRule.getSource().add(individual.getIndividualResource());
								getRoot().getChildRules().add(personRule);							
								personRule.transformSource2Target();
							}		
						}
					//Only RequiredRoles with predefined specified ResourceType are mapped to Staff Queries	
					}else if(thisRequirement instanceof RequiredRole){
						//check to see if role is already created
						if (((RequiredRole)thisRequirement).getResourceType()!=null && StaffUtil.isPredefinedStaff(((RequiredRole)thisRequirement).getResourceType())){
							if (((RequiredRole)thisRequirement).getRole()!=null && !findRole(((RequiredRole)thisRequirement).getRole())){
								RoleRule roleRule = FdlFactory.eINSTANCE.createRoleRule();
								roleRule.getSource().add(((RequiredRole)thisRequirement).getRole());
								getRoot().getChildRules().add(roleRule);
								roleRule.transformSource2Target();	
							}		
						}			
					}
					
					//if(thisRequirement.getOwnedConstraint()!= null && !thisRequirement.getOwnedConstraint().isEmpty()){
						//Call StaffAssignmentRule to map Queries
						StaffAssignmentRule staffRule = FdlFactory.eINSTANCE.createStaffAssignmentRule();
						staffRule.getSource().add(thisRequirement);	
						getRoot().getChildRules().add(staffRule);				
						staffRule.transformSource2Target();	
						//get target and save the doneBy object to activity
						if (!staffRule.getTarget().isEmpty() && activity instanceof BasicActivity){					
							((BasicActivity)activity).getActivityExtensionSetting().getDoneBy().add((DoneBy)staffRule.getTarget().get(0));
							((BasicActivity)activity).getActivityExtensionSetting().setStart(StartExitOption.MANUAL_LITERAL);
						}	
					//}
				
				}
				
			}				 
		}
	}
	
	private boolean findPerson(IndividualResource res){
		TransformationRule rule = TransformationUtil.getRuleForSource(getRoot(), PersonRule.class, res);
		if (rule==null)
			return false;
		else
			return true;
	}
	
	private boolean findRole(Role role){
		TransformationRule rule = TransformationUtil.getRuleForSource(getRoot(), RoleRule.class, role);
		if (rule==null)
			return false;
		else
			return true;
		
	}

	
	/**
	 * Creates a ProgramActivity to represent the Task.
	 * @param inputDs
	 * @param outputDs
	 * @return
	 */
	private Activity createProgramActivity(DataStructure inputDs, DataStructure outputDs){	

		ProgramActivity pa;
		
		InputPinSet inputSet = (InputPinSet)getSource().get(0);
		
		String paName = getNameRegistry().getActivityName(inputSet); 
		String prmName = getNameRegistry().getProgramName(inputSet);
	
		pa = getFdlModelFactoryInstance().createProgramActivity();
		pa.setName(paName);
		pa.setProgramName(prmName);
		
		pa.setInputDataStructure(inputDs);
		pa.setOutputDataStructure(outputDs);
		
		
		String dsc = FdlUtil.getFullName(inputSet);
		//pa.setDescription(getNameRegistry().getDescription(dsc));
		
//		map input value pin if any to activity input container initialization
		List initialList = mapInputValuePin(inputSet, inputDs);
		if((initialList != null) && (!initialList.isEmpty())){
			pa.getInputContainerInitials().addAll(initialList);
		}

		//set pa as target of this rule
		getTarget().add(pa);
		//** cache pa for reExecute
		setTargetActivity(pa);

		//create program
		ProgramRule pr = FdlFactory.eINSTANCE.createProgramRule();
		pr.getSource().add(pa);
		getRoot().getChildRules().add(pr);
		pr.transformSource2Target();	
		
		return pa;
	}
	
} //SANTaskRuleImpl
