/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.Repository;
import com.ibm.btools.bom.model.processes.activities.StoreArtifactPin;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.StoreDatastoreArtifactPinRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.fdl.model.ProgramActivity;

/**
 * <!-- begin-user-doc -->
 * Source: BOM StoreArtifactPin (referencing BOM Datastore only)
 * Target: BOM ProgramActivity (referencing store to datastore program)
 * 
 * Maps a StoreArtifactPin to a store to datastore ProgramActivity.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class StoreDatastoreArtifactPinRuleImpl extends StoreDatastoreArtifactRuleImpl implements StoreDatastoreArtifactPinRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected StoreDatastoreArtifactPinRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.STORE_DATASTORE_ARTIFACT_PIN_RULE;
	}

/**
 * Source: StoreArtifactPin (referencing Datastore)
 * Target: Program Activity (referencing store to datastore program)
 *  
 * Maps a StoreArtifactPin to a store to datastore program activity. 
 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		StoreArtifactPin pin = (StoreArtifactPin)getSource().get(0);
		if(!(isFirstTimeExec())){
			LoggingUtil.traceExit(this, "transformSource2Target","re-execute");		
				return true;	
			}

		//first time execute this rule
		setFirstTimeExec(false);
	
		ProgramActivity pa = createProgramActivity(pin);	

		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
		}
		
/**
 * Creates FDL ProgramActivity to call store to datastore program. 
 * Creates input container initilization by retrieving values from the pin's attributes.
 * 
 * @param pin The store artifact pin
 * @return ProgramActivity - The program activity created
 */

	private ProgramActivity createProgramActivity(StoreArtifactPin pin){
		
		Repository datastore = pin.getRepository();
		
		ValueSpecification lowerBound = pin.getLowerBound();
		if(lowerBound != null){
			 lowerBound = (ValueSpecification)EcoreUtil.copy((EObject)(lowerBound));
		}
		ValueSpecification upperBound = pin.getUpperBound();
		if(upperBound != null){
			upperBound = (ValueSpecification)EcoreUtil.copy((EObject)(upperBound));
		}
	
	//	String pinName =  FdlUtil.getCanonicalName(pin);
	//  String paName = getNameRegistry().getActivityName(pinName, "");
		String pinName = getNameRegistry().getActivityName(pin.getAction()) + "_" + pin.getName();	
	  	String paName = getNameRegistry().getActivityName(pinName, "");
	  	ProgramActivity pa = createStoreDatastoreProgramActivity(
	  		datastore, 
	  		paName,
	  		lowerBound,
	  		upperBound,
	  		pin.getIsInsert(),
	  		pin.getAtBeginning(),
	  		Boolean.TRUE //pin doesn't have overflow attribute
	  		 );
	  
	  
	  	String dsc = getNameRegistry().getDescription(pinName);
	  	pa.setDescription(getNameRegistry().getDescription(dsc));			
		
		getTarget().add(pa);
		return pa;


	}
	
} //StoreDatastoreArtifactPinRuleImpl
