/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.artifacts.ArtifactsFactory;
import com.ibm.btools.bom.model.artifacts.DataType;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.Repository;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.DataTypeRule;
import com.ibm.btools.te.ilm.heuristics.fdl.DatastoreRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.StoreDatastoreArtifactRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlConstants;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.fdl.model.ContainerInitial;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.MemberDeclaration;
import com.ibm.btools.fdl.model.Program;
import com.ibm.btools.fdl.model.ProgramActivity;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Store Datastore Artifact Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public abstract class StoreDatastoreArtifactRuleImpl extends ConnectableRuleImpl implements StoreDatastoreArtifactRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected StoreDatastoreArtifactRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.STORE_DATASTORE_ARTIFACT_RULE;
	}

	/**
	 * Creates the store datastore ProgramActivity.
	 * Initializes input container.
	 * @param datastore	The datastore
	 * @param actName	The name for the program activity to be created
	 * @param lowerBound The lowerBound of the object pin
	 * @param upperBound The upperBound of the object pin
	 * @param isInsert	The value of isInsert attribute for the input container
	 * @param atBeginning	The value of atBeginning attribute for the input container
	 * @param overflow		The value of overflow attribute for the input container  
	 * @return ProgramActivity - The store to datastore program activity created
	 */
	protected ProgramActivity createStoreDatastoreProgramActivity(
		Repository datastore,
		String actName,
		ValueSpecification lowerBound,
		ValueSpecification upperBound,
		Boolean isInsert,
		Boolean atBeginning,
		Boolean overflow){
	
		//	create datastore rule for the datastore
		if(datastore instanceof com.ibm.btools.bom.model.processes.activities.Variable){
			LoggingUtil.logWarning(MessageKeys.VARIABLE_REPOSITORY_NOT_SUPPORTED);
			//TODO: throw exception may cause test case not run
	//		BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.VARIABLE_REPOSITORY_NOT_SUPPORTED));							
	//		throw e;
		}

		DatastoreRule dsr = FdlFactory.eINSTANCE.createDatastoreRule();
		dsr.getSource().add(datastore);	
		getChildRules().add(dsr);
		dsr.transformSource2Target();		

		//we know the second target is the store database program
		Program prm = (Program)dsr.getTarget().get(1);
	
		//create input datastructure according to the pin multiplicity, assuming datstore program can handle any multiplicity	
		DataStructure inputDs = createInputDataStructure(datastore, lowerBound, upperBound);
		DataStructure outputDs = prm.getOutputDataStructure();
	
		//set program to use activity's structure 
		prm.setStructuresFromActivity(new Boolean(true));
		
		ProgramActivity pa = getFdlModelFactoryInstance().createProgramActivity();
		pa.setName(actName);
		pa.setProgramName(prm.getName());		
		pa.setInputDataStructure(inputDs);
		pa.setOutputDataStructure(outputDs);
	
		ContainerInitial ci = getFdlModelFactoryInstance().createContainerInitial(); 
		//get isInsert attribute, index = 1
		ci.setMemberName(((MemberDeclaration)inputDs.getMemberDeclarations().get(1)).getMemberName());
		String initValue = isInsert.booleanValue()? (FdlConstants.TRUE_INT_STRING) : (FdlConstants.FALSE_INT_STRING);
		ci.setInitialValue(initValue);
		pa.getInputContainerInitials().add(ci);
	  
		ci = getFdlModelFactoryInstance().createContainerInitial(); 
		//get atBeginning attribute, index = 2
		ci.setMemberName(((MemberDeclaration)inputDs.getMemberDeclarations().get(2)).getMemberName());
		initValue = atBeginning.booleanValue()?  (FdlConstants.TRUE_INT_STRING) : (FdlConstants.FALSE_INT_STRING);
		ci.setInitialValue(initValue);
		pa.getInputContainerInitials().add(ci);
	
		ci = getFdlModelFactoryInstance().createContainerInitial(); 
		//get overflow attribute, index = 3
		ci.setMemberName(((MemberDeclaration)inputDs.getMemberDeclarations().get(3)).getMemberName());
		initValue = overflow.booleanValue()?  (FdlConstants.TRUE_INT_STRING) : (FdlConstants.FALSE_INT_STRING);
		ci.setInitialValue(initValue);
		pa.getInputContainerInitials().add(ci);	

		//use upper bound for the length
		String length = FdlUtil.getValueSpecificationString(upperBound);
		if(length == null){
			length = "1";
		} 
		ci = getFdlModelFactoryInstance().createContainerInitial(); 
		//get length attribute, index = 4
		ci.setMemberName(((MemberDeclaration)inputDs.getMemberDeclarations().get(4)).getMemberName());
		ci.setInitialValue(length);
		pa.getInputContainerInitials().add(ci);			
			
		return pa;


	}

	/**
	 * Creats input data structure for the store datastore program activity
	 * @param datastore The datastore
	 * @param lowerBound The lower bound of the object pin
	 * @param upperBound The upper bound of the object pin
	 * @return DataStructure - The input data structure for the store datastore program activity 
	 */	
	private DataStructure createInputDataStructure(
		Repository datastore, 
		ValueSpecification lowerBound,
		ValueSpecification upperBound){
	
		//TODO: re-evaluate how to create primitive datatype
		// Ray: should create PrimitiveType instance instead
		//DataType booleanType = ArtifactsFactory.eINSTANCE.createDataType();
		DataType booleanType = ArtifactsFactory.eINSTANCE.createPrimitiveType();
		booleanType.setName("Boolean");
		//DataType intType = ArtifactsFactory.eINSTANCE.createDataType();
		DataType intType = ArtifactsFactory.eINSTANCE.createPrimitiveType();
		intType.setName("Integer");
	
		// create input datastructure for store program		
		// create a new BOM datatype to represent the input		

		Type type = datastore.getType();
		com.ibm.btools.bom.model.artifacts.Class newType = 
			ArtifactsFactory.eINSTANCE.createClass();
			
		String size = FdlUtil.getValueSpecificationString(upperBound);
		if(size == null){
			size = "1";
		}  
		newType.setName(datastore.getName() + "-" + size + FdlConstants.DATASTORE_STORE_IN_SUFFIX);
		
		//create the datastore type attribute with multiplicity value
		com.ibm.btools.bom.model.artifacts.Property	prop =  ArtifactsFactory.eINSTANCE.createProperty();
		prop.setName(FdlUtil.getLowerCase(type.getName()));
		prop.setType(type);
		prop.setLowerBound(lowerBound);
		prop.setUpperBound(upperBound);
		newType.getOwnedAttribute().add(prop);
		
		prop =  ArtifactsFactory.eINSTANCE.createProperty();
		prop.setName(FdlConstants.DATASTORE_IS_INSERT_NAME);		
		prop.setType(booleanType);
		newType.getOwnedAttribute().add(prop);
		
		prop =  ArtifactsFactory.eINSTANCE.createProperty();
		prop.setName(FdlConstants.DATASTORE_AT_BEGINNING_NAME);
		prop.setType(booleanType);		
		newType.getOwnedAttribute().add(prop);
	
		prop =  ArtifactsFactory.eINSTANCE.createProperty();
		prop.setName(FdlConstants.DATASTORE_OVERFLOW_NAME);
		prop.setType(booleanType);		
		newType.getOwnedAttribute().add(prop);	
		
		prop =  ArtifactsFactory.eINSTANCE.createProperty();
		prop.setName(FdlConstants.DATASTORE_LENGTH_NAME);
		prop.setType(intType);		
		newType.getOwnedAttribute().add(prop);			
				
		//input datastructure
		DataTypeRule dtr = FdlFactory.eINSTANCE.createDataTypeRule();
		dtr.getSource().add(newType);
		getRoot().getChildRules().add(dtr);
		dtr.transformSource2Target();
		DataStructure	inDs = (DataStructure)dtr.getTarget().get(0);
		  
		return inDs;	
	}	
	

} //StoreDatastoreArtifactRuleImpl
