package com.ibm.btools.te.ilm.heuristics.helper;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.text.DateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;

import javax.xml.namespace.QName;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.Part;
import org.eclipse.wst.wsdl.PortType;
import org.eclipse.wst.wsdl.WSDLFactory;
import org.eclipse.xsd.XSDComplexTypeDefinition;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDModelGroup;
import org.eclipse.xsd.XSDParticle;
import org.eclipse.xsd.XSDSimpleTypeDefinition;
import org.eclipse.xsd.XSDTypeDefinition;
import org.w3c.dom.Node;

import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.Element;
import com.ibm.btools.bom.model.artifacts.LiteralBoolean;
import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.LiteralNull;
import com.ibm.btools.bom.model.artifacts.LiteralSpecification;
import com.ibm.btools.bom.model.artifacts.LiteralString;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.MultiplicityElement;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.PrimitiveType;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.TypedElement;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.util.StringHelper;
import com.ibm.btools.util.datatype.BTDataType;
import com.ibm.btools.util.datatype.BTDataTypeException;
import com.ibm.btools.util.datatype.BTDataTypeManager;
import com.ibm.btools.util.datatype.IBTDataTypeRegistry;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Assign;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Copy;
import com.ibm.wbit.bpel.Flow;
import com.ibm.wbit.bpel.From;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.PartnerLink;
import com.ibm.wbit.bpel.Source;
import com.ibm.wbit.bpel.Target;
import com.ibm.wbit.bpel.To;
import com.ibm.wbit.bpel.Variable;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.Description;
import com.ibm.wbit.bpelpp.Documentation;
import com.ibm.wbit.bpelpp.JavaCode;

/**
 * This class contains methods that are generic and useful during creation of the 
 * Abstract BPEL representation of the process. 
 */
public class AbstractbpelUtil {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	//private final static String BPELPLUS_NAMESAPCE ="http://www.ibm.com/xmlns/prod/websphere/business-process/v5.1/";
	private final static String BPELPLUS_NAMESAPCE ="http://www.ibm.com/xmlns/prod/websphere/business-process/6.0.0/";
	public final static String BPEL_CORRELATION_INITIATE_YES = "yes";
	public final static String BPEL_CORRELATION_INITIATE_NO = "no";
	//public final static String BPEL_CORRELATION_INITIATE_RENDEZVOUS = "rendezvous";
	public final static String BPEL_CORRELATION_INITIATE_RENDEZVOUS = "join";
		
	/**
	 * Creates an empty <invoke> that has its partnerLink, portType and operation set to the 
	 * values required for use with a JavaScriptActivity.
	 * 
	 * @return Invoke - An <invoke> activity that is ready to be extended with a JavaScriptActivity.
	 */
	public static Invoke createJavaInvoke(){
		Invoke invokeActivity = BPELFactory.eINSTANCE.createInvoke();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		
		Operation javaOperation = WSDLFactory.eINSTANCE.createOperation();
		javaOperation.setName("null"); 
		invokeActivity.setOperation(javaOperation);
		
		PartnerLink javaPartnerLink = BPELFactory.eINSTANCE.createPartnerLink();
		javaPartnerLink.setName("null");
		invokeActivity.setPartnerLink(javaPartnerLink);
	
		PortType javaPortType = WSDLFactory.eINSTANCE.createPortType();
		//hack.. check BPEL plus editor
		//as per BPEL Plus editor, register set the Port namespace to the BPEL wpc namespace
		javaPortType.setQName(new QName(BPELPLUS_NAMESAPCE, "null"));
		//javaPortType.setQName(new QName("", "n-a"));

		invokeActivity.setPortType(javaPortType);
		
		
		return invokeActivity;		
	}
	
	/**
	 * Returns the given string with the first character capitalized.
	 * @param value The string to capitalize the first character of.
	 * @return String - The same as the <code>value</code> string, with the first character capitalized
	 */
	public static String capitalizeFirstChar(String value){
		return value.substring(0, 1).toUpperCase() + value.substring(1); 
	}
	
	/**
	 * Returns the given string with the first character lower case.
	 * @param value The string to lower case the first character.
	 * @return String - The same as the <code>value</code> string, with the first character lower case.
	 */
	public static String lowerCaseFirstChar(String value){
		return value.substring(0, 1).toLowerCase() + value.substring(1); 
	}
	
	/**
	 * Converts a given string to be a valid Java identifier string.  
	 * This is used to convert variable and part names into valid names so that the BPEL and BPEL+
	 * Expressions will work properly, since the names of the variables and parts
	 * are used in the getVariable...() method names, etc.
	 * 
	 * @param inString The string to convert
	 * @return String - The converted String that can be used as a valid Java identifier name 
	 */
	public static String convertString(String inString){	
		/* The specification for a valid java identifier name is as follows:
		 *	 identifier  =  "a..z,$,_"  <  "a..z,$,_,0..9,unicode character over 00C0" >
		 */

		StringBuffer outSB = new StringBuffer();
		
		for (int i=0, length = inString.length(); i<length; i++){
			char c = inString.toLowerCase().charAt(i);
			if ((c >= 'a' && c <= 'z') || c == '$' || c == '_' || (i>0 && (c >= '0' && c <= '9'))){
				/* This is a valid char, so do nothing */
				outSB.append(inString.charAt(i));
			} else {
				/* All invalid characters will be replaced with an underscore ('_') */
				outSB.append("_");
			}				
		}
		
		return outSB.toString();
	}
	
	/**
	 * This method extracts the literal value of LiteralSpecification 
	 * 
	 * @param litSpec The LiteralSpecification from which the value will be harvested
	 * @return Object - The value of the LiteralSpecification
	 */
	public static Object getLiteralValue(LiteralSpecification litSpec){
		if (litSpec instanceof LiteralUnlimitedNatural){
			return ((LiteralUnlimitedNatural)litSpec).getValue();

		} else if (litSpec instanceof LiteralString){
			return ((LiteralString)litSpec).getValue();

		} else if (litSpec instanceof LiteralNull){
			return "";

		} else if (litSpec instanceof LiteralInteger){
			return ((LiteralInteger)litSpec).getValue();

		} else if (litSpec instanceof LiteralBoolean){
			return ((LiteralBoolean)litSpec).getValue();
		}
		
		return "";
	}	
	
	/**
	 * This method extracts the literal value of LiteralSpecification 
	 * 
	 * @param litSpec The LiteralSpecification from which the value will be harvested
	 * @return Object - The value of the LiteralSpecification
	 */
	public static Object getLiteralValue(LiteralSpecification litSpec, Type objType){
		if (litSpec instanceof LiteralUnlimitedNatural){
			return ((LiteralUnlimitedNatural)litSpec).getValue();

		} else if (litSpec instanceof LiteralString){
			if(objType instanceof PrimitiveType){
				 if(IBTDataTypeRegistry.PREDEF_INTEGER.equals(objType.getName())){
					return new Integer(((LiteralString)litSpec).getValue());
				}else if(IBTDataTypeRegistry.PREDEF_SHORT.equals(objType.getName())){
					return new Short(((LiteralString)litSpec).getValue());
				}else if(IBTDataTypeRegistry.PREDEF_DOUBLE.equals(objType.getName())){
					return new Double(((LiteralString)litSpec).getValue());
				 }else if(IBTDataTypeRegistry.PREDEF_LONG.equals(objType.getName())){
					return new Long(((LiteralString)litSpec).getValue());
			 	}else if(IBTDataTypeRegistry.PREDEF_BYTE.equals(objType.getName())){
					return new Byte(((LiteralString)litSpec).getValue());
				 }else if(IBTDataTypeRegistry.PREDEF_FLOAT.equals(objType.getName())){
					return new Float(((LiteralString)litSpec).getValue());
				 }else if(IBTDataTypeRegistry.PREDEF_BOOLEAN.equals(objType.getName())){
					return new Boolean(((LiteralString)litSpec).getValue());
				 }else if(IBTDataTypeRegistry.PREDEF_DATE.equals(objType.getName()) ||IBTDataTypeRegistry.PREDEF_DATETIME.equals(objType.getName())|| IBTDataTypeRegistry.PREDEF_TIME.equals(objType.getName()) ){
					BTDataType type = BTDataTypeManager.instance.newInstance(objType.getName());
					try{
						type.setValue(((LiteralString)litSpec).getValue());
						return type.getMapValue(IBTDataTypeRegistry.MAPLANG_JAVA);
					}catch(BTDataTypeException e){
						
					}
				 }
			}
			//Duration and Strings are mapped to String
			return ((LiteralString)litSpec).getValue();

		} else if (litSpec instanceof LiteralNull){
			return "";

		} else if (litSpec instanceof LiteralInteger){
			return ((LiteralInteger)litSpec).getValue();

		} else if (litSpec instanceof LiteralBoolean){
			return ((LiteralBoolean)litSpec).getValue();
		}
		
		return "";
	}	
	
	/**
	 * Returns the string value that should be used as a parameter in a set(...) method in Java 
	 * code.  This encapsulates string values in "" characters, and leaves others unchanged.
	 * For example, myClass.setValue(1) is correct if 1 is an int, but myClass.setString("hello")
	 * is correct if the attribute is a string
	 * @param value The value to check the type of and return the correct literal assignment string
	 * @return String - The string to use in the assignment statement
	 */
	public static String getLiteralAssignmentString(Object value){
		if (value instanceof String){
			return "\"" + ((String)value) + "\"";
		} else if (value != null){
			return value.toString();
		} else {
			return "";
		}
	}	
	
	/**
	 * Returns the string value that should be used as a parameter in a set(...) method in Java 
	 * code.  This encapsulates string values in "" characters, and leaves others unchanged.
	 * For example, myClass.setValue(1) is correct if 1 is an int, but myClass.setString("hello")
	 * is correct if the attribute is a string. The String  will be build depending
	 * on the value type for example for a Short the method will return "new Short(value.toString()).shortValue())". 
	 * @param value The value to check the type of and return the correct literal assignment string
	 * @return String - The string to use in the assignment statement
	 */
	public static String getLiteralAssignmentString(Object value, Part destPart){
		if (value instanceof String){
			return "\"" + ((String)value) + "\"";
		}else if(value instanceof Short){
			return ("new Short(\"" + ((Short)value).toString()) + "\").shortValue()";
		}else if(value instanceof Integer){
			return ("new Integer(\"" + ((Integer)value).toString()) + "\").intValue()";
		}else if(value instanceof Double){
			return ("new Double(\"" + ((Double)value).toString()) + "\").doubleValue()";
		}else if(value instanceof Long){
			return ("new Long(\"" + ((Long)value).toString()) + "\").longValue()";
		}else if(value instanceof Byte){
			return ("new Byte(\"" + ((Byte)value).toString()) + "\").byteValue()";
		}else if(value instanceof Float){
			return ("new Float(\"" + ((Float)value).toString()) + "\").floatValue()";
		}else if(value instanceof Boolean){
			return ((Boolean)value).toString();
		}else if (value instanceof Date){
			//if the value is the date, will build the assignment based on the destination variable name
			//it is assumed that at this point the destination part can only be simple XSD types-> 
			// will build the assignment based as per the WSAD-IE requirements
			//dateTime -> mapped to Date
			//time and date -> mapped to Calendar (GregorianCalendar)
			Date date = (Date)value;
			if(destPart.getTypeDefinition().getName().equals("dateTime")){
				return ("new java.util.Date(new Long(\"" + ((Date)value).getTime()) + "\").longValue())";
			}else if(destPart.getTypeDefinition().getName().equals("date") || destPart.getTypeDefinition().getName().equals("time")){
				GregorianCalendar gc = new GregorianCalendar();
				gc.setTime(date);
				return "new java.util.GregorianCalendar(" + gc.get(Calendar.YEAR) +"," + gc.get(Calendar.MONTH) + "," + gc.get(Calendar.DATE) + "," + gc.get(Calendar.HOUR) + "," + gc.get(Calendar.MINUTE) + "," + gc.get(Calendar.SECOND)+")" ;
			}else{
				return "new java.util.GregorianCalendar()";
			}
			
		}else if (value != null){
			return value.toString();
		} else {
			return "";
		}

	}
	
	/**
	 * Returns the string value that should be used as a parameter in a set(...) method in Java 
	 * code.  This encapsulates string values in "" characters, and leaves others unchanged.
	 * For example, myClass.setValue(1) is correct if 1 is an int, but myClass.setString("hello")
	 * is correct if the attribute is a string. The String  will be build depending
	 * on the value type for example for a Short the method will return "new Short(value.toString()).shortValue())". 
	 * @param value The value to check the type of and return the correct literal assignment string
	 * @return String - The string to use in the assignment statement
	 */
	public static String getLiteralAssignmentString(Object value, BPELVariable variable){
		if (value instanceof String){
			return "\"" + ((String)value) + "\"";
		}else if(value instanceof Short){
			return ("new Short(\"" + ((Short)value).toString()) + "\")";
		}else if(value instanceof Double){
			return ("new Double(\"" + ((Double)value).toString()) + "\")";
		}else if(value instanceof Integer){
			return ("new Integer(\"" + ((Integer)value).toString()) + "\")";
		}else if(value instanceof Long){
			return ("new Long(\"" + ((Long)value).toString()) + "\")";
		}else if(value instanceof Byte){
			return ("new Byte(\"" + ((Byte)value).toString()) + "\")";
		}else if(value instanceof Float){
			return ("new Float(\"" + ((Float)value).toString()) + "\")";
		}else if(value instanceof Boolean){
		//defect 117086
			//return ((Boolean)value).toString();
			return ("new Boolean(\"" + ((Boolean)value).toString()) + "\")";
		}else if (value instanceof Date){
			//if the value is the date, will build the assignment based on the destination variable name
			//it is assumed that at this point the destination part can only be simple XSD types-> 
			// will build the assignment based as per the WSAD-IE requirements
			//dateTime -> mapped to Date
			//date -> mapped to Calendar (GregorianCalendar)
			//time  mapped to a String defect 118246
			Date date = (Date)value;
			if(variable.getType().getName().equals("dateTime")){
				return ("new java.util.Date(new Long(\"" + ((Date)value).getTime()) + "\").longValue())";
		//	}else if(variable.getType().getName().equals("date") || variable.getType().getName().equals("time")){
			}else if(variable.getType().getName().equals("date")){
				GregorianCalendar gc = new GregorianCalendar();
				gc.setTime(date);
				return "new java.util.GregorianCalendar(" + gc.get(Calendar.YEAR) +"," + gc.get(Calendar.MONTH) + "," + gc.get(Calendar.DATE) + "," + gc.get(Calendar.HOUR) + "," + gc.get(Calendar.MINUTE) + "," + gc.get(Calendar.SECOND)+")" ;
			//defect 118246
			}else if( variable.getType().getName().equals("time")){
				return "\"" + DateFormat.getTimeInstance().format((Date)value) + "\"";
			}else{
				return "new java.util.GregorianCalendar()";
			}
			
		}else if (value != null){
			return value.toString();
		} else {
			return "";
		}

	}	
	
	/**
	 * Returns the string value that should be used as a parameter in a set(...) method in Java 
	 * code.  This encapsulates string values in "" characters, and leaves others unchanged.
	 * For example, myClass.setValue(1) is correct if 1 is an int, but myClass.setString("hello")
	 * is correct if the attribute is a string. The String  will be build depending
	 * on the value type for example for a Short the method will return "new Short(value.toString()).shortValue())". 
	 * @param value The value to check the type of and return the correct literal assignment string
	 * @return String - The string to use in the assignment statement
	 */
	public static String getLiteralAssignmentObject(Object value, Part destPart){
		if (value instanceof String){
			return "\"" + ((String)value) + "\"";
		}else if(value instanceof Short){
			return ("new Short(\"" + ((Short)value).toString()) + "\")";
		}else if(value instanceof Double){
			return ("new Double(\"" + ((Double)value).toString()) + "\")";
		}else if(value instanceof Long){
			return ("new Long(\"" + ((Long)value).toString()) + "\")";
		}else if(value instanceof Byte){
			return ("new Byte(\"" + ((Byte)value).toString()) + "\")";
		}else if(value instanceof Float){
			return ("new Float(\"" + ((Float)value).toString()) + "\")";
		}else if(value instanceof Boolean){
			return ((Boolean)value).toString();
		}else if (value instanceof Date){
			//if the value is the date, will build the assignment based on the destination variable name
			//it is assumed that at this point the destination part can only be simple XSD types-> 
			// will build the assignment based as per the WSAD-IE requirements
			//dateTime -> mapped to Date
			//time and date -> mapped to Calendar (GregorianCalendar)
			Date date = (Date)value;
			if(destPart.getTypeDefinition().getName().equals("dateTime")){
				return ("new java.util.Date(new Long(\"" + ((Date)value).getTime()) + "\").longValue())";
			}else if(destPart.getTypeDefinition().getName().equals("date") || destPart.getTypeDefinition().getName().equals("time")){
				GregorianCalendar gc = new GregorianCalendar();
				gc.setTime(date);
				return "new java.util.GregorianCalendar(" + gc.get(Calendar.YEAR) +"," + gc.get(Calendar.MONTH) + "," + gc.get(Calendar.DATE) + "," + gc.get(Calendar.HOUR) + "," + gc.get(Calendar.MINUTE) + "," + gc.get(Calendar.SECOND)+")" ;
			}else{
				return "new java.util.GregorianCalendar()";
			}
			
		}else if (value != null){
			return value.toString();
		} else {
			return "";
		}

	}	
	
	public static String generateJavaGetterForVariable(String varName) {
		return generateJavaAccessorForVariable("get",varName);
	}

	public static String generateJavaSetterForVariable(String varName) {
		return generateJavaAccessorForVariable("set",varName);
	}
	
	public static String generateJavaSetterForVariableDiamond(String varName, String propertyName, XSDTypeDefinition typeDefinition, String value) {
		return generateSDOSetterForType(varName, propertyName, typeDefinition, value);
	}
	
	private static String generateJavaAccessorForVariable(String accessorString, String varName) {
		varName = capitalizeFirstChar(varName);
		if (varName.startsWith("PartnerLink") ||
			varName.startsWith("Correlationset") ||
			varName.startsWith("Variable")
		) {
			accessorString=accessorString+"Variable";
		}
		return accessorString+varName;
	}
	
	public static String getFullJavaClassName (QName qName) {
		String className=qName.getLocalPart();
		String packageName = qName.getNamespaceURI();
		//feature 109954
		//packageName = StringUtil.replaceFirst(packageName, ProcessUtil.DEFAULT_PREFIX, "");
		packageName = StringHelper.replaceFirst(packageName, ProcessUtil.DEFAULT_PREFIX, "");
		packageName = packageName.replace('/','.');
		return packageName+"."+className;
	}
	
	/**
	 * Returns the fully qualified name of the java class based on the <code>name</code> and <code>targetNamespace</code>
	 * @param qName
	 * @param targetNamespace
	 * @return Fully qualified Java name used for Java code generation.
	 */
	public static String getFullJavaClassName (String qName, String targetNamespace) {
		String className=qName;
		//String packageName = qName.getNamespaceURI();
		String packageName = targetNamespace;
		//feature 109954
	//	packageName = StringUtil.replaceFirst(packageName, ProcessUtil.DEFAULT_PREFIX, "");
		packageName = StringHelper.replaceFirst(packageName, ProcessUtil.DEFAULT_PREFIX, "");
		packageName = packageName.replace('/','.');
		return packageName+"."+className;
	}	
	
	// get the name with java package
	private static String getFullyQualifiedName(String name) {
		if (name == null) {
			return null;
		}

		if (name.equals("Date") || name.equals("DateTime")) {
			return "java.util.Date";
		} else if (name.equals("Int")) {
			return "Integer";
		} else if (name.equals("Time") || name.equals("Duration")) {
			return "String";
		} else if (name.equals("Integer")) {
			return "java.math.BigInteger";
		} else if (name.equals("AnySimpleType")) {
			return "commonj.sdo.DataObject";
		}
		
		return name;
	}
	
	public static Description transformDescription(Element element) {
		Comment desc = BomUtils.getDescription(element);
		if (desc!=null) {
			Description description = BPELPlusFactory.eINSTANCE.createDescription();
//JR24896 - limit description to 254 chars			
			String descrip = desc.getBody();
			if (descrip.length() > 254) {
				description.setValue(descrip.substring(0, 254));
//d160626				String name = null;
//				if (element instanceof NamedElement) {
//					name = ((NamedElement)element).getName();
//				}
//				String[] params = new String[1];
//				if (name == null) {
//					params[0] = element.getUid(); 
//				} else {
//					params[0] = name;
//				}
//				LoggingUtil.logWarning(MessageKeys.TRUNCATED_DESCRIPTION_WARNING, params, null);	
			} else {
				description.setValue(desc.getBody());
			}
			return description;
		}
		return null;
	}
	
	public static Documentation transformDocumentation(Element element) {
		List comments = BomUtils.getAnnotatedComments(element);
		if (comments.size() > 0) {
			StringBuffer sb = new StringBuffer();
			Documentation doc=BPELPlusFactory.eINSTANCE.createDocumentation();
			for (int i=0; i<comments.size(); i++) {
				Comment comment = (Comment)comments.get(i);
				sb.append(comment.getBody());
				if (i!=(comments.size()-1)) {
					sb.append("\r\n");
				}
			}
//GJD JR24896 - limit documentation to 4096 chars			
			if (sb.length() > 4096) {
				doc.setValue(sb.substring(0, 4096));
				String name = null;
				if (element instanceof NamedElement) {
					name = ((NamedElement)element).getName();
				}
				String[] params = new String[1];
				if (name == null) {
					params[0] = element.getUid(); 
				} else {
					params[0] = name;
				}
				LoggingUtil.logWarning(MessageKeys.TRUNCATED_ANNOTATION_WARNING, params, null);	
			} else {
				doc.setValue(sb.toString());
			}
			return doc;
		}		
		return null;
	}	

	public static Assign createSimpleAssign(Variable srcVariable,Part srcVariablePart,Variable destVariable,Part destVariablePart) {
		Assign assign = BPELFactory.eINSTANCE.createAssign();
		Copy copy = BPELFactory.eINSTANCE.createCopy();
		From from = BPELFactory.eINSTANCE.createFrom();
		To to = BPELFactory.eINSTANCE.createTo();
		 												
		from.setVariable(srcVariable);
		from.setPart(srcVariablePart);
			
		to.setVariable(destVariable);
		to.setPart(destVariablePart);
			
		copy.setFrom(from);
		copy.setTo(to);
		assign.getCopy().add(copy);		
		return assign;	
	}
	
	public static Assign createComplexAssign(Variable srcVariable,Part srcVariablePart,boolean isSrcAList, Variable destVariable,Part destVariablePart, boolean isDestAList) {
		// TODO: The correct code should be added in
		return createSimpleAssign(srcVariable,srcVariablePart,destVariable,destVariablePart);
	}
	
	/**
	 * Creates Java Snippets code to copy from 'fromPart' to 'toPart'.
	 * @param fromVariable
	 * @param fromPart
	 * @param isFromAList
	 * @param toVariable
	 * @param toPart
	 * @param isToAList
	 * @param toIndexInitialized - indicate whether the toIndex has been initialized or not.  If not, initialization code will be created
	 * @param toIndexName - name of the toIndex Java variable
	 * @param toIndex - in the case the isToAList is true, this is the index indicating where the copy copies the source to. 
	 * @param useFullGetterForDestination - whether we want to create a full Getter snippets to retrieve the toPart.  This is set to false in the
	 * case when the toPart has not previously been set and that some other Java code will have been created to instantiate the part first.
	 * See Merge and Join rule for this case.
	 * @return
	 */
	public static String createJavaCopySnippets(Variable fromVariable, Part fromPart, boolean isFromAList, Variable toVariable, Part toPart, boolean isToAList, boolean toIndexInitialized, String toIndexName, String toIndex, boolean useFullGetterForDestination) {
		String javaAssign = new String();
		String fromVarName = AbstractbpelUtil.capitalizeFirstChar(fromVariable.getName());
		String fromPartName = AbstractbpelUtil.capitalizeFirstChar(fromPart.getName());
		String toVarName = AbstractbpelUtil.capitalizeFirstChar(toVariable.getName());
		String toPartName = AbstractbpelUtil.capitalizeFirstChar(toPart.getName());
	//	String toElementName = toPart.getElement().getName();
		
		
		if (!isToAList && !isFromAList) {
			javaAssign += AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true)"
					   + ".set"	+ toPartName + "("
								   +  AbstractbpelUtil.generateJavaGetterForVariable(fromVarName) + "()"
								   + ".get" + fromPartName + "()"
					   + ");\n";		
//		if (!isToAList && !isFromAList) {
//			javaAssign += AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true)"
//					  + AbstractbpelUtil.generateJavaGetterForVariable(toPartName) + "()"
//					   + ".set"	+ toPartName + "("
//								   +  AbstractbpelUtil.generateJavaGetterForVariable(fromVarName) + "()"
//								   +  AbstractbpelUtil.generateJavaGetterForVariable(fromPartName) + "()"
//								   + ".get" + pinName + "()"
//					   + ");\n";				
		} else if (isToAList && isFromAList) {
			StringBuffer sb = new StringBuffer();
			if (!toIndexInitialized) {
			   sb.append("int "+toIndexName+"=");
			   sb.append(toIndex);
			   sb.append(";\n");
			   //lastIndexInitialized=true;
			}
			String inListPartGetterName = AbstractbpelUtil.generateJavaGetterForVariable(fromVarName)+"()"
				   + ".get" + fromPartName + "()" 
				   + ".get" + AbstractbpelUtil.capitalizeFirstChar(getNameOfListElement((XSDComplexTypeDefinition)fromPart.getTypeDefinition()));
			String toGetter = toPart.getName();
			if (useFullGetterForDestination) {
				toGetter= AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true).get"+toPartName+"()";
			}
			String outListPartSetterName = toGetter
				  + ".set" + AbstractbpelUtil.capitalizeFirstChar(getNameOfListElement((XSDComplexTypeDefinition)toPart.getTypeDefinition()));
			sb.append("for (int i=0; i<");
			sb.append(inListPartGetterName);
			sb.append("().length; i++) {\n\t");
			sb.append(outListPartSetterName);
			sb.append("("+toIndexName+"++,");
			sb.append(inListPartGetterName);
			sb.append("(i));\n}\n");
			javaAssign += sb.toString();
	   } else if (isToAList && !isFromAList) {
			String toGetter = toPart.getName();
			if (useFullGetterForDestination) {
				toGetter= AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true).get"+toPartName+"()";
			}
			javaAssign += toGetter
				   + ".set" + AbstractbpelUtil.capitalizeFirstChar(getNameOfListElement((XSDComplexTypeDefinition)toPart.getTypeDefinition()))+"("+toIndex+","
				   +  AbstractbpelUtil.generateJavaGetterForVariable(fromVarName) + "()"
				   + ".get" + fromPartName + "()"
				   + ");\n";
	   } else {
			String inListPartGetterName = AbstractbpelUtil.generateJavaGetterForVariable(fromVarName)+"()"
			   + ".get" + fromPartName + "()" 
			   + ".get" + AbstractbpelUtil.capitalizeFirstChar(getNameOfListElement((XSDComplexTypeDefinition)fromPart.getTypeDefinition()));
			javaAssign += AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true)"
				   + ".set"	+ toPartName + "("
				   + inListPartGetterName + "(0)"
				   + ");\n";					
	   }
		return javaAssign;
	}
	
	/**
	 * Creates Java Snippets code to copy from 'fromPart' to 'toPart'.
	 * @param fromVariable
	 * @param fromPart
	 * @param isFromAList
	 * @param toVariable
	 * @param toPart
	 * @param isToAList
	 * @param toIndexInitialized - indicate whether the toIndex has been initialized or not.  If not, initialization code will be created
	 * @param toIndexName - name of the toIndex Java variable
	 * @param toIndex - in the case the isToAList is true, this is the index indicating where the copy copies the source to. 
	 * @param useFullGetterForDestination - whether we want to create a full Getter snippets to retrieve the toPart.  This is set to false in the
	 * case when the toPart has not previously been set and that some other Java code will have been created to instantiate the part first.
	 * See Merge and Join rule for this case.
	 * @return
	 */
	public static String createJavaCopySnippets(Variable fromVariable, Part fromPart, boolean isFromAList, Variable toVariable, Part toPart, boolean isToAList, boolean toIndexInitialized, String toIndexName, String toIndex, boolean useFullGetterForDestination, int fromPinIndex, int toPinIndex) {
		String javaAssign = new String();
		String fromVarName = AbstractbpelUtil.capitalizeFirstChar(fromVariable.getName());
		String fromPartName = AbstractbpelUtil.capitalizeFirstChar(fromPart.getName());
		String toVarName = AbstractbpelUtil.capitalizeFirstChar(toVariable.getName());
		String toPartName = AbstractbpelUtil.capitalizeFirstChar(toPart.getName());
		//String toElementName = toPart.getElement().getName();
		String	fromPinElementName =null;
		String	toPinElementName = null;

		if (!isToAList && !isFromAList) {
			if(toPinIndex != -1){
				toPinElementName = getPinElementName(toPart, toPinIndex);
			}
			if(fromPinIndex != -1){
				fromPinElementName = getPinElementName(fromPart, fromPinIndex);
			}
			javaAssign += AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true)"
					  + "." + ((toPinIndex != -1)?
					   AbstractbpelUtil.generateJavaGetterForVariable(toPartName) + "()"
						+ ".set" + toPinElementName  :
						"set" + toPartName) 
								+ "(" +  AbstractbpelUtil.generateJavaGetterForVariable(fromVarName) + "()"
								   +  "." + AbstractbpelUtil.generateJavaGetterForVariable(fromPartName) + "()"
									+ ((fromPinIndex != -1)? ".get" + fromPinElementName+ "()": "")
					   + ");\n";				
			
		} else if (isToAList && isFromAList) {
			toPinElementName = getPinElementName(toPart, toPinIndex);
			fromPinElementName = getPinElementName(fromPart, fromPinIndex);
			StringBuffer sb = new StringBuffer();
			if (!toIndexInitialized) {
			   sb.append("int "+toIndexName+"=");
			   sb.append(toIndex);
			   sb.append(";\n");
			   //lastIndexInitialized=true;
			}
				String inListPartGetterName = AbstractbpelUtil.generateJavaGetterForVariable(fromVarName)+"()"
				   + ".get" + fromPartName + "()" 
				   + ".get" + AbstractbpelUtil.capitalizeFirstChar(fromPinElementName) 
				   + ((fromPinIndex != -1)? "().get" + AbstractbpelUtil.capitalizeFirstChar(getNameOfListElement((XSDComplexTypeDefinition)getPinTypDefinition(fromPart, fromPinIndex))): "");

			String toGetter = toPinElementName;
			if (useFullGetterForDestination) {
				toGetter= (toPinIndex != -1)? AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true).get"+toPartName+"()" + ".get(" + toPinElementName + ")":
				AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true).get"+toPartName+"()";
			}
			String outListPartSetterName = toGetter
				  //+ ".set" + AbstractbpelUtil.capitalizeFirstChar(getNameOfListElement((XSDComplexTypeDefinition)getPinTypDefinition(toPart, toPinIndex)));
			//	+ ".set" + AbstractbpelUtil.capitalizeFirstChar(toPinElementName);
				+ ((toPinIndex != -1)? ".set" + AbstractbpelUtil.capitalizeFirstChar(getNameOfListElement((XSDComplexTypeDefinition)getPinTypDefinition(toPart, toPinIndex))):
			".set" + AbstractbpelUtil.capitalizeFirstChar(getPinElementName(toPart, toPinIndex))) ;
			
			sb.append("for (int i=0; i<");
			sb.append(inListPartGetterName);
			sb.append("().length; i++) {\n\t");
			sb.append(outListPartSetterName);
			sb.append("("+toIndexName+"++,");
			sb.append(inListPartGetterName);
			sb.append("(i));\n}\n");
			javaAssign += sb.toString();
	   } else if (isToAList && !isFromAList) {
			//String toGetter = toPart.getName();
			toPinElementName = getPinElementName(toPart, toPinIndex);
			if(fromPinIndex != -1){
				fromPinElementName = getPinElementName(fromPart, fromPinIndex);
			}   	
	
			String toGetter = toPinElementName;
			//String toGetter = (toPinIndex != -1)? toPinElementName: toPartName;
			if (useFullGetterForDestination) {
//				toGetter= AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true).get"+toPartName+"()";
				toGetter= (toPinIndex != -1)? AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true).get"+toPartName+"()" + ".get(" + toPinElementName + ")":
				AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true).get"+toPartName+"()";

			}
			javaAssign +=toGetter
					+ ((toPinIndex != -1)? ".set" + AbstractbpelUtil.capitalizeFirstChar(getNameOfListElement((XSDComplexTypeDefinition)getPinTypDefinition(toPart, toPinIndex))):
						".set" + AbstractbpelUtil.capitalizeFirstChar(getPinElementName(toPart, toPinIndex))) 
						+ "("+toIndex+","
					+  AbstractbpelUtil.generateJavaGetterForVariable(fromVarName) + "()"
							+  "." + AbstractbpelUtil.generateJavaGetterForVariable(fromPartName) + "()"
										 + ((fromPinIndex != -1)? ".get" + fromPinElementName + "()": "")
						   + ");\n";

	   } else {
		//from a list, not to a list
			if(toPinIndex != -1){
				toPinElementName = getPinElementName(toPart, toPinIndex);
			}
			fromPinElementName = getPinElementName(fromPart, fromPinIndex);

	   	String inListPartGetterName = AbstractbpelUtil.generateJavaGetterForVariable(fromVarName)+"()"
			+  "." + AbstractbpelUtil.generateJavaGetterForVariable(fromPartName) 
			+ "().get" + AbstractbpelUtil.capitalizeFirstChar(fromPinElementName);


			javaAssign += AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true)"
			+ ((toPinIndex != -1)?  "." + AbstractbpelUtil.generateJavaGetterForVariable(toPartName) + "()"  + ".set" + toPinElementName : 
				".set" + toPartName)
			 + "("
				   + inListPartGetterName + "(0)"
				   + ");\n";										   		   
	   }
		return javaAssign;
	}
	

	/**
	 * Creates Java Snippets code to copy from 'fromPart' to 'toPart'.
	 * @param fromVariable
	 * @param fromPart
	 * @param isFromAList
	 * @param toVariable
	 * @param toPart
	 * @param isToAList
	 * @param toIndexInitialized - indicate whether the toIndex has been initialized or not.  If not, initialization code will be created
	 * @param toIndexName - name of the toIndex Java variable
	 * @param toIndex - in the case the isToAList is true, this is the index indicating where the copy copies the source to. 
	 * @param useFullGetterForDestination - whether we want to create a full Getter snippets to retrieve the toPart.  This is set to false in the
	 * case when the toPart has not previously been set and that some other Java code will have been created to instantiate the part first.
	 * See Merge and Join rule for this case.
	 * @return
	 */
	public static String createJavaCopySnippets(Variable fromVariable, Part fromPart, boolean isFromAList, Variable toVariable, Part toPart, boolean isToAList, boolean toIndexInitialized, String toIndexName, String toIndex, boolean useFullGetterForDestination, int fromPinIndex, int toPinIndex, boolean isPartInitialized) {
		
		String javaAssign = new String();
	
		if(! isPartInitialized){
			javaAssign += AbstractbpelUtil.createJavaVariableForPartCode(toVariable, toPart, true);
		}
		
		String fromVarName = AbstractbpelUtil.capitalizeFirstChar(fromVariable.getName());
		String fromPartName = AbstractbpelUtil.capitalizeFirstChar(fromPart.getName());
		String toVarName = AbstractbpelUtil.capitalizeFirstChar(toVariable.getName());
		String toPartName = AbstractbpelUtil.capitalizeFirstChar(toPart.getName());
		String	fromPinElementName = null;
//		if(fromPinIndex != -1){
//			fromPinElementName =getPinElementName(fromPart, fromPinIndex);	
//		}else{
//			fromPinElementName ="";
//		}
		 String toPinElementName = null;
//			if(toPinIndex != -1){
//				toPinElementName =getPinElementName(toPart, toPinIndex);	
//			}else{
//				toPinElementName ="";
//			}

    	if (!isToAList && !isFromAList) {
			if(toPinIndex != -1){
				toPinElementName =getPinElementName(toPart, toPinIndex);
				if(fromPinIndex != -1){
					fromPinElementName =getPinElementName(fromPart, fromPinIndex);
				}
				javaAssign += toPartName + ".set" + toPinElementName +"("+  AbstractbpelUtil.generateJavaGetterForVariable(fromVarName) + "()"
								+  "." + AbstractbpelUtil.generateJavaGetterForVariable(fromPartName) + "()"
									 + ((fromPinIndex != -1)? ".get" + fromPinElementName+ "()": "")
								+ ");\n";					
			}else{
				if(fromPinIndex != -1){
					fromPinElementName =getPinElementName(fromPart, fromPinIndex);
				}
				javaAssign += AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true)" + ".set" + toPartName 										 
							+ "(" +  AbstractbpelUtil.generateJavaGetterForVariable(fromVarName) + "()"
							   +  "." + AbstractbpelUtil.generateJavaGetterForVariable(fromPartName) + "()"
								+ ((fromPinIndex != -1)? ".get" + fromPinElementName+ "()": "")
				   + ");\n";		

}
				
//				javaAssign += AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true)"
//									  + "." +  AbstractbpelUtil.generateJavaGetterForVariable(toPartName) + "()"
//										+ ".set" + toPinElementName 										 
//												+ "(" +  AbstractbpelUtil.generateJavaGetterForVariable(fromVarName) + "()"
//												   +  "." + AbstractbpelUtil.generateJavaGetterForVariable(fromPartName) + "()"
//													+ ((fromPinIndex != -1)? ".get" + fromPinElementName+ "()": "")
//									   + ");\n";		
//				
//			}
//			javaAssign += AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true)"
//								  + "." + ((toPinIndex != -1)?
//								   AbstractbpelUtil.generateJavaGetterForVariable(toPartName) + "()"
//									+ ".set" + toPinElementName  :
//									"set" + toPartName) 
//											+ "(" +  AbstractbpelUtil.generateJavaGetterForVariable(fromVarName) + "()"
//											   +  "." + AbstractbpelUtil.generateJavaGetterForVariable(fromPartName) + "()"
//												+ ((fromPinIndex != -1)? ".get" + fromPinElementName+ "()": "")
//								   + ");\n";					
			
		} else if (isToAList && isFromAList) {
			fromPinElementName =getPinElementName(fromPart, fromPinIndex);
			toPinElementName =getPinElementName(toPart, toPinIndex);
			
			StringBuffer sb = new StringBuffer();
			if (!toIndexInitialized) {
			   sb.append("int "+toIndexName+"=");
			   sb.append(toIndex);
			   sb.append(";\n");
			   //lastIndexInitialized=true;
			}
			
				String inListPartGetterName = AbstractbpelUtil.generateJavaGetterForVariable(fromVarName)+"()"
				   + ".get" + fromPartName + "()" 
				   + ".get" + AbstractbpelUtil.capitalizeFirstChar(fromPinElementName) 
				   + ((fromPinIndex != -1)? "().get" + AbstractbpelUtil.capitalizeFirstChar(getNameOfListElement((XSDComplexTypeDefinition)getPinTypDefinition(fromPart, fromPinIndex))): "");

			String toGetter = toPinElementName;
			if (useFullGetterForDestination) {
//				toGetter= (toPinIndex != -1)? AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true).get"+toPartName+"()" + ".get(" + toPinElementName + ")":
//				AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true).get"+toPartName+"()";
				toGetter= (toPinIndex != -1)? toPartName+ ".get(" + toPinElementName + ")":
				AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true).get"+toPartName+"()";

			}
			String outListPartSetterName = toGetter
				  //+ ".set" + AbstractbpelUtil.capitalizeFirstChar(getNameOfListElement((XSDComplexTypeDefinition)getPinTypDefinition(toPart, toPinIndex)));
			//	+ ".set" + AbstractbpelUtil.capitalizeFirstChar(toPinElementName);
				+ ((toPinIndex != -1)? ".set" + AbstractbpelUtil.capitalizeFirstChar(getNameOfListElement((XSDComplexTypeDefinition)getPinTypDefinition(toPart, toPinIndex))):
			".set" + AbstractbpelUtil.capitalizeFirstChar(getPinElementName(toPart, toPinIndex))) ;
			
			sb.append("for (int i=0; i<");
			sb.append(inListPartGetterName);
			sb.append("().length; i++) {\n\t");
			sb.append(outListPartSetterName);
			sb.append("("+toIndexName+"++,");
			sb.append(inListPartGetterName);
			sb.append("(i));\n}\n");
			javaAssign += sb.toString();
	   } else if (isToAList && !isFromAList) {
			toPinElementName = getPinElementName(toPart, toPinIndex);
			if(fromPinIndex != -1){
				fromPinElementName = getPinElementName(fromPart, fromPinIndex);
			} 
	   	
			//String toGetter = toPart.getName();
			String toGetter = toPinElementName;
			//String toGetter = (toPinIndex != -1)? toPinElementName: toPartName;
			if (useFullGetterForDestination) {
//		
//				toGetter= (toPinIndex != -1)? AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true).get"+toPartName+"()" + ".get(" + toPinElementName + ")":
//				AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true).get"+toPartName+"()";
				toGetter= (toPinIndex != -1)? toPartName+"()" + ".get(" + toPinElementName + ")":
				AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true).get"+toPartName+"()";

			}
			javaAssign +=toGetter
					+ ((toPinIndex != -1)? ".set" + AbstractbpelUtil.capitalizeFirstChar(getNameOfListElement((XSDComplexTypeDefinition)getPinTypDefinition(toPart, toPinIndex))):
						".set" + AbstractbpelUtil.capitalizeFirstChar(getPinElementName(toPart, toPinIndex))) 
						+ "("+toIndex+","
					+  AbstractbpelUtil.generateJavaGetterForVariable(fromVarName) + "()"
							+  "." + AbstractbpelUtil.generateJavaGetterForVariable(fromPartName) + "()"
										 + ((fromPinIndex != -1)? ".get" + fromPinElementName + "()": "")
						   + ");\n";

	   } else {
//	  from a list, not to a list
			if(toPinIndex != -1){
				toPinElementName = getPinElementName(toPart, toPinIndex);
			}
			fromPinElementName = getPinElementName(fromPart, fromPinIndex);
		
			String inListPartGetterName = AbstractbpelUtil.generateJavaGetterForVariable(fromVarName)+"()"
			+  "." + AbstractbpelUtil.generateJavaGetterForVariable(fromPartName) 
			+ "().get" + AbstractbpelUtil.capitalizeFirstChar(fromPinElementName);


//			javaAssign += AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true)"
//			+ ((toPinIndex != -1)?  "." + AbstractbpelUtil.generateJavaGetterForVariable(toPartName) + "()"  + ".set" + toPinElementName : 
//				".set" + toPartName)
//			 + "("
//				   + inListPartGetterName + "(0)"
//				   + ");\n";		
			if(toPinIndex != -1){
				javaAssign += toPartName + ".set" + toPinElementName + "("
					   + inListPartGetterName + "(0)"
					   + ");\n";										   		   
				
			}else{
				javaAssign += AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true)"
				+ ((toPinIndex != -1)?  "." + AbstractbpelUtil.generateJavaGetterForVariable(toPartName) + "()"  + ".set" + toPinElementName : 
					".set" + toPartName)
				 + "("
					   + inListPartGetterName + "(0)"
					   + ");\n";										   		   
				
			}
	   }
		return javaAssign;
	}


	
	public static String createJavaCopySnippetsDiamond(Variable fromVariable, Part fromPart, boolean isFromAList, Variable toVariable, Part toPart, 
														boolean isToAList, boolean toIndexInitialized, String toIndexName, String toIndex,
														boolean useFullGetterForDestination, int fromPinIndex, int toPinIndex, boolean isPartInitialized) {
		
		String javaAssign = new String();
	
		if(! isPartInitialized){
			javaAssign += AbstractbpelUtil.createJavaVariableForPartCodeDiamond(toVariable, toPart, true, toPinIndex);
		}
		
		String fromVarName = AbstractbpelUtil.capitalizeFirstChar(fromVariable.getName());
		String fromPartName = AbstractbpelUtil.capitalizeFirstChar(fromPart.getName());
		String toVarName = AbstractbpelUtil.capitalizeFirstChar(toVariable.getName());
		String toPartName = AbstractbpelUtil.capitalizeFirstChar(toPart.getName());
		
		XSDTypeDefinition fromPinTypeDefinition = getPinTypDefinition(fromPart, fromPinIndex);
		XSDTypeDefinition toPinTypeDefinition = getPinTypDefinition(toPart, toPinIndex);
		
		if (!isToAList && !isFromAList) {
			if(toPinIndex != -1){
				
				String	toPinElementName = getPinElementName(toPart, toPinIndex);
				String value = null;
				if(fromPinIndex != -1){
					String	fromPinElementName =getPinElementName(fromPart, fromPinIndex);
					value= AbstractbpelUtil.generateSDOGetterForType(fromVarName, fromPinElementName, fromPinTypeDefinition);
				}else{
					value= fromVarName;
				}
				javaAssign += generateJavaSetterForVariableDiamond(toVarName, toPinElementName, toPinTypeDefinition, value);
			}else{
				String	fromPinElementName =getPinElementName(fromPart, fromPinIndex);
				//generate the setter for the variable.. if the variable type is a complex type we can do a simple assign 
				//var = (DataObject)get....
				//if the variable is a simple type (string) the assignment of a variable (DataObject) to a string will not work.. 
				// so we will use the setVariable(variableName, value) API 
				if(fromPinTypeDefinition instanceof XSDComplexTypeDefinition){
					javaAssign += toVarName + "=" + generateSDOGetterForType(fromVarName, fromPinElementName, fromPinTypeDefinition);	
				}else{
					javaAssign += "setVariable(" + toVarName + ", " + generateSDOGetterForType(fromVarName, fromPinElementName, fromPinTypeDefinition) + ");";
				}
				
			}
		} else if (isToAList && isFromAList) {
			
			String	fromPinElementName =getPinElementName(fromPart, fromPinIndex);
			String fromListElementName = getNameOfListElement((XSDComplexTypeDefinition)fromPinTypeDefinition);
			XSDTypeDefinition fromTypeDefinition = getPinTypDefinition(fromPart, -1);
			
			String pinElementName = getNameOfListElement((XSDComplexTypeDefinition)toPinTypeDefinition);
			String inListPartGetterName = null;
			if(fromPinIndex != -1){
				inListPartGetterName = generateSDOGetterForComplexType(fromVarName, fromPinElementName, fromTypeDefinition, fromListElementName, fromPinTypeDefinition);
	
			}else{
				inListPartGetterName = generateSDOGetterForType(fromVarName, fromPinElementName, fromTypeDefinition);
			}
			javaAssign += pinElementName + ".addAll(" + inListPartGetterName + ")";
	
	   } else if (isToAList && !isFromAList) {
	   		String	toPinElementName = getPinElementName(toPart, toPinIndex);
			String pinElementName = getNameOfListElement((XSDComplexTypeDefinition)toPinTypeDefinition);
			String toGetter = pinElementName;
	  		if (useFullGetterForDestination) {
				toGetter= (toPinIndex != -1)? toPartName+"()" + ".get(" + toPinElementName + ")":
				AbstractbpelUtil.generateJavaGetterForVariable(toVarName) + "(true).get"+toPartName+"()";
			}
	  		if(fromPinIndex != -1){
	  			String	fromPinElementName =getPinElementName(fromPart, fromPinIndex);
	  			javaAssign +=toGetter + ".add(" +toIndex+"," +
					AbstractbpelUtil.generateSDOGetterForType(fromVarName, fromPinElementName, fromPinTypeDefinition) + ")";	
	  		}else{
	  			//this is the case of a assign for LocalRepositories
	  			javaAssign +=toGetter + ".add(" +toIndex+"," + fromVarName + ")";
	  		}
	   } else {
	   		//from a list, not to a list
			if(toPinIndex != -1){
			   	String	fromPinElementName =getPinElementName(fromPart, fromPinIndex);
		   		fromPinTypeDefinition = getPinTypDefinition(fromPart, -1);
		   		String sdoType = getSDOType(toPinTypeDefinition);
		   		String inListPartGetterName = "(" + sdoType + ")" + AbstractbpelUtil.generateSDOGetterForType(fromVarName, fromPinElementName, fromPinTypeDefinition)+ ".get(0)";
				
		   		String	toPinElementName = getPinElementName(toPart, toPinIndex);
				javaAssign += generateSDOSetterForType(toVarName, toPinElementName, toPinTypeDefinition, inListPartGetterName);
				
			}else{
				String	fromPinElementName =getPinElementName(fromPart, fromPinIndex);
				XSDTypeDefinition fromTypeDefinition = getPinTypDefinition(fromPart, -1);
				if(fromPinIndex != -1){
					//String	fromPinElementName =getPinElementName(fromPart, fromPinIndex);
					String fromListElementName = getNameOfListElement((XSDComplexTypeDefinition)fromPinTypeDefinition);
				//	XSDTypeDefinition fromTypeDefinition = getPinTypDefinition(fromPart, -1);
				//	String sdoType = getSDOType(fromPinTypeDefinition);
					javaAssign += toVarName + "=(commonj.sdo.DataObject)"  + generateSDOGetterForComplexType(fromVarName, fromPinElementName, fromTypeDefinition, fromListElementName, fromPinTypeDefinition) + ".get(0)";
				}else{
					//fromPinIndex = -1
					String inListPartGetterName = generateSDOGetterForType(fromVarName, fromPinElementName, fromTypeDefinition);
					String sdoType = getSDOType(toPinTypeDefinition);
					inListPartGetterName = "(" + sdoType + ")" + AbstractbpelUtil.generateSDOGetterForType(fromVarName, fromPinElementName, fromPinTypeDefinition)+ ".get(0)";
					//javaAssign += toVarName + "=" + inListPartGetterName;
					//generate the setter for the variable.. if the variable type is a complex type we can do a simple assign 
					//var = (DataObject)get....
					//if the variable is a simple type (string) the assignment of a variable (DataObject) to a string will not work.. 
					// so we will use the setVariable(variableName, value) API 
					if(toPinTypeDefinition instanceof XSDComplexTypeDefinition){
						javaAssign += toVarName + "=" + inListPartGetterName;	
					}else{
						javaAssign += "setVariable(" + toVarName + ", " + inListPartGetterName + ")";
					}
				}
			}
	   }
		return javaAssign;
	}
	
	
	
	public static String createJavaCopySnippetsDiamond(BPELVariable fromVariable, boolean isFromAList, 	BPELVariable toVariable, boolean isToAList,
			boolean toIndexInitialized, String toIndexName, String toIndex,	boolean useFullGetterForDestination) {

		String javaAssign = new String();
		String fromVarName = fromVariable.getName();
		String toVarName = toVariable.getName();
        
		Object fromVarType = null;
		Object toVarType = null;
		//XSDTypeDefinition fromVarTypeDefinition = fromVariable.getType();
		fromVarType = fromVariable.getType();
		if(fromVarType == null){
			fromVarType = fromVariable.getXSDElement();
		}
		toVarType =  toVariable.getType();
		if(toVarType == null){
			toVarType = toVariable.getXSDElement();
		}
//		if(fromVarTypeDefinition == null){
//			XSDElementDeclaration fromVarXSDElement = fromVariable.getXSDElement();
//		}
//		XSDTypeDefinition toVarTypeDefinition = toVariable.getType();
//		if(toVarTypeDefinition == null){
//			XSDElementDeclaration toVarXSDElement = toVariable.getXSDElement();
//		}

		if (!isToAList && !isFromAList) {
			//do a simple assign as per Vlad example
			javaAssign += toVarName	+ "=" +	fromVarName;

		} else if (isToAList && isFromAList) {
			//we should generate something like this:
			//toVarName.getList("listElement").addAll(fromVar.getList("listElement"))			
			//if(fromVarType instanceof XSDElementDeclaration && toVarType instanceof XSDElementDeclaration){
			String	fromListElementName = getNameOfListElement(fromVarType );
			String	toListElementName = getNameOfListElement(toVarType);
			//}else{
			//	fromListElementName = getNameOfListElement((XSDComplexTypeDefinition)fromVarType );
			//	toListElementName = getNameOfListElement((XSDComplexTypeDefinition)toVarType);
			//}
			String inListPartGetterName = fromVarName  + ".getList(\"" + fromListElementName + "\")";
			
			javaAssign += toVarName  + ".getList(\"" + toListElementName + "\").addAll(" + toIndex + "," + inListPartGetterName	+ ")"; 

		} else if (isToAList && !isFromAList) {
			//in this case we should generate something like:
			//toVarName.getList("listElement").add(toIndex, fromVarName)
			String toListElementName  = null;
			if(toVarType instanceof XSDComplexTypeDefinition){
				toListElementName = getNameOfListElement((XSDComplexTypeDefinition) toVarType);
			}else if(toVarType instanceof XSDElementDeclaration){
				toListElementName = getNameOfListElement((XSDElementDeclaration) toVarType);
			}
			//this is the case of a assign for LocalRepositories
			javaAssign += toVarName  + ".getList(\"" + toListElementName + "\").add(" + toIndex + "," + fromVarName	+ ")";
		} else {
			//from a list, not to a list.. this should be an invalid case...
			//we should generate something like this:
			//toVarName = ((commonj.sdo.DataObject)fromVarName.getList("").get(0); if the toVar is a complexType
			//toVarName = ((String)fromVarName.getList("").get(0);. is the toVar is a simpleType
			String sdoType = getSDOType(toVarType);
			String fromListElementName = getNameOfListElement(fromVarType);
			javaAssign += toVarName  + "=(" + sdoType + ")" + fromVarName + ".getList(\"" + fromListElementName + "\").get(0)";
		}
		return javaAssign;
	}

	public static String getPinElementName(Part variablePart, int index){
		
		String elementName = null;
		XSDComplexTypeDefinition type = null;
		if(variablePart.getTypeDefinition() != null && variablePart.getElement()== null){
			return getNameOfListElement((XSDComplexTypeDefinition)variablePart.getTypeDefinition());		 
		}else {
			if(index == -1){
				return getNameOfListElement((XSDComplexTypeDefinition)variablePart.getElementDeclaration().getType());
			}
			type = (XSDComplexTypeDefinition)variablePart.getElementDeclaration().getTypeDefinition();
			//elementName = ((XSDElementDeclaration)((XSDParticle)((XSDModelGroup)type.getComplexType().getContent()).getContents().get(position)).getContent()).getType().getName();
			elementName = ((XSDElementDeclaration)((XSDParticle)((XSDModelGroup)type.getComplexType().getContent()).getContents().get(index)).getContent()).getName();

			return elementName;
		}
	//	return getNameOfListElement(variablePart, index);

	}

	//Ray
	public static boolean isTransformedMultiplicityElementAList(MultiplicityElement mElement,Part part) {
		boolean isList=BomUtils.isMultipled(mElement);
		if (isList) {
			XSDTypeDefinition partType = part.getTypeDefinition();
			//try to get the type of the part from the element declaration, in case the part is defined using the element declaration
			//rather than using the XSD Type
			if(partType == null){
				partType = part.getElementDeclaration().getTypeDefinition();
			}
			if (partType != null) {
				if (partType instanceof XSDComplexTypeDefinition) {
					// it may not be the safest to just check the name ...
					if (!partType.getName().startsWith("ListOf")) {
						isList = false;
					}
				} else {
					isList = false;
				}
			}
		}	
		return isList;	
	}
	
	//Ray
	public static boolean isTransformedMultiplicityElementAList(MultiplicityElement mElement,BPELVariable variable) {
		boolean isList=BomUtils.isMultipled(mElement);
		if (isList) {
			XSDTypeDefinition partType = variable.getType();
			//try to get the type of the part from the element declaration, in case the part is defined using the element declaration
			//rather than using the XSD Type
			if (partType != null) {
				if (partType instanceof XSDComplexTypeDefinition) {
					// it may not be the safest to just check the name ...
					if (!partType.getName().startsWith("ListOf")) {
						isList = false;
					}
				} else {
					isList = false;
				}
			}
		}
		return isList;	
	}
	
	//Ray
	public static boolean isTransformedMultiplicityElementAList(MultiplicityElement mElement,Part part, int positionInSet) {
		boolean isList=BomUtils.isMultipled(mElement);
		if (isList) {
			XSDTypeDefinition partType = part.getTypeDefinition();
			if(partType != null){
				return isTransformedMultiplicityElementAList(mElement, part);
			}
			else{
				//try to get the type of the part from the element declaration, in case the part is defined using the element declaration
				//rather than using the XSD Type
				partType = part.getElementDeclaration().getTypeDefinition();
			}
			if (partType != null) {
				if (partType instanceof XSDComplexTypeDefinition) {
					XSDComplexTypeDefinition complexDefinition = (XSDComplexTypeDefinition)partType;
					//the complexTypeDefinition can be the definition of the ComplexType we create for the PinSet or the definition of the 
					//single Pin in the PinSet
					//it is safe to assume that if the positionInSet = 0 and the TypeDefinition starts with "ListOf" this is a MultiplicityElement
					if(positionInSet == -1 && complexDefinition.getName().startsWith("ListOf")){
						isList = true;
					}
					//if (!getNameOfListElement(complexDefinition, positionInSet).startsWith("ListOf")) {
					else if (!getPinTypDefinition(part, positionInSet).getName().startsWith("ListOf")) {
						isList = false;
					}
				} else {
					isList = false;
				}
			}
		}	
		return isList;	
	}
	
	//Ray
	public static String getNameOfListElement (Object type) {
		String elementName = null;
		XSDElementDeclaration xsdElement = null; 
		if(type instanceof XSDComplexTypeDefinition) {
			XSDComplexTypeDefinition xsdType = (XSDComplexTypeDefinition)type;
			if (xsdType.getComplexType() == null) {
				XSDParticle xsdParticle = (XSDParticle)xsdType.getContent();
				xsdElement = (XSDElementDeclaration)((XSDParticle)((XSDModelGroup)xsdParticle.getContent()).getContents().get(0)).getContent();;
			} else {
				xsdElement = (XSDElementDeclaration)((XSDParticle)((XSDModelGroup)xsdType.getComplexType().getContent()).getContents().get(0)).getContent();
			}
			if (xsdElement.getName() != null) {
				elementName = xsdElement.getName();
			} else {
				xsdElement = xsdElement.getResolvedElementDeclaration();
				elementName = xsdElement.getName();
			}
		} else if(type instanceof XSDElementDeclaration){
			elementName = ((XSDElementDeclaration)type).getName();
		}
		return elementName;
	}	
	
//	Ray
//	  public static String getNameOfListElement (XSDComplexTypeDefinition type, int position) {
//	  	if(position == -1){
//	  		position = 0;
//	  	}
//		  String elementName = null;
//		 
//		//  elementName = ((XSDElementDeclaration)((XSDParticle)((XSDModelGroup)type.getComplexType().getContent()).getContents().get(position)).getContent()).getType().getName();
//		elementName = ((XSDElementDeclaration)((XSDParticle)((XSDModelGroup)type.getComplexType().getContent()).getContents().get(position)).getContent()).getName();
//		  
//		  return elementName;
//	  }	
	  
//	public static String getNameOfListElement (Part part, int position) {
//
//		String elementName = null;
//		XSDComplexTypeDefinition type = null;
//		if(part.getType() != null && part.getElement()== null){
//			return getNameOfListElement((XSDComplexTypeDefinition)part.getType());		 
//		}else {
//			if(position == -1){
//				return getNameOfListElement((XSDComplexTypeDefinition)part.getElement().getType());
//			}
//			type = (XSDComplexTypeDefinition)part.getElement().getTypeDefinition();
//			//elementName = ((XSDElementDeclaration)((XSDParticle)((XSDModelGroup)type.getComplexType().getContent()).getContents().get(position)).getContent()).getType().getName();
//			elementName = ((XSDElementDeclaration)((XSDParticle)((XSDModelGroup)type.getComplexType().getContent()).getContents().get(position)).getContent()).getName();
//
//			return elementName;
//		}
//	}	

	public static String createJavaVariableForPartCode(Variable variable,Part variablePart,boolean useTryCatch) {
		String javaStr = new String();
		String varName = AbstractbpelUtil.capitalizeFirstChar(variable.getName());
		String partName = AbstractbpelUtil.capitalizeFirstChar(variablePart.getName());
		String fullPartTypeName = null;
		if(variablePart.getTypeName() != null){
			fullPartTypeName = AbstractbpelUtil.getFullJavaClassName(variablePart.getTypeName());
		}else{
			fullPartTypeName = AbstractbpelUtil.getFullJavaClassName(variablePart.getElementDeclaration().getTypeDefinition().getQName(), variablePart.getElementDeclaration().getTypeDefinition().getTargetNamespace());
		}
		// Create 'Part' Java variable declaration
		javaStr += fullPartTypeName + " " + variablePart.getName()+";\n";
		if (useTryCatch) {
			javaStr += "try {\n";
			javaStr += "\t"+variablePart.getName()+ " = " + AbstractbpelUtil.generateJavaGetterForVariable(varName)+"(true).get"+partName + "();\n";				
			javaStr += "} catch (java.lang.IllegalArgumentException ex) {\n";
			javaStr += "\t"+variablePart.getName()+ " = new " + fullPartTypeName + "();\n";				
			javaStr += "}\n";
		} else {
			// instantiate the 'Part' Java variable
			javaStr += variablePart.getName()+ " = new " + fullPartTypeName + "();\n";				
		}
		return javaStr;
	}
	
	
	public static String createJavaVariableForPartCode(Variable variable,Part variablePart,boolean useTryCatch, int index) {
		String javaStr = new String();
		String varName = AbstractbpelUtil.capitalizeFirstChar(variable.getName());
		String partName = AbstractbpelUtil.capitalizeFirstChar(variablePart.getName());
//		if(index == -1 && variablePart.getType()!= null && variablePart.getElement()== null){
//			return createJavaVariableForPartCode(variable, variablePart, useTryCatch);
//			String fullPartTypeName = AbstractbpelUtil.getFullJavaClassName(variablePart.getTypeName());
//			javaStr += fullPartTypeName + " " + variablePart.getName()+";\n";
//
//		}else{
			//if(index == -1) index =0;
			XSDTypeDefinition pinPartType = null;
			pinPartType = getPinTypDefinition(variablePart, index);
			String fullPartTypeName = AbstractbpelUtil.getFullJavaClassName(pinPartType.getName(), pinPartType.getTargetNamespace());
		 
			String pinElementName = AbstractbpelUtil.getPinElementName(variablePart, index);
			javaStr += fullPartTypeName + " " + pinElementName +";\n";
			if (useTryCatch) {
				javaStr += "try {\n";
				javaStr += "\t"+pinElementName + " = " + AbstractbpelUtil.generateJavaGetterForVariable(varName)+"(true)." 
								+ AbstractbpelUtil.generateJavaGetterForVariable(partName) + "()"
								+ ((index != -1)? ".get" + pinElementName + "()": "")
								+ ";\n";				
				javaStr += "} catch (java.lang.IllegalArgumentException ex) {\n";
				javaStr += "\t" + pinElementName + " = new " + fullPartTypeName + "();\n";				
				javaStr += "}\n";
	
			} else {
				// instantiate the 'Part' Java variable
				//javaStr += variablePart.getName()+ " = new " + fullPartTypeName + "();\n";
				javaStr += pinElementName + " = new " + fullPartTypeName + "();\n";				
			}
			return javaStr;
	//	}
	}
	
	public static String initializeVariable(Variable variable, Part variablePart,  boolean wsiCompliant){
		String javaString = "";
		//String variableSDO = variable.getName() + "BG";
		String variableName = variable.getName();
		String variableNamespace= "";
		String variableTypeName = null;
		if(wsiCompliant){
			variableNamespace = variablePart.getElementDeclaration().getTypeDefinition().getTargetNamespace();
			variableTypeName = variablePart.getElementDeclaration().getTypeDefinition().getName();
		}else{
			//TODO chech for non wsi compliant
		}
		//TODO AR check what to if variable Namespace is Schema namespace ie. this is the case of a simple type -ex string... 
		//probably the SDO will be initiated differently.. build in SDO.. and how to set it??? 
		javaString += "if (" + variableName + " == null){" + "\n";
		javaString += "\t"+ "com.ibm.websphere.bo.BOFactory factory =  (com.ibm.websphere.bo.BOFactory) new com.ibm.websphere.sca.ServiceManager().locateService(\"com/ibm/websphere/bo/BOFactory\"); \n"; 
//		javaString += "\t"+ "com.ibm.websphere.jservice.ServiceManager serviceManager = new com.ibm.websphere.jservice.ServiceManager();\n";
//		javaString += "\t"+ "com.ibm.wbi.bo.BOFactory factory = (com.ibm.wbi.bo.BOFactory)serviceManager.locateService(\"com/ibm/wbi/bo/BOFactory\"); \n";
		javaString += "\t"+ variableName + " = factory.create( \"" +  variableNamespace + "\", \"" + variableTypeName + "\"); \n";
		javaString += "}\n";
			
		return javaString;
	}
	/**
	 * Initialize the BPEL Variable. If the variable type is a Primitive Type, do not do nothing, else, initialize using the BOFactory.
	 * @param variable a BPEL Variable
	 * @return String reprezenting the JavaCode to initialize the BPELVariable.
	 */
	
	public static String initializeVariable(BPELVariable variable){
		String javaString = "";
		String variableName = variable.getName();
		String variableNamespace= "";
		String variableTypeName = null;
		if (variable.getType() != null) {
			variableNamespace = variable.getType().getTargetNamespace();
			variableTypeName = variable.getType().getName();
		} else if (variable.getXSDElement() != null) {
			variableNamespace = variable.getXSDElement().getTargetNamespace();
			variableTypeName = variable.getXSDElement().getName();			
		}
	
		if(!(variable.getType() instanceof XSDSimpleTypeDefinition)){	
			javaString += "if (" + variableName + " == null){" + "\n";
			//AR 03May 2005
			javaString += "\t"+ "com.ibm.websphere.bo.BOFactory factory =  (com.ibm.websphere.bo.BOFactory) new com.ibm.websphere.sca.ServiceManager().locateService(\"com/ibm/websphere/bo/BOFactory\"); \n";
			javaString += "\t"+ variableName + " = factory.create( \"" +  variableNamespace + "\", \"" + variableTypeName + "\"); \n";
			javaString += "}\n";
		}			
		return javaString;
	}
	
	public static String createJavaVariableForPartCodeDiamond(Variable variable, Part variablePart, boolean alwaysNew, int index) {

		
		String javaStr = "";
		String varName = AbstractbpelUtil.capitalizeFirstChar(variable.getName());
		String partName = AbstractbpelUtil.capitalizeFirstChar(variablePart.getName());
		//This is the variable definition
		XSDTypeDefinition pinPartType = getPinTypDefinition(variablePart, index);
		if(pinPartType instanceof XSDSimpleTypeDefinition){
			//just return the simple empty string, we do not need to initialize any SDO
			return javaStr;
		}
					
		String fullPartTypeName = AbstractbpelUtil.getFullJavaClassName(pinPartType.getName(), pinPartType.getTargetNamespace());
		 
		String pinElementName = AbstractbpelUtil.getPinElementName(variablePart, index);
		javaStr += "commonj.sdo.DataObject " + pinElementName + ";\n";
		if (!alwaysNew) {
			javaStr += "if (" + varName + ".isSet(\"" + pinElementName + "\")){" + "\n";
			javaStr += "\t"+pinElementName + " = " + AbstractbpelUtil.generateSDOGetterForType(varName, pinElementName, pinPartType) + ";\n"; 
			javaStr += "} else {\n";
			javaStr += "\t"+pinElementName + " = " + AbstractbpelUtil.generateSDOCreateDataObject(varName, pinElementName)+  ";\n";				
			javaStr += "}\n";
		} else {
			// instantiate the 'Part' Java variable
			javaStr += pinElementName + " = " + AbstractbpelUtil.generateSDOCreateDataObject(varName, pinElementName)+  ";\n";				

		}
		return javaStr;
	//	}
	}
	

	
	public static String initializeJavaListPropertyDiamond(Variable variable, Part variablePart, boolean alwaysNew, int index) {

		
		String javaStr = "";
		String varName = AbstractbpelUtil.capitalizeFirstChar(variable.getName());
		//This is the variable definition
		XSDTypeDefinition pinPartType = getPinTypDefinition(variablePart, index);
		
		String listElement = AbstractbpelUtil.getNameOfListElement((XSDComplexTypeDefinition)pinPartType);
		String pinElementName = AbstractbpelUtil.getPinElementName(variablePart, index);

		if(pinPartType instanceof XSDSimpleTypeDefinition){
			//just return the simple empty string, we do not need to initialize any SDO
			return javaStr;
		}
		javaStr += "java.util.List " + listElement + ";\n";

		if(index == -1){
			if (!alwaysNew) {
				javaStr += "if (" + varName + ".isSet(\"" + pinElementName + "\")){" + "\n";
				javaStr += "\t"+listElement + " = " + AbstractbpelUtil.generateSDOGetterForType(varName, listElement, pinPartType) + ";\n"; 
				javaStr += "} else {\n";
				javaStr += "\t"+listElement + " = new java.util.LinkedList();\n";		
				javaStr += "}\n";
			} else {
				// instantiate the 'Part' Java variable
				javaStr += listElement + " = new java.util.LinkedList();\n";
			}
		}else{
			if (!alwaysNew) {
				javaStr += "if (" + varName + ".isSet(\"" + pinElementName + "\")){" + "\n";
				XSDTypeDefinition partType = getPinTypDefinition(variablePart, -1);
				javaStr += "\t"+listElement + " = " + AbstractbpelUtil.generateSDOGetterForComplexType(varName, pinElementName, partType, listElement, pinPartType) + ";\n";
				javaStr += "} else {\n";
				javaStr += "\t"+listElement + " = new java.util.LinkedList();\n";		
				javaStr += "}\n";
			} else {
				// instantiate the 'Part' Java variable
				javaStr += listElement + " = new java.util.LinkedList();\n";
			}
		}
		return javaStr;
	}
	
	public static String initializeJavaListPropertyDiamond(BPELVariable variable, boolean alwaysNew, int index) {

		
		String javaStr = "";
		String varName = AbstractbpelUtil.capitalizeFirstChar(variable.getName());
		//This is the variable definition
		//XSDTypeDefinition pinPartType = getPinTypDefinition(variablePart, index);
		XSDTypeDefinition pinPartType = variable.getType();
		
		String listElement = AbstractbpelUtil.getNameOfListElement((XSDComplexTypeDefinition)pinPartType);
		//String pinElementName = AbstractbpelUtil.getPinElementName(variablePart, index);

		if(pinPartType instanceof XSDSimpleTypeDefinition){
			//just return the simple empty string, we do not need to initialize any SDO
			return javaStr;
		}
		javaStr += "java.util.List " + listElement + ";\n";

		//if(index == -1){
			if (!alwaysNew) {
				javaStr += "if (" + varName + ".isSet(\"" + listElement + "\")){" + "\n";
				javaStr += "\t"+listElement + " = " + AbstractbpelUtil.generateSDOGetterForType(varName, listElement, pinPartType) + ";\n"; 
				javaStr += "} else {\n";
				javaStr += "\t"+listElement + " = new java.util.LinkedList();\n";		
				javaStr += "}\n";
			} else {
				// instantiate the 'Part' Java variable
				javaStr += listElement + " = new java.util.LinkedList();\n";
			}
//		}else{
//			if (!alwaysNew) {
//				javaStr += "if (" + varName + ".isSet(\"" + pinElementName + "\")){" + "\n";
//				XSDTypeDefinition partType = getPinTypDefinition(variablePart, -1);
//				javaStr += "\t"+listElement + " = " + AbstractbpelUtil.generateSDOGetterForComplexType(varName, pinElementName, partType, listElement, pinPartType) + ";\n";
//				javaStr += "} else {\n";
//				javaStr += "\t"+listElement + " = new java.util.LinkedList();\n";		
//				javaStr += "}\n";
//			} else {
//				// instantiate the 'Part' Java variable
//				javaStr += listElement + " = new java.util.LinkedList();\n";
//			}
//		}
		return javaStr;
	}
	
	public static String createJavaVariableForPartCode(Variable variable,Part variablePart,boolean useTryCatch, XSDElementDeclaration pinElementDeclaration) {
		String javaStr = new String();
		String varName = AbstractbpelUtil.capitalizeFirstChar(variable.getName());
		String partName = AbstractbpelUtil.capitalizeFirstChar(variablePart.getName());
		XSDTypeDefinition pinPartType = null;
		pinPartType = pinElementDeclaration.getTypeDefinition();
		String fullPartTypeName = AbstractbpelUtil.getFullJavaClassName(pinPartType.getName(), pinPartType.getTargetNamespace());
		 
		String pinElementName = pinElementDeclaration.getName();
			javaStr += fullPartTypeName + " " + pinElementName +";\n";
//			if (useTryCatch) {
//				javaStr += "try {\n";
//				javaStr += "\t"+pinElementName + " = " + AbstractbpelUtil.generateJavaGetterForVariable(varName)+"(true)." 
//								+ AbstractbpelUtil.generateJavaGetterForVariable(partName) + "()"
//								+ ((index != -1)? ".get" + pinElementName + "()": "")
//								+ ";\n";				
//				javaStr += "} catch (java.lang.IllegalArgumentException ex) {\n";
//				javaStr += "\t" + pinElementName + " = new " + fullPartTypeName + "();\n";				
//				javaStr += "}\n";
//	
//			} else {
				// instantiate the 'Part' Java variable
				//javaStr += variablePart.getName()+ " = new " + fullPartTypeName + "();\n";
				javaStr += pinElementName + " = new " + fullPartTypeName + "();\n";				
//			}
			return javaStr;
	//	}
	}
	
	
	public static XSDTypeDefinition getPinTypDefinition(Part variablePart, int index){
		XSDElementDeclaration pinPartDeclaration = null;
		if(index == -1){
			pinPartDeclaration = variablePart.getElementDeclaration();
			if(pinPartDeclaration == null){
				return variablePart.getTypeDefinition();
			}
		}else{
			pinPartDeclaration = ((XSDElementDeclaration)((XSDParticle)((XSDModelGroup)variablePart.getElementDeclaration().getTypeDefinition().getComplexType().getContent()).getContents().get(index)).getContent());	
		}
		
		return pinPartDeclaration.getTypeDefinition();
			
	}
	
	public static String createJavaPartSetterCode(Variable variable,Part variablePart,String sourceJavaFragment) {
		String varName = AbstractbpelUtil.capitalizeFirstChar(variable.getName());
		String partName = AbstractbpelUtil.capitalizeFirstChar(variablePart.getName());
		String javaCode = AbstractbpelUtil.generateJavaGetterForVariable(varName)+ 
			"(true).set"+partName+"("+sourceJavaFragment+");\n";
			return javaCode;
			
//			return 	AbstractbpelUtil.generateJavaGetterForVariable(varName)+ 
//			//"(true).set"+partName+"("+variablePart.getName()+");\n";
//			"(true).set"+partName+"("+sourceJavaFragment+");\n";

	}
	
	public static String createJavaPartSetterCode(Variable variable,Part variablePart,String sourceJavaFragment, boolean createNewPart) {
		String javaCode = new String();
		String varName = AbstractbpelUtil.capitalizeFirstChar(variable.getName());
		String partName = AbstractbpelUtil.capitalizeFirstChar(variablePart.getName());
			if(createNewPart){
				javaCode += createJavaVariableForPartCode(variable, variablePart, true);		
	
			}
				
			javaCode += AbstractbpelUtil.generateJavaGetterForVariable(varName)+ "(true).set" + partName + "("+sourceJavaFragment+");\n";
				return javaCode;
		}			
			

		public static boolean isLinkToExternalActivity(Link link, Flow flow) {
			Source src = null; 
			Target tar = null;
			if (link.getSources().size()>0) {
				src = (Source)link.getSources().get(0);
			}
			if (link.getTargets().size()>0) {
				tar = (Target)link.getTargets().get(0);
			}
			if (src!=null && tar!=null) {
				Activity srcAct=null, tarAct=null;
				srcAct = src.getActivity();
				tarAct = tar.getActivity();
				if (srcAct != null && tarAct !=null) {
					EObject container=srcAct.eContainer();
					while (container!=null && container!=flow) {
						container = container.eContainer();
					}
					if (container==null) {
						return true;
					}
					container=tarAct.eContainer();
					while (container!=null && container!=flow) {
						container = container.eContainer();
					}
					if (container==null) {
						return true;
					}
				} else {
					return true; // if the link has no src or target activity, it will be added later thus the link should be propagated
				}
			} else {
				return true; // if the link has no src or target activity, it will be added later thus the link should be propagated
			}
			return false;		
		}	

	
	public static String createJavaElementSetterCode(Variable variable,Part variablePart,String sourceJavaFragment, int pinIndex) {
		if(pinIndex == -1){
				return createJavaPartSetterCode(variable, variablePart, sourceJavaFragment);
		}
		XSDElementDeclaration toPartDeclaration = ((XSDElementDeclaration)((XSDParticle)((XSDModelGroup)variablePart.getElementDeclaration().getTypeDefinition().getComplexType().getContent()).getContents().get(pinIndex)).getContent());
		String varName = AbstractbpelUtil.capitalizeFirstChar(variable.getName());
		String partName = AbstractbpelUtil.capitalizeFirstChar(variablePart.getName());
		String javaCode = AbstractbpelUtil.generateJavaGetterForVariable(varName)+ "(true)."
						 + AbstractbpelUtil.generateJavaGetterForVariable(partName) + "()"
							+ ".set" + toPartDeclaration.getName() 						 
						 +	"("+sourceJavaFragment+");\n";
			
			return javaCode;
			
			
//		return 	AbstractbpelUtil.generateJavaGetterForVariable(varName)+ 
//				//"(true).set"+partName+"("+variablePart.getName()+");\n";
//				"(true).set"+partName+"("+sourceJavaFragment+");\n";
	}
	
	/**
	* Returns true if the MultiplicityElement has specified a multiplicity different than 1. 
	* @param type a BOM MultiplicityType
	* @return boolean - true if the multiplicity is different than 1
	*/		
	public static boolean isMultipled(MultiplicityElement type){
		// for the upperBound, the ValueSpecification can be either a LiteralUnlimitedNatural or a LiteralInteger		
		ValueSpecification value = type.getUpperBound();
			if(value instanceof LiteralUnlimitedNatural ){
				if(value == null || ((LiteralUnlimitedNatural)value).getValue()!= null && ((LiteralUnlimitedNatural)value).getValue().equals("1")){
					return false;	
				}else{
					return true;
				}
				
			}else if (value instanceof LiteralInteger){
					if(value == null ||((LiteralInteger)value).getValue()!= null && ((LiteralInteger)value).getValue().intValue() == 1){
						return false;		
					}else{
						return true;
					}
				
			}
		// for the lowerBound, the ValueSpecification can be a LiteralInteger
			ValueSpecification lowerBound = type.getLowerBound();
			if(lowerBound instanceof LiteralInteger){
				if(lowerBound == null || ((LiteralInteger)lowerBound).getValue()!= null && (((LiteralInteger)lowerBound).getValue().intValue() == 1 || ((LiteralInteger)lowerBound).getValue().intValue() == 0)){
					return false;
				}else{
					return true;
				}
			}
			return false;
	}	
	
	/** 
	 * Returns the name of the XSDElement name created for the <code>pin</code> in the typeDefinition of the <code>part</part>
	 * from the PinSet <code>set</code>
	 * @param set an Input or OutputPinSet
	 * @param pin an ObjectPin Input or OutputObjectPin
	 * @param part a WSDL Part
	 * @return
	 */
	public static String getPinElementName(NamedElement set, Pin pin, Part part){
		
			XSDElementDeclaration elementDeclaration = part.getElementDeclaration();
			XSDComplexTypeDefinition definition = (XSDComplexTypeDefinition)elementDeclaration.getTypeDefinition();
			XSDParticle particle = (XSDParticle)definition.getContent();
			XSDModelGroup modelGroup = (XSDModelGroup)particle.getContent();
			List elementDeclarations = modelGroup.getContents();
			List pins = getPinsFromSet(set);
			int ind = pins.indexOf(pin);
		
			XSDParticle pinsParticle = (XSDParticle)elementDeclarations.get(ind);
			XSDElementDeclaration  pinElementDeclaration = (XSDElementDeclaration) pinsParticle.getContent();
			return pinElementDeclaration.getName();
		}
		
	/**
	 * Returns the list of ObjectPins from the <code>set</code>
	 * @param set
	 * @return
	 */
	private static List getPinsFromSet(NamedElement set)
	{
		List pins;
		if (set instanceof InputPinSet)
			pins = ((InputPinSet)set).getInputObjectPin();
		else
			pins = ((OutputPinSet)set).getOutputObjectPin();
		return pins;		
	}	
	
	public static int getElementIndex(XSDComplexTypeDefinition definition, String tarPinElementName){
		XSDParticle particle = (XSDParticle)definition.getContent();
		XSDModelGroup modelGroup = (XSDModelGroup)particle.getContent();
		List elementDeclarations = modelGroup.getContents();
		if(elementDeclarations != null && !elementDeclarations.isEmpty()){
			for(int i=0; i< elementDeclarations.size(); i++){
				if(((XSDElementDeclaration)((XSDParticle)elementDeclarations.get(i)).getContent()).getName().equals(tarPinElementName)){
					return i;
				}
			}
		}
		return 0;
		
	}	
	
	public static String createSimpleAssign(String sourceFragment, String tarPart, Variable variable, Part part, String srcPinElementName ){
		
		String javaStr = sourceFragment + "." + AbstractbpelUtil.generateJavaSetterForVariable(tarPart)+ "(" +
							AbstractbpelUtil.generateJavaGetterForVariable(variable.getName()) +"()"
							+ "." + AbstractbpelUtil.generateJavaGetterForVariable(part.getName()) +"()" +
									((srcPinElementName != null && !srcPinElementName.equals(""))?
											"." + AbstractbpelUtil.generateJavaGetterForVariable(srcPinElementName) + "()" :"") + 
											");\n";
		return javaStr;								
		
		
		
	}
	
	public static String generateSDOCreateDataObject(String variableName, String propertyName){
		String sdoStr = "";
		sdoStr += variableName + ".createDataObject(\"" + propertyName +"\")";
		return sdoStr;
	}
	
	
	public static String generateSDOGetterForType(String variableName, String propertyName, XSDTypeDefinition typeDefinition){
		String sdoStr = "";
		if(typeDefinition instanceof XSDComplexTypeDefinition){
			if(isMultiplicityDefinition((XSDComplexTypeDefinition)typeDefinition)){
				sdoStr += variableName + ".getList(\"" + propertyName +"\")";
			}else{
				sdoStr += variableName + ".getDataObject(\"" + propertyName +"\")";
			}
		}else{
			String simpleTypeName = typeDefinition.getName();
			//Defect 111506
			if(simpleTypeName.equals("dateTime")){
				simpleTypeName = "date";
			}
			sdoStr += variableName + ".get" + capitalizeFirstChar(simpleTypeName) + "(\"" + propertyName +"\")";
		}
		return sdoStr;
	}
	
	/**
	 * Returns the SDO class type based on the XSDTypeDefinition. It will be either a the DataObject or the simple Java type. 
	 * @param fromPinTypeDefinition
	 * @return
	 */
	//private static String getSDOType(XSDTypeDefinition fromPinTypeDefinition){
	private static String getSDOType(Object fromPinTypeDefinition){
		String sdoType = "";
		//andyxz added to handle the xsd element 20060912
		if(fromPinTypeDefinition instanceof XSDComplexTypeDefinition || 
				fromPinTypeDefinition instanceof XSDElementDeclaration){
			sdoType += "commonj.sdo.DataObject";
		}else if(fromPinTypeDefinition instanceof XSDSimpleTypeDefinition){
			XSDSimpleTypeDefinition xsdSimpleType = (XSDSimpleTypeDefinition) fromPinTypeDefinition;
			if (xsdSimpleType.getTargetNamespace().equals(ExportOperation.PROCESS_SCHEMA_IMPORT)) {
				sdoType += getFullyQualifiedName(AbstractbpelUtil.capitalizeFirstChar(xsdSimpleType.getName()));
			} else {
				if (xsdSimpleType.getBaseType() != null) {	//according to the XSD model - should be set always
					sdoType += getFullyQualifiedName(AbstractbpelUtil.capitalizeFirstChar((xsdSimpleType.getBaseType()).getName()));					
				} else {
					sdoType += getFullyQualifiedName(AbstractbpelUtil.capitalizeFirstChar(xsdSimpleType.getName()));					
				}
			}
		}
		return sdoType;
	}
	
	
	public static String generateSDOGetterForComplexType(String variableName, String propertyName, XSDTypeDefinition typeDefinition, String propertyName1, XSDTypeDefinition typeDefinition1){
		
		String sdoStr = generateSDOGetterForType(variableName, propertyName, typeDefinition);
		if(typeDefinition1 instanceof XSDComplexTypeDefinition){
			if(isMultiplicityDefinition((XSDComplexTypeDefinition)typeDefinition)){
				sdoStr += ".getList(\"" + propertyName1 +"\")";
			}else{
				sdoStr += ".getDataObject(\"" + propertyName1 +"\")";
			}
		}else{
			String simpleTypeName = typeDefinition.getName();
			//Defect 111506
			if(simpleTypeName.equals("dateTime")){
				simpleTypeName = "date";
			}
			sdoStr += ".get" + capitalizeFirstChar(simpleTypeName) + "(\"" + propertyName1 +"\")";
		}
		return sdoStr;
	}

	public static String generateSDOSetterForType(String variableName, String propertyName, XSDTypeDefinition typeDefinition, String value){
		String sdoStr = "";
		if(typeDefinition instanceof XSDComplexTypeDefinition){
			if(isMultiplicityDefinition((XSDComplexTypeDefinition)typeDefinition)){
				sdoStr += variableName + ".setList(\"" + propertyName +"\", " + value +  ")";
			}else{
				sdoStr += variableName + ".setDataObject(\"" + propertyName +"\", " + value +  ")";	
			}
		}else{
			String simpleTypeName = typeDefinition.getName();
			//sdoStr += variableName + ".set"+ capitalizeFirstChar(simpleTypeName) + "(\"" + propertyName +"\", " + value + ")";
			sdoStr += variableName + "="+ value;
		}
		return sdoStr;
	}
	
	/**
	 * Returns true if the <code>typeDefinition</code> is the definition of a ListOf elements. 
	 * @param typeDefinition
	 * @return
	 */
	public static boolean isMultiplicityDefinition(XSDComplexTypeDefinition typeDefinition){
		
		if(typeDefinition.getTargetNamespace()!= null && typeDefinition.getTargetNamespace().equals("http://RootInformationModel")){
			return true;
		}
		
		if(typeDefinition.getName().startsWith("ListOf")){
			//retrieves the first element of the TypeDefinition and checks the multiplicity.. maxOccurs should be >1
			if(((XSDParticle)((XSDModelGroup)typeDefinition.getComplexType().getContent()).getContents().get(0)).getMaxOccurs()>1){
				return true;
			}
		}
		
		
		return false;
	}

	/**
	 * Check if the multiplicity (incl lower and upper bound) of the two pins are the same
	 * @param op1
	 * @param op2
	 * @return
	 */
	public static boolean isSameMultiplicity(MultiplicityElement op1, MultiplicityElement op2) {
		return (getMultiplicity(op1.getLowerBound()).equals(getMultiplicity(op2.getLowerBound())) &&
				getMultiplicity(op1.getUpperBound()).equals(getMultiplicity(op2.getUpperBound())) );
	}
	
	/**
	 * Convert the multiplicity ValueSpecification to a string
	 * @param mul
	 * @return
	 */
	public static String getMultiplicity(ValueSpecification mul)
	{
		if (mul != null)
		{
			if (mul instanceof LiteralUnlimitedNatural)
			{
				return ((LiteralUnlimitedNatural)mul).getValue();
			} else {
				return ((LiteralInteger)mul).getValue().toString();
			}
		} else {
			return "";
		}
	}	
	
	public static boolean isSameType(TypedElement op1, TypedElement op2) {
		// Ray: try to handle null type
		if (op1.getType()==null || op2.getType()==null) {
			return true;
		}
		return op1.getType().equals(op2.getType());
	}	
	
	public static boolean isModelerGeneratedDefintion(Definition wsdlDefinition){
		    if(wsdlDefinition == null){
		    	return false;
		    }
			List portTypes = wsdlDefinition.getEPortTypes();
			if(portTypes != null && !portTypes.isEmpty()){
				Iterator it = portTypes.iterator();
				while(it.hasNext()){
					PortType portType = (PortType)it.next();
					org.w3c.dom.Element element = portType.getElement();
					Node firstChild = element.getFirstChild();
					if(firstChild.getNodeValue().equals(AbstractbpelConstants.MODELER_VERSION_TAG)){
						return true;
					}
			  }
		}
		return false;
	}
}
