package com.ibm.btools.te.ilm.heuristics.helper;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.ExtensibilityElement;

import com.ibm.btools.bom.command.compound.PredefTypesConst;
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.Element;
import com.ibm.btools.bom.model.artifacts.InstanceSpecification;
import com.ibm.btools.bom.model.artifacts.InstanceValue;
import com.ibm.btools.bom.model.artifacts.LiteralBoolean;
import com.ibm.btools.bom.model.artifacts.LiteralDuration;
import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.LiteralString;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.MultiplicityElement;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.Package;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.artifacts.Slot;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.externalmodels.ExternalSchema;
import com.ibm.btools.bom.model.externalmodels.ExternalService;
import com.ibm.btools.bom.model.externalmodels.ServiceInterface;
import com.ibm.btools.bom.model.models.ProcessModel;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnit;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnitType;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.ActivityEdge;
import com.ibm.btools.bom.model.processes.activities.ConnectableNode;
import com.ibm.btools.bom.model.processes.activities.ControlNode;
import com.ibm.btools.bom.model.processes.activities.InitialNode;
import com.ibm.btools.bom.model.processes.activities.InputControlPin;
import com.ibm.btools.bom.model.processes.activities.InputDeliveryOptionKind;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.LoopNode;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputControlPin;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.activities.TerminationNode;
import com.ibm.btools.bom.model.processes.activities.Variable;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleAction;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleBlock;
import com.ibm.btools.bom.model.processes.humantasks.HumanTask;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.expression.model.Expression;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.attributes.helper.OperationTypeUtil;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.TransformationEngine;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ExpressionRule;
import com.ibm.btools.te.ilm.heuristics.bpel.BpelFactory;
import com.ibm.btools.te.ilm.heuristics.bpel.XPathExprSerializerRule;
import com.ibm.btools.te.ilm.heuristics.xsd.util.OriginalXsdUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.Documentation;

/**
 * This class defines methods that are generally useful for navigating the BOM model.
 */
public class BomUtils {
	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	/**
	 * Temporary class for testing purposes... until a global Constants file is defined
	 * for the tool
	 * TODO: Replace the usage of this Constants class with the real, tool-wide constants class
	 */
	private static class Constants {
		public static final String TASK_ASPECT = "TASK";
		public static final String PROCESS_ASPECT = "PROCESS";
		public static final String SERVICE_ASPECT = "SERVICE";
		public static final String SERVICE_OPERATION_ASPECT = "ServiceOperation";
		public static final String CLASSIFIER_ASPECT = PredefTypesConst.CATEGORY_TEMPLATE_VALUE_ASPECT;
		public static final String CLASSIFIER_VALUE_ASPECT = PredefTypesConst.CATEGORY_VALUE_INSTANCE_ASPECT;
		public static final String BUSINESSRULE_TASK_ASPECT = "BUSINESS_RULE_TASK";
	}
	
	/**
	 * This is a utility method that gets the list of input sets that are reachable from the given 
	 * output set by following each pin from the output set.
	 * 
	 * @param outputSet The staring place
	 * @return List - The list of input sets that are reachable from the given output set
	 */
	public static List getNextInputPinSets(OutputPinSet outputSet){
		List result = new ArrayList();
		List outputPins = new ArrayList(outputSet.getOutputObjectPin());
		outputPins.addAll(outputSet.getOutputControlPin());
		ActivityEdge edge = null;
		
		for (int j=0; j<outputPins.size(); j++){
			edge = ((Pin)outputPins.get(j)).getOutgoing();
			if (edge != null){
				/* The edge could be connected to not only a pin, but also a termination node */
				ConnectableNode cNode = edge.getTarget();
				if (cNode instanceof InputControlPin || cNode instanceof InputObjectPin){
					/* First remove all that already are in the list, so we don't get duplicates */
					result.removeAll(BomUtils.getInputPinSets((Pin)cNode));
					result.addAll(BomUtils.getInputPinSets((Pin)cNode));
				}
			}
		}
		return result;
	}	
	
	/**
	 * Returns the list of input sets in which the given pin belongs 
	 * @param pin - the pin in question.  Must be either an InputControlPin or InputObjectPin
	 * @return List - The list of InputPinSet objects in which the pin is a member
	 */
	public static List getInputPinSets(Pin pin){
		List results = new ArrayList();
		List inputSets = null;
		if (pin instanceof InputControlPin){
			inputSets = ((InputControlPin)pin).getAction().getInputPinSet();
		} else if (pin instanceof InputObjectPin){
			inputSets = ((InputObjectPin)pin).getAction().getInputPinSet();
		} 
		
		Iterator it = inputSets.iterator();
		InputPinSet is = null;
		while (it.hasNext()){
			is = (InputPinSet)it.next();
			if (is.getInputControlPin().contains(pin) || is.getInputObjectPin().contains(pin)){
				results.add(is);
			}
		}
		
		/* Check the attributes that are not supported by the TE */
		if (results.size() > 1){
			String actionName="";
			if (pin instanceof InputControlPin){
				actionName=((InputControlPin)pin).getAction().getName();
			} else if (pin instanceof InputObjectPin){
				actionName=((InputObjectPin)pin).getAction().getName();
			} 
			String params[]={pin.getName(),actionName};
			LoggingUtil.logWarning(MessageKeys.OVERLAPPING_PIN_SET_NOT_SUPPORTED,params,null);
		}		
		
		return results;		
	}
	
	/**
	 * Returns the list of output sets in which the given pin belongs 
	 * @param pin - the pin in question.  Must be either an OutputControlPin or OutputObjectPin
	 * @return List - The list of OutputPinSet objects in which the pin is a member
	 */
	public static List getOutputPinSets(Pin pin){
		List results = new ArrayList();
		List outputSets = null;
		if (pin instanceof OutputControlPin){
			outputSets = ((OutputControlPin)pin).getAction().getOutputPinSet();
		} else if (pin instanceof OutputObjectPin){
			outputSets = ((OutputObjectPin)pin).getAction().getOutputPinSet();
		} 
		
		Iterator it = outputSets.iterator();
		OutputPinSet os = null;
		while (it.hasNext()){
			os = (OutputPinSet)it.next();
			if (os.getOutputControlPin().contains(pin) || os.getOutputObjectPin().contains(pin)){
				results.add(os);
			}
		}
		
		/* Check the attributes that are not supported by the TE */
		if (results.size() > 1){
			String actionName="";
			if (pin instanceof OutputControlPin){
				actionName=((OutputControlPin)pin).getAction().getName();
			} else if (pin instanceof OutputObjectPin){
				actionName=((OutputObjectPin)pin).getAction().getName();
			} 		
			String params[]={pin.getName(),actionName};	
			LoggingUtil.logWarning(MessageKeys.OVERLAPPING_PIN_SET_NOT_SUPPORTED,params,null);
		}		
		
		return results;		
	}
	
	/*
	 * Finds the target set(input/output set) the edge leads to.
	 * Assume pin can only exist in one set.
	 */
	public static NamedElement findTargetSet(ActivityEdge edge){
		
		if (edge == null){
			return null;
		}
		ConnectableNode cNode = edge.getTarget();
		
		List result = null;
		if (cNode instanceof InputObjectPin || cNode instanceof InputControlPin){
			result = BomUtils.getInputPinSets((Pin)cNode);
		} else if (cNode instanceof OutputObjectPin || cNode instanceof OutputControlPin){
			result = BomUtils.getOutputPinSets((Pin)cNode);
		}
		
		/* If we allow a pin to belong to more than one set, this should return the whole list */
		if (result != null && !result.isEmpty()){
			return (NamedElement)result.get(0);
		}
		
		return null;
	}
	
	
	
	/*
	 * Finds the source set(input/output set) the edge coming from.
	 * Assume pin can only exist in one set.
	 */
	public static NamedElement findSourceSet(ActivityEdge edge){
		
		if (edge == null){
			return null;
		}
		
		ConnectableNode cNode = (ConnectableNode)edge.getSource();
		
		List result = null;
		if (cNode instanceof InputObjectPin || cNode instanceof InputControlPin){
			result = BomUtils.getInputPinSets((Pin)cNode);
		} else if (cNode instanceof OutputObjectPin || cNode instanceof OutputControlPin){
			result = BomUtils.getOutputPinSets((Pin)cNode);
		}
		
		/* If we allow a pin to belong to more than one set, this should return the whole list */
		if (result != null && !result.isEmpty()){
			return (NamedElement)result.get(0);
		}
		
		return null;
	}
	
	/**
	 * Returns the canonical name for the given NamedElement by prepending the name of
	 * the owner object up through the ownership hierarchy.  This name can be assumed
	 * to be unique within the given process only. Names are separated by '.'.  
	 * 
	 * Note: This version of the method does not include the containing Activity name
	 * nor the name of the containing top level SAN. 
	 * 
	 * Note also: This method is likely only useful to the TE, because it does not 
	 * guarantee that names are unique across processes since it does not include the name
	 * of the process model, or any parent process models, if nested. 
	 * 
	 * @param element The <code>NamedElement</code> to generate the name for
	 * @return String - The canonical name of <code>element</code>
	 */
	public static String getCanonicalName(NamedElement element){
		return getCanonicalName(element, false, false);
	}
	
	/**
	 * Returns the canonical name for the given NamedElement by prepending the name of
	 * the owner object up through the ownership hierarchy.  This name can be assumed
	 * to be unique within the given process only. 
	 * Names are separated by the <code>separator</code>.  
	 * 
	 * Note: This version of the method does not include the containing Activity name
	 * nor the name of the containing top level SAN. 
	 * 
	 * Note also: This method is likely only useful to the TE, because it does not 
	 * guarantee that names are unique across processes since it does not include the name
	 * of the process model, or any parent process models, if nested. 
	 * 
	 * @param element The <code>NamedElement</code> to generate the name for
	 * @param separator The separator string for names
	 * @return String - The canonical name of <code>element</code>
	 */
	public static String getCanonicalName(NamedElement element, String separator){
		return getCanonicalName(element, false, false, separator);
	}
	/**
	 * Returns the canonical name for the given NamedElement by prepending the name of
	 * the owner object up through the ownership hierarchy.  This name can be assumed
	 * to be unique within the given process only.
	 * 
	 * Note: This method is likely only useful to the TE, because it does not 
	 * guarantee that names are unique across processes since it does not include the name
	 * of the process model, or any parent process models, if nested. 
	 * 
	 * @param element The <code>NamedElement</code> to generate the name for
	 * @param includeActivityName Prepends the name of the containing activity, if true
	 * @param includeTopLevelSANName Prepends the name of the containing top level SAN, if true
	 * @param separator Separator string for names
	 * @return String - The canonical name of <code>element</code>
	 */
	private static String getCanonicalName(NamedElement element, boolean includeActivityName, boolean includeTopLevelSANName, String separator){
		StringBuffer name = new StringBuffer();
		name.append(element.getName());
		NamedElement container = (NamedElement)element.eContainer();
		
		while (container != null && !(container instanceof ProcessModel)){
			if (!includeActivityName && (container instanceof Activity)){
				// do nothing
				
			} else if (!includeTopLevelSANName 
					&& (container instanceof Action) 
					&& isTopLevelProcess((Action)container)){
				
				// do nothing
				
			} else {
				name.insert(0, container.getName() + separator);				
			}
			
			container = (NamedElement)container.eContainer();
		}
		
		return name.toString();
	}
	/**
	 * Returns the canonical name for the given NamedElement by prepending the name of
	 * the owner object up through the ownership hierarchy.  This name can be assumed
	 * to be unique within the given process only. Names are separated by '.'. 
	 * 
	 * Note: This method is likely only useful to the TE, because it does not 
	 * guarantee that names are unique across processes since it does not include the name
	 * of the process model, or any parent process models, if nested. 
	 * 
	 * @param element The <code>NamedElement</code> to generate the name for
	 * @param includeActivityName Prepends the name of the containing activity, if true
	 * @param includeTopLevelSANName Prepends the name of the containing top level SAN, if true
	 * @return String - The canonical name of <code>element</code>
	 */
	public static String getCanonicalName(NamedElement element, boolean includeActivityName, boolean includeTopLevelSANName){
		StringBuffer name = new StringBuffer();
		if(element.getName() != null){
			name.append(element.getName());
		}
	//	NamedElement container = (NamedElement)element.eContainer();
		EObject container = (EObject)element.eContainer();
		
		while (container != null && !(container instanceof ProcessModel)){
			if (!includeActivityName && (container instanceof Activity)){
				// do nothing
				
			} else if (!includeTopLevelSANName 
					&& (container instanceof Action) 
					&& isTopLevelProcess((Action)container)){
				
				// do nothing
				
			} else {
				if(container instanceof NamedElement ){
					if(name.length()>0){
						name.insert(0, ((NamedElement)container).getName() + ".");	
					}
					else{
						name.insert(0, ((NamedElement)container).getName());
					}
				}
			}
			
			//container = (NamedElement)container.eContainer();
			container = container.eContainer();
		}
		return name.toString();
	}
	
	/*
	 * Returns true if the action is a Process
	 */
	public static boolean isProcess(Action action)
	{
		if (action instanceof StructuredActivityNode){
			//			// TODO: Should use the 'aspect' flag to find out if the action is a process
			//			return ((StructuredActivityNode)action).getNodeContents().size()>0;
			return !isService(action) && !isTask(action) && !(action instanceof LoopNode) && 
						!(isServiceOperation(action)) && !(isHumanTask(action)) && !(isBusinessRuleTask(action));
		}
		
		return false;
	}
	
	/**
	 * Returns true if the input action is a top level StructuredActivityNode
	 * 
	 * @param action The action to check
	 * @return boolean - True if <code>action</code> is a top level SAN
	 */
	public static boolean isTopLevelProcess(Action action){
		
		if (action.eContainer() instanceof Activity){
			return true;
		}
		return false;
	}
	
	/**
	 * Returns true if the given action is a local process
	 * @param action The action to check 
	 * @return boolean - True if <code>action</code> is a local process, false otherwise
	 */
	public static boolean isLocalProcess(Action action){		
		return (action instanceof StructuredActivityNode) && (Constants.PROCESS_ASPECT.equalsIgnoreCase(action.getAspect()) &&
				action.getInStructuredNode() != null);
	}
	
	/**
	 * Returns true if the given action is a reusable task
	 * 
	 * @param action The action to check 
	 * @return boolean - True if <code>action</code> is a reusable task, false otherwise
	 */
	public static boolean isReusableTask(Action action){
		if(action instanceof CallBehaviorAction){			 
			String aspect = ((Activity)((CallBehaviorAction)action).getBehavior()).getImplementation().getAspect();
			if(aspect != null && Constants.TASK_ASPECT.equalsIgnoreCase(aspect)){
				return true;
			}
		}else if(action.getInStructuredNode() == null && action.eContainer() instanceof Activity && ((Activity)action.eContainer()).getImplementation()== action){
			String aspect = ((Activity)action.eContainer()).getImplementation().getAspect();
			if(aspect != null && Constants.TASK_ASPECT.equalsIgnoreCase(aspect)){
				return true;
			}
		}
		return false;
	}
	
	public static boolean isReusableHumanTask(Action action){
		Action htAction = null;
		if(action instanceof CallBehaviorAction){			 
			htAction = ((Activity)((CallBehaviorAction)action).getBehavior()).getImplementation();
		}else if(action.getInStructuredNode() == null && action.eContainer() instanceof Activity && ((Activity)action.eContainer()).getImplementation()== action){
			htAction = ((Activity)action.eContainer()).getImplementation();
		}
		if(htAction!=null && isHumanTask(htAction)){
			return true;
		}

		return false;
	}

	
	/**
	 * Returns true if the given action is a reusable br task
	 * @param action The action to check 
	 * @return boolean - True if <code>action</code> is a reusable br task, false otherwise
	 */
	public static boolean isReusableBusinessRuleTask(Action action){
		Action brAction = null;
		if(action instanceof CallBehaviorAction){			 
			brAction = ((Activity)((CallBehaviorAction)action).getBehavior()).getImplementation();
		}else if(action.getInStructuredNode() == null && action.eContainer() instanceof Activity && ((Activity)action.eContainer()).getImplementation() == action){
			brAction = ((Activity)action.eContainer()).getImplementation();
		}
		if(brAction != null && isBusinessRuleTask(brAction)){
			return true;
		}

		return false;
	}

	/**
	 * Returns true if the given action is a reusable service
	 * 
	 * @param action The action to check 
	 * @return boolean - True if <code>action</code> is a reusable service, false otherwise
	 */
	public static boolean isReusableService(Action action){
		if(action instanceof CallBehaviorAction){			 
			String aspect = ((Activity)((CallBehaviorAction)action).getBehavior()).getImplementation().getAspect();
			if(aspect != null && Constants.SERVICE_ASPECT.equalsIgnoreCase(aspect)){
				return true;
			}
		}else if(action.getInStructuredNode() == null && action.eContainer() instanceof Activity && ((Activity)action.eContainer()).getImplementation()== action){
			String aspect = ((Activity)action.eContainer()).getImplementation().getAspect();
			if(aspect != null && Constants.SERVICE_ASPECT.equalsIgnoreCase(aspect)){
				return true;
			}
		}	
		return false;
	}
	
	/**
	 * Returns true if the given action is a task
	 * @param action The action to check 
	 * @return boolean - True if <code>action</code> is a task, false otherwise
	 */
	public static boolean isTask(Action action){		
		return (action instanceof StructuredActivityNode) && (Constants.TASK_ASPECT.equalsIgnoreCase(action.getAspect()));
	}
	
	/**
	 * Returns true is the given action is a business rule task
	 * @param action
	 * @return
	 */
	public static boolean isBusinessRuleTask(Action action) {
		if (action instanceof BusinessRuleAction || Constants.BUSINESSRULE_TASK_ASPECT.equalsIgnoreCase(action.getAspect())) {
			return true;
		}
		return false;
	}
	

	/**
	 * Returns true is the given action is a human task
	 * @param action
	 * @return
	 */
	public static boolean isHumanTask(Action action) {
		if (action instanceof HumanTask || (action.getAspect() != null &&  action.getAspect().equals("HUMAN_TASK"))) { // TODO: shouldn't need to check the aspect
			return true;
		}
		return false;
	}
	
	/**
	 * Returns true if the given action is a service
	 * @param action The action to check 
	 * @return boolean - True if <code>action</code> is a service, false otherwise
	 */
	public static boolean isService(Action action){
		return Constants.SERVICE_ASPECT.equalsIgnoreCase(action.getAspect());
	}
	
	public static boolean isServiceOperation(Action action){
		return action != null && 
		Constants.SERVICE_OPERATION_ASPECT.equals(action.getAspect());
	}
	
	/**
	 * return true if the namedElement is a classifier
	 * @param element
	 * @return
	 */
	public static boolean isClassifier(NamedElement element) {
		if (element.getAspect()!=null && element instanceof OrganizationUnitType && element.getAspect().equalsIgnoreCase(Constants.CLASSIFIER_ASPECT)) {
			return true;
		}else if (element.getAspect() != null && element instanceof OrganizationUnit && element.getAspect().equalsIgnoreCase(Constants.CLASSIFIER_VALUE_ASPECT)){
			return true;
		}
		return false;
	}	
	
	/**
	 * Returns true if the MultiplicityElement has specified a multiplicity different than 1. 
	 * @param type a BOM MultiplicityType
	 * @return boolean - true if the multiplicity is different than 1
	 * TODO check uperbound for M3
	 */		
	public static boolean isMultipled(MultiplicityElement type){
		// for the upperBound, the ValueSpecification can be either a LiteralUnlimitedNatural or a LiteralInteger		
		ValueSpecification value = type.getUpperBound();
		if(value instanceof LiteralUnlimitedNatural ){
			if(value == null || ((LiteralUnlimitedNatural)value).getValue()!= null && ((LiteralUnlimitedNatural)value).getValue().equals("1")){
				return false;	
			}else{
				return true;
			}
			
		}else if (value instanceof LiteralInteger){
			if(value == null ||((LiteralInteger)value).getValue()!= null && ((LiteralInteger)value).getValue().intValue() == 1){
				return false;		
			}else{
				return true;
			}
			
		}
		// for the lowerBound, the ValueSpecification can be a LiteralInteger
		ValueSpecification lowerBound = type.getLowerBound();
		if(lowerBound instanceof LiteralInteger){
			if(lowerBound == null || ((LiteralInteger)lowerBound).getValue()!= null && (((LiteralInteger)lowerBound).getValue().intValue() == 1 || ((LiteralInteger)lowerBound).getValue().intValue() == 0)){
				return false;
			}else{
				return true;
			}
		}
		return false;
	}	
	
	/**
	 * Returns the list of Termination Nodes that are associated with the given output set.
	 * If no such termination nodes can be found, returns an empty list.
	 */
	public static List getTerminationNodesForSet(OutputPinSet set){
		List results = new ArrayList();
		
		StructuredActivityNode san = null;
		if (isProcess(set.getAction()) || set.getAction() instanceof LoopNode){
			san = (StructuredActivityNode)set.getAction();
			
			Object content;
			for (Iterator it = san.getNodeContents().iterator(); it.hasNext(); ){
				content = it.next();
				
				/* A 'null' outcome for a termination node is only permitted in LoopNodes. For a LoopNode,
				 * if the outcome is NOT null, then it must be associated to an exception output set.
				 * So therefore, assume that a termination node with a null
				 * outcome is also associated with the given set. */
				if (content instanceof TerminationNode 
						&& (((TerminationNode)content).getOutcome() == null 
								|| set.equals(((TerminationNode)content).getOutcome()))
								&& !results.contains(content)){
					
					results.add(content);				
				}
			}
		}
		
		return results;
		
	}
	
	/**
	 * Returns the list of Initial Nodes that are associated with the given input set.
	 * If no such initial nodes can be found, returns an empty list.
	 */
	public static List getInitialNodesForSet(InputPinSet set){
		List results = new ArrayList();
		
		StructuredActivityNode san = null;
		if (isProcess(set.getAction()) || set.getAction() instanceof LoopNode){
			san = (StructuredActivityNode)set.getAction();
			
			Object content;
			for (Iterator it = san.getNodeContents().iterator(); it.hasNext(); ){
				content = it.next();
				
				/* A 'null' entryPoint for an initial node indicates that the intial node is an
				 * entry point for all input pin sets. */
				if (content instanceof InitialNode 
						&& (((InitialNode)content).getEntryPoint().isEmpty() 
								|| ((InitialNode)content).getEntryPoint().contains(set))
								&& !results.contains(content)){
					
					results.add(content);				
				}
			}
		}
		
		return results;
		
	}	
	/**
	 * Returns a list that contains all the pins (both object and control) in set.  If there
	 * are no pins in the set, returns an empty list. 
	 */
	public static List getPinsInSet(PinSet pinSet){
		List pinList = new ArrayList();
		if(pinSet instanceof InputPinSet){
			pinList.addAll(((InputPinSet)pinSet).getInputControlPin());
			pinList.addAll(((InputPinSet)pinSet).getInputObjectPin());
		} else if(pinSet instanceof OutputPinSet){
			pinList.addAll(((OutputPinSet)pinSet).getOutputControlPin());
			pinList.addAll(((OutputPinSet)pinSet).getOutputObjectPin());
		}			 
		
		return pinList;		
	}
	
	/**
	 * Creates a Documentation ExtensibilityElement for documentation of the endPointReference.
	 * @param instanceSpecification BOM InstanceSpecification 
	 * @return extensibilityElement - documentation extensibility element.
	 */
	public static ExtensibilityElement createDocumentationExtensibleElement (InstanceSpecification instanceSpecification){
		
		Documentation documentation = BPELPlusFactory.eINSTANCE.createDocumentation();
		String warning = TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.INCOMPLETE_END_POINT_REFERENCE_WARNING);
		//	StringBuffer warning = new StringBuffer("Please update endPointReference binding information. Selection attributes used at modeling time are provided below \n");
		String doc = createDocumentation(instanceSpecification);
		String docString = warning.concat(doc).toString();
//GJD JR24896 - limit documentation to 4096 chars			
		if (docString.length() > 4096) {
			documentation.setValue(docString.substring(0, 4096));
			String name = instanceSpecification.getName();
			String[] params = new String[1];
			params[0] = name;
			LoggingUtil.logWarning(MessageKeys.TRUNCATED_ANNOTATION_WARNING, params, null);	
		} else {
			documentation.setValue(docString);
		}
		return documentation;
	}
	/**
	 * Creates an xml document based on the slots of the instanceSpecification 
	 * @param instanceSpecification
	 * @return string - the xml string representation of the instanceSpecification
	 */
	private static String createDocumentation (InstanceSpecification instanceSpecification){
		
		List slots = instanceSpecification.getSlot();
		Iterator it = slots.iterator();
		String attributeName = null;
		String attributeValue = null;
		StringBuffer sb = new StringBuffer("");
		while(it.hasNext()){
			Slot slot = (Slot)it.next();
			attributeName = slot.getDefiningFeature().getName();
			sb.append("<" + attributeName + ">");
			if (slot.getValue() != null){
				attributeValue = createDocumentationFromValueSpec(slot.getValue());
				sb.append(attributeValue);
				sb.append("</" + attributeName + ">");
			}
		}
		return sb.toString();
	}
	/**
	 * Creates the xml string representation from a list of ValueSpecification
	 * @param valueSpecs - list of BOM ValueSpecifications
	 * @return string - xml representation the list of value specifications
	 */
	private static String createDocumentationFromValueSpec(List valueSpecs){
		StringBuffer sb = new StringBuffer();
		if(valueSpecs != null){
			Iterator it = valueSpecs.iterator();
			while(it.hasNext()){
				sb.append(createDocumentationFromValueSpec((ValueSpecification)it.next()));
			}
		}
		return sb.toString();
	}
	
	/**
	 * Creates the xml string representation from a ValueSpecification
	 * @param valueSpecs a BOM ValueSpecification
	 * @return string - xml representation the list value specifications
	 */
	private static String createDocumentationFromValueSpec(ValueSpecification valueSpecification){
		
		StringBuffer sb = new StringBuffer();
		String attributeValue= "";
		if(valueSpecification instanceof LiteralString){
			attributeValue = ((LiteralString)valueSpecification).getValue();
		}else if(valueSpecification instanceof LiteralBoolean){
			attributeValue = ((LiteralBoolean)valueSpecification).getValue().toString();
		}else if(valueSpecification instanceof LiteralInteger){
			attributeValue = ((LiteralInteger)valueSpecification).getValue().toString();
		}else if(valueSpecification instanceof LiteralDuration){
			attributeValue = ((LiteralDuration)valueSpecification).getValue().toString();
		}else if (valueSpecification instanceof StructuredOpaqueExpression){
			StructuredOpaqueExpression soe = (StructuredOpaqueExpression)valueSpecification;
			Expression bomExpression = soe.getExpression();
			//now create an ExpressionRule and serialize the expression
			ExpressionRule expressionRule = AbstractbpelFactory.eINSTANCE.createExpressionRule();
			expressionRule.getSource().add(bomExpression);
			//getChildRules().add(expressionRule);
			expressionRule.transformSource2Target();
			Expression bpelExpression = (Expression)expressionRule.getTarget().get(0);
			XPathExprSerializerRule serializer = BpelFactory.eINSTANCE.createXPathExprSerializerRule();
			//getChildRules().add(serializer);
			serializer.getSource().add(bpelExpression);
			serializer.transformSource2Target();
			//Eclipse 3.0 migration
			attributeValue =(String) ((com.ibm.wbit.bpel.Expression)serializer.getTarget().get(0)).getBody();
		}else if(valueSpecification instanceof InstanceValue){
			InstanceValue valueSpec = (InstanceValue)valueSpecification;
			if(valueSpec.getOwnedInstance() != null){
				attributeValue = createDocumentation(valueSpec.getOwnedInstance());
			}else if(valueSpec.getInstance() != null){
				attributeValue = createDocumentation(valueSpec.getInstance());
			}
		}
		sb.append(attributeValue);
		return sb.toString();
	}
	
	/**
	 * Find the 'description of the ownedComment of an Action
	 * @param action
	 * @return
	 */
	public static Comment getDescription(Element element) {
		// Ray: need a special case for Activity ...
		if (element instanceof StructuredActivityNode) {
			if (((StructuredActivityNode)element).getActivity()!=null) {
				element = ((StructuredActivityNode)element).getActivity();
			}
		}
		List comments = element.getOwnedComment();
		Comment comment;
		//		for (Iterator it=comments.iterator();it.hasNext();) {
		//			comment = (Comment)it.next();
		//			if (comment.getAnnotatedElement().size()==0) {
		//				return comment;
		//			}
		//		}
		// ignore the kpi stuffs
		if (comments.size() > 0) {
			comment=(Comment)comments.get(0);
			if (comment!=null && comment.getAnnotatedElement().size()==0) {
				if (comment.getBody()!=null && comment.getBody().length()>0) {
					return comment;
				}
			}
		}
		return null;
	}
	
	/**
	 * Find all the annotated comments that annotate this element
	 * @param element
	 * @return
	 */
	public static List getAnnotatedComments(Element element) {
		ArrayList comments = new ArrayList();
		List ownedComments = element.getOwnedComment();
		Comment comment;
		for (Iterator it=ownedComments.iterator();it.hasNext();) {
			comment = (Comment)it.next();
			for (Iterator it2=comment.getAnnotatedElement().iterator();it2.hasNext();) {
				// if the comment is annotated to itself, add it to the return comments list
				Element tempElement = (Element)it2.next();
				if (tempElement==element) {
					comments.add(comment);
				}
			}
			
		}	
		StructuredActivityNode san = getContainingSAN(element);
		if (san != null) {
			for (Iterator it3=san.getOwnedComment().iterator();it3.hasNext();) {
				comment = (Comment)it3.next();
				for (Iterator it4=comment.getAnnotatedElement().iterator();it4.hasNext();) {
					Element tempElement = (Element)it4.next();
					if (tempElement==element) {
						comments.add(comment);
					}
				}
			}				
		}
		return comments;	
	}
	
	/**
	 * Find the containing SAN for this element, if presented
	 * @param element
	 * @return
	 */
	public static StructuredActivityNode getContainingSAN(Element element) {
		StructuredActivityNode san=null;
		if (element instanceof Action) {
			return ((Action)element).getInStructuredNode();
		} else if (element instanceof ControlNode) {
			return ((ControlNode)element).getInStructuredNode();
		} else if (element instanceof ActivityEdge) {
			return ((ActivityEdge)element).getInStructuredNode();
		} else if (element instanceof Variable) {
			return ((Variable)element).getScope();
		}
		return san;
	}	
	
	/**
	 * Get all the correlated pinSets.  If no correlated pinSets found, by default return all the non-correlated pinSets of the containing Action
	 * @param pinSet
	 * @return
	 */
	public static List getCorrelatedPinSets(PinSet pinSet) {
		List correlatedPinSets;
		Action action;
		if (pinSet instanceof InputPinSet) {
			correlatedPinSets = ((InputPinSet)pinSet).getOutputPinSet();
		} else {
			correlatedPinSets = ((OutputPinSet)pinSet).getInputPinSet();
		}
		// if not correlation explicitly set, then try to get the ones that are not correlated
		if (correlatedPinSets.size()==0) {
			if (pinSet instanceof InputPinSet) {
				action = ((InputPinSet)pinSet).getAction();
				// if it is a CBA, needs to get rid of the additional pinSets created by the tool for async invocation
				if (action instanceof CallBehaviorAction) {
					correlatedPinSets = new ArrayList();
					correlatedPinSets.addAll(action.getOutputPinSet());
					correlatedPinSets.remove(action.getOutputPinSet().size()-1); 		
				} else {
					//defect 168331
					//correlatedPinSets = action.getOutputPinSet();
					correlatedPinSets.addAll(action.getOutputPinSet());
					
				}
				// Now filter out those pinSets that are already correlated
				OutputPinSet ops;
				for (int i=correlatedPinSets.size()-1;i>=0;i--) {
					ops = (OutputPinSet)correlatedPinSets.get(i);
					if (ops.getInputPinSet().size()>0) {
						correlatedPinSets.remove(i);
					}
				}
			} else {
				action = ((OutputPinSet)pinSet).getAction();
				correlatedPinSets = action.getInputPinSet();
			}			
		}
		return correlatedPinSets;
	}
	/** 
	 * Returns true if the <code>pinSet</code> contains a single ObjectPin. 
	 * @param pinSet
	 * @return
	 */
	public static boolean isSinglePinInSet(PinSet pinSet){
		if(pinSet instanceof InputPinSet){
			return ((InputPinSet)pinSet).getInputObjectPin().size() == 1;
		}else{
			return ((OutputPinSet)pinSet).getOutputObjectPin().size() == 1;
		}
	}
	
	public static List getObjectPinsForSet(PinSet pinSet){
		if(pinSet instanceof InputPinSet){
			return ((InputPinSet)pinSet).getInputObjectPin();
		}else if (pinSet instanceof OutputPinSet){
			return ((OutputPinSet)pinSet).getOutputObjectPin();
		}else{
			return null;
		}
	}
	public static StructuredActivityNode getContainingProcess(NamedElement element){
		Action action = null;
		if(element instanceof InputPinSet){
			action = ((InputPinSet)element).getAction();
		}else if(element instanceof OutputPinSet){
			action = ((OutputPinSet)element).getAction();
		}else if(element instanceof Action){
			action = (Action)element;
		}
		if(action != null){
			while (action.getInStructuredNode() != null) {
				action = action.getInStructuredNode();
			}
			return ((StructuredActivityNode)action);
		}
		//		NamedElement container = (NamedElement)element.eContainer();
		//		
		//		while (container != null && container instanceof Action){
		//			if(isTopLevelProcess((Action)container)){
		//				return (StructuredActivityNode)container;
		//			}else{
		//				container = (NamedElement)container.eContainer();
		//			}
		//		}
		return null;
	}
	
	/**
	 * This method attempts to find the first pinSet (input or output) that contains a Pin.  Notice a pin can never appear
	 * in both input and output pinset.
	 * @param pin
	 * @return
	 */
	public static PinSet findPinSetForPin(Pin pin) {
		Action action = (Action)pin.eContainer();
		Iterator it = action.getInputPinSet().iterator();
		InputPinSet inPinSet;
		while (it.hasNext()) {
			inPinSet = (InputPinSet)it.next();
			// it can never be control pin
			if (inPinSet.getInputObjectPin().contains(pin)) {
				return inPinSet;
			}
			//verify the case for control cycle link
			else if(inPinSet.getInputControlPin().contains(pin)){
				return inPinSet;
			}
			
		}
		it = action.getOutputPinSet().iterator();
		OutputPinSet outPinSet;
		while (it.hasNext()) {
			outPinSet = (OutputPinSet)it.next();
			// it can never be control pin
			if (outPinSet.getOutputObjectPin().contains(pin)) {
				return outPinSet;
			}
			//verify the case for control cycle link
			else if(outPinSet.getOutputControlPin().contains(pin)){
				return outPinSet;
			}
		}
		return null;  // This should never happened!!
	}
	
	public static String getFullName(NamedElement namedElement)
	{
		String name = null;
		name = namedElement.getName();
		NamedElement container = null;
		if (namedElement.eContainer() instanceof NamedElement) {
			container = (NamedElement)namedElement.eContainer();
		}
		
		if(container == null){
			return name;
		}
		
		while ((container != null) && !(container instanceof PackageableElement)) {
			name = container.getName() + "___" + name;
			container = (NamedElement)container.eContainer();
		}
		if(container != null && container instanceof PackageableElement && container instanceof Activity){
			name = getFullPackageName((Activity)container) + "___" + name;
		}
		
		return name;
	}	
	
	public static String getFullPackageName(PackageableElement pkgElement)
	{
		if(pkgElement instanceof Activity && BomUtils.isProcess(((Activity)pkgElement).getImplementation())){
			//defect 139703, registrer the MasterCopy with the working copy
			//PackageableElement registeredProcess = (PackageableElement)TransformationSessionUtil.getMasterCopy(TransformationEngine.getTransformationSession());
			PackageableElement registeredProcess = (PackageableElement)TransformationSessionUtil.getMasterCopy(TransformationEngine.getTransformationSession(), (Activity)pkgElement);
			
			if(registeredProcess != null){
				pkgElement = registeredProcess;
			}
		}
		
		String name=pkgElement.getName();
		name=name.trim();
		Package belongingPackage = pkgElement.getOwningPackage();
		while (belongingPackage != null && !ProcessUtil.isRootPackage(belongingPackage)) {
			name = belongingPackage.getName() + "___" + name;
			belongingPackage = belongingPackage.getOwningPackage();
		}
		return name;
	}
	
	public static boolean isPredefinedType(Type type) {
		String canonicalName = StaffUtil.getCanonicalName(type);
		if(StaffUtil.predefinedListTypes.contains(canonicalName) || StaffUtil.predefinedOrgTypes.contains(canonicalName) || StaffUtil.predefinedBulkTypes.contains(canonicalName)){
			return true;
		}
		return false;
	}
	
	
	// This class is to categorizes the type of InputPinSets
	public static class PinSetAnalyzer {
		private final static boolean HANDLE_TASK_SAME_AS_PROCESS=true;
		private Action san;
		private List inputPinSetsWithNoCorrelationPredicate;
		private List inputPinSetsWithCorrelationPredicate;
		private List inputPinSetsWithCorrelationPredicateCreateNewInstance;
		private List inputPinSetsAsOneWayOperation;
		private List inputPinSetsAsTwoWayOperation;
		private List outputPinSetsAsCallback;
		private TransformationContext context;
		
		/*
		 * One-way vs Two-way operation analysis
		 * Case 1: An InputPinSet correlates to a single OutputPinSet (non-exceptional) and there is no other InputPinSet 
		 * 			- 2-way WSDL operation will be created
		 * Case 2: An InputPinSet correlates to more than one OutputPinSet (non-exceptional)
		 * 			- One way WSDL operation
		 * 			- Callback operation will be created for each of these OutputPinSet
		 * Case 3: An OutputPinSet correlates to more than one InputPinSet and the pinSets owned by a Process
		 * 			- Same as case 2
		 * Case 4: An InputPinSet is not correlated to any OutputPinSet
		 * 			- One-way WSDL operation will be created
		 * Case 5: An OutputPinSet is not correlated to any InputPinSet
		 * 			- One-way callback WSDL operation will be created
		 */			
		
		
		public PinSetAnalyzer(Action san) {
			this.san = san;
			List inputPinSets = san.getInputPinSet();
			InputPinSet ips;
			// Analyze inputPinSets if correlation predicate is defined
			for (Iterator it=inputPinSets.iterator();it.hasNext();) {
				ips = (InputPinSet)it.next();
				if (ips.getCorrelationPredicate()==null) {
					getInputPinSetsWithNoCorrelationPredicate().add(ips);
				} else {
					if (ips.getNoCorrelationMatches()==InputDeliveryOptionKind.CREATE_NEW_INSTANCE_LITERAL) {
						getInputPinSetsWithCorrelationPredicateCreateNewInstance().add(ips);
					} else {
						getInputPinSetsWithCorrelationPredicate().add(ips);
					}
				}
			}
			// Analyze inputPinSets if it is a two-way, one-way, or one-way with callback operation
			AttributeValueProvider attribProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.IS_ONE_WAY_OPERATION);
			
			Object attribValue;
			for (Iterator it=inputPinSets.iterator();it.hasNext();) {
				ips = (InputPinSet)it.next();
				attribValue=attribProvider.getAttributeValueForType(ips,AttributeNameConstants.IS_ONE_WAY_OPERATION);
				if (attribValue != null && attribValue instanceof Boolean) {
					if (((Boolean)attribValue).booleanValue()) {
						getInputPinSetsAsOneWayOperation().add(ips); 						
					} else {
						getInputPinSetsAsTwoWayOperation().add(ips);						
					}
				}
			}
			
			//			for (Iterator it=inputPinSets.iterator();it.hasNext();) {
			//				ips = (InputPinSet)it.next();
			//				if (checkActionAsProcess(ips.getAction())) {
			//					if (ips.getOutputPinSet().size()==1 && 
			//						((OutputPinSet)ips.getOutputPinSet().get(0)).getInputPinSet().size()==1 &&
			//						// Mixture of inputPinSet correlated to outputPinSet and inputPinSet with 
			//						// correlationPredicate createNewInstance='yes' doesn't seem to be a valid case.
			//						// So we may not want to check it (??)
			//						getInputPinSetsWithCorrelationPredicateCreateNewInstance().size()==0 
			//						 //&& getInputPinSetsWithCorrelationPredicate().size()==0
			//					) {
			//						getInputPinSetsAsTwoWayOperation().add(ips);
			//					} else {
			//						getInputPinSetsAsOneWayOperation().add(ips); 
			//					}	
			//				} else {
			//					if (ips.getOutputPinSet().size()==1) { 
			//						getInputPinSetsAsTwoWayOperation().add(ips);
			//					} else {
			//						getInputPinSetsAsOneWayOperation().add(ips); 
			//					}	
			//				}
			//			}
			
			List outputPinSets = san.getOutputPinSet();
			OutputPinSet ops;
			
			for (Iterator it=outputPinSets.iterator();it.hasNext();) {
				ops = (OutputPinSet)it.next();
				if (OperationTypeUtil.isCallbackResponse(ops).booleanValue()) {
					getOutputPinSetsAsCallbackOperation().add(ops);
				}
			}
			//			for (Iterator it=outputPinSets.iterator();it.hasNext();) {
			//				ops = (OutputPinSet)it.next();
			//				// Analyze whether the outputPinSet is a callback
			//				// The ops does not correlate to any inputPinSet and it contains at least 1 object pin
			//				// OR The ops is correlated to more than one inputPinSet (in the case of a Process) 
			//				// OR The ops is correlated to one inputPinSet but the inputPinSet correlates to more than one outputPinSet 
			//				// OR The ops is correlated to one inputPinSet but the inputPinSet is a one way operation
			//				if ((ops.getInputPinSet().size()==0 && ops.getOutputObjectPin().size()>0) || 
			//					(ops.getInputPinSet().size()>1 && checkActionAsProcess(ops.getAction())) ||
			//					(ops.getInputPinSet().size()==1 && ((InputPinSet)ops.getInputPinSet().get(0)).getOutputPinSet().size()>1) ||
			//					(ops.getInputPinSet().size()==1 && getInputPinSetsAsOneWayOperation().contains(ops.getInputPinSet().get(0)))
			//					) {
			//					getOutputPinSetsAsCallbackOperation().add(ops);
			//				}
			//			}			
		}
		
		public PinSetAnalyzer(Action san, TransformationContext context){
			this.context = context;
			this.san = san;
			List inputPinSets = san.getInputPinSet();
			InputPinSet ips;
			// Analyze inputPinSets if correlation predicate is defined
			for (Iterator it=inputPinSets.iterator();it.hasNext();) {
				ips = (InputPinSet)it.next();
				if (ips.getCorrelationPredicate()==null) {
					getInputPinSetsWithNoCorrelationPredicate().add(ips);
				} else {
					if (ips.getNoCorrelationMatches()==InputDeliveryOptionKind.CREATE_NEW_INSTANCE_LITERAL) {
						getInputPinSetsWithCorrelationPredicateCreateNewInstance().add(ips);
					} else {
						getInputPinSetsWithCorrelationPredicate().add(ips);
					}
				}
			}
			// Analyze inputPinSets if it is a two-way, one-way, or one-way with callback operation
			AttributeValueProvider attribProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.IS_ONE_WAY_OPERATION);
			attribProvider.setContext(context);
			Object attribValue;
			for (Iterator it=inputPinSets.iterator();it.hasNext();) {
				ips = (InputPinSet)it.next();
				attribValue=attribProvider.getAttributeValueForType(ips,AttributeNameConstants.IS_ONE_WAY_OPERATION);
				if (attribValue != null && attribValue instanceof Boolean) {
					if (((Boolean)attribValue).booleanValue()) {
						getInputPinSetsAsOneWayOperation().add(ips); 						
					} else {
						getInputPinSetsAsTwoWayOperation().add(ips);						
					}
				}
			}
			
			//			for (Iterator it=inputPinSets.iterator();it.hasNext();) {
			//				ips = (InputPinSet)it.next();
			//				if (checkActionAsProcess(ips.getAction())) {
			//					if (ips.getOutputPinSet().size()==1 && 
			//						((OutputPinSet)ips.getOutputPinSet().get(0)).getInputPinSet().size()==1 &&
			//						// Mixture of inputPinSet correlated to outputPinSet and inputPinSet with 
			//						// correlationPredicate createNewInstance='yes' doesn't seem to be a valid case.
			//						// So we may not want to check it (??)
			//						getInputPinSetsWithCorrelationPredicateCreateNewInstance().size()==0 
			//						 //&& getInputPinSetsWithCorrelationPredicate().size()==0
			//					) {
			//						getInputPinSetsAsTwoWayOperation().add(ips);
			//					} else {
			//						getInputPinSetsAsOneWayOperation().add(ips); 
			//					}	
			//				} else {
			//					if (ips.getOutputPinSet().size()==1) { 
			//						getInputPinSetsAsTwoWayOperation().add(ips);
			//					} else {
			//						getInputPinSetsAsOneWayOperation().add(ips); 
			//					}	
			//				}
			//			}
			
			List outputPinSets = san.getOutputPinSet();
			OutputPinSet ops;
			
			for (Iterator it=outputPinSets.iterator();it.hasNext();) {
				ops = (OutputPinSet)it.next();
				if (OperationTypeUtil.isCallbackResponse(ops).booleanValue()) {
					getOutputPinSetsAsCallbackOperation().add(ops);
				}
			}
			//			for (Iterator it=outputPinSets.iterator();it.hasNext();) {
			//				ops = (OutputPinSet)it.next();
			//				// Analyze whether the outputPinSet is a callback
			//				// The ops does not correlate to any inputPinSet and it contains at least 1 object pin
			//				// OR The ops is correlated to more than one inputPinSet (in the case of a Process) 
			//				// OR The ops is correlated to one inputPinSet but the inputPinSet correlates to more than one outputPinSet 
			//				// OR The ops is correlated to one inputPinSet but the inputPinSet is a one way operation
			//				if ((ops.getInputPinSet().size()==0 && ops.getOutputObjectPin().size()>0) || 
			//					(ops.getInputPinSet().size()>1 && checkActionAsProcess(ops.getAction())) ||
			//					(ops.getInputPinSet().size()==1 && ((InputPinSet)ops.getInputPinSet().get(0)).getOutputPinSet().size()>1) ||
			//					(ops.getInputPinSet().size()==1 && getInputPinSetsAsOneWayOperation().contains(ops.getInputPinSet().get(0)))
			//					) {
			//					getOutputPinSetsAsCallbackOperation().add(ops);
			//				}
			//			}			
		}
		public List getInputPinSetsWithNoCorrelationPredicate() {
			if (inputPinSetsWithNoCorrelationPredicate==null) {
				inputPinSetsWithNoCorrelationPredicate=new ArrayList();
			}
			return inputPinSetsWithNoCorrelationPredicate;
		}
		public List getInputPinSetsWithCorrelationPredicateCreateNewInstance() {
			if (inputPinSetsWithCorrelationPredicateCreateNewInstance==null) {
				inputPinSetsWithCorrelationPredicateCreateNewInstance=new ArrayList();
			}
			return inputPinSetsWithCorrelationPredicateCreateNewInstance;
		}
		public List getInputPinSetsWithCorrelationPredicate() {
			if (inputPinSetsWithCorrelationPredicate==null) {
				inputPinSetsWithCorrelationPredicate=new ArrayList();
			}
			return inputPinSetsWithCorrelationPredicate;
		}
		
		/**
		 * @return
		 */
		public List getInputPinSetsAsOneWayOperation() {
			if (inputPinSetsAsOneWayOperation==null) {
				inputPinSetsAsOneWayOperation=new ArrayList();
			}
			return inputPinSetsAsOneWayOperation;
		}
		
		/**
		 * @return
		 */
		public List getInputPinSetsAsTwoWayOperation() {
			if (inputPinSetsAsTwoWayOperation==null) {
				inputPinSetsAsTwoWayOperation=new ArrayList();
			}
			return inputPinSetsAsTwoWayOperation;
		}
		
		/**
		 * @return
		 */
		public List getOutputPinSetsAsCallbackOperation() {
			if (outputPinSetsAsCallback==null) {
				outputPinSetsAsCallback=new ArrayList();
			}
			return outputPinSetsAsCallback;
		}
		
		/**
		 * Get the callback outputPinSets 'corresponds' to the inputPinSet
		 * @param ips
		 * @return
		 */
		public List getCallBackOutputPinSets(InputPinSet ips) {
			List list = new ArrayList();
			List ops = ips.getOutputPinSet();
			if (ops.size()>0) {
				list.addAll(ops);
			} else {
				list.addAll(ips.getAction().getOutputPinSet());
			}
			// Eliminate the pin sets that are not categorized as callbacks
			for (int i=list.size()-1; i>=0; i--) {
				if (!getOutputPinSetsAsCallbackOperation().contains(list.get(i))) { 
					list.remove(i); 
				}
			}
			return list;
		}
		
		public boolean checkActionAsProcess(Action action) {
			if (!HANDLE_TASK_SAME_AS_PROCESS) {
				return isProcess(action);
			}
			return true;
		}
	}
	
	/**
	 * This method is responsible for retrieving the owning external service refered by 
	 * the <code>san</code>
	 * @param san represents the service operation
	 * @return
	 */
	public static NamedElement resolveOwningService(StructuredActivityNode san){
		
		if(san == null ||!BomUtils.isServiceOperation(san)){
			return null;
		}
		Activity container = (Activity)san.eContainer();
		Object owningModel = container.getOwningPackage();
		if(owningModel != null && owningModel instanceof ServiceInterface){
			Object extService = ((ServiceInterface)owningModel).getOwningPackage();
			if(extService instanceof ExternalService){
				return (NamedElement)extService;
			}
		}
		return null;
	}
	
	public static NamedElement resolveFileAttachment(StructuredActivityNode san){
		if(san == null || !BomUtils.isServiceOperation(san)){
			return null;
		}
		ExternalService extService = (ExternalService)resolveOwningService(san);
		if(extService == null){
			return null;
		}
		EList  members = extService.getOwnedMember();
		if(members != null && members.size() != 0){
			Iterator mbrIterator = members.iterator();
			while(mbrIterator.hasNext()){
				NamedElement mbr = (NamedElement)mbrIterator.next();
				if("__FILE__ATTACHMENT".equals(mbr.getName()) && 
						"fileAttachment".equals(mbr.getAspect()) &&
						mbr instanceof ExternalSchema){
					EList fileAttachments = ((ExternalSchema)mbr).getOwnedMember();
					if(!fileAttachments.isEmpty()){
						return (NamedElement)fileAttachments.get(0);
					}
				}				
			}
		}    	
		return null;
	}
	
	/**
	 * Resolve the <code>NamedElement</code> that is type of <code>ServiceInterface</code>
	 * @param san
	 * @return
	 */
	public static NamedElement resolveOwningInterface(StructuredActivityNode san){
		if(san == null ||!BomUtils.isServiceOperation(san)){
			return null;
		}
		Activity container = (Activity)san.eContainer();
		Object owningModel = container.getOwningPackage();
		if(owningModel != null && owningModel instanceof ServiceInterface){
			return (NamedElement)owningModel;
		}
		return null;
	}
	
	/**
	 * Check if this Action has more than one regular output pin set
	 * @param action Action
	 * @return boolean
	 */
	public static boolean hasMoreThanOneRegularOutputSet(Action action) {
		if (action == null) {
			return false;
		}
		
		int numberOfRegularOSets = 0;
		
		Iterator <OutputPinSet> it = action.getOutputPinSet().iterator();
		OutputPinSet oSet = null;
		while (it.hasNext() && numberOfRegularOSets < 2) {
			oSet = (OutputPinSet) it.next();
			if ((oSet.getIsException() != null && oSet.getIsException().booleanValue()) ||
					(oSet.getIsStream() != null && oSet.getIsStream().booleanValue())) {
				continue;
			} else {	// treat streaming sets as regular
				numberOfRegularOSets++;
			}
		}
		if (numberOfRegularOSets > 1) {
			return true;
		}
		
		return false;
	}
	/**
	 * Returns true if one of the Pins in the inputSet is a BusinessServiceObject
	 * @param is
	 * @return
	 */
	public static boolean referenceBusinessServiceObjects(PinSet is) {

		List<ObjectPin> objectPins = BomUtils.getObjectPinsForSet(is);
		for(ObjectPin objectPin: objectPins){
			if(OriginalXsdUtil.isExternalType(objectPin.getType())){
				return true;
			}
		}
		return false;
	}
}
