package com.ibm.btools.te.ilm.heuristics.helper;
/*
 * Licensed Materials - Property of IBM
 * 5724-I74, 5724-I75
 *  Copyright IBM Corporation 2008, 2009. All rights reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */


import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.ExtensibleElement;
import org.eclipse.wst.wsdl.Fault;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.Part;
import org.eclipse.wst.wsdl.PortType;
import org.eclipse.xsd.XSDComplexTypeDefinition;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDModelGroup;
import org.eclipse.xsd.XSDParticle;
import org.eclipse.xsd.XSDSimpleTypeDefinition;

import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.actions.Decision;
import com.ibm.btools.bom.model.processes.actions.Fork;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.ActivityEdge;
import com.ibm.btools.bom.model.processes.activities.ConnectableNode;
import com.ibm.btools.bom.model.processes.activities.InputControlPin;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.LoopNode;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputControlPin;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.activities.TerminationNode;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.ProcessFlowPatternRule;
import com.ibm.btools.te.ilm.heuristics.ProcessFlowRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.DecisionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.InvokeRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.OutputRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.VariableRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.util.ScopeRegistryUtil;
import com.ibm.btools.te.ilm.heuristics.bpelp.BpelpRule;
import com.ibm.btools.te.ilm.heuristics.bpelp.impl.BpelpRuleImpl;
import com.ibm.btools.te.ilm.heuristics.bpelp.util.JavaConstants;
import com.ibm.btools.te.ilm.heuristics.extservice.OperationRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.MessageRule;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.util.exception.BTRuntimeException;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Assign;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Case;
import com.ibm.wbit.bpel.Condition;
import com.ibm.wbit.bpel.Copy;
import com.ibm.wbit.bpel.Empty;
import com.ibm.wbit.bpel.Flow;
import com.ibm.wbit.bpel.From;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.OnMessage;
import com.ibm.wbit.bpel.PartnerLink;
import com.ibm.wbit.bpel.PartnerLinks;
import com.ibm.wbit.bpel.Pick;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.bpel.Query;
import com.ibm.wbit.bpel.Receive;
import com.ibm.wbit.bpel.Reply;
import com.ibm.wbit.bpel.Scope;
import com.ibm.wbit.bpel.Source;
import com.ibm.wbit.bpel.Sources;
import com.ibm.wbit.bpel.Switch;
import com.ibm.wbit.bpel.Target;
import com.ibm.wbit.bpel.Targets;
import com.ibm.wbit.bpel.To;
import com.ibm.wbit.bpel.While;
import com.ibm.wbit.bpel.services.partnerlinktype.PartnerlinktypeFactory;
import com.ibm.wbit.bpel.services.partnerlinktype.Role;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.Catch;
import com.ibm.wbit.bpelpp.FaultSource;
import com.ibm.wbit.bpelpp.Generated;
import com.ibm.wbit.bpelpp.Type;
import com.ibm.wbit.wpc.CustomProperty;
import com.ibm.wbit.wpc.TypeEnum;

/**
 * Utility to help cBPEL fault link support
 *
 */
public class CBPELUtil {	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	private static final String FAULT_LINK_VAR_REGISTRY_KEY = "FAULT_LINK_VAR_REGISTRY_KEY";	
	private static final String CBPEL_FAULT_SOURCES = "CBPEL_FAULT_SOURCES";	
	private static final String FAULT_LINK_REG_KEY = "FAULT_LINK_REG_KEY ";
	private static final String SCOPE_FAULT_LINK_REG_KEY = "SCOPE_FAULT_LINK_REG_KEY ";

	//Define the constants for the generate value
	//TODO Discuss with the runtime where to define the constant: com.ibm.bpw.wfg.ra 
	public static final String GEN_BPC_FAULT_SPLIT_ASSIGN = "bpcFaultAssignActivity";	
	public static final String GEN_BPC_FAULT_LINK = "bpcFaultLink";
	public static final String GEN_BPC_REGION_BOUNDARY = "bpcRegionBoundary";
	public static final String GEN_BPC_START_ACTIVITY = "bpcStartActivity";
	public static final String GEN_BPC_END_ACTIVITY = "bpcEndActivity";	

	//TODO find out XPath directive constant
	private static final String QUERY_LANGUAGE = "http://www.w3.org/TR/1999/REC-xpath-19991116";
	//Process originality custom property constants
	private static final String PROCESS_ORIGINALITY_PROPERTY_NAME = "generatedBy";
	private static final String PROCESS_ORIGINALITY_PROPERTY_VALUE = "bpmn";
	/**
	 * Test if the exception output pin set has multiple pins and connects to different targets,
	 * if that's the case, we generate empty activity
	 * @param ops
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static boolean shouldGenerateForkActivity(OutputPinSet ops){		
		if(ops != null){
			List<String> targetNames = new ArrayList<String>();
			if(ops.getIsException() != null && ops.getIsException().booleanValue()){
				List pinList = BomUtils.getPinsInSet(ops);
				if(pinList != null && pinList.size() > 0){
					Iterator pinIt = pinList.iterator();
					while(pinIt.hasNext()){
						Object eachPin = pinIt.next();
						if(eachPin instanceof OutputObjectPin){
							OutputObjectPin oop = (OutputObjectPin)eachPin;
							String targetName = null;
							if(oop.getOutgoing() != null && oop.getOutgoing().getTarget() instanceof ObjectPin){
								ObjectPin pin = (ObjectPin)oop.getOutgoing().getTarget();
								if(pin instanceof InputObjectPin){
									InputPinSet ips = (InputPinSet)BomUtils.getInputPinSets(pin).get(0);
									targetName = ips.getAction().getName();
								}else if(pin instanceof OutputObjectPin){
									//This is the case that the action is nested inside a subprocess or while loop or directly connect
									//to the output pin set of a top level process
									OutputPinSet targetOps = (OutputPinSet)BomUtils.getOutputPinSets(pin).get(0);
									targetName = targetOps.getAction().getName();
								}
								if(targetName != null){
									targetNames.add(targetName);
								}
							}else if(oop.getOutgoing() != null && oop.getOutgoing().getTarget() instanceof TerminationNode){
								TerminationNode tn = (TerminationNode)oop.getOutgoing().getTarget() ;
								if(tn.getOutcome() instanceof OutputPinSet){
									OutputPinSet mappedOPS = (OutputPinSet)tn.getOutcome();
									targetName = mappedOPS.getAction().getName();
									if(targetName != null){
										targetNames.add(targetName);
									}
								}
							}
						}
					}
				}
				//If the fault set includes more than one pins and has more than one one outgoing link
				//we create the assign activity to assign the variable, which is an interface variable
				if(pinList != null && pinList.size() > 1){
					return (targetNames.size() >= 1);
				}
			}			
		}
		return false;
	}

	/**
	 * Assume only one operation per portType
	 * TODO how about multiple operations
	 * @param extPortTypeRule
	 * @return
	 */
	public static List<MessageRule> findMessageRulesForPortType(TransformationRule extPortTypeRule) {
		List<MessageRule> msgRuleList = new ArrayList<MessageRule>();
		if(extPortTypeRule.getChildRules()!= null && extPortTypeRule.getChildRules().size() > 0){
			for(Object childRule:extPortTypeRule.getChildRules()){
				if(childRule instanceof OperationRule || childRule instanceof com.ibm.btools.te.ilm.heuristics.wsdl.OperationRule){
					TransformationRule operationRule = (TransformationRule)childRule;				
					for(Object eachrule:operationRule.getChildRules()){
						if(eachrule instanceof MessageRule){
							msgRuleList.add((MessageRule)eachrule);
						}
					}
				}
			}
		}
		return msgRuleList;
	}

	/**
	 * Assume only one operation per portType
	 * TODO how about multiple operations
	 * @param extPortTypeRule
	 * @return
	 */
	public static List<TransformationRule> findOperationRulesForPortType(TransformationRule extPortTypeRule) {
		List<TransformationRule> operationRuleList = new ArrayList<TransformationRule>();
		if(extPortTypeRule.getChildRules()!= null && extPortTypeRule.getChildRules().size() > 0){
			for(Object childRule:extPortTypeRule.getChildRules()){
				if(childRule instanceof OperationRule || childRule instanceof com.ibm.btools.te.ilm.heuristics.wsdl.OperationRule){
					operationRuleList.add((TransformationRule)childRule);
				}
			}
		}
		return operationRuleList;
	}

	/**
	 * Test utility for the 
	 * @param faultSourceMap
	 * @param ir
	 */
	public static void printFaultSources(Map<String, List<Object>> faultSourceMap,TransformationRule ir){
		if(faultSourceMap == null || faultSourceMap.size() == 0 || ir == null){
			return;
		}
		System.out.println("Fault Sources on BPEL Activity: <"+((Activity)ir.getTarget().get(0)).getName()+">:");
		System.out.println("-------------------------------------------------------------------------------------");
		for(String key:faultSourceMap.keySet()){
			StringBuffer sb = new StringBuffer();
			List sources = faultSourceMap.get(key);
			for(Object source:sources){
				if(source instanceof Link){
					Link link = (Link)source;
					sb.append(link.getName()+"\n");
				}else if(source instanceof Message){
					Message message = (Message)source;
					sb.append("\t"+ message.getQName()+"\n");
				}else if(source instanceof BPELVariable){
					BPELVariable variable = (BPELVariable)source;						
					sb.append("\t\t"+variable.getName()+"\n");
				}
			}			
			System.out.println(sb.toString());		
		}
		System.out.println("-------------------------------------------------------------------------------------");
	}
	/**
	 * This method is to determine whether we need to create variable 
	 * For business service operation, multiple parts message is not supported in iteration 3
	 * we only support anonymous complex type which wraps the multiple pin types 
	 * @param osIt
	 * @return
	 */
	public static boolean shouldCreateWrapperContainer(TransformationContext context,OutputPinSet ops) {
		if(shouldGenerateForkActivity(ops)){
			if(BomUtils.isServiceOperation(ops.getAction())|| ops.getAction() instanceof CallBehaviorAction){
				if(isMultiplePartMessage(context,ops)){	
					//log error not supported in iteration 3, but Iteration 5	
					com.ibm.btools.bom.model.processes.activities.Activity behaviour = null;
					CallBehaviorAction cba = (CallBehaviorAction)ops.getAction() ;
					if(cba.getBehavior()instanceof com.ibm.btools.bom.model.processes.activities.Activity){
						behaviour = (com.ibm.btools.bom.model.processes.activities.Activity)cba.getBehavior();
					}
					String params[]={ops.getName(),behaviour.getImplementation().getName()};
					LoggingUtil.logError(MessageKeys.MULTIPLE_PARTS_MESSAGE_NOT_SUPPORTED, params,null);
					BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.MULTIPLE_PARTS_MESSAGE_NOT_SUPPORTED));
					throw runtimeEx;
				}
				return true;
			}else if(BomUtils.isTask(ops.getAction())){
				//we always create anonymous complex type for local task fault message part
				return true;
			}
		}
		return false;
	}
	/**
	 * Look up the target message in the original WSDL definition
	 *TODO Handle message retrieval for modeler activities, rewrite for cba and local respectively
	 * @param context
	 * @param ops
	 * @return
	 */
	public static Message retrieveMessageForPinSet(TransformationContext context, OutputPinSet ops){
		Message targetMsg = null;
		if(context == null || ops == null){
			return null;
		}
		TransformationRule fromRule = null;
		MessageRule msgRule = null;
		//Handle CBA for business service and global task/service
		if(ops.getAction() instanceof CallBehaviorAction){			
			CallBehaviorAction cba = (CallBehaviorAction)ops.getAction() ;
			if(cba.getBehavior()instanceof com.ibm.btools.bom.model.processes.activities.Activity){
				com.ibm.btools.bom.model.processes.activities.Activity behaviour = (com.ibm.btools.bom.model.processes.activities.Activity)cba.getBehavior();				
				fromRule = ProcessUtil.getProcessInterfaceRule(context,behaviour.getImplementation());					
				OutputPinSet pinSetInSan = getPinSetInSan(behaviour.getImplementation(),ops);
				msgRule = (MessageRule)TransformationUtil.getRuleForSource(fromRule, MessageRule.class, pinSetInSan);				
			}
		}else if(BomUtils.isTask(ops.getAction())){		
			if(ops.getAction()  instanceof StructuredActivityNode){
				StructuredActivityNode san = (StructuredActivityNode)ops.getAction();
				fromRule = ProcessUtil.getProcessInterfaceRule(context,san);
				msgRule = (MessageRule)TransformationUtil.getRuleForSource(fromRule, MessageRule.class, ops);				
			}
		}else if(BomUtils.isLocalProcess(ops.getAction())){
			//sub-process

		}else if(BomUtils.isTopLevelProcess(ops.getAction())){
			//global process
		}
		if(msgRule != null && !msgRule.getTarget().isEmpty()){
			targetMsg = (Message)msgRule.getTarget().get(0);
		}
		return targetMsg;
	}

	/**
	 * Find the output pin set in the SAN, which is the source of the message rule
	 * @param implementation
	 * @param ops
	 * @return
	 */
	private static OutputPinSet getPinSetInSan(StructuredActivityNode implementation, OutputPinSet ops) {
		OutputPinSet opsInSan = null;
		if(implementation == null || ops == null){
			return opsInSan;
		}
		if(implementation.getOutputPinSet()!= null && !implementation.getOutputPinSet().isEmpty()){
			for(Object eachPS:implementation.getOutputPinSet()){
				if(eachPS instanceof OutputPinSet ){
					OutputPinSet targetPS = (OutputPinSet)eachPS;
					if(ops.getName()!= null && ops.getName().equals(targetPS.getName())){
						opsInSan = targetPS;
						break;
					}
				}
			}
		}
		return opsInSan;
	}

	/**
	 * Check if this pin set 
	 * @param pinSet
	 * @return
	 */
	public static boolean isFaultLinkVariable(PinSet pinSet){
		if(pinSet instanceof OutputPinSet){
			OutputPinSet ops = (OutputPinSet)pinSet;
			if(ops.getIsException()!= null && ops.getIsException().booleanValue()){
				return true;
			}
		}
		return false;		
	}
   /**
    * Detect the case that fault pin without outgoing links
    * @param pinSet
    * @return
    */
	public static boolean isFaultSetNoConnections(PinSet pinSet){
		if(pinSet instanceof OutputPinSet){
			OutputPinSet ops = (OutputPinSet)pinSet;
			if(ops.getIsException()!= null && ops.getIsException().booleanValue()){
				List pinList = BomUtils.getPinsInSet(pinSet);
				boolean noConnection = true;
				for(Object eachPin:pinList){
					if(eachPin instanceof OutputObjectPin){
						OutputObjectPin oop = (OutputObjectPin)eachPin;
						if(oop.getOutgoing() instanceof ActivityEdge){
							noConnection = false;
							break;
						}
					}else if(eachPin instanceof OutputControlPin){
						OutputControlPin ocp = (OutputControlPin)eachPin;
						if(ocp.getOutgoing() instanceof ActivityEdge){
							noConnection = false;
							break;
						}
					}
				}
				return noConnection;
			}
		}
		return false;
	}

	/**
	 * This method is to test the business service WSDL style. Since this check happens in the output rule,
	 * we assume the original WSDL interface has already been transformed, so we can easily look up through 
	 * the process interface rule
	 * @param context
	 * @param ops
	 * @return
	 */
	private static boolean isMultiplePartMessage(TransformationContext context, OutputPinSet ops) {
		if(context == null || ops == null){
			return false;
		}
		if(ops.getAction() instanceof CallBehaviorAction){	
			CallBehaviorAction cba = (CallBehaviorAction)ops.getAction() ;
			if(cba.getBehavior()instanceof com.ibm.btools.bom.model.processes.activities.Activity){
				com.ibm.btools.bom.model.processes.activities.Activity behaviour = (com.ibm.btools.bom.model.processes.activities.Activity)cba.getBehavior();				
				TransformationRule fromRule = ProcessUtil.getProcessInterfaceRule(context,behaviour.getImplementation());					
				OutputPinSet pinSetInSan = getPinSetInSan(behaviour.getImplementation(),ops);
				MessageRule msgRule = (MessageRule)TransformationUtil.getRuleForSource(fromRule, MessageRule.class, pinSetInSan);	
				if(msgRule != null && !msgRule.getTarget().isEmpty()){
					Message targetMsg = (Message)msgRule.getTarget().get(0);
					if(targetMsg != null){
						if(targetMsg.getEParts()!= null && targetMsg.getEParts().size() > 1){
							return true;
						}
					}
				}
			}
		}	
		return false;
	}

	@SuppressWarnings("unchecked")
	public static Map<Activity,List<Source>> getFaultLinkRegistry(TransformationContext context) {		
		Map<Activity,List<Source>> registry = (HashMap<Activity,List<Source>>)context.get(FAULT_LINK_REG_KEY);
		if (registry == null) {
			registry = new HashMap<Activity,List<Source>>();
			context.put(FAULT_LINK_REG_KEY,registry);
		}
		return registry;
	}

	@SuppressWarnings("unchecked")
	public static Map<TransformationRule,Map<String,BPELVariable>> getFaultLinkVarRegistry(TransformationContext context) {		
		Map<TransformationRule,Map<String,BPELVariable>> registry = (HashMap<TransformationRule,Map<String,BPELVariable>>)context.get(FAULT_LINK_VAR_REGISTRY_KEY);
		if (registry == null) {
			registry = new HashMap<TransformationRule,Map<String,BPELVariable>>();
			context.put(FAULT_LINK_VAR_REGISTRY_KEY,registry);
		}
		return registry;
	}

	public static BPELVariable getBPELVariableForFaultPinSet(TransformationContext context, TransformationRule containerActivityRule,String opsName){
		BPELVariable wrapperVariable = null;
		if(context == null || containerActivityRule == null || opsName == null){
			return wrapperVariable;
		}
		if(CBPELUtil.getFaultLinkVarRegistry(context).get(containerActivityRule) instanceof Map){
			wrapperVariable = CBPELUtil.getFaultLinkVarRegistry(context).get(containerActivityRule).get(opsName);
		}
		return wrapperVariable;
	}    
	/**
	 * This map is keyed by transformation rule
	 * @param context
	 * @param targetRule
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static HashMap<TransformationRule,Map<String, List<Object>>> getFaultSourcesMap(TransformationContext context) {		
		HashMap<TransformationRule,Map<String, List<Object>>> registry = (HashMap<TransformationRule,Map<String, List<Object>>>)context.get(CBPEL_FAULT_SOURCES);
		if (registry == null) {
			registry = new HashMap<TransformationRule,Map<String, List<Object>>>();
			context.put(CBPEL_FAULT_SOURCES,registry);
		}
		return registry;
	}

	public static void registerFaultSource(TransformationContext context, TransformationRule targetRule,String pinSetName, Object sourceObject){
		if(context == null || targetRule == null || pinSetName == null || sourceObject == null){
			return;
		}
		Map<TransformationRule,Map<String, List<Object>>> faultSourceRegistry =  getFaultSourcesMap(context);
		if(isPinSetInMap(context,targetRule, pinSetName)){//TODO guard same type already in the map
			if(faultSourceRegistry.get(targetRule).get(pinSetName) != null){//&& faultSourceRegistry.get(targetRule).get(pinSetName).isEmpty()){
				faultSourceRegistry.get(targetRule).get(pinSetName).add(sourceObject);
			}
		}else{
			List<Object> sourceList = new ArrayList<Object>();
			sourceList.add(sourceObject);
			//if the registry for the target rule exits, we add the entry for the specific pinset under the
			//target transformation rule
			if(faultSourceRegistry.get(targetRule) instanceof Map){
				faultSourceRegistry.get(targetRule).put(pinSetName, sourceList);
			}else{
				//if the target rule registry doesn't exist, we create the entry for the target rule
				Map<String, List<Object>> faultMap = new HashMap<String, List<Object>>();
				faultMap.put(pinSetName, sourceList);
				faultSourceRegistry.put(targetRule, faultMap);
			}			
		}
	}
	/**
	 * Registry of scope fault links
	 * @param context
	 * @return
	 */
	public static Map<OutputPinSet,Link> getScopeFaultLinkRegistry(TransformationContext context){
		Map<OutputPinSet,Link> registry = (HashMap<OutputPinSet,Link>)context.get(SCOPE_FAULT_LINK_REG_KEY);
		if (registry == null) {
			registry = new HashMap<OutputPinSet,Link>();
			context.put(SCOPE_FAULT_LINK_REG_KEY,registry);
		}
		return registry;
	}

	public static void registerFaultLinkForScope(TransformationContext context,OutputPinSet ops,Link faultLink){
		Map<OutputPinSet,Link> registry = getScopeFaultLinkRegistry(context);
		if(!registry.containsKey(ops)){
			registry.put(ops, faultLink);
		}
	}

	public static Link getFautlLinkForScope(TransformationContext context,OutputPinSet ops){
		Map<OutputPinSet,Link> registry = getScopeFaultLinkRegistry(context);
		return registry.get(ops);
	}

	public static boolean isPinSetInMap(TransformationContext context,TransformationRule targetRule,String pinSetName){
		Map<TransformationRule,Map<String, List<Object>>> faultMap =  getFaultSourcesMap(context);
		if(faultMap != null && pinSetName != null && targetRule != null){
			Map<String,List<Object>> targetMap = faultMap.get(targetRule);
			if(targetMap == null){
				return false;
			}
			return targetMap.containsKey(pinSetName);
		}
		return false;
	}

	public static void registerFaultLinkVariable(TransformationContext context,TransformationRule targetRule,String pinSetName, BPELVariable variable) {
		if(context== null || pinSetName == null || variable == null ){
			return;
		}
		Map<TransformationRule,Map<String,BPELVariable>> registry = getFaultLinkVarRegistry(context);	
		if(registry.get(targetRule) instanceof Map){
			registry.get(targetRule).put(pinSetName, variable);
		}else{
			Map<String,BPELVariable> variableMap = new HashMap<String,BPELVariable>();
			variableMap.put(pinSetName, variable);
			registry.put(targetRule, variableMap);
		}
	}

	public static BPELVariable getFaultLinkVariable(TransformationContext context,TransformationRule targetRule,String pinSetName) {
		BPELVariable variable = null;
		if(context== null || pinSetName == null){
			return variable;
		}
		Map<TransformationRule,Map<String,BPELVariable>> registry = getFaultLinkVarRegistry(context);	
		if(registry != null){
			if(registry.get(targetRule) instanceof Map){
				variable = registry.get(targetRule).get(pinSetName);
			}
		}
		return variable;
	}

	/**
	 * This method is to create assign to fork multiple branches of a fault link, meanwhile, the fault link variable
	 * property will be copied towards the respective variables for the subsequent activities to consume 
	 * @param context
	 * @param containerActivityRule
	 * @param srcSet
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static Assign createAssign(TransformationContext context,TransformationRule containerActivityRule, PinSet srcSet){
		// Creates BPEL Assign 
		BPELVariable srcCon = null;
		BPELVariable tarCon = null;
		Assign assign = BPELFactory.eINSTANCE.createAssign();
		if(srcSet instanceof OutputPinSet){
			OutputPinSet ops = (OutputPinSet)srcSet;
			if(ops.getIsException().booleanValue()){
				List pinList = BomUtils.getPinsInSet(ops);
				if(pinList != null && pinList.size() > 0){
					Iterator pinIt = pinList.iterator();
					ObjectPin srcPin = null;
					//					ObjectPin tarPin = null;					
					while(pinIt.hasNext()){
						Object eachPin = pinIt.next();
						if(eachPin instanceof OutputObjectPin){
							srcPin = (OutputObjectPin)eachPin;							
							if(srcPin.getOutgoing() != null && (srcPin.getOutgoing().getTarget() instanceof ObjectPin ||srcPin.getOutgoing().getTarget() instanceof TerminationNode)){
								//								tarPin = (ObjectPin)srcPin.getOutgoing().getTarget();	

								// Creates BPEL Copy and set to the Assign								
								srcCon = (BPELVariable)CBPELUtil.getBPELVariableForFaultPinSet(context, containerActivityRule, ops.getName());
								Part srcPart = findPart(srcSet,srcPin,srcCon);								

								//Find out the target pinset														
								//								PinSet tarPinSet = BomUtils.findPinSetForPin(tarPin);
								//								NamedElement action = null;
								//								if(tarPinSet instanceof InputPinSet){
								//									action = ((InputPinSet)tarPinSet).getAction();
								//								}else if(tarPinSet instanceof OutputPinSet){
								//									action = ((OutputPinSet)tarPinSet).getAction();
								//								}
								//We don't need the target part
								//MessageRule msgRule = (MessageRule)TransformationUtil.getRuleForSource(ProcessUtil.getProcessInterfaceRule(context,((InputPinSet)tarPinSet).getAction()), MessageRule.class, tarPinSet);
								//								OutputPinSet interfaceOPS = null;
								//								if(tarPinSet instanceof OutputPinSet){
								//									interfaceOPS = ProcessUtil.getInterfaceActivityOutputPinSetForProcessIPS((OutputPinSet)tarPinSet);
								//								}
								//								MessageRule msgRule = null;
								//								if(interfaceOPS == null){
								//									msgRule = (MessageRule)TransformationUtil.getRuleForSource(ProcessUtil.getProcessInterfaceRule(context,action), MessageRule.class, tarPinSet);
								//								}else{
								//									msgRule = (MessageRule)TransformationUtil.getRuleForSource(ProcessUtil.getProcessInterfaceRule(context,interfaceOPS.getAction()), MessageRule.class, interfaceOPS);
								//								}
								//								Part tarPart = null;
								//								if(msgRule != null && !msgRule.getTarget().isEmpty()){
								//									tarPart = (Part)((Message)msgRule.getTarget().get(0)).getOrderedParts(null).get(0);
								//								}

								//Find the target variable, which is the same as the source object pin's varibale
								TransformationRule defRule = ProcessUtil.getProcessDefinitionRule(context);
								List<EObject> sources = new ArrayList<EObject>();
								sources.add(srcPin.getType());
								sources.add(srcPin);
								TransformationRule rule = TransformationUtil.getRuleForSource(defRule, VariableRule.class, sources, BPELVariable.class);
								if(rule != null && !rule.getTarget().isEmpty()){
									tarCon = (BPELVariable)rule.getTarget().get(0);
								}

								Copy copy = BPELFactory.eINSTANCE.createCopy();
								From from = BPELFactory.eINSTANCE.createFrom();
								To to = BPELFactory.eINSTANCE.createTo();
								from.setVariable(srcCon);

								String queryFromString = createQueryString(srcSet, srcPin, srcPart==null?null:srcPart.getElementDeclaration());
								if(queryFromString != null && !queryFromString.equals("")){
									Query queryFrom = BPELFactory.eINSTANCE.createQuery();									
									queryFrom.setValue(queryFromString);
									queryFrom.setQueryLanguage(QUERY_LANGUAGE);
									from.setQuery(queryFrom);
									if(srcPart != null){
										from.setPart(srcPart);
									}
								}else{
									if(srcPart != null){
										from.setPart(srcPart);
									}
								}

								to.setVariable(tarCon);
								//								String queryToString = createQueryString(tarPinSet, tarPin, tarPart==null?null:tarPart.getElementDeclaration());
								//								if(queryToString != null && !queryToString.equals("")){
								//									Query queryTo = BPELFactory.eINSTANCE.createQuery();									
								//									queryTo.setValue(queryToString);
								//									queryTo.setQueryLanguage(QUERY_LANGUAGE);
								//We don't need the query and part name since all the variable are global
								//									to.setQuery(queryTo);
								//									if(tarPart != null){
								//										to.setPart(tarPart);
								//									}
								//							}

								copy.setFrom(from);
								copy.setTo(to);
								assign.getCopy().add(copy);														
							}
						}
					}					
				}
			}
		}	
		return assign;
	}


	/**
	 * This method is to create assign to fork multiple branches of a fault link, meanwhile, the fault link variable
	 * property will be copied towards the respective variables for the subsequent activities to consume, this API is
	 * for scope 
	 * @param context	 
	 * @param srcSet
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static Assign createAssign(TransformationContext context, PinSet srcSet){
		// Creates BPEL Assign 
		BPELVariable srcCon = null;
		BPELVariable tarCon = null;
		Assign assign = BPELFactory.eINSTANCE.createAssign();
		if(srcSet instanceof OutputPinSet){
			OutputPinSet ops = (OutputPinSet)srcSet;
			if(ops.getIsException().booleanValue()){
				List pinList = BomUtils.getPinsInSet(ops);
				if(pinList != null && pinList.size() > 0){
					Iterator pinIt = pinList.iterator();
					ObjectPin srcPin = null;
					//					ObjectPin tarPin = null;					
					while(pinIt.hasNext()){
						Object eachPin = pinIt.next();
						if(eachPin instanceof OutputObjectPin){
							srcPin = (OutputObjectPin)eachPin;							
							if(srcPin.getOutgoing() != null && (srcPin.getOutgoing().getTarget() instanceof ObjectPin ||srcPin.getOutgoing().getTarget() instanceof TerminationNode)){
								//								tarPin = (ObjectPin)srcPin.getOutgoing().getTarget();	

								// Creates BPEL Copy and set to the Assign								
								srcCon = ScopeRegistryUtil.getBpelVariable(context, ops);//(BPELVariable)CBPELUtil.getBPELVariableForFaultPinSet(context, containerActivityRule, ops.getName());
								Part srcPart = findPart(srcSet,srcPin,srcCon);								

								//Find out the target pinset														
								//								PinSet tarPinSet = BomUtils.findPinSetForPin(tarPin);
								//								NamedElement action = null;
								//								if(tarPinSet instanceof InputPinSet){
								//									action = ((InputPinSet)tarPinSet).getAction();
								//								}else if(tarPinSet instanceof OutputPinSet){
								//									action = ((OutputPinSet)tarPinSet).getAction();
								//								}
								//								MessageRule msgRule = (MessageRule)TransformationUtil.getRuleForSource(ProcessUtil.getProcessInterfaceRule(context,((InputPinSet)tarPinSet).getAction()), MessageRule.class, tarPinSet);
								//								MessageRule msgRule = (MessageRule)TransformationUtil.getRuleForSource(ProcessUtil.getProcessInterfaceRule(context,action), MessageRule.class, tarPinSet);
								//								Part tarPart = null;
								//								if(msgRule != null && !msgRule.getTarget().isEmpty()){
								//									tarPart = (Part)((Message)msgRule.getTarget().get(0)).getOrderedParts(null).get(0);
								//								}

								//Find the target variable, which is the same as the source object pin's varibale
								TransformationRule defRule = ProcessUtil.getProcessDefinitionRule(context);
								List<EObject> sources = new ArrayList<EObject>();
								sources.add(srcPin.getType());
								sources.add(srcPin);
								TransformationRule rule = TransformationUtil.getRuleForSource(defRule, VariableRule.class, sources, BPELVariable.class);
								if(rule != null && !rule.getTarget().isEmpty()){
									tarCon = (BPELVariable)rule.getTarget().get(0);
								}

								Copy copy = BPELFactory.eINSTANCE.createCopy();
								From from = BPELFactory.eINSTANCE.createFrom();
								To to = BPELFactory.eINSTANCE.createTo();
								from.setVariable(srcCon);

								String queryFromString = createQueryString(srcSet, srcPin, srcPart==null?srcCon.getXSDElement():srcPart.getElementDeclaration());
								if(queryFromString != null && !queryFromString.equals("")){
									Query queryFrom = BPELFactory.eINSTANCE.createQuery();									
									queryFrom.setValue(queryFromString);
									queryFrom.setQueryLanguage(QUERY_LANGUAGE);
									from.setQuery(queryFrom);	
									if(srcPart != null){
										from.setPart(srcPart);
									}
								}else{
									if(srcPart != null){
										from.setPart(srcPart);
									}
								}

								to.setVariable(tarCon);
								//								String queryToString = createQueryString(tarPinSet, tarPin, tarPart==null?tarCon.getXSDElement():tarPart.getElementDeclaration());
								//								if(queryToString != null && !queryToString.equals("")){
								//									Query queryTo = BPELFactory.eINSTANCE.createQuery();									
								//									queryTo.setValue(queryToString);
								//									queryTo.setQueryLanguage(QUERY_LANGUAGE);
								//									to.setQuery(queryTo);
								//									if(tarPart != null){
								//										to.setPart(tarPart);
								//									}
								//								}

								copy.setFrom(from);
								copy.setTo(to);
								assign.getCopy().add(copy);														
							}
						}
					}					
				}
			}
		}	
		return assign;
	}


	private static Part findPart(PinSet set,Pin pin, BPELVariable con){

		Part part = null;		
		// Get the WSDL message from container
		if(con == null){
			return part;
		}		
		Message msg = con.getMessageType();	
		if(msg != null){			
			part = (Part)msg.getOrderedParts(null).get(0);
		}
		return part;
	}

	/**
	 * TODO Handle the case of non-wrapper complex type case- multiple parts message, which is not 
	 * supported by SOAP
	 * @param set
	 * @param pin
	 * @param part
	 * @return
	 */
	private static String createQueryString( PinSet set, Pin pin, XSDElementDeclaration elementDeclaration){
		if(elementDeclaration == null){      
			return "";
			//this is the case of a single Pin in the PinSet when we did not generate the wrapper complex type
		}else if(!BpelOptionsUtil.isAlwaysGenerateWraperComplexType() && BomUtils.getPinsInSet(set) != null && BomUtils.getPinsInSet(set).size()== 1){
			return "";
		}		
		String pinElementDeclaration = getPinElementName(set, pin, elementDeclaration);		
		return  pinElementDeclaration;
	}

	@SuppressWarnings("unchecked")
	private static String getPinElementName(PinSet set, Pin pin, XSDElementDeclaration elementDeclaration ){
		//		XSDElementDeclaration elementDeclaration = part.getElementDeclaration();
		if(elementDeclaration.getTypeDefinition() instanceof XSDSimpleTypeDefinition){
			//The simple type name is not valid in the XPath
//			return ((XSDSimpleTypeDefinition)elementDeclaration.getTypeDefinition()).getName();
			return "";
		}
		XSDComplexTypeDefinition definition = (XSDComplexTypeDefinition)elementDeclaration.getTypeDefinition();
		XSDParticle particle = (XSDParticle)definition.getContent();
		XSDModelGroup modelGroup = (XSDModelGroup)particle.getContent();
		List elementDeclarations = modelGroup.getContents();
		//If the pin represents a single element, we don't create query string
		if(elementDeclarations != null && set instanceof InputPinSet){ //elementDeclarations.size()== 1 && set instanceof InputPinSet){
			return "";
		}
		List pins = BomUtils.getPinsInSet(set);
		int ind = pins.indexOf(pin);

		XSDParticle pinsParticle = (XSDParticle)elementDeclarations.get(ind);
		XSDElementDeclaration  pinElementDeclaration = (XSDElementDeclaration) pinsParticle.getContent();
		return pinElementDeclaration.getName();
	}

	/**
	 * Create fault sources for the target rule's target, which is a BPEL activity, by iterating through the fault source map
	 * keyed by the pinSet names of the BOM action
	 * @param targetRule
	 * @param faultSourceMap
	 */
	@SuppressWarnings("unchecked")
	public static void createFaultSourcesForBPELActivity(TransformationRule targetRule,	Map<String, List<Object>> faultSourceMap) {
		if(targetRule != null && !targetRule.getTarget().isEmpty()){
			//the first target should be the invoke activity
			if(targetRule.getTarget()== null || 
					targetRule.getTarget().isEmpty() || 
					!(targetRule.getTarget().get(0) instanceof Activity)||
					faultSourceMap == null){
				return;
			}
			Activity activity = (Activity)targetRule.getTarget().get(0);
			//Create FaultSources
			//			FaultSources faultSources = BPELPlusFactory.eINSTANCE.createFaultSources();
			Collection values = faultSourceMap.values();
			List<Source> removableLinks = new ArrayList<Source>();
			if(values != null && !values.isEmpty()){
				for(Object each:values){
					FaultSource faultSource = BPELPlusFactory.eINSTANCE.createFaultSource();					
					if(each instanceof List){						
						Catch eachCatch = BPELPlusFactory.eINSTANCE.createCatch();
						List eachSource = (List)each;
						for(Object theSource:eachSource){
							if(theSource instanceof Link){
								Link link = (Link)theSource;	
								faultSource.setLink(link);
								EList regularSourceLinks = activity.getSources().getChildren();
								if(regularSourceLinks != null && !regularSourceLinks.isEmpty()){
									for(Object regularSource:regularSourceLinks){
										if(regularSource instanceof Source){
											Link eachLink  = ((Source)regularSource).getLink();
											if(link.getName()!= null && link.getName().equals(((Link)eachLink).getName())){
												removableLinks.add((Source)regularSource);
												break;
											}
										}
									}
									//This is to handle the propagated fault links, so we add the source to
									//be removable anyways	
									Source faultLinkSrc = (Source)link.getSources().get(0);
									if(!removableLinks.contains(faultLinkSrc)){
										removableLinks.add(faultLinkSrc);
									}
								}
							}else if(theSource instanceof QName){
								//								Message message = (Message)theSource;
								eachCatch.setFaultName((QName)theSource);
							}else if(theSource instanceof BPELVariable){
								BPELVariable var = (BPELVariable)theSource;
								//								eachCatch.setFaultVariable(var.getName());
								eachCatch.setFaultVariable(var);
							}
						}
						faultSource.setCatch(eachCatch);
					}
					//we skip the fault source w/o connection
					if(faultSource.getLink() != null && activity.getSources()!= null){
						//						faultSources.getFaultSource().add(faultSource);
						activity.getSources().getChildren().add(faultSource);
					}
				}
			}	
			//			if(activity.getExtensibilityElements() != null && !faultSources.getFaultSource().isEmpty()){				
			//				//Add the fault sources as the last extensibility element for the invoke activity
			//				activity.getExtensibilityElements().add(faultSources);				
			//			}
			//remove fault links from the regular sources
			if(!removableLinks.isEmpty()){
				//				activity.getSources().getChildren().removeAll(removableLinks);
				//we need to defer the removal
				registerRemovableLinks(targetRule.getRoot().getContext(),activity,removableLinks);
			}
		}
	}

	private static void registerRemovableLinks(TransformationContext context,Activity activity, List<Source> removableLinks) {		
		if(context == null || activity == null || removableLinks == null ){
			return;
		}
		Map<Activity,List<Source>> registry = getFaultLinkRegistry(context);
		registry.put(activity, removableLinks);
	}

	/**
	 * This method should be called in the last step of the optimization rule
	 * @param context
	 * @param activity
	 */
	@SuppressWarnings("unchecked")
	private static void consolidateFaultLinkSources(TransformationContext context,Activity activity){
		if(activity == null || context == null){
			return;
		}
		Map<Activity,List<Source>> registry = getFaultLinkRegistry(context);
		List<Source> removableSources = registry.get(activity);
		if(removableSources != null && !removableSources.isEmpty()){
			activity.getSources().getChildren().removeAll(removableSources);
		}
	}

	/**
	 * Find the container activity of the output
	 * @param opr
	 * @return
	 */
	public static TransformationRule findContainerBPELActivityRule(OutputRule opr){		
		if(opr == null){
			return null;
		}
		if(opr.getParentRule().getTarget() != null &&  !opr.getParentRule().getTarget().isEmpty()){
			if(opr.getParentRule().getTarget().get(0) instanceof Activity){
				return opr.getParentRule();
			}
		}else if(opr.getParentRule() instanceof InvokeRule){
			return opr.getParentRule();
		}
		return null;
	}
	/**
	 * Override to harvest the links, fault messages and variables needed to populate the 
	 * faultSources of the invoke to enable fault link support
	 * Example:
	 * <bpel:invoke name=thisInvoke ...>
   			...
   		<bpel:sources wpc:type=split>
      			...
   		</bpel:sources>
   		<wpc:faultSources>
      		<wpc:faultSource linkName=faultLink1>
         		<wpc:catch faultName=tns:MyFault1 faultVariable=FaultVariable1/>         		
       		</wpc:faultSource>
      		<wpc:faultSource linkName=faultLink2>
         		<wpc:catchAll/>
      		</wpc:faultSource>
   		</wpc:faultSources>
		</bpel:invoke>
	 * @param context
	 * @param bpelActivityTransformationRule The BPEL activity transformation rule such as InvokeRule
	 * @param portTypeRule
	 */
	public static void processFaultSources(TransformationContext context,TransformationRule bpelActivityTransformationRule, TransformationRule portTypeRule){		
		//system print out directive		
		boolean debug = false;
		//look up the fault link registry
		Map<TransformationRule,Map<String, List<Object>>> faultSourceMap = CBPELUtil.getFaultSourcesMap(context);
		List<TransformationRule> operationRuleList = null;
		if(portTypeRule instanceof com.ibm.btools.te.ilm.heuristics.wsdl.PortTypeRule){
			//native interface
			com.ibm.btools.te.ilm.heuristics.wsdl.PortTypeRule extPortTypeRule = (com.ibm.btools.te.ilm.heuristics.wsdl.PortTypeRule)portTypeRule;
			operationRuleList = findOperationRulesForPortType(extPortTypeRule);
		}else if(portTypeRule instanceof com.ibm.btools.te.ilm.heuristics.extservice.PortTypeRule){
			//external service
			//Retrieve all the child fault message rules and get their targets which are fault messages
			com.ibm.btools.te.ilm.heuristics.extservice.PortTypeRule extPortTypeRule = (com.ibm.btools.te.ilm.heuristics.extservice.PortTypeRule)portTypeRule;
			operationRuleList = findOperationRulesForPortType(extPortTypeRule);
		}
		if(operationRuleList != null){
			for(TransformationRule opRule:operationRuleList){
				if(opRule.getSource().get(0) instanceof InputPinSet && opRule.getTarget().get(0) instanceof Operation){
					InputPinSet ips = (InputPinSet)opRule.getSource().get(0);
					EList opsList = ips.getOutputPinSet();
					if(opsList != null && !opsList.isEmpty()){
						for(Object eachOPS:opsList){
							if(eachOPS instanceof OutputPinSet){
								OutputPinSet ops = (OutputPinSet)eachOPS;
								if(ops.getIsException() != null && ops.getIsException().booleanValue()){
									//Look-up the fault variable
									BPELVariable faultVariable = CBPELUtil.getFaultLinkVariable(context,bpelActivityTransformationRule, ops.getName());
									registerFaultSource(context, bpelActivityTransformationRule,ops.getName(), faultVariable);
									//Look-up the fault QName on the operation
									Operation operation = (Operation)opRule.getTarget().get(0);
									EList faultList = operation.getEFaults();
									QName faultQName = resolveFaulQNametForFaultPinSet(faultList,ops,portTypeRule);
									if(faultQName != null){
										registerFaultSource(context, bpelActivityTransformationRule,ops.getName(), faultQName);
									}
								}
							}
						}
					}					
				}
			}
		}
		createFaultSourcesForBPELActivity(bpelActivityTransformationRule, faultSourceMap.get(bpelActivityTransformationRule));
		if(debug){
			printFaultSources(faultSourceMap.get(bpelActivityTransformationRule), bpelActivityTransformationRule);		
		}
	}

	/**
	 * Resolve the fault QName on the operation corresponding to the fault pin set associated with the input pin set
	 * that is the source of the operation
	 * @param faultList
	 * @param ops
	 * @param operationRule
	 * @return
	 */
	private static QName resolveFaulQNametForFaultPinSet(EList faultList,OutputPinSet ops,TransformationRule portTypeRule) {
		QName faultQName = null;
		if(ops == null || ops == null || portTypeRule == null){
			return faultQName;
		}
		List<MessageRule> msgRuleList = findMessageRulesForPortType(portTypeRule);
		Message targetMsg = null;
		if(msgRuleList != null){
			for(MessageRule eachMsgRule:msgRuleList){
				if(eachMsgRule.getSource().get(0) instanceof OutputPinSet){
					OutputPinSet srcOPS = (OutputPinSet)eachMsgRule.getSource().get(0);
					if(srcOPS != null && srcOPS == ops && srcOPS.getIsException() != null && srcOPS.getIsException().booleanValue()){
						targetMsg = (Message)eachMsgRule.getTarget().get(0);
						break;
					}
				}
			}
		}
		Fault targetFault = null;
		for(Object eachFault:faultList){
			if(eachFault instanceof Fault){
				Fault theFault = (Fault)eachFault;
				if(theFault.getEMessage() == targetMsg){
					//We find the right fault on the operation
					targetFault = theFault;
					break;
				}
			}
		}
		//We use the portType namespace uri as the fault QName namespace uri
		PortType pt = null;
		if(portTypeRule.getTarget() != null && !portTypeRule.getTarget().isEmpty()){
			if(portTypeRule.getTarget().get(0) instanceof PortType){
				pt = (PortType)portTypeRule.getTarget().get(0);
			}
		}
		//The fault QName's namespace URI should be the same as the fault message's
		if(targetFault != null && pt != null){
			faultQName = new QName(pt.getQName().getNamespaceURI(),targetFault.getName());
		}
		return faultQName;
	}

	/**
	 * Analyze the output pin set to determine if a fault link handling needed
	 * @param ips
	 * @return
	 */
	public static boolean shouldHanleFaultLink(InputPinSet ips){
		if(ips == null){
			return false;
		}
		if(ips.getOutputPinSet()!= null && ips.getOutputPinSet().size() > 1){
			for(Object eachPS:ips.getOutputPinSet()){
				if(eachPS instanceof OutputPinSet){
					OutputPinSet ops = (OutputPinSet)eachPS;
					if(ops.getIsException() != null && ops.getIsException().booleanValue()){
						//handle extreme case in which no pins exist
						List pinList = BomUtils.getPinsInSet(ops);
						if(pinList == null ||pinList.isEmpty()){
							continue;
						}
						return true;
					}
				}
			}
		}
		return false;
	}

	/**
	 * Create wpc:type for the gateway
	 * E.G.
	 * BPEL Syntax
		wpc:type=split|merge|fork|join|ior?
	BPEL Example
	<bpel:invoke name=Invoke1 ...>
   		<bpel:targets wpc:type=join>
      		...
   		</bpel:targets>
   		<bpel:sources wpc:type=fork>
      		...
   	</bpel:sources>
   ...
	</bpel:flow>
	 * @param bpelActivity
	 * @param typeEnum
	 * @param isSource
	 */
	public static void createGateWayType(Activity bpelActivity,TypeEnum typeEnum,boolean isSource){		
		if(bpelActivity == null || typeEnum == null){
			return;
		}
		Type type = BPELPlusFactory.eINSTANCE.createType();
		type.setType(typeEnum);
		if(!hasTypeAttribute(bpelActivity,isSource)){
			if(isSource){
				if(bpelActivity.getSources() == null){				
					Sources sources = BPELFactory.eINSTANCE.createSources();
					bpelActivity.setSources(sources);
				}
				bpelActivity.getSources().addExtensibilityElement(type);

			}else{
				if(bpelActivity.getTargets() == null){
					Targets targets = BPELFactory.eINSTANCE.createTargets();
					bpelActivity.setTargets(targets);
				}
				bpelActivity.getTargets().addExtensibilityElement(type);

			}
		}
	}

	private static boolean hasTypeAttribute(Activity bpelActivity, boolean isSource){
		if(bpelActivity == null){
			return false;
		}
		if(isSource){
			if(bpelActivity.getSources() != null && bpelActivity.getSources().getExtensibilityElements() != null){
				for(Object eachExtElement:bpelActivity.getSources().getExtensibilityElements()){
					if(eachExtElement instanceof Type){
						return true;
					}
				}
			}
		}else{
			if(bpelActivity.getTargets() != null && bpelActivity.getTargets().getExtensibilityElements() != null){
				for(Object eachExtElement:bpelActivity.getTargets().getExtensibilityElements()){
					if(eachExtElement instanceof Type){
						return true;
					}
				}
			}
		}
		return false;
	}
	/**
	 * This utility will walk through the optimized BPEL process to assign type attributes for the 
	 * <code>sources</code> or <code>targets</code> of a BPEL activity
	 * @param rule TODO
	 * @param activity the optimized BPEL process
	 * @param context TODO
	 */
	@SuppressWarnings("unchecked")
	public static void optimizeActivityTypeAttribute(TransformationRule rule, Activity activity, TransformationContext context){		
		//Type attribute only applicable to GFlow from 6.2 onward
		com.ibm.wbit.bpelpp.Flow flowActivity = null;
		//Top Level Flow
		if(activity instanceof Flow && activity.eContainer() instanceof Process){
			flowActivity = convertFlowToGFlow((Flow)activity, context);
			if(flowActivity != null){
				((Process)activity.eContainer()).setActivity(flowActivity);
			}
		}		
		if(activity instanceof com.ibm.wbit.bpelpp.Flow){
			flowActivity = (com.ibm.wbit.bpelpp.Flow)activity;
		}
		EList childActivities = null;
		if(flowActivity != null){
			childActivities = flowActivity.getActivities();
		}else if(flowActivity == null && activity instanceof Flow){
			childActivities = ((Flow)activity).getActivities();
		}		
		if(childActivities!= null && !childActivities.isEmpty()){
			for(Object childActivity:childActivities){
				//Ignore start-able activities
				if(childActivity instanceof Receive){
					//					continue;
					processActivityTypeAttribute(context,(Receive)childActivity, rule);
				}else if(childActivity instanceof Scope){
					processActivityTypeAttribute(context,(Scope)childActivity, rule);
					//nested flow for sub-process
					Scope childScope = (Scope)childActivity;
					if(childScope.getActivity() instanceof Flow){
						com.ibm.wbit.bpelpp.Flow scopeGFlow = convertFlowToGFlow((Flow)childScope.getActivity(), context);
						if(scopeGFlow != null){
							childScope.setActivity(scopeGFlow);
							optimizeActivityTypeAttribute(rule, scopeGFlow, context);
						}else{
							optimizeActivityTypeAttribute(rule, (Flow)childScope.getActivity(), context);
						}
					}else if(childScope.getActivity() instanceof While){
						While scopeWhile = (While)childScope.getActivity();
						//This is the case of generalized scope for the while
						processActivityTypeAttribute(context,scopeWhile, rule);
						//nested flow for loop node
						//While whileConstruct = (While)childActivity;
						com.ibm.wbit.bpelpp.Flow whileGFlow = convertFlowToGFlow((Flow)scopeWhile.getActivity(), context);
						if(whileGFlow != null){
							scopeWhile.setActivity(whileGFlow);
							optimizeActivityTypeAttribute(rule, whileGFlow, context);
						}else{
							optimizeActivityTypeAttribute(rule, (Flow)scopeWhile.getActivity(), context);
						}
					}

				}else if (childActivity instanceof While){
					processActivityTypeAttribute(context,(While)childActivity, rule);
					//nested flow for loop node
					While whileConstruct = (While)childActivity;
					com.ibm.wbit.bpelpp.Flow whileGFlow = convertFlowToGFlow((Flow)whileConstruct.getActivity(), context);
					if(whileGFlow != null){
						whileConstruct.setActivity(whileGFlow);
						optimizeActivityTypeAttribute(rule, whileGFlow, context);
					}else{
						optimizeActivityTypeAttribute(rule, (Flow)whileConstruct.getActivity(), context);
					}

				}else if(childActivity instanceof Switch){
					//This is the case that the user choose to generate switch activity for multiple decision gateway
					handleSwitchConstructOptimization((Switch)childActivity,context,rule);
				}else if(childActivity instanceof Pick){
					processActivityTypeAttribute(context,(Pick)childActivity, rule);
					Pick pickAct = (Pick)childActivity;		
					if(pickAct.getMessages()!= null && !pickAct.getMessages().isEmpty()){
						for(Object message:pickAct.getMessages()){
							if(message instanceof OnMessage){
								OnMessage onMsg = (OnMessage)message;
								if(onMsg.getActivity() instanceof Flow){
									com.ibm.wbit.bpelpp.Flow onMessageGFlow = convertFlowToGFlow((Flow)onMsg.getActivity(), context);
									if(onMessageGFlow != null){
										onMsg.setActivity(onMessageGFlow);
										optimizeActivityTypeAttribute(rule, onMessageGFlow, context);
									}else{
										optimizeActivityTypeAttribute(rule, (Flow)onMsg.getActivity(), context);
									}									
								}
							}
						}
					}
				}
				else{
					//atomic activities
					processActivityTypeAttribute(context,(Activity)childActivity, rule);
				}
			}
		}	
	}

	/**
	 * Go through each case flow to see if there is any multiple sources or targets need to have a gateway type
	 * @param childActivity
	 */
	private static void handleSwitchConstructOptimization(Switch childActivity,TransformationContext context,TransformationRule rule) {
		if(childActivity != null){
			processActivityTypeAttribute(context,childActivity, rule);
			EList caseList = childActivity.getCases();
			if(caseList != null && !caseList.isEmpty()){
				for(Object eachCase:caseList){
					if(eachCase instanceof Case){
						Case branch = (Case)eachCase;
						if(branch.getActivity() instanceof Flow){							
							com.ibm.wbit.bpelpp.Flow caseGFlow = convertFlowToGFlow((Flow)branch.getActivity(), context);
							if(caseGFlow != null){
								branch.setActivity(caseGFlow);
								optimizeActivityTypeAttribute(rule, caseGFlow, context);
							}else{
								optimizeActivityTypeAttribute(rule, (Flow)branch.getActivity(), context);
							}							
						}
					}
				}
			}
		}

	}

	/**
	 * Convert the normal flow to GFlow
	 * @param activity
	 * @param scopeGFlow
	 */
	@SuppressWarnings("unchecked")
	private static com.ibm.wbit.bpelpp.Flow convertFlowToGFlow(Flow activity, TransformationContext context) {
		if(!CBPELUtil.isCyclicFlowSupported()){
			return null;
		}
		com.ibm.wbit.bpelpp.Flow scopeGFlow = BPELPlusFactory.eINSTANCE.createFlow();
		if(activity != null){
			scopeGFlow.setName(activity.getName());
			scopeGFlow.getExtensibilityElements().addAll(activity.getExtensibilityElements());
			if (activity.getLinks() != null &&
					activity.getLinks().eContents() != null &&
					!activity.getLinks().eContents().isEmpty()) {
				scopeGFlow.setLinks(activity.getLinks());
			}
			if(activity.getSources() != null){
				scopeGFlow.setSources(activity.getSources());
			}
			if(activity.getTargets() != null){
				scopeGFlow.setTargets(activity.getTargets());		
			}
			scopeGFlow.getActivities().addAll(activity.getActivities());		
		}

		//Convert flow mappings to gflow mappings
		Map<EObject, EObject> flowMapping = CBPELMappingUtil.getFlowMapping(context);
		for(EObject key: flowMapping.keySet()) {
			if (flowMapping.get(key) == activity) {
				flowMapping.put(key, scopeGFlow);
			}
		}
		CBPELUtil.createGeneratedAttribute(scopeGFlow, CBPELUtil.GEN_BPC_REGION_BOUNDARY);
		return scopeGFlow;
	}

	@SuppressWarnings("unchecked")
	private static void processActivityTypeAttribute(TransformationContext context,Activity childActivity, TransformationRule rule) {
		//First get rid of the fault links for regular sources
		consolidateFaultLinkSources(context, childActivity);
		//Set the gateway types
		if(CBPELUtil.isCyclicFlowSupported()){
			if(childActivity != null){
				if(childActivity.getTargets()!= null){
					EList incomingLinks = childActivity.getTargets().getChildren();
					if(incomingLinks != null && incomingLinks.size() > 1){					        			
						//join condition is on the targets of the activity
						if(existJoinConditionOnActivity(childActivity)){
							//we handle the reply gateway using "ior" generically, need to verify with the
							//runtime team whether this semantic works for all cases
							if(childActivity instanceof Reply){
								//type of "ior"
								createGateWayType(childActivity, TypeEnum.IOR_LITERAL, false);		
							}else{
								if(isLogicalAND(childActivity)){
									//type of "join"
									createGateWayType(childActivity, TypeEnum.JOIN_LITERAL, false);									
								}else{
									//type of "merge"
									createGateWayType(childActivity, TypeEnum.MERGE_LITERAL, false);									
								}
							}
							removeJoinCondition(childActivity);
						}else{
							if(childActivity instanceof Scope){
								createGateWayType(childActivity, TypeEnum.MERGE_LITERAL, false);		
							}
						}
					}
				}
				if(isIORDecisionEmpty(rule,context,childActivity)){
					//type of "IOR"
					createGateWayType(childActivity, TypeEnum.IOR_LITERAL, true);
					return;
				}
				if(childActivity.getSources() != null){					
					EList outgoingLinks = childActivity.getSources().getChildren();
					//					if(outgoingLinks != null && outgoingLinks.size() > 1){
					if(isMultipleRegularSource(outgoingLinks)){
						//we need add type attribute for the sources
						boolean existTransitionCondition = false;					
						for(Object eachSource:outgoingLinks){
							if(eachSource instanceof Source){
								if(existTransitionCondition((Source)eachSource)){
									//this means at least one transition condition exists
									existTransitionCondition = true;
									break;
								}							
							}
						}			
						if(!existTransitionCondition){
							//type of "fork"
							createGateWayType(childActivity, TypeEnum.FORK_LITERAL, true);
							//20081109 - wait for valid executable region						} else if (BpelOptimizationUtil.getNonOptimizableElements(context).contains(childActivity)) {							
							//type of "IOR split" - local process/loop - case when multiple pin sets and multiple pins in these sets needs IOR
							//							createGateWayType(childActivity, TypeEnum.IOR_LITERAL, true);
						}else{
							//type of "split"
							createGateWayType(childActivity, TypeEnum.SPLIT_LITERAL, true);
						}
					}
				}
			}
		}		
	}

	private static void removeJoinCondition(Activity activity) {
		if(activity == null){
			return;
		}
		activity.getTargets().setJoinCondition(null);
	}

	private static boolean existJoinConditionOnActivity(Activity activity){
		if(activity != null){
			if(activity.getTargets().getJoinCondition() instanceof Condition){
				return true;
			}
		}
		return false;
	}

	/**
	 * Check the type of the join condition
	 * TODO revisit the logic
	 * @param element
	 * @return
	 */
	private static boolean isLogicalAND(Object element) {
		if(element instanceof Activity){
			Activity activity = (Activity)element;
			if(activity.getTargets().getJoinCondition() instanceof Condition){
				Condition condition = activity.getTargets().getJoinCondition();
				String javaStr = null;
				if(condition.getBody() instanceof String){
					javaStr = (String)condition.getBody();
				}
				if(javaStr != null && 
						(javaStr.indexOf(JavaConstants.AND_OP) != -1 || javaStr.indexOf(JavaConstants.AND_OP.trim()) != -1) && 
						javaStr.indexOf(JavaConstants.OR_OP) == -1 || javaStr.indexOf(JavaConstants.OR_OP.trim()) == -1){
					return true;
				}else if(javaStr != null && (javaStr.indexOf(JavaConstants.OR_OP) != -1 || javaStr.indexOf(JavaConstants.OR_OP.trim()) != -1)){
					return false;
				}
			}
		}
		return false;
	}

	/**
	 * Check if there is transition condition on the source
	 * @param transitionConditionElement
	 * @return
	 */
	private static boolean existTransitionCondition(ExtensibleElement transitionConditionElement){
		if(transitionConditionElement instanceof Source){
			Source source = (Source)transitionConditionElement;
			if(source.getTransitionCondition() instanceof Condition){
				return true;
			}
		}
		return false;
	}

	public static boolean isMultipleRegularPinSet(Action action){
		if(BomUtils.isLocalProcess(action)||action instanceof LoopNode){
			EList opsList = action.getOutputPinSet();
			int i=0;
			if(opsList != null && !opsList.isEmpty()){
				for(Object eachOPS:opsList){
					if(eachOPS instanceof OutputPinSet){
						OutputPinSet ops = (OutputPinSet)eachOPS;
						if(!ops.getIsException().booleanValue()){
							i++;
						}
					}
				}
			}
			if(i > 1){
				return true;
			}
		}
		return false;
	}

	/**
	 * Flag whether we need to support cyclic flow semantics
	 * We will do the following:
	 * 1. Assign type attributes for activities which sources and targets with more than one links
	 * 2. Convert the BPEL Flow into Generalized Flow (GFlow)
	 * @return
	 */
	public static boolean isCyclicFlowSupported() {
		// If there is a loop back flow in a process model or fault link exists
		return true;
		//		return !BpelOptionsUtil.isGenerateForDeployPatternEnabled();
	}

	/**
	 * According to the latest BPEL model serialization scheme, we don't have to explicitly deal with
	 * the fault sources, instead, we add the fault source into the regular sources of the activity,the
	 * fault source serializer will handle the model properly   
	 * @param scope
	 * @return
	 */
	public static Sources getSources(Activity activity){
		//		FaultSources faultSources = null;
		//		if(activity != null){
		//			if(activity.getEExtensibilityElements()!= null && !activity.getEExtensibilityElements().isEmpty()){
		//				for(Object eachExtElement:activity.getEExtensibilityElements()){
		//					if(eachExtElement instanceof FaultSources){
		//						faultSources = (FaultSources)eachExtElement;
		//						break;
		//					}
		//				}
		//			}				
		//		}	
		//		if(faultSources == null){
		//			faultSources = BPELPlusFactory.eINSTANCE.createFaultSources();
		//			activity.addExtensibilityElement(faultSources);
		//		}
		//		return faultSources;
		Sources sources = null;
		if(activity != null){
			if(activity.getSources() != null){
				sources = activity.getSources();
			}else{
				sources = BPELFactory.eINSTANCE.createSources();
				activity.setSources(sources);
			}
		}
		return sources;
	}

	/**
	 * This method is responsible for generating generated attribute for bpel elements including Activities, Links, and
	 * BEPLVariables
	 *TODO: Identify the BPEL elements to be tagged as "Generated" to facilitate the BPEL viewer to mirror the original
	 *process before transformation
	 *@param extensibleElement
	 * @param value
	 */
	@SuppressWarnings("unchecked")
	public static void createGeneratedAttribute(ExtensibleElement extensibleElement,String value){
		boolean shouldTagGenAttribute = false;
		if(shouldTagGenAttribute){
			if((extensibleElement instanceof Activity  || extensibleElement instanceof Link) && value != null){
				Generated genAttribute = BPELPlusFactory.eINSTANCE.createGenerated();
				genAttribute.setGenerated(value);
				extensibleElement.getExtensibilityElements().add(genAttribute);
			}
		}
	}

	/**
	 * Create custom property for the process to identify the originality of the BPEL process
	 * CBPEL Specification:
	 * -------------------
	 * Property Name:	generatedBy
	 * ----------------------------
	 *  Property Values:
	 *  ----------------
		bpel  specifies that the process is really based upon BPEL, i.e. originally created 
		with the BPEL editor of WID. A process developed with the BPEL editor of WID should specify 
		this value. For a process imported to WID, but i.e. exported from Modeler, this property 
		should not be changed automatically.
		-------------------------------------------------------------------------------------------
        bpmn  specifies that the BPEL process is generated out of a BPMN model, i.e. for direct 
        deploy from Modeler to WPS. A process developed with Modeler should specify this value.
        -------------------------------------------------------------------------------------------
	    BusinessStateMachine  specifies that the BPEL process is generated out of a Business 
	    State Machine. A process developed with the BSM editor should specify this value.
	    -------------------------------------------------------------------------------------------
	    Other values are allowed but are ignored by BPC. Default behavior applies in this case.

       BPEL Example
			<bpel:process ... xmlns:bpel=... xmlns:wpc=...>
			   <wpc:customProperty name=generatedBy>
			      <wpc:value>bpmn</wpc:value>
			   </wpc:customProperty>
			   ...
			</bpel:process>

	 * @param process
	 * @param name
	 * @param value
	 */
	@SuppressWarnings("unchecked")
	public static void createProcessOriginProperty(Process process){
		if(process != null){
			CustomProperty property = BPELPlusFactory.eINSTANCE.createCustomProperty();
			property.setName(PROCESS_ORIGINALITY_PROPERTY_NAME);
			property.setValue(PROCESS_ORIGINALITY_PROPERTY_VALUE);			
			process.getEExtensibilityElements().add(property);
			//Handle process self invocation case, where the myRole is not defined
			//for the process itself since the cba transformed first and the partner 
			//is reused
			PartnerLinks  plnks = process.getPartnerLinks();
			if(plnks != null && plnks.getChildren().size() == 1){
				PartnerLink plk = (PartnerLink)plnks.getChildren().get(0);
				if(plk.getPartnerRole() != null && plk.getMyRole() == null){
					Role pr = plk.getPartnerRole();
					Role myRole = PartnerlinktypeFactory.eINSTANCE.createRole();	
					myRole.setName(pr.getName());
					plk.setMyRole(myRole);					
				}
			}
		}			
	}

	/**
	 * Due to the BPEL model serialization change for fault source, we have to tell the regular
	 * source and fault source under the activity's sources
	 * @param outgoingLinks
	 * @return
	 */
	@SuppressWarnings("unchecked")
	private static boolean isMultipleRegularSource(EList outgoingLinks){
		if(outgoingLinks != null){
			if(outgoingLinks.size() > 1){
				int regLinkCounter = 0;
				for(Object eachLink:outgoingLinks){
					if(!(eachLink instanceof FaultSource)){
						regLinkCounter++;
					}
				}
				if(regLinkCounter > 1){
					return true;
				}
			}			
		}
		return false;
	}	

	private static boolean isSourceOfLink(Activity act) {
		return !(act.getSources()==null || (act.getSources() != null && act.getSources().getChildren().isEmpty()));
	}
	private static boolean isTargetOfLink(Activity act) {
		return !(act.getTargets()==null || (act.getTargets() != null && act.getTargets().getChildren().isEmpty()));
	}

	/**
	 * This empty has only one incoming link.The following rules apply as a filter for 
	 * @param tar
	 * @param rule TODO
	 * @param context TODO
	 * @see removeActivity(Activity) {@BpelOptimizationRuleImpl.java}
	 * Rules:
	 * ***********************************************************************
	 * If the source activity is an Empty:
	 * 1. An exclusive branch point nested in an exclusive region -> false
	 * 2. A concurrent branch point nested in an exclusive region -> true
	 * 3. An exclusive branch point nested in a concurrent region -> true
	 * 4. A concurrent branch point nested inside a concurrent region -> true
	 * ***********************************************************************
	 * If the source activity is not an Empty:
	 * 5. If the next activity is not an Empty and it has more than one outgoing links -> true
	 * 7. Otherwise -> false
	 * ************************************************************************
	 * 8. If the empty has only one target and one source -> false	 * 
	 * 9. If the empty's outgoing link connects to its source empty activity -> true
	 * @return
	 */
	public static boolean shouldKeepTargetForGateway(Activity empty,Target tar, TransformationRule rule, TransformationContext context) {
		//If the empty has only one target and one source, it should be optimized
		if(empty.getSources() != null && empty.getSources().getChildren().size() == 1){
			return false;
		}

		if(tar != null){
			Link link = tar.getLink();
			if(link.getSources() != null && link.getSources().get(0) instanceof Source){
				Source source = (Source)link.getSources().get(0);
				//If the next activity is not an Empty and it has more than one outgoing links, we need to keep it
				if(!(source.getActivity() instanceof Empty)){
					Activity nonEmptySourceActivity = source.getActivity();
					if(nonEmptySourceActivity.getSources() != null && nonEmptySourceActivity.getSources().getChildren().size() > 1){
						//If the source number is two but one of them is fault source, we can remove the empty
						if(existFaultSource(nonEmptySourceActivity)){
							return false;
						}
						return true;
					}
					//We need to propagate the type attribute if the decision is a inclusive gateway
					if(isIORDecisionEmpty(rule, context, empty)||isEmptyForIor(empty)){
						createGateWayType(nonEmptySourceActivity,TypeEnum.IOR_LITERAL , true);
					}
					return false;
				} else{
					//Ihe source activity is an empty,we need to handle the nested gateway	
					if(empty.getSources() != null && empty.getSources().getChildren().size() > 1){
						for(Object eachSource:empty.getSources().getChildren()){
							if(eachSource instanceof Source){
								Source theSrc = (Source)eachSource;
								Link theLink = theSrc.getLink();
								if(theLink.getTargets()!= null && !theLink.getTargets().isEmpty()){
									if(theLink.getTargets().get(0) == source.getActivity()){
										return false;
									}
								}
							}
						}
					}
					Activity sourceEmpty = source.getActivity();
					if(sourceEmpty.getSources() != null && !sourceEmpty.getSources().getChildren().isEmpty()){
						if(sourceEmpty.getSources().getChildren().size() >1){
							return true;
						}
					}
					//defect 82855 handle the case of a decision node right after a merge node
					if(sourceEmpty.getSources() != null && sourceEmpty.getSources().getChildren().size() == 1){
						Source src = (Source)sourceEmpty.getSources().getChildren().get(0);
						if(src.getLink().getName().equals(link.getName())){
							//We need to propagate the type attribute if the decision is a inclusive gateway
							if(isIORDecisionEmpty(rule, context, empty)||isEmptyForIor(empty)){
								createGateWayType(source.getActivity(),TypeEnum.IOR_LITERAL , true);
							}
							return false;
						}
					}

					Condition outgoingTC = BpelOptimizationUtil.getTransitionCondition(link);
					int tcCounter = 0;
					int numberOfIncomingLinks = 0;					
					if(sourceEmpty.getTargets() != null && !sourceEmpty.getTargets().getChildren().isEmpty()){
						numberOfIncomingLinks = sourceEmpty.getTargets().getChildren().size();
						for(Object eachTarget:sourceEmpty.getTargets().getChildren()){
							if(eachTarget instanceof Target){
								Target targetObj = (Target)eachTarget;
								Link targetLink = targetObj.getLink();
								Condition eachIncomingTC = BpelOptimizationUtil.getTransitionCondition(targetLink);
								if(eachIncomingTC != null){
									tcCounter++;
								}
							}
						}
					}
					if(outgoingTC != null && tcCounter != 0 && tcCounter == numberOfIncomingLinks){
						return false;
					}
					return true;					
				}
			}
		}
		return false;
	}
	/**
	 * Check if the empty gateway carries the IOR type for an inclusive decision
	 * e.g. a process input criterion directly connects to an inclusive decision
	 * @param empty
	 * @return
	 */
	private static boolean isEmptyForIor(Activity empty) {
		if(empty != null && empty.getSources() != null && empty.getSources().getEExtensibilityElements()!= null){
			if(empty.getSources().getEExtensibilityElements().size() > 0){
				EList extElementsList = empty.getSources().getEExtensibilityElements();
				for(Object eachExtElement:extElementsList){
					if(eachExtElement instanceof Type){
						Type type = (Type)eachExtElement;
						if(TypeEnum.IOR_LITERAL.equals(type.getType())){
							return true;
						}
					}
				}
			}
		}
		return false;
	}

	/**
	 * Check if the source activity has fault source and the actual number of 
	 * sources on the activity are two
	 * @param nonEmptySourceActivity
	 * @return
	 */
	private static boolean existFaultSource(Activity nonEmptySourceActivity) {
		if(nonEmptySourceActivity != null && nonEmptySourceActivity.getSources() != null && nonEmptySourceActivity.getSources().getChildren().size()>1){
			boolean hasFaultSource = false;
			List<String> consolidatedLinkList = new ArrayList<String>();
			for(Object eachSource:nonEmptySourceActivity.getSources().getChildren()){
				String linkName = null;
				if(eachSource instanceof FaultSource){
					FaultSource fs = (FaultSource)eachSource;
					hasFaultSource = true;	
					linkName = fs.getLink().getName();					
				}else if(eachSource instanceof Source){
					Source regSource = (Source)eachSource;
					linkName = regSource.getLink().getName();
				}
				if(linkName!= null && !consolidatedLinkList.contains(linkName)){
					consolidatedLinkList.add(linkName);
				}
			}
			if(hasFaultSource && consolidatedLinkList.size() == 2){
				return true;
			}
		}
		return false;
	}

	/**
	 * This empty has only one outgoing link. The following rules apply as a filter for 
	 * @see removeActivity(Activity) {@BpelOptimizationRuleImpl.java}
	 * Rules:
	 * *******************************************************************
	 * If the target activity is an Empty:
	 * 1. An exclusive branch point nested in an exclusive region -> false
	 * 2. A concurrent branch point nested in an exclusive region -> true
	 * 3. An exclusive branch point nested in a concurrent region -> true
	 * 4. A concurrent branch point nested inside a concurrent region -> true
	 * ********************************************************************
	 * If the target activity is not an Empty: 	 * 
	 * 5. If the next activity is not an Empty and it has more than one incoming links -> true
	 * 6. Otherwise -> false
	 * ********************************************************************
	 * 7. If the empty has only one target and one source -> false
	 * 8. If the target empty has more than one incoming links -> true
	 * @param src
	 * @return
	 */
	public static boolean shouldKeepSourceForGateway(Activity empty,Source src) {
		//If the empty has only one target and one source, it should be optimized
		if(empty.getTargets() != null && empty.getTargets().getChildren().size() == 1){
			return false;
		}
		if(src != null){
			Link link = src.getLink();
			if(link.getTargets()!= null && !link.getTargets().isEmpty() && link.getTargets().get(0) instanceof Target){
				Target target = (Target)link.getTargets().get(0);
				if(target.getActivity() instanceof Reply){
					return false;
				}else if(!(target.getActivity() instanceof Empty)){
					Activity nonEmptyActivity = target.getActivity();
					if(nonEmptyActivity.getTargets()!= null && nonEmptyActivity.getTargets().getChildren().size() > 1){
						//The next activity has more than one incoming link, we need to keep it
						return true;
					}
					return false;
				}else{
					//The target activity is an empty, we need to handle the case of nested gateway
					Activity targetEmpty = target.getActivity();		

					//If the target empty has more than one incoming links, we need to keep it
					if(targetEmpty.getTargets() != null && !targetEmpty.getTargets().getChildren().isEmpty()){
						if(targetEmpty.getTargets().getChildren().size() > 1){
							return true;
						}
					}					
					Condition incomingTC = BpelOptimizationUtil.getTransitionCondition(link);					
					int tcCounter = 0;
					int numberOfOutgoingLinks = 0;
					if(targetEmpty.getSources() != null && !targetEmpty.getSources().getChildren().isEmpty()){
						numberOfOutgoingLinks = targetEmpty.getSources().getChildren().size();
						for(Object eachSource:targetEmpty.getSources().getChildren()){
							if(eachSource instanceof Source){
								Source srcObj = (Source)eachSource;							
								Link srcLink = srcObj.getLink();
								Condition eachOutgoingTC = BpelOptimizationUtil.getTransitionCondition(srcLink);
								if(eachOutgoingTC != null){								
									tcCounter++;
								}
							}
						}
					}
					if(incomingTC != null && tcCounter != 0 && tcCounter == numberOfOutgoingLinks){
						return false;
					}
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * This method is used to determine whether the empty represents the inclusive decision gateway. If yes, we
	 * create "IOR" gateway for this empty. Otherwise, we create gateway attribute based on our regular logic
	 * This is a special handling for modeler user who specifies a multiple-choice decision as inclusive
	 * @param rule
	 * @param context
	 * @param activity
	 * @return
	 */
	private static boolean isIORDecisionEmpty(TransformationRule rule,TransformationContext context,Activity activity){
		if(activity instanceof Empty){
			Empty condidateEmpty = (Empty)activity;
			if(condidateEmpty.getSources() != null && condidateEmpty.getSources().getChildren().size() >= 2){
				Decision decision = findDecisionForEmpty(context,rule,activity);					
				if(decision != null && decision.getMultiplePaths().equals(Boolean.TRUE)){
					return true;						
				}
			}
		}
		return false;
	}

	@SuppressWarnings("unchecked")
	private static Decision findDecisionForEmpty(TransformationContext context,TransformationRule rule, Activity targetEmpty){
		Decision decision = null;
		if(context == null || rule == null || targetEmpty == null){
			return decision;
		}
		TransformationRule bpelprule = null;
		TransformationRule parentRule = rule.getParentRule();
		EList childRules = parentRule.getChildRules();
		//need to find the BpelpRule, the bpelrule should be the sibling rule of the bpeloptimizationrule
		for(Object eachRule:childRules){
//			if(parentRule.getParentRule() instanceof ProcessFlowPatternRule){					
//				ProcessFlowPatternRule pfpr = (ProcessFlowPatternRule)parentRule.getParentRule();
//				//d84885 we need to find out the top level process 
//				if(!pfpr.getSource().isEmpty() && pfpr.getSource().get(0) instanceof com.ibm.btools.bom.model.processes.activities.Activity){
//					com.ibm.btools.bom.model.processes.activities.Activity activity = (com.ibm.btools.bom.model.processes.activities.Activity)pfpr.getSource().get(0);
//					if(!BomUtils.isProcess(activity.getImplementation())){
//						continue;
//					}
//				}
//			}
			if(eachRule instanceof BpelpRuleImpl){
				List switches = ((BpelpRuleImpl)eachRule).getTransformableProcessSwitch();
				if(switches.size() == 0){
					return null;
				}
				Empty empty = null;
				Switch bpelSwitch = null;
				for (Object each: switches){						
					if(!(each instanceof Switch)){
						continue;
					}
					bpelSwitch = (Switch)each;  
					empty = ((BpelpRuleImpl)eachRule).getDecisionActivity(bpelSwitch);	 

					if (empty != null && empty == targetEmpty){							
						decision = getDecisionForSwitch(context,bpelSwitch);	
						break;
					}
				}	    
			}
			if(decision != null){
				break;
			}
		}
		return decision;
	}

	/**
	 * Retrieve the source decision action based on the original switch activity which has been 
	 * transformed into Empty activity
	 * @param context
	 * @param bpelSwitch
	 * @return
	 */
	private static Decision getDecisionForSwitch(TransformationContext context,Switch bpelSwitch){		
		TransformationRule decisionRule = TransformationUtil.getRuleForTarget(ProcessUtil.getProcessDefinitionRule(context), DecisionRule.class,bpelSwitch);
		if (decisionRule != null){
			Decision decision = (Decision)((PinSet)decisionRule.getSource().get(0)).eContainer();
			return decision;
		}
		return null;	    
	}

	/**
	 * If the link is fault link, we don't record it in the list
	 * @param activity
	 * @param link
	 * @return
	 */
	public static boolean isFaultLink(Activity activity,Link link){
		if(activity != null && link != null){
			Sources sources = activity.getSources();
			List<FaultSource> faultSources = new ArrayList<FaultSource>();
			if(sources != null && !sources.getChildren().isEmpty()){
				for(Object eachSource:sources.getChildren()){
					if(eachSource instanceof FaultSource){
						faultSources.add((FaultSource)eachSource);
					}
				}
			}
			if(!faultSources.isEmpty()){
				for(FaultSource eachFaultSource:faultSources){
					Link faultLink = eachFaultSource.getLink();
					if(faultLink != null && (faultLink == link || faultLink.getName().equals(link.getName()))){
						return true;
					}
				}
			}
		}
		return false;
	}

	/**
	 * 1. The fault pin set includes only one out going activity edge
	 * 2. The fault link target is the same as the regular link
	 * @param outputSet
	 * @return
	 */
	public static boolean isFaultLinkInParallelToRegularLink(OutputPinSet outputSet) {
		if(outputSet != null && outputSet.getIsException()!= null && outputSet.getIsException().booleanValue()){
			Action srcAction = outputSet.getAction();
			//TODO handle foreach
			if(BomUtils.isLocalProcess(srcAction)|| srcAction instanceof LoopNode){
				return false;
			}
			List faultPinList = BomUtils.getPinsInSet(outputSet);
			int numofOutGoingLink = 0;
			ConnectableNode faultConnectableNode = null;
			//Find out the only outgoing link
			for(Object eachFaultPin:faultPinList){
				if(eachFaultPin instanceof Pin){
					Pin faultPin = (Pin)eachFaultPin;
					if(faultPin.getOutgoing() != null){
						numofOutGoingLink++;
						faultConnectableNode = faultPin.getOutgoing().getTarget();
					}
				}
			}
			//We only create empty placeholder on the single parallel fault link
			if(numofOutGoingLink == 1){			
				EList ipsList = outputSet.getInputPinSet();
				//assume only one inputpinset associated with the output pinset
				InputPinSet ips = (InputPinSet)ipsList.get(0);
				EList opsList = ips.getOutputPinSet();
				OutputPinSet regularOPS = null;
				if(opsList != null && opsList.contains(outputSet)){
					for(Object eachOPS:opsList){
						if(eachOPS instanceof OutputPinSet){
							OutputPinSet ops = (OutputPinSet)eachOPS;
							if(ops.getIsException() != null && !ops.getIsException().booleanValue()){
								regularOPS = ops;
								break;
							}						
						}
					}
				}
				if(regularOPS != null){
					List regualrPinList = BomUtils.getPinsInSet(regularOPS);				
					PinSet faultTargetPinSet = null;
					if(faultConnectableNode instanceof TerminationNode){
						TerminationNode tn = (TerminationNode)faultConnectableNode;
						faultTargetPinSet = tn.getOutcome();
					}else if(faultConnectableNode instanceof OutputObjectPin || faultConnectableNode instanceof OutputControlPin){
						List faultOpsList = BomUtils.getOutputPinSets((Pin)faultConnectableNode);
						if(faultOpsList != null && !faultOpsList.isEmpty()){
							faultTargetPinSet = (OutputPinSet)faultOpsList.get(0);
						}
					}else if(faultConnectableNode instanceof InputObjectPin || faultConnectableNode instanceof InputControlPin){
						List faultIpsList = BomUtils.getInputPinSets((Pin)faultConnectableNode);
						if(faultIpsList != null && !faultIpsList.isEmpty()){
							faultTargetPinSet = (InputPinSet)faultIpsList.get(0);
						}
					}
					for(Object eachRegularPin:regualrPinList){
						if(eachRegularPin instanceof Pin){
							Pin regularPin = (Pin)eachRegularPin;
							PinSet regularTargetPinSet = null;
							if(regularPin.getOutgoing() != null){
								ConnectableNode regularConnectedNode = regularPin.getOutgoing().getTarget();
								if(regularConnectedNode instanceof TerminationNode){
									regularTargetPinSet = ((TerminationNode)regularConnectedNode).getOutcome();
								}else if(regularConnectedNode instanceof OutputObjectPin || regularConnectedNode instanceof OutputControlPin){
									List regularOpsList = BomUtils.getOutputPinSets((Pin)regularConnectedNode);
									if(regularOpsList != null && !regularOpsList.isEmpty()){
										regularTargetPinSet = (OutputPinSet)regularOpsList.get(0);
									}
								}else if(regularConnectedNode instanceof InputObjectPin || regularConnectedNode instanceof InputControlPin){
									List regularIpsList = BomUtils.getInputPinSets((Pin)regularConnectedNode);
									if(regularIpsList != null && !regularIpsList.isEmpty()){
										regularTargetPinSet = (InputPinSet)regularIpsList.get(0);
									}
								}
								//both pin outgoing links target to the same node, they are in parallel
								if(faultTargetPinSet != null && faultTargetPinSet == regularTargetPinSet){
									return true;
								}
							}
						}
					}
				}
			}
		}
		return false;
	}
    
	/**
	 * Handle the case that fault link split by a fork node, we need to keep the empty activity to maintain the semantic
	 * We need to trace back to the source of the incoming link
	 * @param inputSet InputPinSet on the Fork node
	 * @return
	 */
	public static boolean handleFaultLinkFork(InputPinSet inputSet) {
		if(inputSet != null){
			List pinList = BomUtils.getPinsInSet(inputSet);
			if(pinList != null && !pinList.isEmpty()){
				for(Iterator<Pin> it= pinList.iterator();it.hasNext();){
					Pin eachPin = it.next();
					if(eachPin.getIncoming() != null && eachPin.getIncoming().getSource() instanceof Pin){
						Pin srcPin = (Pin)eachPin.getIncoming().getSource();
						PinSet srcPs = BomUtils.findPinSetForPin(srcPin);
						if(srcPs instanceof OutputPinSet){
							OutputPinSet ops = (OutputPinSet)srcPs;
							if(ops.getIsException()!= null && ops.getIsException().booleanValue()){
								return true;
							}
						}
					}
				}
			}
		}
		return false;
	}
   /**
    * If the exceptional pin set has more than one control pins and only control pins in side
    * we create an empty activity as a place holder to fork 
    * @param outputSet
    * @return
    */
	public static boolean shoudlGenerateEmptyActivity(OutputPinSet outputSet) {
		boolean create = false;
		if(outputSet != null && outputSet.getIsException() != null && outputSet.getIsException().booleanValue()){
			List pinList = BomUtils.getPinsInSet(outputSet);			
			if(pinList != null && !pinList.isEmpty()){
				Iterator<Pin> pinIt = pinList.iterator();
				int pinSize = pinList.size();
				int controlPinCounter = 0;
//				List<EObject> targetList = new ArrayList<EObject>();
				while(pinIt.hasNext()){
					Pin eachPin = pinIt.next();
					if(eachPin instanceof OutputControlPin){
						OutputControlPin ocp = (OutputControlPin)eachPin;
//						ConnectableNode node = ocp.getOutgoing().getTarget();
//						if(!targetList.contains(node.eContainer())){
//							targetList.add(node.eContainer());
//						}
						controlPinCounter++;
					}
				}
				if(controlPinCounter > 1 && controlPinCounter == pinSize ){//&& targetList.size() > 1){
					create = true;
				}
			}
		}
		return create;
	}
/**
 * The subprocess' exceptional output pin set. This method is to detect the case that there are parallel regular and fault links 
 * from a scope targeting a process reply
 * TODO handle for each
 * @param os
 * @return
 */
public static boolean handleParallelLinks(OutputPinSet outputSet) {
	if(outputSet != null && outputSet.getIsException()!= null && outputSet.getIsException().booleanValue()){	
		Action action = outputSet.getAction();
		if(BomUtils.isLocalProcess(action)||action instanceof LoopNode){
			List faultPinList = BomUtils.getPinsInSet(outputSet);
			int numofOutGoingLink = 0;
			ConnectableNode faultConnectableNode = null;
			//Find out the only outgoing link
			for(Object eachFaultPin:faultPinList){
				if(eachFaultPin instanceof Pin){
					Pin faultPin = (Pin)eachFaultPin;
					if(faultPin.getOutgoing() != null){
						numofOutGoingLink++;
						faultConnectableNode = faultPin.getOutgoing().getTarget();
					}
				}
			}
			//We only create empty placeholder on the single parallel fault link
			if(numofOutGoingLink == 1){			
				EList ipsList = outputSet.getInputPinSet();
				//assume only one inputpinset associated with the output pinset
				InputPinSet ips = (InputPinSet)ipsList.get(0);
				EList opsList = ips.getOutputPinSet();
				OutputPinSet regularOPS = null;
				if(opsList != null && opsList.contains(outputSet)){
					for(Object eachOPS:opsList){
						if(eachOPS instanceof OutputPinSet){
							OutputPinSet ops = (OutputPinSet)eachOPS;
							if(ops.getIsException() != null && !ops.getIsException().booleanValue()){
								regularOPS = ops;
								break;
							}						
						}
					}
				}
				if(regularOPS != null){
					List regualrPinList = BomUtils.getPinsInSet(regularOPS);				
					PinSet faultTargetPinSet = null;
					if(faultConnectableNode instanceof TerminationNode){
						TerminationNode tn = (TerminationNode)faultConnectableNode;
						faultTargetPinSet = tn.getOutcome();
					}else if(faultConnectableNode instanceof OutputObjectPin || faultConnectableNode instanceof OutputControlPin){
						List faultOpsList = BomUtils.getOutputPinSets((Pin)faultConnectableNode);
						if(faultOpsList != null && !faultOpsList.isEmpty()){
							faultTargetPinSet = (OutputPinSet)faultOpsList.get(0);
						}
					}else if(faultConnectableNode instanceof InputObjectPin || faultConnectableNode instanceof InputControlPin){
						List faultIpsList = BomUtils.getInputPinSets((Pin)faultConnectableNode);
						if(faultIpsList != null && !faultIpsList.isEmpty()){
							faultTargetPinSet = (InputPinSet)faultIpsList.get(0);
						}
					}
					for(Object eachRegularPin:regualrPinList){
						if(eachRegularPin instanceof Pin){
							Pin regularPin = (Pin)eachRegularPin;
							PinSet regularTargetPinSet = null;
							if(regularPin.getOutgoing() != null){
								ConnectableNode regularConnectedNode = regularPin.getOutgoing().getTarget();
								if(regularConnectedNode instanceof TerminationNode){
									regularTargetPinSet = ((TerminationNode)regularConnectedNode).getOutcome();
								}else if(regularConnectedNode instanceof OutputObjectPin || regularConnectedNode instanceof OutputControlPin){
									List regularOpsList = BomUtils.getOutputPinSets((Pin)regularConnectedNode);
									if(regularOpsList != null && !regularOpsList.isEmpty()){
										regularTargetPinSet = (OutputPinSet)regularOpsList.get(0);
									}
								}else if(regularConnectedNode instanceof InputObjectPin || regularConnectedNode instanceof InputControlPin){
									List regularIpsList = BomUtils.getInputPinSets((Pin)regularConnectedNode);
									if(regularIpsList != null && !regularIpsList.isEmpty()){
										regularTargetPinSet = (InputPinSet)regularIpsList.get(0);
									}
								}
								//both pin outgoing links target to the same node, they are in parallel
								if(faultTargetPinSet != null && faultTargetPinSet == regularTargetPinSet){
									return true;
								}
							}
						}
					}
				}
			}
		}
	}
	return false;
}
}
