package com.ibm.btools.te.ilm.heuristics.helper;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.xsd.XSDComplexTypeDefinition;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDModelGroup;
import org.eclipse.xsd.XSDParticle;
import org.eclipse.xsd.XSDTypeDefinition;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.humantasks.Form;
import com.ibm.btools.bom.model.processes.humantasks.HumanTask;
import com.ibm.btools.model.filemanager.FileMGR;
import com.ibm.btools.model.modelmanager.dependencymanager.Dependency;
import com.ibm.btools.model.modelmanager.dependencymanager.DependencyManager;
import com.ibm.btools.model.modelmanager.dependencymanager.DependencyModel;
import com.ibm.btools.model.resourcemanager.ResourceMGR;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.IlmPlugin;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.form.FormGenerator;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.FormRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessInterfaceRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.SANReusableProcessRule;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils.PinSetAnalyzer;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.MessageRule;
import com.ibm.btools.te.ilm.heuristics.xsd.ElementDeclarationRule;
import com.ibm.btools.te.ilm.heuristics.xsd.PinSetRule;
import com.ibm.btools.te.ilm.heuristics.xsd.util.OriginalXsdUtil;
import com.ibm.btools.te.logicalGroup.model.Group;
import com.ibm.btools.te.logicalGroup.model.LogicalEntityRoot;
import com.ibm.btools.util.precondition.NameIsLegal;
import com.ibm.wbit.tel.DocumentRoot;
import com.ibm.wbit.tel.TCustomClientSettings;
import com.ibm.wbit.tel.TCustomSetting;
import com.ibm.wbit.tel.TTask;
import com.ibm.wbit.tel.TUISettings;
import com.ibm.wbit.tel.TaskFactory;
//import com.ibm.wbit.tel.generation.forms.LotusFormGeneratorConstants;
import com.ibm.wbit.tel.generation.common.forms.LotusFormGeneratorConstants;
import com.ibm.wbit.tel.generation.forms.LotusFormsGenerator;

/**
 * Provides a set of helper APIs for TEL generation
 *
 */
public class HumanTaskUtil implements LotusFormGeneratorConstants{
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	// Constants for TEL creation
	public static final String CLIENT_SETTING_TYPE_LOTUS_FORMS = "Lotus Forms"; //$NON-NLS-1$
	public static final String CUSTOM_SETTING_INPUT_FORM = "inputForm"; //$NON-NLS-1$
	public static final String CLIENT_SETTING_LOTUS_FORMS_INPUT_PARAMETER = CLIENT_PARAM_INPUT_PARAMETERS;
	public static final String CLIENT_SETTING_LOTUS_FORMS_OUTPUT_PARAMETER = CLIENT_PARAM_OUTPUT_PARAMETERS; //$NON-NLS-1$
	public static final String CUSTOM_SETTING_OUTPUT_FORM = "outputForm"; //$NON-NLS-1$
	public static final String JNDI_NAME_STAFF_PLUGIN_PROVIDER_VMM="bpe/staff/samplevmmconfiguration"; //$NON-NLS-1$
	public static final String JNDI_NAME_STAFF_PLUGIN_PROVIDER_USERREGISTRY="bpe/staff/userregistryconfiguration"; //$NON-NLS-1$
	public static final String DURATION_DAY="day"; //$NON-NLS-1$
	public static final String DURATION_DAYS="days"; //$NON-NLS-1$
	public static final String DURATION_HR="hour"; //$NON-NLS-1$
	public static final String DURATION_HRS="hours"; //$NON-NLS-1$
	public static final String DURATION_MIN="minute"; //$NON-NLS-1$
	public static final String DURATION_MINS="minutes"; //$NON-NLS-1$
	public static final String DURATION_SEC="second"; //$NON-NLS-1$
	public static final String DURATION_SECS="seconds"; //$NON-NLS-1$
	public static final String DEFAULT_DURATION_UNTIL_ESCALATION="2"+DURATION_HRS; //$NON-NLS-1$
	public static final String DEFAULT_EMAIL_NAME="DefaultMessage"; //$NON-NLS-1$
	public static final String CONTEXTVAR_POTENTIALOWNERS = "%htm:task.potentialOwners%"; //$NON-NLS-1$
	public static final String CONTEXTVAR_OWNER = "%htm:task.owner%"; //$NON-NLS-1$
	public static final String CONTEXTVAR_RECEIVERFOROTHERESCALATION_PATTERN = "XXX"; //$NON-NLS-1$
	public static final String CONTEXTVAR_RECEIVERFOROTHERESCALATION = "%htm:escalation("+CONTEXTVAR_RECEIVERFOROTHERESCALATION_PATTERN+").receivers%"; //$NON-NLS-1$

	
	// Flag to control the transformation should support first class HumanTask in 6.1
	public final static boolean bUse61HumanTaskSupport = true;
	// Registry for registering in-line vs global HT so that we can distinguish them when creating the SolutionArtifacts to be passed back to ExportOperation
	public final static String HUMAN_TASK_REGISTRY = "HUMAN_TASK_REGISTRY"; //$NON-NLS-1$
	public final static String INLINE_HUMAN_TASK_REGISTRY = "INLINE_HUMAN_TASK_REGISTRY"; //$NON-NLS-1$
	// Registry for escalation
	public final static String HUMAN_TASK_ESALATION_NAME_REGISTRY = "HUMAN_TASK_ESALATION_NAME_REGISTRY"; //$NON-NLS-1$
	public static final String FORM_EXTENSION = ".xfdl"; //$NON-NLS-1$
	
	public static final String CUSTOM_SETTING_DELIMITER ="&_&";
	

	
	/**
	 * Check if the transformed TEL represents a local human task, therefore should be treated as iTEL
	 * @param context
	 * @param docRoot
	 * @return
	 */
	public static boolean isInlineHumanTask(TransformationContext context, DocumentRoot docRoot) {
		return getHumanTaskRegistry(context,INLINE_HUMAN_TASK_REGISTRY).contains(docRoot);
	}
	
	/**
	 * Register in the registry for the tranformed TEL object as inline TEL
	 * @param context
	 * @param docRoot
	 */
	public static void registerInlineHumanTask (TransformationContext context, DocumentRoot docRoot) {
		((List<DocumentRoot>)getHumanTaskRegistry(context,INLINE_HUMAN_TASK_REGISTRY)).add(docRoot);
	}

	/**
	 * Check if the transformed TEL represents a global human task
	 * @param context
	 * @param docRoot
	 * @return
	 */
	public static boolean isHumanTask(TransformationContext context, DocumentRoot docRoot) {
		return getHumanTaskRegistry(context,HUMAN_TASK_REGISTRY).contains(docRoot);
	}
	
	/**
	 * Register in the registry for the tranformed TEL object as standalone TEL
	 * @param context
	 * @param docRoot
	 */
	public static void registerHumanTask (TransformationContext context, DocumentRoot docRoot) {
		((List<DocumentRoot>)getHumanTaskRegistry(context,HUMAN_TASK_REGISTRY)).add(docRoot);
	}

	/**
	 * Get the HumanTask registry from the context
	 * @param context
	 * @param registryKey
	 * @return
	 */
	public static List<DocumentRoot> getHumanTaskRegistry(TransformationContext context,String registryKey) {
		List<DocumentRoot> registry = (List<DocumentRoot>)context.get(registryKey);
		if (registry==null) {
			registry = new ArrayList<DocumentRoot>();
			context.put(registryKey, registry);
		}
		return registry;
	}
	
	/**
	 * Register escalation in the context
	 * @param context
	 * @param bomEscalation
	 * @param telEscalation
	 */
	public static void registerTelEscalationName(TransformationContext context, HumanTask ht, String bomEscalationKey, String telEscalationName) {
		HashMap<HumanTask,HashMap> registry = getEscalationNameRegistry(context);
		HashMap<String,String> nameRegistry = (HashMap<String,String>)registry.get(ht);
		if (nameRegistry==null) {
			nameRegistry = new HashMap<String,String>();
			registry.put(ht, nameRegistry);
		}
		nameRegistry.put(bomEscalationKey, telEscalationName);
	}
	
	/**
	 * Get the registered escalation
	 * @param context
	 * @param bomEscalation
	 * @return
	 */
	public static String getTelEscalationName (TransformationContext context, HumanTask ht, String bomEscalationKey) {
		HashMap<HumanTask,HashMap> registry = getEscalationNameRegistry(context);
		HashMap<String,String> nameRegistry = (HashMap<String,String>)registry.get(ht);
		if (nameRegistry!=null) {
			return (String)nameRegistry.get(bomEscalationKey);
		}
		return null;
	}
	
	/**
	 * Get Escalation registry from the context
	 * @param context
	 * @return
	 */
	public static HashMap<HumanTask,HashMap> getEscalationNameRegistry(TransformationContext context) {
		HashMap<HumanTask,HashMap> registry = (HashMap<HumanTask,HashMap>)context.get(HUMAN_TASK_ESALATION_NAME_REGISTRY);
		if (registry==null) {
			registry = new HashMap<HumanTask,HashMap>();
			context.put(HUMAN_TASK_ESALATION_NAME_REGISTRY,registry);
		}
		return registry;
	}
	
	/**
	 * Check if the action is a local human task
	 * @param action
	 * @return
	 */
	public static boolean isLocalHumanTask(Action action) {
		if (action instanceof HumanTask) {
			if (action.getInStructuredNode()!=null) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Check if it is a 6.1 style HumanTask transformation
	 * @return
	 */
	public static final boolean isUse61HumanTaskSupport() {
		return bUse61HumanTaskSupport;
	}
	
	/**
	 * Get the name of the XFDL file attached to the Bom Form
	 * @param form
	 * @return
	 */
	public static String getAttachedFormFilename(Form form) {
        String projectName = ResourceMGR.getResourceManger().getProjectName(form);
        // Object the attached Form information
        String projectPath = FileMGR.getProjectPath(projectName);
        DependencyModel depMdl = DependencyManager.instance().getDependencyModel(projectName, projectPath);
        List <Dependency>attachedFiles = depMdl.getAllDependencies(form, null, DependencyModel.URL_DEPENDENCY_NAME);
       //defect 92959. Check which of the attachment to pick.. will pick the first one with the same name or the 
        //first xfdl file.
        if (!attachedFiles.isEmpty()) {
        	String formFileName = form.getName() + ".xfdl";
        	String candidateXFDLFile = null;
        	Iterator it = attachedFiles.iterator();
        	String xfdlFileName = null;
        	while(it.hasNext()){
        		String thisFile = ((Dependency)it.next()).getTarget().getUrlDescriptor().getUrl();
        		if(formFileName.equals(thisFile)){
        			xfdlFileName = thisFile;
        			break;
        		}else if(thisFile.endsWith(".xfdl")){
        			candidateXFDLFile = thisFile;
        		}
        	}
        	if(xfdlFileName == null){
        		if(candidateXFDLFile != null){
        			xfdlFileName = candidateXFDLFile;
        		}else{
        			//xfdlFileName = ((Dependency)attachedFiles.get(attachedFiles.size()-1)).getTarget().getUrlDescriptor().getUrl();
        			xfdlFileName = ((Dependency)attachedFiles.get(0)).getTarget().getUrlDescriptor().getUrl();
        		}
        	}
        	return xfdlFileName;
        }
        return null;
	}
	
	/**
	 * Generate Tel name from Naming registry
	 * @param context
	 * @param fromSource
	 * @param telObject
	 * @param propertyName
	 * @return
	 */
	public static String getTelName(TransformationContext context, NamedElement fromSource, Object telObject, String propertyName) {
		AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(propertyName);
		valueProvider.setContext(context);
		String name = (String)valueProvider.getAttributeValueForType(telObject,fromSource,propertyName,NamingUtil.getTELNamingRegistry(context));
		return name;
		
	}

	/**
	 * Returns true is the <code> inputSet</code> is compatible with the correlated outputPinSet.
	 * Matching is done based on the number of Pins and type. 
	 * Assumes one OutputPinSet/InputPinSet correlation, will always pick the first PinSet
	 * @param inputSet
	 * @return
	 */
	public static boolean useSameFormForOutput(InputPinSet inputSet, TransformationContext context){
		
		
		//L017093
		OutputPinSet os =(OutputPinSet) BomUtils.getCorrelatedPinSets(inputSet).get(0);
		
		List startPins = BomUtils.getObjectPinsForSet(inputSet);
		List endPins = BomUtils.getObjectPinsForSet(os);
		if(startPins.size() != endPins.size()){
			return false;
		}
		for(int i=0; i< startPins.size(); i++){
			ObjectPin inputPin = (ObjectPin)startPins.get(i);
			ObjectPin outputPin = (ObjectPin)endPins.get(i);
			if(!inputPin.getType().equals(outputPin.getType())){
				return false;
			}else{
				if(!BomHelper.getInstance().sameMultiplicity(inputPin, outputPin)){
					return false;
				}
			}
		}
		return true;
//		return BomHelper.getInstance().compatibleSets(inputSet, os, context);
//		OutputPinSet os =(OutputPinSet) BomUtils.getCorrelatedPinSets(inputSet).get(0);
//		if(os.getOutputObjectPin().size() == inputSet.getInputObjectPin().size()){
//			for(int i=0; i<inputSet.getInputObjectPin().size(); i++){
//				InputObjectPin inputPin = (InputObjectPin)inputSet.getInputObjectPin().get(i);
//				if(!(inputPin.getType().equals(((OutputObjectPin)os.getOutputObjectPin().get(i)).getType()))){
//					return false;
//				}
//			}
//		}else{
//			return false;
//		}
//	return true;
}
	
	/**
	 * Generates the name of the form based on the name of the action and the name of the PinSet
	 * @param is
	 * @param formName
	 * @return
	 */
	public static String getFormFileName(PinSet is, String formName) {
		Action action = null;
		if(is instanceof InputPinSet){
			action = ((InputPinSet)is).getAction();
		}else{
			action = ((OutputPinSet)is).getAction();
		}
		//replace invalid characters.
		String name = (action.getName()+ "_" + is.getName()) + FORM_EXTENSION;
		for (int i = 0; i < NameIsLegal.illegal_chars.length; i++) {
				int index = name.indexOf(NameIsLegal.illegal_chars[i]);
				if (index != -1) {
					name = name.replace(NameIsLegal.illegal_chars[i], '_');
				}
		}
		return name;
	}
	
	private static String getFormFileName(PinSet is, TTask task) {
		String form = null;
		//defect 86232. Use the task name that will be unique rather then the action name
		if(task != null && task.getName() != null){
			form = task.getName();
		}else{
			Action action = null;
			if(is instanceof InputPinSet){
				action = ((InputPinSet)is).getAction();
			}else{
				action = ((OutputPinSet)is).getAction();
			}
			form = action.getName();
				
		}
		//replace invalid characters.
		String name = (form+ "_" + is.getName()) + FORM_EXTENSION;
		for (int i = 0; i < NameIsLegal.illegal_chars.length; i++) {
				int index = name.indexOf(NameIsLegal.illegal_chars[i]);
				if (index != -1) {
					name = name.replace(NameIsLegal.illegal_chars[i], '_');
				}
		}
		return name;
	}

	public static String getRuntimeUniqueIdentifierForGroup(String modelerIdentifier) {
		//defect 86674
		HashMap <String, String> logicalGroupToRuntimeMap = null; 
		if(modelerIdentifier == null || modelerIdentifier.equals("")){
			return null;
		}else{
			LogicalEntityRoot root =  BpelOptionsUtil.getLogicalGroupMapping();
			if(root == null) return null;
			logicalGroupToRuntimeMap = new HashMap<String, String>();
			Iterator<Group> it = root.getRole().iterator();
			while (it.hasNext()){
				Group group = it.next();
				logicalGroupToRuntimeMap.put(group.getUid(), group.getUniqueName());
			}
			return logicalGroupToRuntimeMap.get(modelerIdentifier);
		}
	}
	
	public static String getRuntimePeopleDirectoryName() {
			LogicalEntityRoot root =  BpelOptionsUtil.getLogicalGroupMapping();
			if(root == null) return null;
			return root.getPeopleDirectory();
	}
	
	/**
	 * Creates the UI Settings.
	 * @param inputSet
	 * @param documentRoot
	 * @param rule
	 * @return
	 */
	public static TUISettings createUISetting(InputPinSet inputSet, com.ibm.wbit.tel.DocumentRoot documentRoot, TransformationRule rule) {
		PinSetAnalyzer ipsa = new BomUtils.PinSetAnalyzer(inputSet.getAction());
		TUISettings uiSettings = TaskFactory.eINSTANCE.createTUISettings();
		TCustomClientSettings customClientSettings = TaskFactory.eINSTANCE.createTCustomClientSettings();
		customClientSettings.setClientType(HumanTaskUtil.CLIENT_SETTING_TYPE_LOTUS_FORMS);
		uiSettings.getCustomClientSettings().add(customClientSettings);
		TCustomSetting customSetting;

		//generate the Form for the InputSet
		TCustomSetting inputCustomSetting = createCustomSetting(inputSet,HumanTaskUtil.CUSTOM_SETTING_INPUT_FORM, documentRoot, true, rule);
		if(inputCustomSetting.getValue() != null) customClientSettings.getCustomSetting().add(inputCustomSetting);

		//if this is a 2 way operation, then should not generate the form for the outputSet, unless specifically set by the user 
		if(ipsa.getInputPinSetsAsTwoWayOperation().contains(inputSet)){
			if(HumanTaskUtil.useSameFormForOutput(inputSet, rule.getRoot().getContext())){
				//hack? In WID, the form to use is in this case the outputForm
				//d88875 start
				//always use form from OutputPinSet, if there is none, use form from InputPinSet, otherwise - create default form based on output
				OutputPinSet oSet = (OutputPinSet)inputSet.getOutputPinSet().get(0);
				if (oSet.getForm() != null || inputSet.getForm() == null) {
					customSetting = createCustomSetting((PinSet)inputSet.getOutputPinSet().get(0),HumanTaskUtil.CUSTOM_SETTING_OUTPUT_FORM, documentRoot, true, rule);
					inputCustomSetting.setValue(customSetting.getValue());
					if(customSetting.getValue() != null) customClientSettings.getCustomSetting().add(customSetting);	
				} else { 		// (inputSet.getForm() != null)
					customSetting = createCustomSetting(inputSet,HumanTaskUtil.CUSTOM_SETTING_OUTPUT_FORM, documentRoot, true, rule);
					if(customSetting.getValue() != null) customClientSettings.getCustomSetting().add(customSetting);		
				} 
				//d88875				customSetting = createCustomSetting((PinSet)inputSet.getOutputPinSet().get(0),HumanTaskUtil.CUSTOM_SETTING_OUTPUT_FORM, documentRoot, true, rule);
				//d88875				inputCustomSetting.setValue(customSetting.getValue());
				//d88875 end			if(customSetting.getValue() != null) customClientSettings.getCustomSetting().add(customSetting);
			}else{
				//if input and output do not match should generate the form for the output also
				customSetting = createCustomSetting((PinSet)inputSet.getOutputPinSet().get(0),HumanTaskUtil.CUSTOM_SETTING_OUTPUT_FORM, documentRoot, true, rule);
				if(customSetting.getValue() != null) customClientSettings.getCustomSetting().add(customSetting);
			}
		}
		updateCustomSettings(uiSettings, inputSet, rule);

		return uiSettings;	
	}
	
	/**
	 * Updates the uiSettings for the LotusForms with the name of the input/output variables (defect 81750)
	 * @param uiSettings
	 * @param inputSet
	 * @param rule
	 */
	private static void updateCustomSettings(TUISettings uiSettings, PinSet inputSet, TransformationRule rule ){
		
		if(uiSettings.getCustomClientSettings() != null && !uiSettings.getCustomClientSettings().isEmpty()){
			TCustomClientSettings customClientSettings = (TCustomClientSettings)uiSettings.getCustomClientSettings().get(0);
			InputPinSet interfaceInputPinSet = null;
		if(inputSet instanceof InputPinSet){
				interfaceInputPinSet = FabricIntegrationUtil.getInterfaceActivityInputPinSetForProcessIPS((InputPinSet)inputSet);
			}
			PinSetRule pinSetRule = null;
			ProcessInterfaceRule procIntRule = null;
			if(interfaceInputPinSet == null){		
				procIntRule = (ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(rule.getRoot().getContext(), inputSet);
				pinSetRule =(PinSetRule)TransformationUtil.getRuleForSource(procIntRule, PinSetRule.class, inputSet);
			}else{
				procIntRule = (ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(rule.getRoot().getContext(), interfaceInputPinSet);
				pinSetRule =(PinSetRule)TransformationUtil.getRuleForSource(procIntRule, PinSetRule.class, interfaceInputPinSet);
		}
			//defect 85855, 87694
			String metaData = getMetaDataForPinSet(procIntRule, inputSet);
			if(metaData != null){
				TCustomSetting customSetting = TaskFactory.eINSTANCE.createTCustomSetting();
				customSetting.setName(HumanTaskUtil.CLIENT_SETTING_LOTUS_FORMS_INPUT_PARAMETER);
				customSetting.setValue(metaData);
				customClientSettings.getCustomSetting().add(customSetting);
			}
			
/*			List<String> parameters = getNameFromPinSetRule(pinSetRule);
			if(parameters != null && !parameters.isEmpty()){
				TCustomSetting customSetting = TaskFactory.eINSTANCE.createTCustomSetting();
				customSetting.setName(HumanTaskUtil.CLIENT_SETTING_LOTUS_FORMS_INPUT_PARAMETER);
				StringBuffer sb = new StringBuffer();
				for(String parameter: parameters){
					sb.append(parameter);
					sb.append(",");
				}
				String value = sb.substring(0, sb.lastIndexOf(","));
				customSetting.setValue(value);
				customClientSettings.getCustomSetting().add(customSetting);
			}
			*/
			List<PinSet> correlatedPins = null;
			if(interfaceInputPinSet == null){
				correlatedPins = BomUtils.getCorrelatedPinSets(inputSet);
			}else{
				correlatedPins = BomUtils.getCorrelatedPinSets(interfaceInputPinSet);
			}
			if(correlatedPins != null && !correlatedPins.isEmpty()){
				PinSet os = null;
				if(interfaceInputPinSet == null){
					os =(PinSet)BomUtils.getCorrelatedPinSets(inputSet).get(0);
				}else{
					os =(PinSet)BomUtils.getCorrelatedPinSets(interfaceInputPinSet).get(0);
				}
				//defect 85855,87694
				String outMetaData = getMetaDataForPinSet(procIntRule, os);
				if(outMetaData != null){
					TCustomSetting customSetting = TaskFactory.eINSTANCE.createTCustomSetting();
					customSetting.setName(HumanTaskUtil.CLIENT_SETTING_LOTUS_FORMS_OUTPUT_PARAMETER);
					customSetting.setValue(outMetaData);
					customClientSettings.getCustomSetting().add(customSetting);
				}
/*				
			pinSetRule =(PinSetRule)TransformationUtil.getRuleForSource(procIntRule, PinSetRule.class, os);
				parameters = getNameFromPinSetRule(pinSetRule);
				if(parameters != null && !parameters.isEmpty()){
					TCustomSetting customSetting = TaskFactory.eINSTANCE.createTCustomSetting();
					customSetting.setName(HumanTaskUtil.CLIENT_SETTING_LOTUS_FORMS_OUTPUT_PARAMETER);
					StringBuffer sb = new StringBuffer();
					for(String parameter: parameters){
						sb.append(parameter);
						sb.append(",");
					}
					String value = sb.substring(0, sb.lastIndexOf(","));
					customSetting.setValue(value);
					customClientSettings.getCustomSetting().add(customSetting);
				}
					*/
			}
		
		}
		
	}
	/**
	 * Returns the metaData based on the wsdlMessage
	 * @param procIntRule
	 * @param inputSet
	 * @return
	 */
	private static String getMetaDataForPinSet(ProcessInterfaceRule procIntRule,	PinSet inputSet) {
		String value = null;
		Message wsdlMessage = createWsdlMessageForForm(inputSet, procIntRule);
		if(wsdlMessage != null){
			LotusFormsGenerator formGenerator = new LotusFormsGenerator();
			value = formGenerator.getMetaData(wsdlMessage);
		}
		return value;
	}

	/**
	 * Returns the name of the inputs that were created for this inputPinSet
	 * @param pinSetRule TransformationRule
	 * @param pinIndex int
	 * @return String
	 */ 
	private static List<String> getNameFromPinSetRule(TransformationRule pinSetRule) {
		if (pinSetRule == null) {
			return null;
		}
		List<String> variableNames = new ArrayList<String>();
		String serializedName = null;
		//hack
		if(pinSetRule.getTarget().isEmpty() && pinSetRule.getParentRule() instanceof ElementDeclarationRule){
			pinSetRule = pinSetRule.getParentRule();
		}
		EObject pinSetTarget = null;
		if(!pinSetRule.getTarget().isEmpty()){
			pinSetTarget = (EObject)pinSetRule.getTarget().get(0);
		}else{
			pinSetTarget = OriginalXsdUtil.getElementDeclarationFromRegistry(pinSetRule.getRoot().getContext(),(NamedElement)pinSetRule.getSource().get(0));
			if(pinSetTarget ==null){
				pinSetTarget =OriginalXsdUtil.getAttributeDeclarationFromRegistry(pinSetRule.getRoot().getContext(),(NamedElement)pinSetRule.getSource().get(0));;
			}
		}
	//	if (pinSetRule.getTarget().get(0) instanceof XSDTypeDefinition) {
		if (pinSetTarget instanceof XSDTypeDefinition) {
			//XSDTypeDefinition xsdType = (XSDTypeDefinition)pinSetRule.getTarget().get(0);
			XSDTypeDefinition xsdType = (XSDTypeDefinition)pinSetTarget;
			if (xsdType instanceof XSDComplexTypeDefinition && ((XSDComplexTypeDefinition)xsdType).getContent() instanceof XSDParticle) {
				XSDModelGroup xsdGroup = (XSDModelGroup)((XSDParticle)((XSDComplexTypeDefinition)xsdType).getContent()).getContent();
//				if (xsdGroup.getContents().size() > pinIndex) {
				for(int pinIndex=0; pinIndex <xsdGroup.getContents().size(); pinIndex++){
					if (xsdGroup.getContents().get(pinIndex) instanceof XSDElementDeclaration) {
						XSDElementDeclaration xsdElement = (XSDElementDeclaration)xsdGroup.getContents().get(pinIndex);
						if (xsdElement.getResolvedElementDeclaration() != null && xsdElement.getResolvedElementDeclaration().getName() != null &&
								!(xsdElement.getResolvedElementDeclaration().getName().equals(xsdElement.getName()))) {
							serializedName = xsdElement.getResolvedElementDeclaration().getName();	// anonymous type - reference to xsdelement
						} else {
							serializedName = xsdElement.getName();
						}
					} else if (xsdGroup.getContents().get(pinIndex) instanceof XSDParticle &&
							((XSDParticle)xsdGroup.getContents().get(pinIndex)).getContent() instanceof XSDElementDeclaration) {
						XSDElementDeclaration xsdElement = (XSDElementDeclaration)((XSDParticle)xsdGroup.getContents().get(pinIndex)).getContent();
						if (xsdElement.getResolvedElementDeclaration() != null && xsdElement.getResolvedElementDeclaration().getName() != null &&
								!(xsdElement.getResolvedElementDeclaration().getName().equals(xsdElement.getName()))) {
							serializedName = xsdElement.getResolvedElementDeclaration().getName();	// anonymous type - reference to xsdelement
						} else {
							serializedName = xsdElement.getName();
						}
						if(serializedName != null) variableNames.add(serializedName);
					}
	//				}											
				}
			}
		//}else if (pinSetRule.getTarget().get(0) instanceof XSDElementDeclaration) {
		}else if (pinSetTarget instanceof XSDElementDeclaration) {
			//XSDElementDeclaration xsdElementWrapper = (XSDElementDeclaration)pinSetRule.getTarget().get(0);
			XSDElementDeclaration xsdElementWrapper = (XSDElementDeclaration)pinSetTarget;
			XSDTypeDefinition xsdType = xsdElementWrapper.getType();
			if (xsdType instanceof XSDComplexTypeDefinition && ((XSDComplexTypeDefinition)xsdType).getContent() instanceof XSDParticle) {
				XSDModelGroup xsdGroup = (XSDModelGroup)((XSDParticle)((XSDComplexTypeDefinition)xsdType).getContent()).getContent();
				for(int pinIndex=0; pinIndex <xsdGroup.getContents().size(); pinIndex++){
//				
//				if (xsdGroup.getContents().size() > pinIndex && xsdGroup.getContents().get(pinIndex) instanceof XSDElementDeclaration) {
					XSDElementDeclaration xsdElement = null;
					if(xsdGroup.getContents().get(pinIndex) instanceof XSDParticle){
						 XSDParticle particle = (XSDParticle)xsdGroup.getContents().get(pinIndex);
						 xsdElement = (XSDElementDeclaration)particle.getContent();
					}else if(xsdGroup.getContents().get(pinIndex) instanceof XSDElementDeclaration){
						xsdElement = (XSDElementDeclaration)xsdGroup.getContents().get(pinIndex);
					}
					if(xsdElement != null){
						if (xsdElement.getResolvedElementDeclaration() != null && xsdElement.getResolvedElementDeclaration().getName() != null &&
								!(xsdElement.getResolvedElementDeclaration().getName().equals(xsdElement.getName()))) {
							serializedName = xsdElement.getResolvedElementDeclaration().getName();	// anonymous type - reference to xsdelement
						} else {
							serializedName = xsdElement.getName();
						}
					}
					if(serializedName != null) variableNames.add(serializedName);
				}
			}								
		}

		return variableNames;
	}
	
	/**
	 * Will generate a default Form in case one does not exist. Will save the Form content as customSetting value until the end. 
	 * During serialization will extract the xfdl and serialize at the desired location. 
	 * @param is
	 * @param formName
	 * @param documentRoot
	 * @param generateDefault
	 * @return
	 */
	private static TCustomSetting createCustomSetting(PinSet is, String formName, com.ibm.wbit.tel.DocumentRoot documentRoot, boolean generateDefault, TransformationRule rule) {
		Form inputForm = null;
		if(rule instanceof SANReusableProcessRule){
			inputForm = is.getForm();
		}else{
			Action action = null;
			if(is instanceof InputPinSet){
				action = ((InputPinSet)is).getAction();
				if(action instanceof HumanTask){
					inputForm = ((HumanTask)action).getInputForm();
				}
			}else{
				action = ((OutputPinSet)is).getAction();
				if(action instanceof HumanTask){
					inputForm = ((HumanTask)action).getOutputForm();
				}
			}
		}
		TCustomSetting customSetting = TaskFactory.eINSTANCE.createTCustomSetting();
		customSetting.setName(formName);
		if(inputForm != null){
			if (TransformationUtil.getRuleForSource(rule.getRoot(), inputForm) == null) {
				FormRule formRule = AbstractbpelFactory.eINSTANCE.createFormRule();
				rule.getRoot().getChildRules().add(formRule);
				formRule.getSource().add(inputForm);
				formRule.transformSource2Target();
			}
			String formFilename = HumanTaskUtil.getAttachedFormFilename(inputForm);
			// set the form uid at this stage and replace it with the actual file location when the xfdl is being persisted 
			customSetting.setValue(inputForm.getUid()+"/"+formFilename);
		}else if(generateDefault && BomUtils.getObjectPinsForSet(is) != null && BomUtils.getObjectPinsForSet(is).size() >0){
			String formValue= "";
			//String formFileName = HumanTaskUtil.getFormFileName(is, formName);
			String formFileName = HumanTaskUtil.getFormFileName(is, documentRoot.getTask());
			FormGenerator formGenerator = FormGenerator.getInstance();
			//feature 190156, form enhancement
			formGenerator.setTransformationRoot(rule.getRoot());
			Message wsdlMessage = createWsdlMessageForForm(is, rule);
			if(wsdlMessage != null){
				formValue = formGenerator.generateXFDL(wsdlMessage, formName);
			}
			customSetting.setValue( formValue + HumanTaskUtil.CUSTOM_SETTING_DELIMITER + formFileName);

		}
		return customSetting;
	}
	


	/**
	 * Need to get the wsdl message that will be used to generate the Forms
	 * @param is
	 * @return
	 */
	private static Message createWsdlMessageForForm(PinSet is, TransformationRule rule) {
		Message wsdlMessage = null;
		//the logic and easy way would be to get the wsdl message that was generated for the interface. 
		//This does not work in the case where the type of the Pin is a BusinessService, as the interface is not yet
		//finished. 
		com.ibm.btools.bom.model.processes.activities.Activity interfaceActivity = FabricIntegrationUtil.getInterfaceActivityIfAny((StructuredActivityNode)is.eContainer());
		    TransformationRule messageRule = null;
		    if(interfaceActivity == null){
		    	ProcessInterfaceRule  processIntRule = (ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(rule.getRoot().getContext(), is);			
		    	messageRule = TransformationUtil.getRuleForSource(processIntRule, MessageRule.class, is );
		    }else{
		    	if(is instanceof OutputPinSet){
		    		OutputPinSet interfaceOPS = FabricIntegrationUtil.getInterfaceActivityOutputPinSetForProcessOPS((OutputPinSet)is);
		    		ProcessInterfaceRule  processIntRule = (ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(rule.getRoot().getContext(), interfaceOPS);	
		    		messageRule = TransformationUtil.getRuleForSource(processIntRule, MessageRule.class, interfaceOPS);
		    	}else{
		    		InputPinSet interfacePinSet = (InputPinSet)interfaceActivity.getImplementation().getInputPinSet().get(0);
		    		ProcessInterfaceRule  processIntRule = (ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(rule.getRoot().getContext(), interfacePinSet);			
		    		messageRule = TransformationUtil.getRuleForSource(processIntRule, MessageRule.class, interfacePinSet );
		    	}
		    }
			if(messageRule != null){
				if(messageRule.getTarget() != null && !messageRule.getTarget().isEmpty()) wsdlMessage = (Message)messageRule.getTarget().get(0);
//				if(!BomUtils.referenceBusinessServiceObjects(is)){
//					return wsdlMessage;
//				}else{
					URI fileURI = URI.createFileURI(IlmPlugin.getDefault().getStateLocation().append("Temp.wsdl").toFile().getAbsolutePath());
					ResourceSet rs = new ResourceSetImpl();
					org.eclipse.emf.ecore.resource.Resource resource = rs.createResource(fileURI);
					resource.getContents().add(((Message)messageRule.getTarget().get(0)).getEnclosingDefinition());
//				}

			}
		return wsdlMessage;
	}

}
