package com.ibm.btools.te.ilm.heuristics.helper;
/*
 * Licensed Materials - Property of IBM
 * 5724-I74, 5724-I75
 *  Copyright IBM Corporation 2003, 2009. All rights reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.PortType;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDImport;

import com.ibm.btools.bom.command.compound.PredefTypesConst;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.Package;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.ActivityNode;
import com.ibm.btools.bom.model.processes.activities.InitialNode;
import com.ibm.btools.bom.model.processes.activities.InputControlPin;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.LoopNode;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.StoreArtifactPin;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.SANNestedProcessRule;
import com.ibm.btools.te.ilm.heuristics.naming.TargetNamespaceProvider;
import com.ibm.btools.te.ilm.heuristics.naming.URINameConverter;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.PartnerLink;
import com.ibm.wsspi.sca.scdl.Component;

/**
 * Provides a set of utility methods related to Process definition and interface
 * transformation
 */
public class ProcessUtil {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	//	public static final String ROOTPROCESSMODEL = "RootProcessModel";
	//	public static final String ROOTINFORMATIONMODEL = "RootInformationModel";
	//	public static final String ROOTORGANIZATIONMODEL = "RootOrganizationModel";
	//	public static final String ROOTRESOURCEMODEL = "RootResourceModel";
	//	public static final String ROOTSERVICESMODEL = "RootServicesModel";
	//	public static final String ROOTREPORTMODEL = "RootReportModel";
	//	public static final String ROOTCEFMODEL = "RootCEFModel";
	//	public static final String ROOTOBSERVATIONMODEL = "RootObservationModel";
	//	public static final String ROOTCLASSIFIERMODEL = "RootClassifierModel";

	public static final String ROOTPROCESSMODEL = PredefTypesConst.ROOT_PROC_MODEL_NAME;
	public static final String ROOTINFORMATIONMODEL = PredefTypesConst.ROOT_INFO_MODEL_NAME;
	public static final String ROOTORGANIZATIONMODEL = PredefTypesConst.ROOT_ORG_MODEL_NAME;
	public static final String ROOTRESOURCEMODEL = PredefTypesConst.ROOT_RES_MODEL_NAME;
	public static final String ROOTSERVICESMODEL = PredefTypesConst.ROOT_SERV_MODEL_NAME;
	public static final String ROOTREPORTMODEL = "RootReportModel";
	public static final String ROOTCEFMODEL = "RootCEFModel";
	//public static final String ROOTOBSERVATIONMODEL = PredefTypesConst.ROOT_OBS_MODEL_NAME;
	public static final String ROOTCLASSIFIERMODEL = PredefTypesConst.ROOT_CATEGORY_MODEL_NAME;
	public static final String ROOTEXTERNALMODEL = PredefTypesConst.ROOT_EXT_MODEL_NAME;

	public static final String PROC_DEF_KEY = "ProcDefInContext";
	public static final String PROC_INT_KEY = "ProcIntInContext";
	public static final String DEFAULT_PREFIX = "http://";
	public static final String BOM_WALKER_KEY = "BomWalkerInContext";
	public static final String SOLUTION_RULE_KEY = "Solution_Rule";
	public static final String PROC_COMPONENT_KEY = "PROC_COMPONENT_KEY";
	public static final String PROC_PARTNER_KEY = "PROC_PARTNER_KEY";

	public static final String UPDATE_COMMANDS_IN_CONTEXT = "_UPDATE_COMMANDS_IN_CONTEXT";

	//key for the Registry of variables that are already in use within a ConcurrentBranchRegion
	public static final String PARALLEL_FLOWS_REGISTERED_VARIABLES = "_PARALLEL_FLOWS_REGISTERED_VARIABLES";

	//key for the Registry of variables that are already in use within a ConcurrentBranchRegion
	public static final String CURRENT_PARALLEL_PATH_REGISTERED_VARIABLES = "_CURRENT_PARALLEL_PATH_REGISTERED_VARIABLES";
	//key used to register the current processing PortType. At one point, only one PortType is in the context.
	public static final String PORT_TYPE_KEY = "_CURRENT_PORT_TYPE_IN_CONTEXT_KEY";
	//key used for the map that keeps a registry of all the processed PortTypes and the XSDImports it requires.
	public static final String TYPE_PORT_TYPE_REGISTRY_MAP_KEY = "TYPE_IMPORT_PORT_TYPE_REGISTRY_MAP_KEY";
	
	public static final String ELEMENT_TO_DISPLAY_NAME_KEY = "ELEMENT_TO_DISPLAY_NAME_KEY";

	private static URINameConverter uriConverter = new URINameConverter();	
	
	public static void putBomWalker(TransformationContext context, BomWalker bomWalker) {
		context.put(BOM_WALKER_KEY,bomWalker);
	}

	public static void putProcessDefinitionRule(TransformationContext context, TransformationRule rule) {
		context.put(PROC_DEF_KEY,rule);
	}

	public static void putProcessDefinitionRule(TransformationContext context, NamedElement namedElement, TransformationRule rule) {
		context.put(createNamespace( namedElement)+PROC_DEF_KEY,rule);
	}

	public static BomWalker getBomWalker(TransformationContext context) {
		return (BomWalker) context.get(BOM_WALKER_KEY);
	}

	public static TransformationRule getProcessDefinitionRule(TransformationContext context) {
		return (TransformationRule) context.get(PROC_DEF_KEY);
	}

	public static void putUpdateCommandsInContext(TransformationContext context, Command command){
		List commandList ;
		if(context.get(UPDATE_COMMANDS_IN_CONTEXT) != null){
			commandList = (List)context.get(UPDATE_COMMANDS_IN_CONTEXT);
		}else{
			commandList = new LinkedList();
			context.put(UPDATE_COMMANDS_IN_CONTEXT, commandList);
		}
		commandList.add(command);
	}
	public static List getUpdateCommandsFromContext(TransformationContext context){

		return (List)context.get(UPDATE_COMMANDS_IN_CONTEXT);
	}

	public static void removeUpdateCommandsFromContext(TransformationContext context){
		List commands = (List)context.get(UPDATE_COMMANDS_IN_CONTEXT);
		commands = new LinkedList();
	}


	public static TransformationRule getProcessDefinitionRule(TransformationContext context, NamedElement namedElement) {
		String namespace = createNamespace(namedElement);
		return (TransformationRule) context.get(namespace+PROC_DEF_KEY);
	}

	public static EObject getProcessDefinition(TransformationContext context) {
		TransformationRule rule = getProcessDefinitionRule(context);
		if (rule != null)
			return (EObject)rule.getTarget().get(0);
		return null;
	}

	public static EObject getProcessDefinition(TransformationContext context, NamedElement namedElement) {
		TransformationRule rule = getProcessDefinitionRule(context,namedElement);
		if (rule != null)
			return (EObject)rule.getTarget().get(0);
		return null;
	}

	public static void putProcessInterfaceRule(TransformationContext context, TransformationRule rule) {
		context.put(PROC_INT_KEY,rule);
	}

	public static void putProcessInterfaceRule(TransformationContext context, NamedElement namedElement, TransformationRule rule) {
		context.put(createNamespace(namedElement)+PROC_INT_KEY,rule);
	}

	public static TransformationRule getProcessInterfaceRule(TransformationContext context) {
		return (TransformationRule) context.get(PROC_INT_KEY);
	}

	public static EObject getProcessInterface(TransformationContext context) {
		TransformationRule rule = getProcessInterfaceRule(context);
		if (rule != null)
			return (EObject)rule.getTarget().get(0);
		return null;
	}

	public static TransformationRule getProcessInterfaceRule(TransformationContext context,NamedElement namedElement) {
		String namespace = createNamespace(namedElement);			
		return (TransformationRule) context.get(namespace+PROC_INT_KEY);
	}

	public static EObject getProcessInterface(
			TransformationContext context,
			NamedElement namedElement) {
		return (EObject) getProcessInterfaceRule(context,namedElement).getTarget().get(0);
	}

	public static PartnerLink getProcessPartner(TransformationContext context,NamedElement namedElement) {
		String namespace = createNamespace(namedElement);
		return (PartnerLink) context.get(namespace+PROC_PARTNER_KEY);
	}

	public static void putProcessPartner(TransformationContext context,NamedElement namedElement,PartnerLink pl) {
		String namespace = createNamespace(namedElement);
		context.put(namespace+PROC_PARTNER_KEY,pl);
	}	

	public static String createFilePathFromNamespace(String namespace) {
		String path = namespace.substring(namespace.indexOf(":")+ 1,namespace.length());
		//		URI uri = URI.createFileURI(path);
		//		return uri.toFileString();
		String fileSeparator = ExportOperation.FILE_SEPARATOR;
		// TODO: path.replaceAll("/",fileSeparator) didn't work ... for some reason it throws an exception
		path = path.replace('/',fileSeparator.charAt(0));
		path = path.replace('\\',fileSeparator.charAt(0));
		while (path.startsWith(fileSeparator)) {
			path = path.substring(1);
		}
		return path;
	}

	public static String createFileName(String namespace)
	{

		String tempNS=namespace.substring(namespace.indexOf(':')+1);
		int ind = tempNS.lastIndexOf('/');
		if (ind != -1) {
			tempNS=tempNS.substring(ind+1);
		}
		return tempNS;

		//		URI uri = URI.create(namespace);
		//		return (uri.getHost()==null?"":uri.getHost()) + (uri.getPath()==null?"":uri.getPath());
	}

	public static String createNamespace(NamedElement namedElement) {	
		TargetNamespaceProvider provider = new TargetNamespaceProvider();
		return provider.getDefaultTargetNamespace(namedElement, false);
		//return createNamespace(context, namedElement, DEFAULT_PREFIX);
	}

	public static String createNamespace(NamedElement namedElement, boolean encoded) {	
		TargetNamespaceProvider provider = new TargetNamespaceProvider();
		return provider.getDefaultTargetNamespace(namedElement, encoded);
		//return createNamespace(context, namedElement, DEFAULT_PREFIX);
	}

	public static boolean isRootPackage(Package pkg) {

		if (pkg != null && pkg.getName() != null &&
				(pkg.getName().equals(ROOTPROCESSMODEL)
						|| pkg.getName().equals(ROOTINFORMATIONMODEL)
						|| pkg.getName().equals(ROOTORGANIZATIONMODEL)
						|| pkg.getName().equals(ROOTRESOURCEMODEL)
						|| pkg.getName().equals(ROOTSERVICESMODEL)
						|| pkg.getName().equals(ROOTREPORTMODEL)
						|| pkg.getName().equals(ROOTCEFMODEL)
						//	|| pkg.getName().equals(ROOTOBSERVATIONMODEL)
						|| pkg.getName().equals(ROOTCLASSIFIERMODEL)
						//AZ Added to handle external model
						|| ROOTEXTERNALMODEL.equals(pkg.getName())
				)
		) {
			// Just comparing by name is not safe ... imagine someone
			// accidentally create a package with the same name as 
			// the root model
			return pkg.getOwningPackage()==null; 
		}
		return false;
	}
	/*


	public static String createNamespace (TransformationContext context, NamedElement namedElement, boolean encoded){

		return createNamespace (context, namedElement, DEFAULT_PREFIX, encoded);
	}

	public static String createNamespace (TransformationContext context, NamedElement namedElement, String namespacePrefix, boolean encoded){
		if(encoded){
			String encodedNamespace = null;
			String targetNamespace = createNamespace(context, namedElement, namespacePrefix);
			try{
				encodedNamespace = URIEncoder.encode(targetNamespace, "UTF8");
				//now replace strip the % out
				StringTokenizer st = new StringTokenizer(encodedNamespace, new String("%"));
				StringBuffer sb = new StringBuffer();
				while(st.hasMoreTokens()){
					sb.append(st.nextToken());
				}
				encodedNamespace = sb.toString();		
				//now check if the namespace is also java compliant.. 
				if (!Character.isJavaIdentifierStart(encodedNamespace.charAt(0))){
					//Add an 'a'
					sb = new StringBuffer();
					sb.append('a');
					sb.append(encodedNamespace);
					encodedNamespace = sb.toString();			
				}

			}catch(Exception ex){
				ex.printStackTrace();
				return targetNamespace;
			}
			return encodedNamespace;
		}else{
			return createNamespace (context, namedElement, DEFAULT_PREFIX);
		}
	}

	public static String createNamespace(TransformationContext context, NamedElement namedElement, String namespacePrefix) {
		String ns = null;
		if (namedElement instanceof Activity) {
			ns = getTopLevelActivityNamespace(context, (PackageableElement)namedElement);		
		} else if (namedElement instanceof Type) {
			Type type = (Type) namedElement;
			Package model = type.getOwningPackage();
			if (model instanceof ServicesModel){
				//ns = model.getName() + "/" + namedElement.getName();
				ns = uriConverter.convertName(NamespaceNamingRegistry.getRegistry(), null, model.getName(), false) + "/" + uriConverter.convertName(NamespaceNamingRegistry.getRegistry(), null, namedElement.getName(), false);
			} else {
				//ns = model.getName();
				ns = uriConverter.convertName(NamespaceNamingRegistry.getRegistry(), null, model.getName());
			}
			Package belongingModel = model.getOwningPackage();
			while (belongingModel != null && !isRootPackage(belongingModel)) {
				//ns = belongingModel.getName() + "/" + ns;
				ns = uriConverter.convertName(NamespaceNamingRegistry.getRegistry(), null, belongingModel.getName() , false) + "/" + ns;
				belongingModel = belongingModel.getOwningPackage();

			}
		} else if (namedElement instanceof PackageableElement) {
			ns = getTopLevelActivityNamespace(context, (PackageableElement)namedElement);
		}else if (namedElement instanceof Operation) {
			ns = getTopLevelActivityNamespace(context, (PackageableElement)namedElement.eContainer()) + "/" + namedElement.getName();
			ns = namedElement.getName();
		} else {
			EObject container = namedElement.eContainer();
			Activity belongingActivity = null;
			while (!(container instanceof Activity)) {
				container = container.eContainer();
			}
			belongingActivity = (Activity) container;
			ns = getTopLevelActivityNamespace(context, belongingActivity);
		}

		//Call URINameConverter
	//	URINameConverter converter = new URINameConverter();		
	//	String newNs = uriConverter.convertName(NamespaceNamingRegistry.getRegistry(), null, ns);
	//	return namespacePrefix + newNs;
		return namespacePrefix + ns;
	}


	private static String getTopLevelActivityNamespace(TransformationContext context, PackageableElement topLevelElement) {
		String ns = topLevelElement.getName();
		ns = uriConverter.convertName(NamespaceNamingRegistry.getRegistry(), null, ns);
		//try to replace the Process which can be a working copy with the Master copy that is saved in the context.. 
		if(topLevelElement instanceof Activity && BomUtils.isProcess(((Activity)topLevelElement).getImplementation())){
			PackageableElement registeredProcess = (PackageableElement)TransformationEngine.getExportSession().getExportOptions().getAdditionalOption(ExportSessionUtil.MASTER_COPY);
			if(registeredProcess != null){
				topLevelElement = registeredProcess;
			}
		}
		Package belongingPackage = topLevelElement.getOwningPackage();
		while (belongingPackage != null && !isRootPackage(belongingPackage)) {
			ns = uriConverter.convertName(NamespaceNamingRegistry.getRegistry(), null, belongingPackage.getName(), false) + "/" + ns;
			belongingPackage = belongingPackage.getOwningPackage();
		}
		return ns;
	}
	 */
	/**
	 * Returns true if the WSDL Import should be use in the WSDL Definition instead of the XSDImport.
	 * WSDL Import is used only for Definition of an Action with ICS Implementation.
	 * @param typedElement
	 * @return
	 */	
	public static  boolean useWSDLImport(NamedElement typedElement){

		return false;

	}	
	/**
	 * Returns the list of variables in the parallel path. If no list was yet initialized, will initialize the list.
	 * @param context
	 * @return
	 */
	public static List getParallelFlowsRegisteredVariables(TransformationContext context){
		List variablesList = (List)context.get(PARALLEL_FLOWS_REGISTERED_VARIABLES) ;
		if(variablesList == null){
			variablesList = new LinkedList();
			context.put(PARALLEL_FLOWS_REGISTERED_VARIABLES, variablesList);
		}
		return variablesList;
	}

	//	/**
	//	 * Registers the <code>variable</code> in the current path. Will not register unique, the variable will be just added to the list. 
	//	 * We will end up with duplicates that are necessary for determining the list of variables used in a given parallel branch.
	//	 * @param context
	//	 * @param variable
	//	 */
	//	public static void registerVariableForParallelFlow(TransformationContext context, BPELVariable variable){
	//		List variableList  = getParallelFlowsRegisteredVariables(context); 
	//	//	if(!variableList.contains(variable))
	//		variableList.add(variable);
	//		context.put(PARALLEL_FLOWS_REGISTERED_VARIABLES, variableList);
	//	}

	/**
	 * Registers te list of <code>variables</code> as variables in the parallel path (forbiden variables). If the <code>initialize</code> is set to true,
	 * will replace the existing variables with the new list, else will add unique variables to the current list.
	 * @param context
	 * @param variables
	 * @param initialize
	 */
	public static void registerVariableForParallelFlow(TransformationContext context, List variables, boolean initialize){
		List variableList;
		if(initialize){
			variableList = new LinkedList();
		}else{
			variableList  = getParallelFlowsRegisteredVariables(context);	
		}

		if(variables != null && !variables.isEmpty()){
			Iterator it = variables.iterator();
			while(it.hasNext()){
				BPELVariable variable = (BPELVariable)it.next();
				if(!variableList.contains(variable)){
					variableList.add(variable);
				}
			}
			//			variableList.addAll(variables);	
		}
		context.put(PARALLEL_FLOWS_REGISTERED_VARIABLES, variableList);

	}

	/**
	 * Registers te list of <code>variables</code> as variables in the parallel path (forbiden variables). If the <code>initialize</code> is set to true,
	 * will replace the existing variables with the new list, else will add unique variables to the current list.
	 * @param context
	 * @param variables
	 * @param initialize
	 */
	public static void deregisterVariableForParallelFlow(TransformationContext context, List variables){
		List variableList  = getParallelFlowsRegisteredVariables(context);	
		if(variables != null && !variables.isEmpty()){
			variableList.removeAll(variables); 
		}
	}
	/**
	 * Returns the list of variables in the current path. If no list was yet initialized, will initialize the list.
	 * @param context
	 * @return
	 */
	public static List getCurrentPathParallelFlowsRegisteredVariables(TransformationContext context){
		List variablesList = (List)context.get(CURRENT_PARALLEL_PATH_REGISTERED_VARIABLES) ;
		if(variablesList == null){
			variablesList = new LinkedList();
			context.put(CURRENT_PARALLEL_PATH_REGISTERED_VARIABLES, variablesList);
		}
		return variablesList;
	}

	/**
	 * Registers the <code>variable</code> in the current path. Will not register unique, the variable will be just added to the list. 
	 * We will end up with duplicates that are necessary for determining the list of variables used in a given parallel branch.
	 * @param context
	 * @param variable
	 */
	public static void registerVariableForParallelFlowCurrentPath(TransformationContext context, BPELVariable variable){
		List variableList  = getCurrentPathParallelFlowsRegisteredVariables(context); 
		//		if(!variableList.contains(variable)){
		variableList.add(variable);
		//		}

	}

	/**
	 * Registers te list of <code>variables</code> as variables in the current path. If the <code>initialize</code> is set to true,
	 * will replace the existing variables with the new list, else will add unique variables to the current list.
	 */
	public static void registerVariablesForCurrentPath(TransformationContext context, List variables, boolean initialize){
		List variableList;
		if(initialize){
			variableList = new LinkedList();
		}else{
			variableList  = getCurrentPathParallelFlowsRegisteredVariables(context);	
		}
		if(variables != null && !variables.isEmpty()){
			Iterator it = variables.iterator();
			while(it.hasNext()){
				BPELVariable variable = (BPELVariable)it.next();
				if(!variableList.contains(variable)){
					variableList.add(variable);
				}
			}
			//			variableList.addAll(variables);	
		}
		context.put(CURRENT_PARALLEL_PATH_REGISTERED_VARIABLES, variableList);
	}

	public static void initializeVariableForCurrentPath(TransformationContext context){
		context.put(CURRENT_PARALLEL_PATH_REGISTERED_VARIABLES, new LinkedList());
	}

	public static void initializeVariableForParallelFlowCurrentPath(TransformationContext context){
		context.put(PARALLEL_FLOWS_REGISTERED_VARIABLES, new LinkedList());
	}

	public static Component getProcessComponent(TransformationContext context) {
		return (Component) context.get(PROC_COMPONENT_KEY);		
	}
	public static void putProcessComponent(TransformationContext context,Component component) {
		context.put(PROC_COMPONENT_KEY,component);		
	}

	/**
	 * Registers the current PortType in the context
	 * @param context
	 * @param portType
	 */
	public static void putCurrentPortType(TransformationContext context, PortType portType){
		context.put(PORT_TYPE_KEY, portType);
	}
	/**
	 * Register the xsdImport created for a required Message with the PortType that this is used by.
	 * @param context a TransformationContext
	 * @param xsdImport an XSDImport for a XSDType 
	 */
	public static void registerMessageTypeForPortType(TransformationContext context, XSDImport xsdImport){

		PortType portType = (PortType)context.get(PORT_TYPE_KEY);
		if(portType != null){
			Map portTypeRegistry =(Map)context.get(TYPE_PORT_TYPE_REGISTRY_MAP_KEY);
			if(portTypeRegistry == null){
				portTypeRegistry = new HashMap();
				context.put(TYPE_PORT_TYPE_REGISTRY_MAP_KEY, portTypeRegistry);
			}
			List portTypeEntry = (List)portTypeRegistry.get(portType);
			if(portTypeEntry == null){
				portTypeEntry = new LinkedList();
				portTypeRegistry.put(portType, portTypeEntry);
			}
			if(!portTypeEntry.contains(xsdImport)){
				portTypeEntry.add(xsdImport);
			}
		}
	}
	
	/**
	 * Register the xsdConstruct created for a Modeler BusinessItem or BusinessItem property against the Modeler displayName.
	 * @param context a TransformationContext
	 * @param xsdConstruct an XSDTypeDefinition or XSDElementDeclaration 
	 */
	public static void registerDisplayNameForXSDElement(TransformationContext context, Object xsdConstruct, String displayName){

		Map elementToDisplayName = (Map)context.get(ELEMENT_TO_DISPLAY_NAME_KEY);
		if(elementToDisplayName == null){
			elementToDisplayName = new HashMap();
			context.put(ELEMENT_TO_DISPLAY_NAME_KEY, elementToDisplayName);
		}
		elementToDisplayName.put(xsdConstruct, displayName);
	}
	
	
	/**
	 * Returns the name that was registered against the specific xsd construct
	 * @param context a TransformationContext
	 * @param xsdConstruct an XSDTypeDefinition or XSDElementDeclaration 
	 */
	public static String getDisplayNameForXSDElement(TransformationContext context, Object xsdConstruct){
		Map<Object, String> elementToDisplayName = (Map)context.get(ELEMENT_TO_DISPLAY_NAME_KEY);
		if(elementToDisplayName == null){
			return null;
		}
		return elementToDisplayName.get(xsdConstruct);
	}
	/**
	 * Retrieves the list of registered XSDImports (XSDTypes it is using) for a given <code>portType</code>
	 * @param context TransformationContext
	 * @param portType an WSDL PortType
	 * @return List of registered XSDImports.
	 */
	public static List getRegisteredTypesForPortType(TransformationContext context, PortType portType){

		List registeredTypes = new LinkedList();
		Map portTypeRegistry =(Map)context.get(TYPE_PORT_TYPE_REGISTRY_MAP_KEY);
		if(portTypeRegistry == null){
			return registeredTypes;
		}else{
			return (List)portTypeRegistry.get(portType);
		}
	}
	
	
	/**
	 * Checks whether this InputSet is the first on the path inside subprocess or loop
	 *  - such a set has pins that are not connected or connected to start node or connected from subprocess input set
	 * @param ipSet InputPinSet
	 * @return boolean
	 */
	public static boolean pinSetIsEntryToSubprocess(InputPinSet ipSet) {
		if (ipSet == null) {
			return false;
		}
		
		// empty set
		if (ipSet.getInputControlPin().size() + ipSet.getInputObjectPin().size() == 0) {
			return false;
		}
		List <Pin> pins = BomUtils.getPinsInSet(ipSet);
		
		boolean onlyConnectedToContainerOrStartNode = true;
		Action sourceAction = null;
		// check if pins are connected to container or Start Node; this pin is the target of connection.
		for (Pin pin: pins) {
			if (pin.getIncoming() != null) {
				if (pin.getIncoming().getSource() instanceof InitialNode) {
					continue;
				} else if (pin.getIncoming().getSource() instanceof InputControlPin) {
					sourceAction = ((InputControlPin) pin.getIncoming().getSource()).getAction();
					if (sourceAction == ipSet.getAction().getInStructuredNode() && 
							(BomUtils.isLocalProcess(ipSet.getAction().getInStructuredNode()) ||
									ipSet.getAction().getInStructuredNode() instanceof LoopNode))
						continue;
				} else if (pin.getIncoming().getSource() instanceof InputObjectPin) {
					sourceAction = ((InputObjectPin) pin.getIncoming().getSource()).getAction();
					if (sourceAction == ipSet.getAction().getInStructuredNode() && 
							(BomUtils.isLocalProcess(ipSet.getAction().getInStructuredNode()) ||
									ipSet.getAction().getInStructuredNode() instanceof LoopNode))
						continue;
				} else {
					onlyConnectedToContainerOrStartNode = false;
					break;
				}
			}
		}
		
		return onlyConnectedToContainerOrStartNode;
	}
	
	
	// Checks whether a local process or loop has multiple links to different (more than 1) activities on start
	public static boolean manyLinksRequired(InputPinSet iSet) {
		if (iSet == null || !(iSet.getAction() instanceof StructuredActivityNode)) {
			return false;
		}
		
		int numberOfLinks = 0;
		
		// check input pins
		List <Pin> pins = BomUtils.getPinsInSet(iSet);
		
		Action targetAction = null;
		List <ActivityNode> activitiesConnected = new ArrayList<ActivityNode>();
		// check if pins are connected to another activity inside this container
		for (Pin pin: pins) {
			if (pin.getOutgoing() != null) {
				if (pin.getOutgoing().getTarget() instanceof InputControlPin) {
					targetAction = ((InputControlPin) pin.getOutgoing().getTarget()).getAction();
					if (!(activitiesConnected.contains(targetAction))) {
						activitiesConnected.add(targetAction);
						numberOfLinks++;
						if (numberOfLinks > 1) {
							break;
						}
					}
				} else if (pin.getOutgoing().getTarget() instanceof InputObjectPin) {
					targetAction = ((InputObjectPin) pin.getOutgoing().getTarget()).getAction();
					if (!(activitiesConnected.contains(targetAction))) {
						activitiesConnected.add(targetAction);
						numberOfLinks++;
						if (numberOfLinks > 1) {
							break;
						}
					}
				}
			}
		}

		// check for initial nodes
		InitialNode allINode = null;
		if (numberOfLinks < 2) {
			List <InitialNode> initialNodes = new ArrayList<InitialNode>();
			for (Object aNode: ((StructuredActivityNode)iSet.getAction()).getNodeContents()) {
				if (aNode instanceof InitialNode) {
					allINode = (InitialNode)aNode;
					if (allINode.getEntryPoint() == null || allINode.getEntryPoint().isEmpty() || allINode.getEntryPoint().contains(iSet)) {
						initialNodes.add(allINode);
					}
				}
			}
			for (InitialNode iNode: initialNodes) {				
				if (iNode.getOutgoing() != null && iNode.getOutgoing().getTarget() instanceof InputControlPin) {
					targetAction = ((InputControlPin) iNode.getOutgoing().getTarget()).getAction();
					if (!(activitiesConnected.contains(targetAction))) {
						activitiesConnected.add(targetAction);
						numberOfLinks++;
						if (numberOfLinks > 1) {
							break;
						}
					}
				}
			}
		}
		
		if (numberOfLinks > 1) {
			return true;
		} else {
			return false;
		}
	}
	
	public static boolean hasLinksToDifferentActivities(OutputPinSet oSet){
		if (oSet == null) {
			return false;
		}
		
		List <Pin> pins = BomUtils.getPinsInSet(oSet);
		int numberOfLinks = 0;
		
		Action targetAction = null;
		List <ActivityNode> activitiesConnected = new ArrayList<ActivityNode>();
		// check if pins are connected to another activity inside this container
		for (Pin pin: pins) {
			if (pin.getOutgoing() != null) {
				if (pin.getOutgoing().getTarget() instanceof InputControlPin) {
					targetAction = ((InputControlPin) pin.getOutgoing().getTarget()).getAction();
					if (!(activitiesConnected.contains(targetAction))) {
						activitiesConnected.add(targetAction);
						numberOfLinks++;
						if (numberOfLinks > 1) {
							break;
						}
					}
				} else if (pin.getOutgoing().getTarget() instanceof InputObjectPin) {
					targetAction = ((InputObjectPin) pin.getOutgoing().getTarget()).getAction();
					if (!(activitiesConnected.contains(targetAction))) {
						activitiesConnected.add(targetAction);
						numberOfLinks++;
						if (numberOfLinks > 1) {
							break;
						}
					}
				}
			//We need to consider local repository case, which will be mapped to a java snippet, a type of BPEL basic actions
			//in which case we need to keep the fork's empty activity to fork the subsequent BPEL actions
			}else if(pin instanceof StoreArtifactPin){
				numberOfLinks++;
				if (numberOfLinks > 1) {
					break;
				}
			}
		}

		if (numberOfLinks > 1) {
			return true;
		} else {
			return false;
		}
	}
	
	public static InputPinSet findEntrySetToSubprocess(TransformationRule thisRule) {
		if (thisRule == null) {
			return null;
		}
		
		while (!(thisRule instanceof SANNestedProcessRule) || thisRule.getParentRule() == null) {
			thisRule = thisRule.getParentRule();
		}
		
		if (thisRule instanceof SANNestedProcessRule) {
			SANNestedProcessRule sanNestedPRule = (SANNestedProcessRule)thisRule;
			if (!sanNestedPRule.getSource().isEmpty() && sanNestedPRule.getSource().get(0) instanceof InputPinSet) {
				return (InputPinSet) sanNestedPRule.getSource().get(0);
			}
		}
		return null;
	}
}
