package com.ibm.btools.te.ilm.heuristics.helper;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.wsdl.Definition;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.xsd.XSDSchema;

import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.externalmodels.ExternalSchema;
import com.ibm.btools.bom.model.externalmodels.ExternalService;
import com.ibm.btools.bom.model.externalmodels.ServiceInterface;
import com.ibm.btools.bom.model.models.ExternalModel;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.Organization;
import com.ibm.btools.fdl.model.Person;
import com.ibm.btools.fdl.model.ProcessCategory;
import com.ibm.btools.fdl.model.Program;
import com.ibm.btools.fdl.model.Role;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.attributes.helper.TechnicalAttributesHelper;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.ilm.ExportOperationUtil;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.model.sa.SAType;
import com.ibm.btools.te.ilm.model.sa.SaFactory;
import com.ibm.btools.te.ilm.model.sa.SolutionArtifact;
import com.ibm.btools.te.ilm.model.sa.UnclassifiedArtifact;
import com.ibm.btools.te.ilm.model.sa.UnclassifiedArtifactType;
import com.ibm.wsspi.sca.scdl.DocumentRoot;

/**
 * Provides a set of helper method to work with SA related model operations
 */
public class SAUtil {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	public static String CA_DECLARATION_KEY = "ComposedArtifactDeclarationInContext";
	public static String SA_KEY="SolutionArtifactUtilRegistryContext";
	
	public static void registerSolutionArtifact(TransformationContext context, Object key, SolutionArtifact sa) {
		HashMap map = (HashMap) context.get(SA_KEY);
		if (map == null) {
			map = new HashMap();
			context.put(SA_KEY,map);
		}
		map.put(key,sa);
	}

	public static void unregisterSolutionArtifact(TransformationContext context, Object key) {
		HashMap map = (HashMap) context.get(SA_KEY);
		if (map != null) {
			map.remove(key);
		}
	}

	public static SolutionArtifact getSolutionArtifact(TransformationContext context, Object key) {
		HashMap map = (HashMap) context.get(SA_KEY);
		if (map != null) {
			return (SolutionArtifact)map.get(key);
		}
		return null;
	}
	
	/**
	 * Add the target object to the unclassified solution artifact
	 * @param unclassifiedSA
	 * @param target
	 */
	public static void updateUnclassifiedSA (SolutionArtifact unclassifiedSA, EObject target)
	{
		UnclassifiedArtifact artifact=null;
		if (target instanceof XSDSchema)
		{
			artifact = getUnclassifiedArtifact(unclassifiedSA,UnclassifiedArtifactType.XML_SCHEMA_LITERAL);
		} else if (target instanceof Definition) {
			artifact = getUnclassifiedArtifact(unclassifiedSA,UnclassifiedArtifactType.WEB_SERVICES_INTERFACE_LITERAL);
		} else if ((target instanceof Program) || (target instanceof ProcessCategory) ||
					(target instanceof DataStructure) || (target instanceof Organization) ||
					(target instanceof Role) || (target instanceof Person)) {
			artifact = getUnclassifiedArtifact(unclassifiedSA,UnclassifiedArtifactType.WORKFLOW_DEFINITION_LITERAL);
		}
		
		if (artifact != null)
		{
			artifact.getArtifacts().add(target);
			unclassifiedSA.getComposedArtifacts().add(artifact);
		}
	}

	/**
	 * Get an unclassified artifact with Type 'type' from the SolutionArtifact unclassifiedSA
	 * @param unclassifiedSA
	 * @param type
	 * @return
	 */
	public static UnclassifiedArtifact getUnclassifiedArtifact(SolutionArtifact unclassifiedSA, UnclassifiedArtifactType type)
	{
		List cas = unclassifiedSA.getComposedArtifacts();
		Iterator casIt = cas.iterator();
		UnclassifiedArtifact uca=null;
		while (casIt.hasNext())
		{
			uca = (UnclassifiedArtifact)casIt.next();
			if (uca.getType()==type)
				return uca;
		}
		uca = SaFactory.eINSTANCE.createUnclassifiedArtifact();
		uca.setType(type);
		return uca;
	}
	
	/**
	 * Creates an unclassified Solution Artifact
	 * @return
	 */
	public static SolutionArtifact createUnclassifiedSolutionArtifact() {
		SolutionArtifact unclassifiedSA = SaFactory.eINSTANCE.createSolutionArtifact();
		unclassifiedSA.setType(SAType.UNCLASSIFIED_LITERAL);
		return unclassifiedSA;					
	}
	
	/**
	 * Create the package structure of the EObject element:
	 * -If element is BOM element, then it retrieves the catalog structure from the BOM element and
	 * construct the same package structure
	 * -If element is a DocumentRoot for a SCDL module, then the package structure is the name of the Module itself
	 * @param element
	 * @return
	 */
	public static String createPackage (EObject element, boolean includePackageableElement) {
		return createPackage(element,includePackageableElement,File.separator);
	}
	
	/**
	 * see createPackage (EObject element, boolean includePackageableElement)
	 * @param element
	 * @param includePackageableElement
	 * @param separator
	 * @return
	 */
	public static String createPackage (EObject element, boolean includePackageableElement, String separator) {
		JavaNCNameConverter converter = new JavaNCNameConverter();
		if (element instanceof DocumentRoot) {
			DocumentRoot dr = (DocumentRoot)element;
			if (dr.getModule()!=null) {
				return dr.getModule().getName()+separator;
			}
		} else {
			PackageableElement pe=null;
			while (!(element instanceof PackageableElement) && element.eContainer()!=null) {
				element = element.eContainer();
			}
			if (element instanceof PackageableElement) {
				pe = (PackageableElement)element;				
			}
			if (pe!=null) {
				StringBuffer sb = new StringBuffer();
				if(!BOMUtil.isServiceOperation(pe)){
					if (includePackageableElement) {
						//defect 116657
						if(pe instanceof Activity ){
							Activity act = (Activity)pe;
							String name = null;
							//for the Process use the BPEL process name (TA or default)
							if(act.getImplementation() != null &&  BomUtils.isProcess(act.getImplementation())){
								name = (String)TechnicalAttributesHelper.getAttributeValue((NamedElement)act.getImplementation(), AttributeNameConstants.BPEL_PROCESS_NAME);
							//for the SAN use the PortType name
							}else if(act.getImplementation() != null && (BomUtils.isReusableTask(act.getImplementation()) || BomUtils.isReusableService(act.getImplementation()))){
								name = (String)TechnicalAttributesHelper.getAttributeValue((NamedElement)act.getImplementation(), AttributeNameConstants.WSDL_PORT_TYPE_NAME);
							}
							if(name != null){
								sb.append(converter.convertName(name).toLowerCase()+separator);
							}else{
								sb.append(converter.convertName(pe.getName()).toLowerCase()+separator);
							}
						}else{
							sb.append(converter.convertName(pe.getName()).toLowerCase()+separator);	
						}
					}
					while (pe.getOwningPackage()!=null) {
						pe = pe.getOwningPackage();
						if (!ProcessUtil.isRootPackage((com.ibm.btools.bom.model.artifacts.Package)pe)) {
							if(pe instanceof ExternalModel && ExportOperationUtil.ignoreDefaultCatalog((ExternalModel)pe)){
								continue;
							}
							sb.insert(0,converter.convertName(pe.getName()).toLowerCase()+separator);						
					}
				  }
				}else{
					while (pe.getOwningPackage()!=null) {
						pe = pe.getOwningPackage();
						if (!ProcessUtil.isRootPackage((com.ibm.btools.bom.model.artifacts.Package)pe)) {
//					if(pe instanceof ServiceInterface || pe instanceof ExternalService){
							//defect 87059, 86515
							if(pe instanceof ServiceInterface){
								continue;
							}
							if(pe instanceof ExternalModel && ExportOperationUtil.ignoreDefaultCatalog((ExternalModel)pe)){
								continue;
							}
							sb.insert(0,converter.convertName(pe.getName()).toLowerCase()+separator);		
						}
					}
				}
				return sb.toString();			
			}			
		}
		return "";
	}
	
	public static String createPackage (EObject element) {
		return createPackage(element,true);
	}

	public static String createPackage (EObject element,String separator) {
		return createPackage(element,true,separator);
	}	
}
