package com.ibm.btools.te.ilm.heuristics.helper;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.ie.exprt.ExportMessage;
import com.ibm.btools.blm.ie.exprt.ExportResult;
import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.emf.cf.BtCompoundCommand;

import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.ilm.TransformationSession;
import com.ibm.btools.te.validation.model.TeValidationRecord;
import com.ibm.btools.util.datatype.BTDateTime;

/**
 * Helper class 
 * */
public class TransformationSessionUtil implements TransformationSessionKeyConstants{

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2008, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	
	
	public static BtCompoundCommand getUpdateCommand(TransformationSession session) {
		BtCompoundCommand command= (BtCompoundCommand)session.get(UPDATE_COMMAND);
		if(command == null){
			command = new BtCompoundCommand();
			session.put(UPDATE_COMMAND, command);
		}
		return command;
	}
	
	public static void appendAndExecuteUpdateCommand(TransformationSession session, Command cmd){
		//List cmdList = getUpdateCommandList(session);
		BtCompoundCommand compoundCommand = getUpdateCommand(session);
		if(cmd != null){
			compoundCommand.appendAndExecute(cmd);
			//log if error
		}
	}
	
	public static List getValidationRecordList(TransformationSession session){
		List recList = (List)session.get(TECH_ATTRIB_VALIDATION_RECORDS);
		if(recList == null){
			recList = new ArrayList();
			session.put(TECH_ATTRIB_VALIDATION_RECORDS, recList);
		}
		return recList;
	}
	
	public static void addValidationRecord(TransformationSession session, TeValidationRecord rec){
		List recList = getValidationRecordList(session);
		recList.add(rec);		
	}
	
	public static void putMasterCopy(TransformationSession session, Activity activity){
		session.put(MASTER_COPY, activity);
	}
	
	public static Object getMasterCopy(TransformationSession session){
		return session.get(MASTER_COPY);
	}
		//defect 139703, registrer the MasterCopy with the working copy
	public static void putMasterCopy(TransformationSession session, Activity activity, Activity workingCopy){
		HashMap entry = new HashMap();
		entry.put(workingCopy, activity);
		session.put(MASTER_COPY, entry);
	}
		//defect 139703, registrer the MasterCopy with the working copy
	public static Object getMasterCopy(TransformationSession session, Activity workingCopy){
		if(session.get(MASTER_COPY) == null){
			return null;
		}else{
			HashMap entry = (HashMap)session.get(MASTER_COPY);
			return entry.get(workingCopy);
		}
		
	}
	
	
	public static void removeValidationRecord(TransformationSession session, NamedElement bomSource, String property){
		List recList = getValidationRecordList(session);
		List obsoleteList = new LinkedList();
		if(recList.size()>0){
			Iterator it = recList.iterator();
			
			while(it.hasNext()){
				TeValidationRecord record = (TeValidationRecord)it.next();
				if(record.getBomElement().equals(bomSource) && record.getPropertyName().equals(property)){
					obsoleteList.add(record);
				}
			}
			recList.removeAll(obsoleteList);
		}
	}
	

	public static void setExportResult(TransformationSession context, ExportResult exportResult){
		context.put(TRANSFORMATION_SESSION_EXPORT_RESULT, exportResult);
		
		
	}
	
	public static ExportResult getExportResult(TransformationSession context){
		return (ExportResult)context.get(TRANSFORMATION_SESSION_EXPORT_RESULT);
		
	}
	
	public static void registerProcessedElementWithNamespace(TransformationSession context, EObject element, String namespace){
		Map modelElementsList = getProcessedElementsRegistry(context);
		
		modelElementsList.put(element, namespace);
		
	}
	private static Map getProcessedElementsRegistry(TransformationSession context){
		Map modelElementsList = (Map)context.get(PROCESSED_ARTIFACTS_REGISTRY);
		if(modelElementsList == null){
			modelElementsList = new HashMap();
			context.put(PROCESSED_ARTIFACTS_REGISTRY, modelElementsList);
		}
		return modelElementsList;
	}
	
	public static String getProcessedElementNamespace(TransformationSession context, EObject element){
		return (String)getProcessedElementsRegistry(context).get(element);
	}
	
	/**
	 * This method returns the value that should be set for the 'valid from' attribute
	 * of the given top-level SAN representing a process, if it exists.  Cases where
	 * this value might be present include when the the transformation engine is 
	 * invoked by the OME for the purposes of creating the export bundle to monitor.
	 * 
	 * @param context The TransformationSession that might contain the valid from override value 
	 * @param topLevelSAN The SAN that represents the process
	 * @return A BTDateTime object containing the valid from value, if it exists.
	 * Null is returned otherwise, indicating that there is no override value provided.
	 */
	public static BTDateTime getValidFromOverride(TransformationSession context, StructuredActivityNode topLevelSAN){
		/* First, find the containing activity of the SAN, if it exists */
		Activity activity = topLevelSAN.getActivity();
		if (activity == null){
			return null;
		}
		
		HashMap overrides = (HashMap)context.get("VALID_FROM_OVERRIDE");
		if (overrides == null){
			return null;
		}
		
		return (BTDateTime)overrides.get(activity);
	}
	
	/**
	 * This method sets the overrides for the 'valid from' attributes of processes 
	 * in the Transformation Session where they can be later accessed.
	 * 
	 * @param context The TransformationSession to set the overrides to 
	 * @param overrides The HashMap of overrides to assign.
	 */
	public static void setValidFromOverride(TransformationSession context, HashMap overrides){
		context.put("VALID_FROM_OVERRIDE", overrides);
	}	
	
	public static ExportSession getExportSession(TransformationSession context) {
		if(context == null) return null;
		return (ExportSession)context.get(EXPORT_SESSION);
	}

	public static void setExportSession(TransformationSession context,ExportSession exportSession) {
		context.put(EXPORT_SESSION,exportSession);
	}
	
	/**
	 * Add VMM group from organization to current list stored in the context under VMMConstants.VMM_GROUP_ORG key
	 */
	public static void registerVMMGroupsFromOrg(TransformationSession session, VMMGroupEntry group) {
		List currentResources = new ArrayList<VMMGroupEntry>(); 
		if (session.get(StaffUtil.VMMConstants.VMM_GROUP_ORG) != null){
			currentResources = (List<VMMGroupEntry>) session.get(StaffUtil.VMMConstants.VMM_GROUP_ORG);
		}
		//Add current group from organization to list of VMM groups stored in the context
		currentResources.add(group);
		//Store updated list into session context
		session.put(StaffUtil.VMMConstants.VMM_GROUP_ORG, currentResources);
		
	}
	
	/**
	 * Add VMM group from role to current list stored in the context under VMMConstants.VMM_GROUP_ROLE key
	 */
	public static void registerVMMGroupsFromRole(TransformationSession session, VMMGroupEntry vmmGroupEntry){
		List currentResources = new ArrayList<VMMGroupEntry>(); 
		if (session.get(StaffUtil.VMMConstants.VMM_GROUP_ROLE) != null){
			currentResources = (List<VMMGroupEntry>) session.get(StaffUtil.VMMConstants.VMM_GROUP_ROLE);
		}
		//Add current group from role to list of VMM groups stored in the context
		currentResources.add(vmmGroupEntry);
		//Store updated list into session context
		session.put(StaffUtil.VMMConstants.VMM_GROUP_ROLE, currentResources);
	}
	
	/**
	 * Getter method for retrieving VMM group from role from context
	 */	
	public static List getVMMGroupsFromRole(TransformationSession session){
		return (List)session.get(StaffUtil.VMMConstants.VMM_GROUP_ROLE);
		
	}
	
	/**
	 * Getter method for retrieving VMM group from organization from context
	 */	
	public static List getVMMGroupsFromOrg(TransformationSession session){
		return (List)session.get(StaffUtil.VMMConstants.VMM_GROUP_ORG);
		
	}

	/**
	 * Add VMM person to current list stored in the context under VMMConstants.VMM_PERSON key
	 */
	public static void registerVMMPerson(TransformationSession session, VMMPersonEntry person) {
		List currentResources = new ArrayList<VMMGroupEntry>(); 
		if (session.get(StaffUtil.VMMConstants.VMM_PERSON) != null){
			currentResources = (List<VMMGroupEntry>) session.get(StaffUtil.VMMConstants.VMM_PERSON);
		}
		//Add current person to list of VMM persons stored in the context
		currentResources.add(person);
		//Store updated list into session context
		session.put(StaffUtil.VMMConstants.VMM_PERSON, currentResources);
		
	}
	
	/**
	 * Getter method for retrieving VMM person from context
	 */	
	public static List getVMMPersons(TransformationSession session){
		return (List)session.get(StaffUtil.VMMConstants.VMM_PERSON);
		
	}

	public static void setHasHumanTask(TransformationSession session) {
		// TODO Auto-generated method stub
		session.put("EXPORT_HUMAN_TASK", Boolean.TRUE);
	}
	
	public static boolean hasHumanTask(TransformationSession session) {
		// TODO Auto-generated method stub
		
		if(session.get("EXPORT_HUMAN_TASK") != null){
			return true;
		}
		return false;
	}
}
