/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.xsd.XSDSchema;

import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.externalmodels.ExternalSchema;
import com.ibm.btools.bom.model.externalmodels.ExternalService;
import com.ibm.btools.bom.model.externalmodels.ServiceInterface;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnit;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.humantasks.Form;
import com.ibm.btools.bom.model.resources.IndividualResource;
import com.ibm.btools.bom.model.resources.Role;
import com.ibm.btools.bom.model.services.Interface;
import com.ibm.btools.bom.model.time.RecurringTimeIntervals;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.impl.TransformationRootImpl;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.CrossProjectReferenceUtil;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.graphs.BPGraph;
import com.ibm.btools.te.ilm.heuristics.DataDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.HeuristicsFactory;
import com.ibm.btools.te.ilm.heuristics.HeuristicsPackage;
import com.ibm.btools.te.ilm.heuristics.HeuristicsTransformation;
import com.ibm.btools.te.ilm.heuristics.InterfaceRule;
import com.ibm.btools.te.ilm.heuristics.OrganizationRule;
import com.ibm.btools.te.ilm.heuristics.ProcessFlowPatternRule;
import com.ibm.btools.te.ilm.heuristics.ResourceRule;
import com.ibm.btools.te.ilm.heuristics.TimeIntervalsRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.FormRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.HumanTaskRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessInterfaceRule;
import com.ibm.btools.te.ilm.heuristics.extservice.ExtserviceFactory;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.FabricIntegrationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.NamespaceNamingRegistry;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.helper.SAUtil;
import com.ibm.btools.te.ilm.heuristics.naming.ComponentNameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NoHashNamingRegistry;
import com.ibm.btools.te.ilm.heuristics.naming.ProjectNamingRegistry;
import com.ibm.btools.te.ilm.heuristics.scdl.util.ScdlUtil;
import com.ibm.btools.te.ilm.heuristics.util.TimeConstants;
import com.ibm.btools.te.ilm.heuristics.wsdl.util.OriginalWsdlUtil;
import com.ibm.btools.te.ilm.model.sa.HumanTask;
import com.ibm.btools.te.ilm.model.sa.ProcessFlow;
import com.ibm.btools.te.ilm.model.sa.ProcessFlowType;
import com.ibm.btools.te.ilm.model.sa.SAType;
import com.ibm.btools.te.ilm.model.sa.SaFactory;
import com.ibm.btools.te.ilm.model.sa.ScdlArtifact;
import com.ibm.btools.te.ilm.model.sa.SolutionArtifact;
import com.ibm.btools.te.ilm.model.sa.UnclassifiedArtifact;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.wbit.businesscalendar.ICalendar;
import com.ibm.wsspi.sca.scdl.Component;
import com.ibm.wsspi.sca.scdl.DocumentRoot;
import com.ibm.wsspi.sca.scdl.Export;
import com.ibm.wsspi.sca.scdl.Import;
import com.ibm.wsspi.sca.scdl.Reference;
import com.ibm.wsspi.sca.scdl.SCDLFactory;
import com.ibm.wsspi.sca.scdl.Wire;


/**
 * This is a TransformationRoot that represent a Transformation Session
 * utilizing Heuristics patterns.  The types of pattern include
 * <p>
 * 		ProcessFlowPattern - Direct mapping BLM process to ILM process, e.g. BPEL, FDL
 * </p>
 * Source : BPGraph
 * Target : a set of generic IT graphs and Data definitions
 */
public class HeuristicsTransformationImpl extends TransformationRootImpl implements HeuristicsTransformation {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private EList firstClassArtifacts;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected HeuristicsTransformationImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return HeuristicsPackage.Literals.HEURISTICS_TRANSFORMATION;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	public boolean isFirstClassArtifact(EObject artifact) {
		if (artifact instanceof Activity)
			return firstClassArtifacts.contains(artifact);
		return false;
	} 

	/**
	 * <!-- begin-user-doc -->
	 * Source : BPGraph
	 * Target : a set of generic IT graphs and Data definitions
	 * Rule:
	 * Go through the BPGraph and extract all top level processes.
	 * For each top level process, it creates a PatternRule based
	 * on user preference (or user interaction).  For example, if
	 * ProcessFlow pattern is preferred, it creates a 
	 * ProcessFlowPatternRule for each process and set the process 
	 * as the source of the ProcessFlowPatternRule.  Other solution 
	 * pattern rules include AEBrokerRule and AEObserverRule.
	 * Creates a SolutionRule and set the generic graphs created 
	 * from the PatternRule as the sources. Extracts the 
	 * DataDefinitionRule in the context and collects the data 
	 * definition targets.  These DataDefinitionRules will be put 
	 * in the context during the transformation e.g. when creating 
	 * WSDL Message Parts, the DataDefinitionRule will be created 
	 * for transforming datatypes (complex types) and will be put 
	 * in the context to be shared by different rules that may 
	 * possibly requires the same complex type definition.
	 * 
	 * AR
	 * A SolutionRule is created for this transformation and saved in the 
	 * context.
	 * As processes are traversed, the LinkRules will be added to the SolutionRule as
	 * child rules. 
	 * 
	 * <!-- end-user-doc -->
	 */
	public boolean transformSource2Target(){
		LoggingUtil.traceEntry(this, "transformSource2Target");
		
		initialize();
		BPGraph bpGraph = (BPGraph)getSource().get(0);
		firstClassArtifacts = bpGraph.getFirstClassArtifact();
		Iterator iterator = firstClassArtifacts.iterator();
		while (iterator.hasNext())
		{
			EObject nextArtifact = (EObject)iterator.next();
			if (nextArtifact instanceof Activity)
			{
				ProcessFlowPatternRule rule = HeuristicsFactory.eINSTANCE.createProcessFlowPatternRule();
				getChildRules().add(rule);
				rule.getSource().add(nextArtifact);
				//Register the interface activity
				Activity activity = (Activity)nextArtifact;
				if(BomUtils.isProcess(activity.getImplementation())){
					FabricIntegrationUtil.getInterfaceActivityIfAny(activity.getImplementation());
				}
			}else if(nextArtifact instanceof Interface){
				InterfaceRule rule = HeuristicsFactory.eINSTANCE.createInterfaceRule();
				getChildRules().add(rule);
				rule.getSource().add(nextArtifact);
			} else if (nextArtifact instanceof Type) {
				if(BOMUtil.isFileAttachment((Type)nextArtifact)){
					//BOMUtil.isInlineComplexType((Type)nextArtifact)){
					//continue;
					//JR31759 Handle service definition export during entire project export
					if(nextArtifact instanceof com.ibm.btools.bom.model.artifacts.Class){
						com.ibm.btools.bom.model.artifacts.Class clazz =(com.ibm.btools.bom.model.artifacts.Class)nextArtifact;
						if(clazz.getOwningPackage() instanceof ExternalSchema){
							ExternalSchema externalSchema = (ExternalSchema)clazz.getOwningPackage();
							if(externalSchema.getOwningPackage()instanceof ExternalService){
								ExternalService externalService = (ExternalService)externalSchema.getOwningPackage();
								if(externalService.getOwnedMember()!= null && !externalService.getOwnedMember().isEmpty()){
									boolean existServiceInterface = false;
									Iterator  members = externalService.getOwnedMember().iterator();
									while(members.hasNext()){
										Object eachItem = members.next();
										if(eachItem instanceof ServiceInterface){
											existServiceInterface = true;
											break;
										}
									}
									//If this is the service definition only
									if(!existServiceInterface){
										//Export the WSDL definition									
										TransformationRule serviceDefinitionRule = ExtserviceFactory.eINSTANCE.createWSDLDefinitionRule();
										serviceDefinitionRule.getSource().add(externalService);										
										Definition serviceDefinition = OriginalWsdlUtil.getOriginalWsdl(getContext(), clazz);
										if(serviceDefinition != null){
											serviceDefinitionRule.getTarget().add(serviceDefinition);
											serviceDefinitionRule.setComplete(true);
											//Initialize the mapping manager in case the un-mapped artifacts need to be
											//mapped when persisting targets
											MappingManager mappingManager = MappingManager.getInstance(getContext());
											if(mappingManager != null){
												mappingManager.addMapDefinition(externalService, serviceDefinition);
											}
										}
										getChildRules().add(serviceDefinitionRule);											
									}
								}
							}
						}
					}	
					continue;
				}
				DataDefinitionRule rule = HeuristicsFactory.eINSTANCE.createDataDefinitionRule();
				getChildRules().add(rule);
				rule.getSource().add(nextArtifact);
			}else if ( (nextArtifact instanceof IndividualResource) || (nextArtifact instanceof Role)) {
				ResourceRule rule = HeuristicsFactory.eINSTANCE.createResourceRule();
				getChildRules().add(rule);
				rule.getSource().add(nextArtifact);
			} else if (nextArtifact instanceof RecurringTimeIntervals) {
				TimeIntervalsRule rule = HeuristicsFactory.eINSTANCE.createTimeIntervalsRule();
				getChildRules().add(rule);
				rule.getSource().add(nextArtifact);
			}else if (nextArtifact instanceof OrganizationUnit) {
				OrganizationRule rule = HeuristicsFactory.eINSTANCE.createOrganizationRule();
				getChildRules().add(rule);
				rule.getSource().add(nextArtifact);
			}else if (nextArtifact instanceof Form) {
				FormRule rule = AbstractbpelFactory.eINSTANCE.createFormRule();
				getChildRules().add(rule);
				rule.getSource().add(nextArtifact);
			}
		}

		boolean ret = executeChildRules();

		processChildTargets();

		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}

	private void initialize() {
		// Reset BOMHelper for each transformation session
		BomHelper.getInstance().reset();
		//reset the namespace naming registry
		NamespaceNamingRegistry.reset();
		// TODO: should do it through extension point
		// TODO: change classname when plugins consolidation is done
		// MetaObjectReferenceResolverFactory.getInstance().registerReferenceResolver("org.eclipse.xsd.XSDParticle","com.ibm.btools.te.ilm.heuristics.xsd.XSDParticleReferenceResolver");
	}
	
	private void processChildTargets() {
		Iterator it = getChildRules().iterator();
		TransformationRule childRule;
		EObject target;
		Iterator targetIt;
		SCDLArtifactUpdater scdlArtifactUpdater = new SCDLArtifactUpdater();
		
		// Process targets
		while (it.hasNext())
		{
			childRule = (TransformationRule)it.next();
			if (!childRule.getTarget().isEmpty()) {
				targetIt = childRule.getTarget().iterator();
				while (targetIt.hasNext()) {
					target = (EObject)targetIt.next();
					if (childRule instanceof ProcessInterfaceRule) {
						SolutionArtifact sa = processUnclaimedProcessInterfaceRule((EObject)childRule.getSource().get(0),target);
						getTarget().add(sa);
					}else if(childRule instanceof HumanTaskRule){
						//d76803
						if (target instanceof ICalendar && childRule.getSource().get(0) instanceof com.ibm.btools.bom.model.processes.humantasks.HumanTask) {
							String encodedNamespace = ProcessUtil.createNamespace((com.ibm.btools.bom.model.processes.humantasks.HumanTask)childRule.getSource().get(0),true);
							ICalendar cal = (ICalendar) target;
							if (cal.getTargetNamespace() != null &&
									cal.getTargetNamespace().equals(encodedNamespace + TimeConstants.CALENDAR_SUFFIX)) {	
								SolutionArtifact sa = processUnclaimedCalendarForHumanTaskRule((EObject)childRule.getSource().get(0), target);
								getTarget().add(sa);
							} else {
								//find RecurringTimeIntervals from the target
								TransformationRule tr = TransformationUtil.getRuleForTarget(this, TimeIntervalsRule.class, target);
								if (tr != null && !getChildRules().contains(tr)) {
									TimeIntervalsRule tiRule = (TimeIntervalsRule)tr;
									if (tiRule.getTarget() != null && tiRule.getTarget().size() == 1 && tiRule.getSource() != null &&
											tiRule.getSource().size() == 1 && tiRule.getSource().get(0) instanceof RecurringTimeIntervals) {
										SolutionArtifact sa = processUnclaimedTimeIntervalsRule((EObject)tiRule.getSource().get(0), target);
										getTarget().add(sa);										
									}
								}
							}
						} else {
							SolutionArtifact sa = null;
							//Use the CBA's package as the tel's package
							if(childRule.getSource().size() >= 3 && (childRule.getSource().get(childRule.getSource().size()-1) instanceof CallBehaviorAction) && 
									FabricIntegrationUtil.isOverriden((CallBehaviorAction)(childRule.getSource().get(childRule.getSource().size()-1)))){
								CallBehaviorAction cba = (CallBehaviorAction)(childRule.getSource().get(childRule.getSource().size()-1));
								sa =  processUnclaimedHumanTaskRule(cba, target);
							}else{
								sa = processUnclaimedHumanTaskRule((EObject)childRule.getSource().get(0), target);
							}
							getTarget().add(sa);
						}
					}else if(childRule instanceof TimeIntervalsRule){ 
						if (childRule.getSource().size() == 1) {
							SolutionArtifact sa = processUnclaimedTimeIntervalsRule((EObject)childRule.getSource().get(0), target);
							getTarget().add(sa);
						}
					} else if(childRule instanceof FormRule){
						SolutionArtifact sa = processUnclaimedFormRule((EObject)childRule.getSource().get(0), target);
						getTarget().add(sa);
					} else {
						if (target instanceof SolutionArtifact)
						{
							getTarget().add(target);
							if (((SolutionArtifact)target).getType()==SAType.SCDL_LITERAL) {
								scdlArtifactUpdater.registerSCDLArtifact((SolutionArtifact)target);
							}
						} else {
							processOtherTargets(target);
						}
					}
				}
			}
		}
		getTarget().addAll(scdlArtifactUpdater.update());
	}
	
	private SolutionArtifact processUnclaimedProcessInterfaceRule(EObject source,EObject target) {
		// if the Interface is for the local task, we should create an UnclassifiedSA for it
		if (source instanceof Action &&
			!BomUtils.isProcess((Action)source) && !BomUtils.isReusableService((Action)source) &&! BomUtils.isReusableTask((Action)source) &&
			!BomUtils.isReusableBusinessRuleTask((Action)source)) {
			SolutionArtifact unclassifiedSA = SAUtil.createUnclassifiedSolutionArtifact();
			SAUtil.updateUnclassifiedSA(unclassifiedSA,target);
			String path = null;
			if (target instanceof Definition) {
				Action action = (Action)source;
				NoHashNamingRegistry noHashReg = null;
				if (NamingUtil.USE_PER_PROJECT_UNIQUENESS) {
					String projectName = CrossProjectReferenceUtil.getProjectNameForBOMElement(action);
					noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(context, NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(projectName);
				} else {
					noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(context, NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(NamingUtil.GLOBAL_XPROJECT_UNIQUENESS);
				}
				Object contextContainer = NamingUtil.getContextContainer(action);
				String wsdlPath = null;

				if (noHashReg.actionExistInLocalRegistry(action, contextContainer)) {
					wsdlPath = noHashReg.retrieveNameFromLocalRegistry(action, contextContainer);
				} else if(noHashReg.actionExistInGlobalRegistry(action)) {
					wsdlPath = noHashReg.retrieveNameFromGlobalRegistry(action);
				}
				if (wsdlPath != null) {
					if (wsdlPath.indexOf(ExportOperation.FILE_SEPARATOR) != -1) {
						path = wsdlPath.substring(0, wsdlPath.lastIndexOf(ExportOperation.FILE_SEPARATOR) + 1);
						path = path.replace(ExportOperation.FILE_SEPARATOR_CHAR, File.separatorChar);
					}
				}
			}	

			if (path != null) {
				unclassifiedSA.setPackage(path);	
			} else {
				unclassifiedSA.setPackage(SAUtil.createPackage(source));
			}
			return unclassifiedSA;
		}
		SolutionArtifact sa = SaFactory.eINSTANCE.createSolutionArtifact();
		sa.setType(SAType.PROCESS_FLOW_LITERAL);
		ProcessFlow pf = SaFactory.eINSTANCE.createProcessFlow();
		sa.getComposedArtifacts().add(pf);
		pf.setType(ProcessFlowType.BPEL_LITERAL);
		pf.getProcessInterface().add(target);
		
		if (source instanceof Action && source.eContainer() instanceof Activity) {
			String path = null;
			if (target instanceof Definition) {
				Action action = (Action)source;
	
				NoHashNamingRegistry noHashReg = null;
				if (NamingUtil.USE_PER_PROJECT_UNIQUENESS) {
					String projectName = CrossProjectReferenceUtil.getProjectNameForBOMElement(action);
					noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(context, NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(projectName);
				} else {
					noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(context, NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(NamingUtil.GLOBAL_XPROJECT_UNIQUENESS);
				}
				Object contextContainer = NamingUtil.getContextContainer(action);
				String wsdlPath = null;
				if(noHashReg.actionExistInGlobalRegistry(action)) {
					wsdlPath = noHashReg.retrieveNameFromGlobalRegistry(action);
				} else if (noHashReg.actionExistInLocalRegistry(action, contextContainer)) {
					wsdlPath = noHashReg.retrieveNameFromGlobalRegistry(action);
				}
				if (wsdlPath != null) {
					if (wsdlPath.indexOf(ExportOperation.FILE_SEPARATOR) != -1) {
						path = wsdlPath.substring(0, wsdlPath.lastIndexOf(ExportOperation.FILE_SEPARATOR) + 1);
						path = path.replace(ExportOperation.FILE_SEPARATOR_CHAR, File.separatorChar);
					}
				}
			}	
			if (path != null) {
				sa.setPackage(path);	
			} else {
				sa.setPackage(SAUtil.createPackage(source));
			}
		} else {
			sa.setPackage(SAUtil.createPackage(source));
		}
		return sa;
	}
	
	private SolutionArtifact processUnclaimedHumanTaskRule(EObject source, EObject target) {
		SolutionArtifact sa = SaFactory.eINSTANCE.createSolutionArtifact();
		sa.setType(SAType.HUMAN_TASK_LITERAL);
		HumanTask ht = SaFactory.eINSTANCE.createHumanTask();
		ht.setInline(false);
		sa.getComposedArtifacts().add(ht);
		sa.setPackage(SAUtil.createPackage(source));
		ht.setHumanTask(target);
		return sa;
	}

	private SolutionArtifact processUnclaimedCalendarForHumanTaskRule(EObject source, EObject target) {
		SolutionArtifact sa = SAUtil.createUnclassifiedSolutionArtifact();
		UnclassifiedArtifact ua = SaFactory.eINSTANCE.createUnclassifiedArtifact();
		sa.getComposedArtifacts().add(ua);
		sa.setPackage(SAUtil.createPackage(source));
		ua.getArtifacts().add(target);
		return sa;
	}

	private SolutionArtifact processUnclaimedTimeIntervalsRule(EObject source, EObject target) {
		SolutionArtifact sa = SAUtil.createUnclassifiedSolutionArtifact();
		UnclassifiedArtifact ua = SaFactory.eINSTANCE.createUnclassifiedArtifact();
		sa.getComposedArtifacts().add(ua);

		String registeredName = null;
		if (source instanceof RecurringTimeIntervals) {
			RecurringTimeIntervals rti = (RecurringTimeIntervals)source;
			NoHashNamingRegistry noHashReg = null;
			if (NamingUtil.USE_PER_PROJECT_UNIQUENESS) {
				String projectName = CrossProjectReferenceUtil.getProjectNameForBOMElement(rti);
				noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(context, NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(projectName);
			} else {
				noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(context, NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(NamingUtil.GLOBAL_XPROJECT_UNIQUENESS);
			}
			
			if(noHashReg.timetableExistInTimetableRegistry(rti)) {
				registeredName = noHashReg.retrieveNameFromTimetableRegistry(rti);
			} else {	// element not registered yet
				String name = JavaNCNameConverter.replaceInvalidName(rti.getName());
				String pathName = SAUtil.createPackage(rti, ExportOperation.FILE_SEPARATOR) + name;
				if (!noHashReg.isUniqueNameInTimetableRegistry(pathName)) {
					String genName = NamingUtil.generateNewTimetableName(pathName, noHashReg);
					noHashReg.registerTimetable(rti, genName);
					registeredName = genName;					
				} else {
					noHashReg.registerTimetable(rti, pathName);
					registeredName = pathName;
				}						
			}
		}

		if (registeredName != null) {
			String packageName = null;
			if (registeredName.indexOf(ExportOperation.FILE_SEPARATOR) > -1) {
				packageName = registeredName.substring(0, registeredName.lastIndexOf(ExportOperation.FILE_SEPARATOR) + 1);
			} else{
				packageName = registeredName;
			}
			
			sa.setPackage(packageName);
		} else {
			sa.setPackage(SAUtil.createPackage(source));
		}
		ua.getArtifacts().add(target);
		return sa;
	}
	
	private SolutionArtifact processUnclaimedFormRule(EObject source, EObject target) {
		SolutionArtifact sa = SAUtil.createUnclassifiedSolutionArtifact();
		UnclassifiedArtifact ua = SaFactory.eINSTANCE.createUnclassifiedArtifact();
		sa.getComposedArtifacts().add(ua);
		sa.setPackage(SAUtil.createPackage(source));
		ua.getArtifacts().add(target);
		return sa;
	}
	
	private SolutionArtifact processOtherTargets(EObject target) {
		SolutionArtifact unclassifiedSA = SAUtil.createUnclassifiedSolutionArtifact();
		getTarget().add(unclassifiedSA);
		SAUtil.updateUnclassifiedSA(unclassifiedSA,target);
		if (target instanceof XSDSchema) {
			TransformationRule tr = TransformationUtil.getRuleForTarget(this,com.ibm.btools.te.ilm.heuristics.xsd.DataDefinitionRule.class,target);
			if (tr!=null) {
				unclassifiedSA.setPackage(SAUtil.createPackage((EObject)tr.getSource().get(0),false));
			}
		}
		return unclassifiedSA;
	}
	
	/**
	 * SCALArtifactUpdater is to process all the registered the SCDL artifacts created in the export session
	 * and perform the following updates if necessary:
	 * 
	 * - If a SCDL Component  contains a reference with multiple wires, this is the case where the component
	 * has a callback to more than one components.  We need to duplicate the component for each of the wire
	 * to ensure there is only one wire per reference.  This fixing has to be done here because this is the end of
	 * the export session which is the only point we know how many wires are created for this component reference
	 * 
	 */
	private class SCDLArtifactUpdater {
		private HashMap compRegistry = new HashMap();
		private HashMap saRegistry = new HashMap();
		private HashMap exportRegistry = new HashMap();
		private HashMap importRegistry = new HashMap();

		/**
		 * Register the component to be processed later when 'updateWire' method is called
		 * @param component
		 */
		public void registerSCDLArtifact(SolutionArtifact sa) {
			List cas = ((SolutionArtifact)sa).getComposedArtifacts();
			for (Iterator it2=cas.iterator();it2.hasNext();) {
				Object ca = it2.next();
				if (ca instanceof ScdlArtifact) {
					if(((ScdlArtifact)ca).getArtifact() instanceof DocumentRoot){
						DocumentRoot dr = (DocumentRoot)((ScdlArtifact)ca).getArtifact();
						if (ScdlUtil.isComponent(dr)) {
							Component component = dr.getComponent();
							compRegistry.put(component.getName(),component);
							saRegistry.put(component,sa);
						} else if (ScdlUtil.isExport(dr)) {
							Export export = dr.getExport();
							exportRegistry.put(export.getTargetName(),export);
						//feature 118131
						}else if(ScdlUtil.isImport(dr)){
							Import scdlImport = dr.getImport();
							importRegistry.put(scdlImport.getName(),scdlImport);
						}
					}
				}
			}
		}

		/**
		 * Update the SCDL artifacts and return a list of new SCDL artifact SAs created as a result of update
		 * @return
		 */
		public List update() {
			return updateComponentWire();
		}
		
		/**
		 * Go through all the components registered and update the Reference that has more than one wires
		 * @return a list of new components created as a result of updating the wires
		 */
		protected List updateComponentWire() {
			List newCmps = new ArrayList();
			java.util.Collection entries = compRegistry.values();
			Component cmp,newCmp;
			for (Iterator it=entries.iterator();it.hasNext();) {
				cmp = (Component)it.next();
				newCmps.addAll(updateComponentWire(cmp));
			}
			return newCmps;
		}
		
		/**
		 * Create a copy of the component cmp and update the reference/wire to wire to the new copied component
		 * @param cmp
		 * @return
		 */
		protected List updateComponentWire(Component cmp) {
			List newSCDLSA = new ArrayList();
			java.util.Collection entries = compRegistry.values();
			Component newCmp;
			List refs = cmp.getReferences();
			for (Iterator it2=refs.iterator();it2.hasNext();) {
				Reference ref = (Reference)it2.next();
				if (ref.getWires().size()>1) {
					List wires = ref.getWires();
					Wire wire;
					for (int i=wires.size()-1; i>0;i--) {
						wire = (Wire)wires.get(i);
						wires.remove(i);
						newCmp=(Component) ScdlUtil.copyComponent(cmp);
						//copiedCompRegistry.put(newCmp,cmp);
						replaceWire(newCmp,wire,ref);
						String target = wire.getTargetName();
						Component wiredCmp = (Component)compRegistry.get(target);
						if(wiredCmp == null){
							continue;
						}
						// rename new Component
						newCmp.setName(ComponentNameProvider.updateName(cmp.getName(),NamingUtil.getKeyFromString(cmp.getName()+target),true));
						// need to rewire the component
						rewireComponent(wiredCmp,cmp,newCmp);
						newSCDLSA.add(createSCDLSA(cmp,newCmp));
						// need to clone the Export for the new component
						Export export = (Export)exportRegistry.get(cmp.getName());
						if (export != null) {
							Export newExport = ScdlUtil.copyExport(export);
							newExport.setTargetName(newCmp.getName());
							newExport.setName(ComponentNameProvider.updateName(export.getName(),NamingUtil.getKeyFromString(export.getName()+newExport.getTargetName()),true));
							newSCDLSA.add(createSCDLSA(cmp,newExport));
						}
					}
				}
			}
			return newSCDLSA;			
		}
		
		private SolutionArtifact createSCDLSA(Component origCmp, Object obj) {
			DocumentRoot dr = SCDLFactory.eINSTANCE.createDocumentRoot();
			if (obj instanceof Component) {
				dr.setComponent((Component)obj);
			} else if (obj instanceof Export) {
				dr.setExport((Export)obj);
			}
			ScdlArtifact scdlArtifact = SaFactory.eINSTANCE.createScdlArtifact();
			scdlArtifact.setArtifact(dr);
			SolutionArtifact sa = SaFactory.eINSTANCE.createSolutionArtifact();
			sa.setType(SAType.SCDL_LITERAL);
			sa.getComposedArtifacts().add(scdlArtifact);
			SolutionArtifact origSA = (SolutionArtifact)saRegistry.get(origCmp);
			sa.setPackage(origSA.getPackage());
			return sa;
		}
		
		private void rewireComponent(Component srcComponent, Component oldTarComponent, Component newTarComponent) {
			List refs=srcComponent.getReferences();
			for (Iterator it=refs.iterator();it.hasNext();) {
				Reference ref = (Reference)it.next();
				List wires = ref.getWires();
				if (!wires.isEmpty()) {
					Wire wire = (Wire)wires.get(0); // should be safe now to assume only one wire exists per reference
					if (wire.getTargetName() == oldTarComponent.getName()) {
						wire.setTargetName(newTarComponent.getName());
					}
				}
			}
		}
		
		private void replaceWire(Component cmp, Wire origWire, Reference origRef) {
			List refs = cmp.getReferences();
			for (Iterator it=refs.iterator();it.hasNext();) {
				Reference ref = (Reference)it.next();
				if (origRef.getName().equals(ref.getName())) {
					ref.getWires().clear();
					ref.getWires().add(origWire);
				}
			}
			
		}
	}	
} //HeuristicsTransformationImpl
