/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import com.ibm.btools.bom.model.artifacts.Classifier;
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.LiteralString;
import com.ibm.btools.bom.model.artifacts.Slot;
import com.ibm.btools.bom.model.resources.IndividualResource;
import com.ibm.btools.bom.model.resources.Qualification;
import com.ibm.btools.bom.model.resources.Role;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;

import com.ibm.btools.te.framework.impl.TransformationRuleImpl;
import com.ibm.btools.te.framework.util.TransformationUtil;

import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.HeuristicsPackage;
import com.ibm.btools.te.ilm.heuristics.ResourceRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.PersonRule;
import com.ibm.btools.te.ilm.heuristics.fdl.RoleRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.heuristics.helper.StaffUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TESettingUtil;
import com.ibm.btools.te.ilm.heuristics.helper.VMMGroupEntry;
import com.ibm.btools.te.ilm.heuristics.helper.VMMPersonEntry;
import com.ibm.btools.te.ilm.heuristics.helper.StaffUtil.StaffConstants;
import com.ibm.btools.te.ilm.heuristics.helper.StaffUtil.VMMConstants;
import com.ibm.btools.fdl.model.Person;
import com.ibm.btools.te.ilm.model.sa.SAType;
import com.ibm.btools.te.ilm.model.sa.SaFactory;
import com.ibm.btools.te.ilm.model.sa.SolutionArtifact;
import com.ibm.btools.te.ilm.model.sa.UnclassifiedArtifact;
import com.ibm.btools.te.ilm.model.sa.UnclassifiedArtifactType;
import com.ibm.btools.te.ilm.resource.MessageKeys;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.EList;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;

import org.eclipse.emf.ecore.util.InternalEList;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Resource Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class ResourceRuleImpl extends TransformationRuleImpl implements ResourceRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ResourceRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return HeuristicsPackage.Literals.RESOURCE_RULE;
	}

	/**
	 * SOURCE: BOM IndividualResource or BOM Role
	 * TARGET: FDL Person or FDL Role
	 * Maps BOM Resource Staff to a FDL Person or maps a BOM Role to FDL Role
	 * @return boolean
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target()
	{	
		LoggingUtil.traceEntry(this, "transformSource2Target");
		TransformationContext context = getContext();
		if (getSource().get(0) instanceof IndividualResource){
			IndividualResource resource = (IndividualResource)getSource().get(0);
			if (TESettingUtil.isResourceTargetFDL(context,resource)){
				handleFDLPersonTransformation(resource);	
			}else{
				handleBPELPersonTransformation(resource);
			}
		} else if (getSource().get(0) instanceof Role){
			Role resource = (Role)getSource().get(0);
			if (TESettingUtil.isResourceTargetFDL(context,resource)){
				handleFDLRoleTransformation(resource);	
			}else{
				handleBPELRoleTransformation(resource);
			}	
		}
		
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}
	
	/**
	 * Verify that role name is provided.
	 * Make call to store VMM group into context.
	 */
	private void handleBPELRoleTransformation(Role resource) {
		String groupName = resource.getName();	
		if(groupName == null){
			String resourceName = resource.getName();
			VMMGroupEntry VMMGroup = new VMMGroupEntry();		
			VMMGroup.setVMMMissingMandatoryAttribute(resourceName);
			StaffUtil.registerVMMGroupEntityForOrg(getContext(), VMMGroup);
			//LoggingUtil.logWarning(MessageKeys.VMM_GROUP_MISSING_MANDATORY_ATTRIBUTES, param, null);
		}
		VMMGroupEntry VMMGroup = new VMMGroupEntry();
		//defect 73912
		if(resource.getOwnedComment() != null && !resource.getOwnedComment().isEmpty()){
			Comment description = (Comment)resource.getOwnedComment().get(0);
			String orgDescription = description.getBody();
			VMMGroup.setVMMDescription(orgDescription);
		}
		
		VMMGroup.setVMMGroupName(groupName);
		StaffUtil.registerVMMGroupEntityForRole(getContext(), VMMGroup);
	}

	/**
	 * Verify that personID of person is provided.
	 * If no lastName or firstName is provided, substitute personID.
	 * Make call to store VMM person into context.
	 */
	private void handleBPELPersonTransformation(IndividualResource resource) {
		boolean isStaff = StaffUtil.isKindOfStaff((Classifier)resource.getClassifier().get(0));
		if(isStaff){
			String personID = null;
			String lastName = null;
			String firstName = null;
			String email = null;
			List slots = resource.getSlot();		
			if(slots != null && !slots.isEmpty()){
				Iterator it = slots.iterator();
				while(it.hasNext()){
					Slot thisSlot = (Slot)it.next();	
					if (thisSlot.getDefiningFeature().getName()!=null){					
						if(thisSlot.getDefiningFeature().getName().equals("personId")){
							if(thisSlot.getValue() != null && !thisSlot.getValue().isEmpty()){
								personID =((LiteralString)thisSlot.getValue().get(0)).getValue();
							}
						}
						else if(thisSlot.getDefiningFeature().getName().equals("lastName")){
							if(thisSlot.getValue() != null && !thisSlot.getValue().isEmpty()){
								lastName =((LiteralString)thisSlot.getValue().get(0)).getValue();
							}
						}
						else if(thisSlot.getDefiningFeature().getName().equals("firstName")){
							if(thisSlot.getValue() != null && !thisSlot.getValue().isEmpty()){
								firstName =((LiteralString)thisSlot.getValue().get(0)).getValue();
							}
						}
						else if(thisSlot.getDefiningFeature().getName().equals("email")){
							if(thisSlot.getValue() != null && !thisSlot.getValue().isEmpty()){
								email =((LiteralString)thisSlot.getValue().get(0)).getValue();
							}
						}
						
					}			
				}
			}
			VMMPersonEntry VMMPerson = new VMMPersonEntry();
			if(personID != null && !personID.equals("")){
				if(lastName == null || lastName.equals("")){
					String resourceName = resource.getName();
					//LoggingUtil.logWarning(MessageKeys.VMM_PERSON_ACCOUNT_MISSING_MANDATORY_ATTRIBUTES_DEFAULT_SET, param, null);
					lastName = personID;
					VMMPerson.setVMMLastNameToDefault(resourceName);
				}
				if(firstName == null || firstName.equals("")){
					String resourceName = resource.getName();
					//LoggingUtil.logWarning(MessageKeys.VMM_PERSON_ACCOUNT_MISSING_MANDATORY_ATTRIBUTES_DEFAULT_SET, param, null);
					firstName = personID;
					VMMPerson.setVMMFirstNameToDefault(resourceName);
				}
			} else {
				//The "personID" in "resource" is mandatory. "personID" is not set.
				String resourceName = resource.getName();
				VMMPerson.setVMMMissingMandatoryAttribute(resourceName);
				StaffUtil.registerVMMPersonEntity(getContext(), VMMConstants.VMM_PERSON, VMMPerson);
				//LoggingUtil.logWarning(MessageKeys.VMM_PERSON_ACCOUNT_MISSING_MANDATORY_ATTRIBUTES, param, null);
				return;
			}

			//add to Context.. 
			VMMPerson.setVMMPersonID(personID);
			VMMPerson.setVMMlastName(lastName);
			VMMPerson.setVMMfirstName(firstName);
			if (!(email == null) && !(email.equals(""))){
				VMMPerson.setVMMemail(email);
			}
		
			//TODO 
			List qualifications = resource.getQualification();
			if(qualifications != null && !qualifications.isEmpty()){
				Iterator it = qualifications.iterator();
				while(it.hasNext()){
					Qualification qualification = (Qualification)it.next();
					Role role = qualification.getRole();
					if(findResourceRole(role)){
						List<String> roles = new ArrayList();
						if (VMMPerson.getVMMroles()!=null) {
							roles = VMMPerson.getVMMroles();
						}
						roles.add(role.getName());
						VMMPerson.setVMMroles(roles);
					}
				}
				
			}
			StaffUtil.registerVMMPersonEntity(getContext(), VMMConstants.VMM_PERSON, VMMPerson);
			
		}
		
	}

	/**
	 * Create SA for the Individual Resource
	 * @param resource - IndividualResource
	 * @return
	 */	
	private void handleFDLPersonTransformation(IndividualResource resource) {
		if (resource!=null && !findPerson(resource) && StaffUtil.isPredefinedStaff(resource.getClassifier())){			
			TransformationContext context = getContext();
			PersonRule rule = FdlFactory.eINSTANCE.createPersonRule();		
			getChildRules().add(rule);
			rule.getSource().add(resource);
			setComplete(rule.transformSource2Target());
			if (isComplete()) {
				EList targets = rule.getTarget();
				Iterator it = targets.iterator();
				EObject obj;
				SolutionArtifact sa;
				UnclassifiedArtifact staffArtifact;
				while (it.hasNext()) {
					obj = (EObject)it.next();
					sa = SaFactory.eINSTANCE.createSolutionArtifact();
					sa.setType(SAType.UNCLASSIFIED_LITERAL);
					sa.setName(((Person)obj).getUserID());	
					staffArtifact = SaFactory.eINSTANCE.createUnclassifiedArtifact();
					staffArtifact.setType(UnclassifiedArtifactType.WORKFLOW_DEFINITION_LITERAL);
					staffArtifact.getArtifacts().add(obj);
					sa.getComposedArtifacts().add(staffArtifact);
					getTarget().add(sa);	
				}
			}
		}
	}
	
	private boolean findPerson (IndividualResource resource){
		TransformationRule rule = TransformationUtil.getRuleForSource(getRoot(), PersonRule.class, resource);
		if (rule==null)
			return false;
		else
			return true;
	}
	
	/**
	 * Check if person resource is being exported in the current session
	 */
	private boolean findResourcePerson (IndividualResource resource){
		TransformationRule rule = TransformationUtil.getRuleForSource(getRoot(), ResourceRule.class, resource);
		if (rule==null)
			return false;
		else
			return true;
	}
	
	/**
	 * Check if role is being exported in the current session
	 */
	private boolean findRole (Role role){
		TransformationRule rule = TransformationUtil.getRuleForSource(getRoot(), RoleRule.class, role);
		if (rule==null)
			return false;
		else
			return true;
	}
	
	/**
	 * Check if role resource is being exported in the current session
	 */
	private boolean findResourceRole (Role role){
		TransformationRule rule = TransformationUtil.getRuleForSource(getRoot(), ResourceRule.class, role);
		if (rule==null)
			return false;
		else
			return true;
	}
	
	/**
	 * Create SA for a Role
	 * @param resource - Role
	 * @return
	 */	
	private void handleFDLRoleTransformation(Role resource) {
		if (resource!=null && !findRole(resource)){
			TransformationContext context = getContext();
			RoleRule rule = FdlFactory.eINSTANCE.createRoleRule();
			getChildRules().add(rule);
			rule.getSource().add(resource);
			setComplete(rule.transformSource2Target());
			if (isComplete()) {
				EList targets = rule.getTarget();
				Iterator it = targets.iterator();
				EObject obj;
				SolutionArtifact sa;
				UnclassifiedArtifact staffArtifact;
				while (it.hasNext()) {
					obj = (EObject)it.next();
					sa = SaFactory.eINSTANCE.createSolutionArtifact();
					sa.setType(SAType.UNCLASSIFIED_LITERAL);
					sa.setName(((com.ibm.btools.fdl.model.Role)obj).getName());	
					staffArtifact = SaFactory.eINSTANCE.createUnclassifiedArtifact();
					staffArtifact.setType(UnclassifiedArtifactType.WORKFLOW_DEFINITION_LITERAL);
					staffArtifact.getArtifacts().add(obj);
					sa.getComposedArtifacts().add(staffArtifact);
					getTarget().add(sa);	
				}
			}
		}
	}

} //ResourceRuleImpl
