package com.ibm.btools.te.ilm.heuristics.naming;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.Fault;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.Part;
import org.eclipse.wst.wsdl.PortType;
import org.eclipse.xsd.XSDElementDeclaration;

import com.ibm.btools.blm.ie.exprt.IExportConstants;
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.services.Behavior;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.wsdl.util.OriginalWsdlUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.util.exception.BTRuntimeException;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Flow;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.bpelpp.Parameter;
import com.ibm.wbit.model.utils.wsdl.WSDLUtils;

/**
* Provider of BPEL Elements names.
*/

public class BPELNameProvider extends NameProvider {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private static String VARIABLE_NAME_SUFFIX = "Variable";
	public BPELNameProvider(){
		nameValidator = new NameValidator();
	}
	/**
	 * Generates an UNIQUE, VALID value for the <code>propertyAttribute</code> of the <code>type</code>
	 * @param forType  a BPEL or WSDL Object
	 * @param propertyName name of the attribute property
	 * @param fromSource  BOM source object
	 * @param registry a NamingRegistry  
	 * @return String default name.
	 */
	public String getDefaultName(Object forType, String propertyName, NamedElement fromSource, NamingRegistry registry, Object parentContext) {
		if(propertyName == null){
			return getDefaultBPELElementName(forType, fromSource, registry);
		}else if(forType instanceof Process){
			return getDefaultBPELElementName(forType, fromSource, registry);
		}else if(forType instanceof Activity){
			return getDefaultActivityName(forType, fromSource, registry);
		}else if(forType instanceof BPELVariable){
			return getDefaultVariableName(forType, fromSource, registry);
		}
		else{
			return getDefaultBPELElementName(forType, fromSource, registry);
		}
	}

	/**
	 * Returns a BPEL ElementName (unique in the registry) based on the <code>fromSource</code> name. If the registry is not set
	 * will create the name based on the source Name, not checking for uniqueness, else, the generated name will be 
	 * UNIQUE withing the given registry.
	 * @param forType a WSDL Element Type
	 * @param fromSource a source BOM NamedElement, source of the Name
	 * @param registry a Registry for identical type name that insures that the name is unique. 
	 * @return String the default BPEL Element name
	 */
	private String getDefaultBPELElementName(Object forType, NamedElement fromSource, NamingRegistry registry){
		String convertedName = null;
		//defect 90062/90011 Resolve message part name to corresponding BPEL Parameter Name
		if(forType instanceof Parameter && fromSource instanceof ObjectPin){			
			ObjectPin pin = (ObjectPin)fromSource;
			if(pin.eContainer() instanceof CallBehaviorAction){
				CallBehaviorAction cba = (CallBehaviorAction)pin.eContainer();
				if(BomUtils.isServiceOperation(cba)){
					if(cba.getBehavior() instanceof com.ibm.btools.bom.model.processes.activities.Activity){
						convertedName = lookupParamterNameForPin(pin, cba);
						return convertedName;
					}					
				}else{
					//defect 92475. synchronize the pin on the CBA with the Pin on the Action 
					NamedElement toBeCalledAction = findToBeProcessedAction(cba);
					if(toBeCalledAction instanceof Action){
						if(pin instanceof InputObjectPin){
							InputPinSet pinSet =(InputPinSet)BomUtils.getInputPinSets(pin).get(0);
							int pinSetIndex = ((CallBehaviorAction)pin.eContainer()).getInputPinSet().indexOf(pinSet);
							List pins = BomUtils.getObjectPinsForSet(pinSet); 
							int pinIndex  = pins.indexOf(pin);
							pin =(InputObjectPin)((InputPinSet)((Action)toBeCalledAction).getInputPinSet().get(pinSetIndex)).getInputObjectPin().get(pinIndex);
							fromSource = pin;
						}else if(pin instanceof OutputObjectPin){
							OutputPinSet pinSet =(OutputPinSet)BomUtils.getOutputPinSets(pin).get(0);
							int pinSetIndex = ((CallBehaviorAction)pin.eContainer()).getOutputPinSet().indexOf(pinSet);
							List pins = BomUtils.getObjectPinsForSet(pinSet); 
							int pinIndex  = pins.indexOf(pin);
							pin =(OutputObjectPin)((OutputPinSet)((Action)toBeCalledAction).getOutputPinSet().get(pinSetIndex)).getOutputObjectPin().get(pinIndex);
							fromSource = pin;
						}
					}
				}
			}
		}
		if(registry == null){
			convertedName = getName(fromSource);
		}else{
			//defect 140558
			String sourceName = null;
			if(forType instanceof Flow){
				sourceName = fromSource.getName() + "_Flow";
			}else{
				sourceName = fromSource.getName();
			}
			JavaNCNameConverter converter = new JavaNCNameConverter();
			//convertedName = converter.convertName(registry, forType,  fromSource.getName());
			convertedName = converter.convertName(registry, forType,  sourceName);
		}
		return convertedName;
	}
	
	/**
	 * This method is responsible for retrieving the parameter names corresponding to the input/output of the
	 * CallBehaviorAction. 
	 * @param pin Input/Output ObjectPin represents the input/output of the BPEL activity
	 * @param cba Business service used by the process as CallBehaviorAction
	 * @return String The parameter name retrieved from the original WSDL definition
	 */
	private String lookupParamterNameForPin(ObjectPin pin, CallBehaviorAction cba) {
		String targetPartName = null;
		try{
			StructuredActivityNode san = (StructuredActivityNode)((com.ibm.btools.bom.model.processes.activities.Activity)cba.getBehavior()).getImplementation();
			PinSet ps = BomUtils.findPinSetForPin(pin);
			Definition originalWSDL = OriginalWsdlUtil.getOriginalWsdl(context,BomUtils.resolveFileAttachment(san));
			if(originalWSDL != null){
				String messagename = ps.getName();
				if(ps instanceof OutputPinSet && ((OutputPinSet)ps).getIsException()){																	
						//find the operation and the fault name
						PortType portType = OriginalWsdlUtil.getPortType(getContext(),BomUtils.resolveOwningInterface(san));
						String faultName = null;
						if(portType != null){
							Operation originalOperation = OriginalWsdlUtil.getOperation(portType,san.getName());
							EList faults = originalOperation.getEFaults();											
							if(faults != null && !faults.isEmpty()){
								for(int i=0; i<faults.size();i++){
									Fault eachFault = (Fault)faults.get(i);
									String eachFaultName = eachFault.getName();
									if(eachFaultName != null){
										if(messagename.startsWith(eachFaultName)){
											messagename = messagename.substring(eachFaultName.length()+1);
											break;
										}
									}
								}
							}
					}
				}
				String partname = pin.getName();				
				Message targetMsg = OriginalWsdlUtil.getMessage(originalWSDL,OriginalWsdlUtil.getTargetNamespaceForContainingService(originalWSDL,pin),messagename, context);
				if(partname != null && partname.indexOf(messagename)!= -1 && partname.startsWith(messagename)){
					//likely the pin name is a combination of message and part. but the part name itself may started with message name
					if(targetMsg != null && targetMsg.getPart(partname) != null ){
						targetPartName = targetMsg.getPart(partname).getName();
					}else{
						//This is a compound name, we need to remove the message prefix and retry. This is our internal pin name handling
						String realPartName = partname.substring(messagename.length()+1);
						if(targetMsg != null && targetMsg.getPart(realPartName) != null){
							targetPartName = targetMsg.getPart(realPartName).getName();
						}
					}
				}else{								
					if(targetMsg != null && targetMsg.getPart(partname) != null){
						targetPartName = targetMsg.getPart(partname).getName();										
					}
				}
				if(targetPartName == null){
					boolean isDocLitWrappedStype = false;
					PortType portType = OriginalWsdlUtil.getPortType(context,BomUtils.resolveOwningInterface(san));						
					if(portType != null){
						Operation operation = OriginalWsdlUtil.getOperation(portType,san.getName());
						if(operation != null){
							WSDLUtils.Compliance isDocLitWrappedCompliance = WSDLUtils.isDocLitWrapped(operation);
							if(isDocLitWrappedCompliance == WSDLUtils.YES){
								isDocLitWrappedStype = true;								
							}	
						}
					}	
					if(isDocLitWrappedStype){
						Part singlePart = (Part)targetMsg.getEParts().get(0);
						List wrappedDeclarations = WSDLUtils.getUnwrappedElementDeclarations(singlePart);
						if(wrappedDeclarations != null && !wrappedDeclarations.isEmpty()){
							//for each declaration we should create a Pin set the name as per the Elenent name
							for(int i=0; i<wrappedDeclarations.size(); i++){
								XSDElementDeclaration declaration = (XSDElementDeclaration)wrappedDeclarations.get(i);
								if(declaration.isElementDeclarationReference()){
									declaration = declaration.getResolvedElementDeclaration();
								}
								if(declaration.getName() != null && declaration.getName().equals(partname)){
									targetPartName = partname;
									break;
								}
							}
						}else{
							//This is a fault message
							XSDElementDeclaration declaration = singlePart.getElementDeclaration();
							if(declaration.isElementDeclarationReference()){
								declaration = declaration.getResolvedElementDeclaration();
							}
							if(declaration.getName() != null && declaration.getName().equals(partname)){
								targetPartName = partname;											
							}else{
								//handle name appended with "_n", this is a lenient check, last resort
								if(partname.startsWith(declaration.getName())){
									targetPartName = declaration.getName();
								}
							}
						}												
					}else{
						//non-doc lit wrapper, handle name appended with "_n", this is a lenient check, last resort
						if(partname.startsWith(messagename)){
							String realPartName = partname.substring(messagename.length()+1);
							EList eParts = targetMsg.getEParts();
							if(eParts != null && !eParts.isEmpty()){
								for(int k=0;k<eParts.size();k++){
									Part eachPart = (Part)eParts.get(k);
									String eachPartName = eachPart.getName();
									if(eachPartName != null && realPartName.startsWith(eachPartName)){
										targetPartName = eachPartName;
										break;
									}
								}
							}
						}						
					}
				    if(targetPartName == null){
						throw new RuntimeException("Cannot find appropriate name for the parameter");
				    }
				}				
			}	
		}catch(Exception e){
			String params[]={pin.getName()};
			LoggingUtil.logError(MessageKeys.ERROR_RETRIEVING_PARAMETER_NAME, params, e);
			BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_RETRIEVING_PARAMETER_NAME));
			runtimeEx.setChainedException(e);			
			throw runtimeEx;
		}
		return targetPartName;
	}
	
	/**
	 * Returns an BPEL VariableName (unique in the registry) based on the <code>fromSource</code> name. If the registry is not set
	 * will create the name based on the source Name, not checking for uniqueness, else, the generated name will be 
	 * UNIQUE withing the given registry.
	 * @param forType a WSDL Element Type
	 * @param fromSource a source BOM NamedElement, source of the Name
	 * @param registry a Registry for identical type name that insures that the name is unique. 
	 * @return String the default BPEL Variable name.
	 */
	private String getDefaultVariableName(Object forType, NamedElement fromSource, NamingRegistry registry){
		String convertedName = null;
		if(registry == null){
			convertedName = getName(fromSource);
		}else{
			JavaNCNameConverter converter = new JavaNCNameConverter();
			//d140822
			String name = null;
			if (((BPELVariable)forType).getType() != null) {
				name = ((BPELVariable)forType).getType().getName();
			} else if (((BPELVariable)forType).getXSDElement() != null) {
				name = ((BPELVariable)forType).getXSDElement().getName();
			}
		    convertedName = converter.convertName(registry, forType,  name + VARIABLE_NAME_SUFFIX);
		}
		return convertedName;
		
	}


	/**
	 * Returns an UNIQUE Name for the BPEL construct (in the registry) based on the <code>fromSource</code> name.
	 * @param forType a BPEL Element Type
	 * @param fromSource a source BOM NamedElement, source of the Name
	 * @param registry a Registry for identical type name that insures that the name is unique. 
	 * @return String the default BPEL Activity name.
	 */
	private String getDefaultActivityName(Object forType, NamedElement fromSource, NamingRegistry registry){
		JavaNCNameConverter converter = new JavaNCNameConverter();
		String convertedName = null;
		if(fromSource instanceof PinSet){
			convertedName = converter.convertName(registry, forType,  ((NamedElement)fromSource.eContainer()).getName() + "_" + fromSource.getName());	
		}else {
			convertedName = converter.convertName(registry, forType,  fromSource.getName());
		}
	    
		return convertedName;
		
	}
	
	/**
	 * Returns the Activity referecned by the CallBehaviorAction, or itself, if <code>action</code>
	 * is not a CallBehaviorAction.
	 * @param action - a BOM Action if Activity
	 * @return - namedElement
	 */
	protected NamedElement findToBeProcessedAction(NamedElement action){
		
		if (action instanceof CallBehaviorAction){
		
			//Action san=action;
			NamedElement san = action;
			Behavior behavior = ((CallBehaviorAction)action).getBehavior();
		
			if (behavior instanceof com.ibm.btools.bom.model.processes.activities.Activity){
				// Set source (SAN) to ActionRule and execute it
				//HeuristicsTransformation root = (HeuristicsTransformation) getRoot();
				// Check if the referenced Process is a first class artifact (i.e. it is a participant of the solution)
				//if (root.isFirstClassArtifact(behavior))
					san = ((com.ibm.btools.bom.model.processes.activities.Activity)behavior).getImplementation();
			}
			return san;
		}else{
			return action;
		}
			
	}	

}
