package com.ibm.btools.te.ilm.heuristics.naming;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.te.ilm.CrossProjectReferenceUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.wsspi.sca.scdl.Component;
import com.ibm.wsspi.sca.scdl.Import;

public class ComponentNameProvider extends NameProvider {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	public static final String COMPONENT_NAME_PREFIX="Component";
	public static final String IMPORT_NAME_PREFIX="ImportedInterface";
	public static final String EXPORT_NAME_PREFIX="PublicInterface";
	
	public static final String NAME_SEPARATOR="_";

	/**
	 * Constructor for the class that instantiates the name validator.
	 *
	 */
	public ComponentNameProvider() {
		nameValidator = new NameValidator();
	}
	
	/**
	 * Returns the default name of a Component that is unique within the given registry and in the context of the parent object.
	 */
	public String getDefaultName(Object forType, String propertyName, NamedElement fromSource, NamingRegistry registry, Object parentContext) {
		return getUniqueName(forType,fromSource);		
	}

	protected String getUniqueName(Object forType,NamedElement fromSource) {
		String fullName = BomUtils.getFullName(fromSource);
//		String prefix=COMPONENT_NAME_PREFIX;
//		// TODO - check if forType is a component, import, or export
//		if (forType instanceof Import) {
//			prefix = IMPORT_NAME_PREFIX;
//		}
//		return prefix+getKeyFromString(fullName);
		
		if (fromSource instanceof Action) {
			Action action = (Action)fromSource;
			String convertedName = null;
			if (forType instanceof Component)	 {
				convertedName = NamingUtil.convertNameForAction(action, getContext());
			} else if (forType instanceof Import) {
				String registeredComponentName = null;
				
				Action referencedAction = null;
				if (action instanceof CallBehaviorAction &&
						((CallBehaviorAction)action).getBehavior() instanceof Activity) {
					if(BomUtils.isReusableBusinessRuleTask((CallBehaviorAction)action)||BomUtils.isReusableHumanTask((CallBehaviorAction)action)||BomUtils.isProcess((CallBehaviorAction)action)){
						referencedAction = ((Activity)((CallBehaviorAction)action).getBehavior()).getImplementation();
					}else{
						referencedAction = action;
					}
				} else {
					referencedAction = action;
				}
				NoHashNamingRegistry noHashReg = null;
				if (NamingUtil.USE_PER_PROJECT_UNIQUENESS) {
					String projectName = CrossProjectReferenceUtil.getProjectNameForBOMElement(referencedAction);
					noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(context, NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(projectName);
				} else {
					noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(context, NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(NamingUtil.GLOBAL_XPROJECT_UNIQUENESS);
				}
				Object contextContainer = NamingUtil.getContextContainer(referencedAction);
				
				String path = null;
				if (contextContainer instanceof StructuredActivityNode &&
						noHashReg.actionExistInLocalRegistry(referencedAction, contextContainer)) {
					path = noHashReg.retrieveNameFromLocalRegistry(referencedAction, contextContainer);
					registeredComponentName = NamingUtil.getNameForArtifact(path);
				} else if(noHashReg.actionExistInGlobalRegistry(referencedAction)) {
					path = noHashReg.retrieveNameFromGlobalRegistry(referencedAction);
					registeredComponentName = NamingUtil.getNameForArtifact(path);
				}
				if (registeredComponentName != null) {
					convertedName = NamingUtil.convertNameForImport(action, registeredComponentName, getContext());
				} else {
					convertedName = NamingUtil.convertNameForImport(action, getContext());
				}
			}
					
			return convertedName;
		}		
		
		JavaNCNameConverter converter = new JavaNCNameConverter();
		return converter.convertName(composeName(fromSource.getName(),NamingUtil.getKeyFromString(fullName)));
	}

	public static String composeName(String name,String key) {
		return name+NAME_SEPARATOR+key;
	}
	
	public static String updateName(String origName,String newkey,boolean mustUseNewKey) {
		String origKey=NamingUtil.getKeyFromString(origName);
		int origKeyPos = origName.lastIndexOf(NAME_SEPARATOR+origKey);
		if (origKeyPos != -1) {
			return composeName(origName.substring(0,origKeyPos),newkey);
		} else {
			if (mustUseNewKey) {
				return composeName(origName,newkey);
			} else {
				return origName;				
			}
		}
		//return composeName(origName.substring(0,origName.lastIndexOf(NAME_SEPARATOR)),newkey);
	}
}
