package com.ibm.btools.te.ilm.heuristics.naming;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.Part;
import org.eclipse.wst.wsdl.PortType;
import org.eclipse.wst.wsdl.WSDLElement;
import org.eclipse.xsd.XSDComponent;
import org.eclipse.xsd.XSDElementDeclaration;

import com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.bpelpp.Parameter;
import com.ibm.wbit.br.core.model.Rule;
import com.ibm.wbit.br.core.model.RuleSet;
import com.ibm.wbit.br.core.model.RuleTemplate;
import com.ibm.wbit.br.core.model.Variable;
import com.ibm.wbit.tel.TEmail;
import com.ibm.wbit.tel.TEscalation;
import com.ibm.wbit.tel.TTask;
import com.ibm.wsspi.sca.scdl.Component;
import com.ibm.wsspi.sca.scdl.Import;

/**
 * Factory class responsible for retrieving a specific NameProvider based on the input Object type.
 */
public class NameProviderFactory {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2008, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	
	private static final boolean TEmail = false;

	/**
	 * Returns a NameProvider based on the type of input Object.
	 * @param forType an Object
	 * @return
	 */
	public static NameProvider getNameProvider(Object forType){
		
		if(forType instanceof Class){
			return getNameProvider((Class)forType);
		}
		NameProvider nameProvider = null;
		if(forType instanceof Activity || forType instanceof Process || forType instanceof BPELVariable || forType instanceof Parameter){
			nameProvider = new BPELNameProvider();
		}else if(forType instanceof XSDComponent){
			nameProvider = new XSDNameProvider();
		}else if(forType instanceof WSDLElement){
			nameProvider = new WSDLNameProvider();
		}else if (forType instanceof Component || forType instanceof Import) {
			nameProvider = new ComponentNameProvider();
		} else if (forType instanceof BusinessRuleGroup || forType instanceof RuleSet ||
				forType instanceof RuleTemplate || forType instanceof Rule || forType instanceof Variable) {
			nameProvider = new BusinessRuleNameProvider();
		} else if (forType instanceof TTask || forType instanceof TEscalation || forType instanceof TEmail) {
			nameProvider = new TELNameProvider();
		}
		return nameProvider;
	}

	/**
	 * Returns a NameProvider based on the type of input Class.
	 * @param forType a Class
	 * @return
	 */
	public static NameProvider getNameProvider(Class forType){
		
		NameProvider nameProvider = null;
		if(forType.equals(Activity.class) || forType.equals(Process.class) || forType.equals(BPELVariable.class)){
			nameProvider = new BPELNameProvider();
		}else if(forType.equals(XSDElementDeclaration.class)){
			nameProvider = new XSDNameProvider();
		}else if(forType.equals(Message.class) || forType.equals(Part.class) || forType.equals(PortType.class) ||forType.equals(Operation.class)){
			nameProvider = new WSDLNameProvider();
		}else if(forType.equals(BusinessRuleGroup.class) || forType.equals(RuleSet.class) || forType.equals(RuleTemplate.class) ||
				 forType.equals(Rule.class) || forType.equals(Variable.class)){
			nameProvider = new BusinessRuleNameProvider();
		}else if (forType.equals(TTask.class) || forType.equals(TEscalation.class) || forType.equals(TEmail.class)) {
			nameProvider = new TELNameProvider();
		}
		return nameProvider;
	}
	
}
