package com.ibm.btools.te.ilm.heuristics.naming;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.Variable;
import com.ibm.btools.te.attributes.helper.TechnicalAttributesHelper;

/**
* Provider for XSD Elements names.
*/
public class XSDNameProvider extends NameProvider {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	/**
	 * Constructor for the class, instantiates the validator.
	 *
	 */
	public XSDNameProvider(){
		nameValidator = new NameValidator();
		
	}
	
	/**
	 * Returns the name of a XSD Element that is unique within the given registry. 
	 */
	public String getName(Object forType, NamedElement fromSource, NamingRegistry registry, String propertyName){
		if(fromSource instanceof Variable){
			return getDefaultName(forType, propertyName, fromSource, registry, parentContext);
					//feature 117494
		}else if (fromSource instanceof Pin){
			//this is the XSDElementDeclaration for Pin so either get the name from the TechnicalAttributes or from the Documentation
			String name = getNameFromTechnicalAttributes((Pin)fromSource, propertyName);
			if(name != null && !name.equals("")){
				return name;
			}
		}
		return super.getName(forType, fromSource, registry, propertyName);
	}

	/**
	 * Returns the default name of a XSD Element that is unique within the given registry and in the context of the parent object.
	 */
	public String getDefaultName(Object forType, String propertyName, NamedElement fromSource, NamingRegistry registry, Object parentContext) {
		String convertedName = null;
		if(registry == null){
			convertedName = getName(fromSource);
		}else{		
			JavaNCNameConverter converter = new JavaNCNameConverter();
			convertedName = converter.convertName(registry, forType, fromSource.getName(), parentContext);
		}
		return convertedName;
	}
	/**
	 * Returns the name of the XSDElementDeclaration attached from the TechnicalAttributes (WSDLParameter) attached to this Pin.
	 * @param fromSource
	 * @param propertyName
	 * @return
	 */
	private String getNameFromTechnicalAttributes(Pin fromSource, String propertyName){
					//feature 117494
		//This should not ask the name from the NameProvider which may try to update the value of the XSDElement (default value generation), 
		//rather should get the name directly from the TechnicalAttributesHelper - as the values are read only
		String name = null;
		String taName = (String)TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, propertyName);
		if(taName != null && !taName.equals("")){
			name = taName;
		}
		return name; 
	}

}
