/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 */
package com.ibm.btools.te.ilm.heuristics.scdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.xml.type.internal.QName;
import org.eclipse.emf.ecore.xml.type.util.XMLTypeUtil;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.ExtensibleElement;
import org.eclipse.wst.wsdl.Import;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.Part;
import org.eclipse.wst.wsdl.Port;
import org.eclipse.wst.wsdl.PortType;
import org.eclipse.wst.wsdl.Service;
import org.eclipse.wst.wsdl.WSDLFactory;
import org.eclipse.wst.wsdl.binding.soap.SOAPAddress;
import org.eclipse.xsd.XSDAttributeDeclaration;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDSchema;
import org.eclipse.xsd.XSDTypeDefinition;
import org.eclipse.xsd.util.XSDConstants;

import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.blm.ie.exprt.option.BpelOptions;
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleAction;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.attributes.helper.TechnicalAttributesHelper;
import com.ibm.btools.te.attributes.model.definition.implementation.wps.AdaptiveEntity;
import com.ibm.btools.te.attributes.model.definition.implementation.wps.BusinessRule;
import com.ibm.btools.te.attributes.model.definition.implementation.wps.DynamicAssembler;
import com.ibm.btools.te.attributes.model.definition.implementation.wps.EnterpriseAccess;
import com.ibm.btools.te.attributes.model.definition.implementation.wps.HumanTask;
import com.ibm.btools.te.attributes.model.definition.implementation.wps.JMS;
import com.ibm.btools.te.attributes.model.definition.implementation.wps.Java;
import com.ibm.btools.te.attributes.model.definition.implementation.wps.SCA;
import com.ibm.btools.te.attributes.model.definition.implementation.wps.WebServices;
import com.ibm.btools.te.attributes.model.definition.implementation.wps.impl.WebServicesImpl;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.impl.TransformationRuleImpl;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.CrossProjectReferenceUtil;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.ExportOperationConstants;
import com.ibm.btools.te.ilm.ExportOperationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.WIDProjectUtil;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.HeuristicsTransformation;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.CallBehaviorActionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.HumanTaskRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.util.AbstractbpelNamingRegistry;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelConstants;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.FabricIntegrationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.HumanTaskUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.helper.SAUtil;
import com.ibm.btools.te.ilm.heuristics.helper.StaffUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionKeyConstants;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NoHashNamingRegistry;
import com.ibm.btools.te.ilm.heuristics.naming.ProjectNamingRegistry;
import com.ibm.btools.te.ilm.heuristics.scdl.ComponentRule;
import com.ibm.btools.te.ilm.heuristics.scdl.ScdlPackage;
import com.ibm.btools.te.ilm.heuristics.scdl.util.JavaImplCreationTemplate;
import com.ibm.btools.te.ilm.heuristics.scdl.util.MediationFlowGenerator;
import com.ibm.btools.te.ilm.heuristics.scdl.util.ScdlUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.util.OriginalWsdlUtil;
import com.ibm.btools.te.ilm.heuristics.xsd.util.DataDefinitionUtil;
import com.ibm.btools.te.ilm.heuristics.xsd.util.OriginalXsdUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.util.exception.BTRuntimeException;
import com.ibm.wbiservers.brules.scdl.brgimpl.BrgimplFactory;
import com.ibm.wbiservers.brules.scdl.brgimpl.BusinessRuleGroupImplementation;
import com.ibm.wbiservers.businessrule.model.brg.BrgFactory;
import com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup;
import com.ibm.wbiservers.businessrule.model.brg.ReferenceGroup;
import com.ibm.wbiservers.businessrule.model.brgt.BrgtFactory;
import com.ibm.wbiservers.businessrule.model.brgt.BusinessRuleGroupTable;
import com.ibm.wbiservers.common.componentdef.CodeParameterDef;
import com.ibm.wbiservers.common.componentdef.ComponentDef;
import com.ibm.wbiservers.common.componentdef.ComponentdefFactory;
import com.ibm.wbiservers.common.componentdef.OperationDef;
import com.ibm.wbiservers.common.componentdef.WSDL;
import com.ibm.wbiservers.common.selectiontables.OperationSelectionTable;
import com.ibm.wbiservers.common.selectiontables.SelectionTables;
import com.ibm.wbiservers.common.selectiontables.SelectiontablesFactory;
import com.ibm.wbit.ae.sacl.CorrelationSet;
import com.ibm.wbit.ae.sacl.FinalState;
import com.ibm.wbit.ae.sacl.InitialState;
import com.ibm.wbit.ae.sacl.Reference;
import com.ibm.wbit.ae.sacl.SACLFactory;
import com.ibm.wbit.ae.sacl.SACLRoot;
import com.ibm.wbit.ae.sacl.State;
import com.ibm.wbit.ae.sacl.StateMachine;
import com.ibm.wbit.ae.sacl.StateMachineDefinition;
import com.ibm.wbit.ae.sacl.Transition;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.PartnerLink;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.bpel.Receive;
import com.ibm.wbit.bpel.Reply;
import com.ibm.wbit.bpel.Sequence;
import com.ibm.wbit.bpel.services.partnerlinktype.PartnerLinkType;
import com.ibm.wbit.bpel.services.partnerlinktype.PartnerlinktypeFactory;
import com.ibm.wbit.bpel.services.partnerlinktype.Role;
import com.ibm.wbit.bpel.services.partnerlinktype.RolePortType;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.DisplayName;
import com.ibm.wbit.bpelpp.Id;
import com.ibm.wbit.bpelpp.Parameter;
import com.ibm.wbit.br.core.util.RuleConstants;
import com.ibm.wbit.sca.scdl.ae.AdaptiveEntityFactory;
import com.ibm.wbit.sca.scdl.ae.AdaptiveEntityImplementation;
import com.ibm.wbit.sca.scdl.ae.TAdaptiveEntity;
import com.ibm.ws.fabric.da.scdl.DAFactory;
import com.ibm.wsspi.sca.scdl.Component;
import com.ibm.wsspi.sca.scdl.DocumentRoot;
import com.ibm.wsspi.sca.scdl.Implementation;
import com.ibm.wsspi.sca.scdl.InteractionStyle;
import com.ibm.wsspi.sca.scdl.InterfaceQualifier;
import com.ibm.wsspi.sca.scdl.InterfaceSet;
import com.ibm.wsspi.sca.scdl.JoinTransaction;
import com.ibm.wsspi.sca.scdl.SCAImportBinding;
import com.ibm.wsspi.sca.scdl.SCDLFactory;
import com.ibm.wsspi.sca.scdl.Transaction;
import com.ibm.wsspi.sca.scdl.TransactionAttribute;
import com.ibm.wsspi.sca.scdl.eis.Destination;
import com.ibm.wsspi.sca.scdl.eis.DestinationUsage;
import com.ibm.wsspi.sca.scdl.eis.EISFactory;
import com.ibm.wsspi.sca.scdl.eis.JMSImportBinding;
import com.ibm.wsspi.sca.scdl.eis.JMSImportMethodBinding;
import com.ibm.wsspi.sca.scdl.eis.OutboundConnection;
import com.ibm.wsspi.sca.scdl.eis.ResourceAdapter;
import com.ibm.wsspi.sca.scdl.java.JavaFactory;
import com.ibm.wsspi.sca.scdl.java.JavaImplementation;
import com.ibm.wsspi.sca.scdl.mfc.MediationFlowFactory;
import com.ibm.wsspi.sca.scdl.mfc.MediationFlowImplementation;
import com.ibm.wsspi.sca.scdl.process.ProcessFactory;
import com.ibm.wsspi.sca.scdl.process.ProcessImplementation;
import com.ibm.wsspi.sca.scdl.process.TProcess;
import com.ibm.wsspi.sca.scdl.task.TTask;
import com.ibm.wsspi.sca.scdl.task.TaskFactory;
import com.ibm.wsspi.sca.scdl.task.TaskImplementation;
import com.ibm.wsspi.sca.scdl.webservice.WebServiceFactory;
import com.ibm.wsspi.sca.scdl.webservice.WebServiceImportBinding;
import com.ibm.wsspi.sca.scdl.wsdl.WSDLPortType;


/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Component Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class ComponentRuleImpl extends TransformationRuleImpl implements ComponentRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	private boolean executedOnce=false;

	public static final int VARIABLE_TYPE = 0;
	public static final int INPUT_VARIABLE_TYPE = 1;
	public static final int OUTPUT_VARIABLE_TYPE = 2;
	public static final int FAULT_VARIABLE_TYPE = 3;

	public static final String JMS_DATA_BINDING = "com.ibm.websphere.sca.jms.data.impl.JMSDataBindingImplJava";
	public static final String JMS_RESOURCE_ADAPTER = "com.ibm.ws.sib.api.jmsra.impl.JmsJcaResourceAdapterImpl";
	public static final String JMS_OUTBOUND_CONNECTION = "com.ibm.ws.sib.api.jmsra.impl.JmsJcaManagedConnectionFactoryImpl";
	public static final String JMS_QUEUE = "com.ibm.ws.sib.api.jms.impl.JmsQueueImpl";
	public static final String JMS_QUEUE_TYPE = "javax.jms.Queue";
	public static final String SERVICE_OPERATION_ASPECT = "ServiceOperation";

	private AttributeValueProvider avp= AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.COMPONENT_NAME);

	private Component component;
	
	private boolean isCBAImplOverride = false;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ComponentRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return ScdlPackage.Literals.COMPONENT_RULE;
	}

	/**
	 * AZ WMT 20070315
	 * Source = BOM Action, one or more wInterface (PortType)
	 * Target = DocumentRoot representing the SCDL artifact
	 */
	public boolean transformSource2Target(){
		LoggingUtil.traceEntry(this, "transformSource2Target");	
		if (executedOnce){ 
			// update the SCDL artifact if the rule is not the first time executed
			updateArtifact();
			executeHandlers();
			LoggingUtil.traceExit(this, "transformSource2Target");
			return isComplete();
		}
		Action action = (Action)getSource().get(0);
		PortType pt = (PortType)getSource().get(1);
		
		if(FabricIntegrationUtil.isSupportedCBA(action)){
			if(getSource().get(getSource().size()-1) instanceof CallBehaviorAction){
				CallBehaviorAction cba = (CallBehaviorAction)getSource().get(getSource().size()-1);
				this.isCBAImplOverride = FabricIntegrationUtil.isOverriden(cba);
			}			
		}
		
		DocumentRoot root = null;
		DocumentRoot rootImport = null;
		DocumentRoot rootComponent = null;
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		//AR for TEST Deploy we should only generate the Import for the CBA, no Export or Component are necessary
		if(BpelOptionsUtil.isGenerateForDeployPatternEnabled()){
			if(this.getParentRule() instanceof CallBehaviorActionRule){
				//import should not be created for the ServiceOperation.		
				if((BomUtils.isProcess(action) && BomUtils.isTopLevelProcess(action)) || BomUtils.isReusableHumanTask(action)|| BomUtils.isReusableBusinessRuleTask(action)){
						rootImport = createSCAImport(action,pt);
					if(rootImport != null) root = rootImport;
					getTarget().add(0, root);
					if(ScdlUtil.isComponent(root)){
						//after creating the component and setting it as target update with implementation. 
						updateComponentImplementation(action, root);
					}
					//defect 115247, 116633, 114270, feature 118131
					updateInteractionStyle(root);
					executedOnce=true;
					setComplete(true);
//					AZ WMT 20070322
					executeHandlers();

					LoggingUtil.traceExit(this, "transformSource2Target");
					return isComplete();
				}
			}
		}
		//defect 84033, do not create additional Imports
		if((BpelOptions.isBestPracticePatternEnabed(exportSession)||
				BpelOptionsUtil.isLibraryEnabledWithModuleProject())&& !BpelOptionsUtil.isGenerateForDeployPatternEnabled()){
			//AZ 20070313 This section is to create .import for every 
			//tasks/services excluding process			
			rootComponent = createSCDLArtifact(action,pt);
			//we create import for each local task but top level process					
			if(rootComponent != null && rootComponent.getComponent() != null){
//				AZ 06/22/07 handle multiple projects export in a cross project reference setting
				//We create SCA import 
				if(!CrossProjectReferenceUtil.isSingleProjectExport()){
					if(CrossProjectReferenceUtil.isReferenced(action)){
						if(rootComponent != null && rootComponent.getComponent() != null){
							if(BomUtils.isTask(action)){
								if(BpelOptions.isBestPracticePatternEnabed(exportSession)||
									BomUtils.isReusableTask(action)){								
									rootImport = createSCAImport(action,pt);
								}
							}
							else{
								//GJD 20070924
								if (action instanceof BusinessRuleAction) {
									if (BomUtils.isReusableBusinessRuleTask(action)) {
										rootImport = createSCAImport(action,pt);	// GJD this should not be called for local BR if
									}
									//should skip the case of exporting to d2d for the BusinessService.. 
								} else if(!(BomUtils.isServiceOperation(action) && BpelOptionsUtil.isGenerateForDeployPatternEnabled())){
									rootImport = createSCAImport(action,pt);
								}
							}
						}	
					}else{
						if(!(BomUtils.isProcess(action) && BomUtils.isTopLevelProcess(action)) && 
								!BomUtils.isHumanTask(action) && !BomUtils.isBusinessRuleTask(action)&&
								BpelOptions.isBestPracticePatternEnabed(exportSession)){
							rootImport = createSCAImport(action,pt);
						}else if(CrossProjectReferenceUtil.createImport(this, action)){
							rootImport = createSCAImport(action,pt);
						}
					}
				}else{
					if(!(BomUtils.isProcess(action) && BomUtils.isTopLevelProcess(action)) && 
							!BomUtils.isHumanTask(action)&&!BomUtils.isBusinessRuleTask(action)){
						if(BomUtils.isTask(action)){
							if(BpelOptions.isBestPracticePatternEnabed(exportSession)||
									BomUtils.isReusableTask(action)){						
								rootImport = createSCAImport(action,pt);
							}
						}else{
							rootImport = createSCAImport(action,pt);
						}
					}			    		
				}		    	
			}else if(rootComponent != null && rootComponent.getImport() != null &&
					BpelOptions.isBestPracticePatternEnabed(exportSession)){//d161850 (do not create component for module and lib option)
				if(!OriginalWsdlUtil.isBindingInfoAvailable(action, pt,getContext())||
						ExportOperationUtil.isDynamicAssembler(avp, action)){
					rootImport = createSCDLComponent(action,pt);
				}
				else if(isCBAImplOverride){//based on type to create 
					CallBehaviorAction cba = (CallBehaviorAction)getSource().get(getSource().size()-1);
					if(ExportOperationUtil.isDynamicAssembler(avp, cba)){
						rootImport = createSCDLComponent(action,pt);
					}
				}
			}

		}else{
			root = createSCDLArtifact(action,pt);	
		}
		

		
		if(BpelOptions.isBestPracticePatternEnabed(exportSession)||
				BpelOptionsUtil.isLibraryEnabledWithModuleProject()){
			if(rootComponent != null && rootImport != null){
				getTarget().add(0,rootComponent);	
				if(ScdlUtil.isComponent(rootComponent)){
					updateComponentImplementation(action, rootComponent);
				}
				updateInteractionStyle(rootComponent);

				getTarget().add(1,rootImport);
				if(ScdlUtil.isComponent(rootImport)){
					updateComponentImplementation(action, rootImport);
				}
				updateInteractionStyle(rootImport);					
			}else{
				if(rootComponent != null){
					root = rootComponent;
				}else if(rootImport != null){
					root = rootImport;
				}
				getTarget().add(0, root);
				if(ScdlUtil.isComponent(root)){
					//after creating the component and setting it as target update with implementation. 
					updateComponentImplementation(action, root);
				}
				//defect 115247, 116633, 114270, feature 118131
				updateInteractionStyle(root);
			}			
		}else{
			
			getTarget().add(0, root);
			if(ScdlUtil.isComponent(root)){
				//after creating the component and setting it as target update with implementation. 
				updateComponentImplementation(action, root);
			}
			//defect 115247, 116633, 114270, feature 118131
			updateInteractionStyle(root);
		}
		executedOnce=true;
		setComplete(true);
//		AZ WMT 20070322
		executeHandlers();

		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}

	/**
	 * Update the content of the SCDL Component with implementation
	 *
	 */
	private void updateComponentImplementation(Action action, DocumentRoot root){
		if(root != null){
			if (action  instanceof BusinessRuleAction && getParentRule() instanceof CallBehaviorActionRule) {
				return;		// do not create impl for CBA pointing to global BR
			}
			Implementation impl = createSCDLImplementation(action);			
			if (impl!=null) {
				ExportOperationUtil.validateImplementationType(action, impl);
 				root.getComponent().setImplementation(impl);
			}
		}
	}

	/**
	 * Update the content of the SCDL artifact
	 *
	 */
	private void updateArtifact() {
		Implementation impl = getImplementationFromTarget();
		// update the implementation type
		if (impl != null) {	
			createImplementation((Action)getSource().get(0),impl);
		} else if (!getSource().isEmpty() &&
				getSource().get(0) instanceof BusinessRuleAction) {		// for CBA pointing to BusinessRuleAction, create/update implementation
			impl = BrgimplFactory.eINSTANCE.createBusinessRuleGroupImplementation();
			createImplementation((Action)getSource().get(0),impl);
			// set default transaction
			Transaction tx = SCDLFactory.eINSTANCE.createTransaction();
			tx.setValue(TransactionAttribute.GLOBAL_LITERAL);
			impl.getImplementationQualifiers().add(tx);
			if (!getTarget().isEmpty() && getTarget().get(0) instanceof DocumentRoot) {
				Component component = getComponentFromTarget();
				if (component != null) {
					component.setImplementation(impl);
				}
			}
		}

		// add additional interfaces
		updateInterfaceSet();
	}

	/**
	 * Add additional interfaces to the interfaceSet of the Component
	 *
	 */
	private void updateInterfaceSet() {
		Component component = getComponentFromTarget();
		if (component != null) {
			InterfaceSet is = component.getInterfaceSet();
			if (is != null) {
				for (int i=0; i<getSource().size();i++) {
					if (getSource().get(i) instanceof PortType) {
						PortType pt = (PortType)getSource().get(i);
						boolean bInterfaceAlreadyExists=false;
						for (int j=0; j<is.getInterfaces().size();j++) {
							if (is.getInterfaces().get(j) instanceof WSDLPortType) {
								WSDLPortType wpt = (WSDLPortType)is.getInterfaces().get(j);
								QName qname = (QName)wpt.getPortType();
								//if (qname.getNamespaceURI()!=pt.getQName().getNamespaceURI() && qname.getLocalPart()!=pt.getQName().getLocalPart()) {
								//wpt = ScdlUtil.createwInterface(pt);
								//is.getInterfaces().add(wpt);
								if (qname.getNamespaceURI()==pt.getQName().getNamespaceURI() && qname.getLocalPart()==pt.getQName().getLocalPart()) {
									bInterfaceAlreadyExists=true;
									break;
								}
							}
						}
						if (!bInterfaceAlreadyExists) {
							WSDLPortType wpt = ScdlUtil.createwInterface(pt);
							is.getInterfaces().add(wpt);
						}
					}
				}				
			}	
			//ScdlUtil.createNSPrefixForWSDLPortType((DocumentRoot)getTarget().get(0));
		}	
		if(component != null){
//			defect 115247, 116633, 114270
			updateComponentInteractionStyle(component);
		}
	}

	/**
	 * Get Component object from the Rule's target list
	 * @return
	 */
	private Component getComponentFromTarget() {
		DocumentRoot root = (DocumentRoot)getTarget().get(0);
		if (root.getComponent()!=null) {
			return root.getComponent();
		}
		return null;
	}

	/**
	 * Get Implementation of a component object from the Rule's target list
	 * @return
	 */
	private Implementation getImplementationFromTarget() {
		Component component = getComponentFromTarget();
		if (component!=null) {
			Implementation impl=null;
			return component.getImplementation();
		}
		return null;
	}


	/**
	 * Create SCDL artifact, either a component or a import
	 * @param action
	 * @param pt
	 * @return
	 */
	private DocumentRoot createSCDLArtifact(Action action, PortType pt) {
		//feature 118131, if this is a Service generated through WSDL Import and with no implementation specified by the user
		//generate an SCDL Import

//		GJD 20060626 START	    
		// find if there is implmenetation for the action in technical attributes
		Object implementation = null;
		if(FabricIntegrationUtil.shouldUseCBAImplementation(this, isCBAImplOverride)){
			CallBehaviorAction cba = (CallBehaviorAction)(getSource().get(getSource().size()-1));
			implementation = avp.getAttributeValueForType(cba, AttributeNameConstants.IMPLEMENTATION);
		}else{
			implementation = avp.getAttributeValueForType(action, AttributeNameConstants.IMPLEMENTATION);
		}
//		Object implementation = avp.getAttributeValueForType(action, AttributeNameConstants.IMPLEMENTATION);
		if (implementation != null) {
			if (implementation instanceof WebServices) {
				//AR Feature 1609 If implementation is Webservice, we would like to generate the MFC and the SCDL Import if there is a valid endpoint
				//defect 82758
				DocumentRoot scdlImport = null;
				if(OriginalWsdlUtil.isBindingInfoAvailable(action, pt, getContext())){
					scdlImport = createSCDLImport(action, pt);
					if(scdlImport != null)getTarget().add(scdlImport);
				}
				DocumentRoot dr = createSCDLComponent(action, pt);
				if(scdlImport != null) {
					com.ibm.wsspi.sca.scdl.Reference reference = ScdlUtil.createSCDLComponentReference(dr, scdlImport, pt, ScdlUtil.MEDIATION_FLOW_REFERENCE_NAME, false);
				}
				return dr;
			//	return createSCDLImport(action, pt);
			} else if (implementation instanceof SCA) {
				return createSCAImport(action, pt);
			} else if (implementation instanceof JMS) {
				return createJMSImport(action, pt);
			}else if(implementation instanceof DynamicAssembler){
				ExportSession exportSession = BpelOptionsUtil.getExportSession();
				if(BpelOptions.isBestPracticePatternEnabed(exportSession)){
					return createSCAImport(action,pt);
				}
				return createSCDLComponent(action,pt);
			}
			return createSCDLComponent(action,pt);
			
		}
//		GJD original handling below is the default for imported from WID with no implementation set by the user
//		GJD 20060626 END

		// create the .import for the user set implementation type 
		if(isServiceAsWebService(action)){
			return createSCDLImport(action, pt);
		}

		// GJD 20061016: create import for external service
		if (action != null && action.getAspect() != null && action.getAspect().equals(SERVICE_OPERATION_ASPECT)) {
			DocumentRoot scdlImport = null;
			//AR Feature 1609 If implementation is Webservice, we would like to generate the MFC and the SCDL Import if there is a valid endpoint
			//defect 82758
			if(OriginalWsdlUtil.isBindingInfoAvailable(action, pt, getContext())){
					scdlImport = createSCDLImport(action, pt);
					if(scdlImport != null)getTarget().add(scdlImport);
				}
				DocumentRoot dr = createSCDLComponent(action, pt);
				if(scdlImport != null) {
					com.ibm.wsspi.sca.scdl.Reference reference = ScdlUtil.createSCDLComponentReference(dr, scdlImport, pt, ScdlUtil.MEDIATION_FLOW_REFERENCE_NAME, false);
				}
				return dr;
		//	return createSCDLImport(action, pt);
		}

		return createSCDLComponent(action,pt);
	}
	/**
	 * Returns true if this Service was generated through WSDL import and the user did not set any of the 
	 * Component Technical Attributes, in which case it should overwrite the default behaviour.
	 * @param action
	 * @return
	 */
	private boolean isServiceAsWebService(Action action){
		//feature 118131, defect 119267
		if(BomUtils.isService(action) && isFromWSDLImport(action)){
			if(isEmpty(action, AttributeNameConstants.COMPONENT_NAME)  &&
					isEmpty(action, AttributeNameConstants.COMPONENT_DISPLAY_NAME) && 
					isEmpty(action, AttributeNameConstants.COMPONENT_DESCRIPTION) && 
					isEmpty(action, AttributeNameConstants.IMPLEMENTATION)){
				return true;
			}
		}

		return false;
	}
	/**
	 * Returns true if the TechnicalAttribute for the <code>source<code> and <code>propertyName</code> does not exist
	 * or is an empty String
	 * @param source
	 * @param propertyName
	 * @return
	 */
	private boolean isEmpty(Action source, String propertyName){
		//defect 119267
		if(propertyName.equals(AttributeNameConstants.IMPLEMENTATION)){
			return TechnicalAttributesHelper.getAttributeValue(source, propertyName) == null;
		}else if(TechnicalAttributesHelper.getAttributeValue(source, propertyName) == null || 
				((String)TechnicalAttributesHelper.getAttributeValue(source, propertyName)).equals("")){
			return true;
		}
		return false;

	}
	/**
	 * Returns true if this action (should be a ReusableService implementation) was generated through WSDL Import.
	 * @param action a StructuredActvityNode, implementation of a Reusable Service
	 * @return boolean
	 */
	private boolean isFromWSDLImport(Action action){
		if(action.getOwnedComment() != null && !action.getOwnedComment().isEmpty()){
			List comments = action.getOwnedComment();
			Iterator it = comments.iterator();
			while(it.hasNext()){
				Comment comment = (Comment)it.next();
				if(comment.getBody().indexOf(AbstractbpelConstants.SERVICE_FROM_WSDL_TAG) != -1){
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Create SCDL Import
	 * @param action
	 * @param pt
	 * @return
	 */
	private DocumentRoot createSCDLImport(Action action, PortType pt) {
		//feature 118131
		com.ibm.wsspi.sca.scdl.Import scdlImport = SCDLFactory.eINSTANCE.createImport();
		// Set Import Name, DisplayName, and Description
		avp.setContext(getContext());

		//d144755		
		Component proxyComponent = SCDLFactory.eINSTANCE.createComponent();
//		String importName = (String)avp.getAttributeValueForType(scdlImport,action,AttributeNameConstants.COMPONENT_NAME,NamingUtil.getDefaultRegistry(getContext(),NamingUtil.DEFAULT_REGISTRY_SCDL));
		String importName = (String)avp.getAttributeValueForType(proxyComponent, action, AttributeNameConstants.COMPONENT_NAME, NamingUtil.getDefaultRegistry(getContext(), NamingUtil.DEFAULT_REGISTRY_SCDL));
		importName = importName+"WS";
		String packageName = null;
		if (!BpelOptionsUtil.isPersistSCDLComponentToRootFolder()) {
			packageName = SAUtil.createPackage(action,ExportOperation.FILE_SEPARATOR);
		}
		if (importName != null && packageName != null && packageName.length() > 1 &&
				!importName.startsWith(packageName.substring(0, packageName.length() - 1))) {
			importName = packageName + importName;				
		}
		scdlImport.setName(importName);
		String importDisplayName = (String)avp.getAttributeValueForType(scdlImport, action,AttributeNameConstants.COMPONENT_DISPLAY_NAME, null);
		scdlImport.setDisplayName(importDisplayName);
		// set description from implementation
		String description = (String)avp.getAttributeValueForType(action,AttributeNameConstants.IMPLEMENTATION_DESCRIPTION);
		if (description!=null && description.length()>0){
			scdlImport.setDescription(description);			
		}
		// Set Import Interface
		scdlImport.setInterfaceSet(ScdlUtil.createInterfaceSet(pt));		
		WebServiceImportBinding newBinding = WebServiceFactory.eINSTANCE.createWebServiceImportBinding();;
		//update binding information from original wsdl if there exists
		updateBindingInfoFromOriginalWSDL(action, pt, newBinding);
		//AR Feature 1609 if there is no service info we should not generate the import, we will generate the MFC..FOR D2D only 
		//AR enable MFC for WID, defect 82758
		//if(newBinding.getService() == null && BpelOptionsUtil.isGenerateForDeployPatternEnabled()) return null;
		if(newBinding.getService() == null) return null;
		scdlImport.setBinding(newBinding);
		DocumentRoot root = SCDLFactory.eINSTANCE.createDocumentRoot();
		root.setImport(scdlImport);
		CrossProjectReferenceUtil.registerTranformTargetWithProjectName(action, root);
		return root;

	}

	/**
	 * For external service, we update the binding information retrieved from the original WSDL
	 * so that in WID implementation the user doesn't have to set the binding information manually
	 * @param action
	 * @param pt
	 * @param newBinding
	 */
	private void updateBindingInfoFromOriginalWSDL(Action action, PortType pt, WebServiceImportBinding newBinding) {
		if (action != null && action.getAspect() != null && action.getAspect().equals(SERVICE_OPERATION_ASPECT)) {
			Definition orgiginalDef = OriginalWsdlUtil.getOriginalWsdl(getContext(),BomUtils.resolveFileAttachment((StructuredActivityNode)action));
			if(orgiginalDef != null){
				//AZ 2007 Assume the first service
				if(orgiginalDef.getEServices() == null || orgiginalDef.getEServices().isEmpty()){
					return;
				}					
				Service svc = (Service)orgiginalDef.getEServices().get(0);
				if(svc.getQName() != null){
					Object svcQName = XMLTypeUtil.createQName(svc.getQName().getNamespaceURI(),svc.getQName().getLocalPart(), null);
					newBinding.setService(svcQName);	
				}
				Port targetPort = null;
				if(svc != null && svc.getEPorts()!= null && !svc.getEPorts().isEmpty()){
					Iterator prtIt = svc.getEPorts().iterator();
					while(prtIt.hasNext()){
						Port each = (Port)prtIt.next();
						if(each.getEBinding()!= null && each.getEBinding().getPortType()!= null){
							if( each.getEBinding().getPortType().getQName() != null && pt.getQName() != null){
								javax.xml.namespace.QName eachQName = each.getEBinding().getPortType().getQName();
								if(eachQName.getLocalPart().equals(pt.getQName().getLocalPart())&&
										eachQName.getNamespaceURI().equals(pt.getQName().getNamespaceURI())){
									targetPort = each;
									break;
								}
							}
						}
					}
				}
				if(targetPort != null){
					if(pt.getQName() != null){
						Object portQName = XMLTypeUtil.createQName(pt.getQName().getNamespaceURI(),targetPort.getName(), null);
						newBinding.setPort(portQName);
					}							
					//AZ 2007 According to the specification: 
					// 1) Binding extensibility elements (1) are used to specify the address information for the port
					// 2) A port MUST NOT specify more than one address.
					// 3) A port MUST NOT specify any binding information other than address information
					if(targetPort.getEExtensibilityElements() != null && !targetPort.getEExtensibilityElements().isEmpty()){
						Object each = targetPort.getEExtensibilityElements().get(0);
						if(each instanceof SOAPAddress){							
							String endpoint = null;
							SOAPAddress address = (SOAPAddress)each;
							endpoint = address.getLocationURI();	
							if(endpoint != null){
								newBinding.setEndpoint(endpoint);
							}
						}	
					}
				}
			}
		}
	}



	/**
	 * Create SCA Import
	 * @param action
	 * @param pt
	 * @return DocumentRoot
	 */
	private DocumentRoot createSCAImport(Action action, PortType pt) {
		String versionName = null;
		com.ibm.wsspi.sca.scdl.Import scdlImport = SCDLFactory.eINSTANCE.createImport();
		// Set Import Name, DisplayName, and Description
		avp.setContext(getContext());

		//d144755
		Component proxyComponent = SCDLFactory.eINSTANCE.createComponent();
		String importNameConverted = null;
		Action cba = action;

		if (getParentRule() instanceof CallBehaviorActionRule) {
			Iterator it = getParentRule().getSource().iterator();
			Object obj = null;
			while (it.hasNext()) {
				obj = it.next();
				if (obj instanceof CallBehaviorAction) {
					cba = (CallBehaviorAction)obj;
					break;
				} else if (obj instanceof InputPinSet) {
					InputPinSet iSet = (InputPinSet)obj;
					if (iSet.getAction() instanceof CallBehaviorAction) {
						cba = iSet.getAction();
						break;			
					}
				}
			}
			if(FabricIntegrationUtil.shouldUseCBAImplementation(this, isCBAImplOverride)){
				importNameConverted = NamingUtil.convertNameForAction(cba, getContext());
			}else{
				importNameConverted = (String)avp.getAttributeValueForType(scdlImport,cba,AttributeNameConstants.COMPONENT_NAME,NamingUtil.getDefaultRegistry(getContext(),NamingUtil.DEFAULT_REGISTRY_SCDL));
			}
		} else {
			importNameConverted = (String)avp.getAttributeValueForType(scdlImport,action,AttributeNameConstants.COMPONENT_NAME,NamingUtil.getDefaultRegistry(getContext(),NamingUtil.DEFAULT_REGISTRY_SCDL));
		}
		//GJD		String importName = (String)avp.getAttributeValueForType(proxyComponent, action, AttributeNameConstants.COMPONENT_NAME, NamingUtil.getDefaultRegistry(getContext(), NamingUtil.DEFAULT_REGISTRY_SCDL));
		String importName = null;
		if (!BpelOptionsUtil.isPersistSCDLComponentToRootFolder()) {
//			importName = SAUtil.createPackage(action,ExportOperation.FILE_SEPARATOR)+ importName;
			importName = importNameConverted;
		} else {
			importName = NamingUtil.getNameForArtifact(importNameConverted);
		}
		scdlImport.setName(importName);
		String importDisplayName = null;
		if(getSource().size() >= 3 && (getSource().get(getSource().size()-1) instanceof CallBehaviorAction)){
			CallBehaviorAction cbaInstance = (CallBehaviorAction)(getSource().get(getSource().size()-1));
			importDisplayName = (String)avp.getAttributeValueForType(scdlImport, cbaInstance,AttributeNameConstants.COMPONENT_DISPLAY_NAME, null);			
		}else{
			importDisplayName = (String)avp.getAttributeValueForType(scdlImport, action,AttributeNameConstants.COMPONENT_DISPLAY_NAME, null);
		}
		scdlImport.setDisplayName(importDisplayName);
		
		// set description from implementation
		String description = null;
		if(getSource().size() >= 3 && (getSource().get(getSource().size()-1) instanceof CallBehaviorAction)){
			CallBehaviorAction cbaInstance = (CallBehaviorAction)(getSource().get(getSource().size()-1));
			description = (String)avp.getAttributeValueForType(cbaInstance,AttributeNameConstants.IMPLEMENTATION_DESCRIPTION);
		}else{
			description = (String)avp.getAttributeValueForType(action,AttributeNameConstants.IMPLEMENTATION_DESCRIPTION);
		}
		if (description!=null && description.length()>0){
			scdlImport.setDescription(description);			
		}
		// Set Import Interface
		InterfaceSet interfaceSet = ScdlUtil.createInterfaceSet(pt);		// creates 1 and only 1 Interface
		//ScdlUtil.updateInterfaceWithOperations((Interface)interfaceSet.getInterfaces().get(0), pt);
		scdlImport.setInterfaceSet(interfaceSet);

		//create the SCA Binding
		SCAImportBinding importBinding = SCDLFactory.eINSTANCE.createSCAImportBinding();
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		boolean skipModuleExport = false;
		if(FabricIntegrationUtil.shouldUseCBAImplementation(this, isCBAImplOverride)){				
			Object implementation = avp.getAttributeValueForType(cba, AttributeNameConstants.IMPLEMENTATION);
			if(implementation instanceof SCA){
				skipModuleExport = true;
				Object docRootRegistry = exportSession.getExportOptions().getAdditionalOption(TransformationSessionKeyConstants.TOP_LEVEL_DOCUMENT_ROOT);
				if(docRootRegistry instanceof Set){
					((Set)docRootRegistry).add(scdlImport);
				}			
			}
		}else if(BomUtils.isTask(action)&& !BomUtils.isReusableTask(action)){
			Object implementation = avp.getAttributeValueForType(action, AttributeNameConstants.IMPLEMENTATION);
			if(implementation instanceof SCA){
				skipModuleExport = true;
			}
		}
		if(BpelOptions.isBestPracticePatternEnabed(exportSession)){
			if(!skipModuleExport){
				//defect 155432
				if(BomUtils.isReusableHumanTask(action)||BomUtils.isReusableBusinessRuleTask(action)||
						(BomUtils.isProcess(action)&& BomUtils.isTopLevelProcess(action))){
					importBinding.setModuleName(BpelOptionsUtil.getWPSModuleName(CrossProjectReferenceUtil.getProjectNameForSourceObject(action)));
				}else{
					importBinding.setModuleName(BpelOptionsUtil.getImplementationModuleName(CrossProjectReferenceUtil.getProjectNameForSourceObject(action)));
				}
				String exportName = null;
				if(getSource().size() >= 3 && (getSource().get(getSource().size()-1) instanceof CallBehaviorAction)){
					CallBehaviorAction cbaInstance = (CallBehaviorAction)(getSource().get(getSource().size()-1));
					exportName = NamingUtil.convertNameForAction(cbaInstance, getContext());

				}else{
					exportName = NamingUtil.convertNameForAction(action, getContext());
				}

				//importBinding.setExportName(importName);
				importBinding.setExportName(exportName);
			}
		}else{
//			if(BomUtils.isReusableHumanTask(action)||BomUtils.isReusableBusinessRuleTask(action)||
//					(BomUtils.isProcess(action)&& BomUtils.isTopLevelProcess(action)) || BomUtils.isServiceOperation(action)){			
			if(!skipModuleExport){
				importBinding.setModuleName(BpelOptionsUtil.getWPSModuleName(CrossProjectReferenceUtil.getProjectNameForSourceObject(action)));
				//d85506 
				String exportName = null;
				if(getSource().size() >= 3 && (getSource().get(getSource().size()-1) instanceof CallBehaviorAction)){
					CallBehaviorAction cbaInstance = (CallBehaviorAction)(getSource().get(getSource().size()-1));
					exportName = NamingUtil.convertNameForAction(cbaInstance, getContext());
				
				}else{
					exportName = NamingUtil.convertNameForAction(action, getContext());
				}
//				String exportName = NamingUtil.convertNameForAction(action, getContext());
				//importBinding.setExportName(importName);
				importBinding.setExportName(exportName);
			}
//			}
		}
		if(BpelOptionsUtil.isGenerateForDeployPatternEnabled()){
			if(BomUtils.isReusableHumanTask(action)||BomUtils.isReusableBusinessRuleTask(action)|| (BomUtils.isProcess(action)&& BomUtils.isTopLevelProcess(action))){
				String moduleName = BpelOptionsUtil.getWPSModuleyNameForElement(((StructuredActivityNode)action).getActivity().getUid());
				importBinding.setModuleName(moduleName);
				String exportName = null;
				if(getSource().size() >= 3 && (getSource().get(getSource().size()-1) instanceof CallBehaviorAction)){
					CallBehaviorAction cbaInstance = (CallBehaviorAction)(getSource().get(getSource().size()-1));
					exportName = NamingUtil.convertNameForAction(cbaInstance, getContext());
				
				}else{
					exportName = NamingUtil.convertNameForAction(action, getContext());
				}
				importBinding.setExportName(exportName);
				versionName = BpelOptionsUtil.getWPSModuleVersionForElement(((StructuredActivityNode)action).getActivity().getUid());

			}
		}
		//AR add version info to the generated import. If no version is set, we will default to 1.0.0
		if(versionName == null){
			versionName = WIDProjectUtil.DEFAULT_VERSION;
		}
		importBinding.setVersionValue(versionName);
		scdlImport.setBinding(importBinding);
		DocumentRoot root = SCDLFactory.eINSTANCE.createDocumentRoot();
		root.setImport(scdlImport);
		return root;
	}


	/**
	 * Create JMS Import
	 * @param action
	 * @param pt
	 * @return documentRoot
	 */
	private DocumentRoot createJMSImport(Action action, PortType pt) {
		com.ibm.wsspi.sca.scdl.Import scdlImport = SCDLFactory.eINSTANCE.createImport();
		// Set Import Name, DisplayName, and Description
		avp.setContext(getContext());       
		//d144755
		Component proxyComponent = SCDLFactory.eINSTANCE.createComponent();
//		String importName = (String)avp.getAttributeValueForType(scdlImport,action,AttributeNameConstants.COMPONENT_NAME,NamingUtil.getDefaultRegistry(getContext(),NamingUtil.DEFAULT_REGISTRY_SCDL));
		String importName = (String)avp.getAttributeValueForType(proxyComponent, action, AttributeNameConstants.COMPONENT_NAME, NamingUtil.getDefaultRegistry(getContext(), NamingUtil.DEFAULT_REGISTRY_SCDL));

		if (!BpelOptionsUtil.isPersistSCDLComponentToRootFolder()) {
			importName = SAUtil.createPackage(action,ExportOperation.FILE_SEPARATOR)+ importName;
		}
		//override the import name based on the cba package
		if(FabricIntegrationUtil.shouldUseCBAImplementation(this, isCBAImplOverride)){
	    	   CallBehaviorAction cba = (CallBehaviorAction)(getSource().get(getSource().size()-1));
	    	   importName = NamingUtil.convertNameForAction(cba, getContext());
	       }
		
		scdlImport.setName(importName);
		String importDisplayName = (String)avp.getAttributeValueForType(scdlImport, action,AttributeNameConstants.COMPONENT_DISPLAY_NAME, null);
		scdlImport.setDisplayName(importDisplayName);
		// set description from implementation
		String description = (String)avp.getAttributeValueForType(action,AttributeNameConstants.IMPLEMENTATION_DESCRIPTION);
		if (description!=null && description.length()>0){
			scdlImport.setDescription(description);			
		}
		// Set Import Interface
		InterfaceSet interfaceSet = ScdlUtil.createInterfaceSet(pt);		// creates 1 and only 1 Interface
		//ScdlUtil.updateInterfaceWithOperations((Interface)interfaceSet.getInterfaces().get(0), pt);
		scdlImport.setInterfaceSet(interfaceSet);

		//create the JMS Binding
		JMSImportBinding importBinding = EISFactory.eINSTANCE.createJMSImportBinding();
		importBinding.setDataBindingType(JMS_DATA_BINDING);

		ResourceAdapter resourceAdapter = EISFactory.eINSTANCE.createResourceAdapter();
		resourceAdapter.setType(JMS_RESOURCE_ADAPTER);
		importBinding.setResourceAdapter(resourceAdapter);

		OutboundConnection connection = EISFactory.eINSTANCE.createOutboundConnection();
		connection.setType(JMS_OUTBOUND_CONNECTION);
		importBinding.setConnection(connection);

		Destination sendDestination = EISFactory.eINSTANCE.createDestination();
		sendDestination.setImplType(JMS_QUEUE);
		sendDestination.setType(JMS_QUEUE_TYPE);
		sendDestination.setUsage(DestinationUsage.SEND_LITERAL);
		importBinding.getDestination().add(sendDestination);

		// method binding - operations within port type
		List operations = pt.getOperations();
		boolean isOneWayOnly = true;
		if (operations != null) {
			Operation op = null;
			Iterator it = operations.iterator();
			while (it.hasNext()) {
				op = (Operation) it.next();
				JMSImportMethodBinding jmsImportMethodBinding = EISFactory.eINSTANCE.createJMSImportMethodBinding();
				jmsImportMethodBinding.setMethod(op.getName());
				importBinding.getMethodBinding().add(jmsImportMethodBinding);
				if (op.getOutput() != null) {
					isOneWayOnly = false;
				}
			}
		}

		if (!isOneWayOnly) {
			Destination receiveDestination = EISFactory.eINSTANCE.createDestination();
			receiveDestination.setImplType(JMS_QUEUE);
			receiveDestination.setType(JMS_QUEUE_TYPE);
			receiveDestination.setUsage(DestinationUsage.RECEIVE_LITERAL);
			importBinding.getDestination().add(receiveDestination);

			Destination callbackDestination = EISFactory.eINSTANCE.createDestination();
			callbackDestination.setImplType(JMS_QUEUE);
			callbackDestination.setType(JMS_QUEUE_TYPE);
			callbackDestination.setUsage(DestinationUsage.CALLBACK_LITERAL);
			importBinding.getDestination().add(callbackDestination);
		}

		scdlImport.setBinding(importBinding);

		DocumentRoot root = SCDLFactory.eINSTANCE.createDocumentRoot();
		root.setImport(scdlImport);
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(!BpelOptions.isBestPracticePatternEnabed(exportSession)){			
			Object docRootRegistry = exportSession.getExportOptions().getAdditionalOption(TransformationSessionKeyConstants.TOP_LEVEL_DOCUMENT_ROOT);
			if(docRootRegistry instanceof Set){
				((Set)docRootRegistry).add(scdlImport);
			}			
		}
		return root;
	}


	/**
	 * Create SCDL Component
	 * @param action
	 * @param pt
	 * @return
	 */
	private DocumentRoot createSCDLComponent(Action action, PortType pt) {

		component = SCDLFactory.eINSTANCE.createComponent();
		// Set Component Name, DisplayName, and Description
		avp.setContext(getContext());
		String convertedComponentName  = null;
		if(FabricIntegrationUtil.shouldUseCBAImplementation(this, isCBAImplOverride)){
			CallBehaviorAction cba = (CallBehaviorAction)(getSource().get(getSource().size()-1));
			convertedComponentName = (String)avp.getAttributeValueForType(component,cba,AttributeNameConstants.COMPONENT_NAME,NamingUtil.getDefaultRegistry(getContext(),NamingUtil.DEFAULT_REGISTRY_SCDL));
		}else{
			convertedComponentName = (String)avp.getAttributeValueForType(component,action,AttributeNameConstants.COMPONENT_NAME,NamingUtil.getDefaultRegistry(getContext(),NamingUtil.DEFAULT_REGISTRY_SCDL));
		}
		String componentName = null;
		if (!BpelOptionsUtil.isPersistSCDLComponentToRootFolder()) {
			if (convertedComponentName.indexOf(ExportOperation.FILE_SEPARATOR) == -1) {		// for component name set in TA
				componentName = SAUtil.createPackage(action,ExportOperation.FILE_SEPARATOR)+convertedComponentName;
			} else {
				componentName = convertedComponentName;
			}
		} else {
			componentName = NamingUtil.getNameForArtifact(convertedComponentName);
		}
		
		component.setName(componentName);
		String componentDisplayName = null;
		if(FabricIntegrationUtil.shouldUseCBAImplementation(this, isCBAImplOverride)){
			CallBehaviorAction cba = (CallBehaviorAction)(getSource().get(getSource().size()-1));
			componentDisplayName = (String)avp.getAttributeValueForType(cba,AttributeNameConstants.COMPONENT_DISPLAY_NAME);
		}else{
			componentDisplayName = (String)avp.getAttributeValueForType(action,AttributeNameConstants.COMPONENT_DISPLAY_NAME);
		}
		component.setDisplayName(componentDisplayName);
		String description = null;
		if(FabricIntegrationUtil.shouldUseCBAImplementation(this, isCBAImplOverride)){
			CallBehaviorAction cba = (CallBehaviorAction)(getSource().get(getSource().size()-1));
			description = (String)avp.getAttributeValueForType(cba,AttributeNameConstants.COMPONENT_DESCRIPTION);
		}else{
			description = (String)avp.getAttributeValueForType(action,AttributeNameConstants.COMPONENT_DESCRIPTION);
		}		
		if (description!=null && description.length()>0){
			component.setDescription(description);			
		}

		// Set Component Interface
		component.setInterfaceSet(ScdlUtil.createInterfaceSet(pt));

		DocumentRoot root = SCDLFactory.eINSTANCE.createDocumentRoot();
		root.setComponent(component);
		//ScdlUtil.createNSPrefixForWSDLPortType(root);

		return root;

	}

	/**
	 * Check if the Implementation type is WebServices or EnterpriseAccess that qualifies to create a SCDL Import
	 * @param action
	 * @return
	 */
	private boolean isCreateImport(Action action) {
		Object implementation = avp.getAttributeValueForType(action,AttributeNameConstants.IMPLEMENTATION);
		return (implementation instanceof WebServices || implementation instanceof EnterpriseAccess);
	}

	/**
	 * Create SCDL Implementation for the Component
	 * @param action
	 * @return
	 */
	private Implementation createSCDLImplementation(Action action) {
		Implementation impl = null;
		Object implementation = null;
		if(!isCBAImplOverride){
			implementation = avp.getAttributeValueForType(action,AttributeNameConstants.IMPLEMENTATION);
		}else if(getSource().get(getSource().size()-1) instanceof CallBehaviorAction){
			CallBehaviorAction cba = (CallBehaviorAction)getSource().get(getSource().size()-1);
			//we need to use the CBA's implementation type
			implementation = avp.getAttributeValueForType(cba,AttributeNameConstants.IMPLEMENTATION);
		}
		if (implementation != null && !implementation.equals(AttributeNameConstants.IMPLEMENTATION)) {
			if (impl== null) {
				if(implementation instanceof HumanTask){
					impl = TaskFactory.eINSTANCE.createTaskImplementation();
				} else if (implementation instanceof com.ibm.btools.te.attributes.model.definition.implementation.wps.Process) {
					impl = ProcessFactory.eINSTANCE.createProcessImplementation();
				} else if (implementation instanceof Java) {
					impl = JavaFactory.eINSTANCE.createJavaImplementation();
				} else if (implementation instanceof BusinessRule) {
					impl = BrgimplFactory.eINSTANCE.createBusinessRuleGroupImplementation();
//					} else if (implementation instanceof Selector) {
//					impl = SelectorimplFactory.eINSTANCE.createSelectorImplementation();	
				} else if (implementation instanceof AdaptiveEntity) {
					impl = AdaptiveEntityFactory.eINSTANCE.createAdaptiveEntityImplementation();
				} 
				else if(implementation instanceof DynamicAssembler){				
					impl = DAFactory.eINSTANCE.createDynamicAssembler();
				}
				else if(implementation instanceof WebServicesImpl){
					impl = MediationFlowFactory.eINSTANCE.createMediationFlowImplementation();
				}
				else {
					impl = SCDLFactory.eINSTANCE.createImplementation();	
				}
			}
			String description = null;
			if(isCBAImplOverride){
				if(getSource().get(getSource().size()-1) instanceof CallBehaviorAction){
					CallBehaviorAction cba = (CallBehaviorAction)getSource().get(getSource().size()-1);
					description = (String)avp.getAttributeValueForType(cba,AttributeNameConstants.IMPLEMENTATION_DESCRIPTION);
				}
			}else{
				description = (String)avp.getAttributeValueForType(action,AttributeNameConstants.IMPLEMENTATION_DESCRIPTION);
			}
			if (description!=null && description.length()>0){
				impl.setDescription(description);			
			}
			createImplementation(action,impl);
		} else {
			// if no implementation is set, but the action is a Process, then we always create a Process type scdl impl
			if (BomUtils.isProcess(action) && ((HeuristicsTransformation)getRoot()).isFirstClassArtifact(action.eContainer())) {
				impl = ProcessFactory.eINSTANCE.createProcessImplementation();
				createImplementation(action,impl);
			} else if (action instanceof BusinessRuleAction) {//TODO should it create implementation for local BR?
																	//&& ((HeuristicsTransformation)getRoot()).isFirstClassArtifact(action.eContainer())) {
				impl = BrgimplFactory.eINSTANCE.createBusinessRuleGroupImplementation();
				createImplementation(action,impl);
			}else if (action instanceof com.ibm.btools.bom.model.processes.humantasks.HumanTask && ((HeuristicsTransformation)getRoot()).isFirstClassArtifact(action.eContainer())) {
				impl = TaskFactory.eINSTANCE.createTaskImplementation();
				createImplementation(action,impl);				
			} else if ((BomUtils.isReusableTask(action) || BomUtils.isReusableService(action)) && (!HumanTaskUtil.isUse61HumanTaskSupport() && StaffUtil.isStaffRequired(action))){
				impl = TaskFactory.eINSTANCE.createTaskImplementation();
				createImplementation(action,impl);
				//16509, add MFC pattern, generate the MFC implementation for all Tasks, Services for the d2d pattern only
				//AR enable MFC for WID, defect 82758
			}else if(BomUtils.isTask(action) || BomUtils.isService(action)|| BomUtils.isReusableTask(action) || BomUtils.isReusableService(action) || BomUtils.isServiceOperation(action)){
//			else if(BpelOptionsUtil.isGenerateForDeployPatternEnabled() && (BomUtils.isTask(action) || BomUtils.isService(action)|| BomUtils.isReusableTask(action) || BomUtils.isReusableService(action) || BomUtils.isServiceOperation(action))){
				impl = MediationFlowFactory.eINSTANCE.createMediationFlowImplementation();
				createImplementation(action,impl);
			}
		}
		if (impl!=null && !(impl instanceof com.ibm.ws.fabric.da.scdl.DynamicAssembler)) {
			// set default transaction
			Transaction tx = SCDLFactory.eINSTANCE.createTransaction();
			tx.setValue(TransactionAttribute.GLOBAL_LITERAL);
			impl.getImplementationQualifiers().add(tx);
		}
		return impl;
	}	

	/**
	 * Create the concrete implementation. We only support two kinds:  BPEL Process and TEL Human Task
	 * @param action
	 * @param impl
	 */
	private void createImplementation(Action action, Implementation impl) {
		String packagePath  = null;
		if(getSource().size() >= 3 && (getSource().get(getSource().size()-1) instanceof CallBehaviorAction)){
			CallBehaviorAction cba = (CallBehaviorAction)(getSource().get(getSource().size()-1));
			packagePath  = ExportOperation.FILE_SEPARATOR+SAUtil.createPackage(cba,ExportOperation.FILE_SEPARATOR);
		}else{
			packagePath  = ExportOperation.FILE_SEPARATOR+SAUtil.createPackage(action,ExportOperation.FILE_SEPARATOR);
		}
		
		if(impl instanceof AdaptiveEntityImplementation && BpelOptionsUtil.generateDefaultComponentImplementation()){
			// RK D131477 - need to encode the namespace (for DBCS chars)
			// SACLRoot aeRoot = createDefaultAE(ProcessUtil.createNamespace(action));
			SACLRoot aeRoot = null;
			if(getSource().size() >= 3 && (getSource().get(getSource().size()-1) instanceof CallBehaviorAction)){
				CallBehaviorAction cba = (CallBehaviorAction)(getSource().get(getSource().size()-1));
				aeRoot = createDefaultAE(ProcessUtil.createNamespace(cba,true));
			}else{
				aeRoot = createDefaultAE(ProcessUtil.createNamespace(action,true));
			}			 
			getTarget().add(aeRoot);
			updateAdaptiveEntity(aeRoot);
			StateMachineDefinition def = aeRoot.getStateMachineDefinition();
			TAdaptiveEntity tAE = AdaptiveEntityFactory.eINSTANCE.createTAdaptiveEntity();
			tAE.setSacl(packagePath + def.getName() + ExportOperation.SACL_FILE_EXT);
			((AdaptiveEntityImplementation)impl).setAdaptiveEntity(tAE);
			//Process process = createDefaultBPEL();
			TProcess tProcess = ProcessFactory.eINSTANCE.createTProcess();
			((AdaptiveEntityImplementation)impl).setProcess(tProcess);
			Process process = BPELFactory.eINSTANCE.createProcess();
			String processName = createProcessImplementationName(process, action);
//			tProcess.setBpel(packagePath + def.getName() + ExportOperation.BPEL_FILE_EXT);
			tProcess.setBpel(packagePath + processName + ExportOperation.BPEL_FILE_EXT);
			//getTarget().add(process);
		}else if (impl instanceof ProcessImplementation) {
			if (action.eContainer() instanceof Activity) {
				com.ibm.wbit.bpel.Process bpelProcess = (com.ibm.wbit.bpel.Process)ProcessUtil.getProcessDefinition(getContext(),(Activity)action.eContainer());
				if (bpelProcess != null) {
					TProcess tProcess = ProcessFactory.eINSTANCE.createTProcess();
					String actionPath = null;
					NoHashNamingRegistry noHashReg = null;
					if (NamingUtil.USE_PER_PROJECT_UNIQUENESS) {
						String projectName = CrossProjectReferenceUtil.getProjectNameForBOMElement(action);
						noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(getContext(), NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(projectName);
					} else {
						noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(getContext(), NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(NamingUtil.GLOBAL_XPROJECT_UNIQUENESS);
					}
					if(noHashReg.actionExistInGlobalRegistry(action)) {
						actionPath = noHashReg.retrieveNameFromGlobalRegistry(action);
					}
					String path = null;
					if (actionPath != null && actionPath.indexOf(ExportOperation.FILE_SEPARATOR) != -1) {
						path = ExportOperation.FILE_SEPARATOR_CHAR + actionPath.substring(0, actionPath.lastIndexOf(ExportOperation.FILE_SEPARATOR) + 1);
					} else {
					//TODO 20070914 is it possible?	
					}
					 
					if (path != null) {
						tProcess.setBpel(path + bpelProcess.getName() + ExportOperation.BPEL_FILE_EXT);	
					} else {
						tProcess.setBpel(packagePath+bpelProcess.getName()+ExportOperation.BPEL_FILE_EXT);
					}
					// d139424: GJD bpel process has to have the path tProcess.setBpel(bpelProcess.getName()+ExportOperation.BPEL_FILE_EXT);
					((ProcessImplementation)impl).setProcess(tProcess);
				}else if(BpelOptionsUtil.generateDefaultComponentImplementation()){
					Object implementation = null;
					if(getSource().size() >= 3 && (getSource().get(getSource().size()-1) instanceof CallBehaviorAction)){
						CallBehaviorAction cba = (CallBehaviorAction)(getSource().get(getSource().size()-1));
						implementation = avp.getAttributeValueForType(cba,AttributeNameConstants.IMPLEMENTATION);
					}else{
						implementation = avp.getAttributeValueForType(action,AttributeNameConstants.IMPLEMENTATION);
					}					
					if (implementation != null && !implementation.equals(AttributeNameConstants.IMPLEMENTATION) && implementation instanceof com.ibm.btools.te.attributes.model.definition.implementation.wps.Process) {	
						TProcess tProcess = ProcessFactory.eINSTANCE.createTProcess();
						//now create a default BPELProcess... 
						Process process = createDefaultBPEL();
						tProcess.setBpel(packagePath + process.getName() + ExportOperation.BPEL_FILE_EXT);
						((ProcessImplementation)impl).setProcess(tProcess);
						getTarget().add(process);
					}
				}
			}else if (BomUtils.isTask(action) || BomUtils.isReusableTask(action) || BomUtils.isReusableService(action) && BpelOptionsUtil.generateDefaultComponentImplementation()){
				TProcess tProcess = ProcessFactory.eINSTANCE.createTProcess();
				//now create a default BPELProcess... 
				Process process = createDefaultBPEL();
				tProcess.setBpel(packagePath + process.getName() + ExportOperation.BPEL_FILE_EXT);
				((ProcessImplementation)impl).setProcess(tProcess);
				getTarget().add(process);
			}
		} else if(impl instanceof TaskImplementation){
			HumanTaskRule humanTask = (HumanTaskRule)TransformationUtil.getRuleForSource(getRoot(), HumanTaskRule.class, this.getSource(), com.ibm.wbit.tel.DocumentRoot.class);
			if (humanTask==null && BpelOptionsUtil.generateDefaultComponentImplementation()){
				Object implementation = null;
				if(getSource().size() >= 3 && (getSource().get(getSource().size()-1) instanceof CallBehaviorAction)){
					CallBehaviorAction cba = (CallBehaviorAction)(getSource().get(getSource().size()-1));
					implementation = avp.getAttributeValueForType(cba,AttributeNameConstants.IMPLEMENTATION);
				}else{
					implementation = avp.getAttributeValueForType(action,AttributeNameConstants.IMPLEMENTATION);
				}				
				if (implementation != null && !implementation.equals(AttributeNameConstants.IMPLEMENTATION) && implementation instanceof com.ibm.btools.te.attributes.model.definition.implementation.wps.HumanTask) {
					humanTask = AbstractbpelFactory.eINSTANCE.createHumanTaskRule();
					//attach the HumanTask to the Root
					getRoot().getChildRules().add(humanTask);
					humanTask.getSource().addAll(this.getSource());
					humanTask.transformSource2Target();					
				}
			}		
			if (humanTask!=null) {
				com.ibm.wbit.tel.DocumentRoot humanTaskRoot = (com.ibm.wbit.tel.DocumentRoot)humanTask.getTarget().get(0);
				TTask tTask = TaskFactory.eINSTANCE.createTTask();
				tTask.setTel(humanTaskRoot.getTask().getName() + ExportOperation.TEL_FILE_EXT);				
				((TaskImplementation)impl).setTask(tTask);				
			}
		}else if(impl instanceof JavaImplementation && BpelOptionsUtil.generateDefaultComponentImplementation()){
			//TODO check for technical attributes if we have an implementation class.. if yes, use that class and 
			//do not generate an Implementation.. 
			PortType pt = (PortType)getSource().get(1);
			com.ibm.btools.te.ilm.heuristics.scdl.util.JavaImplementation javaImpl = new com.ibm.btools.te.ilm.heuristics.scdl.util.JavaImplementation();
			JavaImplCreationTemplate template = new JavaImplCreationTemplate();
			String javaContent = template.generate(pt, component);
			System.out.println(javaContent);
			javaImpl.setJavaImplementation(javaContent);
			javaImpl.setComponent(component);
			((JavaImplementation)impl).setClass(ScdlUtil.createJavaClassName(component));
			getTarget().add(javaImpl);
		}else if(impl instanceof BusinessRuleGroupImplementation && BpelOptionsUtil.generateDefaultComponentImplementation()){
			if (action instanceof BusinessRuleAction) {
				if (((BusinessRuleGroupImplementation)impl).getBrgFile() == null) {
					String brgPath = null;
					NoHashNamingRegistry noHashReg = null;
					if (NamingUtil.USE_PER_PROJECT_UNIQUENESS) {
						String projectName = CrossProjectReferenceUtil.getProjectNameForBOMElement(action);
						noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(getContext(), NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(projectName);
					} else {
						noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(getContext(), NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(NamingUtil.GLOBAL_XPROJECT_UNIQUENESS);
					}
					Object contextContainer = NamingUtil.getContextContainer(action);
	
					if (contextContainer instanceof StructuredActivityNode && noHashReg.actionExistInLocalRegistry(action, contextContainer)) {
						brgPath = noHashReg.retrieveNameFromLocalRegistry(action, contextContainer);
					} else if(noHashReg.actionExistInGlobalRegistry(action)) {
						brgPath = noHashReg.retrieveNameFromGlobalRegistry(action);
					} else {
						//register action
						brgPath = NamingUtil.convertNameForAction(action, getContext()); // name from PortType of TA
					}
					if (brgPath != null) {
						((BusinessRuleGroupImplementation)impl).setBrgFile(ExportOperation.FILE_SEPARATOR + brgPath + ExportOperation.BRG_FILE_EXT);
					}
				}
			} else {				
				BusinessRuleGroup businessRuleGroup = createDefaultBusinessRuleGroup();
				getTarget().add(businessRuleGroup);
				((BusinessRuleGroupImplementation)impl).setBrgFile(packagePath + businessRuleGroup.getName() + ExportOperation.BRG_FILE_EXT);
			}
		}
		else  if(impl instanceof com.ibm.ws.fabric.da.scdl.DynamicAssembler){
			com.ibm.ws.fabric.da.scdl.DynamicAssembler daImpl = (com.ibm.ws.fabric.da.scdl.DynamicAssembler)impl;
			daImpl.setConfigurationFile(component.getName()+ ExportOperationConstants.DA_CONFIGURATION_EXT);
			daImpl.setComponent(component);			
		}else if(impl instanceof MediationFlowImplementation){
			PortType pt = (PortType)getSource().get(1);
			MediationFlowImplementation mfi = (MediationFlowImplementation)impl;
			if(mfi.getMfcFile() != null) return;
			mfi.setMfcFile(component.getName() + ExportOperation.MFC_FILE_EXT);
			mfi.setComponent(component);
			//defect 84336
			Action medFlowAction = null;
			if(getParentRule() != null && getParentRule() instanceof CallBehaviorActionRule){
				medFlowAction  = (Action)((InputPinSet)getParentRule().getSource().get(0)).getAction();
			}else{
				medFlowAction = action;
			}
			MediationFlowGenerator generator = new MediationFlowGenerator(component, medFlowAction, pt);
			com.ibm.wbit.sib.mediation.operation.model.DocumentRoot medFlow = generator.createInterfaceMediationFlow();
			getTarget().add(medFlow);
			EObject flowNodePackage = generator.generateMediationFlow();
			getTarget().add(flowNodePackage);
			com.ibm.wsspi.sca.scdl.Reference reference = ScdlUtil.createSCDLComponentReference(component.getDocumentRoot(), null, pt, ScdlUtil.MEDIATION_FLOW_REFERENCE_NAME, false);
			CrossProjectReferenceUtil.registerTranformTargetWithProjectName(action, medFlow);
			CrossProjectReferenceUtil.registerTranformTargetWithProjectName(action, flowNodePackage);
		}
	}



	/**
	 * Updates the component from the adaptive entity.
	 * @param aeRoot
	 * @return
	 */
	private StateMachineDefinition updateAdaptiveEntity(SACLRoot aeRoot){

		StateMachineDefinition def = aeRoot.getStateMachineDefinition();
		// Create the interfaces
		if (component.getInterfaceSet() != null) {
			// Ensure qualifier present
			List list = component.getInterfaceSet().getInterfaceQualifiers();
			JoinTransaction joinTransaction = null;
			for (int i = 0; i < list.size(); i++) {
				if (list.get(i) instanceof JoinTransaction) {
					joinTransaction = (JoinTransaction)list.get(i);
					break;
				}
			}
			if (joinTransaction == null) {
				joinTransaction = SCDLFactory.eINSTANCE.createJoinTransaction(); 
				list.add(joinTransaction);
			}
			joinTransaction.setValue(Boolean.FALSE); 
			component.getInterfaceSet().getInterfaceQualifiers().add(joinTransaction);
			List interfaces = component.getInterfaceSet().getInterfaces();
			if (interfaces.size() > 0) {
				com.ibm.wbit.ae.sacl.InterfaceSet interfaceSet = def.getInterfaces();
				for (Iterator interfacesIter=interfaces.iterator(); interfacesIter.hasNext();) {
					com.ibm.wsspi.sca.scdl.Interface scdlInterf = (com.ibm.wsspi.sca.scdl.Interface) interfacesIter.next();
					//Assert.isTrue(scdlInterf instanceof com.ibm.wsspi.sca.scdl.wsdl.WSDLPortType);
					scdlInterf.setPreferredInteractionStyle(InteractionStyle.ASYNC_LITERAL);
					com.ibm.wbit.ae.sacl.WSDLPortType portType = SACLFactory.eINSTANCE.createWSDLPortType();
					portType.setPortTypeQName(((com.ibm.wsspi.sca.scdl.wsdl.WSDLPortType)scdlInterf).getPortType());
					interfaceSet.getInterface().add(portType);
				}
			}
		}

		// Create the references
		if (component.getReferenceSet() != null) {
			List references = component.getReferenceSet().getReferences();
			if (references.size() > 0) {
				com.ibm.wbit.ae.sacl.ReferenceSet referenceSet = def.getReferences();
				for (Iterator referencesIter=references.iterator(); referencesIter.hasNext();) {
					com.ibm.wsspi.sca.scdl.Reference scdlReference = (com.ibm.wsspi.sca.scdl.Reference) referencesIter.next();
					Reference reference = SACLFactory.eINSTANCE.createReference();
					reference.setName(scdlReference.getName());
					reference.setDescription(scdlReference.getDescription());
					List scdlRefInterfaces = scdlReference.getInterfaces();
					for (Iterator interfacesIter=scdlRefInterfaces.iterator(); interfacesIter.hasNext();) {
						com.ibm.wsspi.sca.scdl.Interface scdlInterf = (com.ibm.wsspi.sca.scdl.Interface) interfacesIter.next();
						//Assert.isTrue(scdlInterf instanceof com.ibm.wsspi.sca.scdl.wsdl.WSDLPortType);
						com.ibm.wbit.ae.sacl.WSDLPortType portType = SACLFactory.eINSTANCE.createWSDLPortType();
						portType.setPortTypeQName(((com.ibm.wsspi.sca.scdl.wsdl.WSDLPortType)scdlInterf).getPortType());
//						GJD20060911						reference.getInterface().add(portType);
						reference.setInterface(portType);						
					}
					referenceSet.getReference().add(reference);
				}
			}
		}
		return def;

	}
	/**
	 * Creates an empty Adaptive Entity.
	 */
	private SACLRoot createEmptyAE(String tns) {

		String baseName = component.getName().substring(component.getName().lastIndexOf("/")+1);
		// Create the root
		SACLRoot root = SACLFactory.eINSTANCE.createSACLRoot();

		// Create the state machine definition
		StateMachineDefinition stateMachineDef = SACLFactory.eINSTANCE.createStateMachineDefinition();
		stateMachineDef.setName(baseName);
		//stateMachineDef.setDisplayName(baseName);
		stateMachineDef.setDisplayName(component.getDisplayName());
		root.setStateMachineDefinition(stateMachineDef);
		// Set the target namespace
		stateMachineDef.setTargetNamespace(tns);
		// Put target namespace in the prefix map
		root.getXMLNSPrefixMap().put("tns", tns); // TODO fix this
		//now creates the required SACL Objects
		// Create the main state machine
		if (stateMachineDef.getMainStateMachine() == null) {
			stateMachineDef.setMainStateMachine(SACLFactory.eINSTANCE.createStateMachine());
		}
		// Create the correlation set
		if (stateMachineDef.getCorrelationSet() == null) {
			CorrelationSet correlation = SACLFactory.eINSTANCE.createCorrelationSet();
			correlation.setName("defaultCorrelationSet"); //$NON-NLS-1$
			correlation.setProperties(new ArrayList());
			stateMachineDef.setCorrelationSet(correlation);
		}
		// Create the reference set
		if (stateMachineDef.getReferences() == null) {
			stateMachineDef.setReferences(SACLFactory.eINSTANCE.createReferenceSet());
		}
		// Create the interface set
		if (stateMachineDef.getInterfaces() == null) {
			stateMachineDef.setInterfaces(SACLFactory.eINSTANCE.createInterfaceSet());
		}
		// Create the variables
		if (stateMachineDef.getVariables() == null) {
			stateMachineDef.setVariables(SACLFactory.eINSTANCE.createVariables());
		}
		return root;
	}

	/**
	 * Creates an adaptive entity that works, with an intial state, a simple state, a final state and
	 * transitions.
	 */
	private SACLRoot createDefaultAE(String tns) {
		SACLRoot root = createEmptyAE(tns);
		StateMachineDefinition stateMachineDef = (StateMachineDefinition) root.getStateMachineDefinition();
		StateMachine mainMachine = stateMachineDef.getMainStateMachine();

		// Create an intial state
		InitialState initialState = SACLFactory.eINSTANCE.createInitialState();
		initialState.setName("InitialState1"); //$NON-NLS-1$
		initialState.setDisplayName("InitialState1"); //$NON-NLS-1$
		mainMachine.setInitialState(initialState);

		// Create a simple state
		State state = SACLFactory.eINSTANCE.createState();
		state.setName("State1"); //$NON-NLS-1$
		state.setDisplayName("State1"); //$NON-NLS-1$
		mainMachine.getStates().add(state);

		// Create a final state
		FinalState finalState = SACLFactory.eINSTANCE.createFinalState();
		finalState.setName("FinalState1"); //$NON-NLS-1$
		finalState.setDisplayName("FinalState1"); //$NON-NLS-1$
		mainMachine.getFinalStates().add(finalState);

		// Create a transition from the initial state to the simple state
		Transition transition = SACLFactory.eINSTANCE.createTransition();
		transition.setName("Transition1"); //$NON-NLS-1$
		transition.setOperation(SACLFactory.eINSTANCE.createOperation());
		transition.setSourceState(initialState);
		transition.setTargetState(state);
		mainMachine.getTransitions().add(transition);
		//	ExtensionUtils.getExtension(map, transition);

		// Create an automatic transition from the simple state to the final state
		transition = SACLFactory.eINSTANCE.createTransition();
		transition.setName("Transition2"); //$NON-NLS-1$
		transition.setAutomatic(SACLFactory.eINSTANCE.createAutomatic());
		transition.setSourceState(state);
		transition.setTargetState(finalState);
		mainMachine.getTransitions().add(transition);
		//	ExtensionUtils.getExtension(map, transition);

		return root;
	}

	/**
	 * Creates a default BPELProcess as per WID sample. 
	 * @return BPELProcess
	 */
	private Process createDefaultBPEL() {

		Action action = (Action)getSource().get(0);
		PortType portType = (PortType)getSource().get(1);
		Definition wsdlDefinition = portType.getEnclosingDefinition();
		String processTargetNamespace = null;
		//Handle business service
		if(wsdlDefinition.getTargetNamespace().lastIndexOf("Interface") != -1){
			processTargetNamespace = wsdlDefinition.getTargetNamespace().substring(0, wsdlDefinition.getTargetNamespace().lastIndexOf("Interface"));
		}else{
			processTargetNamespace = wsdlDefinition.getTargetNamespace()+"Bpel";
		}

		// Fields to populate that we'll use later
		Message messageIn;
		Message messageOut = null;
		Operation op;
		// Whether or not the operation has output. This will be used to calculate
		// whether to generate a Reply activity in the process.
		boolean hasOutput = false;

		//BPEL Process name should be identical with component name 
		//String baseName = component.getName().substring(component.getName().lastIndexOf("/")+1);
		//String capitalizedBaseName = Character.toUpperCase(baseName.charAt(0)) + baseName.substring(1);
		//Ray - ensure the BPEL Process name aligns with the Artifact wsdl name (which also aligns with the WSDL Definition name)
		//business service case will break
		String convertedActionName = null;//wsdlDefinition.getQName().getLocalPart().substring(0,wsdlDefinition.getQName().getLocalPart().lastIndexOf("Interface"));
		
		op = (Operation)portType.getOperations().get(0);
		hasOutput = op.getOutput() != null;

		messageIn = (Message)op.getInput().getMessage();
		if (hasOutput) {
			messageOut = (Message)op.getOutput().getMessage();
		}

		// Create the BPEL artifacts for the Artifacts WSDL file
		Definition artifactDefn = WSDLFactory.eINSTANCE.createDefinition();
		String artifactTargetNamespace = null;
		if(wsdlDefinition.getTargetNamespace().lastIndexOf("Interface") != -1){
			artifactTargetNamespace = wsdlDefinition.getTargetNamespace().substring(0, wsdlDefinition.getTargetNamespace().lastIndexOf("Interface")) + "Artifacts";
		}else{
			artifactTargetNamespace = wsdlDefinition.getTargetNamespace()+ "Artifacts";
		}
//		String artifactTargetNamespace = wsdlDefinition.getTargetNamespace().substring(0, wsdlDefinition.getTargetNamespace().lastIndexOf("Interface")) + "Artifacts";
		artifactDefn.setTargetNamespace(artifactTargetNamespace);

//		artifactDefn.setQName(new javax.xml.namespace.QName(artifactTargetNamespace,
//				wsdlDefinition.getQName().getLocalPart().substring(0, wsdlDefinition.getQName().getLocalPart().lastIndexOf("Interface")) + "Artifacts"));

		artifactDefn.addNamespace("tns", artifactTargetNamespace); //$NON-NLS-1$
		artifactDefn.addNamespace("xsd", XSDConstants.SCHEMA_FOR_SCHEMA_URI_2001); //$NON-NLS-1$

		PartnerLinkType plt = PartnerlinktypeFactory.eINSTANCE.createPartnerLinkType();
		JavaNCNameConverter converter = new JavaNCNameConverter();
		//Defect 111813
		String pltBase = converter.convertName(action.getName());
		String pltName = pltBase + "PLT";
		plt.setName(pltName); //$NON-NLS-1$
		//	plt.setName(action.getName() + "PartnerLink"); //$NON-NLS-1$
		artifactDefn.addExtensibilityElement(plt);

		Role role = PartnerlinktypeFactory.eINSTANCE.createRole();
		role.setName(pltBase + "Service"); //$NON-NLS-1$
		//role.setName(Messages.getString("NewProcessWizard.ProcessRole_0")); //$NON-NLS-1$
		plt.getRole().add(role);

		RolePortType rpt = PartnerlinktypeFactory.eINSTANCE.createRolePortType();
		rpt.setName(portType);
		role.setPortType(rpt);
		getTarget().add(artifactDefn);

		// Add an import from the Utility WSDL to the Interface WSDL 
		Import wsdlImport = WSDLFactory.eINSTANCE.createImport();
		if (wsdlDefinition.getLocation() != null && ExportOperationUtil.isExternalFile(wsdlDefinition.getLocation())) {
			wsdlImport.setLocationURI(ExportOperationUtil.replaceConvertedSpaces(wsdlDefinition.getLocation()));
		}else{
			wsdlImport.setLocationURI(wsdlDefinition.getQName().getLocalPart()+ExportOperation.WSDL_FILE_EXT);
		}
		wsdlImport.setDefinition(wsdlDefinition);
		wsdlImport.setNamespaceURI(wsdlDefinition.getTargetNamespace());
		if(!artifactDefn.getImports().containsKey(wsdlDefinition.getTargetNamespace())){
			artifactDefn.addImport(wsdlImport);	
		}
		String prefix = DataDefinitionUtil.getPrefixFromNamespaceURI(wsdlDefinition.getTargetNamespace());
		if (artifactDefn.getNamespace(prefix) == null){
			//register the prefix for the namespace
			artifactDefn.addNamespace( prefix, wsdlDefinition.getTargetNamespace());
		}

		//WSDLImportHelper.addImportAndNamespace(artifactDefn, portType.getEnclosingDefinition());

		int id = 0;
		BPELFactory bpelFactory = BPELFactory.eINSTANCE;

		Process process = bpelFactory.createProcess();
		organizeImports(process, wsdlDefinition);
		//		bpelResource.getContents().add(process);
		// process.setName(baseName);
		//Create unique name for the default process BPEL to represent the implementation
		convertedActionName = createProcessImplementationName(process, action);
		process.setName(convertedActionName);
		//setDisplayName(process, baseName);
		setDisplayName(process, action.getName());
		process.setTargetNamespace(processTargetNamespace);
		setId(process, ++id);
		//		extensionMap.put(process, processExtension);
		//setExecutionMode(process, "microflow"); //$NON-NLS-1$
		process.setSuppressJoinFailure(Boolean.TRUE);
		//setExpressionLanguage(process, IBPELUIConstants.EXPRESSION_LANGUAGE_JAVA);
		process.setPartnerLinks(bpelFactory.createPartnerLinks());


		PartnerLink partner = bpelFactory.createPartnerLink();
		//partner.setName(Messages.getString("NewProcessWizard.Client_0")); //$NON-NLS-1$
		partner.setName("Partner"); //$NON-NLS-1$
		partner.setPartnerLinkType(plt);
		partner.setMyRole(role);
		process.getPartnerLinks().getChildren().add(partner);

		Sequence sequence = BPELFactory.eINSTANCE.createSequence();
		process.setActivity(sequence);
		//		setId(sequence, (++id | 0x40000000));

		Receive receive = bpelFactory.createReceive();
		receive.setName("Receive"); //$NON-NLS-1$
		setDisplayName(receive, "Receive"); //$NON-NLS-1$

		receive.setPartnerLink(partner);
		receive.setPortType(portType);
		receive.setOperation(op);
		receive.setCreateInstance(Boolean.TRUE);
		setId(receive, ++id);
		sequence.getActivities().add(receive);

		// In the non-spec-compliant case we'll use wpc:input and wpc:output.
		// Call the helper to get the parts for each
		List inputElements = null;
		List outputElements = null;
		inputElements = ScdlUtil.getTypeDeclarationElements(op, ScdlUtil.INPUT_VARIABLE_TYPE);
		outputElements = ScdlUtil.getTypeDeclarationElements(op, ScdlUtil.OUTPUT_VARIABLE_TYPE);

		// In the spec-compliant case, create a Message-typed variable
		// and assign it to the receive.
		BPELVariable variableIn = null;
		BPELVariable variableOut = null;
		List variables = new LinkedList();
		Map inputVariables = new HashMap(5);
		if (inputElements != null) {
			Iterator it = inputElements.iterator();
			com.ibm.wbit.bpelpp.Output output = BPELPlusFactory.eINSTANCE.createOutput();
			while (it.hasNext()) {
				Object next = it.next();
				BPELVariable variable = bpelFactory.createBPELVariable();
				String paramName = null;
				if (next instanceof Part) {
					org.eclipse.wst.wsdl.Part part = (org.eclipse.wst.wsdl.Part)next;
					paramName = part.getName(); // paramName has to match the part
					if (part.getTypeDefinition() != null) {
						variable.setType(part.getTypeDefinition());
					} else if (part.getElementDeclaration() != null) {
						variable.setXSDElement(part.getElementDeclaration());
					}
				} else if (next instanceof XSDElementDeclaration) {
					// GJD 20060828
					Object obj = resolveXSDObject(next, getContext());
					if (obj == null) {
						String params[] = {((XSDElementDeclaration)next).getName()};
						LoggingUtil.logError(MessageKeys.ERROR_RETRIEVING_ORIGINAL_XSD_DEFINITION, params, null);
						BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_RETRIEVING_ORIGINAL_XSD_DEFINITION));
						throw runtimeEx;
					}
					if (obj instanceof XSDElementDeclaration) {
						XSDElementDeclaration element = (XSDElementDeclaration) obj;
						paramName = element.getName(); // paramName has to match the element
						variable.setName(element.getName() + "Variable");
						XSDTypeDefinition type = element.getTypeDefinition();
						if (type == null) {
							String params[] = {((XSDElementDeclaration)next).getName()};
							LoggingUtil.logError(MessageKeys.ERROR_RETRIEVING_ORIGINAL_XSD_DEFINITION, params, null);
							BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_RETRIEVING_ORIGINAL_XSD_DEFINITION));
							throw runtimeEx;							
						}
						if (type.getName() == null) {
							variable.setXSDElement(element);
						} else {
							variable.setType(type);				        
						}
					} else if (obj instanceof XSDAttributeDeclaration) {
						XSDAttributeDeclaration attribute = (XSDAttributeDeclaration) obj;
						paramName = attribute.getName(); // paramName has to match the element
						variable.setName(attribute.getName() + "Variable");
						variable.setType(attribute.getTypeDefinition());				        
					}
				}

				//process.getVariables().getChildren().add(variable);
				variables.add(variable);
				//add the Variable and its type to the inputVariablesMap. try to reuse the variable on the output
				inputVariables.put(variable, variable.getType());
				organizeImports(process, variable);
				Parameter parameter = BPELPlusFactory.eINSTANCE.createParameter();
				parameter.setName(paramName);
				parameter.setVariable(variable);
				output.getParameter().add(parameter);
			}
			receive.getEExtensibilityElements().add(output);
		}
		//		}

		if (hasOutput) {	
			// Only generate an assign if we're not creating from an
			// existing interface, because otherwise we have no way to
			// intelligently copy input to output
			Reply reply = bpelFactory.createReply();
			reply.setName("Reply"); //$NON-NLS-1$
			setDisplayName(reply, "Reply"); //$NON-NLS-1$
			reply.setPartnerLink(partner);
			reply.setPortType(portType);
			reply.setOperation(op);
			setId(reply, ++id);
			sequence.getActivities().add(reply);
			if (outputElements != null) {
				List outputVariables = new LinkedList();
				BPELVariable variable  = null;
				Iterator it = outputElements.iterator();
				com.ibm.wbit.bpelpp.Input input = BPELPlusFactory.eINSTANCE.createInput();
				while (it.hasNext()) {
					Object next = it.next();
					String paramName = null;
					if (next instanceof Part) {
						variable = bpelFactory.createBPELVariable();
						Part part = (Part)next;
						paramName = part.getName(); // paramName has to match the part
						//	String name = BPELUtil.getUniqueModelName(process, paramName, Collections.singletonList(variable));
						//	variable.setName(name);
						if (part.getTypeDefinition() != null) {
							variable.setType(part.getTypeDefinition());
						} else if (part.getElementDeclaration() != null) {
							variable.setXSDElement(part.getElementDeclaration());
						}
					} else if (next instanceof XSDElementDeclaration) {
						// GJD 20060828
						Object obj = resolveXSDObject(next, getContext());
						if (obj == null) {
							String params[] = {((XSDElementDeclaration)next).getName()};
							LoggingUtil.logError(MessageKeys.ERROR_RETRIEVING_ORIGINAL_XSD_DEFINITION, params, null);
							BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_RETRIEVING_ORIGINAL_XSD_DEFINITION));
							throw runtimeEx;
						}
						if (obj instanceof XSDElementDeclaration) {
							XSDElementDeclaration element = (XSDElementDeclaration) obj;
							paramName = element.getName(); // paramName has to match the element
							XSDTypeDefinition type = element.getTypeDefinition();
							variable = getKeyForValue(inputVariables, type, outputVariables);
							if(variable == null){
								variable = bpelFactory.createBPELVariable();
								variable.setName(element.getName() + "Variable");
								if (element.getTypeDefinition() == null || element.getTypeDefinition().getName() == null) {
									variable.setXSDElement(element);
								} else {
									variable.setType(element.getTypeDefinition());
								}
								outputVariables.add(variable);
							}
						} else if (obj instanceof XSDAttributeDeclaration) {
							XSDAttributeDeclaration attribute = (XSDAttributeDeclaration) obj;
							paramName = attribute.getName(); // paramName has to match the element
							XSDTypeDefinition type = attribute.getTypeDefinition();
							variable = getKeyForValue(inputVariables, type, outputVariables);
							if(variable == null){
								variable = bpelFactory.createBPELVariable();
								variable.setName(attribute.getName() + "Variable");
								variable.setType(attribute.getTypeDefinition());
								outputVariables.add(variable);
							}
						}
					}
					//	process.getVariables().getChildren().add(variable);
					variables.add(variable);
					organizeImports(process, variable);
					Parameter parameter = BPELPlusFactory.eINSTANCE.createParameter();
					parameter.setName(paramName);
					parameter.setVariable(variable);
					input.getParameter().add(parameter);
				}
				reply.getEExtensibilityElements().add(input);
			}

			if(variables != null && !variables.isEmpty()){
				process.setVariables(bpelFactory.createVariables());
				process.getVariables().getChildren().addAll(variables);
			}

		}
		
		//Synchronize the process name with the artifacts name
		artifactDefn.setQName(new javax.xml.namespace.QName(artifactTargetNamespace, process.getName() + "Artifacts"));
		
		//Explictly add the artifacts definition import
		com.ibm.wbit.bpel.Import artifactsImport = BPELFactory.eINSTANCE.createImport();
		artifactsImport.setLocation(process.getName() + ExportOperation.ARTIFACTS_SUFFIX + ExportOperation.WSDL_FILE_EXT);
		artifactsImport.setImportType(AbstractbpelConstants.WSDL_IMPORT_TYPE);
		artifactsImport.setNamespace(artifactTargetNamespace);
		if(process != null){
			process.getImports().add(artifactsImport);
		}

		return process;	
	}

	private BPELVariable getKeyForValue(Map variables, XSDTypeDefinition type, List outputVariables){
		if(outputVariables == null) outputVariables = new LinkedList();
		BPELVariable variable = null;
		if(variables.containsValue(type)){
			Set keys = variables.keySet();
			Iterator it = keys.iterator();
			while(it.hasNext()){
				Object thisVariable = it.next();
				if(variables.get(thisVariable).equals(type) && !outputVariables.contains(thisVariable)){
					variable = (BPELVariable)thisVariable;
					outputVariables.add(variable);
					break;
				}
			}
		}
		return variable;

	}
	/**
	 * Creates and sets an ID on the <code>element</code>
	 * @param element a BPELP ExtensibleElement
	 * @param id an unique id
	 */	
	protected void setId(ExtensibleElement element, int id) {
		if (element == null){
			return;
		}
		Id newId = BPELPlusFactory.eINSTANCE.createId();
		newId.setId(new Integer(id));
		element.getEExtensibilityElements().add(newId);
	}

	/**
	 * Creates a default BusinessRuleGroup as per WID.
	 */
	private BusinessRuleGroup createDefaultBusinessRuleGroup(){
		Action action = (Action)getSource().get(0);
		String baseName = component.getName().substring(component.getName().lastIndexOf("/")+1);
		//create the BusinessRuleGroup  and set it as target
		BusinessRuleGroup businessRuleGroup = BrgFactory.eINSTANCE.createBusinessRuleGroup();
//		businessRuleGroup.setProperty(BRExpressionConstants.PROPERTY_IBM_SYSTEM_VERSION, BRExpressionConstants.PROPERTY_SYSTEM_DEFINED, BRExpressionConstants.PROPERTY_VALUE);

		businessRuleGroup.setName(baseName);
		//businessRuleGroup.setTargetNameSpace(ProcessUtil.createNamespace(action));
		// Defect 124470
		businessRuleGroup.setTargetNameSpace(ProcessUtil.createNamespace(action,true));
		businessRuleGroup.setPresentationTimezone("local");

		WSDL wsdl = ComponentdefFactory.eINSTANCE.createWSDL();
		businessRuleGroup.setWSDL(wsdl);

//		Create the ref group.
		ReferenceGroup refGroup = BrgFactory.eINSTANCE.createReferenceGroup();
		businessRuleGroup.setReferenceGroup(refGroup);


		BusinessRuleGroupTable selectionTables = BrgtFactory.eINSTANCE.createBusinessRuleGroupTable();
		//selectionTables.setTargetNameSpace(ProcessUtil.createNamespace(action));
		//	Defect 124470
		selectionTables.setTargetNameSpace(ProcessUtil.createNamespace(action,true));

		selectionTables.setName(baseName);
		Object qName = XMLTypeUtil.createQName(selectionTables.getTargetNameSpace(), selectionTables.getName(), "selt"); //$NON-NLS-1$
		businessRuleGroup.setBusinessRuleGroupTableName(qName);
		businessRuleGroup.setBusinessRuleGroupTable(selectionTables);
		PortType portType = (PortType)getSource().get(1);
		addPortTypeToComponent(businessRuleGroup, selectionTables, portType);
		return businessRuleGroup;


	}

	private void addPortTypeToComponent(ComponentDef def, SelectionTables table, PortType port) {
		// Create the interface.
		com.ibm.wbiservers.common.componentdef.WSDLPortType pt = ComponentdefFactory.eINSTANCE.createWSDLPortType();
		javax.xml.namespace.QName ptQName = port.getQName();
		pt.setName(XMLTypeUtil.createQName(ptQName.getNamespaceURI(),ptQName.getLocalPart(),null));
		Iterator operations = port.getOperations().iterator();
		// Create the opdefs.
		while (operations.hasNext()) {
			Operation operation = (Operation) operations.next();
			//addOperationToComponent
			OperationDef operationDef = ComponentdefFactory.eINSTANCE.createOperationDef();
			operationDef.setOperationName(operation.getName());
			operationDef.setSelector(RuleConstants.GENERIC_SELECTOR_ID);
			//TODO from SelectorUtil.. check if we can import
			//String CURRENT_DATE_TAG = Utils.GEN_TAG + "com.ibm.wbit.selector.ui.CurrentDate"; //$NON-NLS-1$
			String CURRENT_DATE_TAG = "//@generated:com.ibm.wbit.selector.ui.CurrentDate"; //$NON-NLS-1$
			String CURRENT_TIME_JAVACODE = CURRENT_DATE_TAG + "\r\nreturn new java.util.Date();"; //$NON-NLS-1$
			CodeParameterDef javaParam = ComponentdefFactory.eINSTANCE.createCodeParameterDef();
			javaParam.setCode(CURRENT_TIME_JAVACODE);
			operationDef.getParameterDefs().add(javaParam);
			def.getOperationDefs().add(operationDef);

			//addOperationToTables
			OperationSelectionTable operationSelectionTable = SelectiontablesFactory.eINSTANCE.createOperationSelectionTable();
			operationSelectionTable.setOperationName(operation.getName());		
			table.getOperationSelectionTables().add(operationSelectionTable);
		}
		def.getWSDL().getPortTypes().add(pt);
	}

	/**
	 * Sets the <code>displayName</code> value for the <code>element</code>
	 * @param element
	 * @param displayName
	 */
	private void setDisplayName(ExtensibleElement element, String displayName) {
		if (element == null) return;
		DisplayName dn = BPELPlusFactory.eINSTANCE.createDisplayName();
		dn.setText(displayName);
		element.getEExtensibilityElements().add(dn);

	}
	/**
	 * Returns the resolved xsdObject. It can be either an XSDElementDeclaration or a XSDAttributeDeclaration.
	 * @param xsdObject an XSD Element
	 * @param context TransformationContext
	 * @return Object the resolved object
	 */
	private  Object resolveXSDObject(Object xsdObject, TransformationContext context) {
		if (xsdObject instanceof XSDElementDeclaration) {
			//TODO GJD 20061118: d140600 get the element for references (anonymous types)
			XSDElementDeclaration xsdObjectElement = (XSDElementDeclaration) xsdObject;
			XSDSchema xsd = xsdObjectElement.getSchema();

			XSDElementDeclaration resolvedElement = xsdObjectElement.getResolvedElementDeclaration();
			if (resolvedElement != null) {
				xsdObject = resolvedElement;
				if (resolvedElement.getType() == null) {
					// find the schema, and element declaration
					XSDElementDeclaration xsdNoType = OriginalXsdUtil.getElementFromImported(context, xsd, resolvedElement.getTargetNamespace(), resolvedElement.getName());
					if (xsdNoType != null) {
						xsdObject = xsdNoType;
					} else {
						xsdObject = resolvedElement;
					}
				}
			}
		} else if (xsdObject instanceof XSDAttributeDeclaration) {
			XSDAttributeDeclaration resolvedAttribute = ((XSDAttributeDeclaration)xsdObject).getResolvedAttributeDeclaration();
			if (resolvedAttribute != null) xsdObject = resolvedAttribute;
		}
		return xsdObject;
	}


	/**
	 * Adds the <code>variable</code> namespace to the <code>process</code>
	 * @param process a BPEL Process
	 * @param variable a BPEL variable
	 */
	private void organizeImports(Process process, BPELVariable variable){

		XSDSchema xsdSchema = null;
		if (variable.getType() == null && variable.getXSDElement() != null) {
			xsdSchema = variable.getXSDElement().getSchema();
		} else {
			xsdSchema = variable.getType().getSchema();
		}
		String xsdNamespace = xsdSchema.getTargetNamespace();

		if(!xsdNamespace.equals(XSDConstants.SCHEMA_FOR_SCHEMA_URI_2001) && !containsImport(process,xsdNamespace)){
			//we need to import the schema 
			com.ibm.wbit.bpel.Import xsdImport = BPELFactory.eINSTANCE.createImport();
			if (xsdSchema != null && xsdSchema.getSchemaLocation() != null && ExportOperationUtil.isExternalFile(xsdSchema.getSchemaLocation())) {
				String schemaLocation = null;			
				schemaLocation = xsdSchema.getSchemaLocation();
				xsdImport.setLocation(schemaLocation);
			} else {	// original code
				xsdImport.setLocation(ProcessUtil.createFileName(xsdNamespace)+ ExportOperation.XSD_FILE_EXT);
			}
			xsdImport.setNamespace(xsdNamespace);
			xsdImport.setImportType(AbstractbpelConstants.XSD_IMPORT_TYPE);
			if (!xsdImport.getLocation().equals(ExportOperation.XSD_FILE_EXT)) {
				process.getImports().add(xsdImport);
			}
		}		
	}

	/**
	 * Checks is the Process already contains a wsdl import for the <code>targetNamespace</code> 
	 * @param targetNamespace 
	 * @return
	 */
	private boolean containsImport(Process process, String targetNamespace){

		List imports = process.getImports();
		if(imports != null && !imports.isEmpty()){
			Iterator it = imports.iterator();
			while(it.hasNext()){
				com.ibm.wbit.bpel.Import thisImport = (com.ibm.wbit.bpel.Import)it.next();
				if(thisImport.getImportType() != null && thisImport.getImportType().equals(AbstractbpelConstants.XSD_IMPORT_TYPE) && thisImport.getNamespace().equals(targetNamespace)){
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Adds the import for the <code>portType</code> namespace, if not the same as the
	 * WSDL <code>definition</code> namespace.
	 * @param definition WSDL Definition
	 * @param portType WSDL PortType
	 */	
	private void organizeImports(Process processDef, Definition toBeImportedDef){

		String namespace = toBeImportedDef.getTargetNamespace();
		if(!namespace.equals(processDef.getTargetNamespace())){
			//should add wsdl import.. leave it for the one that is saving the resource
			if(!containsImport(processDef, toBeImportedDef)){
				//we need to import the schema 
				com.ibm.wbit.bpel.Import wsdlImport = BPELFactory.eINSTANCE.createImport();
				if (toBeImportedDef.getLocation() != null && ExportOperationUtil.isExternalFile(toBeImportedDef.getLocation())) {
					wsdlImport.setLocation(ExportOperationUtil.replaceConvertedSpaces(toBeImportedDef.getLocation()));
				} else {
					wsdlImport.setLocation(toBeImportedDef.getQName().getLocalPart()+ExportOperation.WSDL_FILE_EXT);
				}
				wsdlImport.setNamespace(namespace);
				wsdlImport.setImportType(AbstractbpelConstants.WSDL_IMPORT_TYPE);
				if (!wsdlImport.getLocation().equals(ExportOperation.WSDL_FILE_EXT)) {
					processDef.getImports().add(wsdlImport);
				}
			}	
		}
	}
	/**
	 * Checks is the Process already contains a itel import for the <code>targetNamespace</code> 
	 * @param targetNamespace 
	 * @return
	 */
	private boolean containsImport(Process process, Definition toBeImportedDef){

		String targetNamespace = toBeImportedDef.getTargetNamespace();
		String location  = toBeImportedDef.getQName().getLocalPart()+ExportOperation.WSDL_FILE_EXT;
		List imports = process.getImports();
		if(imports != null && !imports.isEmpty()){
			Iterator it = imports.iterator();
			while(it.hasNext()){
				com.ibm.wbit.bpel.Import thisImport = (com.ibm.wbit.bpel.Import)it.next();
				if(thisImport.getImportType() != null && thisImport.getImportType().equals(AbstractbpelConstants.WSDL_IMPORT_TYPE) && thisImport.getNamespace().equals(targetNamespace)
						&& thisImport.getLocation().equals(location)){
					return true;
				}
			}
		}
		return false;
	}
	/**
	 * For all components, the default PreferredInteractionStyle is ASYNCH, except for the Component with JavaImplementation 
	 * for which it should be SYNC. Updates after the fact the PreferredInteraction Style to ANY.
	 * @param component
	 */
	private void updateComponentInteractionStyle(Component component){
		if(component.getImplementation() != null && component.getImplementation() instanceof JavaImplementation){
			List interfaces = component.getInterfaceSet().getInterfaces();
			if(interfaces != null && !interfaces.isEmpty()){
				Iterator it = interfaces.iterator();
				WSDLPortType portType = (WSDLPortType)it.next();
				//portType.setPreferredInteractionStyle(InteractionStyle.ANY_LITERAL);
				portType.setPreferredInteractionStyle(InteractionStyle.SYNC_LITERAL);
			}
		}else if(component.getImplementation()!= null && component.getImplementation() instanceof ProcessImplementation){
			Action action = (Action)getSource().get(0);
			AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.BPEL_PROCESS_IS_LONG_RUNNING);
			valueProvider.setContext(this.getContext());
			Boolean longRunningProcess = (Boolean)valueProvider.getAttributeValueForType(action, AttributeNameConstants.BPEL_PROCESS_IS_LONG_RUNNING);
			if(!longRunningProcess.booleanValue()){
				List interfaces = component.getInterfaceSet().getInterfaces();
				if(interfaces != null && !interfaces.isEmpty()){
					Iterator it = interfaces.iterator();
					WSDLPortType portType = (WSDLPortType)it.next();
					portType.setPreferredInteractionStyle(InteractionStyle.SYNC_LITERAL);
					InterfaceQualifier interfaceQ =(InterfaceQualifier)portType.getInterfaceQualifiers().get(0);
					if(interfaceQ instanceof JoinTransaction){
						((JoinTransaction)interfaceQ).setValue(Boolean.TRUE);
					}


				}

			}

		}
	}

	/**
	 * For all components, the default PreferredInteractionStyle is ASYNCH, except for the Component with JavaImplementation 
	 * for which it should be SYNC. Updates after the fact the PreferredInteraction Style to ANY.
	 * @param component
	 */
	private void updateInteractionStyle(DocumentRoot root){
		if(ScdlUtil.isComponent(root)){
			Component component = root.getComponent();
			if(component.getImplementation() != null && component.getImplementation() instanceof JavaImplementation){
				List interfaces = component.getInterfaceSet().getInterfaces();
				if(interfaces != null && !interfaces.isEmpty()){
					Iterator it = interfaces.iterator();
					WSDLPortType portType = (WSDLPortType)it.next();
					//portType.setPreferredInteractionStyle(InteractionStyle.ANY_LITERAL);
					portType.setPreferredInteractionStyle(InteractionStyle.SYNC_LITERAL);
				}
			}else if(component.getImplementation()!= null && component.getImplementation() instanceof ProcessImplementation){
				Action action = (Action)getSource().get(0);
				AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.BPEL_PROCESS_IS_LONG_RUNNING);
				valueProvider.setContext(this.getContext());
				Boolean longRunningProcess = (Boolean)valueProvider.getAttributeValueForType(action, AttributeNameConstants.BPEL_PROCESS_IS_LONG_RUNNING);
				if(!longRunningProcess.booleanValue()){
					List interfaces = component.getInterfaceSet().getInterfaces();
					if(interfaces != null && !interfaces.isEmpty()){
						Iterator it = interfaces.iterator();
						WSDLPortType portType = (WSDLPortType)it.next();
						portType.setPreferredInteractionStyle(InteractionStyle.SYNC_LITERAL);
						InterfaceQualifier interfaceQ =(InterfaceQualifier)portType.getInterfaceQualifiers().get(0);
						if(interfaceQ instanceof JoinTransaction){
							((JoinTransaction)interfaceQ).setValue(Boolean.TRUE);
						}


					}

				}
			}
			//feature 118131
		}else if(ScdlUtil.isImport(root)){
			com.ibm.wsspi.sca.scdl.Import scdlImport = root.getImport();
			List interfaces = scdlImport.getInterfaceSet().getInterfaces();
			if(interfaces != null && !interfaces.isEmpty()){
				Iterator it = interfaces.iterator();
				WSDLPortType portType = (WSDLPortType)it.next();
				portType.setPreferredInteractionStyle(InteractionStyle.ANY_LITERAL);
				if(portType.getInterfaceQualifiers() != null && !portType.getInterfaceQualifiers().isEmpty()){
					portType.getInterfaceQualifiers().removeAll(portType.getInterfaceQualifiers());
				}
			}
		}
	}
 

	/**
	 * This method tries to derive unique name for component process implementation
	 * @param process
	 * @param action
	 * @return
	 */
   private String createProcessImplementationName(Process process,Action action){
	   String processName = null;
	   if (NamingUtil.getRegistry(getContext(), process)==null)	
			NamingUtil.putRegistry(getContext(), process, new AbstractbpelNamingRegistry());	
		
		AttributeValueProvider procValueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.BPEL_PROCESS_NAME);
		procValueProvider.setContext(this.getContext());
		if(getSource().size() >= 3 && (getSource().get(getSource().size()-1) instanceof CallBehaviorAction)){
			CallBehaviorAction cba = (CallBehaviorAction)(getSource().get(getSource().size()-1));
			processName  = (String)procValueProvider.getAttributeValueForType( process, cba, AttributeNameConstants.BPEL_PROCESS_NAME, NamingUtil.getRegistry(getContext(), process));
		}else{
			processName  = (String)procValueProvider.getAttributeValueForType( process, action, AttributeNameConstants.BPEL_PROCESS_NAME, NamingUtil.getRegistry(getContext(), process));
		}
		JavaNCNameConverter converter = new JavaNCNameConverter();
		processName = converter.convertName(processName);
	   return processName;
   }
} //ComponentRuleImpl
