/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.wsdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Part;
import org.eclipse.wst.wsdl.PortType;
import org.eclipse.wst.wsdl.WSDLFactory;

import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.TypedElement;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.activities.Variable;
import com.ibm.btools.bom.model.services.InputParameterSet;
import com.ibm.btools.bom.model.services.OutputParameterSet;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.attributes.helper.OperationTypeUtil;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.MessageRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.PartRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.WsdlFactory;
import com.ibm.btools.te.ilm.heuristics.wsdl.WsdlPackage;
import com.ibm.btools.te.ilm.heuristics.wsdl.util.OriginalWsdlUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.util.exception.BTRuntimeException;
import com.ibm.btools.util.resource.UtilResourceBundleSingleton;
import com.ibm.wbit.model.utils.wsdl.WSDLUtils;

/**
 * <!-- begin-user-doc -->
 * SOURCE: BOM InputPinSet or OutputPinSet
 * TARGET: WSDL Message
 * 
 * The Rule is responsible for transforming a BOM Input or OutputPinSet to a WSDL Message.
 * If a Message with identical WSDL Parts was already defined in the WSDL the message will be reused. 
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 */
public class MessageRuleImpl extends AbstractProcIntRuleImpl implements MessageRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	private NamedElement srcObj;
	private boolean generateEmptyWSDLMessage = true;
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected MessageRuleImpl() {
		super();
	}

	/**
	 * The WSDL Message
	 */
	private Message wsdlMessage = null;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return WsdlPackage.Literals.MESSAGE_RULE;
	}

	/**
	 * SOURCE: BOM InputPinSet, BOM OutputPinSet.
	 * TARGET: WSDL Message
	 * Transform a BOM Input or OutputPinSet to a WSDL Message.
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");
		if (isComplete()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return true;
		}
		if (getSource() == null || getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return false;
		}
		// Initialization
		srcObj = (NamedElement)getSource().get(0);
		MessageRule mr = (MessageRule)TransformationUtil.getRuleForSource(ProcessUtil.getProcessInterfaceRule(getContext()),srcObj, Message.class);
		if (mr != null){
			getTarget().addAll(mr.getTarget());
			setComplete(true);
			LoggingUtil.traceExit(this, "transformSource2Target");
			return isComplete();
		}
		wsdlMessage = WSDLFactory.eINSTANCE.createMessage();
		AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.WSDL_MESSAGE_NAME);
		valueProvider.setContext(this.getContext());
		String wsdlMessageName = (String)valueProvider.getAttributeValueForType(wsdlMessage, (NamedElement)getSource().get(0), AttributeNameConstants.WSDL_MESSAGE_NAME, NamingUtil.findRegistry(this));	
		wsdlMessage.setQName(new QName(getWSDLDefinition().getTargetNamespace(), wsdlMessageName));
		Message origMsg=null;
		PortType origPortType=null;
		if (getSource().get(0) instanceof PinSet) {
			PinSet ps=(PinSet)getSource().get(0);
			Action action=null;
			if (ps instanceof InputPinSet) {
				action = ((InputPinSet)ps).getAction();
			} else if (ps instanceof OutputPinSet) {
				action = ((OutputPinSet)ps).getAction();
			} 
			if (action!=null) {
				StructuredActivityNode element = null;
				if(action instanceof CallBehaviorAction){
					element =((Activity)((CallBehaviorAction)action).getBehavior()).getImplementation();
				}else if(action instanceof StructuredActivityNode){
					element = (StructuredActivityNode)action;
				}
				Definition origDef = OriginalWsdlUtil.getOriginalWsdl(getContext(),BomUtils.resolveFileAttachment(element));
				origPortType = OriginalWsdlUtil.getPortType(getContext(),BomUtils.resolveOwningInterface(element));
				if (origDef != null) {
				//defect 87357, remove unnecessary validation	
//					// Check to see if there are multiple message parts
//					// If there is, we are not able to transform it
//					Map messages = origDef.getMessages();
//					for (Iterator m = messages.values().iterator(); m.hasNext();) {
//						Message message = (Message)m.next();
//						if (message.getParts().size() >1) {
//							String params[]={element.getName(), srcObj.getName()};
//							String errorMessage = UtilResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.class, MessageKeys.WSDL_MULTIPLE_PARTS, params);
//							LoggingUtil.logError(MessageKeys.WSDL_MULTIPLE_PARTS, params, null);
//							BTRuntimeException runtimeEx = new BTRuntimeException(errorMessage);			
//							throw runtimeEx;
//						}
//					}
					//we have bring the message target namespace and from the record					
					String targetNamespace = null;
					EList comments = ps.getOwnedComment();
					if(comments != null && comments.size() > 0){
						Comment firstComment = (Comment)ps.getOwnedComment().get(0);	
						String content= firstComment==null?null:firstComment.getBody();
						if(content != null && content.indexOf("=") != -1){
							targetNamespace =content.substring(content.indexOf("=") + 1); 	
						}
					}
					//20070212 d146631
					String msgName= null;
					if((ps instanceof OutputPinSet) && ((OutputPinSet)ps).getIsException().booleanValue()){
						msgName = ps.getName().substring(ps.getName().indexOf(".") +1);
					}else{
						msgName = ps.getName();
					}				
					origMsg = OriginalWsdlUtil.getMessage(origDef,targetNamespace,msgName, getContext());
					if (origMsg != null) {
						OriginalWsdlUtil.registerWsdlElement(getContext(),ps,origMsg);
						wsdlMessage=origMsg; 
					}					
				} 		
			}
		}	
		if (origPortType!=null && origMsg!=null) {
			setupChildRulesForOriginalWsdl(origPortType,origMsg);
			setComplete(executeChildRules());
		} else {			
			setupChildRules();
			boolean ret = executeChildRules();
			processChildTargets();
			getWSDLDefinition().addMessage(wsdlMessage);
			setComplete(ret);
		}
		getTarget().add(wsdlMessage); 		
		//AZ WMT 20070322
		executeHandlers();
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}

	private void setupChildRulesForOriginalWsdl(PortType origPortType,Message origMsg) {
		boolean bDocLitWrappedStyle = WSDLUtils.isDocLiteralWrappedPattern(origPortType);
		// Check if the PortType is doc lit wrapped or doc-lit/rpc style
		// If it is a doc-lit wrapped, that means it can only have a single Part in the Message, hence
		// we only create a single PartRule.  Otherwise we create one PartRule for each Pin
		if (!bDocLitWrappedStyle) {
			List parts = origMsg.getEParts();
			if(source.get(0) instanceof InputPinSet){
				InputPinSet ps = (InputPinSet)source.get(0);
				if (parts.size() == ps.getInputObjectPin().size()) {
					// assume the parts and pins are in same order
					for (int i=0;i<parts.size();i++) {
						PartRule partRule = null;
						partRule = WsdlFactory.eINSTANCE.createPartRule();
						getChildRules().add(partRule);
						partRule.getSource().add(ps.getInputObjectPin().get(i));
						partRule.getSource().add(parts.get(i));
						if (parts.size()==1) {
							partRule.getSource().add(ps);
						}
					}
				}
			}else if(source.get(0) instanceof OutputPinSet){
				OutputPinSet ps = (OutputPinSet)source.get(0);
				if (parts.size()==ps.getOutputObjectPin().size()) {
					// assume the parts and pins are in same order
					for (int i=0;i<parts.size();i++) {
						PartRule partRule = null;
						partRule = WsdlFactory.eINSTANCE.createPartRule();
						getChildRules().add(partRule);
						partRule.getSource().add(ps.getOutputObjectPin().get(i));
						partRule.getSource().add(parts.get(i));
						if (parts.size()==1) {
							partRule.getSource().add(ps);
						}
					}
				}
			}			
		} else {
			PartRule partRule = WsdlFactory.eINSTANCE.createPartRule();
			partRule.getSource().add(getSource().get(0));
			//it must have one and only one Part
			partRule.getSource().add(origMsg.getEParts().get(0)); 
			getChildRules().add(partRule);			
		}
	}

	/**
	 * Child PartRules are created based on the source of this rule as follows
	 * - if the source is a PinSet of a WSI compliant Action will create PartRules with same source (PinSet).
	 * - if the source is a PinSet or a Variable for a non WSI compliant Action (target a non WSICompliant WSDL), will create PartRules with source the Pin in the PinSet.
	 * Creates child rules (PartRules). For each BOM ObjectPin in the BOM Set will create 
	 * the PartRules that will be set to the target WSDLMessage. 
	 */
	private void setupChildRules(){
		//NOTE: BOM Control pins are not mapped.
		EObject container = ((EObject)source.get(0)).eContainer();
		if(!isWSICompliant((Action)container)){
			if(source.get(0) instanceof InputPinSet){
				createChildPartRules(((InputPinSet)source.get(0)).getInputObjectPin());
			}else if(source.get(0) instanceof OutputPinSet){
				createChildPartRules(((OutputPinSet)source.get(0)).getOutputObjectPin());
			}else if(source.get(0) instanceof InputParameterSet){
				createChildPartRules(((InputParameterSet)source.get(0)).getParameter());
			}else if(source.get(0) instanceof OutputParameterSet){
				createChildPartRules(((OutputParameterSet)source.get(0)).getParameter());
			}else if(getSource().get(0) instanceof Variable){
				createChildPartRule((TypedElement)source.get(0));
			}
		}else if(getSource().get(0) instanceof Variable && !BpelOptionsUtil.isTargetWSICompliant()) {
			// else the source is a BOM variable
			createChildPartRule((TypedElement)source.get(0));
			//If the target is WSI compliant, only one PartRule will be generated having the source the Pin/ParameterSet
		}else{
			if(!(generateEmptyMessage(((PinSet)getSource().get(0))) && generateEmptyWSDLMessage)){
				PartRule partRule = WsdlFactory.eINSTANCE.createPartRule();
				partRule.getSource().add(getSource().get(0));
				getChildRules().add(partRule);
				//defect 164299
			}else if(generateMessageWithEmptyComplexWrapperType(((PinSet)getSource().get(0)))){
				PartRule partRule = WsdlFactory.eINSTANCE.createPartRule();
				partRule.getSource().add(getSource().get(0));
				getChildRules().add(partRule);
			}
		}

	}

	/**
	 * Creates a PartRule setting the <code>src</code> as source.
	 * @param - src a BOM TypeElement source of the child PartRule
	 */
	private void createChildPartRule(TypedElement src){
		PartRule partRule = null;
		partRule = WsdlFactory.eINSTANCE.createPartRule();
		getChildRules().add(partRule);
		partRule.getSource().add(src);
	}

	/**
	 * Creates a PartRule for each Pin in the <code>pins</code>.
	 * @param - pins List of BOM Pins.
	 */
	private void createChildPartRules(List pins){
		PartRule partRule = null;
		if(pins != null && !pins.isEmpty()){
			for (int i=0; i<pins.size(); i++){
				createChildPartRule((TypedElement)pins.get(i));
			}
		}
	}

	/**
	 * This rule process the child's rules targets that are of type WSDL Part.
	 * It will propagate other child targets
	 */ 
	protected void processChildTargets(){
		TransformationRule childRule = null;
		EObject childTarget = null;
		List childRules = getChildRules();
		List childTargets;
		Iterator childRulesIterator = childRules.iterator();
		while (childRulesIterator.hasNext()){
			childRule = (TransformationRule)childRulesIterator.next();
			childTargets = childRule.getTarget();
			for (int i=0; i<childTargets.size(); i++){
				childTarget = (EObject) childTargets.get(i);
				if (childTarget instanceof Part){
					wsdlMessage.addPart((Part)childTarget);
				}else{
					getTarget().add(childTarget);
				}	

			}
		}
	}

	private boolean isControlOnly(NamedElement pinSet){
		if(pinSet instanceof InputPinSet){
			if(((InputPinSet)pinSet).getInputObjectPin().size()==0){
				return true;
			}
		}else if(pinSet instanceof OutputPinSet){
			if(((OutputPinSet)pinSet).getOutputObjectPin().size()==0){
				return true;
			}
		}
		return false;
	}

	private boolean generateEmptyMessage(PinSet pinSet){
		if(isControlOnly(pinSet)){
			//for an inputPinSet always generate the WSDL Message with Part and empty complex type.. 
			//as this will always map to an Input
			if(pinSet instanceof InputPinSet){
				return false;
				//for an OutputPinSet generate an empty Message in all cases except when the PinSet
				//will generate a Callback.. this should never happen, as we cannot have control only on a SAN
			}else if(pinSet instanceof OutputPinSet){
				BomUtils.PinSetAnalyzer psa = new BomUtils.PinSetAnalyzer(((OutputPinSet)pinSet).getAction(), getContext());
				if(psa.getOutputPinSetsAsCallbackOperation().contains(pinSet)){
					return false;
				}else{
					return true;
				}
			}
		}
		return false;
	}
	
	/**
	 * If it's not one way operation, we always create empty message
	 * create skeleton complex wrapper type to fix defect 164299
	 * @param pinSet
	 * @return
	 */
	private boolean generateMessageWithEmptyComplexWrapperType(PinSet pinSet){	
		if(pinSet instanceof OutputPinSet && isControlOnly(pinSet)){
			OutputPinSet ops = (OutputPinSet)pinSet;
			InputPinSet ips = (InputPinSet)ops.getInputPinSet().get(0);
			Boolean isOneWayOperation = OperationTypeUtil.isOneWayOperation(ips);
			if(!isOneWayOperation.booleanValue()){
				return true;
			}			
		}
		return false;
	}
}


//MessageRuleImpl
