/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.wsdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.wst.wsdl.Fault;
import org.eclipse.wst.wsdl.Input;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.Output;
import org.eclipse.wst.wsdl.WSDLFactory;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.ActivitiesFactory;
import com.ibm.btools.bom.model.processes.activities.Datastore;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.services.InputParameterSet;
import com.ibm.btools.bom.model.services.OutputParameterSet;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.MessageRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.ParameterRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.WsdlFactory;
import com.ibm.btools.te.ilm.heuristics.wsdl.WsdlPackage;
import com.ibm.btools.te.ilm.heuristics.wsdl.util.OriginalWsdlUtil;

/**
 * <!-- begin-user-doc -->
 * SOURCE: BOM InputPinSet and/or OutputPinSet, BOM InputParameter and or OutputParameterSet
 * TARGET: WSDL Parameter
 * The rule is responsible for mapping a BOM InputPinSet and/or OutputPinSet or a 
 * BOM InputParameter and or OutputParameterSet to a WSDL Parameter. 
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class ParameterRuleImpl extends AbstractProcIntRuleImpl implements ParameterRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ParameterRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return WsdlPackage.Literals.PARAMETER_RULE;
	}

	/**
	 * SOURCE: BOM Input and/or OutputSet or InputParameter and/or OutputParameterSet 
	 * 			+ Action (Optional) in the case it is for a callback operation
	 * TARGET: WSDL Parameter
	 * Transforms the source to a WSDL Parameter.  
	*/
	public boolean transformSource2Target(){
		/*
		 * ASSUMPTION: The Message and the PortTypes are part of the same namespace, 
		 * so no need to add import statements for the referenced Messages
		 * Retrieves the message that should have already been created for the source.
		 * Creates a WSDL Input or a WSDL Output depending on the source and adds it to the 
		 * operation.
		 * Set the name based on the Input/OutputPinSet name appending "Message" suffix.
		 * Set the target of the rule to be the WSDL Parameter 
		 */
		LoggingUtil.traceEntry(this, "transformSource2Target");
		if( isComplete()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return true;
		}
		if( getSource() == null || getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return false;
		}
		Message message = null;
		// retrieves the Message already created for this input/outputSet
		//although checking from the root is not good, we may need to start looking from the PortTypeRule which is the grandparent... 
		TransformationRule  rule = TransformationUtil.getRuleForSource(getRoot(),(EObject)getSource().get(0),Message.class);
		if(rule != null){
			message = (Message)rule.getTarget().get(0);
		}
		if( message != null ){
			JavaNCNameConverter converter = new JavaNCNameConverter();
			//AR if the source is a Datastore we need to create the Input and the Output
			if(source.get(0) instanceof Datastore){
					Input input = WSDLFactory.eINSTANCE.createInput();
					input.setMessage(message);
					getTarget().add(input);

					// this is the empty input create for the getVariable
					Input empyInput = createEmptyInput();
					getTarget().add(empyInput);

					Output output = WSDLFactory.eINSTANCE.createOutput();
					output.setMessage(message);
					getTarget().add(output);

					setComplete(true);
					return isComplete();
				}
			else if(source.get(0) instanceof InputPinSet || getSource().get(0) instanceof InputParameterSet){
				Input input = WSDLFactory.eINSTANCE.createInput();
				input.setMessage(message);
			// Check to see if it's a valid name by calling NameConverter
				NameProvider nameProvider = NameProviderFactory.getNameProvider(input);
				String convertedName = nameProvider.getName(Input.class, ((NamedElement)getSource().get(0)), NamingUtil.findRegistry(this));
				input.setName(convertedName);
							
				Operation op = OriginalWsdlUtil.getOperation(getContext(),(NamedElement)getSource().get(0));
				if(op == null){
					NamedElement ne = (NamedElement)getSource().get(0);
					op = resolveOriginalOperationFromParentRule(this,ne);
				}
				Input origInput=null;
				if (op != null) {
					origInput = OriginalWsdlUtil.getInput(op);
					if (origInput!=null ) {
						OriginalWsdlUtil.registerWsdlElement(getContext(),(NamedElement)source.get(0),origInput);
						input = origInput;
					}
				}
				getTarget().add(input);
			}else if(source.get(0) instanceof OutputPinSet || getSource().get(0) instanceof OutputParameterSet){
				// Need to handle the special case for a callback
				if(source.size()==2 && source.get(1) instanceof Action){
					Input input = WSDLFactory.eINSTANCE.createInput();
					input.setMessage(message);
					NameProvider nameProvider = NameProviderFactory.getNameProvider(input);
					String convertedName = nameProvider.getName(Input.class, ((NamedElement)getSource().get(0)), NamingUtil.findRegistry(this));
					input.setName(convertedName);
					getTarget().add(input);
				}else{
					//TODO add faults					
					if (getSource().get(0) instanceof OutputPinSet && ((OutputPinSet)getSource().get(0)).getIsException() != null &&
								((OutputPinSet)getSource().get(0)).getIsException().booleanValue()) {
							Fault fault = WSDLFactory.eINSTANCE.createFault();
							fault.setMessage(message);
							NameProvider nameProvider = NameProviderFactory.getNameProvider(fault);
							String convertedName = nameProvider.getName(Fault.class, ((NamedElement)getSource().get(0)), NamingUtil.findRegistry(this));
							fault.setName(convertedName);
							
							Operation op = OriginalWsdlUtil.getOperation(getContext(),(NamedElement)getSource().get(0));
							if(op == null){
								NamedElement ne = (NamedElement)getSource().get(0);
								op = resolveOriginalOperationFromParentRule(this,ne);
							}
							Fault origFault=null;
							if (op != null) {
								origFault = OriginalWsdlUtil.getFault(op);
								if (origFault != null ) {
									OriginalWsdlUtil.registerWsdlElement(getContext(),(NamedElement)source.get(0), origFault);
									fault = origFault;
								}
							}
							getTarget().add(fault);			
					} else {	//code prior to handling exception output sets
						Output output = WSDLFactory.eINSTANCE.createOutput();
						output.setMessage(message);
						NameProvider nameProvider = NameProviderFactory.getNameProvider(output);
						String convertedName = nameProvider.getName(Output.class, ((NamedElement)getSource().get(0)), NamingUtil.findRegistry(this));
						output.setName(convertedName);
						
						Operation op = OriginalWsdlUtil.getOperation(getContext(),(NamedElement)getSource().get(0));
						if(op == null){
							NamedElement ne = (NamedElement)getSource().get(0);
							op = resolveOriginalOperationFromParentRule(this,ne);
						}
						Output origOutput=null;
						if (op != null) {
							origOutput = OriginalWsdlUtil.getOutput(op);
							if (origOutput!=null ) {
								OriginalWsdlUtil.registerWsdlElement(getContext(),(NamedElement)source.get(0),origOutput);
								output = origOutput;
							}
						}
						getTarget().add(output);
					}
				}
			}
		}
//		AZ WMT 20070322
		executeHandlers();
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");
	return isComplete();	
	}

	private Input createEmptyInput(){
		
		Input emptyInput = WSDLFactory.eINSTANCE.createInput();
		InputPinSet inputPinSet = ActivitiesFactory.eINSTANCE.createInputPinSet();
		inputPinSet.setName("Empty");
		MessageRule messageRule = WsdlFactory.eINSTANCE.createMessageRule();
		messageRule.getSource().add(inputPinSet);
		getChildRules().add(messageRule);
		messageRule.transformSource2Target();

		Message emptyMessage = (Message)messageRule.getTarget().get(0);
		emptyInput.setMessage(emptyMessage);
		
		
		return emptyInput;		
	}
	
	/**
	 * If this operation is not registered with the <code>PinSet</code>, we need to
	 * look up the operation through its parent rule target
	 * @param rule
	 * @param ne
	 * @return
	 */
	private Operation resolveOriginalOperationFromParentRule(ParameterRule rule,NamedElement ne){		
		if(ne instanceof PinSet){
			PinSet ps = (PinSet)ne;
			if(ps.eContainer() instanceof StructuredActivityNode){
				StructuredActivityNode san = (StructuredActivityNode)ps.eContainer();
				if("ServiceOperation".equals(san.getAspect())){
					return (Operation)getParentRule().getTarget().get(0);
				}
			}
		}
		return null;
	}
} //ParameterRuleImpl
