/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.wsdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import javax.wsdl.WSDLException;
import javax.xml.namespace.QName;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.WSDLFactory;
import org.eclipse.xsd.util.XSDConstants;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleAction;
import com.ibm.btools.bom.model.processes.humantasks.HumanTask;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.attributes.helper.TechnicalAttributesHelper;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.TransformationEngine;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.util.AbstractbpelNamingRegistry;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionUtil;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.WSDLDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.WsdlPackage;
import com.ibm.btools.te.ilm.heuristics.wsdl.util.WsdlNamingRegistry;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.util.exception.BTRuntimeException;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.Process;


/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>WSDL Definition Rule</b></em>'.
 * Source: BOM NamedElement
 * Target: Empty WSDL Definition with header set
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class WSDLDefinitionRuleImpl extends AbstractProcIntRuleImpl implements WSDLDefinitionRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected WSDLDefinitionRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return WsdlPackage.Literals.WSDL_DEFINITION_RULE;
	}

	/**
	 * Source: BOM NamedElement
	 * Target: Empty WSDL Definition with header set
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target()
	{
		LoggingUtil.traceExit(this, "transformSource2Target");
		if( isComplete()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return true;
		}
		if( getSource() == null || getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return false;
		}
		try{
			Definition wsdlDefinition = createWSDLDefinition();
			
			getTarget().add(wsdlDefinition);
			
			//Register the WsdlNamingRegistry
			TransformationContext context = getContext();
			if (NamingUtil.getRegistry(context, wsdlDefinition)==null)	
				NamingUtil.putRegistry(getContext(), wsdlDefinition, new WsdlNamingRegistry());			
	
		} catch (WSDLException ex){
			String params[]={((NamedElement)source.get(0)).getName()};
			LoggingUtil.logError(MessageKeys.ERROR_CREATING_WSDL_DEFINITION, params, ex);
			BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_CREATING_WSDL_DEFINITION));
			runtimeEx.setChainedException(ex);			
			throw runtimeEx;
		}
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();		
	}
	
	/**
	* Creates a WSDL definition.
	* @return Definition a WSDL Definition
	* @throws WSDLException
	*/
	protected Definition createWSDLDefinition()throws WSDLException {
		Definition wsdlDefinition;

		NamedElement thisSource = (NamedElement)getSource().get(0);
		if(thisSource instanceof Activity){
			thisSource = ((Activity)thisSource).getImplementation();
		}
		AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.WSDL_TARGET_NAMESPACE);
		valueProvider.setContext(this.getContext());
		wsdlDefinition = WSDLFactory.eINSTANCE.createDefinition();
		String encodedNamespace = (String)valueProvider.getAttributeValueForType(wsdlDefinition, thisSource, AttributeNameConstants.WSDL_TARGET_NAMESPACE, NamingUtil.findRegistry(this));
		//creates the wsdl name.
		String wsdlName=null;
		if (thisSource instanceof Action) {
			Action action = (Action)thisSource;
			//do not append the unique key for Process, Reusable Task and Service
			if (BomUtils.isProcess(action)) {
				Process process = BPELFactory.eINSTANCE.createProcess();	
				if (NamingUtil.getRegistry(getContext(), process)==null)	
					NamingUtil.putRegistry(getContext(), process, new AbstractbpelNamingRegistry());	
				
				AttributeValueProvider procValueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.BPEL_PROCESS_NAME);
				procValueProvider.setContext(this.getContext());
				
				wsdlName  = (String)procValueProvider.getAttributeValueForType( process, thisSource, AttributeNameConstants.BPEL_PROCESS_NAME, NamingUtil.getRegistry(getContext(), process));
				String convertedName = NamingUtil.convertGlobalWSDLNameForAction(action, getContext(), wsdlName); // name from PortType of TA
				String fileName = NamingUtil.getNameForArtifact(convertedName);				
				if (fileName != null && !fileName.equals(wsdlName)) {
					wsdlName = fileName;
				}
			} else if(BomUtils.isReusableTask(action) || BomUtils.isReusableService(action) ||
					(action instanceof BusinessRuleAction && action.getInStructuredNode() == null) ||
					(action instanceof HumanTask && action.getInStructuredNode() == null)){
				JavaNCNameConverter np = new JavaNCNameConverter();
				//defect 116657 use the PortType as the WSDL Definition name
				String name = (String)TechnicalAttributesHelper.getAttributeValue(action, AttributeNameConstants.WSDL_PORT_TYPE_NAME);
				if(name == null || name.equals("")){
					name = thisSource.getName();
				}

				name = np.convertName(name);
				String convertedName = NamingUtil.convertGlobalWSDLNameForAction(action, getContext(), name); // name from PortType of TA
				String fileName = NamingUtil.getNameForArtifact(convertedName);				
				if (fileName != null) {
					wsdlName = fileName;
				} else {
					wsdlName = name;
				}
			} else {
				String name = (String)TechnicalAttributesHelper.getAttributeValue(action, AttributeNameConstants.WSDL_PORT_TYPE_NAME);
				if(name == null || name.equals("")){
					name = thisSource.getName();
				}
				
				JavaNCNameConverter np = new JavaNCNameConverter();
				name = np.convertName(name);				
				String convertedName = NamingUtil.convertWSDLNameForAction(action, getContext(), null); // name from PortType of TA
				String wsdlFileName = NamingUtil.getNameForArtifact(convertedName);
				if (wsdlFileName != null) {
					wsdlName = wsdlFileName;
				} else {
					wsdlName = name;
				}
			}	
			String suffix = ExportOperation.WSDL_SUFFIX.substring(0, 1).toUpperCase() + ExportOperation.WSDL_SUFFIX.substring(1);
			wsdlName = wsdlName+suffix;
		}

		if (wsdlName!=null) {
			//create an unique name based on the TargetNamespace. It will help differentiate between Tasks with same name
			QName qname = new QName(encodedNamespace,wsdlName);
			wsdlDefinition.setQName(qname);			
		}
		
		wsdlDefinition.setTargetNamespace(encodedNamespace);
		wsdlDefinition.addNamespace("tns", encodedNamespace);
		wsdlDefinition.addNamespace("xsd", XSDConstants.SCHEMA_FOR_SCHEMA_URI_2001);
		wsdlDefinition.addNamespace("wsdl", "http://schemas.xmlsoap.org/wsdl/");
		
		TransformationSessionUtil.registerProcessedElementWithNamespace(TransformationEngine.getTransformationSession(), wsdlDefinition, encodedNamespace);

		return wsdlDefinition;	
	}
	
} //WSDLDefinitionRuleImpl
