package com.ibm.btools.te.ilm.heuristics.wsdl.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.Input;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.Output;
import org.eclipse.wst.wsdl.PortType;
import org.eclipse.xsd.XSDElementDeclaration;

import com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry;
import com.ibm.wbit.bpel.services.partnerlinktype.PartnerLinkType;

public class WsdlNamingRegistry implements NamingRegistry{

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	private  HashMap portTypeNameRegistry = new HashMap();
	private  HashMap operationNameRegistry = new HashMap();	
	//AR there is no need for now to have a part registry, we only support single Part messages, 
	//and the Part should be unique within the message, so this is guaranteed
	//private  HashMap partNameRegistry = new HashMap();	
	private  HashMap messageNameRegistry = new HashMap();	
	private  HashMap inputNameRegistry = new HashMap();
	private  HashMap outputNameRegistry = new HashMap();	
	private  HashMap partnerLinkTypeRegistry = new HashMap();
	private  HashMap xsdTypeRegistry = new HashMap();
	
	/**
	 * Returns a boolean whether the name is unique
	 * It searches the registry and and return false if name is not unique
	 * it returns true if name is unique and it stores the name in the registry.
	 * @param obj  the EObject of a construct
	 * @param name  the String name
	 * @return boolean the boolean value whether name is unique or not
	 */		
	public boolean isUniqueName(Object obj, String name)
	{
		HashMap map = getRegistry(obj);
		if (map == null){
			// name for this type of construct does not require name uniqueness
			return true;		 		
		}
		else if (map.values().contains(name)){
			return false;
		}
		else {
			//Store the name in the Registry
			map.put(obj, name);
			return true;
		}
	}
	
	/**
	 * Returns a boolean whether the name is unique
	 * It searches the registry and and return false if name is not unique
	 * it returns true if name is unique and it stores the name in the registry.
	 * @param obj  the EObject of a construct
	 * @param name  the String name
	 * @return boolean the boolean value whether name is unique or not
	 */		
	public boolean isUniqueName(Object obj, Object parentObject, String name)
	{
		HashMap map = getRegistry(obj);
		List entries = null;
		if (map == null){
			// name for this type of construct does not require name uniqueness
			return true;		 		
		}else if(map.get(parentObject) != null){
			entries = (List)map.get(parentObject);
			if(entries.contains(name)){
				return false;
			}else{
				entries.add(name);
				return true;
			}
		}else {
			entries = new LinkedList();
			entries.add(name);
			//Store the name in the Registry
			map.put(parentObject, entries);
			return true;
		}
	}	

	/**
	 * Returns a HashMap for the specific obj	 
	 * @param obj  the EObject of a type of construct	
	 * @return HashMap the corresponding hashmap for obj
	 */		
	private HashMap getRegistry(Object obj)
	{
		if (obj instanceof PortType)
			return portTypeNameRegistry;
		else if (obj instanceof Operation)
			return operationNameRegistry;
//		else if (obj instanceof Part)
//			return partNameRegistry;
		else if (obj instanceof Message)
			return messageNameRegistry;
		else if (obj instanceof Input)
			return inputNameRegistry;
		else if (obj instanceof Output)
			return outputNameRegistry;
		else if (obj instanceof PartnerLinkType)
			return partnerLinkTypeRegistry;
		else if (obj instanceof XSDElementDeclaration)
			return xsdTypeRegistry;
		else
			return null;
	}
	
	/**
	 * Returns a HashMap for the specific obj	 
	 * @param obj  the EObject of a type of construct	
	 * @return HashMap the corresponding hashmap for obj
	 */		
	private HashMap getRegistry(Class obj)
	{
		if (obj == PortType.class)
			return portTypeNameRegistry;
		else if (obj == Operation.class)
			return operationNameRegistry;
//		else if (obj instanceof Part)
//			return partNameRegistry;
		else if (obj == Message.class)
			return messageNameRegistry;
		else if (obj == Input.class)
			return inputNameRegistry;
		else if (obj == Output.class)
			return outputNameRegistry;
		else if (obj == PartnerLinkType.class)
			return partnerLinkTypeRegistry;
		else if (obj == XSDElementDeclaration.class)
			return xsdTypeRegistry;
		else
			return null;
	}

	/* (non-Javadoc)
	 * @see com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry#deregister(java.lang.Object, java.lang.String)
	 */
	public void deregister(Object obj, String name) {
		HashMap map = getRegistry(obj);
		if (map == null){
			// name for this type of construct does not require name uniqueness
			return;		 		
		}
		else if (map.values().contains(name)){
			map.remove(obj);
		}
	}
	
	/* (non-Javadoc)
	 * @see com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry#retrieveFromRegistry(java.lang.String)
	 */
	public Object retrieveFromRegistry(Class obj, String registryEntry) {
		
		HashMap registry = getRegistry(obj);
		if(registry.containsValue(registryEntry)){
			//get the object container that was registered for that entry.. 
			Iterator it = registry.keySet().iterator();
			while(it.hasNext()){
				Object variable = (EObject)it.next();
				if(registry.get(variable).equals(registryEntry)){
					return variable;
				}
			}
			
		}
		return null;
	}
	
	/* (non-Javadoc)
	 * @see com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry#register(java.lang.Object, java.lang.String)
	 */
	public void register(Object obj, String name) {
		HashMap map = getRegistry(obj);
		if(map == null){
			return;
		}
		map.put(obj, name);
	}
}
