/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.xsd.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.Part;
import org.eclipse.xsd.XSDAttributeDeclaration;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDFactory;
import org.eclipse.xsd.XSDSchema;
import org.eclipse.xsd.XSDTypeDefinition;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.ExportOperationConstants;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessInterfaceRule;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.OperationRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.PartRule;
import com.ibm.btools.te.ilm.heuristics.xsd.ElementDeclarationRule;
import com.ibm.btools.te.ilm.heuristics.xsd.PinSetRule;
import com.ibm.btools.te.ilm.heuristics.xsd.XsdFactory;
import com.ibm.btools.te.ilm.heuristics.xsd.XsdPackage;
import com.ibm.btools.te.ilm.heuristics.xsd.util.OriginalXsdUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.util.exception.BTRuntimeException;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Element Declaration Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class ElementDeclarationRuleImpl extends AbstractDataDefRuleImpl implements ElementDeclarationRule {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2008, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	
	/**
	 * The XSDElementDeclaration target of this rule.
	 */
	private XSDElementDeclaration elementDeclaration;
	
	private Definition owningDefinition;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ElementDeclarationRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return XsdPackage.Literals.ELEMENT_DECLARATION_RULE;
	}

	boolean runOnce = false;
	/**
	 * Source: a BOM PinSet (Input/OutputPinSet or Input/OutputParameterSet) and an XSDSchema.
	 * Target : XSDElementDeclaration. 
	 * Child rules: PinSetRule. 
	 * 
	 * Maps the source to an XSDElementDeclaration. This is the global element declaration created for each 
	 * WSDL Message Part. The type of the ElementDeclaration is the wrapper complex type of the created for
	 * the BOM Set. 
	 * This rule is to be used only as part of the Process transformation.
	 * Creates (reuse) the DataDefinitionRule with it's child ClassRule (or ListRule) to retrieve the XSDTypeDefinition for the source TypedElement.
	*/
	public boolean transformSource2Target() {
		/*
		 * This rule is only called as part of the Process transformation, only from the PartRule.
		 */
		LoggingUtil.traceEntry(this, "transformSource2Target");
		if( isComplete()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return true;
		}
		
		if( getSource() == null || getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return false;
		}
		if(runOnce){
			setComplete(true);
			LoggingUtil.traceExit(this, "transformSource2Target");
			return isComplete();
			
		}
		runOnce = true;

		// xsd import enhancement
		if (getSource().get(getSource().size()-1) instanceof XSDElementDeclaration) {
			elementDeclaration=(XSDElementDeclaration)getSource().get(getSource().size()-1);
			OriginalXsdUtil.registerXsdElement(getContext(),(NamedElement)getSource().get(0),elementDeclaration);
		} else if (getSource().get(getSource().size()-1) instanceof XSDAttributeDeclaration) {
		    // not supported
			String params[] = {((XSDAttributeDeclaration)getSource().get(getSource().size()-1)).getName()};
			LoggingUtil.logError(MessageKeys.ELEMENT_DEFINED_AS_XSD_ATTRIBUTE, params, null);
			BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ELEMENT_DEFINED_AS_XSD_ATTRIBUTE));
			throw runtimeEx;
		} else {
		    elementDeclaration = XSDFactory.eINSTANCE.createXSDElementDeclaration();
		    getTarget().add(elementDeclaration);
		
		    //set the name of the ElementDeclaration as per the name of the Operation
		    TransformationRule operationRule = getParentOperationRule();
		    if (operationRule!=null) // Ray - Form generation - there is no operationRule for dummy WSDL generation
		    {
			    String operationName = ((Operation)operationRule.getTarget().get(0)).getName();
			    String convertedName = operationName;
			
			    //for the element declaration corresponding to the outputPinSet, add the Response suffix
			    if(getSource().get(0) instanceof OutputPinSet){
					//need to check if this is a callback operation, in which case, the name of the
					//elementDeclaration should match the name of the operation.
					OutputPinSet src = (OutputPinSet)getSource().get(0);
					BomUtils.PinSetAnalyzer  psa = new BomUtils.PinSetAnalyzer (src.getAction(), getContext());
					if(psa.getOutputPinSetsAsCallbackOperation().contains(src)){
						convertedName = operationName;
					} else if (src.getIsException() && src.getIsException().booleanValue()) {
						//TODO set the name for ExceptionOS to Fault
						if (getParentRule() instanceof PartRule && !getParentRule().getTarget().isEmpty() && 
								getParentRule().getTarget().get(0) instanceof Part) {
							convertedName = ((Part)(getParentRule().getTarget().get(0))).getName() + ExportOperationConstants.FAULT_SUFFIX;
						} else {
							convertedName = operationName + ExportOperationConstants.FAULT_SUFFIX;
						}
					} else {
						convertedName = operationName + "Response";
					}
			    }
				elementDeclaration.setName(convertedName);
		    } else {
		    	elementDeclaration.setName(((NamedElement)getSource().get(0)).getName());
		    }
			XSDSchema schema = (XSDSchema)getSource().get(1);
			ProcessInterfaceRule interfaceRule = (ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(getContext(), (NamedElement)this.getSource().get(0));
			owningDefinition = (Definition)interfaceRule.getTarget().get(0);
			elementDeclaration.setTargetNamespace(owningDefinition.getTargetNamespace());
			//add the elementDeclaration to the schema ...			
			schema.getContents().add(elementDeclaration);
		
		}
		setupChildRules();
		executeChildRules();
		processChildTargets();	
		
		executeHandlers();
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
		
	}
	
	/**
	 * Creates and attach the childRules to this rule. 
	 */
	private void setupChildRules(){
		
   		PinSetRule pinSetRule = XsdFactory.eINSTANCE.createPinSetRule();
   		//pinSetRule.getSource().add(getSource().get(0));
   		pinSetRule.getSource().addAll(getSource());
   		getChildRules().add(pinSetRule);
	}
	

	/**
	 * This rule process the child's rules targets that are of type XSDTypeDefinition.
	 * It will propagate other child targets
	 */ 
	protected void processChildTargets(){
		TransformationRule childRule = null;
		EObject childTarget = null;
		List childRules = getChildRules();
		List childTargets;
		Iterator childRulesIterator = childRules.iterator();
		while (childRulesIterator.hasNext()){
			childRule = (TransformationRule)childRulesIterator.next();
			childTargets = childRule.getTarget();
			for (int i=0; i<childTargets.size(); i++){
				childTarget = (EObject) childTargets.get(i);
				if (childTarget instanceof XSDTypeDefinition){
					elementDeclaration.setAnonymousTypeDefinition((XSDTypeDefinition)childTarget);
				}else{
					getTarget().add(childTarget);
				}	
				
			}
		}
	}
	
	/** 
	 * Returns the OperationRule parent of this Rule. 
	 * @return TransformationRule 
	 */
	private TransformationRule getParentOperationRule(){
		
		TransformationRule parentRule = null;
		parentRule = this.getParentRule();
		// Ray - Form generation - no OperationRule as the parent for dummy WSDL generation for Form
		while(!(parentRule instanceof OperationRule) && parentRule!=null){
		    if (parentRule instanceof com.ibm.btools.te.ilm.heuristics.extservice.OperationRule) {
		        break;
		    }
			parentRule = parentRule.getParentRule();
		}
		return parentRule;
	}

	/** 
	 * Returns the index for the ExceptionOutputSet. 
	 * @return int 
	 */
	private int getIndexForFault(OutputPinSet ops) {
		if (ops == null) {
			return 0;
		}
		
		int index = 0;
		Iterator <OutputPinSet> it = ops.getAction().getOutputPinSet().iterator();
		OutputPinSet outputSet = null;
		while (it.hasNext()) {
			outputSet = (OutputPinSet)it.next();
			if (outputSet.getIsException() != null && outputSet.getIsException().booleanValue()) {
				index++;
				if (outputSet == ops) {
					break;
				}
			}
		}

		return index;
	}
} //ElementDeclarationRuleImpl
