/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.xsd.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDSchema;
import org.eclipse.xsd.XSDTypeDefinition;

import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.xsd.ClassRule;
import com.ibm.btools.te.ilm.heuristics.xsd.DataDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.xsd.XSDSchemaRule;
import com.ibm.btools.te.ilm.heuristics.xsd.XsdFactory;
import com.ibm.btools.te.ilm.heuristics.xsd.XsdPackage;
import com.ibm.btools.te.ilm.heuristics.xsd.util.DataDefinitionUtil;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>XSD Schema Rule</b></em>'.
 * Source = BOM Type
 * Target = XSD Schema
 * Transform a BOM Type to a XSD Schema
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class XSDSchemaRuleImpl extends AbstractDataDefRuleImpl implements XSDSchemaRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected XSDSchemaRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return XsdPackage.Literals.XSD_SCHEMA_RULE;
	}

	/**
	 * Source = BOM Type
	 * Target = XSD Schema
	 * @return boolean
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");
		if(isComplete()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return true;
		}
		if(getSource() == null && getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return false;
		}
		Type type = (Type)getSource().get(0);		
		
		DataDefinitionRule ddr = getAndClaimDataDefinitionRule(type);
		addXSDTypeDefToSchema(ddr,type);
		processChildTargets();
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}
	
	/**
	 * if the DataDefinitionRule is a child rule of this rule instance, either this rule
	 * created it or it claimed it, then we should propagate the result of the DataDefinitionRule
	 * (which is the XSDSchema) up
	 */
	private void processChildTargets() {
		if (!getChildRules().isEmpty()) {
			if (getChildRules().get(0) instanceof DataDefinitionRule) {
				getTarget().addAll(((DataDefinitionRule)(getChildRules().get(0))).getTarget());	
			}
		}
	}
	
	/**
	 * Get and Claim a DataDefinitionRule if it hasn't already been claimed
	 * @param type
	 * @return
	 */
	private DataDefinitionRule getAndClaimDataDefinitionRule(Type type) {
		DataDefinitionRule ddr = DataDefinitionUtil.getDataDefinitionRule(this,type);
		if (ddr == null) {			
			ddr = XsdFactory.eINSTANCE.createDataDefinitionRule();
			ddr.getSource().add(type);
			getChildRules().add(ddr);
			ddr.transformSource2Target();
		} else {
			// Claim the DataDefinition rule if it hasn't been claimed
			if (!(ddr.eContainer() instanceof XSDSchema)) {
				// the DataDefinitionRule has not been claimed by other XSDSchema rule, so we'll claim it here
				getChildRules().add(ddr);
			}	
		}
		return ddr;
	}
	
	/**
	 * Creates and add an XSD type to an XSD schema
	 * @param rule
	 * @param type
	 */
	private void addXSDTypeDefToSchema(DataDefinitionRule rule, Type type) {
			
		//tries to retrieve a rule with the same source rule from the context. 
		TransformationRule xsdRule = TransformationUtil.getRuleForSource(rule, type, XSDTypeDefinition.class);
		if(xsdRule == null){
			XSDTypeDefinition typeDefinition = null;
			//create a typeRule, attach it to the DataDefinitionRule and execute it
			// get the return target and process it
			ClassRule typeRule = XsdFactoryImpl.eINSTANCE.createClassRule();
			typeRule.getSource().add(type);
			rule.getChildRules().add(typeRule);
			boolean ret = rule.transformSource2Target();
			if(typeRule.getTarget()!= null && !typeRule.getTarget().isEmpty()){
			    // GJD 20060817
			    if (typeRule.getTarget().get(0) instanceof XSDElementDeclaration) {
			        typeDefinition = ((XSDElementDeclaration) typeRule.getTarget().get(0)).getType();
			    } else {
					typeDefinition =(XSDTypeDefinition) typeRule.getTarget().get(0);
			    }
			}
		}
	}
	
} //XSDSchemaRuleImpl
