package com.ibm.btools.te.ilm.rulehandler.mapping;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.Input;
import org.eclipse.wst.wsdl.Output;
import org.eclipse.wst.wsdl.PortType;
import org.eclipse.wst.wsdl.Types;
import org.eclipse.wst.wsdl.XSDSchemaExtensibilityElement;
import org.eclipse.xsd.XSDSchema;

import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.blm.ie.exprt.option.BpelOptions;
import com.ibm.btools.blm.ie.exprt.option.WpsExportOptionsBean;
import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.externalmodels.ExternalSchema;
import com.ibm.btools.bom.model.externalmodels.ExternalService;
import com.ibm.btools.bom.model.models.InformationModel;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.actions.Decision;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.ActivityEdge;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleSet;
import com.ibm.btools.model.resourcemanager.ResourceMGR;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.CrossProjectReferenceUtil;
import com.ibm.btools.te.ilm.ExportOperationConstants;
import com.ibm.btools.te.ilm.IlmPlugin;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.LinkRule;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionKeyConstants;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.util.exception.BTRuntimeException;
import com.ibm.wbit.ae.sacl.SACLRoot;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Condition;
import com.ibm.wbit.bpel.Empty;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Pick;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.bpel.Reply;
import com.ibm.wbit.bpel.Scope;
import com.ibm.wbit.bpel.Source;
import com.ibm.wbit.bpel.Switch;
import com.ibm.wbit.bpel.Target;
import com.ibm.wbit.bpelpp.impl.DisplayNameImpl;
import com.ibm.wbit.bpelpp.impl.IdImpl;
import com.ibm.wbit.bpm.map.base.BaseDocument;
import com.ibm.wbit.bpm.map.base.BaseFactory;
import com.ibm.wbit.bpm.map.base.Descriptor;
import com.ibm.wbit.bpm.map.base.LiteralValue;
import com.ibm.wbit.bpm.map.base.impl.BaseFactoryImpl;
import com.ibm.wbit.bpm.map.objectdefinition.EMFRef;
import com.ibm.wbit.bpm.map.objectdefinition.ObjectDefinition;
import com.ibm.wbit.bpm.map.objectdefinition.ObjectDefinitions;
import com.ibm.wbit.bpm.map.objectdefinition.URIRef;
import com.ibm.wbit.bpm.map.objectdefinition.impl.ObjectDefinitionFactoryImpl;
import com.ibm.wbit.bpm.map.objectdefinition.util.ObjectDefinitionResourceFactoryImpl;
import com.ibm.wbit.bpm.trace.model.BPMConstants;
import com.ibm.wbit.businesscalendar.ICalendar;
import com.ibm.wsspi.sca.scdl.DocumentRoot;
import com.ibm.wsspi.sca.scdl.wsdl.WSDLPortType;

/**
 * The MappingManager provides functions for managing (creating, updating, removing, saving) 
 * ObjectDefinitions.
 */
public class MappingManager {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	private static final String MAPPING_MANAGER_REGISTRY="com.ibm.btools.te.ilm.rulehandler.mapping.MAPPING_MANAGER_REGISTRY";
	private static final String OBJECT_DEFINITIONS_REGISTRY="com.ibm.btools.te.ilm.rulehandler.mapping.OBJECT_DEFINITIONS_REGISTRY";

	private static final String PROP_QUALIFIED_PATH="modelPath";

	public static final String DECISION_MERGE_POINT="DECISION_MERGE_POINT_REGISTRY_KEY";
	public static final String DESCRIPTOR_SEPARATOR = "%";
	public static final String DESCRIPTOR_RELATED_PIN_SET = "relatedPinSet";

	public static HashMap uriMappings =null;
	public static HashMap objectDefMappings = null;
	private static MappingManager instance = null;
	private TransformationContext context=null;	
	private static HashMap internalLinks;

	// List of definitions
	private  static List definitions = null;
	//Map that holds all the ObjectDefinitions keyed by the URIRef value
	private static HashMap objDefsRegistry = null;
	private static List tarUID = null;
	
	private static final String BUSINESS_RULE_SET_TYPE = "http:///com/ibm/btools/bom/model/processes/businessrules.ecore#BusinessRuleSet";

	/**
	 * Returns the unique instance of the MappingManager stored in the <code>context</code>
	 * @param context a TransformationContext
	 * @return the unique instance of the MappingManager stored in the <code>context</code>
	 */
	public static MappingManager getInstance(TransformationContext context) {
		instance = (MappingManager)context.get(MAPPING_MANAGER_REGISTRY);
		if (instance==null) {
			instance = new MappingManager(context);
			context.put(MAPPING_MANAGER_REGISTRY,instance);
		}
		return instance;
	}

	/**
	 * This method is used outside of transformation context. The client invoker will
	 * be outside of the transformation context. If the instance is null, an illegalStateException
	 * will be raised.
	 * Internal method is able to directly call this method
	 */
	public static MappingManager getInstance(){
		if(instance == null){
			IllegalStateException ise = new IllegalStateException();
			String params[]={MappingManager.class.getName()};
			LoggingUtil.logError(MessageKeys.ERROR_RETRIEVE_MAPPING_MANAGER, params, ise);
			BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_RETRIEVE_MAPPING_MANAGER));
			runtimeEx.setChainedException(ise);			
			throw runtimeEx;			
		}
		return instance;
	}

	/**
	 * Constructor
	 * @param context
	 */
	private MappingManager(TransformationContext context) {
		this.context=context;
		init();		
	}

	private void init(){
		definitions = new ArrayList();
		tarUID = new LinkedList();
		uriMappings = new HashMap(20);
		internalLinks = new HashMap(20);
		MappingRegistry mapReg = MappingRegistry.getMappingRegistry(context);
		definitions.add(mapReg.getObjectDefinitions());
		objDefsRegistry = mapReg.getObjectRegistry();
	}

	/**
	 * This method will be called when mapping manager finishs save the ODF resources
	 *
	 */
	public static void reset(){
		instance = null;
		definitions = null;
		tarUID = null;
		uriMappings = null;
		internalLinks = null;
		objDefsRegistry = null;
	}

	/**
	 * Creates an ObjectDefinition wrapped for the <code>eObj</code> with the <code>uid</code>
	 * Set timestamp for top level bom elements
	 * @param eObj a BOM or a BPEL/WSDL/XSD Object		
	 * @param uid uniqui ID for the ObjectDefinition	
	 * @return the objectDefinition
	 */
	public ObjectDefinition createObjectDefinition(EObject eObj, String uid) {		
		ObjectDefinition objDef = ObjectDefinitionFactoryImpl.eINSTANCE.createObjectDefinition();	
		ResourceMGR resourceManager = ResourceMGR.getResourceManger();		
		if(eObj instanceof NamedElement){
			NamedElement ne = (NamedElement)eObj;
			String projectName = MappingUtil.getProjectNameForBOM(ne);
			if(projectName != null){
				if(eObj instanceof Activity ){
					Activity theActivity = (Activity)eObj;
					StructuredActivityNode san = theActivity.getImplementation();
					if(BomUtils.isTopLevelProcess(san) ||					
							BomUtils.isReusableService(san)||
							BomUtils.isServiceOperation(san)
					){			
						objDef.setTimeStamp(resourceManager.getLastModifid(projectName,theActivity.getUid()));						
					}
				}if(eObj instanceof CallBehaviorAction){
					if(BomUtils.isReusableTask((CallBehaviorAction)eObj)){
						objDef.setTimeStamp(resourceManager.getLastModifid(projectName,((CallBehaviorAction)eObj).getBehavior().getUid()));						
					}
				}
				else if(eObj instanceof Class){
					Class clazz = (Class)eObj;	
					if(eObj instanceof PackageableElement ||
							"SchemaType".equals(clazz.getAspect())){			
						objDef.setTimeStamp(resourceManager.getLastModifid(projectName, clazz.getUid()));
						//AZ 20070607 set the flag to indicate feedback the object as root element
						objDef.setFeedbackAsRoot(true);
					}			
				}else if(eObj instanceof ExternalSchema){
					ExternalSchema extSchema = (ExternalSchema)eObj;				
					objDef.setTimeStamp(resourceManager.getLastModifid(projectName, extSchema.getUid()));
					objDef.setReportContentChange(true);
				}else if(eObj instanceof ExternalService){
					ExternalService extService = (ExternalService)eObj;				
					objDef.setTimeStamp(resourceManager.getLastModifid(projectName, extService.getUid()));
					objDef.setReportContentChange(true);
					//AZ 20070518 add business item catalog support as root elements
				}else if(eObj instanceof InformationModel){
					InformationModel iModel = (InformationModel)eObj;
					objDef.setTimeStamp(resourceManager.getLastModifid(projectName, iModel.getUid()));
					//Add project name as descriptor
					//TODO Use BPMConstant.DESCRIPTOR_ORIGINAL_PROJECT_NAME
					//objDef.getDescriptor().add(createDescriptor("originalProjectName", projectName));//MappingUtil.getProjectUID(projectName)
					objDef.getDescriptor().add(createDescriptor("originalProjectName", MappingUtil.getProjectUID(projectName)));
				}
			}
		}else if(eObj instanceof Definition || eObj instanceof PortType){
			if(MappingUtil.getCallBackDefinitionRegistry(context)!= null &&
					MappingUtil.getCallBackDefinitionRegistry(context).contains(eObj)){
				//We need to update the descriptor for this definition OD				
				objDef.getDescriptor().add(createDescriptor(MappingUtil.CALL_BACK_WSDL_FLAG, "true"));
			}
		}
		objDef.setUid(uid);		
		//AZ Set the type of the objectdefinition based on the Eobject package uri and 
		//its class name
		objDef.setType(URIUtil.createTypeForEobject(eObj));
		EMFRef emfRef = ObjectDefinitionFactoryImpl.eINSTANCE.createEMFRef();
		emfRef.setEObject(eObj);
		objDef.setObjectReference(emfRef);
		return objDef;
	}
	
	public MappingRegistry getMappingRegistry() {
		return MappingRegistry.getMappingRegistry(context);
	}

	/**
	 * Adds an Register the ObjectDefinition in the MappingRegistry
	 * @param eObj a BOM/BPEL/WSDL/XSD object	
	 * @param objDef the wrapper ObjectDefinition
	 */
	public void addObjectDefinition(EObject eObj,ObjectDefinition objDef) {
		if (getObjectDefinition(eObj)==null) {
			MappingRegistry mapReg = MappingRegistry.getMappingRegistry(context);
			ObjectDefinitions objDefs = mapReg.getObjectDefinitions();
			objDefs.getObjectDefinition().add(objDef);
			mapReg.registerObjectDefinition(eObj,objDef);					
		}
		//Stores the object definition keyed by uid
		uriMappings.put(objDef.getUid(), objDef);		
	}

	/**
	 * Retrieves the ObjectDefinition for the <code>eObj<code> from the context.
	 * @param eObj
	 * @return an ObjectDefinition
	 */
	public ObjectDefinition getObjectDefinition(EObject eObj) {
		MappingRegistry mapReg = MappingRegistry.getMappingRegistry(context);
		return mapReg.getObjectDefinition(eObj);
	}

	/**
	 * Associate the 2 object definitions.
	 * @param src source object Definition	
	 * @param tar target object Definition
	 */
	private void addAssociatedObject(ObjectDefinition src, ObjectDefinition tar){
		List associatedTargets = src.getReferencedDefinitions();
		List uriRefs = new LinkedList();
		List emfRefs = new LinkedList();
		for (Iterator it=associatedTargets.iterator();it.hasNext();) {
			URIRef uriRef = (URIRef)it.next();
			uriRefs.add(uriRef.getUri());
		}

		if(!uriRefs.contains(tar.getUid())){
			URIRef uriRef = ObjectDefinitionFactoryImpl.eINSTANCE.createURIRef();
			uriRef.setUri(tar.getUid());
			associatedTargets.add(uriRef);
		}

		List associatedSources = tar.getReferencedDefinitions();
		for (Iterator it=associatedSources.iterator();it.hasNext();) {
			URIRef uriRef = (URIRef)it.next();
			uriRefs.add(uriRef.getUri());
		}

		if(!uriRefs.contains(src.getUid())){
			URIRef uriRef = ObjectDefinitionFactoryImpl.eINSTANCE.createURIRef();
			uriRef.setUri(src.getUid());
			associatedSources.add(uriRef);
		}		
	}
	
	/**
	 * Convience method which creates a mapping for an scdl import.  The difference is that the project name can be overridden in the case
	 * that we're trying to create a mapping for dependant projects 
	 * @param src
	 * @param tar
	 * @param targetProjectName override
	 */
	public void createMapDefinitionForSCDLImport(EObject src, EObject tar, String targetProjectName) {
		createMapDefinition(src, tar);
		CrossProjectReferenceUtil.registerTranformTargetWithProjectName(src, tar, targetProjectName);
		
	}
	/**
	 * Creates the ObjectDefinition for the <code>src</code> and <code>tar</code> and the association between them
	 * @param src an EObject BOM Object
	 * @param tar an EObject TE target Object
	 */
	public void createMapDefinition(EObject src, EObject tar) {		
		ObjectDefinition srcOD = getObjectDefinition(src);
		if (srcOD==null) {
			String uid = UIDUtil.getObjectDefinitionUID(src);
			srcOD = createObjectDefinition(src,uid);
			srcOD.setSource(true);
			addObjectDefinition(src,srcOD);
			//CrossProjectReferenceUtil.registerObjectDefintionForProjectName(src, srcOD);
		}
		ObjectDefinition tarOD = getObjectDefinition(tar);
		if (tarOD==null) {
			String uid = UIDUtil.getObjectDefinitionUID(src,tar);
			int i=1;
			while(tarUID.contains(uid)){
				uid = uid+i;
			}
			tarOD = createObjectDefinition(tar,uid);				
			tarOD.setSource(false);
			addObjectDefinition(tar,tarOD);
			//CrossProjectReferenceUtil.registerObjectDefintionForProjectName(tar, tarOD);
			tarUID.add(uid);
		}
		if(srcOD.isReportContentChange()){
			tarOD.setReportContentChange(true);
		}
		addAssociatedObject(srcOD, tarOD);
		EObject p = tar.eClass().getEStructuralFeature("eExtensibilityElements") ;
		if(tar.eClass().getEStructuralFeature("eExtensibilityElements") != null){
			EList list = (EList)tar.eGet(tar.eClass().getEStructuralFeature("eExtensibilityElements"));
			if(list != null && !list.isEmpty()){
				Iterator it = list.iterator();
				while(it.hasNext()){
					EObject extElement = (EObject) it.next();
					if(extElement instanceof DisplayNameImpl && !(extElement instanceof IdImpl)){
						addMapDefinition(src, extElement);
					}
				}
			}		
			//do something			
		}		
		if(src instanceof Decision && tar instanceof Switch ){			
			// BPMConstants.DESCRIPTOR_BOM_MERGE_OD
			//The string will be mergepoint: eProxyURI+type+bomName
			if(srcOD != null && tarOD != null){				
				addMergePointDescriptor(src,srcOD);
				addMergePointDescriptor(src,tarOD);
			}
		}
		if(src instanceof InputPinSet && tar instanceof Pick){
			Action action = ((InputPinSet)src).getAction();
			ObjectDefinition srcActionOD = null;
			if(BomUtils.isProcess(action)){
				srcActionOD = getObjectDefinition(((StructuredActivityNode)action).getActivity());				
			}else{
				srcActionOD = getObjectDefinition(action);				
			}		
			if(srcActionOD != null && tarOD != null){	
				addMergePointDescriptor(action,srcActionOD);
				addMergePointDescriptor(action,tarOD);
			}				
		}
	}


	private void addMergePointDescriptor(EObject srcBOM, ObjectDefinition objectDefinition) {
		if(objectDefinition == null ||objectDefinition.getDescriptor()== null|| 
				isDescriptorExist(objectDefinition.getDescriptor(),BPMConstants.DESCRIPTOR_BOM_MERGE_OD)){
			return;
		}
		if(context.get(DECISION_MERGE_POINT) instanceof HashMap){
			Map mergePointMap = (HashMap)context.get(DECISION_MERGE_POINT) ;
			if(mergePointMap.get(srcBOM) instanceof NamedElement){
				NamedElement mergePoint = (NamedElement)mergePointMap.get(srcBOM);
				String mergePointDescriptorString = null;
				if(mergePoint != null){
					String resourceID =  ResourceMGR.getResourceManger().getIDRecord(mergePoint).getId();
					String objectID = mergePoint.getUid();				   
					if(resourceID != null && objectID != null ){
						mergePointDescriptorString = resourceID+ ".xmi"+"#"+objectID; 
						Action mergeAction = null;
						if(mergePoint instanceof OutputPinSet){
							OutputPinSet ops = (OutputPinSet)mergePoint;
							mergeAction = ops.getAction();
						}else if(mergePoint instanceof Activity){
							mergeAction =( (Activity)mergePoint).getImplementation();
						}else if(mergePoint instanceof InputPinSet){
							InputPinSet ips = (InputPinSet)mergePoint;
							mergeAction = ips.getAction();
						}else if(mergePoint instanceof StructuredActivityNode){
							mergeAction = (StructuredActivityNode)mergePoint;
						}
						mergePointDescriptorString +=DESCRIPTOR_SEPARATOR+URIUtil.createTypeForEobject(mergeAction);
						mergePointDescriptorString +=DESCRIPTOR_SEPARATOR+mergeAction.getName();
					}					
				}
				Descriptor mergePointDescriptor = null;
				if(mergePointDescriptorString != null){
					mergePointDescriptor = createDescriptor(BPMConstants.DESCRIPTOR_BOM_MERGE_OD, mergePointDescriptorString);
				}
				if(mergePointDescriptor != null){
					objectDefinition.getDescriptor().add(mergePointDescriptor);
				}
			}
		}

	}	
	
	private boolean isDescriptorExist(EList descriptors,String descriptorName){
		Iterator it = descriptors.iterator();
		while(it.hasNext()){
			Descriptor descriptor = (Descriptor)it.next();
			if(descriptorName.equals(descriptor.getName())){
				return true;
			}
		}
		return false;
	}	
	/**
	 * Sort the predefined project objectdefinition, deep copy the source objectdefinition tree as 
	 * child objectdefinition is referenced
	 * @param objectDefList
	 * @return
	 */
	private static List sortTopLevelObjectDefintion(List objectDefList){
		List sortedList = new ArrayList();
		List tempList = new ArrayList();
		if(objectDefList == null ||objectDefList.isEmpty()){
			return sortedList;
		}
		Iterator odIt = objectDefList.iterator();
		while(odIt.hasNext()){
			ObjectDefinition eachOD = (ObjectDefinition)odIt.next();
			if(eachOD.isSource()){
				while(eachOD.eContainer() instanceof ObjectDefinition){
					eachOD = (ObjectDefinition)eachOD.eContainer();
				}
				if(!tempList.contains(eachOD)){
					tempList.add(eachOD);
					sortedList.add(EcoreUtil.copy(eachOD));
				}
			}else{
				sortedList.add(eachOD);
			}
		}
		return sortedList;
	}

	/**
	 * Registers the src and tar ObjectDefinitions.
	 * @param src
	 * @param tar
	 */
	public void addMapDefinition(EObject src, EObject tar) {
		CrossProjectReferenceUtil.registerTranformTargetWithProjectName(src, tar);
		if(MappingUtil.shouldSkipMapping(src)||tar instanceof SACLRoot||MappingUtil.shouldSkipMapping(tar)){
			return;
		}
		//Only one interface corresponding to its BOM source, we don't create redundant target ODs in order to avoid
		//dangling reference during OD persistence
		if((src instanceof ExternalService || src instanceof Activity) && tar instanceof Definition){
			ObjectDefinition srcOD = getObjectDefinition(src);			
			if(srcOD != null){
				return;
			}
		}
		if(src != null && tar != null){
			
			if (tar instanceof ICalendar)
			{			
				ObjectDefinition srcOD = getObjectDefinition(src);
				if (srcOD==null) {
					String uid = UIDUtil.getObjectDefinitionUID(src);
					srcOD = createObjectDefinition(src,uid);
					srcOD.setSource(true);
					addObjectDefinition(src,srcOD);
					//CrossProjectReferenceUtil.registerObjectDefintionForProjectName(src, srcOD);
				}
			} else {			
				createMapDefinition(src,tar);
			}
		}
	}

	/**
	 * Registers the <code>link</code> with the <code>bpelProcess</code>. The  links are stored in a 
	 * hashMap where the key is the Link and the value is the Rule that has created  this Link.
	 * @param link
	 * @param connectableRule
	 * @param bpelProcess
	 */
	public void addInternalLinks(EObject link, EObject connectableRule, EObject bpelProcess){
		HashMap links = getInternalLinksForKey(bpelProcess);
		links.put(link, connectableRule);
	}

	/**
	 * Returns all the Links register for the <code>process</code>
	 * @param process
	 * @return
	 */
	private HashMap getInternalLinksForKey(EObject process){
		HashMap links = (HashMap)internalLinks.get(process);
		if(links == null){
			links = new HashMap(20);
		}
		internalLinks.put(process,links);
		return links;
	}

	public void addInternalLinksKey(EObject process){
		getInternalLinksForKey(process);
	}	

	/**
	 * Update the object definition by adding a descriptor of object qualified path
	 * named as 'modelPath'
	 *
	 */
	public static void updateObjectQualifiedPath () {
		if (objDefsRegistry!=null) {
			Set set = objDefsRegistry.keySet();
			if (set!=null) {
				for (Iterator it=set.iterator();it.hasNext();) {
					EObject eObj = (EObject)it.next();
					ObjectDefinition objDef = (ObjectDefinition)objDefsRegistry.get(eObj);
					objDef.getDescriptor().add(instance.createDescriptor(PROP_QUALIFIED_PATH,MappingUtil.getObjectQualifiedPath(eObj)));
					//AZ add descriptor for the object name, only applicable to top level elements
					if(eObj instanceof NamedElement ){					
						objDef.getDescriptor().add(instance.createDescriptor(BPMConstants.DESCRIPTOR_OBJECT_NAME,((NamedElement)eObj).getName()));
					}
				}			
			}			
		}
	}
	/**
	 * The object definition hierarchy will be updated based on 
	 * the BOM object hierarchy, also some secondary artifacts object definition
	 * will be removed
	 *
	 */
	public static void updateObjectDefinitionHierarchy() {	
		if (objDefsRegistry!=null) {			
			Set set = objDefsRegistry.keySet();			
			List toBeRemovedObjectDefinition = new ArrayList(); 
			HashMap lastMinuteObjectDefinition = new HashMap();
			for (Iterator it=set.iterator();it.hasNext();) {
				EObject eObj = (EObject)it.next();			
				if (eObj == null) {
					continue;	//TODO GJD 20080401 investigate why
				}
				if (eObj instanceof Link || eObj instanceof BPELVariable || eObj instanceof Switch || eObj instanceof Empty || eObj instanceof Scope) {
					if (eObj.eContainer()==null) {
						toBeRemovedObjectDefinition.add(eObj);
					}
				}	
				if(!MappingUtil.isBOMObject(eObj)){						
					Object container = MappingUtil.getTopLevelElement(eObj);
					if(eObj instanceof DisplayNameImpl && !(container instanceof com.ibm.wbit.bpel.Process) && !(container instanceof ICalendar)){
						toBeRemovedObjectDefinition.add(eObj);						
					}						
				}
				if (!isRootObject(eObj) || eObj instanceof BusinessRuleSet) {
					// if it is not a root object, find it's container ObjectDefinition and add to it
					ObjectDefinition parentObjDef = findParentObjectDefinition(eObj);
					if (parentObjDef != null) {
						ObjectDefinition objDef = (ObjectDefinition)objDefsRegistry.get(eObj);						
						parentObjDef.getObjectDefinition().add(objDef);					
					} else {
						// Special handling for PartnerLinkType because we generate the Artifact.wsdl after transformation
						// ... see ExportOperation.  Hence there is no ObjectDefinition created and let's create one here.
//						if (eObj instanceof PartnerLinkType) {
//						Definition artifactDef = (Definition)eObj.eContainer();
//						ObjectDefinition objDef = (ObjectDefinition)objDefsRegistry.get(eObj);
//						parentObjDef = (ObjectDefinition)lastMinuteObjectDefinition.get(artifactDef);
//						if (parentObjDef == null) {
//						parentObjDef = ObjectDefinitionFactoryImpl.eINSTANCE.createObjectDefinition();
//						// TODO: need to find the src object that transform this artifact wsdl def ... 
//						// which should be the Process object.  The use both the src and tar object to generate the UID 
//						parentObjDef.setUid(UIDGenerator.getUID(UIDUtil.UID_OBJECT_PREFIX));
//						parentObjDef.setType(artifactDef.eClass().getInstanceClassName());
//						EMFRef emfRef = ObjectDefinitionFactoryImpl.eINSTANCE.createEMFRef();
//						emfRef.setEObject(artifactDef);
//						parentObjDef.setObjectReference(emfRef);
//						ObjectDefinitions objDefs = (ObjectDefinitions)objDef.eContainer();
//						objDefs.getObjectDefinition().add(parentObjDef);
//						lastMinuteObjectDefinition.put(artifactDef,parentObjDef);
//						}

//						parentObjDef.getObjectDefinition().add(objDef);
//						}
					}
					//AZ 20070531 handle the hierarchy for business item and data catalog
				}else if(eObj instanceof Class){					
					ObjectDefinition parentObjDef = findInformaModelObjectDefinition(((Class)eObj));
					if (parentObjDef != null) {
						ObjectDefinition objDef = (ObjectDefinition)objDefsRegistry.get(eObj);						
						parentObjDef.getObjectDefinition().add(objDef);					
					} 
				}
				if(isRootObject(eObj)&& !isBusinessItemClass(eObj)){
					CrossProjectReferenceUtil.registerObjectDefintionForProjectName(eObj, (ObjectDefinition)objDefsRegistry.get(eObj));
				}
			}			
			for (Iterator it=toBeRemovedObjectDefinition.iterator();it.hasNext();) {
				EObject obj = (EObject)it.next();
				ObjectDefinition objDef = (ObjectDefinition)objDefsRegistry.get(obj);
				List sources = objDef.getReferencedDefinitions();
				if(sources != null && !sources.isEmpty()){
					List newSources = new LinkedList(sources);
					Iterator it1 = newSources.iterator();
					while(it1.hasNext()){
						URIRef def = (URIRef)it1.next();
						//	EMFRef def = (EMFRef)it1.next();
						ObjectDefinition srcObj = (ObjectDefinition)uriMappings.get(def.getUri());
						//	ObjectDefinition srcObj = def.getObjectDefinition();
						if(srcObj != null){
							List srcURIRefs = srcObj.getReferencedDefinitions();
							List copy = new LinkedList(srcURIRefs);
							Iterator it2 = copy.iterator();
							while(it2.hasNext()){
								URIRef tarDef = (URIRef)it2.next();
//								EMFRef tarDef = (EMFRef)it2.next();
//								if(objDef.equals(tarDef.getObjectDefinition())){
//								srcURIRefs.remove(tarDef);
//								}
								if(objDef.getUid().equals(tarDef.getUri())){
									srcURIRefs.remove(tarDef);
								}
							}
							//we only added this source for the target we just eliminitated.. so eliminate the source also
							//TODO check
							if(srcObj.getObjectDefinition().size() == 0 && srcObj.getReferencedDefinitions().size() == 0) removeObject(srcObj);
						}

					}

				}
				removeObject(objDef);
				objDefsRegistry.remove(obj);
			}		
		}
	}

	private static boolean isBusinessItemClass(EObject obj) {
		if(obj instanceof Class){
			Class clazz = (Class)obj;
			if(clazz.getOwningPackage() instanceof InformationModel){
				return true;
			}	
		}
		return false;
	}

	/** 
	 * Removes an <code>objDef</code> from its container.
	 * @param objDef
	 */
	private static void removeObject(EObject objDef) {

		if (objDef.eContainer() != null) {
			if (objDef.eContainer().eGet(objDef.eContainmentFeature()) instanceof List){
				List list = (List)objDef.eContainer().eGet(objDef.eContainmentFeature());
				list.remove(objDef);
			} else {
				/* If the container isn't a list, just set the new containment value directly */
				objDef.eContainer().eSet(objDef.eContainmentFeature(), null);
			}
		}		

	}
	/**
	 * Returns the first container for this object that has an ObjectDefinition.
	 * @param eObj
	 * @return
	 */
	private static ObjectDefinition findParentObjectDefinition(EObject eObj) {
		eObj = eObj.eContainer();
		ObjectDefinition parentObjDef = null;
		while (parentObjDef==null && eObj!=null) {
			parentObjDef=(ObjectDefinition)objDefsRegistry.get(eObj);			
			eObj = eObj.eContainer();
		}
		return parentObjDef;

	}

	private static ObjectDefinition findInformaModelObjectDefinition(Class clazz){
		if(clazz.getOwningPackage() instanceof InformationModel){
			InformationModel dataCatalog = (InformationModel)clazz.getOwningPackage();		
			return (ObjectDefinition)objDefsRegistry.get(dataCatalog);
		}
		return null;
	}
	/**
	 * Returns true if this is a root BOM Element
	 * @param eObj
	 * @return
	 */
	private static boolean isRootObject(EObject eObj) {
		if (eObj.eContainer()==null) {
			return true;
		} else if (eObj instanceof PackageableElement) { // BOM top level element
			return true;
		}
		return false;
	}

	/** 
	 * Saves all objectDefinitions. The ObjectDefinitions pertaining to BPEL Objects will be saved based on their namespace, in the target folder. 
	 * All BOM ObjectDefinitions are saved in one file (or 2 files). The name of the file will follow the name convention as the following:
	 * Top-Level Module Project:  MainMod_root_objDef.odf
	 * Implementation Module Project:ImplMod_root_objDef.odf
	 * Shared Library Project: SharedLib_root_objDef.odf
	 * @param dirPath
	 * @param objectFilePathMap
	 * @param moduleName
	 * @param libraryName
	 */
	//AZ 20070215 this timestamp represents the export session
	public static void save(String dirPath, HashMap objectFilePathMap, Map sourcePathToTargetPathAbsoluteRegistry) {
		Map projectNameToODRegistry = CrossProjectReferenceUtil.getProjectNameToObjectDefinitionsMap();
		//defect 84111, make sure we reuse the resourceSet across Projects
		ResourceSet resSet = new ResourceSetImpl();
		if(projectNameToODRegistry != null && !projectNameToODRegistry.isEmpty()){
			Set projectNameSet = projectNameToODRegistry.keySet();
			for(Iterator it = projectNameSet.iterator();it.hasNext();){
				String projectName = (String)it.next();		
				if(projectName == null){
					continue;
				}
				List objectDefinitionList = (List)projectNameToODRegistry.get(projectName);		

				String exportSessionTimeStamp = String.valueOf(System.currentTimeMillis());
		//		ResourceSet resSet = new ResourceSetImpl();
				String moduleName=BpelOptionsUtil.getWPSModuleName(projectName);
				String libraryName=BpelOptionsUtil.getWPSLibraryName(projectName);
				String implModuleName=BpelOptionsUtil.getImplementationModuleName(projectName);			
				String moduleFileName = null;
				String libFileName = null;
				String implFileName = null;
				ObjectDefinitions rootObjectDefinition = null;
				if(moduleName != null) {
					moduleFileName = moduleName +File.separator + BPMConstants.OBJECT_DEFINITION_ROOT_FILE_NAME_TOP_MOD;
				}
				if(libraryName != null) {
					libFileName = libraryName +File.separator + BPMConstants.OBJECT_DEFINITION_ROOT_FILE_NAME_SHARED_LIB;
				}
				ExportSession exportSession = BpelOptionsUtil.getExportSession();
				if(BpelOptions.isBestPracticePatternEnabed(exportSession)){			
					implFileName = BpelOptionsUtil.getImplementationModuleName(projectName) +File.separator + BPMConstants.OBJECT_DEFINITION_ROOT_FILE_NAME_IMPL_MOD;
				}
				if (objectDefinitionList ==null) {
					return;
				}
				if(CrossProjectReferenceUtil.isPredefinedProject(projectName)){
					objectDefinitionList = sortTopLevelObjectDefintion(objectDefinitionList);
				}
				//for (Iterator odIt=objectDefinitionList.iterator();odIt.hasNext();) {
				//BaseDocument baseDef = (BaseDocument)odIt.next();			
				//if (baseDef instanceof ObjectDefinitions) {
				//List objDefs = objectDefinitionList;//((ObjectDefinitions)baseDef).getObjectDefinition();		
				for (int i=objectDefinitionList.size()-1;i>=0;i--) {
					ObjectDefinition objDef = (ObjectDefinition)objectDefinitionList.get(i);				
					ObjectDefinitions newDefs = ObjectDefinitionFactoryImpl.eINSTANCE.createObjectDefinitions();
					
					//skip businessruleset OD
					if(BUSINESS_RULE_SET_TYPE.equalsIgnoreCase(objDef.getType())){
						continue;
					}
					
					//AZ 20070215
					newDefs.setTimeStamp(exportSessionTimeStamp);
					newDefs.getObjectDefinition().add(objDef);	
					EObject eObj = ((EMFRef)objDef.getObjectReference()).getEObject();	
					MappingUtil.updateContainerLocation(objDef, sourcePathToTargetPathAbsoluteRegistry);					
					String filePath = (String)objectFilePathMap.get(eObj);	
					// Ray d162625
					String fileExtension = getFileExtensionFromObject(eObj,filePath);
					if(filePath == null){
						Object container = getContainer(eObj, objectFilePathMap);
						filePath = (String)objectFilePathMap.get(container);	
						fileExtension = getFileExtensionFromObject((EObject)container);
					}

					//AZ skip resource creation for lib only export option
					if(MappingUtil.shouldSkipResourceCreation(dirPath,filePath, projectName)){
						continue;
					}

					String fileName = null;
					if (filePath!=null) {
						int k=filePath.lastIndexOf(".");						
						fileName = filePath.substring(0,k)+ fileExtension + BPMConstants.OBJECT_DEFINITION_FILE_SUFFIX + "."+ BPMConstants.OBJECT_DEFINITION_FILE_EXTENSION;
					} else {
						//this is the BOM Element. All BOM Elements should be collected in the same file.
						if(moduleName != null){
							fileName  = moduleFileName;
						}else if(libraryName != null){
							fileName =libFileName;
						}else{
							fileName =projectName + BPMConstants.OBJECT_DEFINITION_FILE_SUFFIX + "." + BPMConstants.OBJECT_DEFINITION_FILE_EXTENSION;
						}
					}
					//capture the root odf instance and add descriptors accordingly
					if(fileName.indexOf(BPMConstants.OBJECT_DEFINITION_ROOT_FILE_NAME_TOP_MOD)!= -1 ||
							fileName.indexOf(BPMConstants.OBJECT_DEFINITION_ROOT_FILE_NAME_SHARED_LIB) != -1){																	
						newDefs.getDescriptor().add(getInstance().createDescriptor(BPMConstants.DESCRIPTOR_PROJECT_UID, MappingUtil.getProjectUID(projectName)));
						
						// Save export options in case we need them for shell sharing compare/merge
						String exportOption = null;
						if(BpelOptions.getWpsOptions(exportSession).getExportOption() == WpsExportOptionsBean.MOD_LIB)
							exportOption = BPMConstants.MODELER_EXPORT_OPTION_MODULEPROJECT_PROJECTLIBRARY;
						else if(BpelOptions.getWpsOptions(exportSession).getExportOption() == WpsExportOptionsBean.TOP_IMP_LIB)
							exportOption = BPMConstants.MODELER_EXPORT_OPTION_RECOMMENDED;
						else if(BpelOptions.getWpsOptions(exportSession).getExportOption() == WpsExportOptionsBean.LIB_ONLY)
							exportOption = BPMConstants.MODELER_EXPORT_OPTION_PROJECTLIBRARY;
						else if(BpelOptions.getWpsOptions(exportSession).getExportOption() == WpsExportOptionsBean.MOD_ONLY)
							exportOption = BPMConstants.MODELER_EXPORT_OPTION_MODULEPROJECT;
						
						if(exportOption != null)
							newDefs.getDescriptor().add(getInstance().createDescriptor(BPMConstants.DESCRIPTOR_EXPORT_OPTION, exportOption));
						
						if(moduleName != null)
							newDefs.getDescriptor().add(getInstance().createDescriptor(BPMConstants.DESCRIPTOR_EXPORT_TOP_MOD_PROJECT_NAME, moduleName));
						
						if(implModuleName != null)
							newDefs.getDescriptor().add(getInstance().createDescriptor(BPMConstants.DESCRIPTOR_EXPORT_IMPL_MOD_PROJECT_NAME, implModuleName));
						
						if(libraryName != null)
							newDefs.getDescriptor().add(getInstance().createDescriptor(BPMConstants.DESCRIPTOR_EXPORT_SHARED_LIB_PROJECT_NAME, libraryName));
						
						if(BpelOptionsUtil.isPartialExport()||
								(CrossProjectReferenceUtil.getReferencingProjects(projectName)!=null && CrossProjectReferenceUtil.getReferencingProjects(projectName).length !=0)){
							newDefs.getDescriptor().add(getInstance().createDescriptor(BPMConstants.DESCRIPTOR_PARTIAL_EXPORTT,"true"));
						}
						if(fileName.indexOf(BPMConstants.OBJECT_DEFINITION_ROOT_FILE_NAME_TOP_MOD)!= -1 && BpelOptions.isBestPracticePatternEnabed(exportSession)){
							MappingUtil.updateRootObjectDefinitionDescriptor(getInstance(),newDefs,BpelOptionsUtil.getImplementationModuleName(projectName), true);
						}
						
						List selectedElements = (List)exportSession.getExportOptions().getAdditionalOption("SELECTED_ELEMENTS");
						if (selectedElements != null
								&& selectedElements.size() == 1
								&& selectedElements.get(0) instanceof Activity) {
							newDefs.getDescriptor().add(getInstance().createDescriptor("topElementUID", ((Activity)selectedElements.get(0)).getUid()));
						}
					}					
					//Resource res = createResource(newDefs,fileName,dirPath, resSet);						
					createResource(newDefs,fileName,dirPath, resSet, objDef);
				}
//				} 
//				}		
				for (Iterator resrcIt=resSet.getResources().iterator();resrcIt.hasNext();) {
					Resource res = (Resource)resrcIt.next();					
					try {
						Map options = new HashMap();
						options.put(XMIResource.OPTION_DECLARE_XML, Boolean.TRUE);
						options.put(XMIResource.OPTION_ENCODING, "UTF-8");
						res.save(options);				
					} catch (Exception ioe) {				
						if ((ioe instanceof IOException) &&
								(res != null && res.getURI() != null) &&
								(res.getURI().toString().length() > 255)) {
							IlmPlugin.getDefault().getLog().log(new Status(IStatus.WARNING, IlmPlugin.getDefault().toString(), 
									res.getURI().toString() + " can't be created because the file path is too long"));
						}
						else {
							String params[]={"save module resource failed"};
							LoggingUtil.logError(MessageKeys.ERROR_SAVE_ODF_FILE, params, ioe);
							BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_SAVE_ODF_FILE));
							runtimeEx.setChainedException(ioe);			
							throw runtimeEx;
						}
					}
				}
				if(moduleName != null && libraryName != null){
					//we must copy the file into the Lib folder also
					URI moduleFileURI = URI.createFileURI(dirPath+ moduleFileName);
					Resource modResource = resSet.getResource(moduleFileURI, false);
					// d84458 - copy the contents of modResource so they can be added to the impl mode and shared lib resources.
					List rootImplContents = new ArrayList(EcoreUtil.copyAll(modResource.getContents()));
					List rootLibContents = new ArrayList(EcoreUtil.copyAll(modResource.getContents()));
					URI libFileURI = URI.createFileURI(dirPath+ libFileName);
					Resource libResource = resSet.getResource(libFileURI, false);
					if(libResource == null){
						libResource = resSet.createResource(libFileURI);
					}

					//AZ WMT 20070323 Create root file for implementation module			
					if(BpelOptions.isBestPracticePatternEnabed(exportSession)){													
						URI implFileURI = URI.createFileURI(dirPath+ implFileName);
						Resource implResource = resSet.getResource(implFileURI, false);
						if(implResource == null){
							implResource = resSet.createResource(implFileURI);
						}
						if(BpelOptions.isBestPracticePatternEnabed(exportSession)){
							MappingUtil.updateImplModuleContent(instance,rootImplContents,moduleName);
						}
						implResource.getContents().addAll(rootImplContents);				
						try {
							Map options = new HashMap();
							options.put(XMIResource.OPTION_DECLARE_XML, Boolean.TRUE);
							options.put(XMIResource.OPTION_ENCODING, "UTF-8");
							implResource.save(options);					
						} catch (Exception ioe) {					
							if ((ioe instanceof IOException) &&
									(implResource != null && implResource.getURI() != null) &&
									(implResource.getURI().toString().length() > 255)) {
								IlmPlugin.getDefault().getLog().log(new Status(IStatus.WARNING, IlmPlugin.getDefault().toString(), 
										implResource.getURI().toString() + " can't be created because the file path is too long"));										
							}														
							else {
								String params[]={"save implementation resource failed"};
								LoggingUtil.logError(MessageKeys.ERROR_SAVE_ODF_FILE, params, ioe);
								BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_SAVE_ODF_FILE));
								runtimeEx.setChainedException(ioe);			
								throw runtimeEx;
							}
						}
					}
					//move the entire content for now in the Lib resource
					if(BpelOptions.isBestPracticePatternEnabed(exportSession)){
						MappingUtil.updateLibModuleContent(rootLibContents);
					}
					libResource.getContents().addAll(rootLibContents);
					try {
						Map options = new HashMap();
						options.put(XMIResource.OPTION_DECLARE_XML, Boolean.TRUE);
						options.put(XMIResource.OPTION_ENCODING, "UTF-8");
						libResource.save(options);					
					} catch (Exception ioe) {
						if ((ioe instanceof IOException) &&
								(libResource != null && libResource.getURI() != null) &&
								(libResource.getURI().toString().length() > 255)) {
							IlmPlugin.getDefault().getLog().log(new Status(IStatus.WARNING, IlmPlugin.getDefault().toString(), 
									libResource.getURI().toString() + " can't be created because the file path is too long"));
						}
						else {	
							String params[]={"save library resource failed"};
							LoggingUtil.logError(MessageKeys.ERROR_SAVE_ODF_FILE, params, ioe);
							BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_SAVE_ODF_FILE));
							runtimeEx.setChainedException(ioe);			
							throw runtimeEx;
						}
					}
				}	
				
				//AZ If there is an empty project there, we delete the root project and the root OD files
				consolidateRootObjectDefinition(dirPath,moduleName,libraryName,implModuleName, resSet);	// d86818 - pass resource set	
			}				
		}		
		//clean up mapping manager after save
		reset();
	}

	private static void consolidateRootObjectDefinition(String dirPath, String moduleName, String libraryName, String implModuleName, ResourceSet rs) {
		if(dirPath == null ||moduleName==null||libraryName==null||implModuleName == null){
			return;
		}
		consolidateProjectRoot(dirPath, moduleName, rs);
		consolidateProjectRoot(dirPath, libraryName, rs);
		consolidateProjectRoot(dirPath, implModuleName, rs);
	}

	private static void consolidateProjectRoot(String dirPath, String projectName, ResourceSet rs) {
		if(CrossProjectReferenceUtil.existProjectDirectory(dirPath, projectName)){
			File projectRootDir = new File(dirPath+projectName);
			if(projectRootDir.isDirectory()){
				String[] children = projectRootDir.list();			   
				if(children != null && children.length == 1) {
					if( children[0].indexOf(BPMConstants.OBJECT_DEFINITION_ROOT_FILE_NAME_TOP_MOD)!=-1 ||
							children[0].indexOf(BPMConstants.OBJECT_DEFINITION_ROOT_FILE_NAME_IMPL_MOD)!=-1||
							children[0].indexOf(BPMConstants.OBJECT_DEFINITION_ROOT_FILE_NAME_SHARED_LIB)!=-1){
						deleteDir(projectRootDir, rs); // d86818
					}
				}
			}			
		}
	}

	public static boolean deleteDir(File dir) {
		return deleteDir(dir, null); // d86818
	}
	
	private static boolean deleteDir(File dir, ResourceSet rs) { // d86818
		if (dir.isDirectory()) {
			String[] children = dir.list();
			for (int i=0; i<children.length; i++) {
				boolean success = deleteDir(new File(dir, children[i]), rs);
				if (!success) {
					return false;
				}
			}
		}else if(rs != null){ 
 			// d86818 - if we have a resource set, make sure we remove the EMF
 			// resurce corresponding to the file being deleted so we don't serialize
 			// it again.
			URI path = URI.createFileURI(dir.getAbsolutePath());
			Resource resource = rs.getResource(path, false);
			if(resource != null){
				rs.getResources().remove(resource);
			}
		}

		// The directory is now empty so delete it
		return dir.delete();
	}

	/**
	 * This method is used to look up the parent document root for the wsdl porttype
	 * @param obj
	 * @param objectFilePathMap TODO
	 * @return
	 */
	private static Object getContainer(EObject obj, Map objectFilePathMap) {
		if(obj instanceof WSDLPortType){
			while(obj.eContainer()!= null){
				if(obj.eContainer() instanceof DocumentRoot){
					return obj.eContainer();
				}
				obj = obj.eContainer();
			}
		}	
		if(obj instanceof Output){
			Output output = (Output)obj;
			if(output.getEMessage() != null){
				if(output.getEMessage().eContainer() instanceof Definition){
					Definition msgContainer = (Definition)output.getEMessage().eContainer();
					return msgContainer;
				}				
			}				
		}
		if(obj instanceof Input){
			Input input = (Input)obj;
			if(input.getEMessage() != null){
				if(input.getEMessage().eContainer() instanceof Definition){
					Definition msgContainer = (Definition)input.getEMessage().eContainer();
					return msgContainer;
				}				
			}			
		}
		if(obj instanceof XSDSchema){
			//handle inline schema case
			if(obj.eContainer() instanceof XSDSchemaExtensibilityElement){
				if(obj.eContainer().eContainer() instanceof Types){
					if(obj.eContainer().eContainer().eContainer() instanceof Definition){
						Definition container = (Definition)obj.eContainer().eContainer().eContainer();
						//look up the same deinifition but different instance in the registry
						//however, the location must be the same
						if(objectFilePathMap != null && !objectFilePathMap.isEmpty()){
							Set keys = objectFilePathMap.keySet();
							for(Iterator keyIt=keys.iterator();keyIt.hasNext();){
								Object eachKey = keyIt.next();
								if(eachKey instanceof Definition){
									Definition  registeredDef = (Definition)eachKey;								
									if(registeredDef.getLocation() != null &&
											registeredDef.getLocation().equals(container.getLocation())){
										return registeredDef;
									}
								}
							}
						}
					}
				}
			}
		}
		return null;
	}	

	/**
	 * Compute the file extension from the <code>EObject</code> and handle the special case for tel vs itel
	 * @param obj
	 * @return
	 */
	// Ray - d162625
	private static String getFileExtensionFromObject(EObject obj,String filePath) {
		if(obj instanceof com.ibm.wbit.tel.DocumentRoot && filePath!=null && filePath.endsWith(ExportOperationConstants.ITEL_FILE_EXT)){
			return "_itel";
		}else if(obj instanceof com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup && filePath!=null && filePath.endsWith(ExportOperationConstants.BRG_FILE_EXT)){
			return "_brg";
		}else if(obj instanceof com.ibm.wbiservers.businessrule.model.brgt.BusinessRuleGroupTable && filePath!=null && filePath.endsWith(ExportOperationConstants.BRGT_FILE_EXT)){
			return "_brgt";
		}else if(obj instanceof com.ibm.wbit.br.core.model.RuleSet && filePath!=null && filePath.endsWith(ExportOperationConstants.RULESET_FILE_EXT)){
			return "_ruleset";			
		}else if(obj instanceof com.ibm.wbit.br.core.model.DocumentRoot && filePath!=null && filePath.endsWith(ExportOperationConstants.RULESET_FILE_EXT)){
			return "_ruleset";					
		} else {
			return getFileExtensionFromObject(obj);
		}
		
		
	}
	/**
	 * Compute the file extension from the <code>EObject</code>
	 * @param obj
	 * @return
	 */
	private static String getFileExtensionFromObject(EObject obj) {
		String fileExtension = "";

		if(obj instanceof Process){
			fileExtension = "_bpel";
		}else if(obj instanceof Definition){
			fileExtension = "_wsdl";
		}else if (obj instanceof DocumentRoot){
			if(((DocumentRoot)obj).getComponent() != null){
				fileExtension = "_component";
			}else if(((DocumentRoot)obj).getExport() != null){
				fileExtension = "_export";
			}else if(((DocumentRoot)obj).getImport() != null){
				fileExtension = "_import";
			}
		}else if(obj instanceof XSDSchema){
			fileExtension = "_xsd";
		}else if(obj instanceof com.ibm.wbit.tel.DocumentRoot){
			fileExtension = "_tel";
		}

		return fileExtension;
	}

	/**
	 * Creates an EMF Resource and add the content of the <code>doc</code> inside.
	 * @param doc
	 * @param fileName
	 * @param dirPath
	 * @param resSet
	 * @param objDef TODO
	 * @return
	 */
	private static void createResource(BaseDocument doc, String fileName, String dirPath, ResourceSet resSet, ObjectDefinition objDef) {
		if (doc!=null) {
			URI fileURI = null;
			//If the file name is already a absolute path, we don't  have to prepend dir path
			EObject eObj = ((EMFRef)objDef.getObjectReference()).getEObject();
			String originalProject = CrossProjectReferenceUtil.getProjectNameForTargetObject(eObj);
			//String[] projectNames = CrossProjectReferenceUtil.getReferencingProjects(originalProject);	
			//defect 82982 skip this checking for d2d. 
			if(CrossProjectReferenceUtil.isImportDocumentRoot(eObj) && !BpelOptionsUtil.isGenerateForDeployPatternEnabled()){
//				String[] projectNames = CrossProjectReferenceUtil.getDependantProjectFor(eObj);
//				for(int i=0;i<projectNames.length;i++){
//					String eachProjectName = projectNames[i];
//					if(originalProject.equals(eachProjectName)){
//						NamedElement target = CrossProjectReferenceUtil.getBOMSOurceObjectFor(eObj);
//						//d155310
//						if(CrossProjectReferenceUtil.shouldSkipCreateImport(target)){
//							continue;
//						}
//					}
//					String targetModuleProject = BpelOptionsUtil.getWPSModuleName(eachProjectName);
//					//defect 164678
//					if(targetModuleProject == null ||targetModuleProject.length() == 0){
//						continue;
//					}
//					String relativePath = null;
//					if(fileName.startsWith(dirPath.substring(0,dirPath.length()-1))){
//						relativePath = fileName.substring(dirPath.length());
//						if(relativePath.indexOf("\\") != -1){
//							String temp = relativePath.substring(relativePath.indexOf("\\"));
//							//Consolidated relative path
//							relativePath = targetModuleProject+temp;
//						}
//					}else{
//						if(fileName.indexOf("\\") != -1){
//							String temp = fileName.substring(fileName.indexOf("\\"));
//							//Consolidated relative path
//							relativePath = targetModuleProject+temp;
//						}
//					}	
//					fileName=dirPath+ relativePath;
//					fileURI = URI.createFileURI(fileName);
//					Resource resource = resSet.getResource(fileURI, false);
//					ObjectDefinitionResourceFactoryImpl  resourceFact = new ObjectDefinitionResourceFactoryImpl();
//					//XMIResourceFactoryImpl resourceFact = new XMIResourceFactoryImpl();
//					if(resource == null){
//						resource = resourceFact.createResource(fileURI);
//					}
//					if(resource.getContents().isEmpty()){
//						resource.getContents().add(doc);
//					}else{
//						if(resource.getContents().get(0) instanceof ObjectDefinitions && doc instanceof ObjectDefinitions){
//							((ObjectDefinitions)resource.getContents().get(0)).getObjectDefinition().addAll(((ObjectDefinitions)doc).getObjectDefinition());
//						}
//					}
//					resSet.getResources().add(resource);						
//				}
				
				
				//d86875 - Check to see if the import was actually created.  Because of the way imports are generated and filtered after the fact, 
				//in CrossProjectReferenceUtil.createResourceForSCDLImport, mappings might still exist for imports that were skipped resulting in
				//a dangling href exception when we save the OD because the resource for the skipped import were never created.  We were not running
				//into this problem with the old code because these mapping was implicitly skipped when we "copy" the import OD for all dependent projects 
				//by calling CrossProjectReferenceUtil.getDependantProjectFor(eObj).  In the new import mapping generation algorithm, we do not reuse the import OD
				//but rather generate a new one for each import, thus we no longer needed to call getDependantProjectFor().
				//Thus, we need to check for these invalid import mappings and skip over them
				if (eObj.eResource() == null) {
					return;
				}
				
				//This check is no longer needed and valid because we don't reuse Document roots anymore (a copy is generated)
//				NamedElement target = CrossProjectReferenceUtil.getBOMSOurceObjectFor(eObj);
//				//d155310
//				if(CrossProjectReferenceUtil.shouldSkipCreateImport(target)){
//					return;
//				}
				
				String targetModuleProject = BpelOptionsUtil.getWPSModuleName(originalProject);
				//defect 164678
				if(targetModuleProject == null ||targetModuleProject.length() == 0){
					return;
				}
				String relativePath = null;
				if(fileName.startsWith(dirPath.substring(0,dirPath.length()-1))){
					relativePath = fileName.substring(dirPath.length());
					if(relativePath.indexOf("\\") != -1){
						String temp = relativePath.substring(relativePath.indexOf("\\"));
						//Consolidated relative path
						relativePath = targetModuleProject+temp;
					}
				}else{
					if(fileName.indexOf("\\") != -1){
						String temp = fileName.substring(fileName.indexOf("\\"));
						//Consolidated relative path
						relativePath = targetModuleProject+temp;
					}
				}	
				fileName=dirPath+ relativePath;
				fileURI = URI.createFileURI(fileName);
				Resource resource = resSet.getResource(fileURI, false);
				ObjectDefinitionResourceFactoryImpl  resourceFact = new ObjectDefinitionResourceFactoryImpl();
				//XMIResourceFactoryImpl resourceFact = new XMIResourceFactoryImpl();
				if(resource == null){
					resource = resourceFact.createResource(fileURI);
				}
				if(resource.getContents().isEmpty()){
					resource.getContents().add(doc);
				}else{
					if(resource.getContents().get(0) instanceof ObjectDefinitions && doc instanceof ObjectDefinitions){
						((ObjectDefinitions)resource.getContents().get(0)).getObjectDefinition().addAll(((ObjectDefinitions)doc).getObjectDefinition());
					}
				}
				resSet.getResources().add(resource);						
//				}
			}else{
				if(dirPath.indexOf("/") != -1){
					dirPath = dirPath.replace("/", File.separator);
				}
				if(fileName.startsWith(dirPath.substring(0,dirPath.length()-1))){
					fileURI = URI.createFileURI(fileName);
				}else{
					fileURI = URI.createFileURI(dirPath+fileName);
				}
				//try to retrieve the resource first. This is the case of the BusinessObject defs that are to be saved with
				//the BomProcess.		
				Resource resource = resSet.getResource(fileURI, false);
				//XMIResourceFactoryImpl resourceFact = new XMIResourceFactoryImpl();
				ObjectDefinitionResourceFactoryImpl resourceFact = new ObjectDefinitionResourceFactoryImpl();
				if(resource == null){
					resource = resourceFact.createResource(fileURI);
				}
				if(resource.getContents().isEmpty()){
					resource.getContents().add(doc);
				}else{
					if(resource.getContents().get(0) instanceof ObjectDefinitions && doc instanceof ObjectDefinitions){
						((ObjectDefinitions)resource.getContents().get(0)).getObjectDefinition().addAll(((ObjectDefinitions)doc).getObjectDefinition());
					}
				}
				resSet.getResources().add(resource);
				//return resource;
			}
		}		
		//return null;
	}
	/**
	 * Creates a Descriptor with the given name and value
	 * @param name
	 * @param value
	 * @return
	 */
	public  Descriptor createDescriptor(String name, String value){
		Descriptor desc=BaseFactoryImpl.eINSTANCE.createDescriptor();
		desc.setName(name);
		LiteralValue litValue = BaseFactoryImpl.eINSTANCE.createLiteralValue();
		litValue.setValue(value);
		desc.setValue(litValue);
		return desc;
	}
	
	/**
	 * Test if the descriptor exists with the given name
	 * @param od
	 * @param name
	 * @return
	 */
	public boolean existsDescriptor(ObjectDefinition od,String name){
		if(od == null || name == null){
			return false;
		}
		if(od.getDescriptor()!= null && !od.getDescriptor().isEmpty() ){
			Iterator<Descriptor> descIt = od.getDescriptor().iterator();
			while(descIt.hasNext()){
				Descriptor eachDesc = (Descriptor)descIt.next();
				if(name.equals(eachDesc.getName())){
					return true;
				}
			}
		}
		return false;
	}

	private void handleUnaccountedLinks(){
		Set processes = internalLinks.keySet();
		Iterator processesIterator = processes.iterator();
		TransformationRule linkRule = null;
		while(processesIterator.hasNext()){
			//get all the BPEL Processes one by one and process its links
			Process bpelProcess = (Process)processesIterator.next();
			HashMap processedLinksMap = getInternalLinksForKey(bpelProcess);
			Collection allProcessedLinks =processedLinksMap.values();
			//get all the Links for this Process including children
			List<Link> allLinks = getAllLinksFromProcess(bpelProcess);
			for (Link thisLink:allLinks) {
				//eliminate all the links that are in the objectRegistry					
				addLinkSourceToTargetsMapping(thisLink);
				//defect 165172
				addLinkTargetToSourcesMapping(thisLink);
				
				//get the entry for this rule
				EObject entry = (EObject)processedLinksMap.get(thisLink);
				if(entry instanceof ActivityEdge){
					NamedElement pinSet = BomUtils.findSourceSet((ActivityEdge)entry);
					if(pinSet instanceof OutputPinSet  && ((OutputPinSet)pinSet).getAction() instanceof Decision ){
						MappingManager.getInstance().addMapDefinition(pinSet, thisLink);
						if(getConditionForLink(thisLink) != null)
							MappingManager.getInstance().addMapDefinition(pinSet, getConditionForLink(thisLink));
					}

//						this object was already mapped
//						continue;
				}else if(entry instanceof LinkRule){
//						this Link was created by a LinkRule.. we may be able associate it with an edge steming from
					//the Activity it links.. source or target
					MappingManager.getInstance().matchEdgeWithLink(thisLink, (LinkRule)entry);
//						continue;
				}else if((linkRule = getRuleForLink(bpelProcess, thisLink))!= null){
					//find and assign link to connection
					MappingManager.getInstance().matchEdgeWithLink(thisLink, linkRule);
				}else{						
					//find and assign link to connection
					MappingManager.getInstance().matchEdgeWithLink(thisLink, null);
				}

				//add the source and target to the BOM Connection
				if(thisLink.getSources() == null || thisLink.getSources().isEmpty()) continue;
				if(thisLink.getTargets() == null || thisLink.getTargets().isEmpty()) continue;
				com.ibm.wbit.bpel.Activity sourceActivity =((Source)thisLink.getSources().get(0)).getActivity();
				com.ibm.wbit.bpel.Activity targetActivity =((Target)thisLink.getTargets().get(0)).getActivity();
				ObjectDefinition linkOD = MappingManager.getInstance().getObjectDefinition(thisLink);
				if(linkOD != null){
					URIRef connectionODRef = (URIRef)linkOD.getReferencedDefinitions().get(0);
					ObjectDefinition connectionOD = (ObjectDefinition)MappingManager.getInstance().getUriMappings().get(connectionODRef.getUri());
					URIRef sourceActivityODRef = null;
					ObjectDefinition sourceActivityBOMOD = null;
					ObjectDefinition sourceActivityOD = MappingManager.getInstance().getObjectDefinition(sourceActivity);
					//defect 154923
					if(sourceActivityOD != null){
						sourceActivityODRef = (URIRef)sourceActivityOD.getReferencedDefinitions().get(0);
						sourceActivityBOMOD = (ObjectDefinition)MappingManager.getInstance().getUriMappings().get(sourceActivityODRef.getUri());
					}
					URIRef targetActivityODRef = null;
					ObjectDefinition targetActivityBOMOD = null;
					ObjectDefinition targetActivityOD = MappingManager.getInstance().getObjectDefinition(targetActivity);
//						defect 154923
					if(targetActivityOD != null){
						targetActivityODRef = (URIRef)targetActivityOD.getReferencedDefinitions().get(0);
						targetActivityBOMOD = (ObjectDefinition)MappingManager.getInstance().getUriMappings().get(targetActivityODRef.getUri());
					}
					if(sourceActivityBOMOD != null && targetActivityBOMOD != null && connectionOD != null){
						Descriptor sourceDescriptor = BaseFactoryImpl.eINSTANCE.createDescriptor();
						sourceDescriptor.setName(BPMConstants.DESCRIPTOR_SOURCE_OD);

						LiteralValue sourceValue = BaseFactory.eINSTANCE.createLiteralValue();
						sourceValue.setValue(sourceActivityBOMOD.getUid());
						sourceDescriptor.setValue(sourceValue);

						Descriptor targetdescriptor = BaseFactoryImpl.eINSTANCE.createDescriptor();
						targetdescriptor.setName(BPMConstants.DESCRIPTOR_TARGET_OD);

						LiteralValue targetValue = BaseFactory.eINSTANCE.createLiteralValue();
						targetValue.setValue(targetActivityBOMOD.getUid());
						targetdescriptor.setValue(targetValue);

						connectionOD.getDescriptor().add(sourceDescriptor);
						connectionOD.getDescriptor().add(targetdescriptor);
					}

				}
			}
		}
	}
	
	/**
	 * Add all links, even though inside Scope and While loops.
	 * @param proc
	 * @return List<link>
	 */
	private List<Link> getAllLinksFromProcess(Process proc) {
		Vector result = new Vector();
		if (proc!=null && proc.eResource()!=null) {
			EList contents = proc.eResource().getContents();
			if (!contents.isEmpty()) {
				Iterator it = EcoreUtil.getAllContents(contents);
				while (it.hasNext()) {
					Object obj = it.next();
					if (obj instanceof Link) {
						result.add((Link) obj);
					}
				}
			}
		}
		return result;
		
	}
    
	/**
	 * Add link source which is normally an InputPinSet to its BPEL targets mapping
	 * @param thisLink
	 */
	private void addLinkSourceToTargetsMapping(Link thisLink) {
		if(thisLink.getTargets() == null || thisLink.getTargets().isEmpty()) return;
		com.ibm.wbit.bpel.Activity targetActivity = ((Target)thisLink.getTargets().get(0)).getActivity();
		if(targetActivity instanceof com.ibm.wbit.bpel.Activity){
			ObjectDefinition  invokeOD = getObjectDefinition(targetActivity);
			if(invokeOD != null){
				URIRef invokeODRef = (URIRef)invokeOD.getReferencedDefinitions().get(0);
				ObjectDefinition ipsOD = (ObjectDefinition)MappingManager.getInstance().getUriMappings().get(invokeODRef.getUri());
				if(ipsOD != null){
					if(ipsOD.getObjectReference() instanceof EMFRef){
						EMFRef emfRef = (EMFRef)ipsOD.getObjectReference();
						EObject ipsEObj = emfRef.getEObject();
						if(!ipsEObj.eIsProxy()){
							com.ibm.wbit.bpel.Targets targets = ((com.ibm.wbit.bpel.Activity)((Target)thisLink.getTargets().get(0)).getActivity()).getTargets();
							MappingManager.getInstance().addMapDefinition(ipsEObj, targets);
						}
					}
				}
			}
		}
	}

	/**
	 * Add link source which is normally an InputPinSet to its BPEL targets mapping
	 * @param thisLink
	 */
	private void addLinkTargetToSourcesMapping(Link thisLink) {
		if(thisLink.getSources() == null || thisLink.getSources().isEmpty()) return;
		com.ibm.wbit.bpel.Activity sourceActivity = ((Source)thisLink.getSources().get(0)).getActivity();
		//if(targetActivity instanceof Invoke){
		if(sourceActivity instanceof com.ibm.wbit.bpel.Activity){
			ObjectDefinition  invokeOD = getObjectDefinition(sourceActivity);
			if(invokeOD != null){
				URIRef invokeODRef = (URIRef)invokeOD.getReferencedDefinitions().get(0);
				ObjectDefinition ipsOD = (ObjectDefinition)MappingManager.getInstance().getUriMappings().get(invokeODRef.getUri());
				if(ipsOD != null){
					if(ipsOD.getObjectReference() instanceof EMFRef){
						EMFRef emfRef = (EMFRef)ipsOD.getObjectReference();
						EObject ipsEObj = emfRef.getEObject();
						if(!ipsEObj.eIsProxy()&& ipsEObj instanceof InputPinSet){
							com.ibm.wbit.bpel.Sources sources = ((com.ibm.wbit.bpel.Activity)((Source)thisLink.getSources().get(0)).getActivity()).getSources();
							MappingManager.getInstance().addMapDefinition(ipsEObj, sources);
						}
					}
				}
			}
		}
	}
	
	private void matchEdgeWithLink(Link thisLink, TransformationRule rule){
		Target target = (Target)thisLink.getTargets().get(0);
		com.ibm.wbit.bpel.Activity bpelTarget = target.getActivity();
		Source sources = (Source)thisLink.getSources().get(0);
		com.ibm.wbit.bpel.Activity bpelSource = sources.getActivity();
		ActivityEdge candidate = null;
		if(rule != null){
			//it is possible that the source or target of this Rule is one of the linked Activities.
			List ruleSources = rule.getSource();
			for(Iterator it = ruleSources.iterator(); it.hasNext();){
				//this should be either a Input or an OutputPinSet
				EObject pinSet = (EObject)it.next();
				Action activity = null;
				if(pinSet instanceof OutputPinSet){
					List pins = BomUtils.getPinsInSet((PinSet)pinSet);
					for(Iterator it2 = pins.iterator(); it2.hasNext();){
						Pin pin = (Pin)it2.next();
						if(!BomUtils.isProcess((Action)pinSet.eContainer()) ){
							candidate = pin.getOutgoing();
						}else{
							candidate = pin.getIncoming();
						}
						if(candidate == null) continue;
						if(targetOfCandidateMatchesActivity(candidate, bpelTarget, rule)){
							MappingManager.getInstance().addMapDefinition(candidate, thisLink);
							//if this Link is the Link stemming from a Decision, register the decision outputSet with the Link
							if(((OutputPinSet)pinSet).getAction() instanceof Decision){
								MappingManager.getInstance().addMapDefinition(pinSet, thisLink);
								MappingManager.getInstance().addMapDefinition(pinSet, getConditionForLink(thisLink));

							}
							return;
						}
					}
				}else if(pinSet instanceof InputPinSet){
					List pins = BomUtils.getPinsInSet((PinSet)pinSet);
					for(Iterator it2 = pins.iterator(); it2.hasNext();){
						Pin pin = (Pin)it2.next();
						candidate = pin.getIncoming();
						if(candidate == null) continue;
						if(targetOfCandidateMatchesActivity(candidate, bpelSource, rule)){
							MappingManager.getInstance().addMapDefinition(candidate, thisLink);
							return;
						}
					}
					//defect 167898
				}else if(pinSet instanceof StructuredActivityNode){
					List pins = BomUtils.getPinsInSet((PinSet)((StructuredActivityNode)pinSet).getOutputPinSet().get(0));
					for(Iterator it2 = pins.iterator(); it2.hasNext();){
						Pin pin = (Pin)it2.next();
						candidate = pin.getIncoming();
						if(candidate == null) continue;
						if(targetOfCandidateMatchesActivity(candidate, bpelTarget, rule)){
							MappingManager.getInstance().addMapDefinition(candidate, thisLink);
							//if this Link is the Link stemming from a Decision, register the decision outputSet with the Link
							return;
						}
					}
				}
			}
		}
	}

	private boolean targetOfCandidateMatchesActivity(ActivityEdge candidate, com.ibm.wbit.bpel.Activity bpelTarget, TransformationRule rule){
		//the activity this candidate edge is connected to
		EObject node = candidate.getTarget().eContainer();
		//defect 167898
		if(node != null && node instanceof StructuredActivityNode && BomUtils.isProcess((StructuredActivityNode)node) && bpelTarget instanceof Reply){
			return true;
		}
		//get all the rules that have the SAN .. check if the bpel source is target
		Collection rules = TransformationUtil.getRulesForSource(ProcessUtil.getProcessDefinitionRule(rule.getRoot().getContext()), node);
		for(Iterator it = rules.iterator(); it.hasNext();){
			List targets = ((TransformationRule)it.next()).getTarget();
			if(targets.contains(bpelTarget)){
				return true;
			}
		}
		return false;
	}

	private Condition getConditionForLink(Link link){		
		return ((Source)link.getSources().get(0)).getTransitionCondition();		
	}

	private boolean existLinkRuleForLinkProcess(Process bpelProcess, Link thisLink){
		HashMap linkMap = getInternalLinksForKey(bpelProcess);
		if(linkMap.get(thisLink) != null){
			return true;
		}else{
			return false;
		}
	}

	private TransformationRule getRuleForLink(Process bpelProcess, Link thisLink){
		TransformationRule linkRule = null;
		HashMap linkMap = getInternalLinksForKey(bpelProcess);
		linkRule = (TransformationRule)linkMap.get(thisLink);
		return linkRule;		
	}

	/**
	 * This method encapsulates the pre-save processing logic, which will be called
	 * before saving map model
	 *
	 */
	public static void postProcessObjectDefinitions(){
		if (MappingManager.objDefsRegistry!=null) {
			MappingManager.getInstance().handleUnaccountedLinks();
			MappingManager.updateObjectQualifiedPath();
			MappingManager.addWSDLFileNameDescriptorToPortTypeOD();
			MappingManager.updateObjectDefinitionHierarchy();			
		}
	}

	/**
	 * 
	 *
	 */
	private static void addWSDLFileNameDescriptorToPortTypeOD() {
		List portTypeList = null;
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null){
			return;
		}
		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
		if(additionalOptions != null){
			Object registry = additionalOptions.get(TransformationSessionKeyConstants.PORTTYPE_LIST_REGISTRY_KEY);
			if(registry instanceof List){
				portTypeList = (List)registry;
			}
		}
		if(portTypeList != null && !portTypeList.isEmpty()){
			Iterator ptIterator = portTypeList.iterator();
			while(ptIterator.hasNext()){
				PortType pt = (PortType)ptIterator.next();
				addPortTypeFileDescriptorFor(pt);
			}
		}
		
		
	}
	
	/**
	 * Add descriptor on invoke: BPMConstants.DESCRIPTOR_PORT_TYPE_FILE
	 * @param mappingManager
	 * @param beplInvoke
	 * @param isCBA TODO
	 */
	private static void addPortTypeFileDescriptorFor(PortType portType){
		if(getInstance() == null || portType == null){
			return;
		}
		ObjectDefinition ptOD = getInstance().getObjectDefinition(portType);
		if(ptOD != null){
			EObject portTypeContainer = portType.eContainer();
			if(portTypeContainer instanceof Definition){
				Definition containerDef = (Definition)portTypeContainer;
				String wsdlFileName = containerDef.getLocation();
				if(wsdlFileName != null && wsdlFileName.length()>0 && wsdlFileName.indexOf("/")>0){
					wsdlFileName = wsdlFileName.substring(wsdlFileName.lastIndexOf("/")+1);
				}else if(wsdlFileName == null){
					wsdlFileName = containerDef.getQName().getLocalPart();
					wsdlFileName += ".wsdl";
				}
				if(wsdlFileName != null && wsdlFileName.length() > 0){					
					Descriptor portTypeFileDescriptor = getInstance().createDescriptor(BPMConstants.DESCRIPTOR_PORT_TYPE_FILE, wsdlFileName);
					ptOD.getDescriptor().add(portTypeFileDescriptor);
				}
			}
		}
	}

	public static HashMap getUriMappings() {
		return uriMappings;
	}

//	/**
//	* Remove the ObjectDefinition if its referenced object is null, or doesn't have proper
//	* container
//	* @param objDefsRegistry
//	* @param toBeRemovedObjectDefinition
//	*/
//	private static void consolidateUnaccountedObjectDefinition(Map objDefsRegistry,List toBeRemovedObjectDefinition){
//	if(objDefsRegistry != null && !objDefsRegistry.values().isEmpty()){
//	Iterator valueIt = objDefsRegistry.values().iterator();
//	while(valueIt.hasNext()){
//	Object value = valueIt.next();
//	if(value instanceof ObjectDefinition){
//	ObjectDefinition odValue = (ObjectDefinition)value;
//	EObject referenceObj = MappingUtil.getReferencedEObjectFor(odValue);
//	if(referenceObj == null){
//	toBeRemovedObjectDefinition.add(odValue);
//	continue;
//	}
//	if(!MappingUtil.isBOMObject(referenceObj)){						
//	Object container = MappingUtil.getTopLevelElement(referenceObj);
//	if(referenceObj instanceof DisplayNameImpl && !(container instanceof com.ibm.wbit.bpel.Process)){
//	toBeRemovedObjectDefinition.add(referenceObj);
//	continue;
//	}						
//	}
//	}
//	}
//	for (Iterator it=toBeRemovedObjectDefinition.iterator();it.hasNext();) {
//	EObject obj = (EObject)it.next();
//	ObjectDefinition objDef;
//	if(obj instanceof ObjectDefinition){
//	objDef = (ObjectDefinition)obj;
//	}else{
//	objDef = (ObjectDefinition)objDefsRegistry.get(obj);
//	}
//	List sources = objDef.getReferencedDefinitions();
//	if(sources != null && !sources.isEmpty()){
//	List newSources = new LinkedList(sources);
//	Iterator it1 = newSources.iterator();
//	while(it1.hasNext()){
//	URIRef def = (URIRef)it1.next();					
//	ObjectDefinition srcObj = (ObjectDefinition)uriMappings.get(def.getUri());					
//	if(srcObj != null){
//	List srcURIRefs = srcObj.getReferencedDefinitions();
//	List copy = new LinkedList(srcURIRefs);
//	Iterator it2 = copy.iterator();
//	while(it2.hasNext()){
//	URIRef tarDef = (URIRef)it2.next();
//	if(objDef.getUid().equals(tarDef.getUri())){
//	srcURIRefs.remove(tarDef);
//	}
//	}
//	//we only added this source for the target we just eliminitated.. so eliminate the source also
//	if(srcObj.getObjectDefinition().size() == 0 && srcObj.getReferencedDefinitions().size() == 0) removeObject(srcObj);
//	}

//	}

//	}
//	removeObject(objDef);
//	objDefsRegistry.remove(obj);
//	}		
//	}
//	}	
}
