package com.ibm.btools.te.ilm.rulehandler.mapping;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.HashMap;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.wbit.bpm.map.objectdefinition.ObjectDefinition;
import com.ibm.wbit.bpm.map.objectdefinition.ObjectDefinitions;
import com.ibm.wbit.bpm.map.objectdefinition.impl.ObjectDefinitionFactoryImpl;

/**
 * This class represents ObjectDefinition registry in a transformation session
 */
public class MappingRegistry {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private static final String MAPPING_REGISTRY = "BPM.MAPPING.REGISTRY";
	private ObjectDefinitions objDefs=null;
	
	private HashMap objectRegistry;
	
	public static MappingRegistry getMappingRegistry(TransformationContext context) {
		MappingRegistry mpreg = (MappingRegistry)context.get(MAPPING_REGISTRY);
		if (mpreg==null) {
			mpreg = new MappingRegistry();
			context.put(MAPPING_REGISTRY,mpreg);
		}
		return mpreg;
	}

	private MappingRegistry () {
		// initialize
		objectRegistry = new HashMap();
	}
	/**
	 * Returns an ObjectDefinitions object.
	 * @return
	 */
	public ObjectDefinitions getObjectDefinitions() {
		if (objDefs == null) {
			objDefs = ObjectDefinitionFactoryImpl.eINSTANCE.createObjectDefinitions();			
		}
		return objDefs;
	}

	/**
	 * Register the <code>eObj</code> with its <code>objDef</code>.
	 * @param eObj
	 * @param objDef
	 */
	public void registerObjectDefinition(EObject eObj,ObjectDefinition objDef) {
		objectRegistry.put(eObj,objDef);
	}
	/**
	 * Returns the ObjectDefinition for this <code>eObj</code>
	 * @param eObj
	 * @return
	 */
	public ObjectDefinition getObjectDefinition(EObject eObj) {
		return (ObjectDefinition)objectRegistry.get(eObj);
	}
	/**
	 * @return Returns the objectRegistry.
	 */
	public HashMap getObjectRegistry() {
		return objectRegistry;
	}
}
