package com.ibm.btools.te.ilm.rulehandler.mapping;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.namespace.QName;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.ExtensibilityElement;
import org.eclipse.wst.wsdl.ExtensibleElement;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Part;
import org.eclipse.wst.wsdl.PortType;
import org.eclipse.xsd.XSDSchema;

import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.artifacts.PrimitiveType;
import com.ibm.btools.bom.model.artifacts.Property;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.externalmodels.ExternalSchema;
import com.ibm.btools.bom.model.externalmodels.ExternalService;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleAction;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleBlock;
import com.ibm.btools.model.filemanager.FileMGR;
import com.ibm.btools.model.modelmanager.dependencymanager.DependencyManager;
import com.ibm.btools.model.resourcemanager.ResourceMGR;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.ilm.CrossProjectReferenceUtil;
import com.ibm.btools.te.ilm.ExportOperationConstants;
import com.ibm.btools.te.ilm.ExportOperationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionKeyConstants;
import com.ibm.btools.te.ilm.model.abstractbpel.Alternative;
import com.ibm.btools.te.ilm.model.abstractbpel.AlternativeActivity;
import com.ibm.btools.te.ilm.model.sa.ProcessFlowType;
import com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup;
import com.ibm.wbiservers.businessrule.model.brgt.BusinessRuleGroupTable;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpm.map.base.Descriptor;
import com.ibm.wbit.bpm.map.base.LiteralValue;
import com.ibm.wbit.bpm.map.base.impl.BaseFactoryImpl;
import com.ibm.wbit.bpm.map.objectdefinition.EMFRef;
import com.ibm.wbit.bpm.map.objectdefinition.ObjectDefinition;
import com.ibm.wbit.bpm.map.objectdefinition.ObjectDefinitions;
import com.ibm.wbit.bpm.trace.model.BPMConstants;
import com.ibm.wbit.br.core.model.RuleSet;
import com.ibm.wbit.businesscalendar.ICalendar;
import com.ibm.wbit.tel.TTask;

/**
 * Utility class .
 */
public class MappingUtil {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	public static int guid=1;
	private static final String SCHEMATYPE_PRIVATE="SchemaType_Private";
	private static final String BUSINESS_SERVICE_OBJECT_ASPECT="SchemaType";
	private static final String CALL_BACK_WSDL_REGISTRY = "CALL_BACK_WSDL_REGISTRY";
	public static final String CALL_BACK_WSDL_FLAG = "isCallBack";
	/**
	 * Returns true if this is an BOM Element.
	 * @param eObj
	 * @return
	 */
	public static boolean isBOMObject (EObject eObj) {
		return eObj instanceof com.ibm.btools.bom.model.artifacts.Element;
	}
	/**
	 * Returns the top level container for the <code>eObject</code>
	 * @param eObj
	 * @return 
	 */
	public static EObject getTopLevelElement(EObject object) {
		while (object.eContainer() != null) {
			object = object.eContainer();
		}
		return object;
	}
	
	private static boolean isRootObject(EObject eObj) {
		if (eObj.eContainer()==null) {
			return true;
		} else if (eObj instanceof PackageableElement) { // BOM top level element
			return true;
		}
		return false;
	}
	
	/**
	 * Add to handle root bom element to support WID compare/merge UI
	 * The qualified path pattern will be "projectName/catalogName1/catalogName2/"
	 * up to the root element's catalog
	 * @param eObj
	 * @return
	 */
	private static String getRootObjectQualifiedPath(EObject eObj){
		StringBuffer sb = new StringBuffer();
		if(eObj instanceof NamedElement){
			com.ibm.btools.bom.model.artifacts.Package pkg = ((PackageableElement)eObj).getOwningPackage();
			while(pkg instanceof com.ibm.btools.bom.model.artifacts.Package && !isRootModel(pkg)){				
				sb.insert(0, pkg.getName() + "/");	
				pkg = pkg.getOwningPackage();
			}			
		}
		String projectName = CrossProjectReferenceUtil.getProjectNameForBOMElement(eObj);
		if(projectName != null){
			sb.insert(0, projectName + "/");	
		}		
		return sb.toString();
	}
	
	/**
	 * Check if the package is the default root model name
	 * @param pkg
	 * @return
	 */
	public static boolean isRootModel(com.ibm.btools.bom.model.artifacts.Package pkg){
		if(pkg != null){
			String modelName = pkg.getName();
			return (IImportFramework.ROOT_PROC_MODEL_NAME.equals(modelName)||
			IImportFramework.ROOT_INFO_MODEL_NAME.equals(modelName)||
			IImportFramework.ROOT_EXTERNAL_MODEL_NAME.equals(modelName)||
			IImportFramework.ROOT_ORG_MODEL_NAME.equals(modelName)||
			IImportFramework.ROOT_RES_MODEL_NAME.equals(modelName)||
			IImportFramework.ROOT_SERV_MODEL_NAME.equals(modelName));
		}
		return false;
	}

	public static String getObjectQualifiedPath(EObject eObj) {
		if (eObj instanceof NamedElement) {
			if(isRootObject(eObj)){
				//projectName/catalogName
				return getRootObjectQualifiedPath(eObj);
			}else{
				return BomUtils.getCanonicalName((NamedElement)eObj,true,false); 
			}			
		} else {
			EObject object = eObj;
			String path="";
			while (object != null) {
				String name =  null;
				Object oName = (Object)findAttribute(object,"name");
				if(oName instanceof String){
					name = (String)oName;
				}else if(oName instanceof TTask){
					name = (String)findAttribute((EObject)oName,"name");

				}
				if (name == null) {
					//QName qname = (QName)findAttribute(object,"qName");
					Object qname = (Object)findAttribute(object,"qName");
					if (qname!=null && qname instanceof QName) {
						name = ((QName)qname).getLocalPart();
					}
				}
				if (name!=null && name.length()>0) {
					if (path.length()>0) {
						path = name+"."+path;						
					} else {
						path = name;
					}
				}
				object = object.eContainer();
			}		
			return path;
//			String className=eObj.getClass().getName();
//			if (className.startsWith("com.ibm.wbit.bpel")) {
//			return getBPELQualifiedPath(eObj);
//			} else if (className.startsWith("org.eclipse.xsd")) {
//			return getXsdQualifiedPath(eObj);
//			} else if (className.startsWith("com.ibm.wsspi.sca.scdl")) {
////			return getScdlQualifiedPath(eObj);
//			} else if (className.startsWith("com.ibm.wbit.tel")) {
////			return getTelElementURI(eObj);
//			} else {
//			return getWsdlQualifiedPath(eObj);
//			}
//			return null;
		}
	}

//	public static String getBOMQualifiedPath(NamedElement object) {
//	return BomUtils.getCanonicalName(object,true,false);
//	}

//	public static String getBPELQualifiedPath(EObject eObj) {
//	EObject object = eObj;
//	String path="";
//	while (object != null) {
//	String name = (String)findAttribute(object,"name");
//	if (name == null) {
//	QName qname = (QName)findAttribute(object,"qName");
//	if (qname!=null) {
//	name = qname.getLocalPart();
//	}
//	}
//	if (name!=null && name.length()>0) {
//	if (path.length()>0) {
//	path = name+"."+path;						
//	} else {
//	path = name;
//	}
//	}
//	object = object.eContainer();
//	}
//	return path;
//	String path = (String)findAttribute(object,"name");
//	object = object.eContainer();
//	while (!(object instanceof PortType || object instanceof Message)) {
//	path = (String)findAttribute(object,"name")+"."+path;
//	object = object.eContainer();
//	}		
//	String type = object.eClass().getName();
//	String name = (String)MappingUtil.findAttribute(object,"name");
//	String id = null;
//	// Should be implemented using generated Switch util class??
//	if (object instanceof ExtensibleElement) {
//	ExtensibilityElement ee=MappingUtil.getExtensibilityElement((ExtensibleElement)object,IdImpl.class);
//	if (ee!=null) {
//	id = ((IdImpl)ee).getId().toString();
//	} else {
//	ee=MappingUtil.getExtensibilityElement((ExtensibleElement)object,VariableIdImpl.class);
//	if (ee != null) {
//	id = ((VariableIdImpl)ee).getVariableId().toString();					
//	}
//	}
//	}
//	if (id==null) {
//	id=name;
//	}
//	return id;

//	}	

//	private static String getXsdQualifiedPath(EObject object) {
//	String path = (String)findAttribute(object,"name");
//	object = object.eContainer();
//	while (!(object instanceof XSDSchema)) {
//	String name = (String)findAttribute(object,"name");
//	if (name!=null) {
//	path = name+"."+path;				
//	}
//	object = object.eContainer();
//	}
//	// special handling for in-line schema in WSDL
//	if (object instanceof XSDSchema && object.eContainer()!=null) {
//	path=((XSDSchema)object).getTargetNamespace()+":"+path;
//	}
//	return path;
//	}

//	private static String getWsdlQualifiedPath(EObject object) {
//	String path = (String)findAttribute(object,"name");
//	while (object!=null) {
//	path = (String)findAttribute(object,"name")+"."+path;
//	object = object.eContainer();
//	}
//	QName qname = (QName)findAttribute(object,"qName");
//	if (qname!=null) {
//	path=qname.getLocalPart()+"."+path;			
//	}
//	return path;
//	}
	/**
	 * Returns the ExtensibilityElement attached to e that has the Class c
	 * 
	 * @param haystack The ExtensibleElement to search through 
	 * @param needle The Class of ExtensibilityElement to look for
	 * @return The ExtensibilityElement whose class matches c;
	 */
	public static ExtensibilityElement getExtensibilityElement(ExtensibleElement haystack, Class needle){
		ExtensibilityElement e = null;
		for (Iterator it = haystack.getEExtensibilityElements().iterator(); it.hasNext(); ){
			e = (ExtensibilityElement)it.next();
			if (e.getClass() == needle){
				return e;
			}
		}

		return null;            
	}

	public static Object findAttribute (EObject object, String attribName) {
		EAttribute attribute=findAttribute(object.eClass(),attribName);
		if (attribute != null) {
			return object.eGet(attribute);			
		} else {
			return null;
		}
	}

	public static EAttribute findAttribute(EClass eClass, String attribName) {
		List attributes = eClass.getEAttributes();
		for (Iterator it=attributes.iterator();it.hasNext();) {
			EAttribute attribute = (EAttribute)it.next();
			if (attribute.getName().equalsIgnoreCase(attribName)) {
				return attribute;
			}
		}
		for (Iterator it=eClass.getESuperTypes().iterator();it.hasNext();) {
			EAttribute attribute = findAttribute((EClass)it.next(),attribName);
			if (attribute!=null) {
				return attribute;
			}
		}
		return null;
	}

	/**
	 * Returns the alternative activity that corresponds to the given type
	 * @param altAct The AlternativeActivity whose alternatives to search
	 * @param type The type of alternative sought
	 * @return The activity of the given type, or null if none exists
	 */
	public static Activity getAlternative(AlternativeActivity altAct, ProcessFlowType type){
		Alternative act = null;
		for (Iterator it = altAct.getAlternative().iterator(); it.hasNext(); ){
			act = (Alternative)it.next();

			if (act.getType().getValue() == type.getValue()){
				return act.getActivity();
			}
		}

		return null;
	}

	/**
	 * This method is to update the root object definition file's reference project descriptor
	 * @param odefs
	 * @param isTopLevelModule
	 */
	public static void updateRootObjectDefinitionDescriptor(MappingManager instance,ObjectDefinitions odefs,String referenencedModuleName, boolean isTopLevelModule){
		if(odefs == null){
			return;
		}
		EList descriptors = odefs.getDescriptor();
		Descriptor partialExportDesc = null;
		boolean found = false;
		if(descriptors != null && !descriptors.isEmpty()){
			Iterator descIt = descriptors.iterator();    		
			while(descIt.hasNext()){
				Descriptor each = (Descriptor)descIt.next();
				if(BPMConstants.DESCRIPTOR_PROJECT_REFERENCE.equals(each.getName())){
					LiteralValue litValue = BaseFactoryImpl.eINSTANCE.createLiteralValue();
					litValue.setValue(referenencedModuleName);
					each.setValue(litValue);  
					found = true;    				
				}
				if(BPMConstants.DESCRIPTOR_PARTIAL_EXPORTT.equals(each.getName())){
					partialExportDesc = each;
				}
			}    		
		}
		if(!found){
			//Create the descriptor for the top level module     		
			if(odefs instanceof ObjectDefinitions && referenencedModuleName != null){
				ObjectDefinitions odfs =  (ObjectDefinitions)odefs;
				odfs.getDescriptor().add(instance.createDescriptor(BPMConstants.DESCRIPTOR_PROJECT_REFERENCE,referenencedModuleName));
			}
		}
		if(partialExportDesc != null && found && !isTopLevelModule){
			//descriptors.remove(partialExportDesc);
		}
	}

	/**
	 * This method is to update the descriptor of the impl. module odf
	 * @param contents
	 */
	public static void updateImplModuleContent(MappingManager instance,List contents,String topModuleName) {
		if(contents == null || contents.isEmpty()){
			return;
		}
		Object firstContent = contents.get(0);
		if(firstContent instanceof ObjectDefinitions){
			ObjectDefinitions target = (ObjectDefinitions)firstContent;
			updateRootObjectDefinitionDescriptor(instance,target,topModuleName, false);
		}


	}

	/**
	 * This method is to update the descriptor of the lib module odf
	 * @param rootImplContents
	 */
	public static void updateLibModuleContent(List rootImplContents) {
		if(rootImplContents == null || rootImplContents.isEmpty()){
			return;
		}
		List descToRemove=new ArrayList();
		Object firstContent = rootImplContents.get(0);
		if(firstContent instanceof ObjectDefinitions){
			ObjectDefinitions target = (ObjectDefinitions)firstContent;
			EList descriptors= target.getDescriptor();
			//AZ 20070528 add support for project UID
			if(descriptors != null && !descriptors.isEmpty()){
				for(Iterator descIt=descriptors.iterator();descIt.hasNext();){
					Descriptor descObj = (Descriptor)descIt.next();
					if(BPMConstants.DESCRIPTOR_PROJECT_REFERENCE.equals(descObj.getName())){
						descToRemove.add(descObj);
					}
				}
			}
			if(descToRemove != null && !descToRemove.isEmpty()){
				for(Iterator removeIt=descToRemove.iterator();removeIt.hasNext();){
					target.getDescriptor().remove(removeIt.next());
				}
			}
		}		
	}

	/**
	 * Update the container's location
	 * @param objDefinition
	 * @param sourcePathToTargetPathAbsoluteRegistry
	 */
	public static void updateContainerLocation(ObjectDefinition objDefinition,Map sourcePathToTargetPathAbsoluteRegistry){
		if(objDefinition == null||sourcePathToTargetPathAbsoluteRegistry ==null ||
				sourcePathToTargetPathAbsoluteRegistry.isEmpty()
		){
			return;
		}
		EObject eObj = ((EMFRef)objDefinition.getObjectReference()).getEObject();
		if(eObj instanceof Message){					
			//we need to update its container definition path
			Definition msgContainer = (Definition)((Message)eObj).eContainer();
			if(msgContainer != null){
				String docBaseURI = msgContainer.getDocument().getDocumentURI();
				docBaseURI = ExportOperationUtil.replaceConvertedSpaces(docBaseURI);
				if(sourcePathToTargetPathAbsoluteRegistry.containsKey(docBaseURI)){
					//update the location as the target location
					String updatedPath = (String)(((List)sourcePathToTargetPathAbsoluteRegistry.get(docBaseURI)).get(1));
					msgContainer.getDocument().setDocumentURI(updatedPath);
				}
			}
		}
		if(eObj instanceof Part){
			Definition partDefContainer = (Definition)((Part)eObj).eContainer().eContainer();
			if(partDefContainer != null){
				String location = partDefContainer.getLocation();
				location = ExportOperationUtil.replaceConvertedSpaces(location);
				if(sourcePathToTargetPathAbsoluteRegistry.containsKey(location)){
					//update the location as the target location
					String updatedPath = (String)(((List)sourcePathToTargetPathAbsoluteRegistry.get(location)).get(1));
					partDefContainer.setLocation(updatedPath);
				}
			}
		}		
	}
	/**
	 * This method is responsible for adding mapping for untransformed external service
	 * @param objectFilePathMap
	 * @param imprtDefinition
	 * @param targetPath
	 * @param absoluteImportedLocationPath
	 */
	public static void addMapForUnTransformedService(Map objectFilePathMap,
			Definition imprtDefinition,
			String targetPath,
			String absoluteImportedLocationPath){	
		if(imprtDefinition != null){
			//If the import location has been updated, the schema/wsdl has already been mapped
			if(absoluteImportedLocationPath != null && absoluteImportedLocationPath.indexOf(ExportOperationConstants.FILE_ATTACHMENT) == -1){
				return;
			}
			com.ibm.btools.bom.model.artifacts.Class attachmentBOM = resolveAttachmentBOM(convertFileURIToAbsolutePath(absoluteImportedLocationPath));
			if(attachmentBOM != null){		
				MappingManager mappingMgr = MappingManager.getInstance();
				if(mappingMgr != null && targetPath != null){
					EObject src = resolveUnreferencedService(convertFileURIToAbsolutePath(absoluteImportedLocationPath));
					if(mappingMgr.getObjectDefinition(src) == null){
						//resolveProjectNameByImportLocation(absoluteImportedLocationPath, src, imprtDefinition);
						mappingMgr.addMapDefinition(src, imprtDefinition);
						String objFilePath = targetPath.substring(0,targetPath.lastIndexOf(File.separatorChar)+1)+
						ExportOperationUtil.computeNameFromDefinitionObject(imprtDefinition);
						objectFilePathMap.put(imprtDefinition,objFilePath);
						//clean up the loaded resource
						ResourceMGR.getResourceManger().unloadMatchingResources(absoluteImportedLocationPath);
					}
				}
			}
		}
	}

	/**
	 * This method is responsible for adding mapping for untransformed external schema
	 * @param objectFilePathMap
	 * @param schema
	 * @param targetPath
	 * @param absoluteImportedLocationPath
	 */
	public static void addMapForUnTransformedSchema(Map objectFilePathMap,
			XSDSchema schema,
			String targetPath,
			String absoluteImportedLocationPath){
		if(schema != null){
			//If the import location has been updated, the schema/wsdl has already been mapped
			if(absoluteImportedLocationPath != null && absoluteImportedLocationPath.indexOf(ExportOperationConstants.FILE_ATTACHMENT) == -1){
				return;
			}
			com.ibm.btools.bom.model.artifacts.Class attachmentBOM = resolveAttachmentBOM(convertFileURIToAbsolutePath(absoluteImportedLocationPath));
			if(attachmentBOM != null){
				MappingManager mappingMgr = MappingManager.getInstance();
				if(mappingMgr != null && targetPath != null){
					EObject src = resolveUnreferencedSchema(convertFileURIToAbsolutePath(absoluteImportedLocationPath));
					if(mappingMgr.getObjectDefinition(src) == null){
						//resolveProjectNameByImportLocation(absoluteImportedLocationPath, src, schema);
						mappingMgr.addMapDefinition(src, schema);
						targetPath = convertFileURIToAbsolutePath(targetPath);
						String objFilePath = targetPath.substring(0,targetPath.lastIndexOf(File.separatorChar)+1)+
						ExportOperationUtil.computeNameFromDefinitionObject(schema);
						objectFilePathMap.put(schema,objFilePath);
						//clean up the loaded resource
						ResourceMGR.getResourceManger().unloadMatchingResources(absoluteImportedLocationPath);
					}
				}
			}
		}		
	}

	/**
	 * This method applies to the registry which only have unique value entry 
	 * @param registry
	 * @param value
	 * @return
	 */
	public static Object resolveRegistryPathKey(Map registry, Object value){
		if(registry == null || registry.isEmpty() || value == null){
			return null;
		}
		Set keys = registry.keySet();
		for(Iterator it = keys.iterator();it.hasNext();){
			Object key = it.next();
			if(registry.get(key).equals(value)){
				return key;
			}
		}
		return null;
	}

	/**
	 * 
	 * @param absoluteImportedLocationPath
	 * @return
	 */
	public static ExternalService resolveUnreferencedService(String absoluteImportedLocationPath){
		EList roots = ResourceMGR.getResourceManger().getRootObjects(getServiceModelXMIAbsoluteURI(convertFileURIToAbsolutePath(absoluteImportedLocationPath)));
		if(roots == null || roots.isEmpty()){
			return null;
		}
		Object extSvcObject = roots.get(0);
		if(extSvcObject instanceof  ExternalService){
			ExternalService temp = (ExternalService)extSvcObject;
			//Resolve the actual BOM root objects in our workspace
			List<EObject>  rootElements = ResourceMGR.getResourceManger().getAllElementsWithUID(temp.getUid());
			if(rootElements != null && !rootElements.isEmpty() && rootElements.get(0) instanceof ExternalService){
				return (ExternalService)rootElements.get(0);
			}
			
		}
		return null;
	}

	/**
	 * 
	 * @param absoluteImportedLocationPath
	 * @return
	 */
	public static ExternalSchema resolveUnreferencedSchema(String absoluteImportedLocationPath){
		EList roots = ResourceMGR.getResourceManger().getRootObjects(getServiceModelXMIAbsoluteURI(convertFileURIToAbsolutePath(absoluteImportedLocationPath)));
		if(roots == null || roots.isEmpty()){
			return null;
		}
		Object extSvcObject = roots.get(0);
		if(extSvcObject instanceof  ExternalSchema){
			ExternalSchema temp = (ExternalSchema)extSvcObject;
			//Resolve the actual BOM root objects in our workspace
			List<EObject>  rootElements = ResourceMGR.getResourceManger().getAllElementsWithUID(temp.getUid());
			if(rootElements != null && !rootElements.isEmpty()&& rootElements.get(0) instanceof ExternalSchema){
				return (ExternalSchema)rootElements.get(0);
			}			
		}
		return null;
	}

	/**
	 * 
	 * @param fileAttachmentAbsolutePath
	 * @return
	 */
	private static String getServiceModelXMIAbsoluteURI(String fileAttachmentAbsolutePath){
		String temp = null;
		if(fileAttachmentAbsolutePath == null || fileAttachmentAbsolutePath.length()== 0){
			return temp;
		}
		if(fileAttachmentAbsolutePath.indexOf(ExportOperationConstants.FILE_ATTACHMENT) != -1){
			temp = fileAttachmentAbsolutePath.substring(0,fileAttachmentAbsolutePath.indexOf(ExportOperationConstants.FILE_ATTACHMENT));
			temp = temp + ExportOperationConstants.MODEL_XMI;
		}
		return temp;
	}

	private static String convertFileURIToAbsolutePath(String fileUri){	
		if(fileUri != null && fileUri.startsWith(ExportOperationConstants.URI_FILE)&& fileUri.indexOf(ExportOperationConstants.FILE_SEPARATOR) != -1){
			fileUri = fileUri.replace(ExportOperationConstants.FILE_SEPARATOR, File.separator);
			fileUri = fileUri.substring(ExportOperationConstants.URI_FILE.length()+1);
			return fileUri;
		}
		return fileUri;
	}

	/**
	 * 
	 * @param fileAttachmentAbsolutePath
	 * @return
	 */
	private static String getModelXMIAbsoluteURI(String fileAttachmentAbsolutePath){
		String temp = null;
		if(fileAttachmentAbsolutePath == null || fileAttachmentAbsolutePath.length()== 0){
			return temp;
		}	
		temp = fileAttachmentAbsolutePath.substring(0,(fileAttachmentAbsolutePath.lastIndexOf(File.separator)+1));
		temp = temp + ExportOperationConstants.MODEL_XMI;		
		return temp;
	}

	/**
	 * 
	 * @param absoluteImportedLocationPath
	 * @return
	 */
	private static com.ibm.btools.bom.model.artifacts.Class resolveAttachmentBOM(String absoluteImportedLocationPath){
		EList roots = ResourceMGR.getResourceManger().getRootObjects(getModelXMIAbsoluteURI(absoluteImportedLocationPath));
		if(roots == null || roots.isEmpty()){
			return null;
		}
		Object extSvcAttachmentObject = roots.get(0);
		if(extSvcAttachmentObject instanceof  com.ibm.btools.bom.model.artifacts.Class){
			return (com.ibm.btools.bom.model.artifacts.Class)extSvcAttachmentObject;
		}
		return null;
	}

	/**
	 * This method is to weed out the irrelevant ODF file if the user chooses the export
	 * option as library only
	 * @param projectName TODO
	 * @param objectFilePathMap
	 * @param eObj
	 * @return
	 */
	public static  boolean shouldSkipResourceCreation(String dirPath,String filePath, String projectName){
		if(BpelOptionsUtil.isLibraryOnly()){
			if(filePath == null || dirPath == null||projectName==null){
				return false;
			}
			String libName = BpelOptionsUtil.getWPSLibraryName(projectName);
			if(libName !=null && filePath.startsWith(libName)){
				return false;
			}else if(filePath.startsWith(dirPath.substring(0,dirPath.length()-1))){
				String temp = filePath.substring(dirPath.length());
				if(libName !=null && temp.startsWith(libName)){					
					return false;
				}
			}
			return true;
		}
		return false;
	}

	/**
	 * This method handles the XSD SimpleType definition which has restriction of primitive types
	 * We will skip the mapping for those elements
	 * <xs:simpleType name="StringLength1to64">
		<xs:annotation>
			<xs:documentation xml:lang="en">Used for Character Strings, length 1 to 64</xs:documentation>
		</xs:annotation>
		<xs:restriction base="xs:string">
			<xs:minLength value="1"/>
			<xs:maxLength value="64"/>
		</xs:restriction>
	</xs:simpleType>
	 * @param property
	 * @return
	 */
	public static boolean isPrimitiveType(Property property){		
		if(property == null){
			return false;
		}		
		if(property.getType() instanceof PrimitiveType){
			return true;
		}else  if(property.getType() instanceof com.ibm.btools.bom.model.artifacts.Class){
			com.ibm.btools.bom.model.artifacts.Class clazz = ( com.ibm.btools.bom.model.artifacts.Class) property.getType();
			if(!clazz.getOwnedAttribute().isEmpty()){
				Object attr = clazz.getOwnedAttribute().get(0);
				if(attr instanceof Property){
					Property attrProperty = (Property)attr;
					if(clazz.getName() != null){
						if(clazz.getName().equals(attrProperty.getName())){
							if(attrProperty.getType() instanceof PrimitiveType){
								return true;
							}
						}
					}
				}
			}

		}
		return false;
	}

	/**
	 * We skip the mapping for private type's property
	 * @param property
	 * @return
	 */
	public static boolean isPrivateTypeProperty(Property property){
		if(property == null){
			return false;
		}
		if(property.eContainer() instanceof com.ibm.btools.bom.model.artifacts.Class){
			com.ibm.btools.bom.model.artifacts.Class container = (com.ibm.btools.bom.model.artifacts.Class)property.eContainer();
			if(SCHEMATYPE_PRIVATE.equals(container.getAspect())||BUSINESS_SERVICE_OBJECT_ASPECT.equals(container.getAspect())){
				return true;
			}
		}
		return false;
	}
    
	/**
	 * We skip the mapping for business service object and inline schema types
	 * @param type
	 * @return
	 */
	public static boolean isPrivateType(Type type){		
		if(type == null){
			return false;
		}
		if(SCHEMATYPE_PRIVATE.equals(type.getAspect()) 
				|| BUSINESS_SERVICE_OBJECT_ASPECT.equals(type.getAspect())){
			return true;
		}	
		return false;

	}

	public static EObject getReferencedEObjectFor(ObjectDefinition odf){
		if(odf == null){
			return null;
		}
		return ((EMFRef)odf.getObjectReference()).getEObject();
	}

	/**
	 * Add support for the project UID in the CMT file
	 * @param projectName
	 * @return
	 */
	public static String getProjectUID(String projectName){
		if(projectName == null || projectName.length()==0){
			return null;
		}
		String projectPath=FileMGR.getProjectPath(projectName);
		if(projectPath == null){
			return null;
		}
		if(DependencyManager.instance().getDependencyModel(projectName, projectPath).getProjectIdentifier() != null){
			return DependencyManager.instance().getDependencyModel(projectName, projectPath).getProjectIdentifier().getValue();
		}
		return null;
	}

	/**
	 * Get the project name that the bom element resides
	 * @param ne
	 * @return
	 */
	public static String getProjectNameForBOM(NamedElement ne){
		String projectName = null;
		if(ne == null){
			return projectName;
		}
		ResourceMGR resourceManager = ResourceMGR.getResourceManger();	
		List projects = resourceManager.getProjectsForUID(ne.getUid());
		if(projects !=null && !projects.isEmpty()){
			projectName = resourceManager.getProjectsForUID(ne.getUid()).get(0);
		}	
		return projectName;
	}
	
	/**
	 * SKip the map model generation for human task and business rule for now
	 * TODO need to turn off this check if we decide to support
	 * @param src
	 * @return
	 */
	public static boolean shouldSkipMapping(EObject src) {
		if( src instanceof com.ibm.btools.bom.model.processes.activities.Activity){
			if(((com.ibm.btools.bom.model.processes.activities.Activity)src).getImplementation() instanceof StructuredActivityNode){
				Action action = ((com.ibm.btools.bom.model.processes.activities.Activity)src).getImplementation();
				if(BomUtils.isBusinessRuleTask(action)|| BomUtils.isHumanTask(action)){
					return false;
				}
			}
		}else if(src instanceof com.ibm.btools.bom.model.processes.humantasks.HumanTask ||
				src instanceof BusinessRuleAction){
			return false;
		}else if(src instanceof PrimitiveType){
			return true;
		}else if (src != null && src.eContainer() instanceof Action && BomUtils.isServiceOperation((Action)src.eContainer())) {
			//skip mapping for business services if we're not D2D and if the source object isn't a fault PS and not a Input PS
			if (!BpelOptionsUtil.isGenerateForDeployPatternEnabled() && 
					!(src instanceof OutputPinSet && ((OutputPinSet)src).getIsException()) && 
					!(src instanceof InputPinSet)) {
				return true;
			}
		}else if (//(src instanceof Action && BomUtils.isServiceOperation((Action)src))||
				(src instanceof BusinessRuleBlock) ||
//				(src != null && src.eContainer() instanceof Action && BomUtils.isServiceOperation((Action)src.eContainer()) &&
//						!(src instanceof OutputPinSet && ((OutputPinSet)src).getIsException()) &&
//						!(src instanceof InputPinSet)) || 
				(src instanceof com.ibm.btools.bom.model.processes.activities.Activity && 
						BomUtils.isServiceOperation(((com.ibm.btools.bom.model.processes.activities.Activity)src).getImplementation())) ){
			return true;
		}
//			//defect 161993 suppress the business rule artifacts mapping
//		}else if(src instanceof BusinessRuleGroup || src instanceof BusinessRuleGroupTable || src instanceof RuleSet){
//			return true;
//		}else if (src instanceof ICalendar)
//			return true;
		return false;
	}
	
	/**
	 * Register the call back WSDL definition/portType in the transformation context
	 * @param context
	 * @param obj
	 */
	public static void registerCallBackArtifacts(TransformationContext context, EObject obj) {
		if(context == null || obj == null){
			return;
		}
		List<EObject> callBackWsdlRegistry = null;
		if(context.get(CALL_BACK_WSDL_REGISTRY) instanceof List){
			callBackWsdlRegistry = (List)context.get(CALL_BACK_WSDL_REGISTRY);
			if(!callBackWsdlRegistry.contains(obj)){
				callBackWsdlRegistry.add(obj);
			}
		}else{
			callBackWsdlRegistry = new ArrayList<EObject>();
			if(!callBackWsdlRegistry.contains(obj)){
				callBackWsdlRegistry.add(obj);
			}
		}
		context.put(CALL_BACK_WSDL_REGISTRY, callBackWsdlRegistry);
	}
	
	/**
	 * Retrieve call back WSDL definition from the context
	 * @param context
	 * @return
	 */
	public static List getCallBackDefinitionRegistry(TransformationContext context){
		if(context != null && context.get(CALL_BACK_WSDL_REGISTRY) instanceof List){
			return (List)context.get(CALL_BACK_WSDL_REGISTRY);
		}
		return null;
	}
	
	/**
	 * Resolve the src project name and register the project name with the src and tar
	 * @param importLocation
	 * @param src
	 * @param tar
	 */
//	private static void resolveProjectNameByImportLocation(String importLocation,EObject src,EObject tar){
//		String wbmProjectName = getProjectNameForBOM((NamedElement)src);
//		ExportSession exportSession = BpelOptionsUtil.getExportSession();
//		if(exportSession == null||importLocation == null){
//			return;
//		}
//		if( wbmProjectName != null){
//				Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
//				if(additionalOptions != null){
//					Object registry = additionalOptions.get(TransformationSessionKeyConstants.TARGET_TO_PROJECT_NAME_REGISTRY_KEY);
//					if(!(registry instanceof Map)){
//						registry = new HashMap();
//						additionalOptions.put(TransformationSessionKeyConstants.TARGET_TO_PROJECT_NAME_REGISTRY_KEY, registry);
//					}
//					if(!((Map)registry).containsKey(tar)){
//						((Map)registry).put(tar, wbmProjectName);
//					}
//				}
//				//register source to project name
//				if(additionalOptions != null){
//					Object registry = additionalOptions.get(TransformationSessionKeyConstants.SOURCE_TO_PROJECT_NAME_REGISTRY_KEY);
//					if(!(registry instanceof Map)){
//						registry = new HashMap();
//						additionalOptions.put(TransformationSessionKeyConstants.SOURCE_TO_PROJECT_NAME_REGISTRY_KEY, registry);
//					}
//					if(!((Map)registry).containsKey(src)){
//						((Map)registry).put(src, wbmProjectName);
//					}
//				}			
//
//				//Add target to soucr map
//				if(additionalOptions != null){
//					Object registry = additionalOptions.get(TransformationSessionKeyConstants.TARGET_TO_SOURCE_REGISTRY_KEY);
//					if(!(registry instanceof Map)){
//						registry = new HashMap();
//						additionalOptions.put(TransformationSessionKeyConstants.TARGET_TO_SOURCE_REGISTRY_KEY, registry);
//					}
//					if(!((Map)registry).containsKey(tar)){
//						((Map)registry).put(tar, src);
//					}
//				}				
//				if(additionalOptions != null){
//					Object registry = additionalOptions.get(TransformationSessionKeyConstants.TARGET_TO_SOURCE_REGISTRY_KEY);
//					if(!(registry instanceof Map)){
//						registry = new HashMap();
//						additionalOptions.put(TransformationSessionKeyConstants.TARGET_TO_SOURCE_REGISTRY_KEY, registry);
//					}
//					if(!((Map)registry).containsKey(tar)){
//						((Map)registry).put(tar, src);
//					}
//				}
//			}
//		}
}
