package com.ibm.btools.te.ilm.rulehandler.mapping.wps;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleAction;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleSetSelectionRecord;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup;
import com.ibm.wbiservers.businessrule.model.brgt.BRGTDateTimeRangeKey;
import com.ibm.wbiservers.businessrule.model.brgt.BusinessRuleGroupTable;
import com.ibm.wbiservers.businessrule.model.brgt.RuleSetSelectionData;
import com.ibm.wbiservers.common.selectiontables.OperationSelectionRecord;
import com.ibm.wbiservers.common.selectiontables.OperationSelectionTable;
import com.ibm.wbiservers.common.selectiontables.SelectionData;
import com.ibm.wbiservers.common.selectiontables.SelectionKey;
import com.ibm.wbit.br.core.model.RuleSet;
/**
 This handler is responsbile for creating the ObjectDefinition mapping between
 *the source and target: 
 *SOURCE: BusinessRuleGroup and PortType
 *TARGET:BusinessRuleGroupTable
 */
public class BRGroupRuleHandler extends ConnectableRuleHandler {
	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 	
	/**	
	 * @param rule The BRReusableTaskRule that has just executed.
	 */
	public void handleRule(TransformationRule rule) {		
		com.ibm.btools.bom.model.processes.businessrules.BusinessRuleGroup src =null;
		BusinessRuleGroupTable target = null;
		
		if(rule.getSource().get(0) instanceof com.ibm.btools.bom.model.processes.businessrules.BusinessRuleGroup){
			src = (com.ibm.btools.bom.model.processes.businessrules.BusinessRuleGroup)rule.getSource().get(0);
		}
		
		if(rule.getTarget().get(0) instanceof BusinessRuleGroupTable){
			target = (BusinessRuleGroupTable)rule.getTarget().get(0) ;
		}		
		if(src != null && target != null){
			MappingManager mapMgr = getMappingManager(rule);
			mapMgr.addMapDefinition(src, target);			
			
			EList selectionRecords = src.getSelectionRecords();
			EList operationSelectionTables = target.getOperationSelectionTables();	
			EList operationSelectionRecords = ((OperationSelectionTable)operationSelectionTables.get(0)).getOperationSelectionRecords();
			
			mapListOfSrc2Target(selectionRecords,operationSelectionRecords,mapMgr);

			if(target.getOperationSelectionTables().get(0) instanceof OperationSelectionTable){
				OperationSelectionTable ost = (OperationSelectionTable)target.getOperationSelectionTables().get(0);
				SelectionData sd= ost.getDefaultSelectionData();
				mapMgr.addMapDefinition(src, sd);
				
				if (src.getAvailableTargets().size() == ost.getAvailableTargets().size()) {
					for (int i=0; i < src.getAvailableTargets().size(); i++) {
						mapMgr.addMapDefinition((EObject)src.getAvailableTargets().get(i), (EObject)ost.getAvailableTargets().get(i));
					}
				}
			}		
		}		
	}	
	private void mapListOfSrc2Target(EList selectionRecords, EList operationSelectionTables,MappingManager mm){
		if(selectionRecords != null && operationSelectionTables != null && operationSelectionTables.size()> 0 &&  
				operationSelectionTables.size() == selectionRecords.size()){
			int size = selectionRecords.size();
			//the source should correspond to the target in the collection
			for(int i =0;i<size;i++ ){
				if(selectionRecords.get(i)instanceof BusinessRuleSetSelectionRecord && 
						operationSelectionTables.get(i)instanceof OperationSelectionRecord){
					BusinessRuleSetSelectionRecord brsr = (BusinessRuleSetSelectionRecord)selectionRecords.get(i);
					OperationSelectionRecord osr = (OperationSelectionRecord)operationSelectionTables.get(i);
					mm.addMapDefinition(brsr, osr);			
					//map brSelRecord to the SelectionKey and BRGTDateTimeRangeKey
					mm.addMapDefinition(brsr, osr.getSelectionKey());
					mm.addMapDefinition(brsr, (BRGTDateTimeRangeKey)osr.getSelectionKey().getSelectionKeyElements().get(0));
					if(	brsr.getRuleSet() != null) {
						mm.addMapDefinition(brsr,osr.getSelectionData());
						mm.addMapDefinition(brsr.getRuleSet(),osr.getSelectionData());
					}
				}
			}				
		}
	}
}
