package com.ibm.btools.te.ilm.rulehandler.mapping.wps;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRule;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleAction;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleTemplate;
import com.ibm.btools.bom.model.processes.businessrules.IfThenRule;
import com.ibm.btools.bom.model.processes.businessrules.TemplateParameter;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.heuristics.br.BRTemplateRule;
import com.ibm.btools.te.ilm.heuristics.br.util.BrUtil;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup;
import com.ibm.wbiservers.businessrule.model.brgt.BusinessRuleGroupTable;
import com.ibm.wbit.br.core.model.ModelFactory;
import com.ibm.wbit.br.core.model.Rule;
import com.ibm.wbit.br.core.model.RuleSet;
import com.ibm.wbit.br.core.model.RuleTemplate;
import com.ibm.wbit.br.core.model.Variable;
/**
 This handler is responsbile for creating the ObjectDefinition mapping between
 *the source and target: 
 *SOURCE:
 *TARGET:
 */
public class BRTemplateRuleHandler extends ConnectableRuleHandler {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 	
	/**	
	 * @param rule The BRTemplateRule that has just executed.
	 */
	public void handleRule(TransformationRule rule) {
		if(rule instanceof BRTemplateRule){
			BusinessRuleTemplate brt = (BusinessRuleTemplate)rule.getSource().get(0);
			if(rule.getTarget().get(0) instanceof RuleTemplate){
				RuleTemplate rt = (RuleTemplate)rule.getTarget().get(0);
				MappingManager mm = getMappingManager(rule);
				mm.addMapDefinition(brt, rt);

				//map params
				EList srcParamList = brt.getParameters();
				EList targetParamList = rt.getParameter();				
				mapListOfSrc2Target(srcParamList,targetParamList,mm);

				if (brt.getRule() != null && brt.getRule() instanceof IfThenRule) {					
					BusinessRule br = brt.getRule();
					com.ibm.wbit.br.core.model.IfThenRule rtRule = (com.ibm.wbit.br.core.model.IfThenRule)rt.getRule();
                    
					//map rule of the template, they may be mapped in other handlers
					mm.addMapDefinition(br,rtRule);				

					IfThenRule ifThenRule = (IfThenRule)brt.getRule();

					if (ifThenRule.getRuleCondition() != null) {
						if (ifThenRule.getRuleCondition() != null && ifThenRule.getRuleCondition() instanceof StructuredOpaqueExpression &&
								((StructuredOpaqueExpression)ifThenRule.getRuleCondition()).getExpression() != null) {

							//map if condition
							mm.addMapDefinition(ifThenRule.getRuleCondition(),rtRule.getIf());
							mm.addMapDefinition(ifThenRule.getRuleCondition(),rtRule.getIf().getConditionExpression());
                             
							//map consequence/action
							EList consequences = ifThenRule.getRuleConsequence();
							EList actions = rtRule.getThen().getAction();
							if (!consequences.isEmpty() && !actions.isEmpty()) {
								mapListOfSrc2Target(consequences,actions,mm);
							} else {
								for (int i=0; i < actions.size(); i++) {
									mm.addMapDefinition(rtRule, (EObject)actions.get(i));
								}
							}
//							mapListOfSrc2Target(consequences,actions,mm);
						}
					}
				}	
			}
	}
	}	
	
	/**
	 * Map a list of source EObject to a list of target EObject, assume that their
	 * counterparts in the same position (index)
	 * @param srcs
	 * @param targets
	 * @param mm
	 */
	private void mapListOfSrc2Target(EList srcs, EList targets,MappingManager mm){
		if(srcs != null && targets != null && targets.size()> 0 &&  
				targets.size() == srcs.size()){
			int size = srcs.size();
			//the source should correspond to the target in the collection
			for(int i =0;i<size;i++ ){
				mm.addMapDefinition((EObject)srcs.get(i), (EObject)targets.get(i));
			}
		}				
	}
}
