package com.ibm.btools.te.ilm.rulehandler.mapping.wps;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.nio.channels.FileChannel.MapMode;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.BpelOptimizationRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.impl.BpelOptimizationRuleImpl;
import com.ibm.btools.te.ilm.heuristics.helper.CBPELMappingUtil;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.wbit.bpel.Case;
import com.ibm.wbit.bpel.Flow;
/**
 * This handler is responsible for mapping between OutputPinSet of a decision 
 *and the case activity (Flow, skip Empty)and mapping between Input/OutputPinSet
 * of a Process/Task and the onMessage activity (Flow, skip Empty)
 *
 */
public class BpelOptimizationRuleHandler extends ConnectableRuleHandler {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	public void handleRule(TransformationRule rule) {
		if(rule instanceof BpelOptimizationRule){
			BpelOptimizationRuleImpl optmizeRule = (BpelOptimizationRuleImpl)rule;
			HashMap caseActivityMap = optmizeRule.getCaseActivityMappings();
			if(caseActivityMap != null && !caseActivityMap.keySet().isEmpty()){
				mapPinSetToFlow(rule, caseActivityMap);
			}
			HashMap onMessageActivityMap = optmizeRule.getOnMessageActivityMappings();
			if(onMessageActivityMap != null && ! onMessageActivityMap.keySet().isEmpty()){
				mapPinSetToFlow(rule, onMessageActivityMap);	
			}
		}
		
		//Convert flow mappings to gflow mappings
		MappingManager mapMgr = getMappingManager(rule);
		Map<EObject, EObject> flowMapping = CBPELMappingUtil.getFlowMapping(rule.getRoot().getContext());
		for(EObject key: flowMapping.keySet()) {
			mapMgr.addMapDefinition(key, flowMapping.get(key));
//			if (flowMapping.get(key) == activity) {
//				flowMapping.put(key, scopeGFlow);
//			}
		}
	}

	private void mapPinSetToFlow(TransformationRule rule, HashMap caseActivityMap) {
		Set caseKeys = caseActivityMap.keySet();
		for(Iterator keyIt = caseKeys.iterator();keyIt.hasNext();){
			Object eachKey = keyIt.next();
			Object eachValue = caseActivityMap.get(eachKey);
			if(eachValue instanceof Flow || eachValue instanceof com.ibm.wbit.bpelpp.Flow){
				MappingManager mapMgr = getMappingManager(rule);
				mapMgr.addMapDefinition((EObject)eachKey, (EObject)eachValue);
				if(((EObject)eachValue).eContainer() instanceof Case){
					Case thisCase = (Case)((EObject)eachValue).eContainer();
					if(thisCase != null && thisCase.getCondition() != null){
						mapMgr.addMapDefinition((EObject)eachKey, thisCase.getCondition());
					}
					
				}
			}
		}
	}
}
