package com.ibm.btools.te.ilm.rulehandler.mapping.wps;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xsd.XSDComplexTypeContent;
import org.eclipse.xsd.XSDComplexTypeDefinition;
import org.eclipse.xsd.XSDModelGroup;
import org.eclipse.xsd.XSDParticle;
import org.eclipse.xsd.XSDParticleContent;

import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingUtil;

/**
 * This handler is responsbile for creating the ObjectDefinition mapping between
 * the source and target: 
 * SOURCE: BOM Type
 * TARGET: XSDTypeDefinition 
 *  
 */

public class ClassRuleHandler extends ConnectableRuleHandler {
	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 
	
	/**
	 * Creates the ObjectDefinitions for the BOM Type and the XSDTypeDefinitions. 
	 * @param rule the ClassRule that has just executed
	 */
	public void handleRule(TransformationRule rule) {
		//rule will execute only if DataType is not a PrimitiveType
		if(!(rule.getSource().get(0) instanceof com.ibm.btools.bom.model.artifacts.PrimitiveType)){
			if(!MappingUtil.isPrivateType((Type)rule.getSource().get(0))){
				EList targets = rule.getTarget();
				if(targets != null && !targets.isEmpty()){
					Object each = targets.get(0);
					if(each instanceof XSDComplexTypeDefinition){
						XSDComplexTypeDefinition complexTypeDef = (XSDComplexTypeDefinition)each;
						XSDComplexTypeContent content = complexTypeDef.getContent();
						if(content instanceof XSDParticle){
							XSDParticle particle = (XSDParticle)content;
							XSDParticleContent particleContent = particle.getContent();
							if(particleContent instanceof XSDModelGroup){
								XSDModelGroup xsdModelGroup = (XSDModelGroup)particleContent;
								//This is the target to map
								MappingManager mapMgr = getMappingManager(rule);
								mapMgr.addMapDefinition((EObject)rule.getSource().get(0),xsdModelGroup);
							}
						}
					}
				}
				super.handleRule(rule);
			}
		}
	}

}
