package com.ibm.btools.te.ilm.rulehandler.mapping.wps;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.emf.common.util.EList;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.DecisionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.InvokeWithCallbackRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.SANReusableProcessRule;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.MergePointFinder;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingRuleHandler;
import com.ibm.wbit.bpm.map.base.Descriptor;

/**
 * This super class is a placeholder for the class hierarchy, which has no
 * actual logic for <code>ObjectDefinition</code> mapping, its sub-classes will
 * have the actually mapping logic
 */
public abstract class ConnectableRuleHandler extends MappingRuleHandler {    
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	/**
	 * The descriptor is to establish the correlation between the objectpin and the pinset it resides
	 * @param mappingManager
	 * @param pinSet
	 * @param objectPin
	 */
	protected void createAssociatedPinSetDescriptor(MappingManager mappingManager,PinSet pinSet, ObjectPin objectPin){
		if(mappingManager == null|| pinSet == null || objectPin == null){
			return;
		}
		if(mappingManager.getObjectDefinition(pinSet) != null && mappingManager.getObjectDefinition(objectPin)!= null){
			String associatedPinSetInfo = mappingManager.getObjectDefinition(pinSet).getUid()+
											MappingManager.DESCRIPTOR_SEPARATOR +
											pinSet.getName();
			Descriptor relatedPinSetDescriptor = mappingManager.createDescriptor(MappingManager.DESCRIPTOR_RELATED_PIN_SET, associatedPinSetInfo);
			if(relatedPinSetDescriptor != null){
				EList descriptors = mappingManager.getObjectDefinition(objectPin).getDescriptor();
				if(descriptors != null && !isrelatedPinSetDescriptorExist(descriptors)){
					descriptors.add(relatedPinSetDescriptor);
				}
			}
		}
	}
	
	private boolean isrelatedPinSetDescriptorExist(EList descriptors){
		Iterator it = descriptors.iterator();
		while(it.hasNext()){
			Descriptor descriptor = (Descriptor)it.next();
			if(MappingManager.DESCRIPTOR_RELATED_PIN_SET.equals(descriptor.getName())){
				return true;
			}
		}
		return false;
	}
	
	protected void registerMergePoint(TransformationRule rule, InputPinSet inputPinSet, Action san) {
		
		//NamedElement mergePoint = BomHelper.getInstance().findExclusiveMergePoint(inputPinSet);	
		//defect 162516. handle the Merge of the Pick
		MergePointFinder mpf = new MergePointFinder();
		NamedElement mergePoint = null;
		if(rule instanceof SANReusableProcessRule || rule instanceof InvokeWithCallbackRule){
			mergePoint = mpf.findExclusiveMergePoint(san.getInputPinSet());
		}else if(rule instanceof DecisionRule){
			mergePoint = mpf.findExclusiveMergePoint(inputPinSet);
		}
	    if(mergePoint != null){
	    	Map mergePointMap = null;
	    	if(rule.getRoot().getContext().get(MappingManager.DECISION_MERGE_POINT) instanceof Map){
	    		mergePointMap = (Map) rule.getRoot().getContext().get(MappingManager.DECISION_MERGE_POINT);
	    		mergePointMap.put(san, mergePoint);
	    	}else{
	    		mergePointMap = new HashMap();
	    		mergePointMap.put(san, mergePoint);
	    		rule.getRoot().getContext().put(MappingManager.DECISION_MERGE_POINT, mergePointMap);
	    	}	    	
	    }
	}
}
