package com.ibm.btools.te.ilm.rulehandler.mapping.wps;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.artifacts.Constraint;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.humantasks.EmailEscalationAction;
import com.ibm.btools.bom.model.processes.humantasks.EmailMessage;
import com.ibm.btools.bom.model.processes.humantasks.Escalation;
import com.ibm.btools.bom.model.processes.humantasks.EscalationAction;
import com.ibm.btools.bom.model.processes.humantasks.EscalationChain;
import com.ibm.btools.bom.model.processes.humantasks.HumanTask;
import com.ibm.btools.bom.model.resources.ResourceRequirement;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.expression.model.FunctionArgument;
import com.ibm.btools.expression.model.FunctionExpression;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.HumanTaskRule;
import com.ibm.btools.te.ilm.heuristics.util.TimeConstants;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.wbit.businesscalendar.ICalendar;
import com.ibm.wbit.tel.DocumentRoot;
import com.ibm.wbit.tel.ParameterType;
import com.ibm.wbit.tel.TEMailReceiver;
import com.ibm.wbit.tel.TEmail;
import com.ibm.wbit.tel.TEscalation;
import com.ibm.wbit.tel.TEscalationChain;
import com.ibm.wbit.tel.TEscalationReceiver;
import com.ibm.wbit.tel.TPotentialOwner;
import com.ibm.wbit.tel.TTask;
/**
 This handler is responsbile for creating the ObjectDefinition mapping between
 *the source and target: 
 *SOURCE: BOM Action
 *TARGET: TEL DocumentRoot
 */
public class HumanTaskRuleHandler extends ConnectableRuleHandler {
	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 	
	/**	
	 * @param rule The HumanTaskRule that has just executed.
	 */
	public void handleRule(TransformationRule rule) {
		
		Action action = (Action)rule.getSource().get(0);		
		EObject component = (EObject)rule.getTarget().get(0);
		MappingManager mapMgr = getMappingManager(rule);
		TTask  tTask = null;
		if(component instanceof DocumentRoot){
			DocumentRoot telDocRoot = (DocumentRoot)component;
			tTask= telDocRoot.getTask();
		}
		
		if (action instanceof HumanTask) {
			//GJD add calendar mapping
			ICalendar calendar = null;
			if (rule instanceof HumanTaskRule) {
				calendar = getCalendar(((HumanTaskRule)rule).getTarget(), tTask);
			}
			
			if (((HumanTask)action).getActivity() != null) { // we hit a global human task
				
				mapMgr.addMapDefinition(((HumanTask)action).getActivity(),component);	
				if(tTask != null) {
					mapMgr.addMapDefinition(((HumanTask)action).getActivity(),tTask);
										
//					for (int i=0; i < tTask.getDisplayName().size(); i++) 
//						mapMgr.addMapDefinition(((HumanTask)action).getActivity(), (TDisplayName)tTask.getDisplayName().get(i));					
//					
//					for (int i=0; i < tTask.getDescription().size(); i++)
//						mapMgr.addMapDefinition(((HumanTask)action).getActivity(), (TDescription)tTask.getDescription().get(i));															
				}
				// GJD add calendar mapping
				if(calendar != null){
					mapMgr.addMapDefinition(((HumanTask)action).getActivity(),calendar);
				}

			} else { // we hit a local human task
				
				mapMgr.addMapDefinition(action,component);	
				if(tTask != null) {
					mapMgr.addMapDefinition(action,tTask);
					
//					for (int i=0; i < tTask.getDisplayName().size(); i++) 
//						mapMgr.addMapDefinition(action, (TDisplayName)tTask.getDisplayName().get(i));					
//					
//					for (int i=0; i < tTask.getDescription().size(); i++)
//						mapMgr.addMapDefinition(action, (TDescription)tTask.getDescription().get(i));
				}
				// GJD add calendar mapping
				if(calendar != null){
					mapMgr.addMapDefinition(action, calendar);
				}
			}
		} else { // we hit a normal task implemented by human task

			mapMgr.addMapDefinition(action,component);	
			if(tTask != null)
				mapMgr.addMapDefinition(action,tTask);
			
			// in this case, we don't need to map the content of the human task
			// because the generated human task is just a template.
			return;
		}
		
		// now, mapping the contents
		HumanTask ht = (HumanTask)action;
		
		mapPotentialOwner(mapMgr, ht, tTask);
		
		List emailMessage = ht.getEmailMessages();
		if(emailMessage != null && !emailMessage.isEmpty()) {
			for (int i = 0; i<emailMessage.size(); i++){
				//TODO 1-to-1 mapping from BOM email messages to WID email messages
				EmailMessage message = (EmailMessage)emailMessage.get(i);
				TEmail email = (TEmail)tTask.getEmail().get(i);
				if (message != null && email != null)
					mapMgr.addMapDefinition(message, email);
			}
		}
		
		mapEscalationChains(mapMgr, ht, tTask);
	}	
	
	protected void mapPotentialOwner(MappingManager mapMgr, HumanTask ht, TTask tTask) {
		if(ht.getPotentialOwner() == null) {
			return;
		}
	
		ResourceRequirement potentialOwner = ht.getPotentialOwner();
		TPotentialOwner widOwner = tTask.getStaffSettings().getPotentialOwner();		
		//map the BOM PrimaryOwner to the WID PotentialOwner
		if(tTask.getStaffSettings() != null)
			mapMgr.addMapDefinition(potentialOwner, tTask.getStaffSettings());
		
		if (potentialOwner != null && widOwner != null) {
			mapMgr.addMapDefinition(potentialOwner, widOwner);
			
			if (widOwner.getVerb() != null)
				mapMgr.addMapDefinition(potentialOwner, widOwner.getVerb());
			
			if (!potentialOwner.getOwnedConstraint().isEmpty()) {
				// TODO always one constraint per potential owner
				Constraint constraint = (Constraint)potentialOwner.getOwnedConstraint().get(0);
				StructuredOpaqueExpression resReqConstraint = (StructuredOpaqueExpression)constraint.getSpecification();
				FunctionExpression function = resReqConstraint != null? (FunctionExpression)resReqConstraint.getExpression(): null;
				
				//map the BOM Function to the WID verb
				if (function != null && widOwner.getVerb() != null) {
//					mapMgr.addMapDefinition(function, widOwner.getVerb());
								
					if(function.getArguments() != null && !function.getArguments().isEmpty()){						
						List<FunctionArgument> mappedArgs = new ArrayList<FunctionArgument>();
						
						List arguments = function.getArguments();
						for (int i=0; i < arguments.size(); i++) {
							FunctionArgument thisFuncArgument = (FunctionArgument)arguments.get(i);
							if (thisFuncArgument.getArgumentValue() != null)
								mappedArgs.add(thisFuncArgument);							
						}
							
						if (!mappedArgs.isEmpty()) {
							int size = Math.min(mappedArgs.size(), widOwner.getVerb().getParameter().size());
							for(int i=0; i<size; i++){
								FunctionArgument thisFuncArgument = (FunctionArgument)mappedArgs.get(i);
								ParameterType parameter =(ParameterType)widOwner.getVerb().getParameter().get(i);
								mapMgr.addMapDefinition(thisFuncArgument, parameter);							
							}
						}
					}
				}
			}
		}
	}
	
	protected void mapEscalationChains(MappingManager mapMgr, HumanTask ht, TTask tTask) {
		if(ht.getEscalationChains() == null || ht.getEscalationChains().isEmpty())
			return;
		
		List<EscalationChain> escalationChains = ht.getEscalationChains();
		//Iterator<EscalationChain> it = escalationChains.iterator();
		for(int i=0; i<escalationChains.size(); i++){
			// TODO 1-to-1 mapping between BOM escalation chain and WID escalation chain?
			EscalationChain thisChain = escalationChains.get(i);
			TEscalationChain widChain = (TEscalationChain)tTask.getEscalationSettings().getEscalationChain().get(i);
			
			if (thisChain == null || widChain == null)
				break;
			
			mapMgr.addMapDefinition(thisChain, widChain);
			
			if(thisChain.getEscalations() != null && !thisChain.getEscalations().isEmpty()) {
				List<Escalation> escalations = thisChain.getEscalations();
				for(int j=0; j<escalations.size(); j++){
					// TODO 1-to-1 mapping between BOM escalation and WID escalation? 
					Escalation thisEscalation = escalations.get(j);
					TEscalation widEscalation = (TEscalation)widChain.getEscalation().get(j);
					
					if (thisEscalation == null || widEscalation == null)
						continue; // d73815 - we cannot break the if loop, as there are more escalation chains to map
					
					mapMgr.addMapDefinition(thisEscalation, widEscalation);
					if (widEscalation.getDocumentation().size() == 1) {
						mapMgr.addMapDefinition(thisEscalation, (EObject)widEscalation.getDocumentation().get(0));
					}
						
					if(thisEscalation.getEscalationAction() != null) {
						EscalationAction escAction = thisEscalation.getEscalationAction();
						if(escAction.getEscalationReceiver() != null) {
							ResourceRequirement escalationReceiver = escAction.getEscalationReceiver();
							TEscalationReceiver widReceiver = widEscalation.getEscalationReceiver();
							if(escalationReceiver == null || widReceiver == null) 
								continue;
							mapMgr.addMapDefinition(escalationReceiver, widReceiver);
							
							if (widReceiver.getVerb() != null)
								mapMgr.addMapDefinition(escalationReceiver, widReceiver.getVerb());
								
							if (!escalationReceiver.getOwnedConstraint().isEmpty()) {
								// TODO always one constraint per escalation receiver
								Constraint escConstraint = (Constraint)escalationReceiver.getOwnedConstraint().get(0);
								StructuredOpaqueExpression escFuncConstraint = (StructuredOpaqueExpression)escConstraint.getSpecification();
								FunctionExpression escFunction = (FunctionExpression)escFuncConstraint.getExpression();
								
								if (escFunction == null || widReceiver.getVerb() == null)
									continue; // d73815 - we cannot break the if loop, as there are more escalation chains to map
								
								//map the BOM Function to the WID verb
								//mapMgr.addMapDefinition(escFunction, widReceiver.getVerb());

								
								if(escFunction.getArguments() != null && !escFunction.getArguments().isEmpty()) {																	
									List<FunctionArgument> mappedArgs = new ArrayList<FunctionArgument>();
									
									List escArguments = escFunction.getArguments();
									for (int k=0; k < escArguments.size(); k++) {
										FunctionArgument thisFuncArgument = (FunctionArgument)escArguments.get(k);
										if (thisFuncArgument.getArgumentValue() != null)
											mappedArgs.add(thisFuncArgument);							
									}
								
									if (!mappedArgs.isEmpty()){
										int size = Math.min(mappedArgs.size(), widReceiver.getVerb().getParameter().size());
										for(int k=0; k< size; k++){
											FunctionArgument thisFuncArgument = (FunctionArgument)mappedArgs.get(k);
											ParameterType parameter =(ParameterType)widReceiver.getVerb().getParameter().get(k);
											if (thisFuncArgument != null && parameter != null)
												mapMgr.addMapDefinition(thisFuncArgument, parameter);									
										}
									}
								}								
							}
							
							if(escAction instanceof EmailEscalationAction) {
								EmailEscalationAction emailAction = (EmailEscalationAction)escAction;
								ResourceRequirement emailReceiver = emailAction.getEmailReceiver();
								TEMailReceiver widEmailReceiver = widEscalation.getEMailReceiver();
								if(emailReceiver == null || widEmailReceiver == null) 
									continue; // d73815 - we cannot break the if loop, as there are more escalation chains to map
								mapMgr.addMapDefinition(emailReceiver, widEmailReceiver);
								
								if (!emailReceiver.getOwnedConstraint().isEmpty()) {
									// TODO always one constraint per email receiver
									Constraint emaiConstraint = (Constraint)emailReceiver.getOwnedConstraint().get(0);
									StructuredOpaqueExpression emailFuncConstraint = (StructuredOpaqueExpression)emaiConstraint.getSpecification();
									FunctionExpression emailFunc = (FunctionExpression)emailFuncConstraint.getExpression();
									
									if (emailFunc == null || widEmailReceiver.getVerb() == null)
										continue; // d73815 - we cannot break the if loop, as there are more escalation chains to map
									
									//map the BOM Function to the WID verb
									mapMgr.addMapDefinition(emailFunc, widEmailReceiver.getVerb());
								
									if(emailFunc.getArguments() !=null && !emailFunc.getArguments().isEmpty()) {										
										List<FunctionArgument> mappedArgs = new ArrayList<FunctionArgument>();
										
										List escArguments = emailFunc.getArguments();
										for (int k=0; k < escArguments.size(); k++) {
											FunctionArgument thisFuncArgument = (FunctionArgument)escArguments.get(k);
											if (thisFuncArgument.getArgumentValue() != null)
												mappedArgs.add(thisFuncArgument);							
										}
									
										if(!mappedArgs.isEmpty()){
											int size = Math.min(mappedArgs.size(), widEmailReceiver.getVerb().getParameter().size());											
											for(int k=0; k<size; k++){
												FunctionArgument thisFuncArgument = (FunctionArgument)mappedArgs.get(k);
												ParameterType parameter =(ParameterType)widEmailReceiver.getVerb().getParameter().get(k);
												if (thisFuncArgument != null && parameter != null)
													mapMgr.addMapDefinition(thisFuncArgument, parameter);											
											}
										}
									}
								}
							}		
						}
					}
				}
			}
		}
	}
	
	private ICalendar getCalendar (List targets, TTask  tTask) {
		if (targets == null || tTask ==  null) {
			return null;
		}

		ICalendar calendar = null;
		Iterator it = targets.iterator();
		Object obj = null;
		while (it.hasNext()) {
			obj = it.next();
			if (obj instanceof ICalendar) {
				calendar = (ICalendar)obj;
				if (calendar.getTargetNamespace().equals(tTask.getTargetNamespace().toString() + TimeConstants.CALENDAR_SUFFIX)) {
					break;
				}
			}
			calendar = null;
		}
		return calendar;
	}

}
