package com.ibm.btools.te.ilm.rulehandler.mapping.wps;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.Iterator;
import java.util.List;

import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.wbit.bpelpp.Input;
import com.ibm.wbit.bpelpp.Parameter;

/**
 * This handler is responsbile for creating the ObjectDefinition mapping between
 * the source and target: 
 * SOURCE: BOM PinSet/ObjectPin
 * TARGET: BPEL input Parameter
 */
public class InputRuleHandler extends ConnectableRuleHandler {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 
	
	/**
	 * @param rule the InputRule that has just executed
	 */
	public void handleRule(TransformationRule rule) {
		PinSet pinSet = (PinSet)rule.getSource().get(0);
		Input input = (Input)rule.getTarget().get(0);
		List objectPins = BomUtils.getObjectPinsForSet(pinSet);
		List inputParams = input.getParameter();
		MappingManager mapMgr = null;
		if(objectPins != null && !objectPins.isEmpty()){
			Iterator it = objectPins.iterator();
			for (int i=0; i<objectPins.size();i++) {
				ObjectPin objectPin = (ObjectPin)objectPins.get(i);
				Parameter parameter = (Parameter)inputParams.get(i);
				mapMgr = getMappingManager(rule);
				mapMgr.addMapDefinition(objectPin,parameter);				
			}
		}

		//we should also map the BPEL Input to the PinSet
		super.handleRule(rule);
		if(objectPins != null && !objectPins.isEmpty()){
			for (int i=0; i<objectPins.size();i++) {
				ObjectPin objectPin = (ObjectPin)objectPins.get(i);
				createAssociatedPinSetDescriptor(mapMgr,pinSet,objectPin);
			}			
		}
	}
	
}
