package com.ibm.btools.te.ilm.rulehandler.mapping.wps;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.Definition;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ConnectableRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.impl.ConnectableRuleImpl;
import com.ibm.btools.te.ilm.heuristics.helper.CBPELUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingUtil;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Assign;
import com.ibm.wbit.bpel.Copy;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Source;
import com.ibm.wbit.bpelpp.Description;
import com.ibm.wbit.bpelpp.Documentation;
import com.ibm.wbit.bpelpp.Task;
import com.ibm.wbit.bpelpp.impl.AutonomyImpl;
import com.ibm.wbit.bpelpp.impl.ExecutionModeImpl;
import com.ibm.wbit.bpelpp.impl.TaskImpl;
import com.ibm.wbit.bpm.map.base.Descriptor;
import com.ibm.wbit.bpm.map.objectdefinition.ObjectDefinition;
import com.ibm.wbit.bpm.trace.model.BPMConstants;
import com.ibm.wbit.wpc.TypeEnum;


/**
 * This handler is responsible for creating the ObjectDefinition mapping between
 * the source and target: 
 * SOURCE: BOM InputPinSet (on a SAN or CBA)
 * TARGET: BPEL Invoke
 */
public class InvokeRuleHandler extends ConnectableRuleHandler {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 
	
	/**	 
	 * @param rule The InvokeRule that has just executed.
	 */
	@SuppressWarnings("unchecked")
	public void handleRule(TransformationRule rule) {
		InputPinSet inputPinSet = (InputPinSet)rule.getSource().get(0);	   
	    MappingManager mapMgr = getMappingManager(rule);
		/* The SAN can have a maximum of one correlated output pin set */
	    //we need to map the fork activity for exceptional output	    
	    if (inputPinSet.getOutputPinSet().size() > 0){	        
	        for(Object ops:inputPinSet.getOutputPinSet()){
	        	if(ops instanceof OutputPinSet){
	        		if(CBPELUtil.shouldGenerateForkActivity((OutputPinSet)ops)){
	        			if(rule instanceof ConnectableRuleImpl){
	        				ConnectableRuleImpl cr = (ConnectableRuleImpl)rule;
	        			    Activity generatedActivity = cr.getMappedActivity((NamedElement)ops);
	        			    mapMgr.addMapDefinition((OutputPinSet)ops, generatedActivity);        			
	        			    
	        			    //Map the copy elements inside the assign.  This is for the case that the assign
	        			    //is generated from a fault with a complex part
	        			    if (generatedActivity instanceof Assign) {
	        			    	Assign assign = (Assign)generatedActivity;
	        			    	for (Copy copy : (List<Copy>)assign.getCopy()) {
	        			    		mapMgr.addMapDefinition((OutputPinSet)ops, copy);
	        			    	}
	        			    }
	        			    
//	        			    //IPS --> Assign Sources
//	        			    mapMgr.addMapDefinition((InputPinSet)rule.getSource().get(0), generatedActivity.getSources());
//	        			    //EPS --> Assign Source
//	        			    for (Source src : (List<Source>)generatedActivity.getSources().getChildren()) {
//	        			    	mapMgr.addMapDefinition((OutputPinSet)ops, src);
//	        			    }
	        			    //All connections from this EPS --> link from invoke to assign
//	        			    for (OutputObjectPin op : (List<OutputObjectPin>)((OutputPinSet)ops).getOutputObjectPin()) {
//	        			    	mapMgr.addMapDefinition(op.getOutgoing(), faultSource.getLink());
//	        			    }
	        			    
	        			}
	        		}
	        	}
	        }
	    }
		
		Invoke bpelInvoke = (Invoke)rule.getTarget().get(0);	
		//defect 162856
		if(inputPinSet.eContainer() instanceof CallBehaviorAction){
			mapMgr.addMapDefinition(inputPinSet.eContainer(),bpelInvoke);
			//addPortTypeFileDescriptorFor(mapMgr,bpelInvoke, true);
		}else{
			mapMgr.addMapDefinition(inputPinSet,bpelInvoke);
			//addPortTypeFileDescriptorFor(mapMgr,bpelInvoke, false);
		}		
		List extElements = bpelInvoke.getEExtensibilityElements();
			if(extElements != null && !extElements.isEmpty()){
				Iterator it = extElements.iterator();
				while(it.hasNext()){
					EObject extElement = (EObject)it.next();
					if(extElement instanceof Documentation || extElement instanceof Description){
						mapMgr.addMapDefinition(inputPinSet, extElement);
					}
				}
		}

//		MapRecord coreRecord = getCoreMapRecord(san, inputPinSet); 
//		if (coreRecord != null){
//		    /* There may be new variables to map (corresponding to new pins) that 
//		     * weren't available the first time the rule was executed */
//		    associateInputOutput(inputPinSet, outputPinSet, (BasicNode)coreRecord.getOmModelObject());
//		    return;
//		}

	    /* Create a new <invoke> everytime this handler is called, since it must
	     * be for a different input pin set on the same SAN */
//	    BasicNode omInvokeNode = new BasicNode();
	    
	    /* Check for the existence of the <staff> tag, since its presence indicates that something
	     * is a local staff activity and not a human task invoke, otherwise if a BOM task has both a 
	     * resource requirement and an implementation, the HashMap will have it look like a local staff activity 
	     * when really it should be represented as an external human task, which it is in the BPEL */
        Task staffTask = (Task)MappingUtil.getExtensibilityElement(bpelInvoke, TaskImpl.class);
        if (staffTask!=null) {
        	mapMgr.addMapDefinition(inputPinSet,staffTask);	
        }
	    
	    /* Check if the <invoke> actually contains a staff activity, and set the type appropriated */
//	    if (staffTask != null && getInternalMapRecord(RuleHandlerUtils.getInternalKey(san, RuleHandlerUtils.STAFF_ACTIVITY_KEY_TAG))!= null){
//	        omInvokeNode.setType(new ElementType(ElementType.BPEL_STAFF));
//		    omInvokeNode.setName(bpelInvoke.getName());
//	    } else if (getInternalMapRecord(RuleHandlerUtils.getInternalKey(san, RuleHandlerUtils.HUMAN_TASK_ACTIVITY_KEY_TAG))!= null){
//	    	MapRecord htRecord = getInternalMapRecord(RuleHandlerUtils.getInternalKey(san, RuleHandlerUtils.HUMAN_TASK_ACTIVITY_KEY_TAG));
//	        omInvokeNode.setType(new ElementType(ElementType.BPEL_HUMAN_TASK_INVOKE));
//		    omInvokeNode.setName(((BasicNode)htRecord.getOmModelObject()).getName());
//	    } else if (isReusableProcess(san)) {
//	        omInvokeNode.setType(new ElementType(ElementType.BPEL_PROCESS_INVOKE));
//		    omInvokeNode.setName(bpelInvoke.getName());
//	    } else {
//	        omInvokeNode.setType(new ElementType(ElementType.BPEL_INVOKE));
//		    omInvokeNode.setName(bpelInvoke.getName());
//	    }
//	    /* The InvokeRule is only called in the receive / reply case, so synchronous */
//	    omInvokeNode.setSynchonousCall(true);
//	    omInvokeNode.setEssential(true);
//	    associateInputOutput(inputPinSet, outputPinSet, omInvokeNode);
//
//	    /* Put this in the hashMap, but only after connecting it with any pre-existing
//	     * representations for the pin sets */
//	    recordAndConnectWithPinSets(san, inputPinSet, new MapRecord(omInvokeNode, bpelInvoke));
	    	   
	}
	/**
	 * Add descriptor on invoke: BPMConstants.DESCRIPTOR_PORT_TYPE_FILE
	 * @param mappingManager
	 * @param beplInvoke
	 * @param isCBA TODO
	 */
	private void addPortTypeFileDescriptorFor(MappingManager mappingManager,Invoke beplInvoke, boolean isCBA){
		if(mappingManager == null || beplInvoke == null){
			return;
		}
		ObjectDefinition invokeOD = mappingManager.getObjectDefinition(beplInvoke);
		if(invokeOD != null){
			EObject portTypeContainer = beplInvoke.getPortType().eContainer();
			if(portTypeContainer instanceof Definition){
				Definition containerDef = (Definition)portTypeContainer;
				String wsdlFileName = null;
				if(!isCBA){
					wsdlFileName = containerDef.getQName().getLocalPart();
					wsdlFileName += ".wsdl";
				}else{
					wsdlFileName = containerDef.getLocation();
					if(wsdlFileName.indexOf("/")>0){
						wsdlFileName = wsdlFileName.substring(wsdlFileName.lastIndexOf("/")+1);
					}else if(wsdlFileName == null){
						wsdlFileName = containerDef.getQName().getLocalPart();
						wsdlFileName += ".wsdl";
					}
				}
				if(wsdlFileName != null && wsdlFileName.length() > 0){					
					Descriptor portTypeFileDescriptor = mappingManager.createDescriptor(BPMConstants.DESCRIPTOR_PORT_TYPE_FILE, wsdlFileName);
					invokeOD.getDescriptor().add(portTypeFileDescriptor);
				}
			}
		}
	}
	
}
