package com.ibm.btools.te.ilm.rulehandler.mapping.wps;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.impl.InvokeWithCallbackRuleImpl;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Pick;


/**
 * This class is responsible for creating the ObjectDefinitions corresponding
 * to BOM Tasks / CBAs in the case where the InvokeWithCallbackRule was executed.
 * 
 * The InvokeWithCallbackRule is executed for every <invoke> that is created
 * as a 'one way' operation.  There are two types of 'one way' operation:
 *   1. 'one way' operation with no response (no callback)
 *   2. 'one way' operation with callback
 * 
 * In both cases, an <invoke> is created that does not have an 'output' 
 * extensibility element.  In the second case, either a <receive> or a <pick>
 * is also created (depending on if there is more than one output pin set correlated
 * to the source input pin set) to capture the reslut of the <invoke>.  The 
 * <recieve> or the <pick> is accessible via the InvokeWithCallbackRuleImpl.getCallbackActivity()
 * method
 * Source: BOM InputPinSet (on a SAN or CBA)
 * Target: BPEL Invoke (first target), ... , BPEL <recieve> or <pick>, ...
 */
public class InvokeWithCallbackRuleHandler extends ConnectableRuleHandler {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 
	
	/**	 
	 * @param rule The InvokeWithCallbackRule that has just executed.
	 */
	public void handleRule(TransformationRule rule) {
		InputPinSet inputPinSet = (InputPinSet)rule.getSource().get(0);
	    Action san = (Action)inputPinSet.getAction();		
		Invoke bpelInvoke = (Invoke)rule.getTarget().get(0);
		
		MappingManager mapMgr = getMappingManager(rule);
		mapMgr.addMapDefinition(inputPinSet,bpelInvoke);

		Activity callbackActivity = ((InvokeWithCallbackRuleImpl)rule).getCallbackActivity();
		if(callbackActivity != null){
			//this callback activity can be either a Pick or a Receive. 
			//we have to set as source the OutputPinSets. 
			List outputPinSets = getOutputPinSets(inputPinSet);
			if(outputPinSets.size() >1){
			//this should be a Pick
				for(int i=0; i<outputPinSets.size(); i++){
					mapMgr.addMapDefinition( (EObject)outputPinSets.get(i), callbackActivity);
					registerMergePoint(rule, inputPinSet, san);
				}
			//this should be a Receive	
			}else{
				mapMgr.addMapDefinition((EObject)outputPinSets.get(0), callbackActivity);
			}
		}		
	}	
	
	private List getOutputPinSets(InputPinSet inputSet) {
		List outputPinSets = BomUtils.getCorrelatedPinSets(inputSet);
		if (inputSet.getAction() instanceof CallBehaviorAction) {
		// if it is a CBA, need to re-adjust the order of the outputPinSets and make sure they match the CBA's order
		// It is important because the operation (Source of this rule) is in order
			List filteredOrigOPS = new ArrayList();
			List origOPS = inputSet.getAction().getOutputPinSet();
			OutputPinSet tempOPS;
			for (Iterator it=origOPS.iterator();it.hasNext();) {
				tempOPS = (OutputPinSet)it.next();
				if (outputPinSets.contains(tempOPS)) {
					filteredOrigOPS.add(tempOPS);
				}
			}
			outputPinSets = filteredOrigOPS;
		}
		return outputPinSets;		
	}

}
