package com.ibm.btools.te.ilm.rulehandler.mapping.wps;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.Iterator;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.ActivityEdge;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.DisplayName;

/**
 * This handler is responsbile for creating the ObjectDefinition mapping between
 * the source and target: 
 * SOURCE: BOM Activity Edge
 * TARGET: BPEL link
 */
public class LinkRuleHandler extends ConnectableRuleHandler {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 	

	/**	
	 * @param rule The LinkRule that has just executed.
	 */
	public void handleRule(TransformationRule rule) {
		ActivityEdge e = null;
		Object o = null;
		for (Iterator it = rule.getSource().iterator(); it.hasNext();) {
			o = it.next();
			if (o instanceof ActivityEdge) {
				e = (ActivityEdge) o;
				break;
			}
		}
		Process bpelProcess = (Process)ProcessUtil.getProcessDefinition(rule.getRoot().getContext());
		MappingManager mapMgr = getMappingManager(rule);
		Iterator it = rule.getTarget().iterator();
		while (it.hasNext()) {
			EObject obj = (EObject) it.next();
			if (obj instanceof Link) {
				if (e == null) {
					mapMgr.addInternalLinks(obj, rule, bpelProcess);
					//mapMgr.addMapDefinition(e, obj);
				} else {
					mapMgr.addMapDefinition(e, obj);
					mapMgr.addInternalLinks(obj, e, bpelProcess);
					NamedElement pinSet = BomUtils.findSourceSet(e);
					if(pinSet instanceof OutputPinSet){
						Link theLink = (Link)obj;
						DisplayName displayName = BPELPlusFactory.eINSTANCE.createDisplayName();
						displayName.setText(e.getName());
						theLink.getExtensibilityElements().add(displayName);
					}
				}
			}
		}
	}
}