package com.ibm.btools.te.ilm.rulehandler.mapping.wps;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.impl.MergeRuleImpl;
import com.ibm.btools.te.ilm.model.abstractbpel.AlternativeActivity;
import com.ibm.btools.te.ilm.model.sa.ProcessFlowType;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingUtil;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpelpp.JavaScriptActivity;
import com.ibm.wbit.bpelpp.impl.JavaScriptActivityImpl;

/**
 *This handler is responsbile for creating the ObjectDefinition mapping between
 *the source and target: 
 *SOURCE: BOM Action
 *TARGET: AlternativeActivity/JavaScriptActivity
 */
public class MergeRuleHandler extends ConnectableRuleHandler {
	
	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 
	
	/**	 
	 * @param rule The MergeRule that has just executed.
	 */
	public void handleRule(TransformationRule rule) {		
	    InputPinSet inputPinSet = (InputPinSet)rule.getSource().get(0);
	    Action san = (Action)inputPinSet.getAction();
		
	    Activity bpelMergeActivity = (Activity)((MergeRuleImpl)rule).getMergeActivity();
	    MappingManager mapMgr = getMappingManager(rule);
	    
	    if (bpelMergeActivity instanceof AlternativeActivity){
	        bpelMergeActivity = MappingUtil.getAlternative((AlternativeActivity)bpelMergeActivity, ProcessFlowType.BPELPP_LITERAL);
	        mapMgr.addMapDefinition(san, bpelMergeActivity);
	    } 
	    
	    Activity bpelIsActivity = (Activity)((MergeRuleImpl)rule).getInputSetActivity();
	    if (bpelIsActivity != null) {
	    	mapMgr.addMapDefinition(inputPinSet, bpelIsActivity);
	    } 
	    
	    Activity bpelOsActivity = (Activity)((MergeRuleImpl)rule).getOutputSetActivity();
	    if (bpelOsActivity != null) {
	    	OutputPinSet ops = (OutputPinSet) inputPinSet.getOutputPinSet().get(0);
	    	mapMgr.addMapDefinition(ops, bpelOsActivity);
	    } 
	    
	    //
	    if(bpelMergeActivity instanceof Invoke && bpelMergeActivity.getEExtensibilityElements()!= null && !bpelMergeActivity.getEExtensibilityElements().isEmpty()){
	    	JavaScriptActivity scriptActivity = (JavaScriptActivity)MappingUtil.getExtensibilityElement(bpelMergeActivity, JavaScriptActivityImpl.class);
	    	if(scriptActivity != null){
	    		
	    		mapMgr.addMapDefinition(san, scriptActivity);
	    	}
	    }


	}

}
