package com.ibm.btools.te.ilm.rulehandler.mapping.wps;

/*
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 5724-I74, 5724-I75
 *  Copyright IBM Corporation 2003, 2009. All rights reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

import java.util.Iterator;
import java.util.List;

import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpelpp.Output;
import com.ibm.wbit.bpelpp.Parameter;

/**
 * This handler is responsbile for creating the ObjectDefinition mapping between
 * the source and target: 
 * source = BOM InputPinSet/OutputSet
 * target = BPELPlus Output
 */
public class OutputRuleHandler extends ConnectableRuleHandler {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 

	/**
	 * @param rule the OutputRule that has just executed
	 */
	public void handleRule(TransformationRule rule) {
		PinSet pinSet = (PinSet)rule.getSource().get(0);		
		MappingManager mapMgr = null;
		if(pinSet instanceof OutputPinSet && ((OutputPinSet)pinSet).getIsException().booleanValue()){
			//we need to map the wrapper fault variable to the output pin set
			mapMgr = getMappingManager(rule);
			if(rule.getTarget() != null && !rule.getTarget().isEmpty()){
				if(rule.getTarget().get(0) instanceof BPELVariable){
					BPELVariable variable = (BPELVariable)rule.getTarget().get(0);
					mapMgr.addMapDefinition(pinSet,variable);	
				}			
			}			
		}else{	
			Output output = (Output)rule.getTarget().get(0);
			List objectPins = BomUtils.getObjectPinsForSet(pinSet);
			List params = output.getParameter();
			if(objectPins != null && !objectPins.isEmpty()){
				Iterator it = objectPins.iterator();
				for (int i=0; i<objectPins.size();i++) {
					ObjectPin objectPin = (ObjectPin)objectPins.get(i);
					Parameter parameter = (Parameter)params.get(i);
					mapMgr = getMappingManager(rule);
					mapMgr.addMapDefinition(objectPin,parameter);				
				}
			}
			super.handleRule(rule);
			if(objectPins != null && !objectPins.isEmpty()){
				for (int i=0; i<objectPins.size();i++) {
					ObjectPin objectPin = (ObjectPin)objectPins.get(i);
					createAssociatedPinSetDescriptor(mapMgr,pinSet,objectPin);
				}			
			}
		}
	}
}
