package com.ibm.btools.te.ilm.rulehandler.mapping.wps;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.xsd.XSDComplexTypeDefinition;
import org.eclipse.xsd.XSDModelGroup;
import org.eclipse.xsd.XSDParticle;

import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;

/**
 * This handler is responsbile for creating the ObjectDefinition mapping between 
 * BOM <code>PinSet</code> and XSD <code>XSDParticle </code> content
 */
public class PinSetRuleHandler extends ConnectableRuleHandler {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 
	
	/**
	 * @param rule the PinSetRule that has just executed
	 */
	public void handleRule(TransformationRule rule) {
		//TODO added for fault - no complex type created in PinSetRule		
		if (rule.getSource().isEmpty() || rule.getTarget().isEmpty()) {
			return;
		}
		
		EObject src =(EObject) rule.getSource().get(0);
		EObject tar =(EObject) rule.getTarget().get(0);
		if (src instanceof PinSet && tar instanceof XSDComplexTypeDefinition) {
			PinSet pinSet = (PinSet)src;
			XSDComplexTypeDefinition compType = (XSDComplexTypeDefinition)tar;
			List pins = collectPins(pinSet);
			List elements = collectElements(compType);
			for (int i=0;i<pins.size();i++){
				EObject pin = (EObject)pins.get(i);
				EObject element = (EObject)elements.get(i);
				
				MappingManager mapMgr = getMappingManager(rule);
				mapMgr.addMapDefinition(pin,element);
			}
			
		}
	}
	/**
	 * Collects the ObjectPins is the <code>pinSet</code>.
	 * @param pinSet a BOM PinSet.
	 * @return List
	 */
	private List collectPins(EObject pinSet){
		
		List pins = new ArrayList();
		if(pinSet instanceof InputPinSet){
			pins = ((InputPinSet)pinSet).getInputObjectPin();
		}else if(pinSet instanceof OutputPinSet){
			pins = ((OutputPinSet)pinSet).getOutputObjectPin();
//		}else if(pinSet instanceof InputParameterSet){
//			pins = ((InputParameterSet)pinSet).getParameter();
//		}else if(pinSet instanceof OutputParameterSet){
//			pins = ((OutputParameterSet)pinSet).getParameter();
		}
		return pins;
	}	
	
	private List collectElements(XSDComplexTypeDefinition compType) {
		List elements = new ArrayList();
		XSDParticle compTypeContent = (XSDParticle)compType.getContent();
		if (compTypeContent!=null) {
			XSDModelGroup contentGroup = (XSDModelGroup)compTypeContent.getContent();
			if (contentGroup!=null) {
				List particles = contentGroup.getContents();
				for (Iterator it=particles.iterator();it.hasNext();){
					XSDParticle particle = (XSDParticle)it.next();
					elements.add(particle.getContent());
				}
			}
		}
		return elements;
	}
}
