package com.ibm.btools.te.ilm.rulehandler.mapping.wps;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.PortType;

import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.te.framework.TransformationRule;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.CallBehaviorActionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.SANNestedProcessRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.SANTaskRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.impl.CallBehaviorActionRuleImpl;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.impl.ProcessRuleImpl;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.util.ScopeRegistryUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.CBPELMappingUtil;
import com.ibm.btools.te.ilm.heuristics.helper.CBPELUtil;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Assign;
import com.ibm.wbit.bpel.Copy;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Source;
import com.ibm.wbit.bpel.Target;
import com.ibm.wbit.bpelpp.FaultSource;
import com.ibm.wbit.bpelpp.FaultSources;

public class ProcessRuleHandler extends ConnectableRuleHandler {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 
	
	private void mapAssignContents(MappingManager mapMgr, ProcessRule rule, PinSet pinset, Assign assign)
	{
		for (Copy copy : (List<Copy>)assign.getCopy()) {
			mapMgr.addMapDefinition(pinset, copy);
		}
		
		//Map this ips to the bpel variable
		mapMgr.addMapDefinition(pinset, ScopeRegistryUtil.getBpelVariable(rule.getRoot().getContext(), pinset));
	}
	
	/**
	 * Handle the ProcessRule that just was executed, for a sub process only
	 * Source: IPS of BOM process
	 * Target: 
	 * Establishes the following mappings :
	 * IPS --> generated assign
	 * 		same IPS --> copy element in the generated assign
	 * 		same IPS --> variables used in the generated assign
	 * OPS --> generated assigns
	 * 		same OPS --> copy element in the generated assign
	 * 		same OPS --> variables used in the generated assign
	 * All IPS --> Empty Activity
	 * OPS --> generated assigns from cross boundary fault links
	 * 		same OPS --> copy element in the generated assign
	 * 		same OPS --> variables used in the generated assign
	 * 		same OPS --> the bpel link generated for the fault link
	 */
	public void handleRule(TransformationRule rule) {
		if(rule == null || !(rule instanceof ProcessRule) || !(rule.getParentRule() instanceof SANNestedProcessRule)){
			return;
		}
		
		MappingManager mapMgr = getMappingManager(rule);
		
		//Map IPS and OPS to their respective bpel (assign) activities
		//Lookup the process rule
		ProcessRuleImpl pr = (ProcessRuleImpl)rule;
		InputPinSet ips = (InputPinSet)rule.getSource().get(0);
		Activity activity = pr.getPinSetActivity(ips);
		if (activity != null) {
			//Map the assign generated from multiple IPS
			mapMgr.addMapDefinition(ips, activity);
			//Map the copy elements inside the assign.
			
			if (activity instanceof Assign) {
				Assign assign = (Assign)activity;
				mapAssignContents(mapMgr, pr, ips, assign);
			}
		}
		
		
		//Map OPS to their respective bpel (assign) activities
		for (OutputPinSet ops : (List<OutputPinSet>)ips.getOutputPinSet()) {
			Activity bpelActivity = pr.getPinSetActivity(ops);
			if (bpelActivity != null) {
				mapMgr.addMapDefinition(ops, bpelActivity);
				//Map the copy elements inside the assign.
				if (activity instanceof Assign) {
					Assign assign = (Assign)activity;
					mapAssignContents(mapMgr, pr, ops, assign);
				}
			}
			
			//Map generated assign from cross boundary fault links
			if (ops.getIsException() && CBPELUtil.shouldGenerateForkActivity(ops)) {
				Activity generatedAssign = pr.getMappedActivity(ops);
				if (generatedAssign != null) {
					mapMgr.addMapDefinition(ops, generatedAssign);
					if (generatedAssign instanceof Assign) {
						Assign assign = (Assign)generatedAssign;
						//Map the content of the assign
						mapAssignContents(mapMgr, pr, ops, assign);
						
						//An assign was generated as a result of a cross boundary fault.  We need to map this bpel link
						if (assign.getTargets() != null) {
							Target target = (Target) assign.getTargets().getChildren().get(0);
							mapMgr.addMapDefinition(ops, target.getLink());
						}
					}
				}
				
			}
		}
		
		
		//Map generated empty on the input set if there are multiple input sets
		Activity startScopeActivity = ScopeRegistryUtil.getBpelActivity(rule.getRoot().getContext(), ips.getAction());
		if (startScopeActivity != null) {
			mapMgr.addMapDefinition(ips, startScopeActivity);
		}
		
//		for (OutputPinSet outputPinSet : (List<OutputPinSet>)ips.getOutputPinSet()) {
//			if (outputPinSet.getIsException() && CBPELUtil.shouldGenerateForkActivity(outputPinSet)) {
//				Activity generatedAssign = pr.getMappedActivity(outputPinSet);
//				if (generatedAssign != null) {
//					mapMgr.addMapDefinition(outputPinSet, generatedAssign);
//					
//					if (generatedAssign.getTargets() != null) {
//						Target target = (Target) generatedAssign.getTargets().getChildren().get(0);
//						mapMgr.addMapDefinition(outputPinSet, target.getLink());
//					}
//				}
//				
//			}
//		}
	}
}
