package com.ibm.btools.te.ilm.rulehandler.mapping.wps;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.impl.ProcessRuleImpl;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.util.ScopeRegistryUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.CBPELMappingUtil;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingRegistry;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Scope;
import com.ibm.wbit.bpel.Source;
import com.ibm.wbit.bpel.Target;
import com.ibm.wbit.bpelpp.Description;
import com.ibm.wbit.bpelpp.Documentation;
import com.ibm.wbit.bpm.map.objectdefinition.ObjectDefinition;

/**
 * This handler is responsbile for creating the ObjectDenifition mapping between 
 * BOM <code> StructuredActivityNode </code> and BPEL <code>Scope </code>
 */
public class SANNestedProcessRuleHandler extends ConnectableRuleHandler {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 
	/**	
	 * @param rule the SANNestedProcessRule that has just executed
	 */
	public void handleRule(TransformationRule rule) {
		/**
		 * Handles the SANReusableProcessRule
		 * Source: BOM StructuredActivityNode
		 * Target: BPEL Scope
		 */
		InputPinSet pinSet = (InputPinSet)rule.getSource().get(0);
		Action san = pinSet.getAction();
		MappingManager mapMgr = getMappingManager(rule);

		if(rule.getTarget() != null && !rule.getTarget().isEmpty()){
			Scope scope = (Scope)rule.getTarget().get(0);
			mapMgr.addMapDefinition(san, scope);
			//this is the Flow
			if(scope.getActivity() != null){
				CBPELMappingUtil.registerFlowMapping(san, scope.getActivity(), rule.getRoot().getContext());
				mapMgr.addMapDefinition(san, scope.getActivity());
				
				List extElements = scope.getEExtensibilityElements();
				if(extElements != null && !extElements.isEmpty()){
					Iterator it = extElements.iterator();
					while(it.hasNext()){
						EObject extElement = (EObject)it.next();
						if(extElement instanceof Documentation || extElement instanceof Description){
							mapMgr.addMapDefinition(san, extElement);
						}
					}
				}
				
				
			}
		}		
		
		//incoming bom connections of the ips --> downstream link from assigns generated from multiple input sets
		ProcessRuleImpl pr = (ProcessRuleImpl) rule.getChildRules().get(0);
		Activity activity = pr.getMappedActivity(pinSet);
		if (activity != null) {
			if (activity.getSources() == null) {
				return;
			}
			
			Link downstreamLink = ((Source)activity.getSources().getChildren().get(0)).getLink();
			List pins = BomUtils.getPinsInSet(pinSet);
			for (Pin pin : (List<Pin>)pins) {
				if(pin instanceof ObjectPin){
					mapMgr.addMapDefinition(pin.getIncoming(), downstreamLink);
				}
			}
		}
	}
}
