package com.ibm.btools.te.ilm.rulehandler.mapping.wps;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import org.eclipse.emf.ecore.EClass;

import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.rulehandler.IRuleHandlerFactory;
import com.ibm.btools.te.framework.rulehandler.RuleHandler;
import com.ibm.btools.te.ilm.heuristics.HeuristicsPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.bpelp.BpelpPackage;
import com.ibm.btools.te.ilm.heuristics.br.BrPackage;
import com.ibm.btools.te.ilm.heuristics.extservice.ExtservicePackage;
import com.ibm.btools.te.ilm.heuristics.scdl.ScdlPackage;
import com.ibm.btools.te.ilm.heuristics.wsdl.WsdlPackage;
import com.ibm.btools.te.ilm.heuristics.xsd.XsdPackage;

/**
 * This class is responsible for determining which RuleHandler subclass should
 * be invoked following the executing of a TransformationRule.
 */
public class WpsRuleHandlerFactory implements IRuleHandlerFactory {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * This method returns the instance of a RuleHandler subclass that is 
	 * applicable for handling the given transformation rule.  It is called once 
	 * per type of TransformationRule, and the given instance is used from that
	 * point on to handle rules of the requested type.
	 * 
	 * @param rule The transformation rule that has just executed for the first time.
	 */
	public RuleHandler createHandler(TransformationRule rule) {
		
		EClass ruleClass = rule.eClass();
		AbstractbpelPackage p = AbstractbpelPackage.eINSTANCE;
		XsdPackage x = XsdPackage.eINSTANCE;
		HeuristicsPackage h = HeuristicsPackage.eINSTANCE;
		BpelpPackage b = BpelpPackage.eINSTANCE;
		ScdlPackage s = ScdlPackage.eINSTANCE;
		WsdlPackage w = WsdlPackage.eINSTANCE;
		ExtservicePackage extService = ExtservicePackage.eINSTANCE;
		BrPackage brpkg = BrPackage.eINSTANCE;
		
		if (ruleClass == p.getSANReusableProcessRule()){
			return new SANReusableProcessRuleHandler();
		} else if (ruleClass == p.getProcessDefinitionRule()){
			return new ProcessDefinitionRuleHandler();
		} else if (ruleClass == s.getComponentRule()){
			return new ComponentRuleHandler();
		}else if (ruleClass == p.getHumanTaskRule()){
			return new HumanTaskRuleHandler();
		}else if (ruleClass == s.getExportRule()){
			return new ExportRuleHandler();
		}else if (ruleClass == w.getPortTypeRule()||
				ruleClass == extService.getPortTypeRule()){
			return new PortTypeRuleHandler();
		} else if (ruleClass == w.getMessageRule()){
			return new MessageRuleHandler();
		} else if (ruleClass == w.getOperationRule()||
				ruleClass == extService.getOperationRule()){
			return new OperationRuleHandler();
//		}
//		else if (ruleClass == w.getPartnerLinkRule()){
//			return new PartnerLinkRuleHandler();
		} else if (ruleClass == p.getPartnerRule()){
			return new PartnerRuleHandler();
		} else if (ruleClass == w.getPartRule()){
			return new PartRuleHandler();
		} else if (ruleClass == w.getParameterRule()){
			return new ParameterRuleHandler();
		} else if (ruleClass == p.getInvokeRule()){
		    return new InvokeRuleHandler();
		} else if (ruleClass == p.getInputRule()){
		    return new InputRuleHandler();
		} else if (ruleClass == p.getOutputRule()){
		    return new OutputRuleHandler();
		} else if (ruleClass == p.getLinkRule()){
		    return new LinkRuleHandler();
		} else if (ruleClass == p.getVariableRule()){
		    return new VariableRuleHandler();
		} else if (ruleClass == p.getProcessInterfaceRule()){
		    return new ProcessInterfaceRuleHandler();
		} else if (ruleClass == x.getElementDeclarationRule()){
		    return new ElementDeclarationRuleHandler();
		} else if (ruleClass == x.getPinSetRule()){
		    return new PinSetRuleHandler();
		} else if(ruleClass == x.getClassRule()){
			return new ClassRuleHandler();
		}else if(ruleClass == x.getDataDefinitionRule()){
			return new DataDefinitionRuleHandler();
		}else if(ruleClass == x.getPropertyRule()){
			return new PropertyRuleHandler();
		}else if (ruleClass == p.getInvokeWithCallbackRule()){
		    return new InvokeWithCallbackRuleHandler();
		} else if (ruleClass == p.getDecisionRule()){
		    return new DecisionRuleHandler();
		} else if (ruleClass == p.getMapRule()){
		    return new MapRuleHandler();
		} else if (ruleClass == p.getLoopNodeRule()){
		    return new LoopNodeRuleHandler();
		} else if (ruleClass == b.getBpelpRule()){
		    return new BpelpRuleHandler();
		}  else if (ruleClass == p.getMergeRule()){
		    return new MergeRuleHandler();
		}   else if (ruleClass == p.getForkRule()) {
		    return new ForkRuleHandler();
		} else if (ruleClass == p.getJoinRule()){
		    return new JoinRuleHandler();
		} else if(ruleClass == p.getSANNestedProcessRule()){
		    return new SANNestedProcessRuleHandler();
		}else if (ruleClass == p.getBRReusableTaskRule()) {
			return new BRReusableTaskRuleHandler();
		}else if (ruleClass == p.getBRTaskRule()){
			return new BRTaskRuleHandler();
		}else if(ruleClass == p.getCallBehaviorActionRule()){
			return new CallBehaviorActionRuleHandler();
		}else if(ruleClass == p.getAssignRule()){
			return new AssignRuleHandler();
		}		
		//if (ruleClass == h.getProcessFlowPatternRule()){
//		    return new ProcessFlowPatternRuleHandler();
//		} else if (ruleClass == p.getSANReusableProcessRule()){
//		    return new SANReusableProcessRuleHandler();
//		} else if (ruleClass == p.getVariableRule()){
//		    return new VariableRuleHandler();
//		} else if (ruleClass == p.getInvokeRule()){
//		    return new InvokeRuleHandler();
//		} else if (ruleClass == p.getInvokeWithCallbackRule()){
//		    return new InvokeWithCallbackRuleHandler();
////		} else if (ruleClass == p.getGenericActionRule()){
////		    return new InvokeRuleHandler();
//		} else if (ruleClass == p.getInputValuePinRule()){
//		    return new InputValuePinRuleHandler();
//		} else if (ruleClass == p.getVariableAccessRule()){
//		    return new VariableAccessRuleHandler();
//		} else if (ruleClass == p.getDatastoreAccessRule()){
//		    return new VariableAccessRuleHandler();
//		} else if (ruleClass == p.getDecisionRule()){
//		    return new DecisionRuleHandler();
//		} else if (ruleClass == b.getBpelpRule()){
//		    return new BpelpRuleHandler();
//		} else if (ruleClass == p.getLinkRule()){
//		    return new LinkRuleHandler();
//		} else if (ruleClass == p.getForkRule()){
//		    return new ForkRuleHandler();
//		} else if (ruleClass == p.getJoinRule()){
//		    return new JoinRuleHandler();
//		} else if (ruleClass == p.getMapRule()){
//		    return new MapRuleHandler();
//		} else if (ruleClass == p.getMergeRule()){
//		    return new MergeRuleHandler();
//		} else if (ruleClass == p.getStaffActivityRule()){
//		    return new StaffActivityRuleHandler();
//		} else if (ruleClass == p.getHumanTaskRule()){
//		    return new HumanTaskRuleHandler();
//		} else if (ruleClass == p.getSANNestedProcessRule()){
//		    return new SANNestedProcessRuleHandler();
		else if (ruleClass == p.getHTTaskRule()){
			return new HTTaskRuleHandler();
		}
		else if (ruleClass == p.getSANTaskRule()){
		    return new SanTaskRuleHandler();
		} else if (ruleClass == p.getProcessRule()){
		    return new ProcessRuleHandler();
		} else if (ruleClass == p.getBpelOptimizationRule()){
		    return new BpelOptimizationRuleHandler();
		}else if(ruleClass == p.getExpressionRule()){
			return new ExpressionRuleHandler();
		}else if(ruleClass == x.getListRule()){
			return new ListRuleHandler();
		}else if(ruleClass == h.getTimeIntervalsRule()){
			return new TimetableRuleHandler();
		}else if(ruleClass == brpkg.getBRSetRule()){
			return new BRSetRuleHandler();
		}else if(ruleClass == brpkg.getBRTemplateRule()){
			return new BRTemplateRuleHandler();
		}else if(ruleClass == brpkg.getBRRule()){
			return new BRRuleHandler();
		}else if(ruleClass == brpkg.getBRGroupRule()){
			return new BRGroupRuleHandler();
		}
//		} else if (ruleClass == p.getLoopNodeRule()){
//		    return new LoopNodeRuleHandler();
//		} else if (ruleClass == x.getClassRule()){
//			return new ClassRuleHandler();
//		} else if (ruleClass == x.getPropertyRule()){
//			return new PropertyRuleHandler();
//		} else if (ruleClass == s.getComponentRule()){
//			return new ComponentRuleHandler();
//		} else if (ruleClass == h.getHeuristicsTransformation()){
//			return new HeuristicsTransformationHandler();
//		}
		return null;
	}

}
